@@ -1,11 +1,11 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 #-------------------------------------------------------------------------------
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -214,11 +214,6 @@ class cs_compile(object):
             flags.insert(0, "-I" + pkgincludedir)
             if self.pkg.config.libs['ple'].variant == "internal":
                 flags.insert(0, "-I" + self.pkg.get_dir("includedir"))
-            if self.pkg.name != os.path.basename(pkgincludedir):
-                d = os.path.join(self.pkg.get_dir("includedir"),
-                                 self.pkg.name)
-                if os.path.isdir(d):
-                    flags.insert(0, "-I" + d)
 
         elif flag == 'ldflags':
             flags.insert(0, "-L" + self.pkg.get_dir("libdir"))
@@ -231,7 +226,7 @@ class cs_compile(object):
 
     #---------------------------------------------------------------------------
 
-    def get_flags(self, flag):
+    def get_flags(self, flag, base_name=None):
         """
         Determine compilation flags for a given flag type.
         """
@@ -244,6 +239,17 @@ class cs_compile(object):
                 and (not self.pkg.config.libs[lib].dynamic_load)):
                 cmd_line += separate_args(self.pkg.config.libs[lib].flags[flag])
 
+        if flag == 'cppflags' and base_name:
+            add_dirs = []
+            if base_name in self.pkg.config.exec_include:
+                add_dirs.append(self.pkg.config.exec_include[base_name])
+            for a_d in add_dirs:
+                d = os.path.join(self.pkg.get_dir("includedir"), a_d)
+                if os.path.isdir(d):
+                    add_flag = "-I" + d
+                    if add_flag not in cmd_line:
+                        cmd_line.insert(0, add_flag)
+
         # Specific handling of low-level libraries, which should come last,
         # such as -lm and -lpthread:
         # If -lm appears multiple times, only add it at the end of the
@@ -321,7 +327,7 @@ class cs_compile(object):
 
     #-------------------------------------------------------------------------------
 
-    def compile_src(self, src_list=None,
+    def compile_src(self, base_name=None, src_list=None,
                     opt_cflags=None, opt_cxxflags=None, opt_fcflags=None,
                     keep_going=False,
                     stdout=sys.stdout, stderr=sys.stderr):
@@ -374,7 +380,7 @@ class cs_compile(object):
             if os.path.basename(f) == 'cs_base.c':
                 cmd += ['-DLOCALEDIR=\\"' + pkg.get_dir('localedir') + '\\"', \
                         '-DPKGDATADIR=\\"' + pkg.get_dir('pkgdatadir') + '\\"']
-            cmd += self.get_flags('cppflags')
+            cmd += self.get_flags('cppflags', base_name=base_name)
             cmd += separate_args(pkg.config.flags['cflags'])
             cmd += ["-c", f]
             if run_command(cmd, pkg=pkg, echo=True,
@@ -393,7 +399,7 @@ class cs_compile(object):
             for d in c_include_dirs:
                 cmd += ["-I", d]
             cmd.append('-DHAVE_CONFIG_H')
-            cmd += self.get_flags('cppflags')
+            cmd += self.get_flags('cppflags', base_name=base_name)
             cmd += separate_args(pkg.config.flags['cxxflags'])
             cmd += ["-c", f]
             if run_command(cmd, pkg=pkg, echo=True,
@@ -425,6 +431,9 @@ class cs_compile(object):
             if f_base == 'cs_user_porosity.f90':
                 o_name = "cs_f_user_porosity.o"
                 cmd += ["-o", o_name]
+            if f_base == 'cs_user_source_terms.f90':
+                o_name = "cs_f_user_source_terms.o"
+                cmd += ["-o", o_name]
             if opt_fcflags != None:
                 cmd += separate_args(opt_fcflags)
             for d in f_include_dirs:
@@ -502,12 +511,6 @@ class cs_compile(object):
         if o_files:
             cmd += o_files
 
-        # If present, address sanitizer needs to come first
-
-        if '-lasan' in p_libs:
-            p_libs.remove('-lasan')
-            cmd += ['-lasan']
-
         if os.path.basename(exec_name) in self.pkg.config.exec_libs:
             cmd += [self.pkg.config.exec_libs[os.path.basename(exec_name)]]
 
@@ -519,6 +522,17 @@ class cs_compile(object):
         if pkg.config.rpath != "":
             cmd += self.so_dirs_path(cmd)
 
+        # Clean paths in link flags
+        cmd_new = []
+        for c in cmd:
+            if c[:2] == '-L':
+                c = '-L' + os.path.normpath(c[2:])
+                if cmd_new.count(c) < 1:
+                    cmd_new.append(c)
+            else:
+                cmd_new.append(c)
+        cmd = cmd_new
+
         # Call linker
 
         if retval == 0:
@@ -541,7 +555,7 @@ class cs_compile(object):
 
     #---------------------------------------------------------------------------
 
-    def compile_and_link(self, srcdir, destdir=None, src_list=None,
+    def compile_and_link(self, base_name, srcdir, destdir=None, src_list=None,
                          opt_cflags=None, opt_cxxflags=None, opt_fcflags=None,
                          opt_libs=None, force_link=False, keep_going=False,
                          stdout=sys.stdout, stderr=sys.stderr):
@@ -552,7 +566,7 @@ class cs_compile(object):
 
         # Determine executable name
 
-        exec_name = self.pkg.solver
+        exec_name = base_name
         if destdir != None:
             exec_name = os.path.join(destdir, exec_name)
 
@@ -575,7 +589,7 @@ class cs_compile(object):
         for f in dir_files:
             src_list.append(os.path.join(srcdir, f))
 
-        retval, obj_list = self.compile_src(src_list,
+        retval, obj_list = self.compile_src(base_name, src_list,
                                             opt_cflags, opt_cxxflags, opt_fcflags,
                                             keep_going, stdout, stderr)
 
@@ -598,7 +612,7 @@ class cs_compile(object):
 
 #---------------------------------------------------------------------------
 
-def compile_and_link(pkg, srcdir, destdir=None,
+def compile_and_link(pkg, base_name, srcdir, destdir=None,
                      opt_cflags=None, opt_cxxflags=None, opt_fcflags=None,
                      opt_libs=None, force_link=False, keep_going=False,
                      stdout=sys.stdout, stderr=sys.stderr):
@@ -607,7 +621,8 @@ def compile_and_link(pkg, srcdir, destdir=None,
     """
     c = cs_compile(pkg)
 
-    retcode = c.compile_and_link(srcdir,
+    retcode = c.compile_and_link(base_name,
+                                 srcdir,
                                  destdir=destdir,
                                  opt_cflags=opt_cflags,
                                  opt_cxxflags=opt_cxxflags,
@@ -629,7 +644,10 @@ def main(argv, pkg):
     Main function.
     """
 
-    from cs_exec_environment import set_modules, source_rcfile
+    try:
+        from code_saturne.cs_exec_environment import set_modules, source_rcfile
+    except Exception:
+        from cs_exec_environment import set_modules, source_rcfile
 
     test_mode, force_link, keep_going, src_dir, dest_dir, \
         version, cflags, cxxflags, fcflags, libs = process_cmd_line(argv, pkg)
@@ -644,6 +662,7 @@ def main(argv, pkg):
         dest_dir = None
 
     retcode = compile_and_link(pkg,
+                               pkg.solver,
                                src_dir,
                                destdir=dest_dir,
                                opt_cflags=cflags,
@@ -661,7 +680,7 @@ if __name__ == '__main__':
 
     # Retrieve package information (name, version, installation dirs, ...)
 
-    from cs_package import package
+    from code_saturne.cs_package import package
     pkg = package()
 
     # Create an instance of the main script
