@@ -1,11 +1,11 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 #-------------------------------------------------------------------------------
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -46,6 +46,7 @@ rank_id = -1
 
 debuggers = {"gdb": "GNU gdb debugger",
              "cgdb": "Console front-end to gdb",
+             "cuda-gdb": "CUDA debugger",
              "gdbgui": "gdbgui gdb web browser interface",
              "ddd": "Data Display Debugger",
              "emacs": "Emacs with gdb debugger",
@@ -322,8 +323,11 @@ def process_cmd_line(argv, pkg):
 
     try:
         if cmds['debugger']:
+            idx_insert = 1
             for idx in range(idx_s, idx_e):
-                cmds['debugger'].insert(idx-idx_s + 1, argv[idx])
+                if argv[idx] not in cmds['debugger']:
+                    cmds['debugger'].insert(idx_insert, argv[idx])
+                    idx_insert += 1
     except Exception:
         pass
 
@@ -518,8 +522,12 @@ def run_gdb_debug(path, args=None, gdb_cmds=None,
 
     f = gen_cmd_file(cmds)
 
-    cmd.append('-x')
-    cmd.append(f)
+    if debugger_ui == 'gdbgui':
+        cmd.append('--gdb-cmd')
+        cmd.append(gdb + ' -x ' + f + ' ' + path)
+    else:
+        cmd.append('-x')
+        cmd.append(f)
 
     if debugger_command_file:
         fp = open(debugger_command_file)
@@ -532,18 +540,16 @@ def run_gdb_debug(path, args=None, gdb_cmds=None,
 
     # Finalize command
 
-    cmd.append(path)
+    if debugger_ui != 'gdbgui':
+        cmd.append(path)
 
     # Start building command to run
 
     if debugger_ui in ['terminal', 'cgdb']:
-        if rank_id < 0 and not target:
-            cmd = [str(debugger)] + cmd
-        else:
-            cmd_string = str(debugger)
-            for c in cmd:
-                cmd_string += ' ' + enquote_arg(str(c))
-            cmd = [term, '-e', cmd_string]
+        cmd_string = str(debugger)
+        for c in cmd:
+            cmd_string += ' ' + enquote_arg(str(c))
+        cmd = [term, '-e', cmd_string]
 
     elif debugger_ui == 'gdbgui':
         cmd.insert(0, debugger)
@@ -567,7 +573,8 @@ def run_gdb_debug(path, args=None, gdb_cmds=None,
         else:
             cmd_string += ' -i=mi'   # emacs 24 and newer
         for c in cmd:
-            cmd_string += ' ' + enquote_arg(str(c))
+            if cmd != gdb:
+                cmd_string += ' ' + enquote_arg(str(c))
         cmd_string += r'\")"'
         cmd = [debugger, '--eval', cmd_string]
 
@@ -575,6 +582,7 @@ def run_gdb_debug(path, args=None, gdb_cmds=None,
         cmd[0] = 'gdb'
 
     if not debugger_ui in ("emacs", "emacs23"):
+        print(cmd)
         p = subprocess.Popen(cmd)
     else:
         cmd_line = cmd[0]
@@ -623,6 +631,13 @@ def run_vgdb_debug(path,
             idx = output.find("target remote")
             if idx > -1:
                 cmd = output[idx:].rstrip()
+                # Work around bug on some Debian 10 systems
+                c = cmd.split()
+                if not os.path.isfile(c[3]):
+                    if c[3][:9] == '/usr/lib/':
+                        c[3] = '/usr/bin/vgdb'
+                        if os.path.isfile(c[3]):
+                            cmd = ' '.join(c)
                 try:
                     vgdb_pid = int(cmd[cmd.index("--pid")+6:])
                 except Exception:
@@ -761,7 +776,7 @@ def run_debug(cmds):
     if 'debugger' in cmds.keys():
         debugger = cmds['debugger'][0]
         dbg_name = os.path.basename(debugger)
-        if dbg_name in debuggers.keys() and dbg_name != 'gdb':
+        if dbg_name in debuggers.keys() and dbg_name not in ('gdb', 'cuda-gdb'):
             debugger_ui = dbg_name
         elif dbg_name.find("emacs23") > -1:
             debugger_ui = 'emacs23'
