@@ -1,11 +1,11 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 #-------------------------------------------------------------------------------
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -39,7 +39,12 @@ import tempfile
 
 python_version = sys.version[:3]
 
-import cs_batch
+try:
+    from code_saturne import cs_batch
+except Exception:
+    import cs_batch
+
+saved_os_env_vars = {}
 
 #===============================================================================
 # Utility functions
@@ -82,25 +87,25 @@ def separate_args(s):
         sep = False
         protected = False
         in_quotes = ''
-        for i in range(len(s)):
+        for i, c in enumerate(s):
             if protected:
-                a += s[i]
+                a += c
                 protected = False
             else:
-                if s[i] == '\\' and s[i+1:i+1].isalnum():
+                if c == '\\' and s[i+1:i+1].isalnum():
                     protected = True
-                elif s[i] == '"' or s[i] == "'":
-                    if in_quotes == s[i]:
-                        a += s[i]
+                elif c == '"' or c == "'":
+                    if in_quotes == c:
+                        a += c
                         in_quotes = ''
                     elif in_quotes != '':
-                            a += s[i]
+                            a += c
                     else:
-                        a += s[i]
-                        in_quotes = s[i]
+                        a += c
+                        in_quotes = c
                 elif in_quotes != '':
-                    a += s[i]
-                elif (s[i] == ' ' or s[i] == '\t'):
+                    a += c
+                elif (c == ' ' or c == '\t'):
                     if a != '':
                         if (a[0] == a[-1:]) and (a[0] == '"' or a[0] == "'"):
                             l.append(a[1:-1])
@@ -108,10 +113,13 @@ def separate_args(s):
                             l.append(a)
                         a = ''
                 else:
-                    a += s[i]
+                    a += c
 
         if a != '':
-            l.append(a)
+            if (a[0] == a[-1:]) and (a[0] == '"' or a[0] == "'"):
+                l.append(a[1:-1])
+            else:
+                l.append(a)
 
     return l
 
@@ -153,6 +161,58 @@ def assemble_args(cmd):
             l += ' ' + s
     return l.strip()
 
+#-------------------------------------------------------------------------------
+# Update command line arguments with no associated value
+#-------------------------------------------------------------------------------
+
+def update_command_no_value(args, options, present):
+    """
+    Adds, updates, or removes parts of a command to pass a given option.
+    The command is provided as a list, and options defining a value may be
+    defined as a tuple (to allow for multiple variants).
+
+    If no option was previously present and a value is added, the first
+    syntax of the options tuple will be used.
+    """
+
+    # Update first occurence
+
+    count = 0
+    target = 0
+    if present:
+        target = 1
+
+    for opt in options:
+        j = 0
+        while j < len(args):
+            if args[j] == opt:
+                count += 1
+                if count > target:
+                    args.pop(j)       # option
+                else:
+                    j += 1
+            else:
+                j += 1
+
+    if count < target:
+        args.append(options[0])
+
+    # Special cases at end
+
+    i = 0
+    args_tail = []
+    while i < len(args):
+        if args[i][0] == '$' or args[i] == '&':
+             a = args.pop(i)
+             args_tail.append(a)
+        else:
+             i = i+1
+    args += args_tail
+
+    # Return updated list
+
+    return args
+
 #-------------------------------------------------------------------------------
 # Update command line arguments for a given value
 #-------------------------------------------------------------------------------
@@ -322,7 +382,7 @@ def append_shell_shebang(l):
         l.append('@echo off')
     else:
         user_shell = get_shell_type()
-        l.append('#!' + user_shell + '\n\n')
+        l.append('#!' + user_shell)
     l.append('')
 
 #-------------------------------------------------------------------------------
@@ -469,7 +529,7 @@ def get_script_positional_args():
     if sys.platform.startswith('win'):
         args = '%*'
     else:
-        args = '$@'
+        args = '"$@"'
     return args
 
 #-------------------------------------------------------------------------------
@@ -487,17 +547,62 @@ def get_script_return_code():
 
 #-------------------------------------------------------------------------------
 
-def run_command(cmd, pkg = None, echo = False,
+def clean_os_environ_for_shell():
+    """
+    Clean environment relative to sub-shells.
+    """
+    # In case LMOD is present, avoid warnings on macros
+    # for called scripts.
+
+    is_bash = False
+    shell = get_shell_type()
+    if 'bash' in shell:
+        is_bash = True
+
+    # In case LMOD is present, avoid warnings on macros
+    # for called scripts.
+    for ev in ('BASH_FUNC_module%%', 'BASH_FUNC_ml%%'):
+        if ev in os.environ:
+            if is_bash:
+                saved_os_env_vars[ev] = os.environ[ev]
+            del os.environ[ev]
+
+#-------------------------------------------------------------------------------
+
+def full_os_environ_for_shell(env=None):
+    """
+    Return environment relative to sub-shells.
+    """
+
+    if saved_os_env_vars:
+        if env:
+            s_env = env.copy()
+        else:
+            s_env = os.environ.copy()
+        for key in saved_os_env_vars.keys():
+            s_env[key] = saved_os_env_vars[key]
+
+    else:
+        if env:
+            s_env = env
+        else:
+            s_env = os.environ
+
+    return s_env
+
+#-------------------------------------------------------------------------------
+
+def run_command(args, pkg = None, echo = False,
                 stdout = sys.stdout, stderr = sys.stderr, env = None):
     """
     Run a command.
     """
     if echo == True:
-        if type(cmd) == str:
-            stdout.write(str(cmd) + '\n')
+        if type(args) == str:
+            stdout.write(str(args) + '\n')
         else:
             l = ''
-            for s in cmd:
+            for s in args:
                 if (s.find(' ') > -1):
                     l += ' ' + enquote_arg(s)
                 else:
@@ -525,17 +630,33 @@ def run_command(cmd, pkg = None, echo = False,
     if (stderr != sys.stderr):
         kwargs['stderr'] = stderr
 
-    if type(cmd) == str:
-        p = subprocess.Popen(cmd,
-                             shell=True,
-                             executable=get_shell_type(),
-                             universal_newlines=True,
-                             env = env,
-                             **kwargs)
-    else:
-        p = subprocess.Popen(cmd, universal_newlines=True, env = env, **kwargs)
-
-    p.communicate()
+    returncode = 1
+    try:
+        if type(args) == str:
+            p = subprocess.Popen(args,
+                                 shell=True,
+                                 executable=get_shell_type(),
+                                 universal_newlines=True,
+                                 env=full_os_environ_for_shell(env),
+                                 **kwargs)
+        else:
+            p = subprocess.Popen(args,
+                                 universal_newlines=True,
+                                 env=env,
+                                 **kwargs)
+        p.communicate()
+        returncode = p.returncode
+    except Exception as e:
+        import traceback
+        exc_type, exc_value, exc_traceback = sys.exc_info()
+        traceback.print_exception(exc_type, exc_value, exc_traceback,
+                                  limit=2, file=sys.stderr)
+        print("")
+        print("Failed calling subprocess with:")
+        print("  args = " + str(args))
+        print("  env = " + str(env))
+        print("  kwargs = " + str(kwargs))
+        sys.exit(1)
 
     # Reset the PATH to its previous value
 
@@ -543,7 +664,7 @@ def run_command(cmd, pkg = None, echo = False,
         if pkg.config.features['relocatable'] == "yes":
             os.environ['PATH'] = saved_path
 
-    return p.returncode
+    return returncode
 
 #-------------------------------------------------------------------------------
 
@@ -608,13 +729,29 @@ def source_shell_script(path):
 
     if not os.path.isfile(path):
         sys.stderr.write('Warning:\n'
-                         + '   file ' + path + '\n'
+                         + '  file ' + path + '\n'
                          + 'not present, so cannot be sourced.\n\n')
 
     if sys.platform.startswith('win'):
         return
 
-    user_shell = os.getenv('SHELL')
+    user_shell = None
+
+    # If shebang is present, try to use it.
+    # (TODO: handle more complex cases with env)
+    try:
+        f = open(path, 'r')
+        l = f.readline()
+        f.close()
+        if l[:2] == '#!':
+            e = l[2:].split()[0]
+            if e[-3:] != 'env':
+                user_shell = e
+    except Exception:
+        pass
+
+    if not user_shell:
+        user_shell = os.getenv('SHELL')
     if not user_shell:
         user_shell = '/bin/sh'
 
@@ -625,7 +762,8 @@ def source_shell_script(path):
                          executable=user_shell,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE,
-                         universal_newlines=True)
+                         universal_newlines=True,
+                         env=full_os_environ_for_shell())
 
     output = p.communicate()[0]
 
@@ -681,7 +819,42 @@ def source_rcfile(pkg):
 
 #-------------------------------------------------------------------------------
 
-def source_syrthes_env(pkg):
+def get_ld_library_path_additions(pkg):
+    """
+    return library search path additions.
+
+    With ELF shared libraries (Linux/Unix), We may need to add
+    paths to LD_LIBRARY_PATH, because if the compiler is configured
+    to use --enable-new-datgs,
+    DT_RUNPATH will be present instead of DT_RPATH in the library header,
+    and will have lower priority than LD_LIBRARY_PATH.
+
+    This function determines which paths should be added to
+    ld_library_path for safety.
+    """
+
+    lib_dirs = []
+
+    ld_library_path = os.getenv('LD_LIBRARY_PATH')
+    if ld_library_path:
+        ld_library_path_dirs = ld_library_path.split(':')
+
+        for lib in pkg.config.deplibs:
+            if (pkg.config.libs[lib].have == "yes"
+                and (not pkg.config.libs[lib].dynamic_load)):
+                ldflags = separate_args(pkg.config.libs[lib].flags['ldflags'])
+                for f in ldflags:
+                    if f[0:2] == "-L":
+                        d = f[2:]
+                        if os.path.isdir(d):
+                            if not (d in ld_library_path_dirs or d in lib_dirs):
+                                lib_dirs.append(d)
+
+    return lib_dirs
+
+#-------------------------------------------------------------------------------
+
+def source_syrthes_env(pkg, verbose=True):
     """
     Source SYRTHES environment
     """
@@ -715,18 +888,28 @@ def source_syrthes_env(pkg):
     # for consistency with case creation parameters; it this is not enough,
     # use existing environment or load one based on current configuration.
 
-    try:
-        for p in sys.path:
-            if p[-14:] == '/share/syrthes' or p[-14:] == '\share\syrthes':
-                syr_profile = os.path.join(p[:,-14], 'bin', 'syrthes.profile')
-                if os.path.isfile(syr_profile):
+    for p in sys.path:
+        if p[-14:] == '/share/syrthes' or p[-14:] == '\share\syrthes':
+            syr_profile = os.path.join(p[:-14], 'bin', 'syrthes.profile')
+            if os.path.isfile(syr_profile):
+                if verbose:
+                    sys.stdout.write("Sourcing SYRTHES environment: " \
+                                     + syr_profile + "\n")
+                try:
                     source_shell_script(syr_profile)
-    except Exception:
-        pass
+                except Exception:
+                    sys.stderr.write("  Failed sourcing SYRTHES environment.\n")
+                # Ensure exit from the loop once syrthes is found
+                break
 
     env_syrthes_home = os.getenv('SYRTHES4_HOME')
+    if not env_syrthes_home:
+        env_syrthes_home = os.getenv('SYRTHES_HOME')
 
     if not syrthes_home:
+        if verbose:
+            sys.stdout.write("Set syrthes_home based on SYRTHES4_HOME: " \
+                             + str(env_syrthes_home) + "\n")
         syrthes_home = env_syrthes_home
 
     if not syrthes_home:
@@ -743,15 +926,21 @@ def source_syrthes_env(pkg):
     if syrthes_home != env_syrthes_home:
         syr_profile = os.path.join(config.get('install', 'syrthes'),
                                    'bin', 'syrthes.profile')
+        if verbose:
+            sys.stdout.write("Sourcing SYRTHES environment: " \
+                             + syr_profile + "\n")
         source_shell_script(syr_profile)
 
     # Finally, ensure module can be imported
 
+    syrthes_bin = os.path.join(syrthes_home, 'bin')
     syr_datapath = os.path.join(syrthes_home,
                                 os.path.join('share', 'syrthes'))
-    if sys.path.count(syr_datapath) > 0:
-        sys.path.remove(syr_datapath)
-    sys.path.insert(0, syr_datapath)
+
+    for p in (syrthes_bin, syr_datapath):
+        if sys.path.count(p) > 0:
+            sys.path.remove(p)
+        sys.path.insert(0, p)
 
 #-------------------------------------------------------------------------------
 
@@ -762,10 +951,9 @@ def get_parent_process_path():
 
     path = None
 
-    if sys.platform.startswith('win'):
+    if not sys.platform.startswith('win'):
         try:
             f = open("/proc/" + str(os.getppid()) + "/cmdline")
-            l = f.readlines()
             l = f.read()
             path = l.split('\x00')[1]
         except Exception:
@@ -864,7 +1052,14 @@ class batch_info:
             rtime = get_command_output(cmd)
         elif self.batch_type == 'SLURM':
             cmd = "squeue -h -j $SLURM_JOBID -o %L"
-            rtime = cs_batch.parse_wall_time_slurm(get_command_output(cmd))
+            # In case of job array, check on first line
+            rs = get_command_output(cmd)
+            if rs:
+                rtime = cs_batch.parse_wall_time_slurm(rs.splitlines()[0])
+            else:
+                msg = "Error: command\n  " + cmd + "\n\n"
+                msg += "returned no output\n"
+                msg += "unable to determine remaining time\n"
 
         return rtime
 
@@ -948,7 +1143,7 @@ class resource_info(batch_info):
             s = os.getenv('LSB_MCPU_HOSTS')
             if s != None:
                 mcpu_list = s.split(' ')
-                self.n_nodes = len(mcpu_list)/2
+                self.n_nodes = len(mcpu_list) // 2
                 for i in range(self.n_nodes):
                     self.n_procs += int(mcpu_list[i*2 + 1])
             else:
@@ -981,7 +1176,7 @@ class resource_info(batch_info):
                     self.n_procs = self.n_nodes*16
                     if n_threads:
                         if n_threads > 4:
-                            self.n_procs = self.n_nodes*16*4/n_threads
+                            self.n_procs = self.n_nodes*16*4 // n_threads
             s = os.getenv('LOADL_HOSTFILE')
             if s != None:
                 self.manager = 'LOADL'
@@ -1061,7 +1256,7 @@ class resource_info(batch_info):
             if self.n_procs != None:
                 if self.n_procs != n_procs:
                     sys.stderr.write('Warning:\n'
-                                     +'   Will try to use ' + str(n_procs)
+                                     +'  Will try to use ' + str(n_procs)
                                      + ' processes while resource manager ('
                                      + self.manager + ')\n   allows for '
                                      + str(self.n_procs) + '.\n\n')
@@ -1075,7 +1270,7 @@ class resource_info(batch_info):
         if n_threads != None:
             if self.n_threads != None and n_threads != self.n_threads:
                 sys.stderr.write('Warning:\n'
-                                 +'   Will try to use ' + str(self.n_threads)
+                                 +'  Will try to use ' + str(self.n_threads)
                                  + ' threads per task while resource manager ('
                                  + self.manager + ')\n   allows for '
                                  + str(n_threads) + '.\n\n')
@@ -1091,7 +1286,7 @@ class resource_info(batch_info):
 
         ppn = None
         if self.n_procs != None and  self.n_nodes != None:
-            ppn = self.n_procs / self.n_nodes
+            ppn = self.n_procs // self.n_nodes
 
         return ppn
 
@@ -1168,7 +1363,7 @@ class resource_info(batch_info):
             if s != None:
                 mcpu_list = s.split(' ')
                 hosts_list = []
-                for i in range(len(mcpu_list)/2):
+                for i in range(len(mcpu_list) // 2):
                     host = mcpu_list[i*2]
                     count = int(mcpu_list[i*2 + 1])
                     for j in range(count):
@@ -1309,7 +1504,6 @@ class mpi_environment:
                                'MSMPI':self.__init_msmpi__,
                                'OpenMPI':self.__init_openmpi__,
                                'BullxMPI':self.__init_openmpi__,
-                               'BGQ_MPI':self.__init_bgq__,
                                'Platform_MPI':self.__init_platform_mpi__}
             if self.type in mpi_env_by_type:
                 init_method = mpi_env_by_type[self.type]
@@ -1444,7 +1638,7 @@ class mpi_environment:
             return 'gforker' # might also be remshell
 
         sys.stderr.write('Warning:\n'
-                         + '   Unable to determine MPICH program manager:'
+                         + '  Unable to determine MPICH program manager:'
                          + ' assume "Hydra".\n\n')
 
         return 'hydra'
@@ -1581,7 +1775,7 @@ class mpi_environment:
                     hosts = True
             if hosts == True:
                 sys.stderr.write('Warning:\n'
-                                 + '   Hosts list will be ignored by'
+                                 + '  Hosts list will be ignored by'
                                  + ' MPICH gforker program manager.\n\n')
 
         # Info commands
@@ -1679,46 +1873,6 @@ class mpi_environment:
 
     #---------------------------------------------------------------------------
 
-    def __init_bgq__(self, p, resource_info=None, wdir = None):
-
-        """
-        Initialize for Blue Gene/Q environment.
-        """
-
-        # Set base executable path
-
-        self.mpiexec = 'runjob'
-
-        # Determine processor count and MPMD handling
-
-        self.mpiexec_n = ' --np '
-        self.mpmd = MPI_MPMD_configfile
-
-        rm = None
-        ppn = 1
-        if resource_info != None:
-            rm = resource_info.manager
-            ppn = resource_info.n_procs_per_node()
-        if rm == 'SLURM':
-            self.mpiexec = 'srun'
-            self.mpiexec_n = ' --ntasks='
-            if ppn:
-                self.mpiexec_n_per_node = ' --ntasks-per-node=' + str(ppn)
-        else:
-            if ppn != 1:
-                self.mpiexec_n_per_node = ' --ranks-per-node ' + str(ppn)
-            self.mpiexec_separator = ':'
-
-        # Other options to add
-
-        # self.mpiexec_exe = '--exe'
-        # self.mpiexec_args = '--args'
-        # self.mpiexec_envs = '--envs OMP_NUM_THREADS=' + str(omp_num_threads)
-
-        # Info commands
-
-    #---------------------------------------------------------------------------
-
     def __init_platform_mpi__(self, p, resource_info=None, wdir = None):
         """
         Initialize for Platform MPI environment.
@@ -1881,7 +2035,7 @@ class exec_environment:
 
 if __name__ == '__main__':
 
-    import cs_package
+    from code_saturne import cs_package
     pkg = cs_package.package()
     e = exec_environment(pkg)
 
