@@ -1,10 +1,10 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 #-------------------------------------------------------------------------------
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -31,19 +31,39 @@ This module defines the following functions:
 - launch_manual
 """
 
-
 #-------------------------------------------------------------------------------
 # Library modules import
 #-------------------------------------------------------------------------------
 
-
 import os, sys, shutil, stat, fnmatch
 from optparse import OptionParser
 
 #-------------------------------------------------------------------------------
-# Processes the passed command line arguments
+# Licence text
 #-------------------------------------------------------------------------------
 
+licence_text = \
+"""
+Copyright (C) 1998-2021 EDF S.A.
+
+This program is free software; you can redistribute it and/or modify it under \
+the terms of the GNU General Public License as published by the Free Software \
+Foundation; either version 2 of the License, or (at your option) any later \
+version.
+
+This program is distributed in the hope that it will be useful, but WITHOUT \
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS \
+FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more \
+details.
+
+You should have received a copy of the GNU General Public License along with \
+this program; if not, write to the Free Software Foundation, Inc., 51 Franklin \
+Street, Fifth Floor, Boston, MA 02110-1301, USA.
+"""
+
+#-------------------------------------------------------------------------------
+# Processes the passed command line arguments
+#-------------------------------------------------------------------------------
 
 def process_cmd_line(argv, pkg):
     """
@@ -71,6 +91,10 @@ def process_cmd_line(argv, pkg):
                       metavar="<guide>", action="append",
                       help="open a manual " + str(get_docs(pkg)))
 
+    parser.add_option("--modules", dest="modules",
+                      action="store_true",
+                      help="print evironment modules")
+
     parser.add_option("--version", dest="version",
                       action="store_true",
                       help="print version number")
@@ -85,6 +109,10 @@ def process_cmd_line(argv, pkg):
         print_version(pkg)
         sys.exit(0)
 
+    if options.modules:
+        print_modules(pkg)
+        sys.exit(0)
+
     if len(args) > 0 or len(options.guides) == 0:
         parser.print_help()
         sys.exit(1)
@@ -96,7 +124,6 @@ def process_cmd_line(argv, pkg):
 # Print Code_Saturne version
 #-------------------------------------------------------------------------------
 
-
 def print_version(pkg):
     """
     Print Code_Saturne version.
@@ -104,6 +131,21 @@ def print_version(pkg):
 
     print(pkg.code_name + " version: " + pkg.version_full)
 
+#-------------------------------------------------------------------------------
+# Print Environment modules info
+#-------------------------------------------------------------------------------
+
+def print_modules(pkg):
+    """
+    Print Code_Saturne environment modules info.
+    """
+
+    from code_saturne import cs_config
+    c = cs_config.config()
+
+    if c.env_modulecmd:
+        print("Module command:      " + str(c.env_modulecmd))
+        print("Environment modules: " + str(c.env_modules))
 
 #-------------------------------------------------------------------------------
 # Launch the PDF manual
@@ -123,7 +165,6 @@ def get_docs(pkg):
                 l.append('Doxygen')
     return l
 
-
 def launch_manual(reader, m, pkg):
     """
     Launch the corresponding PDF manual.
