@@ -1,11 +1,11 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 #-------------------------------------------------------------------------------
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -31,12 +31,9 @@ try:
 except Exception:
     import configparser  # Python3
 
-try:
-    from code_saturne.cs_exec_environment import separate_args, assemble_args, \
-        enquote_arg, get_command_single_value, update_command_single_value
-except exception:
-    from cs_exec_environment import separate_args, assemble_args, \
-        enquote_arg, get_command_single_value, update_command_single_value
+from code_saturne.cs_exec_environment import separate_args, assemble_args, \
+    enquote_arg, get_command_single_value, update_command_single_value, \
+    update_command_no_value
 
 #===============================================================================
 # Class used to manage runcase files
@@ -47,11 +44,10 @@ class runcase(object):
     def __init__(self,
                  path,
                  package=None,
-                 create_if_missing=True,
-                 rebuild=False,
-                 study_name=None,
-                 case_name=None,
-                 ignore_batch=False):
+                 submit=False,
+                 job_header=None,
+                 prologue=None,
+                 epilogue=None):
         """
         Initialize runcase info object.
         """
@@ -59,41 +55,26 @@ class runcase(object):
         self.path = path
         self.package = package
 
-        try:
+        if submit:
+            self.build_template(job_header=job_header,
+                                prologue=prologue, epilogue=epilogue)
+        else:
             f = open(self.path, mode = 'r')
             self.lines = f.readlines()
             f.close()
-            if rebuild:
-                self.get_run_command()
-                command_line = self.lines[self.run_cmd_line_id]
-                self.build_template(package,
-                                    study_name,
-                                    case_name,
-                                    ignore_batch)
-                self.lines[self.run_cmd_line_id] = command_line
 
-        except IOError:
-            if create_if_missing or rebuild:
-                self.build_template(package, study_name, case_name, ignore_batch)
-            else:
-                print("Error: can not open or read %s\n" % self.path)
-                sys.exit(1)
+            for i in range(len(self.lines)):
+                self.lines[i] = self.lines[i].rstrip()
 
-        for i in range(len(self.lines)):
-            self.lines[i] = self.lines[i].rstrip()
-
-        self.get_run_command()
+            self.get_run_command()
 
     #---------------------------------------------------------------------------
 
-    def save(self, path=None):
+    def save(self):
         """
-        Save runcase, optionally to a different path.
+        Save runcase.
         """
 
-        if (path):
-            self.path = path
-
         f = open(self.path, mode = 'w')
         for line in self.lines:
             if line.startswith('export PATH='):
@@ -129,7 +110,7 @@ class runcase(object):
             if j > -1:
                 line = line[0:j]
 
-            args = separate_args(line.rstrip())
+            args = separate_args(line.strip())
             if args.count('run') == 1:
                 if args.index('run') == 1: # "<package_name> run"
                     for name in ('code_saturne', 'neptune_cfd'):
@@ -145,11 +126,15 @@ class runcase(object):
                             self.cmd_name = name
                             self.run_cmd_line_id = i
                             return
+                        elif args[0] == name:
+                            self.cmd_name = name
+                            self.run_cmd_line_id = i
+                            return
 
         # We should have exited before reaching this.
 
-        err_str = "Error: unable to determine the name of the script for " + self.path
-        raise ValueError(err_str)
+        err_str = "Error: unable to determine the name of the script for " + self.path + os.linesep
+        sys.stderr.write(err_str)
 
     #---------------------------------------------------------------------------
 
@@ -162,100 +147,75 @@ class runcase(object):
 
     #---------------------------------------------------------------------------
 
-    def set_run_args(self, args):
-        """
-        Set the run command and arguments from a list
-        """
-
-        self.lines[self.run_cmd_line_id] = assemble_args(args)
-
-    #---------------------------------------------------------------------------
-
-    def build_template(self, package=None, study_name=None, case_name=None,
-                       ignore_batch=False):
+    def build_template(self, job_header=None,
+                       prologue=None, epilogue=None):
         """
         Build batch file template
         """
 
         import os, stat
-        from cs_exec_environment import append_shell_shebang, \
-            append_script_comment, prepend_path_command
+        from code_saturne.cs_exec_environment import append_shell_shebang, \
+            append_script_comment
 
-        if not package:
-            import cs_package
-            package = cs_package.package()
+        if not self.package:
+            from code_saturne import cs_package
+            self.package = cs_package.package()
+
+        # Use alternate wrapper if configured
+
+        wrapper_postfix = None
+
+        config = configparser.ConfigParser()
+        config.read(self.package.get_configfiles())
+        if config.has_option('install', 'wrapper_postfix'):
+            wrapper_postfix = config.get('install', 'wrapper_postfix')
 
         self.lines = []
 
         append_shell_shebang(self.lines)
 
-        # Add batch system info if necessary
+        # Add batch system info and user prologue if necessary
 
-        batch_template = None
-        config = configparser.ConfigParser()
-        config.read(package.get_configfiles())
-
-        if not ignore_batch and config.has_option('install', 'batch'):
-
-            batch_template = config.get('install', 'batch')
-
-            if not os.path.isabs(batch_template):
-                batch_template = os.path.join(package.get_batchdir(),
-                                             'batch.' + batch_template)
-
-            fdt = open(batch_template, 'r')
-
-            import re, string
-            kwd1 = re.compile('nameandcase')
-
-            # Determine or build default names if required
-
-            if not case_name or not study_name:
-
-                topdir, scriptdir = os.path.split(os.path.split(self.path)[0])
-                if scriptdir == 'SCRIPTS':
-                    studydir, casedir = os.path.split(topdir)
-                    studydir = os.path.split(studydir)[1]
-                else:
-                    casedir = ''
-                    studydir = scriptdir
-
-                if not case_name:
-                    if casedir:
-                        case_name = casedir
-                    else:
-                        case_name = ''
-                if not study_name:
-                    study_name = studydir
-
-            studycasename = study_name.lower() + case_name.lower()
-
-            # For some systems, names are limited to 15 caracters
-            studycasename = studycasename[:15]
-
-            for line in fdt:
-                line = line.rstrip()
-                line = re.sub(kwd1, studycasename, line)
+        if job_header:
+            for line in job_header.split(os.linesep):
                 self.lines.append(line)
+            self.lines.append('')
 
-            fdt.close()
+        # Ensure switch to script directory
+
+        append_script_comment(self.lines,
+                              'Set working directory:' + os.linesep)
+        self.lines.append('cd ' + enquote_arg(os.path.dirname(self.path)))
+        self.lines.append('')
+
+        if prologue:
+            append_script_comment(self.lines,
+                                  'User prologue:' + os.linesep)
+            for line in prologue.split(os.linesep):
+                self.lines.append(line)
+            self.lines.append('')
 
         # Add command to execute.
 
-        append_script_comment(self.lines, 'Ensure the correct command is found:')
+        append_script_comment(self.lines, 'Run command:' + os.linesep)
 
-        self.lines.append(prepend_path_command('PATH',
-                                               package.get_dir("bindir")))
-        self.lines.append('')
-        append_script_comment(self.lines, 'Run command:\n')
-        # On Linux systems, add a backslash to prevent aliases
-        if sys.platform.startswith('win'):
-            run_cmd = ''
+        if wrapper_postfix == None:
+            exec_path = os.path.join(self.package.get_dir("bindir"), self.package.name)
         else:
-            run_cmd = '\\'
-        run_cmd += package.name + ' run'
+            exec_path = '\\' + self.package.name + wrapper_postfix
+
+        run_cmd = enquote_arg(exec_path) + ' run'
+        self.cmd_name = self.package.name
         self.run_cmd_line_id = len(self.lines)
         self.lines.append(run_cmd)
+        self.lines.append('')
+
+        if epilogue:
+            append_script_comment(self.lines,
+                                  'User epilogue:' + os.linesep)
+            for line in epilogue.split(os.linesep):
+                self.lines.append(line)
+            self.lines.append('')
 
         self.save()
 
@@ -278,50 +238,6 @@ class runcase(object):
 
     #---------------------------------------------------------------------------
 
-    def set_compute_build(self, parameters):
-        """
-        Set the compute-build option in the run command
-        """
-
-        line = self.lines[self.run_cmd_line_id]
-
-        args = update_command_single_value(separate_args(line),
-                                           ('--compute-build',
-                                            '--compute-build='),
-                                           enquote_arg(parameters))
-
-        self.lines[self.run_cmd_line_id] = assemble_args(args)
-
-    #---------------------------------------------------------------------------
-
-    def get_coupling(self):
-        """
-        Get the coupling option in the run command
-        """
-
-        args = separate_args(self.lines[self.run_cmd_line_id])
-
-        return get_command_single_value(args,
-                                        ('--coupling',
-                                         '--coupling='))
-
-    #---------------------------------------------------------------------------
-
-    def set_coupling(self, coupling):
-        """
-        Set the coupling option in the run command
-        """
-
-        line = self.lines[self.run_cmd_line_id]
-
-        args = update_command_single_value(separate_args(line),
-                                           ('--coupling',),
-                                           enquote_arg(coupling))
-
-        self.lines[self.run_cmd_line_id] = assemble_args(args)
-
-    #---------------------------------------------------------------------------
-
     def get_parameters(self):
         """
         Get the parameters option in the run command
@@ -334,21 +250,6 @@ class runcase(object):
 
     #---------------------------------------------------------------------------
 
-    def set_parameters(self, parameters):
-        """
-        Set the parameters option in the run command
-        """
-
-        line = self.lines[self.run_cmd_line_id]
-
-        args = update_command_single_value(separate_args(line),
-                                           ('--param', '--param=', '-p'),
-                                           enquote_arg(parameters))
-
-        self.lines[self.run_cmd_line_id] = assemble_args(args)
-
-    #---------------------------------------------------------------------------
-
     def get_nprocs(self):
         """
         Get the nprocs option in the run command
@@ -361,21 +262,6 @@ class runcase(object):
 
     #---------------------------------------------------------------------------
 
-    def set_nprocs(self, parameters):
-        """
-        Set the nprocs option in the run command
-        """
-
-        line = self.lines[self.run_cmd_line_id]
-
-        args = update_command_single_value(separate_args(line),
-                                           ('--nprocs', '--nprocs=', '-n'),
-                                           enquote_arg(parameters))
-
-        self.lines[self.run_cmd_line_id] = assemble_args(args)
-
-    #---------------------------------------------------------------------------
-
     def get_nthreads(self):
         """
         Get the nthreads option in the run command
@@ -390,23 +276,6 @@ class runcase(object):
 
     #---------------------------------------------------------------------------
 
-    def set_nthreads(self, parameters):
-        """
-        Set the nthreads option in the run command
-        """
-
-        line = self.lines[self.run_cmd_line_id]
-
-        args = update_command_single_value(separate_args(line),
-                                           ('--threads-per-task',
-                                            '--threads-per-task=',
-                                            '-nt'),
-                                           enquote_arg(parameters))
-
-        self.lines[self.run_cmd_line_id] = assemble_args(args)
-
-    #---------------------------------------------------------------------------
-
     def get_run_id(self):
         """
         Get the run id, id_prefix, and id_suffix options in the run command
@@ -424,28 +293,108 @@ class runcase(object):
 
     #---------------------------------------------------------------------------
 
-    def set_run_id(self, run_id=None, run_id_prefix=None, run_id_suffix=None):
+    def get_run_stage(self, stage):
         """
-        Set the run id, id_prefix, and id_suffix options in the run command
+        Return True if a given stage is specified in the run command,
+        False otherwise
         """
 
-        line = self.lines[self.run_cmd_line_id]
+        args = separate_args(self.lines[self.run_cmd_line_id])
 
-        args = separate_args(line)
-        if run_id != None:
-            args = update_command_single_value(args,
-                                               ('--id', '--id='),
-                                               enquote_arg(run_id))
-        if run_id_prefix != None:
-            args = update_command_single_value(args,
-                                               ('--id-prefix', '--id-prefix='),
-                                               enquote_arg(run_id_prefix))
-        if run_id_suffix != None:
-            args = update_command_single_value(args,
-                                               ('--id-suffix', '--id-suffix='),
-                                               enquote_arg(run_id_suffix))
+        s_arg = '--' + stage
+        if s_arg in args:
+            return True
 
-        self.lines[self.run_cmd_line_id] = assemble_args(args)
+        return False
+
+    #---------------------------------------------------------------------------
+
+    def run_conf_sections(self,
+                          resource_name='job_defaults',
+                          batch_template=None):
+        """
+        Build "run_conf" sections from an existing runcase.
+        """
+
+        sections = {}
+
+        param = self.get_parameters()
+
+        setup_dict = {}
+        if param != 'setup.xml':
+            setup_dict['param'] = param
+        if len(setup_dict) > 0:
+            sections['setup'] = setup_dict
+
+        run_dict = {}
+        compute_build = self.get_compute_build()
+        if compute_build:
+            run_dict['compute_build'] = compute_build
+        run_id, run_id_prefix, run_id_suffix = self.get_run_id()
+        if run_id:
+            run_dict['id'] = run_id
+        if run_id_prefix:
+            run_dict['id_prefix'] = run_id_prefix
+        if run_id_suffix:
+            run_dict['id_suffix'] = run_id_suffix
+        stage_map = {'stage': 'stage',
+                     'initialize': 'preprocess',
+                     'execute': 'compute',
+                     'finalize': 'finalize'}
+        for stage in stage_map.keys():
+            if self.get_run_stage(stage):
+                run_dict[stage_map[stage]] = True
+        if len(run_dict) > 0:
+            sections['run'] = run_dict
+
+        resource_dict = {}
+        nprocs = self.get_nprocs()
+        if nprocs:
+            resource_dict['n_procs'] = nprocs
+        nthreads = self.get_nthreads()
+        if nthreads:
+            resource_dict['n_threads'] = nthreads
+
+        # Handle batch info
+        if batch_template:
+            from code_saturne import cs_batch
+            batch_src = cs_batch.batch(self.package)
+            batch_src.parse_lines(self.lines)
+
+            topdir, scriptdir = os.path.split(os.path.split(self.path)[0])
+            if scriptdir == 'SCRIPTS':
+                studydir, casedir = os.path.split(topdir)
+                studydir = os.path.split(studydir)[1]
+            else:
+                casedir = ''
+                studydir = scriptdir
+            job_name = studydir.lower() + casedir.lower()
+
+            dst_header = cs_batch.generate_header(batch_template=batch_template,
+                                                  job_name=job_name,
+                                                  package=self.package)
+            batch_dst = cs_batch.batch(self.package)
+            batch_dst.parse_lines(dst_header)
+            for k in batch_src.params.keys():
+                if batch_src.params[k] != None:
+                    batch_dst.params[k] = batch_src.params[k]
+            batch_dst.update_lines(dst_header)
+
+            br = os.linesep
+
+            job_header = ''
+            for i, l in enumerate(dst_header):
+                if i == 0:
+                    job_header = l
+                else:
+                    job_header += br + l
+
+            resource_dict['job_header'] = job_header
+
+        if len(resource_dict) > 0:
+            sections[resource_name] = resource_dict
+
+        return sections
 
 #-------------------------------------------------------------------------------
 # End
