@@ -1,10 +1,10 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 #-------------------------------------------------------------------------------
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -40,15 +40,20 @@ class master_script:
                          'bdump':self.bdump,
                          'compile':self.compile,
                          'config':self.config,
+                         'cplgui':self.gui,
                          'create':self.create,
                          'gui':self.gui,
                          'studymanagergui':self.studymanager_gui,
                          'smgrgui':self.studymanager_gui,
                          'trackcvg':self.trackcvg,
                          'info':self.info,
+                         'parametric':self.parametric,
                          'run':self.run,
                          'salome':self.salome,
-                         'submit':self.submit}
+                         'submit':self.submit,
+                         'symbol2line':self.symbol2line,
+                         'update':self.update,
+                         'up':self.update}
 
         if package != None:
             sys.path.insert(1, package.get_dir('pythondir'))
@@ -106,15 +111,19 @@ Topics:
   bdump
   compile
   config
+  cplgui
   create
   gui
+  parametric
   studymanagergui
   smgrgui
   trackcvg
+  update
+  up
   info
   run
-  salome
   submit
+  symbol2line
 
 Options:
   -h, --help  show this help message and exit"""
@@ -122,57 +131,78 @@ Options:
         print(usage % {'prog':sys.argv[0]})
 
     def studymanager(self, options = None):
-        import cs_studymanager
+        from code_saturne import cs_studymanager
         return cs_studymanager.main(options, self.package)
 
     def bdiff(self, options = None):
-        import cs_bdiff
+        from code_saturne import cs_bdiff
         return cs_bdiff.main(options, self.package)
 
     def bdump(self, options = None):
-        import cs_bdump
+        from code_saturne import cs_bdump
         return cs_bdump.main(options, self.package)
 
     def compile(self, options = None):
-        import cs_compile
+        from code_saturne import cs_compile
         return cs_compile.main(options, self.package)
 
     def config(self, options = None):
-        import cs_config
+        from code_saturne import cs_config
         return cs_config.main(options, self.package)
 
     def create(self, options = None):
-        import cs_create
+        from code_saturne import cs_create
         return cs_create.main(options, self.package)
 
     def gui(self, options = None):
-        import cs_gui
+        if self.package.config.features["gui"] == "no":
+            raise Exception("This code_saturne build does not include the GUI.")
+        from code_saturne import cs_gui
         return cs_gui.main(options, self.package)
 
     def studymanager_gui(self, options = None):
-        import cs_studymanager_gui
+        from code_saturne import cs_studymanager_gui
         return cs_studymanager_gui.main(options, self.package)
 
     def trackcvg(self, options = None):
-        import cs_trackcvg
+        from code_saturne import cs_trackcvg
         return cs_trackcvg.main(options, self.package)
 
     def info(self, options = None):
-        import cs_info
+        from code_saturne import cs_info
         return cs_info.main(options, self.package)
 
+    def parametric(self, options = None):
+        from code_saturne import cs_parametric_study
+        return cs_parametric_study.main(options, self.package)
+
     def run(self, options = None):
-        import cs_run
+        from code_saturne import cs_run
         return cs_run.main(options, self.package)
 
     def salome(self, options = None):
-        import cs_salome
-        return cs_salome.main(options, self.package)
+        salome_cfd = \
+            """%(prog)s salome
+
+Command has been replaced by:
+  salome_cfd
+command"""
+
+        print(salome_cfd % {'prog':sys.argv[0]})
+        return 1
 
     def submit(self, options = None):
-        import cs_submit
+        from code_saturne import cs_submit
         return cs_submit.main(options, self.package)
 
+    def update(self, options = None):
+        from code_saturne import cs_update
+        return cs_update.main(options, self.package)
+
+    def symbol2line(self, options = None):
+        from code_saturne import cs_debug_symbol
+        return cs_debug_symbol.main(options, self.package)
+
 #-------------------------------------------------------------------------------
 # End
 #-------------------------------------------------------------------------------
