@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -41,13 +41,15 @@ if not hasattr(sys, 'version_info') or sys.version_info <= (2, 4, 0, 'final'):
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-# to use PyQt API 2
-#if sys.version_info[0] == 2:
-#    import sip
-#    sip.setapi('QString', 2)
-#    sip.setapi('QVariant', 2)
-#    sip.setapi('QTime', 2)
-#    sip.setapi('QUrl', 2)
+# Force PyQt API 2 for PyQt4
+
+try:
+    import sip
+    for api in ('QDate', 'QDateTime', 'QString', 'QTextStream',
+                'QTime', 'QUrl', 'QVariant'):
+        sip.setapi(api, 2)
+except Exception:
+    pass
 
 try:
     from code_saturne.Base.QtCore    import *
@@ -72,7 +74,7 @@ if list(map(int, PYQT_VERSION_STR.split("."))) < [4, 3, 0]:
 # Application modules
 #-------------------------------------------------------------------------------
 
-import cs_config
+from code_saturne import cs_config
 
 #-------------------------------------------------------------------------------
 # Processes the passed command line arguments
@@ -113,18 +115,12 @@ def main(argv, pkg):
     Start Qt and a session of the application.
     """
 
-    from cs_exec_environment import set_modules, source_rcfile
+    from code_saturne.cs_exec_environment import set_modules, source_rcfile
     set_modules(pkg)
     source_rcfile(pkg)
 
     # Test the package name to know which modules have to be imported
-    if pkg.name == 'code_saturne':
-        images_path = os.path.join(pkg.get_dir('pkgdatadir'), 'images')
-        sys.path.insert(1, os.path.join(pkg.get_dir('pkgpythondir'), 'Base'))
-    else:
-        images_path = os.path.join(pkg.get_dir('pkgpythondir'), 'core', 'icons')
-        sys.path.insert(1, os.path.join(pkg.get_dir('pkgpythondir'), 'core'))
-        sys.path.insert(1, pkg.get_dir('pythondir'))
+    images_path = os.path.join(pkg.get_dir('pkgdatadir'), 'images')
 
     case = process_cmd_line(argv)
 
