@@ -1,10 +1,10 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 #-------------------------------------------------------------------------------
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -106,9 +106,9 @@ class Parser:
     #---------------------------------------------------------------------------
 
     def __init__(self,
-                 fileName,
-                 root_str = None,
-                 version_str = None):
+                 filename = 'setup.xml',
+                 doc = None,
+                 version_str = '2.0'):
 
         self.dict = {}
         self.dict['mesh_dir'] = None
@@ -116,26 +116,34 @@ class Parser:
 
         self.root = None
 
-        if fileName == None:
-            return
+        if doc != None:
+            self.doc = doc
 
-        if not os.path.isfile(fileName):
-            raise XMLError('XML file: ' + fileName + ' not found')
+        else:
+            if fileName == None:
+                return
 
-        try:
-            self.doc = minidom.parse(fileName)
-        except Exception:
-            raise XMLError('Error parsing XML file: ' + fileName)
+            if not os.path.isfile(fileName):
+                raise XMLError('XML file: ' + fileName + ' not found')
+
+            try:
+                self.doc = minidom.parse(fileName)
+            except Exception:
+                raise XMLError('Error parsing XML file: ' + fileName)
 
         self.root = self.doc.documentElement
 
-        if root_str != None:
-            if root_str != self.root.tagName:
-                errStr = '%s\n' \
-                    +'type: "%s", but "%s" expected.'
-                raise XMLError(errStr % (fileName,
-                                         self.root.tagName,
-                                         root_str))
+        root_names = ('Code_Saturne_GUI',
+                      'NEPTUNE_CFD_GUI')
+
+        if self.root.tagName not in root_names:
+            errStr = '%s\n' \
+                     +'type: "%s", but one of "%s" expected.'
+            raise XMLError(errStr % (fileName,
+                                     self.root.tagName,
+                                     str(root_names)))
+
+        self.dict['xml_root_name'] = self.root.tagName
 
         if version_str != None:
             version = self.root.getAttribute('version')
@@ -210,6 +218,12 @@ class Parser:
         if sol_domain_node == None:
             return
 
+        # Check whether additionnal preprocessing is done upn restart
+
+        val = getDataFromNode(sol_domain_node, 'preprocess_on_restart')
+        if val in ('yes', 'on'):
+            self.dict['preprocess_on_restart'] = True
+
         # Get mesh_input if available; in this case, no mesh
         # import will be necessary, so we are done.
 
@@ -292,13 +306,24 @@ class Parser:
         if not calc_node:
             return
 
-        node = getChildNode(calc_node, 'start_restart')
-        if node != None:
-            node = getChildNode(node, 'restart')
-        if node != None:
-            path = str(node.getAttribute('path'))
-            if path:
-                self.dict['restart_input'] = path
+        is_restart = False
+
+        sr_node = getChildNode(calc_node, 'start_restart')
+
+        if sr_node != None:
+            node = getChildNode(sr_node, 'restart')
+            if node != None:
+                path = str(node.getAttribute('path'))
+                if path:
+                    self.dict['restart_input'] = path
+                    is_restart = True
+
+        if sr_node != None:
+            node = getChildNode(sr_node, 'restart_mesh')
+            if node != None:
+                path = str(node.getAttribute('path'))
+                if path:
+                    self.dict['restart_mesh_input'] = path
 
         node = getChildNode(calc_node, 'partitioning')
         if node != None:
@@ -322,12 +347,10 @@ class Parser:
         if log_node != None:
             attr = str(log_node.getAttribute('main'))
             if attr == 'stdout':
-                logging_args += '--log 0 '
+                logging_args += '--trace '
             attr = str(log_node.getAttribute('parallel'))
-            if attr == 'stdout':
-                logging_args += '--logp 0'
-            elif attr == 'listing':
-                logging_args += '--logp 1'
+            if attr == 'listing':
+                logging_args += '--logp'
         logging_args.strip()
 
         if logging_args:
