@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -72,6 +72,7 @@ violet    = (0.5803, 0.    , 0.8275)
 turquoise = (0.    , 0.8078, 0.8196)
 brown     = (0.5451, 0.2706, 0.0745)
 red       = (1.    , 0.    , 0.    )
+black     = (0.    , 0.    , 0.    )
 
 #-------------------------------------------------------------------------------
 # log config
@@ -109,7 +110,7 @@ class Plot(object):
         self.f = open(file, 'r')
 
         # Read mandatory attributes
-        self.subplots = [int(s) for s in parser.getAttribute(node,"fig").split()]
+        self.subplots = [int(s) for s in parser.getAttribute(node,"spids").split()]
         ycol         = int(parser.getAttribute(node, "ycol"))
 
         # Read optional attributes
@@ -118,11 +119,11 @@ class Plot(object):
         self.fmt = parser.getAttribute(node, "fmt", "")
         xcol  = int(parser.getAttribute(node,   "xcol",  0))
         xplus = float(parser.getAttribute(node, "xplus", 0))
-        xfois = float(parser.getAttribute(node, "xfois", 1))
+        xscale = float(parser.getAttribute(node, "xscale", 1))
         yplus = float(parser.getAttribute(node, "yplus", 0))
-        yfois = float(parser.getAttribute(node, "yfois", 1))
+        yscale = float(parser.getAttribute(node, "yscale", 1))
 
-        self.uploadData(xcol, ycol, xplus, xfois, yplus, yfois)
+        self.uploadData(xcol, ycol, xplus, xscale, yplus, yscale)
 
         try:
             xerr = [int(s) for s in parser.getAttribute(node, "xerr").split()]
@@ -157,8 +158,8 @@ class Plot(object):
 
         # List of additional matplotlib commands
         for k, v in parser.getAttributes(node).items():
-            if k not in ('fig', 'fmt', 'legend', 'xcol', 'ycol', \
-                         'xplus', 'yplus', 'xfois', 'yfois', \
+            if k not in ('spids', 'fmt', 'legend', 'xcol', 'ycol', \
+                         'xplus', 'yplus', 'xscale', 'yscale', \
                          'xerr', 'yerr', 'xerrp', 'yerrp', 'id'):
                 self.cmd.append("plt.setp(lines, " + k + "=" + v + ")")
 
@@ -178,7 +179,7 @@ class Plot(object):
 
     #---------------------------------------------------------------------------
 
-    def uploadData(self, xcol, ycol, xplus, xfois, yplus, yfois):
+    def uploadData(self, xcol, ycol, xplus, xscale, yplus, yscale):
         """
         Upload and parse data
         """
@@ -190,12 +191,21 @@ class Plot(object):
                 j += 1
                 line = line.replace(", ", " ") # compatibility with CSV
                 line = line.lstrip()
+
+                # for CSV files, try to detect a header to skip it
+                if j == 1:
+                    try:
+                        val = float(line.split()[0])
+                    # if it can not be converted to float
+                    except ValueError as not_float:
+                        continue
+
                 if xcol:
-                    self.xspan.append(float(line.split()[xcol-1])*xfois + xplus)
+                    self.xspan.append(float(line.split()[xcol-1])*xscale + xplus)
                 else:
                     self.xspan.append(j)
 
-                self.yspan.append(float(line.split()[ycol-1])*yfois + yplus)
+                self.yspan.append(float(line.split()[ycol-1])*yscale + yplus)
 
     #---------------------------------------------------------------------------
 
@@ -214,19 +224,38 @@ class Plot(object):
 
             if len(errorbar) == 2:
                 error = [ [], [] ]
+                j = 0
                 for line in self.f.readlines():
-                    line = line.replace(", ", " ") # compatibility with CSV
                     line = line.lstrip()
                     if line and line[0] != '#':
+                        j += 1
+                        line = line.replace(", ", " ") # compatibility with CSV
+                        line = line.lstrip()
+                        # for CSV files, try to detect a header to skip it
+                        if j == 1:
+                            try:
+                                val = float(line.split()[0])
+                            except ValueError as not_float:
+                                continue
                         error[0].append(float(line.split()[errorbar[0]-1]))
                         error[1].append(float(line.split()[errorbar[1]-1]))
                 return error
 
             elif len(errorbar) == 1:
                 error = []
+                j = 0
                 for line in self.f.readlines():
                     line = line.lstrip()
                     if line and line[0] != '#':
+                        j += 1
+                        line = line.replace(", ", " ") # compatibility with CSV
+                        line = line.lstrip()
+                        # for CSV files, try to detect a header to skip it
+                        if j == 1:
+                            try:
+                                val = float(line.split()[0])
+                            except ValueError as not_float:
+                                continue
                         error.append(float(line.split()[errorbar[0]-1]))
                 return error
         elif errorp:
@@ -236,10 +265,19 @@ class Plot(object):
                 sys.exit(1)
             else:
                 error = []
+                j = 0
                 for line in self.f.readlines():
-                    line = line.replace(", ", " ") # compatibility with CSV
                     line = line.lstrip()
                     if line and line[0] != '#':
+                        j += 1
+                        line = line.replace(", ", " ") # compatibility with CSV
+                        line = line.lstrip()
+                        # for CSV files, try to detect a header to skip it
+                        if j == 1:
+                            try:
+                                val = float(line.split()[0])
+                            except ValueError as not_float:
+                                continue
                         error.append(errorp/100.*float(line.split()[col-1]))
                 return error
 
@@ -270,9 +308,23 @@ class Probes(object):
 
         f = open(file_name, 'r')
 
+        j = 0
+
         for line in f.readlines():
             line = line.lstrip()
             if line and line[0] != '#':
+                j += 1
+                line = line.replace(", ", " ") # compatibility with CSV
+                line = line.lstrip()
+
+                # for CSV files, try to detect a header to skip it
+                if j == 1:
+                    try:
+                        val = float(line.split()[0])
+                    # if it can not be converted to float
+                    except ValueError as not_float:
+                        continue
+
                 self.xspan.append(float(line.split()[xcol - 1]))
                 self.yspan.append(float(line.split()[ycol - 1]))
 
@@ -363,7 +415,7 @@ class Figure(object):
             elif k == "dpi":
                 self.dpi = int(v)
 
-        # Store te list of subplot objects associated to the current figure
+        # Store the list of subplot objects associated to the current figure
         self.subplots = []
         for id in [int(s) for s in node.attributes["idlist"].value.split()]:
             for p in subplots:
@@ -521,11 +573,12 @@ class Plotter(object):
                                      case.label, case.resu,
                                      dest, "monitoring", file_name)
                     if not os.path.isfile(f):
-                        raise ValueError("\n\nThis file does not exist: %s\n (call with path: %s)\n" % (file_name, f))
+                        print("\n\nThis file does not exist: %s\n (last call with path: %s)\n" % (file_name, f))
 
-                    for ycol in range(2, self.__number_of_column(f) + 1):
-                        curve = Probes(f, fig, ycol)
-                        self.curves.append(curve)
+                    else:
+                        for ycol in range(2, self.__number_of_column(f) + 1):
+                            curve = Probes(f, fig, ycol)
+                            self.curves.append(curve)
 
         # Read the files of results of cases
         for case in study_object.cases:
@@ -560,12 +613,13 @@ class Plotter(object):
                                 break
 
                     if not iok:
-                        raise ValueError("\n\nThis file does not exist: %s\n (last call with path: %s)\n" % (file_name, f))
+                        print("\n\nThis file does not exist: %s\n (last call with path: %s)\n" % (file_name, f))
 
-                    for nn in plots:
-                        curve = Plot(nn, self.parser, f)
-                        curve.setMeasurement(False)
-                        self.curves.append(curve)
+                    else:
+                        for nn in plots:
+                            curve = Plot(nn, self.parser, f)
+                            curve.setMeasurement(False)
+                            self.curves.append(curve)
 
         # Read the files of results of postpro
         script, label, nodes, args = self.parser.getPostPro(study_label)
@@ -733,12 +787,14 @@ class Plotter(object):
         log.debug("plot_figure --> layout: nbcol: %s nbrow: %s " % (nbcol, nbrow))
         plt.subplots_adjust(hspace=hs, wspace=ws, right=ri, left=le, bottom=0.2)
 
+        subplots = {}
+
         # draw curves in the right subplot
         for p in figure.subplots:
             idx = figure.subplots.index(p)
             log.debug("plot_figure --> plot draw: id = %s" % p.id)
             ax = plt.subplot(nbrow, nbcol, idx + 1)
-
+            subplots[idx] = ax
             for curve in p.curves:
                 self.__draw_curve(ax, curve, p)
 
@@ -747,7 +803,7 @@ class Plotter(object):
 
         for p in figure.subplots:
             idx = figure.subplots.index(p)
-            ax = plt.subplot(nbrow, nbcol, idx + 1)
+            ax = subplots[idx]
 
             if p.title:
                 ax.set_title(p.title)
@@ -764,7 +820,7 @@ class Plotter(object):
         # additional matplotlib raw commands for subplot
         for p in figure.subplots:
             idx = figure.subplots.index(p)
-            ax = plt.subplot(nbrow, nbcol, idx + 1)
+            ax = subplots[idx]
             for cmd in p.cmd:
                 try:
                     exec(cmd)
