@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -121,8 +121,11 @@ class Parser(object):
                 data = current.firstChild.data
             else:
                 data = None
+        elif len(l) == 0:
+            print("Error: in getDataFromNode no markup %s found." %  childName)
+            sys.exit(1)
         else:
-            print("Error: in getDataFromNode several markup %s found." %  childName)
+            print("Error: in getDataFromNode several markups %s found." %  childName)
             sys.exit(1)
 
         return data
@@ -320,13 +323,15 @@ class Parser(object):
         """
         data = []
 
+        setup_filter_keys = ("notebook", "parametric", "kw_args")
+
         for node in self.getStudyNode(l).getElementsByTagName("case"):
             if str(node.attributes["status"].value) == 'on':
                 d = {}
                 d['node']    = node
                 d['label']   = str(node.attributes["label"].value)
                 d['compute'] = str(node.attributes["compute"].value)
-                d['post'] = str(node.attributes["post"].value)
+                d['post']    = str(node.attributes["post"].value)
 
                 try:
                     d['compare'] = str(node.attributes["compare"].value)
@@ -349,10 +354,37 @@ class Parser(object):
                 except:
                     d['tags'] = None
 
+                try:
+                    d['depends'] = str(node.attributes["depends"].value)
+                except:
+                    d['depends'] = None
+
+                try:
+                    d['n_iter'] = str(node.attributes["n_iter"].value)
+                except:
+                    d['n_iter'] = None
+
+                try:
+                    d['estim_wtime'] = str(node.attributes["estim_wtime"].value)
+                except:
+                    d['estim_wtime'] = None
+
+                for k in setup_filter_keys:
+                    d[k] = None
+
                 for n in node.childNodes:
-                    if n.nodeType == minidom.Node.ELEMENT_NODE and n.childNodes:
-                        if n.tagName not in ("compare", "prepro", "script", "data"):
-                            d[n.tagName] = n.childNodes[0].data
+                    if n.nodeType == minidom.Node.ELEMENT_NODE:
+                        if n.childNodes:
+                            if n.tagName not in ("compare", "prepro", "script", "data", "depends"):
+                                d[n.tagName] = n.childNodes[0].data
+                        else:
+                            if n.tagName in setup_filter_keys:
+                                args = n.getAttribute("args")
+                                if not d[n.tagName]:
+                                    d[n.tagName] = str(args)
+                                else:
+                                    d[n.tagName] += " " + str(args)
+
                 data.append(d)
 
         return data
