@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 #-------------------------------------------------------------------------------
@@ -243,7 +243,7 @@ class compile_install(cs_compile):
             # Strangely, on MinGW, Windows paths are not correctly handled here
             # So, assuming we always build on MinGW, here is a little trick!
             if sys.platform.startswith("win"):
-                if pkg.get_cross_compile() != 'cygwin': #mingw32 or mingw64
+                if pkg.get_cross_compile() != 'cygwin': # mingw64
                     libdir = os.path.normpath('C:\\MinGW\\msys\\1.0' + libdir)
             if self.destdir:
                 libdir = dest_subdir(self.destdir, libdir)
@@ -292,7 +292,7 @@ class compile_install(cs_compile):
             # handled here. So, assuming we always build on MinGW,
             # here is a little trick!
             if sys.platform.startswith("win"):
-                if pkg.get_cross_compile() != 'cygwin': #mingw32 or mingw64
+                if pkg.get_cross_compile() != 'cygwin': # mingw64
                     libdir = os.path.normpath('C:\\MinGW\\msys\\1.0' + libdir)
             if self.destdir:
                 libdir = dest_subdir(self.destdir, libdir)
@@ -315,6 +315,9 @@ class compile_install(cs_compile):
                 and (not pkg.config.libs[lib].dynamic_load)):
                 cmd_line += separate_args(pkg.config.libs[lib].flags[flag])
 
+        print(cmd_line)
+        print(self.pkg.config.rpath)
+
 #===============================================================================
 # Functions
 #===============================================================================
@@ -329,10 +332,11 @@ def install_exec_name(pkg, exec_name, destdir=None):
     # Strangely, on MinGW, Windows paths are not correctly handled here...
     # So, assuming we always build on MinGW, here is a little trick!
     if sys.platform.startswith("win"):
-        if pkg.get_cross_compile() != 'cygwin': #mingw32 or mingw64
+        if pkg.get_cross_compile() != 'cygwin': # mingw64
             exec_name = os.path.normpath('C:\\MinGW\\msys\\1.0' + exec_name)
         else:
-            exec_name = pkg.dirs['pkglibexecdir'][1] + "/" + pkg.solver
+            exec_name = os.path.join(pkg.dirs['pkglibexecdir'][1],
+                                     os.path.basename(exec_name))
     if destdir:
         exec_name = dest_subdir(destdir, exec_name)
     dirname = os.path.dirname(exec_name)
@@ -349,7 +353,10 @@ if __name__ == '__main__':
 
     from cs_package import package
 
-    pkg = package()
+    # Note: some subtle Python issue requires reloading the package here
+    # (incomplete config loaded otherwise on 2021 Debian sid with PETSc)
+
+    pkg = package(reload_config=True)
 
     # Check mode and options
     options, src_files = process_cmd_line(sys.argv[1:], pkg)
@@ -362,7 +369,9 @@ if __name__ == '__main__':
 
     exec_name=options.out_file
     if not exec_name:
-        exec_name = pkg.solver
+        exec_name = 'cs_solver'
+        if os.path.basename(sys.argv[0]) == 'neptune_cfd':
+            exec_name = 'nc_solver'
 
     if options.mode == 'install':
         c = compile_install(pkg, src_dir, destdir=options.dest_dir)
