@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 #-------------------------------------------------------------------------------
@@ -59,25 +59,25 @@ def version_from_news(srcdir):
     release = ''
     extra = ''
 
-    trunk = -1
+    master = -1
     version = [-1]
     release = [-1, '', '', '']
 
-    f = open(os.path.join(srcdir, "NEWS"))
+    f = open(os.path.join(srcdir, "NEWS.md"))
     lines = f.readlines()
     f.close()
 
     for i in range(1, len(lines)):
 
-        # Release names are underlined by '==='
+        # Release names are underlined by '---'
 
-        if lines[i][0:7] == '=======':
+        if lines[i][0:7] == '-------':
 
             j = i-1
             l = lines[j]
 
-            if l[0:6] == 'Trunk ' and trunk < 0:
-                trunk = j
+            if l[0:7] == 'Master ' and master < 0:
+                master = j
 
             # If release appears, at which line, and is there a date,
             # or is it not yet released ?
@@ -95,9 +95,9 @@ def version_from_news(srcdir):
                             release = [j] + n.split('.')
                             break
 
-    if trunk > -1:
+    if master > -1:
         if version[0] > -1:
-            major, minor = trunk_major_minor(version[1], version[2])
+            major, minor = master_major_minor(version[1], version[2])
             release = ''
             extra = '-alpha'
 
@@ -112,9 +112,9 @@ def version_from_news(srcdir):
 
 #-------------------------------------------------------------------------------
 
-def trunk_major_minor(prev_major, prev_minor):
+def master_major_minor(prev_major, prev_minor):
     """
-    For trunk, determine next version based on previous branches
+    For master, determine next version based on previous branches
     """
 
     # Current release cycle plans for 4 minor releases per major release
@@ -184,18 +184,19 @@ def git_version(srcdir, defaults):
     # Universal newlines options may fail in some cases with
     # Python 3, so check for newlines using '\n' and '\\n' later.
 
-    p = subprocess.Popen(cmd,
-                         stdout=subprocess.PIPE,
-                         stderr=subprocess.PIPE,
-                         universal_newlines=False)
-    output = p.communicate()
-    if p.returncode != 0:
-        return major, minor, release, extra
-    else:
-        commit_rev = output[0]
-        if sys.version[0] == '3':
-            commit_rev = commit_rev.decode('utf-8')
-        revision += '-' + commit_rev
+    try:
+        p = subprocess.Popen(cmd,
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE,
+                             universal_newlines=False)
+        output = p.communicate()
+        if p.returncode == 0:
+            commit_rev = output[0]
+            if sys.version[0] == '3':
+                commit_rev = commit_rev.decode('utf-8')
+            revision += '-' + commit_rev
+    except Exception:
+        pass
 
     return major, minor, release, extra, revision
 
@@ -211,20 +212,23 @@ def git_version_is_modified(srcdir):
 
     cmd = ['git', '--no-pager', '--git-dir='+os.path.join(srcdir, '.git'),
            '--work-tree='+srcdir, 'status', '-s', '-uno']
-    p = subprocess.Popen(cmd,
-                         stdout=subprocess.PIPE,
-                         stderr=subprocess.PIPE,
-                         universal_newlines=True)
-    output = p.communicate()
-    o0 = str(output[0])
-    if p.returncode == 0:
-        changes = o0.split('\n')
-        n_changes = len(changes) - 1
-        for c in changes:
-            if c[-3:] == '.po':
-                n_changes -= 1
-        if n_changes > 0:
-            return True
+    try:
+        p = subprocess.Popen(cmd,
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE,
+                             universal_newlines=True)
+        output = p.communicate()
+        o0 = str(output[0])
+        if p.returncode == 0:
+            changes = o0.split('\n')
+            n_changes = len(changes) - 1
+            for c in changes:
+                if c[-3:] == '.po':
+                    n_changes -= 1
+            if n_changes > 0:
+                return True
+    except Exception:
+        pass
 
     return False
 
@@ -245,6 +249,63 @@ def build_version_string(major, minor, release, extra, revision = None):
     version = version.replace('\r', '')
     return version
 
+#-------------------------------------------------------------------------------
+
+def replace_in_file(src, dest, major, minor, release, extra, revision, modified):
+    """
+    Replace expressions in file based on version numbers.
+    """
+
+    f = open(src, 'r')
+    lines = f.readlines()
+    f.close()
+
+    cs_version = build_version_string(major, minor, release, extra)
+    cs_version_full = build_version_string(major, minor, release, extra,
+                                           revision)
+    cs_version_short = build_version_string(major, minor, '', extra)
+
+    if modified:
+        cs_version_full += '-m'
+    cs_revision = revision[1:]
+    if modified:
+        revision += '-m'
+
+    for i, l in enumerate(lines):
+        l = l.replace("@cs_version@", cs_version)
+        l = l.replace("@cs_version_full@", cs_version_full)
+        l = l.replace("@cs_version_short@", cs_version_short)
+        l = l.replace("@cs_revision@", cs_revision)
+        lines[i] = l
+
+    if not dest:
+        for l in lines:
+            sys.stdout.write(l)
+
+    else:
+        changed = True
+
+        cmp_lines = []
+        if os.path.isfile(dest):
+            f = open(dest, 'r')
+            cmp_lines = f.readlines()
+            f.close()
+            changed = False
+
+        if len(cmp_lines) != len(lines):
+            changed = True
+        else:
+            for i, l in enumerate(lines):
+                if l != cmp_lines[i]:
+                    changed = True
+                    break
+
+        if changed:
+            f = open(dest, 'w')
+            for l in lines:
+                f.write(l)
+            f.close()
+
 #-------------------------------------------------------------------------------
 # Main
 #-------------------------------------------------------------------------------
@@ -257,17 +318,22 @@ if __name__ == '__main__':
 
     # Default command: return simple string based on local information
 
-    if len(sys.argv) == 1:
+    n_args = len(sys.argv) - 1
+
+    if n_args == 0:
         major, minor, release, extra = version_from_news(srcdir)
         sys.stdout.write(build_version_string(major, minor, release, extra))
 
     elif sys.argv[1] == '--short':
         major, minor, release, extra = version_from_news(srcdir)
+        if not extra in ['-alpha', '-beta']:
+            extra = ''
         sys.stdout.write(build_version_string(major, minor, '', extra))
 
     # Call tools
 
-    elif sys.argv[1] in ['--full', '--revision-only', '--verbose']:
+    elif sys.argv[1] in ['--full', '--revision-only', '--verbose',
+                         '--replace']:
 
         defaults = version_from_news(srcdir)
         release = ''
@@ -323,6 +389,13 @@ if __name__ == '__main__':
             print('extra:    ' + extra)
             print('revision: ' + revision[1:])
             print('modified: ' + str(modified))
+        elif sys.argv[1] == '--replace' and n_args in (2, 3):
+            src = sys.argv[2]
+            dest = None
+            if n_args == 3:
+                dest = sys.argv[3]
+            replace_in_file(src, dest, major, minor, release,
+                            extra, revision, modified)
 
     else:
         usage = \
@@ -331,10 +404,13 @@ if __name__ == '__main__':
 Default:              print version string
 
 Options:
-  --full              print full version string with available version control info
-  --short             print short version string with major, minor, and extras
-  --revision-only     print version string with version control info only
-  --verbose           print fields on different lines
+  --full                      print full version string with available
+                              version control info
+  --short                     print short version string with major, minor,
+                              and extras
+  --revision-only             print version string with version control info only
+  --update <template> [path]  build or rebuild output file from template
+  --verbose                   print fields on different lines
 
 Options:
   -h, --help  show this help message and exit"""
