@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -25,7 +25,7 @@
 /*-----------------------------------------------------------------------------*/
 
 /*!
-  \page f_parameters Input of calculation parameters (Fortran modules)
+  \page f_parameters Input of calculation parameters (Fortran subroutines in cs_user_parameters.f90)
 
   \section cs_f_user_parameters_h_intro Introduction
 
@@ -68,84 +68,6 @@
 
   \snippet cs_user_parameters.f90 usipsu
 
-  \subsection cs_f_user_parameters_h_usipsu_1 Special fields
-
-  Enforce existence of 'tplus' and 'tstar' fields, so that
-  a Nusselt number may be computed using the
-  \ref post_boundary_nusselt subroutine.
-  When postprocessing this quantity is activated, those fields
-  are present, but if we need to compute them in the
-  \ref cs_user_extra_operations user subroutine without postprocessing them,
-  forcing the definition of these fields to save the values computed
-  for the boundary layer is necessary.
-
-  \snippet cs_user_parameters-output.f90 usipsu_ex_1
-
-  Save contribution of slope test for variables in special fields.
-  These fields are automatically created, with postprocessing output enabled,
-  if the matching variable is convected, does not use a pure upwind scheme,
-  and has a slope test (the slope_test_upwind_id key value for a given
-  variable's field is automatically set to the matching postprocessing field's
-  id, or -1 if not applicable).
-
-  \snippet cs_user_parameters-output.f90 usipsu_ex_2
-
-  \section cs_f_user_parameters_h_usipes  Input-output related examples (usipes)
-
-  \subsection cs_f_user_parameters_h_example_base Basic options
-
-  Frequency of log output.
-
-  \snippet cs_user_parameters-output.f90 usipes_ex_01
-
-  Log (listing) verbosity.
-
-  \snippet cs_user_parameters-output.f90 usipes_ex_02
-
-  Activate or deactivate logging output.
-  By default, logging is active for most variables. In the following
-  example, logging for velocity is deactivated.
-
-  \snippet cs_user_parameters-output.f90 usipes_ex_03
-
-  Change a property's label (here for density, first checking if it
-  is variable). A field's name cannot be changed, but its label,
-  used for logging and postprocessing output, may be redefined.
-
-  \snippet cs_user_parameters-output.f90 usipes_ex_04
-
-  \subsection cs_f_user_parameters_h_example_structures Structures output
-
-  Structures output step.
-
-  \snippet cs_user_parameters-output.f90 usipes_ex_05
-
-  \subsection cs_f_user_parameters_h_example_post Postprocessing output
-
-  Activate or deactivate postprocessing output.
-  By default, output is active for most variables. In the following
-  example, the output for velocity is deactivated.
-
-  \snippet cs_user_parameters-output.f90 usipes_ex_07
-
-  Activate or deactivate probes output.
-
-  \snippet cs_user_parameters-output.f90 usipes_ex_08
-
-  Probes for Radiative Transfer (Luminance and radiative density flux vector).
-
-  \snippet cs_user_parameters-output.f90 usipes_ex_10
-
-  \subsection cs_f_user_parameters_h_example_1 Postprocess at boundary
-
-   Force postprocessing of projection of some variables at boundary
-   with no reconstruction.
-   This is handled automatically if the second bit of a field's
-   'post_vis' key value is set to 1 (which amounts to adding 2
-   to that key value).
-
-  \snippet cs_user_parameters-output.f90 usipes_ex_09
-
   \section cs_f_user_parameters_h_usppmo Specific physic activation (usppmo)
 
   The \ref usppmo routine can be found in the \ref cs_user_parameters.f90 file.
