@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -37,117 +37,14 @@
   \section cs_user_extra_operations_examples_cs_user_extra_op_examples Extra operations examples
   Here is the list of examples dedicated to different physics:
 
-  - \subpage cs_user_extra_operations_examples_energy_balance_p
+  - \subpage cs_user_extra_operations_examples_vorticity_field_p
   - \subpage cs_user_extra_operations_examples_balance_by_zone_p
+  - \subpage cs_user_extra_operations_examples_scalar_balance_p
+  - \subpage cs_user_extra_operations_examples_energy_balance_p
   - \subpage cs_user_extra_operations_examples_force_temperature_p
-  - \subpage cs_user_extra_operations_examples_global_efforts_p
+  - \subpage cs_user_extra_operations_examples_boundary_forces_p
   - \subpage cs_user_extra_operations_examples_parallel_operations_p
-
-*/
-// __________________________________________________________________________________
-/*!
-
-  \page cs_user_extra_operations_examples_energy_balance_p Energy balance
-
-  \section cs_user_extra_operations_examples_energy_balance Energy balance
-
-  \subsection cs_user_extra_operations_examples_loc_var_eb Local variables to be added
-
-  The following local variables need to be defined for the examples
-  in this section:
-
-  \snippet cs_user_extra_operations-energy_balance.f90 loc_var_dec
-
-  \subsection cs_user_extra_operations_examples_init Initialization and finalization
-
-  The following initialization block needs to be added for the following examples:
-
-  \snippet cs_user_extra_operations-energy_balance.f90 init
-
-  Ad the end of the subroutine, it is recommended to deallocate the work array:
-
-  \snippet cs_user_extra_operations-energy_balance.f90 finalize
-
-  In theory Fortran 95 deallocates locally-allocated arrays automatically,
-  but deallocating arrays in a symmetric manner to their allocation is good
-  practice, and it avoids using a different logic for C and Fortran.
-
-  \subsection cs_user_extra_operations_examples_eb_body Body
-
-  This example computes energy balance relative to temperature
-  We assume that we want to compute balances  (convective and diffusive)
-  at the boundaries of the calculation domain represented below
-  (with boundaries marked by colors).
-
-  The scalar considered if the temperature. We will also use the
-  specific heat (to obtain balances in Joules)
-
-
-  Domain and associated boundary colors:
-  - 2, 4, 7 : adiabatic walls
-  - 6       : wall with fixed temperature
-  - 3       : inlet
-  - 5       : outlet
-  - 1       : symmetry
-
-
-  To ensure calculations have physical meaning, it is best to use
-  a spatially uniform time step (\ref optcal::idtvar "idtvar" = 0 or 1).
-  In addition, when restarting a calculation, the balance is
-  incorrect if \ref optcal::inpdt0 "inpdt0"= 1 (visct not initialized
-  and t(n-1) not known).
-
-
-  Temperature variable
-  - ivar = \ref isca(\ref optcal::iscalt "iscalt")
-
-  Boundary coefficients coefap/coefbp are those of
-  \ref numvar::ivarfl "ivarfl"(ivar).
-
-
-  The balance at time step n is equal to:
-
-  \f[
-  \begin{array}{r c l}
-  Balance^n &=& \displaystyle
-               \sum_{\celli=1}^{\ncell}
-                  \norm{\vol{\celli}} C_p \rho_\celli
-                  \left(T_\celli^{n-1} -T_\celli^n \right)  \\
-           &+& \displaystyle
-               \sum_{\fib}
-                  C_p \Delta t_\celli \norm{\vect{S}_\ib}
-                  \left(A_\ib^f + B_\ib^f T_\celli^n \right) \\
-           &+& \displaystyle
-               \sum_{\fib}
-                  C_p \Delta t_\celli \dot{m}_\ib
-                  \left(A_\ib^g + B_\ib^g T_\celli^n \right)
-  \end{array}
-  \f]
-
-  The first term is negative if the amount of energy in the volume
-  has increased (it is 0 in a steady regime).
-
-  The other terms (convection, diffusion) are positive if the amount
-  of energy in the volume has increased due to boundary conditions.
-
-  In a steady regime, a positive balance thus indicates an energy gain.
-
-
-  With \f$ \rho \f$ (\c rom) calculated using the density law from the
-  \ref usphyv subroutine, for example:
-
-  \f[
-  \rho^{n-1}_\celli = P_0 / \left( R T_\celli^{n-1} + T_0 \right)
-  \f]
-  where \f$ R\f$ is \c cs_physical_constants_r and \f$ T_0 \f$ is \c tkelv.
-
-
-  \f$ C_p \f$ and \f$ \lambda/C_p \f$ may vary.
-
-
-  Here is the corresponding code:
-
-  \snippet cs_user_extra_operations-energy_balance.f90 example_1
+  - \subpage cs_user_extra_operations_examples_stopping_criterion_p
 
 */
 // __________________________________________________________________________________
@@ -234,7 +131,7 @@
   of coordinates).
 
   The box criterion can be used as follows:
-  box[\f$x_{min}\f$, \f$y_{min}\f$, \f$z_{min}\f$, \f$x_{max}\f$, \f$y_{max}\f$, \f$z_{max}\f$].
+  box[\f$ x_{min}\f$, \f$ y_{min}\f$, \f$ z_{min}\f$, \f$ x_{max}\f$, \f$ y_{max}\f$, \f$ z_{max}\f$].
 
   Here is the corresponding code:
 
@@ -274,6 +171,38 @@
   components of the pressure drop computation:
   \snippet cs_user_extra_operations-balance_by_zone.c example_6
 
+*/
+// __________________________________________________________________________________
+/*!
+
+  \page cs_user_extra_operations_examples_scalar_balance_p Scalar balance on full domain
+
+  \section cs_user_extra_operations_examples_scalar_balance Scalar balance on full domain
+
+  This is an example of \ref cs_user_extra_operations which performs a scalar
+  balance on the full computational domain. It is possible to customize the output to extract
+  the contribution of some boundary zones of interest.
+
+  Define local variables
+
+  \snippet cs_user_extra_operations-scalar_balance.c local_variables
+
+  Get the physical fields
+
+  \snippet cs_user_extra_operations-scalar_balance.c local_variables
+
+  Initialization step
+
+  \snippet cs_user_extra_operations-scalar_balance.c init
+
+  Computation step
+
+  \snippet cs_user_extra_operations-scalar_balance.c computation
+
+  Write the balance at time step n
+
+  \snippet cs_user_extra_operations-scalar_balance.c computation
+
 */
 // __________________________________________________________________________________
 /*!
@@ -303,22 +232,62 @@
 // __________________________________________________________________________________
 /*!
 
-  \page cs_user_extra_operations_examples_global_efforts_p Global efforts
+  \page cs_user_extra_operations_examples_vorticity_field_p Compute vorticity field values
 
-  \section cs_user_extra_operations_examples_global_efforts Global efforts
+  \section cs_user_extra_operations_examples_vorticity_field Compute vorticity field values
 
-  This is an example of \ref cs_user_extra_operations which computes global efforts
+  This is an example of \ref cs_user_extra_operations
+  which computes the vorticity field values over the whole domain.
 
-  \subsection cs_user_extra_operations_examples_loc_var_geff Local variables to be added
+  First number of cells in the current sub-domain (or the whole domain for a sequential calculation)
+  is retrieved. The number of cells with ghosts (i.e. including halo cells) is retrieved first, then
+  the number of standard cells. The array that will host the velocity gradient values is finally
+  declared, as a 3x3 real matrix array per cell.
 
-  \snippet cs_user_extra_operations-global_efforts.f90 loc_var_dec
+  \snippet cs_user_extra_operations-vorticity_field.c vorticity_d
 
-  \subsection cs_user_extra_operations_examples_geff_body Body
+  The array hosting the gradient values has to be allocated consistantly with his type.
 
-  Example: compute global efforts on a subset of faces.
-  If boundary stresses have been calculated correctly:
+  \snippet cs_user_extra_operations-vorticity_field.c vorticity_a
 
-  \snippet cs_user_extra_operations-global_efforts.f90 example_1
+  Then the gradient of the velocity is computed. This is done as follows, by calling the appropriate
+  field operator:
+
+  \snippet cs_user_extra_operations-vorticity_field.c vorticity_g
+
+  The vorticity field has to be retrieved as follows below. Note that if it doesn't exist the pointer
+  will be set to NULL (this is the behavior of the "_try" variant of the field accesser).
+  The vorticity field can have been added through the GUI (menu postprocessing > additional user
+  arrays) or in \ref cs_user_model.
+
+  \snippet cs_user_extra_operations-vorticity_field.c vorticity_f
+
+  Finally the vorticity values are computed in each standard cell if the field "vorticity" has been
+  well retrieved previously only. Notice the way the gradient values are accessed.
+
+  \snippet cs_user_extra_operations-vorticity_field.c vorticity_cv
+
+  The array holding the gradient values has to be deallocated at the end.
+
+  \snippet cs_user_extra_operations-vorticity_field.c vorticity_da
+
+*/
+// __________________________________________________________________________________
+/*!
+
+  \page cs_user_extra_operations_examples_boundary_forces_p Boundary forces
+
+  \section cs_user_extra_operations_examples_boundary_forces Boundary forces
+
+  This is an example of \ref cs_user_extra_operations which computes boundary forces
+
+  Example 1: compute total forces on a boundary zone (subset of boundary faces).
+
+  \snippet cs_user_extra_operations-boundary_forces.c boundary_forces_ex1
+
+  Example 2: compute pressure forces on a boundary zone (subset of boundary faces).
+
+  \snippet cs_user_extra_operations-boundary_forces.c boundary_forces_ex2
 
 */
 // __________________________________________________________________________________
@@ -442,3 +411,37 @@
   \snippet cs_user_extra_operations-parallel_operations.f90 example_15
 
 */
+// __________________________________________________________________________________
+/*!
+
+  \page cs_user_extra_operations_examples_stopping_criterion_p Stopping criterion based on L2 time residuals
+
+  \section cs_user_extra_operations_examples_stopping_criterion Stopping criterion based on L2 time residuals
+
+  This is an example of \ref cs_user_extra_operations allowing to properly stop a computation
+  when the L2 time residuals (displayed in the run_solver.log file) of all
+  solved variables have decreased below a value of 1e-3.
+
+  L2 time residuals of a variable at a given time step are a relative measure of
+  the unsteady term of its transport equation:
+
+  \f[
+  \sqrt{\int_\Omega \left| \der{\varia}{t} \right|^2 \dd \Omega / \int_\Omega \left| \varia \right|^2 \dd \Omega}
+  \f]
+
+  \snippet cs_user_extra_operations-stopping_criterion.c extra_stopping_criterion
+
+*/
+// __________________________________________________________________________________
+/*!
+
+  \page cs_user_extra_operations_examples_verif_cdo_diff Postprocessing of results obtained with CDO schemes for a scalar transport equation.
+
+  \section cs_user_extra_operations_examples_verif_cdo_diff Postprocessing of results obtained with CDO schemes for a scalar transport equation.
+
+  This is an example of \ref cs_user_extra_operations allowing to add operations on results produced by CDO schemes. It allows to define advanced
+  postprocessing.
+
+  \snippet cs_user_extra_operations-verif_cdo_diffusion.c extra_verif_cdo_diff
+
+*/
