@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -197,7 +197,7 @@
 !> The values of the 3 \c rcodcl components are:
 !>  - \c rcodcl(ifac, ivar, 1):
 !>     - Dirichlet for the variable          if \c icodcl(ifac, ivar) = 1 or 13
-!>     - Wall value (sliding velocity, temp) if \c icodcl(ifac, ivar) = 5
+!>     - Wall value (sliding velocity, temp) if \c icodcl(ifac, ivar) =  5
 !>     .
 !>     The dimension of \c rcodcl(ifac, ivar, 1) is that of the
 !>     resolved variable, for instance:
@@ -330,7 +330,7 @@
 !>
 !> Cell value field ids
 !>
-!> - Density:                        \c irom
+!> - Density:                        \c icrom
 !> - Dynamic molecular viscosity:    \c iviscl
 !> - Turbulent viscosity:            \c ivisct
 !> - Specific heat:                  \c icp
@@ -447,10 +447,10 @@ double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
-integer ilelt, ifac, nlelt
-
 integer, allocatable, dimension(:) :: lstelt
 
+integer          ielt, nlelt, ifac
+
 !===============================================================================
 
 
@@ -472,13 +472,13 @@ allocate(lstelt(nfabor))  ! temporary array for boundary faces selection
 call getfbr('1',nlelt,lstelt)
 
 if (ttcabs.lt.3.8d0) then
-  do ilelt = 1, nlelt
-    ifac = lstelt(ilelt)
+  do ielt = 1, nlelt
+    ifac = lstelt(ielt)
     rcodcl(ifac,isca(2),1) = 20.d0 + 100.d0*ttcabs
   enddo
 else
-  do ilelt = 1, nlelt
-    ifac = lstelt(ilelt)
+  do ielt = 1, nlelt
+    ifac = lstelt(ielt)
     rcodcl(ifac,isca(2),1) = 400.d0
   enddo
 endif
