@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -57,8 +57,7 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Toolbox import GuiParam
-from code_saturne.Base.Common import LABEL_LENGTH_MAX
+from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam
 
 #-------------------------------------------------------------------------------
 # log config
@@ -81,6 +80,97 @@ else:
     TEXT_TYPES = (str,)
 
 #==============================================================================
+# Classes used to handle ComboBox subsections (groups)
+#==============================================================================
+
+# Qt Role used to differentiate whether a ComboBox entry is a group child
+# or not
+GroupRole = Qt.UserRole
+
+
+class GroupDelegate(QStyledItemDelegate):
+    """
+    Class used to create a group delegate which applies a tab to a combobox
+    entry if it is a group child. Only applied to the visual style!
+    """
+    def initStyleOption(self, option, index):
+        super(GroupDelegate, self).initStyleOption(option, index)
+        if not index.data(GroupRole):
+            option.text = "   " + option.text
+
+class GroupItem():
+    """
+    GroupItem class.
+    Handles the subsections in a ComboModel.
+    """
+
+    def __init__(self, name, index):
+        """
+        Init method. Inputs are:
+        name  : String with the name of the group
+        index : Position of the group name in the combobox list of elements
+        """
+        self._name  = name
+        self._index = index
+
+        self._children = []
+        self._number_of_children = 0
+
+
+    def addChild(self, name):
+        """
+        Method to add a child with the given name.
+        Returns the index of the child to be used if the 'insertRow' method.
+        """
+        self._children.append(name)
+        self._number_of_children += 1
+
+        child_index = self._index + self._number_of_children
+
+        return child_index
+
+    def getIndex(self):
+        """
+        Method which returns the index of the group
+        """
+        return self._index
+
+    def getChildIndex(self, childName):
+        """
+        Method to retrieve the index of a given child
+        """
+        if childName not in self._children:
+            msg = "%s is not part of group %s\n" % (childName, self._name)
+            raise Exception(msg)
+
+        child_index = self._children.index(childName) + self._index + 1
+        return child_index
+
+
+    def getNumberOfChildren(self):
+        """
+        Method which returns the number of children in the group
+        """
+        return self._number_of_children
+
+
+    def incrementIndex(self):
+
+        self._index += 1
+
+    def generateItem(self):
+
+        item  = QStandardItem(str(self._name))
+        item.setData(True, GroupRole)
+
+        font = item.font()
+        font.setBold(True)
+        font.setItalic(True)
+        item.setFont(font)
+        item.setFlags(item.flags() & ~Qt.ItemIsSelectable)
+
+        return item
+#==============================================================================
 # Strings
 #==============================================================================
 
@@ -112,66 +202,38 @@ def to_text_string(obj, encoding=None):
 
 #==============================================================================
 # QVariant conversion utilities
+#
+# Note: to_qvariant was removed recently; from_qvariant may be removed in
+#       many_places and the "to_text_string" variants could be replaced
+#       by "to_str"; where the value is already known to be of the correct
+#       type, or None handled in the associated code,
+#       this could even be ignored
 #==============================================================================
-PYQT_API_1 = False
+
 import collections
 
 if os.environ.get('QT_API', 'pyqt') == 'pyqt':
-    import sip
-    if QT_API == "PYQT4":
-        try:
-            PYQT_API_1 = sip.getapi('QVariant') == 1
-        except AttributeError:
-            PYQT_API_1 = True
-
-    def to_qvariant(pyobj=None):
-        """Convert Python object to QVariant
-        This is a transitional function from PyQt API #1 (QVariant exist)
-        to PyQt API #2 and Pyside (QVariant does not exist)"""
-        if PYQT_API_1:
-            from code_saturne.Base.QtCore import QVariant
-            return QVariant(pyobj)
-        else:
-            return pyobj
 
     def from_qvariant(qobj=None, convfunc=None):
         """Convert QVariant object to Python object
         This is a transitional function from PyQt API #1 (QVariant exists)
         to PyQt API #2 and Pyside (QVariant does not exist)"""
-        if PYQT_API_1:
-            assert isinstance(convfunc, collections.Callable)
+        if (qobj != None):
             if convfunc in TEXT_TYPES or convfunc is to_text_string:
-                return convfunc(qobj.toString())
-            elif convfunc is bool:
-                return qobj.toBool()
+                return str(qobj)
             elif convfunc is int:
-                return qobj.toInt()[0]
+                return int(qobj)
             elif convfunc is float:
-                return qobj.toDouble()[0]
-            else:
-                return convfunc(qobj)
-        else:
-            if (qobj != None):
-                if convfunc in TEXT_TYPES or convfunc is to_text_string:
-                    return str(qobj)
-                elif convfunc is int:
-                    return int(qobj)
-                elif convfunc is float:
-                    try:
-                        return float(qobj)
-                    except Exception:
-                        return locale.atof(qobj)
-                else:
-                    return qobj
+                try:
+                    return float(qobj)
+                except Exception:
+                    return locale.atof(qobj)
             else:
                 return qobj
+        else:
+            return qobj
 
 else:
-    def to_qvariant(obj=None):
-        """Convert Python object to QVariant
-        This is a transitional function from PyQt API#1 (QVariant exist)
-        to PyQt API#2 and Pyside (QVariant does not exist)"""
-        return obj
 
     def from_qvariant(qobj=None, pytype=None):
         """Convert QVariant object to Python object
@@ -184,6 +246,12 @@ def qbytearray_to_str(qba):
     return str(bytes(qba.toHex().data()).decode())
 
 
+def to_str(s):
+    """Convert argument to string, using an empty string for None"""
+    if s is None:
+        return ''
+    return str(s)
+
 #==============================================================================
 # Wrappers around QFileDialog static methods
 #==============================================================================
@@ -343,11 +411,48 @@ class ComboModel:
         self.dicoV2M = {}
         self.dicoM2V = {}
 
+        self.item_groups = {}
+
         self.items   = []
         self.combo.setModel(self.model)
+        self.combo.setItemDelegate(GroupDelegate(self.combo))
 
 
-    def addItem(self, str_view, str_model="", warn=False):
+    def addItemGroup(self, group_name):
+
+        if group_name in self.item_groups.keys():
+            return
+
+        index = self.last
+        self.item_groups[group_name] = GroupItem(group_name, index)
+
+        item = self.item_groups[group_name].generateItem()
+        self.model.setItem(index, item)
+
+        self.items.append(group_name)
+
+        self.last += 1
+
+    def addItemList(self, list_of_views_and_models, warn=False, groupName=None):
+        """
+        Insert of a list of items in the model.
+        list_of_views_and_models is a list of 2-element list
+          e.g. [ ["View1", "Model1"], ["View2", "Model2"] ]
+        """
+        for view, model in list_of_views_and_models:
+            self.addItem(view, model, warn, groupName)
+
+    def hasItem(self, index=None, str_view="", str_model=""):
+        if index is not None:
+            return index < self.last
+
+        elif str_model:
+            return (str_model in self.items)
+
+        elif str_view:
+            return (str_view in self.dicoV2M.keys())
+
+    def addItem(self, str_view, str_model="", warn=False, groupName=None):
         """
         Insert an item in the model.
 
@@ -359,17 +464,39 @@ class ComboModel:
 
         warn: If True, entry is marked with a color.
         """
-        item  = QStandardItem(str(str_view))
+        if self.hasItem(str_view=str_view):
+            return
 
-        index = self.last
-        self.model.setItem(index, item)
+        item = QStandardItem(str(str_view))
+        item.setData(True, GroupRole)
+
+        if groupName in self.item_groups.keys():
+            # Insert the child after the group name in the ComboModel
+            item.setData(False, GroupRole)
+            index = self.item_groups[groupName].addChild(str(str_view))
+            self.model.setItem(index, item)
+
+            # In case groups were created before, update the index of
+            # the following groups in the list
+            for key in self.item_groups.keys():
+                gpe = self.item_groups[key]
+                if gpe.getIndex() >= index:
+                    gpe.incrementIndex()
+
+                    idx = gpe.getIndex()
+                    it  = gpe.generateItem()
+                    self.model.setItem(idx, it)
+
+        else:
+            index = self.last
+            self.model.setItem(index, item)
 
         if warn:
             self.combo.setItemData(index,
                                    QColor(Qt.red),
                                    Qt.TextColorRole)
 
-        self.last = index + 1
+        self.last += 1
 
         if not str_model: str_model = str_view
 
@@ -395,8 +522,14 @@ class ComboModel:
             del self.dicoV2M[str_view]
 
             self.dicoM2V[new_str_model] = new_str_view
-            self.dicoV2M[new_str_view]  = new_str_model
+            self.dicoV2M[new_str_view] = new_str_model
 
+    def flushItems(self):
+        """
+        Remove all items
+        """
+        while self.last > 0:
+            self.delItem(0)
 
     def delItem(self, index=None, str_model="", str_view=""):
         """
@@ -484,14 +617,35 @@ class ComboModel:
             self.combo.setCurrentIndex(index)
 
         elif str_model:
-            index = self.items.index(str_model)
-            self.combo.setCurrentIndex(index)
+            try:
+                index = self.items.index(str_model)
+                self.combo.setCurrentIndex(index)
+            except Exception:
+                self._displayWarning(str_model)
+                # Throw signals to ensure XML is updated (not very elegant)
+                self.combo.activated[str].emit(self.dicoM2V[self.items[0]])
+                self.combo.currentTextChanged[str].emit(self.dicoM2V[self.items[0]])
+                self.combo.currentIndexChanged[int].emit(0)
+                index = 0
 
         elif str_view:
-            str_model = self.dicoV2M[str_view]
-            index = self.items.index(str_model)
-            self.combo.setCurrentIndex(index)
+            try:
+                str_model = self.dicoV2M[str_view]
+                index = self.items.index(str_model)
+                self.combo.setCurrentIndex(index)
+            except Exception:
+                self._displayWarning(str_model)
+                index = 0
 
+    def _displayWarning(self, str_model):
+        if self.combo.accessibleName() != "":
+            name = self.combo.accessibleName()
+        else:
+            name = self.combo.objectName()
+        title = "Warning in " + name
+        msg = str_model + " is not in list: " + str(self.items) + "\n"
+        msg += "Value reset to " + str(self.items[0]) + " but should be checked.\n"
+        QMessageBox.warning(self.combo, title, msg)
 
     def enableItem(self, index=None, str_model="", str_view=""):
         """
@@ -669,10 +823,7 @@ class IntValidator(QIntValidator):
 
         self.state = state
 
-        if PYQT_API_1:
-            return (state, pos)
-        else:
-            return (state, stri, pos)
+        return (state, stri, pos)
 
 
     def tr(self, text):
@@ -799,10 +950,7 @@ class DoubleValidator(QDoubleValidator):
 
         self.state = state
 
-        if PYQT_API_1:
-            return (state, pos)
-        else:
-            return (state, stri, pos)
+        return (state, stri, pos)
 
 
     def tr(self, text):
@@ -815,13 +963,14 @@ class RegExpValidator(QRegExpValidator):
     """
     Validator for regular expression.
     """
-    def __init__(self, parent, rx):
+    def __init__(self, parent, rx, forbidden_labels=None):
         """
         Initialization for validator
         """
         QRegExpValidator.__init__(self, parent)
         self.parent = parent
         self.state = QRegExpValidator.Invalid
+        self.forbidden = forbidden_labels
 
         self.__validator = QRegExpValidator(rx, parent)
 
@@ -840,6 +989,10 @@ class RegExpValidator(QRegExpValidator):
         """
         state = self.__validator.validate(stri, pos)[0]
 
+        if self.forbidden:
+            if stri in self.forbidden:
+                state = QValidator.Intermediate
+
         palette = self.parent.palette()
 
         if state == QValidator.Intermediate:
@@ -851,10 +1004,7 @@ class RegExpValidator(QRegExpValidator):
 
         self.state = state
 
-        if PYQT_API_1:
-            return (state, pos)
-        else:
-            return (state, stri, pos)
+        return (state, stri, pos)
 
 
     def tr(self, text):
@@ -948,5 +1098,299 @@ class BufferSpinBoxWidget(QSpinBox):
             return QAbstractSpinBox.StepUpEnabled | QAbstractSpinBox.StepDownEnabled
 
 #-------------------------------------------------------------------------------
-# End of QtPage
+# Delegates to use
 #-------------------------------------------------------------------------------
+
+class LabelDelegate(QItemDelegate):
+    """
+    Delegate for lines
+    """
+
+    def __init__(self, parent=None, xml_model=None,
+                 forbidden_labels=None,
+                 accepted_regex=None,
+                 auto_completion=[]):
+        super(LabelDelegate, self).__init__(parent)
+
+        self.parent = parent
+        self.mdl    = xml_model
+
+        self._forbidden_labels = forbidden_labels
+
+        # Regex
+        rx = accepted_regex
+        if rx == None:
+            rx = "[ -~]*"
+#            "[\-_A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX) + "}"
+        self.regExp = QRegExp(rx)
+
+        # Auto completion
+        self._comp_list = auto_completion
+
+    def createEditor(self, parent, option, index):
+
+        editor = QLineEdit(parent)
+
+        v = RegExpValidator(editor, self.regExp, self._forbidden_labels)
+        editor.setValidator(v)
+
+        # Autocompletion if necessary:
+        if len(self._comp_list) > 0:
+            completer = QCompleter()
+            editor.setCompleter(completer)
+            mc = QStringListModel()
+            completer.setModel(mc)
+            mc.setStringList(self._comp_list)
+
+        # return editor
+        return editor
+
+    def setEditorData(self, editor, index):
+
+        editor.setAutoFillBackground(True)
+
+        v = from_qvariant(index.model().data(index, Qt.DisplayRole),
+                          to_text_string)
+        self.p_value = str(v)
+
+        editor.setText(v)
+
+    def setModelData(self, editor, model, index):
+
+        if not editor.isModified():
+            return
+
+        if editor.validator().state == QValidator.Acceptable:
+            p_value = str(editor.text())
+            model.setData(index, p_value, Qt.DisplayRole)
+
+class FloatDelegate(QItemDelegate):
+
+    def __init__(self, parent=None, xml_model=None,
+                 minVal=-1.e99, maxVal=+1.e99):
+
+        super(FloatDelegate, self).__init__(parent)
+
+        self.parent = parent
+        self.mdl    = xml_model
+
+        self._min   = minVal
+        if type(self._min) != float:
+            self._min = -1.e99
+
+        self._max   = maxVal
+        if type(self._max) != float:
+            self._max = +1.e99
+
+    def createEditor(self, parent, option, index):
+
+        editor = QLineEdit(parent)
+
+        validator = DoubleValidator(editor,
+                                    min=self._min,
+                                    max=self._max)
+
+        editor.setValidator(validator)
+
+        return editor
+
+    def setEditorData(self, editor, index):
+
+        editor.setAutoFillBackground(True)
+
+        value = from_qvariant(index.model().data(index, Qt.DisplayRole),
+                              to_text_string)
+        editor.setText(value)
+
+    def setModelData(self, editor, model, index):
+
+        if editor.validator().state == QValidator.Acceptable:
+            value = from_qvariant(editor.text(), float)
+            selectionModel = self.parent.selectionModel()
+
+            for idx in selectionModel.selectedIndexes():
+                if idx.column() == index.column():
+                    model.setData(idx, value, Qt.DisplayRole)
+
+class IntegerDelegate(QItemDelegate):
+
+    def __init__(self, parent=None, xml_model=None,
+                 minVal=-10000000000, maxVal=+10000000000):
+
+        super(IntegerDelegate, self).__init__(parent)
+
+        self.parent  = parent
+        self.mdl     = xml_model
+
+        self._min = minVal
+        if type(self._min) != int:
+            self._min = -10000000000
+        self._max = maxVal
+        if type(self._max) != int:
+            self._max = +10000000000
+
+
+    def createEditor(self, parent, option, index):
+
+        editor = QLineEdit(parent)
+
+        validator = IntValidator(editor,
+                                 min=self._min,
+                                 max=self._max)
+
+        editor.setValidator(validator)
+
+        return editor
+
+    def setEditorData(self, editor, index):
+
+        editor.setAutoFillBackground(True)
+
+        value = from_qvariant(index.model().data(index, Qt.DisplayRole),
+                              to_text_string)
+        editor.setText(value)
+
+    def setModelData(self, editor, model, index):
+
+        value = from_qvariant(editor.text(), int)
+
+        if editor.validator().state == QValidator.Acceptable:
+            selectionModel = self.parent.selectionModel()
+            for idx in selectionModel.selectedIndexes():
+                if idx.column() == index.column():
+                    model.setData(idx, value, Qt.DisplayRole)
+
+
+class ComboDelegate(QItemDelegate):
+
+    def __init__(self, parent=None, xml_model=None,
+                 opts_list=[], opts_state=None):
+
+        super(ComboDelegate, self).__init__(parent)
+
+        self.parent       = parent
+        self.mdl          = xml_model
+
+        if opts_state:
+            if type(opts_state) != list:
+                raise Exception("Wrong type for opts_state")
+            if len(opts_state) != len(opts_list):
+                raise Exception("Wrong length of opts_state")
+        else:
+            # To simplify the code which follows, we ensure that
+            # opts_state is a list with the correct length
+            opts_state = ["on"]*len(opts_list)
+
+        self.opts_list = []
+        for i, opt in enumerate(opts_list):
+
+            if opts_state[i] not in ["on", "na", "off"]:
+                msg="Wrong state for opts %s : %s" % (opt,opts_state[i])
+                raise Exception(msg)
+
+            ac = True
+            av = True
+            if opts_state[i] == "na":
+                av = False
+            elif opts_state[i] == "off":
+                ac = False
+                av = False
+
+            self.opts_list.append({'name':opt,
+                                   'active':ac,
+                                   'available':av})
+
+
+    def createEditor(self, parent, option, index):
+
+        editor = QComboBox(parent)
+        for opt in self.opts_list:
+            name     = opt['name']
+            isActive = opt['active']
+            isAvail  = opt['available']
+
+            editor.addItem(name)
+            idx = editor.findText(name)
+            editor.model().item(idx).setEnabled(isActive)
+            if not isAvail:
+                editor.setItemData(idx, QColor(Qt.red), Qt.TextColorRole)
+
+        editor.installEventFilter(self)
+
+        return editor
+
+
+    def setEditorData(self, comboBox, index):
+
+        string = from_qvariant(index.model().data(index, Qt.DisplayRole),
+                               to_text_string)
+        comboBox.setEditText(string)
+
+    def setModelData(self, comboBox, model, index):
+
+        value = comboBox.currentText()
+
+        selectionModel = self.parent.selectionModel()
+
+        for idx in selectionModel.selectedIndexes():
+            if idx.column() == index.column():
+                model.setData(idx, value, Qt.DisplayRole)
+
+
+class BasicTableModel(QAbstractTableModel):
+
+    def __init__(self, parent=QModelIndex(), xml_model=None, data=[[]], headers=[], default_row=[]):
+        super(BasicTableModel, self).__init__(parent)
+        self.parent = parent
+        self.xml_model = xml_model
+        self.data_table = data
+        self.headers = headers
+        self.default_row = default_row
+
+    def rowCount(self, parent=QModelIndex()):
+        return len(self.data_table)
+
+    def columnCount(self, parent=QModelIndex()):
+        return len(self.headers)
+
+    def data(self, index, role=Qt.DisplayRole):
+        if not index.isValid():
+            return QVariant()
+        elif role != Qt.DisplayRole:
+            return QVariant()
+        return self.data_table[index.row()][index.column()]
+
+    def headerData(self, section, orientation, role=Qt.DisplayRole):
+        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
+            return self.headers[section]
+        return None
+
+    def flags(self, index):
+        if not index.isValid():
+            return Qt.NoItemFlags
+        else:
+            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
+
+    def setData(self, index, value, role=Qt.EditRole):
+        if (role == Qt.EditRole) and (index.isValid()):
+            self.data_table[index.row()][index.column()] = value
+            self.dataChanged.emit(index, index)
+            return True
+        return QAbstractTableModel.setData(index, value, role)
+
+    def insertRows(self, position, rows=1, index=QModelIndex()):
+        self.beginInsertRows(QModelIndex(), position, position + rows - 1)
+        for row in range(rows):
+            self.data_table.insert(position + row, self.default_row)
+        self.endInsertRows()
+        return True
+
+    def removeRows(self, position, rows=1, index=QModelIndex()):
+        self.beginRemoveRows(QModelIndex(), position, position + rows - 1)
+        del self.data_table[position:position + rows]
+        self.endRemoveRows()
+        return True
+
+# -------------------------------------------------------------------------------
+# End of QtPage
+# -------------------------------------------------------------------------------
