@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -44,11 +44,11 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import GuiParam, GuiLabelManager
 from code_saturne.Base.QtPage import RegExpValidator, IntValidator
-from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
+from code_saturne.Base.QtPage import from_qvariant, to_text_string
+from code_saturne.Base.QtPage import LabelDelegate, IntegerDelegate
 from code_saturne.Pages.BalanceForm import Ui_BalanceForm
-from code_saturne.Pages.BalanceModel import BalanceModel
 from code_saturne.Pages.FacesSelectionView import StandardItemModelFaces
 
 #-------------------------------------------------------------------------------
@@ -60,66 +60,6 @@ log = logging.getLogger("BalanceView")
 log.setLevel(GuiParam.DEBUG)
 
 
-#-------------------------------------------------------------------------------
-# Line edit delegate for selection
-#-------------------------------------------------------------------------------
-
-class LineEditDelegateSelector(QItemDelegate):
-    """
-    Use of a QLineEdit in the table.
-    """
-    def __init__(self, parent=None):
-        QItemDelegate.__init__(self, parent)
-
-
-    def createEditor(self, parent, option, index):
-        editor = QLineEdit(parent)
-        return editor
-
-
-    def setEditorData(self, editor, index):
-        editor.setAutoFillBackground(True)
-        value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
-        editor.setText(value)
-
-
-    def setModelData(self, editor, model, index):
-        value = editor.text()
-        model.setData(index, to_qvariant(value), Qt.DisplayRole)
-
-
-#-------------------------------------------------------------------------------
-# Line edit delegate for index
-#-------------------------------------------------------------------------------
-
-class LineEditDelegateIndex(QItemDelegate):
-    """
-    Use of a QLineEdit in the table.
-    """
-    def __init__(self, parent=None):
-        QItemDelegate.__init__(self, parent)
-
-
-    def createEditor(self, parent, option, index):
-        editor = QLineEdit(parent)
-        validator = IntValidator(editor, min=0)
-        editor.setValidator(validator)
-        editor.installEventFilter(self)
-        return editor
-
-
-    def setEditorData(self, editor, index):
-        editor.setAutoFillBackground(True)
-        value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
-        editor.setText(value)
-
-
-    def setModelData(self, editor, model, index):
-        if editor.validator().state == QValidator.Acceptable:
-            value = from_qvariant(editor.text(), int)
-            model.setData(index, to_qvariant(value), Qt.DisplayRole)
-
-
 #-------------------------------------------------------------------------------
 # Model class
 #-------------------------------------------------------------------------------
@@ -146,18 +86,18 @@ class StandardItemModelPressureDrop(QStandardItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         if role == Qt.ToolTipRole:
-            return to_qvariant(self.tooltip[index.column()])
+            return self.tooltip[index.column()]
 
         if role == Qt.DisplayRole:
             row = index.row()
             col = index.column()
             if index.column() in (0, 1):
-                return to_qvariant(self._data[row][col])
+                return self._data[row][col]
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -171,8 +111,8 @@ class StandardItemModelPressureDrop(QStandardItemModel):
 
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
-            return to_qvariant(self.headers[section])
-        return to_qvariant()
+            return self.headers[section]
+        return None
 
 
     def setData(self, index, value, role):
@@ -246,19 +186,19 @@ class StandardItemModelScalarBalance(QStandardItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         if role == Qt.ToolTipRole:
-            return to_qvariant(self.tooltip[index.column()])
+            return self.tooltip[index.column()]
 
         if role == Qt.DisplayRole:
             row = index.row()
             col = index.column()
 
             if index.column() in (0, 1, 2):
-                return to_qvariant(self._data[row][col])
+                return self._data[row][col]
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -272,8 +212,8 @@ class StandardItemModelScalarBalance(QStandardItemModel):
 
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
-            return to_qvariant(self.headers[section])
-        return to_qvariant()
+            return self.headers[section]
+        return None
 
 
     def setData(self, index, value, role):
@@ -340,7 +280,15 @@ class BalanceView(QWidget, Ui_BalanceForm):
 
         self.case = case
         self.case.undoStopGlobal()
-        self.mdl = BalanceModel(self.case)
+        if self.case.xmlRootNode().tagName == "Code_Saturne_GUI":
+            from code_saturne.model.BalanceModel import BalanceModel
+            self.mdl = BalanceModel(self.case)
+        elif self.case.xmlRootNode().tagName == "NEPTUNE_CFD_GUI":
+            from code_saturne.model.BalanceModelNeptune import BalanceModelNeptune
+            self.mdl = BalanceModelNeptune(self.case)
+
+        # Autocompletion for zone selection
+        _zone_autocompletion = GuiLabelManager().getCompleter("mesh_selection")
 
         # tableView Pressure Drop
         self.pressureModel = StandardItemModelPressureDrop(self.mdl)
@@ -349,10 +297,12 @@ class BalanceView(QWidget, Ui_BalanceForm):
         self.tableViewPressureDrop.setSelectionBehavior(QAbstractItemView.SelectRows)
         self.tableViewPressureDrop.setSelectionMode(QAbstractItemView.SingleSelection)
 
-        delegateIdxPres = LineEditDelegateIndex(self.tableViewPressureDrop)
+        delegateIdxPres = IntegerDelegate(self.tableViewPressureDrop,
+                                          minVal=0)
         self.tableViewPressureDrop.setItemDelegateForColumn(0, delegateIdxPres)
 
-        delegateSelector = LineEditDelegateSelector(self.tableViewPressureDrop)
+        delegateSelector = LabelDelegate(self.tableViewPressureDrop,
+                                         auto_completion=_zone_autocompletion)
         self.tableViewPressureDrop.setItemDelegateForColumn(1, delegateSelector)
 
         self.tableViewPressureDrop.resizeColumnsToContents()
@@ -370,13 +320,15 @@ class BalanceView(QWidget, Ui_BalanceForm):
         self.tableViewScalarBalance.setSelectionBehavior(QAbstractItemView.SelectRows)
         self.tableViewScalarBalance.setSelectionMode(QAbstractItemView.SingleSelection)
 
-        delegateIdxScalar = LineEditDelegateIndex(self.tableViewScalarBalance)
+        delegateIdxScalar = IntegerDelegate(self.tableViewScalarBalance,
+                                            minVal=0)
         self.tableViewScalarBalance.setItemDelegateForColumn(0, delegateIdxScalar)
 
-        delegateVariable = LineEditDelegateSelector(self.tableViewScalarBalance)
+        delegateVariable = LabelDelegate(self.tableViewScalarBalance,
+                                         auto_completion=_zone_autocompletion)
         self.tableViewScalarBalance.setItemDelegateForColumn(1, delegateVariable)
 
-        delegateSelector = LineEditDelegateSelector(self.tableViewScalarBalance)
+        delegateSelector = LabelDelegate(self.tableViewScalarBalance)
         self.tableViewScalarBalance.setItemDelegateForColumn(1, delegateSelector)
 
         self.tableViewScalarBalance.resizeColumnsToContents()
@@ -544,14 +496,6 @@ class BalanceView(QWidget, Ui_BalanceForm):
         self.tableViewScalarBalance.clearSelection()
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
-
-
 #-------------------------------------------------------------------------------
 # Testing part
 #-------------------------------------------------------------------------------
