@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -45,34 +45,38 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Toolbox import GuiParam
-from code_saturne.Base.QtPage  import DoubleValidator, ComboModel, from_qvariant
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import DoubleValidator, IntValidator, ComboModel, from_qvariant
 
 from code_saturne.Pages.BoundaryConditionsScalarsForm import Ui_BoundaryConditionsScalarsForm
-from code_saturne.Pages.LocalizationModel             import LocalizationModel, Zone
-from code_saturne.Pages.DefineUserScalarsModel        import DefineUserScalarsModel
-from code_saturne.Pages.ThermalScalarModel            import ThermalScalarModel
-from code_saturne.Pages.QMeiEditorView                import QMeiEditorView
-from code_saturne.Pages.Boundary                      import Boundary
-from code_saturne.Pages.CompressibleModel             import CompressibleModel
-from code_saturne.Pages.AtmosphericFlowsModel         import AtmosphericFlowsModel
-from code_saturne.Pages.NotebookModel                 import NotebookModel
+from code_saturne.model.LocalizationModel             import LocalizationModel, Zone
+from code_saturne.model.DefineUserScalarsModel        import DefineUserScalarsModel
+from code_saturne.model.ThermalScalarModel            import ThermalScalarModel
+from code_saturne.Pages.QMegEditorView                import QMegEditorView
+from code_saturne.model.Boundary                      import Boundary
+from code_saturne.model.CompressibleModel             import CompressibleModel
+from code_saturne.model.AtmosphericFlowsModel         import AtmosphericFlowsModel
+from code_saturne.model.NotebookModel import NotebookModel
+from code_saturne.model.ConjugateHeatTransferModel import ConjugateHeatTransferModel
 
-#-------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------
 # log config
-#-------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------
 
 logging.basicConfig()
 log = logging.getLogger("BoundaryConditionsScalarsView")
 log.setLevel(GuiParam.DEBUG)
 
-#-------------------------------------------------------------------------------
+
+
+# -------------------------------------------------------------------------------
 # Main class
-#-------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------
 
 class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
     """
     """
+
     def __init__(self, parent):
         """
         Constructor
@@ -83,15 +87,29 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.setupUi(self)
 
 
+    def __get_thermal_type__(self):
+        """
+        Return thermal type and conversion option
+        """
+        thermal_type_s = self.thermal_type.split(':')
+        thermal_type = thermal_type_s[0]
+        convert = None
+        if len(thermal_type_s) > 1:
+            convert = thermal_type_s[1]
+        return thermal_type, convert
+
+
     def setup(self, case):
         """
         Setup the widget
         """
-        self.__case = case
+        self.case = case
         self.__boundary = None
 
-        self.__case.undoStopGlobal()
-        self.notebook = NotebookModel(self.__case)
+        self.case.undoStopGlobal()
+
+        self.notebook = NotebookModel(self.case)
+        self.cht_model = ConjugateHeatTransferModel(self.case)
 
         self.lineEditValueThermal.textChanged[str].connect(self.slotValueThermal)
         self.lineEditValueSpecies.textChanged[str].connect(self.slotValueSpecies)
@@ -110,13 +128,16 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.comboBoxMeteo.activated[str].connect(self.slotMeteoChoice)
         self.comboBoxTypeMeteo.activated[str].connect(self.slotMeteoTypeChoice)
 
+        # Syrthes coupling
+        self.lineEditSyrthesInstance.editingFinished.connect(self.slotChooseSyrthesInstance)
+
         ## Validators
         validatorValueThermal = DoubleValidator(self.lineEditValueThermal)
         validatorValueSpecies = DoubleValidator(self.lineEditValueSpecies)
-        validatorValueMeteo   = DoubleValidator(self.lineEditValueMeteo)
-        validatorExThermal    = DoubleValidator(self.lineEditExThermal)
-        validatorExSpecies    = DoubleValidator(self.lineEditExSpecies)
-        validatorExMeteo      = DoubleValidator(self.lineEditExMeteo)
+        validatorValueMeteo = DoubleValidator(self.lineEditValueMeteo)
+        validatorExThermal = DoubleValidator(self.lineEditExThermal)
+        validatorExSpecies = DoubleValidator(self.lineEditExSpecies)
+        validatorExMeteo = DoubleValidator(self.lineEditExMeteo)
 
         self.lineEditValueThermal.setValidator(validatorValueThermal)
         self.lineEditValueSpecies.setValidator(validatorValueSpecies)
@@ -125,7 +146,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.lineEditExSpecies.setValidator(validatorExSpecies)
         self.lineEditExMeteo.setValidator(validatorExMeteo)
 
-        self.__case.undoStartGlobal()
+        self.case.undoStartGlobal()
 
 
     def __setBoundary(self, boundary):
@@ -134,41 +155,66 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         """
         self.__boundary = boundary
 
-        self.nature  = boundary.getNature()
-        self.therm   = ThermalScalarModel(self.__case)
-        self.sca_mo  = DefineUserScalarsModel(self.__case)
-        self.comp    = CompressibleModel(self.__case)
-        self.atm     = AtmosphericFlowsModel(self.__case)
+        self.nature = boundary.getNature()
+        self.therm = ThermalScalarModel(self.case)
+        self.sca_mo = DefineUserScalarsModel(self.case)
+        self.comp = CompressibleModel(self.case)
+        self.atm = AtmosphericFlowsModel(self.case)
+
+        self.model_th = self.therm.getThermalScalarModel()
 
         self.modelTypeThermal = ComboModel(self.comboBoxTypeThermal, 1, 1)
         self.modelTypeSpecies = ComboModel(self.comboBoxTypeSpecies, 1, 1)
-        self.modelTypeMeteo   = ComboModel(self.comboBoxTypeMeteo, 1, 1)
+        self.modelTypeMeteo = ComboModel(self.comboBoxTypeMeteo, 1, 1)
 
         self.modelTypeThermal.addItem(self.tr("Prescribed value"), 'dirichlet')
-        self.modelTypeSpecies.addItem(self.tr("Prescribed value"), 'dirichlet')
-        self.modelTypeMeteo.addItem(  self.tr("Prescribed value"), 'dirichlet')
+        if self.model_th == "enthalpy":
+            self.modelTypeThermal.addItem(self.tr("Prescribed temperature value"),
+                                          'dirichlet:temperature')
 
-        self.modelTypeThermal.addItem(self.tr("Prescribed value (user law)"), 'dirichlet_formula')
-        self.modelTypeSpecies.addItem(self.tr("Prescribed value (user law)"), 'dirichlet_formula')
-        self.modelTypeMeteo.addItem(  self.tr("Prescribed value (user law)"), 'dirichlet_formula')
+        self.modelTypeSpecies.addItem(self.tr("Prescribed value"), 'dirichlet')
+        self.modelTypeMeteo.addItem(self.tr("Prescribed value"), 'dirichlet')
+
+        self.modelTypeThermal.addItem(self.tr("Prescribed value (user law)"),
+                                      'dirichlet_formula')
+        if self.model_th == "enthalpy":
+            self.modelTypeThermal.addItem(self.tr("Prescribed temperature value (user law)"),
+                                          'dirichlet_formula:temperature')
+        self.modelTypeSpecies.addItem(self.tr("Prescribed value (user law)"),
+                                      'dirichlet_formula')
+        self.modelTypeMeteo.addItem(self.tr("Prescribed value (user law)"),
+                                    'dirichlet_formula')
 
         if self.nature == 'outlet':
             self.modelTypeThermal.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
             self.modelTypeSpecies.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
             self.modelTypeMeteo.addItem(  self.tr("Prescribed (outgoing) flux"), 'neumann')
         elif self.nature == 'wall':
+            self.initSyrthesInstanceList()
+
             self.modelTypeThermal.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
             self.modelTypeSpecies.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
-            self.modelTypeMeteo.addItem(  self.tr("Prescribed (outgoing) flux"), 'neumann')
-            self.modelTypeThermal.addItem(self.tr("Prescribed (outgoing) flux (user law)"), 'neumann_formula')
-            self.modelTypeSpecies.addItem(self.tr("Prescribed (outgoing) flux (user law)"), 'neumann_formula')
-            self.modelTypeMeteo.addItem(  self.tr("Prescribed (outgoing) flux (user law)"), 'neumann_formula')
-            self.modelTypeThermal.addItem(self.tr("Exchange coefficient"), 'exchange_coefficient')
-            self.modelTypeSpecies.addItem(self.tr("Exchange coefficient"), 'exchange_coefficient')
-            self.modelTypeMeteo.addItem(  self.tr("Exchange coefficient"), 'exchange_coefficient')
-            self.modelTypeThermal.addItem(self.tr("Exchange coefficient (user law)"), 'exchange_coefficient_formula')
-            self.modelTypeSpecies.addItem(self.tr("Exchange coefficient (user law)"), 'exchange_coefficient_formula')
-            self.modelTypeMeteo.addItem(  self.tr("Exchange coefficient (user law)"), 'exchange_coefficient_formula')
+            self.modelTypeMeteo.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
+            self.modelTypeThermal.addItem(self.tr("Prescribed (outgoing) flux (user law)"),
+                                          'neumann_formula')
+            self.modelTypeSpecies.addItem(self.tr("Prescribed (outgoing) flux (user law)"),
+                                          'neumann_formula')
+            self.modelTypeMeteo.addItem(self.tr("Prescribed (outgoing) flux (user law)"),
+                                        'neumann_formula')
+            self.modelTypeThermal.addItem(self.tr("Exchange coefficient"),
+                                          'exchange_coefficient')
+            self.modelTypeSpecies.addItem(self.tr("Exchange coefficient"),
+                                          'exchange_coefficient')
+            self.modelTypeMeteo.addItem(self.tr("Exchange coefficient"),
+                                        'exchange_coefficient')
+            self.modelTypeThermal.addItem(self.tr("Exchange coefficient (user law)"),
+                                          'exchange_coefficient_formula')
+            self.modelTypeSpecies.addItem(self.tr("Exchange coefficient (user law)"),
+                                          'exchange_coefficient_formula')
+            self.modelTypeMeteo.addItem(self.tr("Exchange coefficient (user law)"),
+                                        'exchange_coefficient_formula')
+            self.modelTypeThermal.addItem(self.tr("SYRTHES coupling"), "syrthes_coupling")
+
         elif self.nature == 'groundwater':
             self.modelTypeSpecies.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
 
@@ -189,17 +235,19 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         else:
             self.groupBoxSpecies.hide()
 
-        self.model_th = self.therm.getThermalScalarModel()
         if self.model_th != 'off' and self.comp.getCompressibleModel() == 'off':
             self.groupBoxThermal.show()
             self.modelThermal = ComboModel(self.comboBoxThermal,1,1)
             self.thermal = self.therm.getThermalScalarName()
-            self.modelThermal.addItem(self.tr(self.thermal),self.thermal)
+            self.thermal_type = self.__boundary.getScalarChoice(self.thermal)
+            cnv = self.__boundary.getScalarConvert(self.thermal)
+            if cnv:
+                self.thermal_type += ':' + cnv
+            self.modelThermal.addItem(self.tr(self.thermal), self.thermal)
             self.modelThermal.setItem(str_model = self.thermal)
         else:
             self.groupBoxThermal.hide()
 
-        self.meteo_list = ""
         self.meteo_list = self.sca_mo.getMeteoScalarsNameList()
 
         self.groupBoxMeteo.hide()
@@ -217,24 +265,30 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
            (self.nature == 'inlet' or self.nature == 'outlet')):
             label = self.__boundary.getLabel()
             nature = "meteo_" + self.nature
-            bb = Boundary(nature, label, self.__case)
+            bb = Boundary(nature, label, self.case)
 
             if bb.getMeteoDataStatus() == 'off':
                 self.groupBoxMeteo.hide()
-                self.groupBoxThermal.show()
+                if self.model_th != 'off':
+                    self.groupBoxThermal.show()
                 self.modelMeteo = ComboModel(self.comboBoxMeteo, 1, 1)
                 if len(self.meteo_list) > 0:
                     self.groupBoxMeteo.show()
                     for m in self.meteo_list:
                         self.modelMeteo.addItem(self.tr(m), m)
                     self.meteo = self.meteo_list[0]
-                    self.modelMeteo.setItem(str_model = self.meteo)
+                    self.modelMeteo.setItem(str_model=self.meteo)
             else:
                 self.groupBoxMeteo.hide()
                 self.groupBoxThermal.hide()
 
         self.initializeVariables()
 
+    def initSyrthesInstanceList(self):
+        syrthes_instances = self.cht_model.getSyrthesInstancesList()
+        current_instance = self.__boundary.getConjugateHeatTransferCoupling()
+        if current_instance:
+            self.lineEditSyrthesInstance.setText(str(current_instance))
 
     def initializeVariables(self):
         """
@@ -253,18 +307,21 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.labelValueThermal.hide()
         self.pushButtonThermal.setEnabled(False)
         self.pushButtonThermal.setStyleSheet("background-color: None")
+#        self.groupBoxSyrthes.hide()
+        self.labelSyrthesInstance.hide()
+        self.lineEditSyrthesInstance.hide()
 
         if self.model_th != 'off' and self.comp.getCompressibleModel() == 'off':
-            self.thermal_type = self.__boundary.getScalarChoice(self.thermal)
-            self.modelTypeThermal.setItem(str_model = self.thermal_type)
+            self.modelTypeThermal.setItem(str_model=self.thermal_type)
             self.labelValueThermal.setText('Value')
             self.groupBoxThermal.setTitle('Thermal')
 
-            if self.thermal_type in ('dirichlet', 'exchange_coefficient', 'neumann'):
+            thermal_type, convert = self.__get_thermal_type__()
+            if thermal_type in ('dirichlet', 'exchange_coefficient', 'neumann'):
                 self.labelValueThermal.show()
                 self.lineEditValueThermal.show()
 
-                if self.thermal_type == 'exchange_coefficient':
+                if thermal_type == 'exchange_coefficient':
                     self.lineEditExThermal.show()
                     self.labelExThermal.show()
                     v = self.__boundary.getScalarValue(self.thermal, 'dirichlet')
@@ -272,23 +329,31 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                     self.lineEditValueThermal.setText(str(v))
                     self.lineEditExThermal.setText(str(w))
                 else:
-                    v = self.__boundary.getScalarValue(self.thermal, self.thermal_type)
+                    v = self.__boundary.getScalarValue(self.thermal, thermal_type)
                     self.lineEditValueThermal.setText(str(v))
 
-                if self.thermal_type == 'neumann':
+                if thermal_type == 'neumann':
                     self.labelValueThermal.setText('Flux')
                     if self.nature == 'outlet':
                         self.groupBoxThermal.setTitle('Thermal for backflow')
 
-            elif self.thermal_type in ('exchange_coefficient_formula', 'dirichlet_formula', 'neumann_formula'):
+            elif thermal_type in ('exchange_coefficient_formula', 'dirichlet_formula',
+                                  'neumann_formula'):
                 self.pushButtonThermal.setEnabled(True)
-                exp = self.__boundary.getScalarFormula(self.thermal, self.thermal_type)
+                exp = self.__boundary.getScalarFormula(self.thermal, thermal_type)
                 if exp:
                     self.pushButtonThermal.setStyleSheet("background-color: green")
                     self.pushButtonThermal.setToolTip(exp)
                 else:
                     self.pushButtonThermal.setStyleSheet("background-color: red")
 
+            elif self.thermal_type == "syrthes_coupling":
+                self.labelSyrthesInstance.show()
+                self.lineEditSyrthesInstance.show()
+                syrCompleter = QCompleter(self.cht_model.getSyrthesInstancesList())
+                self.lineEditSyrthesInstance.setCompleter(syrCompleter)
+
+
         # Initalize species
         self.labelValueSpecies.hide()
         self.lineEditValueSpecies.hide()
@@ -297,7 +362,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
 
         if self.species_list != None and self.species_list != []:
             self.species_type = self.__boundary.getScalarChoice(self.species)
-            self.modelTypeSpecies.setItem(str_model = self.species_type)
+            self.modelTypeSpecies.setItem(str_model=self.species_type)
             self.labelValueSpecies.setText('Value')
             self.groupBoxSpecies.setTitle('Species')
 
@@ -324,7 +389,8 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                     if self.nature == 'outlet':
                         self.groupBoxSpecies.setTitle('Species for backflow')
 
-            elif self.species_type in ('exchange_coefficient_formula', 'dirichlet_formula', 'neumann_formula'):
+            elif self.species_type in ('exchange_coefficient_formula',
+                                       'dirichlet_formula', 'neumann_formula'):
                 self.pushButtonSpecies.setEnabled(True)
                 exp = self.__boundary.getScalarFormula(self.species, self.species_type)
                 if exp:
@@ -348,7 +414,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                 nature = "meteo_" + self.nature
             else:
                 nature = self.nature
-            bb = Boundary(nature, label, self.__case)
+            bb = Boundary(nature, label, self.case)
 
             if self.nature == 'wall' or bb.getMeteoDataStatus() == 'off':
                 self.meteo_type = self.__boundary.getScalarChoice(self.meteo)
@@ -376,7 +442,8 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                     if self.nature == 'outlet':
                         self.groupBoxMeteo.setTitle('Meteo for backflow')
 
-                if self.meteo_type in ('exchange_coefficient_formula', 'dirichlet_formula', 'neumann_formula'):
+                if self.meteo_type in ('exchange_coefficient_formula',
+                                       'dirichlet_formula', 'neumann_formula'):
                     self.pushButtonMeteo.setEnabled(True)
                     exp = self.__boundary.getScalarFormula(self.meteo, self.meteo_type)
                     if exp:
@@ -390,9 +457,9 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         """
         Show the widget
         """
-        if DefineUserScalarsModel(self.__case).getScalarNameList() or\
-           DefineUserScalarsModel(self.__case).getMeteoScalarsNameList() or\
-           DefineUserScalarsModel(self.__case).getThermalScalarName():
+        if DefineUserScalarsModel(self.case).getScalarNameList() or\
+           DefineUserScalarsModel(self.case).getMeteoScalarsNameList() or\
+           DefineUserScalarsModel(self.case).getThermalScalarName():
             self.__setBoundary(boundary)
             self.show()
         else:
@@ -421,9 +488,14 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         INPUT label for choice of zone
         """
         self.thermal_type = self.modelTypeThermal.dicoV2M[str(text)]
-        self.__boundary.setScalarChoice(self.thermal, self.thermal_type)
-        self.initializeVariables()
+        thermal_type, convert = self.__get_thermal_type__()
+        self.__boundary.setScalarChoice(self.thermal, thermal_type)
+        if not convert:
+            convert = None
+        self.__boundary.setScalarConvert(self.thermal, convert=convert)
 
+        self.initializeVariables()
+        return
 
     @pyqtSlot(str)
     def slotSpeciesChoice(self, text):
@@ -468,13 +540,18 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         """
         """
         name = self.thermal
-        exp = self.__boundary.getScalarFormula(self.thermal, self.thermal_type)
+        variable_name = name
+        thermal_type, convert = self.__get_thermal_type__()
+        if convert:
+            name = convert
+
+        exp = self.__boundary.getScalarFormula(self.thermal, thermal_type)
         exa = """#example: """
-        if self.thermal_type == 'dirichlet_formula':
+        if thermal_type == 'dirichlet_formula':
             req = [(name, str(name))]
-        elif self.thermal_type == 'neumann_formula':
+        elif thermal_type == 'neumann_formula':
             req = [("flux", "flux")]
-        elif self.thermal_type == 'exchange_coefficient_formula':
+        elif thermal_type == 'exchange_coefficient_formula':
             req = [(name, str(name)),("hc", "heat coefficient")]
 
         sym = [('x', "X face's gravity center"),
@@ -482,21 +559,28 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
 
         for (nme, val) in self.notebook.getNotebookList():
             sym.append((nme, 'value (notebook) = ' + str(val)))
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
+        c = self.__boundary.getScalarChoice(variable_name)
+
+        dialog = QMegEditorView(parent      = self,
+                                function_type = 'bnd',
+                                zone_name     = self.__boundary._label,
+                                variable_name = name,
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = c,
+                                examples      = exa)
+
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotThermalFormula -> %s" % str(result))
-            self.__boundary.setScalarFormula(self.thermal, self.thermal_type, str(result))
+            self.__boundary.setScalarFormula(self.thermal, thermal_type, str(result))
             self.pushButtonThermal.setStyleSheet("background-color: green")
             self.pushButtonThermal.setToolTip(exp)
 
@@ -519,17 +603,23 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
 
         for (nme, val) in self.notebook.getNotebookList():
             sym.append((nme, 'value (notebook) = ' + str(val)))
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
+        c = self.__boundary.getScalarChoice(self.species)
+        dialog = QMegEditorView(parent        = self,
+                                function_type = 'bnd',
+                                zone_name     = self.__boundary._label,
+                                variable_name = self.species,
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = c,
+                                examples      = exa)
+
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotSpeciesFormula -> %s" % str(result))
@@ -557,17 +647,22 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
 
         for (nme, val) in self.notebook.getNotebookList():
             sym.append((nme, 'value (notebook) = ' + str(val)))
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
+        dialog = QMegEditorView(parent        = self,
+                                function_type = 'bnd',
+                                zone_name     = self.__boundary._label,
+                                variable_name = self.meteo,
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = self.meteo_type,
+                                examples=exa)
+
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotMeteoFormula -> %s" % str(result))
@@ -582,9 +677,10 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         """
         if self.lineEditValueThermal.validator().state == QValidator.Acceptable:
             value = from_qvariant(var, float)
-            if self.thermal_type in ('dirichlet', 'neumann'):
-                self.__boundary.setScalarValue(self.thermal, self.thermal_type, value)
-            elif self.thermal_type == 'exchange_coefficient':
+            thermal_type = self.thermal_type.split(':')[0]
+            if thermal_type in ('dirichlet', 'neumann'):
+                self.__boundary.setScalarValue(self.thermal, thermal_type, value)
+            elif thermal_type == 'exchange_coefficient':
                 self.__boundary.setScalarValue(self.thermal, 'dirichlet', value)
 
 
@@ -629,7 +725,6 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
             value = from_qvariant(var, float)
             self.__boundary.setScalarValue(self.species, 'exchange_coefficient', value)
 
-
     @pyqtSlot(str)
     def slotExMeteo(self, var):
         """
@@ -638,12 +733,24 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
             value = from_qvariant(var, float)
             self.__boundary.setScalarValue(self.meteo, 'exchange_coefficient', value)
 
+    @pyqtSlot()
+    def slotChooseSyrthesInstance(self):
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
+        value = str(self.lineEditSyrthesInstance.text())
+        if value:
+            bnd_label = self.__boundary.getLabel()
+            value_p = self.__boundary.getConjugateHeatTransferCoupling()
+            bnd_label = self.__boundary.getLabel()
+            if value != value_p:
+                self.cht_model.deleteSyrthesCoupling(value_p, bnd_label)
+
+            self.__boundary.setConjugateHeatTransferCoupling(value)
+
+            if value not in self.cht_model.getSyrthesInstancesList():
+                self.cht_model.addSyrthesCoupling(value)
+            self.cht_model.addBoundaryLabel(value, bnd_label)
+
+        return
 
 #-------------------------------------------------------------------------------
 # End
