@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -46,12 +46,12 @@ from code_saturne.Base.QtWidgets import *
 #-------------------------------------------------------------------------------
 
 from code_saturne.Pages.BoundaryConditionsTurbulenceInletForm import Ui_BoundaryConditionsTurbulenceInletForm
-from code_saturne.Pages.TurbulenceModel import TurbulenceModel
+from code_saturne.model.TurbulenceModel import TurbulenceModel
 
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import GuiParam
 from code_saturne.Base.QtPage import DoubleValidator, ComboModel, from_qvariant
-from code_saturne.Pages.QMeiEditorView import QMeiEditorView
-from code_saturne.Pages.NotebookModel import NotebookModel
+from code_saturne.Pages.QMegEditorView import QMegEditorView
+from code_saturne.model.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -83,11 +83,11 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
         """
         Setup the widget
         """
-        self.__case = case
+        self.case = case
         self.__boundary = None
 
-        self.__case.undoStopGlobal()
-        self.notebook = NotebookModel(self.__case)
+        self.case.undoStopGlobal()
+        self.notebook = NotebookModel(self.case)
 
         self.comboBoxTurbulence.activated[str].connect(self.__slotChoiceTurbulence)
 
@@ -109,7 +109,7 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
         self.lineEditDiameterIntens.setValidator(validatorDiam)
         self.lineEditIntensity.setValidator(validatorIntensity)
 
-        self.__case.undoStartGlobal()
+        self.case.undoStartGlobal()
 
 
     def showWidget(self, boundary):
@@ -118,7 +118,7 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
         """
         self.__boundary = boundary
 
-        if TurbulenceModel(self.__case).getTurbulenceVariable():
+        if TurbulenceModel(self.case).getTurbulenceVariable():
             turb_choice = boundary.getTurbulenceChoice()
             self.__modelTurbulence.setItem(str_model=turb_choice)
             self.pushButtonTurb.setEnabled(False)
@@ -215,57 +215,67 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
         """
         INPUT user formula
         """
-        turb_model = TurbulenceModel(self.__case).getTurbulenceModel()
+        sym = [('x','cell center coordinate'),
+               ('y','cell center coordinate'),
+               ('z','cell center coordinate'),
+               ('t','time'),
+               ('dt','time step'),
+               ('iter','number of time step'),
+               ('surface', 'Boundary zone surface')]
+
+        for (nme, val) in self.notebook.getNotebookList():
+            sym.append((nme, 'value (notebook) = ' + str(val)))
+
+        exa_base = """#example:
+uref2 = 10.;
+dh = 0.2;
+rho0 = 1.17862;
+mu0 = 1.83e-05;
+
+re = sqrt(uref2)*dh*rho0/mu0;
+
+if (re < 2000) {
+  # in this case u*^2 is directly calculated to
+  # avoid a problem with xlmbda=64/Re when Re->0
+  ustar2 = 8.*mu0*sqrt(uref2)/rho0/dh;
+}
+else if (re < 4000) {
+  xlmbda = 0.021377 + 5.3115e-6*re;
+  ustar2 = uref2*xlmbda/8.;
+}
+else {
+  xlmbda = 1/( 1.8*log(re)/log(10.)-1.64)^2;
+  ustar2 = uref2*xlmbda/8.;
+}
+cmu = 0.09;
+kappa = 0.42;
+"""
+
+        turb_model = TurbulenceModel(self.case).getTurbulenceModel()
         if turb_model in ('k-epsilon', 'k-epsilon-PL'):
 
             exp = self.__boundary.getTurbFormula()
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
-            exa = """#example :
-uref2 = 10.;
-dh = 0.2;
-re = sqrt(uref2)*dh*rho0/mu0;
+            exa = exa_base + """
 
-if (re < 2000){
-#     in this case u*^2 is directly calculated to not have a problem with
-#     xlmbda=64/Re when Re->0
-
-  ustar2 = 8.*mu0*sqrt(uref2)/rho0/dh;}
-
-else if (re<4000){
-
-  xlmbda = 0.021377 + 5.3115e-6*re;
-  ustar2 = uref2*xlmbda/8.;}
-
-else {
-
-  xlmbda = 1/( 1.8*log(re)/log(10.)-1.64)^2;
-  ustar2 = uref2*xlmbda/8.;}
-
-cmu = 0.09;
-kappa = 0.42;
-k   = ustar2/sqrt(cmu);
+k = ustar2/sqrt(cmu);
 epsilon = ustar2^1.5/(kappa*dh*0.1);"""
 
             req = [('k', "turbulent energy"),
                    ('epsilon', "turbulent dissipation")]
 
-            sym = [('x','cell center coordinate'),
-                   ('y','cell center coordinate'),
-                   ('z','cell center coordinate'),
-                   ('t','time'),
-                   ('dt','time step'),
-                   ('iter','number of time step')]
+            dialog = QMegEditorView(parent        = self,
+                                    function_type = 'bnd',
+                                    zone_name     = self.__boundary._label,
+                                    variable_name = 'turbulence_ke',
+                                    expression    = exp,
+                                    required      = req,
+                                    symbols       = sym,
+                                    condition     = 'formula',
+                                    examples      = exa)
 
-            for (nme, val) in self.notebook.getNotebookList():
-                sym.append((nme, 'value (notebook) = ' + str(val)))
 
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
-                                    expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
@@ -279,32 +289,11 @@ epsilon = ustar2^1.5/(kappa*dh*0.1);"""
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
 
-            exa = """#exemple :
-uref2 = 10.;
-dh = 0.2;
-re = sqrt(uref2)*dh*rho0/mu0;
-
-if (re < 2000){
-#     in this case u*^2 is directly calculated to not have a problem with
-#     xlmbda=64/Re when Re->0
-
-  ustar2 = 8.*mu0*sqrt(uref2)/rho0/dh;}
-
-else if (re<4000){
-
-  xlmbda = 0.021377 + 5.3115e-6*re;
-  ustar2 = uref2*xlmbda/8.;}
-
-else {
-
-  xlmbda = 1/( 1.8*log(re)/log(10.)-1.64)^2;
-  ustar2 = uref2*xlmbda/8.;}
-
-cmu = 0.09;
-kappa = 0.42;
-k   = ustar2/sqrt(cmu);
-epsilon = ustar2^1.5/(kappa*dh*0.1);
+            exa = exa_base + """
 d2s3 = 2/3;
+
+k = ustar2/sqrt(cmu);
+epsilon = ustar2^1.5/(kappa*dh*0.1);
 r11 = d2s3*k;
 r22 = d2s3*k;
 r33 = d2s3*k;
@@ -317,26 +306,19 @@ r23 = 0;
                    ('r22', "Reynolds stress R22"),
                    ('r33', "Reynolds stress R33"),
                    ('r12', "Reynolds stress R12"),
-                   ('r13', "Reynolds stress R13"),
-                   ('r23', "Reynolds stress R23"),
+                   ('r23', "Reynolds stress R13"),
+                   ('r13', "Reynolds stress R23"),
                    ('epsilon', "turbulent dissipation")]
 
-            sym = [('x','cell center coordinate'),
-                   ('y','cell center coordinate'),
-                   ('z','cell center coordinate'),
-                   ('t','time'),
-                   ('dt','time step'),
-                   ('iter','number of time step')]
+            dialog = QMegEditorView(parent        = self,
+                                    function_type = 'bnd',
+                                    zone_name     = self.__boundary._label,
+                                    variable_name = 'turbulence_rije',
+                                    expression    = exp,
+                                    required      = req,
+                                    symbols       = sym,
+                                    examples      = exa)
 
-            for (nme, val) in self.notebook.getNotebookList():
-                sym.append((nme, 'value (notebook) = ' + str(val)))
-
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
-                                    expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
@@ -350,32 +332,11 @@ r23 = 0;
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
 
-            exa = """#exemple :
-uref2 = 10.;
-dh = 0.2;
-re = sqrt(uref2)*dh*rho0/mu0;
-
-if (re < 2000){
-#     in this case u*^2 is directly calculated to not have a problem with
-#     xlmbda=64/Re when Re->0
-
-  ustar2 = 8.*mu0*sqrt(uref2)/rho0/dh;}
-
-else if (re<4000){
-
-  xlmbda = 0.021377 + 5.3115e-6*re;
-  ustar2 = uref2*xlmbda/8.;}
-
-else {
-
-  xlmbda = 1/( 1.8*log(re)/log(10.)-1.64)^2;
-  ustar2 = uref2*xlmbda/8.;}
-
-cmu = 0.09;
-kappa = 0.42;
-k   = ustar2/sqrt(cmu);
-epsilon = ustar2^1.5/(kappa*dh*0.1);
+            exa = exa_base + """
 d2s3 = 2/3;
+
+k = ustar2/sqrt(cmu);
+epsilon = ustar2^1.5/(kappa*dh*0.1);
 r11 = d2s3*k;
 r22 = d2s3*k;
 r33 = d2s3*k;
@@ -389,27 +350,20 @@ alpha =  1.;
                    ('r22', "Reynolds stress R22"),
                    ('r33', "Reynolds stress R33"),
                    ('r12', "Reynolds stress R12"),
-                   ('r13', "Reynolds stress R13"),
-                   ('r23', "Reynolds stress R23"),
+                   ('r23', "Reynolds stress R13"),
+                   ('r13', "Reynolds stress R23"),
                    ('epsilon', "turbulent dissipation"),
                    ('alpha', "alpha")]
 
-            sym = [('x','cell center coordinate'),
-                   ('y','cell center coordinate'),
-                   ('z','cell center coordinate'),
-                   ('t','time'),
-                   ('dt','time step'),
-                   ('iter','number of time step')]
+            dialog = QMegEditorView(parent        = self,
+                                    function_type = 'bnd',
+                                    zone_name     = self.__boundary._label,
+                                    variable_name = 'turbulence_rij_ebrsm',
+                                    expression    = exp,
+                                    required      = req,
+                                    symbols       = sym,
+                                    examples      = exa)
 
-            for (nme, val) in self.notebook.getNotebookList():
-                sym.append((nme, 'value (notebook) = ' + str(val)))
-
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
-                                    expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
@@ -423,30 +377,9 @@ alpha =  1.;
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
 
-            exa = """#exemple :
-uref2 = 10.;
-dh = 0.2;
-re = sqrt(uref2)*dh*rho0/mu0;
-
-if (re < 2000){
-#     in this case u*^2 is directly calculated to not have a problem with
-#     xlmbda=64/Re when Re->0
-
-  ustar2 = 8.*mu0*sqrt(uref2)/rho0/dh;}
-
-else if (re<4000){
-
-  xlmbda = 0.021377 + 5.3115e-6*re;
-  ustar2 = uref2*xlmbda/8.;}
-
-else {
-
-  xlmbda = 1/( 1.8*log(re)/log(10.)-1.64)^2;
-  ustar2 = uref2*xlmbda/8.;}
-
-cmu = 0.09;
-kappa = 0.42;
+            exa = exa_base + """
 d2s3 = 2/3;
+
 k   = ustar2/sqrt(cmu);
 epsilon = ustar2^1.5/(kappa*dh*0.1);
 phi = d2s3;
@@ -457,22 +390,16 @@ alpha = 0;"""
                    ('phi', "variable phi in v2f model"),
                    ('alpha', "variable alpha in v2f model")]
 
-            sym = [('x','cell center coordinate'),
-                   ('y','cell center coordinate'),
-                   ('z','cell center coordinate'),
-                   ('t','time'),
-                   ('dt','time step'),
-                   ('iter','number of time step')]
-
-            for (nme, val) in self.notebook.getNotebookList():
-                sym.append((nme, 'value (notebook) = ' + str(val)))
-
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
+            dialog = QMegEditorView(parent = self,
+                                    function_type = 'bnd',
+                                    zone_name = self.__boundary._label,
+                                    variable_name = 'turbulence_v2f',
                                     expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
+                                    required = req,
+                                    symbols = sym,
+                                    condition = 'formula',
+                                    examples = exa)
+
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
@@ -486,52 +413,25 @@ alpha = 0;"""
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
 
-            exa = """#exemple :
-uref2 = 10.;
-dh = 0.2;
-re = sqrt(uref2)*dh*rho0/mu0;
+            exa = exa_base + """
 
-if (re < 2000){
-#     in this case u*^2 is directly calculated to not have a problem with
-#     xlmbda=64/Re when Re->0
-
-  ustar2 = 8.*mu0*sqrt(uref2)/rho0/dh;}
-
-else if (re<4000){
-
-  xlmbda = 0.021377 + 5.3115e-6*re;
-  ustar2 = uref2*xlmbda/8.;}
-
-else {
-
-  xlmbda = 1/( 1.8*log(re)/log(10.)-1.64)^2;
-  ustar2 = uref2*xlmbda/8.;}
-
-cmu = 0.09;
-kappa = 0.42;
-k   = ustar2/sqrt(cmu);
-eps = ustar2^1.5/(kappa*dh*0.1);
-omega = eps/(cmu * k);"""
+k = ustar2/sqrt(cmu);
+epsilon = ustar2^1.5/(kappa*dh*0.1);
+omega = epsilon/(cmu * k);"""
 
             req = [('k', "turbulent energy"),
                    ('omega', "specific dissipation rate")]
 
-            sym = [('x','cell center coordinate'),
-                   ('y','cell center coordinate'),
-                   ('z','cell center coordinate'),
-                   ('t','time'),
-                   ('dt','time step'),
-                   ('iter','number of time step')]
+            dialog = QMegEditorView(parent        = self,
+                                    function_type = 'bnd',
+                                    zone_name     = self.__boundary._label,
+                                    variable_name = 'turbulence_kw',
+                                    expression    = exp,
+                                    required      = req,
+                                    symbols       = sym,
+                                    condition     = 'formula',
+                                    examples      = exa)
 
-            for (nme, val) in self.notebook.getNotebookList():
-                sym.append((nme, 'value (notebook) = ' + str(val)))
-
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
-                                    expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
@@ -545,51 +445,25 @@ omega = eps/(cmu * k);"""
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
 
-            exa = """#exemple :
-uref2 = 10.;
-dh = 0.2;
-re = sqrt(uref2)*dh*rho0/mu0;
+            exa = exa_base + """
 
-if (re < 2000){
-#     in this case u*^2 is directly calculated to not have a problem with
-#     xlmbda=64/Re when Re->0
+k = ustar2/sqrt(cmu);
+epsilon = ustar2^1.5/(kappa*dh*0.1);
 
-  ustar2 = 8.*mu0*sqrt(uref2)/rho0/dh;}
-
-else if (re<4000){
-
-  xlmbda = 0.021377 + 5.3115e-6*re;
-  ustar2 = uref2*xlmbda/8.;}
-
-else {
-
-  xlmbda = 1/( 1.8*log(re)/log(10.)-1.64)^2;
-  ustar2 = uref2*xlmbda/8.;}
-
-cmu = 0.09;
-kappa = 0.42;
-k   = ustar2/sqrt(cmu);
-eps = ustar2^1.5/(kappa*dh*0.1);
-nu_tilda = eps/(cmu * k);"""
+nu_tilda = _epsilon/(cmu * _k);"""
 
             req = [('nu_tilda', "nu_tilda")]
 
-            sym = [('x','cell center coordinate'),
-                   ('y','cell center coordinate'),
-                   ('z','cell center coordinate'),
-                   ('t','time'),
-                   ('dt','time step'),
-                   ('iter','number of time step')]
+            dialog = QMegEditorView(parent        = self,
+                                    function_type = 'bnd',
+                                    zone_name     = self.__boundary._label,
+                                    variable_name = 'turbulence_spalart',
+                                    expression    = exp,
+                                    required      = req,
+                                    symbols       = sym,
+                                    condition     = 'formula',
+                                    examples      = exa)
 
-            for (nme, val) in self.notebook.getNotebookList():
-                sym.append((nme, 'value (notebook) = ' + str(val)))
-
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
-                                    expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
@@ -598,12 +472,6 @@ nu_tilda = eps/(cmu * k);"""
                 self.pushButtonTurb.setToolTip(result)
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
 #-------------------------------------------------------------------------------
 # End
 #-------------------------------------------------------------------------------
