@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -47,15 +47,15 @@ from code_saturne.Base.QtWidgets import *
 
 from code_saturne.Pages.BoundaryConditionsVelocityInletForm import Ui_BoundaryConditionsVelocityInletForm
 
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import GuiParam
 from code_saturne.Base.QtPage import DoubleValidator, ComboModel, from_qvariant
-from code_saturne.Pages.LocalizationModel import LocalizationModel, Zone
-from code_saturne.Pages.Boundary import Boundary
-from code_saturne.Pages.CompressibleModel import CompressibleModel
-from code_saturne.Pages.GasCombustionModel import GasCombustionModel
+from code_saturne.model.LocalizationModel import LocalizationModel, Zone
+from code_saturne.model.Boundary import Boundary
+from code_saturne.model.CompressibleModel import CompressibleModel
+from code_saturne.model.GasCombustionModel import GasCombustionModel
 
-from code_saturne.Pages.QMeiEditorView import QMeiEditorView
-from code_saturne.Pages.NotebookModel import NotebookModel
+from code_saturne.Pages.QMegEditorView import QMegEditorView
+from code_saturne.model.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -88,14 +88,14 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         """
         Setup the widget
         """
-        self.__case = case
+        self.case = case
         self.__boundary = None
 
-        self.__case.undoStopGlobal()
+        self.case.undoStopGlobal()
 
-        self.mdl = CompressibleModel(self.__case)
-        self.gas = GasCombustionModel(self.__case)
-        self.notebook = NotebookModel(self.__case)
+        self.mdl = CompressibleModel(self.case)
+        self.gas = GasCombustionModel(self.case)
+        self.notebook = NotebookModel(self.case)
 
         # Connections
         self.comboBoxVelocity.activated[str].connect(self.__slotChoiceVelocity)
@@ -181,7 +181,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         self.pushButtonVelocityFormula.clicked.connect(self.__slotVelocityFormula)
         self.pushButtonDirectionFormula.clicked.connect(self.__slotDirectionFormula)
 
-        self.__case.undoStartGlobal()
+        self.case.undoStartGlobal()
 
 
     def showWidget(self, boundary):
@@ -345,6 +345,21 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         self.hide()
 
 
+    def getVelocityExpression(self, boundary):
+        """
+        Get the mathematical expression for velocity
+        """
+        exp = boundary.getVelocity()
+        c = boundary.getVelocityChoice()
+        if c == 'norm_formula':
+            n = 'u_norm'
+        elif c == 'flow1_formula':
+            n = 'q_m'
+        elif c == 'flow2_formula':
+            n = 'q_v'
+
+        return exp, n
+
     @pyqtSlot(str)
     def __slotChoiceVelocity(self, text):
         """
@@ -413,9 +428,13 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         """
         exp = self.__boundary.getVelocity()
         c = self.__boundary.getVelocityChoice()
+        sym = []
         if c == 'norm_formula':
             exa = "u_norm = 1.0;"
             req = [('u_norm', 'Norm of the velocity')]
+            sym.extend((('x', "X face's gravity center"),
+                        ('y', "Y face's gravity center"),
+                        ('z', "Z face's gravity center")))
         elif c == 'flow1_formula':
             exa = "q_m = 1.0;"
             req = [('q_m', 'mass flow rate')]
@@ -423,22 +442,24 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             exa = "q_v = 1.0;"
             req = [('q_v', 'volumic flow rate')]
 
-        sym = [('x', "X face's gravity center"),
-               ('y', "Y face's gravity center"),
-               ('z', "Z face's gravity center"),
-               ('dt', 'time step'),
-               ('t', 'current time'),
-               ('iter', 'number of iteration')]
+        sym.extend((('dt', 'time step'),
+                    ('t', 'current time'),
+                    ('iter', 'number of iteration'),
+                    ('surface', 'Boundary zone surface')))
 
         for (nme, val) in self.notebook.getNotebookList():
             sym.append((nme, 'value (notebook) = ' + str(val)))
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
+        dialog = QMegEditorView(parent        = self,
+                                function_type = "bnd",
+                                zone_name     = self.__boundary._label,
+                                variable_name = "velocity",
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = c,
+                                examples      = exa)
+
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotFormulaVelocity -> %s" % str(result))
@@ -528,17 +549,22 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
 
         for (nme, val) in self.notebook.getNotebookList():
             sym.append((nme, 'value (notebook) = ' + str(val)))
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
+        dialog = QMegEditorView(parent        = self,
+                                function_type = "bnd",
+                                zone_name     = self.__boundary._label,
+                                variable_name = "direction",
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = "formula",
+                                examples      = exa)
+
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotFormulaDirection -> %s" % str(result))
@@ -698,12 +724,6 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         self.initialize()
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
 #-------------------------------------------------------------------------------
 # End
 #-------------------------------------------------------------------------------
