@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -52,16 +52,16 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Common import LABEL_LENGTH_MAX
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam
 from code_saturne.Base.QtPage import ComboModel, DoubleValidator, RegExpValidator
-from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
+from code_saturne.Base.QtPage import from_qvariant, to_text_string
 
 from code_saturne.Pages.DefineUserScalarsForm import Ui_DefineUserScalarsForm
-from code_saturne.Pages.LocalizationModel import LocalizationModel
-from code_saturne.Pages.DefineUserScalarsModel import DefineUserScalarsModel
-from code_saturne.Pages.TurbulenceModel import TurbulenceModel
-from code_saturne.Pages.GroundwaterModel import GroundwaterModel
+from code_saturne.model.LocalizationModel import LocalizationModel
+from code_saturne.model.DefineUserScalarsModel import DefineUserScalarsModel
+from code_saturne.model.TurbulenceModel import TurbulenceModel
+from code_saturne.model.GroundwaterModel import GroundwaterModel
+from code_saturne.model.GasCombustionModel import GasCombustionModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -110,10 +110,11 @@ class NameDelegate(QItemDelegate):
         if editor.validator().state == QValidator.Acceptable:
             new_pname = str(editor.text())
 
-            if new_pname in model.mdl.getScalarNameList():
+            if new_pname in model.mdl.getScalarNameList() + model.mdl.getGasCombScalarsNameList() + model.mdl.getThermalScalarName():
                 default = {}
-                default['name']  = self.old_pname
-                default['list']   = model.mdl.getScalarNameList()
+                default['label']  = self.old_pname
+                default['name']   = self.old_pname
+                default['list'] = model.mdl.getScalarNameList()
                 default['regexp'] = self.regExp
                 log.debug("setModelData -> default = %s" % default)
 
@@ -121,12 +122,15 @@ class NameDelegate(QItemDelegate):
                 dialog = VerifyExistenceLabelDialogView(self.parent, default)
                 if dialog.exec_():
                     result = dialog.get_result()
-                    new_pname = result['name']
+                    new_pname = result['label']
+                    result['name'] = new_pname
+                    result['list'] = model.mdl.getScalarNameList()
+                    result['regexp'] = self.regExp
                     log.debug("setModelData -> result = %s" % result)
                 else:
                     new_pname = self.old_pname
 
-            model.setData(index, to_qvariant(str(new_pname)), Qt.DisplayRole)
+            model.setData(index, str(new_pname), Qt.DisplayRole)
 
 
 #-------------------------------------------------------------------------------
@@ -172,15 +176,9 @@ class GGDHDelegate(QItemDelegate):
     def setModelData(self, comboBox, model, index):
         txt = str(comboBox.currentText())
         value = self.modelCombo.dicoV2M[txt]
-        model.setData(index, to_qvariant(value), Qt.DisplayRole)
+        model.setData(index, value, Qt.DisplayRole)
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
 #-------------------------------------------------------------------------------
 # Line edit delegate for the variance name
 #-------------------------------------------------------------------------------
@@ -219,8 +217,9 @@ class VarianceNameDelegate(QItemDelegate):
         if editor.validator().state == QValidator.Acceptable:
             new_pname = str(editor.text())
 
-            if new_pname in model.mdl.getScalarNameList():
+            if new_pname in model.mdl.getScalarNameList() + model.mdl.getGasCombScalarsNameList() + model.mdl.getThermalScalarName():
                 default = {}
+                default['label']  = self.old_pname
                 default['name']  = self.old_pname
                 default['list']   = model.mdl.getScalarNameList()
                 default['regexp'] = self.regExp
@@ -235,7 +234,7 @@ class VarianceNameDelegate(QItemDelegate):
                 else:
                     new_pname = self.old_pname
 
-            model.setData(index, to_qvariant(str(new_pname)), Qt.DisplayRole)
+            model.setData(index, str(new_pname), Qt.DisplayRole)
 
 
 #-------------------------------------------------------------------------------
@@ -260,6 +259,11 @@ class VarianceDelegate(QItemDelegate):
 
     def setEditorData(self, editor, index):
         editor.setAutoFillBackground(True)
+        value = from_qvariant(index.model().data(index, Qt.DisplayRole),
+                              to_text_string)
+        if value in index.model().mdl.getGasCombScalarsNameList():
+            self.modelCombo.addItem(value, value)
+            return
         l1 = index.model().mdl.getScalarNameList()
         for label in index.model().mdl.getThermalScalarName():
             l1.append(label)
@@ -273,15 +277,9 @@ class VarianceDelegate(QItemDelegate):
     def setModelData(self, comboBox, model, index):
         txt = str(comboBox.currentText())
         value = self.modelCombo.dicoV2M[txt]
-        model.setData(index, to_qvariant(value), Qt.DisplayRole)
+        model.setData(index, value, Qt.DisplayRole)
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
 #-------------------------------------------------------------------------------
 # StandarItemModel class
 #-------------------------------------------------------------------------------
@@ -298,9 +296,6 @@ class StandardItemModelScalars(QStandardItemModel):
 
         self.setColumnCount(len(self.headers))
 
-        self.toolTipRole = [self.tr("Code_Saturne keyword: NSCAUS"),
-                            self.tr("Code_Saturne keyword: ITURT")]
-
         self._data = []
         self.parent = parent
         self.mdl  = mdl
@@ -308,17 +303,15 @@ class StandardItemModelScalars(QStandardItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         row = index.row()
         col = index.column()
 
-        if role == Qt.ToolTipRole:
-            return to_qvariant(self.toolTipRole[col])
         if role == Qt.DisplayRole:
-            return to_qvariant(self._data[row][col])
+            return self._data[row][col]
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -333,8 +326,8 @@ class StandardItemModelScalars(QStandardItemModel):
 
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
-            return to_qvariant(self.headers[section])
-        return to_qvariant()
+            return self.headers[section]
+        return None
 
 
     def setData(self, index, value, role):
@@ -348,6 +341,8 @@ class StandardItemModelScalars(QStandardItemModel):
         # Name
         if col == 0:
             old_pname = self._data[row][col]
+            if self.mdl.getScalarType(old_pname) == 'var_model':
+                return
             new_pname = str(from_qvariant(value, to_text_string))
             self._data[row][col] = new_pname
             self.mdl.renameScalarLabel(old_pname, new_pname)
@@ -382,6 +377,21 @@ class StandardItemModelScalars(QStandardItemModel):
         self._data.append(scalar)
 
 
+    def newModelItem(self, existing_name=None):
+        """
+        Add only an item in the table view (do not create a scalar)
+        """
+        row = self.rowCount()
+        var = self.mdl.getScalarVariance(existing_name)
+        if var not in ("", "no variance", "no_variance"):
+            return
+        turbFlux = self.mdl.getTurbulentFluxModel(existing_name)
+        scalar = [existing_name, turbFlux]
+
+        self.setRowCount(row+1)
+        self._data.append(scalar)
+
+
     def getItem(self, row):
         """
         Return the values for an item.
@@ -414,7 +424,7 @@ class StandardItemModelVariance(QStandardItemModel):
         QStandardItemModel.__init__(self)
 
         self.headers = [self.tr("Variance"),
-                        self.tr("Species_Name")]
+                        self.tr("Associated variable")]
 
         self.setColumnCount(len(self.headers))
 
@@ -425,17 +435,15 @@ class StandardItemModelVariance(QStandardItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         row = index.row()
         col = index.column()
 
-        if role == Qt.ToolTipRole:
-            return to_qvariant(self.toolTipRole[col])
         if role == Qt.DisplayRole:
-            return to_qvariant(self._data[row][col])
+            return self._data[row][col]
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -446,8 +454,8 @@ class StandardItemModelVariance(QStandardItemModel):
 
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
-            return to_qvariant(self.headers[section])
-        return to_qvariant()
+            return self.headers[section]
+        return None
 
 
     def setData(self, index, value, role):
@@ -461,6 +469,8 @@ class StandardItemModelVariance(QStandardItemModel):
         # name
         if col == 0:
             old_pname = self._data[row][col]
+            if self.mdl.getScalarType(old_pname) == 'var_model':
+                return
             new_pname = str(from_qvariant(value, to_text_string))
             self._data[row][col] = new_pname
             self.mdl.renameScalarLabel(old_pname, new_pname)
@@ -471,6 +481,8 @@ class StandardItemModelVariance(QStandardItemModel):
             variance = str(from_qvariant(value, to_text_string))
             self._data[row][col] = variance
             name = self._data[row][0]
+            if self.mdl.getScalarType(name) == 'var_model':
+                return
             self.mdl.setScalarVariance(name, variance)
 
         self.dataChanged.emit(index, index)
@@ -506,6 +518,21 @@ class StandardItemModelVariance(QStandardItemModel):
         self._data.append(scalar)
 
 
+    def newModelItem(self, existing_name=None):
+        """
+        Add an item in the table view
+        """
+
+        row = self.rowCount()
+        var = self.mdl.getScalarVariance(existing_name)
+        if var in ("", "no variance", "no_variance"):
+            return
+        scalar = [existing_name, var]
+
+        self.setRowCount(row+1)
+        self._data.append(scalar)
+
+
     def getItem(self, row):
         """
         Return the values for an item.
@@ -595,6 +622,10 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
             self.modelScalars.newItem(name)
         for name in self.mdl.getScalarsVarianceList():
             self.modelVariance.newItem(name)
+        if GasCombustionModel(self.case).getGasCombustionModel() == "d3p":
+            for name in self.mdl.getGasCombScalarsNameList():
+                self.modelScalars.newModelItem(name)
+                self.modelVariance.newModelItem(name)
 
         if GroundwaterModel(self.case).getGroundwaterModel() != "off":
             self.groupBox_3.hide()
@@ -626,6 +657,8 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
 
         for row in lst:
             name = self.modelScalars.getItem(row)[0]
+            if self.mdl.getScalarType(name) == 'var_model':
+                return
             if self.mdl.getScalarType(name) == 'user':
                 self.mdl.deleteScalar(name)
                 self.modelScalars.deleteItem(row)
@@ -672,6 +705,8 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
 
         for row in lst:
             name = self.modelVariance.getItem(row)[0]
+            if self.mdl.getScalarType(name) == 'var_model':
+                return
             self.mdl.deleteScalar(name)
             self.modelVariance.deleteItem(row)
 
@@ -686,12 +721,6 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
             self.tableView.resizeColumnToContents(col)
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
 #-------------------------------------------------------------------------------
 # End
 #-------------------------------------------------------------------------------
