@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -49,8 +49,8 @@ from code_saturne.Base.QtWidgets import *
 #-------------------------------------------------------------------------------
 
 from code_saturne.Base.QtPage import RegExpValidator, DoubleValidator
-from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.Base.QtPage import from_qvariant, to_text_string
+from code_saturne.model.Common import GuiParam, GuiLabelManager
 from code_saturne.Pages.FacesSelectionForm import Ui_FacesSelectionForm
 
 #-------------------------------------------------------------------------------
@@ -87,7 +87,7 @@ class LineEditDelegateVerbosity(QItemDelegate):
 
     def setModelData(self, lineEdit, model, index):
         value = lineEdit.text()
-        model.setData(index, to_qvariant(value), Qt.DisplayRole)
+        model.setData(index, value, Qt.DisplayRole)
 
 #-------------------------------------------------------------------------------
 # Line edit delegate for selection
@@ -105,17 +105,30 @@ class LineEditDelegateSelector(QItemDelegate):
         editor = QLineEdit(parent)
         validator =  RegExpValidator(editor, QRegExp("[ -~]*"))
         editor.setValidator(validator)
+
+        # Autocompletion for selection criteria!
+        comp_list = GuiLabelManager().getCompleter('mesh_selection')
+
+        completer = QCompleter()
+        editor.setCompleter(completer)
+        model = QStringListModel()
+        completer.setModel(model)
+        model.setStringList(comp_list)
+
         return editor
 
 
     def setEditorData(self, lineEdit, index):
+        # This line is used to avoid an overlay of old and new text
+        lineEdit.setAutoFillBackground(True)
+
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         lineEdit.setText(value)
 
 
     def setModelData(self, lineEdit, model, index):
         value = lineEdit.text()
-        model.setData(index, to_qvariant(value), Qt.DisplayRole)
+        model.setData(index, value, Qt.DisplayRole)
 
 #-------------------------------------------------------------------------------
 # Line edit delegate for Fraction and Plane
@@ -143,7 +156,7 @@ class FractionPlaneDelegate(QItemDelegate):
     def setModelData(self, editor, model, index):
         if editor.validator().state == QValidator.Acceptable:
             value = from_qvariant(editor.text(), float)
-            model.setData(index, to_qvariant(value), Qt.DisplayRole)
+            model.setData(index, value, Qt.DisplayRole)
 
 #-------------------------------------------------------------------------------
 # Model class
@@ -203,27 +216,27 @@ class StandardItemModelFaces(QStandardItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         row = index.row()
         col = index.column()
 
         if role == Qt.ToolTipRole:
-            return to_qvariant(self.tooltip[col])
+            return self.tooltip[col]
 
         if role == Qt.DisplayRole:
             if col == 0:
-                return to_qvariant(self.dataFaces[row]['fraction'])
+                return self.dataFaces[row]['fraction']
             elif col == 1:
-                return to_qvariant(self.dataFaces[row]['plane'])
+                return self.dataFaces[row]['plane']
             elif col == 2:
-                return to_qvariant(self.dataFaces[row]['verbosity'])
+                return self.dataFaces[row]['verbosity']
             elif col == 3:
-                return to_qvariant(self.dataFaces[row]['visualization'])
+                return self.dataFaces[row]['visualization']
             elif col == 4:
-                return to_qvariant(self.dataFaces[row]['selector'])
+                return self.dataFaces[row]['selector']
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -234,8 +247,8 @@ class StandardItemModelFaces(QStandardItemModel):
 
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
-            return to_qvariant(self.headers[section])
-        return to_qvariant()
+            return self.headers[section]
+        return None
 
 
     def setData(self, index, value, role):
@@ -365,13 +378,6 @@ class FacesSelectionView(QWidget, Ui_FacesSelectionForm):
             break
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
-
 #-------------------------------------------------------------------------------
 # Testing part
 #-------------------------------------------------------------------------------
