@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -47,19 +47,18 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import GuiParam
 from code_saturne.Base.QtPage import ComboModel, IntValidator, DoubleValidator
 from code_saturne.Base.QtPage import from_qvariant, to_text_string
 from code_saturne.Pages.LagrangianStatisticsForm import Ui_LagrangianStatisticsForm
-from code_saturne.Pages.LagrangianStatisticsModel import LagrangianStatisticsModel
-from code_saturne.Pages.LagrangianModel import LagrangianModel
+from code_saturne.model.LagrangianStatisticsModel import LagrangianStatisticsModel
+from code_saturne.model.LagrangianModel import LagrangianModel
 
 
 #-------------------------------------------------------------------------------
 # log config
 #-------------------------------------------------------------------------------
 
-
 logging.basicConfig()
 log = logging.getLogger("LagrangianStatisticsView")
 log.setLevel(GuiParam.DEBUG)
@@ -69,7 +68,6 @@ log.setLevel(GuiParam.DEBUG)
 # StandarItemModel for volumic variables names
 #-------------------------------------------------------------------------------
 
-
 class StandardItemModelVolumicNames(QStandardItemModel):
     def __init__(self, model):
         """
@@ -78,7 +76,7 @@ class StandardItemModelVolumicNames(QStandardItemModel):
 
         self.headers = [self.tr("Variable Name (Mean value)"),
                         self.tr("Variable Name (Variance)"),
-                        self.tr("Post-processing")]
+                        self.tr("Active")]
         self.setColumnCount(len(self.headers))
         self.model = model
         self.initData()
@@ -89,14 +87,17 @@ class StandardItemModelVolumicNames(QStandardItemModel):
         self.dataVolumicNames = []
         vnames = self.model.getVariablesNamesVolume()
         for vname in vnames:
-            if vname == "Part_statis_weight":
-                labelv = ""
-                postprocessing = self.model.getPostprocessingVolStatusFromName(vname)
-                line = [vname, labelv, postprocessing]
+            if vname[0:5] == "mean_":
+                labelv = "var_" + vname[5:]
+                compute = self.model.getComputeVolStatusFromName(vname)
+                line = [vname, labelv, compute]
+            elif vname[0:4] == "var_":
+                compute = self.model.getComputeVolStatusFromName(vname)
+                continue # group with mean
             else:
-                labelv = "var_" + vname
-                postprocessing = self.model.getPostprocessingVolStatusFromName(vname)
-                line = [vname, labelv, postprocessing]
+                labelv = ""
+                compute = self.model.getComputeVolStatusFromName(vname)
+                line = [vname, labelv, compute]
 
             row = self.rowCount()
             self.setRowCount(row+1)
@@ -166,7 +167,10 @@ class StandardItemModelVolumicNames(QStandardItemModel):
                 self.dataVolumicNames[index.row()][index.column()] = "on"
 
             vname = self.dataVolumicNames[index.row()][0]
-            self.model.setPostprocessingVolStatusFromName(vname, status)
+            self.model.setComputeVolStatusFromName(vname, status)
+            vname = self.dataVolumicNames[index.row()][1]
+            if vname:
+                self.model.setComputeVolStatusFromName(vname, status)
 
         self.dataChanged.emit(index, index)
         return True
@@ -176,15 +180,15 @@ class StandardItemModelVolumicNames(QStandardItemModel):
 # StandarItemModel for boundaries variables names
 #-------------------------------------------------------------------------------
 
-
 class StandardItemModelBoundariesNames(QStandardItemModel):
     def __init__(self, model):
         """
         """
         QStandardItemModel.__init__(self)
 
-        self.headers = [self.tr("Variable Name"),
-                        self.tr("Post-processing")]
+        self.headers = [self.tr("Variable Name (Mean value)"),
+                        self.tr("Variable Name (Variance)"),
+                        self.tr("Active")]
         self.setColumnCount(len(self.headers))
         self.model = model
         self.initData()
@@ -195,8 +199,17 @@ class StandardItemModelBoundariesNames(QStandardItemModel):
         self.dataBoundariesNames = []
         vnames = self.model.getVariablesNamesBoundary()
         for vname in vnames:
-            postprocessing = self.model.getPostprocessingStatusFromName(vname)
-            line = [vname, postprocessing]
+            if vname[0:5] == "mean_":
+                labelv = "var_" + vname[5:]
+                compute = self.model.getComputeBoundaryStatusFromName(vname)
+                line = [vname, labelv, compute]
+            elif vname[0:4] == "var_":
+                compute = self.model.getComputeBoundaryStatusFromName(vname)
+                continue # group with mean
+            else:
+                labelv = ""
+                compute = self.model.getComputeBoundaryStatusFromName(vname)
+                line = [vname, labelv, compute]
 
             row = self.rowCount()
             self.setRowCount(row+1)
@@ -205,26 +218,22 @@ class StandardItemModelBoundariesNames(QStandardItemModel):
 
     def data(self, index, role):
 
-        self.kwords = [ "IFLMBD",   "INBRBD",   "IANGBD",   "IVITBD",
-                        "IENCNBBD", "IENCMABD", "IENCDIBD", "IENCCKBD"]
         if not index.isValid():
             return
 
         # ToolTips
         if role == Qt.ToolTipRole:
-            if index.column() == 0:
-                return self.tr("Code_Saturne key word: NOMBRD")
-            elif index.column() == 1:
-                return self.tr("Code_Saturne key word: " + self.kwords[index.row()])
+            if index.column() in [0, 1]:
+                return self.tr("field label base")
 
         # Display
         if role == Qt.DisplayRole:
-            if index.column() == 0:
+            if index.column() in [0,1]:
                 return self.dataBoundariesNames[index.row()][index.column()]
 
         # CheckState
         elif role == Qt.CheckStateRole:
-            if index.column() ==1:
+            if index.column() ==2:
                 if self.dataBoundariesNames[index.row()][index.column()] == 'on':
                     return Qt.Checked
                 else:
@@ -250,7 +259,17 @@ class StandardItemModelBoundariesNames(QStandardItemModel):
 
     def setData(self, index, value, role):
 
-        if index.column() == 1:
+        if index.column() == 0:
+            self.dataBoundariesNames[index.row()][index.column()] = \
+                        str(from_qvariant(value, to_text_string))
+            vname = self.dataBoundariesNames[index.row()][0]
+
+        elif index.column() == 1:
+            labelv = str(from_qvariant(value, to_text_string))
+            self.dataBoundariesNames[index.row()][index.column()] = labelv
+            name = self.dataBoundariesNames[index.row()][0]
+
+        elif index.column() == 2:
             v = from_qvariant(value, int)
             if v == Qt.Unchecked:
                 status = "off"
@@ -260,8 +279,10 @@ class StandardItemModelBoundariesNames(QStandardItemModel):
                 self.dataBoundariesNames[index.row()][index.column()] = "on"
 
             vname = self.dataBoundariesNames[index.row()][0]
-            self.model.setPostprocessingStatusFromName(vname, status)
-
+            self.model.setComputeBoundaryStatusFromName(vname, status)
+            vname = self.dataBoundariesNames[index.row()][1]
+            if vname:
+                self.model.setComputeBoundaryStatusFromName(vname, status)
 
         self.dataChanged.emit(index, index)
         return True
@@ -271,7 +292,6 @@ class StandardItemModelBoundariesNames(QStandardItemModel):
 # Main class
 #-------------------------------------------------------------------------------
 
-
 class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
     """
     """
@@ -313,13 +333,6 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
         self.lineEditSEUIL.setValidator(validatorSEUIL)
 
         # initialize Widgets
-        # FIXME
-        # test if restart lagrangian is on
-##         mdl.lagr = LagrangianModel()
-##         is_restart = mdl_lagr.getRestart()
-##         if is_restart == "off":
-##             self.lineEditISUIST.setEnabled(False)
-##             self.checkBoxISUIST.setEnabled(False)
         status = self.model.getRestartStatisticsStatus()
         if status == "on":
             self.checkBoxISUIST.setChecked(True)
@@ -506,13 +519,6 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
             self.model.setIterationStartBoundary(value)
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
-
 #-------------------------------------------------------------------------------
 # Testing part
 #-------------------------------------------------------------------------------
