@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -45,12 +45,12 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Common import LABEL_LENGTH_MAX
-from code_saturne.Base.Toolbox import GuiParam
-from code_saturne.Base.QtPage import from_qvariant, to_qvariant, to_text_string
+from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam, GuiLabelManager
+from code_saturne.Base.QtPage import from_qvariant, to_text_string
 from code_saturne.Base.QtPage import DoubleValidator, RegExpValidator
+from code_saturne.Base.QtPage import LabelDelegate, FloatDelegate, ComboDelegate
 from code_saturne.Pages.NotebookForm import Ui_NotebookForm
-from code_saturne.Pages.NotebookModel import NotebookModel
+from code_saturne.model.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -60,6 +60,11 @@ logging.basicConfig()
 log = logging.getLogger("NotebookView")
 log.setLevel(GuiParam.DEBUG)
 
+#-------------------------------------------------------------------------------
+# List of labels which are forbidden as notebook variables names
+#-------------------------------------------------------------------------------
+
+_forbidden_labels = GuiLabelManager().getForbidden("notebook")
 
 #-------------------------------------------------------------------------------
 # item class
@@ -68,14 +73,17 @@ class item_class(object):
     '''
     custom data object
     '''
-    def __init__(self, idx, name, value):
+    def __init__(self, idx, name, value, oturns_var, editable, description):
         self.index  = idx
         self.name   = name
-        self.value = value
+        self.value  = value
+        self.oturns = oturns_var
+        self.edit   = editable
+        self.descr  = description
 
     def __repr__(self):
-        return "variable : %s // value : %s"\
-               % (self.name, self.value)
+        return "variable : %s // value : %s // used with openturns : %s"\
+               % (self.name, self.value, self.oturns)
 
 #-------------------------------------------------------------------------------
 # Treeitem class
@@ -105,21 +113,27 @@ class TreeItem(object):
 
 
     def columnCount(self):
-        return 2
+        return 5
 
 
     def data(self, column, role):
         if self.item == None:
             if column == 0:
-                return to_qvariant(self.header)
+                return self.header
             else:
-                return to_qvariant()
+                return None
         else:
             if column == 0 and role == Qt.DisplayRole:
-                return to_qvariant(self.item.name)
+                return self.item.name
             elif column == 1 and role == Qt.DisplayRole:
-                return to_qvariant(self.item.value)
-        return to_qvariant()
+                return self.item.value
+            elif column == 2 and role == Qt.DisplayRole:
+                return self.item.oturns
+            elif column == 3 and role == Qt.DisplayRole:
+                return self.item.edit
+            elif column == 4 and role == Qt.DisplayRole:
+                return self.item.descr
+        return None
 
 
     def parent(self):
@@ -132,75 +146,6 @@ class TreeItem(object):
         return 0
 
 
-#-------------------------------------------------------------------------------
-#
-#-------------------------------------------------------------------------------
-
-class LabelDelegate(QItemDelegate):
-    """
-    """
-    def __init__(self, parent=None, xml_model=None):
-        super(LabelDelegate, self).__init__(parent)
-        self.parent = parent
-        self.mdl = xml_model
-
-
-    def createEditor(self, parent, option, index):
-        editor = QLineEdit(parent)
-        rx = "[\-_A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX) + "}"
-        self.regExp = QRegExp(rx)
-        v =  RegExpValidator(editor, self.regExp)
-        editor.setValidator(v)
-        return editor
-
-
-    def setEditorData(self, editor, index):
-        v = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
-        self.p_value = str(v)
-        editor.setText(v)
-
-    def setModelData(self, editor, model, index):
-        if not editor.isModified():
-            return
-
-        if editor.validator().state == QValidator.Acceptable:
-            p_value = str(editor.text())
-            model.setData(index, to_qvariant(p_value), Qt.DisplayRole)
-
-
-#-------------------------------------------------------------------------------
-#
-#-------------------------------------------------------------------------------
-
-class ValueDelegate(QItemDelegate):
-    """
-    """
-    def __init__(self, parent=None, xml_model=None):
-        super(ValueDelegate, self).__init__(parent)
-        self.parent = parent
-        self.mdl = xml_model
-
-
-    def createEditor(self, parent, option, index):
-        editor = QLineEdit(parent)
-        vd = DoubleValidator(editor)
-        editor.setValidator(vd)
-        return editor
-
-
-    def setEditorData(self, editor, index):
-        value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
-        editor.setText(str(value))
-
-    def setModelData(self, editor, model, index):
-        if not editor.isModified():
-            return
-
-        if editor.validator().state == QValidator.Acceptable:
-            value = from_qvariant(editor.text(), float)
-            model.setData(index, to_qvariant(value), Qt.DisplayRole)
-
-
 #-------------------------------------------------------------------------------
 # StandarItemModelOutput class
 #-------------------------------------------------------------------------------
@@ -227,31 +172,37 @@ class VariableStandardItemModel(QAbstractItemModel):
         if parent and parent.isValid():
             return parent.internalPointer().columnCount()
         else:
-            return 2
+            return 5
 
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         item = index.internalPointer()
 
         # ToolTips
         if role == Qt.ToolTipRole:
-            return to_qvariant()
+            return None
 
         # StatusTips
         if role == Qt.StatusTipRole:
             if index.column() == 0:
-                return to_qvariant(self.tr("variable name"))
+                return self.tr("variable name")
             elif index.column() == 1:
-                return to_qvariant(self.tr("value"))
+                return self.tr("value")
+            elif index.column() == 2:
+                return self.tr("OpenTurns Variable")
+            elif index.column() == 3:
+                return self.tr("Editable")
+            elif index.column() == 4:
+                return self.tr("Description")
 
         # Display
         if role == Qt.DisplayRole:
             return item.data(index.column(), role)
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -263,16 +214,28 @@ class VariableStandardItemModel(QAbstractItemModel):
             return Qt.ItemIsEnabled
 
         itm = index.internalPointer()
-        return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable
+        if index.column() == 3:
+            if "Yes" in self.mdl.getVariableOt(index.row()):
+                return Qt.NoItemFlags
+            else:
+                return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable
+        else:
+            return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable
 
 
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
             if section == 0:
-                return to_qvariant(self.tr("variable name"))
+                return self.tr("variable name")
             elif section == 1:
-                return to_qvariant(self.tr("value"))
-        return to_qvariant()
+                return self.tr("value")
+            elif section == 2:
+                return self.tr("OpenTurns Variable")
+            elif section == 3:
+                return self.tr("Editable")
+            elif section == 4:
+                return self.tr("Description")
+        return None
 
 
     def index(self, row, column, parent = QModelIndex()):
@@ -324,9 +287,12 @@ class VariableStandardItemModel(QAbstractItemModel):
     def populateModel(self):
         for idx in self.mdl.getVarList():
             parentItem = self.rootItem
-            cname = self.mdl.getVariableName(idx)
+            cname  = self.mdl.getVariableName(idx)
             value  = self.mdl.getVariableValue(idx)
-            item = item_class(idx, cname, value)
+            oturns = self.mdl.getVariableOt(idx)
+            edit   = self.mdl.getVariableEditable(idx)
+            descr  = self.mdl.getVariableDescription(idx)
+            item = item_class(idx, cname, value, oturns, edit, descr)
             new_item = TreeItem(item, cname, parentItem)
             parentItem.appendChild(new_item)
 
@@ -342,8 +308,22 @@ class VariableStandardItemModel(QAbstractItemModel):
         elif index.column() == 1:
             value = str(from_qvariant(value, float))
             item.item.value = value
-            self.mdl.setVariableValue(item.item.index, item.item.value)
+            self.mdl.setVariableValue(item.item.value, idx=item.item.index)
 
+        elif index.column() == 2:
+            value = from_qvariant(value, to_text_string)
+            item.item.oturns = value
+            self.mdl.setVariableOt(item.item.index, item.item.oturns)
+
+        elif index.column() == 3:
+            editable = from_qvariant(value, to_text_string)
+            item.item.edit = editable
+            self.mdl.setVariableEditable(item.item.index, item.item.edit)
+
+        elif index.column() == 4:
+            description = from_qvariant(value, to_text_string)
+            item.item.descr = description
+            self.mdl.setVariableDescription(item.item.index, item.item.descr)
         self.dataChanged.emit(QModelIndex(), QModelIndex())
 
         return True
@@ -382,22 +362,58 @@ class NotebookView(QWidget, Ui_NotebookForm):
         self.treeViewNotebook.setSelectionBehavior(QAbstractItemView.SelectRows)
         self.treeViewNotebook.setDragEnabled(False)
 
-        nameDelegate = LabelDelegate(self.treeViewNotebook, self.mdl)
+        nameDelegate = LabelDelegate(self.treeViewNotebook,
+                                     xml_model=self.mdl,
+                                     forbidden_labels=_forbidden_labels)
         self.treeViewNotebook.setItemDelegateForColumn(0, nameDelegate)
 
-        valDelegate = ValueDelegate(self.treeViewNotebook, self.mdl)
+        valDelegate = FloatDelegate(self.treeViewNotebook, self.mdl)
         self.treeViewNotebook.setItemDelegateForColumn(1, valDelegate)
 
+        OTOptions = ["No", "Yes: Input", "Yes: Output"]
+        otvalDelegate = ComboDelegate(self.treeViewNotebook,
+                                      opts_list=OTOptions)
+        self.treeViewNotebook.setItemDelegateForColumn(2, otvalDelegate)
+
+        EditableOptions = ["No","Yes"]
+        editableDelegate = ComboDelegate(self.treeViewNotebook,
+                                         opts_list=EditableOptions)
+        self.treeViewNotebook.setItemDelegateForColumn(3, editableDelegate)
+
+        descriptionDelegate = LabelDelegate(self.treeViewNotebook)
+        self.treeViewNotebook.setItemDelegateForColumn(4, descriptionDelegate)
+
         self.treeViewNotebook.resizeColumnToContents(0)
+        self.treeViewNotebook.resizeColumnToContents(2)
+        self.treeViewNotebook.resizeColumnToContents(4)
 
         # Connections
         self.toolButtonAdd.clicked.connect(self.slotAddVariable)
         self.toolButtonDelete.clicked.connect(self.slotDeleteVariable)
         self.toolButtonImport.clicked.connect(self.slotImportVariable)
 
+        self.modelVar.dataChanged.connect(self.dataChanged)
+
         self.case.undoStartGlobal()
 
 
+    def dataChanged(self):
+        row = self.treeViewNotebook.currentIndex().row()
+        col = self.treeViewNotebook.currentIndex().column()
+        next_item = self.treeViewNotebook.model().index(row, col+1)
+        # Check for updates
+        ot_status = self.mdl.getVariableOt(row)
+        edit_status = self.mdl.getVariableEditable(row)
+        if ot_status == "Yes: Input" and edit_status == "Yes":
+            self.mdl.setVariableEditable(row, "No")
+            self.modelVar.setData(next_item, "No")
+        elif ot_status == "Yes: Output" and edit_status == "No":
+            self.mdl.setVariableEditable(row, "Yes")
+            self.modelVar.setData(next_item, "Yes")
+
+        self.update()
+
+
     @pyqtSlot()
     def slotAddVariable(self):
         """
@@ -445,13 +461,6 @@ class NotebookView(QWidget, Ui_NotebookForm):
         self.treeViewNotebook.expandAll()
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
-
 #-------------------------------------------------------------------------------
 # Testing part
 #-------------------------------------------------------------------------------
