@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -45,10 +45,11 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import GuiParam
 from code_saturne.Base.QtPage import ComboModel, DoubleValidator, from_qvariant
 from code_saturne.Pages.NumericalParamGlobalForm import Ui_NumericalParamGlobalForm
-from code_saturne.Pages.NumericalParamGlobalModel import NumericalParamGlobalModel
+from code_saturne.model.NumericalParamGlobalModel import NumericalParamGlobalModel
+from code_saturne.model.GroundwaterModel import GroundwaterModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -82,32 +83,50 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
 
         self.labelSRROM.hide()
         self.lineEditSRROM.hide()
-        self.line_5.hide()
 
         # Combo models
-        self.modelEXTRAG = ComboModel(self.comboBoxEXTRAG,2,1)
-        self.modelIMRGRA = ComboModel(self.comboBoxIMRGRA,5,1)
+        self.modelGradientType = ComboModel(self.comboBoxGradientType, 5, 1)
+        self.modelGradientType.addItem(self.tr("Automatic"), 'default')
+        self.modelGradientType.addItem(self.tr("Green-Gauss with iterative handling of non-orthogonalities"),
+                                       'green_iter')
+        self.modelGradientType.addItem(self.tr("Least squares"), 'lsq')
+        self.modelGradientType.addItem(self.tr("Green-Gauss with least squares gradient face values"),
+                                       'green_lsq')
+        self.modelGradientType.addItem(self.tr("Green-Gauss with vertex interpolated face values"),
+                                       'green_vtx')
 
-        self.modelEXTRAG.addItem(self.tr("Neumann 1st order"), 'neumann')
-        self.modelEXTRAG.addItem(self.tr("Extrapolation"), 'extrapolation')
+        self.modelExtNeighbors = ComboModel(self.comboBoxExtNeighbors, 5, 1)
+        self.modelExtNeighbors.addItem(self.tr("Automatic"), 'default')
+        self.modelExtNeighbors.addItem(self.tr("None (face adjacent only)"), 'none')
+        self.modelExtNeighbors.addItem(self.tr("Full (all vertex adjacent)"), 'complete')
+        self.modelExtNeighbors.addItem(self.tr("Opposite adjacent cell centers"),
+                                       'cell_center_opposite')
+        self.modelExtNeighbors.addItem(self.tr("Non-orthogonal faces threshold (legacy)"),
+                                       'non_ortho_max')
 
-        self.modelIMRGRA.addItem(self.tr("Iterative handling of non-orthogonalities"),'0')
-        self.modelIMRGRA.addItem(self.tr("Least squares method over neighboring cells"),'1')
-        self.modelIMRGRA.addItem(self.tr("Least squares method over extended cell neighborhood"),'2')
-        self.modelIMRGRA.addItem(self.tr("Least squares method over partial extended cell neighborhood"),'3')
-        self.modelIMRGRA.addItem(self.tr("Iterative method with least squares initialization"),'4')
-
-        self.comboBoxEXTRAG.setSizeAdjustPolicy(QComboBox.AdjustToContents)
+        self.modelDensityVar = ComboModel(self.comboBoxDensityVar, 6, 1)
+        self.modelDensityVar.addItem(self.tr("Automatic"), 'default')
+        self.modelDensityVar.addItem(self.tr("Boussinesq approximation (rho constant except in the buoyant term)"),
+                                       'boussi')
+        self.modelDensityVar.addItem(self.tr("Dilatable steady algorithm"), 'dilat_std')
+        self.modelDensityVar.addItem(self.tr("Dilatable unsteady algorithm"),
+                                       'dilat_unstd')
+        self.modelDensityVar.addItem(self.tr("Low-Mach algorithm"),
+                                       'low_mach')
+        self.modelDensityVar.addItem(self.tr("Algorithm for fire"),
+                                       'algo_fire')
 
         # Connections
         self.checkBoxIVISSE.clicked.connect(self.slotIVISSE)
+
         self.checkBoxIPUCOU.clicked.connect(self.slotIPUCOU)
-        self.checkBoxICFGRP.clicked.connect(self.slotICFGRP)
         self.checkBoxImprovedPressure.clicked.connect(self.slotImprovedPressure)
-        self.comboBoxEXTRAG.activated[str].connect(self.slotEXTRAG)
+        self.checkBoxICFGRP.clicked.connect(self.slotICFGRP)
         self.lineEditRELAXP.textChanged[str].connect(self.slotRELAXP)
-        self.comboBoxIMRGRA.activated[str].connect(self.slotIMRGRA)
+        self.comboBoxGradientType.activated[str].connect(self.slotGradientType)
+        self.comboBoxExtNeighbors.activated[str].connect(self.slotExtNeighbors)
         self.lineEditSRROM.textChanged[str].connect(self.slotSRROM)
+        self.comboBoxDensityVar.activated[str].connect(self.slotDensityVar)
 
         # Validators
         validatorRELAXP = DoubleValidator(self.lineEditRELAXP, min=0., max=1.)
@@ -127,9 +146,11 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
         else:
             self.checkBoxIPUCOU.setChecked(False)
 
-        import code_saturne.Pages.FluidCharacteristicsModel as FluidCharacteristics
+        import code_saturne.model.FluidCharacteristicsModel as FluidCharacteristics
         fluid = FluidCharacteristics.FluidCharacteristicsModel(self.case)
-        modl_atmo, modl_joul, modl_thermo, modl_gas, modl_coal, modl_comp = fluid.getThermoPhysicalModel()
+        modl_atmo, modl_joul, modl_thermo, modl_gas, modl_coal, modl_comp, modl_hgn = \
+            fluid.getThermoPhysicalModel()
+        modl_gwf  = GroundwaterModel(self.case).getGroundwaterModel()
 
         if self.model.getHydrostaticPressure() == 'on':
             self.checkBoxImprovedPressure.setChecked(True)
@@ -137,47 +158,55 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
             self.checkBoxImprovedPressure.setChecked(False)
 
         self.lineEditRELAXP.setText(str(self.model.getPressureRelaxation()))
-        self.modelEXTRAG.setItem(str_model=self.model.getWallPressureExtrapolation())
-        self.modelIMRGRA.setItem(str_model=str(self.model.getGradientReconstruction()))
+        self.modelGradientType.setItem(str_model=str(self.model.getGradientReconstruction()))
+        self.modelExtNeighbors.setItem(str_model=str(self.model.getExtendedNeighborType()))
+        self.modelDensityVar.setItem(str_model=str(self.model.getDensityVar()))
+
+        if self.model.getGradientReconstruction() == 'green_iter':
+            self.labelExtNeighbors.hide()
+            self.comboBoxExtNeighbors.hide()
+
+        from code_saturne.model.TimeStepModel import TimeStepModel
+        idtvar = TimeStepModel(self.case).getTimePassing()
 
         if modl_joul != 'off' or modl_gas != 'off' or modl_coal != 'off':
             self.labelSRROM.show()
             self.lineEditSRROM.show()
             self.lineEditSRROM.setText(str(self.model.getDensityRelaxation()))
-            self.line_5.show()
 
         if modl_comp != 'off':
-            self.labelICFGRP.show()
             self.checkBoxICFGRP.show()
-            self.line_4.show()
             if self.model.getHydrostaticEquilibrium() == 'on':
                 self.checkBoxICFGRP.setChecked(True)
             else:
                 self.checkBoxICFGRP.setChecked(False)
             self.checkBoxIPUCOU.hide()
-            self.labelIPUCOU.hide()
             self.lineEditRELAXP.hide()
             self.labelRELAXP.hide()
             self.checkBoxImprovedPressure.hide()
-            self.labelImprovedPressure.hide()
-            self.line_2.hide()
-            self.line_5.hide()
-            self.line_7.hide()
-            self.line_8.hide()
         else:
-            self.labelICFGRP.hide()
             self.checkBoxICFGRP.hide()
-            self.line_4.hide()
-            self.checkBoxIPUCOU.show()
-            self.labelIPUCOU.show()
+            if idtvar == -1:
+                self.checkBoxIPUCOU.setEnabled(False)
+            else:
+                self.checkBoxIPUCOU.show()
             self.lineEditRELAXP.show()
             self.labelRELAXP.show()
             self.checkBoxImprovedPressure.show()
-            self.labelImprovedPressure.show()
-            self.line_2.show()
-            self.line_5.show()
-            self.line_7.show()
-            self.line_8.show()
+
+        self.modelDensityVar.disableItem(str_model = 'algo_fire')
+        if modl_gas != 'off':
+            self.modelDensityVar.enableItem(str_model = 'algo_fire')
+
+        # For the moment, the Low Mach algorithm is disabled in the GUI
+        self.modelDensityVar.disableItem(str_model = 'low_mach')
+
+        # Hide non relevant info for ground water flows
+
+        if modl_gwf != 'off':
+            self.groupBoxTerms.hide()
+            self.groupBoxVelPres.hide()
+            self.groupBoxDensityVar.hide()
 
         # Update the Tree files and folders
         self.browser.configureTree(self.case)
@@ -229,16 +258,6 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
             self.model.setHydrostaticPressure("off")
 
 
-    @pyqtSlot(str)
-    def slotEXTRAG(self, text):
-        """
-        Set value for parameter EXTRAG
-        """
-        extrag = self.modelEXTRAG.dicoV2M[str(text)]
-        self.model.setWallPressureExtrapolation(extrag)
-        log.debug("slotEXTRAG-> %s" % extrag)
-
-
     @pyqtSlot(str)
     def slotRELAXP(self, text):
         """
@@ -262,21 +281,39 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
 
 
     @pyqtSlot(str)
-    def slotIMRGRA(self, text):
+    def slotGradientType(self, text):
         """
-        Set value for parameter IMRGRA
+        Set value for parameter GradientType
         """
-        imrgra = self.modelIMRGRA.dicoV2M[str(text)]
-        self.model.setGradientReconstruction(int(imrgra))
-        log.debug("slotIMRGRA-> %s" % imrgra)
+        grd_type = self.modelGradientType.dicoV2M[str(text)]
+        self.model.setGradientReconstruction(grd_type)
+        log.debug("slotGradientType-> %s" % grd_type)
+
+        if grd_type == 'green_iter':
+            self.labelExtNeighbors.hide()
+            self.comboBoxExtNeighbors.hide()
+        else:
+            self.labelExtNeighbors.show()
+            self.comboBoxExtNeighbors.show()
 
 
-    def tr(self, text):
+    @pyqtSlot(str)
+    def slotExtNeighbors(self, text):
         """
-        Translation
+        Set extended neighborhood type
         """
-        return text
+        enh_type = self.modelExtNeighbors.dicoV2M[str(text)]
+        self.model.setExtendedNeighborType(enh_type)
+        log.debug("slotExtNeighbors-> %s" % enh_type)
 
+    @pyqtSlot(str)
+    def slotDensityVar(self, text):
+        """
+        Set algorithm for density variation in time
+        """
+        dv_type = self.modelDensityVar.dicoV2M[str(text)]
+        self.model.setDensityVar(dv_type)
+        log.debug("slotDensityVar-> %s" % dv_type)
 
 #-------------------------------------------------------------------------------
 # Testing part
