@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -32,7 +32,7 @@ This module contains the following classes and function:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import sys, re, os, string, logging
+import sys, re, os, logging
 import os.path
 
 #-------------------------------------------------------------------------------
@@ -46,7 +46,7 @@ from code_saturne.Base.QtGui  import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import GuiParam
 from code_saturne.Base.QtPage import getopenfilename
 
 #-------------------------------------------------------------------------------
@@ -66,10 +66,10 @@ def preprocessorFile(parent, initdir):
     Verify if the choses of file is correct
     """
     file_name = ""
-    title = tr("Select a Code_Saturne Preprocessor listing")
+    title = parent.tr("Select a preprocessor log")
     filetypes = "Preprocessor log (*.log);;All Files (*)"
     filt = "All files (*)"
-    initdir = os.path.join(initdir, 'check_mesh')
+    initdir = os.path.join(initdir, 'preprocessor.log')
     file_name, _selfilter = getopenfilename(parent, title, initdir, filetypes)
     file_name = str(file_name)
 
@@ -80,27 +80,20 @@ def preprocessorFile(parent, initdir):
 
         j=-1
         for i in range(len(lines)):
-            index = string.rfind(lines[i], "Code_Saturne")
+            index = lines[i].rfind("Code_Saturne")
             if index != -1:
                 j = i
                 break
 
         if j == -1:
-            title = tr("Informations")
-            msg = tr("Warning : the selected file is not a correct file.\n\n"\
-                     "Verify your selection")
+            title = parent.tr("Informations")
+            msg = parent.tr("Warning : the selected file is not a correct file.\n\n"\
+                            "Verify your selection")
             QMessageBox.information(parent, title, msg)
 
     return file_name
 
 
-
-def tr(text):
-    """
-    Translation
-    """
-    return text
-
 #-------------------------------------------------------------------------------
 # Informations class
 #-------------------------------------------------------------------------------
@@ -114,15 +107,13 @@ class Informations:
 
         lines = self.readFile(file)
         if not lines:
-            raise ValueError("Code_Saturne Preprocessor log language unknown.")
+            raise ValueError("Preprocessor log language unknown.")
 
         refList, groupList = self.getLists(lines)
 
         self.refList = refList
         self.groupList = groupList
 
-        self.updateListes()
-
 
     def readFile(self, file):
         if not file:
@@ -134,7 +125,7 @@ class Informations:
 
             lang = ""
             for i in range(len(lines)):
-                index = string.find(lines[i], "familles de faces et cellules")
+                index = lines[i].find("familles de faces et cellules")
                 if index > 0:
                     lang = 'fr'
                     break
@@ -196,11 +187,6 @@ class Informations:
         return refList, groupList
 
 
-    # TODO: delete this method
-    def updateListes(self):
-        return self.refList, self.groupList
-
-
     def getLocalizations(self):
         lst = self.refList + self.groupList
         return list(map(str, lst))
