@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -23,8 +23,6 @@
 #-------------------------------------------------------------------------------
 
 """
-This module defines the 'Meshes and Enveloppe' page.
-
 This module contains the following classes:
 - ProfilesView
 """
@@ -33,7 +31,6 @@ This module contains the following classes:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import string
 import logging
 
 #-------------------------------------------------------------------------------
@@ -48,13 +45,13 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Common import LABEL_LENGTH_MAX
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam
 from code_saturne.Base.QtPage import IntValidator, DoubleValidator, RegExpValidator, ComboModel
-from code_saturne.Base.QtPage import to_qvariant, from_qvariant
+from code_saturne.Base.QtPage import from_qvariant
 from code_saturne.Pages.ProfilesForm import Ui_ProfilesForm
-from code_saturne.Pages.ProfilesModel import ProfilesModel
-from code_saturne.Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.model.ProfilesModel import ProfilesModel
+from code_saturne.Pages.QMegEditorView import QMegEditorView
+from code_saturne.model.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -80,12 +77,12 @@ class StandardItemModelProfile(QStandardItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
         if role == Qt.DisplayRole:
-            return to_qvariant(self.dataProfile[index.row()][index.column()])
+            return self.dataProfile[index.row()][index.column()]
         elif role == Qt.TextAlignmentRole:
-            return to_qvariant(Qt.AlignCenter)
-        return to_qvariant()
+            return Qt.AlignCenter
+        return None
 
 
     def flags(self, index):
@@ -98,12 +95,12 @@ class StandardItemModelProfile(QStandardItemModel):
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
             if section == 0:
-                return to_qvariant(self.tr("Filename"))
+                return self.tr("Filename")
             elif section == 1:
-                return to_qvariant(self.tr("Variables"))
+                return self.tr("Variables")
         elif role == Qt.TextAlignmentRole:
-            return to_qvariant(Qt.AlignCenter)
-        return to_qvariant()
+            return Qt.AlignCenter
+        return None
 
 
     def setData(self, index, value, role):
@@ -168,6 +165,7 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
         self.case = case
         self.case.undoStopGlobal()
         self.mdl = ProfilesModel(self.case)
+        self.notebook = NotebookModel(self.case)
 
         #  Initialize variables concerning the display of the Hlist
 
@@ -222,6 +220,9 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
         self.lineEditFreqTime.textChanged[str].connect(self.slotFrequenceTime)
         self.lineEditNbPoint.textChanged[str].connect(self.slotNbPoint)
 
+        self.checkBoxSnapToCenter.clicked.connect(self.slotSnapToCenter)
+        self.checkBoxActivateInterpolation.clicked.connect(self.slotActivateInterpolation)
+
         # Validators
         validatorFreq = IntValidator(self.lineEditFreq, min=0)
         validatorFreq.setExclusiveMin(True)
@@ -401,12 +402,28 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
 
         self.lineEditNbPoint.setText(str(nb_point))
 
+        if self.mdl.getSnapMode(label) == "snap_to_center":
+            self.checkBoxSnapToCenter.setChecked(True)
+        else:
+            self.checkBoxSnapToCenter.setChecked(False)
+
+        if self.mdl.getProfileInterpolation(label) == "yes":
+            self.checkBoxActivateInterpolation.setChecked(True)
+        else:
+            self.checkBoxActivateInterpolation.setChecked(False)
+
+
         self.modelDrop.setStringList([])
         liste = [str(s) for s in liste]
 
         self.modelDrop.setStringList(liste)
 
-        self.pushButtonFormula.setStyleSheet("background-color: red")
+        exp = self.mdl.getFormula(self.label_select)
+        if exp:
+            self.pushButtonFormula.setStyleSheet("background-color: green")
+            self.pushButtonFormula.setToolTip(exp)
+        else:
+            self.pushButtonFormula.setStyleSheet("background-color: red")
 
 
     @pyqtSlot()
@@ -473,8 +490,13 @@ z = z1*s + z0*(1.-s);"""
                ('z', "z formula")]
         sym = [('s', 'parameter')]
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.case['package'].get_check_syntax(),
+        for (nme, val) in self.notebook.getNotebookList():
+            sym.append((nme, 'value (notebook) = ' + str(val)))
+
+        dialog = QMegEditorView(self,
+                                function_type = 'pfl',
+                                zone_name     = self.label_select,
+                                variable_name = 'coords',
                                 expression = exp,
                                 required   = req,
                                 symbols    = sym,
@@ -526,11 +548,23 @@ z = z1*s + z0*(1.-s);"""
             self.mdl.setNbPoint(self.label_select, int(text))
 
 
-    def tr(self, text):
+    @pyqtSlot()
+    def slotSnapToCenter(self):
         """
-        Translation
         """
-        return text
+        if self.checkBoxSnapToCenter.isChecked():
+            self.mdl.setSnapMode(self.label_select, "snap_to_center")
+        else:
+            self.mdl.setSnapMode(self.label_select, "none")
+
+    @pyqtSlot()
+    def slotActivateInterpolation(self):
+        """
+        """
+        if self.checkBoxActivateInterpolation.isChecked():
+            self.mdl.setProfileInterpolation(self.label_select, "yes")
+        else:
+            self.mdl.setProfileInterpolation(self.label_select, "no")
 
 
 #-------------------------------------------------------------------------------
