@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -32,8 +32,9 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.QtCore import *
-from code_saturne.Base.QtGui  import *
+from code_saturne.Base.QtCore     import *
+from code_saturne.Base.QtGui      import *
+from code_saturne.Base.QtWidgets  import *
 
 import vtk
 
@@ -43,24 +44,21 @@ import SMESH
 import CFDSTUDYGUI_DataModel
 
 try:
-    from libvtkRenderingCorePython import *
-    from libvtkFiltersSourcesPython import *
+    from vtkmodules.vtkRenderingCorePython import *
+    from vtkmodules.vtkFiltersSourcesPython import *
 except:
     try:
+        from vtkmodules.vtkRenderingCore import *
+        from vtkmodules.vtkFiltersSources import *
+    except:
         from vtkRenderingCorePython import *
         from vtkFiltersSourcesPython import *
-    except:
-        # for compatibility with salome 6.6
-        try:
-            from libvtkRenderingPython import *
-        except:
-            from vtkRenderingPython import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import GuiParam
 
 #-------------------------------------------------------------------------------
 # log config
@@ -585,6 +583,3 @@ class ProfilActors(Actors):
 #-------------------------------------------------------------------------------
 #
 #-------------------------------------------------------------------------------
-
-
-
