@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -45,10 +45,10 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import GuiParam
 from code_saturne.Base.QtPage import ComboModel, IntValidator, DoubleValidator, from_qvariant
 from code_saturne.Pages.TimeStepForm import Ui_TimeStepForm
-from code_saturne.Pages.TimeStepModel import TimeStepModel
+from code_saturne.model.TimeStepModel import TimeStepModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -79,34 +79,48 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
         self.mdl = TimeStepModel(self.case)
         self.browser = tree
 
-       # Combo model
+        # Check if xml contains deprecated steady algorithme
+        has_deprecated_steady = False
+        if str(self.mdl.getTimePassing()) == '-1':
+            has_deprecated_steady = True
 
-        self.modelTimeOptions = ComboModel(self.comboBoxOptions,4,1)
+       # Combo model
+        if has_deprecated_steady:
+            self.modelTimeOptions = ComboModel(self.comboBoxOptions,4,1)
+        else:
+            self.modelTimeOptions = ComboModel(self.comboBoxOptions,3,1)
         self.modelTimeOptions.addItem(self.tr("Constant"), '0')
         self.modelTimeOptions.addItem(self.tr("Time varying (adaptive)"), '1')
-        self.modelTimeOptions.addItem(self.tr("Space & time varying (pseudo-steady)"), '2')
-        self.modelTimeOptions.addItem(self.tr("Steady (constant relaxation coefficient)"), '-1')
+        self.modelTimeOptions.addItem(self.tr("Steady (local time step)"), '2')
+        if has_deprecated_steady:
+            self.modelTimeOptions.addItem(self.tr("Deprecated: steady (constant relaxation coefficient)"), '-1')
 
         self.modelNTERUP = ComboModel(self.comboBoxNTERUP,3,1)
         self.modelNTERUP.addItem(self.tr("SIMPLE"), 'simple')
         self.modelNTERUP.addItem(self.tr("SIMPLEC"), 'simplec')
-        self.modelNTERUP.addItem(self.tr("PISO"), 'piso')
+        self.modelNTERUP.addItem(self.tr("Inner iterations"), 'piso')
         self.comboBoxNTERUP.setSizeAdjustPolicy(QComboBox.AdjustToContents)
 
+        self.modelTimeStop = ComboModel(self.comboBoxStopCrit, 2, 1)
+        self.modelTimeStop.addItem(self.tr("Number of time steps"), "iterations")
+        self.modelTimeStop.addItem(self.tr("Physical time (s)"), "maximum_time")
+        self.modelTimeStop.addItem(self.tr("Additional time steps"), "iterations_add")
+        self.modelTimeStop.addItem(self.tr("Additional physical time (s)"), "maximum_time_add")
+
         # Connections
         self.comboBoxOptions.activated[str].connect(self.slotTimePassing)
         self.lineEditDTREF.textChanged[str].connect(self.slotTimeStep)
         self.lineEditRELXST.textChanged[str].connect(self.slotRelaxCoef)
-        self.lineEditNTMABS.textChanged[str].connect(self.slotIter)
         self.lineEditCOUMAX.textChanged[str].connect(self.slotTimeOptionCOUMAX)
         self.lineEditFOUMAX.textChanged[str].connect(self.slotTimeOptionFOUMAX)
         self.lineEditCDTMIN.textChanged[str].connect(self.slotTimeOptionCDTMIN)
         self.lineEditCDTMAX.textChanged[str].connect(self.slotTimeOptionCDTMAX)
         self.lineEditVARRDT.textChanged[str].connect(self.slotTimeOptionVARRDT)
         self.checkBoxIPTLRO.clicked.connect(self.slotThermalTimeStep)
-        self.checkBoxINPDT0.clicked.connect(self.slotZeroTimeStep)
         self.comboBoxNTERUP.activated[str].connect(self.slotNTERUP)
         self.spinBoxNTERUP.valueChanged[int].connect(self.slotNTERUP2)
+        self.comboBoxStopCrit.activated[str].connect(self.slotStopCritModel)
+        self.lineEditStop.textChanged[str].connect(self.slotStopCritValue)
 
         # Validators
 
@@ -114,7 +128,6 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
         validatorDTREF.setExclusiveMin(True)
         validatorRELXST = DoubleValidator(self.lineEditRELXST, min=0.0, max=1.0)
         validatorRELXST.setExclusiveMin(True)
-        validatorNTMABS = IntValidator(self.lineEditNTMABS, min=1)
         validatorCOUMAX = DoubleValidator(self.lineEditCOUMAX, min=0.0)
         validatorCOUMAX.setExclusiveMin(True)
         validatorFOUMAX = DoubleValidator(self.lineEditFOUMAX, min=0.0)
@@ -122,18 +135,20 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
         validatorCDTMIN = DoubleValidator(self.lineEditCDTMIN, min=0.0, max=1.0)
         validatorCDTMIN.setExclusiveMin(True)
         validatorCDTMAX = DoubleValidator(self.lineEditCDTMAX, min=1.0)
-        validatorVARRDT = DoubleValidator(self.lineEditVARRDT, min=0.0, max=1.0)
+        validatorVARRDT = DoubleValidator(self.lineEditVARRDT, min=0.0)
         validatorVARRDT.setExclusiveMin(True)
 
         self.lineEditDTREF.setValidator(validatorDTREF)
         self.lineEditRELXST.setValidator(validatorRELXST)
-        self.lineEditNTMABS.setValidator(validatorNTMABS)
         self.lineEditCOUMAX.setValidator(validatorCOUMAX)
         self.lineEditFOUMAX.setValidator(validatorFOUMAX)
         self.lineEditCDTMIN.setValidator(validatorCDTMIN)
         self.lineEditCDTMAX.setValidator(validatorCDTMAX)
         self.lineEditVARRDT.setValidator(validatorVARRDT)
 
+        self.validatorNTABS = IntValidator(self.lineEditStop, min=0)
+        self.validatorTABS = DoubleValidator(self.lineEditStop, min=0.0)
+
         # Initialization
 
         idtvar = self.mdl.getTimePassing()
@@ -141,79 +156,81 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
 
         # Constraints on time step from Turbulence model
 
-        from code_saturne.Pages.TurbulenceModel import TurbulenceModel
+        from code_saturne.model.TurbulenceModel import TurbulenceModel
         model = TurbulenceModel(self.case).getTurbulenceModel()
         del TurbulenceModel
 
         if model in ('LES_Smagorinsky', 'LES_dynamique', 'LES_WALE'):
             idtvar = 0
-            self.modelTimeOptions.disableItem(str_model='1')
-            self.modelTimeOptions.disableItem(str_model='2')
-            self.modelTimeOptions.disableItem(str_model='-1')
+
+            # Deactivate all modes except constant time step
+            for itm in self.modelTimeOptions.getItems():
+                if str(itm) != "0":
+                    self.modelTimeOptions.disableItem(str_model=itm)
 
         # Constraints on time step from Lagrangian model
 
-        from code_saturne.Pages.LagrangianModel import LagrangianModel
+        from code_saturne.model.LagrangianModel import LagrangianModel
         model = LagrangianModel(self.case).getLagrangianModel()
         if model in ['one_way', 'two_way']:
             if idtvar not in [0, 1]:
                 idtvar = 0
-            self.modelTimeOptions.disableItem(str_model='2')
-            self.modelTimeOptions.disableItem(str_model='-1')
-            if model == 'two_way':
-                idtvar = 0
-                self.modelTimeOptions.disableItem(str_model='1')
+
+            # Deactivate certain modes
+            for itm in self.modelTimeOptions.getItems():
+                if str(itm) == "2":
+                    self.modelTimeOptions.disableItem(str_model=itm)
+                elif str(itm) == "1" and model == 'two_way':
+                    self.modelTimeOptions.disableItem(str_model=itm)
 
         # Constraints on time step from compressible model
 
-        from code_saturne.Pages.CompressibleModel import CompressibleModel
+        from code_saturne.model.CompressibleModel import CompressibleModel
         model = CompressibleModel(self.case).getCompressibleModel()
         if model != 'off':
             if idtvar not in [0, 1]:
                 idtvar = 0
-            self.modelTimeOptions.disableItem(str_model='1')
-            self.modelTimeOptions.disableItem(str_model='2')
-            self.modelTimeOptions.disableItem(str_model='-1')
+
+            # Deactivate steady time step
+            # Using list to allow additions in the future
+            for itm in self.modelTimeOptions.getItems():
+                if str(itm) in ["2"]:
+                    self.modelTimeOptions.disableItem(str_model=itm)
+
             self.labelNTERUP.setText("Velocity-Pressure algorithm\nsub-iterations on Navier-Stokes")
             self.comboBoxNTERUP.hide()
             self.spinBoxNTERUP.show()
 
         # Constraints on time step from groundwater model
 
-        from code_saturne.Pages.GroundwaterModel import GroundwaterModel
+        from code_saturne.model.GroundwaterModel import GroundwaterModel
         model = GroundwaterModel(self.case).getGroundwaterModel()
         if model != 'off':
             if idtvar not in [0, 1]:
                 idtvar = 0
-            self.modelTimeOptions.disableItem(str_model='1')
-            self.modelTimeOptions.disableItem(str_model='2')
-            self.modelTimeOptions.disableItem(str_model='-1')
+
+            # Deactivate all modes except constant time step
+            for itm in self.modelTimeOptions.getItems():
+                if str(itm) != "0":
+                    self.modelTimeOptions.disableItem(str_model=itm)
 
         # Change time step option if required by model constraints
 
         if idtvar_p != idtvar:
             self.mdl.setTimePassing(idtvar)
 
-        ntmabs = self.mdl.getIterationsNumber()
-        self.lineEditNTMABS.setText(str(ntmabs))
-
         if self.mdl.thermalCase():
             if self.mdl.getThermalTimeStep() == 'on':
                 self.checkBoxIPTLRO.setChecked(True)
             else:
                 self.checkBoxIPTLRO.setChecked(False)
         else:
-            self.lineIPTLRO.hide()
             self.labelIPTLRO.hide()
             self.checkBoxIPTLRO.hide()
             self.mdl.RemoveThermalTimeStepNode()
 
-        if self.mdl.getZeroTimeStep() == 'on':
-            self.checkBoxINPDT0.setChecked(True)
-        else:
-            self.checkBoxINPDT0.setChecked(False)
-
         self.__setTimePassingDisplay(idtvar)
+        self.__setStopCritDisplay()
 
         self.case.undoStartGlobal()
 
@@ -225,19 +242,36 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
 
         self.modelTimeOptions.setItem(str_model=str(idtvar))
 
-        if idtvar == -1:
-            self.modelNTERUP.enableItem(str_model = 'simple')
-            self.modelNTERUP.disableItem(str_model = 'simplec')
-            self.modelNTERUP.disableItem(str_model = 'piso')
-        elif idtvar == 2:
-            self.modelNTERUP.disableItem(str_model = 'simple')
-            self.modelNTERUP.enableItem(str_model = 'simplec')
+        if idtvar in (-1, 2):
+            if idtvar == -1:
+                self.modelNTERUP.enableItem(str_model = 'simple')
+                self.modelNTERUP.disableItem(str_model = 'simplec')
+            elif idtvar == 2:
+                self.modelNTERUP.disableItem(str_model = 'simple')
+                self.modelNTERUP.enableItem(str_model = 'simplec')
             self.modelNTERUP.disableItem(str_model = 'piso')
+
+            m_prev, c_prev = self.mdl.getStopCriterion()
+            for m in ('maximum_time', 'maximum_time_add'):
+                if m_prev == m:
+                    dtref = self.mdl.getTimeStep()
+                    value = int(float(c_prev) / float(dtref))
+                    model = 'iterations'
+                    if m[-4:] == '_add':
+                        model += '_add'
+                    self.mdl.setStopCriterion(model, value)
+                    self.__setStopCritDisplay()
+
+                self.modelTimeStop.disableItem(str_model=m)
+
         else:
             self.modelNTERUP.disableItem(str_model = 'simple')
             self.modelNTERUP.enableItem(str_model = 'simplec')
             self.modelNTERUP.enableItem(str_model = 'piso')
 
+            self.modelTimeStop.enableItem(str_model='maximum_time')
+            self.modelTimeStop.enableItem(str_model='maximum_time_add')
+
         algo = self.mdl.getVelocityPressureAlgorithm()
         self.modelNTERUP.setItem(str_model=algo)
         if algo == 'piso':
@@ -245,7 +279,7 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
         else:
             self.spinBoxNTERUP.hide()
 
-        value = self.mdl.getPisoSweepNumber()
+        value = self.mdl.getVelocityPressureParamSweepNumber()
         self.spinBoxNTERUP.setValue(value)
 
         if idtvar == -1:
@@ -279,9 +313,44 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
             self.lineEditCDTMAX.setText(str(time_step_max_factor))
             self.lineEditVARRDT.setText(str(time_step_var))
 
-            self.groupBoxLabels.show()
+            self.labelCOUMAX.show()
+            self.lineEditCOUMAX.show()
+            self.labelFOUMAX.show()
+            self.lineEditFOUMAX.show()
+            self.labelCDTMIN.show()
+            self.lineEditCDTMIN.show()
+            self.labelCDTMAX.show()
+            self.lineEditCDTMAX.show()
+            self.labelVARRDT.show()
+            self.lineEditVARRDT.show()
+
         else:
-            self.groupBoxLabels.hide()
+            self.labelCOUMAX.hide()
+            self.lineEditCOUMAX.hide()
+            self.labelFOUMAX.hide()
+            self.lineEditFOUMAX.hide()
+            self.labelCDTMIN.hide()
+            self.lineEditCDTMIN.hide()
+            self.labelCDTMAX.hide()
+            self.lineEditCDTMAX.hide()
+            self.labelVARRDT.hide()
+            self.lineEditVARRDT.hide()
+
+
+    def __setStopCritDisplay(self):
+        """
+        Stop criterion option
+        """
+
+        model, value = self.mdl.getStopCriterion()
+
+        if model in ("iterations", "iterations_add"):
+            self.lineEditStop.setValidator(self.validatorNTABS)
+        elif model in ("maximum_time", "maximum_time_add"):
+            self.lineEditStop.setValidator(self.validatorTABS)
+
+        self.modelTimeStop.setItem(str_model=str(model))
+        self.lineEditStop.setText(str(value))
 
 
     @pyqtSlot(str)
@@ -296,6 +365,47 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
         self.__setTimePassingDisplay(idtvar)
 
 
+    @pyqtSlot(str)
+    def slotStopCritModel(self, text):
+        """
+        Select stop criterion model
+        """
+        m_prev, c_prev = self.mdl.getStopCriterion()
+        model = self.modelTimeStop.dicoV2M[str(text)]
+
+        value = c_prev
+        if m_prev in ("iterations", "iterations_add"):
+            if model in ("maximum_time", "maximum_time_add"):
+                dtref = self.mdl.getTimeStep()
+                value = float(c_prev) * float(dtref)
+        elif m_prev in ("maximum_time", "maximum_time_add"):
+            if model in ("iterations", "iterations_add"):
+                dtref = self.mdl.getTimeStep()
+                value = int(float(c_prev) / float(dtref))
+
+        self.mdl.setStopCriterion(model, value)
+
+        self.__setStopCritDisplay()
+
+
+    @pyqtSlot(str)
+    def slotStopCritValue(self, text):
+        """
+        Input stop criterion
+        """
+        if self.lineEditStop.validator().state == QValidator.Acceptable:
+
+            model, c_prev = self.mdl.getStopCriterion()
+
+            value = c_prev
+            if model in ("iterations", "iterations_add"):
+                value = from_qvariant(text, int)
+                self.mdl.setStopCriterion(model, value)
+            elif model in ("maximum_time", "maximum_time_add"):
+                value = from_qvariant(text, float)
+                self.mdl.setStopCriterion(model, value)
+
+
     @pyqtSlot(str)
     def slotTimeStep(self, text):
         """
@@ -315,7 +425,7 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
         self.mdl.setVelocityPressureAlgorithm(NTERUP)
         if NTERUP == 'piso':
             self.spinBoxNTERUP.show()
-            value = self.mdl.getPisoSweepNumber()
+            value = self.mdl.getVelocityPressureParamSweepNumber()
             self.spinBoxNTERUP.setValue(value)
         else:
             self.spinBoxNTERUP.hide()
@@ -326,9 +436,9 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
     @pyqtSlot(int)
     def slotNTERUP2(self, var):
         """
-        Set value for parameter piso sweep number
+        Set value for velocity-pressure parameter sweep number
         """
-        self.mdl.setPisoSweepNumber(var)
+        self.mdl.setVelocityPressureParamSweepNumber(var)
         log.debug("slotNTERUP2-> %s" % var)
 
 
@@ -342,16 +452,6 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
             self.mdl.setRelaxCoefficient(relax_coef)
 
 
-    @pyqtSlot(str)
-    def slotIter(self, text):
-        """
-        Input NTMABS.
-        """
-        if self.lineEditNTMABS.validator().state == QValidator.Acceptable:
-            iteration = from_qvariant(text, int)
-            self.mdl.setIterationsNumber(iteration)
-
-
     @pyqtSlot(str)
     def slotTimeOptionCOUMAX(self, text):
         """
@@ -413,23 +513,6 @@ class TimeStepView(QWidget, Ui_TimeStepForm):
             self.mdl.setThermalTimeStep("off")
 
 
-    @pyqtSlot()
-    def slotZeroTimeStep(self):
-        """
-        Input INPDT0.
-        """
-        if self.checkBoxINPDT0.isChecked():
-            self.mdl.setZeroTimeStep("on")
-        else:
-            self.mdl.setZeroTimeStep("off")
-
-
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
 #-------------------------------------------------------------------------------
 # Testing part
 #-------------------------------------------------------------------------------
