@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -48,11 +48,11 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Toolbox import GuiParam
+from code_saturne.model.Common import GuiParam
 from code_saturne.Base.QtPage import ComboModel, DoubleValidator, from_qvariant
 from code_saturne.Pages.TurbulenceForm import Ui_TurbulenceForm
 from code_saturne.Pages.TurbulenceAdvancedOptionsDialogForm import Ui_TurbulenceAdvancedOptionsDialogForm
-from code_saturne.Pages.TurbulenceModel import TurbulenceModel
+from code_saturne.model.TurbulenceModel import TurbulenceModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -82,19 +82,35 @@ class TurbulenceAdvancedOptionsDialogView(QDialog, Ui_TurbulenceAdvancedOptionsD
         self.case = case
         self.case.undoStopGlobal()
 
+        self.labelTurbDiff.hide()
+        self.comboBoxTurbDiff.hide()
+        self.turbDiff = None
+
+        self.default = default
+        self.result  = self.default.copy()
+
         if default['model'] in ('k-epsilon', 'k-epsilon-PL'):
             title = self.tr("Options for k-epsilon model")
         elif default['model'] in ('Rij-epsilon', 'Rij-SSG', 'Rij-EBRSM'):
             title = self.tr("Options for Rij-epsilon model")
+
+            self.labelTurbDiff.show()
+            self.comboBoxTurbDiff.show()
+            self.turbDiff = ComboModel(self.comboBoxTurbDiff, 2, 1)
+            self.turbDiff.addItem(self.tr("Scalar diffusivity (Shir model)"), 'shir')
+            self.turbDiff.addItem(self.tr("Tensorial diffusivity (Daly and Harlow model)"), 'daly_harlow')
+
+            # Initialization of turb diff model
+            self.turbDiff.setItem(str_model=str(self.result['turb_diff']))
+
         elif default['model'] == 'k-omega-SST':
             title = self.tr("Options for k-omega-SST model")
         elif default['model'] == 'v2f-BL-v2/k':
             title = self.tr("Options for v2f-BL-v2/k model")
         elif default['model'] == 'Spalart-Allmaras':
             title = self.tr("Options for Spalart-Allmaras model")
+
         self.setWindowTitle(title)
-        self.default = default
-        self.result  = self.default.copy()
 
         self.checkBoxGravity.setEnabled(True)
         self.comboBoxWallFunctions.setEnabled(True)
@@ -163,7 +179,9 @@ class TurbulenceAdvancedOptionsDialogView(QDialog, Ui_TurbulenceAdvancedOptionsD
         else:
             self.result['gravity_terms'] = "off"
         self.result['wall_function'] = \
-        int(self.wallFunctions.dicoV2M[str(self.comboBoxWallFunctions.currentText())])
+          int(self.wallFunctions.dicoV2M[str(self.comboBoxWallFunctions.currentText())])
+        if self.turbDiff:
+            self.result['turb_diff'] = self.turbDiff.dicoV2M[str(self.comboBoxTurbDiff.currentText())]
 
         QDialog.accept(self)
 
@@ -175,12 +193,6 @@ class TurbulenceAdvancedOptionsDialogView(QDialog, Ui_TurbulenceAdvancedOptionsD
         QDialog.reject(self)
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
 #-------------------------------------------------------------------------------
 # Main view class
 #-------------------------------------------------------------------------------
@@ -207,18 +219,75 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
         self.modelTurbModel = ComboModel(self.comboBoxTurbModel,10,1)
 
         self.modelTurbModel.addItem(self.tr("No model (i.e. laminar flow)"), "off")
-        self.modelTurbModel.addItem(self.tr("Mixing length"), "mixing_length")
-        self.modelTurbModel.addItem(self.tr("k-epsilon"), "k-epsilon")
-        self.modelTurbModel.addItem(self.tr("k-epsilon Linear Production"), "k-epsilon-PL")
-        self.modelTurbModel.addItem(self.tr("Rij-epsilon LRR"), "Rij-epsilon")
-        self.modelTurbModel.addItem(self.tr("Rij-epsilon SSG"), "Rij-SSG")
-        self.modelTurbModel.addItem(self.tr("Rij-epsilon EBRSM"), "Rij-EBRSM")
-        self.modelTurbModel.addItem(self.tr("v2f BL-v2/k"), "v2f-BL-v2/k")
-        self.modelTurbModel.addItem(self.tr("k-omega SST"), "k-omega-SST")
-        self.modelTurbModel.addItem(self.tr("Spalart-Allmaras"), "Spalart-Allmaras")
-        self.modelTurbModel.addItem(self.tr("LES (Smagorinsky)"), "LES_Smagorinsky")
-        self.modelTurbModel.addItem(self.tr("LES (classical dynamic model)"), "LES_dynamique")
-        self.modelTurbModel.addItem(self.tr("LES (WALE)"), "LES_WALE")
+
+        # RANS - Algebraic
+        self.modelTurbModel.addItemGroup(self.tr("RANS - Algebraic"))
+        self.modelTurbModel.addItem(self.tr("Mixing length"), "mixing_length", groupName="RANS - Algebraic")
+
+        # RANS - 1st order
+        self.modelTurbModel.addItemGroup(self.tr("RANS - 1st order"))
+
+        e = {"k-epsilon-PL": "k-\u03B5 Linear Production",
+             "v2f-BL-v2/k": "v\u00B2-f BL-v\u00B2/k",
+             "k-omega-SST": "k-\u03C9 SST",
+             "Spalart-Allmaras": "Spalart-Allmaras"}
+
+        if QT_API == "PYQT4":
+            e["k-epsilon-PL"] = "k-epsilon Linear Production"
+            e["v2f-BL-v2/k"] = "v2f BL-v2/k"
+            e["k-omega-SST"] = "k-omega SST"
+
+        for k in ("k-epsilon-PL", "v2f-BL-v2/k",
+                  "k-omega-SST", "Spalart-Allmaras"):
+            self.modelTurbModel.addItem(self.tr(e[k]), k,
+                                        groupName="RANS - 1st order")
+
+        # RANS - 2nd order
+        self.modelTurbModel.addItemGroup(self.tr("RANS - 2nd order"))
+
+        e = {"Rij-SSG": "R\u1D62\u2C7C-\u03B5 SSG",
+             "Rij-EBRSM": "R\u1D62\u2C7C-\u03B5 EBRSM"}
+
+        if QT_API == "PYQT4":
+            e["Rij-SSG"] = "Rij-SSG"
+            e["Rij-EBRSM"] = "Rij-EBRSM"
+
+        for k in ("Rij-SSG", "Rij-EBRSM"):
+            self.modelTurbModel.addItem(self.tr(e[k]), k,
+                                        groupName="RANS - 2nd order")
+
+        # LES
+        self.modelTurbModel.addItemGroup(self.tr("LES"))
+
+        self.modelTurbModel.addItem(self.tr("Smagorinsky"),
+                                    "LES_Smagorinsky",
+                                    groupName="LES")
+        self.modelTurbModel.addItem(self.tr("Standard dynamic model"),
+                                    "LES_dynamique",
+                                    groupName="LES")
+        self.modelTurbModel.addItem(self.tr("WALE"),
+                                    "LES_WALE",
+                                    groupName="LES")
+
+        # Others
+        self.modelTurbModel.addItemGroup(self.tr(""))
+        self.modelTurbModel.addItemGroup(self.tr("Others"))
+
+        e = {"Rij-epsilon": "R\u1D62\u2C7C-\u03B5 LRR",
+             "k-epsilon": "k-\u03B5"}
+
+        if QT_API == "PYQT4":
+            e["Rij-epsilon"] = "Rij-LRR"
+            e["k-epsilon"] = "k-epsilon"
+
+        for k in ("Rij-epsilon", "k-epsilon"):
+            self.modelTurbModel.addItem(self.tr(e[k]), k,
+                                        groupName="Others")
+
+        self.modelLength = ComboModel(self.comboBoxLength,2,1)
+        self.modelLength.addItem(self.tr("Automatic"), 'automatic')
+        self.modelLength.addItem(self.tr("Prescribed"), 'prescribed')
+        self.comboBoxLength.setSizeAdjustPolicy(QComboBox.AdjustToContents)
 
         # Connections
 
@@ -226,17 +295,26 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
         self.pushButtonAdvanced.clicked.connect(self.slotAdvancedOptions)
         self.lineEditLength.textChanged[str].connect(self.slotLengthScale)
 
+        self.lineEditV0.textChanged[str].connect(self.slotVelocity)
+        self.comboBoxLength.activated[str].connect(self.slotLengthChoice)
+        self.lineEditL0.textChanged[str].connect(self.slotLength)
+
         # Frames display
 
         self.frameAdvanced.hide()
         self.frameLength.hide()
 
-        # Validator
+        # Validators
 
         validator = DoubleValidator(self.lineEditLength, min=0.0)
         validator.setExclusiveMin(True)
         self.lineEditLength.setValidator(validator)
 
+        validatorV0 = DoubleValidator(self.lineEditV0, min=0.0)
+        self.lineEditV0.setValidator(validatorV0)
+
+        validatorL0 = DoubleValidator(self.lineEditL0, min=0.0)
+        self.lineEditL0.setValidator(validatorL0)
 
         # Update the turbulence models list with the calculation features
 
@@ -255,6 +333,23 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
         l_scale = self.model.getLengthScale()
         self.lineEditLength.setText(str(l_scale))
 
+        # Initialization
+
+        v = self.model.getVelocity()
+        self.lineEditV0.setText(str(v))
+
+        init_length_choice = self.model.getLengthChoice()
+        self.modelLength.setItem(str_model=init_length_choice)
+        if init_length_choice == 'automatic':
+            self.lineEditL0.setText(str())
+            self.lineEditL0.hide()
+            self.labelUnitL0.hide()
+        else:
+            self.lineEditL0.show()
+            self.labelUnitL0.show()
+            l = self.model.getLength()
+            self.lineEditL0.setText(str(l))
+
         self.case.undoStartGlobal()
 
 
@@ -267,6 +362,10 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
 
         self.frameAdvanced.hide()
         self.frameLength.hide()
+        self.groupBoxReferenceValues.hide()
+
+        if model not in ('off', 'mixing_length', 'LES_Smagorinsky', 'LES_dynamique', 'LES_WALE'):
+            self.groupBoxReferenceValues.show()
 
         if model == 'mixing_length':
             self.frameLength.show()
@@ -303,6 +402,48 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
         self.model.setTurbulenceModel(model)
         self.__initializeView()
 
+    @pyqtSlot(str)
+    def slotVelocity(self,  text):
+        """
+        Private slot.
+        Input reference velocity.
+        """
+        if self.lineEditV0.validator().state == QValidator.Acceptable:
+            v = from_qvariant(text, float)
+            self.model.setVelocity(v)
+
+
+    @pyqtSlot(str)
+    def slotLengthChoice(self,text):
+        """
+        Private slot.
+        Input mode for reference length.
+        """
+        choice = self.modelLength.dicoV2M[str(text)]
+        self.model.setLengthChoice(choice)
+        if choice == 'automatic':
+            self.lineEditL0.setText(str())
+            self.lineEditL0.hide()
+            self.labelUnitL0.hide()
+        else:
+            self.lineEditL0.show()
+            self.labelUnitL0.show()
+            value = self.model.getLength()
+            self.lineEditL0.setText(str(value))
+        log.debug("slotlengthchoice-> %s" % choice)
+
+
+    @pyqtSlot(str)
+    def slotLength(self,  text):
+        """
+        Private slot.
+        Input reference length.
+        """
+        if self.lineEditL0.validator().state == QValidator.Acceptable:
+            l = from_qvariant(text, float)
+            self.model.setLength(l)
+
+
     @pyqtSlot()
     def slotAdvancedOptions(self):
         """
@@ -312,6 +453,7 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
         default = {}
         default['model']         = self.model.getTurbulenceModel()
         default['wall_function'] = self.model.getWallFunction()
+        default['turb_diff']     = self.model.getTurbDiffModel()
         default['gravity_terms'] = self.model.getGravity()
         log.debug("slotAdvancedOptions -> %s" % str(default))
 
@@ -321,15 +463,10 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
             log.debug("slotAdvancedOptions -> %s" % str(result))
             self.model.setTurbulenceModel(result['model'])
             self.model.setWallFunction(result['wall_function'])
+            self.model.setTurbDiffModel(result['turb_diff'])
             self.model.setGravity(result['gravity_terms'])
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
 #-------------------------------------------------------------------------------
 # End
 #-------------------------------------------------------------------------------
