@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -39,9 +39,8 @@ import os, unittest
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Common import *
-import code_saturne.Base.Toolbox as Tool
-from code_saturne.Base.XMLvariables import Model, Variables
+from code_saturne.model.Common import *
+from code_saturne.model.XMLvariables import Model, Variables
 
 #-------------------------------------------------------------------------------
 # Model class
@@ -534,27 +533,31 @@ class ManageCasesModel(Model):
         nn['label'] = name
 
 
-    def getPostScriptInput(self, study_name, idx):
+    def getPostScriptInput(self, study_name, case_idx):
         """
         Get post script status from node with index
         """
-        self.isInt(idx)
+        self.isInt(case_idx)
         study_node = self.case.xmlGetNode('study', label = study_name)
-        node = study_node.xmlGetNode("case", id = idx)
-        nn = node.xmlGetNode("input")
+        node = study_node.xmlGetNode("case", id = case_idx)
         name = ""
-        if nn:
-            name = nn['file']
+        lst = node.xmlGetNodeList("input")
+        if lst:
+            if len(lst) == 1:
+                nn = lst[0]
+                name = nn['file']
+            elif len(lst) >= 1:
+                name = '<multiple inputs>'
         return name
 
 
-    def setPostScriptInput(self, study_name, idx, name):
+    def setPostScriptInput(self, study_name, case_idx, name):
         """
         Put post script status from node with index
         """
-        self.isInt(idx)
+        self.isInt(case_idx)
         study_node = self.case.xmlGetNode('study', label = study_name)
-        node = study_node.xmlGetNode("case", id = idx)
+        node = study_node.xmlGetNode("case", id = case_idx)
         nn = node.xmlInitChildNode("input")
         nn['file'] = name
 
