@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -45,9 +45,8 @@ from code_saturne.Base.QtWidgets import *
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Common import LABEL_LENGTH_MAX
-from code_saturne.Base.Toolbox import GuiParam
-from code_saturne.Base.QtPage import RegExpValidator, to_qvariant, from_qvariant, to_text_string
+from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam
+from code_saturne.Base.QtPage import RegExpValidator, from_qvariant, to_text_string
 from code_saturne.studymanager_gui.ManageCasesForm import Ui_ManageCasesForm
 from code_saturne.studymanager_gui.ManageCasesModel import ManageCasesModel
 
@@ -113,33 +112,33 @@ class TreeItem(object):
     def data(self, column, role):
         if self.item == None:
             if column == 0:
-                return to_qvariant(self.header)
+                return self.header
             else:
-                return to_qvariant()
+                return None
         else:
             if column == 0 and role == Qt.DisplayRole:
-                return to_qvariant(self.item.name)
+                return self.item.name
             elif column == 1 and role == Qt.CheckStateRole:
                 value = self.item.status
                 if value == 'on':
-                    return to_qvariant(Qt.Checked)
+                    return Qt.Checked
                 else:
-                    return to_qvariant(Qt.Unchecked)
+                    return Qt.Unchecked
             elif column == 2 and role == Qt.CheckStateRole:
                 value = self.item.compute
                 if value == 'on':
-                    return to_qvariant(Qt.Checked)
+                    return Qt.Checked
                 else:
-                    return to_qvariant(Qt.Unchecked)
+                    return Qt.Unchecked
             elif column == 3 and role == Qt.CheckStateRole:
                 value = self.item.post
                 if value == 'on':
-                    return to_qvariant(Qt.Checked)
+                    return Qt.Checked
                 else:
-                    return to_qvariant(Qt.Unchecked)
+                    return Qt.Unchecked
             elif column == 4 and role == Qt.DisplayRole:
-                return to_qvariant(self.item.run_id)
-        return to_qvariant()
+                return self.item.run_id
+        return None
 
 
     def parent(self):
@@ -186,7 +185,7 @@ class LabelDelegate(QItemDelegate):
 
         if editor.validator().state == QValidator.Acceptable:
             p_value = str(editor.text())
-            model.setData(index, to_qvariant(p_value), Qt.DisplayRole)
+            model.setData(index, p_value, Qt.DisplayRole)
 
 
 #-------------------------------------------------------------------------------
@@ -225,26 +224,26 @@ class CaseStandardItemModel(QAbstractItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         item = index.internalPointer()
 
         # ToolTips
         if role == Qt.ToolTipRole:
-            return to_qvariant()
+            return None
 
         # StatusTips
         if role == Qt.StatusTipRole:
             if index.column() == 0:
-                return to_qvariant(self.tr("Case name"))
+                return self.tr("Case name")
             elif index.column() == 1:
-                return to_qvariant(self.tr("Status"))
+                return self.tr("Status")
             elif index.column() == 2:
-                return to_qvariant(self.tr("Compute"))
+                return self.tr("Compute")
             elif index.column() == 3:
-                return to_qvariant(self.tr("Post-processing"))
+                return self.tr("Post-processing")
             elif index.column() == 4:
-                return to_qvariant(self.tr("Run_id"))
+                return self.tr("Run_id")
 
         # Display
         if role == Qt.DisplayRole:
@@ -252,7 +251,7 @@ class CaseStandardItemModel(QAbstractItemModel):
         elif role == Qt.CheckStateRole:
             return item.data(index.column(), role)
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -275,16 +274,16 @@ class CaseStandardItemModel(QAbstractItemModel):
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
             if section == 0:
-                return to_qvariant(self.tr("Case name"))
+                return self.tr("Case name")
             elif section == 1:
-                return to_qvariant(self.tr("Status"))
+                return self.tr("Status")
             elif section == 2:
-                return to_qvariant(self.tr("Compute"))
+                return self.tr("Compute")
             elif section == 3:
-                return to_qvariant(self.tr("Post-\nprocessing"))
+                return self.tr("Post-\nprocessing")
             elif section == 4:
-                return to_qvariant(self.tr("run_id"))
-        return to_qvariant()
+                return self.tr("run_id")
+        return None
 
 
     def index(self, row, column, parent = QModelIndex()):
@@ -462,6 +461,7 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
 
         self.groupBoxPrepro.hide()
         self.groupBoxPost.hide()
+        self.groupBoxInput.hide()
         self.groupBoxCompare.hide()
 
         self.lineEditPrepro.setEnabled(False)
@@ -558,6 +558,7 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
         self.treeViewCases.setModel(self.modelCases)
         self.groupBoxPrepro.hide()
         self.groupBoxPost.hide()
+        self.groupBoxInput.hide()
         self.treeViewCases.expandAll()
         self.slotChangeSelection()
 
@@ -574,6 +575,7 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
         self.treeViewCases.setModel(self.modelCases)
         self.groupBoxPrepro.hide()
         self.groupBoxPost.hide()
+        self.groupBoxInput.hide()
         self.treeViewCases.expandAll()
         self.slotChangeSelection()
 
@@ -594,6 +596,7 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
         self.treeViewCases.setModel(self.modelCases)
         self.groupBoxPrepro.hide()
         self.groupBoxPost.hide()
+        self.groupBoxInput.hide()
         self.treeViewCases.expandAll()
         self.slotChangeSelection()
 
@@ -610,6 +613,7 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
         self.treeViewCases.setModel(self.modelCases)
         self.groupBoxPrepro.hide()
         self.groupBoxPost.hide()
+        self.groupBoxInput.hide()
         self.treeViewCases.expandAll()
         self.slotChangeSelection()
 
@@ -626,6 +630,7 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
         self.treeViewCases.setModel(self.modelCases)
         self.groupBoxPrepro.hide()
         self.groupBoxPost.hide()
+        self.groupBoxInput.hide()
         self.treeViewCases.expandAll()
         self.slotChangeSelection()
 
@@ -639,6 +644,7 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
         self.pushButtonAdd.setEnabled(True)
         self.groupBoxPrepro.hide()
         self.groupBoxPost.hide()
+        self.groupBoxInput.hide()
         self.groupBoxCompare.hide()
 
         current = self.treeViewCases.currentIndex()
@@ -649,9 +655,6 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
             # study
             self.pushButtonDeleteStudy.setEnabled(True)
             self.groupBoxPost.show()
-            self.pushButtonInput.hide()
-            self.lineEditInput.hide()
-            self.labelinput.hide()
             study = current.internalPointer().item.name
 
             status = self.mdl.getStudyPostScriptStatus(study)
@@ -678,10 +681,8 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
             self.toolButtonDuplicate.setEnabled(True)
             self.groupBoxPrepro.show()
             self.groupBoxPost.show()
+            self.groupBoxInput.show()
             self.groupBoxCompare.show()
-            self.pushButtonInput.show()
-            self.lineEditInput.show()
-            self.labelinput.show()
             study = current.parent().internalPointer().item.name
 
             # prepro
@@ -726,6 +727,10 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
 
             input_name = self.mdl.getPostScriptInput(study, idx)
             self.lineEditInput.setText(str(input_name))
+            if input_name == '<multiple inputs>':
+                self.pushButtonInput.setEnabled(False)
+            else:
+                self.pushButtonInput.setEnabled(True)
 
             # compare
             status = self.mdl.getCompareStatus(study, idx)
@@ -844,13 +849,16 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
         study = current.parent().internalPointer().item.name
 
         cur_path = os.getcwd()
-        rep = os.path.abspath(os.path.join(self.mdl.repo, study, "MESH"))
-        os.chdir(rep)
+        base_dir = os.path.abspath(os.path.join(self.mdl.repo, study))
+        rep = os.path.abspath(os.path.join(base_dir, "MESH"))
+        if not os.path.isdir(rep):
+            rep = os.path.abspath(os.path.join(os.path.split(base_dir)[0], "MESH"))
+        if not os.path.isdir(rep):
+            rep = base_dir
         title = self.tr("preprocess script")
         filetypes = self.tr("(*py*);;All Files (*)")
         file = QFileDialog.getOpenFileName(self, title, rep, filetypes)[0]
         file = str(file)
-        os.chdir(cur_path)
 
         if not file:
             return
@@ -878,13 +886,16 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
             study = current.parent().internalPointer().item.name
 
         cur_path = os.getcwd()
-        rep = os.path.abspath(os.path.join(self.mdl.repo, study, "POST"))
-        os.chdir(rep)
+        base_dir = os.path.abspath(os.path.join(self.mdl.repo, study))
+        rep = os.path.abspath(os.path.join(base_dir, "POST"))
+        if not os.path.isdir(rep):
+            rep = os.path.abspath(os.path.join(os.path.split(base_dir)[0], "POST"))
+        if not os.path.isdir(rep):
+            rep = base_dir
         title = self.tr("postprocess script")
         filetypes = self.tr("All Files (*)")
         file = QFileDialog.getOpenFileName(self, title, rep, filetypes)[0]
         file = str(file)
-        os.chdir(cur_path)
 
         if not file:
             return
@@ -911,13 +922,16 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
         study = current.parent().internalPointer().item.name
 
         cur_path = os.getcwd()
-        rep = os.path.abspath(os.path.join(self.mdl.repo, study, "POST"))
-        os.chdir(rep)
+        base_dir = os.path.abspath(os.path.join(self.mdl.repo, study))
+        rep = os.path.abspath(os.path.join(base_dir, "POST"))
+        if not os.path.isdir(rep):
+            rep = os.path.abspath(os.path.join(os.path.split(base_dir)[0], "POST"))
+        if not os.path.isdir(rep):
+            rep = base_dir
         title = self.tr("input file for postprocess script")
         filetypes = self.tr("All Files (*)")
         file = QFileDialog.getOpenFileName(self, title, rep, filetypes)[0]
         file = str(file)
-        os.chdir(cur_path)
 
         if not file:
             return
@@ -931,13 +945,6 @@ class ManageCasesView(QWidget, Ui_ManageCasesForm):
             self.mdl.setPostScriptInput(study, idx, file)
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
-
 #-------------------------------------------------------------------------------
 # Testing part
 #-------------------------------------------------------------------------------
