@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -39,9 +39,8 @@ import sys, unittest
 # Application modules import
 #-------------------------------------------------------------------------------
 
-import code_saturne.Base.Toolbox as Tool
-from code_saturne.Base.XMLvariables import Model, Variables
-from code_saturne.Base.XMLmodel import ModelTest
+from code_saturne.model.XMLvariables import Model, Variables
+from code_saturne.model.XMLmodel import ModelTest
 
 #-------------------------------------------------------------------------------
 # Model class
@@ -114,7 +113,7 @@ class ManagePlotterModel(Model):
         study_node = self.case.xmlGetNode("study", label = study)
         lst = []
         for node in study_node.xmlGetNodeList("measurement"):
-            lst.append(node['file'])
+            lst.append((node['file'], node['path']))
         return lst
 
 
@@ -169,7 +168,7 @@ class ManagePlotterModel(Model):
                     nn['idlist'] = new_lst
 
         for nn in study_node.xmlGetNodeList("plot"):
-            lst = nn['fig']
+            lst = nn['spids']
             new_lst = ''
             if lst:
                 for idl in lst:
@@ -181,7 +180,7 @@ class ManagePlotterModel(Model):
                             new_lst = new_lst + " " + str(new_idl)
                         else:
                             new_lst = str(new_idl)
-                    nn['fig'] = new_lst
+                    nn['spids'] = new_lst
 
 
     def getSubplotTitle(self, study, idx):
@@ -548,11 +547,11 @@ class ManagePlotterModel(Model):
             idx = idx + 1
 
 
-    def getMeasurementPlotList(self, study, measurement_name):
+    def getMeasurementPlotList(self, study, name, path):
         """
         """
         study_node = self.case.xmlGetNode("study", label = study)
-        measurement_node = study_node.xmlGetNode("measurement", file = measurement_name)
+        measurement_node = study_node.xmlGetNode("measurement", file=name, path=path)
         lst = []
         for node in measurement_node.xmlGetNodeList("plot"):
             lst.append(node['id'])
@@ -837,13 +836,13 @@ class ManagePlotterModel(Model):
 
     def getMeasurementIdList(self, study, measurement_idx, idx):
         """
-        Return subplot id list for a figure
+        Return subplot id list for a plot of a measurement
         """
         study_node = self.case.xmlGetNode("study", label = study)
         lst = study_node.xmlGetNodeList("measurement")
         node = lst[measurement_idx]
         plot = node.xmlGetNode("plot", id = idx)
-        lst = plot['fig']
+        lst = plot['spids']
         if not lst:
             lst = ""
         return lst
@@ -851,13 +850,13 @@ class ManagePlotterModel(Model):
 
     def setMeasurementIdList(self, study, measurement_idx, idx, idlist):
         """
-        Set subplot id list for a figure
+        Set subplot id list for a plot of a measurement
         """
         study_node = self.case.xmlGetNode("study", label = study)
         lst = study_node.xmlGetNodeList("measurement")
         node = lst[measurement_idx]
         plot = node.xmlGetNode("plot", id = idx)
-        plot['fig'] = idlist
+        plot['spids'] = idlist
 
 
 #-------------------------------------------------------------------------------
@@ -887,7 +886,7 @@ class ManagePlotterModel(Model):
         """
         study_node = self.case.xmlGetNode("study", label = study)
         nn = study_node.xmlGetNode("case", id = case_idx)
-        nn.xmlInitChildNode("data", file=name)
+        nn.xmlInitChildNode("data", dest="", file=name)
 
 
     def delCaseDataFile(self, study, case_idx, idx):
@@ -1256,7 +1255,7 @@ class ManagePlotterModel(Model):
         case = study_node.xmlGetNode("case", id = case_idx)
         node = case.xmlGetNodeList("data")[data_idx]
         n = node.xmlGetNode("plot", id = plot_id)
-        lst = n['fig']
+        lst = n['spids']
         if not lst:
             lst = ""
         return lst
@@ -1270,7 +1269,7 @@ class ManagePlotterModel(Model):
         case = study_node.xmlGetNode("case", id = case_idx)
         node = case.xmlGetNodeList("data")[data_idx]
         plot = node.xmlGetNode("plot", id = plot_id)
-        plot['fig'] = idlist
+        plot['spids'] = idlist
 
 #-------------------------------------------------------------------------------
 # End
