@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -46,10 +46,9 @@ import os
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from code_saturne.Base.Toolbox import GuiParam
-from code_saturne.Base.Common import LABEL_LENGTH_MAX
+from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam
 from code_saturne.Base.QtPage import ComboModel, DoubleValidator, IntValidator
-from code_saturne.Base.QtPage import RegExpValidator, to_qvariant
+from code_saturne.Base.QtPage import RegExpValidator
 from code_saturne.Base.QtPage import from_qvariant, to_text_string
 from code_saturne.studymanager_gui.ManagePlotForm import Ui_ManagePlotForm
 from code_saturne.studymanager_gui.ManagePlotterForm import Ui_ManagePlotterForm
@@ -138,13 +137,6 @@ class ManageSubplotDialogView(QDialog, Ui_ManagePlotterSubplotForm):
         QDialog.reject(self)
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
-
 #-------------------------------------------------------------------------------
 # Advanced dialog for plot
 #-------------------------------------------------------------------------------
@@ -218,13 +210,6 @@ class ManagePlotDialogView(QDialog, Ui_ManagePlotForm):
         QDialog.reject(self)
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
-
 #-------------------------------------------------------------------------------
 # Line edit delegate for float
 #-------------------------------------------------------------------------------
@@ -254,7 +239,7 @@ class FloatDelegate(QItemDelegate):
             selectionModel = self.parent.selectionModel()
             for idx in selectionModel.selectedIndexes():
                 if idx.column() == index.column():
-                    model.setData(idx, to_qvariant(value))
+                    model.setData(idx, value)
 
 #-------------------------------------------------------------------------------
 # Line edit delegate for integer
@@ -285,7 +270,7 @@ class IntDelegate(QItemDelegate):
             selectionModel = self.parent.selectionModel()
             for idx in selectionModel.selectedIndexes():
                 if idx.column() == index.column():
-                    model.setData(idx, to_qvariant(value))
+                    model.setData(idx, value)
 
 
 #-------------------------------------------------------------------------------
@@ -322,7 +307,7 @@ class FormatFigureDelegate(QItemDelegate):
         selectionModel = self.parent.selectionModel()
         for idx in selectionModel.selectedIndexes():
             if idx.column() == index.column():
-                model.setData(idx, to_qvariant(value))
+                model.setData(idx, value)
 
 
 #-------------------------------------------------------------------------------
@@ -355,7 +340,7 @@ class LabelDelegate(QItemDelegate):
             return
 
         p_value = str(editor.text())
-        model.setData(index, to_qvariant(p_value), Qt.DisplayRole)
+        model.setData(index, p_value, Qt.DisplayRole)
 
 
 #-------------------------------------------------------------------------------
@@ -419,7 +404,7 @@ class StandardItemModelSubplot(QStandardItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         row = index.row()
         column = index.column()
@@ -427,23 +412,23 @@ class StandardItemModelSubplot(QStandardItemModel):
         key = self.keys[column]
 
         if dico[key] == None:
-            return to_qvariant()
+            return None
 
         if role == Qt.DisplayRole and column != 4:
-            return to_qvariant(dico[key])
+            return dico[key]
 
 
         elif role == Qt.TextAlignmentRole:
-            return to_qvariant(Qt.AlignCenter)
+            return Qt.AlignCenter
 
         elif role == Qt.CheckStateRole and column == 4:
             st = dico[key]
             if st == 'on':
-                return to_qvariant(Qt.Checked)
+                return Qt.Checked
             else:
-                return to_qvariant(Qt.Unchecked)
+                return Qt.Unchecked
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -459,8 +444,8 @@ class StandardItemModelSubplot(QStandardItemModel):
 
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
-            return to_qvariant(self.headers[section])
-        return to_qvariant()
+            return self.headers[section]
+        return None
 
 
     def setData(self, index, value, role=None):
@@ -566,7 +551,7 @@ class StandardItemModelFigure(QStandardItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         row = index.row()
         column = index.column()
@@ -574,15 +559,15 @@ class StandardItemModelFigure(QStandardItemModel):
         key = self.keys[column]
 
         if dico[key] == None:
-            return to_qvariant()
+            return None
 
         if role == Qt.DisplayRole:
-            return to_qvariant(dico[key])
+            return dico[key]
 
         elif role == Qt.TextAlignmentRole:
-            return to_qvariant(Qt.AlignCenter)
+            return Qt.AlignCenter
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -596,8 +581,8 @@ class StandardItemModelFigure(QStandardItemModel):
 
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
-            return to_qvariant(self.headers[section])
-        return to_qvariant()
+            return self.headers[section]
+        return None
 
 
     def setData(self, index, value, role=None):
@@ -687,17 +672,17 @@ class TreeItem(object):
     def data(self, column, role):
         if self.item == None:
             if column == 0:
-                return to_qvariant(self.header)
+                return self.header
             else:
-                return to_qvariant()
+                return None
         else:
             if column == 0 and role == Qt.DisplayRole:
-                return to_qvariant(self.item.tpe)
+                return self.item.tpe
             elif column == 1 and role == Qt.DisplayRole:
-                return to_qvariant(self.item.name)
+                return self.item.name
             elif column == 2 and role == Qt.DisplayRole:
-                return to_qvariant(self.item.idlist)
-        return to_qvariant()
+                return self.item.idlist
+        return None
 
 
     def parent(self):
@@ -723,9 +708,7 @@ class StandardItemModelMeasurement(QAbstractItemModel):
         self.study = study
 
         self.noderoot = {}
-        self.prtlist = []
-        for name in self.mdl.getMeasurementList(self.study):
-            self.prtlist.append(name)
+        self.prtlist = self.mdl.getMeasurementList(self.study)
 
         self.rootItem = TreeItem(None, "ALL", None)
         self.parents = {0 : self.rootItem}
@@ -742,28 +725,28 @@ class StandardItemModelMeasurement(QAbstractItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         item = index.internalPointer()
 
         # ToolTips
         if role == Qt.ToolTipRole:
-            return to_qvariant()
+            return None
 
         # StatusTips
         if role == Qt.StatusTipRole:
             if index.column() == 0:
-                return to_qvariant(self.tr("type"))
+                return self.tr("measurement file")
             elif index.column() == 1:
-                return to_qvariant(self.tr("identification"))
+                return self.tr("plot id")
             elif index.column() == 2:
-                return to_qvariant(self.tr("subplot id list"))
+                return self.tr("subplot id")
 
         # Display
         if role == Qt.DisplayRole:
             return item.data(index.column(), role)
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -776,12 +759,12 @@ class StandardItemModelMeasurement(QAbstractItemModel):
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
             if section == 0:
-                return to_qvariant(self.tr("type"))
+                return self.tr("measurement file")
             elif section == 1:
-                return to_qvariant(self.tr("identification"))
+                return self.tr("plot id")
             elif section == 2:
-                return to_qvariant(self.tr("subplot id list"))
-        return to_qvariant()
+                return self.tr("subplot id")
+        return None
 
 
     def index(self, row, column, parent = QModelIndex()):
@@ -831,19 +814,20 @@ class StandardItemModelMeasurement(QAbstractItemModel):
 
 
     def populateModel(self):
-        for name in self.prtlist:
+        for (name, path) in self.prtlist:
             row = self.rowCount()
-            item = item_class("measurement", name, "")
+            ms_file = os.path.join(path, name)
+            item = item_class(ms_file, "", "")
             newparent = TreeItem(item, name, self.rootItem)
             self.rootItem.appendChild(newparent)
             self.noderoot[name] = newparent
 
         measurement_idx = 0
-        for name in self.prtlist:
-            for idx in self.mdl.getMeasurementPlotList(self.study, name):
+        for (name, path) in self.prtlist:
+            for idx in self.mdl.getMeasurementPlotList(self.study, name, path):
                 parentItem = self.noderoot[name]
                 idlist = self.mdl.getMeasurementIdList(self.study, measurement_idx, idx)
-                item = item_class("plot", str(idx), idlist)
+                item = item_class("", str(idx), idlist)
                 new_item = TreeItem(item, "", parentItem)
                 parentItem.appendChild(new_item)
             measurement_idx = measurement_idx + 1
@@ -881,28 +865,28 @@ class StandardItemModelCase(QAbstractItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         item = index.internalPointer()
 
         # ToolTips
         if role == Qt.ToolTipRole:
-            return to_qvariant()
+            return None
 
         # StatusTips
         if role == Qt.StatusTipRole:
             if index.column() == 0:
-                return to_qvariant(self.tr("type"))
+                return self.tr("type")
             elif index.column() == 1:
-                return to_qvariant(self.tr("identification"))
+                return self.tr("identification")
             elif index.column() == 2:
-                return to_qvariant(self.tr("subplot id list"))
+                return self.tr("subplot id list")
 
         # Display
         if role == Qt.DisplayRole:
             return item.data(index.column(), role)
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -915,12 +899,12 @@ class StandardItemModelCase(QAbstractItemModel):
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
             if section == 0:
-                return to_qvariant(self.tr("type"))
+                return self.tr("type")
             elif section == 1:
-                return to_qvariant(self.tr("identification"))
+                return self.tr("identification")
             elif section == 2:
-                return to_qvariant(self.tr("subplot id list"))
-        return to_qvariant()
+                return self.tr("subplot id list")
+        return None
 
 
     def index(self, row, column, parent = QModelIndex()):
@@ -1695,12 +1679,6 @@ class ManagePlotterView(QWidget, Ui_ManagePlotterForm):
         self.slotChangeSelectionCases()
 
 
-    def tr(self, text):
-        """
-        Translation
-        """
-        return text
-
 #-------------------------------------------------------------------------------
 # Testing part
 #-------------------------------------------------------------------------------
