@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2018 EDF S.A.
+# Copyright (C) 1998-2021 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -58,10 +58,9 @@ from code_saturne.Base.QtWidgets import *
 # Application modules
 #-------------------------------------------------------------------------------
 
-import cs_info
-from cs_exec_environment import \
+from code_saturne import cs_info
+from code_saturne.cs_exec_environment import \
     separate_args, update_command_single_value, assemble_args, enquote_arg
-import cs_runcase
 
 try:
     from code_saturne.trackcvg.MainForm import Ui_MainForm
@@ -69,13 +68,8 @@ except:
     sys.path.insert(1, os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "Base"))
     from code_saturne.trackcvg.MainForm import Ui_MainForm
 
-try:
-    import code_saturne.trackcvg
-except:
-    sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
-
 from code_saturne.Base.QtPage import getexistingdirectory
-from code_saturne.Base.QtPage import DoubleValidator, from_qvariant, to_qvariant
+from code_saturne.Base.QtPage import DoubleValidator, from_qvariant, to_text_string
 
 import numpy
 import matplotlib
@@ -155,23 +149,23 @@ class TreeItem(object):
     def data(self, column, role):
         if self.item == None:
             if column == 0:
-                return to_qvariant(self.header)
+                self.header
             else:
-                return to_qvariant()
+                return None
         else:
             if column == 0 and role == Qt.DisplayRole:
-                return to_qvariant(self.item.name)
+                return self.item.name
             elif column == 1 and role == Qt.CheckStateRole:
                 value = self.item.status
                 if value == 'on':
-                    return to_qvariant(Qt.Checked)
+                    return Qt.Checked
                 elif value == 'onoff':
-                    return to_qvariant(Qt.PartiallyChecked)
+                    return Qt.PartiallyChecked
                 else:
-                    return to_qvariant(Qt.Unchecked)
+                    return Qt.Unchecked
             elif column == 2 and role == Qt.DisplayRole:
-                return to_qvariant(self.item.subplot_id)
-        return to_qvariant()
+                return self.item.subplot_id
+        return None
 
 
     def parent(self):
@@ -215,22 +209,22 @@ class CaseStandardItemModel(QAbstractItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return to_qvariant()
+            return None
 
         item = index.internalPointer()
 
         # ToolTips
         if role == Qt.ToolTipRole:
-            return to_qvariant()
+            return None
 
         # StatusTips
         if role == Qt.StatusTipRole:
             if index.column() == 0:
-                return to_qvariant(self.tr("File name"))
+                return self.tr("File name")
             elif index.column() == 1:
-                return to_qvariant(self.tr("Status"))
+                return self.tr("Status")
             elif index.column() == 2:
-                return to_qvariant(self.tr("Subplot"))
+                return self.tr("Subplot")
 
         # Display
         if role == Qt.DisplayRole:
@@ -238,7 +232,7 @@ class CaseStandardItemModel(QAbstractItemModel):
         elif role == Qt.CheckStateRole:
             return item.data(index.column(), role)
 
-        return to_qvariant()
+        return None
 
 
     def flags(self, index):
@@ -267,12 +261,12 @@ class CaseStandardItemModel(QAbstractItemModel):
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
             if section == 0:
-                return to_qvariant(self.tr("File name"))
+                return self.tr("File name")
             elif section == 1:
-                return to_qvariant(self.tr("Status"))
+                return self.tr("Status")
             elif section == 2:
-                return to_qvariant(self.tr("Subplot"))
-        return to_qvariant()
+                return self.tr("Subplot")
+        return None
 
 
     def index(self, row, column, parent = QModelIndex()):
@@ -348,8 +342,10 @@ class CaseStandardItemModel(QAbstractItemModel):
             else:
                 item.item.status = "off"
             if item in self.noderoot.values():
-                (name, name_long, status, subplot_id, probes_number) = self.lst[index.row()]
-                self.lst[index.row()] = (name, name_long, item.item.status, subplot_id, probes_number)
+                (name, name_long, status,
+                 subplot_id, probes_number) = self.lst[index.row()]
+                self.lst[index.row()] = (name, name_long, item.item.status,
+                                         subplot_id, probes_number)
                 for it in self.lstFileProbes[name]:
                     it.status = item.item.status
             else:
@@ -369,15 +365,19 @@ class CaseStandardItemModel(QAbstractItemModel):
                 for (name, name_long, status, subplot_id, probes_number) in self.lst:
                     nm, ext = os.path.splitext(name)
                     if nm == item.parentItem.item.name:
-                        self.lst[idx] = (name, name_long, item.parentItem.item.status, subplot_id, probes_number)
+                        self.lst[idx] = (name, name_long,
+                                         item.parentItem.item.status,
+                                         subplot_id, probes_number)
                     idx = idx + 1
 
         elif index.column() == 2:
             v = from_qvariant(value, int)
             item.item.subplot_id = v
             if item in self.noderoot.values():
-                (name, name_long, status, subplot_id, probes_number) = self.lst[index.row()]
-                self.lst[index.row()] = (name, name_long, status, item.item.subplot_id, probes_number)
+                (name, name_long, status,
+                 subplot_id, probes_number) = self.lst[index.row()]
+                self.lst[index.row()] = (name, name_long, status,
+                                         item.item.subplot_id, probes_number)
                 for it in self.lstFileProbes[name]:
                     it.subplot_id = item.item.subplot_id
             else:
@@ -392,7 +392,8 @@ class CaseStandardItemModel(QAbstractItemModel):
                 for (name, name_long, status, subplot_id, probes_number) in self.lst:
                     nm, ext = os.path.splitext(name)
                     if nm == item.parentItem.item.name:
-                        self.lst[idx] = (name, name_long, status, subplotid, probes_number)
+                        self.lst[idx] = (name, name_long, status,
+                                         subplotid, probes_number)
                     idx = idx + 1
 
         self.dataChanged.emit(QModelIndex(), QModelIndex())
@@ -457,14 +458,16 @@ class MyMplCanvas(FigureCanvas):
 
                 lbl = name + "_s" + str(j)
 
-                self.axes[lstProbes[j].subplot_id - 1].plot(self.xAxe, self.yAxe, label = lbl)
+                self.axes[lstProbes[j].subplot_id - 1].plot(self.xAxe, self.yAxe,
+                                                            label = lbl)
                 self.axes[lstProbes[j].subplot_id - 1].legend(loc="upper left",
                                                               bbox_to_anchor=(1.02,1.0),
                                                               borderaxespad=0.0,
                                                               ncol=1,
                                                               fancybox=True,
                                                               shadow=True,
-                                                              prop={'size':'medium', 'style': 'italic'})
+                                                              prop={'size':'medium',
+                                                                    'style': 'italic'})
 
 
     def update_figure_listing(self, name, data, nb_probes, lstProbes):
@@ -475,14 +478,16 @@ class MyMplCanvas(FigureCanvas):
 
                 lbl = "t res. " + name[j]
 
-                self.axes[lstProbes[j].subplot_id - 1].plot(self.xAxe, self.yAxe, label = lbl)
+                self.axes[lstProbes[j].subplot_id - 1].plot(self.xAxe, self.yAxe,
+                                                            label = lbl)
                 self.axes[lstProbes[j].subplot_id - 1].legend(loc="upper left",
                                                               bbox_to_anchor=(1.02,1.0),
                                                               borderaxespad=0.0,
                                                               ncol=1,
                                                               fancybox=True,
                                                               shadow=True,
-                                                              prop={'size':'medium', 'style': 'italic'})
+                                                              prop={'size':'medium',
+                                                                    'style': 'italic'})
 
 
 
@@ -552,14 +557,19 @@ class MainView(object):
         self.setAttribute(Qt.WA_DeleteOnClose)
         MainView.Instances.add(self)
 
-        self.setWindowTitle(self.package.code_name + " TRACKING CONVERGENCE" + " - " + self.package.version)
+        iconpath = os.path.split(os.path.dirname(os.path.abspath(__file__)))[0]
+        iconpath = os.path.join(iconpath, "Base", "logo_salome_cfd.png")
+        icon = QIcon(QPixmap(iconpath))
+        self.setWindowIcon(icon)
+
+        self.setWindowTitle(self.package.code_name + " convergence plot" + " - " + self.package.version)
 
         # Validator
         validator = DoubleValidator(self.lineEditTime, min=0.0)
         self.lineEditTime.setValidator(validator)
 
         # connections
-        self.fileCloseAction.triggered.connect(self.close)
+        self.fileCloseAction.triggered.connect(self.caseClose)
         self.fileQuitAction.triggered.connect(self.fileQuit)
         self.actionSave_state.triggered.connect(self.SaveState)
         self.actionLoad_state.triggered.connect(self.LoadState)
@@ -631,7 +641,9 @@ class MainView(object):
         self.lineEditTime.setText(str(self.timeRefresh))
 
         # treeViewDirectory
-        self.modelCases = CaseStandardItemModel(self.parent, self.fileList, self.listFileProbes)
+        self.modelCases = CaseStandardItemModel(self.parent,
+                                                self.fileList,
+                                                self.listFileProbes)
         self.treeViewDirectory.setModel(self.modelCases)
         self.treeViewDirectory.setAlternatingRowColors(True)
         self.treeViewDirectory.setSelectionBehavior(QAbstractItemView.SelectItems)
@@ -652,7 +664,8 @@ class MainView(object):
 
         # gestion des figures
         l = QVBoxLayout(self.widget)
-        self.dc = MyMplCanvas(self.widget, subplotNb=self.subplotNumber, width=5, height=4, dpi=50)
+        self.dc = MyMplCanvas(self.widget, subplotNb=self.subplotNumber,
+                              width=5, height=4, dpi=50)
         l.addWidget(self.dc)
 
         # this is the Navigation widget
@@ -665,6 +678,32 @@ class MainView(object):
         self.updateView()
 
 
+    def caseClose(self):
+        """
+        public slot
+
+        try to quit all the current MainWindow
+        """
+        if self.caseName != None:
+            title = self.tr("Close Case")
+            msg   = self.tr("Save current state?")
+            reply = QMessageBox.question(self, title, msg,
+                                         QMessageBox.Yes | QMessageBox.No)
+            if reply == QMessageBox.Yes:
+                self.SaveState()
+
+        self.toolButtonDir.setStyleSheet("background-color: red")
+        self.lineEditCase.setText("")
+        self.caseName = None
+        self.fileList = []
+        self.listingVariable = []
+        self.listFileProbes = {}
+        self.modelCases = CaseStandardItemModel(self.parent, [], [])
+        self.treeViewDirectory.setModel(self.modelCases)
+        self.modelCases.dataChanged.connect(self.treeViewChanged)
+        self.updateView()
+
+
     def closeEvent(self, event):
         """
         public slot
@@ -672,19 +711,14 @@ class MainView(object):
         try to quit all the current MainWindow
         """
         if self.caseName != None:
-            title = self.tr("Quit")
-            msg   = self.tr("Save current state?")
-            reply = QMessageBox.question(self, title, msg,
-                                         QMessageBox.Yes | QMessageBox.No)
-            if reply == QMessageBox.Yes:
-                self.SaveState()
+            self.caseClose()
 
-            settings = QSettings()
+        settings = QSettings()
 
-            settings.setValue("MainWindow/Geometry",
-                              self.saveGeometry())
-            settings.setValue("MainWindow/State",
-                              self.saveState())
+        settings.setValue("MainWindow/Geometry",
+                          self.saveGeometry())
+        settings.setValue("MainWindow/State",
+                          self.saveState())
 
         event.accept()
 
@@ -725,7 +759,7 @@ class MainView(object):
               self.package.bugreport + "\n\n"               +\
               "Please visit our site:\n"                    +\
               self.package.url
-        QMessageBox.about(self, self.package.name + ' Interface', msg)
+        QMessageBox.about(self, self.package.name + ' convergence plot', msg)
 
 
     def displayLicence(self):
@@ -734,7 +768,8 @@ class MainView(object):
 
         GNU GPL license dialog window
         """
-        QMessageBox.about(self, self.package.code_name + ' Interface', "see COPYING file")
+        QMessageBox.about(self, self.package.code_name + ' convergence plot',
+                          cs_info.licence_text)
 
 
     def displayConfig(self):
@@ -743,7 +778,8 @@ class MainView(object):
 
         configuration information window
         """
-        QMessageBox.about(self, self.package.code_name + ' Interface', "see config.py")
+        QMessageBox.about(self, self.package.code_name + ' convergence plot',
+                          "see config.py")
 
 
     def setColor(self):
@@ -798,8 +834,6 @@ class MainView(object):
             if self.palette_default:
                 app.setPalette(self.palette_default)
             if self.font_default:
-                print(self.font_default)
-                print(self.font())
                 self.setFont(self.font_default)
                 app.setFont(self.font_default)
             settings = QSettings()
@@ -897,6 +931,14 @@ class MainView(object):
 
 
     def slotOpenCase(self):
+
+        # Save previous case before switching
+
+        if self.caseName != None:
+            self.caseClose()
+
+        # Load new case
+
         title = self.tr("Choose a result directory")
 
         path = os.getcwd()
@@ -954,21 +996,28 @@ class MainView(object):
                 if name == 'residuals.csv':
                     data = self.ReadCsvFile(fle, probes_number)
                     if status == "on" or status == "onoff":
-                        self.dc.update_figure_listing(self.listingVariable, data, probes_number, self.listFileProbes[name])
+                        self.dc.update_figure_listing(self.listingVariable,
+                                                      data,
+                                                      probes_number,
+                                                      self.listFileProbes[name])
                 elif name == 'residuals.dat':
                     data = self.ReadDatFile(fle, probes_number)
                     if status == "on" or status == "onoff":
-                        self.dc.update_figure_listing(self.listingVariable, data, probes_number, self.listFileProbes[name])
+                        self.dc.update_figure_listing(self.listingVariable,
+                                                      data, probes_number,
+                                                      self.listFileProbes[name])
                 elif ext == ".csv":
                     data = self.ReadCsvFile(fle, probes_number)
                     nm, ext = os.path.splitext(name)
                     nm = nm[7:]
-                    self.dc.update_figure(nm, data, probes_number, self.listFileProbes[name])
+                    self.dc.update_figure(nm, data, probes_number,
+                                          self.listFileProbes[name])
                 elif ext == ".dat":
                     data = self.ReadDatFile(fle, probes_number)
                     nm, ext = os.path.splitext(name)
                     nm = nm[7:]
-                    self.dc.update_figure(nm, data, probes_number, self.listFileProbes[name])
+                    self.dc.update_figure(nm, data, probes_number,
+                                          self.listFileProbes[name])
         self.dc.drawFigure()
 
 
@@ -980,9 +1029,11 @@ class MainView(object):
         ficIn= open(name, 'r')
         typ = 0 # O time / 1 iteration
         for line in ficIn.readlines():
-            line = line.strip('\n')
+            e_id = line.find('\n')
+            if e_id < 0:
+                continue
             if comp > 0:
-                content = line.split(',')
+                content = line[:e_id].split(',')
 
                 if comp == 1:
                     if type(content[0]) == int:
@@ -990,15 +1041,11 @@ class MainView(object):
 
                 if typ == 0:
                     for el in content:
-                        el = el.lstrip()
-                        el = el.rstrip()
-                        data.append(el)
+                        data.append(float(el))
                 else:
                     data.append(float(content[0]))
                     for el in content[1:]:
-                        el = el.lstrip()
-                        el = el.rstrip()
-                        data.append(el)
+                        data.append(float(el))
             comp = comp + 1
         ficIn.close()
         A = numpy.array(data)
@@ -1037,7 +1084,10 @@ class MainView(object):
         for line in ficIn.readlines():
             if not line.startswith("#"):
                 comp = comp + 1
-                line = line.lstrip()
+                e_id = line.find('\n')
+                if e_id < 0:
+                    continue
+                line = line[:e_id].lstrip()
                 content = line.split()
 
                 if comp == 1:
@@ -1046,11 +1096,11 @@ class MainView(object):
 
                 if typ == 0:
                     for el in content:
-                        data.append(el)
+                        data.append(float(el))
                 else:
                     data.append(float(content[0]))
                     for el in content[1:]:
-                        data.append(el)
+                        data.append(float(el))
         ficIn.close()
         A = numpy.array(data)
         n = A.shape[0]
@@ -1085,15 +1135,12 @@ class MainView(object):
         """
         ficIn= open(name, 'r')
         lst = []
-        if self.package.name == 'code_saturne':
-            line  = ficIn.readline()
-            line = line.strip('\n')
-            line = line.lstrip()
-            content = line.split(',')
-            for el in content[1:]:
-                lst.append(el)
-        elif self.package.name == 'neptune_cfd':
-            print("to complete")
+        line  = ficIn.readline()
+        line = line.strip('\n')
+        line = line.lstrip()
+        content = line.split(',')
+        for el in content[1:]:
+            lst.append(el)
         ficIn.close()
         return lst
 
@@ -1103,14 +1150,11 @@ class MainView(object):
         """
         ficIn= open(name, 'r')
         lst = []
-        if self.package.name == 'code_saturne':
-            line  = ficIn.readline()
-            line = line.lstrip()
-            content = line.split()
-            for el in content[1:]:
-                lst.append(el)
-        elif self.package.name == 'neptune_cfd':
-            print("to complete")
+        line  = ficIn.readline()
+        line = line.lstrip()
+        content = line.split()
+        for el in content[1:]:
+            lst.append(el)
         ficIn.close()
         return lst
 
@@ -1184,23 +1228,15 @@ class MainView(object):
     def slotRefresh(self):
         """
         """
+        if not self.caseName:
+            return
+
         name = os.path.join(self.caseName, 'control_file')
         ficIn= open(name, 'w')
         ficIn.write('flush\n')
         ficIn.close()
 
 
-    def tr(self, text):
-        """
-        private method
-
-        translation
-
-        @param text: text to translate
-        @return: translated text
-        """
-        return text
-
 #-------------------------------------------------------------------------------
 # Main Window for Code_Saturne
 #-------------------------------------------------------------------------------
@@ -1250,7 +1286,6 @@ class MainViewSaturne(QMainWindow, Ui_MainForm, MainView):
         self.displayCSManualAction.triggered.connect(self.displayCSManual)
         self.displayCSTutorialAction.triggered.connect(self.displayCSTutorial)
         self.displayCSTheoryAction.triggered.connect(self.displayCSTheory)
-        self.displayCSSmgrAction.triggered.connect(self.displayCSSmgr)
         self.displayCSRefcardAction.triggered.connect(self.displayCSRefcard)
         self.displayCSDoxygenAction.triggered.connect(self.displayCSDoxygen)
 
@@ -1264,8 +1299,6 @@ class MainViewSaturne(QMainWindow, Ui_MainForm, MainView):
             self.displayCSManualAction.setEnabled(False)
         if 'theory.pdf' not in liste:
             self.displayCSTheoryAction.setEnabled(False)
-        if 'studymanager.pdf' not in liste:
-            self.displayCSSmgrAction.setEnabled(False)
         if 'refcard.pdf' not in liste:
             self.displayCSRefcardAction.setEnabled(False)
         if 'doxygen' not in liste:
@@ -1289,7 +1322,7 @@ class MainViewSaturne(QMainWindow, Ui_MainForm, MainView):
         open the tutorial for Code_Saturne
         """
         msg = "See " + self.package.url + " web site for tutorials."
-        QMessageBox.about(self, self.package.name + ' Interface', msg)
+        QMessageBox.about(self, self.package.name + ' convergence plot', msg)
 
 
     def displayCSTheory(self):
