@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 #-------------------------------------------------------------------------------
 # Library modules import
@@ -7,7 +7,7 @@
 import sys
 
 if sys.version_info[:2] < (2,6):
-    sys.stderr.write("This script needs Python 2.6 at least\n")
+    sys.stderr.write("This script needs Python 3.4 at least\n")
 
 import platform
 
@@ -536,9 +536,6 @@ class Setup:
         # Download packages
         self.download = 'yes'
 
-        # Code_Saturne language (may be en/fr)
-        self.language = 'en'
-
         # Code_Saturne installation with debugging symbols
         self.debug = 'no'
 
@@ -597,9 +594,9 @@ class Setup:
             Package(name="HDF5",
                     description="Hierarchical Data Format",
                     package="hdf5",
-                    version="1.10.4",
-                    archive="hdf5-1.10.4.tar.gz",
-                    url="https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-1.10/hdf5-1.10.4/src/%s")
+                    version="1.10.6",
+                    archive="hdf5-1.10.6.tar.gz",
+                    url="https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-1.10/hdf5-1.10.6/src/%s")
 
         p = self.packages['hdf5']
         p.config_opts = "--enable-build-mode=production"
@@ -610,9 +607,9 @@ class Setup:
             Package(name="CGNS",
                     description="CFD General Notation System",
                     package="cgns",
-                    version="3.3.1",
-                    archive="CGNS-3.3.1.tar.gz",
-                    url="https://github.com/CGNS/CGNS/archive/v3.3.1.tar.gz")
+                    version="4.1.2",
+                    archive="CGNS-4.1.2.tar.gz",
+                    url="https://github.com/CGNS/CGNS/archive/v4.1.2.tar.gz")
 
         p = self.packages['cgns']
         p.config_opts = "-DCGNS_ENABLE_64BIT=ON -DCGNS_ENABLE_SCOPING=ON"
@@ -623,8 +620,8 @@ class Setup:
             Package(name="MED",
                     description="Model for Exchange of Data",
                     package="med",
-                    version="4.0.0",
-                    archive="med-4.0.0.tar.gz",
+                    version="4.1.0",
+                    archive="med-4.1.0.tar.gz",
                     url="http://files.salome-platform.org/Salome/other/%s")
 
         p = self.packages['med']
@@ -646,9 +643,9 @@ class Setup:
             Package(name="scotch",
                     description="PT-Scotch",
                     package="scotch",
-                    version="6.0.6",
-                    archive="scotch_6.0.6.tar.gz",
-                    url="https://gforge.inria.fr/frs/download.php/file/37622/%s")
+                    version="6.1.0",
+                    archive="scotch_6.1.0.tar.gz",
+                    url="https://gitlab.inria.fr/scotch/scotch/-/archive/v6.1.0/%s")
 
     #---------------------------------------------------------------------------
 
@@ -659,7 +656,7 @@ class Setup:
         self.cxx = find_executable(['c++', 'g++', 'icpc', 'xlc++', 'clang++'], 'CXX')
         self.mpicc = find_executable(['mpicc', 'mpicc.openmpi', 'mpicc.mpich'])
         self.mpicxx = find_executable(['mpicxx', 'mpicxx.openmpi', 'mpicxx.mpich'])
-        self.python = find_executable(['python'], 'PYTHON')
+        self.python = find_executable(['python3'], 'PYTHON')
 
         # Architecture name
         self.arch = os.uname()[0] + '_' + os.uname()[4]
@@ -760,7 +757,6 @@ to start the installation.
                     if not list[1] in ['default', 'auto']:
                         self.prefix = list[1]
                 elif key == 'debug': self.debug = list[1]
-                elif key == 'language': self.language = list[1]
                 elif key == 'use_arch': self.use_arch = list[1]
                 elif key == 'arch':
                     self.arch = list[1]
@@ -857,14 +853,6 @@ Check the setup file and some utilities presence.
                              "Please check your setup file.\n\n")
             sys.exit(1)
 
-        # Testing language option
-        if self.language not in ['en', 'fr']:
-            sys.stderr.write("\n*** Aborting installation:\n"
-                             "\'language\' option in the setup file "
-                             "should be \'en\' or \'fr'.\n"
-                             "Please check your setup file.\n\n")
-            sys.exit(1)
-
         # Testing prefix directory
         if self.prefix and not os.path.isdir(self.prefix):
             try:
@@ -1040,7 +1028,7 @@ Check the setup file and some utilities presence.
             if hdf5.install_dir:
                 config_opts = config_opts + " --with-hdf5=" + hdf5.install_dir
                 med.config_opts += " --with-hdf5=" + hdf5.install_dir
-                cgns.config_opts += "-DCMAKE_PREFIX_PATH=" + hdf5.install_dir \
+                cgns.config_opts += " -DCMAKE_PREFIX_PATH=" + hdf5.install_dir \
                     + " -DHDF5_INCLUDE_PATH=" + hdf5.install_dir + "/include"
 
         # CGNS
@@ -1083,11 +1071,6 @@ Check the setup file and some utilities presence.
         if self.salome:
             config_opts = config_opts + " --with-salome=" + self.salome
 
-        # Language
-
-        if self.language == 'fr':
-            config_opts = config_opts + " --enable-french"
-
         # Build type
 
         if self.shared:
@@ -1163,13 +1146,6 @@ Check the setup file and some utilities presence.
 download  %(download)s
 #
 #--------------------------------------------------------
-# Language
-#   default: "en" english
-#   others:  "fr" french
-#--------------------------------------------------------
-language  %(lang)s
-#
-#--------------------------------------------------------
 # Install Code_Saturne with debugging symbols
 #--------------------------------------------------------
 debug     %(debug)s
@@ -1299,7 +1275,7 @@ salome    %(salome)s
 
         sf.write(setupMain
                  % { 'download':self.download, 'prefix':prefix,
-                     'lang':self.language, 'debug':self.debug,
+                     'debug':self.debug,
                      'use_arch':self.use_arch, 'arch':arch,
                      'cc':cc, 'mpicc':mpicc,
                      'fc':fc,
