@@ -9,7 +9,7 @@
   This file is part of the "Parallel Location and Exchange" library,
   intended to provide mesh or particle-based code coupling services.
 
-  Copyright (C) 2005-2018  EDF S.A.
+  Copyright (C) 2005-2020  EDF S.A.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -295,7 +295,7 @@ ple_locator_extend_search(ple_locator_t               *this_locator,
                           float                        tolerance_fraction,
                           ple_lnum_t                   n_points,
                           const ple_lnum_t             point_list[],
-                          const ple_lnum_t             point_tag[],
+                          const int                    point_tag[],
                           const ple_coord_t            point_coords[],
                           float                        distance[],
                           ple_mesh_extents_t          *mesh_extents_f,
