@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -386,7 +386,7 @@ _print_help(void)
   _fct_prt("-o", _("<file>"),
            _(": output file name"));
   sprintf(opt_str, _("  (default file: \"%s\")"),
-          "mesh_input");
+          "mesh_input.csm");
   _fct_prt("", "", opt_str);
 
   _fct_prt("--out", _("<file>"), _(": same"));
@@ -965,7 +965,7 @@ ecs_cmd__lit_arg(int    argc,
       else
         ecs_loc_cmd__aff_opt_en_double(argc, argv, argv[iarg]);
 
-      if (argc - 1 > iarg && strncmp(argv[iarg + 1], "-", 1))
+      if (argc - 2 > iarg && strncmp(argv[iarg + 1], "-", 1))
         outfic = argv[++iarg];
       else
         outfic = "preprocessor.log";
@@ -1004,7 +1004,7 @@ ecs_cmd__lit_arg(int    argc,
       if (argc - 1 > iarg && strncmp(argv[iarg + 1], "-", 1))
         outfic = argv[++iarg];
       else
-        outfic = "mesh_input";
+        outfic = "mesh_input.csm";
 
       ECS_MALLOC(cmd->nom_out, strlen(outfic) + 1, char);
 
@@ -1135,9 +1135,9 @@ ecs_cmd__lit_arg(int    argc,
 
   if (cmd->nom_out == NULL && bool_no_write == false) {
     ECS_MALLOC(cmd->nom_out,
-               strlen("mesh_input") + 1,
+               strlen("mesh_input.csm") + 1,
                char);
-    strcpy(cmd->nom_out, "mesh_input");
+    strcpy(cmd->nom_out, "mesh_input.csm");
   }
 
   /*-----------------------------------------*/
