@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -869,9 +869,10 @@ ecs_loc_pre_ens__ele_lin(int32_t     *connect_loc,
     for (iloc = 0; iloc < taille_ele_lin; iloc++)
       connect_loc[ipos_lin++] = connect_loc[ipos_ens++];
 
-    ECS_REALLOC(connect_loc, ipos_lin, int32_t);
   }
 
+  ECS_REALLOC(connect_loc, ipos_lin, int32_t);
+
   return connect_loc;
 }
 
@@ -1709,11 +1710,12 @@ ecs_loc_pre_ens__concat_elems(ecs_maillage_t         *maillage,
   if (liste_noeuds->id_noeud != NULL) {
 
     for (ient = ECS_ENTMAIL_FAC ; ient < ECS_N_ENTMAIL ; ient++)
-      ecs_maillage_pre__label_en_indice
-        (liste_noeuds->nbr_noeuds,
-         elt_pos_som_ent[ient][cpt_elt_ent[ient]] - 1,
-         liste_noeuds->id_noeud,
-         elt_val_som_ent[ient]) ;
+      if (elt_val_som_ent[ient] != NULL)
+        ecs_maillage_pre__label_en_indice
+          (liste_noeuds->nbr_noeuds,
+           elt_pos_som_ent[ient][cpt_elt_ent[ient]],
+           liste_noeuds->id_noeud,
+           elt_val_som_ent[ient]) ;
 
     ECS_FREE(liste_noeuds->id_noeud) ;
   }
@@ -1835,7 +1837,7 @@ ecs_loc_pre_ens__c_bin_endian_6(ecs_file_t  *fic,
     ecs_file_read_check_error(fic, 0);
 
     ecs_error(__FILE__, __LINE__, 0,
-              _("EnSight: positionning (seek) error\n"
+              _("EnSight: positioning (seek) error\n"
                 "in file \"%s\"."), ecs_file_get_name(fic));
   }
 
@@ -2590,4 +2592,3 @@ ecs_pre_ens__lit_maillage(const char  *nom_fic_case,
 }
 
 /*----------------------------------------------------------------------------*/
-
