@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -620,15 +620,15 @@ _read_physical_names(ecs_file_t       *mesh_file,
  *----------------------------------------------------------------------------*/
 
 static void
-_read_entities(int             n_phys_names_max,
-               const int       phys_name_id[],
-               const char    **phys_name,
-               ecs_file_t     *mesh_file,
-               int            *line_num,
-               int             version_fmt_gmsh,
-               int             type_fmt_gmsh,
-               int             fam_shift[ECS_N_ENTMAIL],
-               ecs_famille_t  *vect_famille_tete[ECS_N_ENTMAIL])
+_read_entities(int                 n_phys_names_max,
+               const int           phys_name_id[],
+               const char  *const *phys_name,
+               ecs_file_t         *mesh_file,
+               int                *line_num,
+               int                 version_fmt_gmsh,
+               int                 type_fmt_gmsh,
+               int                 fam_shift[ECS_N_ENTMAIL],
+               ecs_famille_t      *vect_famille_tete[ECS_N_ENTMAIL])
 {
   char   line[ECS_LOC_LNG_MAX_CHAINE_GMSH];
   int    retval;
@@ -1645,9 +1645,7 @@ ecs_loc_pre_gmsh__lit_elements_v4(ecs_maillage_t  *maillage,
   unsigned long  nbr_elt;
   unsigned long  n_ent_blocks;
 
-  long        label;
   ecs_int_t   type_ecs;
-  ecs_int_t   ind_elt;
   ecs_int_t   nbr_elt_lus;
   ecs_int_t   ind_nod_elt;
   ecs_int_t   ind_som_elt;
@@ -1669,8 +1667,9 @@ ecs_loc_pre_gmsh__lit_elements_v4(ecs_maillage_t  *maillage,
   ecs_int_t   *elt_val_som_ent    [ECS_N_ENTMAIL]; /* Numeros des sommets   */
   int         *elt_val_fam_ent    [ECS_N_ENTMAIL]; /* Groupes (familles)    */
 
-  ecs_int_t cpt_are   = 0;
-  ecs_int_t cpt_point = 0;
+  ecs_int_t  ind_elt = 0;
+  ecs_int_t  cpt_are   = 0;
+  ecs_int_t  cpt_point = 0;
 
   int        retour;
   bool       ligne_decodee = true;
@@ -1783,9 +1782,9 @@ ecs_loc_pre_gmsh__lit_elements_v4(ecs_maillage_t  *maillage,
         ecs_error(__FILE__, __LINE__, 0,
                   _("Error reading a Gmsh mesh file:\n"
                     "at line %ld of file \"%s\".\n"
-                    "Type identifier <%d> for element <%ld> is not recognized."),
+                    "Type identifier <%d> for element bloc <%ld> is not recognized."),
                   (long)(*num_ligne), ecs_file_get_name(fic_maillage),
-                  (int)type_gmsh, (long)label);
+                  (int)type_gmsh, (long)eb);
 
       type_ecs         = ecs_gmsh_elt_liste_c[type_gmsh - 1].ecs_typ;
       nbr_nod_elt_gmsh = ecs_gmsh_elt_liste_c[type_gmsh - 1].nbr_som;
@@ -1821,8 +1820,6 @@ ecs_loc_pre_gmsh__lit_elements_v4(ecs_maillage_t  *maillage,
           continue;
         }
 
-        label = atol(ssch);
-
         /* Lecture des numéros des sommets de l'élément courant */
 
         for (ind_nod_elt = 0; ind_nod_elt < nbr_nod_elt_gmsh; ind_nod_elt++) {
@@ -1948,8 +1945,6 @@ ecs_loc_pre_gmsh__lit_elements_v4(ecs_maillage_t  *maillage,
             continue;
           }
 
-          label = data[0];
-
           /* Lecture des numéros des sommets de l'élément courant */
 
           for (ind_nod_elt = 0; ind_nod_elt < nbr_nod_elt_gmsh; ind_nod_elt++)
@@ -1972,8 +1967,6 @@ ecs_loc_pre_gmsh__lit_elements_v4(ecs_maillage_t  *maillage,
             continue;
           }
 
-          label = data[0];
-
           /* Lecture des numéros des sommets de l'élément courant */
 
           for (ind_nod_elt = 0; ind_nod_elt < nbr_nod_elt_gmsh; ind_nod_elt++)
@@ -2183,7 +2176,7 @@ ecs_pre_gmsh__lit_maillage(const char  *nom_fic_maillage)
       ecs_famille_t  *vect_famille[ECS_N_ENTMAIL];
       _read_entities(n_phys_names_max,
                      phys_name_id,
-                     (const char**)phys_name,
+                     (const char *const *)phys_name,
                      fic_maillage,
                      &num_ligne,
                      version_fmt_gmsh,
