@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,6 +54,7 @@
 #include "cs_mesh_bad_cells.h"
 #include "cs_mesh_quantities.h"
 #include "cs_parall.h"
+#include "cs_parameters.h"
 #include "cs_sles_default.h"
 
 /*----------------------------------------------------------------------------
@@ -158,8 +159,7 @@ cs_bad_cells_regularisation_scalar(cs_real_t *var)
   cs_real_t epsilp = 1.e-12;
 
   /* Matrix block size */
-  int ibsize = 1;
-  int db_size[4] = {ibsize, ibsize, ibsize, ibsize*ibsize};
+  cs_lnum_t db_size[4] = {1, 1, 1, 1};
 
   cs_sles_solve_native(-1, /* f_id */
                        "potential_regularisation_scalar",
@@ -221,7 +221,7 @@ cs_bad_cells_regularisation_vector(cs_real_3_t  *var,
   cs_lnum_t n_i_faces = mesh->n_i_faces;
   cs_lnum_t n_b_faces = mesh->n_b_faces;
   const cs_lnum_2_t *i_face_cells = (const cs_lnum_2_t *)mesh->i_face_cells;
-  const int *b_face_cells = mesh->b_face_cells;
+  const cs_lnum_t *b_face_cells = mesh->b_face_cells;
 
   const cs_real_t *surfn = mq->i_face_surf;
   const cs_real_t *surfbn = mq->b_face_surf;
@@ -337,8 +337,7 @@ cs_bad_cells_regularisation_vector(cs_real_3_t  *var,
   cs_real_t epsilp = 1.e-12;
 
   /* Matrix block size */
-  int ibsize = 3;
-  int db_size[4] = {ibsize, ibsize, ibsize, ibsize*ibsize};
+  cs_lnum_t db_size[4] = {3, 3, 3, 3*3};
 
   cs_sles_solve_native(-1, /* f_id */
                        "potential_regularisation_vector",
@@ -402,18 +401,12 @@ cs_bad_cells_regularisation_sym_tensor(cs_real_6_t  *var,
   cs_lnum_t n_cells_ext = mesh->n_cells_with_ghosts;
   cs_lnum_t n_cells = mesh->n_cells;
   cs_lnum_t n_i_faces = mesh->n_i_faces;
-  cs_lnum_t n_b_faces = mesh->n_b_faces;
   const cs_lnum_2_t *i_face_cells = (const cs_lnum_2_t *)mesh->i_face_cells;
-  const int *b_face_cells = mesh->b_face_cells;
 
   const cs_real_t *surfn = mq->i_face_surf;
-  const cs_real_t *surfbn = mq->b_face_surf;
   double *dist = mq->i_dist;
-  double *distbr = mq->b_dist;
   double *volume  = mq->cell_vol;
 
-  const cs_real_3_t *surfbo = (const cs_real_3_t *) mq->b_face_normal;
-
   cs_real_66_t *dam;
   cs_real_6_t *rhs;
   cs_real_t *xam;
@@ -488,8 +481,15 @@ cs_bad_cells_regularisation_sym_tensor(cs_real_6_t  *var,
     }
   }
 
-  /* Boudanry projection... should be consistent with BCs... */
+  /* Boundary projection... should be consistent with BCs... */
+#if 0
   if (boundary_projection == 1) {
+    cs_lnum_t n_b_faces = mesh->n_b_faces;
+    const cs_lnum_t *b_face_cells = mesh->b_face_cells;
+    const cs_real_3_t *surfbo = (const cs_real_3_t *) mq->b_face_normal;
+    const cs_real_t *surfbn = mq->b_face_surf;
+    double *distbr = mq->b_dist;
+
     for (cs_lnum_t face_id = 0; face_id < n_b_faces; face_id++) {
       if (cs_glob_bc_type[face_id] == CS_SMOOTHWALL ||
           cs_glob_bc_type[face_id] == CS_ROUGHWALL  ||
@@ -501,13 +501,14 @@ cs_bad_cells_regularisation_sym_tensor(cs_real_6_t  *var,
             for (int j = 0; j < 3; j++) {
               double nn =   surfbo[face_id][i]/surfbn[face_id]
                           * surfbo[face_id][j]/surfbn[face_id];
-//TODO ???              dam[cell_id][i][j] += ssd * nn;
+              //TODO ???    dam[cell_id][i][j] += ssd * nn;
             }
           }
         }
       }
     }
   }
+#endif
 
   cs_real_t rnorm = sqrt(cs_gdot(6*n_cells,
                                  (const cs_real_t *)rhs,
@@ -520,8 +521,7 @@ cs_bad_cells_regularisation_sym_tensor(cs_real_6_t  *var,
   cs_real_t epsilp = 1.e-12;
 
   /* Matrix block size */
-  int ibsize = 6;
-  int db_size[4] = {ibsize, ibsize, ibsize, ibsize*ibsize};
+  cs_lnum_t db_size[4] = {6, 6, 6, 6*6};
 
   cs_sles_solve_native(-1, /* f_id */
                        "potential_regularisation_sym_tensor",
@@ -584,9 +584,7 @@ cs_bad_cells_regularisation_tensor(cs_real_9_t  *var,
   cs_lnum_t n_cells_ext = mesh->n_cells_with_ghosts;
   cs_lnum_t n_cells = mesh->n_cells;
   cs_lnum_t n_i_faces = mesh->n_i_faces;
-  cs_lnum_t n_b_faces = mesh->n_b_faces;
   const cs_lnum_2_t *i_face_cells = (const cs_lnum_2_t *)mesh->i_face_cells;
-  const int *b_face_cells = mesh->b_face_cells;
 
   const cs_real_t *surfn = mq->i_face_surf;
   const cs_real_t *surfbn = mq->b_face_surf;
@@ -670,8 +668,11 @@ cs_bad_cells_regularisation_tensor(cs_real_9_t  *var,
     }
   }
 
+#if 0
   /* Boudanry projection... should be consistent with BCs... */
   if (boundary_projection == 1) {
+    cs_lnum_t n_b_faces = mesh->n_b_faces;
+    const cs_lnum_t *b_face_cells = mesh->b_face_cells;
     for (cs_lnum_t face_id = 0; face_id < n_b_faces; face_id++) {
       if (cs_glob_bc_type[face_id] == CS_SMOOTHWALL ||
           cs_glob_bc_type[face_id] == CS_ROUGHWALL  ||
@@ -690,6 +691,7 @@ cs_bad_cells_regularisation_tensor(cs_real_9_t  *var,
       }
     }
   }
+#endif
 
   cs_real_t rnorm = sqrt(cs_gdot(9*n_cells,
                                  (const cs_real_t *)rhs,
@@ -702,8 +704,7 @@ cs_bad_cells_regularisation_tensor(cs_real_9_t  *var,
   cs_real_t epsilp = 1.e-12;
 
   /* Matrix block size */
-  int ibsize = 9;
-  int db_size[4] = {ibsize, ibsize, ibsize, ibsize*ibsize};
+  cs_lnum_t db_size[4] = {9, 9, 9, 9*9};
 
   cs_sles_solve_native(-1, /* f_id */
                        "potential_regularisation_tensor",
@@ -746,7 +747,6 @@ cs_bad_cells_regularisation_tensor(cs_real_9_t  *var,
   BFT_FREE(rhs);
 }
 
-
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
