@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -58,6 +58,7 @@
 #include "cs_log.h"
 #include "cs_math.h"
 #include "cs_mesh.h"
+#include "cs_convection_diffusion.h"
 #include "cs_field.h"
 #include "cs_field_operator.h"
 #include "cs_field_pointer.h"
@@ -69,8 +70,7 @@
 #include "cs_parameters.h"
 #include "cs_prototypes.h"
 #include "cs_timer.h"
-#include "cs_stokes_model.h"
-#include "cs_convection_diffusion.h"
+#include "cs_velocity_pressure.h"
 
 /*----------------------------------------------------------------------------
  *  Header for the current file
@@ -119,8 +119,11 @@ BEGIN_C_DECLS
  * Options for the convective scheme:
  * - blencp = 0: upwind scheme for the advection
  * - blencp = 1: no upwind scheme except in the slope test
- * - ischcp = 0: second order
+ * - ischcp = 0: SOLU
  * - ischcp = 1: centered
+ * - ischcp = 2: SOLU with upwind gradient reconstruction
+ * - ischcp = 3: blending SOLU centered
+ * - ischcp = 4: NVD-TVD
  * - imucpp = 0: do not multiply the convective part by \f$ C_p \f$
  * - imucpp = 1: multiply the convective part by \f$ C_p \f$
  *
@@ -206,7 +209,8 @@ cs_balance_scalar(int                idtvar,
   cs_var_cal_opt_t var_cal_opt_loc;
 
   if (f_id < 0) {
-    var_cal_opt_loc.iwarni   = var_cal_opt->iwarni;
+    var_cal_opt_loc = cs_parameters_var_cal_opt_default();
+    var_cal_opt_loc.verbosity   = var_cal_opt->verbosity;
     var_cal_opt_loc.iconv    = var_cal_opt->iconv;
     var_cal_opt_loc.istat    = -1; /* unused in balance */
     var_cal_opt_loc.idiff    = var_cal_opt->idiff;
@@ -378,8 +382,11 @@ cs_balance_scalar(int                idtvar,
  * Options for the convective scheme:
  * - blencp = 0: upwind scheme for the advection
  * - blencp = 1: no upwind scheme except in the slope test
- * - ischcp = 0: second order
+ * - ischcp = 0: SOLU
  * - ischcp = 1: centered
+ * - ischcp = 2: SOLU with upwind gradient reconstruction
+ * - ischcp = 3: blending SOLU centered
+ * - ischcp = 4: NVD-TVD
  *
  * \param[in]     idtvar        indicator of the temporal scheme
  * \param[in]     f_id          field id (or -1)
@@ -460,7 +467,8 @@ cs_balance_vector(int                  idtvar,
   cs_var_cal_opt_t var_cal_opt_loc;
 
   if (f_id < 0) {
-    var_cal_opt_loc.iwarni   = var_cal_opt->iwarni;
+    var_cal_opt_loc = cs_parameters_var_cal_opt_default();
+    var_cal_opt_loc.verbosity   = var_cal_opt->verbosity;
     var_cal_opt_loc.iconv    = var_cal_opt->iconv;
     var_cal_opt_loc.istat    = -1; /* unused in balance */
     var_cal_opt_loc.idiff    = var_cal_opt->idiff;
@@ -564,7 +572,6 @@ cs_balance_vector(int                  idtvar,
                                             cofbfv,
                                             i_visc,
                                             b_visc,
-                                            secvif,
                                             viscel,
                                             weighf,
                                             weighb,
@@ -613,8 +620,11 @@ cs_balance_vector(int                  idtvar,
  * Options for the convective scheme:
  * - blencp = 0: upwind scheme for the advection
  * - blencp = 1: no upwind scheme except in the slope test
- * - ischcp = 0: second order
+ * - ischcp = 0: SOLU
  * - ischcp = 1: centered
+ * - ischcp = 2: SOLU with upwind gradient reconstruction
+ * - ischcp = 3: blending SOLU centered
+ * - ischcp = 4: NVD-TVD
  *
  * \param[in]     idtvar        indicator of the temporal scheme
  * \param[in]     f_id          field id (or -1)
@@ -683,7 +693,8 @@ cs_balance_tensor(int                 idtvar,
   cs_var_cal_opt_t var_cal_opt_loc;
 
   if (f_id < 0) {
-    var_cal_opt_loc.iwarni   = var_cal_opt->iwarni;
+    var_cal_opt_loc = cs_parameters_var_cal_opt_default();
+    var_cal_opt_loc.verbosity   = var_cal_opt->verbosity;
     var_cal_opt_loc.iconv    = var_cal_opt->iconv;
     var_cal_opt_loc.istat    = -1; /* unused in balance */
     var_cal_opt_loc.idiff    = var_cal_opt->idiff;
