@@ -4,7 +4,7 @@
 
 /* This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -60,6 +60,7 @@
 #include "cs_ext_neighborhood.h"
 #include "cs_mesh_quantities.h"
 #include "cs_parameters.h"
+#include "cs_porous_model.h"
 #include "cs_prototypes.h"
 #include "cs_timer.h"
 
@@ -107,15 +108,15 @@ BEGIN_C_DECLS
 
 void CS_PROCF (inimav, INIMAV)
 (
- const cs_int_t   *const  f_id,
- const cs_int_t   *const  itypfl,
- const cs_int_t   *const  iflmb0,
- const cs_int_t   *const  init,
- const cs_int_t   *const  inc,
- const cs_int_t   *const  imrgra,
- const cs_int_t   *const  nswrgu,
- const cs_int_t   *const  imligu,
- const cs_int_t   *const  iwarnu,
+ const int        *const  f_id,
+ const int        *const  itypfl,
+ const int        *const  iflmb0,
+ const int        *const  init,
+ const int        *const  inc,
+ const int        *const  imrgra,
+ const int        *const  nswrgu,
+ const int        *const  imligu,
+ const int        *const  iwarnu,
  const cs_real_t  *const  epsrgu,
  const cs_real_t  *const  climgu,
  const cs_real_t          rom[],
@@ -158,10 +159,10 @@ void CS_PROCF (inimav, INIMAV)
 
 void CS_PROCF (divmas, DIVMAS)
 (
- const cs_int_t  *const   init,
- const cs_real_t          i_massflux[],
- const cs_real_t          b_massflux[],
- cs_real_t                diverg[]
+ const int       *const init,
+ const cs_real_t        i_massflux[],
+ const cs_real_t        b_massflux[],
+ cs_real_t              diverg[]
 )
 {
   const cs_mesh_t  *m = cs_glob_mesh;
@@ -179,10 +180,10 @@ void CS_PROCF (divmas, DIVMAS)
 
 void CS_PROCF (divmat, DIVMAT)
 (
- const cs_int_t    *const   init,
- const cs_real_3_t          i_massflux[],
- const cs_real_3_t          b_massflux[],
- cs_real_3_t                diverg[]
+ const int         *const init,
+ const cs_real_3_t        i_massflux[],
+ const cs_real_3_t        b_massflux[],
+ cs_real_3_t              diverg[]
 )
 {
   const cs_mesh_t  *m = cs_glob_mesh;
@@ -200,8 +201,8 @@ void CS_PROCF (divmat, DIVMAT)
 
 void CS_PROCF (projts, PROJTS)
 (
- const cs_int_t  *const   init,
- const cs_int_t  *const   nswrgu,
+ const int       *const   init,
+ const int       *const   nswrgu,
  const cs_real_3_t        frcxt[],
  const cs_real_t          cofbfp[],
  cs_real_t                i_massflux[],
@@ -237,9 +238,9 @@ void CS_PROCF (projts, PROJTS)
 
 void CS_PROCF (projtv, PROJTV)
 (
- const cs_int_t  *const   init,
- const cs_int_t  *const   nswrgu,
- const cs_int_t  *const   ircflp,
+ const int       *const   init,
+ const int       *const   nswrgu,
+ const int       *const   ircflp,
  const cs_real_3_t        frcxt[],
  const cs_real_t          cofbfp[],
  const cs_real_t          i_visc[],
@@ -273,15 +274,15 @@ void CS_PROCF (projtv, PROJTV)
 
 void CS_PROCF (divrij, DIVRIJ)
 (
- const cs_int_t   *const  f_id,
- const cs_int_t   *const  itypfl,
- const cs_int_t   *const  iflmb0,
- const cs_int_t   *const  init,
- const cs_int_t   *const  inc,
- const cs_int_t   *const  imrgra,
- const cs_int_t   *const  nswrgu,
- const cs_int_t   *const  imligu,
- const cs_int_t   *const  iwarnu,
+ const int        *const  f_id,
+ const int        *const  itypfl,
+ const int        *const  iflmb0,
+ const int        *const  init,
+ const int        *const  inc,
+ const int        *const  imrgra,
+ const int        *const  nswrgu,
+ const int        *const  imligu,
+ const int        *const  iwarnu,
  const cs_real_t  *const  epsrgu,
  const cs_real_t  *const  climgu,
  const cs_real_t          rom[],
@@ -345,8 +346,7 @@ void CS_PROCF (divrij, DIVRIJ)
  * \param[in]     itypfl        indicator (take rho into account or not)
  *                               - 1 compute \f$ \rho\vect{u}\cdot\vect{s} \f$
  *                               - 0 compute \f$ \vect{u}\cdot\vect{s} \f$
- * \param[in]     iflmb0        the mass flux is set to 0 on walls and
- *                               symmetries if = 1
+ * \param[in]     iflmb0        the mass flux is set to 0 on symmetries if = 1
  * \param[in]     init          the mass flux is initialized to 0 if > 0
  * \param[in]     inc           indicator
  *                               - 0 solve an increment
@@ -444,7 +444,7 @@ cs_mass_flux(const cs_mesh_t          *m,
   const cs_real_3_t *restrict dofij
     = (const cs_real_3_t *restrict)fvq->dofij;
 
-  char var_name[32];
+  char var_name[64];
 
   cs_real_3_t *qdm, *f_momentum, *coefaq;
   cs_real_33_t *grdqdm;
@@ -462,7 +462,7 @@ cs_mass_flux(const cs_mesh_t          *m,
   /* Choose gradient type */
 
   cs_halo_type_t halo_type = CS_HALO_STANDARD;
-  cs_gradient_type_t gradient_type = CS_GRADIENT_ITER;
+  cs_gradient_type_t gradient_type = CS_GRADIENT_GREEN_ITER;
 
   cs_gradient_type_by_imrgra(imrgra,
                              &gradient_type,
@@ -470,13 +470,14 @@ cs_mass_flux(const cs_mesh_t          *m,
 
   if (f_id != -1) {
     f = cs_field_by_id(f_id);
-    snprintf(var_name, 31, "%s", f->name); var_name[31] = '\0';
+    snprintf(var_name, 63, "%s", f->name);
   }
   else {
-    strcpy(var_name, "Work array"); var_name[31] = '\0';
+    strncpy(var_name, "[momentum]", 63);
   }
+  var_name[63] = '\0';
 
-  /* ---> Momentum computation */
+  /* Momentum computation */
 
   if (init == 1) {
 #   pragma omp parallel for
@@ -602,19 +603,21 @@ cs_mass_flux(const cs_mesh_t          *m,
           f_momentum[face_id][isou] = romb[face_id]*vel[cell_id][isou];
         }
       }
-      /* With porosity */
-    } else if (porosi != NULL && porosf == NULL) {
+    } /* With porosity */
+    else if (porosi != NULL && porosf == NULL) {
 #     pragma omp parallel for if(m->n_b_faces > CS_THR_MIN)
       for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
         cs_lnum_t cell_id = b_face_cells[face_id];
         for (int isou = 0; isou < 3; isou++) {
           coefaq[face_id][isou] = romb[face_id]
                                  *coefav[face_id][isou]*porosi[cell_id];
-          f_momentum[face_id][isou] = romb[face_id]*vel[cell_id][isou]*porosi[cell_id];
+          f_momentum[face_id][isou] =  romb[face_id]*vel[cell_id][isou]
+                                      *porosi[cell_id];
         }
       }
-      /* With anisotropic porosity */
-    } else if (porosi != NULL && porosf != NULL) {
+
+    } /* With anisotropic porosity */
+    else if (porosi != NULL && porosf != NULL) {
 #     pragma omp parallel for if(m->n_b_faces > CS_THR_MIN)
       for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
         cs_lnum_t cell_id = b_face_cells[face_id];
@@ -658,8 +661,8 @@ cs_mass_flux(const cs_mesh_t          *m,
           f_momentum[face_id][isou] = vel[cell_id][isou];
         }
       }
-      /* With porosity */
-    } else if (porosi != NULL && porosf == NULL) {
+    } /* With porosity */
+    else if (porosi != NULL && porosf == NULL) {
 #     pragma omp parallel for if(m->n_b_faces > CS_THR_MIN)
       for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
         cs_lnum_t cell_id = b_face_cells[face_id];
@@ -668,8 +671,8 @@ cs_mass_flux(const cs_mesh_t          *m,
           f_momentum[face_id][isou] = vel[cell_id][isou]*porosi[cell_id];
         }
       }
-      /* With anisotropic porosity */
-    } else if (porosi != NULL && porosf != NULL) {
+    } /* With anisotropic porosity */
+    else if (porosi != NULL && porosf != NULL) {
 #     pragma omp parallel for if(m->n_b_faces > CS_THR_MIN)
       for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
         cs_lnum_t cell_id = b_face_cells[face_id];
@@ -754,7 +757,7 @@ cs_mass_flux(const cs_mesh_t          *m,
   }
 
   /*==========================================================================
-    4. Compute mass flux with reconstruction technics if the mesh is
+    4. Compute mass flux with reconstruction method if the mesh is
        non orthogonal
     ==========================================================================*/
 
@@ -762,8 +765,7 @@ cs_mass_flux(const cs_mesh_t          *m,
 
     BFT_MALLOC(grdqdm, n_cells_ext, cs_real_33_t);
 
-
-    /* Computation of qdm gradient
+    /* Computation of momentum gradient
        (vectorial gradient, the periodicity has already been treated) */
 
     cs_gradient_vector(var_name,
@@ -899,8 +901,7 @@ cs_mass_flux(const cs_mesh_t          *m,
  * \param[in]     itypfl        indicator (take rho into account or not)
  *                               - 1 compute \f$ \rho\vect{u}\cdot\vect{s} \f$
  *                               - 0 compute \f$ \vect{u}\cdot\vect{s} \f$
- * \param[in]     iflmb0        the mass flux is set to 0 on walls and
- *                               symmetries if = 1
+ * \param[in]     iflmb0        the mass flux is set to 0 on symmetries if = 1
  * \param[in]     init          the mass flux is initialized to 0 if > 0
  * \param[in]     inc           indicator
  *                               - 0 solve an increment
@@ -980,7 +981,7 @@ cs_tensor_face_flux(const cs_mesh_t          *m,
 
   /* Local variables */
 
-  char var_name[32];
+  char var_name[64];
 
   cs_real_6_t *c_mass_var, *b_mass_var, *coefaq;
 
@@ -997,7 +998,7 @@ cs_tensor_face_flux(const cs_mesh_t          *m,
   /* Choose gradient type */
 
   cs_halo_type_t halo_type = CS_HALO_STANDARD;
-  cs_gradient_type_t gradient_type = CS_GRADIENT_ITER;
+  cs_gradient_type_t gradient_type = CS_GRADIENT_GREEN_ITER;
 
   cs_gradient_type_by_imrgra(imrgra,
                              &gradient_type,
@@ -1005,11 +1006,12 @@ cs_tensor_face_flux(const cs_mesh_t          *m,
 
   if (f_id != -1) {
     f = cs_field_by_id(f_id);
-    snprintf(var_name, 31, "%s", f->name); var_name[31] = '\0';
+    snprintf(var_name, 63, "%s", f->name);
   }
   else {
-    strcpy(var_name, "Work array"); var_name[31] = '\0';
+    strncpy(var_name, "[tensor face flux]", 63);
   }
+  var_name[63] = '\0';
 
   /* ---> Momentum computation */
 
@@ -1276,21 +1278,21 @@ cs_tensor_face_flux(const cs_mesh_t          *m,
     BFT_MALLOC(c_grad_mvar, n_cells_ext, cs_real_63_t);
 
     /* Computation of c_mass_var gradient
-       (vectorial gradient, the periodicity has already been treated) */
+       (tensor gradient, the periodicity has already been treated) */
 
-    cs_gradient_tensor(var_name,
-                       gradient_type,
-                       halo_type,
-                       inc,
-                       nswrgu,
-                       iwarnu,
-                       imligu,
-                       epsrgu,
-                       climgu,
-                       coefaq,
-                       coefbv,
-                       c_mass_var,
-                       c_grad_mvar);
+    cs_gradient_tensor_synced_input(var_name,
+                                    gradient_type,
+                                    halo_type,
+                                    inc,
+                                    nswrgu,
+                                    iwarnu,
+                                    imligu,
+                                    epsrgu,
+                                    climgu,
+                                    (const cs_real_6_t *)coefaq,
+                                    (const cs_real_66_t *)coefbv,
+                                    (const cs_real_6_t *)c_mass_var,
+                                    c_grad_mvar);
 
     /* Mass flow through interior faces */
 
@@ -1647,7 +1649,6 @@ cs_ext_force_flux(const cs_mesh_t          *m,
     = (const cs_lnum_2_t *restrict)m->i_face_cells;
   const cs_lnum_t *restrict b_face_cells
     = (const cs_lnum_t *restrict)m->b_face_cells;
-  const cs_real_t *restrict weight = fvq->weight;
   const cs_real_t *restrict i_dist = fvq->i_dist;
   const cs_real_t *restrict b_dist = fvq->b_dist;
   const cs_real_t *restrict i_f_face_surf = fvq->i_f_face_surf;
@@ -1674,8 +1675,8 @@ cs_ext_force_flux(const cs_mesh_t          *m,
   if (f_i_poro_duq_0 != NULL) {
     is_p = 1;
     i_poro_duq_0 = f_i_poro_duq_0->val;
-    i_poro_duq_1 = cs_field_by_name_try("i_poro_duq_1")->val;
-    b_poro_duq = cs_field_by_name_try("b_poro_duq")->val;
+    i_poro_duq_1 = cs_field_by_name("i_poro_duq_1")->val;
+    b_poro_duq = cs_field_by_name("b_poro_duq")->val;
   } else {
     i_poro_duq_0 = &_f_ext;
     i_poro_duq_1 = &_f_ext;
