@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -62,10 +62,10 @@ BEGIN_C_DECLS
 
 void CS_PROCF (viscfa, VISCFA)
 (
- const cs_int_t  *const   visc_mean_type,
- cs_real_t                c_visc[],
- cs_real_t                i_visc[],
- cs_real_t                b_visc[]
+ const int   *const visc_mean_type,
+ cs_real_t          c_visc[],
+ cs_real_t          i_visc[],
+ cs_real_t          b_visc[]
 );
 
 /*----------------------------------------------------------------------------
@@ -74,10 +74,10 @@ void CS_PROCF (viscfa, VISCFA)
 
 void CS_PROCF (vistnv, VISTNV)
 (
- const cs_int_t  *const   visc_mean_type,
- cs_real_6_t              c_visc[],
- cs_real_33_t             i_visc[],
- cs_real_t                b_visc[]
+ const int     *const visc_mean_type,
+ cs_real_6_t          c_visc[],
+ cs_real_33_t         i_visc[],
+ cs_real_t            b_visc[]
 );
 
 /*----------------------------------------------------------------------------
@@ -86,12 +86,12 @@ void CS_PROCF (vistnv, VISTNV)
 
 void CS_PROCF (vitens, VITENS)
 (
- cs_real_6_t              c_visc[],
- const cs_int_t  *const   iwarnp,
- cs_real_2_t              weighf[],
- cs_real_t                weighb[],
- cs_real_t                i_visc[],
- cs_real_t                b_visc[]
+ cs_real_6_t         c_visc[],
+ const int    *const iwarnp,
+ cs_real_2_t         weighf[],
+ cs_real_t           weighb[],
+ cs_real_t           i_visc[],
+ cs_real_t           b_visc[]
 );
 
 /*============================================================================
@@ -122,7 +122,7 @@ void CS_PROCF (vitens, VITENS)
 
 void
 cs_face_viscosity(const cs_mesh_t               *m,
-                  cs_mesh_quantities_t          *fvq,
+                  const cs_mesh_quantities_t    *fvq,
                   const int                      visc_mean_type,
                   cs_real_t            *restrict c_visc,
                   cs_real_t            *restrict i_visc,
@@ -147,12 +147,12 @@ cs_face_viscosity(const cs_mesh_t               *m,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_face_anisotropic_viscosity_vector(const cs_mesh_t               *m,
-                                     cs_mesh_quantities_t          *fvq,
-                                     const int                      visc_mean_type,
-                                     cs_real_6_t          *restrict c_visc,
-                                     cs_real_33_t         *restrict i_visc,
-                                     cs_real_t            *restrict b_visc);
+cs_face_anisotropic_viscosity_vector(const cs_mesh_t             *m,
+                                     const cs_mesh_quantities_t  *fvq,
+                                     const int                    visc_mean_type,
+                                     cs_real_6_t        *restrict c_visc,
+                                     cs_real_33_t       *restrict i_visc,
+                                     cs_real_t          *restrict b_visc);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -181,7 +181,7 @@ cs_face_anisotropic_viscosity_vector(const cs_mesh_t               *m,
 
 void
 cs_face_anisotropic_viscosity_scalar(const cs_mesh_t               *m,
-                                     cs_mesh_quantities_t          *fvq,
+                                     const cs_mesh_quantities_t    *fvq,
                                      cs_real_6_t          *restrict c_visc,
                                      const int                      iwarnp,
                                      cs_real_2_t          *restrict weighf,
