@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -96,53 +96,6 @@ static cs_real_t  *_drdxyz = NULL, *_wdrdxy = NULL;
  * Private function definitions
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Update drdxyz and wdrdxy for periodic ghost cells.
- *
- * Called by PERMAS.
- *
- * parameters:
- *   h_cell_id --> cell id in halo
- *   cell_id   --> cell id
- *   rom       --> density array
- *   call_id   --> first or second call
- *   drdxyz    <-> Gradient on components of Rij (Reynolds stress tensor)
- *   wdrdxy    <-> associated working array.
- *----------------------------------------------------------------------------*/
-
-static void
-_update_drdxyz(cs_lnum_t         h_cell_id,
-               cs_lnum_t         cell_id,
-               const cs_real_t   rom[],
-               int               call_id,
-               cs_real_t         drdxyz[],
-               cs_real_t         wdrdxy[])
-{
-  cs_lnum_t  i, j, id;
-
-  if (call_id == 1) { /* First call */
-
-    for (i = 0; i < 2*3; i++) {
-      for (j = 0; j < 3; j++) {
-        id = j + 3*i + 3*6*h_cell_id;
-        wdrdxy[id] = drdxyz[id];
-        drdxyz[id] *= rom[cell_id];
-      }
-    }
-
-  }
-  else if (call_id == 2) { /* Second call */
-
-    for (i = 0; i < 2*3; i++) {
-      for (j = 0; j < 3; j++) {
-        id = j + 3*i + 3*6*h_cell_id;
-        drdxyz[id] = wdrdxy[id];
-      }
-    }
-
-  } /* End if second call */
-}
-
 /*----------------------------------------------------------------------------
  * Save ghost cell values of an initial Rij component gradient.
  *
@@ -222,91 +175,6 @@ _save_halo_perio_rij(int           comp_id,
  * Public function definitions for Fortran API
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Periodicity management for INIMAS
- *
- * If INIMAS is called by NAVSTO :
- *    We assume that gradient on ghost cells given by a rotation is known
- *    and is equal to the velocity one for the previous time step.
- * If INIMAS is called by DIVRIJ
- *    We assume that (more justifiable than in previous case) gradient on
- *    ghost cells given by rotation is equal to Rij gradient for the previous
- *    time step.
- *
- * Fortran Interface:
- *
- * subroutine permas
- * *****************
- *
- * integer          iappel      :  -> : indicateur d'appel dans inimas
- *                                          = 1 si appel au debut
- *                                          = 2 si appel a la fin
- * double precision rom(ncelet) :  -> : masse volumique aux cellules
- *
- * Size of DRDXYZ and WDRDXY = n_ghost_cells*6*3
- *----------------------------------------------------------------------------*/
-
-void
-CS_PROCF (permas, PERMAS)(const cs_int_t  *iappel,
-                          cs_real_t        rom[])
-{
-  int  rank_id, t_id;
-  cs_lnum_t  i, cell_id, shift;
-  cs_lnum_t  start_std, end_std, start_ext, end_ext;
-
-  cs_mesh_t  *mesh = cs_glob_mesh;
-  cs_halo_t  *halo = mesh->halo;
-  cs_halo_type_t  halo_type = mesh->halo_type;
-  const fvm_periodicity_t  *periodicity = mesh->periodicity;
-
-  if (halo_type == CS_HALO_N_TYPES)
-    return;
-
-  if (*iappel == 1)
-    cs_halo_sync_var(mesh->halo, mesh->halo_type, rom);
-
-  for (t_id = 0; t_id < mesh->n_transforms; t_id++) {
-
-    if (   fvm_periodicity_get_type(periodicity, t_id)
-        >= FVM_PERIODICITY_ROTATION) {
-
-      shift = 4 * halo->n_c_domains * t_id;
-
-      for (rank_id = 0; rank_id < halo->n_c_domains; rank_id++) {
-
-        start_std = halo->perio_lst[shift + 4*rank_id];
-        end_std = start_std + halo->perio_lst[shift + 4*rank_id + 1];
-
-        for (i = start_std; i < end_std; i++) {
-
-          cell_id = mesh->n_cells + i;
-
-          _update_drdxyz(i, cell_id, rom, *iappel, _drdxyz, _wdrdxy);
-
-        } /* End of loop on halo elements */
-
-        if (halo_type == CS_HALO_EXTENDED) {
-
-          start_ext = halo->perio_lst[shift + 4*rank_id + 2];
-          end_ext = start_ext + halo->perio_lst[shift + 4*rank_id + 3];
-
-          for (i = start_ext; i < end_ext; i++) {
-
-            cell_id = mesh->n_cells + i;
-
-            _update_drdxyz(i, cell_id, rom, *iappel, _drdxyz, _wdrdxy);
-
-          } /* End of loop on halo elements */
-
-        } /* End if extended halo */
-
-      } /* End of loop on ranks */
-
-    } /* End of test on rotation */
-
-  } /* End of loop on transformation */
-}
-
 /*----------------------------------------------------------------------------
  * Preparation rotation periodicity for Reynolds stresses.
  *
@@ -322,11 +190,10 @@ CS_PROCF (permas, PERMAS)(const cs_int_t  *iappel,
 
 void CS_PROCF (perinr, PERINR)
 (
- const cs_int_t   *const imrgra,  /* <-- gradient computation mode            */
- const cs_int_t   *const iwarnp,  /* <-- verbosity level                      */
- const cs_real_t  *const epsrgp,  /* <-- precision for iterative gradient
+ const int        *const imrgra,  /* <-- gradient computation mode            */
+ const int        *const iwarnp,  /* <-- verbosity level                      */
+ const cs_real_t  *const epsrgp   /* <-- precision for iterative gradient
                                          calculation                          */
- const cs_real_t  *const extrap   /* <-- extrapolate gradient at boundary     */
 )
 {
   int i;
@@ -335,7 +202,7 @@ void CS_PROCF (perinr, PERINR)
   cs_field_t  *f = NULL;
 
   cs_halo_type_t halo_type = CS_HALO_STANDARD;
-  cs_gradient_type_t gradient_type = CS_GRADIENT_ITER;
+  cs_gradient_type_t gradient_type = CS_GRADIENT_GREEN_ITER;
 
   const cs_mesh_t  *mesh = cs_glob_mesh;
 
@@ -353,8 +220,10 @@ void CS_PROCF (perinr, PERINR)
 
     f = cs_field_by_name_try(r_name[i]);
 
-    if (f == NULL)
+    if (f == NULL) {
+      BFT_FREE(grad);
       return;
+    }
 
     /* We do not reconstruct and do not limit as we do not know the gradient of
        neighbors (we are trying to compute it here). Starting from the second
@@ -381,7 +250,6 @@ void CS_PROCF (perinr, PERINR)
                        *iwarnp,
                        -1,              /* clip_mode */
                        *epsrgp,
-                       *extrap,
                        1.5,             /* clip_coeff (ignored here) */
                        NULL,            /* f_ext */
                        f->bc_coeffs->a,
@@ -676,7 +544,7 @@ cs_gradient_perio_init_rij_tensor(int           *tr_dim,
  * We assume that is correct to treat periodicities implicitly for the other
  * variables when reconstructing gradients.
  *
- * \param[in]       f_id     field index
+ * \param[in]       f_id     field id
  * \param[in, out]  grad     gradient of field
  *
  * size of _drdxyz and _wdrdxy = n_ghost_cells*6*3
@@ -684,13 +552,13 @@ cs_gradient_perio_init_rij_tensor(int           *tr_dim,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_gradient_perio_process_rij(const cs_int_t    *f_id,
-                              cs_real_3_t        grad[])
+cs_gradient_perio_process_rij(int          f_id,
+                              cs_real_3_t  grad[])
 {
   cs_lnum_t  d_var = -1;
   cs_mesh_t  *mesh = cs_glob_mesh;
 
-  cs_field_t  *f = cs_field_by_id(*f_id);
+  cs_field_t  *f = cs_field_by_id(f_id);
 
   if (f->name[0] == 'r') {
     if (strlen(f->name) == 3) {
