@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,34 +54,6 @@ BEGIN_C_DECLS
  * Public function prototypes for Fortran API
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Periodicity management for INIMAS
- *
- * If INIMAS is called by NAVSTO :
- *    We assume that gradient on ghost cells given by a rotation is known
- *    and is equal to the velocity one for the previous time step.
- * If INIMAS is called by DIVRIJ
- *    We assume that (more justifiable than in previous case) gradient on
- *    ghost cells given by rotation is equal to Rij gradient for the previous
- *    time step.
- *
- * Fortran Interface:
- *
- * subroutine permas
- * *****************
- *
- * integer          iappel      :  -> : indicateur d'appel dans inimas
- *                                          = 1 si appel au debut
- *                                          = 2 si appel a la fin
- * double precision rom(ncelet) :  -> : masse volumique aux cellules
- *
- * Size of DRDXYZ and WDRDXY = n_ghost_cells*6*3
- *----------------------------------------------------------------------------*/
-
-void
-CS_PROCF (permas, PERMAS)(const cs_int_t  *iappel,
-                          cs_real_t        rom[]);
-
 /*----------------------------------------------------------------------------
  * Preparation rotation periodicity for Reynolds stresses.
  *
@@ -97,11 +69,10 @@ CS_PROCF (permas, PERMAS)(const cs_int_t  *iappel,
 
 void CS_PROCF (perinr, PERINR)
 (
- const cs_int_t   *const imrgra,  /* <-- gradient computation mode            */
- const cs_int_t   *const iwarnp,  /* <-- verbosity level                      */
- const cs_real_t  *const epsrgp,  /* <-- precision for iterative gradient
+ const int        *const imrgra,  /* <-- gradient computation mode            */
+ const int        *const iwarnp,  /* <-- verbosity level                      */
+ const cs_real_t  *const epsrgp   /* <-- precision for iterative gradient
                                          calculation                          */
- const cs_real_t  *const extrap   /* <-- extrapolate gradient at boundary     */
 );
 
 /*=============================================================================
@@ -196,7 +167,7 @@ cs_gradient_perio_init_rij_tensor(int           *tr_dim,
  * variables when reconstructing gradients.
  *
  * parameters:
- *   f_id      <--   field index
+ *   f_id      <--   field id
  *   grad      -->   gradient of field
  *
  * size of _drdxyz and _wdrdxy = n_ghost_cells*6*3
@@ -204,8 +175,8 @@ cs_gradient_perio_init_rij_tensor(int           *tr_dim,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_gradient_perio_process_rij(const cs_int_t    *f_id,
-                              cs_real_3_t        grad[]);
+cs_gradient_perio_process_rij(int          f_id,
+                              cs_real_3_t  grad[]);
 
 /*----------------------------------------------------------------------------*/
 
