@@ -9,7 +9,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -107,8 +107,8 @@ extern const char *cs_grid_coarsening_type_name[];
 
 cs_grid_t *
 cs_grid_create_from_shared(cs_lnum_t              n_faces,
-                           const int             *diag_block_size,
-                           const int             *extra_diag_block_size,
+                           const cs_lnum_t       *diag_block_size,
+                           const cs_lnum_t       *extra_diag_block_size,
                            const cs_lnum_2_t     *face_cell,
                            const cs_real_t       *cell_cen,
                            const cs_real_t       *cell_vol,
@@ -117,6 +117,25 @@ cs_grid_create_from_shared(cs_lnum_t              n_faces,
                            const cs_matrix_t     *a_conv,
                            const cs_matrix_t     *a_diff);
 
+/*----------------------------------------------------------------------------
+ * Create base grid by mapping from parent (possibly shared) matrix.
+ *
+ * Note that as arrays given as arguments are shared by the created grid
+ * (which can only access them, not modify them), the grid should be
+ * destroyed before those arrays.
+ *
+ * parameters:
+ *   a       <-- associated matrix
+ *   n_ranks <-- number of active ranks (<= 1 to restrict to local values)
+ *
+ * returns:
+ *   base grid structure
+ *----------------------------------------------------------------------------*/
+
+cs_grid_t *
+cs_grid_create_from_parent(const cs_matrix_t  *a,
+                           int                 n_ranks);
+
 /*----------------------------------------------------------------------------
  * Destroy a grid structure.
  *
@@ -160,8 +179,8 @@ void
 cs_grid_get_info(const cs_grid_t  *g,
                  int              *level,
                  bool             *symmetric,
-                 int              *db_size,
-                 int              *eb_size,
+                 cs_lnum_t        *db_size,
+                 cs_lnum_t        *eb_size,
                  int              *n_ranks,
                  cs_lnum_t        *n_rows,
                  cs_lnum_t        *n_cols_ext,
@@ -250,17 +269,37 @@ cs_grid_get_matrix(const cs_grid_t  *g);
 MPI_Comm
 cs_grid_get_comm(const cs_grid_t  *g);
 
+/*----------------------------------------------------------------------------
+ * Get the MPI subcommunicator for a given merge stride.
+ *
+ * parameters:
+ *   parent       <-- parent MPI communicator
+ *   merge_stride <-- associated merge stride
+ *
+ * returns:
+ *   MPI communicator
+ *----------------------------------------------------------------------------*/
+
+MPI_Comm
+cs_grid_get_comm_merge(MPI_Comm  parent,
+                       int       merge_stride);
+
 #endif
 
 /*----------------------------------------------------------------------------
  * Create coarse grid from fine grid.
  *
  * parameters:
- *   f                    <-- Fine grid structure
- *   verbosity            <-- Verbosity level
- *   coarsening_type      <-- Coarsening criteria type
- *   aggregation_limit    <-- Maximum allowed fine rows per coarse row
- *   relaxation_parameter <-- P0/P1 relaxation factor
+ *   f                          <-- Fine grid structure
+ *   coarsening_type            <-- Coarsening criteria type
+ *   aggregation_limit          <-- Maximum allowed fine rows per coarse rows
+ *   verbosity                  <-- Verbosity level
+ *   merge_stride               <-- Associated merge stride
+ *   merge_rows_mean_threshold  <-- mean number of rows under which
+ *                                  merging should be applied
+ *   merge_rows_glob_threshold  <-- global number of rows under which
+ *                                  merging should be applied
+ *   relaxation_parameter       <-- P0/P1 relaxation factor
  *
  * returns:
  *   coarse grid structure
@@ -268,11 +307,31 @@ cs_grid_get_comm(const cs_grid_t  *g);
 
 cs_grid_t *
 cs_grid_coarsen(const cs_grid_t  *f,
-                int               verbosity,
                 int               coarsening_type,
                 int               aggregation_limit,
+                int               verbosity,
+                int               merge_stride,
+                int               merge_rows_mean_threshold,
+                cs_gnum_t         merge_rows_glob_threshold,
                 double            relaxation_parameter);
 
+/*----------------------------------------------------------------------------
+ * Create coarse grid with only one row per rank from fine grid.
+ *
+ * parameters:
+ *   f            <-- Fine grid structure
+ *   merge_stride <-- Associated merge stride
+ *   verbosity    <-- Verbosity level
+ *
+ * returns:
+ *   coarse grid structure
+ *----------------------------------------------------------------------------*/
+
+cs_grid_t *
+cs_grid_coarsen_to_single(const cs_grid_t  *f,
+                          int               merge_stride,
+                          int               verbosity);
+
 /*----------------------------------------------------------------------------
  * Compute coarse row variable values from fine row values
  *
@@ -292,22 +351,6 @@ cs_grid_restrict_row_var(const cs_grid_t  *f,
                          const cs_real_t  *f_var,
                          cs_real_t        *c_var);
 
-/*----------------------------------------------------------------------------
- * Compute fine row integer values from coarse row values
- *
- * parameters:
- *   c       <-- Fine grid structure
- *   f       <-- Fine grid structure
- *   c_num   --> Variable defined on coarse grid rows
- *   f_num   <-- Variable defined on fine grid rows
- *----------------------------------------------------------------------------*/
-
-void
-cs_grid_prolong_row_num(const cs_grid_t  *c,
-                        const cs_grid_t  *f,
-                        int              *c_num,
-                        int              *f_num);
-
 /*----------------------------------------------------------------------------
  * Compute fine row variable values from coarse row values
  *
@@ -387,16 +430,6 @@ cs_grid_project_diag_dom(const cs_grid_t  *g,
                          cs_lnum_t         n_base_rows,
                          cs_real_t         diag_dom[]);
 
-/*----------------------------------------------------------------------------
- * Return the merge_stride if merging is active.
- *
- * returns:
- *   grid merge stride if merging is active, 1 otherwise
- *----------------------------------------------------------------------------*/
-
-int
-cs_grid_get_merge_stride(void);
-
 /*----------------------------------------------------------------------------
  * Finalize global info related to multigrid solvers
  *----------------------------------------------------------------------------*/
@@ -418,46 +451,6 @@ cs_grid_dump(const cs_grid_t  *g);
  * Public function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Query the global multigrid parameters for parallel grid merging.
- *
- * parameters:
- *   rank_stride         --> number of ranks over which merging
- *                           takes place, or NULL
- *   rows_mean_threshold --> mean number of rows under which merging
- *                           should be applied, or NULL
- *   rows_glob_threshold --> global number of rows under which merging
- *                           should be applied, or NULL
- *   min_ranks           --> number of active ranks under which
- *                           no merging takes place, or NULL
- *----------------------------------------------------------------------------*/
-
-void
-cs_grid_get_merge_options(int         *rank_stride,
-                          int         *rows_mean_threshold,
-                          cs_gnum_t   *rows_glob_threshold,
-                          int         *min_ranks);
-
-/*----------------------------------------------------------------------------
- * Set global multigrid parameters for parallel grid merging.
- *
- * parameters:
- *   rank_stride         <-- number of ranks over which merging
- *                           takes place
- *   rows_mean_threshold <-- mean number of rows under which merging
- *                           should be applied
- *   rows_glob_threshold <-- global number of rows under which merging
- *                           should be applied
- *   min_ranks           <-- number of active ranks under which
- *                           no merging takes place
- *----------------------------------------------------------------------------*/
-
-void
-cs_grid_set_merge_options(int         rank_stride,
-                          int         rows_mean_threshold,
-                          cs_gnum_t   rows_glob_threshold,
-                          int         min_ranks);
-
 /*----------------------------------------------------------------------------
  * Set matrix tuning behavior for multigrid coarse meshes.
  *
@@ -473,30 +466,6 @@ void
 cs_grid_set_matrix_tuning(cs_matrix_fill_type_t  fill_type,
                           int                    max_level);
 
-/*----------------------------------------------------------------------------
- * Force matrix variant selection for multigrid coarse meshes.
- *
- * The finest mesh (level 0) is handled by the default options,
- * so only coarser meshes are considered here.
- *
- * parameters:
- *   fill_type <-- associated matrix fill type
- *   level     <-- level for which variant is assiged
- *   mv        <-- matrix variant to assign (NULL to unassign)
- *----------------------------------------------------------------------------*/
-
-void
-cs_grid_set_matrix_variant(cs_matrix_fill_type_t       fill_type,
-                           int                         level,
-                           const cs_matrix_variant_t  *mv);
-
-/*----------------------------------------------------------------------------
- * Log the current settings for multigrid parallel merging.
- *----------------------------------------------------------------------------*/
-
-void
-cs_grid_log_merge_options(void);
-
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
