@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,11 +54,15 @@ BEGIN_C_DECLS
 
 typedef enum {
 
-  CS_MATRIX_NATIVE,     /* Native matrix format */
-  CS_MATRIX_CSR,        /* Compressed Sparse Row storage format */
-  CS_MATRIX_CSR_SYM,    /* Compressed Symmetric Sparse Row storage format */
-  CS_MATRIX_MSR,        /* Modified Compressed Sparse Row storage format */
-  CS_MATRIX_N_TYPES     /* Number of known matrix types */
+  CS_MATRIX_NATIVE,           /*!< Native (edge-based) matrix storage */
+  CS_MATRIX_CSR,              /*!< Compressed Sparse Row storage */
+  CS_MATRIX_CSR_SYM,          /*!< Compressed Symmetric Sparse Row storage */
+  CS_MATRIX_MSR,              /*!< Modified Compressed Sparse Row storage
+                                (separate diagonal) */
+
+  CS_MATRIX_N_BUILTIN_TYPES,  /*!< Number of known and built-in matrix types */
+
+  CS_MATRIX_N_TYPES           /*!< Number of known matrix types */
 
 } cs_matrix_type_t;
 
@@ -205,8 +209,7 @@ cs_matrix_structure_create_msr(cs_matrix_type_t        type,
                                const cs_numbering_t   *numbering);
 
 /*----------------------------------------------------------------------------
- * Create an MSR matrix structure sharing an existing connectivity definition
- * as well as an optional edge-based definition.
+ * Create an MSR matrix structure sharing an existing connectivity definition.
  *
  * Note that as the structure created maps to the given existing
  * cell global number, face -> cell connectivity arrays, and cell halo
@@ -226,7 +229,7 @@ cs_matrix_structure_create_msr(cs_matrix_type_t        type,
  *                        info, or NULL
  *
  * returns:
- *   a pointer to a created CDO matrix structure
+ *   a pointer to a created matrix structure
  *----------------------------------------------------------------------------*/
 
 cs_matrix_structure_t *
@@ -282,24 +285,6 @@ cs_matrix_structure_destroy(cs_matrix_structure_t  **ms);
 cs_matrix_t *
 cs_matrix_create(const cs_matrix_structure_t  *ms);
 
-/*----------------------------------------------------------------------------
- * Create a matrix container using a given variant.
- *
- * If the matrix variant is incompatible with the structure, it is ignored,
- * and defaults for that structure are used instead.
- *
- * parameters:
- *   ms <-- associated matrix structure
- *   mv <-- associated matrix variant
- *
- * returns:
- *   pointer to created matrix structure;
- *----------------------------------------------------------------------------*/
-
-cs_matrix_t *
-cs_matrix_create_by_variant(const cs_matrix_structure_t  *ms,
-                            const cs_matrix_variant_t    *mv);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Create a matrix directly from assembler.
@@ -335,6 +320,23 @@ cs_matrix_create_from_assembler(cs_matrix_type_t        type,
 cs_matrix_t *
 cs_matrix_create_by_copy(cs_matrix_t   *src);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create a matrix based on the local restriction of a base matrix.
+ *
+ * Coefficients are copied. Some coefficients may be shared with the
+ * parent matrix, so the base matrix must not be destroyed before the
+ * restriction matrix.
+ *
+ * \param[in]  src  reference matrix structure
+ *
+ * \return  pointer to created matrix structure;
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_matrix_t *
+cs_matrix_create_by_local_restrict(const cs_matrix_t  *src);
+
 /*----------------------------------------------------------------------------
  * Destroy a matrix structure.
  *
@@ -802,6 +804,24 @@ cs_matrix_get_msr_arrays(const cs_matrix_t   *matrix,
                          const cs_real_t    **d_val,
                          const cs_real_t    **x_val);
 
+/*----------------------------------------------------------------------------
+ * Assign functions based on a variant to a given matrix.
+ *
+ * If the matrix variant is incompatible with the structure, it is ignored,
+ * and defaults for that structure are used instead.
+ *
+ * parameters:
+ *   m <-> associated matrix structure
+ *   mv <-- associated matrix variant
+ *
+ * returns:
+ *   pointer to created matrix structure;
+ *----------------------------------------------------------------------------*/
+
+void
+cs_matrix_apply_variant(cs_matrix_t                *m,
+                        const cs_matrix_variant_t  *mv);
+
 /*----------------------------------------------------------------------------
  * Matrix.vector product y = A.x
  *
@@ -871,42 +891,149 @@ cs_matrix_pre_vector_multiply_sync(cs_halo_rotation_t   rotation_mode,
                                    const cs_matrix_t   *matrix,
                                    cs_real_t           *x);
 
-/*----------------------------------------------------------------------------
- * Build list of variants for tuning or testing.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Function for initialization of CSR matrix coefficients using
+ *        local row ids and column indexes.
  *
- * parameters:
- *   n_fill_types <-- number of fill types tuned for
- *   fill_types   <-- array of fill types tuned for
- *   type_filter  <-- true for matrix types tuned for, false for others
- *   numbering    <-- vectorization or thread-related numbering info,
- *                    or NULL
- *   n_variants   --> number of variants
- *   m_variant    --> array of matrix variants
- *----------------------------------------------------------------------------*/
+ * \warning  The matrix pointer must point to valid data when the selection
+ *           function is called, so the life cycle of the data pointed to
+ *           should be at least as long as that of the assembler values
+ *           structure.
+ *
+ * \param[in, out]  matrix_p  untyped pointer to matrix description structure
+ * \param[in]       db_size   optional diagonal block sizes
+ * \param[in]       eb_size   optional extra-diagonal block sizes
+ */
+/*----------------------------------------------------------------------------*/
 
 void
-cs_matrix_variant_build_list(int                      n_fill_types,
-                             cs_matrix_fill_type_t    fill_types[],
-                             bool                     type_filter[],
-                             const cs_numbering_t    *numbering,
+cs_matrix_csr_assembler_values_init(void             *matrix_p,
+                                    const cs_lnum_t   db_size[4],
+                                    const cs_lnum_t   eb_size[4]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Function pointer for addition to CSR matrix coefficients using
+ *        local row ids and column indexes.
+ *
+ * Values whose associated row index is negative should be ignored;
+ * Values whose column index is -1 are assumed to be assigned to a
+ * separately stored diagonal. Other indexes shoudl be valid.
+ *
+ * \warning  The matrix pointer must point to valid data when the selection
+ *           function is called, so the life cycle of the data pointed to
+ *           should be at least as long as that of the assembler values
+ *           structure.
+ *
+ * \remark  Note that we pass column indexes (not ids) here; as the
+ *          caller is already assumed to have identified the index
+ *          matching a given column id.
+ *
+ * \param[in, out]  matrix_p  untyped pointer to matrix description structure
+ * \param[in]       n         number of values to add
+ * \param[in]       stride    associated data block size
+ * \param[in]       row_id    associated local row ids
+ * \param[in]       col_idx   associated local column indexes
+ * \param[in]       vals      pointer to values (size: n*stride)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_matrix_csr_assembler_values_add(void             *matrix_p,
+                                   cs_lnum_t         n,
+                                   cs_lnum_t         stride,
+                                   const cs_lnum_t   row_id[],
+                                   const cs_lnum_t   col_idx[],
+                                   const cs_real_t   vals[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Function for initialization of MSR matrix coefficients using
+ *        local row ids and column indexes.
+ *
+ * \warning  The matrix pointer must point to valid data when the selection
+ *           function is called, so the life cycle of the data pointed to
+ *           should be at least as long as that of the assembler values
+ *           structure.
+ *
+ * \param[in, out]  matrix_p  untyped pointer to matrix description structure
+ * \param[in]       db_size   optional diagonal block sizes
+ * \param[in]       eb_size   optional extra-diagonal block sizes
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_matrix_msr_assembler_values_init(void             *matrix_p,
+                                    const cs_lnum_t   db_size[4],
+                                    const cs_lnum_t   eb_size[4]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Function pointer for addition to MSR matrix coefficients using
+ *        local row ids and column indexes.
+ *
+ * Values whose associated row index is negative should be ignored;
+ * Values whose column index is -1 are assumed to be assigned to a
+ * separately stored diagonal. Other indexes shoudl be valid.
+ *
+ * \warning  The matrix pointer must point to valid data when the selection
+ *           function is called, so the life cycle of the data pointed to
+ *           should be at least as long as that of the assembler values
+ *           structure.
+ *
+ * \remark  Note that we pass column indexes (not ids) here; as the
+ *          caller is already assumed to have identified the index
+ *          matching a given column id.
+ *
+ * \param[in, out]  matrix_p  untyped pointer to matrix description structure
+ * \param[in]       n         number of values to add
+ * \param[in]       stride    associated data block size
+ * \param[in]       row_id    associated local row ids
+ * \param[in]       col_idx   associated local column indexes
+ * \param[in]       vals      pointer to values (size: n*stride)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_matrix_msr_assembler_values_add(void             *matrix_p,
+                                   cs_lnum_t         n,
+                                   cs_lnum_t         stride,
+                                   const cs_lnum_t   row_id[],
+                                   const cs_lnum_t   col_idx[],
+                                   const cs_real_t   vals[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Build list of variants for tuning or testing.
+ *
+ * The matrix coefficients should be assigned, so the fill type can
+ * be determined.
+ *
+ * \param[in]   m             associated matrix
+ * \param[out]  n_variants    number of variants
+ * \param[out]  m_variant     array of matrix variants
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_matrix_variant_build_list(const cs_matrix_t       *m,
                              int                     *n_variants,
                              cs_matrix_variant_t    **m_variant);
 
-/*----------------------------------------------------------------------------
- * Build matrix variant
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Build matrix variant
  *
  * The variant will initially use default matrix-vector functions,
  * which can be later modified using cs_matrix_variant_set_func().
  *
- * parameters:
- *   type         <-- type of matrix considered
- *   numbering    <-- vectorization or thread-related numbering info,
- *                    or NULL
- *----------------------------------------------------------------------------*/
+ * \param[in]  m   pointer to matrix
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_matrix_variant_t *
-cs_matrix_variant_create(cs_matrix_type_t         type,
-                         const cs_numbering_t    *numbering);
+cs_matrix_variant_create(cs_matrix_t  *m);
 
 /*----------------------------------------------------------------------------
  * Destroy a matrix variant structure.
@@ -918,6 +1045,19 @@ cs_matrix_variant_create(cs_matrix_type_t         type,
 void
 cs_matrix_variant_destroy(cs_matrix_variant_t  **mv);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Apply a variant to a given matrix
+ *
+ * \param[in, out]  m   pointer to matrix
+ * \param[in]       mv  pointer to matrix variant pointer
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_matrix_variant_apply(cs_matrix_t          *m,
+                        cs_matrix_variant_t  *mv);
+
 /*----------------------------------------------------------------------------
  * Select the sparse matrix-vector product function to be used by a
  * matrix variant for a given fill type.
@@ -925,23 +1065,26 @@ cs_matrix_variant_destroy(cs_matrix_variant_t  **mv);
  * Currently, possible variant functions are:
  *
  *   CS_MATRIX_NATIVE  (all fill types)
+ *     default
  *     standard
- *     fixed           (for CS_MATRIX_??_BLOCK_D or CS_MATRIX_??_BLOCK_D_SYM)
  *     omp             (for OpenMP with compatible numbering)
  *     vector          (For vector machine with compatible numbering)
  *
  *   CS_MATRIX_CSR     (for CS_MATRIX_SCALAR or CS_MATRIX_SCALAR_SYM)
+ *     default
  *     standard
  *     mkl             (with MKL)
  *
  *   CS_MATRIX_CSR_SYM (for CS_MATRIX_SCALAR_SYM)
+ *     default
  *     standard
  *     mkl             (with MKL)
  *
  *   CS_MATRIX_MSR     (all fill types except CS_MATRIX_33_BLOCK)
+ *     default
  *     standard
- *     generic         (for CS_MATRIX_??_BLOCK_D or CS_MATRIX_??_BLOCK_D_SYM)
  *     mkl             (with MKL, for CS_MATRIX_SCALAR or CS_MATRIX_SCALAR_SYM)
+ *     omp_sched       (For OpenMP with scheduling)
  *
  * parameters:
  *   mv        <-> pointer to matrix variant
@@ -958,27 +1101,6 @@ cs_matrix_variant_set_func(cs_matrix_variant_t     *mv,
                            int                      ed_flag,
                            const char              *func_name);
 
-/*----------------------------------------------------------------------------
- * Merge a functions to a matrix variant from another variant sharing
- * the same structure.
- *
- * Functions from the structure to merge for the selected fill type are
- * assigned to the main variant.
- *
- * This can be useful when tuning has been done separately for different fill
- * types, and the resulting selected structure is identical.
- *
- * parameters:
- *   mv        <-> pointer to matrix variant
- *   mv_merge  <-- pointer to matrix variant to merge
- *   fill_type <-- matrix fill type to merge from
- *----------------------------------------------------------------------------*/
-
-void
-cs_matrix_variant_merge(cs_matrix_variant_t        *mv,
-                        const cs_matrix_variant_t  *mv_merge,
-                        cs_matrix_fill_type_t       fill_type);
-
 /*----------------------------------------------------------------------------
  * Get the type associated with a matrix variant.
  *
@@ -989,26 +1111,6 @@ cs_matrix_variant_merge(cs_matrix_variant_t        *mv,
 cs_matrix_type_t
 cs_matrix_variant_type(const cs_matrix_variant_t  *mv);
 
-/*----------------------------------------------------------------------------
- * Test local matrix.vector product operations.
- *
- * parameters:
- *   n_rows         <-- number of local rows
- *   n_cols_ext     <-- number of columns + ghosts
- *   n_edges        <-- local number of (undirected) graph edges
- *   edges          <-- edges (symmetric row <-> column) connectivity
- *   halo           <-- cell halo structure
- *   numbering      <-- vectorization or thread-related numbering info, or NULL
- *----------------------------------------------------------------------------*/
-
-void
-cs_matrix_variant_test(cs_lnum_t              n_rows,
-                       cs_lnum_t              n_cols_ext,
-                       cs_lnum_t              n_edges,
-                       const cs_lnum_2_t     *edges,
-                       const cs_halo_t       *halo,
-                       const cs_numbering_t  *numbering);
-
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
