@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -50,7 +50,6 @@ BEGIN_C_DECLS
 
 #define CS_MATRIX_DISTANT_ROW_USE_COL_IDX     (1 << 0)
 #define CS_MATRIX_DISTANT_ROW_USE_COL_G_ID    (1 << 1)
-
 #define CS_MATRIX_EXTERNAL_HALO               (1 << 2)
 
 /*============================================================================
@@ -121,6 +120,46 @@ typedef void
                                     const cs_lnum_t   col_idx[],
                                     const cs_real_t   vals[]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Function pointer for addition to matrix coefficients using
+ *        local row ids and column indexes in case of CDO schemes.
+ *        Contrary to other add_values() function. Here values are structured
+ *        in a CSR-like way. By this way one avoids to allocate specific
+ *        arrays for this step.
+ *
+ * Values whose associated row index is negative should be ignored;
+ * Values whose column index is -1 are assumed to be assigned to a
+ * separately stored diagonal. Other indexes should be valid.
+ *
+ * \warning  The matrix pointer must point to valid data when the selection
+ *           function is called, so the life cycle of the data pointed to
+ *           should be at least as long as that of the assembler values
+ *           structure.
+ *
+ * \remark  Note that we pass column indexes (not ids) here; as the
+ *          caller is already assumed to have identified the index
+ *          matching a given column id.
+ *
+ * \param[in, out] matrix_p   untyped pointer to matrix description structure
+ * \param[in]      row_id     local row id to deal with
+ * \param[in]      stride     associated data block size
+ * \param[in]      n_cols     number of columns for the row to deal with
+ * \param[in]      diag_val   pointer to the diagonal value(s)
+ * \param[in]      col_idx    associated local column indexes
+ * \param[in]      vals       pointer to values (size: n*stride*row_size)
+ */
+/*----------------------------------------------------------------------------*/
+
+typedef void
+(cs_matrix_assembler_values_cdo_add_t) (void               *matrix,
+                                        cs_lnum_t           row_id,
+                                        cs_lnum_t           stride,
+                                        cs_lnum_t           n_cols,
+                                        const cs_real_t     diag_val[],
+                                        const cs_lnum_t     col_idx[],
+                                        const cs_real_t     vals[]);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Function pointer for addition to matrix coefficients using
