@@ -4,7 +4,7 @@
 
 /* This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -59,6 +59,7 @@
 #include "cs_ext_neighborhood.h"
 #include "cs_mesh_quantities.h"
 #include "cs_parameters.h"
+#include "cs_porous_model.h"
 #include "cs_prototypes.h"
 #include "cs_timer.h"
 
@@ -151,9 +152,9 @@ void CS_PROCF (matrix, MATRIX)
 
 void CS_PROCF (matrdt, MATRDT)
 (
- const cs_int_t  *const   iconvp,
- const cs_int_t  *const   idiffp,
- const cs_int_t  *const   isym,
+ const int       *const   iconvp,
+ const int       *const   idiffp,
+ const int       *const   isym,
  const cs_real_t          coefbp[],
  const cs_real_t          cofbfp[],
  const cs_real_t          i_massflux[],
@@ -262,9 +263,11 @@ cs_matrix_wrapper_scalar(int               iconvp,
   }
 
   /* If a whole line of the matrix is 0, the diagonal is set to 1 */
+  if (mq->has_disable_flag == 1) {
 # pragma omp parallel for
-  for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-    da[cell_id] += mq->c_solid_flag[CS_MIN(cs_glob_porous_model, 1)*cell_id];
+    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
+      da[cell_id] += mq->c_disable_flag[cell_id];
+    }
   }
 
 }
@@ -365,9 +368,11 @@ cs_matrix_wrapper_scalar_conv_diff(int               iconvp,
   }
 
   /* If a whole line of the matrix is 0, the diagonal is set to 1 */
+  if (mq->has_disable_flag == 1) {
 # pragma omp parallel for
-  for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-    da[cell_id] += mq->c_solid_flag[CS_MIN(cs_glob_porous_model, 1)*cell_id];
+    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
+      da[cell_id] += mq->c_disable_flag[cell_id];
+    }
   }
 
 }
@@ -383,7 +388,7 @@ cs_matrix_wrapper_vector(int                  iconvp,
                          int                  tensorial_diffusion,
                          int                  ndircp,
                          int                  isym,
-                         int                  eb_size[4],
+                         cs_lnum_t            eb_size[4],
                          double               thetap,
                          const cs_real_33_t   coefbp[],
                          const cs_real_33_t   cofbfp[],
@@ -408,6 +413,7 @@ cs_matrix_wrapper_vector(int                  iconvp,
   if (tensorial_diffusion == 1) {
     /* Symmetric matrix */
     if (isym == 1) {
+      assert(eb_size[0] == 1);
       cs_sym_matrix_vector(m,
                            idiffp,
                            thetap,
@@ -419,7 +425,8 @@ cs_matrix_wrapper_vector(int                  iconvp,
                            xa);
 
     /* Non-symmetric matrix */
-    } else {
+    }
+    else {
       cs_matrix_vector(m,
                        mq,
                        iconvp,
@@ -487,11 +494,13 @@ cs_matrix_wrapper_vector(int                  iconvp,
   }
 
   /* If a whole line of the matrix is 0, the diagonal is set to 1 */
+  if (mq->has_disable_flag == 1) {
 # pragma omp parallel for
-  for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-    for (int isou = 0; isou < 3; isou++)
-      da[cell_id][isou][isou]
-        += mq->c_solid_flag[CS_MIN(cs_glob_porous_model, 1)*cell_id];
+    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
+      for (int isou = 0; isou < 3; isou++)
+        da[cell_id][isou][isou]
+          += (cs_real_t)(mq->c_disable_flag[cell_id]);
+    }
   }
 
 }
@@ -567,13 +576,14 @@ cs_matrix_wrapper_tensor(int                  iconvp,
                                                  thetap,
                                                  cofbfts,
                                                  fimp,
-                                                 (cs_real_66_t *)i_visc,
+                                                 (const cs_real_66_t *)i_visc,
                                                  b_visc,
                                                  da,
                                                  (cs_real_66_t *)xa);
 
     /* Non-symmetric matrix */
-    } else {
+    }
+    else {
       cs_matrix_anisotropic_diffusion_tensor(m,
                                              iconvp,
                                              idiffp,
@@ -607,10 +617,11 @@ cs_matrix_wrapper_tensor(int                  iconvp,
   }
 
   /* If a whole line of the matrix is 0, the diagonal is set to 1 */
-  for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-    cs_lnum_t corr_cell_id = CS_MIN(cs_glob_porous_model, 1)*cell_id;
-    for (int isou = 0; isou < 6; isou++) {
-      da[cell_id][isou][isou] += mq->c_solid_flag[corr_cell_id];
+  if (mq->has_disable_flag == 1) {
+    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
+      for (int isou = 0; isou < 6; isou++) {
+        da[cell_id][isou][isou] += mq->c_disable_flag[cell_id];
+      }
     }
   }
 
@@ -691,53 +702,61 @@ cs_sym_matrix_scalar(const cs_mesh_t          *m,
     }
   }
 
-# pragma omp parallel for
-  for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
-    xa[face_id] = 0.;
+  /* 2. Computation of extradiagonal terms and contribution to the diagonal */
+
+  if (idiffp) {
+
+    for (int g_id = 0; g_id < n_i_groups; g_id++) {
+#     pragma omp parallel for firstprivate(thetap)
+      for (int t_id = 0; t_id < n_i_threads; t_id++) {
+        for (cs_lnum_t face_id = i_group_index[(t_id*n_i_groups + g_id)*2];
+             face_id < i_group_index[(t_id*n_i_groups + g_id)*2 + 1];
+             face_id++) {
+
+          cs_lnum_t ii = i_face_cells[face_id][0];
+          cs_lnum_t jj = i_face_cells[face_id][1];
+
+          cs_real_t aij = -thetap*i_visc[face_id];
+
+          xa[face_id] = aij;
+          da[ii] -= aij;
+          da[jj] -= aij;
+
+        }
+      }
+    }
+
   }
 
-  /* 2. Computation of extradiagonal terms */
+  else {
 
-# pragma omp parallel for firstprivate(thetap, idiffp)
-  for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
-    xa[face_id] = -thetap*idiffp*i_visc[face_id];
-  }
-
-  /* 3. Contribution of the extra-diagonal terms to the diagonal */
-
-  for (int g_id = 0; g_id < n_i_groups; g_id++) {
 #   pragma omp parallel for
-    for (int t_id = 0; t_id < n_i_threads; t_id++) {
-      for (cs_lnum_t face_id = i_group_index[(t_id*n_i_groups + g_id)*2];
-          face_id < i_group_index[(t_id*n_i_groups + g_id)*2 + 1];
-          face_id++) {
-
-        cs_lnum_t ii = i_face_cells[face_id][0];
-        cs_lnum_t jj = i_face_cells[face_id][1];
-
-        da[ii] -= xa[face_id];
-        da[jj] -= xa[face_id];
-
-      }
+    for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
+      xa[face_id] = 0.;
     }
+
   }
 
-  /* 4. Contribution of border faces to the diagonal */
+  /* 3. Contribution of border faces to the diagonal */
 
-  for (int g_id = 0; g_id < n_b_groups; g_id++) {
-#   pragma omp parallel for firstprivate(thetap, idiffp) \
-                        if(m->n_b_faces > CS_THR_MIN)
-    for (int t_id = 0; t_id < n_b_threads; t_id++) {
-      for (cs_lnum_t face_id = b_group_index[(t_id*n_b_groups + g_id)*2];
-        face_id < b_group_index[(t_id*n_b_groups + g_id)*2 + 1];
-        face_id++) {
+  if (idiffp) {
 
-        cs_lnum_t ii = b_face_cells[face_id];
+    for (int g_id = 0; g_id < n_b_groups; g_id++) {
+#     pragma omp parallel for firstprivate(thetap, idiffp)        \
+                          if(m->n_b_faces > CS_THR_MIN)
+      for (int t_id = 0; t_id < n_b_threads; t_id++) {
+        for (cs_lnum_t face_id = b_group_index[(t_id*n_b_groups + g_id)*2];
+             face_id < b_group_index[(t_id*n_b_groups + g_id)*2 + 1];
+             face_id++) {
 
-        da[ii] += thetap*idiffp*b_visc[face_id]*cofbfp[face_id];
+          cs_lnum_t ii = b_face_cells[face_id];
 
+          da[ii] += thetap*b_visc[face_id]*cofbfp[face_id];
+
+        }
       }
     }
+
   }
 
 }
@@ -817,11 +836,11 @@ cs_matrix_scalar(const cs_mesh_t          *m,
   const cs_lnum_t *restrict b_face_cells
     = (const cs_lnum_t *restrict)m->b_face_cells;
 
-  /*===============================================================================*/
+  /*========================================================================== */
 
-  /*===============================================================================
+  /*==========================================================================
     1. Initialization
-    ===============================================================================*/
+    ========================================================================== */
 
 # pragma omp parallel for
   for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
@@ -834,7 +853,6 @@ cs_matrix_scalar(const cs_mesh_t          *m,
     }
   }
 
-
 # pragma omp parallel for
   for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
     xa[face_id][0] = 0.;
@@ -944,8 +962,10 @@ cs_matrix_scalar(const cs_mesh_t          *m,
            * D_jj = -theta (m_ij)^- + m_ij
            *      = -X_ji + (1-theta)*m_ij
            */
-          da[ii] -= xa[face_id][0] + iconvp*(1. - thetap)*xcpp[ii]*i_massflux[face_id];
-          da[jj] -= xa[face_id][1] - iconvp*(1. - thetap)*xcpp[jj]*i_massflux[face_id];
+          da[ii] -= xa[face_id][0]
+            + iconvp*(1. - thetap)*xcpp[ii]*i_massflux[face_id];
+          da[jj] -= xa[face_id][1]
+            - iconvp*(1. - thetap)*xcpp[jj]*i_massflux[face_id];
 
         }
       }
@@ -1267,7 +1287,7 @@ cs_matrix_vector(const cs_mesh_t            *m,
                  const cs_mesh_quantities_t *mq,
                  int                         iconvp,
                  int                         idiffp,
-                 int                         eb_size[4],
+                 cs_lnum_t                   eb_size[4],
                  double                      thetap,
                  const cs_real_33_t          coefbp[],
                  const cs_real_33_t          cofbfp[],
@@ -1298,7 +1318,7 @@ cs_matrix_vector(const cs_mesh_t            *m,
   int is_p = 0;
 
   if (cs_glob_porous_model == 3) {
-    i_f_face_factor = mq->i_f_face_factor;
+    i_f_face_factor = (const cs_real_2_t *)(mq->i_f_face_factor);
     b_f_face_factor = mq->b_f_face_factor;
     is_p = 1;
   }
@@ -1661,7 +1681,6 @@ cs_matrix_tensor(const cs_mesh_t          *m,
 
 }
 
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Build the diagonal of the advection/diffusion matrix
@@ -1893,7 +1912,7 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t            *m,
 
   /* Is it porous? */
   if (cs_glob_porous_model == 3) {
-    i_f_face_factor = mq->i_f_face_factor;
+    i_f_face_factor = (const cs_real_2_t *)(mq->i_f_face_factor);
     b_f_face_factor = mq->b_f_face_factor;
     is_p = 1;
   }
@@ -2224,7 +2243,7 @@ cs_matrix_anisotropic_diffusion_tensor(const cs_mesh_t          *m,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_sym_matrix_anisotropic_diffusion(const cs_mesh_t           *m,
+cs_sym_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
                                     int                       idiffp,
                                     double                    thetap,
                                     const cs_real_33_t        cofbfp[],
