@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -76,8 +76,8 @@ BEGIN_C_DECLS
 
 void
 cs_matrix_vector_native_multiply(bool                symmetric,
-                                 const int           db_size[4],
-                                 const int           eb_size[4],
+                                 const cs_lnum_t     db_size[4],
+                                 const cs_lnum_t     eb_size[4],
                                  cs_halo_rotation_t  rotation_mode,
                                  int                 f_id,
                                  const cs_real_t    *dam,
@@ -119,9 +119,9 @@ cs_matrix_update_mesh(void);
  *----------------------------------------------------------------------------*/
 
 cs_matrix_t  *
-cs_matrix_default(bool        symmetric,
-                  const int  *diag_block_size,
-                  const int  *extra_diag_block_size);
+cs_matrix_default(bool             symmetric,
+                  const cs_lnum_t  *diag_block_size,
+                  const cs_lnum_t  *extra_diag_block_size);
 
 /*----------------------------------------------------------------------------
  * Return MSR matrix for a given fill type
@@ -136,9 +136,9 @@ cs_matrix_default(bool        symmetric,
  *----------------------------------------------------------------------------*/
 
 cs_matrix_t  *
-cs_matrix_msr(bool        symmetric,
-              const int  *diag_block_size,
-              const int  *extra_diag_block_size);
+cs_matrix_msr(bool             symmetric,
+              const cs_lnum_t  *diag_block_size,
+              const cs_lnum_t  *extra_diag_block_size);
 
 /*----------------------------------------------------------------------------
  * Return native matrix for a given fill type
@@ -153,12 +153,12 @@ cs_matrix_msr(bool        symmetric,
  *----------------------------------------------------------------------------*/
 
 cs_matrix_t  *
-cs_matrix_native(bool        symmetric,
-                 const int  *diag_block_size,
-                 const int  *extra_diag_block_size);
+cs_matrix_native(bool              symmetric,
+                 const cs_lnum_t  *diag_block_size,
+                 const cs_lnum_t  *extra_diag_block_size);
 
 /*----------------------------------------------------------------------------
- * Force matrix variant for a given fill type
+ * Determine or apply default tuning for a given matrix type
  *
  * Information from the variant used fo this definition is copied,
  * so it may be freed after calling this function.
@@ -169,33 +169,7 @@ cs_matrix_native(bool        symmetric,
  *----------------------------------------------------------------------------*/
 
 void
-cs_matrix_set_variant(cs_matrix_fill_type_t       fill_type,
-                      const cs_matrix_variant_t  *mv);
-
-/*----------------------------------------------------------------------------
- * Set matrix tuning behavior for a given fill type
- *
- * parameters:
- *   fill type  <-- Fill type for which tuning behavior is set
- *   tune       <-- 1 to activate tuning, 0 to deactivate
- *----------------------------------------------------------------------------*/
-
-void
-cs_matrix_set_tuning(cs_matrix_fill_type_t   fill_type,
-                     int                     tune);
-
-/*----------------------------------------------------------------------------
- * Return matrix tuning behavior for a given fill type.
- *
- * parameters:
- *   fill type  <-- Fill type for which tuning behavior is set
- *
- * returns:
- *   1 if tuning is active, 0 otherwise
- *----------------------------------------------------------------------------*/
-
-int
-cs_matrix_get_tuning(cs_matrix_fill_type_t   fill_type);
+cs_matrix_default_set_tuned(cs_matrix_t  *m);
 
 /*----------------------------------------------------------------------------
  * Set number of matrix computation runs for tuning.
@@ -227,6 +201,19 @@ void
 cs_matrix_get_tuning_runs(int     *n_min_products,
                           double  *t_measure);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set default matrix type for a given fill type.
+ *
+ * \param[in] fill type  Fill type for which tuning behavior is set
+ * \param[in] type       Matrix type to use
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_matrix_default_set_type(cs_matrix_fill_type_t  fill_type,
+                           cs_matrix_type_t       type);
+
 /*----------------------------------------------------------------------------
  * Return a (0-based) global block row numbering.
  *
@@ -267,13 +254,13 @@ cs_matrix_get_block_row_g_id(cs_lnum_t         n_rows,
 /*----------------------------------------------------------------------------*/
 
 cs_matrix_t *
-cs_matrix_set_coefficients_coupled(const cs_field_t  *f,
-                                   cs_matrix_type_t   type,
-                                   bool               symmetric,
-                                   const int         *diag_block_size,
-                                   const int         *extra_diag_block_size,
-                                   const cs_real_t   *da,
-                                   const cs_real_t   *xa);
+cs_matrix_set_coefficients_by_assembler(const cs_field_t  *f,
+                                        cs_matrix_type_t   type,
+                                        bool               symmetric,
+                                        const cs_lnum_t   *diag_block_size,
+                                        const cs_lnum_t   *extra_diag_block_size,
+                                        const cs_real_t   *da,
+                                        const cs_real_t   *xa);
 
 /*----------------------------------------------------------------------------*/
 
