@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -330,28 +330,14 @@ struct _cs_matrix_t {
 
 struct _cs_matrix_variant_t {
 
-  char                   name[32];     /* Variant name */
+  char                   name[2][32]; /* Variant names */
 
-  cs_matrix_type_t       type;         /* Matrix storage and definition type */
+  cs_matrix_type_t       type;        /* Matrix storage and definition type */
+  cs_matrix_fill_type_t  fill_type;   /* Matrix storage and definition type */
 
-  /* Function pointer arrays, with variants:
-     fill_type + exclude_diagonal_flag */
-
-  cs_matrix_vector_product_t   *vector_multiply[CS_MATRIX_N_FILL_TYPES][2];
-
-  /* Measured structure creation cost, or -1 otherwise */
-
-  double  matrix_create_cost;
-
-  /* Measured assignment costs for each available fill type, or -1 otherwise */
-
-  double  matrix_assign_cost[CS_MATRIX_N_FILL_TYPES];
-
-  /* Measured operation costs for each available operation, or -1 otherwise
-     fill_type*2 + exclude_diagonal_flag */
-
-  double  matrix_vector_cost[CS_MATRIX_N_FILL_TYPES][2][2];
+  /* Function pointer arrays, with and without exclude_diagonal_flag */
 
+  cs_matrix_vector_product_t   *vector_multiply[2];
 };
 
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
