@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
+#include "cs_grid.h"
 #include "cs_sles.h"
 #include "cs_sles_it.h"
 #include "cs_sles_pc.h"
@@ -49,6 +50,19 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Multigrid types
+ *----------------------------------------------------------------------------*/
+
+typedef enum {
+
+  CS_MULTIGRID_V_CYCLE,          /*!< Use a V-cycle */
+  CS_MULTIGRID_K_CYCLE,          /*!< Use a K-cycle */
+  CS_MULTIGRID_K_CYCLE_HPC,      /*!< Use a K-cycle tuned for HPC systems */
+  CS_MULTIGRID_N_TYPES           /*!< Number of multigrid types */
+
+} cs_multigrid_type_t;
+
 /* Multigrid linear solver context (opaque) */
 
 typedef struct _cs_multigrid_t  cs_multigrid_t;
@@ -57,6 +71,10 @@ typedef struct _cs_multigrid_t  cs_multigrid_t;
  *  Global variables
  *============================================================================*/
 
+/* Names for multigrid types */
+
+extern const char *cs_multigrid_type_name[];
+
 /*=============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -75,16 +93,6 @@ cs_multigrid_initialize(void);
 void
 cs_multigrid_finalize(void);
 
-/*----------------------------------------------------------------------------
- * Indicate if multigrid solver API is used for at least one system.
- *
- * returns:
- *   true if at least one system uses a multigrid solver, false otherwise
- *----------------------------------------------------------------------------*/
-
-bool
-cs_multigrid_needed(void);
-
 /*----------------------------------------------------------------------------
  * Define and associate a multigrid sparse linear system solver
  * for a given field or equation name.
@@ -103,29 +111,33 @@ cs_multigrid_needed(void);
  * If needed, cs_sles_find() may be used to obtain a pointer to the
  * matching cs_sles_t container.
  *
- * parameters:
- *   f_id <-- associated field id, or < 0
- *   name <-- associated name if f_id < 0, or NULL
+ * \param[in]  f_id     associated field id, or < 0
+ * \param[in]  name     associated name if f_id < 0, or NULL
+ * \param[in]  mg_type  type of multigrid algorithm to use
  *
  * \return  pointer to new multigrid info and context
  */
 /*----------------------------------------------------------------------------*/
 
 cs_multigrid_t *
-cs_multigrid_define(int          f_id,
-                    const char  *name);
+cs_multigrid_define(int                   f_id,
+                    const char           *name,
+                    cs_multigrid_type_t   mg_type);
 
-/*----------------------------------------------------------------------------
- * Create multigrid linear system solver info and context.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create multigrid linear system solver info and context.
  *
  * The multigrid variant is an ACM (Additive Corrective Multigrid) method.
  *
- * returns:
- *   pointer to new multigrid info and context
- *----------------------------------------------------------------------------*/
+ * \param[in]  mg_type  type of multigrid algorithm to use
+ *
+ * \return  pointer to new multigrid info and context
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_multigrid_t *
-cs_multigrid_create(void);
+cs_multigrid_create(cs_multigrid_type_t  mg_type);
 
 /*----------------------------------------------------------------------------
  * Destroy multigrid linear system solver info and context.
@@ -159,13 +171,10 @@ cs_multigrid_copy(const void  *context);
  *
  * parameters:
  *   mg                     <-> pointer to multigrid info and context
- *   aggregation_limit      <-- maximum allowed fine cells per coarse cell
- *   coarsening_type        <-- coarsening type:
- *                              0: algebraic, natural face traversal;
- *                              1: algebraic, face traveral by criteria;
- *                              2: algebraic, Hilbert face traversal;
+ *   aggregation_limit      <-- maximum allowed fine rows per coarse cell
+ *   coarsening_type        <-- coarsening type; see cs_grid_coarsening_t
  *   n_max_levels           <-- maximum number of grid levels
- *   min_g_cells            <-- global number of cells on coarse grids
+ *   min_g_rows             <-- global number of rows on coarse grids
  *                              under which no coarsening occurs
  *   p0p1_relax             <-- p0/p1 relaxation_parameter
  *   postprocess_block_size <-- if > 0, postprocess coarsening
@@ -177,7 +186,7 @@ cs_multigrid_set_coarsening_options(cs_multigrid_t  *mg,
                                     int              aggregation_limit,
                                     int              coarsening_type,
                                     int              n_max_levels,
-                                    cs_gnum_t        min_g_cells,
+                                    cs_gnum_t        min_g_rows,
                                     double           p0p1_relax,
                                     int              postprocess_block_size);
 
@@ -252,7 +261,7 @@ cs_multigrid_setup(void               *context,
                    int                 verbosity);
 
 /*----------------------------------------------------------------------------
- * Setup multigrid sparse linear equation solver with seperate
+ * Setup multigrid sparse linear equation solver with separate
  * convection-diffusion matrixes
  *
  * parameters:
@@ -315,7 +324,7 @@ cs_multigrid_solve(void                *context,
  * Free iterative sparse linear equation solver setup context.
  *
  * Note that this function should free resolution-related data, such as
- * buffers and preconditioning but doesd not free the whole context,
+ * buffers and preconditioning but does not free the whole context,
  * as info used for logging (especially performance data) is maintained.
 
  * parameters:
@@ -339,15 +348,18 @@ void
 cs_multigrid_log(const void  *context,
                  cs_log_t     log_type);
 
-/*----------------------------------------------------------------------------
- * Create a multigrid preconditioner.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create a multigrid preconditioner.
  *
- * returns:
- *   pointer to newly created preconditioner object.
- *----------------------------------------------------------------------------*/
+ * \param[in]  mg_type  type of multigrid algorithm to use
+ *
+ * \return  pointer to newly created preconditioner object.
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_sles_pc_t *
-cs_multigrid_pc_create(void);
+cs_multigrid_pc_create(cs_multigrid_type_t  mg_type);
 
 /*----------------------------------------------------------------------------
  * Error handler for multigrid sparse linear equation solver.
@@ -391,6 +403,46 @@ cs_multigrid_set_plot_options(cs_multigrid_t  *mg,
                               const char      *base_name,
                               bool             use_iteration);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Query the global multigrid parameters for parallel grid merging.
+ *
+ * \param[in]   mg                   pointer to multigrid info and context
+ * \param[out]  rank_stride          number of ranks over which merging
+ *                                   takes place, or NULL
+ * \param[out]  rows_mean_threshold  mean number of rows under which merging
+ *                                   should be applied, or NULL
+ * \param[out]  rows_glob_threshold  global number of rows under which
+ *                                   merging should be applied, or NULL
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_multigrid_get_merge_options(const cs_multigrid_t  *mg,
+                               int                   *rank_stride,
+                               int                   *rows_mean_threshold,
+                               cs_gnum_t             *rows_glob_threshold);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set global multigrid parameters for parallel grid merging behavior.
+ *
+ * \param[in, out]  mg                   pointer to multigrid info and context
+ * \param[in]       rank_stride          number of ranks over which merging
+ *                                       takes place
+ * \param[in]       rows_mean_threshold  mean number of rows under which
+ *                                       merging should be applied
+ * \param[in]       rows_glob_threshold  global number of rows under which
+ *                                       merging should be applied
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_multigrid_set_merge_options(cs_multigrid_t  *mg,
+                               int              rank_stride,
+                               int              rows_mean_threshold,
+                               cs_gnum_t        rows_glob_threshold);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
