@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -365,6 +365,7 @@ static cs_sles_t **_cs_sles_systems[3] = {NULL, NULL, NULL};
 
 /* Timer statistics */
 
+static cs_timer_counter_t   _sles_t_tot;     /* Total time in linear solvers */
 static int _sles_stat_id = -1;
 
 /*============================================================================
@@ -657,7 +658,7 @@ _needs_solving(const  char        *name,
 
   /* Initialize residue, check for immediate return */
 
-  const int *diag_block_size = cs_matrix_get_diag_block_size(a);
+  const cs_lnum_t *diag_block_size = cs_matrix_get_diag_block_size(a);
   const cs_lnum_t _diag_block_size = diag_block_size[1];
   assert(diag_block_size[0] == diag_block_size[1]);
 
@@ -789,7 +790,7 @@ _ensure_alloc_post(cs_sles_t          *sles,
 {
   if (sles->post_info != NULL) {
 
-    const int *diag_block_size = cs_matrix_get_diag_block_size(a);
+    const cs_lnum_t *diag_block_size = cs_matrix_get_diag_block_size(a);
     const cs_lnum_t n_vals = cs_matrix_get_n_columns(a) * diag_block_size[1];
 
     sles->post_info->n_rows = cs_matrix_get_n_rows(a);
@@ -889,6 +890,8 @@ _post_function(void                  *sles_p,
 void
 cs_sles_initialize(void)
 {
+  CS_TIMER_COUNTER_INIT(_sles_t_tot);
+
   int stats_root = cs_timer_stats_id_by_name("operations");
 
   if (stats_root > -1) {
@@ -947,9 +950,23 @@ cs_sles_finalize(void)
 void
 cs_sles_log(cs_log_t  log_type)
 {
+  int n_tot_systems = 0;
+  for (int i = 0; i < 3; i++)
+    n_tot_systems += _cs_sles_n_systems[i];
+
+  if (n_tot_systems < 1)
+    return;
+
   int log_order[] = {2, 0, 1}; /* log previous setups first, then fields,
                                   then others */
 
+  if (log_type == CS_LOG_PERFORMANCE)
+    cs_log_printf
+      (log_type,
+       _("\n"
+         "Total elapsed time for linear equation system solvers:  %.3f s\n"),
+       _sles_t_tot.wall_nsec*1e-9);
+
   const char *option_category[]
     = {N_("Linear solver options modified during run (previous values)"),
        N_("Linear solver options for fields"),
@@ -1178,7 +1195,7 @@ cs_sles_find_or_add(int          f_id,
  * operations involving access through a field id.
  *
  * \deprecated This function is provided to allow some peculiar
- * calling sequences, in which \ref cs_c_bindings::codits (codits) is
+ * calling sequences, in which \ref cs_equation_iterative_solve_scalar is
  * called with a nonzero \c ivar value, but specific solver options
  * must still be set.
  * In the future, a more consistent mechanism (using a zero \c ivar
@@ -1205,7 +1222,7 @@ cs_sles_push(int          f_id,
   if (retval->name != NULL)
     bft_error
       (__FILE__, __LINE__, 0,
-       _("cs_slesh_push() only allows a stack of depth 1:\n"
+       _("cs_sles_push() only allows a stack of depth 1:\n"
          "  it  may not be called multiple times for a given field (id %d)\n"
          "  without calling cs_sles_pop between those calls."), f_id);
   else {
@@ -1510,6 +1527,8 @@ void
 cs_sles_setup(cs_sles_t          *sles,
               const cs_matrix_t  *a)
 {
+  cs_timer_t t0 = cs_timer_time();
+
   if (sles->context == NULL)
     _cs_sles_define_default(sles->f_id, sles->name, a);
 
@@ -1535,6 +1554,9 @@ cs_sles_setup(cs_sles_t          *sles,
   }
 
   cs_timer_stats_switch(t_top_id);
+
+  cs_timer_t t1 = cs_timer_time();
+  cs_timer_counter_add_diff(&_sles_t_tot, &t0, &t1);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1583,6 +1605,8 @@ cs_sles_solve(cs_sles_t           *sles,
               size_t               aux_size,
               void                *aux_vectors)
 {
+  cs_timer_t t0 = cs_timer_time();
+
   if (sles->context == NULL)
     _cs_sles_define_default(sles->f_id, sles->name, a);
 
@@ -1660,6 +1684,9 @@ cs_sles_solve(cs_sles_t           *sles,
 
   cs_timer_stats_switch(t_top_id);
 
+  cs_timer_t t1 = cs_timer_time();
+  cs_timer_counter_add_diff(&_sles_t_tot, &t0, &t1);
+
   return state;
 }
 
@@ -1854,7 +1881,7 @@ cs_sles_post_error_output_def(const char          *name,
 
     const cs_lnum_t n_cols = cs_matrix_get_n_columns(a);
     const cs_lnum_t n_rows = cs_matrix_get_n_rows(a);
-    const int *diag_block_size = cs_matrix_get_diag_block_size(a);
+    const cs_lnum_t *diag_block_size = cs_matrix_get_diag_block_size(a);
 
     /* Check for mesh location */
 
@@ -2018,17 +2045,10 @@ cs_sles_post_output_var(const char      *name,
     if (flag > 0) {
 
       char type_name[32];
-      size_t l = strlen(name);
-
-      l += strlen("_fp_type");
-
-      if (l > 31)
-        l = 31;
-
-      l -= strlen("_fp_type");
+      size_t l = 31 - strlen("_fp_type");
 
       strncpy(type_name, name, l);
-      type_name[l] = '\0';
+      type_name[31] = '\0';
 
       strcat(type_name, "_fp_type");
 
