@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -51,6 +51,7 @@
 
 #include "cs_base.h"
 #include "cs_field.h"
+#include "cs_field_default.h"
 #include "cs_grid.h"
 #include "cs_halo.h"
 #include "cs_internal_coupling.h"
@@ -127,15 +128,7 @@ _sles_default_native(int                f_id,
 
   if (name != NULL) {
 
-    if (!strcmp(name, "wall_distance")) { /* distpr.f90 */
-      sles_it_type = CS_SLES_PCG;
-      multigrid = 1;
-    }
-    if (!strcmp(name, "yplus_wall")) { /* distyp.f90 */
-      sles_it_type = CS_SLES_P_SYM_GAUSS_SEIDEL;
-    }
-    else if (   !strcmp(name, "hydrostatic_p")  /* calhyd.f90 */
-             || !strcmp(name, "potential")) {   /* predfl.f90 */
+    if (!strcmp(name, "potential")) {   /* predfl.f90 */
       /* Copy from pressure if possible */
       cs_field_t *cvar_p = (cs_field_by_name_try("pressure"));
       cs_sles_t *src = NULL;
@@ -149,38 +142,35 @@ _sles_default_native(int                f_id,
           return;
       }
       /* If copying from pressure failed, default to multigrid */
-      sles_it_type = CS_SLES_PCG;
+      sles_it_type = CS_SLES_FCG;
       multigrid = 1;
     }
-    else if (!strcmp(name, "Prhydro")) { /* prehyd.f90 */
-      sles_it_type = CS_SLES_PCG;
-    }
     else if (!strcmp(name, "Pr compress")) { /* resopv.f90 */
       sles_it_type = CS_SLES_P_SYM_GAUSS_SEIDEL;
     }
-    else if (!strcmp(name, "PoissonL")) { /* lageqp.f90 */
-      sles_it_type = CS_SLES_PCG;
+    else if (!strcmp(name, "PoissonL")) { /* _lageqp */
+      sles_it_type = CS_SLES_FCG;
       n_max_iter = 1000;
     }
-    else if (!strcmp(name, "radiation_p1")) { /* raypun.f90 */
-      sles_it_type = CS_SLES_PCG;
+    else if (!strcmp(name, "radiation_p1")) { /* cs_rad_transfer_pun */
+      sles_it_type = CS_SLES_FCG;
       multigrid = 1;
     }
     /* cs_bad_cells_regularisation.c */
     else if (!strcmp(name, "potential_regularisation_scalar")) {
-      sles_it_type = CS_SLES_PCG;
+      sles_it_type = CS_SLES_FCG;
     }
     else if (!strcmp(name, "potential_regularisation_vector")) {
-      sles_it_type = CS_SLES_PCG;
+      sles_it_type = CS_SLES_FCG;
     }
     else if (!strcmp(name, "potential_regularisation_sym_tensor")) {
-      sles_it_type = CS_SLES_PCG;
+      sles_it_type = CS_SLES_FCG;
     }
   }
   else if (f_id > -1) {
     const cs_field_t *f = cs_field_by_id(f_id);
     if (!strcmp(f->name, "hydraulic_head")) {
-      sles_it_type = CS_SLES_PCG;
+      sles_it_type = CS_SLES_FCG;
       multigrid = 2;
     }
   }
@@ -198,7 +188,7 @@ _sles_default_native(int                f_id,
     }
 
     if (symmetric) {
-      sles_it_type = CS_SLES_PCG;
+      sles_it_type = CS_SLES_FCG;
       if (f_id > -1 && coupling_id < 0)
         multigrid = 1;
     }
@@ -214,7 +204,8 @@ _sles_default_native(int                f_id,
 
     /* Multigrid used as preconditioner if possible, as solver otherwise */
 
-    if ((matrix_type == CS_MATRIX_MSR) || (matrix_type == CS_MATRIX_N_TYPES)) {
+    if (   (matrix_type == CS_MATRIX_MSR)
+        || (matrix_type >= CS_MATRIX_N_TYPES)) {
       if (sles_it_type == CS_SLES_PCG && cs_glob_n_threads > 1)
         sles_it_type = CS_SLES_FCG;
       cs_sles_it_t *c = cs_sles_it_define(f_id,
@@ -243,6 +234,191 @@ _sles_default_native(int                f_id,
 
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Setup sparse linear equation solver using native matrix arrays.
+ *
+ * \param[in]       f_id                   associated field id, or < 0
+ * \param[in]       name                   associated name if f_id < 0, or NULL
+ * \param[in]       setup_id               associated setup id
+ * \param[in]       sc                     associated solver context
+ * \param[in]       symmetric              indicates if matrix coefficients
+ *                                         are symmetric
+ * \param[in]       diag_block_size        block sizes for diagonal, or NULL
+ * \param[in]       extra_diag_block_size  block sizes for extra diagonal,
+ *                                         or NULL
+ * \param[in]       da                     diagonal values (NULL if zero)
+ * \param[in]       xa                     extradiagonal values (NULL if zero)
+ */
+/*----------------------------------------------------------------------------*/
+
+static void
+_sles_setup_matrix_native(int                  f_id,
+                          const char          *name,
+                          int                  setup_id,
+                          cs_sles_t           *sc,
+                          bool                 symmetric,
+                          const cs_lnum_t     *diag_block_size,
+                          const cs_lnum_t     *extra_diag_block_size,
+                          const cs_real_t     *da,
+                          const cs_real_t     *xa)
+{
+  cs_matrix_t *a = NULL;
+
+  const cs_mesh_t *m = cs_glob_mesh;
+
+  bool need_msr = false;
+
+  /* If context has not been defined yet, temporarily set
+     matrix coefficients (using native matrix, which has lowest
+     overhead as coefficients are provided in that form)
+     to define the required context.
+
+     The matrix type might be modified later based on solver
+     constraints. */
+
+  if (cs_sles_get_context(sc) == NULL) {
+    int eb_size = 1;
+    if (extra_diag_block_size != NULL)
+      eb_size = extra_diag_block_size[1];
+    if (eb_size > 1)
+      a = cs_matrix_native(symmetric,
+                           diag_block_size,
+                           extra_diag_block_size);
+    else
+      a = cs_matrix_msr(symmetric,
+                        diag_block_size,
+                        extra_diag_block_size);
+
+    cs_matrix_set_coefficients(a,
+                               symmetric,
+                               diag_block_size,
+                               extra_diag_block_size,
+                               m->n_i_faces,
+                               (const cs_lnum_2_t *)(m->i_face_cells),
+                               da,
+                               xa);
+
+    cs_sles_define_t  *sles_default_func = cs_sles_get_default_define();
+    sles_default_func(f_id, name, a);
+    cs_matrix_release_coefficients(a);
+  }
+
+  assert(cs_sles_get_context(sc) != NULL);
+
+  cs_sles_pc_t  *pc = NULL;
+  cs_multigrid_t *mg = NULL;
+
+  if (strcmp(cs_sles_get_type(sc), "cs_sles_it_t") == 0) {
+    cs_sles_it_t *c = cs_sles_get_context(sc);
+    cs_sles_it_type_t s_type = cs_sles_it_get_type(c);
+    if (   s_type >= CS_SLES_P_GAUSS_SEIDEL
+        && s_type <= CS_SLES_TS_B_GAUSS_SEIDEL)
+      need_msr = true;
+    else {
+      pc = cs_sles_it_get_pc(c);
+      if (pc != NULL) {
+        if (strcmp(cs_sles_pc_get_type(pc), "multigrid") == 0)
+          mg = cs_sles_pc_get_context(pc);
+      }
+    }
+  }
+  else if (strcmp(cs_sles_get_type(sc), "cs_multigrid_t") == 0)
+    mg = cs_sles_get_context(sc);
+
+  if (mg != NULL) {
+    cs_sles_it_type_t fs_type = cs_multigrid_get_fine_solver_type(mg);
+    if (   fs_type >= CS_SLES_P_GAUSS_SEIDEL
+        && fs_type <= CS_SLES_TS_B_GAUSS_SEIDEL)
+      need_msr = true;
+  }
+
+  /* MSR not supported yet for full blocks */
+  if (extra_diag_block_size != NULL) {
+    if (extra_diag_block_size[0] > 1)
+      need_msr = false;
+  }
+
+  if (need_msr)
+    a = cs_matrix_msr(symmetric,
+                      diag_block_size,
+                      extra_diag_block_size);
+  else
+    a = cs_matrix_default(symmetric,
+                          diag_block_size,
+                          extra_diag_block_size);
+
+  cs_matrix_set_coefficients(a,
+                             symmetric,
+                             diag_block_size,
+                             extra_diag_block_size,
+                             m->n_i_faces,
+                             (const cs_lnum_2_t *)(m->i_face_cells),
+                             da,
+                             xa);
+
+  cs_matrix_default_set_tuned(a);
+
+  _matrix_setup[setup_id][0] = a;
+  _matrix_setup[setup_id][1] = NULL;
+  _matrix_setup[setup_id][2] = NULL;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Call sparse linear equation solver setup for systems requiring
+ *        a general matrix assember..
+ *
+ * \param[in]  f_id                   associated field id, or < 0
+ * \param[in]  setup_id               associated setup id
+ * \param[in]  sc                     associated solver context
+ * \param[in]  symmetric              indicates if matrix coefficients
+ *                                    are symmetric
+ * \param[in]  diag_block_size        block sizes for diagonal, or NULL
+ * \param[in]  extra_diag_block_size  block sizes for extra diagonal,
+ *                                    or NULL
+ * \param[in]  da                     diagonal values (NULL if zero)
+ * \param[in]  xa                     extradiagonal values (NULL if zero)
+ */
+/*----------------------------------------------------------------------------*/
+
+static void
+_sles_setup_matrix_by_assembler(int               f_id,
+                                int               setup_id,
+                                cs_sles_t        *sc,
+                                bool              symmetric,
+                                const cs_lnum_t  *diag_block_size,
+                                const cs_lnum_t  *extra_diag_block_size,
+                                const cs_real_t  *da,
+                                const cs_real_t  *xa)
+{
+  cs_matrix_t *a = NULL;
+
+  /* Check field and associated internal coupling if present */
+
+  const cs_field_t *f = NULL;
+  if (f_id > -1)
+    f = cs_field_by_id(f_id);
+
+  cs_matrix_type_t m_type = CS_MATRIX_MSR;
+  if (strcmp(cs_sles_get_type(sc), "cs_sles_amgx_t") == 0)
+    m_type = CS_MATRIX_CSR;
+
+  /* Set coefficients */
+
+  a = cs_matrix_set_coefficients_by_assembler(f,
+                                              m_type,
+                                              symmetric,
+                                              diag_block_size,
+                                              extra_diag_block_size,
+                                              da,
+                                              xa);
+  cs_matrix_default_set_tuned(a);
+
+  _matrix_setup[setup_id][0] = a;
+  _matrix_setup[setup_id][1] = a; /* so it is freed later */
+}
+
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*============================================================================
@@ -286,8 +462,6 @@ cs_sles_default_setup(void)
   cs_sles_set_default_define(cs_sles_default);
   cs_sles_set_default_verbosity(cs_sles_default_get_verbosity);
 
-  int key_cal_opt_id = cs_field_key_id("var_cal_opt");
-
   /* Define for all variable fields */
 
   const int n_fields = cs_field_n_fields();
@@ -295,26 +469,24 @@ cs_sles_default_setup(void)
   /* Define solver for all variable fields if not already done,
      based on convection/diffusion */
 
-  if (key_cal_opt_id > -1) {
+  for (int f_id = 0; f_id < n_fields; f_id++) {
+    const cs_field_t *f = cs_field_by_id(f_id);
+    if (f->type & CS_FIELD_VARIABLE) {
 
-    for (int f_id = 0; f_id < n_fields; f_id++) {
-      const cs_field_t *f = cs_field_by_id(f_id);
-      if (f->type & CS_FIELD_VARIABLE) {
+      void *context = NULL;
+      cs_sles_t *sc = cs_sles_find(f->id, NULL);
+      if (sc != NULL)
+        context = cs_sles_get_context(sc);
 
-        void *context = NULL;
-        cs_sles_t *sc = cs_sles_find(f->id, NULL);
-        if (sc != NULL)
-          context = cs_sles_get_context(sc);
-
-        if (context == NULL) {
-          /* Get the calculation option from the field */
-          cs_var_cal_opt_t var_cal_opt;
-          cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-          bool symmetric = (var_cal_opt.iconv > 0) ? false : true;
+      if (context == NULL) {
+        /* Get the calculation option from the field */
+        const cs_equation_param_t *eqp = cs_field_get_equation_param_const(f);
+        if (eqp != NULL) {
+          bool symmetric = (eqp->iconv > 0) ? false : true;
           _sles_default_native(f_id, NULL, CS_MATRIX_N_TYPES, symmetric);
         }
-
       }
+
     }
 
   }
@@ -324,9 +496,6 @@ cs_sles_default_setup(void)
   cs_log_printf(CS_LOG_SETUP, "\n");
   cs_log_separator(CS_LOG_SETUP);
 
-  if (cs_multigrid_needed() && cs_glob_n_ranks > 1)
-    cs_grid_log_merge_options();
-
   cs_sles_it_log_parallel_options();
 
   cs_sles_log(CS_LOG_SETUP);
@@ -381,7 +550,7 @@ cs_sles_default_get_verbosity(int          f_id,
     if (f->type & CS_FIELD_VARIABLE) {
       cs_var_cal_opt_t var_cal_opt;
       cs_field_get_key_struct(f, k_cal_opt_id, &var_cal_opt);
-      retval = var_cal_opt.iwarni;
+      retval = var_cal_opt.verbosity;
     }
     else
       retval = cs_field_get_key_int(f, k_log);
@@ -412,8 +581,8 @@ cs_sles_default_get_verbosity(int          f_id,
 void
 cs_sles_setup_native_conv_diff(int                  f_id,
                                const char          *name,
-                               const int           *diag_block_size,
-                               const int           *extra_diag_block_size,
+                               const cs_lnum_t     *diag_block_size,
+                               const cs_lnum_t     *extra_diag_block_size,
                                const cs_real_t     *da,
                                const cs_real_t     *xa,
                                const cs_real_t     *da_conv,
@@ -449,27 +618,6 @@ cs_sles_setup_native_conv_diff(int                  f_id,
          "If this is not an error, increase CS_SLES_DEFAULT_N_SETUPS\n"
          "  in file %s.", CS_SLES_DEFAULT_N_SETUPS, __FILE__);
 
-    /* Set extended contribution for domain coupling */
-
-    if (f_id > -1) {
-
-      const cs_field_t *f = cs_field_by_id(f_id);
-      int coupling_id
-        = cs_field_get_key_int(f, cs_field_key_id("coupling_entity"));
-
-      if (coupling_id > -1) {
-        /* TODO base matrix type on solver type (for external libraries) */
-        a = cs_matrix_set_coefficients_coupled(f,
-                                               CS_MATRIX_MSR,
-                                               false, /* symmetric */
-                                               diag_block_size,
-                                               extra_diag_block_size,
-                                               da,
-                                               xa);
-      }
-
-    }
-
     if (a == NULL) {
 
       a = cs_matrix_msr(false,
@@ -485,11 +633,14 @@ cs_sles_setup_native_conv_diff(int                  f_id,
                                  da,
                                  xa);
 
+    }
+
+    if (a_conv == NULL) {
+
       cs_matrix_t *a_ref = cs_matrix_default(false,
                                              diag_block_size,
                                              extra_diag_block_size);
-      if (a_conv == NULL)
-        a_conv = cs_matrix_create_by_copy(a_ref);
+      a_conv = cs_matrix_create_by_copy(a_ref);
 
       cs_matrix_set_coefficients(a_conv,
                                  false,
@@ -500,8 +651,14 @@ cs_sles_setup_native_conv_diff(int                  f_id,
                                  da_conv,
                                  xa_conv);
 
-      if (a_diff == NULL)
-        a_diff = cs_matrix_create_by_copy(a_ref);
+    }
+
+    if (a_diff == NULL) {
+
+      cs_matrix_t *a_ref = cs_matrix_default(false,
+                                             diag_block_size,
+                                             extra_diag_block_size);
+      a_diff = cs_matrix_create_by_copy(a_ref);
 
       cs_matrix_set_coefficients(a_diff,
                                  false,
@@ -526,6 +683,8 @@ cs_sles_setup_native_conv_diff(int                  f_id,
     a_diff = _matrix_setup[setup_id][2];
   }
 
+  cs_matrix_default_set_tuned(a);
+
   /* Setup system */
 
   if (strcmp(cs_sles_get_type(sc), "cs_multigrid_t") != 0)
@@ -539,93 +698,6 @@ cs_sles_setup_native_conv_diff(int                  f_id,
   cs_multigrid_setup_conv_diff(mg, name, a, a_conv, a_diff, verbosity);
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Call sparse linear equation solver setup for systems with internal
- *        coupling.
- *
- * \param[in]  f_id                   associated field id, or < 0
- * \param[in]  name                   associated name if f_id < 0, or NULL
- * \param[in]  symmetric              indicates if matrix coefficients
- *                                    are symmetric
- * \param[in]  diag_block_size        block sizes for diagonal, or NULL
- * \param[in]  extra_diag_block_size  block sizes for extra diagonal,
- *                                    or NULL
- * \param[in]  da                     diagonal values (NULL if zero)
- * \param[in]  xa                     extradiagonal values (NULL if zero)
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_sles_setup_native_coupling(int               f_id,
-                              const char       *name,
-                              bool              symmetric,
-                              const int        *diag_block_size,
-                              const int        *extra_diag_block_size,
-                              const cs_real_t  *da,
-                              const cs_real_t  *xa)
-{
-  cs_matrix_t *a = NULL;
-
-  /* Check if this system has already been setup */
-
-  cs_sles_t *sc = cs_sles_find_or_add(f_id, name);
-
-  int setup_id = 0;
-  while (setup_id < _n_setups) {
-    if (_sles_setup[setup_id] == sc)
-      break;
-    else
-      setup_id++;
-  }
-
-  if (setup_id >= _n_setups) {
-
-    _n_setups += 1;
-
-    if (_n_setups > CS_SLES_DEFAULT_N_SETUPS)
-      bft_error
-        (__FILE__, __LINE__, 0,
-         "Too many linear systems solved without calling cs_sles_free_native\n"
-         "  maximum number of systems: %d\n"
-         "If this is not an error, increase CS_SLES_DEFAULT_N_SETUPS\n"
-         "  in file %s.", CS_SLES_DEFAULT_N_SETUPS, __FILE__);
-
-    /* Set extended contribution for domain coupling */
-
-    if (f_id > -1) {
-
-      const cs_field_t *f = cs_field_by_id(f_id);
-      int coupling_id
-        = cs_field_get_key_int(f, cs_field_key_id("coupling_entity"));
-
-      if (coupling_id > -1) {
-        /* TODO base matrix type on solver type (for external libraries) */
-        a = cs_matrix_set_coefficients_coupled(f,
-                                               CS_MATRIX_MSR,
-                                               symmetric,
-                                               diag_block_size,
-                                               extra_diag_block_size,
-                                               da,
-                                               xa);
-      }
-
-    }
-
-    _sles_setup[setup_id] = sc;
-    _matrix_setup[setup_id][0] = a;
-    _matrix_setup[setup_id][1] = a; /* so it is freed later */
-
-  }
-  else {
-    a = _matrix_setup[setup_id][0];
-  }
-
-  /* Setup system */
-
-  cs_sles_setup(sc, a);
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Call sparse linear equation solver using native matrix arrays.
@@ -656,8 +728,8 @@ cs_sles_convergence_state_t
 cs_sles_solve_native(int                  f_id,
                      const char          *name,
                      bool                 symmetric,
-                     const int           *diag_block_size,
-                     const int           *extra_diag_block_size,
+                     const cs_lnum_t     *diag_block_size,
+                     const cs_lnum_t     *extra_diag_block_size,
                      const cs_real_t     *da,
                      const cs_real_t     *xa,
                      cs_halo_rotation_t   rotation_mode,
@@ -673,8 +745,6 @@ cs_sles_solve_native(int                  f_id,
 
   const cs_mesh_t *m = cs_glob_mesh;
 
-  bool need_msr = false;
-
   /* Check if this system has already been setup */
 
   cs_sles_t *sc = cs_sles_find_or_add(f_id, name);
@@ -699,97 +769,53 @@ cs_sles_solve_native(int                  f_id,
          "If this is not an error, increase CS_SLES_DEFAULT_N_SETUPS\n"
          "  in file %s.", CS_SLES_DEFAULT_N_SETUPS, __FILE__);
 
-    /* If context has not been defined yet, temporarily set
-       matrix coefficients (using native matrix, which has lowest
-       overhead as coefficients are provided in that form)
-       to define the required context.
 
-       The matrix type might be modified later based on solver
-       constraints. */
+    /* Check if we need to used a matrix assembler */
 
-    if (cs_sles_get_context(sc) == NULL) {
-      a = cs_matrix_native(symmetric,
-                           diag_block_size,
-                           extra_diag_block_size);
+    bool need_matrix_assembler = false;
 
-      cs_matrix_set_coefficients(a,
-                                 symmetric,
-                                 diag_block_size,
-                                 extra_diag_block_size,
-                                 m->n_i_faces,
-                                 (const cs_lnum_2_t *)(m->i_face_cells),
-                                 da,
-                                 xa);
-
-      cs_sles_define_t  *sles_default_func = cs_sles_get_default_define();
-      sles_default_func(f_id, name, a);
-      cs_matrix_release_coefficients(a);
+    if (f_id > -1) {
+      const cs_field_t *f = cs_field_by_id(f_id);
+      int coupling_id
+        = cs_field_get_key_int(f, cs_field_key_id("coupling_entity"));
+      if (coupling_id > -1)
+        need_matrix_assembler = true;
+    }
+    const char *sles_type = cs_sles_get_type(sc);
+    if (strcmp(sles_type, "cs_sles_amgx_t") == 0) {
+      if (m->n_init_perio > 0)
+        need_matrix_assembler = true;
     }
 
-    assert(cs_sles_get_context(sc) != NULL);
+    if (need_matrix_assembler)
+      _sles_setup_matrix_by_assembler(f_id,
+                                      setup_id,
+                                      sc,
+                                      symmetric,
+                                      diag_block_size,
+                                      extra_diag_block_size,
+                                      da,
+                                      xa);
 
-    cs_sles_pc_t  *pc = NULL;
-    cs_multigrid_t *mg = NULL;
-
-    if (strcmp(cs_sles_get_type(sc), "cs_sles_it_t") == 0) {
-      cs_sles_it_t *c = cs_sles_get_context(sc);
-      cs_sles_it_type_t s_type = cs_sles_it_get_type(c);
-      if (   s_type >= CS_SLES_P_GAUSS_SEIDEL
-          && s_type <= CS_SLES_TS_B_GAUSS_SEIDEL)
-        need_msr = true;
-      else {
-        pc = cs_sles_it_get_pc(c);
-        if (pc != NULL) {
-          if (strcmp(cs_sles_pc_get_type(pc), "multigrid") == 0)
-            mg = cs_sles_pc_get_context(pc);
-        }
-      }
-    }
-    else if (strcmp(cs_sles_get_type(sc), "cs_multigrid_t") == 0)
-      mg = cs_sles_get_context(sc);
-
-    if (mg != NULL) {
-      cs_sles_it_type_t fs_type = cs_multigrid_get_fine_solver_type(mg);
-      if (   fs_type >= CS_SLES_P_GAUSS_SEIDEL
-          && fs_type <= CS_SLES_TS_B_GAUSS_SEIDEL)
-        need_msr = true;
-    }
-
-    /* MSR not supported yet for full blocks */
-    if (extra_diag_block_size != NULL) {
-      if (extra_diag_block_size[0] > 1)
-        need_msr = false;
-    }
-
-    if (need_msr)
-      a = cs_matrix_msr(symmetric,
-                        diag_block_size,
-                        extra_diag_block_size);
     else
-      a = cs_matrix_default(symmetric,
-                            diag_block_size,
-                            extra_diag_block_size);
-
-    cs_matrix_set_coefficients(a,
-                               symmetric,
-                               diag_block_size,
-                               extra_diag_block_size,
-                               m->n_i_faces,
-                               (const cs_lnum_2_t *)(m->i_face_cells),
-                               da,
-                               xa);
+      _sles_setup_matrix_native(f_id,
+                                name,
+                                setup_id,
+                                sc,
+                                symmetric,
+                                diag_block_size,
+                                extra_diag_block_size,
+                                da,
+                                xa);
 
     _sles_setup[setup_id] = sc;
-    _matrix_setup[setup_id][0] = a;
-    _matrix_setup[setup_id][1] = NULL;
-    _matrix_setup[setup_id][2] = NULL;
-
   }
-  else
-    a = _matrix_setup[setup_id][0];
+
+  a = _matrix_setup[setup_id][0];
 
   /* If system uses specific halo (i.e. when matrix contains more than
-     face->cell nonzeroes), allocate specific buffers. */
+     face->cell nonzeroes), allocate specific buffers and synchronize
+     right hand side. */
 
   cs_real_t *_vx = vx, *_rhs = NULL;
   const cs_real_t *rhs_p = rhs;
@@ -813,6 +839,7 @@ cs_sles_solve_native(int                  f_id,
     }
     cs_matrix_pre_vector_multiply_sync(rotation_mode, a, _rhs);
     rhs_p = _rhs;
+
   }
 
   /* Solve system */
@@ -995,7 +1022,7 @@ cs_sles_default_error(cs_sles_t                    *sles,
 
       bft_printf(_("\n\n"
                    "%s [%s]: divergence\n"
-                   "  fallback from multigrid to %s-preconditionned CG solver\n"
+                   "  fallback from multigrid to %s-preconditioned CG solver\n"
                    "  for re-try and subsequent solves.\n"),
                    "Multigrid", name, "Jacobi");
 
@@ -1016,7 +1043,7 @@ cs_sles_default_error(cs_sles_t                    *sles,
   /* Reset solution if new solve is expected */
 
   if (alternative) {
-    const int *db_size = cs_matrix_get_diag_block_size(a);
+    const cs_lnum_t *db_size = cs_matrix_get_diag_block_size(a);
     const cs_lnum_t n_cols = cs_matrix_get_n_columns(a) * db_size[1];
     for (cs_lnum_t i = 0; i < n_cols; i++)
       vx[i] = 0;
