@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -131,8 +131,8 @@ cs_sles_default_finalize(void);
 void
 cs_sles_setup_native_conv_diff(int                  f_id,
                                const char          *name,
-                               const int           *diag_block_size,
-                               const int           *extra_diag_block_size,
+                               const cs_lnum_t     *diag_block_size,
+                               const cs_lnum_t     *extra_diag_block_size,
                                const cs_real_t     *da,
                                const cs_real_t     *xa,
                                const cs_real_t     *da_conv,
@@ -167,8 +167,8 @@ cs_sles_convergence_state_t
 cs_sles_solve_native(int                  f_id,
                      const char          *name,
                      bool                 symmetric,
-                     const int           *diag_block_size,
-                     const int           *extra_diag_block_size,
+                     const cs_lnum_t     *diag_block_size,
+                     const cs_lnum_t     *extra_diag_block_size,
                      const cs_real_t     *da,
                      const cs_real_t     *xa,
                      cs_halo_rotation_t   rotation_mode,
@@ -196,7 +196,7 @@ cs_sles_free_native(int          f_id,
  * sparse linear equation solver.
  *
  * In case of divergence with an iterative solver, this error handler
- * switches to a default preconditionner, then resets the solution vector.
+ * switches to a default preconditioner, then resets the solution vector.
  *
  * The default error for the solver type handler is then  set, in case
  * the solution fails again.
