@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -59,6 +59,7 @@
 
 #include "cs_base.h"
 #include "cs_log.h"
+#include "cs_fp_exception.h"
 #include "cs_halo.h"
 #include "cs_matrix.h"
 #include "cs_matrix_default.h"
@@ -93,7 +94,7 @@ BEGIN_C_DECLS
   This function is called the end of the setup stage for a KSP solver.
 
   Note that using the advanced KSPSetPostSolve and KSPSetPreSolve functions,
-  this also allows setting furthur function pointers for pre and post-solve
+  this also allows setting further function pointers for pre and post-solve
   operations (see the PETSc documentation).
 
   Note: if the context pointer is non-NULL, it must point to valid data
@@ -137,9 +138,9 @@ struct _cs_sles_petsc_t {
 
   int                  n_iterations_last;  /* Number of iterations for last
                                               system resolution */
-  int                  n_iterations_min;   /* Minimum number ot iterations
+  int                  n_iterations_min;   /* Minimum number of iterations
                                               in system resolution history */
-  int                  n_iterations_max;   /* Maximum number ot iterations
+  int                  n_iterations_max;   /* Maximum number of iterations
                                               in system resolution history */
   int long long        n_iterations_tot;   /* Total accumulated number of
                                               iterations */
@@ -188,12 +189,99 @@ static PetscLogStage _log_stage[2];
  * Private function definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Export the linear system using PETSc Viewer mechanism
+ *
+ * parameters:
+ *   name <-- name of the related linear system
+ *   ksp  <-- Pointer to PETSc KSP structure
+ *   rhs  <-- PETSc vector
+ *----------------------------------------------------------------------------*/
+
+static void
+_export_petsc_system(const char   *name,
+                     KSP           ksp,
+                     Vec           b)
+{
+  const char *p = getenv("CS_PETSC_SYSTEM_VIEWER");
+
+  if (p == NULL)
+    return;
+
+  /* Get system and preconditioner matrices */
+
+  Mat a, pa;
+  KSPGetOperators(ksp, &a, &pa);
+
+  char  *filename = NULL;
+  int len = strlen(name) + strlen("_matrix.dat") + 1;
+  BFT_MALLOC(filename, len, char);
+
+  if (strcmp(p, "BINARY") == 0) {
+
+    PetscViewer viewer;
+
+    /* Matrix */
+    sprintf(filename, "%s_matrix.dat", name);
+    PetscViewerBinaryOpen(PETSC_COMM_WORLD, filename, FILE_MODE_WRITE, &viewer);
+    MatView(a, viewer);
+    PetscViewerDestroy(&viewer);
+
+    /* Right-hand side */
+    sprintf(filename, "%s_rhs.dat", name);
+    PetscViewerBinaryOpen(PETSC_COMM_WORLD, filename, FILE_MODE_WRITE, &viewer);
+    VecView(b, viewer);
+    PetscViewerDestroy(&viewer);
+
+  }
+  else if (strcmp(p, "ASCII") == 0) {
+
+    PetscViewer viewer;
+
+    /* Matrix */
+    sprintf(filename, "%s_matrix.txt", name);
+    PetscViewerASCIIOpen(PETSC_COMM_WORLD, filename, &viewer);
+    MatView(a, viewer);
+    PetscViewerDestroy(&viewer);
+
+    /* Right-hand side */
+    sprintf(filename, "%s_rhs.txt", name);
+    PetscViewerASCIIOpen(PETSC_COMM_WORLD, filename, &viewer);
+    VecView(b, viewer);
+    PetscViewerDestroy(&viewer);
+
+  }
+  else if (strcmp(p, "MATLAB") == 0) {
+
+    PetscViewer viewer;
+
+    /* Matrix */
+    sprintf(filename, "%s_matrix.m", name);
+    PetscViewerASCIIOpen(PETSC_COMM_WORLD, filename, &viewer);
+    PetscViewerPushFormat(viewer, PETSC_VIEWER_ASCII_MATLAB);
+    MatView(a, viewer);
+    PetscViewerPopFormat(viewer);
+    PetscViewerDestroy(&viewer);
+
+    /* Right-hand side */
+    sprintf(filename, "%s_rhs.m", name);
+    PetscViewerASCIIOpen(PETSC_COMM_WORLD, filename, &viewer);
+    PetscViewerPushFormat(viewer, PETSC_VIEWER_ASCII_MATLAB);
+    VecView(b, viewer);
+    PetscViewerPopFormat(viewer);
+    PetscViewerDestroy(&viewer);
+
+  }
+
+  BFT_FREE(filename);
+}
+
 /*----------------------------------------------------------------------------
  * Local matrix.vector product y = A.x with shell matrix.
  *
  * parameters:
  *   a  <-- Pointer to PETSc matrix structure
- *   x  <-- Multipliying vector values
+ *   x  <-- Multiplying vector values
  *   y  --> Resulting vector
  *----------------------------------------------------------------------------*/
 
@@ -265,6 +353,8 @@ _shell_get_row(Mat                 a,
                const PetscInt     *cols[],
                const PetscScalar  *vals[])
 {
+  CS_UNUSED(nnz);
+
   _mat_shell_t *sh;
 
   MatShellGetContext(a, &sh);
@@ -292,6 +382,8 @@ _shell_mat_duplicate(Mat                  a,
                      MatDuplicateOption   op,
                      Mat                 *m)
 {
+  CS_UNUSED(op);
+
   _mat_shell_t *sh;
 
   MatShellGetContext(a, &sh);
@@ -316,7 +408,7 @@ _shell_mat_duplicate(Mat                  a,
 }
 
 /*----------------------------------------------------------------------------
- * Duplicate matrix
+ * Destroy matrix
  *
  * parameters:
  *   a  <-- Pointer to PETSc matrix structure
@@ -329,6 +421,8 @@ _shell_mat_destroy(Mat                  a,
                    MatDuplicateOption   op,
                    Mat                 *m)
 {
+  CS_UNUSED(m);
+
   _mat_shell_t *sh;
 
   MatShellGetContext(a, &sh);
@@ -376,10 +470,10 @@ _cs_ksp_converged(KSP                  ksp,
 /*----------------------------------------------------------------------------
  * Function pointer for user settings of a PETSc KSP solver setup.
  *
- * This function is called the end of the setup stage for a KSP solver.
+ * This function is called at the end of the setup stage for a KSP solver.
  *
  * Note that using the advanced KSPSetPostSolve and KSPSetPreSolve functions,
- * this also allows setting furthur function pointers for pre and post-solve
+ * this also allows setting further function pointers for pre and post-solve
  * operations (see the PETSc documentation).
  *
  * Note: if the context pointer is non-NULL, it must point to valid data
@@ -388,7 +482,7 @@ _cs_ksp_converged(KSP                  ksp,
  *
  * parameters:
  *   context <-> pointer to optional (untyped) value or structure
- *   ksp     <-> pointer to PETSc KSP context
+ *   ksp     <-> PETSc KSP context
  *----------------------------------------------------------------------------*/
 
 void
@@ -1137,7 +1231,7 @@ cs_sles_petsc_setup(void               *context,
   /* KSPSetup could be called here for better separation of setup/solve
      logging, but calling it systematically seems to cause issues
      at least with the performance of the GAMG preconditioner
-     (possibly calling unneed operations). So we avoid it for now,
+     (possibly calling unneeded operations). So we avoid it for now,
      noting that the user always has to option of calling it at the
      end of the setup hook. */
 
@@ -1215,6 +1309,9 @@ cs_sles_petsc_solve(void                *context,
                     size_t               aux_size,
                     void                *aux_vectors)
 {
+  CS_UNUSED(aux_size);
+  CS_UNUSED(aux_vectors);
+
   cs_sles_convergence_state_t cvg = CS_SLES_ITERATING;
 
   cs_timer_t t0;
@@ -1304,8 +1401,15 @@ cs_sles_petsc_solve(void                *context,
 
   /* Resolution */
 
+  cs_fp_exception_disable_trap();
+
   KSPSolve(sd->ksp, b, x);
 
+  cs_fp_exception_restore_trap();
+
+  if (getenv("CS_PETSC_SYSTEM_VIEWER") != NULL)
+    _export_petsc_system(name, sd->ksp, b);
+
   VecDestroy(&x);
   VecDestroy(&b);
 
@@ -1421,6 +1525,11 @@ cs_sles_petsc_error_post_and_abort(cs_sles_t                    *sles,
                                    const cs_real_t              *rhs,
                                    cs_real_t                    *vx)
 {
+  CS_UNUSED(a);
+  CS_UNUSED(rotation_mode);
+  CS_UNUSED(rhs);
+  CS_UNUSED(vx);
+
   if (state >= CS_SLES_BREAKDOWN)
     return false;
 
@@ -1490,7 +1599,6 @@ cs_sles_petsc_log(const void  *context,
                     s_type, p_type, norm_type_name, m_type);
 
   }
-
   else if (log_type == CS_LOG_PERFORMANCE) {
 
     int n_calls = c->n_solves;
