@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -31,6 +31,9 @@
  * PETSc headers
  *----------------------------------------------------------------------------*/
 
+#include <petscversion.h>
+#include <petscconf.h>
+#include <petscviewer.h>
 #include <petscksp.h>
 
 /*----------------------------------------------------------------------------
@@ -38,9 +41,8 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
-#include "cs_halo_perio.h"
 #include "cs_matrix.h"
-#include "cs_time_plot.h"
+#include "cs_sles.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -57,10 +59,10 @@ BEGIN_C_DECLS
 /*----------------------------------------------------------------------------
  * Function pointer for user settings of a PETSc KSP solver setup.
  *
- * This function is called the end of the setup stage for a KSP solver.
+ * This function is called at the end of the setup stage for a KSP solver.
  *
  * Note that using the advanced KSPSetPostSolve and KSPSetPreSolve functions,
- * this also allows setting furthur function pointers for pre and post-solve
+ * this also allows setting further function pointers for pre and post-solve
  * operations (see the PETSc documentation).
  *
  * Note: if the context pointer is non-NULL, it must point to valid data
@@ -91,10 +93,10 @@ typedef struct _cs_sles_petsc_t  cs_sles_petsc_t;
 /*----------------------------------------------------------------------------
  * Function pointer for user settings of a PETSc KSP solver setup.
  *
- * This function is called the end of the setup stage for a KSP solver.
+ * This function is called at the end of the setup stage for a KSP solver.
  *
  * Note that using the advanced KSPSetPostSolve and KSPSetPreSolve functions,
- * this also allows setting furthur function pointers for pre and post-solve
+ * this also allows setting further function pointers for pre and post-solve
  * operations (see the PETSc documentation).
  *
  * Note: if the context pointer is non-NULL, it must point to valid data
@@ -110,6 +112,52 @@ void
 cs_user_sles_petsc_hook(void               *context,
                         KSP                 ksp);
 
+/*=============================================================================
+ * Static inline public function prototypes
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------
+ * Initialize PETSc if needed
+ *----------------------------------------------------------------------------*/
+
+static inline void
+cs_sles_petsc_init(void)
+{
+  /* Initialization must be called before setting options;
+     it does not need to be called before calling
+     cs_sles_petsc_define(), as this is handled automatically. */
+
+  PetscBool is_initialized;
+  PetscInitialized(&is_initialized);
+
+  if (is_initialized == PETSC_FALSE) {
+#if defined(HAVE_MPI)
+    PETSC_COMM_WORLD = cs_glob_mpi_comm;
+#endif
+    PetscInitializeNoArguments();
+  }
+}
+
+/*----------------------------------------------------------------------------
+ * \brief Output the settings of a KSP structure
+ *
+ * \param[in]  ksp     Krylov SubSpace structure
+ *----------------------------------------------------------------------------*/
+
+static inline void
+cs_sles_petsc_log_setup(KSP          ksp)
+{
+  PetscViewer  v;
+
+  PetscViewerCreate(PETSC_COMM_WORLD, &v);
+  PetscViewerSetType(v, PETSCVIEWERASCII);
+  PetscViewerFileSetMode(v, FILE_MODE_APPEND);
+  PetscViewerFileSetName(v, "petsc_setup.log");
+
+  KSPView(ksp, v);
+  PetscViewerDestroy(&v);
+}
+
 /*=============================================================================
  * Public function prototypes
  *============================================================================*/
