@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -36,7 +36,7 @@
 */
 
 #if (SIZEOF_LONG < 8) && (_FILE_OFFSET_BITS == 64)
-# if defined(__linux__) || defined(__blrts__) || defined(__bg__)
+# if defined(__linux__)
 #  if !defined(_POSIX_SOURCE)
 #    define _GNU_SOURCE 1
 #  endif
@@ -1441,15 +1441,22 @@ _extract_section_values(_cs_io_t    *inp,
     /* Check type name */
 
     if (type_name[0] == 'c') {
-      char *_data = NULL;
-      MEM_MALLOC(_data, n_vals + 1, char);
-      memcpy(_data, data, n_vals);
-      for (i = 0; i < n_vals; i++)
-        if (_data[i] == '\0')
-          _data[i] = '\n';
-      _data[n_vals] = '\0';
-      printf("%s", _data);
-      MEM_FREE(_data);
+      if (inp->location_id == 0) {
+        char *_data = NULL;
+        MEM_MALLOC(_data, n_vals + 1, char);
+        memcpy(_data, data, n_vals);
+        for (i = 0; i < n_vals; i++)
+          if (_data[i] == '\0')
+            _data[i] = '\n';
+        _data[n_vals] = '\0';
+        printf("%s", _data);
+        MEM_FREE(_data);
+      }
+      else {
+        char *_data = data;
+        for (i = 0; i < n_vals; i++)
+          printf("%d\n", (int)(_data[i]));
+      }
     }
 
 #if (__STDC_VERSION__ >= 199901L)
@@ -2199,7 +2206,7 @@ _compare_ints(_cs_io_t         *inp1,
  *   block_size  <-- buffer size
  *   n_echo      <-- number of values to echo
  *   n_echo_cur  <-> number of values already echoed
- *   f_stats     <-> max, and total difference
+ *   f_stats     <-> max, total, max relative, total relative difference
  *
  * returns:
  *   number of different values
@@ -2216,7 +2223,7 @@ _compare_floats(_cs_io_t         *inp1,
                 size_t            block_size,
                 size_t            n_echo,
                 long long        *n_echo_cur,
-                double            f_stats[2])
+                double            f_stats[4])
 {
   size_t i;
   size_t n_diffs = 0;
@@ -2226,10 +2233,14 @@ _compare_floats(_cs_io_t         *inp1,
     if (delta < 0.0)
       delta = -delta;
     if (delta > f_threshold) {
+      double delta_r = delta / CS_MAX(CS_ABS(cmp1[i]), CS_ABS(cmp2[i]));
       n_diffs++;
       if (delta > f_stats[0])
         f_stats[0] = delta;
+      if (delta_r > f_stats[2])
+        f_stats[2] = delta_r;
       f_stats[1] += delta;
+      f_stats[3] += delta_r;
     }
   }
 
@@ -2360,7 +2371,7 @@ _compare_sections(_cs_io_t    *inp1,
     size_t max_block_size = 2 << 16;
     void *buf1 = NULL, *buf2 = NULL;
     void *cmp1 = NULL, *cmp2 = NULL;
-    double f_stats[2] = {0.0, 0.0};
+    double f_stats[4] = {0.0, 0.0, 0.0, 0.0};
     const size_t type_size1 = _type_size_from_name(type1);
     const size_t type_size2 = _type_size_from_name(type2);
 
@@ -2422,8 +2433,10 @@ _compare_sections(_cs_io_t    *inp1,
 
     if (n_diffs > 0) {
       if (type1[0] == 'r')
-        printf(_("    Differences: %llu; Max: %g; Mean: %g\n\n"),
-               n_diffs, f_stats[0], f_stats[1]/n_diffs);
+        printf(_("    Differences: %llu; Max: %g; Mean: %g; "
+                 "Rel Max: %6.2e; Rel Mean: %6.2e\n\n"),
+               n_diffs, f_stats[0], f_stats[1]/n_diffs,
+               f_stats[2], f_stats[3]/n_diffs);
       else
         printf(_("    Differences: %llu\n\n"), n_diffs);
       retval = 1;
