@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -43,8 +43,7 @@
 #include "bft_mem.h"
 #include "bft_printf.h"
 
-#include "fvm_selector.h"
-
+#include "cs_ale.h"
 #include "cs_all_to_all.h"
 #include "cs_base.h"
 #include "cs_base_fortran.h"
@@ -52,6 +51,7 @@
 #include "cs_boundary_zone.h"
 #include "cs_calcium.h"
 #include "cs_cdo_main.h"
+#include "cs_cell_to_vertex.h"
 #include "cs_control.h"
 #include "cs_coupling.h"
 #include "cs_ctwr.h"
@@ -64,9 +64,14 @@
 #include "cs_gradient.h"
 #include "cs_gradient_perio.h"
 #include "cs_gui.h"
+#include "cs_gui_boundary_conditions.h"
+#include "cs_gui_conjugate_heat_transfer.h"
+#include "cs_gui_mesh.h"
+#include "cs_gui_mobile_mesh.h"
 #include "cs_gui_output.h"
 #include "cs_gui_particles.h"
 #include "cs_gui_radiative_transfer.h"
+#include "cs_gui_util.h"
 #include "cs_io.h"
 #include "cs_join.h"
 #include "cs_lagr.h"
@@ -84,8 +89,10 @@
 #include "cs_mesh_quantities.h"
 #include "cs_mesh_bad_cells.h"
 #include "cs_mesh_smoother.h"
+#include "cs_notebook.h"
 #include "cs_opts.h"
 #include "cs_param_cdo.h"
+#include "cs_paramedmem_coupling.h"
 #include "cs_parameters.h"
 #include "cs_partition.h"
 #include "cs_physical_properties.h"
@@ -94,9 +101,12 @@
 #include "cs_preprocess.h"
 #include "cs_preprocessor_data.h"
 #include "cs_probe.h"
+#include "cs_property.h"
 #include "cs_prototypes.h"
 #include "cs_random.h"
 #include "cs_restart.h"
+#include "cs_restart_map.h"
+#include "cs_runaway_check.h"
 #include "cs_sles.h"
 #include "cs_sles_default.h"
 #include "cs_sat_coupling.h"
@@ -107,6 +117,7 @@
 #include "cs_timer_stats.h"
 #include "cs_tree.h"
 #include "cs_turbomachinery.h"
+#include "cs_volume_mass_injection.h"
 #include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------*/
@@ -123,23 +134,6 @@ BEGIN_C_DECLS
  * Public function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Main function for Code_Saturne run.
- *
- * This function is called either by main() in the standard case, or by
- * a SALOME standalone module's yacsinit() function. As the latter cannot
- * return, almost all of the code's execution steps (except command-line
- * initialization, required to know if we are running in standard mode or
- * pluging a SALOME module) are done here.
- *
- * As yacsinit() can take no arguments, the command-line options must be
- * defined as a static global variable by main() so as to be usable
- * by run().
- *----------------------------------------------------------------------------*/
-
-void
-cs_run(void);
-
 /*============================================================================
  * Static global variables
  *============================================================================*/
@@ -159,21 +153,13 @@ static cs_opts_t  opts;
 /*----------------------------------------------------------------------------
  * Main function for Code_Saturne run.
  *
- * This function is called either by main() in the standard case, or by
- * a SALOME standalone module's yacsinit() function. As the latter cannot
- * return, almost all of the code's execution steps (except command-line
- * initialization, required to know if we are running in standard mode or
- * pluging a SALOME module) are done here.
- *
- * As yacsinit() can take no arguments, the command-line options must be
- * defined as a static global variable by main() so as to be usable
- * by run().
+ * This function is called by main().
  *----------------------------------------------------------------------------*/
 
-void
-cs_run(void)
+static void
+_run(void)
 {
-  cs_int_t  ivoset = 0;
+  int  ivoset = 0;
 
   int  check_mask = 0;
   cs_halo_type_t halo_type = CS_HALO_STANDARD;
@@ -196,10 +182,14 @@ cs_run(void)
   cs_user_parallel_io();
   cs_file_defaults_info();
 
+  cs_gui_mpi_algorithms();
+
   cs_partition_external_library_info();
 
   bft_printf("\n");
 
+  cs_base_update_status("initializing\n");
+
   /* Initialize random number generator
      (used in only some cases, but safe to do, and inexpensive) */
 
@@ -218,6 +208,12 @@ cs_run(void)
 
   cs_turbomachinery_define();
 
+  /* Check if an internally generated cartesian mesh is used */
+  if (cs_gui_mesh_build_cartesian())
+    cs_gui_mesh_cartesian_define();
+
+  cs_user_mesh_cartesian_define();
+
   /* Call main calculation initialization function or help */
 
   cs_io_log_initialize();
@@ -237,18 +233,23 @@ cs_run(void)
   /* Create a new structure for the computational domain */
   cs_glob_domain = cs_domain_create();
 
-  /* Update mesh zones */
-  cs_domain_update_wall_zones(cs_glob_domain);
+  /* Define MPI-based Couplings if applicable */
+
+  cs_gui_syrthes_coupling();
+  cs_user_syrthes_coupling();
+  cs_user_saturne_coupling();
 
   /* Initialize Fortran API and calculation setup */
 
   if ((opts.preprocess | opts.verif) == false && opts.benchmark <= 0) {
 
-    cs_int_t _rank_id = cs_glob_rank_id, _n_ranks = cs_glob_n_ranks;
+    int _rank_id = cs_glob_rank_id, _n_ranks = cs_glob_n_ranks;
 
     cs_base_fortran_bft_printf_to_f();
 
-    cs_gui_init();
+    const char default_restart_mesh[] = "restart_mesh_input";
+    if (cs_file_isreg(default_restart_mesh))
+      cs_restart_map_set_mesh_input(default_restart_mesh);
 
     CS_PROCF(csinit, CSINIT)(&_rank_id, &_n_ranks);
 
@@ -258,8 +259,18 @@ cs_run(void)
     if (ivoset)
       halo_type = CS_HALO_EXTENDED;
 
+    if (cs_glob_ale > 0) {
+      cs_gui_mobile_mesh_get_boundaries(cs_glob_domain);
+      if (cs_glob_mesh->time_dep < CS_MESH_TRANSIENT_COORDS)
+        cs_glob_mesh->time_dep = CS_MESH_TRANSIENT_COORDS;
+    }
+
     cs_cdo_initialize_setup(cs_glob_domain);
 
+    /* Setup linear solvers */
+    cs_gui_linear_solvers();
+    cs_user_linear_solvers();
+
     cs_base_fortran_bft_printf_to_c();
 
     cs_ctwr_build_zones();
@@ -275,7 +286,7 @@ cs_run(void)
      may have the option of assigning a name to this instance. */
 
 #if defined(HAVE_MPI)
-  cs_coupling_discover_mpi_apps(opts.app_name);
+  cs_coupling_discover_mpi_apps(opts.app_name, NULL);
 #endif
 
   if (opts.app_name != NULL)
@@ -286,6 +297,8 @@ cs_run(void)
   cs_syr_coupling_all_init();
   cs_sat_coupling_all_init();
 
+  cs_paramedmem_coupling_all_init();
+
   /* Initialize main post-processing */
 
   cs_gui_postprocess_writers();
@@ -322,6 +335,9 @@ cs_run(void)
 
   cs_post_init_meshes(check_mask);
 
+  cs_user_mesh_modify_partial(cs_glob_mesh,
+                              cs_glob_mesh_quantities);
+
   /* Compute iterations or quality criteria depending on verification options */
 
   if (opts.verif == true) {
@@ -339,7 +355,7 @@ cs_run(void)
   }
 
   if (check_mask && cs_syr_coupling_n_couplings())
-    bft_error(__FILE__, __LINE__, errno,
+    bft_error(__FILE__, __LINE__, 0,
               _("Coupling with SYRTHES is not possible in mesh preprocessing\n"
                 "or verification mode."));
 
@@ -406,10 +422,14 @@ cs_run(void)
 
           cs_ctwr_build_all();
 
+          cs_volume_mass_injection_flag_zones();
+
           /* Setup couplings and fixed-mesh postprocessing */
 
           cs_syr_coupling_init_meshes();
 
+          cs_paramedmem_coupling_define_mesh_fields();
+
           cs_post_default_write_meshes();
 
           cs_turbomachinery_restart_mesh();
@@ -434,12 +454,6 @@ cs_run(void)
 
       }
 
-      /* Finalize sparse linear systems resolution */
-
-      cs_sles_default_finalize();
-
-      cs_matrix_finalize();
-
     }
 
     /* Finalize gradient computation */
@@ -449,10 +463,22 @@ cs_run(void)
 
     /* Finalize synthetic inlet condition generation */
 
-    cs_inflow_finalize();
+    cs_les_inflow_finalize();
 
   }
 
+  /* Finalize linear system resolution */
+
+  cs_sles_default_finalize();
+
+  /* Finalize sparse linear systems resolution */
+
+  cs_matrix_finalize();
+
+  /* Finalize user extra operations */
+  if (opts.verif == false)
+    cs_user_extra_operations_finalize(cs_glob_domain);
+
   /* Switch logging back to C (may be moved depending on Fortran dependencies) */
 
   cs_base_fortran_bft_printf_to_c();
@@ -473,11 +499,15 @@ cs_run(void)
   cs_syr_coupling_all_finalize();
 #if defined(HAVE_MPI)
   cs_sat_coupling_all_finalize();
+  cs_paramedmem_coupling_all_finalize();
   cs_coupling_finalize();
 #endif
 
   cs_control_finalize();
 
+  /* Free the checkpoint multiwriter structure */
+  cs_restart_multiwriters_destroy_all();
+
   /* Print some mesh statistics */
 
   cs_gui_usage_log();
@@ -492,8 +522,9 @@ cs_run(void)
 
   cs_internal_coupling_finalize();
 
-  /* Free thermal physical properties */
+  /* Free memory related to properties */
 
+  cs_property_destroy_all();
   cs_thermal_table_finalize();
 
   /* Free turbomachinery related structures */
@@ -516,6 +547,8 @@ cs_run(void)
   cs_gui_radiative_transfers_finalize();
   cs_gui_finalize();
 
+  cs_notebook_destroy_all();
+
   cs_field_pointer_destroy_all();
   cs_field_destroy_all();
   cs_field_destroy_all_keys();
@@ -526,6 +559,7 @@ cs_run(void)
 
   /* Free main mesh after printing some statistics */
 
+  cs_cell_to_vertex_free();
   cs_mesh_adjacencies_finalize();
 
   cs_boundary_zone_finalize();
@@ -549,6 +583,10 @@ cs_run(void)
 
   cs_base_time_summary();
   cs_base_mem_finalize();
+
+  cs_log_printf_flush(CS_LOG_N_TYPES);
+
+  cs_runaway_check_finalize();
 }
 
 /*============================================================================
@@ -619,29 +657,26 @@ main(int    argc,
 
   cs_base_error_init(opts.sig_defaults);
 
-  /* Open 'listing' (log) files */
+  /* Open 'run_solver.log' (log) files */
 
-  cs_base_fortran_bft_printf_set("listing", opts.ilisr0, opts.ilisrp);
+  cs_base_trace_set(opts.trace);
+  cs_base_fortran_bft_printf_set("run_solver", opts.logrp);
 
   /* Log-file header and command line arguments recap */
 
   cs_base_logfile_head(argc, argv);
 
-  /* Running as a standalone SALOME component, load YACS component
-     library and run yacsinit() component initialization and event loop,
-     which should itself include the standard run routine */
+  /* Load setup parameters if present */
 
-  if (opts.yacs_module != NULL) {
-    cs_calcium_load_yacs(opts.yacs_module);
-    BFT_FREE(opts.yacs_module);
-    cs_calcium_start_yacs(); /* Event-loop does not return as of this version */
-    cs_calcium_unload_yacs();
+  const char s_param[] = "setup.xml";
+  if (cs_file_isreg(s_param)) {
+    cs_gui_load_file(s_param);
+    cs_notebook_load_from_file();
   }
 
-  /* In standard case, simply call regular run() method */
+  /* Call main run() method */
 
-  else
-    cs_run();
+  _run();
 
   /* Return */
 
