@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -33,7 +33,6 @@ subroutine atini1
 
 use paramx
 use dimens
-use ihmpre
 use numvar
 use optcal
 use cstphy
@@ -46,7 +45,6 @@ use atincl
 use atsoil
 use atchem
 use atimbr
-use siream
 use field
 use cs_c_bindings
 
@@ -58,22 +56,14 @@ implicit none
 
 integer          ii, isc, jj
 double precision turb_schmidt
-
-type(var_cal_opt) :: vcopt
+double precision visls_0
 
 !===============================================================================
 
 !===============================================================================
-! 0. VERIFICATIONS
+! 1. VERIFICATIONS
 !===============================================================================
 
-if (ippmod(iatmos).ge.2) then
-  if (itytur.ne.2) then
-    write(nfecra, 1002)
-    call csexit(1)
-  endif
-endif
-
 if (ippmod(iatmos).le.1) then
   if (iatra1.eq.1.or.iatsoil.eq.1) then
     write(nfecra, 1003)
@@ -81,25 +71,6 @@ if (ippmod(iatmos).le.1) then
   endif
 endif
 
-!===============================================================================
-! 1. INFORMATIONS GENERALES
-!===============================================================================
-
-!--> constants used in the atmospheric physics module
-!    (see definition in atincl.h):
-
-ps = 1.0d5
-rvsra = 1.608d0
-cpvcpa = 1.866d0
-clatev = 2.501d6
-gammat = -6.5d-03
-rvap = rvsra*rair
-
-! Density and viscosity
-
-irovar = 0
-ivivar = 0
-
 !===============================================================================
 ! 2. Transported variables for IPPMOD(IATMOS) = 0, 1 or 2
 !===============================================================================
@@ -119,7 +90,7 @@ if (ippmod(iatmos).eq.0) then
     jj = iscapp(isc)
 
     if (iscavr(jj).le.0) then
-      visls0(jj) = viscl0
+      call field_set_key_double(ivarfl(isca(jj)), kvisl0, viscl0)
     endif
 
   enddo
@@ -140,10 +111,21 @@ if (ippmod(iatmos).eq.1) then
 
     jj = iscapp(isc)
 
-    call field_get_key_struct_var_cal_opt(ivarfl(isca(jj)), vcopt)
-
     if (iscavr(jj).le.0) then
-      visls0(jj) = viscl0
+      call field_get_key_double(ivarfl(isca(jj)), kvisl0, visls_0)
+      ! If not set elsewhere (user, GUI, ...)
+      if (visls_0.lt.-grand) then
+
+        ! For the temperature, the diffusivity factor is directly the thermal conductivity
+        ! lambda = Cp * mu / Pr
+        ! where Pr is the (molecular) Prandtl number
+        if (itherm .eq. 1.and.jj.eq.iscalt) then
+          visls_0 = viscl0 * cp0
+        else
+          visls_0 = viscl0
+        endif
+        call field_set_key_double(ivarfl(isca(jj)), kvisl0, visls_0)
+      endif
     endif
 
   enddo
@@ -164,10 +146,8 @@ if (ippmod(iatmos).eq.2) then
 
     jj = iscapp(isc)
 
-    call field_get_key_struct_var_cal_opt(ivarfl(isca(jj)), vcopt)
-
     if (iscavr(jj).le.0) then
-      visls0(jj) = viscl0
+      call field_set_key_double(ivarfl(isca(jj)), kvisl0, viscl0)
     endif
 
   enddo
@@ -186,70 +166,36 @@ if (nscal.gt.0) then
 endif
 
 !===============================================================================
-! 6. Force RIJ Matrix stabilisation for all atmospheric models
+! 6. Force Rij Matrix stabilisation for all atmospheric models
 !===============================================================================
 
 if (itytur.eq.3) irijnu = 1
 
+!===============================================================================
+! 7. Some initialization for meteo...
+!===============================================================================
+
+if (ippmod(iatmos).ge.0) then
+
+  call init_meteo
+
+  if (imbrication_flag) then
+    call activate_imbrication
+  endif
+
+  call cs_at_data_assim_build_ops
+
+  if (ifilechemistry.ge.1) then
+    call init_chemistry
+  endif
+
+endif
+
+
 !--------
-! FORMATS
+! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1002 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@                                                            ',/,&
-'@  Seul le modele de turbulence k-eps est disponible avec    ',/,&
-'@   le module atmosphere humide (ippmod(iatmos) = 2).        ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier usipsu (cs_user_parameters.f90)                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 1003 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@                                                            ',/,&
-'@  Les modeles de sol (iatsoil) et de rayonnement (iatra1)   ',/,&
-'@   ne sont disponilbes qu''avec le module atmosphere        ',/,&
-'@   humide (ippomod(iatmos) = 2).                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier usipsu (cs_user_parameters.f90)                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
- 1002 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@  WARNING:   STOP WHILE READING INPUT DATA               ',/,&
-'@    =========                                               ',/,&
-'@                ATMOSPHERIC  MODULE                         ',/,&
-'@                                                            ',/,&
-'@  Only k-eps turbulence model is available with humid       ',/,&
-'@   atmosphere module (ippmod(iatmos) = 2).                  ',/,&
-'@  Computation CAN NOT run.                                  ',/,&
-'@                                                            ',/,&
-'@  Check the input data given through the User Interface     ',/,&
-'@   or in cs_user_parameters.f90.                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
  1003 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -269,8 +215,6 @@ if (itytur.eq.3) irijnu = 1
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
 !----
 ! End
 !----
