@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -25,17 +25,16 @@
 !>
 !> Initialise for example the meteorological field for each cell of
 !> the domain by interpolation of the data from the meteo file
+!> First stage, before GUI.
+!
 !-------------------------------------------------------------------------------
 ! Arguments
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[in]   nvar        total number of variables
-!> \param[in]   nscal       total number of scalars
-!> \param[in]   dt          time step value
 !-------------------------------------------------------------------------------
 
-subroutine atiniv  ( nvar, nscal, dt )
+subroutine atiniv0
 
 !===============================================================================
 ! Module files
@@ -53,26 +52,25 @@ use ppincl
 use atincl
 use mesh
 use atchem
-use siream
+use sshaerosol
 use field
+use cs_c_bindings
 
 !===============================================================================
 
 implicit none
 
-integer          nvar   , nscal
-
-double precision dt(ncelet)
-
 ! Local variables
 
 integer          imode, iel
 double precision d2s3
-double precision zent,xuent,xvent,xkent,xeent,tpent,qvent,ncent
+double precision zent,xuent,xvent, xwent, xkent,xeent,tpent,qvent,ncent
 
 integer k,ii, isc
 double precision xcent
 
+type(var_cal_opt) :: vcopt_p, vcopt_u
+
 double precision, dimension(:,:), pointer :: vel
 double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_phi
 double precision, dimension(:), pointer :: cvar_fb, cvar_omg, cvar_nusa
@@ -81,12 +79,19 @@ double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
 double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
 double precision, dimension(:), pointer :: cvar_despgi, cvar_sc
 double precision, dimension(:), pointer :: cvar_scalt, cvar_totwt, cvar_ntdrp
+double precision, dimension(:,:), pointer :: cpro_met_vel
+double precision, dimension(:), pointer :: cpro_met_potemp
+double precision, dimension(:), pointer :: cpro_met_qv, cpro_met_nc
+double precision, dimension(:), pointer :: cpro_met_k, cpro_met_eps
 
 !===============================================================================
 
 ! Map field arrays
 call field_get_val_v(ivarfl(iu), vel)
 
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
+
 !===============================================================================
 ! 1.  INITIALISATION VARIABLES LOCALES
 !===============================================================================
@@ -120,17 +125,29 @@ elseif (iturb.eq.70) then
   call field_get_val_s(ivarfl(inusa), cvar_nusa)
 endif
 
+if (imeteo.ge.2) then
+  call field_get_val_s_by_name('meteo_pot_temperature', cpro_met_potemp)
+  call field_get_val_v_by_name('meteo_velocity', cpro_met_vel)
+  call field_get_val_s_by_name('meteo_tke', cpro_met_k)
+  call field_get_val_s_by_name('meteo_eps', cpro_met_eps)
+  if (ippmod(iatmos).eq.2) then
+    call field_get_val_s_by_name('meteo_humidity', cpro_met_qv)
+    call field_get_val_s_by_name('meteo_drop_nb', cpro_met_nc)
+  endif
+endif
+
 !===============================================================================
 ! 2. READING THE METEO PROFILE FILE (IF IMETEO = 1 DEFAULT OPTION):
 !===============================================================================
 
-if (imeteo.gt.0) then
-
+! Meteo file
+if (imeteo.eq.1) then
   imode = 1
-  call atlecm &
-  !==========
-  ( imode )
+  call atlecm(imode)
 
+! Recomputed from cs_glog_atmo_option values
+else if (imeteo.eq.2) then
+  call cs_atmo_compute_meteo_profiles()
 endif
 
 if (iatra1.gt.0) then
@@ -151,24 +168,16 @@ if (ifilechemistry.ge.1) then
   !==========
   ( imode)
 
-  ! Computation of the conversion factor matrix used for
-  ! the reaction rates jaccobian matrix
-  do ii = 1, nespg
-    do k = 1, nespg
-      conv_factor_jac((chempoint(k)-1)*nespg+chempoint(ii)) = dmmk(ii)/dmmk(k)
-    enddo
-  enddo
-
   ! Volume initilization with profiles for species present
   ! in the chemical profiles file
-  if (isuite.ne.0.and.init_at_chem.eq.1) then
+  if (isuite.eq.0 .or. (isuite.ne.0.and.init_at_chem.eq.1)) then
+
     do k = 1, nespgi
       call field_get_val_s(ivarfl(isca(isca_chem(idespgi(k)))), cvar_despgi)
 
       do iel = 1, ncel
         zent = xyzcen(3,iel)
         call intprf                                                         &
-        !==========
         (nbchmz, nbchim,                                                    &
          zproc, tchem, espnum(1+(k-1)*nbchim*nbchmz), zent  , ttcabs, xcent )
         ! The first nespg user scalars are supposed to be chemical species
@@ -177,18 +186,39 @@ if (ifilechemistry.ge.1) then
 
     enddo
   endif
+
+endif
+
+! Atmospheric gaseous chemistry
+if (ichemistry.ge.1) then
+
+  ! Computation of the conversion factor matrix used for
+  ! the reaction rates jaccobian matrix
+  do ii = 1, nespg
+    do k = 1, nespg
+      conv_factor_jac((chempoint(k)-1)*nespg+chempoint(ii)) = dmmk(ii)/dmmk(k)
+    enddo
+  enddo
+
 endif
 
 ! Atmospheric aerosol chemistry
-if (iaerosol.eq.1) then
+if (iaerosol.ne.CS_ATMO_AEROSOL_OFF) then
 
-  ! Reading intial concentrations and numbers
+  ! Reading intial concentrations and numbers from file
+  !   or from the aerosol library
   call atleca()
 
   ! Initialization
-  if (init_at_chem.eq.1) then
-    do ii = 1, nesp_aer*nbin_aer + nbin_aer
-      isc = (isca_chem(1) - 1) + nespg_siream + ii
+  if (isuite.eq.0 .or. init_at_chem.eq.1) then
+
+    ! Writing
+    if (vcopt_u%iwarni.ge.1.or.vcopt_p%iwarni.ge.1) then
+      write(nfecra,2001)
+    endif
+
+    do ii = 1, nlayer_aer*n_aer + n_aer
+      isc = isca_chem(nespg + ii)
       call field_get_val_s(ivarfl(isca(isc)), cvar_sc)
       do iel = 1, ncel
         cvar_sc(iel) = dlconc0(ii)
@@ -196,22 +226,43 @@ if (iaerosol.eq.1) then
     enddo
   endif
 
+  ! Do not free memory allocated in atleca, dlconc0 is used in attycl
+
 endif
 
-! Verifications
-if ((iatra1.eq.1.or.ichemistry.ge.1).and.(syear.eq.-999.or.squant.eq.-999.or.shour.eq.-999&
-.or.smin.eq.-999.or.ssec.eq.-999)) then
+! Check simulation times used by atmo
+! radiative transfer or chemistry models
+if (     (iatra1.eq.1.or.ifilechemistry.ge.1)              &
+    .and.(syear.eq.-1.or.squant.eq.-1.or.shour.eq.-1 &
+          .or.smin.eq.-1.or.ssec.le.-1.d0)) then
   if (iatra1.eq.1) write(nfecra,1000)
-  if (ichemistry.ge.1) write(nfecra,1001)
+  if (ifilechemistry.ge.1) write(nfecra,1001)
   call csexit (1)
 endif
 
-if ((iatra1.eq.1.or.ichemistry.ge.1).and.(xlat.ge.rinfin*0.5.or.xlon.ge.rinfin*0.5)) then
-  if (iatra1.eq.1) write(nfecra,1002)
-  if (ichemistry.ge.1) write(nfecra,1003)
+! Check radiative module latitude / longitude
+if (iatra1.eq.1 .and. (xlat.ge.rinfin*0.5 .or. xlon.ge.rinfin*0.5)) then
+  write(nfecra,1002)
   call csexit (1)
 endif
 
+! Check latitude / longitude from meteo file
+if (imeteo.eq.1) then
+  if (maxval(xmet).ge.rinfin*0.5 .or. maxval(ymet).ge.rinfin*0.5) then
+    write(nfecra,1003)
+    call csexit (1)
+  endif
+endif
+
+! Check latitude / longitude from chemistry file
+if (ifilechemistry.ge.1) then
+  if (maxval(xchem).ge.rinfin*0.5 .or. maxval(ychem).ge.rinfin*0.5) then
+    write(nfecra,1004)
+    call csexit (1)
+  endif
+endif
+
+
 !===============================================================================
 ! 3. Dry atmosphere: default initialization of potential temperature
 !===============================================================================
@@ -225,7 +276,7 @@ if (isuite.eq.0) then
       call field_get_val_s(ivarfl(isca(iscalt)), cvar_scalt)
     else if (ippmod(iatmos).eq.2) then
       call field_get_val_s(ivarfl(isca(iscalt)), cvar_scalt)
-      call field_get_val_s(ivarfl(isca(itotwt)), cvar_totwt)
+      call field_get_val_s(ivarfl(isca(iymw)), cvar_totwt)
       call field_get_val_s(ivarfl(isca(intdrp)), cvar_ntdrp)
     endif
 
@@ -250,35 +301,47 @@ if (isuite.eq.0) then
     ! Only if meteo file is present:
     else
 
+      ! Writing
+      if (vcopt_u%iwarni.ge.1.or.vcopt_p%iwarni.ge.1) then
+        write(nfecra,2000)
+      endif
+
       do iel = 1, ncel
 
         zent = xyzcen(3,iel)
 
-        call intprf                                                   &
-        !==========
-       (nbmetd, nbmetm,                                               &
-        zdmet, tmmet, umet , zent  , ttcabs, xuent )
+        ! Meteo file
+        if (imeteo.eq.1) then
+          call intprf &
+            (nbmetd, nbmetm,                                               &
+            zdmet, tmmet, umet , zent  , ttcabs, xuent )
 
-        call intprf                                                   &
-        !==========
-       (nbmetd, nbmetm,                                               &
-        zdmet, tmmet, vmet , zent  , ttcabs, xvent )
+          call intprf &
+            (nbmetd, nbmetm,                                               &
+            zdmet, tmmet, vmet , zent  , ttcabs, xvent )
 
-        call intprf                                                   &
-        !==========
-       (nbmetd, nbmetm,                                               &
-        zdmet, tmmet, ekmet, zent  , ttcabs, xkent )
+          call intprf &
+            (nbmetd, nbmetm,                                               &
+            zdmet, tmmet, ekmet, zent  , ttcabs, xkent )
 
-        call intprf                                                   &
-        !==========
-       (nbmetd, nbmetm,                                               &
-        zdmet, tmmet, epmet, zent  , ttcabs, xeent )
+          call intprf &
+            (nbmetd, nbmetm,                                               &
+            zdmet, tmmet, epmet, zent  , ttcabs, xeent )
+
+          xwent = 0.d0
+        else
+          xuent = cpro_met_vel(1, iel)
+          xvent = cpro_met_vel(2, iel)
+          xwent = cpro_met_vel(3, iel)
+          xkent = cpro_met_k(iel)
+          xeent = cpro_met_eps(iel)
+        endif
 
         vel(1,iel) = xuent
         vel(2,iel) = xvent
-        vel(3,iel) = 0.d0
+        vel(3,iel) = xwent
 
-    !     ITYTUR est un indicateur qui vaut ITURB/10
+        ! ITYTUR est un indicateur qui vaut ITURB/10
         if    (itytur.eq.2) then
 
           cvar_k(iel)  = xkent
@@ -321,36 +384,42 @@ if (isuite.eq.0) then
 
         endif
 
-
         if (ippmod(iatmos).eq.1) then
-          ! The thermal scalar is potential temperature
-            call intprf                                                 &
-            !==========
-         (nbmett, nbmetm,                                               &
-          ztmet, tmmet, tpmet, zent  , ttcabs, tpent )
+          if (imeteo.eq.1) then
+            ! The thermal scalar is potential temperature
+            call intprf &
+              (nbmett, nbmetm,                                               &
+              ztmet, tmmet, tpmet, zent  , ttcabs, tpent )
+          else
+            tpent = cpro_met_potemp(iel)
+          endif
 
-            cvar_scalt(iel) = tpent
+          cvar_scalt(iel) = tpent
         endif
 
         if (ippmod(iatmos).eq.2) then
-          ! The thermal scalar is liquid potential temperature
-            call intprf                                                 &
-            !==========
-         (nbmett, nbmetm,                                               &
-          ztmet, tmmet, tpmet, zent  , ttcabs, tpent )
-            cvar_scalt(iel) = tpent
 
-            call intprf                                                 &
-            !==========
-         (nbmett, nbmetm,                                               &
-          ztmet, tmmet, qvmet, zent  , ttcabs, qvent )
-            cvar_totwt(iel) = qvent
+          if (imeteo.eq.1) then
+            ! The thermal scalar is liquid potential temperature
+            call intprf &
+              (nbmett, nbmetm,                                               &
+              ztmet, tmmet, tpmet, zent  , ttcabs, tpent )
 
-            call intprf                                                 &
-            !==========
-         (nbmett, nbmetm,                                               &
-          ztmet, tmmet, ncmet, zent  , ttcabs, ncent )
-            cvar_ntdrp(iel) = ncent
+            call intprf &
+              (nbmett, nbmetm,                                               &
+              ztmet, tmmet, qvmet, zent  , ttcabs, qvent )
+
+            call intprf &
+              (nbmett, nbmetm,                                               &
+              ztmet, tmmet, ncmet, zent  , ttcabs, ncent )
+          else
+            tpent = cpro_met_potemp(iel)
+            qvent = cpro_met_qv(iel)
+            ncent = cpro_met_nc(iel)
+          endif
+          cvar_scalt(iel) = tpent
+          cvar_totwt(iel) = qvent
+          cvar_ntdrp(iel) = ncent
         endif
 
       enddo
@@ -360,94 +429,17 @@ if (isuite.eq.0) then
 
 endif
 
-!===============================================================================
-! 4. USER  OPTIONS
-!===============================================================================
+!--------
+! Formats
+!--------
 
-call cs_user_f_initialization &
-!==========================
-( nvar   , nscal  ,                                            &
-  dt     )
+ 2000 format(/,                                                   &
+'   ** INIT DYNAMIC VARIABLES FROM METEO FILE'                ,/,&
+'      --------------------------------------'                ,/)
 
-!----
-! FORMATS
-!----
-
-
-#if defined(_CS_LANG_FR)
-
- 1000 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@    MODELE DE RAYONNEMENT (IATRA1) DEMANDE                  ',/,&
-'@                                                            ',/,&
-'@    Le temps de la simulation est mal defini                ',/,&
-'@    Revoir les variables syear, squant, shour, smin, ssec   ',/,&
-'@                                                            ',/,&
-'@    Par priorite decroissante ces variables peuvent        ',/,&
-'@    etre definies dans cs_user_parameters.f90 ou le fichier ',/,&
-'@    meteo ou le fichier chimie eventuel                     ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
- 1001 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@    MODULE DE CHIMIE (ICHEMISTRY) DEMANDE                   ',/,&
-'@                                                            ',/,&
-'@    Le temps de la simulation est mal defini                ',/,&
-'@    Revoir les variables syear, squant, shour, smin, ssec   ',/,&
-'@                                                            ',/,&
-'@    Par priorite decroissante ces variables peuvent         ',/,&
-'@    etre definies dans cs_user_parameters.f90 ou le fichier ',/,&
-'@    meteo ou le fichier chimie                             ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
- 1002 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@    MODELE DE RAYONNEMENT (IATRA1) DEMANDE                  ',/,&
-'@                                                            ',/,&
-'@    Les coordonnees xlat et xlon du domaine sont mal definies',/,&
-'@                                                            ',/,&
-'@    Voir cs_user_parameters.f90                             ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
- 1003 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@    MODULE DE CHIMIE (ICHEMISTRY) DEMANDE                   ',/,&
-'@                                                            ',/,&
-'@    Les coordonnees xlat et xlon du domaine sont mal definies',/,&
-'@                                                            ',/,&
-'@    Voir cs_user_parameters.f90                             ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-
-#else
+ 2001 format(/,                                                   &
+'   ** INIT ATMO CHEMISTRY VARIABLE FROM FILE'                ,/,&
+'      --------------------------------------'                ,/)
 
  1000 format(                                                     &
 '@                                                            ',/,&
@@ -461,7 +453,7 @@ call cs_user_f_initialization &
 '@    The simulation time is wrong                            ',/,&
 '@    Check variables syear, squant, shour, smin, ssec        ',/,&
 '@                                                            ',/,&
-'@    By decreasing priority these variablse can be defined   ',/,&
+'@    By decreasing priority, these variables can be defined  ',/,&
 '@    in cs_user_parameters or the meteo file                 ',/,&
 '@    or the chemistry file                                   ',/,&
 '@                                                            ',/,&
@@ -479,7 +471,7 @@ call cs_user_f_initialization &
 '@    The simulation time is wrong                            ',/,&
 '@    Check variables syear, squant, shour, smin, ssec        ',/,&
 '@                                                            ',/,&
-'@    By decreasing priority these variablse can be defined   ',/,&
+'@    By decreasing priority, these variables can be defined  ',/,&
 '@    in cs_user_parameters or the meteo file                 ',/,&
 '@    or the chemistry file                                   ',/,&
 '@                                                            ',/,&
@@ -495,7 +487,7 @@ call cs_user_f_initialization &
 '@                ATMOSPHERIC  MODULE                         ',/,&
 '@                RADITIVE MODEL (IATRA1)                     ',/,&
 '@                                                            ',/,&
-'@    Wrong xlat and xlon coordinates                         ',/,&
+'@    Wrong xlat and xlon coordinates.                        ',/,&
 '@                                                            ',/,&
 '@    See cs_user_parameters.f90                              ',/,&
 '@                                                            ',/,&
@@ -508,21 +500,276 @@ call cs_user_f_initialization &
 '@                                                            ',/,&
 '@ @@  WARNING:   STOP WHILE READING INPUT DATA               ',/,&
 '@    =========                                               ',/,&
-'@      ATMOSPHERIC CHEMISTRY                                 ',/,&
+'@      ATMOSPHERIC MODULE                                    ',/,&
 '@                                                            ',/,&
-'@    Wrong xlat and xlon coordinates                         ',/,&
+'@    Wrong coordinates xmet, ymet for the meteo profile.     ',/,&
 '@                                                            ',/,&
 '@    See cs_user_parameters.f90                              ',/,&
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
+ 1004 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@  WARNING:   STOP WHILE READING INPUT DATA               ',/,&
+'@    =========                                               ',/,&
+'@      ATMOSPHERIC CHEMISTRY                                 ',/,&
+'@                                                            ',/,&
+'@    Wrong xchem, ychem coordinates for the concentration    ',/,&
+'@    profiles (chemistry model).                             ',/,&
+'@                                                            ',/,&
+'@    See cs_user_parameters.f90                              ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
 
 !----
-! FIN
+! End
 !----
 
 return
 
-end subroutine atiniv
+end subroutine atiniv0
+
+!-------------------------------------------------------------------------------
+!> \file atiniv.f90
+!> \brief Initialisation of calculation variables for the atmospheric module,
+!> it is the counterpart of usiniv.f90.
+!>
+!> Initialise for example the meteorological field for each cell of
+!> the domain by interpolation of the data from the meteo file
+!> Second stage.
+!
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!-------------------------------------------------------------------------------
+
+subroutine atiniv1
+
+!===============================================================================
+! Module files
+!===============================================================================
+
+use paramx
+use numvar
+use optcal
+use cstphy
+use cstnum
+use entsor
+use ppppar
+use ppthch
+use ppincl
+use atincl
+use mesh
+use atchem
+use sshaerosol
+use field
+use cs_c_bindings
+
+!===============================================================================
+
+implicit none
+
+! Local variables
+
+integer          iel
+double precision d2s3
+double precision zent,xuent,xvent, xwent, xkent,xeent,tpent,qvent,ncent
+
+double precision, dimension(:,:), pointer :: vel
+double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_phi
+double precision, dimension(:), pointer :: cvar_fb, cvar_omg, cvar_nusa
+double precision, dimension(:,:), pointer :: cvar_rij
+double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
+double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
+double precision, dimension(:), pointer :: cvar_scalt, cvar_totwt, cvar_ntdrp
+double precision, dimension(:,:), pointer :: cpro_met_vel
+double precision, dimension(:), pointer :: cpro_met_potemp
+double precision, dimension(:), pointer :: cpro_met_qv, cpro_met_nc
+double precision, dimension(:), pointer :: cpro_met_k, cpro_met_eps
+
+!===============================================================================
+
+! Map field arrays
+call field_get_val_v(ivarfl(iu), vel)
+
+!===============================================================================
+! 1.  INITIALISATION VARIABLES LOCALES
+!===============================================================================
+
+d2s3 = 2.d0/3.d0
+
+if (itytur.eq.2) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+elseif (itytur.eq.3) then
+  if (irijco.eq.1) then
+    call field_get_val_v(ivarfl(irij), cvar_rij)
+  else
+    call field_get_val_s(ivarfl(ir11), cvar_r11)
+    call field_get_val_s(ivarfl(ir22), cvar_r22)
+    call field_get_val_s(ivarfl(ir33), cvar_r33)
+    call field_get_val_s(ivarfl(ir12), cvar_r12)
+    call field_get_val_s(ivarfl(ir23), cvar_r23)
+    call field_get_val_s(ivarfl(ir13), cvar_r13)
+  endif
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+elseif (iturb.eq.50) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+  call field_get_val_s(ivarfl(iphi), cvar_phi)
+  call field_get_val_s(ivarfl(ifb), cvar_fb)
+elseif (iturb.eq.60) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iomg), cvar_omg)
+elseif (iturb.eq.70) then
+  call field_get_val_s(ivarfl(inusa), cvar_nusa)
+endif
+
+if (imeteo.ge.2) then
+  call field_get_val_s_by_name('meteo_pot_temperature', cpro_met_potemp)
+  call field_get_val_v_by_name('meteo_velocity', cpro_met_vel)
+  call field_get_val_s_by_name('meteo_tke', cpro_met_k)
+  call field_get_val_s_by_name('meteo_eps', cpro_met_eps)
+  if (ippmod(iatmos).eq.2) then
+    call field_get_val_s_by_name('meteo_humidity', cpro_met_qv)
+    call field_get_val_s_by_name('meteo_drop_nb', cpro_met_nc)
+  endif
+endif
+
+!===============================================================================
+! 2. Checks of the reading the meteo profile file (if imeteo = 1 default option)
+!===============================================================================
+
+! Check simulation times used by atmo
+! radiative transfer or chemistry models
+if (     (iatra1.eq.1.or.ifilechemistry.ge.1)              &
+    .and.(syear.eq.-1.or.squant.eq.-1.or.shour.eq.-1 &
+          .or.smin.eq.-1.or.ssec.le.-1.d0)) then
+  if (iatra1.eq.1) write(nfecra,1000)
+  if (ifilechemistry.ge.1) write(nfecra,1001)
+  call csexit (1)
+endif
+
+! Check radiative module latitude / longitude
+if (iatra1.eq.1 .and. (xlat.ge.rinfin*0.5 .or. xlon.ge.rinfin*0.5)) then
+  write(nfecra,1002)
+  call csexit (1)
+endif
+
+! Check latitude / longitude from meteo file
+if (imeteo.eq.1) then
+  if (maxval(xmet).ge.rinfin*0.5 .or. maxval(ymet).ge.rinfin*0.5) then
+    write(nfecra,1003)
+    call csexit (1)
+  endif
+endif
+
+! Check latitude / longitude from chemistry file
+if (ifilechemistry.ge.1) then
+  if (maxval(xchem).ge.rinfin*0.5 .or. maxval(ychem).ge.rinfin*0.5) then
+    write(nfecra,1004)
+    call csexit (1)
+  endif
+endif
+
+!--------
+! Formats
+!--------
+
+ 1000 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@  WARNING:   STOP WHILE READING INPUT DATA               ',/,&
+'@    =========                                               ',/,&
+'@                ATMOSPHERIC  MODULE                         ',/,&
+'@                RADITIVE MODEL (IATRA1)                     ',/,&
+'@                                                            ',/,&
+'@    The simulation time is wrong                            ',/,&
+'@    Check variables syear, squant, shour, smin, ssec        ',/,&
+'@                                                            ',/,&
+'@    By decreasing priority, these variables can be defined  ',/,&
+'@    in cs_user_parameters or the meteo file                 ',/,&
+'@    or the chemistry file                                   ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+
+ 1001 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@  WARNING:   STOP WHILE READING INPUT DATA               ',/,&
+'@    =========                                               ',/,&
+'@      ATMOSPHERIC CHEMISTRY                                 ',/,&
+'@                                                            ',/,&
+'@    The simulation time is wrong                            ',/,&
+'@    Check variables syear, squant, shour, smin, ssec        ',/,&
+'@                                                            ',/,&
+'@    By decreasing priority, these variables can be defined  ',/,&
+'@    in cs_user_parameters or the meteo file                 ',/,&
+'@    or the chemistry file                                   ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+
+ 1002 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@  WARNING:   STOP WHILE READING INPUT DATA               ',/,&
+'@    =========                                               ',/,&
+'@                ATMOSPHERIC  MODULE                         ',/,&
+'@                RADITIVE MODEL (IATRA1)                     ',/,&
+'@                                                            ',/,&
+'@    Wrong xlat and xlon coordinates.                        ',/,&
+'@                                                            ',/,&
+'@    See cs_user_parameters.f90                              ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+
+ 1003 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@  WARNING:   STOP WHILE READING INPUT DATA               ',/,&
+'@    =========                                               ',/,&
+'@      ATMOSPHERIC MODULE                                    ',/,&
+'@                                                            ',/,&
+'@    Wrong coordinates xmet, ymet for the meteo profile.     ',/,&
+'@                                                            ',/,&
+'@    See cs_user_parameters.f90                              ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+
+ 1004 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@  WARNING:   STOP WHILE READING INPUT DATA               ',/,&
+'@    =========                                               ',/,&
+'@      ATMOSPHERIC CHEMISTRY                                 ',/,&
+'@                                                            ',/,&
+'@    Wrong xchem, ychem coordinates for the concentration    ',/,&
+'@    profiles (chemistry model).                             ',/,&
+'@                                                            ',/,&
+'@    See cs_user_parameters.f90                              ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+
+!----
+! End
+!----
+
+return
+
+end subroutine atiniv1
