@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -57,6 +57,7 @@ use ppppar
 use atincl
 use numvar
 use atchem
+use sshaerosol, only : init_gas_with_lib, sshaerosol_get_gas
 
 implicit none
 
@@ -68,18 +69,19 @@ integer           imode
 
 ! Local variables
 
+logical, save :: switch_to_labels
 integer f_id
 integer itp, ii, ios, k
 integer sjday,minute
 double precision second
 integer year, month, quant, hour, day, jday
-character(len=80) :: ccomnt, label, oneline
+character(len=80) :: ccomnt, label, oneline, fname
 character(len=1) :: csaute
 
 ! altitudes and concentrations of every nespgi species
 double precision  zconctemp(nespgi+1)
-! names of species defined in the file in case of a user defined chemical scheme
-character(len=80), allocatable, dimension(:) ::      namespg
+! names of species in the initialization file
+character(len=80), dimension(:), allocatable :: labels
 
 !===============================================================================
 
@@ -148,7 +150,7 @@ else
     call csexit (1)
   endif
 
-  ! --> if the date and time are not completed in usati1.f90 nor and if no meteo
+  ! --> if the date and time are not completed in usppmo/cs_user_model if no meteo
   ! --> file is given,
   ! --> the date and time of the first concentration profile are taken as the
   !     starting time of the simulation
@@ -187,7 +189,6 @@ else
     if(tchem(itp).lt.tchem(itp-1)) then
       write(nfecra,8000)
       call csexit (1)
-      !==========
     endif
   endif
 endif
@@ -207,150 +208,172 @@ else
   read(impmec,*,err=999,end=999) xchem(itp), ychem(itp)
 endif
 
-!===============================================================================
-! 4. reading the number of reactions
-!===============================================================================
-if (ifilechemistry.eq.4) then
+if (init_gas_with_lib) then
 
- 103  read (impmec,'(A80)',err=999,end=999) ccomnt
+  !===============================================================================
+  ! 4a. set the number of species initialized
+  !===============================================================================
 
-if(ccomnt(1:1).eq.csaute) go to 103
-backspace(impmec)
+  if (imode.eq.0) then
+    nespgi = nespg
+  endif
+
+  !===============================================================================
+  ! 5a. read the values of the species initialized
+  !===============================================================================
+
+  if (imode.ne.0) then
+    do ii = 1, nespgi
+      idespgi(ii) = ii
+    enddo
+  endif
+
+  !===============================================================================
+  ! 6a. read the concentrations
+  !===============================================================================
+
+  if (imode.eq.0) then
+
+    nbchmz = 1
+    nbchim = 1
+    return
+
+  else
+
+    zproc(1) = 0.
+
+    call sshaerosol_get_gas(espnum)
+
+    ! Conversion from microg / m^3 to ppm
+    do ii = 1, nespgi
+      espnum(ii) = espnum(ii) / (1.d-3 * ro0)
+    enddo
+
+  endif
 
-if (imode.eq.0) then
-  read(impmec,*,err=999,end=999) nrg
 else
-  read(impmec,*,err=999,end=999)
-endif
 
-!===============================================================================
-! 4. reading the number of species
-!===============================================================================
-
- 104  read (impmec,'(A80)',err=999,end=999) ccomnt
-
-if(ccomnt(1:1).eq.csaute) go to 104
-backspace(impmec)
-
-if (imode.eq.0) then
-  read(impmec,*,err=999,end=999) nespg
-else
-  read(impmec,*,err=999,end=999)
-endif
-
-!===============================================================================
-! 5. reading the names of the species
-!===============================================================================
-
- 105  read(impmec,'(A80)',err=999,end=999) ccomnt
-
-if (ccomnt(1:1).eq.csaute) go to 105
-backspace(impmec)
-
-allocate(namespg(nespg))
-read(impmec,*,err=999,end=999) namespg(1:nespg)
-do k = 1, nespg
-  f_id = ivarfl(isca(isca_chem(k)))
-  call field_set_key_str(f_id, keylbl, namespg(k))
-enddo
-deallocate (namespg)
-
-!===============================================================================
-! 6. reading the molar masses of the species
-!===============================================================================
-
- 106  read (impmec,'(a80)',err=999,end=999) ccomnt
-
-if(ccomnt(1:1).eq.csaute) go to 106
-backspace(impmec)
-
-if (imode.eq.0) then
-  read(impmec,*,err=999,end=999)
-else
-  ! Molar masses must be given in the same order as species
-  read(impmec,*,err=999,end=999) dmmk(1:nespg)
-endif
-
-endif ! End test on ifilechemistry
-!===============================================================================
-! 7. reading the number of species initialized by the chemistry file
-!===============================================================================
+  !===============================================================================
+  ! 4b. reading the number of species initialized by the chemistry file
+  !===============================================================================
 
  107  read (impmec,'(a80)',err=999,end=999) ccomnt
 
-if (ccomnt(1:1).eq.csaute) go to 107
-backspace(impmec)
+  if (ccomnt(1:1).eq.csaute) go to 107
+  backspace(impmec)
 
 
-if (imode.eq.0) then
-  read (impmec,*,err=999,end=999) nespgi
-  if (nespgi.gt.size(isca_chem)) then
-    write(nfecra,8002) size(isca_chem), nespgi
-    call csexit (1)
-    !==========
+  if (imode.eq.0) then
+    read (impmec,*,err=999,end=999) nespgi
+    ! If nespgi < 0, we will read labels
+    ! If nepsgi > 0, we will read scalar ids (default)
+    if (nespgi < 0) then
+      nespgi = - nespgi
+      switch_to_labels = .true.
+    else
+     switch_to_labels = .false.
+    endif
+    if (nespgi.gt.size(isca_chem)) then
+      write(nfecra,8002) size(isca_chem), nespgi
+      call csexit (1)
+      !==========
+    endif
+  else
+    read (impmec,*,err=999,end=999)
   endif
-else
-  read (impmec,*,err=999,end=999)
-endif
 
-!===============================================================================
-! 8. reading the number of species initialized by the chemistry file
-!===============================================================================
-if (nespgi.ge.1) then
+  if (nespgi.ge.1) then
+
+    !===============================================================================
+    ! 5b. reading the species initialized by the chemistry file, only if nespgi > 1
+    !===============================================================================
 
  108  read(impmec,'(a80)',err=999,end=999) ccomnt
 
-  if(ccomnt(1:1).eq.csaute) go to 108
-  backspace(impmec)
+    if(ccomnt(1:1).eq.csaute) go to 108
+    backspace(impmec)
 
+    if (imode.eq.0) then
+      read(impmec,*,err=999,end=999)
+    else
+      if (switch_to_labels) then
+        allocate(labels(nespgi))
+        read(impmec,*,err=999,end=999) labels
 
-  if (imode.eq.0) then
-    read(impmec,*,err=999,end=999)
-  else
-    read(impmec,*,err=999,end=999) idespgi(1:nespgi)
-  endif
+        do ii = 1, nespgi
 
-!===============================================================================
-! 9. reading the concentration profiles
-!===============================================================================
+          ! Initialize idespgi
+          idespgi(ii) = -1
+
+          ! Find the field matching the given label
+          ! Update idespgi and break innermost for loop
+          do k = 1, nespg
+            call field_get_label(ivarfl(isca(isca_chem(k))),fname)
+            if (trim(labels(ii)).eq.fname) then
+              idespgi(ii) = k
+              exit
+            endif
+          enddo
+
+          ! Verification
+          if (idespgi(ii).lt.1 .or. idespgi(ii).gt.size(isca_chem)) then
+            write(nfecra,8003) labels(ii)
+            call csexit (1)
+            !==========
+          endif
+
+        enddo
+
+        deallocate(labels)
+
+      else
+        read(impmec,*,err=999,end=999) idespgi(1:nespgi)
+      endif
+    endif
+
+    !===============================================================================
+    ! 6b. reading the concentration profiles, only if nespgi >= 1
+    !===============================================================================
 
 109 read(impmec,'(a80)',err=999,end=999) ccomnt
 
-  if(ccomnt(1:1).eq.csaute) go to 109
-  backspace(impmec)
+    if(ccomnt(1:1).eq.csaute) go to 109
+    backspace(impmec)
 
-  if (imode.eq.0) then
-    read(impmec,*,err=999,end=999) nbchmz
-    if(nbchmz.le.1) then
-      write(nfecra,8001)
-      call csexit (1)
-    endif
+    if (imode.eq.0) then
+      read(impmec,*,err=999,end=999) nbchmz
+      if(nbchmz.le.1) then
+        write(nfecra,8001)
+        call csexit (1)
+      endif
 
-    do ii = 1, nbchmz
-      read (impmec,*,err=999,end=999)
-    enddo
-
-  else
-
-    read(impmec,*,err=999,end=999)
-
-    do ii = 1, nbchmz
-
-      !     Altitudes and concentrations of every nespgi scpecies
-      read (impmec,*,err=999,end=999) zconctemp
-
-      zproc(ii) = zconctemp(1)
-      do k = 2, nespgi+1
-        espnum(ii+(itp-1)*nbchmz+(k-2)*nbchmz*nbchim) = zconctemp(k)
+      do ii = 1, nbchmz
+        read (impmec,*,err=999,end=999)
       enddo
 
-    enddo
-  endif
+    else
 
-endif ! fin test nespgi
+      read(impmec,*,err=999,end=999)
+
+      do ii = 1, nbchmz
+
+        !     Altitudes and concentrations of every nespgi scpecies
+        read (impmec,*,err=999,end=999) zconctemp
+
+        zproc(ii) = zconctemp(1)
+        do k = 2, nespgi+1
+          espnum(ii+(itp-1)*nbchmz+(k-2)*nbchmz*nbchim) = zconctemp(k)
+        enddo
+
+      enddo
+    endif
+
+  endif ! fin test nespgi
+
+endif ! fin test init_gas_with_lib
 
 !===============================================================================
-! 10. logging
+! 7. logging
 !===============================================================================
 
 if (imode.eq.1) then
@@ -367,7 +390,7 @@ if (imode.eq.1) then
 7996 format(1x, f10.2)
   write(nfecra, '(a)', advance='no') 'zproc, '
   do ii = 1, nespgi
-    f_id = ivarfl(isca(isca_chem(ii)))
+    f_id = ivarfl(isca(isca_chem(idespgi(ii))))
     call field_get_label(f_id, label)
     if (ii .lt. nespgi) then
       write(nfecra, '(a)', advance='no') trim(label)//', '
@@ -404,139 +427,18 @@ return
 !============================
 
  99   continue
-write ( nfecra,9998 )
-call csexit (1)
-!==========
+write(nfecra, 9998)
+
+return
 
  999  continue
-write ( nfecra,9999 )
+write(nfecra, 9999)
 call csexit (1)
-!==========
 
 !--------
 ! FORMATS
 !--------
 
-#if defined(_CS_LANG_FR)
- 8000 format (                                                          &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecc)      ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@    MODULE DE CHIMIE (ICHEMISTRY) DEMANDE                   ',/,&
-'@                                                            ',/,&
-'@              Erreur  dans le fichier chimie :              ',/,&
-'@      ordre chronologique des profils non respecte          ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8001 format (                                                          &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecc)      ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@    MODULE DE CHIMIE (ICHEMISTRY) DEMANDE                   ',/,&
-'@                                                            ',/,&
-'@              Erreur  dans le fichier chimie :              ',/,&
-'@  le nombre de mesures de concentrations doit etre          ',/,&
-'@  superieur a 2                                             ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8002 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecc)      ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@    MODULE DE CHIMIE (ICHEMISTRY) DEMANDE                   ',/,&
-'@                                                            ',/,&
-'@  Le nombre d''especes a initialiser avec le fichier chimie ',/,&
-'@  est superieur au nombre de scalaires chimie               ',/,&
-'@                                                            ',/,&
-'@   Nombre de scalaires chimie       declares : ',I10         ,/,&
-'@   Nombre d''especes chimiques a initialiser : ',I10         ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8005 format (                                                    &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ATTENTION:VERIFICATION A L''ENTREE DES DONNEES (atlecc)  ',/,&
-'@   =========                                                ',/,&
-'@        PHYSIQUE PARTICULIERE ATMOSPHERIQUE                 ',/,&
-'@                                                            ',/,&
-'@  erreur a la lecture de la date du fichier meteo           ',/,&
-'@  verifier le format (entiers, reel)                        ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                             ',/)
- 9998 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecc)      ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@    MODULE DE CHIMIE (ICHEMISTRY) DEMANDE                   ',/,&
-'@                                                            ',/,&
-'@  Erreur a l''ouverture du fichier chimie                   ',/,&
-'@  Verifier le nom du fichier chimie                         ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9999 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecc)      ',/,&
-'@    =========                                               ',/,&
-'@    PHYSIQUE PARTICULIERE (ATMOSPHERIQUE) DEMANDEE          ',/,&
-'@    MODULE DE CHIMIE (ICHEMISTRY) DEMANDE                   ',/,&
-'@                                                            ',/,&
-'@  Erreur a la lecture du fichier chimie.                    ',/,&
-'@    Le fichier a ete ouvert mais est peut etre incomplet    ',/,&
-'@    ou son format inadapte.                                 ',/,&
-'@                                                            ',/,&
-'@    year (integer), month (integer), day (integer),         ',/,&
-'@    hour (integer), minute (integer), second (dble prec)    ',/,&
-'@    of the profile                                          ',/,&
-'@    OU                                                      ',/,&
-'@    year (integer), quantile (integer), hour (integer),     ',/,&
-'@    minute (integer), second (dble prec) of the profile     ',/,&
-'@    location of the concentration profile (x,y) (dble prec) ',/,&
-'@    number of reactions (integer)                           ',/,&
-'@    number of species (integer)                             ',/,&
-'@    names of the species                                    ',/,&
-'@    molar mass of the species                               ',/,&
-'@ concentration profiles:                                    ',/,&
-'@   number of altitudes (integer)                            ',/,&
-'@   alt, concentration for each species                      ',/,&
-'@ NO LINE AT THE END OF THE FILE                             ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
  8000 format (                                                          &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -586,6 +488,22 @@ call csexit (1)
 '@  The computation will not be run                           ',/,&
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+ 8003 format(                                                           &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@ WARNING : STOP WHILE READING INPUT DATA (atlecc)        ',/,&
+'@    =========                                               ',/,&
+'@      ATMOSPHERIC CHEMISTRY FROM SPACK                      ',/,&
+'@                                                            ',/,&
+'@  Could not identify the given species label                ',/,&
+'@                                                            ',/,&
+'@   Given species label : ',A80                               ,/,&
+'@                                                            ',/,&
+'@  The computation will not be run                           ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
  8005 format (                                                    &
 '@                                                            ',/,&
@@ -606,15 +524,14 @@ call csexit (1)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
-'@ @@  WARNING:   STOP WHILE READING INPUT DATA (atlecc)      ',/,&
-'@     =======                                                ',/,&
-'@      ATMOSPHERIC CHEMISTRY                                 ',/,&
-'@                                                            ',/,&
-'@  Error opening the chemistry profile file                  ',/,&
-'@  check the name of the chemistry file                      ',/,&
-'@                                                            ',/,&
-'@  The computation will not be run                           ',/,&
-'@                                                            ',/,&
+'@ @@  WARNING:   ATMOSPHERIC CHEMISTRY (atlecc)'              ,/,&
+'@     ======='                                                ,/,&
+'@'                                                            ,/,&
+'@  No chemistry profile given'                                ,/,&
+'@  check the name of the chemistry file'                      ,/,&
+'@'                                                            ,/,&
+'@  The chemistry variable will be set to 0.'                  ,/,&
+'@'                                                            ,/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
  9999 format(                                                           &
@@ -651,6 +568,4 @@ call csexit (1)
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
 end subroutine atlecc
