@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,8 +26,8 @@
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[in]   imode       IMODE = 0 : READING FOR DIMENSIONS ONLY
-!>                          IMODE = 1 : READING ACTUAL METEO DATA
+!> \param[in]   imode        0: reading for dimensions and starting time only
+!>                           1: reading actual meteo data
 !-------------------------------------------------------------------------------
 subroutine atlecm ( imode )
 
@@ -41,7 +41,10 @@ use entsor
 use cstnum
 use cstphy
 use ppppar
+use optcal
 use atincl
+use ctincl, only: cp_a, cp_v
+use cs_c_bindings
 
 !===============================================================================
 
@@ -53,10 +56,11 @@ integer           imode
 
 ! a function used in the routine
 ! for the diagnostic of liquid water content
-double precision qsatliq
 
 ! Local variables
 
+character(len=80) :: ficmet
+
 integer itp, ii, ios, k
 integer year, quant,hour,minute, month, day
 integer ih2o
@@ -66,6 +70,7 @@ double precision sjday, jday
 double precision rap,rscp,tmoy, rhmoy
 double precision ztop, zzmax, tlkelv, pptop, dum
 double precision rhum,q0,q1
+double precision cpvcpa
 
 character(len=80) :: ccomnt,oneline
 character(len=1)  :: csaute
@@ -82,9 +87,11 @@ endif
 ! 0. Initialization
 !===============================================================================
 
-CSAUTE = '/'
+csaute = '/'
 
 ! --> Opens the meteo file
+call atmo_get_meteo_file_name(ficmet)
+
 open (unit=impmet, file=ficmet,                                  &
      status='old', form='formatted', access='sequential',       &
      iostat=ios, err=99)
@@ -93,6 +100,8 @@ rewind(unit=impmet, err=99)
 itp=0
 ih2o = 0
 
+cpvcpa = cp_v / cp_a
+
 if (imode.eq.1) then
   rscp=rair/cp0
 
@@ -125,41 +134,39 @@ backspace(impmet)
 ! --> year, quant-day, hour, minute, second  of the profile (UTC)
 ! NB: second is real, all others are integers
 
-if (imode.eq.0) then
-  read(impmet, *, err=999, end=906)
-else
-  second = -9999.d0
-  read(impmet, '(a80)', err=999, end=906) oneline
-  read(oneline, *, err=907, end=907) year, month, day,  &
-                                                  hour, minute, second
+second = -9999.d0
+read(impmet, '(a80)', err=999, end=906) oneline
+read(oneline, *, err=907, end=907) year, month, day,  &
+                                                hour, minute, second
 ! --> catch some read errors
-  if (month.gt.12.or.day.gt.31) then
-    write(nfecra,8005)
-    call csexit (1)
-  endif
-  call comp_quantile(day, month, year, quant)
-  goto 908
+if (month.gt.12.or.day.gt.31) then
+  write(nfecra,8005)
+  call csexit (1)
+endif
+call comp_quantile(day, month, year, quant)
+goto 908
 907  continue
-  read(oneline, *, err=999, end=906) year, quant, hour, minute, second
+read(oneline, *, err=999, end=906) year, quant, hour, minute, second
 908 continue
 ! --> catch some read errors
-  if (second.lt.0d0.or.quant.gt.366) then
-    write(nfecra,8005)
-    call csexit (1)
-  endif
+if (second.lt.0d0.or.quant.gt.366) then
+  write(nfecra,8005)
+  call csexit (1)
+endif
 
-  ! --> if the date and time are not completed in usati1.f90,
-  !     the date and time of the first meteo profile are taken as the
-  !     starting time of the simulation
+! --> if the date and time are not completed in usppmo / cs_user_model
+!     the date and time of the first meteo profile are taken as the
+!     starting time of the simulation
 
-  if (syear.lt.0) then
-    syear = year
-    squant = quant
-    shour = hour
-    smin = minute
-    ssec = second
-  endif
+if (syear.lt.0) then
+  syear = year
+  squant = quant
+  shour = hour
+  smin = minute
+  ssec = second
+endif
 
+if (imode.ne.0) then
   !--> Compute the relative time to the starting time of the simulation
 
   ! --> Compute the julian day for the starting day of the simulation
@@ -188,7 +195,6 @@ else
       !==========
     endif
   endif
-
 endif
 
 !===============================================================================
@@ -318,6 +324,19 @@ else
 
 endif
 
+! For Boussinesq approximation, initialize p0, rho0 and theta0 at the first level
+if (imode.eq.1.and.idilat.eq.0.and.itp.eq.1) then
+  p0 = pmer(itp)
+  t0 = (ttmet(1,itp)+tkelvi)*(ps/p0)**rscp ! theta0
+  ! Humid atmosphere
+  if (ippmod(iatmos).eq.2) then
+    rhum = rair*(1.d0+(rvsra-1.d0)*qvmet(1,itp)*ih2o)
+  else
+    rhum = rair
+  endif
+  ro0 = p0 / (ttmet(1,itp) + tkelvi)/rhum
+endif
+
 !===============================================================================
 ! 6. Compute hydro pressure profile  (Laplace integration)
 !===============================================================================
@@ -336,11 +355,11 @@ if (imode.eq.1) then
     do k = 2, nbmaxt
       tmoy = 0.5d0*(ttmet(k-1,itp) + ttmet(k,itp)) + tkelvi
 
-      if(ippmod(iatmos).eq.2) then ! take liquid water into account
-        q0 = min( qvmet(k-1,itp), qsatliq( ttmet(k-1,itp) &
-            + tkelvi, phmet(k-1,itp)))
-        q1 = min( qvmet(k  ,itp), qsatliq( ttmet(k  ,itp) &
-            + tkelvi, phmet(k-1,itp)))
+      if (ippmod(iatmos).eq.2) then ! take liquid water into account
+        q0 = min( qvmet(k-1,itp), cs_air_yw_sat( ttmet(k-1,itp) &
+            , phmet(k-1,itp)))
+        q1 = min( qvmet(k  ,itp), cs_air_yw_sat( ttmet(k  ,itp) &
+            , phmet(k-1,itp)))
         !in q1=.. phmet(k-1,itp) is not a mistake: we can not use phmet(k,itp)
         !since this is what we want to estimate.
       else
@@ -360,8 +379,8 @@ if (imode.eq.1) then
       tmoy = 0.5d0*(ttmet(k+1,itp) + ttmet(k,itp)) + tkelvi
 
       if(ippmod(iatmos).eq.2) then ! take liquid water into account
-        q0 = min( qvmet(k,itp), qsatliq( ttmet(k,itp)+tkelvi, phmet(k+1,itp)))
-        q1 = min( qvmet(k+1  ,itp), qsatliq( ttmet(k+1,itp)+tkelvi, phmet(k+1,itp)))
+        q0 = min( qvmet(k,itp), cs_air_yw_sat( ttmet(k,itp), phmet(k+1,itp)))
+        q1 = min( qvmet(k+1  ,itp), cs_air_yw_sat( ttmet(k+1,itp), phmet(k+1,itp)))
         !in q0=.. phmet(k+1,itp) is not a mistake: we can not use phmet(k,itp)
         !since this is what we want to estimate.
       else
@@ -441,7 +460,7 @@ endif
 ! 9. Printings
 !===============================================================================
 
-if (imode.eq.1) then
+if (imode.eq.1.and.imeteo.eq.1) then
   if (itp.eq.1) then
     write(nfecra, *)
     write(nfecra, *) '==================================================='
@@ -452,7 +471,7 @@ if (imode.eq.1) then
 7995 format(1x, i4, i5, 2i4, f10.2)
   write(nfecra, *) 'tmmet(itp):'
   write(nfecra, 7996) tmmet(itp)
-7996 format(1x, f10.2)
+7996 format(1x, e10.3)
   write(nfecra, *) 'zdmet, umet, vmet, ekmet, epmet:'
   do ii = 1, nbmetd
     write(nfecra, 7997)                                            &
@@ -479,7 +498,7 @@ if (imode.eq.1) then
       write(nfecra,7999)                                            &
            ztmet(ii), ttmet(ii,itp), tpmet(ii,itp),              &
            rmet(ii,itp), phmet(ii,itp), qvmet(ii,itp),           &
-           qsatliq(ttmet(ii,itp)+tkelvi , phmet(ii,itp)),         &
+           cs_air_yw_sat(ttmet(ii,itp) , phmet(ii,itp)),         &
            ncmet(ii,itp)
 7999  format(1x, 3f8.2,f8.4,f12.3,e10.3,e10.3,e12.5)
     enddo
@@ -523,147 +542,6 @@ call csexit (1)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-8000 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecm)      ',/,&
-     '@    =========                                               ',/,&
-     '@      PHYSIQUE PARTICULIERE ATMOSPHERIQUE                   ',/,&
-     '@                                                            ',/,&
-     '@              Erreur  dans le fichier meteo:                ',/,&
-     '@      ordre chronologique des profils non respecte          ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/)
-8001 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecm)      ',/,&
-     '@    =========                                               ',/,&
-     '@      PHYSIQUE PARTICULIERE ATMOSPHERIQUE                   ',/,&
-     '@                                                            ',/,&
-     '@              Erreur  dans le fichier meteo:                ',/,&
-     '@  le nombre de mesures de temperatures doit etre            ',/,&
-     '@  superieur a 2                                             ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/)
-8002 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecm)      ',/,&
-     '@    =========                                               ',/,&
-     '@      PHYSIQUE PARTICULIERE ATMOSPHERIQUE                   ',/,&
-     '@                                                            ',/,&
-     '@              Erreur  dans le fichier meteo:                ',/,&
-     '@  le nombre de mesures de vitesses doit etre                ',/,&
-     '@  superieur a 2                                             ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/)
-8003 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ATTENTION:VERIFICIATION A L''ENTREE DES DONNEES (atlecm)',/,&
-     '@   =========                                               ',/,&
-     '@        PHYSIQUE PARTICULIERE ATMOSPHERIQUE                 ',/,&
-     '@                                                            ',/,&
-     '@     erreur a la lecture du fichier meteo                   ',/,&
-     '@  humidites specifiques irrealistes                         ',/,&
-     '@  verifier les unites (kg/kg)                               ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                             ',/)
-8004 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ATTENTION:VERIFICATION A L''ENTREE DES DONNEES (atlecm)' ,/,&
-     '@   =========                                               ',/,&
-     '@        PHYSIQUE PARTICULIERE ATMOSPHERIQUE                 ',/,&
-     '@                                                            ',/,&
-     '@  erreur a la lecture du fichier meteo                      ',/,&
-     '@  nombre de gouttes lus <0                                  ',/,&
-     '@  verifier le fichier meteo                                 ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                             ',/)
-8005 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ATTENTION:VERIFICATION A L''ENTREE DES DONNEES (atlecm)  ',/,&
-     '@   =========                                                ',/,&
-     '@        PHYSIQUE PARTICULIERE ATMOSPHERIQUE                 ',/,&
-     '@                                                            ',/,&
-     '@  erreur a la lecture de la date du fichier meteo           ',/,&
-     '@  verifier le format (entiers, reel)                        ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                             ',/)
-9998 format(                                                     &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecm)      ',/,&
-     '@    =========                                               ',/,&
-     '@      PHYSIQUE PARTICULIERE ATMOSPHERIQUE                   ',/,&
-     '@                                                            ',/,&
-     '@  Erreur a l''ouverture du fichier meteo                    ',/,&
-     '@  Verifier le nom du fichier meteo                          ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/)
-9999 format(                                                     &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecm)      ',/,&
-     '@    =========                                               ',/,&
-     '@      PHYSIQUE PARTICULIERE ATMOSPHERIQUE                   ',/,&
-     '@                                                            ',/,&
-     '@  Erreur a la lecture du fichier meteo.                     ',/,&
-     '@    Le fichier a ete ouvert mais est peut etre incomplet    ',/,&
-     '@    ou son format inadapte.                                 ',/,&
-     '@                                                            ',/,&
-     '@    year (integer), quantile (integer), hour (integer),    ',/,&
-     '@          minute (integer), second (dble prec) of the profile',/,&
-     '@    location of the meteo profile (x,y) (dble prec)         ',/,&
-     '@    sea level pressure (double precision)                   ',/,&
-     '@ temperature profile:                                       ',/,&
-     '@   number of altitudes (integer)                            ',/,&
-     '@   alt.,temperature  in celcius,humidity in kg/kg (dble prec)',/,&
-     '@ wind profile:                                              ',/,&
-     '@   number of altitudes (integer)                            ',/,&
-     '@   alt.,u,v,k,eps (double precision)                        ',/,&
-     '@ NO LINE AT THE END OF THE FILE                             ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/)
-
-#else
-
 8000 format (                                                    &
      '@                                                            ',/,&
      '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -801,8 +679,6 @@ call csexit (1)
      '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
      '@                                                            ',/)
 
-#endif
-
 end subroutine atlecm
 
 
@@ -836,5 +712,3 @@ quant = distrib-(booll*retrait)+jour
 
 return
 end subroutine comp_quantile
-
-
