@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -33,14 +33,34 @@
 !> \param[in]   itypfb          boundary face types
 !> \param[in]   izfppp          boundary face zone number for atmospheric module
 !> \param[out]  icodcl          face boundary condition code
+!>                               - 1 Dirichlet
+!>                               - 2 Radiative outlet
+!>                               - 3 Neumann
+!>                               - 4 sliding and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 5 smooth wall and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 6 rough wall and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 9 free inlet/outlet
+!>                                 (input mass flux blocked to 0)
+!>                               - 13 Dirichlet for the advection operator and
+!>                                    Neumann for the diffusion operator
 !> \param[out]  rcodcl          Boundary conditions value
-!>- rcodcl(1) = valeur du dirichlet
-!>- rcodcl(2) = valeur du coef. d'echange ext. (infinie si pas d'echange)
-!>- rcodcl(3) = valeur de la densite de flux (negatif si gain) w/m2 ou
-!> hauteur de rugosite (m) si icodcl=6  \n
-!>    pour les vitesses (vistl+visct)*gradu \n
-!>    pour la pression             dt*gradp \n
-!>    pour les scalaires cp*(viscls+visct/turb_schmidt)*gradt
+!>                               - rcodcl(1) value of the dirichlet
+!>                               - rcodcl(2) value of the exterior exchange
+!>                                 coefficient (infinite if no exchange)
+!>                               - rcodcl(3) value flux density
+!>                                 (negative if gain) in w/m2 or roughness
+!>                                 in m if icodcl=6
+!>                                 -# for the velocity \f$ (\mu+\mu_T)
+!>                                    \gradv \vect{u} \cdot \vect{n}  \f$
+!>                                 -# for the pressure \f$ \Delta t
+!>                                    \grad P \cdot \vect{n}  \f$
+!>                                 -# for a scalar \f$ cp \left( K +
+!>                                     \dfrac{K_T}{\sigma_T} \right)
+!>                                     \grad T \cdot \vect{n} \f$
+!
 !-------------------------------------------------------------------------------
 subroutine attycl ( itypfb, izfppp, icodcl, rcodcl )
 
@@ -64,7 +84,7 @@ use field
 use atincl
 use atchem
 use atimbr
-use siream
+use sshaerosol
 use cs_c_bindings
 
 !===============================================================================
@@ -83,11 +103,15 @@ double precision rcodcl(nfabor,nvar,3)
 integer          ifac, iel, izone
 integer          ii
 integer jsp, isc
-double precision d2s3, zent, vs, xuent, xvent
+double precision d2s3, zent, vs, xuent, xvent, xwent
 double precision xkent, xeent, tpent, qvent,ncent
 double precision xcent
 double precision viscla, uref2, rhomoy, dhy, xiturb
 double precision, dimension(:), pointer :: brom, coefap, viscl
+double precision, dimension(:,:), pointer :: cpro_met_vel
+double precision, dimension(:), pointer :: cpro_met_potemp
+double precision, dimension(:), pointer :: cpro_met_qv, cpro_met_nc
+double precision, dimension(:), pointer :: cpro_met_k, cpro_met_eps
 
 ! arrays for cressman interpolation
 double precision , dimension(:),allocatable :: u_bord
@@ -113,6 +137,17 @@ tpent = 0.d0
 call field_get_val_s(ibrom, brom)
 call field_get_val_s(iviscl, viscl)
 
+if (imeteo.ge.2) then
+  call field_get_val_s_by_name('meteo_pot_temperature', cpro_met_potemp)
+  call field_get_val_v_by_name('meteo_velocity', cpro_met_vel)
+  call field_get_val_s_by_name('meteo_tke', cpro_met_k)
+  call field_get_val_s_by_name('meteo_eps', cpro_met_eps)
+  if (ippmod(iatmos).eq.2) then
+    call field_get_val_s_by_name('meteo_humidity', cpro_met_qv)
+    call field_get_val_s_by_name('meteo_drop_nb', cpro_met_nc)
+  endif
+endif
+
 !===============================================================================
 ! 2.  SI IPROFM = 1 : CHOIX ENTREE/SORTIE SUIVANT LE PROFIL METEO SI
 !                       ITYPFB N'A PAS ETE MODIFIE
@@ -230,9 +265,9 @@ call field_get_coefa_s(ivarfl(ipr), coefap)
 do ifac = 1, nfabor
 
   izone = izfppp(ifac)
+  iel = ifabor(ifac)
 
-  if (iprofm(izone).eq.1.and.imeteo.eq.1) then
-
+  if (iprofm(izone).eq.1.and.imeteo.ge.1) then
 !     On recupere les valeurs du profil et on met a jour RCODCL s'il n'a pas
 !       ete modifie. Il servira si la face est une face d'entree ou si c'est une
 !       face de sortie (si le flux est rentrant).
@@ -240,48 +275,55 @@ do ifac = 1, nfabor
 
     if (imbrication_flag .and.cressman_u) then
       xuent = u_bord(ifac)
-    else
-      call intprf                                                    &
-      !==========
+    else if (imeteo.eq.1) then
+      call intprf &
       (nbmetd, nbmetm,                                               &
-       zdmet, tmmet, umet , zent  , ttcabs, xuent )
+        zdmet, tmmet, umet , zent  , ttcabs, xuent )
+    else
+      xuent = cpro_met_vel(1, iel)
     endif
 
+    xwent = 0.d0
     if (imbrication_flag .and.cressman_v) then
       xvent = v_bord(ifac)
-    else
-      call intprf                                                    &
-      !==========
+    else if (imeteo.eq.1) then
+      call intprf &
       (nbmetd, nbmetm,                                               &
        zdmet, tmmet, vmet , zent  , ttcabs, xvent )
+    else
+      xvent = cpro_met_vel(2, iel)
+      xwent = cpro_met_vel(3, iel)
     endif
 
     if (imbrication_flag .and.cressman_tke) then
       xkent = tke_bord(ifac)
-    else
-      call intprf                                                    &
-      !==========
+    else if (imeteo.eq.1) then
+      call intprf &
       (nbmetd, nbmetm,                                               &
        zdmet, tmmet, ekmet, zent  , ttcabs, xkent )
+    else
+      xkent = cpro_met_k(iel)
     endif
 
     if (imbrication_flag .and.cressman_eps) then
       xeent = eps_bord(ifac)
-    else
-      call intprf                                                    &
-      !==========
+    else if (imeteo.eq.1) then
+      call intprf &
       (nbmetd, nbmetm,                                               &
        zdmet, tmmet, epmet, zent  , ttcabs, xeent )
+    else
+      xeent = cpro_met_eps(iel)
     endif
 
     if(imbrication_flag .and.cressman_theta                          &
        .and. ippmod(iatmos).ge.1 ) then
-        tpent = theta_bord(ifac)
-    else
-      call intprf                                                    &
-      !==========
+       tpent = theta_bord(ifac)
+    else if (imeteo.eq.1) then
+      call intprf &
       (nbmett, nbmetm,                                               &
        ztmet, tmmet, tpmet, zent  , ttcabs, tpent )
+    else
+      tpent = cpro_met_potemp(iel)
     endif
 
     vs = xuent*surfbo(1,ifac) + xvent*surfbo(2,ifac)
@@ -292,7 +334,7 @@ do ifac = 1, nfabor
     ! ete  specifiee par utilisateur.
 
     if (iautom(ifac).ge.1) then
-      if (vs.gt.0) then
+      if (vs.gt.epzero) then
         itypfb(ifac) = isolib
       else
         if (itypfb(ifac).eq.0) itypfb(ifac) = ientre
@@ -303,40 +345,42 @@ do ifac = 1, nfabor
 
       if (rcodcl(ifac,iu,1).gt.rinfin*0.5d0) rcodcl(ifac,iu,1) = xuent
       if (rcodcl(ifac,iv,1).gt.rinfin*0.5d0) rcodcl(ifac,iv,1) = xvent
-      if (rcodcl(ifac,iw,1).gt.rinfin*0.5d0) rcodcl(ifac,iw,1) = 0.d0
+      if (rcodcl(ifac,iw,1).gt.rinfin*0.5d0) rcodcl(ifac,iw,1) = xwent
 
       call turbulence_bc_set_uninit_inlet_k_eps(ifac, xkent, xeent, rcodcl)
 
       if (iscalt.ne.-1) then
 
         if (rcodcl(ifac,isca(iscalt),1).gt.rinfin*0.5d0) &
-             rcodcl(ifac,isca(iscalt),1) = tpent
+          rcodcl(ifac,isca(iscalt),1) = tpent
 
-          !  Humid Atmosphere
-          if ( ippmod(iatmos).eq.2 ) then
-            if (rcodcl(ifac,isca(itotwt),1).gt.rinfin*0.5d0)  then
-              if (imbrication_flag .and. cressman_qw)then
-                qvent = qw_bord(ifac)
-              else
-                call intprf &
-                !==========
+        !  Humid Atmosphere
+        if ( ippmod(iatmos).eq.2 ) then
+          if (rcodcl(ifac,isca(iymw),1).gt.rinfin*0.5d0)  then
+            if (imbrication_flag .and. cressman_qw)then
+              qvent = qw_bord(ifac)
+            else if (imeteo.eq.1) then
+              call intprf &
                 (nbmett, nbmetm, ztmet, tmmet, qvmet, zent, ttcabs, qvent )
-              endif
-              rcodcl(ifac,isca(itotwt),1) = qvent
-            endif
-
-            if (rcodcl(ifac,isca(intdrp),1).gt.rinfin*0.5d0)  then
-              if (imbrication_flag .and. cressman_nc)then
-                ncent = nc_bord(ifac)
-              else
-                call intprf &
-                !==========
-                (nbmett, nbmetm, ztmet, tmmet, ncmet, zent, ttcabs, ncent )
-              endif
-              rcodcl(ifac,isca(intdrp),1) = ncent
+            else
+              qvent = cpro_met_qv(iel)
             endif
+            rcodcl(ifac,isca(iymw),1) = qvent
           endif
 
+          if (rcodcl(ifac,isca(intdrp),1).gt.rinfin*0.5d0)  then
+            if (imbrication_flag .and. cressman_nc)then
+              ncent = nc_bord(ifac)
+            else if (imeteo.eq.1) then
+              call intprf &
+                (nbmett, nbmetm, ztmet, tmmet, ncmet, zent, ttcabs, ncent )
+            else
+              ncent = cpro_met_nc(iel)
+            endif
+            rcodcl(ifac,isca(intdrp),1) = ncent
+          endif
+        endif
+
       endif
 
     endif
@@ -345,8 +389,9 @@ do ifac = 1, nfabor
 
       if (iautom(ifac).ge.1) then
 
-        ! Dirichlet on the pressure
-        icodcl(ifac,ipr) = 1
+        ! Dirichlet on the pressure: expressed in solved pressure directly
+        ! (not in total pressure), that is why -1 is used (transformed as 1 in typecl.f90).
+        icodcl(ifac,ipr) = -1
         rcodcl(ifac,ipr,1) = coefap(ifac)
 
         ! Dirichlet on turbulent variables
@@ -370,7 +415,7 @@ do ifac = 1, nfabor
           icodcl(ifac,iw) = 1
           rcodcl(ifac,iu,1) = xuent
           rcodcl(ifac,iv,1) = xvent
-          rcodcl(ifac,iw,1) = 0.d0
+          rcodcl(ifac,iw,1) = xwent
 
         endif
 
@@ -394,7 +439,7 @@ do ifac = 1, nfabor
         call turbulence_bc_inlet_hyd_diam(ifac,                            &
                                           uref2, dhy, rhomoy, viscla,      &
                                           rcodcl)
-      else if (icalke(izone).eq.1) then
+      else if (icalke(izone).eq.2) then
         uref2 =   rcodcl(ifac,iu,1)**2                       &
                 + rcodcl(ifac,iv,1)**2                       &
                 + rcodcl(ifac,iw,1)**2
@@ -427,7 +472,7 @@ if (ifilechemistry.ge.1) then
 
     ! For species present in the concentration profiles file,
     ! profiles are used here as boundary conditions if boundary conditions have
-    ! not been treated earier (eg, in usatcl)
+    ! not been treated earlier (eg, in usatcl)
     do ii = 1, nespgi
       if (rcodcl(ifac,isca(isca_chem(idespgi(ii))),1).gt.0.5d0*rinfin) then
         call intprf                                                    &
@@ -441,7 +486,7 @@ if (ifilechemistry.ge.1) then
 
    endif
 
-   ! For other species zero dirichlet conditions are imposed,
+   ! For other species zero Dirichlet conditions are imposed,
    ! unless they have already been treated earlier (eg, in usatcl)
    do ii =1 , nespg
     if (rcodcl(ifac,isca(isca_chem(ii)),1).gt.0.5d0*rinfin) then
@@ -456,7 +501,7 @@ if (ifilechemistry.ge.1) then
 endif
 
 ! Atmospheric aerosol chemistry
-if (iaerosol.eq.1) then
+if (iaerosol.ne.CS_ATMO_AEROSOL_OFF) then
 
   do ifac = 1, nfabor
 
@@ -465,8 +510,8 @@ if (iaerosol.eq.1) then
      izone = izfppp(ifac)
 
       if (iprofa(izone).eq.1) then
-        do jsp = 1, nesp_aer*nbin_aer+nbin_aer
-          isc = (isca_chem(1) - 1)  + nespg_siream + jsp
+        do jsp = 1, nlayer_aer*n_aer+n_aer
+          isc = isca_chem(nespg + jsp)
           if (rcodcl(ifac,isca(isc),1).gt.0.5d0*rinfin) &
               rcodcl(ifac,isca(isc),1) = dlconc0(jsp)
         enddo
@@ -474,8 +519,8 @@ if (iaerosol.eq.1) then
 
       ! For other species zero dirichlet conditions are imposed,
       ! unless they have already been treated earlier (eg, in usatcl)
-      do ii = 1, nesp_aer*nbin_aer+nbin_aer
-        isc = (isca_chem(1) - 1)  + nespg_siream + ii
+      do ii = 1, nlayer_aer*n_aer+n_aer
+        isc = isca_chem(nespg + ii)
         if (rcodcl(ifac,isca(isc),1).gt.0.5d0*rinfin) &
             rcodcl(ifac,isca(isc),1) = 0.0d0
       enddo
@@ -484,10 +529,10 @@ if (iaerosol.eq.1) then
        ! (for example species not present in the third gaseous scheme,
        ! which can be treated in usatcl of with the file chemistry)
        ! zero dirichlet conditions are imposed
-       do ii = 1, nespg_siream
-        isc = (isca_chem(1) - 1)  + ii
-        if (rcodcl(ifac,isca(isc),1).gt.0.5d0*rinfin) &
-            rcodcl(ifac,isca(isc),1) = 0.0d0
+       do ii = 1, nespg
+         isc = isca_chem(ii)
+         if (rcodcl(ifac,isca(isc),1).gt.0.5d0*rinfin) &
+           rcodcl(ifac,isca(isc),1) = 0.0d0
        enddo
 
     endif
