@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -54,11 +54,11 @@ use cstnum
 use ppppar
 use ppthch
 use ppincl
-use ihmpre
 use atincl
 use atchem
 use field
-use siream
+use sshaerosol
+use cs_c_bindings
 
 !===============================================================================
 
@@ -68,12 +68,9 @@ implicit none
 
 integer        ii, jj, isc, f_id
 integer        kscmin, kscmax
-character(len=80) :: name, label
-character(len=2) :: nbin
-character(len=10), dimension(nesp_aer) :: f_esp_siream
 
 integer(c_int) :: n_chem_species
-integer(c_int), dimension(65) :: species_f_id
+integer(c_int), dimension(:), allocatable :: species_f_id
 
 !===============================================================================
 ! Interfaces
@@ -100,16 +97,9 @@ call field_get_key_id("min_scalar_clipping", kscmin)
 call field_get_key_id("max_scalar_clipping", kscmax)
 
 !===============================================================================
-! 1. GUI and model information
+! 1. Model information
 !===============================================================================
 
-if (iihmpr.eq.1) then
-  call uiati1 (imeteo, ficmet, len(ficmet))
-endif
-
-! Set base default model parameters and call usati1
-
-call atini0
 
 !===============================================================================
 ! 1. Add variables
@@ -142,8 +132,8 @@ if (ippmod(iatmos).eq.2) then
   call field_set_key_double(f_id, kscmin, 200.d0)
 
   ! total water content
-  call add_model_scalar_field('total_water', 'TotWater', itotwt)
-  f_id = ivarfl(isca(itotwt))
+  call add_model_scalar_field('ym_water', 'Ym water', iymw)
+  f_id = ivarfl(isca(iymw))
   call field_set_key_double(f_id, kscmin, 0.d0)
 
   ! total number of droplets
@@ -159,11 +149,11 @@ endif
 
 ! Atmospheric gaseous chemistry
 ! Do not change this order
-if (iaerosol.eq.1) ichemistry = 3
+if (iaerosol.eq.CS_ATMO_AEROSOL_SSH) ichemistry = 4
 ! if a chemical scheme is solved, a concentration profiles
-! file must be used
+! file  named 'chemistry' should be used
 if (ichemistry.ge.1) ifilechemistry = ichemistry
-if (inogaseouschemistry.eq.1) ichemistry = 0
+if (nogaseouschemistry .and. iaerosol.eq.CS_ATMO_AEROSOL_OFF) ichemistry = 0
 
 if (ifilechemistry.ge.1) then
 
@@ -176,25 +166,28 @@ if (ifilechemistry.ge.1) then
   if (ifilechemistry.eq.1) then
     nrg = 5
     nespg = 4
-    allocate(isca_chem(nespg))
-    allocate(dmmk(nespg))
-    allocate(chempoint(nespg))
-    call add_model_scalar_field('chemistry_no',  'NO',  isca_chem(1))
-    call add_model_scalar_field('chemistry_no2', 'NO2', isca_chem(2))
-    call add_model_scalar_field('chemistry_o3',  'O3',  isca_chem(3))
-    call add_model_scalar_field('chemistry_o3p', 'O3P', isca_chem(4))
-    dmmk(1)=30.d-3 ! Molar mass NO
-    dmmk(2)=46.d-3 ! Molar mass NO2
-    dmmk(3)=48.d-3 ! Molar mass O3
-    dmmk(4)=16.d-3 ! Molar mass O3P
+
+    ! Map isca_chem, dmmk, chempoint and allocate it if needed
+    call init_chemistry_pointers()
+
+    call add_model_scalar_field('species_no',  'NO',  isca_chem(1))
+    call add_model_scalar_field('species_no2', 'NO2', isca_chem(2))
+    call add_model_scalar_field('species_o3',  'O3',  isca_chem(3))
+    call add_model_scalar_field('species_o3p', 'O3P', isca_chem(4))
+    dmmk(1)=30.d0  ! Molar mass (g/mol) NO
+    dmmk(2)=46.d0  ! Molar mass (g/mol) NO2
+    dmmk(3)=48.d0  ! Molar mass (g/mol) O3
+    dmmk(4)=16.d0  ! Molar mass (g/mol) O3P
     chempoint = (/ 4, 3, 2, 1 /)
-  ! scheme with 20 species and 34 reactions
+
+  ! scheme with 20 species and 34 reactions! Note pas de COV
   else if (ifilechemistry.eq.2) then
     nrg = 34
     nespg = 20
-    allocate(isca_chem(nespg))
-    allocate(dmmk(nespg))
-    allocate(chempoint(nespg))
+
+    ! Map isca_chem, dmmk, chempoint and allocate it if needed
+    call init_chemistry_pointers()
+
     call add_model_scalar_field('species_no',    'NO',    isca_chem(1))
     call add_model_scalar_field('species_no2',   'NO2',   isca_chem(2))
     call add_model_scalar_field('species_o3',    'O3',    isca_chem(3))
@@ -215,40 +208,36 @@ if (ifilechemistry.ge.1) then
     call add_model_scalar_field('species_xo2',   'XO2',   isca_chem(18))
     call add_model_scalar_field('species_so2',   'SO2',   isca_chem(19))
     call add_model_scalar_field('species_h2so4', 'H2SO4', isca_chem(20))
-    dmmk(1)=30.d-3     ! Molar mass NO
-    dmmk(2)=46.d-3     ! Molar mass NO2
-    dmmk(3)=48.d-3     ! Molar mass O3
-    dmmk(4)=16.d-3     ! Molar mass O3P
-    dmmk(5)=16.d-3     ! Molar mass O1D
-    dmmk(6)=17.01d-3   ! Molar mass OH
-    dmmk(7)=33.01d-3   ! Molar mass HO2
-    dmmk(8)=34.01d-3   ! Molar mass H2O2
-    dmmk(9)=62.01d-3   ! Molar mass NO3
-    dmmk(10)=108.01d-3 ! Molar mass N2O5
-    dmmk(11)=47.01d-3  ! Molar mass HONO
-    dmmk(12)=63.01d-3  ! Molar mass HNO3
-    dmmk(13)=28.01d-3  ! Molar mass CO
-    dmmk(14)=30.03d-3  ! Molar mass HCHO
-    dmmk(15)=44.05d-3  ! Molar mass ALD2
-    dmmk(16)=75.04d-3  ! Molar mass C2O3
-    dmmk(17)=121.05d-3 ! Molar mass PAN
-    dmmk(18)=47.03d-3  ! Molar mass XO2
-    dmmk(19)=64.06d-3  ! Molar mass SO2
-    dmmk(20)=98.08d-3  ! Molar mass H2SO4
+    dmmk(1)=30.d0      ! Molar mass (g/mol) NO
+    dmmk(2)=46.d0      ! Molar mass (g/mol) NO2
+    dmmk(3)=48.d0      ! Molar mass (g/mol) O3
+    dmmk(4)=16.d0      ! Molar mass (g/mol) O3P
+    dmmk(5)=16.d0      ! Molar mass (g/mol) O1D
+    dmmk(6)=17.01d0    ! Molar mass (g/mol) OH
+    dmmk(7)=33.01d0    ! Molar mass (g/mol) HO2
+    dmmk(8)=34.01d0    ! Molar mass (g/mol) H2O2
+    dmmk(9)=62.01d0    ! Molar mass (g/mol) NO3
+    dmmk(10)=108.01d0  ! Molar mass (g/mol) N2O5
+    dmmk(11)=47.01d0   ! Molar mass (g/mol) HONO
+    dmmk(12)=63.01d0   ! Molar mass (g/mol) HNO3
+    dmmk(13)=28.01d0   ! Molar mass (g/mol) CO
+    dmmk(14)=30.03d0   ! Molar mass (g/mol) HCHO
+    dmmk(15)=44.05d0   ! Molar mass (g/mol) ALD2
+    dmmk(16)=75.04d0   ! Molar mass (g/mol) C2O3
+    dmmk(17)=121.05d0  ! Molar mass (g/mol) PAN
+    dmmk(18)=47.03d0   ! Molar mass (g/mol) XO2
+    dmmk(19)=64.06d0   ! Molar mass (g/mol) SO2
+    dmmk(20)=98.08d0   ! Molar mass (g/mol) H2SO4
     chempoint = (/ 20, 19, 16, 17, 2, 15, 14, 3, 18, 7, 8, 9, 4, &
                    10, 1, 12, 11, 13, 5, 6 /)
+
   ! scheme CB05 with 52 species and 155 reactions
   else if (ifilechemistry.eq.3) then
-    if (iaerosol.eq.1) then
-      nrg = 162
-      nespg = 65
-    else
-      nrg = 155
-      nespg = 52
-    endif
-    allocate(isca_chem(nespg))
-    allocate(dmmk(nespg))
-    allocate(chempoint(nespg))
+    nrg = 155
+    nespg = 52
+
+    ! Map isca_chem, dmmk, chempoint and allocate it if needed
+    call init_chemistry_pointers()
 
     call add_model_scalar_field('species_no',    'NO',    isca_chem(1))
     call add_model_scalar_field('species_no2',   'NO2',   isca_chem(2))
@@ -302,158 +291,139 @@ if (ifilechemistry.ge.1) then
     call add_model_scalar_field('species_so2',   'SO2',   isca_chem(50))
     call add_model_scalar_field('species_h2so4', 'H2SO4', isca_chem(51))
     call add_model_scalar_field('species_hco3',  'HCO3',  isca_chem(52))
-    dmmk(1)=30.d-3     ! Molar mass NO
-    dmmk(2)=46.d-3     ! Molar mass NO2
-    dmmk(3)=48.d-3     ! Molar mass O3
-    dmmk(4)=16.d-3     ! Molar mass O3P
-    dmmk(5)=16.d-3     ! Molar mass O1D
-    dmmk(6)=17.01d-3   ! Molar mass OH
-    dmmk(7)=33.01d-3   ! Molar mass HO2
-    dmmk(8)=34.01d-3   ! Molar mass H2O2
-    dmmk(9)=62.01d-3   ! Molar mass NO3
-    dmmk(10)=108.01d-3 ! Molar mass N2O5
-    dmmk(11)=47.01d-3  ! Molar mass HONO
-    dmmk(12)=63.01d-3  ! Molar mass HNO3
-    dmmk(13)=79.01d-3  ! Molar mass HNO4
-    dmmk(14)=28.01d-3  ! Molar mass CO
-    dmmk(15)=30.03d-3  ! Molar mass HCHO
-    dmmk(16)=44.05d-3  ! Molar mass ALD2
-    dmmk(17)=75.04d-3  ! Molar mass C2O3
-    dmmk(18)=121.05d-3 ! Molar mass PAN
-    dmmk(19)=43.04d-3  ! Molar mass ALDX
-    dmmk(20)=74.04d-3  ! Molar mass CXO3
-    dmmk(21)=120.04d-3 ! Molar mass PANX
-    dmmk(22)=47.03d-3  ! Molar mass XO2
-    dmmk(23)=47.03d-3  ! Molar mass XO2N
-    dmmk(24)=77.04d-3  ! Molar mass NTR
-    dmmk(25)=46.07d-3  ! Molar mass ETOH
-    dmmk(26)=16.04d-3  ! Molar mass CH4
-    dmmk(27)=47.03d-3  ! Molar mass MEO2
-    dmmk(28)=32.04d-3  ! Molar mass MEOH
-    dmmk(29)=48.04d-3  ! Molar mass MEPX
-    dmmk(30)=46.03d-3  ! Molar mass FACD
-    dmmk(31)=30.07d-3  ! Molar mass ETHA
-    dmmk(32)=47.03d-3  ! Molar mass ROOH
-    dmmk(33)=60.05d-3  ! Molar mass AACD
-    dmmk(34)=76.05d-3  ! Molar mass PACD
-    dmmk(35)=15.03d-3  ! Molar mass PAR
-    dmmk(36)=16.d-3    ! Molar mass ROR
-    dmmk(37)=28.05d-3  ! Molar mass ETH
-    dmmk(38)=27.05d-3  ! Molar mass OLE
-    dmmk(39)=56.11d-3  ! Molar mass IOLE
-    dmmk(40)=68.18d-3  ! Molar mass ISOP
-    dmmk(41)=70.09d-3  ! Molar mass ISPD
-    dmmk(42)=136.24d-3 ! Molar mass TERP
-    dmmk(43)=92.14d-3  ! Molar mass TOL
-    dmmk(44)=106.16d-3 ! Molar mass XYL
-    dmmk(45)=108.14d-3 ! Molar mass CRES
-    dmmk(46)=141.15d-3 ! Molar mass TO2
-    dmmk(47)=48.04d-3  ! Molar mass OPEN
-    dmmk(48)=108.14d-3 ! Molar mass CRO
-    dmmk(49)=72.06d-3  ! Molar mass MGLY
-    dmmk(50)=64.06d-3  ! Molar mass SO2
-    dmmk(51)=98.08d-3  ! Molar mass H2SO4
-    dmmk(52)=63.03d-3  ! Molar mass HCO3
-    if (iaerosol.eq.1) then
-      call add_model_scalar_field('species_hc8',      'HC8',      isca_chem(53))
-      call add_model_scalar_field('species_api',      'API',      isca_chem(54))
-      call add_model_scalar_field('species_lim',      'LIM',      isca_chem(55))
-      call add_model_scalar_field('species_cvar01',   'CVARO1',   isca_chem(56))
-      call add_model_scalar_field('species_cvar02',   'CVARO2',   isca_chem(57))
-      call add_model_scalar_field('species_cvalk1',   'CVALK1',   isca_chem(58))
-      call add_model_scalar_field('species_cvole1',   'CVOLE1',   isca_chem(59))
-      call add_model_scalar_field('species_cvapi1',   'CVAPI1',   isca_chem(60))
-      call add_model_scalar_field('species_cvapi2',   'CVAPI2',   isca_chem(61))
-      call add_model_scalar_field('species_cvlim1',   'CVLIM1',   isca_chem(62))
-      call add_model_scalar_field('species_cvlim2',   'CVLIM2',   isca_chem(63))
-      call add_model_scalar_field('species_nh3',      'NH3',      isca_chem(64))
-      call add_model_scalar_field('species_hcl',      'HCL',      isca_chem(65))
-      call add_model_scalar_field('species_cvbibmp',  'CVBIBMP',  isc)
-      call add_model_scalar_field('species_cvanclp',  'CVANCLP',  isc)
-      call add_model_scalar_field('species_cvbiiso1', 'CVBIISO1', isc)
-      call add_model_scalar_field('species_cvbiiso2', 'CVBIISO2', isc)
-      dmmk(53)=114.0d0   ! Molar mass HC8
-      dmmk(54)=136.0d0   ! Molar mass API
-      dmmk(55)=136.0d0   ! Molar mass LIM
-      dmmk(56)=150.0d0   ! Molar mass CVARO1
-      dmmk(57)=150.0d0   ! Molar mass CVARO2
-      dmmk(58)=140.0d0   ! Molar mass CVALK1
-      dmmk(59)=140.0d0   ! Molar mass CVOLE1
-      dmmk(60)=184.0d0   ! Molar mass CVAPI1
-      dmmk(61)=184.0d0   ! Molar mass CVAPI2
-      dmmk(62)=200.0d0   ! Molar mass CVLIM1
-      dmmk(63)=200.0d0   ! Molar mass CVLIM2
-      dmmk(64)=17.0d0    ! Molar mass NH3
-      dmmk(65)=36.5d0    ! Molar mass HCL
+    dmmk(1)=30.d0      ! Molar mass (g/mol) NO
+    dmmk(2)=46.d0      ! Molar mass (g/mol) NO2
+    dmmk(3)=48.d0      ! Molar mass (g/mol) O3
+    dmmk(4)=16.d0      ! Molar mass (g/mol) O3P
+    dmmk(5)=16.d0      ! Molar mass (g/mol) O1D
+    dmmk(6)=17.01d0    ! Molar mass (g/mol) OH
+    dmmk(7)=33.01d0    ! Molar mass (g/mol) HO2
+    dmmk(8)=34.01d0    ! Molar mass (g/mol) H2O2
+    dmmk(9)=62.01d0    ! Molar mass (g/mol) NO3
+    dmmk(10)=108.01d0  ! Molar mass (g/mol) N2O5
+    dmmk(11)=47.01d0   ! Molar mass (g/mol) HONO
+    dmmk(12)=63.01d0   ! Molar mass (g/mol) HNO3
+    dmmk(13)=79.01d0   ! Molar mass (g/mol) HNO4
+    dmmk(14)=28.01d0   ! Molar mass (g/mol) CO
+    dmmk(15)=30.03d0   ! Molar mass (g/mol) HCHO
+    dmmk(16)=44.05d0   ! Molar mass (g/mol) ALD2
+    dmmk(17)=75.04d0   ! Molar mass (g/mol) C2O3
+    dmmk(18)=121.05d0  ! Molar mass (g/mol) PAN
+    dmmk(19)=43.04d0   ! Molar mass (g/mol) ALDX
+    dmmk(20)=74.04d0   ! Molar mass (g/mol) CXO3
+    dmmk(21)=120.04d0  ! Molar mass (g/mol) PANX
+    dmmk(22)=47.03d0   ! Molar mass (g/mol) XO2
+    dmmk(23)=47.03d0   ! Molar mass (g/mol) XO2N
+    dmmk(24)=77.04d0   ! Molar mass (g/mol) NTR
+    dmmk(25)=46.07d0   ! Molar mass (g/mol) ETOH
+    dmmk(26)=16.04d0   ! Molar mass (g/mol) CH4
+    dmmk(27)=47.03d0   ! Molar mass (g/mol) MEO2
+    dmmk(28)=32.04d0   ! Molar mass (g/mol) MEOH
+    dmmk(29)=48.04d0   ! Molar mass (g/mol) MEPX
+    dmmk(30)=46.03d0   ! Molar mass (g/mol) FACD
+    dmmk(31)=30.07d0   ! Molar mass (g/mol) ETHA
+    dmmk(32)=47.03d0   ! Molar mass (g/mol) ROOH
+    dmmk(33)=60.05d0   ! Molar mass (g/mol) AACD
+    dmmk(34)=76.05d0   ! Molar mass (g/mol) PACD
+    dmmk(35)=15.03d0   ! Molar mass (g/mol) PAR
+    dmmk(36)=16.d0     ! Molar mass (g/mol) ROR
+    dmmk(37)=28.05d0   ! Molar mass (g/mol) ETH
+    dmmk(38)=27.05d0   ! Molar mass (g/mol) OLE
+    dmmk(39)=56.11d0   ! Molar mass (g/mol) IOLE
+    dmmk(40)=68.18d0   ! Molar mass (g/mol) ISOP
+    dmmk(41)=70.09d0   ! Molar mass (g/mol) ISPD
+    dmmk(42)=136.24d0  ! Molar mass (g/mol) TERP
+    dmmk(43)=92.14d0   ! Molar mass (g/mol) TOL
+    dmmk(44)=106.16d0  ! Molar mass (g/mol) XYL
+    dmmk(45)=108.14d0  ! Molar mass (g/mol) CRES
+    dmmk(46)=141.15d0  ! Molar mass (g/mol) TO2
+    dmmk(47)=48.04d0   ! Molar mass (g/mol) OPEN
+    dmmk(48)=108.14d0  ! Molar mass (g/mol) CRO
+    dmmk(49)=72.06d0   ! Molar mass (g/mol) MGLY
+    dmmk(50)=64.06d0   ! Molar mass (g/mol) SO2
+    dmmk(51)=98.08d0   ! Molar mass (g/mol) H2SO4
+    dmmk(52)=63.03d0   ! Molar mass (g/mol) HCO3
+    chempoint = (/ 48, 52, 47, 43, 1, 42, 50, 17, 44, 9, 15, 38, 13, 37,&
+                   41, 45, 51, 10, 35, 46, 14, 49, 39, 33, 2, 3, 40, 11,&
+                   19, 20, 4, 21, 36, 22, 34, 16, 23, 24, 25, 31, 32, 26,&
+                   5, 6, 27, 12, 28, 30, 29, 7, 8, 18 /)
+
+  ! User defined chemistry using SPACK file and routines
+  else if (ifilechemistry.eq.4) then
+
+    ! This function read the number of species, their molar mass
+    ! and creates variables
+    call cs_atmo_declare_chem_from_spack()
+
+    ! Read the number of reactions
+    call ssh_dimensions(ii, nrg, jj)
+
+    ! Verification
+    if (ii.ne.nespg) then
+      write(nfecra,1003)
+      call csexit (1)
     endif
-    if (iaerosol.eq.1) then
-      chempoint = (/ 64, 65, 59, 57, 3, 55, 61, 20, 56, 16, 23, 50,&
-                     17, 51, 54, 60, 62, 13, 48, 58, 18, 63, 52, 46,&
-                     4, 5, 53, 14, 22, 35, 6, 32, 49, 33, 47, 19, 34,&
-                     36, 37, 44, 45, 39, 7, 8, 40, 15, 41, 43, 42, 9,&
-                     10, 21, 11, 24, 27, 30, 31, 12, 38, 25, 26, 28, 29,&
-                     1, 2 /)
-    else
-      chempoint = (/ 48, 52, 47, 43, 1, 42, 50, 17, 44, 9, 15, 38, 13, 37,&
-                     41, 45, 51, 10, 35, 46, 14, 49, 39, 33, 2, 3, 40, 11,&
-                     19, 20, 4, 21, 36, 22, 34, 16, 23, 24, 25, 31, 32, 26,&
-                     5, 6, 27, 12, 28, 30, 29, 7, 8, 18 /)
-    endif
- endif
+
+    ! Map isca_chem, dmmk, chempoint and allocate it if needed
+    call init_chemistry_pointers()
+
+  endif
+
+  ! Finish initialization of C chemistry when SPACK was not used
+  if (ifilechemistry.ne.4) then
+    call cs_atmo_chem_init_c_chemistry()
+  endif
 
 endif
 
 ! Atmospheric aerosol chemistry
-if (iaerosol.eq.1) then
-
-  write (nfecra,*) "Atmospheric aerosol chemistry activated"
+if (iaerosol.ne.CS_ATMO_AEROSOL_OFF) then
 
   ! logical unit and name of the chemical profiles file
   impmea=28
   ficmea='aerosols'
 
-  ! 4 species not followed in gaseous scheme 3 but necessary to siream
-  nespg_siream=nespg+4
-
   ! Verification
-  if (ifilechemistry.ne.3) then
+  if (ifilechemistry.ne.4) then
     write(nfecra,1004)
     call csexit (1)
   endif
 
-  ! Filling the names array
-  esp_siream=(/'MD    ', 'BC    ', 'NA    ', 'H2SO4 ', 'NH3   ', 'HNO3  ',&
- 'HCL   ','ARO1  ', 'ARO2  ', 'ALK1  ', 'OLE1  ', 'API1  ', 'API2  ',&
- 'LIM1  ', 'LIM2  ', 'ANCLP ', 'BIISO1', 'BIISO2',&
- 'BIBMP ', 'POA   ', 'H2O   '/)
+  ! Load shared library
+  ! Initialise external aerosol code
+  ! Create variables
+  call cs_atmo_aerosol_initialize()
 
-  f_esp_siream=(/'md    ', 'bc    ', 'na    ', 'h2so4 ', 'nh3   ', 'hno3  ',&
- 'hcl   ','aro1  ', 'aro2  ', 'alk1  ', 'ole1  ', 'api1  ', 'api2  ',&
- 'lim1  ', 'lim2  ', 'anclp ', 'biiso1', 'biiso2',&
- 'bibmp ', 'poa   ', 'h2o   '/)
+  ! Remap pointers following bft_realloc when initializing aerosols
+  call init_aerosol_pointers()
 
-  do jj = 1, nesp_aer
-    do ii = 1, nbin_aer
-      write(nbin,"(i2)") ii
-      name = 'species_'//trim(f_esp_siream(jj))//'_bin'//trim(adjustl(nbin))
-      label = trim(esp_siream(jj))//'_bin'//trim(adjustl(nbin))
-      call add_model_scalar_field(name, label,  isc)
-    enddo
+endif
+
+! Set clippings for gas and aerosol species
+if (ichemistry.ge.1) then
+  do ii = 1, nespg
+    f_id = ivarfl(isca(isca_chem(ii)))
+    call field_set_key_double(f_id, kscmin, 0.d0)
   enddo
-
-  do ii = 1, nbin_aer
-    write(nbin,"(i2)") ii
-    name = 'species_'//'naero_bin'//trim(adjustl(nbin))
-    label = 'Naero_bin'//trim(adjustl(nbin))
-    call add_model_scalar_field(name, label,  isc)
+endif
+if (iaerosol.ne.CS_ATMO_AEROSOL_OFF) then
+  do ii = nespg + 1, nespg + n_aer * (nlayer_aer + 1)
+    f_id = ivarfl(isca(isca_chem(ii)))
+    call field_set_key_double(f_id, kscmin, 0.d0)
+  enddo
+  ! Allow large aerosol numbers
+  do ii = nespg + n_aer * nlayer_aer + 1, nespg + n_aer * (nlayer_aer + 1)
+    f_id = ivarfl(isca(isca_chem(ii)))
+    call field_set_key_double(f_id, kscmax, 1.d40)
   enddo
-
 endif
 
 !===============================================================================
 ! 4. Map to fields and GUI
 !===============================================================================
 
+allocate(species_f_id(nespg))
+
 n_chem_species = nespg
 do isc = 1, nespg
   species_f_id(isc) = ivarfl(isca(isca_chem(isc)))
@@ -461,6 +431,8 @@ enddo
 
 call cs_field_pointer_map_atmospheric(n_chem_species, species_f_id)
 
+deallocate(species_f_id)
+
 !===============================================================================
 ! 5. General field and physical properties
 !===============================================================================
@@ -472,28 +444,20 @@ icp = -1
 ! Formats
 !----
 
-#if defined(_CS_LANG_FR)
-
- 1004 format(                                                           &
+ 1003 format(                                                           &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
+'@ @@  WARNING:   STOP WHILE READING INPUT DATA               ',/,&
 '@    =========                                               ',/,&
-'@    CHIMIE ATMOSPHERIQUE PARTICULAIRE DEMANDEE              ',/,&
+'@    ATMOSPHERIC CHEMISTRY FROM SPACK                        ',/,&
 '@                                                            ',/,&
-'@  Lorsque le modele de chimie particulaire siream est utilise',/,&
-'@   un schema gazeux complet(CB05) est automatiquement utilise',/,&
-'@  L''utilisateur ne doit pas en specifier d''autre (ichemistry)',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier usati1 (cs_user_parameters.f90)                  ',/,&
+'@  The number of gaseous species read from the SPACK file    ',/,&
+'@  is not equal to the one read in the SPACK source file     ',/,&
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#else
-
  1004 format(                                                           &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -502,7 +466,7 @@ icp = -1
 '@    =========                                               ',/,&
 '@    ATMOSPHERIC AEROSOL CHEMISTRY MODULE                    ',/,&
 '@                                                            ',/,&
-'@  When aerosol chemistry model siream is used               ',/,&
+'@  When aerosol chemistry model is used                      ',/,&
 '@   a full gaseous scheme (CB05) is automatically used       ',/,&
 '@  The user cannot specify any other scheme (ichemistry)     ',/,&
 '@  Computation CAN NOT run.                                  ',/,&
@@ -513,8 +477,6 @@ icp = -1
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
 !----
 ! End
 !----
