@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -59,7 +59,6 @@ use mesh
 use field
 use dimens
 use atchem
-use siream
 
 implicit none
 
@@ -114,11 +113,11 @@ do iel = 1, ncel
   enddo
 
   do ii = 1, nespg
-    conv_factor(chempoint(ii)) = rom*navo*(1.0d-12)/dmmk(ii)
+    conv_factor(chempoint(ii)) = rom*navo*(1.0d-9)/dmmk(ii)
     source(ii) = 0.0d0
   enddo
 
-  if ((isepchemistry.eq.1).or.(ntcabs.eq.1)) then
+  if ((isepchemistry.eq.1).or.(ntcabs.lt.ntinit)) then
     ! -----------------------------
     ! -- splitted Rosenbrock solver
     ! -----------------------------
@@ -144,13 +143,9 @@ do iel = 1, ncel
     else if (ichemistry.eq.2) then
       call fexchem_2 (nespg,nrg,dlconc,rk,source,conv_factor,dchema)
     else if (ichemistry.eq.3) then
-      if (iaerosol.eq.1) then
-        call fexchem_siream (nespg,nrg,dlconc,rk,source,conv_factor,dchema)
-      else
-        call fexchem_3 (nespg,nrg,dlconc,rk,source,conv_factor,dchema)
-      endif
+      call fexchem_3 (nespg,nrg,dlconc,rk,source,conv_factor,dchema)
     else if (ichemistry.eq.4) then
-      call fexchem (nespg,nrg,dlconc,rk,source,conv_factor,dchema)
+      call fexchem_4 (nespg,nrg,dlconc,rk,source,conv_factor,dchema)
     endif
 
     ! Explicit contribution from dynamics as a source term:
@@ -167,14 +162,14 @@ do iel = 1, ncel
 
   ! The maximum time step used for chemistry resolution is dtchemmax
   if (dtc.le.dtchemmax) then
-    call roschem (dlconc,source,source,conv_factor,dtc,rk,rk)
+    call chem_roschem (dlconc,source,source,conv_factor,dtc,rk,rk)
   else
     ncycle = int(dtc/dtchemmax)
     dtrest = mod(dtc,dtchemmax)
     do ii = 1, ncycle
-      call roschem (dlconc,source,source,conv_factor,dtchemmax,rk,rk)
+      call chem_roschem (dlconc,source,source,conv_factor,dtchemmax,rk,rk)
     enddo
-    call roschem (dlconc,source,source,conv_factor,dtrest,rk,rk)
+    call chem_roschem (dlconc,source,source,conv_factor,dtrest,rk,rk)
   endif
 
   ! Update of values at current time step
@@ -186,10 +181,9 @@ enddo
 
 deallocate(cvar_espg, cvara_espg)
 
-! TODO: clipping or not ?
 ! Clipping
 do ii = 1, nespg
-  call clpsca (ii)
+  call clpsca(isca_chem(ii))
 enddo
 
 !--------
