@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -273,7 +273,7 @@ cs_at_data_assim_build_ops(void)
         bft_printf("    Obs %i\n",ii);
         for (cs_lnum_t jj = proj_idx[ii]; jj < proj_idx[ii+1]; jj++)
           bft_printf("    Point %i x %.2f y %.2f z %.2f coef %.2f\n",
-                     jj, (proj + jj*4)[1], (proj + jj*4)[2],
+                     (int)jj, (proj + jj*4)[1], (proj + jj*4)[2],
                      (proj + jj*4)[3], (proj + jj*4)[0]);
         bft_printf("\n");
       }
