@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -749,10 +749,12 @@ cs_at_opt_interp_read_file(char const           filename[50],
   while (fgets(line, MAX_LINE_SIZE, fichier)) {
     /* Reading Number of observations */
     if (strncmp(line, "_nobs_", 6) == 0) {
-      fscanf(fichier, "%i", &(ms->nb_measures));
+      int n_measures;
+      fscanf(fichier, "%i", &n_measures);
+      ms->nb_measures = n_measures;
 
 #if _OI_DEBUG_
-      bft_printf("   *Reading _nobs_ : %i\n", ms->nb_measures);
+      bft_printf("   *Reading _nobs_ : %i\n", n_measures);
 #endif
 
       if (ms->nb_measures <= 0) {
@@ -920,7 +922,7 @@ cs_at_opt_interp_read_file(char const           filename[50],
       }
     }
 
-    /* Reading the number of obs/times to print in the listing */
+    /* Reading the number of obs/times to print in the log */
     if (strncmp(line, "_n_log_data_", 12) == 0) {
       fscanf(fichier, "%i", &(oi->n_log_data));
       if (oi->n_log_data < 0)
