@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -47,7 +47,7 @@ use mesh
 use field
 use atincl
 use atchem
-use siream
+use cs_c_bindings
 
 implicit none
 
@@ -55,7 +55,8 @@ implicit none
 
 ! Local variables
 
-integer iel,ii
+integer iel,ii,iphotolysis
+integer          met_qv_id
 double precision temp, dens          ! temperature, density
 double precision press, hspec        ! pressure, specific humidity (kg/kg)
 double precision rk(nrg)             ! kinetic rates
@@ -66,10 +67,14 @@ double precision qureel              ! julian day
 double precision heurtu              ! yime (UTC)
 double precision albe                ! albedo, useless here
 double precision fo                  ! solar constant, useless here
+double precision omega
 
 double precision, dimension(:), pointer :: crom
 double precision, dimension(:), pointer :: cvar_totwt
 double precision, dimension(:), pointer :: cpro_tempc, cpro_liqwt
+double precision, dimension(:), pointer :: cpro_met_temp
+double precision, dimension(:), pointer :: cpro_met_p
+double precision, dimension(:), pointer :: cpro_met_qv
 
 ! Initialisation
 
@@ -78,13 +83,28 @@ dens = ro0
 press = dens*rair*temp ! ideal gas law
 hspec = 0.0d0
 
+if (imeteo.ge.2) then
+  call field_get_val_s_by_name('meteo_temperature', cpro_met_temp)
+  call field_get_val_s_by_name('meteo_pressure', cpro_met_p)
+  call field_get_id_try('meteo_humidity', met_qv_id)
+  if (met_qv_id.ge.0) then
+    call field_get_val_s(met_qv_id, cpro_met_qv)
+  endif
+endif
+
+if (photolysis) then
+  iphotolysis = 1
+else
+  iphotolysis = 2
+endif
+
 if (ippmod(iatmos).ge.1) then
   call field_get_val_s(icrom, crom)
   call field_get_val_s(itempc, cpro_tempc)
 endif
 
 if (ippmod(iatmos).ge.2) then
-  call field_get_val_s(ivarfl(isca(itotwt)), cvar_totwt)
+  call field_get_val_s(ivarfl(isca(iymw)), cvar_totwt)
   call field_get_val_s(iliqwt, cpro_liqwt)
 endif
 
@@ -93,13 +113,11 @@ endif
 ! Computation of zenith angle
 qureel = float(squant)
 heurtu = float(shour) + float(smin)/60.d0+ssec/3600.d0
-call raysze(xlat,xlon,qureel,heurtu,0,albe,dlmuzero,fo)
+if (idtvar.eq.0 .or. idtvar.eq.1) heurtu = heurtu + ttcabs/3600.d0
+call raysze(xlat,xlon,qureel,heurtu,0,albe,dlmuzero, omega, fo)
 azi = dabs(dacos(dlmuzero)*180.d0/pi)
 
-! To be sure to cut photolysis (SPACK does not) if azi>90
-if (azi.gt.90.0d0) then
-  iphotolysis = 2
-endif
+! Note: Photolysis should be cut in SPACK and not here even if the azimuthal angle is > 90
 
 ! Loop on cells
 do iel = 1, ncel
@@ -117,19 +135,21 @@ do iel = 1, ncel
 
     ! Hydrostatic pressure
     call intprf                                                   &
-    !==========
    (nbmett, nbmetm,                                               &
     ztmet , tmmet, phmet, zent, ttcabs, press )
 
     ! Temperature
     call intprf                                                   &
-    !==========
    (nbmett, nbmetm,                                               &
     ztmet , tmmet, ttmet, zent, ttcabs, temp )
-    temp=temp+tkelvi
+
+    temp = temp + tkelvi
+  else
+    press = cpro_met_p(iel)
+    temp = cpro_met_temp(iel) + tkelvi
   endif
 
-  ! Specific hymidity
+  ! Specific humidity
   ! Humid atmosphere
   if (ippmod(iatmos).ge.2) then
     hspec = (cvar_totwt(iel)-cpro_liqwt(iel))    &
@@ -139,10 +159,15 @@ do iel = 1, ncel
   else if (imeteo.eq.1) then
 
     call intprf                                                   &
-    !==========
    (nbmett, nbmetm,                                               &
     ztmet , tmmet, qvmet, zent, ttcabs, hspec )
 
+  else if (met_qv_id.ge.0) then
+    hspec = cpro_met_qv(iel)
+
+  ! Dry
+  else
+    hspec = 0.d0
   endif
 
   ! Call the computation of kinetic rates
@@ -151,13 +176,9 @@ do iel = 1, ncel
   else if (ichemistry.eq.2) then
     call kinetic_2(nrg,rk,temp,hspec,press,azi,1.0d0,iphotolysis)
   else if (ichemistry.eq.3) then
-    if (iaerosol.eq.1) then
-      call kinetic_siream(nrg,rk,temp,hspec,press,azi,1.0d0,iphotolysis)
-    else
-      call kinetic_3(nrg,rk,temp,hspec,press,azi,1.0d0,iphotolysis)
-    endif
+    call kinetic_3(nrg,rk,temp,hspec,press,azi,1.0d0,iphotolysis)
   else if (ichemistry.eq.4) then
-    call kinetic(nrg,rk,temp,hspec,press,azi,1.0d0,iphotolysis)
+    call kinetic_4(nrg,rk,temp,hspec,press,azi,1.0d0,iphotolysis)
   endif
 
   ! Storage of kinetic rates
@@ -168,10 +189,10 @@ do iel = 1, ncel
 enddo
 
 !--------
-! FORMATS
+! Formats
 !--------
 !----
-! END
+! End
 !----
 
 return
