@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -39,6 +39,7 @@
 !> \param[in]   dt      ratio of the local time step to the reference one
 !> \param[in]   rcodcl  Boundary conditions type
 !-------------------------------------------------------------------------------
+
 subroutine solvar ( temp , qv ,rom , dt, rcodcl )
 
 !===============================================================================
@@ -58,22 +59,17 @@ use ppppar
 use ppthch
 use ppincl
 use atincl
+use ctincl, only: cp_a, cp_v
 use atsoil
 use mesh
 use field
-
+use cs_c_bindings
 
 implicit none
 
-!===============================================================================
-
-integer idim2
-parameter ( idim2 = 26 )
 !==============================================================================
 
-
-!... declaration des variables externes
-
+! Arguments
 
 double precision rcodcl(nfabor,nvar,3)
 
@@ -81,8 +77,7 @@ double precision temp(ncelet)
 double precision qv(ncelet)
 double precision rom(ncelet),dt(ncelet)
 
-
-!... declaration des variables internes
+! Local variables
 
 integer ifac,iel,isol
 double precision rcodcx, rcodcy, rcodcz, rcodsn
@@ -104,18 +99,26 @@ double precision ray1,chas1,chal1,rapp1,premem
 double precision ray2,chas2,chal2,rapp2,secmem
 double precision w1min,w1max,w2min,w2max
 double precision r1,r2,tseuil,dum
+double precision cpvcpa
 
 double precision, dimension(:,:), pointer :: vel
+double precision, dimension(:), pointer :: cpro_met_p
 
 !===============================================================================
 
 ! Map field arrays
 call field_get_val_v(ivarfl(iu), vel)
 
+if (imeteo.eq.2) then
+  call field_get_val_s_by_name('meteo_pressure', cpro_met_p)
+endif
+
 !     ==========================
 !     1) initialisations locales
 !     ==========================
 
+cpvcpa = cp_v / cp_a
+
 b = 5.d0
 c = 5.d0
 d = 5.d0
@@ -134,7 +137,6 @@ precip = 0.d0
 tseuil = 16.d0 + tkelvi
 
 do isol = 1, nfmodsol
-  ifac = indsol(isol)
 
   tssol = solution_sol(isol)%temp_sol + tkelvi
   qvsol = solution_sol(isol)%total_water
@@ -142,8 +144,8 @@ do isol = 1, nfmodsol
   w2    = solution_sol(isol)%w2
 
   z0t    = solution_sol(isol)%constantes%rugthe
-  emis   = solution_sol(ifac)%constantes%emissi
-  albedo = solution_sol(ifac)%constantes%albedo
+  emis   = solution_sol(isol)%constantes%emissi
+  albedo = solution_sol(isol)%constantes%albedo
   csol   = solution_sol(isol)%constantes%csol
   veg    = solution_sol(isol)%constantes%vegeta
   c1w    = solution_sol(isol)%constantes%c1w
@@ -159,6 +161,7 @@ do isol = 1, nfmodsol
   !     2) calcul du vecteur vitesse tangent (identique a celui fait ds fr
   !     ==================================================================
 
+  ifac = indsol(isol)
   ! ---> NORMALE UNITAIRE
 
   rnx = surfbo(1,ifac)/surfbn(ifac)
@@ -203,15 +206,18 @@ do isol = 1, nfmodsol
     !     ====================================
 
     ! on impose t = tmer et hr = 100 %
-    esat = 610.78d0*exp(17.2694d0*tmer/(tmer + tkelvi-35.86d0))
+    esat  = cs_air_pwv_sat(tmer)
 
     if (imeteo.eq.0) then
       call atmstd(zreel,pres1,dum,dum)
-    else
+
+
+    else if (imeteo.eq.1) then
       call intprf                                                       &
-         !==========
-         (nbmett, nbmetm,                                               &
-          ztmet, tmmet, phmet, zreel, ttcabs, pres1)
+        (nbmett, nbmetm,                                               &
+        ztmet, tmmet, phmet, zreel, ttcabs, pres1)
+    else
+      pres1 = cpro_met_p(iel)
     endif
 
     tsplus = tmer + tkelvi
@@ -230,11 +236,12 @@ do isol = 1, nfmodsol
 
     if (imeteo.eq.0) then
       call atmstd(zreel,pres1,dum,dum)
+    else if (imeteo.eq.1) then
+      call intprf &
+        (nbmett, nbmetm,                                               &
+        ztmet, tmmet, phmet, zreel, ttcabs, pres1)
     else
-      call intprf                                                       &
-         !==========
-         (nbmett, nbmetm,                                               &
-          ztmet, tmmet, phmet, zreel, ttcabs, pres1)
+      pres1 = cpro_met_p(iel)
     endif
 
     tpot1 = solution_sol(isol)%tempp
@@ -243,8 +250,12 @@ do isol = 1, nfmodsol
     tpotv1 = tpot1*(1.d0 + (rvsra - 1.d0)*qvsol)
     tpotv2 = tpot2*(1.d0 + (rvsra - 1.d0)*qv(iel))
 
-    rib = 2.d0*abs(gz)*distb(ifac)*(tpotv2 - tpotv1)/(tpotv1 + tpotv2)      &
-         /vtmod/vtmod
+    if (abs(vtmod).le.epzero) then
+      rib = 2.d0*abs(gz)*distb(ifac)*(tpotv2 - tpotv1)/(tpotv1 + tpotv2)      &
+        /vtmod/vtmod
+    else
+      rib = 0.d0
+    endif
     if(rib.ge.0.d0) then
       fh = 1.d0/(1.d0 + 3.d0*b*rib*sqrt(1.d0 + d*rib))
     else
@@ -286,12 +297,12 @@ do isol = 1, nfmodsol
     !     7) calcul de la pression de vapeur saturante esat et de d(qsat)/dt
     !     ==================================================================
 
-    esat = 610.78d0*exp(17.2694d0*(tssol - tkelvi)/(tssol - 35.86d0))
+    esat  = cs_air_pwv_sat(tssol-tkelvi)
     rapsat = rvsra*pres1+esat*(1.d0-rvsra)
     qsat = esat/rapsat
-    cstder = 17.2694d0*(tkelvi - 35.86d0)
+    cstder = 17.438d0*(239.78d0)!TODO code it in  cs_air_prop.c
     dqsat = pres1*rvsra/rapsat/rapsat*cstder*esat                     &
-         /(tssol - 35.86d0)/(tssol - 35.86d0)
+         /(tssol -tkelvi+ 239.78d0)/(tssol -tkelvi+ 239.78d0)!TODO code it in  cs_air_prop.c
 
     !     ===========================================================
     !     8) calcul du premier membre de l'equation d'evolution de tssol
@@ -356,6 +367,8 @@ do isol = 1, nfmodsol
 
 enddo
 
+call cs_user_atmo_soil(temp, qv, rom, dt, rcodcl)
+
 return
 
 end subroutine solvar
