@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2015 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -161,17 +161,6 @@ contains
     ! Formats
     !--------
 
-#if defined(_CS_LANG_FR)
-
-1120 format (                                                     &
-'@                                                            ',/,&
-'@ @@ ERREUR : dans la fonction hypser                        ',/,&
-'@    ======                                                  ',/,&
-'@  Le parametre x doit verifier |x| < 1,  x = ',E12.5         ,/,&
-'@                                                            '  )
-
-#else
-
  1120 format (                                                    &
 '@                                                            ',/,&
 '@ @@ ERROR: in hypser function                               ',/,&
@@ -179,8 +168,6 @@ contains
 '@  The x parameter should verify |x| < 1,  x = ', E12.5       ,/,&
 '@                                                            '  )
 
-#endif
-
   end function hypser
 
   !=============================================================================
