@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,6 +27,8 @@ module albase
 
   !=============================================================================
 
+  use, intrinsic :: iso_c_binding
+
   implicit none
 
   !=============================================================================
@@ -36,11 +38,14 @@ module albase
   !> \addtogroup albase
   !> \{
 
-  !> Activates (=1) or not (=0), activate the ALE module
-  integer, save :: iale
+  !> Activates (=1 o) or not (=0), activate the ALE module:
+  !>  - 0: not activated
+  !>  - 1: legacy solver
+  !>  - 2: CDO solver
+  integer(c_int), pointer, save :: iale
   !> the number of sub-iterations of initialization of the fluid
   integer, save :: nalinf
-  !> maximum number of imlicitation iterations of of the structure displacement
+  !> maximum number of implicitation iterations of the structure displacement
   integer, save :: nalimx
   !> relative precision of implicitation of the structure displacement
   double precision, save :: epalim
@@ -54,8 +59,6 @@ module albase
   !> from the boundary conditions defined in cs user boundary conditions,
   !> or even from their coordinates.
   integer, allocatable, dimension(:) :: ialtyb
-  !> initial mesh coordinates
-  double precision, allocatable, dimension(:,:) :: xyzno0
   !> Pointer to field over vertices: mesh displacement
   integer, save :: fdiale
   !> \}
@@ -73,11 +76,7 @@ contains
 
     integer, intent(in) :: nfabor, nnod
 
-    if (iale.eq.1) then
-      allocate(xyzno0(3,nnod))
-    endif
-
-    if (iale.eq.1) then
+    if (iale.ge.1) then
       allocate(impale(nnod))
       allocate(ialtyb(nfabor))
     endif
@@ -90,11 +89,7 @@ contains
 
     use cplsat
 
-    if (iale.eq.1) then
-      deallocate(xyzno0)
-    endif
-
-    if (iale.eq.1) then
+    if (iale.ge.1) then
       deallocate(impale)
       deallocate(ialtyb)
     endif
@@ -103,4 +98,21 @@ contains
 
   !=============================================================================
 
+  subroutine map_ale
+
+    use, intrinsic :: iso_c_binding
+    use cs_c_bindings
+
+    ! Local variables
+
+    type(c_ptr) :: c_iale
+
+    call cs_f_ale_get_pointers(c_iale)
+
+    call c_f_pointer(c_iale, iale)
+
+  end subroutine map_ale
+
+  !=============================================================================
+
 end module albase
