@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -47,6 +47,7 @@
 !>                               - 13 Dirichlet for the advection operator and
 !>                                    Neumann for the diffusion operator
 !> \param[in,out] impale        imposed displacement indicator
+!> \param[in]     init          partial treatment (before time loop) if true
 !> \param[in]     dt            time step (per cell)
 !> \param[in,out] rcodcl        boundary condition values:
 !>                               - rcodcl(1) value of the dirichlet
@@ -62,13 +63,12 @@
 !>                                 -# for a scalar \f$ cp \left( K +
 !>                                     \dfrac{K_T}{\sigma_T} \right)
 !>                                     \grad T \cdot \vect{n} \f$
-!> \param[in]     xyzno0        initial mesh nodes coordinates
 !______________________________________________________________________________
 
 subroutine altycl &
- ( itypfb , ialtyb , icodcl , impale ,                            &
+ ( itypfb , ialtyb , icodcl , impale , init ,                    &
    dt     ,                                                      &
-   rcodcl , xyzno0)
+   rcodcl )
 
 !===============================================================================
 ! Module files
@@ -84,7 +84,7 @@ use entsor
 use parall
 use mesh
 use field
-use albase, only: fdiale
+use albase, only: fdiale, iale
 use cs_c_bindings
 
 !===============================================================================
@@ -97,9 +97,10 @@ integer          itypfb(nfabor)
 integer          ialtyb(nfabor), icodcl(nfabor,nvar)
 integer          impale(nnod)
 
+logical          init
+
 double precision dt(ncelet)
 double precision rcodcl(nfabor,nvar,3)
-double precision xyzno0(3,nnod)
 
 ! Local variables
 
@@ -110,6 +111,8 @@ double precision srfbnf, rnx, rny, rnz
 double precision rcodcx, rcodcy, rcodcz, rcodsn
 
 double precision, dimension(:,:), pointer :: disale
+double precision, dimension(:,:), pointer :: xyzno0
+double precision, allocatable, dimension(:,:) :: b_fluid_vel
 
 !===============================================================================
 
@@ -130,6 +133,7 @@ icoder(3) = -1
 icoder(4) = -1
 
 call field_get_val_v(fdiale, disale)
+call field_get_val_v_by_name("vtx_coord0", xyzno0)
 
 ! Set to 0 non specified RCODCL arrays
 do ifac = 1, nfabor
@@ -144,8 +148,32 @@ enddo
 !===============================================================================
 ! 2. Check the consistency of BC types (IALTYB) given in USALCL
 !===============================================================================
-!  (valeur 0 autorisee)
 
+! When using CDO solver, no need of checks.
+if (iale.eq.2) then
+
+  allocate(b_fluid_vel(3, nfabor))
+
+  do ifac = 1, nfabor
+    b_fluid_vel(1, ifac) = 0.d0
+    b_fluid_vel(2, ifac) = 0.d0
+    b_fluid_vel(3, ifac) = 0.d0
+  enddo
+
+  call cs_ale_update_bcs(ialtyb, b_fluid_vel)
+
+  ! Copy back to deprecated rcodcl
+  do ifac = 1, nfabor
+    rcodcl(ifac, iuma, 1) = b_fluid_vel(1, ifac)
+    rcodcl(ifac, ivma, 1) = b_fluid_vel(2, ifac)
+    rcodcl(ifac, iwma, 1) = b_fluid_vel(3, ifac)
+  enddo
+
+  goto 100
+
+endif
+
+!  (valeur 0 autorisee)
 do ifac = 1, nfabor
   if (ialtyb(ifac).ne.0      .and.                                &
       ialtyb(ifac).ne.ibfixe .and.                                &
@@ -273,6 +301,9 @@ enddo
 ! 4. Check ICODCL consistency
 !===============================================================================
 
+! When called before time loop, some values are not yet available.
+if (init .eqv. .true.) return
+
 do ifac = 1, nfabor
 
   if (icodcl(ifac,iuma).ne.1 .and.                                &
@@ -356,6 +387,8 @@ endif
 ! 5. Fluid velocity BCs for walls and symmetries (due to mesh movment)
 !===============================================================================
 
+100  continue
+
 ! Pour les symetries on rajoute toujours la vitesse de maillage, car on
 !   ne conserve que la vitesse normale
 ! Pour les parois, on prend la vitesse de maillage si l'utilisateur n'a
@@ -368,7 +401,7 @@ do ifac = 1, nfabor
 
   if (ialtyb(ifac).eq.ivimpo) then
 
-    !Warning: only the normal component is kept in clsyvt
+    ! Warning: only the normal component is kept in clsyvt
     if (itypfb(ifac).eq.isymet) then
       rcodcl(ifac,iu,1) = rcodcl(ifac,iuma,1)
       rcodcl(ifac,iv,1) = rcodcl(ifac,ivma,1)
@@ -422,63 +455,6 @@ enddo
 ! Formats
 !========
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(                                                     &
-'@'                                                            ,/,&
-'@ METHODE ALE'                                                ,/,&
-'@'                                                            ,/,&
-'@ Au moins une face de bord a un type de bord non reconnu.'   ,/,&
-'@    Le calcul ne sera pas execute.'                          ,/,&
-'@'                                                            ,/,&
-'@ Verifier les conditions aux limites dans l''Interface'      ,/,&
-'@    ou dans usalcl.f90.'                                     ,/,&
-'@'                                                              )
- 2000 format(                                                     &
-'@'                                                            ,/,&
-'@ METHODE ALE'                                                ,/,&
-'@'                                                            ,/,&
-'@ Au moins une face de bord a un type de condition a la'      ,/,&
-'@    limite (icodcl) non reconnu pour la vitesse de maillage' ,/,&
-'@    selon' ,a1,' (',a4,').'                                  ,/,&
-'@'                                                            ,/,&
-'@ Les seules valeurs autorisees pour icodcl sont'             ,/,&
-'@   1 : Dirichlet'                                            ,/,&
-'@   3 : Neumann'                                              ,/,&
-'@   4 : Glissement'                                           ,/,&
-'@'                                                              )
- 3000 format(                                                     &
-'@'                                                            ,/,&
-'@ METHODE ALE'                                                ,/,&
-'@'                                                            ,/,&
-'@ Incoherence dans les types de conditions a la limite'       ,/,&
-'@   pour la vitesse de maillage.'                              ,/,&
-'@'                                                            ,/,&
-'@ Au moins une face de bord a les conditions aux limites'     ,/,&
-'@   suivantes :'                                              ,/,&
-'@'                                                            ,/,&
-'@   itypcl : ',i10                                            ,/,&
-'@   icodcl(.,iuma) : ', i10                                   ,/,&
-'@   icodcl(.,ivma) : ', i10                                   ,/,&
-'@   icodcl(.,iwma) : ', i10                                   ,/,&
-'@'                                                            ,/,&
-'@ Si une composante est traitee en glissement (icodcl=4),'    ,/,&
-'@   toutes les composantes doivent etre traitees en'          ,/,&
-'@   glissement.'                                              ,/,&
-'@'                                                              )
- 4000 format(                                                     &
-'@'                                                            ,/,&
-'@ METHODE ALE'                                                ,/,&
-'@'                                                            ,/,&
-'@ Incoherence dans les conditions aux limites pour la vitesse',/,&
-'@   de maillage'                                              ,/,&
-'@   (cf. message(s) ci-dessus)'                               ,/,&
-'@'                                                            ,/,&
-'@ Verifier dans l''interface ou dans usalcl.f90'              ,/,&
-'@'                                                              )
-
-#else
-
  1000 format(                                                     &
 '@'                                                            ,/,&
 '@ ALE METHOD'                                                 ,/,&
@@ -531,7 +507,5 @@ enddo
 '@ Check boundary conditions in the GUI or in usalcl.f90.'     ,/,&
 '@'                                                              )
 
-#endif
-
 return
 end subroutine
