@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,26 +20,19 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine caltri
-!================
+!> \file caltri.f90
+!> \brief Main time loop.
+!>
+!------------------------------------------------------------------------------
 
-!===============================================================================
-! Purpose:
-! --------
-
-! Main solver subroutine (read, time loop, write)
-
-!-------------------------------------------------------------------------------
+!------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!__________________.____._____.________________________________________________.
+!------------------------------------------------------------------------------
+!   mode          name          role
+!------------------------------------------------------------------------------
+!______________________________________________________________________________
 
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
+subroutine caltri
 
 !===============================================================================
 ! Module files
@@ -62,8 +55,6 @@ use ppincl
 use coincl
 use cpincl
 use lagran
-use vorinc
-use ihmpre
 use radiat
 use cplsat
 use atincl
@@ -74,11 +65,11 @@ use field
 use post
 use atchem
 use atimbr
-use siream
 use ptrglo
 use turbomachinery
 use cs_c_bindings
 use cs_f_interfaces
+use cdomod
 
 use, intrinsic :: iso_c_binding
 
@@ -96,16 +87,14 @@ implicit none
 
 ! Local variables
 
-logical(kind=c_bool) mesh_modified
+logical(kind=c_bool) :: mesh_modified, log_active
+integer(c_int) :: ierr
 
-integer          modhis, iappel, modntl, iisuit, imrgrl
+integer          modhis, iappel, iisuit
 integer          iel
-
 integer          inod   , idim, ifac
 integer          itrale , ntmsav
-
-integer          nent
-
+integer          iterns
 integer          stats_id, restart_stats_id, lagr_stats_id, post_stats_id
 
 double precision titer1, titer2
@@ -113,11 +102,18 @@ double precision titer1, titer2
 integer          ivoid(1)
 double precision rvoid(1)
 
+logical          legacy_mass_st_zones
+
 double precision, save :: ttchis
 
 double precision, pointer, dimension(:)   :: dt => null()
 double precision, pointer, dimension(:)   :: porosi => null()
 double precision, pointer, dimension(:,:) :: disale => null()
+double precision, dimension(:,:), pointer :: xyzno0 => null()
+
+integer, allocatable, dimension(:,:) :: icodcl
+integer, allocatable, dimension(:) :: isostd
+double precision, allocatable, dimension(:,:,:) :: rcodcl
 
 !===============================================================================
 ! Interfaces
@@ -125,6 +121,30 @@ double precision, pointer, dimension(:,:) :: disale => null()
 
 interface
 
+  subroutine condli_ini &
+  ( nvar   , nscal  ,                                              &
+    itrale ,                                                       &
+    icodcl , isostd ,                                              &
+    dt     , rcodcl )
+
+    use mesh, only: nfac, nfabor
+
+    implicit none
+
+    integer          nvar, nscal
+    integer          itrale
+
+    integer, dimension(nfabor,nvar) :: icodcl
+    integer, dimension(nfabor+1) :: isostd
+
+    double precision, dimension(nfabor,nvar,3) :: rcodcl
+
+    double precision, pointer, dimension(:)   :: dt
+
+  end subroutine condli_ini
+
+  !=============================================================================
+
   subroutine tridim(itrale, nvar, nscal, dt)
 
     implicit none
@@ -144,19 +164,97 @@ interface
 
   !=============================================================================
 
-  subroutine cs_gui_postprocess_activate()  &
-    bind(C, name='cs_gui_postprocess_activate')
+  subroutine cs_time_step_increment(dt) &
+    bind(C, name='cs_time_step_increment')
     use, intrinsic :: iso_c_binding
     implicit none
-  end subroutine cs_gui_postprocess_activate
+    real(kind=c_double), value :: dt
+  end subroutine cs_time_step_increment
 
   !=============================================================================
 
-  subroutine cs_gui_profile_output()  &
-    bind(C, name='cs_gui_profile_output')
+  subroutine cs_les_inflow_initialize()  &
+    bind(C, name='cs_les_inflow_initialize')
     use, intrinsic :: iso_c_binding
     implicit none
-  end subroutine cs_gui_profile_output
+  end subroutine cs_les_inflow_initialize
+
+  !=============================================================================
+
+  subroutine cs_meg_post_activate()  &
+    bind(C, name='cs_meg_post_activate')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_meg_post_activate
+
+  !=============================================================================
+
+  subroutine cs_restart_map_build()  &
+    bind(C, name='cs_restart_map_build')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_restart_map_build
+
+  !=============================================================================
+
+  subroutine cs_restart_map_free()  &
+    bind(C, name='cs_restart_map_free')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_restart_map_free
+
+  !=============================================================================
+
+  subroutine cs_restart_lagrangian_checkpoint_read()  &
+    bind(C, name='cs_restart_lagrangian_checkpoint_read')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_restart_lagrangian_checkpoint_read
+
+  !=============================================================================
+
+  subroutine cs_restart_lagrangian_checkpoint_write()  &
+    bind(C, name='cs_restart_lagrangian_checkpoint_write')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_restart_lagrangian_checkpoint_write
+
+  !=============================================================================
+
+  subroutine cs_les_synthetic_eddy_restart_read()  &
+    bind(C, name='cs_les_synthetic_eddy_restart_read')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_les_synthetic_eddy_restart_read
+
+  !=============================================================================
+
+  subroutine cs_les_synthetic_eddy_restart_write()  &
+    bind(C, name='cs_les_synthetic_eddy_restart_write')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_les_synthetic_eddy_restart_write
+
+  !=============================================================================
+
+  subroutine cs_volume_mass_injection_build_lists(ncetsm, icetsm, izctsm) &
+    bind(C, name='cs_volume_mass_injection_build_lists')
+    use, intrinsic :: iso_c_binding
+    implicit none
+    integer(kind=c_int), value :: ncetsm
+    integer(kind=c_int), dimension(*), intent(out) :: icetsm, izctsm
+  end subroutine cs_volume_mass_injection_build_lists
+
+  !=============================================================================
+
+  function cs_runaway_check() result(ierr) &
+    bind(C, name='cs_runaway_check')
+    use, intrinsic :: iso_c_binding
+    implicit none
+    integer(c_int) :: ierr
+  end function cs_runaway_check
+
+  !=============================================================================
 
 end interface
 
@@ -189,19 +287,6 @@ if (iilagr.gt.0) then
                                 "particle displacement")
 endif
 
-!===============================================================================
-! Geometry
-!===============================================================================
-
-! Filter extended neighborhood for least-squares gradients
-
-imrgrl = abs(imrgra)
-imrgrl = modulo(imrgrl,10)
-
-if (imrgrl.eq.3 .or. imrgrl.eq.6 .or. imrgrl.eq.9) then
-  call redvse(anomax)
-endif
-
 !===============================================================================
 ! End of modules initialization
 !===============================================================================
@@ -216,8 +301,9 @@ call initi2
 ! First pass for every subroutine
 iappel = 1
 
+legacy_mass_st_zones = .false.
+
 ! Allocate temporary arrays for zones definition
-allocate(izcpdc(ncel))
 allocate(izctsm(ncel))
 allocate(izftcd(ncel)) ! should be in init_pcond only
 
@@ -229,6 +315,7 @@ allocate(izftcd(ncel)) ! should be in init_pcond only
 
 ncetsm = volume_zone_n_type_cells(VOLUME_ZONE_MASS_SOURCE_TERM)
 ncepdc = volume_zone_n_type_cells(VOLUME_ZONE_HEAD_LOSS)
+nfbpcd = 0
 
 ! Total number of cells with head-loss
 ncpdct = ncepdc
@@ -266,20 +353,27 @@ endif
 ! Mass source terms
 ! -----------------
 
-call cs_user_mass_source_terms &
-( nvar   , nscal  , ncepdc ,                                     &
-  ncetsm , iappel ,                                              &
-  ivoid  ,                                                       &
-  ivoid  , ivoid  , izctsm ,                                     &
-  rvoid  ,                                                       &
-  ckupdc , rvoid  )
-
 ! Total number of cells with mass source term
 nctsmt = ncetsm
 if (irangp.ge.0) then
   call parcpt(nctsmt)
 endif
 
+! If already defined through zones, do not use first (counting)
+! call for legacy user mass source terms.
+if (nctsmt.eq.0) then
+  call cs_user_mass_source_terms(nvar, nscal, ncepdc, ncetsm, 1,   &
+                                 ivoid, ivoid, ivoid, izctsm,      &
+                                 rvoid, ckupdc, rvoid )
+  nctsmt = ncetsm
+  if (irangp.ge.0) then
+    call parcpt(nctsmt)
+  endif
+  if (nctsmt.gt.0) then
+    legacy_mass_st_zones = .true.
+  endif
+endif
+
 if (nctsmt.gt.0) then
   write(nfecra,2002) nctsmt
   write(nfecra,3000)
@@ -326,27 +420,11 @@ endif
 call cs_1d_wall_thermal_check(iappel, isuit1)
 
 ! Free memory if relevant
-if (ncpdct.eq.0) deallocate(izcpdc)
 if (nctsmt.eq.0) deallocate(izctsm)
 if (nftcdt.eq.0) deallocate(izftcd)
 if (nfpt1t.eq.0) call cs_1d_wall_thermal_finalize
 
 ! Formats
-#if defined(_CS_LANG_FR)
- 2001 format(                                                   &
- /,/,'TRAITEMENT DES PERTES DE CHARGES ACTIVE ',/,              &
- '                 SUR  UN TOTAL DE NCEPDC = ',I10,' CELLULES',/)
- 2002 format(                                          &
- /,/,'TRAITEMENT DES SOURCES DE MASSE ACTIVE ',/,      &
-   '                 SUR  UN TOTAL DE ',I10,' CELLULES')
- 2003 format(                                           &
- /,/,'TRAITEMENT DES SOURCES DE CONDENSATION ACTIVE ',/,&
-   '                 SUR  UN TOTAL DE ',I10,' CELLULES')
- 2004 format(                                                     &
-    /,'MODULE THERMIQUE 1D EN PAROI ACTIVE     ',/,&
-      '   SUR UN TOTAL DE ',I10,' FACES DE BORD',/,               &
-      '   (',I10,' FACES DE BORD EN LOCAL)',/)
-#else
  2001 format(                                               &
  /,/,'HEAD LOSS TERMS TREATMENT ACTIVATED ',/,              &
  '                 ON   A TOTAL OF NCEPDC = ',I10,' CELLS',/)
@@ -360,8 +438,6 @@ if (nfpt1t.eq.0) call cs_1d_wall_thermal_finalize
  /,'1D-WALL THERMAL MODULE ACTIVATED ',/,     &
    '   ON A TOTAL OF ',I10,' BOUNDARY FACES',/,             &
    '   (',I10,' LOCAL BOUNDARY FACES)',/)
-#endif
-
 
 !===============================================================================
 ! Memory management
@@ -374,30 +450,18 @@ call init_aux_arrays(ncelet, nfabor)
 call turbomachinery_init
 
 if (ippmod(iatmos).ge.0) then
-
-  call init_meteo
-
-  if (imbrication_flag) then
-    call activate_imbrication
-  endif
-
-  call cs_at_data_assim_build_ops
+  call init_atmo_autom(nfabor)
 
   if (ifilechemistry.ge.1) then
-    call init_chemistry
+    call init_chemistry_reacnum
   endif
-
-  if (iaerosol.eq.1) then
-    call init_aerosols
-  endif
-
 endif
 
 if (ippmod(icompf).ge.0) then
   call init_compf (nfabor)
 endif
 
-if (iale.eq.1) then
+if (iale.ge.1) then
   call init_ale (nfabor, nnod)
 endif
 
@@ -406,7 +470,7 @@ if (ncpdct.gt.0) then
 endif
 
 if (nctsmt.gt.0) then
-  call init_tsma ( nvar )
+  call init_tsma (nvar)
 endif
 
 if (nftcdt.gt.0) then
@@ -427,6 +491,10 @@ if (iilagr.gt.0) then
   call init_lagr_arrays(tslagr)
 endif
 
+if (i_les_balance.gt.0) then
+  call les_balance_create
+endif
+
 !===============================================================================
 ! Default initializations
 !===============================================================================
@@ -442,9 +510,16 @@ call iniva0(nscal)
 ! Compute the porosity if needed
 if (iporos.ge.1) then
 
-  if (iihmpr.eq.1) then
-    call uiporo
+  ! Make fluid surfaces of mesh quantity point to the created fields
+  call cs_porous_model_set_has_disable_flag(1)
+  call cs_porous_model_init_fluid_quantities()
+
+  ! Compute porosity from scan
+  if (compute_porosity_from_scan) then
+    call cs_compute_porosity_from_scan()
   endif
+
+  call uiporo
   call usporo
 
   ! C version
@@ -457,25 +532,25 @@ if (iporos.ge.1) then
   endif
 
   do iel = 1, ncelet
+    ! Penalisation of solid cells
+    if (porosi(iel).lt.epzero) then
+      porosi  (iel) = 0.d0
+      isolid_0(iel) = 1
+    endif
     cell_f_vol(iel) = volume(iel) * porosi(iel)
   enddo
 
   ! For integral formulation, in case of 0 fluid volume, clip fluid faces
   if (iporos.eq.3) then
     do ifac = 1, nfac
-      !TODO compute i_f_face_factor with porosi AND fluid surface and surface: epsilon_i*surface/f_surface
-      if (porosi(ifacel(1, ifac)).lt.epzero) then
-        porosi(ifacel(1, ifac)) = 0.d0
-        isolid_0(ifacel(1, ifac)) = 1
-
+      !TODO compute i_f_face_factor with porosi AND fluid surface and surface:
+      ! epsilon_i*surface/f_surface
+      if (isolid_0(ifacel(1, ifac)) .eq. 1) then
         suffac(1, ifac) = 0.d0
         suffac(2, ifac) = 0.d0
         suffac(3, ifac) = 0.d0
         suffan(ifac) = 0.d0
-      else if (porosi(ifacel(2, ifac)).lt.epzero) then
-        porosi(ifacel(2, ifac)) = 0.d0
-        isolid_0(ifacel(2, ifac)) = 1
-
+      else if (isolid_0(ifacel(2, ifac)).eq.1) then
         suffac(1, ifac) = 0.d0
         suffac(2, ifac) = 0.d0
         suffac(3, ifac) = 0.d0
@@ -484,11 +559,9 @@ if (iporos.ge.1) then
     enddo
 
     do ifac = 1, nfabor
-      !TODO compute i_f_face_factor with porosi AND fluid surface and surface: epsilon_i*surface/f_surface
-      if (porosi(ifabor(ifac)).lt.epzero) then
-        porosi(ifabor(ifac)) = 0.d0
-        isolid_0(ifabor(ifac)) = 1
-
+      !TODO compute i_f_face_factor with porosi AND fluid surface and surface:
+      ! epsilon_i*surface/f_surface
+      if (isolid_0(ifabor(ifac)) .eq. 1) then
         suffbo(1, ifac) = 0.d0
         suffbo(2, ifac) = 0.d0
         suffbo(3, ifac) = 0.d0
@@ -497,6 +570,16 @@ if (iporos.ge.1) then
     enddo
   endif
 
+  call cs_f_mesh_quantities_fluid_vol_reductions
+
+endif
+
+!===============================================================================
+! Initialization for the atmospheric soil model
+!===============================================================================
+
+if (ippmod(iatmos).ge.2.and.iatsoil.eq.1) then
+  call atmsol()
 endif
 
 !==============================================================================
@@ -525,13 +608,18 @@ if (nftcdt.gt.0) then
 
   ! the Condensation model coupled with a 0-D thermal model
   ! to take into account the metal mass structures effects.
-  if(itagms.eq.1) then
-
+  if (itagms.eq.1) then
     call init_tagms
   endif
 
 endif
 
+!===============================================================================
+! Initialization for the Synthetic turbulence Inlets
+!===============================================================================
+
+call cs_les_inflow_initialize
+
 !===============================================================================
 ! Possible restart
 !===============================================================================
@@ -545,12 +633,15 @@ if (isuite.eq.1) then
 
   call timer_stats_start(restart_stats_id)
 
+  call cs_restart_map_build
+
   call lecamo
 
   ! Using ALE, geometric parameters must be recalculated
-  if (iale.eq.1) then
+  if (iale.ge.1) then
 
     call field_get_val_v(fdiale, disale)
+    call field_get_val_v_by_name("vtx_coord0", xyzno0)
 
     do inod = 1, nnod
       do idim = 1, ndim
@@ -565,6 +656,29 @@ if (isuite.eq.1) then
 
   endif
 
+  ! Radiative module restart */
+  if (iirayo.gt.0) then
+    call cs_rad_transfer_read
+  endif
+
+  ! Lagrangian module restart (particles) */
+  if (iilagr.gt.0) then
+    call cs_restart_lagrangian_checkpoint_read()
+  endif
+
+  call cs_les_synthetic_eddy_restart_read
+
+  ! TODO
+  ! cs_restart_map_free may not be called yet, because
+  ! cs_lagr_solve_initialize and the first call of cs_lagr_solve_time_step
+  ! may also need restart data for particles and statistics respectively.
+  ! This should be solved by moving the corresponding stages at least to
+  ! cs_lagr_solve_initialize sor as to free mapping data before the time loop.
+
+  if (iilagr.lt.1) then
+    call cs_restart_map_free
+  endif
+
   call timer_stats_stop(restart_stats_id)
 
 endif
@@ -576,21 +690,12 @@ endif
 
 call inivar(nvar, nscal)
 
-!===============================================================================
-! Radiative transfer: possible restart
-!===============================================================================
-
-if (iirayo.gt.0) then
-   call cs_rad_transfer_read
+if (icdo.ge.1) then ! CDO mode
+  call cs_f_initialize_cdo_systems
 endif
 
-!===============================================================================
-! Initialize particles for Lagrangian module
-!===============================================================================
-
-if (iilagr.gt.0) then
-  call laglec()
-endif
+iterns = -1
+call phyvar(nvar, nscal, iterns, dt)
 
 !===============================================================================
 ! Initializations for the 1D thermal wall module
@@ -620,36 +725,33 @@ if (nfpt1t.gt.0) then
 
   else
 
-!     Creation du maillage, initialisation de la temperature.
+    ! Create mesh, initialize temperature.
     call cs_1d_wall_thermal_mesh_create
 
   endif
 
 endif
 
-!     Les infos sur l'epaisseur de la paroi, le nombre de points de
-!     discretisation et la raison geometrique ont ete transmises a
-!     la structure C. Elles sont maintenant inutiles dans le Fortran.
-!     -> on libere la memoire.
+! First pass for the BCs:
+! - initilalize itypfb, reference pressure point...
+!--------------------------------------------------
 
-!===============================================================================
-! Initialization for the Synthetic turbulence Inlets
-!===============================================================================
+! Deprecated, only for compatibility reason
+nvarcl = nvar
 
-nent = 0
+allocate(icodcl(nfabor,nvar))
+allocate(rcodcl(nfabor,nvar,3))
+allocate(isostd(nfabor+1))
 
-call defsyn(nent)
+! First pass for initialization BC types
+! -- Couplage Code_Saturne/Code_Saturne
 
-if (isuisy.eq.1) then
-  call lecsyn('les_inflow'//c_null_char)
-endif
+call cscini(nvar)
+call condli_ini(nvar, nscal, itrale, icodcl, isostd, dt, rcodcl)
 
-! ATMO MODULE : INITIALIZATION FOR THE SOIL MODEL (ippmo(iatmos) = 2)
-!===============================================================================
-
-if (ippmod(iatmos).ge.2.and.iatsoil.eq.1) then
-  call atmsol()
-endif
+deallocate(icodcl)
+deallocate(rcodcl)
+deallocate(isostd)
 
 !===============================================================================
 ! Arrays for time block, to discard afterwards
@@ -667,64 +769,29 @@ if (ncpdct.gt.0) then
 
 endif
 
-! On appelle cs_user_mass_source_terms lorsqu'il y a sur un processeur au moins
-!     des cellules avec terme source de masse.
-!     On ne fait que remplir le tableau d'indirection des cellules
-!     On appelle cependant cs_user_mass_source_terms avec tous les processeurs,
-!     au cas ou l'utilisateur aurait mis en oeuvre des operations globales.
+! Build volume mass injection cell lists when present on at least one rank.
+! This is a collective call for consistency and in case the user requires it.
 
 if (nctsmt.gt.0) then
 
-  call volume_zone_select_type_cells(VOLUME_ZONE_MASS_SOURCE_TERM, icetsm)
-
-  iappel = 2
-  call cs_user_mass_source_terms &
-( nvar   , nscal  , ncepdc ,                                     &
-  ncetsm , iappel ,                                              &
-  icepdc ,                                                       &
-  icetsm , itypsm , izctsm ,                                     &
-  dt     ,                                                       &
-  ckupdc , smacel )
+  if (.not. legacy_mass_st_zones) then
+    call cs_volume_mass_injection_build_lists(ncetsm, icetsm, izctsm)
+  else
+    call cs_user_mass_source_terms(nvar, nscal, ncepdc, ncetsm, 2,   &
+                                   icepdc, icetsm, itypsm, izctsm,   &
+                                   dt, ckupdc, smacel)
+  endif
 
 endif
 
-! -- Methode des vortex pour la L.E.S.
-!    (dans verini on s'est deja assure que ITYTUR=4 si IVRTEX=1)
-
-if (ivrtex.eq.1) then
-
-  allocate(irepvo(nfabor))
-
-  call vorin0(nfabor)
-
-  iappel = 1
-
-  call usvort &
- ( nvar   , nscal  , iappel ,                                     &
-   dt     )
-
-  call vorver (nfabor, iappel)
-
-  call init_vortex
-
-  call vorpre
-
-endif
-
-! -- Fin de zone Methode des vortex pour la L.E.S.
-
 ! -- Structures mobiles en ALE
 
-if (iale.eq.1) then
+if (iale.ge.1) then
   call strini(dt)
 endif
 
 ! -- Fin de zone Structures mobiles en ALE
 
-! -- Couplage Code_Saturne/Code_Saturne
-
-call cscini(nvar)
-
 ! Lagrangian initialization
 
 if (iilagr.gt.0) then
@@ -737,6 +804,18 @@ if (iilagr.gt.0) then
 
 endif
 
+! CDO module (user-defined equations)
+!====================================
+
+if (icdo.eq.1) then
+   ! FV and CDO activated
+   call cs_f_cdo_solve_steady_state_domain
+endif
+
+! Logging of initial values
+
+call log_iteration
+
 !===============================================================================
 ! Start of time loop
 !===============================================================================
@@ -750,10 +829,6 @@ if (iturbo.eq.2)  ttcmob = ttpmob
 
 write(nfecra,3000)
 
-if(inpdt0.eq.1) then
-  ntmabs = ntcabs
-endif
-
 !     Nb d'iter ALE (nb relatif a l'execution en cours)
 !     Si ITALIN=1, on fait une iteration d'initialisation
 !     (si ITALIN=-999, c'est qu'on a fait une suite de calculs
@@ -778,7 +853,7 @@ if (itrale.gt.0) then
     call cplsyn (ntmabs, ntcabs, dtref)
   endif
 
-  if (ntmabs .eq. ntcabs .and. inpdt0.eq.0) then
+  if (ntmabs .eq. ntcabs .and. ntmabs.gt.ntpabs) then
     call csexit (1)
   endif
 
@@ -799,15 +874,18 @@ call timer_stats_stop(post_stats_id)
 
  100  continue
 
-if (inpdt0.eq.0 .and. itrale.gt.0) then
-  ntcabs = ntcabs + 1
+if (ttmabs.gt.0 .and. ttmabs.gt.ttcabs) then
+  ntmabs = ntcabs + (ttmabs-ttcabs)/dtref
+  if (ntmabs.le.ntcabs) ntmabs = ntcabs + 1
+endif
+
+if (itrale.gt.0 .and. ntmabs.gt.ntpabs) then
   call timer_stats_increment_time_step
-  if(idtvar.eq.0.or.idtvar.eq.1) then
-    ttcabs = ttcabs + dt(1)
+  if (idtvar.eq.0.or.idtvar.eq.1) then
+    call cs_time_step_increment(dt(1))
   else
-    ttcabs = ttcabs + dtref
+    call cs_time_step_increment(dtref)
   endif
-  write(nfecra,3001) ttcabs,ntcabs
   if (iturbo.eq.2) then
     if(idtvar.eq.0.or.idtvar.eq.1) then
       ttcmob = ttcmob + dt(1)
@@ -817,7 +895,6 @@ if (inpdt0.eq.0 .and. itrale.gt.0) then
   endif
 endif
 
-
 !===============================================================================
 ! Step forward in time
 !===============================================================================
@@ -830,19 +907,41 @@ if (      (idtvar.eq.0 .or. idtvar.eq.1)                          &
   ntmabs = ntcabs
 endif
 
+! Check for runaway (diverging) computation
+ierr = cs_runaway_check()
+
+! Set default logging (always log 10 first iterations and last one=)
+log_active = .false.
+if (ntcabs - ntpabs.le.10 .or. ntcabs.eq.ntmabs) then
+  log_active = .true.
+else if (ntlist.gt.0) then
+  if (mod(ntcabs,ntlist) .eq. 0) log_active = .true.
+endif
+call cs_log_default_activate(log_active)
+
+if (ntmabs.gt.ntpabs .and. itrale.gt.0) then
+  if (log_active) then
+    write(nfecra,3001) ttcabs, ntcabs
+  endif
+endif
+
 mesh_modified = .false.
 call cs_volume_zone_build_all(mesh_modified)
 call cs_boundary_zone_build_all(mesh_modified)
 
 call dmtmps(titer1)
 
-call tridim                                                       &
- ( itrale ,                                                       &
-   nvar   , nscal  ,                                              &
-   dt     )
+call tridim(itrale, nvar, nscal, dt)
 
+if (ntmabs.gt.ntpabs .and. itrale.gt.0) then
 
-if (inpdt0.eq.0 .and. itrale.gt.0) then
+  ! CDO module (user-defined equations)
+  !====================================
+
+  if (icdo.eq.1) then
+     ! FV and CDO activated
+     call cs_f_cdo_solve_unsteady_state_domain
+  endif
 
   ! Lagrangian module
   !==================
@@ -857,13 +956,32 @@ if (inpdt0.eq.0 .and. itrale.gt.0) then
 
   endif
 
+  ! Update gradients needed in LES balance computation
+  !=============================================================================
+
+  if (i_les_balance.gt.0) then
+    call les_balance_update_gradients
+  endif
+
   ! Compute temporal means (accumulation)
-  !======================================
+  !=============================================================================
 
   call time_moment_update_all
 
 endif
 
+!===============================================================================
+! Update mesh (ALE)
+!===============================================================================
+
+if (iale.ge.1 .and. ntmabs.gt.ntpabs) then
+
+  if (itrale.eq.0 .or. itrale.gt.nalinf) then
+    call cs_ale_update_mesh(itrale)
+  endif
+
+endif
+
 !===============================================================================
 ! Optional processing by user
 !===============================================================================
@@ -874,31 +992,18 @@ if (itrale.gt.0) then
 
   ! 1D profiles postprocessing output
 
-  if (iihmpr.eq.1) then
-    call cs_gui_profile_output()
-    call uiexop()
-  endif
+  call uiexop()
 
   call cs_f_user_extra_operations(nvar, nscal, dt)
 
   call user_extra_operations()
 
-  call timer_stats_stop(post_stats_id)
-
-endif
-
-!===============================================================================
-! Update mesh (ALE)
-!===============================================================================
-
-if (iale.eq.1 .and. inpdt0.eq.0) then
-
-  if (itrale.eq.0 .or. itrale.gt.nalinf) then
-
-    call cs_ale_update_mesh(itrale, xyzno0)
-
+  if (i_les_balance.gt.0) then
+    call les_balance_compute
   endif
 
+  call timer_stats_stop(post_stats_id)
+
 endif
 
 !===============================================================================
@@ -932,7 +1037,7 @@ if (iisuit.eq.1) then
   if(ntcabs.lt.ntmabs) then
     write(nfecra,3020) ntcabs, ttcabs
   else if(ntcabs.eq.ntmabs) then
-    write(nfecra,3021)ntcabs,ttcabs
+    write(nfecra,3021) ntcabs,ttcabs
   endif
 
   call ecrava
@@ -945,20 +1050,25 @@ if (iisuit.eq.1) then
     call cs_1d_wall_thermal_write
   endif
 
-  if (nent.gt.0) then
-    call ecrsyn('les_inflow'//c_null_char)
-  endif
+  call cs_les_synthetic_eddy_restart_write
 
   if (iilagr.gt.0) then
-    call lagout()
+    call cs_restart_lagrangian_checkpoint_write()
   endif
 
   if (iirayo.gt.0) then
     call cs_rad_transfer_write
   endif
 
+  if (i_les_balance.gt.0) then
+    call les_balance_write_restart
+  endif
+
   call stusui
 
+  ! Remove all unnecessary previous dumps of checkpoint files
+  call restart_clean_multiwriters_history
+
   call timer_stats_stop(restart_stats_id)
 
 endif ! iisuit = 1
@@ -971,11 +1081,10 @@ call timer_stats_start(post_stats_id)
 
 call post_activate_by_time_step
 
-call cs_gui_postprocess_activate
+call cs_meg_post_activate;
 call cs_user_postprocess_activate(ntmabs, ntcabs, ttcabs)
 
-! If ITRALE=0, deactivate all writers, as geometry has not
-!              been output yet.
+! If ITRALE=0, deactivate all writers, as geometry has not been output yet.
 if (itrale.eq.0) then
   call post_activate_writer(0, .false.)
 endif
@@ -986,6 +1095,14 @@ endif
 
 call pstvar(nvar, nscal)
 
+! CDO module (user-defined equations)
+!====================================
+
+if (icdo.eq.1) then
+  ! FV and CDO activated
+  call cs_f_cdo_post_domain
+endif
+
 !===============================================================================
 ! Structures
 !===============================================================================
@@ -1004,7 +1121,6 @@ if ((nthist.gt.0 .or.frhist.gt.0.d0) .and. itrale.gt.0) then
   if (modhis.eq.1) then
 
     ttchis = ttcabs
-
     if (ihistr.eq.1) then
       call strhis(modhis)
     endif
@@ -1014,19 +1130,12 @@ if ((nthist.gt.0 .or.frhist.gt.0.d0) .and. itrale.gt.0) then
 endif
 
 !===============================================================================
-! Write to "listing" every ntlist iterations
+! Write to "run_solver.log" every ntlist iterations
 !===============================================================================
 
-if (ntlist.gt.0) then
-  modntl = mod(ntcabs,ntlist)
-elseif(ntlist.eq.-1.and.ntcabs.eq.ntmabs) then
-  modntl = 0
-else
-  modntl = 1
-endif
-if (modntl.eq.0) then
+if (log_active) then
 
-  call ecrlis(ncelet, ncel, dt, volume)
+  call ecrlis(ncelet, ncel, dt, cell_f_vol)
 
   call log_iteration
 
@@ -1058,6 +1167,12 @@ endif
 
 ! LIBERATION DES TABLEAUX INTERMEDIAIRES (PDC+TSM)
 
+if (isuite.eq.1.and.iilagr.gt.0) then
+  call timer_stats_start(restart_stats_id)
+  call cs_restart_map_free
+  call timer_stats_stop(restart_stats_id)
+endif
+
 !===============================================================================
 ! Finalize probes
 !===============================================================================
@@ -1080,10 +1195,6 @@ endif
 
 ! Free main arrays
 
-if (ivrtex.eq.1) then
-  deallocate(irepvo)
-endif
-
 call turbomachinery_finalize
 
 call radiat_finalize
@@ -1093,9 +1204,9 @@ call turbulence_model_free_bc_ids
 
 call finalize_aux_arrays
 
-if (ippmod(iatmos).ge.0) then
+call finalize_meteo
 
-  call finalize_meteo
+if (ippmod(iatmos).ge.0) then
 
   if(imbrication_flag)then
     call finalize_imbrication
@@ -1117,7 +1228,7 @@ if (ippmod(igmix).ge.0) then
   call finalize_gas_mix
 endif
 
-if (iale.eq.1) then
+if (iale.ge.1) then
   call finalize_ale
 endif
 
@@ -1150,8 +1261,8 @@ if (nfpt1d.gt.0) then
   call cs_1d_wall_thermal_finalize
 endif
 
-if (ivrtex.eq.1) then
-  call finalize_vortex
+if (i_les_balance.gt.0) then
+  call les_balance_finalize
 endif
 
 write(nfecra,7000)
@@ -1160,64 +1271,6 @@ write(nfecra,7000)
 ! Formats
 !----
 
-#if defined(_CS_LANG_FR)
-
- 2000 format(/,/,                                                 &
-'===============================================================',&
-                                                              /,/,&
-                                                                /,&
-                                                                /,&
-'                       CORPS DU CALCUL                       ',/,&
-'                       ===============                       ',/,&
-                                                                /,&
-                                                                /,&
-'===============================================================',&
-                                                              /,/,&
-                                                                /)
- 3000 format(/,                                                   &
-'===============================================================',&
- /)
- 3001 format(/,' INSTANT ',E18.9,        '   ITERATION NUMERO ',I15,/,  &
-' ============================================================= ',&
- /,/)
- 3002 format(/,' INSTANT ',E18.9,        '   INITIALISATION ALE ',/,    &
-' ============================================================= ',&
- /,/)
- 3012 format(/,' TEMPS POUR L''INITIALISATION ALE :    ',E14.5,/,/,     &
-'===============================================================',&
- /)
- 3020 format(/,/,                                                 &
- ' Sortie intermediaire de fichiers suite',/,                     &
- '   Sauvegarde a l''iteration ', I10, ', Temps physique ',E14.5,/,/)
- 3021 format(/,/,                                                 &
- ' Sortie finale de fichiers suite',/,                     &
- '   Sauvegarde a l''iteration ', I10, ', Temps physique ',E14.5,/,/)
-
- 4000 format(/,/,                                                 &
-'===============================================================',&
-                                                              /,/,&
-                                                                /,&
-                                                                /,&
-'                   ETAPES FINALES DU CALCUL                  ',/,&
-'                   ========================                  ',/,&
-                                                                /,&
-                                                                /,&
-' =========================================================== ',/,&
-                                                                /,&
-                                                                /)
- 7000 format(/,/,                                                 &
-' =========================================================== ',/,&
-                                                              /,/,&
-                                                                /,&
-                                                                /,&
-'                 FIN DE L''EXECUTION DU CALCUL               ',/,&
-'                 ============================                ',/,&
-                                                                /,&
-                                                                /,&
-'===============================================================')
-
-#else
-
  2000 format(/,/,                                                 &
 '===============================================================',&
                                                               /,/,&
@@ -1272,8 +1325,6 @@ write(nfecra,7000)
                                                                 /,&
 '===============================================================')
 
-#endif
-
 !----
 ! End
 !----
