@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -89,7 +89,6 @@ use numvar
 use optcal
 use cstphy
 use cstnum
-use dimens, only: nvar
 use pointe
 use entsor
 use albase
@@ -105,25 +104,26 @@ implicit none
 
 integer          nscal
 
-integer          icodcl(nfabor,nvar)
+integer, pointer, dimension(:,:) :: icodcl
 
-double precision rcodcl(nfabor,nvar,3)
-double precision velipb(nfabor,ndim), rijipb(nfabor,6)
+double precision, pointer, dimension(:,:,:) :: rcodcl
+double precision, dimension(:,:) :: velipb
+double precision, pointer, dimension(:,:) :: rijipb
 
 ! Local variables
 
 integer          ifac, ii, isou
-integer          iel, f_dim
+integer          iel, f_dim, clsyme
 integer          iscal , ivar, idftnp
+integer          kturt, turb_flux_model, turb_flux_model_type
 
-double precision rnx, rny, rnz, rxnn
+double precision rnx, rny, rnz
 double precision upx, upy, upz, usn
 double precision tx, ty, tz, txn, t2x, t2y, t2z
-double precision clsyme
 double precision eloglo(3,3), alpha(6,6)
 double precision srfbnf, rcodcn, hint, visclc, visctc, distbf
 double precision cpp
-double precision vismsh(3), hintv(6), rnn(6), htnn(6)
+double precision hintv(6), rnn(6), htnn(6)
 double precision visci(3,3), fikis, viscis, distfi
 double precision fcoefa(6), fcoefb(6), fcofaf(6), fcofbf(6), fcofad(6), fcofbd(6)
 
@@ -276,6 +276,9 @@ if (itytur.eq.3) then
   endif
 endif
 
+! Get the turbulent flux model key id
+call field_get_key_id('turbulent_flux_model', kturt)
+
 ! --- Begin the loop over boundary faces
 do ifac = 1, nfabor
 
@@ -303,7 +306,7 @@ do ifac = 1, nfabor
     !   TX a partir de la vitesse tangentielle absolue car l'orientation
     !   de TX et T2X est sans importance pour les symetries)
     rcodcn = 0.d0
-    if (iale.eq.1) then
+    if (iale.ge.1) then
       rcodcn = rcodcl(ifac,iu,1)*rnx                           &
              + rcodcl(ifac,iv,1)*rny                           &
              + rcodcl(ifac,iw,1)*rnz
@@ -333,27 +336,16 @@ do ifac = 1, nfabor
 
       else
 
-        ! If the velocity is zero, vector T is normal and random;
-        !   we need it for the reference change for Rij, and we cancel the velocity.
+        ! If the velocity is zero,
+        !  Tx, Ty, Tz is not used (we cancel the velocity), so we assign any
+        !  value (zero for example)
 
-        if(abs(rny).ge.epzero.or.abs(rnz).ge.epzero)then
-          rxnn = sqrt(rny**2+rnz**2)
-          tx  =  0.d0
-          ty  =  rnz/rxnn
-          tz  = -rny/rxnn
-        elseif(abs(rnx).ge.epzero.or.abs(rnz).ge.epzero)then
-          rxnn = sqrt(rnx**2+rnz**2)
-          tx  =  rnz/rxnn
-          ty  =  0.d0
-          tz  = -rnx/rxnn
-        else
-          write(nfecra,1000)ifac,rnx,rny,rnz
-          call csexit (1)
-        endif
+        tx  = 0.d0
+        ty  = 0.d0
+        tz  = 0.d0
 
       endif
 
-
       ! --> T2 = RN X T (where X is the cross product)
 
       t2x = rny*tz - rnz*ty
@@ -379,37 +371,10 @@ do ifac = 1, nfabor
       eloglo(3,2) = -rnz
       eloglo(3,3) =  t2z
 
-      ! --> Commpute alpha(6,6)
+      ! Compute Reynolds stress transformation matrix
 
-      ! Let f be the center of the boundary faces and
-      !   I the center of the matching cell
-
-      ! We noteE Rg (resp. Rl) indexed by f or by I
-      !   the Reynolds Stress tensor in the global basis (resp. local)
-
-      ! The alpha matrix applied to the global vector in I'
-      !   (Rg11,I'|Rg22,I'|Rg33,I'|Rg12,I'|Rg13,I'|Rg23,I')t
-      !    must provide the values to prescribe to the face
-      !   (Rg11,f |Rg22,f |Rg33,f |Rg12,f |Rg13,f |Rg23,f )t
-      !    except for the Dirichlet boundary conditions (added later)
-
-      ! We define it by computing Rg,f as a function of Rg,I' as follows
-
-      !   RG,f = ELOGLO.RL,f.ELOGLOt (matrix products)
-
-      !                     | RL,I'(1,1)     B*U*.Uk     C*RL,I'(1,3) |
-      !      with    RL,f = | B*U*.Uk       RL,I'(2,2)       0        |
-      !                     | C*RL,I'(1,3)     0         RL,I'(3,3)   |
-
-      !             with    RL,I = ELOGLOt.RG,I'.ELOGLO
-      !                     B = 0
-      !              and    C = 0 at the wall (1 with symmetry)
-
-      ! We compute in fact  ELOGLO.projector.ELOGLOt
-
-      clsyme=1.d0
-      call clca66 ( clsyme , eloglo , alpha )
-      !==========
+      clsyme = 1
+      call turbulence_bc_rij_transform(clsyme, eloglo, alpha)
 
     endif
 
@@ -646,8 +611,13 @@ enddo
 !===========================================================================
 
 do iscal = 1, nscal
+
+  ! Get the associated turbulent flux model
+  call field_get_key_int(ivarfl(isca(iscal)), kturt, turb_flux_model)
+  turb_flux_model_type = turb_flux_model / 10
+
   ! u'T'
-  if (ityturt(iscal).eq.3) then
+  if (turb_flux_model_type.eq.3) then
     call clsyvt_scalar(iscal, icodcl)
   endif
 
@@ -754,22 +724,6 @@ if (iale.eq.1) then
 
 endif
 
-!===============================================================================
-! 5. Formats
-!===============================================================================
-
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,' LA NORMALE A LA FACE DE BORD DE SYMETRIE ',I10,/,&
-         ' EST NULLE ; COORDONNEES : ',3E12.5)
-
-#else
-
- 1000 format(/,' THE NORMAL TO THE SYMMETRY BOUNDARY FACE ',I10,/,&
-         ' IS NULL; COORDINATES: ',3E12.5)
-
-#endif
-
 !----
 ! End
 !----
@@ -838,9 +792,10 @@ integer          icodcl(nfabor,nvar)
 
 integer          ivar, f_id
 integer          ifac, iel, isou, jsou
-integer          ifcvsl
+integer          iscacp, ifcvsl
+integer          kturt, turb_flux_model, turb_flux_model_type
 
-double precision cpp, rkl, visclc
+double precision cpp, rkl, visclc, visls_0
 double precision distbf, srfbnf
 double precision rnx, rny, rnz
 double precision hintt(6)
@@ -858,7 +813,12 @@ double precision, dimension(:), pointer :: viscl, viscls, cpro_cp
 
 !===============================================================================
 
-if (ityturt(iscal).ne.3) return
+! Get the turbulent flux model for the scalar
+call field_get_key_id('turbulent_flux_model', kturt)
+call field_get_key_int(ivarfl(isca(iscal)), kturt, turb_flux_model)
+turb_flux_model_type = turb_flux_model / 10
+
+if (turb_flux_model_type.ne.3) return
 
 ivar = isca(iscal)
 f_id = ivarfl(ivar)
@@ -883,6 +843,9 @@ if (ifcvsl .ge. 0) then
   call field_get_val_s(ifcvsl, viscls)
 endif
 
+! Does the scalar behave as a temperature ?
+call field_get_key_int(f_id, kscacp, iscacp)
+
 ! Turbulent diffusive flux of the scalar T
 ! (blending factor so that the component v'T' have only
 !  mu_T/(mu+mu_T)* Phi_T)
@@ -893,6 +856,9 @@ call field_get_name(ivarfl(ivar), fname)
 ! Index of the corresponding turbulent flux
 call field_get_id(trim(fname)//'_turbulent_flux', f_id)
 
+! Reference diffusivity
+call field_get_key_double(f_id, kvisl0, visls_0)
+
 call field_get_coefa_v(f_id,coefaut)
 call field_get_coefb_v(f_id,coefbut)
 call field_get_coefaf_v(f_id,cofafut)
@@ -901,7 +867,7 @@ call field_get_coefad_v(f_id,cofarut)
 call field_get_coefbd_v(f_id,cofbrut)
 
 ! EB-GGDH/AFM/DFM alpha boundary conditions
-if (iturt(iscal).eq.11 .or. iturt(iscal).eq.21 .or. iturt(iscal).eq.31) then
+if (turb_flux_model.eq.11 .or. turb_flux_model.eq.21 .or. turb_flux_model.eq.31) then
 
   ! Name of the scalar ivar
   call field_get_name(ivarfl(ivar), fname)
@@ -925,7 +891,7 @@ do ifac = 1, nfabor
     ! --- Physical Propreties
     visclc = viscl(iel)
     cpp = 1.d0
-    if (iscacp(iscal).eq.1) then
+    if (iscacp.eq.1) then
       if (icp.ge.0) then
         cpp = cpro_cp(iel)
       else
@@ -942,7 +908,7 @@ do ifac = 1, nfabor
     rnz = surfbo(3,ifac)/srfbnf
 
     if (ifcvsl .lt. 0) then
-      rkl = visls0(iscal)/cpp
+      rkl = visls_0/cpp
     else
       rkl = viscls(iel)/cpp
     endif
@@ -998,7 +964,7 @@ do ifac = 1, nfabor
     enddo
 
     ! EB-GGDH/AFM/DFM alpha boundary conditions
-    if (iturt(iscal).eq.11 .or. iturt(iscal).eq.21 .or. iturt(iscal).eq.31) then
+    if (turb_flux_model.eq.11 .or. turb_flux_model.eq.21 .or. turb_flux_model.eq.31) then
 
       ! Dirichlet Boundary Condition
       !-----------------------------
@@ -1092,7 +1058,7 @@ integer          ivar, f_id
 integer          ifac, iel
 integer          ifcvsl
 
-double precision rkl, visclc
+double precision rkl, visclc, visls_0
 double precision distbf, srfbnf
 double precision rnx, rny, rnz, temp
 double precision hintt(6)
@@ -1140,6 +1106,9 @@ endif
 ! retrieve turbulent Schmidt value for current scalar
 call field_get_key_double(ivarfl(ivar), ksigmas, turb_schmidt)
 
+! Reference diffusivity
+call field_get_key_double(f_id, kvisl0, visls_0)
+
 ! --- Loop on boundary faces
 do ifac = 1, nfabor
 
@@ -1159,7 +1128,7 @@ do ifac = 1, nfabor
     rnz = surfbo(3,ifac)/srfbnf
 
     if (ifcvsl .lt. 0) then
-      rkl = visls0(iscal)
+      rkl = visls_0
     else
       rkl = viscls(iel)
     endif
