@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -46,9 +46,6 @@
 !> \param[in]     ifbpcd        index of faces with condensation source terms
 !> \param[in]     izzftcd       faces zone with condensation source terms imposed
 !>                              (at previous and current time steps)
-!> \param[in]     tpar          temperature imposed at the cold wall
-!>                              as constant or variable in time
-!>                              with a 1D thermal model
 !> \param[out]    gam_s         value associated to each variable in the
 !>                              condensation source terms (Lambda_cond)
 !> \param[out]    hpcond        value associated to the fluid exchange coeff.
@@ -58,7 +55,6 @@
 
 subroutine condensation_copain_model &
  ( nvar   , nfbpcd , ifbpcd , izzftcd ,  &
-   tpar   ,                              &
    gam_s  , hpcond )
 
 !===============================================================================
@@ -92,7 +88,6 @@ implicit none
 
 integer          nvar, nfbpcd, ifbpcd(nfbpcd), izzftcd(nfbpcd)
 
-double precision tpar
 double precision gam_s(nfbpcd,nvar)
 double precision hpcond(nfbpcd)
 
@@ -100,6 +95,7 @@ double precision hpcond(nfbpcd)
 
 integer          ii, iz, iel, ifac, iesp
 integer          ivar, f_id, ifcvsl
+integer          ustar_id
 
 double precision flux
 double precision sink_term, gamma_cond
@@ -122,6 +118,7 @@ double precision C_k1,C_k2,C_k3,C_k4,C_k5,C_k6
 double precision C_k7,C_k8,C_k9,t_wall
 double precision pr_c,t_c,dtheta
 double precision lcond
+double precision uk, rough_t
 
 type(gas_mix_species_prop) s_h2o_g, s_k
 type(var_cal_opt) :: vcopt
@@ -132,6 +129,7 @@ double precision, dimension(:), pointer :: cpro_rho, cpro_viscl, cpro_cp, cpro_v
 double precision, dimension(:), pointer :: cvar_enth, cvar_yk
 double precision, dimension(:), pointer :: y_h2o_g
 double precision, dimension(:), pointer :: yplbr
+double precision, dimension(:), pointer :: bpro_ustar
 
 !===============================================================================
 ! Allocate a temporary array for cells selection
@@ -157,6 +155,11 @@ if (iyplbr.ge.0) call field_get_val_s(iyplbr, yplbr)
 call field_get_id("y_h2o_g", f_id)
 call field_get_key_struct_gas_mix_species_prop(f_id, s_h2o_g)
 
+call field_get_id_try('ustar', ustar_id)
+if (ustar_id.ge.0) then
+  call field_get_val_s(ustar_id, bpro_ustar)
+endif
+
 !===============================================================================
 ! 1 - Constant physical values of the Copain modelling
 !     with (C_k) constants of the saturated pressure and
@@ -353,6 +356,15 @@ do ii = 1, nfbpcd
   !-- kinematic viscosity --------------------------
   xnu = cpro_viscl(iel)/cpro_rho(iel)
 
+  ! Thermal roughness
+  rough_t = 0.d0
+  ! Turbulent friction velocity
+  if (ustar_id.ge.0) then
+    uk = bpro_ustar(ifac)
+  else
+    uk = 0.d0
+  endif
+
   !-- Compute the t_in temperature far away
   !-- from the wall. t_inf := t_atmos, is the
   !-- temperature associate to the cell.
@@ -439,7 +451,8 @@ do ii = 1, nfbpcd
     dplus = 0.d0
     yplus = yplbr(ifac)
     sigmat = 0.9d0
-    call hturbp(iwalfs,schdt,sigmat,yplus,dplus,hflui,ypth)
+
+    call hturbp(iwalfs,xnu,schdt,sigmat,rough_t,uk,yplus,dplus,hflui,ypth)
 
     hcdt =  diff_m(iel)/distbf*hflui
 
@@ -534,7 +547,7 @@ do ii = 1, nfbpcd
   sigmat = 0.9d0
   Prdtl = cpro_viscl(iel)/cpro_venth(iel)
 
-  call hturbp(iwalfs,Prdtl,sigmat,yplus,dplus,hpflui,ypth)
+  call hturbp(iwalfs,xnu,Prdtl,sigmat,rough_t,uk,yplus,dplus,hpflui,ypth)
   hw_enth = cpro_venth(iel)/distbf*hpflui
 
   h1max = max(h1max,hw_enth)
@@ -586,7 +599,7 @@ deallocate(mix_mol_mas, x_ncond, mol_mas_ncond)
 deallocate(x_h2o_g, diff_m)
 
 !===============================================================================
-! 5 - Print in the listing every ntlist of the min/max values
+! 5 - Print in the log every ntlist of the min/max values
 !         for exchange coefficient and flux computed
 !===============================================================================
 
@@ -631,43 +644,6 @@ endif
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========                                               ',/,&
-'@    DONNEES DE CALCUL INCOHERENTES                          ',/,&
-'@                                                            ',/,&
-'@      usipsu indique que la chaleur specifique est uniforme ',/,&
-'@        ICP = ',I10   ,' alors que                          ',/,&
-'@      copain model impose une chaleur specifique variable.  ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne sera pas execute.                          ',/,&
-'@                                                            ',/,&
-'@    Modifier usipsu ou copain model.                        ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 1010 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========                                               ',/,&
-'@    DONNEES DE CALCUL INCOHERENTES                          ',/,&
-'@                                                            ',/,&
-'@    La diffusivite du scalaire ',i10,' est uniforme alors   ',/,&
-'@      que copain model impose une diffusivite variable.     ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne sera pas execute.                          ',/,&
-'@                                                            ',/,&
-'@    Modifier usipsu ou copain model.                        ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#else
  1000 format(                                                     &
 '@',/,                                                            &
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -705,7 +681,6 @@ endif
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',/)
-#endif
  1061 format(/,                                                   &
 ' ** Condensation source terms (Gamma) added:',E14.5           ,/,&
 '    ----------------------------------------',/)
