@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -109,7 +109,6 @@ use lagran
 use radiat
 use field
 use field_operator
-use ihmpre, only: iihmpr
 use mesh
 use parall
 use period
@@ -148,13 +147,13 @@ logical          lprev
 character(len=80) :: chaine, fname
 integer          ivar
 integer          ii, ifac , iel, isou
-integer          iprev , inc   , iccocg, iiun, ibcl
+integer          iprev , inc   , iccocg, ibcl
 integer          ivarsc
-integer          iiscav
+integer          iiscav, iscacp
 integer          ifcvsl, iflmas, iflmab, f_oi_id
 integer          nswrgp, imligp, iwarnp
 integer          iconvp, idiffp, ndircp
-integer          nswrsp, ircflp, ischcp, isstpp, iescap
+integer          imrgrp, nswrsp, ircflp, ischcp, isstpp, iescap
 integer          imucpp, idftnp, iswdyp
 integer          iflid , f_id, st_prv_id, st_id,  keydri, iscdri
 integer          f_id_al
@@ -164,17 +163,23 @@ integer          icrom_scal
 integer          key_buoyant_id, is_buoyant_fld
 integer          key_t_ext_id
 integer          iviext
+integer          key_turb_schmidt
+integer          t_scd_id
+integer          kturt, variance_turb_flux_model, scalar_turb_flux_model
+integer          scalar_turb_flux_model_type, variance_turb_flux_model_type
 
 integer          ivoid(1)
 
-double precision epsrgp, climgp, extrap, relaxp, blencp, epsilp
+double precision epsrgp, climgp, relaxp, blencp, epsilp
 double precision epsrsp
 double precision rhovst, xk    , xe    , sclnor
 double precision thetv , thets , thetap, thetp1
-double precision smbexp, dvar, cprovol, prod, expkdt
-double precision temp, idifftp, roskpl, kplskm, ctot_tmp
-double precision turb_schmidt
+double precision smbexp, dvar, cprovol, prod
+double precision temp, idifftp
+double precision turb_schmidt, visls_0
 double precision xR, prdtl, alpha_theta
+double precision normp
+double precision l2norm, l2errork
 
 double precision rvoid(1)
 
@@ -203,10 +208,14 @@ double precision, dimension(:,:), pointer :: cvara_rij
 double precision, dimension(:), pointer :: cvar_al
 double precision, dimension(:), pointer :: visct, viscl, cpro_cp, cproa_scal_st
 double precision, dimension(:), pointer :: cpro_scal_st
+double precision, dimension(:), pointer :: cpro_turb_schmidt
 double precision, dimension(:), pointer :: cpro_viscls, cpro_visct
 double precision, dimension(:), pointer :: cpro_tsscal
 double precision, dimension(:), pointer :: cpro_x2icla
 double precision, dimension(:), pointer :: cvar_var, cvara_var, cvara_varsca
+double precision, dimension(:), pointer :: cvark_var
+double precision, allocatable, dimension(:), target :: wcvark_var
+double precision, allocatable, dimension(:) :: errork
 double precision, allocatable, dimension(:) :: divflu
 ! Darcy arrays
 double precision, allocatable, dimension(:) :: diverg
@@ -220,6 +229,22 @@ type(var_cal_opt) :: vcopt, vcopt_varsc
 type(gwf_soilwater_partition) :: sorption_scal
 
 !===============================================================================
+! Interfaces
+!===============================================================================
+
+interface
+
+  subroutine cs_syr_coupling_volume_source_terms(field_id, st_exp, st_imp) &
+    bind(C, name = 'cs_syr_coupling_volume_source_terms')
+
+    use, intrinsic :: iso_c_binding
+    implicit none
+    integer(kind=c_int), value :: field_id
+    real(kind=c_double), dimension(*), intent(inout) :: st_exp, st_imp
+
+  end subroutine cs_syr_coupling_volume_source_terms
+
+ end interface
 
 !===============================================================================
 ! 1. Initialization
@@ -244,7 +269,7 @@ call field_get_key_id("time_extrapolated", key_t_ext_id)
 ! If the scalar is buoyant, it is inside the Navier Stokes loop, and so iterns >=1
 ! otherwise it is outside of the loop and iterns = -1.
 if (  (is_buoyant_fld.eq. 1 .and. iterns.eq.-1) &
-  .or.(is_buoyant_fld.eq.-1 .and. iterns.ne.-1)) return
+  .or.(is_buoyant_fld.eq. 0 .and. iterns.ne.-1)) return
 
 ! Key id for drift scalar
 call field_get_key_id("drift_scalar_model", keydri)
@@ -259,7 +284,7 @@ call field_get_key_int(iflid, kbmasf, iflmab) ! boundary mass flux
 ! Pointer to the Boundary mass flux
 call field_get_val_s(iflmab, bmasfl)
 
-call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+call field_get_key_struct_var_cal_opt(iflid, vcopt)
 
 if (vcopt%iwgrec.eq.1) then
   ! Id weighting field for gradient
@@ -295,8 +320,19 @@ else
   ivarsc = 0
 endif
 
+!  Get the turbulent flux model for the scalar and its variance
+call field_get_key_id('turbulent_flux_model', kturt)
+
+call field_get_key_int(ivarfl(isca(iscal)) , kturt, scalar_turb_flux_model)
+if (ivarsc > 0) then
+  call field_get_key_int(ivarfl(ivarsc), kturt, variance_turb_flux_model)
+  variance_turb_flux_model_type = variance_turb_flux_model / 10
+endif
+
+scalar_turb_flux_model_type = scalar_turb_flux_model / 10
+
 ! --- Numero des grandeurs physiques
-call field_get_key_int (ivarfl(isca(iscal)), kromsl, icrom_scal)
+call field_get_key_int (iflid, kromsl, icrom_scal)
 
 if (icrom_scal.eq.-1) then
   icrom_scal = icrom
@@ -310,11 +346,13 @@ endif
 call field_get_val_s(ivisct, visct)
 call field_get_val_s(iviscl, viscl)
 
-call field_get_key_int (ivarfl(isca(iscal)), kivisl, ifcvsl)
+call field_get_key_int (iflid, kivisl, ifcvsl)
 if (ifcvsl.ge.0) then
   call field_get_val_s(ifcvsl, cpro_viscls)
 endif
 
+call field_get_key_double(iflid, kvisl0, visls_0)
+
 if (idilat.ge.4) then
   call field_get_val_s(iustdy(iscal), cpro_tsscal)
 endif
@@ -342,13 +380,12 @@ endif
 
 imucpp = 0
 if (iscavr(iscal).gt.0) then
-  if (abs(iscacp(iscavr(iscal))).eq.1) then
-    imucpp = 1
-  endif
+  call field_get_key_int(ivarfl(isca(iscavr(iscal))), kscacp, iscacp)
 else
-  if (abs(iscacp(iscal)).eq.1) then
-    imucpp = 1
-  endif
+  call field_get_key_int(iflid, kscacp, iscacp)
+endif
+if (iscacp.eq.1) then
+  imucpp = 1
 endif
 
 allocate(xcpp(ncelet))
@@ -376,7 +413,13 @@ if (irangp.ge.0.or.iperio.eq.1) then
 endif
 
 ! Retrieve turbulent Schmidt value for current scalar
-call field_get_key_double(ivarfl(isca(iscal)), ksigmas, turb_schmidt)
+call field_get_key_double(iflid, ksigmas, turb_schmidt)
+! If turbulent Schmidt is variable, id of the corresponding field
+call field_get_key_id("turbulent_schmidt_id", key_turb_schmidt)
+call field_get_key_int(iflid, key_turb_schmidt, t_scd_id)
+if (t_scd_id.ge.0) then
+  call field_get_val_s(t_scd_id, cpro_turb_schmidt)
+endif
 
 !===============================================================================
 ! 2. Source terms
@@ -389,24 +432,22 @@ do iel = 1, ncel
   smbrs(iel) = 0.d0
 enddo
 
-if (iihmpr.eq.1) then
-  if (iscal.ne.iscalt) then
-    call uitssc &
-    ( ippmod(idarcy), iflid  , cvar_var , smbrs  , rovsdt )
-  else
-    call uitsth &
-    ( iflid  , cvar_var , smbrs  , rovsdt )
-  endif
+if (iscal.ne.iscalt) then
+  call uitssc(ippmod(idarcy), iflid, cvar_var, smbrs, rovsdt)
+else
+  call uitsth(iflid, cvar_var, smbrs, rovsdt)
 endif
 
 call ustssc &
-!==========
 ( nvar   , nscal  , ncepdp , ncesmp ,                            &
   iscal  ,                                                       &
   icepdc , icetsm , itypsm ,                                     &
   dt     ,                                                       &
   ckupdc , smacel , smbrs  , rovsdt )
 
+! C version
+call user_source_terms(iflid, smbrs, rovsdt)
+
 ! Take into account radioactive decay rate (implicit source term)
 if (ippmod(idarcy).eq.1) then
   call cs_gwf_decay_rate(ivarfl(ivar), rovsdt)
@@ -451,17 +492,16 @@ endif
 ! Atmospheric chemistry
 ! In case of a semi-coupled resolution, computation of the explicit
 ! chemical source term to be considered during dynamical resolution
-! The first nespg user scalars are supposed to be chemical species
-if ((ichemistry.ge.1).and.(isepchemistry.eq.2)                    &
-     .and.(iscal.le.nespg).and.(ntcabs.gt.1)) then
-  call chem_source_terms(iscal, smbrs, rovsdt)
+if ((ichemistry.ge.1) .and. (isepchemistry.eq.2) .and. (ntcabs.ge.ntinit)) then
+  if ((isca_chem(1).le.iscal).and.(iscal.le.isca_chem(nespg))) then
+    call chem_source_terms(iscal, smbrs, rovsdt)
+  endif
 endif
 
-
 ! Precipitation/dissolution for lagrangian module
 ! Calculation of source terms du to precipitation and dissolution phenomena
 if (ipreci.eq.1.and.iscal.eq.1) then
-  call precst(dtref, crom, cvar_var, smbrs)
+  call cs_lagr_precipitation_mass_st(dtref, crom, cvar_var, smbrs)
 endif
 
 ! Si on extrapole les TS :
@@ -511,7 +551,7 @@ endif
 !     Ordre 2 non pris en compte
 
 if (iscal.eq.iscalt) then
-  call cptssy(iscal, smbrs, rovsdt)
+  call cs_syr_coupling_volume_source_terms(iflid, smbrs, rovsdt)
 endif
 
 ! --> Physique particulieres
@@ -612,9 +652,6 @@ endif
 
 if (ncesmp.gt.0) then
 
-  ! Entier egal a 1 (pour navsto : nb de sur-iter)
-  iiun = 1
-
   allocate(srcmas(ncesmp))
 
   ! When treating the Temperature, the equation is multiplied by Cp
@@ -627,12 +664,9 @@ if (ncesmp.gt.0) then
   enddo
 
   ! On incremente SMBRS par -Gamma RTPA et ROVSDT par Gamma
-  call catsma &
-  !==========
- ( ncelet , ncel   , ncesmp , iiun   , isso2t(iscal) ,            &
-   icetsm , itypsm(1,ivar) ,                                      &
-   cell_f_vol , cvara_var    , smacel(1,ivar) , srcmas   ,        &
-   smbrs  , rovsdt , w1)
+  call catsma(ncesmp, 1, icetsm, itypsm(:,ivar),               &
+              cell_f_vol, cvara_var, smacel(:,ivar), srcmas,   &
+              smbrs, rovsdt, w1)
 
   deallocate(srcmas)
 
@@ -669,8 +703,7 @@ if (nfbpcd.gt.0) then
   enddo
 
   call condensation_source_terms &
-  !=============================
-  (ncelet , ncel ,                                       &
+  (ncelet ,                                              &
    iscal  ,                                              &
    nfbpcd , ifbpcd  , itypcd(1,ivar) ,                   &
    0      , ivoid   , ivoid          ,                   &
@@ -702,8 +735,7 @@ if (icondv.eq.0) then
   enddo
 
   call condensation_source_terms &
-  !=============================
-  (ncelet , ncel ,                                       &
+  (ncelet ,                                              &
    iscal  ,                                              &
    0      , ivoid   , ivoid          ,                   &
    ncmast , ltmast  , itypst(1,ivar) ,                   &
@@ -757,16 +789,16 @@ if (itspdv.eq.1) then
 
     call field_get_key_struct_var_cal_opt(ivarfl(ivarsc), vcopt_varsc)
 
+    imrgrp = vcopt_varsc%imrgra
     nswrgp = vcopt_varsc%nswrgr
     imligp = vcopt_varsc%imligr
     iwarnp = vcopt_varsc%iwarni
     epsrgp = vcopt_varsc%epsrgr
     climgp = vcopt_varsc%climgr
-    extrap = vcopt_varsc%extrag
 
     call gradient_s                                                          &
-     ( ivarfl(ivarsc)  , imrgra , inc    , iccocg , nswrgp , imligp ,        &
-       iwarnp          , epsrgp , climgp , extrap ,                          &
+     ( ivarfl(ivarsc)  , imrgrp , inc    , iccocg , nswrgp , imligp ,        &
+       iwarnp          , epsrgp , climgp ,                                   &
        cvara_varsca    , coefa_p, coefb_p,                                   &
        grad )
 
@@ -774,7 +806,8 @@ if (itspdv.eq.1) then
 
     ! Production Term
 
-    ! NB: diffusivity is clipped to 0 because in LES, it might be negative. Problematic
+    ! NB: diffusivity is clipped to 0 because in LES, it might be negative.
+    ! Problematic
     ! for the variance even if it is strange to use variance and LES ...
 
     ! Time extrapolation (2nd order)
@@ -787,7 +820,7 @@ if (itspdv.eq.1) then
 
       ! iscal is the variance of the scalar iiscav
       ! with modelized turbulent fluxes GGDH or AFM or DFM
-      if (ityturt(iiscav).ge.1) then
+      if (variance_turb_flux_model_type.ge.1) then
 
         ! Name of the scalar iiscav associated to the variance iscal
         call field_get_name(ivarfl(ivarsc), fname)
@@ -806,12 +839,22 @@ if (itspdv.eq.1) then
         enddo
       ! SGDH model
       else
-        do iel = 1, ncel
-          cproa_scal_st(iel) = cproa_scal_st(iel)                             &
-               + 2.d0*xcpp(iel)*max(cpro_visct(iel),zero)                     &
-               *cell_f_vol(iel)/turb_schmidt                                 &
-               *(grad(1,iel)**2 + grad(2,iel)**2 + grad(3,iel)**2)
-        enddo
+        ! Variable Schmidt number
+        if (t_scd_id.ge.0) then
+          do iel = 1, ncel
+            cproa_scal_st(iel) = cproa_scal_st(iel)                             &
+                 + 2.d0*xcpp(iel)*max(cpro_visct(iel),zero)                     &
+                 *cell_f_vol(iel)/cpro_turb_schmidt(iel)                        &
+                 *(grad(1,iel)**2 + grad(2,iel)**2 + grad(3,iel)**2)
+          enddo
+        else
+          do iel = 1, ncel
+            cproa_scal_st(iel) = cproa_scal_st(iel)                             &
+                 + 2.d0*xcpp(iel)*max(cpro_visct(iel),zero)                     &
+                 *cell_f_vol(iel)/turb_schmidt                                 &
+                 *(grad(1,iel)**2 + grad(2,iel)**2 + grad(3,iel)**2)
+          enddo
+        endif
       endif
 
     ! If not time extrapolation...
@@ -821,8 +864,7 @@ if (itspdv.eq.1) then
 
       ! iscal is the variance of the scalar iiscav
       ! with modelized turbulent fluxes GGDH or AFM or DFM
-      if (ityturt(iiscav).ge.1) then
-
+      if (variance_turb_flux_model_type.ge.1) then
 
         ! Name of the scalar ivarsc associated to the variance iscal
         call field_get_name(ivarfl(ivarsc), fname)
@@ -852,22 +894,42 @@ if (itspdv.eq.1) then
 
       ! SGDH model
       else
-        do iel = 1, ncel
-          smbrs(iel) = smbrs(iel)                                            &
-                     + 2.d0*xcpp(iel)*max(cpro_visct(iel),zero)           &
-                     * cell_f_vol(iel)/turb_schmidt                       &
-                     * (grad(1,iel)**2 + grad(2,iel)**2 + grad(3,iel)**2)
-        enddo
+        ! Variable Schmidt number
+        if (t_scd_id.ge.0) then
+          do iel = 1, ncel
+            smbrs(iel) = smbrs(iel)                                         &
+                       + 2.d0*xcpp(iel)*max(cpro_visct(iel),zero)           &
+                       * cell_f_vol(iel)/cpro_turb_schmidt(iel)             &
+                       * (grad(1,iel)**2 + grad(2,iel)**2 + grad(3,iel)**2)
+          enddo
+        else
+          do iel = 1, ncel
+            smbrs(iel) = smbrs(iel)                                         &
+                       + 2.d0*xcpp(iel)*max(cpro_visct(iel),zero)           &
+                       * cell_f_vol(iel)/turb_schmidt                       &
+                       * (grad(1,iel)**2 + grad(2,iel)**2 + grad(3,iel)**2)
+          enddo
+        endif
       endif
 
       ! Production term for a variance  TODO compute ustdy when isso2t >0
       if (idilat.ge.4) then
-        do iel = 1, ncel
-          cpro_tsscal(iel) = cpro_tsscal(iel) +                   &
-               2.d0*xcpp(iel)*max(cpro_visct(iel),zero)        &
-             *cell_f_vol(iel)/turb_schmidt                     &
-             *(grad(1,iel)**2 + grad(2,iel)**2 + grad(3,iel)**2)
-        enddo
+        ! Variable Schmidt number
+        if (t_scd_id.ge.0) then
+          do iel = 1, ncel
+            cpro_tsscal(iel) = cpro_tsscal(iel) +                &
+                 2.d0*xcpp(iel)*max(cpro_visct(iel),zero)        &
+               *cell_f_vol(iel)/cpro_turb_schmidt(iel)           &
+               *(grad(1,iel)**2 + grad(2,iel)**2 + grad(3,iel)**2)
+          enddo
+        else
+          do iel = 1, ncel
+            cpro_tsscal(iel) = cpro_tsscal(iel) +                &
+                 2.d0*xcpp(iel)*max(cpro_visct(iel),zero)        &
+               *cell_f_vol(iel)/turb_schmidt                     &
+               *(grad(1,iel)**2 + grad(2,iel)**2 + grad(3,iel)**2)
+          enddo
+        endif
       endif
     endif
 
@@ -894,7 +956,9 @@ if (itspdv.eq.1) then
         call field_get_val_prev_s(ivarfl(ir33), cvara_r33)
       endif
       ! EB- AFM or EB-DFM or EB-GGDH
-      if (iturt(iiscav).eq.11 .or. iturt(iiscav).eq.21 .or. iturt(iiscav).eq.31) then
+      if (variance_turb_flux_model.eq.11 .or. &
+          variance_turb_flux_model.eq.21 .or. &
+          variance_turb_flux_model.eq.31      ) then
         ! Name of the scalar corresponding to the current variance
         call field_get_name(ivarfl(ivarsc), fname)
         ! Index of the corresponding alpha
@@ -929,7 +993,9 @@ if (itspdv.eq.1) then
       ! Rh = (1-alpha_T) * Pr + R * alpha_T
       ! with - R = 0.5
       !      - alpha_T = 1.0 for GGDH/DFM/AFM
-      if(iturt(iiscav).eq.11.or.iturt(iiscav).eq.21.or.iturt(iiscav).eq.31) then
+      if (variance_turb_flux_model.eq.11 .or. &
+          variance_turb_flux_model.eq.21 .or. &
+          variance_turb_flux_model.eq.31      ) then
         alpha_theta = cvar_al(iel)
       else
         alpha_theta = 1.d0
@@ -938,7 +1004,7 @@ if (itspdv.eq.1) then
       if (ifcvsl.ge.0) then
         prdtl = viscl(iel)*xcpp(iel)/cpro_viscls(iel)
       else
-        prdtl = viscl(iel)*xcpp(iel)/visls0(iscal)
+        prdtl = viscl(iel)*xcpp(iel)/visls_0
       endif
       xR = ( 1.d0 - alpha_theta ) * prdtl + alpha_theta * rvarfl(iscal)
 
@@ -969,7 +1035,7 @@ if (ippmod(icompf).ge.0.or.(idilat.gt.1.and.ipredfl.eq.1.and.irovar.eq.1)) then
 
 ! Low Mach compressible algos (conservative in time).
 ! Same algo. for Volume of Fluid method
-else if ((idilat.gt.1.or.ivofmt.ge.0).and.irovar.eq.1) then
+else if ((idilat.gt.1.or.ivofmt.gt.0).and.irovar.eq.1) then
   if (iterns.eq.1) then
     call field_get_val_prev2_s(icrom_scal, pcrom)
   else
@@ -993,12 +1059,14 @@ if (vcopt%idiff.ge.1) then
 
   ! AFM model or DFM models: add div(Cp*rho*T'u') to smbrs
   ! Compute T'u' for GGDH
-  if (ityturt(iscal).ge.1) then
+  if (scalar_turb_flux_model_type.ge.1) then
 
     ! EB-GGDH/AFM/DFM: solving alpha for the scalar
-    if (iturt(iscal).eq.11.or.iturt(iscal).eq.21.or.iturt(iscal).eq.31) then
+    if (scalar_turb_flux_model.eq.11 .or. &
+        scalar_turb_flux_model.eq.21 .or. &
+        scalar_turb_flux_model.eq.31      ) then
       ! Name of the scalar
-      call field_get_name(ivarfl(isca(iscal)), fname)
+      call field_get_name(iflid, fname)
 
       ! Index of the corresponding turbulent flux
       call field_get_id(trim(fname)//'_alpha', f_id)
@@ -1021,21 +1089,33 @@ if (vcopt%idiff.ge.1) then
   if (iand(vcopt%idften, ISOTROPIC_DIFFUSION).ne.0) then
 
     idifftp = vcopt%idifft
-    if (ityturt(iscal).eq.3) then
+    if (scalar_turb_flux_model_type.eq.3) then
       idifftp = 0
     endif
 
     if (ifcvsl.lt.0) then
       do iel = 1, ncel
-        w1(iel) = visls0(iscal)                                     &
-           + idifftp*xcpp(iel)*max(visct(iel),zero)/turb_schmidt
+        w1(iel) = visls_0
       enddo
     else
       do iel = 1, ncel
-        w1(iel) = cpro_viscls(iel)                                &
-           + idifftp*xcpp(iel)*max(visct(iel),zero)/turb_schmidt
+        w1(iel) = cpro_viscls(iel)
       enddo
     endif
+    if (idifftp.ge.1) then
+      ! Variable Schmidt number
+      if (t_scd_id.ge.0) then
+        do iel = 1, ncel
+          w1(iel) = w1(iel)                                &
+             + idifftp*xcpp(iel)*max(visct(iel),zero)/cpro_turb_schmidt(iel)
+        enddo
+      else
+        do iel = 1, ncel
+          w1(iel) = w1(iel)                                &
+             + idifftp*xcpp(iel)*max(visct(iel),zero)/turb_schmidt
+        enddo
+      endif
+    endif
 
     if (vcopt%iwgrec.eq.1) then
       ! Weighting for gradient
@@ -1068,14 +1148,16 @@ if (vcopt%idiff.ge.1) then
       call field_get_val_v(ivstes, visten)
     endif
 
-    if (iturt(iscal).eq.11.or.iturt(iscal).eq.20.or.iturt(iscal).eq.21) then
+    if (scalar_turb_flux_model.eq.11 .or. &
+        scalar_turb_flux_model.eq.21 .or. &
+        scalar_turb_flux_model.eq.31      ) then
       if (ifcvsl.lt.0) then
         do iel = 1, ncel
 
           temp = vcopt%idifft*xcpp(iel)
-          viscce(1,iel) = temp*vistet(1,iel) + visls0(iscal)
-          viscce(2,iel) = temp*vistet(2,iel) + visls0(iscal)
-          viscce(3,iel) = temp*vistet(3,iel) + visls0(iscal)
+          viscce(1,iel) = temp*vistet(1,iel) + visls_0
+          viscce(2,iel) = temp*vistet(2,iel) + visls_0
+          viscce(3,iel) = temp*vistet(3,iel) + visls_0
           viscce(4,iel) = temp*vistet(4,iel)
           viscce(5,iel) = temp*vistet(5,iel)
           viscce(6,iel) = temp*vistet(6,iel)
@@ -1099,9 +1181,9 @@ if (vcopt%idiff.ge.1) then
         do iel = 1, ncel
 
           temp = vcopt%idifft*xcpp(iel)*ctheta(iscal)/csrij
-          viscce(1,iel) = temp*visten(1,iel) + visls0(iscal)
-          viscce(2,iel) = temp*visten(2,iel) + visls0(iscal)
-          viscce(3,iel) = temp*visten(3,iel) + visls0(iscal)
+          viscce(1,iel) = temp*visten(1,iel) + visls_0
+          viscce(2,iel) = temp*visten(2,iel) + visls_0
+          viscce(3,iel) = temp*visten(3,iel) + visls_0
           viscce(4,iel) = temp*visten(4,iel)
           viscce(5,iel) = temp*visten(5,iel)
           viscce(6,iel) = temp*visten(6,iel)
@@ -1249,11 +1331,22 @@ endif
 ! 3. Solving
 !===============================================================================
 
+if (iterns.ge.1) then
+  allocate(wcvark_var(ncelet))
+  cvark_var => wcvark_var
+  do iel = 1, ncelet
+    cvark_var(iel) = cvar_var(iel)
+  enddo
+else
+  call field_get_val_s(ivarfl(ivar), cvark_var)
+endif
+
 iconvp = vcopt%iconv
 idiffp = vcopt%idiff
 idftnp = vcopt%idften
-ndircp = ndircl(ivar)
+ndircp = vcopt%ndircl
 nswrsp = vcopt%nswrsm
+imrgrp = vcopt%imrgra
 nswrgp = vcopt%nswrgr
 imligp = vcopt%imligr
 ircflp = vcopt%ircflu
@@ -1267,10 +1360,10 @@ epsilp = vcopt%epsilo
 epsrsp = vcopt%epsrsm
 epsrgp = vcopt%epsrgr
 climgp = vcopt%climgr
-extrap = vcopt%extrag
 relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
 
 call field_get_coefa_s(ivarfl(ivar), coefap)
 call field_get_coefb_s(ivarfl(ivar), coefbp)
@@ -1278,14 +1371,13 @@ call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 call codits &
-!==========
  ( idtvar , iterns , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
-   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   iwarnp ,                                                       &
-   blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
+   iwarnp , normp  ,                                              &
+   blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
    relaxp , thetv  ,                                              &
-   cvara_var       , cvar_var        ,                            &
+   cvara_var       , cvark_var       ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
    imasfl , bmasfl ,                                              &
    viscf  , viscb  , viscf  , viscb  , viscce ,                   &
@@ -1295,7 +1387,7 @@ call codits &
    xcpp   , rvoid  )
 
 !===============================================================================
-! 4. Writing and clipping
+! 4. clipping and log
 !===============================================================================
 
 call clpsca(iscal)
@@ -1368,7 +1460,25 @@ if (vcopt%iwarni.ge.2) then
   write(nfecra,1200)chaine(1:16) ,sclnor
 endif
 
+! Log in case of velocity/pressure inner iterations
+if (iterns.ge.1.and.vcopt%iwarni.ge.1) then
+
+  allocate(errork(ncelet))
+  do iel = 1, ncel
+    errork(iel) = cvar_var(iel) - cvark_var(iel)
+  enddo
+
+  l2errork = sqrt(cs_gres(ncel, cell_f_vol, errork, errork))
+  deallocate(errork)
+
+  l2norm = sqrt(cs_gres(ncel, cell_f_vol, cvara_var, cvara_var))
+
+  write(nfecra,2601) ivarfl(ivar), iterns, l2errork, l2errork/l2norm, l2norm
+
+endif
+
 ! Free memory
+if (allocated(wcvark_var)) deallocate(wcvark_var)
 deallocate(w1)
 deallocate(smbrs, rovsdt)
 if (allocated(viscce)) deallocate(viscce)
@@ -1381,28 +1491,6 @@ if (allocated(diverg)) deallocate(diverg)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,                                                   &
-'   ** RESOLUTION POUR LA VARIABLE ',A16                       ,/,&
-'      ---------------------------                            ',/)
- 1200 format(1X,A16,' : BILAN EXPLICITE = ',E14.5)
- 9000 format( &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ERREUR DANS COVOFI                          ',/,&
-'@    =========                                               ',/,&
-'@    IVARSC DOIT ETRE UN ENTIER POSITIF STRICTEMENT          ',/,&
-'@    IL VAUT ICI ',I10                                        ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne peut etre execute.                           ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
  1000 format(/,                                                   &
 '   ** SOLVING VARIABLE ',A16                                  ,/,&
 '      ----------------'                                       ,/)
@@ -1421,7 +1509,8 @@ if (allocated(diverg)) deallocate(diverg)
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
+2601 format('Inner iteration scalar',I10, 'iter=', I10, 'L2 error = ',E12.4, &
+            ' L2 normalized error', E12.4, 'L2 nomr', E12.4 ,/)
 
 !----
 ! End
