@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -124,7 +124,7 @@ integer          iiscav
 integer          ifcvsl, iflmas, iflmab
 integer          nswrgp, imligp, iwarnp
 integer          iconvp, idiffp, ndircp
-integer          nswrsp, ircflp, ischcp, isstpp, iescap, ivissv
+integer          imrgrp, nswrsp, ircflp, ischcp, isstpp, iescap, ivissv
 integer          idftnp, iswdyp
 integer          iflid , st_prv_id, st_id,  keydri, iscdri
 integer          icvflb, f_dim, iflwgr
@@ -138,7 +138,7 @@ double precision sclnor
 double precision thetv , thets , thetap, thetp1
 double precision smbexp(3)
 double precision temp, idifftp
-double precision turb_schmidt
+double precision turb_schmidt, visls_0
 
 double precision rvoid(1)
 
@@ -190,7 +190,7 @@ call field_get_key_int(iflid, key_buoyant_id, is_buoyant_fld)
 ! If the vector is buoyant, it is inside the Navier Stokes loop, and so iterns >=1
 ! otherwise it is outside of the loop and iterns = -1.
 if (  (is_buoyant_fld.eq. 1 .and. iterns.eq.-1) &
-  .or.(is_buoyant_fld.eq.-1 .and. iterns.ne.-1)) return
+  .or.(is_buoyant_fld.eq. 0 .and. iterns.ne.-1)) return
 
 ! Key id for drift scalar
 call field_get_key_id("drift_scalar_model", keydri)
@@ -263,13 +263,16 @@ thetv  = vcopt%thetav
 
 call field_get_name(iflid, chaine)
 
-if(vcopt%iwarni.ge.1) then
+if (vcopt%iwarni.ge.1) then
   write(nfecra,1000) chaine(1:16)
 endif
 
 ! Retrieve turbulent Schmidt value for current vector
 call field_get_key_double(ivarfl(isca(iscal)), ksigmas, turb_schmidt)
 
+! Reference diffusivity
+call field_get_key_double(ivarfl(isca(iscal)), kvisl0, visls_0)
+
 !===============================================================================
 ! 2. Source terms
 !===============================================================================
@@ -286,13 +289,15 @@ do iel = 1, ncel
 enddo
 
 call ustsvv &
-!==========
 ( nvar   , nscal  , ncepdp , ncesmp ,                            &
   iscal  ,                                                       &
   icepdc , icetsm , itypsm ,                                     &
   dt     ,                                                       &
   ckupdc , smacel , smbrv  , fimp )
 
+! C version
+call user_source_terms(ivarfl(isca(iscal)), smbrv, fimp)
+
 ! Store the source terms for convective limiter
 call field_get_key_int(iflid, kst, st_id)
 if (st_id .ge.0) then
@@ -316,7 +321,7 @@ end if
 
 ! Si on extrapole les TS :
 !   SMBRV recoit -theta TS du pas de temps precedent
-!     (on aurait pu le faire avant ustssc, mais avec le risque que
+!     (on aurait pu le faire avant ustsvv, mais avec le risque que
 !      l'utilisateur l'ecrase)
 !   SMBRV recoit la partie du terme source qui depend de la variable
 !   A l'ordre 2, on suppose que le ROVSDT fourni par l'utilisateur est <0
@@ -358,7 +363,7 @@ endif
 !     Ordre 2 non pris en compte
 
 if (ippmod(iphpar).ge.1) then
-  call pptsvv(iscal, smbrv, fimp)
+  call pptsvv(iscal, smbrv)
 endif
 
 ! Mass source term
@@ -369,11 +374,9 @@ if (ncesmp.gt.0) then
 
   ! On incremente SMBRV par -Gamma RTPA et ROVSDT par Gamma
   allocate(gavinj(3,ncelet))
-  call catsmv &
- ( ncelet , ncel   , ncesmp , iiun   , isso2t(iscal) ,            &
-   icetsm , itypsm(1,ivar) ,                                      &
-   cell_f_vol , cvara_var    , smacel(1,ivar) , smacel(1,ipr),    &
-   smbrv  , fimp , gavinj)
+  call catsmv(ncesmp, 1, icetsm, itypsm(:,ivar),                     &
+              cell_f_vol, cvara_var, smacel(:,ivar), smacel(:,ipr),  &
+              smbrv, fimp, gavinj)
 
   ! Si on extrapole les TS on met Gamma Pinj dans cproa_vect_st
   if (st_prv_id .ge. 0) then
@@ -409,7 +412,7 @@ if (ippmod(icompf).ge.0.or.(idilat.gt.1.and.ipredfl.eq.1.and.irovar.eq.1)) then
 
 ! Low Mach compressible algos (conservative in time).
 ! Same algo for Volume of Fluid method.
-else if ((idilat.gt.1.or.ivofmt.ge.0).and.irovar.eq.1) then
+else if ((idilat.gt.1.or.ivofmt.gt.0).and.irovar.eq.1) then
   if (iterns.eq.1) then
     call field_get_val_prev2_s(icrom, pcrom)
   else
@@ -438,7 +441,7 @@ if (vcopt%idiff.ge.1) then
     idifftp = vcopt%idifft
     if (ifcvsl.lt.0) then
       do iel = 1, ncel
-        w1(iel) = visls0(iscal)                                     &
+        w1(iel) = visls_0                                         &
            + idifftp*max(visct(iel),zero)/turb_schmidt
       enddo
     else
@@ -479,9 +482,9 @@ if (vcopt%idiff.ge.1) then
       do iel = 1, ncel
 
         temp = vcopt%idifft*ctheta(iscal)/csrij
-        viscce(1,iel) = temp*visten(1,iel) + visls0(iscal)
-        viscce(2,iel) = temp*visten(2,iel) + visls0(iscal)
-        viscce(3,iel) = temp*visten(3,iel) + visls0(iscal)
+        viscce(1,iel) = temp*visten(1,iel) + visls_0
+        viscce(2,iel) = temp*visten(2,iel) + visls_0
+        viscce(3,iel) = temp*visten(3,iel) + visls_0
         viscce(4,iel) = temp*visten(4,iel)
         viscce(5,iel) = temp*visten(5,iel)
         viscce(6,iel) = temp*visten(6,iel)
@@ -631,9 +634,10 @@ endif
 iconvp = vcopt%iconv
 idiffp = vcopt%idiff
 idftnp = vcopt%idften
-ndircp = ndircl(ivar)
+ndircp = vcopt%ndircl
 nswrsp = vcopt%nswrsm
 nswrgp = vcopt%nswrgr
+imrgrp = vcopt%imrgra
 imligp = vcopt%imligr
 ircflp = vcopt%ircflu
 ischcp = vcopt%ischcv
@@ -660,7 +664,7 @@ call field_get_coefbf_v(iflid, cofbfp)
 call coditv &
 !==========
  ( idtvar , iterns , iflid  , iconvp , idiffp , ndircp ,          &
-   imrgra , nswrsp , nswrgp , imligp , ircflp , ivissv ,          &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp , ivissv ,          &
    ischcp , isstpp , iescap , idftnp , iswdyp ,                   &
    iwarnp ,                                                       &
    blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
@@ -708,28 +712,6 @@ if (allocated(diverg)) deallocate(diverg)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,                                                   &
-'   ** RESOLUTION POUR LA VARIABLE ',A16                       ,/,&
-'      ---------------------------                            ',/)
- 1200 format(1X,A16,' : BILAN EXPLICITE = ',E14.5)
-! 9000 format( &
-!'@                                                            ',/,&
-!'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-!'@                                                            ',/,&
-!'@ @@ ATTENTION : ERREUR DANS COVOFV                          ',/,&
-!'@    =========                                               ',/,&
-!'@    IVARSC DOIT ETRE UN ENTIER POSITIF STRICTEMENT          ',/,&
-!'@    IL VAUT ICI ',I10                                        ,/,&
-!'@                                                            ',/,&
-!'@  Le calcul ne peut etre execute.                           ',/,&
-!'@                                                            ',/,&
-!'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-!'@                                                            ',/)
-
-#else
-
  1000 format(/,                                                   &
 '   ** SOLVING VARIABLE ',A16                                  ,/,&
 '      ----------------'                                       ,/)
@@ -748,8 +730,6 @@ if (allocated(diverg)) deallocate(diverg)
 !'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 !'@                                                            ',/)
 
-#endif
-
 !----
 ! End
 !----
