@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -267,7 +267,7 @@ cs_1d_wall_thermal_create(void)
   _1d_wall_thermal.nmxt1d = 0;
 
   /* Allocate the izft1d array */
-  BFT_MALLOC(_1d_wall_thermal.izft1d, n_b_faces, int);
+  BFT_MALLOC(_1d_wall_thermal.izft1d, n_b_faces, cs_lnum_t);
 
   for (cs_lnum_t ifac = 0 ; ifac < n_b_faces ; ifac++) {
     _1d_wall_thermal.izft1d[ifac] = 0;
@@ -596,7 +596,7 @@ cs_1d_wall_thermal_read(void)
   cs_lnum_t           ii, jj, ifac, indfac, ierror;
   cs_lnum_t           n_b_faces = cs_glob_mesh->n_b_faces;
 
-  char nomsui[] = "1dwall_module";
+  char nomsui[] = "1dwall_module.csc";
 
   cs_restart_t             *suite;
   cs_mesh_location_type_t   support;
@@ -646,11 +646,11 @@ cs_1d_wall_thermal_read(void)
     /* Read the header */
     cs_lnum_t  *tabvar;
 
-    BFT_MALLOC(tabvar, 1, int);
+    BFT_MALLOC(tabvar, 1, cs_lnum_t);
 
     nbvent  = 1;
     support = CS_MESH_LOCATION_NONE;
-    typ_val = CS_TYPE_cs_int_t;
+    typ_val = CS_TYPE_int;
 
     ierror = cs_restart_read_section(suite,
                                      "version_fichier_suite_module_1d",
@@ -682,14 +682,14 @@ cs_1d_wall_thermal_read(void)
     cs_lnum_t  *tabvar;
     cs_lnum_t  mfpt1d;
     cs_gnum_t  mfpt1t;
-    cs_lnum_t  iok;
+    int  iok;
 
-    BFT_MALLOC(tabvar, n_b_faces, int);
+    BFT_MALLOC(tabvar, n_b_faces, cs_lnum_t);
 
     const char nomrub[] = "nb_pts_discretis";
     nbvent  = 1;
     support = CS_MESH_LOCATION_BOUNDARY_FACES;
-    typ_val = CS_TYPE_cs_int_t;
+    typ_val = CS_TYPE_int;
 
     ierror = cs_restart_read_section(suite,
                                      nomrub,
@@ -982,7 +982,7 @@ cs_1d_wall_thermal_write(void)
   cs_lnum_t            nbvent;
   cs_lnum_t            ii, jj, ifac;
 
-  char nomsui[] = "1dwall_module";
+  char nomsui[] = "1dwall_module.csc";
 
   cs_lnum_t n_b_faces = cs_glob_mesh->n_b_faces;
 
@@ -1011,7 +1011,7 @@ cs_1d_wall_thermal_write(void)
 
     nbvent  = 1;
     support = CS_MESH_LOCATION_NONE;
-    typ_val = CS_TYPE_cs_int_t;
+    typ_val = CS_TYPE_int;
 
     cs_restart_write_section(suite,
                              "version_fichier_suite_module_1d",
@@ -1032,7 +1032,7 @@ cs_1d_wall_thermal_write(void)
 
     nbvent  = 1;
     support = CS_MESH_LOCATION_BOUNDARY_FACES;
-    typ_val = CS_TYPE_cs_int_t;
+    typ_val = CS_TYPE_int;
 
     for (ii = 0 ; ii < _1d_wall_thermal.nfpt1d ; ii++) {
       ifac = _1d_wall_thermal.ifpt1d[ii] - 1;
@@ -1193,6 +1193,7 @@ cs_1d_wall_thermal_free(void)
     BFT_FREE(_1d_wall_thermal.local_models->z);
   BFT_FREE(_1d_wall_thermal.local_models);
   BFT_FREE(_1d_wall_thermal.ifpt1d);
+  BFT_FREE(_1d_wall_thermal.tppt1d);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1210,7 +1211,7 @@ cs_1d_wall_thermal_finalize(void)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Provide acces to cs_glob_1d_wall_thermal.
+ * \brief Provide access to cs_glob_1d_wall_thermal.
  */
 /*----------------------------------------------------------------------------*/
 
