@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -36,11 +36,45 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
+#include "cs_domain.h"
 
 /*----------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
 
+/*============================================================================
+ * Type definitions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------
+ * ALE type
+ *----------------------------------------------------------------------------*/
+
+enum {
+
+  CS_ALE_NONE = 0,
+  CS_ALE_LEGACY = 1,
+  CS_ALE_CDO = 2
+
+};
+
+/*=============================================================================
+ * Global variables
+ *============================================================================*/
+
+extern int cs_glob_ale;
+
+/*============================================================================
+ * Fortran wrapper function definitions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------
+ * Get pointer to cs_glob_ale
+ *----------------------------------------------------------------------------*/
+
+void
+cs_f_ale_get_pointers(int **iale);
+
 /*============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -65,7 +99,7 @@ cs_ale_update_mesh_quantities(cs_real_t  *min_vol,
 /*!
  * \brief  Project the displacement on mesh vertices (solved on cell center).
  *
- * \param[in]       ialtyb        Type of boundary for ALE
+ * \param[in]       ale_bc_type   Type of boundary for ALE
  * \param[in]       meshv         Mesh velocity
  * \param[in]       gradm         Mesh velocity gradient
  *                                (du_i/dx_j : gradv[][i][j])
@@ -77,7 +111,7 @@ cs_ale_update_mesh_quantities(cs_real_t  *min_vol,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_ale_project_displacement(const int           ialtyb[],
+cs_ale_project_displacement(const int           ale_bc_type[],
                             const cs_real_3_t  *meshv,
                             const cs_real_33_t  gradm[],
                             const cs_real_3_t  *claale,
@@ -90,12 +124,24 @@ cs_ale_project_displacement(const int           ialtyb[],
  * \brief  Update mesh in the ALE framework.
  *
  * \param[in]       itrale        number of the current ALE iteration
- * \param[in]       xyzno0        nodes coordinates of the initial mesh
  */
 /*----------------------------------------------------------------------------*/
+
 void
-cs_ale_update_mesh(const int           itrale,
-                   const cs_real_3_t  *xyzno0);
+cs_ale_update_mesh(const int           itrale);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Update ALE BCs for required for the fluid
+ *
+ * \param[out]      ale_bc_type   type of ALE bcs
+ * \param[out]      b_fluid_vel   Fluid velocity at boundary faces
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_ale_update_bcs(int         *ale_bc_type,
+                  cs_real_3_t *b_fluid_vel);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -105,17 +151,77 @@ cs_ale_update_mesh(const int           itrale,
  * so that it can be used to update mass fluxes (due to mesh displacement).
  *
  * \param[in]       iterns        Navier-Stokes iteration number
- * \param[in]       ndircl        Number of Dirichlet BCs for mesh velocity
  * \param[in]       impale        Indicator for fixed node displacement
- * \param[in]       ialtyb        Type of boundary for ALE
+ * \param[in]       ale_bc_type   Type of boundary for ALE
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_ale_solve_mesh_velocity(const int   iterns,
-                           const int   ndircl,
                            const int  *impale,
-                           const int  *ialtyb);
+                           const int  *ale_bc_type);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Activate the mesh velocity solving with CDO
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_ale_activate(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Test if mesh velocity solving with CDO is activated
+ *
+ * \return true ifmesh velocity solving with CDO is requested, false otherwise
+ */
+/*----------------------------------------------------------------------------*/
+
+bool
+cs_ale_is_activated(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Setup the equations related to mesh deformation.
+ *
+ * \param[in, out]   domain     pointer to a cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_ale_init_setup(cs_domain_t   *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Setup the equations solving the mesh velocity
+ *
+ * \param[in]   domain       pointer to a cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_ale_setup_boundaries(const cs_domain_t   *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Finalize the setup stage for the equation of the mesh velocity
+ *
+ * \param[in, out]  domain       pointer to a cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_ale_finalize_setup(cs_domain_t   *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Free the main structure related to the ALE mesh velocity solving
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_ale_destroy_all(void);
 
 /*----------------------------------------------------------------------------*/
 
