@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -48,6 +48,16 @@
 #include <dlfcn.h>
 #endif
 
+#if defined(HAVE_CATALYST)
+#  define _CS_EXIT_DEPLIB_CRASH_WORKAROUND 1
+#else
+#  define _CS_EXIT_DEPLIB_CRASH_WORKAROUND 0
+#endif
+
+#if _CS_EXIT_DEPLIB_CRASH_WORKAROUND
+#include <setjmp.h>
+#endif
+
 /*----------------------------------------------------------------------------
  * PLE library headers
  *----------------------------------------------------------------------------*/
@@ -68,6 +78,7 @@
 #include "cs_fp_exception.h"
 #include "cs_log.h"
 #include "cs_timer.h"
+#include "cs_version.h"
 
 /*----------------------------------------------------------------------------
  *  Header for the current file
@@ -92,7 +103,7 @@ BEGIN_C_DECLS
 
 /*
  * 'usual' maximum name length; a longer name is possible, but will
- * provoque a dynamic memory allocation.
+ * provoke a dynamic memory allocation.
  */
 
 #define CS_BASE_N_STRINGS                               5
@@ -136,24 +147,31 @@ static char  cs_glob_base_str[CS_BASE_N_STRINGS][CS_BASE_STRING_LEN + 1];
 
 /* Global variables associated with signal handling */
 
+static bool _cs_base_sighandlers_set = false;
+
 #if defined(SIGHUP)
 static _cs_base_sighandler_t cs_glob_base_sighup_save = SIG_DFL;
 #endif
 
+static _cs_base_sighandler_t cs_glob_base_sigabrt_save = SIG_DFL;
 static _cs_base_sighandler_t cs_glob_base_sigint_save = SIG_DFL;
 static _cs_base_sighandler_t cs_glob_base_sigterm_save = SIG_DFL;
 static _cs_base_sighandler_t cs_glob_base_sigfpe_save = SIG_DFL;
 static _cs_base_sighandler_t cs_glob_base_sigsegv_save = SIG_DFL;
 
-#if defined(__bgq__)
-static _cs_base_sighandler_t cs_glob_base_sigabrt_save = SIG_DFL;
-static _cs_base_sighandler_t cs_glob_base_sigtrap_save = SIG_DFL;
-#endif
-
 #if defined(SIGXCPU)
 static _cs_base_sighandler_t cs_glob_base_sigcpu_save = SIG_DFL;
 #endif
 
+/* Workaround for SIGSEGV at exit
+   with some ParaView Catalyst/OpenGL driver combinations */
+
+#if _CS_EXIT_DEPLIB_CRASH_WORKAROUND
+static jmp_buf _cs_exit_jmp_buf;
+#endif
+
+/* Dynamic library handling */
+
 #if defined(HAVE_DLOPEN)
 #if defined(CS_DLOPEN_USE_RTLD_GLOBAL)
 static int _cs_dlopen_flags = RTLD_LAZY | RTLD_GLOBAL;
@@ -173,13 +191,23 @@ static char *_cs_base_env_pkglibdir = NULL;
 
 /* Log file */
 
+static FILE  *_bft_printf_file = NULL;
 static char  *_bft_printf_file_name = NULL;
 static bool   _bft_printf_suppress = false;
+static bool   _cs_trace = false;
 
 /* Additional cleanup steps */
 
 static cs_base_atexit_t  * _cs_base_atexit = NULL;
 
+/* Additional MPI communicators */
+
+#if defined(HAVE_MPI)
+static int        _n_step_comms = 0;
+static int       *_step_ranks = NULL;
+static MPI_Comm  *_step_comm = NULL;
+#endif
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -198,6 +226,17 @@ _cs_base_bft_printf_null(const char  *format,
   return 0;
 }
 
+/*----------------------------------------------------------------------------
+ * False print of a message to standard output for discarded logs
+ *----------------------------------------------------------------------------*/
+
+static int
+_cs_base_bft_printf_file(const char  *format,
+                         va_list      arg_ptr)
+{
+  return  vfprintf(_bft_printf_file, format, arg_ptr);
+}
+
 /*----------------------------------------------------------------------------
  * Flush of log output buffer
  *----------------------------------------------------------------------------*/
@@ -218,6 +257,16 @@ _cs_base_bft_printf_flush_null(void)
   return 0;
 }
 
+/*----------------------------------------------------------------------------
+ * False flush of log output buffer for discarded logs
+ *----------------------------------------------------------------------------*/
+
+static int
+_cs_base_bft_printf_flush_file(void)
+{
+  return fflush(_bft_printf_file);
+}
+
 /*----------------------------------------------------------------------------
  * Print a message to the error output
  *
@@ -296,25 +345,104 @@ _cs_base_err_vprintf(const char  *format,
  * The message is repeated on the standard output and an error file.
  *----------------------------------------------------------------------------*/
 
+#if defined(__GNUC__)
+__attribute__((format(printf, 1, 2)))
+#endif
+
 static void
 _cs_base_err_printf(const char  *format,
                     ...)
 {
-  /* Initialisation de la liste des arguments */
+  /* Initialize arguments list */
 
   va_list  arg_ptr;
-
   va_start(arg_ptr, format);
 
-  /* message sur les sorties */
+  /* message on outputs */
 
   _cs_base_err_vprintf(format, arg_ptr);
 
-  /* Finalisation de la liste des arguments */
+  /* Finalize arguments list */
 
   va_end(arg_ptr);
 }
 
+#if _CS_EXIT_DEPLIB_CRASH_WORKAROUND
+
+/*----------------------------------------------------------------------------
+ * Handle a signal as a warning at exit
+ *----------------------------------------------------------------------------*/
+
+static void
+_cs_base_sig_exit_crash_workaround(int  signum)
+{
+  bft_printf_flush();
+
+  char sig_name[32] = "";
+
+  switch (signum) {
+
+  case SIGFPE:
+    snprintf(sig_name, 31, "SIGFPE");
+    break;
+
+  case SIGSEGV:
+    snprintf(sig_name, 31, "SIGSEGV");
+    break;
+
+  }
+
+  if (cs_glob_rank_id <= 0) {
+    fprintf(stderr, _("Warning: signal %s received during program exit.\n"),
+            sig_name);
+    cs_base_backtrace_dump(stderr, 3);
+  }
+
+  longjmp(_cs_exit_jmp_buf, 1);
+}
+
+/*----------------------------------------------------------------------------
+ * Set signal handlers to workaround an external library issue at exit.
+ *
+ * This is used at exit to transform errors into warnings on rank 0
+ * only during a standard exit routine.
+ *
+ * This is an ugly workaround to an ugly crash observed when exiting after
+ * have used VTK (Paraview Catalyst) libraries (on Debian Stretch with
+ * NVIDIA 390 driver at least).
+ *
+ * It is based on changing to a different signal handler for SIGSEGV
+ * (but could also be used for other signals), and using setjmp/longjmp
+ * to return from that handler and call _exit to avoid further issues.
+ * This avoids returning a nonzero error code to the caller when the
+ * code had finished correctly before calling exit(EXIT_SUCCESS).
+ *----------------------------------------------------------------------------*/
+
+static void
+_set_atexit_crash_workaround(void)
+{
+#if defined(HAVE_SIGACTION)
+
+  if (_cs_base_sighandlers_set) {
+    struct sigaction sa;
+    sa.sa_handler = *_cs_base_sig_exit_crash_workaround;
+    sigfillset(&sa.sa_mask);
+    sa.sa_flags = 0;
+    sa.sa_restorer = NULL;
+    if (sigaction(SIGSEGV, &sa, NULL) == -1 && cs_glob_rank_id <= 0)
+      perror("sigaction");
+
+    int i = setjmp(_cs_exit_jmp_buf);
+
+    if (i == 1)
+      _exit(EXIT_SUCCESS); /* This handler is only used during normal exit */
+  }
+
+#endif
+}
+
+#endif /* _CS_EXIT_DEPLIB_CRASH_WORKAROUND */
+
 /*----------------------------------------------------------------------------
  * Exit function
  *----------------------------------------------------------------------------*/
@@ -322,6 +450,9 @@ _cs_base_err_printf(const char  *format,
 static void
 _cs_base_exit(int status)
 {
+  if (status == EXIT_SUCCESS)
+    cs_base_update_status(NULL);
+
 #if defined(HAVE_MPI)
   {
     int mpi_flag;
@@ -359,6 +490,13 @@ _cs_base_exit(int status)
   }
 #endif /* HAVE_MPI */
 
+#if _CS_EXIT_DEPLIB_CRASH_WORKAROUND
+
+  if (status == EXIT_SUCCESS)
+    _set_atexit_crash_workaround();
+
+#endif
+
   exit(status);
 }
 
@@ -477,9 +615,8 @@ _cs_mem_error_handler(const char  *file_name,
  *----------------------------------------------------------------------------*/
 
 static void
-_cs_base_backtrace_print(int  niv_debut)
+_cs_base_backtrace_print(int  lv_start)
 {
-  size_t  ind;
   bft_backtrace_t  *tr = NULL;
 
   tr = bft_backtrace_create();
@@ -492,23 +629,23 @@ _cs_base_backtrace_print(int  niv_debut)
     const char *s_func;
     const char *s_addr;
 
-    const char s_inconnu[] = "?";
+    const char s_unknown[] = "?";
     const char s_vide[] = "";
     const char *s_prefix = s_vide;
 
-    size_t nbr = bft_backtrace_size(tr);
+    int nbr = bft_backtrace_size(tr);
 
     if (nbr > 0)
       _cs_base_err_printf(_("\nCall stack:\n"));
 
-    for (ind = niv_debut ; ind < nbr ; ind++) {
+    for (int ind = lv_start; ind < nbr; ind++) {
 
       s_file = bft_backtrace_file(tr, ind);
       s_func = bft_backtrace_function(tr, ind);
       s_addr = bft_backtrace_address(tr, ind);
 
       if (s_file == NULL)
-        s_file = s_inconnu;
+        s_file = s_unknown;
       if (s_func == NULL)
         strcpy(s_func_buf, "?");
       else {
@@ -517,10 +654,10 @@ _cs_base_backtrace_print(int  niv_debut)
         strcat(s_func_buf, ">");
       }
       if (s_addr == NULL)
-        s_addr = s_inconnu;
+        s_addr = s_unknown;
 
       _cs_base_err_printf("%s%4d: %-12s %-32s (%s)\n", s_prefix,
-                          ind-niv_debut+1, s_addr, s_func_buf, s_file);
+                          ind-lv_start+1, s_addr, s_func_buf, s_file);
 
     }
 
@@ -555,6 +692,10 @@ _cs_base_sig_fatal(int  signum)
     break;
 #endif
 
+  case SIGABRT:
+    _cs_base_err_printf(_("SIGABRT signal (abort) intercepted.\n"));
+    break;
+
   case SIGINT:
     _cs_base_err_printf(_("SIGINT signal (Control+C or equivalent) received.\n"
                           "--> computation interrupted by user.\n"));
@@ -575,14 +716,6 @@ _cs_base_sig_fatal(int  signum)
                           "intercepted!\n"));
     break;
 
-#if defined(__bgq__)
-  case SIGABRT:
-    _cs_base_err_printf(_("SIGABRT signal (Abort) "
-                          "intercepted.\n"));
-    cs_glob_base_sigabrt_save  = signal(SIGABRT, cs_glob_base_sigabrt_save);
-    break;
-#endif
-
 #if defined(SIGXCPU)
   case SIGXCPU:
     _cs_base_err_printf(_("SIGXCPU signal (CPU time limit reached) "
@@ -617,7 +750,7 @@ _get_path(const char   *dir_path,
     const char *cs_root_dir = NULL;
     const char *rel_path = NULL;
 
-    /* Allow for displacable install */
+    /* Allow for displaceable install */
 
     if (*env_path != NULL)
       return *env_path;
@@ -686,7 +819,28 @@ _get_path(const char   *dir_path,
 #if defined(HAVE_MPI)
 
 /*----------------------------------------------------------------------------
- *  Finalisation MPI
+ * Destroy a set of reduced communicators
+ *----------------------------------------------------------------------------*/
+
+static void
+_finalize_reduced_communicators(void)
+{
+  int comm_id;
+
+  for (comm_id = 1; comm_id < _n_step_comms; comm_id++) {
+    if (   _step_comm[comm_id] != MPI_COMM_NULL
+        && _step_comm[comm_id] != cs_glob_mpi_comm)
+      MPI_Comm_free(&(_step_comm[comm_id]));
+  }
+
+  BFT_FREE(_step_comm);
+  BFT_FREE(_step_ranks);
+
+  _n_step_comms = 0;
+}
+
+/*----------------------------------------------------------------------------
+ *  MPI finalization
  *----------------------------------------------------------------------------*/
 
 static void
@@ -976,84 +1130,6 @@ cs_base_logfile_head(int    argc,
        "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
   struct tm time_cnv;
 
-  /* Define MPI Information */
-
-#if defined(MPI_SUBVERSION)
-
-  char mpi_vendor_lib[32] = "";
-  char mpi_lib[32] = "";
-
-  /* Base MPI library information */
-
-#if defined(MPI_VENDOR_NAME)
-
-#if defined(OMPI_MAJOR_VERSION)
-  snprintf(mpi_lib, 31, "%s %d.%d.%d",
-           MPI_VENDOR_NAME,
-           OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION, OMPI_RELEASE_VERSION);
-#elif defined(MPICH2_VERSION)
-  snprintf(mpi_lib, 31, "%s %s", MPI_VENDOR_NAME, MPICH2_VERSION);
-#elif defined(MPICH_VERSION)
-  snprintf(mpi_lib, 31, "%s %s", MPI_VENDOR_NAME, MPICH_VERSION);
-#else
-  snprintf(mpi_lib, 31, "%s", MPI_VENDOR_NAME);
-#endif
-
-#elif defined(OPEN_MPI)
-#if defined(OMPI_MAJOR_VERSION)
-  snprintf(mpi_lib, 31, "Open MPI %d.%d.%d",
-           OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION, OMPI_RELEASE_VERSION);
-#else
-  snprintf(mpi_lib, 31, "Open MPI");
-#endif
-
-#elif defined(MPICH2)
-#if defined(MPICH2_VERSION)
-  snprintf(mpi_lib, 31, "MPICH2 %s", MPICH2_VERSION);
-#else
-  snprintf(mpi_lib, 31, "MPICH2");
-#endif
-#elif defined(MPICH_NAME)
-#if defined(MPICH_VERSION)
-  snprintf(mpi_lib, 31, "MPICH %s", MPICH_VERSION);
-#else
-  snprintf(mpi_lib, 31, "MPICH");
-#endif
-
-#elif defined(LAM_MPI)
-#if defined(LAM_VERSION)
-  snprintf(mpi_lib, 31, "LAM/MPI %s", LAM_VERSION);
-#else
-  snprintf(mpi_lib, 31, "LAM/MPI");
-#endif
-#endif
-
-  mpi_lib[31] = '\0';
-
-  /* Possible additional MPI vendor information */
-
-#if defined(MVAPICH2_VERSION)
-  snprintf(mpi_vendor_lib, 31, "MVAPICH2 %s", MVAPICH2_VERSION);
-#elif defined(MSMPI_VER)
-  snprintf(mpi_vendor_lib, 31, "MS-MPI");
-#elif defined(PLATFORM_MPI)
-  {
-    int v, v0, v1, v2, v3;
-    v0 =  PLATFORM_MPI>>24;
-    v =  (PLATFORM_MPI - (v0<<24));
-    v1 =  v>>16;
-    v =  (v - (v1<<16));
-    v2 =  v>>8;
-    v3 =  (v - (v2<<8));
-    snprintf(mpi_vendor_lib, 31, "Platform MPI %x.%x.%x.%x\n",
-             v0, v1, v2, v3);
-  }
-#endif
-
-  mpi_vendor_lib[31] = '\0';
-
-#endif /* defined(MPI_SUBVERSION) */
-
   /* Determine compilation date */
 
   for (ii = 0; ii < 12; ii++) {
@@ -1094,7 +1170,7 @@ cs_base_logfile_head(int    argc,
              "                      Version %s\n\n",
              CS_APP_VERSION);
 
-  bft_printf("\n  Copyright (C) 1998-2018 EDF S.A., France\n\n");
+  bft_printf("\n  Copyright (C) 1998-2021 EDF S.A., France\n\n");
 
 #if defined(CS_REVISION)
   if (strlen(CS_REVISION) > 0)
@@ -1104,22 +1180,6 @@ cs_base_logfile_head(int    argc,
   bft_printf(_("  build %s\n"), str);
 
 #if defined(MPI_SUBVERSION)
-  if (mpi_vendor_lib[0] != '\0') {
-    if (mpi_lib[0] != '\0')
-      bft_printf(_("  MPI version %d.%d (%s, based on %s)\n\n"),
-                 MPI_VERSION, MPI_SUBVERSION, mpi_vendor_lib, mpi_lib);
-    else
-      bft_printf(_("  MPI version %d.%d (%s)\n\n"),
-                 MPI_VERSION, MPI_SUBVERSION, mpi_vendor_lib);
-  }
-  else {
-    if (mpi_lib[0] != '\0')
-      bft_printf(_("  MPI version %d.%d (%s)\n\n"),
-                 MPI_VERSION, MPI_SUBVERSION, mpi_lib);
-    else
-      bft_printf(_("  MPI version %d.%d\n\n"),
-                 MPI_VERSION, MPI_SUBVERSION);
-  }
 #endif
 
   bft_printf("\n");
@@ -1252,6 +1312,81 @@ cs_base_mpi_init(int    *argc,
 #endif
 }
 
+/*----------------------------------------------------------------------------
+ * Return a reduced communicator matching a multiple of the total
+ * number of ranks.
+ *
+ * This updates the number of reduced communicators if necessary.
+ *
+ * parameters:
+ *   rank_step <-- associated multiple of total ranks
+ *----------------------------------------------------------------------------*/
+
+MPI_Comm
+cs_base_get_rank_step_comm(int  rank_step)
+{
+  if (rank_step <= 1)
+    return cs_glob_mpi_comm;
+
+  int n_ranks = cs_glob_n_ranks / rank_step;
+  if (cs_glob_n_ranks % rank_step > 0)
+    n_ranks += 1;
+
+  if (n_ranks <= 1)
+    return MPI_COMM_NULL;
+
+  int comm_id = 0;
+  if (_n_step_comms > 0) {
+    while (   _step_ranks[comm_id] != n_ranks
+           && comm_id < _n_step_comms)
+      comm_id++;
+  }
+
+  /* Add communicator if required */
+
+  if (comm_id >= _n_step_comms) {
+
+    _n_step_comms += 1;
+    BFT_REALLOC(_step_comm, _n_step_comms, MPI_Comm);
+    BFT_REALLOC(_step_ranks, _n_step_comms, int);
+
+    _step_ranks[comm_id] = n_ranks;
+
+    if (n_ranks == cs_glob_n_ranks)
+      _step_comm[comm_id] = cs_glob_mpi_comm;
+
+    else if (n_ranks == 1)
+      _step_comm[comm_id] = MPI_COMM_NULL;
+
+    else {
+
+      int ranges[1][3];
+      MPI_Group old_group, new_group;
+
+      MPI_Barrier(cs_glob_mpi_comm); /* For debugging */
+
+      MPI_Comm_size(cs_glob_mpi_comm, &n_ranks);
+      MPI_Comm_group(cs_glob_mpi_comm, &old_group);
+
+      ranges[0][0] = 0;
+      ranges[0][1] = n_ranks - 1;
+      ranges[0][2] = rank_step;
+
+      MPI_Group_range_incl(old_group, 1, ranges, &new_group);
+      MPI_Comm_create(cs_glob_mpi_comm, new_group, &(_step_comm[comm_id]));
+      MPI_Group_free(&new_group);
+
+      MPI_Group_free(&old_group);
+
+      MPI_Barrier(cs_glob_mpi_comm); /* For debugging */
+
+    }
+
+  }
+
+  return _step_comm[comm_id];
+}
+
 #endif /* HAVE_MPI */
 
 /*----------------------------------------------------------------------------
@@ -1327,6 +1462,8 @@ cs_base_error_init(bool  signal_defaults)
       cs_glob_base_sighup_save  = signal(SIGHUP, _cs_base_sig_fatal);
 #endif
 
+    cs_glob_base_sigabrt_save  = signal(SIGABRT, _cs_base_sig_fatal);
+
     if (cs_glob_rank_id <= 0) {
       cs_glob_base_sigint_save  = signal(SIGINT, _cs_base_sig_fatal);
       cs_glob_base_sigterm_save = signal(SIGTERM, _cs_base_sig_fatal);
@@ -1335,16 +1472,12 @@ cs_base_error_init(bool  signal_defaults)
     cs_glob_base_sigfpe_save  = signal(SIGFPE, _cs_base_sig_fatal);
     cs_glob_base_sigsegv_save = signal(SIGSEGV, _cs_base_sig_fatal);
 
-#if defined(__bgq__)
-    cs_glob_base_sigabrt_save  = signal(SIGABRT, _cs_base_sig_fatal);
-    cs_glob_base_sigtrap_save  = signal(SIGTRAP, _cs_base_sig_fatal);
-#endif
-
 #if defined(SIGXCPU)
     if (cs_glob_rank_id <= 0)
       cs_glob_base_sigcpu_save = signal(SIGXCPU, _cs_base_sig_fatal);
 #endif
 
+    _cs_base_sighandlers_set = true;
   }
 }
 
@@ -1425,20 +1558,22 @@ void
 cs_base_mem_finalize(void)
 {
   int    ind_bil, itot;
-  double valreal[2];
+  double valreal[4];
 
 #if defined(HAVE_MPI)
   int  imax = 0, imin = 0;
-  double val_sum[2];
-  int  ind_min[2];
-  _cs_base_mpi_double_int_t  val_in[2], val_min[2], val_max[2];
+  double val_sum[4];
+  int  ind_min[4];
+  _cs_base_mpi_double_int_t  val_in[4], val_min[4], val_max[4];
 #endif
 
-  int   ind_val[2] = {1, 1};
+  int   ind_val[4] = {1, 1, 1, 1};
   const char  unit[8] = {'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'};
 
   const char  * type_bil[] = {N_("Total memory used:                       "),
-                              N_("Theoretical instrumented dynamic memory: ")};
+                              N_("Theoretical instrumented dynamic memory: "),
+                              N_("Virtual memory used:                     "),
+                              N_("Shared libraries memory used:            ")};
 
   /* Memory summary */
 
@@ -1447,30 +1582,32 @@ cs_base_mem_finalize(void)
 
   valreal[0] = (double)bft_mem_usage_max_pr_size();
   valreal[1] = (double)bft_mem_size_max();
+  valreal[2] = (double)bft_mem_usage_max_vm_size();
+  valreal[3] = (double)bft_mem_usage_shared_lib_size();
 
   /* Ignore inconsistent measurements */
 
-  for (ind_bil = 0; ind_bil < 2; ind_bil++) {
+  for (ind_bil = 0; ind_bil < 4; ind_bil++) {
     if (valreal[ind_bil] < 1.0)
       ind_val[ind_bil] = 0;
   }
 
 #if defined(HAVE_MPI)
   if (cs_glob_n_ranks > 1) {
-    MPI_Reduce(ind_val, ind_min, 2, MPI_INT, MPI_MIN,
+    MPI_Reduce(ind_val, ind_min, 4, MPI_INT, MPI_MIN,
                0, cs_glob_mpi_comm);
-    MPI_Reduce(valreal, val_sum, 2, MPI_DOUBLE, MPI_SUM,
+    MPI_Reduce(valreal, val_sum, 4, MPI_DOUBLE, MPI_SUM,
                0, cs_glob_mpi_comm);
-    for (ind_bil = 0; ind_bil < 2; ind_bil++) {
+    for (ind_bil = 0; ind_bil < 4; ind_bil++) {
       val_in[ind_bil].val = valreal[ind_bil];
       val_in[ind_bil].rank = cs_glob_rank_id;
     }
-    MPI_Reduce(val_in, val_min, 2, MPI_DOUBLE_INT, MPI_MINLOC,
+    MPI_Reduce(val_in, val_min, 4, MPI_DOUBLE_INT, MPI_MINLOC,
                0, cs_glob_mpi_comm);
-    MPI_Reduce(val_in, val_max, 2, MPI_DOUBLE_INT, MPI_MAXLOC,
+    MPI_Reduce(val_in, val_max, 4, MPI_DOUBLE_INT, MPI_MAXLOC,
                0, cs_glob_mpi_comm);
     if (cs_glob_rank_id == 0) {
-      for (ind_bil = 0; ind_bil < 2; ind_bil++) {
+      for (ind_bil = 0; ind_bil < 4; ind_bil++) {
         ind_val[ind_bil]  = ind_min[ind_bil];
         valreal[ind_bil] = val_sum[ind_bil];
       }
@@ -1480,7 +1617,7 @@ cs_base_mem_finalize(void)
 
   /* Similar handling of several instrumentation methods */
 
-  for (ind_bil = 0 ; ind_bil < 2 ; ind_bil++) {
+  for (ind_bil = 0; ind_bil < 4; ind_bil++) {
 
     /* If an instrumentation method returns an apparently consistent
        result, print it. */
@@ -1506,9 +1643,14 @@ cs_base_mem_finalize(void)
 
       /* Print to log file */
 
-      cs_log_printf(CS_LOG_PERFORMANCE,
-                    _("  %s %12.3f %ciB\n"),
-                    _(type_bil[ind_bil]), valreal[ind_bil], unit[itot]);
+      if (ind_bil < 2 || cs_glob_n_ranks < 2)
+        cs_log_printf(CS_LOG_PERFORMANCE,
+                      _("  %s %12.3f %ciB\n"),
+                      _(type_bil[ind_bil]), valreal[ind_bil], unit[itot]);
+      else
+        cs_log_printf(CS_LOG_PERFORMANCE,
+                      _("  %s\n"),
+                      _(type_bil[ind_bil]));
 
 #if defined(HAVE_MPI)
       if (cs_glob_n_ranks > 1 && cs_glob_rank_id == 0) {
@@ -1529,6 +1671,13 @@ cs_base_mem_finalize(void)
   cs_log_printf(CS_LOG_PERFORMANCE, "\n");
   cs_log_separator(CS_LOG_PERFORMANCE);
 
+  /* Finalize extra communicators now as they use memory allocated through
+     bft_mem_* API */
+
+#if defined(HAVE_MPI)
+  _finalize_reduced_communicators();
+#endif
+
   /* Finalize memory handling */
 
   if (cs_glob_base_bft_mem_init == true) {
@@ -1620,34 +1769,143 @@ cs_base_time_summary(void)
   cs_log_separator(CS_LOG_PERFORMANCE);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Update status file.
+ *
+ * If the format string is NULL, the file is removed.
+
+ * \param[in]  format  format string, or NULL
+ * \param[in]  ...     format arguments
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_update_status(const char  *format,
+                      ...)
+{
+  static const char _status_file_name[] = "run_status.running";
+  static FILE *_status_file = NULL;
+
+  if (cs_glob_rank_id < 1) {
+
+    if (format == NULL) {
+      if (_status_file != NULL) {
+        if (fclose(_status_file) == 0) {
+          _status_file = NULL;
+          remove(_status_file_name);
+        }
+      }
+    }
+
+    else {
+
+      va_list  arg_ptr;
+      va_start(arg_ptr, format);
+
+      /* Output to trace */
+
+#if defined(va_copy) || defined(__va_copy)
+      if (_cs_trace && format != NULL) {
+        va_list arg_ptr_2;
+#if defined(va_copy)
+        va_copy(arg_ptr_2, arg_ptr);
+#else
+        __va_copy(arg_ptr_2, arg_ptr);
+#endif
+        vprintf(format, arg_ptr_2);
+        va_end(arg_ptr_2);
+      }
+#endif
+
+      /* Status file */
+
+      if (_status_file == NULL)
+        _status_file = fopen(_status_file_name, "w");
+
+      if (_status_file != NULL) {
+        long p_size = ftell(_status_file);
+        fseek(_status_file, 0, SEEK_SET);
+        vfprintf(_status_file, format, arg_ptr);
+        long c_size = ftell(_status_file);
+
+        while (p_size > c_size) {
+          size_t l = 0;
+          char buf[64];
+          while (l < 64 && p_size > c_size) {
+            buf[l++] = ' ';
+            p_size--;
+          }
+          fwrite(buf, 1, l, _status_file);
+        }
+      }
+
+      va_end(arg_ptr);
+
+    }
+
+  }
+}
+
+/*----------------------------------------------------------------------------
+ * Set tracing of progress on or off.
+ *
+ * This function should be called before cs_base_bft_printf_set() if tracing
+ * is activated.
+ *
+ * parameters:
+ *   trace  <-- trace progress to stdout
+ *----------------------------------------------------------------------------*/
+
+void
+cs_base_trace_set(bool  trace)
+{
+  if (_bft_printf_file_name == NULL)
+    _cs_trace = trace;
+}
+
 /*----------------------------------------------------------------------------
  * Set output file name and suppression flag for bft_printf().
  *
  * This allows redirecting or suppressing logging for different ranks.
  *
  * parameters:
- *   log_name    <-- base file name for log, or NULL for stdout
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to <log_name> file
+ *   log_name    <-- base file name for log
  *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to <log_name>_n*" file;
- *                   2: redirected to "/dev/null" (suppressed)
+ *                   false: to "/dev/null" (suppressed)
+ *                   true: to <log_name>_r*.log" file;
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_bft_printf_init(const char  *log_name,
-                        int          r0_log_flag,
-                        int          rn_log_flag)
+                        bool         rn_log_flag)
 {
   BFT_FREE(_bft_printf_file_name);
   _bft_printf_suppress = false;
 
+  const char ext[] = ".log";
+
+  /* Allow bypassing this with environment variable to accommodate
+     some debug habits */
+
+  bool log_to_stdout = false;
+  const char *p = getenv("CS_LOG_TO_STDOUT");
+  if (p != NULL) {
+    if (atoi(p) > 0)
+      log_to_stdout = true;
+  }
+
   /* Rank 0 */
 
-  if (cs_glob_rank_id < 1 && r0_log_flag == 1 && log_name != NULL) {
+  if (   cs_glob_rank_id < 1
+      && log_name != NULL
+      && log_to_stdout == false) {
 
-    BFT_MALLOC(_bft_printf_file_name, strlen(log_name) + 1, char);
+    BFT_MALLOC(_bft_printf_file_name,
+               strlen(log_name) + strlen(ext) + 1,
+               char);
     strcpy(_bft_printf_file_name, log_name);
+    strcat(_bft_printf_file_name, ext);
 
   }
 
@@ -1655,21 +1913,24 @@ cs_base_bft_printf_init(const char  *log_name,
 
   else if (cs_glob_rank_id > 0) {
 
-    if (log_name != NULL && rn_log_flag == 1) { /* Non-suppressed logs */
+    if (log_name != NULL && rn_log_flag > 0) { /* Non-suppressed logs */
 
-      int i;
-      int n_dec = 1;
-      for (i = cs_glob_n_ranks; i >= 10; i /= 10, n_dec += 1);
-      BFT_MALLOC(_bft_printf_file_name, strlen(log_name) + n_dec + 3, char);
-      sprintf(_bft_printf_file_name,
-              "%s_r%0*d",
-              log_name,
-              n_dec,
-              cs_glob_rank_id);
+      if (log_to_stdout == false) {
+        int n_dec = 1;
+        for (int i = cs_glob_n_ranks; i >= 10; i /= 10, n_dec += 1);
+        BFT_MALLOC(_bft_printf_file_name,
+                   strlen(log_name) + n_dec + 3 + strlen(ext), char);
+        sprintf(_bft_printf_file_name,
+                "%s_r%0*d%s",
+                log_name,
+                n_dec,
+                cs_glob_rank_id,
+                ext);
+      }
 
     }
 
-    else if (rn_log_flag == 2) { /* Suppressed logs */
+    else { /* Suppressed logs */
 
       _bft_printf_suppress = true;
       bft_printf_proxy_set(_cs_base_bft_printf_null);
@@ -1687,45 +1948,59 @@ cs_base_bft_printf_init(const char  *log_name,
  * This allows redirecting or suppressing logging for different ranks.
  *
  * parameters:
- *   log_name    <-- base file name for log, or NULL for stdout
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to <log_name> file
+ *   log_name    <-- base file name for log
  *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to <log_name>_n*" file;
- *                   2: suppressed
+ *                   false: to "/dev/null" (suppressed)
+ *                   true: to <log_name>_r*.log" file;
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_bft_printf_set(const char  *log_name,
-                       int          r0_log_flag,
-                       int          rn_log_flag)
+                       bool         rn_log_flag)
 {
-  cs_base_bft_printf_init(log_name, r0_log_flag, rn_log_flag);
+  cs_base_bft_printf_init(log_name, rn_log_flag);
 
   if (_bft_printf_file_name != NULL && _bft_printf_suppress == false) {
 
-    bft_printf_proxy_set(vprintf);
-    bft_printf_flush_proxy_set(_cs_base_bft_printf_flush);
-    ple_printf_function_set(vprintf);
-
     /* Redirect log */
 
     if (_bft_printf_file_name != NULL) {
 
-      FILE *fp = freopen(_bft_printf_file_name, "w", stdout);
+      bft_printf_proxy_set(vprintf);
+      bft_printf_flush_proxy_set(_cs_base_bft_printf_flush);
+      ple_printf_function_set(vprintf);
 
-      if (fp == NULL)
-        bft_error(__FILE__, __LINE__, errno,
-                  _("It is impossible to redirect the standard output "
-                    "to file:\n%s"), _bft_printf_file_name);
+      if (cs_glob_rank_id > 0 || _cs_trace == false) {
+
+        FILE *fp = freopen(_bft_printf_file_name, "w", stdout);
+
+        if (fp == NULL)
+          bft_error(__FILE__, __LINE__, errno,
+                    _("It is impossible to redirect the standard output "
+                      "to file:\n%s"), _bft_printf_file_name);
 
 #if defined(HAVE_DUP2)
-      if (dup2(fileno(fp), fileno(stderr)) == -1)
-        bft_error(__FILE__, __LINE__, errno,
-                  _("It is impossible to redirect the standard error "
-                    "to file:\n%s"), _bft_printf_file_name);
+        if (dup2(fileno(fp), fileno(stderr)) == -1)
+          bft_error(__FILE__, __LINE__, errno,
+                    _("It is impossible to redirect the standard error "
+                      "to file:\n%s"), _bft_printf_file_name);
 #endif
 
+      }
+      else {
+
+        _bft_printf_file = fopen(_bft_printf_file_name, "w");
+        if (_bft_printf_file == NULL)
+          bft_error(__FILE__, __LINE__, errno,
+                    _("Error opening log file:\n%s"),
+                    _bft_printf_file_name);
+
+        bft_printf_proxy_set(_cs_base_bft_printf_file);
+        bft_printf_flush_proxy_set(_cs_base_bft_printf_flush_file);
+        ple_printf_function_set(_cs_base_bft_printf_file);
+
+      }
+
     }
 
   }
@@ -1784,7 +2059,7 @@ cs_base_warn(const char  *file_name,
  * Define a function to be called when entering cs_exit() or bft_error().
  *
  * Compared to the C atexit(), only one function may be called (latest
- * setting wins), but the function is called slighty before exit,
+ * setting wins), but the function is called slightly before exit,
  * so it is well adapted to cleanup such as flushing of non-C API logging.
  *
  * parameters:
@@ -1870,9 +2145,9 @@ cs_base_string_f_to_c_create(const char  *f_str,
 void
 cs_base_string_f_to_c_free(char  **c_str)
 {
-  cs_int_t ind;
+  int ind;
 
-  for (ind = 0 ; ind < CS_BASE_N_STRINGS ; ind++) {
+  for (ind = 0; ind < CS_BASE_N_STRINGS; ind++) {
     if (*c_str == cs_glob_base_str[ind]) {
       cs_glob_base_str_is_free[ind] = true;
       *c_str = NULL;
@@ -2055,8 +2330,8 @@ cs_base_dlopen(const char *filename)
   void *retval = NULL;
 
   /* Disable floating-point traps as the initialization of some libraries
-     may interfere with this (for example, embree, and optional Paraview
-     depedency) */
+     may interfere with this (for example, embree, and optional ParaView
+     dependency) */
 
   cs_fp_exception_disable_trap();
 
@@ -2079,7 +2354,7 @@ cs_base_dlopen(const char *filename)
 /*!
  * \brief Load a plugin's dynamic library
  *
- * This function is similar to \ref cs_base_dlopen, execpt that only
+ * This function is similar to \ref cs_base_dlopen, except that only
  * the base plugin file name (with no extension) needs to be given.
  * It is assumed the file is available in the code's "pkglibdir" directory,
  *
@@ -2209,6 +2484,175 @@ cs_base_get_dl_function_pointer(void        *handle,
 
 #endif /* defined(HAVE_DLOPEN) */
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Dump a stack trace to a file
+ *
+ * \param[in]  f         pointer to file in which to dump trace
+ * \param[in]  lv_start  start level in stack trace
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_backtrace_dump(FILE  *f,
+                       int    lv_start)
+{
+  bft_backtrace_t  *tr = NULL;
+
+  tr = bft_backtrace_create();
+
+  if (tr != NULL) {
+
+    char s_func_buf[67];
+
+    const char *s_file;
+    const char *s_func;
+    const char *s_addr;
+
+    const char s_unknown[] = "?";
+    const char s_vide[] = "";
+    const char *s_prefix = s_vide;
+
+    int nbr = bft_backtrace_size(tr);
+
+    if (nbr > 0)
+      fprintf(f, "\nCall stack:\n");
+
+    for (int ind = lv_start; ind < nbr; ind++) {
+
+      s_file = bft_backtrace_file(tr, ind);
+      s_func = bft_backtrace_function(tr, ind);
+      s_addr = bft_backtrace_address(tr, ind);
+
+      if (s_file == NULL)
+        s_file = s_unknown;
+      if (s_func == NULL)
+        strcpy(s_func_buf, "?");
+      else {
+        s_func_buf[0] = '<';
+        strncpy(s_func_buf + 1, s_func, 64);
+        strcat(s_func_buf, ">");
+      }
+      if (s_addr == NULL)
+        s_addr = s_unknown;
+
+      fprintf(f, "%s%4d: %-12s %-32s (%s)\n", s_prefix,
+              ind-lv_start+1, s_addr, s_func_buf, s_file);
+
+    }
+
+    bft_backtrace_destroy(tr);
+
+    if (nbr > 0)
+      fprintf(f, "End of stack\n\n");
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Query run-time directory info, using working directory names.
+ *
+ * Returned names are allocated if non-NULL, so should be deallocated by
+ * the caller when no longer needed.
+ *
+ * Names are extracted from the working directory structure, which is expected
+ * to be of the form:
+ * <prefix>/study_name/case_name/RESU/run_id
+ *
+ * or, in the case o a coupled run:
+ * <prefix>/study_name/RESU_COUPLING/run_id/case_name
+ *
+ * If some names cannot be queried, NULL is returned.
+ *
+ * \param[out]  run_id      run_id, or NULL
+ * \param[out]  case_name   case name, or NULL
+ * \param[out]  study_name  study name, or NULL
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_get_run_identity(char  **run_id,
+                         char  **case_name,
+                         char  **study_name)
+{
+  /* Use execution directory if name is unavailable */
+
+  const char *c[4] = {NULL, NULL, NULL, NULL};
+
+  if (run_id != NULL)
+    *run_id = NULL;
+  if (case_name != NULL)
+    *case_name = NULL;
+  if (study_name != NULL)
+    *study_name = NULL;
+
+#if defined(HAVE_GETCWD)
+
+  int buf_size = 128;
+  char *wd = NULL, *buf = NULL;
+
+  while (wd == NULL) {
+    buf_size *= 2;
+    BFT_REALLOC(buf, buf_size, char);
+    wd = getcwd(buf, buf_size);
+    if (wd == NULL && errno != ERANGE)
+      bft_error(__FILE__, __LINE__, errno,
+                _("Error querying working directory.\n"));
+  }
+
+  int i = strlen(buf) - 1;
+
+  for (int comp_id = 0; comp_id < 4; comp_id++) {
+    while (i > 0 && buf[i-1] != '/')
+      i--;
+    if (i >= 0) {
+      c[comp_id] = buf+i;
+      if (i > 0) {
+        i--;
+        buf[i] = '\0';
+      }
+    }
+    else
+      break;
+  }
+
+  const char *_run_id = c[0];
+  const char *_case_name = NULL;
+  const char *_study_name = NULL;
+
+  if (c[1] != NULL) {
+    if (strcmp(c[1], "RESU") == 0) {
+      _case_name = c[2];
+      _study_name = c[3];
+    }
+    else if (c[2] != NULL) {
+      if (strcmp(c[2], "RESU_COUPLING") == 0) {
+        _run_id = c[1];
+        _case_name = c[0];
+        _study_name = c[3];
+      }
+    }
+  }
+
+  if (run_id != NULL && _run_id != NULL) {
+    BFT_MALLOC(*run_id, strlen(_run_id) + 1, char);
+    strcpy(*run_id, _run_id);
+  }
+  if (case_name != NULL && _case_name != NULL) {
+    BFT_MALLOC(*case_name, strlen(_case_name) + 1, char);
+    strcpy(*case_name, _case_name);
+  }
+  if (study_name != NULL && _study_name != NULL) {
+    BFT_MALLOC(*study_name, strlen(_study_name) + 1, char);
+    strcpy(*study_name, _study_name);
+  }
+
+  BFT_FREE(buf);
+
+#endif /* defined(HAVE_GETCWD) */
+
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
