@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -65,13 +65,9 @@
  * be better optimized on memory-aligned data (if 0 here, no alignment
  * is leveraged). This alignment is not exploited yet in Code_Saturne. */
 
-#if defined(__bgq__)
-#define CS_MEM_ALIGN 32
-#else
 #define CS_MEM_ALIGN 0
-#endif
 
-#define CS_BASE_STRING_LEN                             64
+#define CS_BASE_STRING_LEN                             80
 
 /*----------------------------------------------------------------------------*/
 
@@ -108,9 +104,9 @@ static inline const char *
 cs_base_strtf(bool  boolean)
 {
   if (boolean)
-    return "true";
+    return "**True**";
   else
-    return "false";
+    return "**False**";
 }
 
 /*----------------------------------------------------------------------------
@@ -163,6 +159,20 @@ void
 cs_base_mpi_init(int    *argc,
                  char  **argv[]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return a reduced communicator matching a multiple of the total
+ *        number of ranks.
+ *
+ * This updates the number of reduced communicators if necessary.
+ *
+ * \param[in]  rank_step  associated multiple of total ranks
+ */
+/*----------------------------------------------------------------------------*/
+
+MPI_Comm
+cs_base_get_rank_step_comm(int  rank_step);
+
 #endif /* defined(HAVE_MPI) */
 
 /*----------------------------------------------------------------------------
@@ -212,24 +222,48 @@ cs_base_mem_finalize(void);
 void
 cs_base_time_summary(void);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Update status file.
+ *
+ * If the format string is NULL, the file is removed.
+
+ * \param[in]  format  format string, or NULL
+ * \param[in]  ...     format arguments
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_update_status(const char  *format,
+                      ...);
+
+/*----------------------------------------------------------------------------
+ * Set tracing of progress on or off.
+ *
+ * parameters:
+ *   trace  <-- trace progress to stdout
+ *----------------------------------------------------------------------------*/
+
+void
+cs_base_trace_set(bool trace);
+
+
 /*----------------------------------------------------------------------------
  * Set output file name and suppression flag for bft_printf().
  *
  * This allows redirecting or suppressing logging for different ranks.
  *
  * parameters:
- *   log_name    <-- base file name for log, or NULL for stdout
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to <log_name> file
+ *   log_name    <-- base file name for log
  *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to <log_name>_n*" file;
- *                   2: redirected to "/dev/null" (suppressed)
+ *   rn_log_flag <-- redirection for ranks > 0 log:
+ *                   false: to "/dev/null" (suppressed)
+ *                   true: to <log_name>_r*.log" file;
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_bft_printf_init(const char  *log_name,
-                        int          r0_log_flag,
-                        int          rn_log_flag);
+                        bool         rn_log_flag);
 
 /*----------------------------------------------------------------------------
  * Replace default bft_printf() mechanism with internal mechanism.
@@ -238,17 +272,11 @@ cs_base_bft_printf_init(const char  *log_name,
  *
  * parameters:
  *   log_name    <-- base file name for log
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to "listing" file
- *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to "listing_n*" file;
- *                   2: redirected to "/dev/null" (suppressed)
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_bft_printf_set(const char  *log_name,
-                       int          r0_log_flag,
-                       int          rn_log_flag);
+                       bool         rn_log_flag);
 
 /*----------------------------------------------------------------------------
  * Return name of default log file.
@@ -292,7 +320,7 @@ cs_base_warn(const char  *file_name,
  * Define a function to be called when entering cs_exit() or bft_error().
  *
  * Compared to the C atexit(), only one function may be called (latest
- * setting wins), but the function is called slighty before exit,
+ * setting wins), but the function is called slightly before exit,
  * so it is well adapted to cleanup such as flushing of non-C API logging.
  *
  * parameters:
@@ -431,11 +459,11 @@ cs_base_dlopen(const char *filename);
 /*!
  * \brief Load a plugin's dynamic library
  *
- * This function is similar to \ref cs_base_dlopen, execpt that only
+ * This function is similar to \ref cs_base_dlopen, except that only
  * the base plugin file name (with no extension) needs to be given.
  * It is assumed the file is available in the code's "pkglibdir" directory,
  *
- * \param[in]  filename  path to shared library file
+ * \param[in]  name  path to shared library file
  *
  * \return  handle to shared library
  */
@@ -504,6 +532,46 @@ cs_base_get_dl_function_pointer(void        *handle,
 
 #endif /* defined(HAVE_DLOPEN) */
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Dump a stack trace to a file
+ *
+ * \param[in]  f         pointer to file in which to dump trace
+ * \param[in]  lv_start  start level in stack trace
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_backtrace_dump(FILE  *f,
+                       int    lv_start);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Query run-time directory info, using working directory names.
+ *
+ * Returned names are allocated if non-NULL, so should be deallocated by
+ * the caller when no longer needed.
+ *
+ * Names are extracted from the working directory structure, which is expected
+ * to be of the form:
+ * <prefix>/study_name/case_name/RESU/run_id
+ *
+ * or, in the case o a coupled run:
+ * <prefix>/study_name/RESU_COUPLING/run_id/case_name
+ *
+ * If some names cannot be queried, NULL is returned.
+ *
+ * \param[out]  run_id      run_id, or NULL
+ * \param[out]  case_name   case name, or NULL
+ * \param[out]  study_name  study name, or NULL
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_get_run_identity(char  **run_id,
+                         char  **case_name,
+                         char  **study_name);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
