@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -96,23 +96,23 @@ extern void cs_f_flush_logs(void);
 
 extern void CS_PROCF (csprnt, CSPRNT)
 (
-  char       *cs_buf_print,
-  cs_int_t   *msgsize
+  char  *cs_buf_print,
+  int   *msgsize
 );
 
 /*----------------------------------------------------------------------------
- * Initialize Fortran log (listing) files
+ * Initialize Fortran log files
  *----------------------------------------------------------------------------*/
 
 extern void CS_PROCF (csopli, CSOPLI)
 (
- const cs_int_t  *infecr,  /* <-- value to assign to nfecra */
- const cs_int_t  *isuppr,  /* <-- supress output if 1 */
- const cs_int_t  *ierror   /* --> error code (0 if sucess) */
+ const int  *infecr,  /* <-- value to assign to nfecra */
+ const int  *isuppr,  /* <-- supress output if 1 */
+ const int  *ierror   /* --> error code (0 if sucess) */
 );
 
 /*----------------------------------------------------------------------------
- * Close log (listing) handled by Fortran: (CLose LIsting)
+ * Close log handled by Fortran: (CLose LIsting)
  *----------------------------------------------------------------------------*/
 
 extern void CS_PROCF (csclli, CSCLLI)
@@ -146,7 +146,7 @@ static int
 _bft_printf_f(const char     *const format,
               va_list         arg_ptr)
 {
-  cs_int_t  msgsize;
+  int  msgsize;
 
   /* Buffer for printing from C code: print to a character string, which will
      be printed to file by Fortran code.
@@ -237,7 +237,7 @@ _close_c_log_file(void)
 
 void CS_PROCF (csexit, CSEXIT)
 (
-  const cs_int_t  *status
+  const int  *status
 )
 {
   cs_exit(*status);
@@ -276,8 +276,8 @@ void CS_PROCF (dmtmps, DMTMPS)
 
 void CS_PROCF (csmkdr, CSMKDR)
 (
- const char       *dirnam,
- const cs_int_t   *dirlen
+ const char  *dirnam,
+ const int   *dirlen
 )
 {
   char    *bufname;
@@ -364,7 +364,7 @@ void CS_PROCF (cserf, CSERF)
 
 void CS_PROCF (cslogname, CSLOGNAME)
 (
- const cs_int_t   *len,
+ const int        *len,
  char             *dir
  CS_ARGF_SUPP_CHAINE              /*     (possible 'length' arguments added
                                          by many Fortran compilers) */
@@ -412,7 +412,7 @@ void CS_PROCF (cslogname, CSLOGNAME)
 
 void CS_PROCF (csdatadir, CSDATADIR)
 (
- const cs_int_t   *len,
+ const int        *len,
  char             *dir
  CS_ARGF_SUPP_CHAINE              /*     (possible 'length' arguments added
                                          by many Fortran compilers) */
@@ -448,31 +448,39 @@ void CS_PROCF (csdatadir, CSDATADIR)
  *
  * parameters:
  *   log_name    <-- base file name for log, or NULL for stdout
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to <log_name> file
  *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to <log_name>_n*" file;
- *                   2: redirected to "/dev/null" (suppressed)
+ *                   false:  to "/dev/null" (suppressed)
+ *                   true: redirected to <log_name>_n*.log" file;
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_fortran_bft_printf_set(const char  *log_name,
-                               int          r0_log_flag,
-                               int          rn_log_flag)
+                               bool         rn_log_flag)
 {
   const char *name = NULL;
   bool suppress = false;
-  cs_int_t infecr = 6, isuppr = 0, ierror = 0;
+  int  infecr = 6, isuppr = 0, ierror = 0;
 
   /* C output */
 
-  cs_base_bft_printf_init(log_name, r0_log_flag, rn_log_flag);
+  cs_base_bft_printf_init(log_name, rn_log_flag);
 
   name = cs_base_bft_printf_name();
   suppress = cs_base_bft_printf_suppressed();
 
   if (suppress == false) {
 
+    /* Allow bypassing this with environment variable to accommodate
+       some debug habits */
+
+    const char *p = getenv("CS_LOG_TO_STDOUT");
+    if (p != NULL) {
+      if (atoi(p) > 0)
+        name = NULL;
+    }
+
+    /* In standard case, redirect if possible */
+
     if (name != NULL) {
 
       _bft_printf_file = fopen(name, "w");
@@ -568,7 +576,7 @@ cs_base_fortran_bft_printf_to_f(void)
 
   if (name != NULL) {
 
-    cs_int_t nfecra = 9, isuppr = 0, ierror = 0;
+    int nfecra = 9, isuppr = 0, ierror = 0;
 
     /* Close C output */
 
@@ -600,6 +608,64 @@ cs_base_fortran_bft_printf_to_f(void)
   ple_printf_function_set(_bft_printf_f);
 }
 
+/*----------------------------------------------------------------------------
+ * Wrappers to cs_user_*
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_extra_operations_initialize_wrapper(void)
+{
+  cs_user_extra_operations_initialize(cs_glob_domain);
+}
+
+void
+cs_user_physical_properties_wrapper(void)
+{
+  cs_user_physical_properties(cs_glob_domain);
+}
+
+void
+cs_user_source_terms_wrapper(int         f_id,
+                             cs_real_t  *st_exp,
+                             cs_real_t  *st_imp)
+{
+  cs_user_source_terms(cs_glob_domain,
+                       f_id,
+                       st_exp,
+                       st_imp);
+}
+
+void
+cs_user_extra_operations_wrapper(void)
+{
+  cs_user_extra_operations(cs_glob_domain);
+}
+
+void
+cs_user_initialization_wrapper(void)
+{
+  cs_user_initialization(cs_glob_domain);
+}
+
+void
+cs_user_parameters_wrapper(void)
+{
+  cs_user_parameters(cs_glob_domain);
+}
+
+void
+cs_user_finalize_setup_wrapper(void)
+{
+  cs_user_finalize_setup(cs_glob_domain);
+}
+
+void
+cs_user_porosity_wrapper(void)
+{
+  cs_user_porosity(cs_glob_domain);
+}
+
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
