@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -66,7 +66,7 @@ BEGIN_C_DECLS
 
 void CS_PROCF (csexit, CSEXIT)
 (
-  const cs_int_t  *status
+  const int  *status
 );
 
 /*----------------------------------------------------------------------------
@@ -99,8 +99,8 @@ void CS_PROCF (dmtmps, DMTMPS)
 
 void CS_PROCF (csmkdr, CSMKDR)
 (
- const char       *dirnam,
- const cs_int_t   *dirlen
+ const char  *dirnam,
+ const int   *dirlen
 );
 
 /*----------------------------------------------------------------------------
@@ -156,8 +156,8 @@ void CS_PROCF (cserf, CSERF)
 
 void CS_PROCF (cslogname, CSLOGNAME)
 (
- const cs_int_t   *len,
- char             *dir
+ const int   *len,
+ char        *dir
  CS_ARGF_SUPP_CHAINE              /*     (possible 'length' arguments added
                                          by many Fortran compilers) */
 );
@@ -180,8 +180,8 @@ void CS_PROCF (cslogname, CSLOGNAME)
 
 void CS_PROCF (csdatadir, CSDATADIR)
 (
- const cs_int_t   *len,
- char             *dir
+ const int   *len,
+ char        *dir
  CS_ARGF_SUPP_CHAINE              /*     (possible 'length' arguments added
                                          by many Fortran compilers) */
 );
@@ -200,17 +200,14 @@ void CS_PROCF (csdatadir, CSDATADIR)
  *
  * parameters:
  *   log_name    <-- base file name for log, or NULL for stdout
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to <log_name> file
  *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to <log_name>_n*" file;
- *                   2: redirected to "/dev/null" (suppressed)
+ *                   false:  to "/dev/null" (suppressed)
+ *                   true: redirected to <log_name>_n*.log" file;
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_fortran_bft_printf_set(const char  *log_name,
-                               int          r0_log_flag,
-                               int          rn_log_flag);
+                               bool         rn_log_flag);
 
 /*----------------------------------------------------------------------------
  * Switch bft_printf() mechanism to C output.
@@ -230,6 +227,36 @@ cs_base_fortran_bft_printf_to_c(void);
 void
 cs_base_fortran_bft_printf_to_f(void);
 
+/*----------------------------------------------------------------------------
+ * Wrappers for C functions
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_extra_operations_initialize_wrapper(void);
+
+void
+cs_user_physical_properties_wrapper(void);
+
+void
+cs_user_source_terms_wrapper(int         f_id,
+                             cs_real_t  *st_exp,
+                             cs_real_t  *st_imp);
+
+void
+cs_user_extra_operations_wrapper(void);
+
+void
+cs_user_initialization_wrapper(void);
+
+void
+cs_user_parameters_wrapper(void);
+
+void
+cs_user_finalize_setup_wrapper(void);
+
+void
+cs_user_porosity_wrapper(void);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
