@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -77,7 +77,7 @@ extern const int  *cs_glob_bc_face_zone;
 /*----------------------------------------------------------------------------
  * Handling of boundary condition definition errors and associated output.
  *
- * This function checks for errors, and simply returns if no errors are
+ * This function checks for errors, and simply returns if no error is
  * encountered. In case of error, it outputs helpful information so as to
  * make it easier to locate the matching faces.
  *
@@ -178,6 +178,38 @@ cs_boundary_conditions_mapped_set(const cs_field_t          *f,
                                   int                        nvar,
                                   cs_real_t                  rcodcl[]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Add location of locate shifted boundary face coordinates on
+ *        cells or boundary faces for automatic interpolation.
+ *
+ * \note
+ * This function is currently restricted to mapping of boundary face
+ * locations (usually from boundary zones) to cell of boundary face
+ * locations, but could be extended to other location types in the future.
+ *
+ * \param[in]  bc_location_id      id of selected boundary mesh location;
+ *                                 currently restricted to subsets of
+ *                                 boundary faces (i.e. boundary zone
+ *                                 location ids).
+ * \param[in]  source_location_id  id of selected location  mesh location
+ *                                 (usually CS_MESH_LOCATION_CELLS but can be
+ *                                 a more restricted cell or boundary face zone
+ *                                 location location id).
+ * \param[in]  coord_shift      coordinates shift relative to selected
+ *                              boundary faces
+ * \param[in]  tolerance        relative tolerance for point location.
+ *
+ * \return  id of added map
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_boundary_conditions_add_map(int         bc_location_id,
+                               int         source_location_id,
+                               cs_real_t   coord_shift[3],
+                               double      tolerance);
+
 /*----------------------------------------------------------------------------
  * Create the boundary conditions face type and face zone arrays
  *----------------------------------------------------------------------------*/
@@ -186,7 +218,9 @@ void
 cs_boundary_conditions_create(void);
 
 /*----------------------------------------------------------------------------
- * Free the boundary conditions face type and face zone arrays
+ * Free the boundary conditions face type and face zone arrays.
+ *
+ * This also frees boundary condition mappings which may have been defined.
  *----------------------------------------------------------------------------*/
 
 void
@@ -416,6 +450,44 @@ cs_boundary_conditions_set_dirichlet_vector_aniso(cs_real_3_t    a,
   bf[2][0] = hintt[5];
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Update per variable boundary condition codes.
+ *
+ * \param[in]       nvar             number of variables requiring BC's
+ * \param[in]       itypfb           type of boundary for each face
+ * \param[in, out]  icodcl           boundary condition codes
+ * \param[in, out]  rcodcl           boundary condition values
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_boundary_conditions_compute(int     nvar,
+                               int     itypfb[],
+                               int     icodcl[],
+                               double  rcodcl[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Automatic adjustments for boundary condition codes.
+ *
+ * Currently handles mapped inlets, after the call to \ref stdtcl.
+ * As portions of stdtcl are migrated to C, they should be called here,
+ * before mapped inlets.
+ *
+ * \param[in]       nvar             number of variables requiring BC's
+ * \param[in]       itypfb           type of boundary for each face
+ * \param[in, out]  icodcl           boundary condition codes
+ * \param[in, out]  rcodcl           boundary condition values
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_boundary_conditions_complete(int     nvar,
+                                int     itypfb[],
+                                int     icodcl[],
+                                double  rcodcl[]);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
