@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,7 +54,7 @@ BEGIN_C_DECLS
  *============================================================================*/
 
 /*!
- * @defgroup volume_zone_flags Flags specifying general boundary zone type
+ * @defgroup boundary_zone_flags Flags specifying general boundary zone type
  *
  * @{
  */
@@ -324,7 +324,7 @@ cs_boundary_zone_log_setup(void);
  */
 /*----------------------------------------------------------------------------*/
 
-cs_lnum_t
+int
 cs_boundary_zone_n_type_zones(int  type_flag);
 
 /*----------------------------------------------------------------------------*/
@@ -390,6 +390,15 @@ cs_boundary_zone_update_face_class_id(void);
 int
 cs_boundary_zone_max_class_or_zone_id(void);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print boundary zones information to listing file
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_boundary_zone_print_info(void);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
