@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -61,6 +61,7 @@
 
 #include "cs_file.h"
 #include "cs_log.h"
+#include "cs_notebook.h"
 #include "cs_parall.h"
 #include "cs_post.h"
 #include "cs_resource.h"
@@ -465,7 +466,7 @@ _comm_read_sock_to_queue(cs_control_queue_t  *queue,
 
     /* Check for end of command (end of line if not escaped
        by continuation character such as "/" or "," */
-    size_t cut_id = start_id + ret;
+    size_t cut_id = start_id + ret - 1;
     bool escape = false;
     queue->buf_idx[2] = cut_id;
     while (cut_id > 0 && queue->buf[cut_id] != '\0') {
@@ -810,24 +811,30 @@ _read_next_opt_int(const char  **s,
  * Read next value, expecting double precision floating point value
  *
  * parameters:
- *   cur_line <-- pointer to line buffer
- *   s        <-> current string position
- *   val      --> value read
+ *   skip_prev <-- skip previous string ?
+ *   cur_line  <-- pointer to line buffer
+ *   s         <-> current string position
+ *   val       --> value read
  *
  * returns:
  *   number of values read (1 for success, 0 otherwise)
  *----------------------------------------------------------------------------*/
 
 static int
-_read_next_double(const char   *cur_line,
+_read_next_double(bool          skip_prev,
+                  const char   *cur_line,
                   const char  **s,
                   double       *val)
 {
   int n_val = 0;
 
   const char *p = *s;
-  while (*p != '\0' && *p != ' ' && *p != '\t')
-    p++;
+
+  if (skip_prev) {
+    while (*p != '\0' && *p != ' ' && *p != '\t')
+      p++;
+  }
+
   while (*p != '\0' && (*p == ' ' || *p == '\t'))
     p++;
   *s = p;
@@ -844,7 +851,7 @@ _read_next_double(const char   *cur_line,
  * Read next value, expecting string
  *
  * parameters:
- *   skip_prev <-- skipe previous string ?
+ *   skip_prev <-- skip previous string ?
  *   s         <-> current string position
  *   s_val     --> pointer to string read
  *----------------------------------------------------------------------------*/
@@ -900,7 +907,7 @@ _control_checkpoint(const char   *cur_line,
   }
   else if (strncmp(*s, "time_value ", 11) == 0) {
     double t;
-    if (_read_next_double(cur_line, s, &t) > 0) {
+    if (_read_next_double(true, cur_line, s, &t) > 0) {
       cs_restart_checkpoint_set_next_tv(t);
       bft_printf("  %-32s %12.5g\n",
                  "checkpoint_time_value", t);
@@ -908,7 +915,7 @@ _control_checkpoint(const char   *cur_line,
   }
   else if (strncmp(*s, "wall_time ", 10) == 0) {
     double wt;
-    if (_read_next_double(cur_line, s, &wt) > 0) {
+    if (_read_next_double(true, cur_line, s, &wt) > 0) {
       cs_restart_checkpoint_set_next_wt(wt);
       bft_printf("  %-32s %12.5g\n",
                  "checkpoint_wall_time", wt);
@@ -924,7 +931,7 @@ _control_checkpoint(const char   *cur_line,
   }
   else if (strncmp(*s, "time_value_interval ", 20) == 0) {
     double t;
-    if (_read_next_double(cur_line, s, &t) > 0) {
+    if (_read_next_double(true, cur_line, s, &t) > 0) {
       if (t > 0) {
         cs_restart_checkpoint_set_defaults(-1, t, -1.);
         bft_printf("  %-32s %12.5g\n",
@@ -937,14 +944,14 @@ _control_checkpoint(const char   *cur_line,
   }
   else if (strncmp(*s, "wall_time_interval ", 19) == 0) {
     double wt;
-    if (_read_next_double(cur_line, s, &wt) > 0) {
+    if (_read_next_double(true, cur_line, s, &wt) > 0) {
       if (wt > 0) {
         cs_restart_checkpoint_set_defaults(-1, -1., wt);
         bft_printf("  %-32s %12.5g\n",
                    "checkpoint_wall_time_interval", wt);
       }
       else
-        bft_printf("  %-32s %12.5g %s\n",
+         bft_printf("  %-32s %12.5g %s\n",
                    "checkpoint_wall_time_interval", wt, _("ignored"));
     }
   }
@@ -952,6 +959,64 @@ _control_checkpoint(const char   *cur_line,
     bft_printf(_("   ignored: \"%s\"\n"), cur_line);
 }
 
+/*----------------------------------------------------------------------------
+ * Handle command file line relative to notebook
+ *
+ * parameters:
+ *   ts       <-- pointer to time step status
+ *   s        <-> pointer to current position in line
+ *----------------------------------------------------------------------------*/
+
+static void
+_control_notebook(const cs_time_step_t   *ts,
+                  char                   *cur_line,
+                  char                  **s)
+{
+  *s += 9; /* shift in string by length of "notebook_" part */
+
+  bool ignored = true;
+  if (strncmp(*s, "set ", 4) == 0) {
+    char *name;
+    double val = 0.;
+    _read_next_string(true, s, &name);
+    if (_read_next_double(false, cur_line, (const char **)s, &val) == 1) {
+      int editable;
+      cs_notebook_parameter_is_present(name, &editable);
+      if (editable) {
+        cs_notebook_parameter_set_value(name, val);
+        bft_printf("  %-32s \"%s\" set to %12.5g\n",
+                   "notebook", name, val);
+      }
+      ignored = false;
+    }
+  } else if (strncmp(*s, "get ", 4) == 0) {
+    char *name;
+    double val = 0.;
+    _read_next_string(true, s, &name);
+    int editable;
+    int is_present = cs_notebook_parameter_is_present(name,
+                                                      &editable);
+    if (is_present > 0) {
+      val = cs_notebook_parameter_value_by_name(name);
+      bft_printf("  %-32s \"%s\" get : %12.5g\n",
+                 "notebook", name, val);
+      ignored = false;
+
+#if defined(HAVE_SOCKET)
+      if (_cs_glob_control_comm != NULL) {
+        char reply[20] = "\0";
+        sprintf(reply, "get: %.3f", val);
+        _comm_write_sock(_cs_glob_control_comm, reply, 1, strlen(reply) + 1);
+      }
+#endif
+
+    }
+  }
+
+  if (ignored)
+    bft_printf(_("   ignored: \"%s\"\n"), cur_line);
+}
+
 /*----------------------------------------------------------------------------
  * Handle command file line relative to postprocessing
  *
@@ -965,7 +1030,7 @@ _control_postprocess(const cs_time_step_t   *ts,
                      char                   *cur_line,
                      const char            **s)
 {
-  *s += 12; /* shift in string by lenght of "postprocess_" part */
+  *s += 12; /* shift in string by length of "postprocess_" part */
 
   if (strncmp(*s, "time_step ", 10) == 0) {
     int nt = 0, writer_id = 0;
@@ -984,7 +1049,7 @@ _control_postprocess(const cs_time_step_t   *ts,
   else if (strncmp(*s, "time_value ", 11) == 0) {
     int writer_id = 0;
     double t = 0.;
-    if (_read_next_double(cur_line, s, &t) > 0) {
+    if (_read_next_double(true, cur_line, s, &t) > 0) {
       if (_read_next_opt_int(s, &writer_id) == 0)
         writer_id = 0;
       if (t >= 0)
@@ -1113,7 +1178,7 @@ _parse_control_buffer(const char         *name,
     }
     else if (strncmp(s, "max_time_value ", 15) == 0) {
       double t_max;
-      if (_read_next_double(cur_line, (const char **)&s, &t_max) > 0)
+      if (_read_next_double(true, cur_line, (const char **)&s, &t_max) > 0)
         t_max = CS_MAX(t_max, ts->t_cur);
       cs_time_step_define_t_max(t_max);
       bft_printf("  %-32s %12.5g (%s %12.5g)\n",
@@ -1121,7 +1186,7 @@ _parse_control_buffer(const char         *name,
     }
     else if (strncmp(s, "max_wall_time ", 14) == 0) {
       double wt_max;
-      if (_read_next_double(cur_line, (const char **)&s, &wt_max) > 0)
+      if (_read_next_double(true, cur_line, (const char **)&s, &wt_max) > 0)
         wt_max = CS_MAX(wt_max, cs_timer_wtime());
       bft_printf("  %-32s %12.5g (%s %12.5g)\n",
                  "max_wall_time", wt_max, _("current:"),
@@ -1133,7 +1198,7 @@ _parse_control_buffer(const char         *name,
 
     else if (strncmp(s, "control_file_wtime_interval ", 28) == 0) {
       double wt;
-      if (_read_next_double(cur_line, (const char **)&s, &wt) > 0)
+      if (_read_next_double(true, cur_line, (const char **)&s, &wt) > 0)
         _control_file_wt_interval = wt;
     }
 
@@ -1147,6 +1212,11 @@ _parse_control_buffer(const char         *name,
     else if (strncmp(s, "postprocess_", 12) == 0)
       _control_postprocess(ts, cur_line, (const char **)&s);
 
+    /* Notebook options */
+
+    else if (strncmp(s, "notebook_", 9) == 0)
+      _control_notebook(ts, cur_line, &s);
+
     /* Force flush of logs */
 
     else if (strncmp(s, "flush", 5) == 0) {
@@ -1175,6 +1245,7 @@ _parse_control_buffer(const char         *name,
 
     else if (strncmp(s, "disconnect ", 11) == 0) {
       cs_control_comm_finalize();
+      break;
     }
 
     /* Advance */
@@ -1354,9 +1425,17 @@ cs_control_check_file(void)
 
   /* Test control queue and connection second */
 
-  if (_control_advance_steps > 0)
+  if (_control_advance_steps > 0) {
+
     _control_advance_steps -= 1;
 
+#if defined(HAVE_SOCKET)
+    char reply[13] = "Iteration OK";
+    if (_cs_glob_control_comm != NULL)
+      _comm_write_sock(_cs_glob_control_comm, reply, 1, strlen(reply)+1);
+#endif
+  }
+
   if (   _cs_glob_control_queue != NULL
       && _control_advance_steps < 1) {
 
