@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -119,9 +119,9 @@ static double _cs_coupling_ts_multiplier = 1.0;
 
 void CS_PROCF(cplsyn, CPLSYN)
 (
- cs_int_t         *ntmabs,
- const cs_int_t   *ntcabs,
- cs_real_t        *dtref
+ int         *ntmabs,
+ const int   *ntcabs,
+ cs_real_t   *dtref
 )
 {
   int  current_ts_id = *ntcabs;
@@ -151,7 +151,7 @@ void CS_PROCF(cplsyn, CPLSYN)
 
 void CS_PROCF(cplact, CPLACT)
 (
- cs_int_t         *isync
+ int         *isync
 )
 {
   if (cs_coupling_is_sync_active())
@@ -170,12 +170,14 @@ void CS_PROCF(cplact, CPLACT)
 /*!
  * \brief Discover other applications in the same MPI root communicator.
  *
- * \param[in]  app_name  name of this instance of Code_Saturne
+ * \param[in]  app_name         name of this instance of Code_Saturne
+ * \param[in]  forced_app_type  name of app type, or NULL
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_coupling_discover_mpi_apps(const char  *app_name)
+cs_coupling_discover_mpi_apps(const char  *app_name,
+                              const char  *forced_app_type)
 {
   int mpi_flag;
   int world_size;
@@ -209,12 +211,20 @@ cs_coupling_discover_mpi_apps(const char  *app_name)
       bft_printf_flush();
     }
 
-    _cs_glob_coupling_mpi_app_world
-      = ple_coupling_mpi_set_create(_cs_coupling_sync_flag,
-                                    app_type,
-                                    app_name,
-                                    MPI_COMM_WORLD,
-                                    cs_glob_mpi_comm);
+    if (forced_app_type == NULL)
+      _cs_glob_coupling_mpi_app_world
+        = ple_coupling_mpi_set_create(_cs_coupling_sync_flag,
+                                      app_type,
+                                      app_name,
+                                      MPI_COMM_WORLD,
+                                      cs_glob_mpi_comm);
+    else
+      _cs_glob_coupling_mpi_app_world
+        = ple_coupling_mpi_set_create(_cs_coupling_sync_flag,
+                                      forced_app_type,
+                                      app_name,
+                                      MPI_COMM_WORLD,
+                                      cs_glob_mpi_comm);
 
     n_apps = ple_coupling_mpi_set_n_apps(_cs_glob_coupling_mpi_app_world);
     app_id = ple_coupling_mpi_set_get_app_id(_cs_glob_coupling_mpi_app_world);
@@ -304,7 +314,7 @@ cs_coupling_get_sync_flag(void)
  *
  * This flag is used by all couplings based on the PLE (Parallel Location
  * and Exchange) group synchronization mechanism, which include couplings
- * with SYRTHES 4, Code_Saturne, and NEPTUNE_CFD.
+ * with SYRTHES, code_saturne, and neptune_cfd.
  *
  * It is defined by a mask, so for example flags f1, f2, and f3 may be
  * combined using the "f1 | f2 | f2" syntax.
@@ -314,17 +324,17 @@ cs_coupling_get_sync_flag(void)
  * iteration, but the actual time step is recomputed later.
  *
  * Possible flags are:
- *   PLE_COUPLING_TS_MIN        Use smallest time step
- *   PLE_COUPLING_TS_LEADER     Prescribe time step for the group
- *                              (only one member may set this flag)
- *   PLE_COUPLING_UNSTEADY      Inform others that this instance is
- *                              using an unsteady solution approach
- *   PLE_COUPLING_STEADY        Inform others that this instance is
- *                              using a teady solution approach
- *   PLE_COUPLING_USER_1        User definable flag
- *   PLE_COUPLING_USER_2        User definable flag
- *   PLE_COUPLING_USER_3        User definable flag
- *   PLE_COUPLING_USER_4        User definable flag
+ *  - PLE_COUPLING_TS_MIN        Use smallest time step
+ *  - PLE_COUPLING_TS_LEADER     Prescribe time step for the group
+ *                               (only one member may set this flag)
+ *  - PLE_COUPLING_UNSTEADY      Inform others that this instance is
+ *                               using an unsteady solution approach
+ *  - PLE_COUPLING_STEADY        Inform others that this instance is
+ *                               using a steady solution approach
+ *  - PLE_COUPLING_USER_1        User definable flag
+ *  - PLE_COUPLING_USER_2        User definable flag
+ *  - PLE_COUPLING_USER_3        User definable flag
+ *  - PLE_COUPLING_USER_4        User definable flag
  *
  * To force stopping, PLE_COUPLING_STOP may be set. In this case,
  * the calculation will stop at the first synchronization, even if
@@ -442,7 +452,7 @@ cs_coupling_sync_apps(int      flags,
     }
     sync_flags = sync_flags | flags | stop_mask;
 
-    if (current_ts_id >= *max_ts_id)
+    if (current_ts_id >= *max_ts_id && *max_ts_id >= 0)
       sync_flags = sync_flags | PLE_COUPLING_STOP;
     else {
       sync_flags = sync_flags | PLE_COUPLING_NEW_ITERATION;
@@ -507,7 +517,7 @@ cs_coupling_sync_apps(int      flags,
       /* Handle time stepping behavior */
 
       if (app_status[i] & PLE_COUPLING_STOP) {
-        if (*max_ts_id > current_ts_id) {
+        if (*max_ts_id > current_ts_id || *max_ts_id < 0) {
           ai = ple_coupling_mpi_set_get_info(_cs_glob_coupling_mpi_app_world, i);
           bft_printf
             (_("\nApplication \"%s\" (%s) requested calculation stop.\n"),
@@ -533,7 +543,7 @@ cs_coupling_sync_apps(int      flags,
       }
 
       if (app_status[i] & PLE_COUPLING_LAST) {
-        if (*max_ts_id > current_ts_id + 1) {
+        if (*max_ts_id > current_ts_id + 1 || *max_ts_id < 0) {
           ai = ple_coupling_mpi_set_get_info(_cs_glob_coupling_mpi_app_world, i);
           bft_printf
             (_("\nApplication \"%s\" (%s) requested last iteration.\n"),
