@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -82,9 +82,9 @@ BEGIN_C_DECLS
 
 void CS_PROCF(cplsyn, CPLSYN)
 (
- cs_int_t         *ntmabs,
- const cs_int_t   *ntcabs,
- cs_real_t        *dtref
+ int         *ntmabs,
+ const int   *ntcabs,
+ cs_real_t   *dtref
  );
 
 /*----------------------------------------------------------------------------
@@ -101,7 +101,7 @@ void CS_PROCF(cplsyn, CPLSYN)
 
 void CS_PROCF(cplact, CPLACT)
 (
- cs_int_t         *isync
+ int         *isync
 );
 
 /*============================================================================
@@ -114,11 +114,13 @@ void CS_PROCF(cplact, CPLACT)
  * Discover other applications in the same MPI root communicator.
  *
  * parameters:
- *   app_name <-- name of this instance of Code_Saturne.
+ *   app_name        <-- name of this instance of Code_Saturne.
+ *   forced_app_type <-- name of app type, or NULL
  *----------------------------------------------------------------------------*/
 
 void
-cs_coupling_discover_mpi_apps(const char  *app_name);
+cs_coupling_discover_mpi_apps(const char  *app_name,
+                              const char  *forced_app_type);
 
 /*----------------------------------------------------------------------------
  * Finalize MPI coupling helper structures.
@@ -156,7 +158,7 @@ cs_coupling_get_sync_flag(void);
  *
  * This flag is used by all couplings based on the PLE (Parallel Location
  * and Exchange) group synchronization mechanism, which include couplings
- * with SYRTHES 4, Code_Saturne, and NEPTUNE_CFD.
+ * with SYRTHES, code_saturne, and neptune_cfd.
  *
  * It is defined by a mask, so for example flags f1, f2, and f3 may be
  * combined using the "f1 | f2 | f2" syntax.
@@ -172,7 +174,7 @@ cs_coupling_get_sync_flag(void);
  *   PLE_COUPLING_UNSTEADY      Inform others that this instance is
  *                              using an unsteady solution approach
  *   PLE_COUPLING_STEADY        Inform others that this instance is
- *                              using a teady solution approach
+ *                              using a steady solution approach
  *   PLE_COUPLING_USER_1        User definable flag
  *   PLE_COUPLING_USER_2        User definable flag
  *   PLE_COUPLING_USER_3        User definable flag
