@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -41,8 +41,11 @@ BEGIN_C_DECLS
  * Additional doxygen documentation
  *============================================================================*/
 
+/* The following block applies to cs_defs.h directly, as it mainly
+   concerns itself with type definitions */
+
 /*!
-  \file cs_defs.c
+  \file cs_defs.h
         Base macro and typedef definitions for system portability.
 
   \typedef cs_gnum_t
@@ -71,15 +74,6 @@ BEGIN_C_DECLS
            not depend on mesh size, so as not to pollute the readability
            and comprehensibility of the code.
 
-  \typedef cs_int_t
-            \brief Fortran-compatible integer
-            \deprecated
-            Currently, this integer type is necessarily of the same
-            type as \ref cs_lnum_t, but it should only be used in Fortran
-            wrapper function definitions. Moving to ISO_C_BINDINGS,
-            and converting more code to C, this type should eventually
-            disappear.
-
   \typedef cs_lnum_2_t
            \brief vector of 2 local mesh-entity ids
 
@@ -130,12 +124,17 @@ BEGIN_C_DECLS
  * Global variables
  *============================================================================*/
 
+/* Empty string */
+
+const char   cs_empty_string[] = "";
+
 /* Sizes associated with datatypes */
 
 const size_t  cs_datatype_size[] = {0,
                                     1,
                                     sizeof(float),
                                     sizeof(double),
+                                    sizeof(unsigned short int),
                                     4,
                                     8,
                                     4,
@@ -145,6 +144,7 @@ const char   *cs_datatype_name[] = {"",
                                     "char",
                                     "float",
                                     "double",
+                                    "flag",
                                     "int32",
                                     "int64",
                                     "uint32",
@@ -158,6 +158,7 @@ MPI_Datatype  cs_datatype_to_mpi[] = {MPI_DATATYPE_NULL,
                                       MPI_CHAR,
                                       MPI_FLOAT,
                                       MPI_DOUBLE,
+                                      MPI_UNSIGNED_SHORT,
                                       MPI_INT,            /* CS_INT32 */
                                       MPI_LONG_INT,       /* CS_INT64 */
                                       MPI_UNSIGNED,       /* CS_UINT32 */
