@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -58,6 +58,11 @@ extern "C" {
 
 #else
 
+#  define _LIBINTL_H /* Prevent inclusion of <libintl.h> by other files
+                        with incorrect or missing checks;
+                        TODO locate files causing issues to avoid
+                        requiring this workaround */
+
 #  define _(String) (String)
 #  define N_(String) String
 #  define textdomain(String) (String)
@@ -288,43 +293,53 @@ typedef enum {
   typedef unsigned  cs_gnum_t;
 #endif
 
+/* Local integer index or number */
+
+#if defined(HAVE_LONG_LNUM)
+  typedef long  cs_lnum_t;
+#else
+  typedef int   cs_lnum_t;
+#endif
+
 /* Other types */
+typedef double            cs_coord_t;  /* Real number (coordinate value) */
 
-typedef int                 cs_lnum_t;    /* Local integer index or number */
-typedef double              cs_coord_t;   /* Real number (coordinate value) */
-
-typedef int                 cs_int_t;    /* Fortran integer */
 typedef double              cs_real_t;   /* Fortran double precision */
 typedef char                cs_byte_t;   /* Byte (untyped memory unit) */
-typedef unsigned short int  cs_flag_t;   /* Flag for storing metadata */
+typedef unsigned short int  cs_flag_t;   /* Flag storing metadata */
 
 /* Vector or array block types */
 
-typedef int        cs_lnum_2_t[2];      /* Vector of 2 local numbers */
-typedef int        cs_lnum_3_t[3];      /* Vector of 3 local numbers */
+typedef cs_lnum_t  cs_lnum_2_t[2];      /* Vector of 2 local numbers */
+typedef cs_lnum_t  cs_lnum_3_t[3];      /* Vector of 3 local numbers */
 
-typedef double     cs_coord_3_t[3];     /* Vector of 3 real (coordinate)
-                                           values */
+typedef cs_coord_t cs_coord_3_t[3];         /* Vector of 3 real (coordinate)
+                                               values */
 
-typedef cs_real_t  cs_real_2_t[2];      /* Vector of 2 real values */
-typedef cs_real_t  cs_real_3_t[3];      /* Vector of 3 real values */
-typedef cs_real_t  cs_real_4_t[4];      /* Vector of 4 real values */
-typedef cs_real_t  cs_real_6_t[6];      /* Vector of 6 real values
-                                           (for symmetric tensor) */
-typedef cs_real_t  cs_real_9_t[9];      /* Vector of 9 real values */
+typedef cs_real_t  cs_real_2_t[2];          /* Vector of 2 real values */
+typedef cs_real_t  cs_real_3_t[3];          /* Vector of 3 real values */
+typedef cs_real_t  cs_real_4_t[4];          /* Vector of 4 real values */
+typedef cs_real_t  cs_real_6_t[6];          /* Vector of 6 real values
+                                               (for symmetric tensor) */
+typedef cs_real_t  cs_real_9_t[9];          /* Vector of 9 real values */
+typedef cs_real_t  cs_real_10_t[10];        /* Vector of 10 real values */
 
-typedef cs_real_t  cs_real_33_t[3][3];  /* Matrix of 3x3 real values */
-typedef cs_real_t  cs_real_66_t[6][6];  /* Matrix of 6x6 real values */
-typedef cs_real_t  cs_real_99_t[9][9];  /* Matrix of 9x9 real values */
+typedef cs_real_t  cs_real_33_t[3][3];      /* Matrix of 3x3 real values */
+typedef cs_real_t  cs_real_66_t[6][6];      /* Matrix of 6x6 real values */
+typedef cs_real_t  cs_real_99_t[9][9];      /* Matrix of 9x9 real values */
 
-typedef cs_real_t  cs_real_34_t[3][4];  /* Matrix of 3x4 real values */
+typedef cs_real_t  cs_real_333_t[3][3][3];  /* tensor of 3x3x3 real values */
 
-typedef cs_real_t  cs_real_63_t[6][3];  /* Matrix of 6x3 real values */
+typedef cs_real_t  cs_real_34_t[3][4];      /* Matrix of 3x4 real values */
 
-typedef cs_real_33_t  cs_real_332_t[2];  /* vector of 2 3x3 matrices
-                                            of real values */
-typedef cs_real_66_t  cs_real_662_t[2];  /* vector of 2 6x6 matrices
-                                            of real values */
+typedef cs_real_t  cs_real_63_t[6][3];      /* Matrix of 6x3 real values */
+
+typedef cs_real_t  cs_real_69_t[6][9];      /* Matrix of 6x9 real values */
+
+typedef cs_real_33_t  cs_real_332_t[2];     /* vector of 2 3x3 matrices
+                                               of real values */
+typedef cs_real_66_t  cs_real_662_t[2];     /* vector of 2 6x6 matrices
+                                               of real values */
 
 typedef struct {
 
@@ -347,7 +362,6 @@ typedef struct {
 
 #if defined(HAVE_MPI) && !defined(CS_IGNORE_MPI)
 
-#  define CS_MPI_INT       MPI_INT         /* If cs_int_t is an int */
 #  define CS_MPI_REAL      MPI_DOUBLE      /* If cs_real_t is a double */
 
 /* MPI type for cs_gnum_t integer type (depends on configuration) */
@@ -369,8 +383,16 @@ typedef struct {
 #    define CS_MPI_GNUM       MPI_UNSIGNED
 #  endif
 
+/* MPI type for cs_lnum_t type */
+
+#  if defined(HAVE_LONG_LNUM)
+#    define CS_MPI_LNUM     MPI_LONG
+#  else
+#    define CS_MPI_LNUM     MPI_INT
+#  endif
+
+#  define CS_MPI_EFLAG    MPI_UNSIGNED       /* MPI type for cs_mflag_t type */
 #  define CS_MPI_FLAG     MPI_UNSIGNED_SHORT /* MPI type for cs_flag_t type */
-#  define CS_MPI_LNUM     MPI_INT            /* MPI type for cs_lnum_t type */
 #  define CS_MPI_COORD    MPI_DOUBLE         /* MPI type for cs_coord_t type */
 
 #endif /* defined(HAVE_MPI) && !defined(CS_IGNORE_MPI) */
@@ -386,10 +408,18 @@ typedef struct {
 # define CS_GNUM_TYPE     CS_UINT32
 #endif
 
-#if (SIZEOF_INT == 8)
-# define CS_LNUM_TYPE     CS_INT64
+#if defined(HAVE_LONG_LNUM)
+# if (SIZEOF_LONG == 8)
+#  define CS_LNUM_TYPE     CS_INT64
+# else
+#  define CS_LNUM_TYPE     CS_INT32
+# endif
 #else
-# define CS_LNUM_TYPE     CS_INT32
+# if (SIZEOF_INT == 8)
+#  define CS_LNUM_TYPE     CS_INT64
+# else
+#  define CS_LNUM_TYPE     CS_INT32
+# endif
 #endif
 
 #if (SIZEOF_INT == 8)
@@ -398,19 +428,22 @@ typedef struct {
 # define CS_INT_TYPE      CS_INT32
 #endif
 
+#if (SIZEOF_INT == 8)
+# define CS_UINT_TYPE     CS_UINT64
+#else
+# define CS_UINT_TYPE     CS_UINT32
+#endif
+
 #define CS_FLAG_TYPE      CS_UINT16
+#define CS_EFLAG_TYPE     CS_UINT_TYPE
 #define CS_REAL_TYPE      CS_DOUBLE
 #define CS_COORD_TYPE     CS_DOUBLE
 
 /* Minimum size for OpenMP loops
- *  (based on initial benchmarking, which will need updates)
- *-----------------------------------------------------------*/
+ *  (will need benchmarking and tuning for various systems)
+ *---------------------------------------------------------*/
 
-#if defined(__bgq__) && defined(__xlc__)
-#  define CS_THR_MIN 1014
-#else
-#  define CS_THR_MIN 128
-#endif
+#define CS_THR_MIN 128
 
 /* Cache line size, or multiple thereof */
 /*--------------------------------------*/
@@ -421,9 +454,9 @@ typedef struct {
  * Type independent min an max (caution: the argument is evaluated)
  *----------------------------------------------------------------------------*/
 
-#define CS_ABS(a)     ((a) <  0  ? -(a) : (a))  /* Absolute value of a */
-#define CS_MIN(a,b)   ((a) < (b) ?  (a) : (b))  /* Minimum of a et b */
-#define CS_MAX(a,b)   ((a) > (b) ?  (a) : (b))  /* Maximum of a et b */
+#define CS_ABS(a)     ((a) <  0  ? -(a) : (a))  /*!< Absolute value of a */
+#define CS_MIN(a,b)   ((a) < (b) ?  (a) : (b))  /*!< Minimum of a et b */
+#define CS_MAX(a,b)   ((a) > (b) ?  (a) : (b))  /*!< Maximum of a et b */
 
 /*----------------------------------------------------------------------------
  * Variable interlace type:
@@ -495,6 +528,10 @@ typedef enum {
  * Global variables
  *============================================================================*/
 
+/* Empty but non-NULL string */
+
+extern const char     cs_empty_string[];
+
 /* Sizes and names associated with datatypes */
 
 extern const size_t   cs_datatype_size[];
