@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,7 +45,12 @@
 #include <EOS/API/EOS_Field.hxx>
 #include <EOS/API/EOS_Fields.hxx>
 #include <EOS/API/EOS_Error_Field.hxx>
+
+#if defined(EOS_PRE_V1_6)
 #include <EOS/API/EOS_enums.hxx>
+#else
+#include <EOS/API/EOS_properties.hxx>
+#endif
 
 /*----------------------------------------------------------------------------
  *  Local headers
@@ -74,12 +79,10 @@ NEPTUNE::EOS *eos;
 /*============================================================================
  * Private function definitions
  *============================================================================*/
-#ifdef __cplusplus
-extern "C"
-#endif
-void
-_eos_error_code(const char *function_name,
-                NEPTUNE::EOS_Error return_code)
+
+static void
+_eos_error_code(const char          *function_name,
+                NEPTUNE::EOS_Error   return_code)
 {
     // Generic error code returned by EOS.
     // For a field, the generic error is the worst
@@ -132,10 +135,10 @@ _eos_error_code(const char *function_name,
 extern "C"
 #endif
 void
-cs_eos_create(char *EOSMethod,
-              char *EOSRef)
+cs_eos_create(const char *EOSMethod,
+              char       *EOSRef)
 {
-    eos = new NEPTUNE::EOS(EOSMethod, EOSRef);
+  eos = new NEPTUNE::EOS(EOSMethod, EOSRef);
 }
 
 /*----------------------------------------------------------------------------
@@ -148,7 +151,7 @@ extern "C"
 void
 cs_eos_destroy(void)
 {
-    delete eos;
+  delete eos;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -167,6 +170,7 @@ cs_eos_destroy(void)
 #ifdef __cplusplus
 extern "C"
 #endif
+
 void
 cs_phys_prop_eos(cs_phys_prop_thermo_plane_type_t   thermo_plane,
                  cs_phys_prop_type_t                property,
@@ -192,7 +196,8 @@ cs_phys_prop_eos(cs_phys_prop_thermo_plane_type_t   thermo_plane,
   }
   else {
     bft_error(__FILE__, __LINE__, 0,
-              _("bad choice: you chose to work in the %i plane with EOS."), thermo_plane);
+              _("bad choice: you chose to work in the %i plane with EOS."),
+              thermo_plane);
   }
 
   NEPTUNE::EOS_Fields output(1);
@@ -200,8 +205,9 @@ cs_phys_prop_eos(cs_phys_prop_thermo_plane_type_t   thermo_plane,
 
   switch (property) {
     case CS_PHYS_PROP_PRESSURE:
-      bft_error(__FILE__, __LINE__, 0,
-                _("bad choice: you can't choose pressure as an output variable"));
+      bft_error
+        (__FILE__, __LINE__, 0,
+         _("bad choice: you can't choose pressure as an output variable"));
       break;
     case CS_PHYS_PROP_TEMPERATURE:
       /* temperature is in K */
@@ -259,7 +265,6 @@ cs_phys_prop_eos(cs_phys_prop_thermo_plane_type_t   thermo_plane,
 
   _eos_error_code("cs_phys_prop_eos", eos_error);
   BFT_FREE(error);
-
 }
 
 /*----------------------------------------------------------------------------*/
