@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -91,11 +91,12 @@ BEGIN_C_DECLS
  * \param[in]     idtvar        indicator of the temporal scheme
  * \param[in]     iterns        external sub-iteration number
  * \param[in]     f_id          field id (or -1)
- * \param[in]     ndircp        indicator (0 if the diagonal is stepped aside)
  * \param[in]     iescap        compute the predictor indicator if 1
  * \param[in]     imucpp        indicator
  *                               - 0 do not multiply the convectiv term by Cp
  *                               - 1 do multiply the convectiv term by Cp
+ * \param[in]     normp         Reference norm to solve the system (optional)
+ *                              if negative: recomputed here
  * \param[in]     var_cal_opt   pointer to a cs_var_cal_opt_t structure which
  *                              contains variable calculation options
  * \param[in]     pvara         variable at the previous time step
@@ -149,9 +150,9 @@ cs_equation_iterative_solve_scalar(int                   idtvar,
                                    int                   iterns,
                                    int                   f_id,
                                    const char           *name,
-                                   int                   ndircp,
                                    int                   iescap,
                                    int                   imucpp,
+                                   cs_real_t             normp,
                                    cs_var_cal_opt_t     *var_cal_opt,
                                    const cs_real_t       pvara[],
                                    const cs_real_t       pvark[],
@@ -226,7 +227,6 @@ cs_equation_iterative_solve_scalar(int                   idtvar,
  * \param[in]     iterns        external sub-iteration number
  * \param[in]     f_id          field id (or -1)
  * \param[in]     name          associated name if f_id < 0, or NULL
- * \param[in]     ndircp        indicator (0 if the diagonal is stepped aside)
  * \param[in]     ivisep        indicator to take \f$ \divv
  *                               \left(\mu \gradt \transpose{\vect{a}} \right)
  *                               -2/3 \grad\left( \mu \dive \vect{a} \right)\f$
@@ -286,7 +286,6 @@ cs_equation_iterative_solve_vector(int                   idtvar,
                                    int                   iterns,
                                    int                   f_id,
                                    const char           *name,
-                                   int                   ndircp,
                                    int                   ivisep,
                                    int                   iescap,
                                    cs_var_cal_opt_t     *var_cal_opt,
@@ -309,7 +308,7 @@ cs_equation_iterative_solve_vector(int                   idtvar,
                                    const cs_real_t       weighb[],
                                    int                   icvflb,
                                    const int             icvfli[],
-                                   const cs_real_33_t    fimp[],
+                                   cs_real_33_t          fimp[],
                                    cs_real_3_t           smbrp[],
                                    cs_real_3_t           pvar[],
                                    cs_real_3_t           eswork[]);
@@ -361,7 +360,6 @@ cs_equation_iterative_solve_vector(int                   idtvar,
  *
  * \param[in]     idtvar        indicator of the temporal scheme
  * \param[in]     f_id          field id (or -1)
- * \param[in]     ndircp        indicator (0 if the diagonal is stepped aside)
  * \param[in]     var_cal_opt   pointer to a cs_var_cal_opt_t structure which
  *                              contains variable calculation options
  * \param[in]     pvara         variable at the previous time step
@@ -410,7 +408,6 @@ void
 cs_equation_iterative_solve_tensor(int                   idtvar,
                                    int                   f_id,
                                    const char           *name,
-                                   int                   ndircp,
                                    cs_var_cal_opt_t     *var_cal_opt,
                                    const cs_real_6_t     pvara[],
                                    const cs_real_6_t     pvark[],
