@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,7 @@
  *----------------------------------------------------------------------------*/
 
 #include <assert.h>
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -46,6 +47,7 @@
 #include "bft_printf.h"
 
 #include "cs_field.h"
+#include "cs_field_default.h"
 #include "cs_gradient.h"
 #include "cs_gradient_perio.h"
 #include "cs_halo.h"
@@ -56,6 +58,7 @@
 #include "cs_parameters.h"
 #include "cs_parall.h"
 #include "cs_mesh.h"
+#include "cs_mesh_adjacencies.h"
 #include "cs_mesh_location.h"
 #include "cs_mesh_quantities.h"
 #include "cs_internal_coupling.h"
@@ -110,7 +113,7 @@ BEGIN_C_DECLS
 void
 cs_f_field_gradient_scalar(int                    f_id,
                            int                    use_previous_t,
-                           int                    imrgra,
+                           int                    imrgr0,
                            int                    inc,
                            int                    recompute_cocg,
                            cs_real_3_t  *restrict grad);
@@ -118,7 +121,7 @@ cs_f_field_gradient_scalar(int                    f_id,
 void
 cs_f_field_gradient_potential(int                    f_id,
                               int                    use_previous_t,
-                              int                    imrgra,
+                              int                    imrgr0,
                               int                    inc,
                               int                    recompute_cocg,
                               int                    hyd_p_flag,
@@ -128,15 +131,20 @@ cs_f_field_gradient_potential(int                    f_id,
 void
 cs_f_field_gradient_vector(int                     f_id,
                            int                     use_previous_t,
-                           int                     imrgra,
+                           int                     imrgr0,
                            int                     inc,
                            cs_real_33_t  *restrict grad);
 
-void cs_f_field_gradient_tensor(int                     f_id,
-                                int                     use_previous_t,
-                                int                     imrgra,
-                                int                     inc,
-                                cs_real_63_t  *restrict grad);
+void
+cs_f_field_gradient_tensor(int                     f_id,
+                           int                     use_previous_t,
+                           int                     imrgr0,
+                           int                     inc,
+                           cs_real_63_t  *restrict grad);
+
+void
+cs_f_field_set_volume_average(int       f_id,
+                              cs_real_t va);
 
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
@@ -206,7 +214,7 @@ _field_interpolate_by_gradient(const cs_field_t   *f,
                 " not implemented for fields on location %s."),
               f->name, cs_mesh_location_type_name[f->location_id]);
 
-  /* Get the calculation option from the field */
+  /* Compute field cell gradient */
 
   cs_real_t *grad;
   BFT_MALLOC(grad, 3*dim*n_cells_ext, cs_real_t);
@@ -227,7 +235,7 @@ _field_interpolate_by_gradient(const cs_field_t   *f,
     bft_error(__FILE__, __LINE__, 0,
               _("Field gradient interpolation for field %s of dimension %d:\n"
                 " not implemented."),
-              f->name, f->dim);
+              f->name, (int)dim);
 
   /* Now interpolated values */
 
@@ -271,67 +279,76 @@ _local_extrema_scalar(const cs_real_t *restrict pvar,
 {
   const cs_mesh_t *m = cs_glob_mesh;
   const cs_lnum_t n_cells = m->n_cells;
-  const cs_lnum_t n_cells_ext = m->n_cells_with_ghosts;
-  const int n_i_groups = m->i_face_numbering->n_groups;
-  const int n_i_threads = m->i_face_numbering->n_threads;
-  const cs_lnum_t *restrict i_group_index = m->i_face_numbering->group_index;
+  const cs_lnum_t n_vertices = m->n_vertices;
 
-  const cs_lnum_t *restrict cell_cells_idx
-    = (const cs_lnum_t *restrict) m->cell_cells_idx;
-  const cs_lnum_t *restrict cell_cells_lst
-    = (const cs_lnum_t *restrict) m->cell_cells_lst;
-  const cs_lnum_2_t *restrict i_face_cells
-    = (const cs_lnum_2_t *restrict)m->i_face_cells;
+  const cs_adjacency_t  *c2v = cs_mesh_adjacencies_cell_vertices();
+  const cs_lnum_t *c2v_idx = c2v->idx;
+  const cs_lnum_t *c2v_ids = c2v->ids;
 
-# pragma omp parallel for
-  for (cs_lnum_t ii = 0; ii < n_cells_ext; ii++) {
+# pragma omp parallel for if (n_cells > CS_THR_MIN)
+  for (cs_lnum_t ii = 0; ii < n_cells; ii++) {
     local_max[ii] = pvar[ii];
     local_min[ii] = pvar[ii];
   }
 
-  /* Contribution from interior faces */
+  cs_real_t *v_min, *v_max;
+  BFT_MALLOC(v_min, n_vertices, cs_real_t);
+  BFT_MALLOC(v_max, n_vertices, cs_real_t);
 
-  for (int g_id = 0; g_id < n_i_groups; g_id++) {
-#   pragma omp parallel for
-    for (int t_id = 0; t_id < n_i_threads; t_id++) {
-      for (cs_lnum_t face_id = i_group_index[(t_id*n_i_groups + g_id)*2];
-          face_id < i_group_index[(t_id*n_i_groups + g_id)*2 + 1];
-          face_id++) {
+# pragma omp parallel for if (n_vertices > CS_THR_MIN)
+  for (cs_lnum_t ii = 0; ii < n_vertices; ii++) {
+    v_max[ii] = -HUGE_VAL;
+    v_min[ii] = HUGE_VAL;
+  }
 
-        cs_lnum_t ii = i_face_cells[face_id][0];
-        cs_lnum_t jj = i_face_cells[face_id][1];
+  /* Scatter min/max values to vertices */
 
-        cs_real_t pi = pvar[ii];
-        cs_real_t pj = pvar[jj];
-
-        local_max[ii] = CS_MAX(local_max[ii], pj);
-        local_max[jj] = CS_MAX(local_max[jj], pi);
-        local_min[ii] = CS_MIN(local_min[ii], pj);
-        local_min[jj] = CS_MIN(local_min[jj], pi);
-      }
+  for (cs_lnum_t c_id = 0; c_id < n_cells; c_id++) {
+    cs_lnum_t s_id = c2v_idx[c_id];
+    cs_lnum_t e_id = c2v_idx[c_id+1];
+    cs_real_t _c_var = pvar[c_id];
+    for (cs_lnum_t j = s_id; j < e_id; j++) {
+      cs_lnum_t v_id = c2v_ids[j];
+      if (_c_var < v_min[v_id])
+        v_min[v_id] = _c_var;
+      if (_c_var > v_max[v_id])
+        v_max[v_id] = _c_var;
     }
   }
 
-  /* Contribution from extended neighborhood */
+  if (m->vtx_interfaces != NULL) {
+    cs_interface_set_min(m->vtx_interfaces,
+                         m->n_vertices,
+                         1,
+                         true,
+                         CS_REAL_TYPE,
+                         v_min);
+    cs_interface_set_max(m->vtx_interfaces,
+                         m->n_vertices,
+                         1,
+                         true,
+                         CS_REAL_TYPE,
+                         v_max);
+  }
 
-  if (halo_type == CS_HALO_EXTENDED) {
-#   pragma omp parallel for
-    for (cs_lnum_t ii = 0; ii < n_cells; ii++) {
-      for (cs_lnum_t cidx = cell_cells_idx[ii];
-           cidx < cell_cells_idx[ii + 1];
-           cidx++) {
-        cs_lnum_t jj = cell_cells_lst[cidx];
+  /* Gather min/max values from vertices */
 
-        cs_real_t pi = pvar[ii];
-        cs_real_t pj = pvar[jj];
-
-        local_max[ii] = CS_MAX(local_max[ii], pj);
-        local_max[jj] = CS_MAX(local_max[jj], pi);
-        local_min[ii] = CS_MIN(local_min[ii], pj);
-        local_min[jj] = CS_MIN(local_min[jj], pi);
-      }
+# pragma omp parallel for if (n_cells > CS_THR_MIN)
+  for (cs_lnum_t c_id = 0; c_id < n_cells; c_id++) {
+    cs_lnum_t s_id = c2v_idx[c_id];
+    cs_lnum_t e_id = c2v_idx[c_id+1];
+    for (cs_lnum_t j = s_id; j < e_id; j++) {
+      cs_lnum_t v_id = c2v_ids[j];
+      if (v_min[v_id] < local_min[c_id])
+        local_min[c_id] = v_min[v_id];
+      if (v_max[v_id] > local_max[c_id])
+        local_max[c_id] = v_max[v_id];
     }
-  } /* End for extended neighborhood */
+  }
+
+  /* Free memory */
+  BFT_FREE(v_min);
+  BFT_FREE(v_max);
 
   /* Synchronisation */
   if (m->halo != NULL) {
@@ -353,7 +370,7 @@ _local_extrema_scalar(const cs_real_t *restrict pvar,
  * parameters:
  *   f_id           <-- field id
  *   use_previous_t <-- should we use values from the previous time step ?
- *   imrgra         <-- gradient reconstruction mode
+ *   imrgr0         <-- ignored
  *   inc            <-- if 0, solve on increment; 1 otherwise
  *   recompute_cocg <-- should COCG FV quantities be recomputed ?
  *   grad           --> gradient
@@ -362,17 +379,19 @@ _local_extrema_scalar(const cs_real_t *restrict pvar,
 void
 cs_f_field_gradient_scalar(int                    f_id,
                            int                    use_previous_t,
-                           int                    imrgra,
+                           int                    imrgr0,
                            int                    inc,
                            int                    recompute_cocg,
                            cs_real_3_t  *restrict grad)
 {
+  CS_UNUSED(imrgr0);
+
   bool _use_previous_t = use_previous_t ? true : false;
   bool _recompute_cocg = recompute_cocg ? true : false;
 
   const cs_field_t *f = cs_field_by_id(f_id);
 
-    cs_field_gradient_scalar(f,
+  cs_field_gradient_scalar(f,
                            _use_previous_t,
                            inc,
                            _recompute_cocg,
@@ -386,7 +405,7 @@ cs_f_field_gradient_scalar(int                    f_id,
  * parameters:
  *   f_id           <-- field id
  *   use_previous_t <-- should we use values from the previous time step ?
- *   imrgra         <-- gradient reconstruction mode
+ *   imrgr0         <-- ignored
  *   halo_type      <-- halo type
  *   inc            <-- if 0, solve on increment; 1 otherwise
  *   recompute_cocg <-- should COCG FV quantities be recomputed ?
@@ -398,19 +417,18 @@ cs_f_field_gradient_scalar(int                    f_id,
 void
 cs_f_field_gradient_potential(int                    f_id,
                               int                    use_previous_t,
-                              int                    imrgra,
+                              int                    imrgr0,
                               int                    inc,
                               int                    recompute_cocg,
                               int                    hyd_p_flag,
                               cs_real_3_t            f_ext[],
                               cs_real_3_t  *restrict grad)
 {
+  CS_UNUSED(imrgr0);
+
   bool _use_previous_t = use_previous_t ? true : false;
   bool _recompute_cocg = recompute_cocg ? true : false;
 
-  if (imrgra < 0)
-    imrgra = 0;
-
   const cs_field_t *f = cs_field_by_id(f_id);
 
   cs_field_gradient_potential(f,
@@ -429,7 +447,7 @@ cs_f_field_gradient_potential(int                    f_id,
  * parameters:
  *   f_id           <-- field id
  *   use_previous_t <-- should we use values from the previous time step ?
- *   imrgra         <-- gradient reconstruction mode
+ *   imrgr0         <-- ignored
  *   inc            <-- if 0, solve on increment; 1 otherwise
  *   recompute_cocg <-- should COCG FV quantities be recomputed ?
  *   grad           --> gradient
@@ -438,10 +456,12 @@ cs_f_field_gradient_potential(int                    f_id,
 void
 cs_f_field_gradient_vector(int                     f_id,
                            int                     use_previous_t,
-                           int                     imrgra,
+                           int                     imrgr0,
                            int                     inc,
                            cs_real_33_t  *restrict grad)
 {
+  CS_UNUSED(imrgr0);
+
   bool _use_previous_t = use_previous_t ? true : false;
 
   const cs_field_t *f = cs_field_by_id(f_id);
@@ -459,28 +479,49 @@ cs_f_field_gradient_vector(int                     f_id,
  * parameters:
  *   f_id           <-- field id
  *   use_previous_t <-- should we use values from the previous time step ?
- *   imrgra         <-- gradient reconstruction mode
+ *   imrgr0         <-- ignored
  *   inc            <-- if 0, solve on increment; 1 otherwise
  *   recompute_cocg <-- should COCG FV quantities be recomputed ?
  *   grad           --> gradient
  *----------------------------------------------------------------------------*/
 
-void cs_f_field_gradient_tensor(int                     f_id,
-                                int                     use_previous_t,
-                                int                     imrgra,
-                                int                     inc,
-                                cs_real_63_t  *restrict grad)
+void
+cs_f_field_gradient_tensor(int                     f_id,
+                           int                     use_previous_t,
+                           int                     imrgr0,
+                           int                     inc,
+                           cs_real_63_t  *restrict grad)
 {
+  CS_UNUSED(imrgr0);
+
   bool _use_previous_t = use_previous_t ? true : false;
 
   const cs_field_t *f = cs_field_by_id(f_id);
 
-
   cs_field_gradient_tensor(f,
                            _use_previous_t,
                            inc,
                            grad);
 }
+
+/*----------------------------------------------------------------------------
+ * Shift field values in order to set its spatial average to a given value.
+ *
+ * parameters:
+ *   f_id           <-- field id
+ *   va             <-- real value of volume average to be set
+ *----------------------------------------------------------------------------*/
+
+void
+cs_f_field_set_volume_average(int       f_id,
+                              cs_real_t va)
+{
+  cs_field_t *f = cs_field_by_id(f_id);
+
+  cs_field_set_volume_average(f,
+                              va);
+}
+
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*=============================================================================
@@ -509,16 +550,31 @@ cs_field_gradient_scalar(const cs_field_t          *f,
                          cs_real_3_t      *restrict grad)
 {
   cs_halo_type_t halo_type = CS_HALO_STANDARD;
-  cs_gradient_type_t gradient_type = CS_GRADIENT_ITER;
+  cs_gradient_type_t gradient_type = CS_GRADIENT_GREEN_ITER;
 
-  static int key_cal_opt_id = -1;
-  if (key_cal_opt_id < 0)
-    key_cal_opt_id = cs_field_key_id("var_cal_opt");
+  /* Does the field have a parent (variable) ?
+     Field is its own parent if not parent is specified */
+
+  const cs_field_t *parent_f = f;
+
+  const int f_parent_id
+    = cs_field_get_key_int(f, cs_field_key_id("parent_field_id"));
+  if (f_parent_id > -1)
+    parent_f = cs_field_by_id(f_parent_id);
+
+  int imrgra = cs_glob_space_disc->imrgra;
+  cs_var_cal_opt_t eqp_default = cs_parameters_var_cal_opt_default();
 
   /* Get the calculation option from the field */
-  cs_var_cal_opt_t var_cal_opt;
-  cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-  cs_gradient_type_by_imrgra(var_cal_opt.imrgra,
+  const cs_equation_param_t
+    *eqp = cs_field_get_equation_param_const(parent_f);
+
+  if (eqp != NULL)
+    imrgra = eqp->imrgra;
+  else
+    eqp = &eqp_default;
+
+  cs_gradient_type_by_imrgra(imrgra,
                              &gradient_type,
                              &halo_type);
 
@@ -527,52 +583,61 @@ cs_field_gradient_scalar(const cs_field_t          *f,
   cs_real_t *c_weight = NULL;
   cs_internal_coupling_t  *cpl = NULL;
 
-  if (f->type & CS_FIELD_VARIABLE && var_cal_opt.iwgrec == 1) {
-    if (var_cal_opt.idiff > 0) {
+  if (parent_f->type & CS_FIELD_VARIABLE && eqp->idiff > 0) {
+
+    if (eqp->iwgrec == 1) {
       /* Weighted gradient coefficients */
       int key_id = cs_field_key_id("gradient_weighting_id");
-      int diff_id = cs_field_get_key_int(f, key_id);
+      int diff_id = cs_field_get_key_int(parent_f, key_id);
       if (diff_id > -1) {
         cs_field_t *f_weight = cs_field_by_id(diff_id);
         c_weight = f_weight->val;
         w_stride = f_weight->dim;
       }
     }
+
+    /* Internal coupling structure */
+    int key_id = cs_field_key_id_try("coupling_entity");
+    if (key_id > -1) {
+      int coupl_id = cs_field_get_key_int(parent_f, key_id);
+      if (coupl_id > -1)
+        cpl = cs_internal_coupling_by_id(coupl_id);
+    }
+
   }
 
-  if (f->type & CS_FIELD_VARIABLE) {
-    if (var_cal_opt.idiff > 0) {
-      /* Internal coupling structure */
-      int key_id = cs_field_key_id_try("coupling_entity");
-      if (key_id > -1) {
-        int coupl_id = cs_field_get_key_int(f, key_id);
-        if (coupl_id > -1)
-          cpl = cs_internal_coupling_by_id(coupl_id);
-      }
-    }
-  }
+  if (f->n_time_vals < 2 && use_previous_t)
+    bft_error(__FILE__, __LINE__, 0,
+              _("%s: field %s does not maintain previous time step values\n"
+                "so \"use_previous_t\" can not be handled."),
+              __func__, f->name);
 
   cs_real_t *var = (use_previous_t) ? f->val_pre : f->val;
 
   cs_gradient_perio_init_rij(f, &tr_dim, grad);
 
+  const cs_real_t *bc_coeff_a = NULL, *bc_coeff_b = NULL;
+  if (f->bc_coeffs != NULL) {
+    bc_coeff_a = f->bc_coeffs->a;
+    bc_coeff_b = f->bc_coeffs->b;
+  }
+
   cs_gradient_scalar(f->name,
                      gradient_type,
                      halo_type,
                      inc,
                      recompute_cocg,
-                     var_cal_opt.nswrgr,
+                     eqp->nswrgr,
                      tr_dim,
                      0, /* hyd_p_flag */
                      w_stride,
-                     var_cal_opt.iwarni,
-                     var_cal_opt.imligr,
-                     var_cal_opt.epsrgr,
-                     var_cal_opt.extrag,
-                     var_cal_opt.climgr,
+                     eqp->verbosity,
+                     eqp->imligr,
+                     eqp->epsrgr,
+                     eqp->climgr,
                      NULL, /* f_ext */
-                     f->bc_coeffs->a,
-                     f->bc_coeffs->b,
+                     bc_coeff_a,
+                     bc_coeff_b,
                      var,
                      c_weight,
                      cpl, /* internal coupling */
@@ -606,67 +671,90 @@ cs_field_gradient_potential(const cs_field_t          *f,
                             cs_real_3_t      *restrict grad)
 {
   cs_halo_type_t halo_type = CS_HALO_STANDARD;
-  cs_gradient_type_t gradient_type = CS_GRADIENT_ITER;
+  cs_gradient_type_t gradient_type = CS_GRADIENT_GREEN_ITER;
 
-  static int key_cal_opt_id = -1;
-  if (key_cal_opt_id < 0)
-    key_cal_opt_id = cs_field_key_id("var_cal_opt");
+  /* Does the field have a parent (variable) ?
+     Field is its own parent if not parent is specified */
+
+  const cs_field_t *parent_f = f;
+
+  const int f_parent_id
+    = cs_field_get_key_int(f, cs_field_key_id("parent_field_id"));
+  if (f_parent_id > -1)
+    parent_f = cs_field_by_id(f_parent_id);
+
+  int imrgra = cs_glob_space_disc->imrgra;
+  cs_var_cal_opt_t eqp_default = cs_parameters_var_cal_opt_default();
 
   /* Get the calculation option from the field */
-  cs_var_cal_opt_t var_cal_opt;
-  cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-  cs_gradient_type_by_imrgra(var_cal_opt.imrgra,
+  const cs_equation_param_t
+    *eqp = cs_field_get_equation_param_const(parent_f);
+
+  if (eqp != NULL)
+    imrgra = eqp->imrgra;
+  else
+    eqp = &eqp_default;
+
+  cs_gradient_type_by_imrgra(imrgra,
                              &gradient_type,
                              &halo_type);
 
+  if (f->n_time_vals < 2 && use_previous_t)
+    bft_error(__FILE__, __LINE__, 0,
+              _("%s: field %s does not maintain previous time step values\n"
+                "so \"use_previous_t\" can not be handled."),
+              __func__, f->name);
+
   int w_stride = 1;
   cs_real_t *var = (use_previous_t) ? f->val_pre : f->val;
 
   cs_real_t *c_weight = NULL;
   cs_internal_coupling_t  *cpl = NULL;
 
-  if (f->type & CS_FIELD_VARIABLE && var_cal_opt.iwgrec == 1) {
-    if (var_cal_opt.idiff > 0) {
+  if (parent_f->type & CS_FIELD_VARIABLE && eqp->idiff > 0) {
+
+    if (eqp->iwgrec == 1) {
       int key_id = cs_field_key_id("gradient_weighting_id");
-      int diff_id = cs_field_get_key_int(f, key_id);
+      int diff_id = cs_field_get_key_int(parent_f, key_id);
       if (diff_id > -1) {
         cs_field_t *f_weight = cs_field_by_id(diff_id);
         c_weight = f_weight->val;
         w_stride = f_weight->dim;
       }
     }
-  }
 
-  if (f->type & CS_FIELD_VARIABLE) {
-    if (var_cal_opt.idiff > 0) {
-      /* Internal coupling structure */
-      int key_id = cs_field_key_id_try("coupling_entity");
-      if (key_id > -1) {
-        int coupl_id = cs_field_get_key_int(f, key_id);
-        if (coupl_id > -1)
-          cpl = cs_internal_coupling_by_id(coupl_id);
-      }
+    /* Internal coupling structure */
+    int key_id = cs_field_key_id_try("coupling_entity");
+    if (key_id > -1) {
+      int coupl_id = cs_field_get_key_int(parent_f, key_id);
+      if (coupl_id > -1)
+        cpl = cs_internal_coupling_by_id(coupl_id);
     }
+
   }
 
+  const cs_real_t *bc_coeff_a = NULL, *bc_coeff_b = NULL;
+  if (f->bc_coeffs != NULL) {
+    bc_coeff_a = f->bc_coeffs->a;
+    bc_coeff_b = f->bc_coeffs->b;
+  }
 
   cs_gradient_scalar(f->name,
                      gradient_type,
                      halo_type,
                      inc,
                      recompute_cocg,
-                     var_cal_opt.nswrgr,
+                     eqp->nswrgr,
                      0, /* tr_dim */
                      hyd_p_flag,
                      w_stride,
-                     var_cal_opt.iwarni,
-                     var_cal_opt.imligr,
-                     var_cal_opt.epsrgr,
-                     var_cal_opt.extrag,
-                     var_cal_opt.climgr,
+                     eqp->verbosity,
+                     eqp->imligr,
+                     eqp->epsrgr,
+                     eqp->climgr,
                      f_ext,
-                     f->bc_coeffs->a,
-                     f->bc_coeffs->b,
+                     bc_coeff_a,
+                     bc_coeff_b,
                      var,
                      c_weight,
                      cpl, /* internal coupling */
@@ -692,24 +780,29 @@ cs_field_gradient_vector(const cs_field_t          *f,
                          cs_real_33_t     *restrict grad)
 {
   cs_halo_type_t halo_type = CS_HALO_STANDARD;
-  cs_gradient_type_t gradient_type = CS_GRADIENT_ITER;
+  cs_gradient_type_t gradient_type = CS_GRADIENT_GREEN_ITER;
 
-  static int key_cal_opt_id = -1;
-  if (key_cal_opt_id < 0)
-    key_cal_opt_id = cs_field_key_id("var_cal_opt");
+  int imrgra = cs_glob_space_disc->imrgra;
+  cs_var_cal_opt_t eqp_default = cs_parameters_var_cal_opt_default();
 
   /* Get the calculation option from the field */
-  cs_var_cal_opt_t var_cal_opt;
-  cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-  cs_gradient_type_by_imrgra(var_cal_opt.imrgra,
+  const cs_equation_param_t *eqp = cs_field_get_equation_param_const(f);
+
+  if (eqp != NULL)
+    imrgra = eqp->imrgra;
+  else
+    eqp = &eqp_default;
+
+  cs_gradient_type_by_imrgra(imrgra,
                              &gradient_type,
                              &halo_type);
 
   cs_real_t *c_weight = NULL;
   cs_internal_coupling_t  *cpl = NULL;
 
-  if (f->type & CS_FIELD_VARIABLE && var_cal_opt.iwgrec == 1) {
-    if (var_cal_opt.idiff > 0) {
+  if (f->type & CS_FIELD_VARIABLE && eqp->idiff > 0) {
+
+    if (eqp->iwgrec == 1) {
       /* Weighted gradient coefficients */
       int key_id = cs_field_key_id("gradient_weighting_id");
       int diff_id = cs_field_get_key_int(f, key_id);
@@ -718,33 +811,43 @@ cs_field_gradient_vector(const cs_field_t          *f,
         c_weight = f_weight->val;
       }
     }
+
+    int key_id = cs_field_key_id_try("coupling_entity");
+    if (key_id > -1) {
+      int coupl_id = cs_field_get_key_int(f, key_id);
+      if (coupl_id > -1)
+        cpl = cs_internal_coupling_by_id(coupl_id);
+    }
+
   }
 
-  if (f->type & CS_FIELD_VARIABLE) {
-    if (var_cal_opt.idiff > 0) {
-      int key_id = cs_field_key_id_try("coupling_entity");
-      if (key_id > -1) {
-        int coupl_id = cs_field_get_key_int(f, key_id);
-        if (coupl_id > -1)
-          cpl = cs_internal_coupling_by_id(coupl_id);
-      }
-    }
-  }
+  if (f->n_time_vals < 2 && use_previous_t)
+    bft_error(__FILE__, __LINE__, 0,
+              _("%s: field %s does not maintain previous time step values\n"
+                "so \"use_previous_t\" can not be handled."),
+              __func__, f->name);
 
   cs_real_3_t *var = (use_previous_t) ? (cs_real_3_t *)(f->val_pre)
                                       : (cs_real_3_t *)(f->val);
 
+  const cs_real_3_t *bc_coeff_a = NULL;
+  const cs_real_33_t *bc_coeff_b = NULL;
+  if (f->bc_coeffs != NULL) {
+    bc_coeff_a = (const cs_real_3_t *)f->bc_coeffs->a;
+    bc_coeff_b = (const cs_real_33_t *)f->bc_coeffs->b;
+  }
+
   cs_gradient_vector(f->name,
                      gradient_type,
                      halo_type,
                      inc,
-                     var_cal_opt.nswrgr,
-                     var_cal_opt.iwarni,
-                     var_cal_opt.imligr,
-                     var_cal_opt.epsrgr,
-                     var_cal_opt.climgr,
-                     (const cs_real_3_t *)(f->bc_coeffs->a),
-                     (const cs_real_33_t *)(f->bc_coeffs->b),
+                     eqp->nswrgr,
+                     eqp->verbosity,
+                     eqp->imligr,
+                     eqp->epsrgr,
+                     eqp->climgr,
+                     bc_coeff_a,
+                     bc_coeff_b,
                      var,
                      c_weight,
                      cpl,
@@ -770,33 +873,50 @@ cs_field_gradient_tensor(const cs_field_t          *f,
                          cs_real_63_t     *restrict grad)
 {
   cs_halo_type_t halo_type = CS_HALO_STANDARD;
-  cs_gradient_type_t gradient_type = CS_GRADIENT_ITER;
+  cs_gradient_type_t gradient_type = CS_GRADIENT_GREEN_ITER;
 
-  static int key_cal_opt_id = -1;
-  if (key_cal_opt_id < 0)
-    key_cal_opt_id = cs_field_key_id("var_cal_opt");
+  int imrgra = cs_glob_space_disc->imrgra;
+  cs_var_cal_opt_t eqp_default = cs_parameters_var_cal_opt_default();
 
   /* Get the calculation option from the field */
-  cs_var_cal_opt_t var_cal_opt;
-  cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-  cs_gradient_type_by_imrgra(var_cal_opt.imrgra,
+  const cs_equation_param_t *eqp = cs_field_get_equation_param_const(f);
+
+  if (eqp != NULL)
+    imrgra = eqp->imrgra;
+  else
+    eqp = &eqp_default;
+
+  cs_gradient_type_by_imrgra(imrgra,
                              &gradient_type,
                              &halo_type);
 
+  if (f->n_time_vals < 2 && use_previous_t)
+    bft_error(__FILE__, __LINE__, 0,
+              _("%s: field %s does not maintain previous time step values\n"
+                "so \"use_previous_t\" can not be handled."),
+              __func__, f->name);
+
   cs_real_6_t *var = (use_previous_t) ? (cs_real_6_t *)(f->val_pre)
                                       : (cs_real_6_t *)(f->val);
 
+  const cs_real_6_t *bc_coeff_a = NULL;
+  const cs_real_66_t *bc_coeff_b = NULL;
+  if (f->bc_coeffs != NULL) {
+    bc_coeff_a = (const cs_real_6_t *)f->bc_coeffs->a;
+    bc_coeff_b = (const cs_real_66_t *)f->bc_coeffs->b;
+  }
+
   cs_gradient_tensor(f->name,
                      gradient_type,
                      halo_type,
                      inc,
-                     var_cal_opt.nswrgr,
-                     var_cal_opt.iwarni,
-                     var_cal_opt.imligr,
-                     var_cal_opt.epsrgr,
-                     var_cal_opt.climgr,
-                     (const cs_real_6_t *)(f->bc_coeffs->a),
-                     (const cs_real_66_t *)(f->bc_coeffs->b),
+                     eqp->nswrgr,
+                     eqp->verbosity,
+                     eqp->imligr,
+                     eqp->epsrgr,
+                     eqp->climgr,
+                     bc_coeff_a,
+                     bc_coeff_b,
                      var,
                      grad);
 }
@@ -853,7 +973,7 @@ cs_field_interpolate(cs_field_t              *f,
  *
  * This assumes the field values have been synchronized.
  *
- * \param[in]       f id        scalar field id
+ * \param[in]       f_id        scalar field id
  * \param[in]       halo_type   halo type
  * \param[in, out]  local_max   local maximum value
  * \param[in, out]  local_min   local minimum value
@@ -893,6 +1013,45 @@ cs_field_local_extrema_scalar(int              f_id,
   }
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Shift field values in order to set its spatial average (fluid volume
+ * average) to a given value.
+ *
+ * \param[in]   f   pointer to field
+ * \param[in]   va  real value of fluid volume average to be set
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_field_set_volume_average(cs_field_t     *f,
+                            const cs_real_t va)
+{
+  const cs_lnum_t n_cells = cs_glob_mesh->n_cells;
+
+  cs_mesh_quantities_t  *mq = cs_glob_mesh_quantities;
+  const cs_real_t *cell_f_vol = mq->cell_f_vol;
+  const cs_real_t tot_f_vol = mq->tot_f_vol;
+
+  cs_real_t *restrict val = f->val;
+  cs_real_t p_va = 0.;
+
+# pragma omp parallel for  reduction(+:p_va)
+  for (cs_lnum_t c_id = 0 ; c_id < n_cells ; c_id++) {
+    p_va += cell_f_vol[c_id]*val[c_id];
+  }
+
+  cs_parall_sum(1, CS_DOUBLE, &p_va);
+  p_va = p_va / tot_f_vol;
+
+  cs_real_t shift = va - p_va;
+
+# pragma omp parallel for
+  for (cs_lnum_t c_id = 0 ; c_id < n_cells ; c_id++) {
+    val[c_id] += shift;
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Synchronize current parallel and periodic field values.
