@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -165,7 +165,7 @@ cs_field_interpolate(cs_field_t              *f,
  *
  * This assumes the field values have been synchronized.
  *
- * \param[in]       f id        scalar field id
+ * \param[in]       f_id        scalar field id
  * \param[in]       halo_type   halo type
  * \param[in, out]  local_max   local maximum value
  * \param[in, out]  local_min   local minimum value
@@ -178,6 +178,20 @@ cs_field_local_extrema_scalar(int              f_id,
                               cs_real_t       *local_max,
                               cs_real_t       *local_min);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Shift field values in order to set its spatial average to a given
+ * value.
+ *
+ * \param[in]   f   pointer to field
+ * \param[in]   va  real value of volume average to be set
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_field_set_volume_average(cs_field_t     *f,
+                            const cs_real_t mean);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Synchronize current parallel and periodic field values.
