@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -416,7 +416,7 @@ _postprocess(const char   *error_mesh_name,
                           1,
                           false, /* no interlace */
                           true,  /* use parents */
-                          CS_POST_TYPE_cs_int_t,
+                          CS_POST_TYPE_int,
                           NULL,
                           NULL,
                           _flag,
