@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -32,13 +32,9 @@
 !------------------------------------------------------------------------------
 !   mode          name          role
 !------------------------------------------------------------------------------
-!> \param[in]     nvar          total number of variables
-!> \param[in]     nscal         total number of scalars
-!> \param[in]     dt            time step
 !______________________________________________________________________________
 
-subroutine cs_gas_mix_initialization &
- ( nvar   , nscal  , dt )
+subroutine cs_gas_mix_initialization
 
 !===============================================================================
 ! Module files
@@ -53,7 +49,6 @@ use pointe
 use entsor
 use parall
 use period
-use ihmpre
 use ppppar
 use ppthch
 use ppincl
@@ -67,10 +62,6 @@ implicit none
 
 ! Arguments
 
-integer          nvar   , nscal
-
-double precision dt(ncelet)
-
 ! Local variables
 
 integer          iel, iesp
@@ -128,16 +119,9 @@ call field_get_key_struct_gas_mix_species_prop(f_id, s_d)
 call field_get_val_s(igmxml, mix_mol_mas)
 
 !===============================================================================
-! 2. User initialization
-!===============================================================================
-
-call cs_user_f_initialization &
-( nvar   , nscal  ,                                            &
-  dt     )
-
-!===============================================================================
-! 3. Deduce the mass fraction (y_d) from the mass fractions (yk) of
+! 2. Deduce the mass fraction (y_d) from the mass fractions (yk) of
 !    the noncondensable gases transported
+!    Note: mass fraction has been provided by the user in the GUI or routines
 !===============================================================================
 
 if (isuite.eq.0) then
@@ -206,7 +190,7 @@ if (isuite.eq.0) then
   endif
 
   !===============================================================================
-  ! 4. Print to the listing to check the variables intialization
+  ! 3. Print to the log to check the variables intialization
   !===============================================================================
 
   write(nfecra, 200)
@@ -215,7 +199,7 @@ if (isuite.eq.0) then
 endif
 
 !===============================================================================
-! 5. Stop if problem
+! 4. Stop if problem
 !===============================================================================
 
 if (iok.gt.0) then
