@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -60,7 +60,6 @@ use pointe
 use albase
 use parall
 use period
-use ihmpre
 use ppppar
 use ppthch
 use ppincl
@@ -440,11 +439,11 @@ deallocate(s_k)
 '@',/,                                                            &
 '@      usipsu specifies that the specific heat is uniform',/,    &
 '@        icp = ',i10   ,' while',/,                              &
-'@      usphyv prescribes a variable specific heat.',/,           &
+'@      cs_user_physical_properties prescribes a variable specific heat.',/,&
 '@',/,                                                            &
 '@    The calculation will not be run.',/,                        &
 '@',/,                                                            &
-'@    Modify usipsu or usphyv.',/,                                &
+'@    Modify usipsu or cs_user_physical_properties.',/,           &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',/)
