@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -42,6 +42,7 @@
 #include "cs_base.h"
 #include "cs_matrix_assembler.h"
 #include "cs_mesh.h"
+#include "cs_mesh_quantities.h"
 #include "cs_parameters.h"
 
 /*----------------------------------------------------------------------------*/
@@ -61,6 +62,9 @@ BEGIN_C_DECLS
 
 typedef struct {
 
+  /* Id */
+  int id;
+
   /* Locator + tag for exchanging variables */
   ple_locator_t   *locator;
   int             *c_tag;
@@ -69,6 +73,14 @@ typedef struct {
   char  *cells_criteria;
   char  *faces_criteria;
 
+  char *interior_faces_group_name;
+  char *exterior_faces_group_name;
+
+  /* Associated zone ids */
+
+  int   n_volume_zones;
+  int  *volume_zone_ids;
+
   cs_lnum_t  n_local; /* Number of faces */
   cs_lnum_t *faces_local; /* Coupling boundary faces, numbered 0..n-1 */
 
@@ -87,13 +99,6 @@ typedef struct {
   /* OF vectors  */
   cs_real_3_t *offset_vect;
 
-  /* Gradient reconstruction */
-  cs_real_33_t *cocgb_s_lsq;
-  cs_real_33_t *cocg_it;
-
-  /* User information */
-  char *namesca;
-
 } cs_internal_coupling_t;
 
 /*============================================================================
@@ -115,7 +120,7 @@ cs_internal_coupling_n_couplings(void);
 /*!
  * \brief Define coupling volume using given selection criteria.
  *
- * Then, this volume must be seperated from the rest of the domain with a wall.
+ * Then, this volume must be separated from the rest of the domain with a wall.
  *
  * \param[in, out] mesh            pointer to mesh structure to modify
  * \param[in]      criteria_cells  criteria for the first group of cells
@@ -142,6 +147,57 @@ void
 cs_internal_coupling_add_volume(cs_mesh_t  *mesh,
                                 const char criteria_cells[]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define coupling volume using a cs_zone_t. Then, this volume will
+ * be separated from the rest of the domain with thin walls.
+ *
+ * \param[in, out] mesh  pointer to mesh structure to modify
+ * \param[in]      z     pointer to cs_volume_zone_t
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_internal_coupling_add_volume_zone(cs_mesh_t       *mesh,
+                                     const cs_zone_t *z);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define coupling volume using given cs_zone_t. Then, this volume will
+ * be separated from the rest of the domain with thin walls.
+ *
+ * \param[in]  n_zones   number of associated volume zones
+ * \param[in]  zone_ids  ids of associated volume zones
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_internal_coupling_add_volume_zones(int        n_zones,
+                                      const int  zone_ids[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define internal coupling volume boundary group names.
+ *
+ * This is used only for internal couplings based on a separation of volumes
+ * (cs_internal_coupling_add_volume, cs_internal_coupling_add_volume_zone,
+ * cs_internal_coupling_add_volume_zones).
+ *
+ * The interior name is used for faces adjacent to the main volume, and
+ * the exterio name for faces adjacent to the selected (exterior) volume.
+ *
+ * This allows filtering faces on each side of the boundary in a simpler manner.
+ *
+ * \param[in, out] cpl             pointer to mesh structure to modify
+ * \param[in]      criteria_cells  criteria for the first group of cells
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_internal_coupling_add_boundary_groups(cs_internal_coupling_t  *cpl,
+                                         const char              *interior_name,
+                                         const char              *exterior_name);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Impose wall BCs to internal coupled faces if not yet defined.
@@ -518,9 +574,9 @@ cs_internal_coupling_matrix_add_values(const cs_field_t              *f,
 void
 cs_internal_coupling_coupled_faces(const cs_internal_coupling_t  *cpl,
                                    cs_lnum_t                     *n_local,
-                                   cs_lnum_t                     *faces_local[],
+                                   const cs_lnum_t               *faces_local[],
                                    cs_lnum_t                     *n_distant,
-                                   cs_lnum_t                     *faces_distant[]);
+                                   const cs_lnum_t               *faces_distant[]);
 
 /*----------------------------------------------------------------------------
  * Log information about a given internal coupling entity
@@ -546,7 +602,7 @@ cs_internal_coupling_dump(void);
  * Add preprocessing operations required by coupling volume using given
  * criteria.
  *
- * The volume is seperated from the rest of the domain with inserted
+ * The volume is separated from the rest of the domain with inserted
  * boundaries.
  *
  * parameters:
