@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -294,6 +294,20 @@ cs_io_set_indexed_position(cs_io_t             *inp,
                            cs_io_sec_header_t  *header,
                            size_t               id);
 
+/*----------------------------------------------------------------------------
+ * Set a section's final data type to int.
+ *
+ * It the datatype is not compatible, throw an error.
+ *
+ * parameters:
+ *   header <-- header structure
+ *   cs_io  --> kernel IO structure
+ *----------------------------------------------------------------------------*/
+
+void
+cs_io_set_int(cs_io_sec_header_t  *header,
+              const cs_io_t       *cs_io);
+
 /*----------------------------------------------------------------------------
  * Set a message's final data type to cs_lnum_t.
  *
