@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -66,7 +66,7 @@ BEGIN_C_DECLS
  *-----------------------------------------------------------------------------*/
 
 /*-----------------------------------------------------------------------------
- * Local static variable definitions
+ * Local variable definitions
  *-----------------------------------------------------------------------------*/
 
 static bool  _cs_log_atexit_set = false;
@@ -75,6 +75,9 @@ static FILE* _cs_log[] = {NULL, NULL, NULL};
 static const char* _cs_log_name[] = {"",
                                      "setup.log",
                                      "performance.log"};
+
+static bool  _cs_log_default_active = true;
+
 int cs_glob_log_frequency = 1;
 
 /*============================================================================
@@ -83,7 +86,7 @@ int cs_glob_log_frequency = 1;
  *============================================================================*/
 
 void
-cs_f_log_frequency_get_pointer(int     **ntlist);
+cs_f_log_frequency_get_pointer(int  **ntlist);
 
 /*============================================================================
  * Private function definitions
@@ -241,6 +244,41 @@ cs_f_log_frequency_get_pointer(int     **ntlist)
  * Public function definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Update "active" or "inactive" flag of default log.
+ *
+ * This does not prevent output to the log file, but the flag can be queried
+ * using \ref cs_log_default_is_active, so in most cases, this status
+ * should be checked before logging output
+ *
+ * \param[in]  activate  true to activate, false to deactivate.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_log_default_activate(bool  activate)
+{
+  _cs_log_default_active = activate;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Update "active" or "inactive" flag of default log.
+ *
+ * This does not prevent output to the log file, but the flag can be queried
+ * using \
+ *
+ * \return  true if active, false otherwise.
+ */
+/*----------------------------------------------------------------------------*/
+
+bool
+cs_log_default_is_active(void)
+{
+  return _cs_log_default_active;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Count printable length of a character string.
@@ -294,7 +332,7 @@ cs_log_strlen(const char  *str)
           if (c <= 0x7F) {
             retval++;
           }
-          else if (c >= 0xC0 || c <= 0xFD) {
+          else {
             multibyte = true;
             retval++;
           }
@@ -361,6 +399,38 @@ cs_log_strpadl(char        *dest,
   _log_strpad(dest, src, width, destsize, 1);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Pretty-print int-32 based bit field to string
+ *
+ * \param[in]   code  value to print
+ * \param[out]  buf   output buffer (must be at least 33 bytes).
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_log_binary_pp_int32(int32_t     code,
+                       char     buf[33])
+{
+  int i;
+  int32_t n = code;
+
+  for (i = 0; i < 33; i++)
+    buf[i] = ' ';
+  buf[32] = '\0';
+  buf[31] = '0';
+
+  i = 31;
+  while (n && i > -1) {
+    if (n & 1)
+      buf[i] = '1';
+    else
+      buf[i] = '0';
+    n = n >> 1;
+    i--;
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Print log info to a given log type.
@@ -525,7 +595,7 @@ cs_log_separator(cs_log_t log)
 
   for (i = 0; i < 80; i++)
     separator[i] = '-';
-    separator[80] = '\0';
+  separator[80] = '\0';
 
   cs_log_printf(log, "%s\n", separator);
 }
