@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -41,17 +41,21 @@
  * Local headers
  *----------------------------------------------------------------------------*/
 
+#include "cs_atmo.h"
 #include "cs_base.h"
+#include "cs_boundary.h"
 #include "cs_boundary_zone.h"
 #include "cs_ctwr.h"
+#include "cs_combustion_model.h"
+#include "cs_domain.h"
 #include "cs_fan.h"
 #include "cs_field.h"
 #include "cs_log.h"
+#include "cs_mesh_quantities.h"
 #include "cs_parameters.h"
 #include "cs_physical_constants.h"
 #include "cs_sles.h"
 #include "cs_sles_default.h"
-#include "cs_stokes_model.h"
 #include "cs_thermal_model.h"
 #include "cs_time_moment.h"
 #include "cs_turbomachinery.h"
@@ -59,6 +63,7 @@
 #include "cs_rotation.h"
 #include "cs_turbulence_model.h"
 #include "cs_lagr_log.h"
+#include "cs_velocity_pressure.h"
 #include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------
@@ -102,6 +107,10 @@ BEGIN_C_DECLS
 static void
 _log_global_model_options(void)
 {
+  /* Mesh quantity options */
+
+  cs_mesh_quantities_log_setup();
+
   cs_log_printf(CS_LOG_SETUP,
                 _("\n"
                   "Physical model options\n"
@@ -125,11 +134,27 @@ _log_global_model_options(void)
   cs_turb_constants_log_setup();
 
   cs_time_step_log_setup();
+  cs_time_scheme_log_setup();
 
-  /* Stokes model*/
-  cs_stokes_model_log_setup();
+  /* Velocity-pressure model*/
+  cs_velocity_pressure_model_log_setup();
 
-  /* TODO : Partie iroext etc... */
+  /* Velocity-pressure parameters */
+  cs_velocity_pressure_param_log_setup();
+
+  /* Atmospheric */
+  cs_atmo_log_setup();
+
+  /* Atmospheric chemistry */
+  cs_atmo_chemistry_log_setup();
+
+  /* Atmospheric aerosols */
+  cs_atmo_aerosol_log_setup();
+
+  /* Combustion */
+  cs_combustion_log_setup();
+
+  /* TODO: iroext, etc... */
 
   /* Face viscosity */
   cs_space_disc_log_setup();
@@ -155,6 +180,10 @@ _log_global_model_options(void)
   /* Zone information */
   cs_volume_zone_log_setup();
   cs_boundary_zone_log_setup();
+
+  /* BC information */
+  cs_boundary_log_setup(cs_glob_domain->boundaries);
+  cs_boundary_log_setup(cs_glob_domain->ale_boundaries);
 }
 
 
