@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -81,10 +81,13 @@ static cs_real_t  _machine_epsilon = 1.11e-16;
 /* Numerical constants */
 
 const cs_real_t cs_math_zero_threshold = FLT_MIN;
-const cs_real_t cs_math_onethird = 1./3.;
-const cs_real_t cs_math_onesix = 1./6.;
-const cs_real_t cs_math_onetwelve = 1./12.;
-const cs_real_t cs_math_one24 = 1./24.;
+const cs_real_t cs_math_1ov3 = 1./3.;
+const cs_real_t cs_math_2ov3 = 2./3.;
+const cs_real_t cs_math_4ov3 = 4./3.;
+const cs_real_t cs_math_5ov3 = 5./3.;
+const cs_real_t cs_math_1ov6 = 1./6.;
+const cs_real_t cs_math_1ov12 = 1./12.;
+const cs_real_t cs_math_1ov24 = 1./24.;
 
 /*! epsilon \f$ 10^{-12}\f$ */
 const cs_real_t cs_math_epzero = 1e-12;
@@ -102,6 +105,38 @@ const cs_real_t cs_math_pi = 3.14159265358979323846;
  * Private function definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute a rotation of the elements in a 3x3 real values matrix
+ *
+ * \param[in, out]  m   matrix of 3x3 real values
+ * \param[in]       i   1st index
+ * \param[in]       j   2nd index
+ * \param[in]       k   3rd index
+ * \param[in]       l   4th index
+ * \param[in]       s   rate
+ * \param[in]       t   rate
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+_rotate_ind_33(cs_real_t  m[3][3],
+               cs_lnum_t  i,
+               cs_lnum_t  j,
+               cs_lnum_t  k,
+               cs_lnum_t  l,
+               cs_real_t  s,
+               cs_real_t  t)
+{
+  /* Save values of m[i][j] and m[k][l] */
+  cs_real_t m_ij = m[i][j];
+  cs_real_t m_kl = m[k][l];
+
+  /* Modify the values of (i,j) and (k,l) */
+  m[i][j] = m_ij - s*(m_kl + m_ij*t);
+  m[k][l] = m_kl + s*(m_ij - m_kl*t);
+}
+
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*============================================================================
@@ -224,12 +259,12 @@ cs_math_sym_33_eigen(const cs_real_t  m[6],
 
     cs_real_6_t  n;
     cs_real_t  tr = (m[0] + m[1] + m[2]);
-    cs_real_t  tr_third = cs_math_onethird * tr;
+    cs_real_t  tr_third = cs_math_1ov3 * tr;
 
     e1 = m[0] - tr_third, e2 = m[1] - tr_third, e3 = m[2] - tr_third;
     cs_real_t  p2 = e1*e1 + e2*e2 + e3*e3 + 2.*p1;
 
-    cs_real_t  p = sqrt(p2*cs_math_onesix);
+    cs_real_t  p = sqrt(p2*cs_math_1ov6);
     assert(p > 0.);
     cs_real_t  ovp = 1./p;
 
@@ -241,21 +276,21 @@ cs_math_sym_33_eigen(const cs_real_t  m[6],
     }
 
     /* r should be between -1 and 1 but truncation error and bad conditionning
-       can lead to slighty under/over-shoot */
+       can lead to slightly under/over-shoot */
     cs_real_t  r = 0.5 * cs_math_sym_33_determinant(n);
 
     cs_real_t  cos_theta, cos_theta_2pi3;
     if (r <= -1.) {
-      cos_theta = 0.5; // theta = pi/3;
+      cos_theta = 0.5; /* theta = pi/3; */
       cos_theta_2pi3 = -1.;
     }
     else if (r >= 1.) {
-      cos_theta = 1.; // theta = 0.;
+      cos_theta = 1.; /* theta = 0.; */
       cos_theta_2pi3 = -0.5;
     }
     else {
-      cos_theta = cos(cs_math_onethird*acos(r));
-      cos_theta_2pi3 = cos(cs_math_onethird*(acos(r) + 2.*cs_math_pi));
+      cos_theta = cos(cs_math_1ov3*acos(r));
+      cos_theta_2pi3 = cos(cs_math_1ov3*(acos(r) + 2.*cs_math_pi));
     }
 
     /* eigenvalues computed should satisfy e1 < e2 < e3 */
@@ -264,7 +299,7 @@ cs_math_sym_33_eigen(const cs_real_t  m[6],
     e2 = tr - e1 -e3; // since tr(m) = e1 + e2 + e3
 
   }
-  else { // m is diagonal
+  else { /* m is diagonal */
 
     e1 = m[0], e2 = m[1], e3 = m[2];
 
@@ -275,6 +310,7 @@ cs_math_sym_33_eigen(const cs_real_t  m[6],
   else {
     if (e2 < e1) e = e2, e2 = e1, e1 = e;
   }
+
   /* Return values */
   eig_vals[0] = e1;
   eig_vals[1] = e2;
@@ -314,17 +350,17 @@ cs_math_33_eigen(const cs_real_t     m[3][3],
 
   cs_real_t  p1 = m[0][1]*m[0][1] + m[0][2]*m[0][2] + m[1][2]*m[1][2];
 
-  if (p1 > 0.0) { // m is not diagonal
+  if (p1 > 0.0) { /* m is not diagonal */
 
     cs_real_t  theta;
     cs_real_t  n[3][3];
-    cs_real_t  tr = cs_math_onethird*(m[0][0] + m[1][1] + m[2][2]);
+    cs_real_t  tr = cs_math_1ov3*(m[0][0] + m[1][1] + m[2][2]);
 
     e1 = m[0][0] - tr, e2 = m[1][1] - tr, e3 = m[2][2] - tr;
     cs_real_t  p2 = e1*e1 + e2*e2 + e3*e3 + 2*p1;
 
     assert(p2 > 0);
-    cs_real_t  p = sqrt(p2*cs_math_onesix);
+    cs_real_t  p = sqrt(p2*cs_math_1ov6);
     cs_real_t  ovp = 1./p;
 
     for (int  i = 0; i < 3; i++) {
@@ -336,23 +372,23 @@ cs_math_33_eigen(const cs_real_t     m[3][3],
     }
 
     /* r should be between -1 and 1 but truncation error and bad conditionning
-       can lead to slighty under/over-shoot */
+       can lead to slightly under/over-shoot */
     cs_real_t  r = 0.5 * cs_math_33_determinant((const cs_real_t (*)[3])n);
 
     if (r <= -1)
-      theta = cs_math_onethird*cs_math_pi;
+      theta = cs_math_1ov3*cs_math_pi;
     else if (r >= 1)
       theta = 0.;
     else
-      theta = cs_math_onethird*acos(r);
+      theta = cs_math_1ov3*acos(r);
 
-    // eigenvalues computed should satisfy e1 < e2 < e3
+    /* eigenvalues computed should satisfy e1 < e2 < e3 */
     e3 = tr + 2*p*cos(theta);
-    e1 = tr + 2*p*cos(theta + 2*cs_math_pi*cs_math_onethird);
+    e1 = tr + 2*p*cos(theta + 2*cs_math_pi*cs_math_1ov3);
     e2 = 3*tr - e1 -e3; // since tr(m) = e1 + e2 + e3
 
   }
-  else { // m is diagonal
+  else { /* m is diagonal */
 
     e1 = m[0][0], e2 = m[1][1], e3 = m[2][2];
     if (e3 < e2) e = e3, e3 = e2, e2 = e;
@@ -460,7 +496,134 @@ cs_math_voltet(const cs_real_t   xv[3],
   cs_math_3_length_unitv(xe, xc, &lec, uec);
   cs_math_3_cross_product(uev, uef, ucp);
 
-  return  cs_math_onesix *lev*lef*lec* fabs(cs_math_3_dot_product(ucp, uec));
+  return  cs_math_1ov6 *lev*lef*lec* fabs(cs_math_3_dot_product(ucp, uec));
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Evaluate eigenvalues and eigenvectors
+ *         of a real symmetric matrix m1[3,3]: m1*m2 = lambda*m2
+ *
+ * Use of Jacobi method for symmetric matrices
+ * (adapted from the book Numerical Recipies in C, Chapter 11.1)
+ *
+ * \param[in]     m_in         matrix of 3x3 real values (initial)
+ * \param[in]     tol_err      absolute tolerance (sum of off-diagonal elements)
+ * \param[out]    eig_val      vector of 3 real values (eigenvalues)
+ * \param[out]    eig_vec      matrix of 3x3 real values (eigenvectors)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_math_33_eig_val_vec(const cs_real_t  m_in[3][3],
+                       const cs_real_t  tol_err,
+                       cs_real_t        eig_val[restrict 3],
+                       cs_real_t        eig_vec[restrict 3][3])
+{
+  /* Declaration of local variables
+   * vec1, vec2:  vectors of 3 real values (copies of diagonal values)
+   * m:           matrix of 3x3 real values (copy of m_in)
+   * epsilon:     error (like machine epsilon for floats) */
+  cs_real_t vec1[3], vec2[3];
+  cs_real_t m[3][3] = {{m_in[0][0], m_in[0][1], m_in[0][2]},
+                       {m_in[1][0], m_in[1][1], m_in[1][2]},
+                       {m_in[2][0], m_in[2][1], m_in[2][2]}};
+  cs_real_t epsilon = 1.0e-16;
+
+  for (int id = 0; id < 3; id++) {
+    eig_val[id] = m_in[id][id];
+    vec1[id] = eig_val[id];
+    vec2[id] = 0.0;
+  }
+
+  /* The strategy here is to adopt a cyclic Jacobi method
+   * where the diagonalisation is carried out by several sweeps
+   * (each sweep to increase the precision of the result)
+   * Here, we perform up to 50 sweeps (the algorithm stops when
+   * reaching the given tolerance) */
+
+  // Error = sum off-diagonal elements
+  cs_real_t error
+    = cs_math_fabs(m[0][1]) + cs_math_fabs(m[0][2]) + cs_math_fabs(m[1][2]);
+
+  // Iterative solution
+  for (int i_sweep = 0; (i_sweep < 50) && (error > tol_err); i_sweep++) {
+
+    // Start loop on off-diagonal elements
+    for (int id1 = 0; id1 < 2; id1 ++) {
+      for (int id2 = id1+1; id2 < 3; id2 ++) {
+        // After 4 sweeps, skip rotation if off-diagonal element is small
+        if (cs_math_fabs(m[id1][id2]) < epsilon) {
+          if (i_sweep > 4)
+            m[id1][id2] = 0.0;
+        }
+        // Otherwise, ...
+        else { // if (abs(m[id1][id2]) >= epsilon)
+          cs_real_t val1, val2, val3, val4, val5;    // Declare five variables val.
+          // Get val1
+          cs_real_t diff = eig_val[id2] - eig_val[id1];
+          if (cs_math_fabs(m[id1][id2]) < epsilon)
+            val1 = m[id1][id2] / diff;
+          else {
+            cs_real_t theta = 0.5 * diff / m[id1][id2];
+            val1 = 1.0 / (cs_math_fabs(theta)+sqrt(1.0+cs_math_pow2(theta)));
+            if ( theta < 0 )
+              val1 = -val1;
+          }
+          // Get val2, val3 and val4
+          val3 = 1.0 / sqrt(1.0+cs_math_pow2(val1));
+          val2 = val1*val3;
+          val4 = val2 / (1.0 + val3);
+          val5 = val1 * m[id1][id2];
+          // Accumulate correction to diagonal elements
+          vec2[id1] -= val5;
+          vec2[id2] += val5;
+          eig_val[id1] -= val5;
+          eig_val[id2] += val5;
+
+          m[id1][id2] = 0.0;
+          // Rotate
+          for (int id3 = 0; id3 <= id1-1; id3++) // Rotations 0 <= j < p
+            _rotate_ind_33(m, id3, id1, id3, id2, val2, val4);
+          for (int id3 = id1+1; id3 <= id2-1; id3++) // Rotations p < j < q
+            _rotate_ind_33(m, id1, id3, id3, id2, val2, val4);
+          for (int id3 = id2+1; id3 < 3; id3++) // Rotations q < j <= n
+            _rotate_ind_33(m, id1, id3, id2, id3, val2, val4);
+          for (int id3 = 0; id3 < 3; id3++)
+            _rotate_ind_33(eig_vec, id3, id1, id3, id2, val2, val4);
+        }
+      }
+    }
+
+    // Update d and reinitialize z
+    for (int id = 0; id < 3; id++ ) {
+      vec1[id] += vec2[id];
+      eig_val[id] = vec1[id];
+      vec2[id] = 0.0;
+    }
+
+    // Update the error
+    error = cs_math_fabs(m[0][1]) + cs_math_fabs(m[0][2]) + cs_math_fabs(m[1][2]);
+  }
+
+  /* Sort eigenvalues and eigenvectors with ascending order */
+  for (int id1 = 0; id1 < 2; id1++) {
+    cs_lnum_t ind_min = id1;
+    for (int id2 = id1+1; id2 < 3; id2++) {
+      if ( eig_val[id2] < eig_val[id1] )
+        ind_min = id2;
+    }
+    if ( ind_min != id1 ) {
+      cs_real_t temp = eig_val[ind_min];
+      eig_val[ind_min] = eig_val[id1];
+      eig_val[id1] = temp;
+      for (int id2 = 0; id2 < 3; id2++) {
+        temp = eig_vec[id2][ind_min];
+        eig_vec[id2][ind_min] = eig_vec[id2][id1];
+        eig_vec[id2][id1] = temp;
+      }
+    }
+  }
 }
 
 /*----------------------------------------------------------------------------*/
