@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -56,6 +56,19 @@ BEGIN_C_DECLS
  * Type definition
  *============================================================================*/
 
+/* Symetric tensor component name */
+
+typedef enum {
+
+  XX,
+  YY,
+  ZZ,
+  XY,
+  YZ,
+  XZ
+
+} cs_math_sym_tensor_component_t;
+
 /*============================================================================
  *  Global variables
  *============================================================================*/
@@ -63,10 +76,13 @@ BEGIN_C_DECLS
 /* Numerical constants */
 
 extern const cs_real_t cs_math_zero_threshold;
-extern const cs_real_t cs_math_onethird;
-extern const cs_real_t cs_math_onesix;
-extern const cs_real_t cs_math_onetwelve;
-extern const cs_real_t cs_math_one24;
+extern const cs_real_t cs_math_1ov3;
+extern const cs_real_t cs_math_2ov3;
+extern const cs_real_t cs_math_4ov3;
+extern const cs_real_t cs_math_5ov3;
+extern const cs_real_t cs_math_1ov6;
+extern const cs_real_t cs_math_1ov12;
+extern const cs_real_t cs_math_1ov24;
 extern const cs_real_t cs_math_epzero;
 extern const cs_real_t cs_math_infinite_r;
 extern const cs_real_t cs_math_big_r;
@@ -88,14 +104,14 @@ extern const cs_real_t cs_math_pi;
 /*----------------------------------------------------------------------------*/
 
 static inline int
-cs_math_binom(short int  n,
-              short int  k)
+cs_math_binom(int  n,
+              int  k)
 {
   int ret = 1;
   assert(n >= k);
 
-  const short int n_iter = (k > n-k) ? n-k : k;
-  for (short int j = 1; j <= n_iter; j++, n--) {
+  const int n_iter = (k > n-k) ? n-k : k;
+  for (int j = 1; j <= n_iter; j++, n--) {
     if (n % j == 0)
       ret *= n/j;
     else if (ret % j == 0)
@@ -107,6 +123,62 @@ cs_math_binom(short int  n,
   return ret;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the absolute value of a real value.
+ *
+ * \param[in]  x  value
+ *
+ * \return the real of the given value
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline cs_real_t
+cs_math_fabs(cs_real_t  x)
+{
+  cs_real_t ret = (x <  0) ? -x : x;
+
+  return ret;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the min value of two real values.
+ *
+ * \param[in]  x, y  values
+ *
+ * \return the min value
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline cs_real_t
+cs_math_fmin(cs_real_t  x,
+             cs_real_t  y)
+{
+  cs_real_t ret = (x <  y) ? x : y;
+
+  return ret;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the max value of two real values.
+ *
+ * \param[in]  x, y  values
+ *
+ * \return the max value
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline cs_real_t
+cs_math_fmax(cs_real_t  x,
+             cs_real_t  y)
+{
+  cs_real_t ret = (x <  y) ? y : x;
+
+  return ret;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the square of a real value.
@@ -243,8 +315,8 @@ cs_math_3_square_distance(const cs_real_t  xa[3],
 /*!
  * \brief Compute the dot product of two vectors of 3 real values.
  *
- * \param[in]     u             vector of 3 real values
- * \param[in]     v             vector of 3 real values
+ * \param[in]   u   vector of 3 real values
+ * \param[in]   v   vector of 3 real values
  *
  * \return the resulting dot product u.v.
  */
@@ -261,13 +333,11 @@ cs_math_3_dot_product(const cs_real_t  u[3],
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Compute the dot product of
- * a tensor t with two vectors n1, and n2
- * n1 t n2
+ * \brief Compute the dot product of a tensor t with two vectors, n1 and n2.
  *
- * \param[in]     n1            vector of 3 real values
- * \param[in]     t             vector of 3 real values
- * \param[in]     n2            vector of 3 real values
+ * \param[in]     n1    vector of 3 real values
+ * \param[in]     t     tensor of 3x3 real values
+ * \param[in]     n2    vector of 3 real values
  *
  * \return the resulting dot product n1.t.n2.
  */
@@ -278,13 +348,40 @@ cs_math_3_33_3_dot_product(const cs_real_t  n1[3],
                            const cs_real_t  t[3][3],
                            const cs_real_t  n2[3])
 {
-  cs_real_t n_t_n =
-    ( n1[0] * t[0][0] * n2[0] + n1[1] * t[1][0] * n2[0] + n1[2] * t[2][0] * n2[0]
-    + n1[0] * t[0][1] * n2[1] + n1[1] * t[1][1] * n2[1] + n1[2] * t[2][1] * n2[1]
-    + n1[0] * t[0][2] * n2[2] + n1[1] * t[1][2] * n2[2] + n1[2] * t[2][2] * n2[2]);
+  cs_real_t n_t_n
+    = (  n1[0]*t[0][0]*n2[0] + n1[1]*t[1][0]*n2[0] + n1[2]*t[2][0]*n2[0]
+       + n1[0]*t[0][1]*n2[1] + n1[1]*t[1][1]*n2[1] + n1[2]*t[2][1]*n2[1]
+       + n1[0]*t[0][2]*n2[2] + n1[1]*t[1][2]*n2[2] + n1[2]*t[2][2]*n2[2]);
   return n_t_n;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute the dot product of a symmetric tensor t with two vectors,
+ *        n1 and n2.
+ *
+ * \param[in]     n1    vector of 3 real values
+ * \param[in]     t     tensor of 6 real values
+ *                      [ 0 3 5 ]
+ *                      [ 3 1 4 ]
+ *                      [ 5 4 2 ]
+ * \param[in]     n2    vector of 3 real values
+ *
+ * \return the resulting dot product n1.t.n2.
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline cs_real_t
+cs_math_3_sym_33_3_dot_product(const cs_real_t  n1[3],
+                               const cs_real_t  t[6],
+                               const cs_real_t  n2[3])
+{
+  cs_real_t n_t_n
+    = (  n1[0]*t[0]*n2[0] + n1[1]*t[3]*n2[0] + n1[2]*t[5]*n2[0]
+       + n1[0]*t[3]*n2[1] + n1[1]*t[1]*n2[1] + n1[2]*t[4]*n2[1]
+       + n1[0]*t[5]*n2[2] + n1[1]*t[4]*n2[2] + n1[2]*t[2]*n2[2]);
+  return n_t_n;
+}
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -322,7 +419,33 @@ cs_math_3_square_norm(const cs_real_t v[3])
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Normalise a vector of 3 real values.
+ * \brief  Normalize a vector of 3 real values.
+ *
+ * \deprecated: use \ref cs_math_3_normalize instead.
+ *
+ * \param[in]     vin    vector
+ * \param[out]    vout   normalized vector
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_math_3_normalise(const cs_real_t  vin[3],
+                    cs_real_t        vout[3])
+{
+  cs_real_t norm = cs_math_3_norm(vin);
+
+  cs_real_t inv_norm = ((norm > cs_math_zero_threshold) ?  1. / norm : 0);
+
+  vout[0] = inv_norm * vin[0];
+  vout[1] = inv_norm * vin[1];
+  vout[2] = inv_norm * vin[2];
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Normalise a vector of 3 real values.
+ *
+ * To normalize in-place, vin and vout may point to the same array.
  *
  * \param[in]     vin           vector
  * \param[out]    vout          normalized vector
@@ -330,8 +453,8 @@ cs_math_3_square_norm(const cs_real_t v[3])
 /*----------------------------------------------------------------------------*/
 
 static inline void
-cs_math_3_normalise(const cs_real_t  vin[3],
-                    cs_real_t        vout[restrict 3])
+cs_math_3_normalize(const cs_real_t  vin[3],
+                    cs_real_t        vout[3])
 {
   cs_real_t norm = cs_math_3_norm(vin);
 
@@ -345,11 +468,11 @@ cs_math_3_normalise(const cs_real_t  vin[3],
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Orthogonal projection of a vector with respect to a normalised
- * vector.
+ *        vector.
  *
- * \param[in]     n             normal vector direction
- * \param[in]     v             vector to be projected
- * \param[out]    vout          projection
+ * \param[in]   n     normal vector direction
+ * \param[in]   v     vector to be projected
+ * \param[out]  vout  projection
  */
 /*----------------------------------------------------------------------------*/
 
@@ -366,18 +489,18 @@ cs_math_3_orthogonal_projection(const cs_real_t  n[3],
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Add the dot product with a normal vector to the normal direction
- * to a vector.
+ *        to a vector.
  *
- * \param[in]     n             normalised face normal vector
- * \param[in]     factor        factor
- * \param[in,out] v             vector to be scaled
+ * \param[in]       n       normalised face normal vector
+ * \param[in]       factor  factor
+ * \param[in, out]  v       vector to be scaled
  */
 /*----------------------------------------------------------------------------*/
 
 static inline void
 cs_math_3_normal_scaling(const cs_real_t  n[3],
                          cs_real_t        factor,
-                         cs_real_3_t      v)
+                         cs_real_t        v[3])
 {
   cs_real_t v_dot_n = (factor -1.) * cs_math_3_dot_product(v, n);
   for (int i = 0; i < 3; i++)
@@ -392,14 +515,14 @@ cs_math_3_normal_scaling(const cs_real_t  n[3],
  *
  * \param[in]     n             normalised face normal vector
  * \param[in]     factor        factor
- * \param[in,out] t             vector to be scaled
+ * \param[in,out] t             matrix to be scaled
  */
 /*----------------------------------------------------------------------------*/
 
 static inline void
 cs_math_33_normal_scaling_add(const cs_real_t  n[3],
                               cs_real_t        factor,
-                              cs_real_33_t     t)
+                              cs_real_t        t[3][3])
 {
   cs_real_t n_t_n = (factor -1.) *
     ( n[0] * t[0][0] * n[0] + n[1] * t[1][0] * n[0] + n[2] * t[2][0] * n[0]
@@ -424,7 +547,7 @@ cs_math_33_normal_scaling_add(const cs_real_t  n[3],
 static inline void
 cs_math_33_3_product(const cs_real_t  m[3][3],
                      const cs_real_t  v[3],
-                     cs_real_3_t      mv)
+                     cs_real_t        mv[restrict 3])
 {
   mv[0] = m[0][0]*v[0] + m[0][1]*v[1] + m[0][2]*v[2];
   mv[1] = m[1][0]*v[0] + m[1][1]*v[1] + m[1][2]*v[2];
@@ -445,7 +568,7 @@ cs_math_33_3_product(const cs_real_t  m[3][3],
 static inline void
 cs_math_33_3_product_add(const cs_real_t  m[3][3],
                          const cs_real_t  v[3],
-                         cs_real_3_t      mv)
+                         cs_real_t        mv[restrict 3])
 {
   mv[0] += m[0][0]*v[0] + m[0][1]*v[1] + m[0][2]*v[2];
   mv[1] += m[1][0]*v[0] + m[1][1]*v[1] + m[1][2]*v[2];
@@ -466,7 +589,7 @@ cs_math_33_3_product_add(const cs_real_t  m[3][3],
 static inline void
 cs_math_33t_3_product(const cs_real_t  m[3][3],
                       const cs_real_t  v[3],
-                      cs_real_3_t      mv)
+                      cs_real_t        mv[restrict 3])
 {
   mv[0] = m[0][0]*v[0] + m[1][0]*v[1] + m[2][0]*v[2];
   mv[1] = m[0][1]*v[0] + m[1][1]*v[1] + m[2][1]*v[2];
@@ -517,6 +640,22 @@ cs_math_sym_33_3_product_add(const cs_real_t  m[6],
   mv[2] += m[5] * v[0] + m[4] * v[1] + m[2] * v[2];
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute the trace of a symmetric tensor.
+ *
+ * \param[in]   t   vector of 6 real values (symmetric tensor)
+ *
+ * \return trace (t[0] + t[1] + t[2])
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline cs_real_t
+cs_math_6_trace(const cs_real_t  t[6])
+{
+  return (t[0] + t[1] + t[2]);
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Compute the product of a matrix of 6x6 real values by
@@ -605,9 +744,9 @@ cs_math_sym_33_determinant(const cs_real_6_t   m)
 /*!
  * \brief Compute the cross product of two vectors of 3 real values.
  *
- * \param[in]     u             vector of 3 real values
- * \param[in]     v             vector of 3 real values
- * \param[out]    uv            vector of 3 real values
+ * \param[in]     u    vector of 3 real values
+ * \param[in]     v    vector of 3 real values
+ * \param[out]    uv   cross-product of u an v
  */
 /*----------------------------------------------------------------------------*/
 
@@ -616,15 +755,41 @@ cs_math_sym_33_determinant(const cs_real_6_t   m)
 #endif
 
 static inline void
-cs_math_3_cross_product(const cs_real_t u[3],
-                        const cs_real_t v[3],
-                        cs_real_t       uv[restrict 3])
+cs_math_3_cross_product(const cs_real_t  u[3],
+                        const cs_real_t  v[3],
+                        cs_real_t        uv[restrict 3])
 {
   uv[0] = u[1]*v[2] - u[2]*v[1];
   uv[1] = u[2]*v[0] - u[0]*v[2];
   uv[2] = u[0]*v[1] - u[1]*v[0];
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute the triple product
+ *
+ * \param[in]     u   vector of 3 real values
+ * \param[in]     v   vector of 3 real values
+ * \param[in]     w   vector of 3 real values
+ *
+ * \return the scalar triple product: (uxv).w
+ */
+/*----------------------------------------------------------------------------*/
+
+#if defined(__INTEL_COMPILER)
+#pragma optimization_level 0 /* Bug with O1 or above with icc 15.0.1 20141023 */
+#endif
+
+static inline cs_real_t
+cs_math_3_triple_product(const cs_real_t  u[3],
+                         const cs_real_t  v[3],
+                         const cs_real_t  w[3])
+{
+  return    (u[1]*v[2] - u[2]*v[1]) * w[0]
+          + (u[2]*v[0] - u[0]*v[2]) * w[1]
+          + (u[0]*v[1] - u[1]*v[0]) * w[2];
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Inverse a 3x3 matrix
@@ -731,8 +896,8 @@ cs_math_33_inv_cramer_sym_in_place(cs_real_t  a[3][3])
  * \remark Symmetric matrix coefficients are stored as follows:
  *         (s11, s22, s33, s12, s23, s13)
  *
- * \param[in]     s             symmetric matrix
- * \param[out]    sout          sout = 1/s1
+ * \param[in]     s      symmetric matrix
+ * \param[out]    sout   sout = 1/s1
  */
 /*----------------------------------------------------------------------------*/
 
@@ -761,19 +926,18 @@ cs_math_sym_33_inv_cramer(const cs_real_t  s[6],
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Compute the product of a matrix of 3x3 real values by a matrix of 3x3
- * real values.
+ * \brief Compute the product of two 3x3 real valued matrixes.
  *
- * \param[in]     m1            matrix of 3x3 real values
- * \param[in]     m2            matrix of 3x3 real values
- * \param[out]    mout          matrix of 3x3 real values
+ * \param[in]     m1     matrix of 3x3 real values
+ * \param[in]     m2     matrix of 3x3 real values
+ * \param[out]    mout   m1.m2 product
  */
 /*----------------------------------------------------------------------------*/
 
 static inline void
 cs_math_33_product(const cs_real_t  m1[3][3],
                    const cs_real_t  m2[3][3],
-                   cs_real_33_t     mout)
+                   cs_real_t        mout[3][3])
 {
   mout[0][0] = m1[0][0]*m2[0][0] + m1[0][1]*m2[1][0] + m1[0][2]*m2[2][0];
   mout[0][1] = m1[0][0]*m2[0][1] + m1[0][1]*m2[1][1] + m1[0][2]*m2[2][1];
@@ -790,8 +954,134 @@ cs_math_33_product(const cs_real_t  m1[3][3],
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Compute the product of a matrix of 3x3 real values by a matrix of 3x3
- * real values and add.
+ * \brief Compute transformation from relative to absolute
+ *        reference frame Q^t M Q
+ *
+ * \param[in]     m      matrix of 3x3 real values
+ * \param[in]     q      transformation matrix of 3x3 real values
+ * \param[out]    mout   Q^t M Q
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_math_33_transform_r_to_a(const cs_real_t  m[3][3],
+                            const cs_real_t  q[3][3],
+                            cs_real_t        mout[3][3])
+{
+  /* _m = M.Q */
+  cs_real_33_t _m;
+  _m[0][0] = m[0][0]*q[0][0] + m[0][1]*q[1][0] + m[0][2]*q[2][0];
+  _m[0][1] = m[0][0]*q[0][1] + m[0][1]*q[1][1] + m[0][2]*q[2][1];
+  _m[0][2] = m[0][0]*q[0][2] + m[0][1]*q[1][2] + m[0][2]*q[2][2];
+
+  _m[1][0] = m[1][0]*q[0][0] + m[1][1]*q[1][0] + m[1][2]*q[2][0];
+  _m[1][1] = m[1][0]*q[0][1] + m[1][1]*q[1][1] + m[1][2]*q[2][1];
+  _m[1][2] = m[1][0]*q[0][2] + m[1][1]*q[1][2] + m[1][2]*q[2][2];
+
+  _m[2][0] = m[2][0]*q[0][0] + m[2][1]*q[1][0] + m[2][2]*q[2][0];
+  _m[2][1] = m[2][0]*q[0][1] + m[2][1]*q[1][1] + m[2][2]*q[2][1];
+  _m[2][2] = m[2][0]*q[0][2] + m[2][1]*q[1][2] + m[2][2]*q[2][2];
+
+  /* mout = Q^t _m */
+  mout[0][0] = q[0][0]*_m[0][0] + q[1][0]*_m[1][0] + q[2][0]*_m[2][0];
+  mout[0][1] = q[0][0]*_m[0][1] + q[1][0]*_m[1][1] + q[2][0]*_m[2][1];
+  mout[0][2] = q[0][0]*_m[0][2] + q[1][0]*_m[1][2] + q[2][0]*_m[2][2];
+
+  mout[1][0] = q[0][1]*_m[0][0] + q[1][1]*_m[1][0] + q[2][1]*_m[2][0];
+  mout[1][1] = q[0][1]*_m[0][1] + q[1][1]*_m[1][1] + q[2][1]*_m[2][1];
+  mout[1][2] = q[0][1]*_m[0][2] + q[1][1]*_m[1][2] + q[2][1]*_m[2][2];
+
+  mout[2][0] = q[0][2]*_m[0][0] + q[1][2]*_m[1][0] + q[2][2]*_m[2][0];
+  mout[2][1] = q[0][2]*_m[0][1] + q[1][2]*_m[1][1] + q[2][2]*_m[2][1];
+  mout[2][2] = q[0][2]*_m[0][2] + q[1][2]*_m[1][2] + q[2][2]*_m[2][2];
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute transformation from absolute to relative
+ *        reference frame Q M Q^t
+ *
+ * \param[in]     m      matrix of 3x3 real values
+ * \param[in]     q      transformation matrix of 3x3 real values
+ * \param[out]    mout   Q M Q^t
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_math_33_transform_a_to_r(const cs_real_t  m[3][3],
+                            const cs_real_t  q[3][3],
+                            cs_real_t        mout[3][3])
+{
+  /* _m = M.Q^t */
+  cs_real_33_t _m;
+  _m[0][0] = m[0][0]*q[0][0] + m[0][1]*q[0][1] + m[0][2]*q[0][2];
+  _m[0][1] = m[0][0]*q[1][0] + m[0][1]*q[1][1] + m[0][2]*q[1][2];
+  _m[0][2] = m[0][0]*q[2][0] + m[0][1]*q[2][1] + m[0][2]*q[2][2];
+
+  _m[1][0] = m[1][0]*q[0][0] + m[1][1]*q[0][1] + m[1][2]*q[0][2];
+  _m[1][1] = m[1][0]*q[1][0] + m[1][1]*q[1][1] + m[1][2]*q[1][2];
+  _m[1][2] = m[1][0]*q[2][0] + m[1][1]*q[2][1] + m[1][2]*q[2][2];
+
+  _m[2][0] = m[2][0]*q[0][0] + m[2][1]*q[0][1] + m[2][2]*q[0][2];
+  _m[2][1] = m[2][0]*q[1][0] + m[2][1]*q[1][1] + m[2][2]*q[1][2];
+  _m[2][2] = m[2][0]*q[2][0] + m[2][1]*q[2][1] + m[2][2]*q[2][2];
+
+  /* mout = Q _m */
+  mout[0][0] = q[0][0]*_m[0][0] + q[0][1]*_m[1][0] + q[0][2]*_m[2][0];
+  mout[0][1] = q[0][0]*_m[0][1] + q[0][1]*_m[1][1] + q[0][2]*_m[2][1];
+  mout[0][2] = q[0][0]*_m[0][2] + q[0][1]*_m[1][2] + q[0][2]*_m[2][2];
+
+  mout[1][0] = q[1][0]*_m[0][0] + q[1][1]*_m[1][0] + q[1][2]*_m[2][0];
+  mout[1][1] = q[1][0]*_m[0][1] + q[1][1]*_m[1][1] + q[1][2]*_m[2][1];
+  mout[1][2] = q[1][0]*_m[0][2] + q[1][1]*_m[1][2] + q[1][2]*_m[2][2];
+
+  mout[2][0] = q[2][0]*_m[0][0] + q[2][1]*_m[1][0] + q[2][2]*_m[2][0];
+  mout[2][1] = q[2][0]*_m[0][1] + q[2][1]*_m[1][1] + q[2][2]*_m[2][1];
+  mout[2][2] = q[2][0]*_m[0][2] + q[2][1]*_m[1][2] + q[2][2]*_m[2][2];
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Extract from the given matrix its symmetric
+ *        and anti-symmetric part
+ *
+ * \param[in]     m        matrix of 3x3 real values
+ * \param[out]    m_sym    matrix of 3x3 real values (symmetric part)
+ * \param[out]    m_ant    matrix of 3x3 real values (anti-symmetric part)
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_math_33_extract_sym_ant(const cs_real_t  m[3][3],
+                           cs_real_t        m_sym[3][3],
+                           cs_real_t        m_ant[3][3])
+{
+  /* sym = 0.5 (m + m_transpose) */
+  m_sym[0][0] = 0.5 * (m[0][0] + m[0][0]);
+  m_sym[0][1] = 0.5 * (m[0][1] + m[1][0]);
+  m_sym[0][2] = 0.5 * (m[0][2] + m[2][0]);
+  m_sym[1][0] = 0.5 * (m[1][0] + m[0][1]);
+  m_sym[1][1] = 0.5 * (m[1][1] + m[1][1]);
+  m_sym[1][2] = 0.5 * (m[1][2] + m[2][1]);
+  m_sym[2][0] = 0.5 * (m[2][0] + m[0][2]);
+  m_sym[2][1] = 0.5 * (m[2][1] + m[1][2]);
+  m_sym[2][2] = 0.5 * (m[2][2] + m[2][2]);
+
+  /* ant = 0.5 (m - m_transpose) */
+  m_ant[0][0] = 0.5 * (m[0][0] - m[0][0]);
+  m_ant[0][1] = 0.5 * (m[0][1] - m[1][0]);
+  m_ant[0][2] = 0.5 * (m[0][2] - m[2][0]);
+  m_ant[1][0] = 0.5 * (m[1][0] - m[0][1]);
+  m_ant[1][1] = 0.5 * (m[1][1] - m[1][1]);
+  m_ant[1][2] = 0.5 * (m[1][2] - m[2][1]);
+  m_ant[2][0] = 0.5 * (m[2][0] - m[0][2]);
+  m_ant[2][1] = 0.5 * (m[2][1] - m[1][2]);
+  m_ant[2][2] = 0.5 * (m[2][2] - m[2][2]);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Add the product of two 3x3 real matrixes to a matrix.
  *
  * \param[in]     m1            matrix of 3x3 real values
  * \param[in]     m2            matrix of 3x3 real values
@@ -802,7 +1092,7 @@ cs_math_33_product(const cs_real_t  m1[3][3],
 static inline void
 cs_math_33_product_add(const cs_real_t  m1[3][3],
                        const cs_real_t  m2[3][3],
-                       cs_real_33_t     mout)
+                       cs_real_t        mout[restrict 3][3])
 {
   mout[0][0] += m1[0][0]*m2[0][0] + m1[0][1]*m2[1][0] + m1[0][2]*m2[2][0];
   mout[0][1] += m1[0][0]*m2[0][1] + m1[0][1]*m2[1][1] + m1[0][2]*m2[2][1];
@@ -821,8 +1111,9 @@ cs_math_33_product_add(const cs_real_t  m1[3][3],
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Compute the product of two symmetric matrices.
+ *
  * Warning: this is valid if and only if s1 and s2 commute (otherwise sout is
- * not symmetric).
+ *          not symmetric).
  *
  * \remark Symmetric matrix coefficients are stored as follows:
  *         (s11, s22, s33, s12, s23, s13)
@@ -855,7 +1146,7 @@ cs_math_sym_33_product(const cs_real_t s1[6],
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Compute a 6x6 matrix A, equivalent to a 3x3 matrix s, such as:
- * A*R_6 = R*s^t + s*R
+ *        A*R_6 = R*s^t + s*R
  *
  * \param[in]     s            3x3 matrix
  * \param[out]    sout         6x6 matrix
@@ -863,8 +1154,8 @@ cs_math_sym_33_product(const cs_real_t s1[6],
 /*----------------------------------------------------------------------------*/
 
 static inline void
-cs_math_reduce_sym_prod_33_to_66(const cs_real_t s[3][3],
-                                 cs_real_t       sout[restrict 6][6])
+cs_math_reduce_sym_prod_33_to_66(const cs_real_t  s[3][3],
+                                 cs_real_t        sout[restrict 6][6])
 {
   int tens2vect[3][3];
   int iindex[6], jindex[6];
@@ -917,7 +1208,7 @@ cs_math_sym_33_double_product(const cs_real_t  s1[6],
                               const cs_real_t  s3[6],
                               cs_real_t        sout[restrict 3][3])
 {
-  cs_real_33_t _sout;
+  cs_real_t _sout[3][3];
 
   /* S11 */
   _sout[0][0] = s1[0]*s2[0] + s1[3]*s2[3] + s1[5]*s2[5];
@@ -1102,6 +1393,27 @@ cs_math_voltet(const cs_real_t   xv[3],
                const cs_real_t   xf[3],
                const cs_real_t   xc[3]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Evaluate eigenvalues and eigenvectors
+ *         of a real symmetric matrix m1[3,3]: m1*m2 = lambda*m2
+ *
+ * Use of Jacobi method for symmetric matrices
+ * (adapted from the book Numerical Recipies in C, Chapter 11.1)
+ *
+ * \param[in]     m_in         matrix of 3x3 real values (initial)
+ * \param[in]     tol_err      absolute tolerance (sum of off-diagonal elements)
+ * \param[out]    eig_val      vector of 3 real values (eigenvalues)
+ * \param[out]    eig_vec      matrix of 3x3 real values (eigenvectors)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_math_33_eig_val_vec(const cs_real_t  m_in[3][3],
+                       const cs_real_t  tol_err,
+                       cs_real_t        eig_val[restrict 3],
+                       cs_real_t        eig_vec[restrict 3][3]);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Compute LU factorization of an array of dense matrices
