@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2015 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -68,6 +68,8 @@
 #include "cs_mesh.h"
 #include "cs_measures_util.h"
 
+#include "cs_field.h"
+
 /*----------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
@@ -169,8 +171,8 @@ _mesh_interpol_create_connect(cs_interpol_grid_t   *ig)
 
       MPI_Reduce(&val_in, &val_min, 1, MPI_DOUBLE_INT, MPI_MINLOC, 0,
                   cs_glob_mpi_comm);
-      MPI_Bcast(&val_min.rank, 1, CS_MPI_INT, 0, cs_glob_mpi_comm);
-      MPI_Bcast(&location[ii], 1, CS_MPI_INT, val_min.rank,
+      MPI_Bcast(&val_min.rank, 1, MPI_INT, 0, cs_glob_mpi_comm);
+      MPI_Bcast(&location[ii], 1, MPI_INT, val_min.rank,
                 cs_glob_mpi_comm);
 
       ig->rank_connect[ii] = val_min.rank;
@@ -178,9 +180,10 @@ _mesh_interpol_create_connect(cs_interpol_grid_t   *ig)
   }
 #endif
 
+  /* copy location and switch to 0-based */
 #   pragma omp parallel for
   for (ii = 0; ii < nb_points; ii++) {
-    ig->cell_connect[ii] = location[ii];
+    ig->cell_connect[ii] = location[ii] - 1;
   }
 
   nodal_mesh = fvm_nodal_destroy(nodal_mesh);
@@ -189,12 +192,12 @@ _mesh_interpol_create_connect(cs_interpol_grid_t   *ig)
 }
 
 /*----------------------------------------------------------------------------
- * Interpolate mesh field on interpol grid strcuture.
+ * Interpolate mesh field on interpol grid structure.
  *
  * parameters:
  *   ig                   <-- pointer to the interpolation grid structure
  *   values_to_interpol   <-- field on mesh (size = n_cells)
- *   interpolated_values  --> interpolated values on the interpol grid
+ *   interpolated_values  --> interpolated values on the interpolation grid
  *                            structure (size = ig->nb_point)
  *----------------------------------------------------------------------------*/
 
@@ -210,10 +213,10 @@ cs_interpol_field_on_grid(cs_interpol_grid_t         *ig,
 
 #   pragma omp parallel for private(jj)
   for (ii = 0; ii < nb_points; ii++) {
-    if (ig->cell_connect[ii] > 0 && ig->cell_connect[ii] < mesh->n_cells +1)
+    if (ig->cell_connect[ii] > -1 && ig->cell_connect[ii] < mesh->n_cells)
       for (jj = 0; jj < ms_dim; jj++)
         interpoled_values[ii*ms_dim +jj] =
-          values_to_interpol[(ig->cell_connect[ii] -1)*ms_dim + jj];
+          values_to_interpol[(ig->cell_connect[ii])*ms_dim + jj];
   }
 
 #if defined(HAVE_MPI)
@@ -245,17 +248,8 @@ cs_cressman_interpol(cs_measures_set_t         *ms,
                      cs_real_t                 *interpolated_values,
                      int                        id_type)
 {
-  cs_lnum_t ii;
-  cs_lnum_t jj;
   cs_lnum_t n_elts = 0;
-  cs_real_t interpolated_value;
-  cs_real_t dist_x;
-  cs_real_t dist_y;
-  cs_real_t dist_z;
-  cs_real_t weight;
-  cs_real_t total_weight;
-  cs_real_t r2;
-  cs_real_t *xyz_cen;
+  cs_real_t *xyz_cen = NULL;
   const cs_mesh_t *mesh = cs_glob_mesh;
   const cs_mesh_quantities_t *mesh_quantities = cs_glob_mesh_quantities;
 
@@ -267,30 +261,29 @@ cs_cressman_interpol(cs_measures_set_t         *ms,
     n_elts = mesh->n_b_faces;
     xyz_cen = mesh_quantities->b_face_cog;
   }
+  assert(xyz_cen != NULL);
 
-# pragma omp parallel for private(jj)
-  for (ii = 0; ii < n_elts; ii++) {
-    total_weight = 0.;
-    interpolated_value = 0.;
-    for (jj = 0; jj < ms->nb_measures; jj++) {
+# pragma omp parallel for
+  for (cs_lnum_t ii = 0; ii < n_elts; ii++) {
+    cs_real_t total_weight = 0.;
+    cs_real_t interpolated_value = 0.;
+    for (cs_lnum_t jj = 0; jj < ms->nb_measures; jj++) {
       if (ms->is_cressman[jj] == 1) {
-        dist_x = (xyz_cen[ii*3   ] - ms->coords[jj*3   ])
-                *ms->inf_radius[jj*3   ];
-        dist_y = (xyz_cen[ii*3 +1] - ms->coords[jj*3 +1])
-                *ms->inf_radius[jj*3 +1];
-        dist_z = (xyz_cen[ii*3 +2] - ms->coords[jj*3 +2])
-                *ms->inf_radius[jj*3 +2];
+        cs_real_t dist_x = (xyz_cen[ii*3   ] - ms->coords[jj*3   ])
+                          *ms->inf_radius[jj*3   ];
+        cs_real_t dist_y = (xyz_cen[ii*3 +1] - ms->coords[jj*3 +1])
+                          *ms->inf_radius[jj*3 +1];
+        cs_real_t dist_z = (xyz_cen[ii*3 +2] - ms->coords[jj*3 +2])
+                          *ms->inf_radius[jj*3 +2];
 
-        r2 = dist_x*dist_x + dist_y*dist_y + dist_z*dist_z;
+        cs_real_t r2 = dist_x*dist_x + dist_y*dist_y + dist_z*dist_z;
 
-          if (r2/4. > 700.)
-            weight = 0.;
-          else
-            weight = exp(-r2/4.);
+        cs_real_t weight = 0.;
+        if (r2/4. <= 700.)
+          weight = exp(-r2/4.);
 
-
-          total_weight += weight;
-          interpolated_value += (ms->measures[jj])*weight;
+        total_weight += weight;
+        interpolated_value += (ms->measures[jj])*weight;
       }
     }
 
@@ -402,7 +395,7 @@ cs_interpol_grid_create(const char   *name)
  * Create interpolation grid.
  *
  * parameters:
- *   ig        <-- pointer to an interpol grid strcuture
+ *   ig        <-- pointer to an interpol grid structure
  *   nb_points <-- number of considered points
  *   coord     <-- coordonates of considered points
  *----------------------------------------------------------------------------*/
@@ -415,8 +408,8 @@ cs_interpol_grid_init(cs_interpol_grid_t    *ig,
   cs_lnum_t ii;
   BFT_MALLOC(ig->cell_connect, nb_points, cs_lnum_t);
 #if defined(HAVE_MPI)
-      if (cs_glob_n_ranks > 1)
-        BFT_MALLOC(ig->rank_connect, nb_points, int);
+  if (cs_glob_n_ranks > 1)
+    BFT_MALLOC(ig->rank_connect, nb_points, int);
 #endif
   BFT_MALLOC(ig->coords, 3*nb_points, cs_real_t);
 
@@ -430,7 +423,6 @@ cs_interpol_grid_init(cs_interpol_grid_t    *ig,
   _mesh_interpol_create_connect(ig);
 
   ig->is_connect = true;
-
 }
 
 /*----------------------------------------------------------------------------
@@ -441,7 +433,6 @@ cs_interpol_grid_init(cs_interpol_grid_t    *ig,
  * parameters:
  *   name        <-- measures set name
  *   type_flag   <-- mask of field property and category values (not used yet)
- *   location_id <-- id of associated location
  *   dim         <-- measure set dimension (number of components)
  *   interleaved <-- if dim > 1, indicate if field is interleaved
  *
@@ -529,7 +520,7 @@ cs_measures_set_create(const char   *name,
     ms->is_cressman = NULL;
     ms->is_interpol = NULL;
     ms->inf_radius = NULL;
-
+    ms->comp_ids = NULL;
   }
   else {
     BFT_FREE(ms->coords);
@@ -537,6 +528,7 @@ cs_measures_set_create(const char   *name,
     BFT_FREE(ms->is_cressman);
     BFT_FREE(ms->is_interpol);
     BFT_FREE(ms->inf_radius);
+    BFT_FREE(ms->comp_ids);
   }
 
   return ms;
@@ -555,7 +547,7 @@ cs_measures_set_create(const char   *name,
  *                          0: not taken into account
  *                          1: taken into account
  *   measures_coords  <-- measures spaces coordonates
- *   measures         <-- measures values (associated to coordonates)
+ *   measures         <-- measures values (associated to coordinates)
  *   influence_radius <-- influence radius for interpolation (xyz interleaved)
  *----------------------------------------------------------------------------*/
 
@@ -618,7 +610,6 @@ cs_measures_set_map_values(cs_measures_set_t       *ms,
         ms->inf_radius[ii*3 +jj] = influence_radius[ii*3 + jj];
       }
   }
-
 }
 
 /*----------------------------------------------------------------------------
@@ -710,7 +701,7 @@ cs_measures_set_add_values(cs_measures_set_t       *ms,
  *  id <-- measures set id
  *
  * return:
- *   pointer to the grid structure
+ *   pointer to the measures set structure
  *
  *----------------------------------------------------------------------------*/
 
@@ -760,7 +751,7 @@ cs_interpol_grid_by_id(int  id)
  * name <-- measure set name
  *
  * return:
- *   pointer to the measure set structure
+ *   pointer to the measures set structure
  *----------------------------------------------------------------------------*/
 
 cs_measures_set_t  *
@@ -819,6 +810,8 @@ cs_measures_sets_destroy(void)
     BFT_FREE(ms->coords);
     BFT_FREE(ms->is_interpol);
     BFT_FREE(ms->is_cressman);
+    BFT_FREE(ms->inf_radius);
+    BFT_FREE(ms->comp_ids);
   }
 
   BFT_FREE(_measures_sets);
@@ -877,11 +870,11 @@ cs_interpol_grids_destroy(void)
 
 void CS_PROCF(mestcr, MESTCR)
 (
- const char          *name,
- const cs_int_t      *lname,
- const cs_int_t      *idim,
- const cs_int_t      *ilved,
- cs_int_t            *imeset
+ const char   *name,
+ const int    *lname,
+ const int    *idim,
+ const int    *ilved,
+ int          *imeset
 )
 {
   char *bufname;
@@ -916,9 +909,9 @@ void CS_PROCF(mestcr, MESTCR)
 
 void CS_PROCF(gridcr, GRIDCR)
 (
- const char          *name,
- const cs_int_t      *lname,
- cs_int_t            *igrid
+ const char     *name,
+ const int      *lname,
+ int            *igrid
 )
 {
   char *bufname;
@@ -952,13 +945,13 @@ void CS_PROCF(gridcr, GRIDCR)
 
 void CS_PROCF(mesmap, MESMAP)
 (
- const cs_int_t         *imeset,
- const cs_int_t         *inbmes,
- const cs_real_t        *meset,
- const cs_real_t        *coords,
- const cs_int_t         *cressm,
- const cs_int_t         *interp,
- const cs_real_t        *infrad
+ const int         *imeset,
+ const int         *inbmes,
+ const cs_real_t   *meset,
+ const cs_real_t   *coords,
+ const int         *cressm,
+ const int         *interp,
+ const cs_real_t   *infrad
 )
 {
 
@@ -979,7 +972,7 @@ void CS_PROCF(mesmap, MESMAP)
  *
  * Fortran interface
  *
- * subroutine gridmp (name, lname, igrid)
+ * subroutine gridmap (name, lname, igrid)
  * *****************
  *
  * integer          igrid       : <-- : Measures set id
@@ -989,9 +982,9 @@ void CS_PROCF(mesmap, MESMAP)
 
 void CS_PROCF(grimap, GRIMAP)
 (
- const cs_int_t         *igrid,
- const cs_int_t         *inpts,
- const cs_real_t        *coords
+ const int         *igrid,
+ const int         *inpts,
+ const cs_real_t   *coords
 )
 {
   cs_interpol_grid_t *ig = cs_interpol_grid_by_id(*igrid);
@@ -1020,13 +1013,13 @@ void CS_PROCF(grimap, GRIMAP)
 
 void CS_PROCF(mesadd, MESADD)
 (
- const cs_int_t         *imeset,
- const cs_int_t         *inbmes,
- const cs_real_t        *meset,
- const cs_real_t        *coords,
- const cs_int_t         *cressm,
- const cs_int_t         *interp,
- const cs_real_t        *infrad
+ const int         *imeset,
+ const int         *inbmes,
+ const cs_real_t   *meset,
+ const cs_real_t   *coords,
+ const int         *cressm,
+ const int         *interp,
+ const cs_real_t   *infrad
 )
 {
   cs_measures_set_t *ms = cs_measures_set_by_id(*imeset);
@@ -1058,9 +1051,9 @@ void CS_PROCF(mesadd, MESADD)
 
 void CS_PROCF(mscrss, MSCRSS)
 (
- const cs_int_t         *imeset,
- const cs_int_t         *type,
- cs_real_t              *pldval
+ const int         *imeset,
+ const int         *type,
+ cs_real_t         *pldval
 )
 {
   cs_measures_set_t *ms = cs_measures_set_by_id(*imeset);
@@ -1085,9 +1078,9 @@ void CS_PROCF(mscrss, MSCRSS)
 
 void CS_PROCF(gripol, GRIPOL)
 (
- const cs_int_t         *igrid,
- const cs_real_t        *inval,
- cs_real_t              *pldval
+ const int         *igrid,
+ const cs_real_t   *inval,
+ cs_real_t         *pldval
 )
 {
   cs_interpol_grid_t *ig = cs_interpol_grid_by_id(*igrid);
