@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -37,13 +37,12 @@
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 !> \param[in]     nfbpcd        number of faces with condensation source terms
-!> \param[in]     ifbpcd        index of faces with condensation source terms
 !> \param[in]     izzftcd       faces zone with condensation source terms imposed
 !>                              (at previous and current time steps)
 !_______________________________________________________________________________
 
 subroutine cs_mesh_tagmr &
- ( nfbpcd , ifbpcd, izzftcd )
+ ( nfbpcd , izzftcd )
 
 !===============================================================================
 ! Module files
@@ -71,7 +70,7 @@ implicit none
 
 ! Arguments
 
-integer          nfbpcd, ifbpcd(nfbpcd), izzftcd(nfbpcd)
+integer          nfbpcd, izzftcd(nfbpcd)
 
 ! Local variables
 
