@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,6 +54,7 @@
 
 #include "cs_base.h"
 #include "cs_gui_util.h"
+#include "cs_notebook.h"
 #include "cs_parameters.h"
 #include "cs_partition.h"
 #include "cs_system_info.h"
@@ -125,17 +126,10 @@ _arg_env_help(const char  *name)
           "                   (usually automatic, only required for\n"
           "                   undetermined MPI libraries)\n"));
   fprintf
-    (e, _(" --log             output redirection for rank -1 or 0:\n"
-          "                     0: standard output\n"
-          "                     1: output in \"listing\" (default)\n"));
+    (e, _(" --trace           trace progress in standard output\n"));
   fprintf
-    (e, _(" --logp            output redirection for rank > 0:\n"
-          "                    -1: remove output (default)\n"
-          "                     0: no redirection (if independant\n"
-          "                        terminals, debugger type)\n"
-          "                     1: output in \"listing_n<rank>\"\n"));
-  fprintf
-    (e, _(" -p, --param       <file_name> parameter file\n"));
+    (e, _(" --logp            output redirection for ranks > 0\n"
+          "                   to \"run_solver_n<rank>.log\"\n"));
 
   fprintf
     (e, _(" --preprocess      mesh preprocessing mode\n"));
@@ -172,44 +166,6 @@ _print_version(void)
   printf(_("%s version %s\n"), CS_APP_NAME, CS_APP_VERSION);
 }
 
-/*----------------------------------------------------------------------------
- * Convert an argument to an integer and check its validity
- *
- * parameters:
- *   arg_id  <-- index of argument in argv
- *   argc    <-- number of command line arguments
- *   argv    <-- array of command line arguments
- *   argerr  --> error indicator
- *
- * returns:
- *   integer value
- *----------------------------------------------------------------------------*/
-
-static int
-_arg_to_int(int    arg_id,
-            int    argc,
-            char  *argv[],
-            int   *argerr)
-{
-  char  *start = NULL;
-  char  *end = NULL;
-  int  retval = 0;
-
-  *argerr = 0;
-
-  if (arg_id < argc) {
-    start = argv[arg_id];
-    end = start + strlen(start);
-    retval = strtol(start, &end, 0);
-    if (end != start + strlen(start)) *argerr = 1;
-  }
-  else {
-    *argerr = 1;
-  }
-
-  return retval;
-}
-
 /*============================================================================
  * Public function definitions for Fortran API
  *============================================================================*/
@@ -236,17 +192,14 @@ cs_opts_define(int         argc,
   /* Local variables */
 
   const char *s;
-  const char *s_param = NULL;
   int arg_id = 0, argerr = 0;
 
-  const char moduleoptbase[] = "--yacs-module=";
-
   /* Default initialization */
 
   opts->app_name = NULL;
 
-  opts->ilisr0 = 1;
-  opts->ilisrp = 2;
+  opts->trace = false;
+  opts->logrp = false;
 
   opts->sig_defaults = false;
 
@@ -254,8 +207,6 @@ cs_opts_define(int         argc,
   opts->verif = false;
   opts->benchmark = 0;
 
-  opts->yacs_module = NULL;
-
   /* Parse command line arguments */
 
   while (++arg_id < argc && argerr == 0) {
@@ -298,28 +249,12 @@ cs_opts_define(int         argc,
 
 #endif
 
-    else if (strcmp(s, "--log") == 0) {
-      int n1 = 0;
-      n1 = _arg_to_int(++arg_id, argc, argv, &argerr);
-      if (n1 == 0)
-        opts->ilisr0 = 0;
-      else if (n1 == 1)
-        opts->ilisr0 = 1;
-      else
-        argerr = 1;
+    else if (strcmp(s, "--trace") == 0) {
+      opts->trace = true;
     }
 
     else if (strcmp(s, "--logp") == 0) {
-      int n1 = 0;
-      n1 = _arg_to_int(++arg_id, argc, argv, &argerr);
-      if (n1 == -1)
-        opts->ilisrp = 2;
-      else if (n1 == 0)
-        opts->ilisrp = 0;
-      else if (n1 == 1)
-        opts->ilisrp = 1;
-      else
-        argerr = 1;
+      opts->logrp = true;
     }
 
 #if defined(HAVE_MPI)
@@ -339,37 +274,12 @@ cs_opts_define(int         argc,
 
 #endif /* defined(HAVE_MPI) */
 
-    else if (strcmp(s, "-p") == 0 || strcmp(s, "--param") == 0) {
-      if (arg_id + 1 < argc)
-        s_param = argv[++arg_id];
-      else
-        argerr = 1;
-    }
-
     else if (strcmp(s, "--preprocess") == 0)
       opts->preprocess = true;
 
     else if (strcmp(s, "-q") == 0 || strcmp(s, "--quality") == 0)
       opts->verif = true;
 
-    /* Library loader options (do not appear in help as they
-       are not destined to be used directly by a user) */
-
-    else if (strncmp(s, moduleoptbase, strlen(moduleoptbase)) == 0) {
-      if (cs_glob_rank_id <= 0) {
-        const char *_s = s + strlen(moduleoptbase);
-#if defined(HAVE_DLOPEN)
-        BFT_MALLOC(opts->yacs_module, strlen(_s) + 1, char);
-        strcpy(opts->yacs_module, _s);
-#else
-        fprintf(stderr, _("%s was built without dynamic loader support,\n"
-                          "so module file \"%s\" may not be loaded.\n"),
-                argv[0], _s);
-        cs_exit(EXIT_FAILURE);
-#endif /* defined(HAVE_DLOPEN) */
-      }
-    }
-
     /* signal handling */
 
     else if (strcmp(s, "--sig-defaults") == 0)
@@ -423,9 +333,6 @@ cs_opts_define(int         argc,
       cs_exit(EXIT_SUCCESS);
   }
 
-  if (s_param != NULL)
-    cs_gui_load_file(s_param);
-
   /* If application name has not been defined, use working directory
      base name as default. */
 
