@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -149,7 +149,7 @@ typedef struct
 
 /* Minimum recommended scatter/gather buffer size */
 
-static size_t _cs_parall_min_coll_buf_size = 1024*1024*8;
+static size_t _cs_parall_min_coll_buf_size = 1024*1024;
 
 #endif
 
@@ -584,51 +584,6 @@ cs_f_parall_barrier(void)
 
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
-/*============================================================================
- * Public function definitions for Fortran API
- *============================================================================*/
-
-/*----------------------------------------------------------------------------
- * Return the value associated to a probe.
- *
- * Fortran Interface :
- *
- * subroutine parhis (node, ndrang, var, varcap)
- * *****************
- *
- * integer          node        : <-- : local number of the element related to
- *                                      a measure node
- * integer          ndrang      : <-- : rank of the process owning the closest
- *                                      node from the measure node
- * double precision var(*)      : <-- : values of the variable on local elements
- * double precision varcap      : --> : value of the variable for the element
- *                                      related to the measure node
- *----------------------------------------------------------------------------*/
-
-void
-CS_PROCF (parhis, PARHIS)(cs_int_t   *node,
-                          cs_int_t   *ndrang,
-                          cs_real_t   var[],
-                          cs_real_t  *varcap)
-{
-#if defined(HAVE_MPI)
-
-  if (cs_glob_n_ranks > 1) {
-
-    assert(sizeof(double) == sizeof(cs_real_t));
-
-    if (*ndrang == cs_glob_rank_id)
-      *varcap = var[*node - 1];
-    else
-      *varcap = 0.0;
-
-    MPI_Bcast(varcap, 1, CS_MPI_REAL, *ndrang, cs_glob_mpi_comm);
-
-  }
-
-#endif
-}
-
 /*=============================================================================
  * Public function definitions
  *============================================================================*/
@@ -767,13 +722,6 @@ cs_parall_max_loc_vals(int         n,
   }
 
 #endif
-
-  if (cs_glob_n_ranks == 1) {
-
-    for (int i = 0; i < n; i++)
-      max_loc_vals[i] = max[i];
-
-  }
 }
 
 /*----------------------------------------------------------------------------*/
@@ -812,13 +760,6 @@ cs_parall_min_loc_vals(int         n,
   }
 
 #endif
-
-  if (cs_glob_n_ranks == 1) {
-
-    for (int i = 0; i < n; i++)
-      min_loc_vals[i] = min[i];
-
-  }
 }
 
 /*----------------------------------------------------------------------------*/
@@ -906,10 +847,10 @@ cs_parall_allgather_r(int        n_elts,
 
     assert(sizeof(double) == sizeof(cs_real_t));
 
-    BFT_MALLOC(count, n_domains, cs_int_t);
-    BFT_MALLOC(shift, n_domains, cs_int_t);
+    BFT_MALLOC(count, n_domains, int);
+    BFT_MALLOC(shift, n_domains, int);
 
-    MPI_Allgather(&n_elts, 1, CS_MPI_INT, count, 1, CS_MPI_INT,
+    MPI_Allgather(&n_elts, 1, MPI_INT, count, 1, MPI_INT,
                   cs_glob_mpi_comm);
 
     shift[0] = 0;
