@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -84,14 +84,14 @@ struct _cs_part_to_block_t {
 
   size_t       n_block_ents;    /* Number of entities to receive (this block) */
   size_t       n_part_ents;     /* Number of entities to send (partition) */
-  size_t       recv_size;       /* Size of receive buffer for MPI_Alltoallv
+  size_t       recv_size;       /* Size of receive buffer for MPI_Gatherv
                                    (send_size not necessary, as send_size
                                    should always be equal to n_part_ents,
                                    though elements may be assembled in a
                                    different order) */
 
-  int         *recv_count;      /* Receive counts for MPI_Alltoall */
-  int         *recv_displ;      /* Receive displs for MPI_Alltoall */
+  int         *recv_count;      /* Receive counts for MPI_Gatherv */
+  int         *recv_displ;      /* Receive displs for MPI_Gatherv */
 
   int         *block_rank_id;   /* Block id for each part entity
                                    (NULL if based on global_ent_num) */
