@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -68,6 +68,7 @@ BEGIN_C_DECLS
   \file cs_physical_constants.c
         Base physical constants and fluid properties data.
 */
+
 /*----------------------------------------------------------------------------*/
 
 /*! \struct cs_physical_constants_t
@@ -98,14 +99,7 @@ BEGIN_C_DECLS
 
   \var  cs_fluid_properties_t::ixyzp0
         filling \ref xyzp0 indicator
-  \var  cs_fluid_properties_t::ieos
-        indicator of equation of state
 
-        useful only for the compressible module. Ideal gas with a constant
-        adiabatic coefficient (\ref ieos=1), stiffened gas (\ref ieos=2) and
-        mix of ideal gas (\ref ieos=3) are available, but the user can complete
-        the file \ref cs_cf_thermo.h, which is not a user source, to add new
-        equations of state.
   \var  cs_fluid_properties_t::icp
         indicates if the isobaric specific heat \f$C_p\f$ is variable
         - -1: uniform, no property field is declared
@@ -122,23 +116,26 @@ BEGIN_C_DECLS
         1\f$\leqslant\f$N\f$\leqslant\f$\ref dimens::nscal "nscal" so that
         iscsth(n)=1 (there is a scalar temperature) or with the compressible
         module for non perfect gases.
+
   \var  cs_fluid_properties_t::icv
         property index of the isochoric specific heat
         - -1: uniform isochoric specific heat (no property field defined)
+
   \var  cs_fluid_properties_t::irovar
         variable density field \f$ \rho \f$:
         - 1: true, its variation law be given either
         in the GUI, or in the user subroutine
         \ref cs_user_physical_properties .\n
-        See \subpage physical_properties for more informations.
+        See \ref physical_properties for more informations.
         - 0: false, its value is the reference density
         \ref ro0.
+
   \var  cs_fluid_properties_t::ivivar
         variable viscosity field \f$ \mu \f$:
            - 1: true, its variation law be given either
         in the GUI, or in the user subroutine
         \ref cs_user_physical_properties .\n
-        See \subpage physical_properties for more informations.
+        See \ref physical_properties for more informations.
            - 0: false, its value is the reference molecular
         dynamic viscosity \ref viscl0
   \var  cs_fluid_properties_t::ivsuth
@@ -146,6 +143,7 @@ BEGIN_C_DECLS
         Only useful in gas mix (igmix) specific physics
         - 1: Sutherland law
         - 0: low temperature law (linear except for helium)
+
   \var  cs_fluid_properties_t::ro0
         reference density
 
@@ -157,7 +155,7 @@ BEGIN_C_DECLS
         therefore better to specify its value.
 
         Always useful otherwise, even if a law defining the density is given by
-        the user subroutines \ref usphyv or \ref cs_user_physical_properties.
+        the user subroutines \ref cs_user_physical_properties.
         Indeed, except with the compressible module, CS  does not use the total
         pressure \f$P\f$ when solving the Navier-Stokes equation, but a reduced
         pressure \f$ P^*=P-\rho_0\vect{g}.(\vect{x}-\vect{x}_0)+ P^*_0-P_0 \f$,
@@ -169,16 +167,27 @@ BEGIN_C_DECLS
         \rho\f$, the more \f$P^*\f$ will tend to
         represent only the dynamic part of the pressure and the faster and more
         precise its solution will be. Whatever the value of \ref ro0, both \f$
-        P\f$ and \f$P^*\f$ appear in the listing and the post-processing
+        P\f$ and \f$P^*\f$ appear in the log and the post-processing
         outputs with the compressible module, the calculation is made directly
         on the total pressure.
+
   \var  cs_fluid_properties_t::viscl0
         reference molecular dynamic viscosity
 
         Negative value: not initialized.
 
         Always useful, it is the used value unless the user specifies the
-        viscosity in the subroutine \ref usphyv.
+        viscosity in the subroutine \ref cs_user_physical_properties.
+
+  \var  cs_fluid_properties_t::viscv0
+        reference volume viscosity
+
+        Noted \f$\kappa\f$ in the equation expressing \f$\tens{\sigma}\f$ in
+        the paragraph dedicated to \ref iviscv)
+
+        Used by the compressible model, unless the user specifies a variable
+        volume viscosity in the GUI or \ref cs_user_physical_properties.
+
   \var  cs_fluid_properties_t::p0
         reference pressure for the total pressure
 
@@ -188,6 +197,7 @@ BEGIN_C_DECLS
         \ref xyzp0).
         With the compressible module, the total pressure is solved directly.
         Always useful.
+
   \var  cs_fluid_properties_t::pred0
         reference value for the reduced pressure \f$P^*\f$ (see \ref ro0)
 
@@ -200,6 +210,7 @@ BEGIN_C_DECLS
         It is therefore initialized to \ref p0 and not \ref pred0 (see
         \ref ro0).
         Always useful, except with the compressible module.
+
   \var  cs_fluid_properties_t::xyzp0[3]
         coordinates of the reference point \f$\vect{x}_0\f$ for the total
         pressure
@@ -227,6 +238,7 @@ BEGIN_C_DECLS
         therefore not used.
 
         Always useful, except with the compressible module.
+
   \var  cs_fluid_properties_t::t0
         reference temperature
 
@@ -234,51 +246,59 @@ BEGIN_C_DECLS
         of the density), for the electricity modules to initialize the domain
         temperature and for the compressible module (initializations).
         It must be given in Kelvin.
+
   \var  cs_fluid_properties_t::cp0
         reference specific heat
 
-        Useful if there is 1 <= n <= nscaus
-        so that \ref cs_thermal_model_t::iscalt "cs_glob_thermal_model->iscalt" = n
-        and \ref cs_thermal_model_t::itherm "cs_glob_thermal_model->itherm" =  1
-       (there is a scalar "temperature"), unless the
-        user specifies the specific heat in the user subroutine \ref usphyv
+        Useful if \ref cs_thermal_model_t::itherm "cs_glob_thermal_model->itherm"
+        != CS_THERMAL_MODEL_NONE, unless the
+        user specifies the specific heat in the user subroutine \ref cs_user_physical_properties
         (\ref cstphy::icp "icp" > 0) with the compressible module or coal combustion,
         \ref cp0 is also needed even when there is no user scalar. \note
         None of the scalars from the specific physics is a temperature. \note
         When using the Graphical Interface, \ref cp0 is also used to
         calculate the diffusivity of the thermal scalars, based on their
         conductivity; it is therefore needed, unless the diffusivity is also
-        specified in \ref usphyv.
+        specified in \ref cs_user_physical_properties.
+
   \var  cs_fluid_properties_t::cv0
         reference isochoric specific heat (J/kg/K)
 
-        Useful for the compressible module
+        Useful for the compressible module.
+
+  \var  cs_fluid_properties_t::lambda0
+        reference heat conductivity (W/m/K)
+
+        Always useful. This was previously only available through the GUI,
+        so in most cases, is set to 1.
+
   \var  cs_fluid_properties_t::xmasmr
         molar mass of the perfect gas in \f$ kg/mol \f$
         (if \ref cstphy::ieos "ieos"=1)
 
         Always useful.
-  \var  cs_fluid_properties_t::psginf
-        stiffened gas (\ref cs_fluid_properties_t::ieos "ieos"=2) limit pressure
-        (zero in perfect gas)
-  \var  cs_fluid_properties_t::gammasg
-        stiffened gas (\ref cs_fluid_properties_t::ieos "ieos"=2) polytropic coefficient
-        (dimensionless)
+
   \var  cs_fluid_properties_t::ipthrm
         uniform variable thermodynamic pressure:
         - 0: false (ie not variable)
         - 1: true
+
   \var  cs_fluid_properties_t::pther
         Thermodynamic pressure for the current time step.
+
   \var  cs_fluid_properties_t::pthera
         thermodynamic pressure for the previous time step
+
   \var  cs_fluid_properties_t::pthermax
         thermodynamic maximum pressure for user clipping, used to model a
         venting effect
+
   \var  cs_fluid_properties_t::sleak
         Leak surface
+
   \var  cs_fluid_properties_t::kleak
         Leak head loss (2.9 by default, from Idelcick)
+
   \var  cs_fluid_properties_t::roref
         Initial reference density
 */
@@ -309,7 +329,6 @@ static cs_physical_constants_t _physical_constants = {
 
 static cs_fluid_properties_t  _fluid_properties = {
   .ixyzp0   = -1,
-  .ieos     = -1,
   .icp      = -1,
   .icv      = -1,
   .irovar   = 0,
@@ -317,15 +336,18 @@ static cs_fluid_properties_t  _fluid_properties = {
   .ivsuth   = 0,
   .ro0      = 1.17862,
   .viscl0   = 1.83337e-5,
+  .viscv0   = 0.,
   .p0       = 1.01325e5,
   .pred0    = 0.,
   .xyzp0    = {-1.e30, -1.e30, -1.e30},
   .t0       = 293.15,
   .cp0      = 1017.24,
   .cv0      = 0.,
+  .lambda0  = 1.,
+  .r_pg_cnst = 287.058, /* dry air pperfect gas constant J/mol/K */
+  .rvsra    = 1.607768, /* Note: Rv = 461.52272377 J/mol/K */
+  .clatev   = 2.501e6,
   .xmasmr   = 0.028966, /* air molar mass */
-  .psginf   = 0.,
-  .gammasg  = 1.4,
   .ipthrm   = 0,
   .pther    = 1.013e5,
   .pthera   = 0.,
@@ -342,11 +364,11 @@ static cs_fluid_properties_t  _fluid_properties = {
 
 /*! Ideal gas constant (\f$J.mol^{-1}.K^{-1}\f$) */
 
-const double cs_physical_constants_r = 8.31446;
+const double cs_physical_constants_r = 8.31446261815324;
 
 /*! Boltzmann constant (\f$J.K^{-1}\f$) */
 
-const double cs_physical_constants_kb = 1.38e-23;
+const double cs_physical_constants_kb = 1.380649e-23;
 
 /*! Conversion from Celsius to Kelvin: 275.15   */
 
@@ -377,7 +399,6 @@ cs_f_physical_constants_get_pointers(double  **gx,
 
 void
 cs_f_fluid_properties_get_pointers(int     **ixyzp0,
-                                   int     **ieos,
                                    int     **icp,
                                    int     **icv,
                                    int     **irovar,
@@ -391,9 +412,11 @@ cs_f_fluid_properties_get_pointers(int     **ixyzp0,
                                    double  **t0,
                                    double  **cp0,
                                    double  **cv0,
+                                   double  **lambda0,
+                                   double  **rair,
+                                   double  **rvsra,
+                                   double  **clatev,
                                    double  **xmasmr,
-                                   double  **psginf,
-                                   double  **gammasg,
                                    int     **ipthrm,
                                    double  **pther,
                                    double  **pthera,
@@ -402,6 +425,9 @@ cs_f_fluid_properties_get_pointers(int     **ixyzp0,
                                    double  **kleak,
                                    double  **roref);
 
+void
+cs_f_fluid_properties_pp_get_pointers(double  **viscv0);
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -443,7 +469,6 @@ cs_f_physical_constants_get_pointers(double  **gx,
  *
  * parameters:
  *   ixyzp0   --> pointer to cs_glob_fluid_properties->ixyzp0
- *   ieos     --> pointer to cs_glob_fluid_properties->ieos
  *   icp      --> pointer to cs_glob_fluid_properties->icp
  *   icv      --> pointer to cs_glob_fluid_properties->icv
  *   irovar   --> pointer to cs_glob_fluid_properties->irovar
@@ -457,9 +482,11 @@ cs_f_physical_constants_get_pointers(double  **gx,
  *   t0       --> pointer to cs_glob_fluid_properties->t0
  *   cp0      --> pointer to cs_glob_fluid_properties->cp0
  *   cv0      --> pointer to cs_glob_fluid_properties->cv0
+ *   lambda0  --> pointer to cs_glob_fluid_properties->lambda0
+ *   rair     --> pointer to cs_glob_fluid_properties->r_pg_cnst
+ *   rvsra    --> pointer to cs_glob_fluid_properties->rvsra
+ *   clatev   --> pointer to cs_glob_fluid_properties->clatev
  *   xmasmr   --> pointer to cs_glob_fluid_properties->xmasmr
- *   psginf   --> pointer to cs_glob_fluid_properties->psginf
- *   gammasg  --> pointer to cs_glob_fluid_properties->gammasg
  *   ipthrm   --> pointer to cs_glob_fluid_properties->ipthrm
  *   pther    --> pointer to cs_glob_fluid_properties->pther
  *   pthera   --> pointer to cs_glob_fluid_properties->pthera
@@ -471,7 +498,6 @@ cs_f_physical_constants_get_pointers(double  **gx,
 
 void
 cs_f_fluid_properties_get_pointers(int     **ixyzp0,
-                                   int     **ieos,
                                    int     **icp,
                                    int     **icv,
                                    int     **irovar,
@@ -485,9 +511,11 @@ cs_f_fluid_properties_get_pointers(int     **ixyzp0,
                                    double  **t0,
                                    double  **cp0,
                                    double  **cv0,
+                                   double  **lambda0,
+                                   double  **rair,
+                                   double  **rvsra,
+                                   double  **clatev,
                                    double  **xmasmr,
-                                   double  **psginf,
-                                   double  **gammasg,
                                    int     **ipthrm,
                                    double  **pther,
                                    double  **pthera,
@@ -497,7 +525,6 @@ cs_f_fluid_properties_get_pointers(int     **ixyzp0,
                                    double  **roref)
 {
   *ixyzp0   = &(_fluid_properties.ixyzp0);
-  *ieos     = &(_fluid_properties.ieos);
   *icp      = &(_fluid_properties.icp);
   *icv      = &(_fluid_properties.icv);
   *irovar   = &(_fluid_properties.irovar);
@@ -511,9 +538,11 @@ cs_f_fluid_properties_get_pointers(int     **ixyzp0,
   *t0       = &(_fluid_properties.t0);
   *cp0      = &(_fluid_properties.cp0);
   *cv0      = &(_fluid_properties.cv0);
+  *lambda0  = &(_fluid_properties.lambda0);
+  *rair     = &(_fluid_properties.r_pg_cnst);
+  *rvsra    = &(_fluid_properties.rvsra);
+  *clatev   = &(_fluid_properties.clatev);
   *xmasmr   = &(_fluid_properties.xmasmr);
-  *psginf   = &(_fluid_properties.psginf);
-  *gammasg  = &(_fluid_properties.gammasg);
   *ipthrm   = &(_fluid_properties.ipthrm);
   *pther    = &(_fluid_properties.pther);
   *pthera   = &(_fluid_properties.pthera);
@@ -523,6 +552,22 @@ cs_f_fluid_properties_get_pointers(int     **ixyzp0,
   *roref    = &(_fluid_properties.roref);
 }
 
+/*----------------------------------------------------------------------------
+ * Get pointers to members of the global fluid properties structure.
+ *
+ * This function is intended for use by Fortran wrappers, and
+ * enables mapping to Fortran global pointers.
+ *
+ * parameters:
+ *   visv0   --> pointer to cs_glob_fluid_properties->viscv0
+ *----------------------------------------------------------------------------*/
+
+void
+cs_f_fluid_properties_pp_get_pointers(double  **viscv0)
+{
+  *viscv0   = &(_fluid_properties.viscv0);
+}
+
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*=============================================================================
@@ -531,7 +576,7 @@ cs_f_fluid_properties_get_pointers(int     **ixyzp0,
 
 /*----------------------------------------------------------------------------
  *!
- * \brief Provide acces to cs_glob_physical_constants
+ * \brief Provide access to cs_glob_physical_constants
  *
  * needed to initialize structure with GUI
  *----------------------------------------------------------------------------*/
@@ -544,7 +589,7 @@ cs_get_glob_physical_constants(void)
 
 /*----------------------------------------------------------------------------
  *!
- * \brief Provide acces to cs_glob_fluid_properties
+ * \brief Provide access to cs_glob_fluid_properties
  *
  * needed to initialize structure with GUI
  *----------------------------------------------------------------------------*/
@@ -573,14 +618,18 @@ cs_physical_constants_log_setup(void)
 
   cs_log_printf
     (CS_LOG_SETUP,
-     _("    gravity:    [%14.5e, (Gravity x component)\n"
-       "                 %14.5e, (Gravity y component)\n"
-       "                 %14.5e] (Gravity z component)\n"
-       "    icorio:      %14d (Coriolis source terms)\n\n"),
-       cs_glob_physical_constants->gravity[0],
-       cs_glob_physical_constants->gravity[1],
-       cs_glob_physical_constants->gravity[2],
-       cs_glob_physical_constants->icorio);
+     _("    gravity vector:    [%g, %g, %g]\n"),
+     cs_glob_physical_constants->gravity[0],
+     cs_glob_physical_constants->gravity[1],
+     cs_glob_physical_constants->gravity[2]);
+
+  const char *icorio_value_str[]
+    = {N_("    0 (ignore Coriolis source terms)"),
+       N_("    1 (apply Coriolis source terms)")};
+
+  cs_log_printf(CS_LOG_SETUP,
+                _("    icorio:    %s\n"),
+                _(icorio_value_str[cs_glob_physical_constants->icorio]));
 }
 
 /*----------------------------------------------------------------------------
@@ -594,18 +643,17 @@ cs_fluid_properties_log_setup(void)
 {
   cs_log_printf
     (CS_LOG_SETUP,
-     _("   Continuous phase:\n"
+     _("\n"
+       "  Continuous phase:\n"
        "    ro0:         %14.5e (Reference density)\n"
        "    viscl0:      %14.5e (Ref. molecular dyn. visc.)\n"
        "    cp0:         %14.5e (Ref. specific heat)\n"
-       "    icp:         %14d (> 0: Variable cp (cs_user_physical_properties))\n"
+       "    icp:         %4d (> 0: Variable Cp (cs_user_physical_properties))\n"
        "    p0:          %14.5e (Ref. total pressure)\n"
        "    pred0:       %14.5e (Ref. reduced pressure)\n"
        "    t0:          %14.5e (Ref. temperature)\n\n"
-       "    irovar:      %14d (Density constant(0) or not(1))\n"
-       "    ivivar:      %14d (Molec. visc cst.(0) or not(1))\n\n"
        "    Initial reference point for pressure\n"
-       "    xyzp0:       %14.5e %14.5e %14.5e\n"),
+       "    xyzp0:       %14.5e %14.5e %14.5e\n\n"),
        cs_glob_fluid_properties->ro0,
        cs_glob_fluid_properties->viscl0,
        cs_glob_fluid_properties->cp0,
@@ -613,11 +661,22 @@ cs_fluid_properties_log_setup(void)
        cs_glob_fluid_properties->p0,
        cs_glob_fluid_properties->pred0,
        cs_glob_fluid_properties->t0,
-       cs_glob_fluid_properties->irovar,
-       cs_glob_fluid_properties->ivivar,
        cs_glob_fluid_properties->xyzp0[0],
        cs_glob_fluid_properties->xyzp0[1],
        cs_glob_fluid_properties->xyzp0[2]);
+
+  const char *irovar_value_str[] = {N_("0 (constant density)"),
+                                    N_("1 (variable density)")};
+  const char *ivivar_value_str[] = {N_("0 (constant molecular viscosity)"),
+                                    N_("1 (variable molecular viscosity)")};
+
+  cs_log_printf(CS_LOG_SETUP,
+                _("    irovar:    %s\n"),
+                _(irovar_value_str[cs_glob_fluid_properties->irovar]));
+
+  cs_log_printf(CS_LOG_SETUP,
+                _("    ivivar:    %s\n"),
+                _(ivivar_value_str[cs_glob_fluid_properties->ivivar]));
 }
 
 /*----------------------------------------------------------------------------*/
