@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -63,23 +63,28 @@ BEGIN_C_DECLS
 
 /* Default writer ids and filters */
 
-#define CS_POST_WRITER_ALL_ASSOCIATED  0       /* all associated writers */
+#define CS_POST_WRITER_ALL_ASSOCIATED  0       /*!< all associated writers */
 
-#define CS_POST_WRITER_DEFAULT        -1       /* default visualisation */
-#define CS_POST_WRITER_ERRORS         -2       /* error visualisation */
-#define CS_POST_WRITER_PARTICLES      -3       /* particle visualisation */
-#define CS_POST_WRITER_TRAJECTORIES   -4       /* trajectories visualisation */
-#define CS_POST_WRITER_PROBES         -5       /* probe monitoring */
-#define CS_POST_WRITER_PROFILES       -6       /* profiles */
-#define CS_POST_WRITER_HISTOGRAMS     -7       /* histograms */
+#define CS_POST_WRITER_DEFAULT        -1       /*!< default visualisation */
+#define CS_POST_WRITER_ERRORS         -2       /*!< error visualisation */
+#define CS_POST_WRITER_PARTICLES      -3       /*!< particle visualisation */
+#define CS_POST_WRITER_TRAJECTORIES   -4       /*!< trajectories visualisation */
+#define CS_POST_WRITER_PROBES         -5       /*!< probe monitoring */
+#define CS_POST_WRITER_PROFILES       -6       /*!< profiles */
+#define CS_POST_WRITER_HISTOGRAMS     -7       /*!< histograms */
 
 /* Default mesh ids */
 
-#define CS_POST_MESH_VOLUME           -1       /* volume mesh output */
-#define CS_POST_MESH_BOUNDARY         -2       /* boundary mesh output */
-#define CS_POST_MESH_PARTICLES        -3       /* particle output */
-#define CS_POST_MESH_TRAJECTORIES     -4       /* particle output */
-#define CS_POST_MESH_PROBES           -5       /* probes output */
+#define CS_POST_MESH_VOLUME           -1       /*!< volume mesh output */
+#define CS_POST_MESH_BOUNDARY         -2       /*!< boundary mesh output */
+#define CS_POST_MESH_PARTICLES        -3       /*!< particle output */
+#define CS_POST_MESH_TRAJECTORIES     -4       /*!< particle output */
+#define CS_POST_MESH_PROBES           -5       /*!< probes output */
+
+/* Additional categories (no associated default mesh) */
+
+#define CS_POST_MESH_SURFACE         -12       /* surface (boundary and/or
+                                                  interior) mesh */
 
 /*============================================================================
  * Local type definitions
@@ -88,9 +93,8 @@ BEGIN_C_DECLS
 /* Datatype enumeration */
 
 typedef enum {
-  CS_POST_TYPE_cs_int_t,
-  CS_POST_TYPE_cs_real_t,
   CS_POST_TYPE_int,
+  CS_POST_TYPE_cs_real_t,
   CS_POST_TYPE_float,
   CS_POST_TYPE_double
 } cs_post_type_t;
@@ -200,7 +204,7 @@ typedef void
 /*!
  * \brief Define a writer; this objects manages a case's name, directory,
  *        and format, as well as associated mesh's time dependency, and the
- *        default output frequency for associated variables.
+ *        default output interval for associated variables.
  *
  * This function must be called before the time loop. If a writer with a
  * given id is defined multiple times, the last definition supercedes the
@@ -277,9 +281,9 @@ typedef void
  *                              connectivity changes
  * \param[in]  output_at_start  force output at calculation start if true
  * \param[in]  output_at_end    force output at calculation end if true
- * \param[in]  frequency_n      default output frequency in time-steps, or < 0
- * \param[in]  frequency_t      default output frequency in seconds, or < 0
- *                              (has priority over frequency_n)
+ * \param[in]  interval_n       default output interval in time-steps, or < 0
+ * \param[in]  interval_t       default output interval in seconds, or < 0
+ *                              (has priority over interval_n)
  */
 /*----------------------------------------------------------------------------*/
 
@@ -292,8 +296,8 @@ cs_post_define_writer(int                     writer_id,
                       fvm_writer_time_dep_t   time_dep,
                       bool                    output_at_start,
                       bool                    output_at_end,
-                      int                     frequency_n,
-                      double                  frequency_t);
+                      int                     interval_n,
+                      double                  interval_t);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -618,6 +622,30 @@ void
 cs_post_mesh_detach_writer(int  mesh_id,
                            int  writer_id);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Associate a writer to a postprocessing mesh.
+ *
+ * This function must be called during the postprocessing output definition
+ * stage, before any output actually occurs.
+ *
+ * If the field should already be output automatically based on the mesh
+ * category and field output keywords, it will not be added.
+ *
+ * \param[in]  mesh_id    id of associated mesh
+ * \param[in]  writer_id  id of specified associated writer,
+ *                        or \ref CS_POST_WRITER_ALL_ASSOCIATED for all
+ * \param[in]  field_id   id of field to attach
+ * \param[in]  comp_id    id of field component (-1 for all)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_post_mesh_attach_field(int  mesh_id,
+                          int  writer_id,
+                          int  field_id,
+                          int  comp_id);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Get a postprocessing meshes entity presence flag.
@@ -864,7 +892,7 @@ cs_post_get_free_mesh_id(void);
 /*!
  * \brief Update "active" or "inactive" flag of writers based on the time step.
  *
- * Writers are activated if their output frequency is a divisor of the
+ * Writers are activated if their output interval is a divisor of the
  * current time step, or if their optional time step and value output lists
  * contain matches for the current time step.
  *
@@ -1164,17 +1192,6 @@ void
 cs_post_renum_faces(const cs_lnum_t  init_i_face_num[],
                     const cs_lnum_t  init_b_face_num[]);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Configure the post-processing output so that a mesh displacement
- *        field may be output automatically for meshes based on the global
- *        volume mesh.
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_post_set_deformable(void);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Configure the post-processing output so that mesh connectivity
