@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -81,8 +81,8 @@ BEGIN_C_DECLS
 
 typedef struct {
 
-  const cs_int_t   *nvar;
-  const cs_int_t   *nscal;
+  const int   *nvar;
+  const int   *nscal;
 
 } cs_post_default_input_t;
 
@@ -101,7 +101,7 @@ static bool                     _default_input_is_set = false;
 
 /*----------------------------------------------------------------------------
  * Default additional output of mesh and time-dependent variables for the
- * call to pstvar / cs_post_write_vars.
+ * call to cs_post_write_vars.
  *
  * Note: if the input pointer is non-NULL, it must point to valid data
  * when the output function is called, so either:
@@ -146,8 +146,8 @@ _write_additional_vars(void                  *input,
 
   cs_post_default_input_t  *_input = input;
 
-  cs_int_t   nummai = mesh_id;
-  cs_int_t   numtyp = cat_id;
+  int   nummai = mesh_id;
+  int   numtyp = cat_id;
 
   cs_real_t  *var_trav = NULL;
   cs_real_t  *cel_vals = NULL;
@@ -365,8 +365,8 @@ _write_q_criterion(void                  *input,
 
 void CS_PROCF (pstvar, PSTVAR)
 (
- const cs_int_t   *nvar,
- const cs_int_t   *nscal
+ const int   *nvar,
+ const int   *nscal
 )
 {
   /* Define or update map of variables */
