@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -56,6 +56,7 @@
 #include "cs_file.h"
 #include "cs_interface.h"
 #include "cs_mesh.h"
+#include "cs_mesh_cartesian.h"
 #include "cs_mesh_from_builder.h"
 #include "cs_mesh_group.h"
 #include "cs_parall.h"
@@ -154,7 +155,7 @@ _mesh_file_info_t  *_mesh_file_info = NULL;
  *   min_size <-- minimum data size
  *
  * returns:
- *   the data size requirde including alignment
+ *   the data size required including alignment
  *----------------------------------------------------------------------------*/
 
 static inline size_t
@@ -171,23 +172,37 @@ _align_size(size_t  min_size)
 static void
 _set_default_input_if_needed(void)
 {
-  const char input_default[] = "mesh_input";
+  const char input_default[] = "mesh_input.csm";
+  const char cp_input_default[] = "restart/mesh_input.csm";
+
+  const char input_default_noext[] = "mesh_input";
+  const char cp_input_default_noext[] = "restart/mesh_input";
+
+  const char input_default_folder[] = "mesh_input";
 
   if (_n_mesh_files == 0) {
 
+    /* First check for mesh_input.csm file */
     if (cs_file_isreg(input_default))
       cs_preprocessor_data_add_file(input_default, 0, NULL, NULL);
 
-    else if (cs_file_isdir(input_default)) {
+    else if (cs_file_isreg(cp_input_default))
+      cs_preprocessor_data_add_file(cp_input_default, 0, NULL, NULL);
+
+    /* If not present, check without extension */
+    else if (cs_file_isreg(input_default_noext))
+      cs_preprocessor_data_add_file(input_default_noext, 0, NULL, NULL);
+
+    else if (cs_file_isdir(input_default_folder)) {
       int i;
-      char **dir_files = cs_file_listdir(input_default);
+      char **dir_files = cs_file_listdir(input_default_folder);
       for (i = 0; dir_files[i] != NULL; i++) {
         char *tmp_name = NULL;
         BFT_MALLOC(tmp_name,
-                   strlen(input_default) + 1 + strlen(dir_files[i]) + 1,
+                   strlen(input_default_noext) + 1 + strlen(dir_files[i]) + 1,
                    char);
         sprintf(tmp_name, "%s%c%s",
-                input_default, _dir_separator, dir_files[i]);
+                input_default_folder, _dir_separator, dir_files[i]);
         if (cs_file_isreg(tmp_name))
           cs_preprocessor_data_add_file(tmp_name, 0, NULL, NULL);
         BFT_FREE(tmp_name);
@@ -196,6 +211,9 @@ _set_default_input_if_needed(void)
       BFT_FREE(dir_files);
     }
 
+    else if (cs_file_isreg(cp_input_default_noext))
+      cs_preprocessor_data_add_file(cp_input_default_noext, 0, NULL, NULL);
+
     else
       bft_error(__FILE__, __LINE__, 0,
                 _("No \"%s\" file or directory found."), input_default);
@@ -294,18 +312,17 @@ _mesh_reader_destroy(_mesh_reader_t  **mr)
  *----------------------------------------------------------------------------*/
 
 static void
-_add_periodicity(cs_mesh_t *mesh,
-                 cs_int_t   perio_type,
-                 cs_int_t   perio_num,
-                 cs_real_t  matrix[3][4])
+_add_periodicity(cs_mesh_t  *mesh,
+                 int         perio_type,
+                 int         perio_num,
+                 cs_real_t   matrix[3][4])
 {
-  cs_int_t  i, j;
   double  _matrix[3][4];
 
   fvm_periodicity_type_t _perio_type = (fvm_periodicity_type_t)perio_type;
 
-  for (i = 0; i < 3; i++) {
-    for (j = 0; j < 4; j++)
+  for (int i = 0; i < 3; i++) {
+    for (int j = 0; j < 4; j++)
       _matrix[i][j] = matrix[i][j];
   }
 
@@ -329,14 +346,13 @@ _set_block_ranges(cs_mesh_t          *mesh,
 {
   int i;
 
-  int min_block_size = 0;
+  size_t min_block_size = 0;
   int rank_id = cs_glob_rank_id;
   int n_ranks = cs_glob_n_ranks;
 
 #if defined(HAVE_MPI)
-  int block_rank_step = 1;
-  cs_file_get_default_comm(&block_rank_step, &min_block_size, NULL, NULL);
-  mb->min_rank_step = block_rank_step;
+  mb->min_rank_step = 1;
+  min_block_size = cs_parall_get_min_coll_buf_size();
 #endif
 
   /* Always build per_face_range in case of periodicity */
@@ -471,7 +487,7 @@ _mesh_groups_rename(cs_mesh_t          *mesh,
 
     /* Set mesh modification flag */
 
-    mesh->modified = 1;
+    mesh->modified |= CS_MESH_MODIFIED;
 
   }
 
@@ -493,14 +509,13 @@ static void
 _colors_to_groups(cs_mesh_t  *mesh,
                   int         n_gc)
 {
-  cs_int_t  i, j;
   int  n_colors = 0;
   int  color_names_size = 0;
 
   /* Counting pass */
 
-  for (j = 0; j < mesh->n_max_family_items; j++) {
-    for (i = mesh->n_families - n_gc; i < mesh->n_families; i++) {
+  for (int j = 0; j < mesh->n_max_family_items; j++) {
+    for (int i = mesh->n_families - n_gc; i < mesh->n_families; i++) {
       if (mesh->family_item[mesh->n_families*j + i] > 0) {
         int color_id = mesh->family_item[mesh->n_families*j + i];
         int name_size = 1;
@@ -518,13 +533,13 @@ _colors_to_groups(cs_mesh_t  *mesh,
 
   if (n_colors > 0) {
     if (mesh->n_groups > 0) {
-      BFT_REALLOC(mesh->group_idx, mesh->n_groups + n_colors + 1, cs_int_t);
+      BFT_REALLOC(mesh->group_idx, mesh->n_groups + n_colors + 1, int);
       BFT_REALLOC(mesh->group,
                   mesh->group_idx[mesh->n_groups] + color_names_size,
                   char);
     }
     else {
-      BFT_MALLOC(mesh->group_idx, n_colors + 1, cs_int_t);
+      BFT_MALLOC(mesh->group_idx, n_colors + 1, int);
       BFT_MALLOC(mesh->group, color_names_size, char);
       mesh->group_idx[0] = 0;
     }
@@ -532,12 +547,12 @@ _colors_to_groups(cs_mesh_t  *mesh,
 
   /* Assignment */
 
-  for (j = 0; j < mesh->n_max_family_items; j++) {
-    for (i = mesh->n_families - n_gc; i < mesh->n_families; i++) {
+  for (int j = 0; j < mesh->n_max_family_items; j++) {
+    for (int i = mesh->n_families - n_gc; i < mesh->n_families; i++) {
       if (mesh->family_item[mesh->n_families*j + i] > 0) {
         int color_id = mesh->family_item[mesh->n_families*j + i];
         int name_size = 1;
-        cs_int_t group_end = mesh->group_idx[mesh->n_groups];
+        int group_end = mesh->group_idx[mesh->n_groups];
         sprintf(mesh->group + group_end, "%d", color_id);
         while (color_id > 0) {
           color_id /= 10;
@@ -663,7 +678,6 @@ _read_dimensions(cs_mesh_t          *mesh,
                  _mesh_reader_t     *mr,
                  int                 file_id)
 {
-  cs_int_t  i, j;
   cs_io_sec_header_t  header;
 
   cs_gnum_t  n_elts = 0;
@@ -690,8 +704,6 @@ _read_dimensions(cs_mesh_t          *mesh,
 
   /* Initialize reading of Preprocessor output */
 
-  bft_printf(_(" Reading metadata from file: \"%s\"\n"), f->filename);
-
 #if defined(HAVE_MPI)
   pp_in = cs_io_initialize(f->filename,
                            "Face-based mesh definition, R0",
@@ -813,7 +825,7 @@ _read_dimensions(cs_mesh_t          *mesh,
         bft_error(__FILE__, __LINE__, 0,
                   _(unexpected_msg), header.sec_name, cs_io_get_name(pp_in));
       else {
-        cs_io_set_cs_lnum(&header, pp_in);
+        cs_io_set_int(&header, pp_in);
         cs_io_read_global(&header, &n_gc, pp_in);
         mesh->n_families += n_gc;
       }
@@ -826,17 +838,17 @@ _read_dimensions(cs_mesh_t          *mesh,
         bft_error(__FILE__, __LINE__, 0,
                   _(unexpected_msg), header.sec_name, cs_io_get_name(pp_in));
       else {
-        cs_io_set_cs_lnum(&header, pp_in);
+        cs_io_set_int(&header, pp_in);
         cs_io_read_global(&header, &n_gc_props_max, pp_in);
         if (n_gc_props_max > mesh->n_max_family_items) {
           /* Update (pad) previous definitions */
           BFT_REALLOC(mesh->family_item,
                       mesh->n_families*n_gc_props_max,
-                      cs_int_t);
-          for (i = mesh->n_max_family_items;
+                      int);
+          for (int i = mesh->n_max_family_items;
                i < n_gc_props_max;
                i++) {
-            for (j = 0; j < mesh->n_families - n_gc; j++)
+            for (int j = 0; j < mesh->n_families - n_gc; j++)
               mesh->family_item[(mesh->n_families - n_gc)*i + j] = 0;
           }
           mesh->n_max_family_items = n_gc_props_max;
@@ -851,7 +863,7 @@ _read_dimensions(cs_mesh_t          *mesh,
         bft_error(__FILE__, __LINE__, 0,
                   _(unexpected_msg), header.sec_name, cs_io_get_name(pp_in));
       else {
-        cs_io_set_cs_lnum(&header, pp_in);
+        cs_io_set_int(&header, pp_in);
         cs_io_read_global(&header, &n_groups, pp_in);
         mesh->n_groups += n_groups;
       }
@@ -860,21 +872,22 @@ _read_dimensions(cs_mesh_t          *mesh,
     else if (strncmp(header.sec_name, "group_name_index",
                      CS_IO_NAME_LEN) == 0) {
 
-      if ((cs_int_t)header.n_vals != n_groups + 1)
+      if ((int)header.n_vals != n_groups + 1)
         bft_error(__FILE__, __LINE__, 0,
                   _(unexpected_msg), header.sec_name, cs_io_get_name(pp_in));
       else {
-        cs_io_set_cs_lnum(&header, pp_in);
-        cs_lnum_t *_group_idx;
-        BFT_MALLOC(_group_idx, n_groups + 1, cs_lnum_t);
+        cs_io_set_int(&header, pp_in);
+        int *_group_idx;
+        BFT_MALLOC(_group_idx, n_groups + 1, int);
         cs_io_read_global(&header, _group_idx, pp_in);
         if (mesh->group_idx == NULL) {
           BFT_MALLOC(mesh->group_idx, mesh->n_groups + 1, int);
-          for (i = 0; i < n_groups+1; i++)
+          for (int i = 0; i < n_groups+1; i++)
             mesh->group_idx[i] = _group_idx[i] - 1;
         }
         else {
           BFT_REALLOC(mesh->group_idx, mesh->n_groups + 1, int);
+          int i, j;
           for (i = 0, j = mesh->n_groups - n_groups; i < n_groups; i++, j++)
             mesh->group_idx[j + 1] = (   mesh->group_idx[j]
                                       + _group_idx[i+1] - _group_idx[i]);
@@ -887,13 +900,13 @@ _read_dimensions(cs_mesh_t          *mesh,
                      CS_IO_NAME_LEN) == 0) {
 
       if (   mesh->group_idx == NULL
-          || (  (cs_int_t)header.n_vals
+          || (  (int)header.n_vals
               != (  mesh->group_idx[mesh->n_groups]
                   - mesh->group_idx[mesh->n_groups - n_groups])))
         bft_error(__FILE__, __LINE__, 0,
                   _(unexpected_msg), header.sec_name, cs_io_get_name(pp_in));
       else {
-        i = mesh->group_idx[mesh->n_groups - n_groups] - mesh->group_idx[0];
+        int i = mesh->group_idx[mesh->n_groups - n_groups] - mesh->group_idx[0];
         BFT_REALLOC(mesh->group, i + header.n_vals + 1, char);
         cs_io_read_global(&header, mesh->group + i, pp_in);
       }
@@ -907,24 +920,25 @@ _read_dimensions(cs_mesh_t          *mesh,
         bft_error(__FILE__, __LINE__, 0,
                   _(unexpected_msg), header.sec_name, cs_io_get_name(pp_in));
       else {
+        cs_io_set_int(&header, pp_in);
         if (mesh->family_item == NULL)
-          BFT_MALLOC(mesh->family_item, n_elts, cs_int_t);
+          BFT_MALLOC(mesh->family_item, n_elts, int);
         if (mesh->n_families == n_gc)
           cs_io_read_global(&header, mesh->family_item, pp_in);
         else {
-          cs_int_t *_family_item = NULL;
-          BFT_REALLOC(mesh->family_item, n_elts, cs_int_t);
-          BFT_MALLOC(_family_item, header.n_vals, cs_int_t);
+          int *_family_item = NULL;
+          BFT_REALLOC(mesh->family_item, n_elts, int);
+          BFT_MALLOC(_family_item, header.n_vals, int);
           cs_io_read_global(&header, _family_item, pp_in);
           /* Shift previous data */
-          for (j = mesh->n_max_family_items - 1; j > 0; j--) {
-            for (i = mesh->n_families - n_gc - 1; i > -1; i--)
+          for (int j = mesh->n_max_family_items - 1; j > 0; j--) {
+            for (int i = mesh->n_families - n_gc - 1; i > -1; i--)
               mesh->family_item[mesh->n_families*j + i]
                 = mesh->family_item[(mesh->n_families - n_gc)*j + i];
           }
-          for (i = 0; i < n_gc; i++) {
+          for (int i = 0; i < n_gc; i++) {
             /* Copy group class data, shifting group names if necessary */
-            for (j = 0; j < n_gc_props_max; j++) {
+            for (int j = 0; j < n_gc_props_max; j++) {
               int _family_item_j = _family_item[n_gc*j + i];
               if (_family_item_j < 0)
                 _family_item_j -= (mesh->n_groups - n_groups);
@@ -932,7 +946,7 @@ _read_dimensions(cs_mesh_t          *mesh,
                 = _family_item_j;
             }
             /* Pad if necessary */
-            for (j = n_gc_props_max; j < mesh->n_max_family_items; j++)
+            for (int j = n_gc_props_max; j < mesh->n_max_family_items; j++)
               mesh->family_item[mesh->n_families*j + (mesh->n_families-n_gc+i)]
                 = 0;
           }
@@ -976,7 +990,7 @@ _read_dimensions(cs_mesh_t          *mesh,
 
         mb->n_perio = mesh->n_init_perio;
 
-        for (i = mesh->n_init_perio - n_init_perio;
+        for (int i = mesh->n_init_perio - n_init_perio;
              i < mesh->n_init_perio;
              i++) {
           mb->n_per_face_couples[i] = 0;
@@ -993,7 +1007,7 @@ _read_dimensions(cs_mesh_t          *mesh,
         bft_error(__FILE__, __LINE__, 0,
                   _(unexpected_msg), header.sec_name, cs_io_get_name(pp_in));
       else {
-        cs_int_t n_rot_perio;
+        int n_rot_perio;
         cs_io_set_cs_lnum(&header, pp_in);
         cs_io_read_global(&header, &n_rot_perio, pp_in);
         if (n_rot_perio > 0)
@@ -1004,7 +1018,7 @@ _read_dimensions(cs_mesh_t          *mesh,
     else if (strncmp(header.sec_name, "n_periodic_faces",
                      CS_IO_NAME_LEN) == 0) {
 
-      if ((cs_int_t)header.n_vals != n_init_perio)
+      if ((int)header.n_vals != n_init_perio)
         bft_error(__FILE__, __LINE__, 0,
                   _(unexpected_msg), header.sec_name, cs_io_get_name(pp_in));
       else {
@@ -1013,7 +1027,7 @@ _read_dimensions(cs_mesh_t          *mesh,
         cs_io_read_global(&header,
                           mb->n_g_per_face_couples + dest_offset,
                           pp_in);
-        for (i = dest_offset; i < mesh->n_init_perio; i++)
+        for (int i = dest_offset; i < mesh->n_init_perio; i++)
           mb->n_g_per_face_couples[i] /= 2;
       }
 
@@ -1030,6 +1044,81 @@ _read_dimensions(cs_mesh_t          *mesh,
   cs_io_finalize(&pp_in);
 }
 
+/*----------------------------------------------------------------------------
+ * Get dimensions of cartesian mesh mesh
+ *
+ * This function updates the information in the mesh
+ *
+ * parameters
+ *   mesh     <-> pointer to mesh structure
+ *   mb       <-> pointer to mesh builder helper structure
+ *----------------------------------------------------------------------------*/
+
+static void
+_read_cartesian_dimensions(cs_mesh_t         *mesh,
+                           cs_mesh_builder_t *mb)
+{
+  cs_gnum_t _nx = cs_mesh_cartesian_get_ncells(0);
+  cs_gnum_t _ny = cs_mesh_cartesian_get_ncells(1);
+  cs_gnum_t _nz = cs_mesh_cartesian_get_ncells(2);
+
+  /* Get total number of cells */
+  mesh->n_g_cells = _nx * _ny * _nz;
+
+  /* Total number of faces */
+  mb->n_g_faces = 3 * _nx * _ny * _nz
+                + _nx * _ny
+                + _nx * _nz
+                + _ny * _nz;
+
+  /* Vertices */
+  mesh->n_g_vertices = (_nx + 1) * (_ny + 1) * (_nz + 1);
+
+  /* Face to vertices connectivity.
+   * Quadrangles means we have 4 vertices per face.
+   */
+  mb->n_g_face_connect_size = 4 * mb->n_g_faces;
+
+  /* One family */
+  mesh->n_families = 7;
+  mesh->n_max_family_items = 1;
+  if (mesh->family_item == NULL)
+    BFT_MALLOC(mesh->family_item, mesh->n_families, int);
+  for (int i = 0; i < mesh->n_families - 1; i++)
+    mesh->family_item[i] = -(i+1);
+
+  mesh->family_item[mesh->n_families - 1] = 0;
+
+  /* Add groups : 6 boundary face */
+  mesh->n_groups = 6;
+  BFT_MALLOC(mesh->group_idx, mesh->n_groups + 1, int);
+  mesh->group_idx[0] = 0;
+  for (int i = 0; i < mesh->n_groups; i++)
+    mesh->group_idx[i+1] = mesh->group_idx[i] + 3;
+
+  BFT_REALLOC(mesh->group, 19, char);
+
+  for (int i = 0; i < mesh->n_groups; i++) {
+    int i0 = 3*i;
+    char *_buf = mesh->group + i0;
+    if (i == 0)
+      strcpy(_buf, "X0");
+    if (i == 1)
+      strcpy(_buf, "X1");
+    if (i == 2)
+      strcpy(_buf, "Y0");
+    if (i == 3)
+      strcpy(_buf, "Y1");
+    if (i == 4)
+      strcpy(_buf, "Z0");
+    if (i == 5)
+      strcpy(_buf, "Z1");
+
+    _buf[2] = '\0';
+  }
+  mesh->group[18] = '\0';
+}
+
 /*----------------------------------------------------------------------------
  * Compute value range information for a given data section.
  *
@@ -1167,7 +1256,7 @@ _inverse_transf_matrix(double  a[3][4],
 
   double abs_pivot, abs_a_ki, factor;
   double _a[3][4];
-  double _a_swap[3], _b_swap[3];
+  double _a_swap[4], _b_swap[4];
 
   /* Copy array (avoid modifying a), and initialize inverse */
 
@@ -1333,13 +1422,13 @@ _read_data(int                 file_id,
            _mesh_reader_t     *mr,
            long                echo)
 {
-  cs_int_t  perio_id, perio_type;
+  int  perio_id, perio_type;
   cs_file_access_t  method;
   cs_io_sec_header_t  header;
 
   cs_real_t  perio_matrix[3][4];
 
-  cs_int_t  perio_num = -1;
+  int        perio_num = -1;
   bool       end_read = false;
   bool       data_read = false;
   cs_io_t  *pp_in = NULL;
@@ -1369,7 +1458,7 @@ _read_data(int                 file_id,
     MPI_Info           hints;
     MPI_Comm           block_comm, comm;
     cs_file_get_default_access(CS_FILE_MODE_READ, &method, &hints);
-    cs_file_get_default_comm(NULL, NULL, &block_comm, &comm);
+    cs_file_get_default_comm(NULL, &block_comm, &comm);
     assert(comm == cs_glob_mpi_comm || comm == MPI_COMM_NULL);
     pp_in = cs_io_initialize(f->filename,
                              "Face-based mesh definition, R0",
@@ -1511,10 +1600,10 @@ _read_data(int                 file_id,
 
         /* Allocate for first file read */
         if (mb->cell_gc_id == NULL)
-          BFT_MALLOC(mb->cell_gc_id, n_vals, cs_int_t);
+          BFT_MALLOC(mb->cell_gc_id, n_vals, int);
 
         /* Read data */
-        cs_io_set_cs_lnum(&header, pp_in);
+        cs_io_set_int(&header, pp_in);
         cs_io_read_block(&header, gnum_range_cur[0], gnum_range_cur[1],
                          mb->cell_gc_id + val_offset_cur, pp_in);
 
@@ -1551,10 +1640,10 @@ _read_data(int                 file_id,
 
         /* Allocate for first file read */
         if (mb->face_gc_id == NULL)
-          BFT_MALLOC(mb->face_gc_id, n_vals, cs_int_t);
+          BFT_MALLOC(mb->face_gc_id, n_vals, int);
 
         /* Read data */
-        cs_io_set_cs_lnum(&header, pp_in);
+        cs_io_set_int(&header, pp_in);
         cs_io_read_block(&header, gnum_range_cur[0], gnum_range_cur[1],
                          mb->face_gc_id + val_offset_cur, pp_in);
 
@@ -1568,6 +1657,38 @@ _read_data(int                 file_id,
         }
       }
 
+      /* Face level values */
+
+      else if (strncmp(header.sec_name, "face_refinement_generation",
+                       CS_IO_NAME_LEN) == 0) {
+
+        mb->have_face_r_gen = true;
+
+        /* Compute range for current file  */
+        _data_range(&header,
+                    pp_in,
+                    mb->n_g_faces,
+                    mr->n_g_faces_read,
+                    1,
+                    0,
+                    mb->face_bi.gnum_range,
+                    gnum_range_cur,
+                    &n_g_faces,
+                    &n_vals,
+                    &n_vals_cur);
+
+        n_faces = n_vals_cur;
+        val_offset_cur = mr->n_faces_read;
+
+        /* Allocate for first file read */
+        if (mb->face_r_gen == NULL)
+          BFT_MALLOC(mb->face_r_gen, n_vals, char);
+
+        /* Read data */
+        cs_io_read_block(&header, gnum_range_cur[0], gnum_range_cur[1],
+                         mb->face_r_gen + val_offset_cur, pp_in);
+      }
+
       /* Face -> vertices connectivity */
 
       else if (strncmp(header.sec_name, "face_vertices_index",
@@ -1707,7 +1828,7 @@ _read_data(int                 file_id,
           _transform_coords(range_size,
                             mb->vertex_coords + val_offset_cur,
                             f->matrix);
-          mesh->modified = 1;
+          mesh->modified |= CS_MESH_MODIFIED;
         }
       }
 
@@ -1833,8 +1954,8 @@ _read_data(int                 file_id,
  *----------------------------------------------------------------------------*/
 
 void
-CS_PROCF(ledevi, LEDEVI)(cs_int_t   *iperio,
-                         cs_int_t   *iperot)
+CS_PROCF(ledevi, LEDEVI)(int  *iperio,
+                         int  *iperot)
 {
   cs_mesh_t  *m = cs_glob_mesh;
 
@@ -1989,6 +2110,10 @@ int
 cs_preprocessor_check_perio(void)
 {
   int retval;
+
+  if (cs_mesh_cartesian_need_build())
+    return 0;
+
   _mesh_reader_t *mr = NULL;
 
   int perio_flag = 0;
@@ -2030,21 +2155,27 @@ cs_preprocessor_data_read_headers(cs_mesh_t          *mesh,
 {
   int file_id;
 
-  _mesh_reader_t *mr = NULL;
-
   /* Initialize reading of Preprocessor output */
 
-  _set_default_input_if_needed();
+  if (cs_mesh_cartesian_need_build()) {
 
-  _cs_glob_mesh_reader = _mesh_reader_create(&_n_mesh_files,
-                                             &_mesh_file_info);
+    _read_cartesian_dimensions(mesh, mesh_builder);
 
-  _n_max_mesh_files = 0;
+  } else {
+    _mesh_reader_t *mr = NULL;
 
-  mr = _cs_glob_mesh_reader;
+    _set_default_input_if_needed();
 
-  for (file_id = 0; file_id < mr->n_files; file_id++)
-    _read_dimensions(mesh, mesh_builder, mr, file_id);
+    _cs_glob_mesh_reader = _mesh_reader_create(&_n_mesh_files,
+                                               &_mesh_file_info);
+
+    _n_max_mesh_files = 0;
+
+    mr = _cs_glob_mesh_reader;
+
+    for (file_id = 0; file_id < mr->n_files; file_id++)
+      _read_dimensions(mesh, mesh_builder, mr, file_id);
+  }
 
   /* Return values */
 
@@ -2133,11 +2264,17 @@ cs_preprocessor_data_read_mesh(cs_mesh_t          *mesh,
   else
     _set_block_ranges(mesh, mesh_builder);
 
-  for (file_id = 0; file_id < mr->n_files; file_id++)
-    _read_data(file_id, mesh, mesh_builder, mr, echo);
+  if (cs_mesh_cartesian_need_build()) {
+    cs_mesh_cartesian_connectivity(mesh, mesh_builder, echo);
+    mesh->modified |= CS_MESH_MODIFIED;
+  }
+  else {
+    for (file_id = 0; file_id < mr->n_files; file_id++)
+      _read_data(file_id, mesh, mesh_builder, mr, echo);
 
-  if (mr->n_files > 1)
-    mesh->modified = 1;
+    if (mr->n_files > 1)
+      mesh->modified |= CS_MESH_MODIFIED;
+  }
 
   /* Partition data */
 
@@ -2153,7 +2290,8 @@ cs_preprocessor_data_read_mesh(cs_mesh_t          *mesh,
 
   /* Free temporary memory */
 
-  _mesh_reader_destroy(&mr);
+  if (mr != NULL)
+    _mesh_reader_destroy(&mr);
   _cs_glob_mesh_reader = mr;
 
   /* Remove family duplicates */
