@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -53,6 +53,8 @@
 #include "cs_mesh_connect.h"
 #include "cs_mesh_location.h"
 #include "cs_mesh_quantities.h"
+#include "cs_order.h"
+#include "cs_parall.h"
 #include "cs_selector.h"
 #include "cs_timer.h"
 
@@ -87,8 +89,10 @@ BEGIN_C_DECLS
 #define CS_PROBE_TRANSIENT   (1 << 0) //   1: locations of probes may change
 #define CS_PROBE_BOUNDARY    (1 << 1) //   2: locations only on the border mesh
 #define CS_PROBE_ON_CURVE    (1 << 2) //   4: locations are on a curve
-#define CS_PROBE_AUTO_VAR    (1 << 3) //   8: automatic output of variables
-#define CS_PROBE_OVERWRITE   (1 << 4) //  16: allow re-creation of probe set
+#define CS_PROBE_OVERWRITE   (1 << 3) //   8: allow re-creation of probe set
+#define CS_PROBE_AUTO_VAR    (1 << 4) //  16: automatic output of variables
+#define CS_PROBE_AUTO_S      (1 << 5) //  32: output curvilinear coordinates
+#define CS_PROBE_AUTO_COORD  (1 << 6) //  64: output cartesian coordinates
 
 /*=============================================================================
  * Local Structure Definitions
@@ -114,6 +118,10 @@ struct _cs_probe_set_t {
   cs_lnum_t     n_probes;       /* Number of probes really used */
   cs_lnum_t     n_loc_probes;   /* Number of probes located on this rank */
 
+  cs_real_3_t  *coords_ini;     /* Initial coordinates of the set of probes
+                                   (only when snap mode is on and
+                                   location is transient) */
+
   cs_real_3_t  *coords;         /* Coordinates of the set of probes
                                    Initially allocated to n_max_probes. */
   cs_real_t    *s_coords;       /* NULL or curvilinear coordinates
@@ -127,7 +135,7 @@ struct _cs_probe_set_t {
   void                         *p_define_input;  /* Advanced local definition
                                                     input */
 
-  int          *loc_id;         /* ids of probes located on local domain */
+  cs_lnum_t    *loc_id;         /* ids of probes located on local domain */
 
   cs_lnum_t    *elt_id;         /* Element ids where the probes have been
                                    located (size: n_loc_probes); -1 for
@@ -137,11 +145,21 @@ struct _cs_probe_set_t {
 
   char         *located;        /* 1 for located probes, 0 for unlocated */
 
+  int           interpolation;  /* 0: no interpolation;
+                                   1: local gradient-based interpolation */
+
   /* User-defined writers associated to this set of probes */
 
   int           n_writers;      /* Number of writers (-1 if unset) */
   int          *writer_ids;     /* List of writer ids */
 
+  /* User-defined fields associated to this set of probes */
+
+  int           n_fields;       /* Number of associated fields in list */
+  int           n_max_fields;   /* Maximum number of associated fields in list */
+  int          *field_info;     /* List of associated field tuples:
+                                   (writer_id, field_id, component_id) */
+
 };
 
 /* List of available keys for setting a set of probes */
@@ -152,6 +170,7 @@ typedef enum {
   PSETKEY_BOUNDARY,
   PSETKEY_SELECT_CRIT,
   PSETKEY_TOLERANCE,
+  PSETKEY_INTERPOLATION,
   PSETKEY_ERROR
 
 } psetkey_t;
@@ -199,6 +218,8 @@ _print_psetkey(psetkey_t  key)
     return "selection_criteria";
   case PSETKEY_TOLERANCE:
     return "tolerance";
+  case PSETKEY_INTERPOLATION:
+    return "interpolation";
   default:
     assert(0);
   }
@@ -230,6 +251,8 @@ _get_psetkey(const char  *keyname)
     key = PSETKEY_SELECT_CRIT;
   else if (strcmp(keyname, "tolerance") == 0)
     key = PSETKEY_TOLERANCE;
+  else if (strcmp(keyname, "interpolation") == 0)
+    key = PSETKEY_INTERPOLATION;
 
   return key;
 }
@@ -273,6 +296,7 @@ _probe_set_free(cs_probe_set_t   *pset)
     return;
 
   BFT_FREE(pset->name);
+  BFT_FREE(pset->coords_ini);
   BFT_FREE(pset->coords);
   BFT_FREE(pset->sel_criter);
   BFT_FREE(pset->loc_id);
@@ -291,6 +315,9 @@ _probe_set_free(cs_probe_set_t   *pset)
 
   if (pset->n_writers > 0)
     BFT_FREE(pset->writer_ids);
+
+  if (pset->n_fields > 0)
+    BFT_FREE(pset->field_info);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -348,6 +375,8 @@ _probe_set_create(const char    *name,
   pset->n_probes = 0;
   pset->n_loc_probes = 0;
 
+  pset->coords_ini = NULL;
+
   BFT_MALLOC(pset->coords, n_max_probes, cs_real_3_t);
   pset->s_coords = NULL;
   pset->labels = NULL;
@@ -360,9 +389,15 @@ _probe_set_create(const char    *name,
   pset->vtx_id = NULL;
   pset->located = NULL;
 
+  pset->interpolation = 0;
+
   pset->n_writers = -1;
   pset->writer_ids = NULL;
 
+  pset->n_fields = 0;
+  pset->n_max_fields = 0;
+  pset->field_info = NULL;
+
   return pset;
 }
 
@@ -431,14 +466,164 @@ _build_local_probe_set(cs_probe_set_t  *pset)
                       &s);
 
   pset->n_probes = n_elts;
-  pset->coords = coords;
-  pset->s_coords = s;
+
+  /* Order by curvilinear coordinates to avoid issues with some
+     plot formats/tools in single-rank mode (not needed in parallel,
+     as this is already handled through IO numbering in that case) */
+
+  if (cs_glob_n_ranks <= 1) {
+    BFT_MALLOC(pset->coords, n_elts, cs_real_3_t);
+    BFT_MALLOC(pset->s_coords, n_elts, cs_real_t);
+
+    cs_lnum_t *order = NULL;
+    BFT_MALLOC(order, n_elts, cs_lnum_t);
+    cs_order_real_allocated(NULL, s, order, n_elts);
+
+    for (cs_lnum_t i = 0; i < n_elts; i++) {
+      cs_lnum_t j = order[i];
+      for (cs_lnum_t k = 0; k < 3; k++)
+        pset->coords[i][k] = coords[j][k];
+      pset->s_coords[i] = s[j];
+    }
+
+    BFT_FREE(order);
+
+    BFT_FREE(coords);
+    BFT_FREE(s);
+  }
+
+  else {
+    pset->coords = coords;
+    pset->s_coords = s;
+  }
 }
 
-/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Merge identical probes when snapped to center.
+ *
+ * \param[in, out]  pset    pointer to a cs_probe_set_t structure
+ * \param[in]       center  cell centers
+ */
+/*----------------------------------------------------------------------------*/
+
+#if defined(__INTEL_COMPILER)
+#if __INTEL_COMPILER < 1800
+#pragma optimization_level 1 /* Bug with O2 or above with icc 17.0.0 20160721 */
+#endif
+#endif
+
+static void
+_merge_snapped_to_center(cs_probe_set_t   *pset,
+                         const cs_real_t   centers[])
+{
+  if (pset == NULL)
+    return;
+
+  int *tag;
+  BFT_MALLOC(tag, pset->n_probes, int);
+
+  for (int i = 0; i < pset->n_probes; i++)
+    tag[i] = 0;
+
+  cs_lnum_t *order;
+  BFT_MALLOC(order, pset->n_loc_probes, cs_lnum_t);
+  cs_order_lnum_allocated(NULL, pset->elt_id, order, pset->n_loc_probes);
+
+  cs_lnum_t e_id = 0;
+
+  while (e_id < pset->n_loc_probes) {
+
+    cs_lnum_t s_id = e_id;
+    cs_lnum_t j_min = order[s_id];
+    int l = pset->elt_id[j_min];
+    int k = pset->loc_id[j_min];
+    cs_real_t d_min = cs_math_3_distance(pset->coords[k], centers + l*3);
+    tag[pset->loc_id[j_min]] = 1;
+
+    for (e_id = s_id+1; e_id < pset->n_loc_probes; e_id++) {
+      cs_lnum_t j = order[e_id];
+      if (pset->elt_id[j] != l)
+        break;
+      else {
+        cs_real_t d = cs_math_3_distance(pset->coords[k], centers + l*3);
+        if (d < d_min) {
+          tag[pset->loc_id[j_min]] = 0;
+          tag[pset->loc_id[j]] = 1;
+          j_min = j;
+          d_min = d;
+        }
+      }
+    }
+
+  }
+
+  BFT_FREE(order);
+
+  /* Now all points to keep are tagged */
+
+  cs_parall_max(pset->n_probes, CS_INT_TYPE, tag);
+
+  /* Discard untagged points
+     ----------------------- */
+
+  /* Update global arrays; transfer tag to renumbering array */
+
+  int n_probes = 0;
+
+  for (int i = 0; i < pset->n_probes; i++) {
+    if (tag[i] > 0) {
+      tag[i] = n_probes;
+      for (int j = 0; j < 3; j++)
+        pset->coords[n_probes][j] = pset->coords[i][j];
+      if (pset->s_coords != NULL)
+        pset->s_coords[n_probes] = pset->s_coords[i];
+      if (pset->labels != NULL)
+        pset->labels[n_probes] = pset->labels[i];
+      pset->located[n_probes] = pset->located[i];
+      n_probes++;
+    }
+    else {
+      if (pset->labels != NULL)
+        BFT_FREE(pset->labels[i]);
+      tag[i] = -1;
+    }
+  }
+  pset->n_probes = n_probes;
+
+  BFT_REALLOC(pset->coords, n_probes, cs_real_3_t);
+  if (pset->s_coords != NULL)
+    BFT_REALLOC(pset->s_coords, n_probes, cs_real_t);
+  if (pset->labels != NULL)
+    BFT_REALLOC(pset->labels, n_probes, char *);
+  BFT_REALLOC(pset->located, n_probes, char);
+
+  /* Update local arrays */
+
+  int n_loc_probes = 0;
+  for (int i = 0; i < pset->n_loc_probes; i++) {
+    int j = pset->loc_id[i];
+    if (tag[j] > -1) {
+      pset->loc_id[n_loc_probes] = tag[j];
+      pset->elt_id[n_loc_probes] = pset->elt_id[i];
+      pset->vtx_id[n_loc_probes] = pset->vtx_id[i];
+      n_loc_probes += 1;
+    }
+  }
+  pset->n_loc_probes = n_loc_probes;
+
+  BFT_REALLOC(pset->loc_id, n_loc_probes, cs_lnum_t);
+  BFT_REALLOC(pset->elt_id, n_loc_probes, cs_lnum_t);
+  BFT_REALLOC(pset->vtx_id, n_loc_probes, cs_lnum_t);
+
+  BFT_FREE(tag);
+}
 
 /*============================================================================
- * Public function definitions
+ * Semi-private function definitions
+ *
+ * The following functions are intended to be used by the postprocessing layer
+ * (cs_post.c), not directly by the user.
  *============================================================================*/
 
 /*----------------------------------------------------------------------------*/
@@ -460,6 +645,42 @@ cs_probe_finalize(void)
   BFT_FREE(_probe_set_array);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Transfer info on associated fields to the caller.
+ *
+ * This function transfert the property of the associated arrays to the caller
+ * and removes it from the probe set info.
+ *
+ * \param[in]   pset        pointer to a cs_probe_set_t structure
+ * \param[out]  n_fields    number of associated fields
+ * \param[out]  field_info  associated field info (array of tuples with
+ *                          writer id, field id, component id)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_probe_set_transfer_associated_field_info(cs_probe_set_t   *pset,
+                                            int              *n_fields,
+                                            int             **field_info)
+{
+  if (pset == NULL)
+    bft_error(__FILE__, __LINE__, 0, _(_err_empty_pset));
+
+  *n_fields = pset->n_fields;
+  *field_info = pset->field_info;
+
+  pset->n_fields = 0;
+  pset->n_max_fields = 0;
+  pset->field_info = NULL;
+}
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
+/*============================================================================
+ * Public function definitions
+ *============================================================================*/
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Retrieve the number of probe sets defined.
@@ -554,6 +775,8 @@ cs_probe_set_get_name(cs_probe_set_t   *pset)
  * \param[out] on_boundary     true if probes are located on boundary
  * \param[out] on_curve        true if the probe set has cuvilinear coordinates
  * \param[out] auto_variables  true if set of variables to output is predefined
+ * \param[out] auto_curve_coo  true if curvilinear coordinates should be output
+ * \param[out] auto_cart_coo   true if cartesian coordinates should be output
  * \param[out] n_writers       number of associated  user-defined writers,
  *                             or -1 if default unchanged
  * \param[out] writer_ids      pointer to a list of writer ids
@@ -566,6 +789,8 @@ cs_probe_set_get_post_info(const cs_probe_set_t   *pset,
                            bool                   *on_boundary,
                            bool                   *on_curve,
                            bool                   *auto_variables,
+                           bool                   *auto_curve_coo,
+                           bool                   *auto_cart_coo,
                            int                    *n_writers,
                            int                    *writer_ids[])
 {
@@ -576,6 +801,10 @@ cs_probe_set_get_post_info(const cs_probe_set_t   *pset,
     *time_varying = (pset->flags & CS_PROBE_TRANSIENT) ? true : false;
   if (auto_variables != NULL)
     *auto_variables = (pset->flags & CS_PROBE_AUTO_VAR) ? true: false;
+  if (auto_curve_coo != NULL)
+    *auto_curve_coo = (pset->flags & CS_PROBE_AUTO_S) ? true: false;
+  if (auto_cart_coo != NULL)
+    *auto_cart_coo = (pset->flags & CS_PROBE_AUTO_COORD) ? true: false;
   if (on_curve != NULL)
     *on_curve = (pset->flags & CS_PROBE_ON_CURVE) ? true : false;
   if (on_boundary != NULL)
@@ -607,6 +836,26 @@ cs_probe_set_get_location_criteria(cs_probe_set_t   *pset)
   return pset->sel_criter;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the interpolation option for a given probe set
+ *
+ * Interpolation will be applied only where possible.
+ *
+ * \param[in]   pset       pointer to a cs_probe_set_t structure
+ *
+ * \return interpolation option value
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_probe_set_get_interpolation(cs_probe_set_t   *pset) {
+  if (pset == NULL)
+    return  0;
+
+  return pset->interpolation;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Create a new set of probes.
@@ -833,6 +1082,57 @@ cs_probe_set_allow_overwrite(const char  *name)
     pset->flags = pset->flags | CS_PROBE_OVERWRITE;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Assign curvilinear abscissa for the given probe set
+ *
+ * This is effective only when using probe sets to which curvilinear
+ * coordinates have not already been assigned.
+ *
+ * So this may be used following \ref cs_probe_set_create (and probe additions)
+ * or \ref cs_probe_set_create_from_array, but will be ignored for probe sets
+ * defined using \ref cs_probe_set_create_from_segment and
+ * \ref cs_probe_set_create_from_local.
+ *
+ * If provided, the array of curvilinear absicssa must match the size of the
+ * probe set. If set to NULL, it is assumed a unifor spacing is provided
+ * between points.
+ *
+ * \param[in]  pset  pointer to a cs_probe_set_t structure
+ * \param[in]  s     array of curvilinear abscissa, or NULL
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_probe_set_assign_curvilinear_abscissa(cs_probe_set_t   *pset,
+                                         const cs_real_t  *s)
+{
+  if (pset == NULL)
+    return;
+
+  if (pset->flags & CS_PROBE_ON_CURVE)
+    return;
+
+  pset->flags |= CS_PROBE_ON_CURVE;
+
+  BFT_REALLOC(pset->s_coords, pset->n_probes, cs_real_t);
+
+  if (s != NULL) {
+    for (int i = 0; i < pset->n_probes; i++)
+      pset->s_coords[i] = s[i];
+  }
+  else if (pset->n_probes > 0) {
+    pset->s_coords[0] = 0.;
+    if (pset->n_probes > 1) {
+      int npm1 = pset->n_probes - 1;
+      cs_real_t a = 1./(cs_real_t)npm1;
+      for (int i = 1; i < npm1; i++)
+        pset->s_coords[i] = i*a;;
+      pset->s_coords[npm1] = 1.;
+    }
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Associate a list of writers to a probe set.
@@ -862,6 +1162,50 @@ cs_probe_set_associate_writers(cs_probe_set_t   *pset,
     pset->writer_ids[i] = writer_ids[j];
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Associate a field to a probe set.
+ *
+ * This function must be called during the postprocessing output definition
+ * stage, before any output actually occurs.
+ *
+ * If the field should already be output automatically based on the mesh
+ * category and field output keywords, it will be filtered at a later stage.
+ *
+ * \param[in]  pset       pointer to a cs_probe_set_t structure
+ * \param[in]  writer_id  id of specified associated writer,
+ *                        or 0 (\ref CS_POST_WRITER_ALL_ASSOCIATED) for all
+ * \param[in]  field_id   id of field to attach
+ * \param[in]  comp_id    id of field component (-1 for all)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_probe_set_associate_field(cs_probe_set_t   *pset,
+                             int               writer_id,
+                             int               field_id,
+                             int               comp_id)
+{
+  if (pset == NULL)
+    bft_error(__FILE__, __LINE__, 0, _(_err_empty_pset));
+
+  if (pset->n_max_fields <= pset->n_fields) {
+    if (pset->n_max_fields == 0)
+      pset->n_max_fields = 8;
+    else
+      pset->n_max_fields *= 2;
+    BFT_REALLOC(pset->field_info, 3*(pset->n_max_fields), int);
+  }
+
+  int *afi = pset->field_info + 3*pset->n_fields;
+
+  afi[0] = writer_id;
+  afi[1] = field_id;
+  afi[2] = comp_id;
+
+  pset->n_fields += 1;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Set to true or false the automatic post-processing of variables
@@ -886,6 +1230,54 @@ cs_probe_set_auto_var(cs_probe_set_t   *pset,
     pset->flags |= CS_PROBE_AUTO_VAR;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set automatic output of curvilinear coordinates.
+ *
+ * \param[in, out] pset     pointer to a cs_probe_set_t structure
+ * \param[in]      mode     true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_probe_set_auto_curvilinear_coords(cs_probe_set_t   *pset,
+                                     bool              mode)
+{
+  if (pset == NULL)
+    bft_error(__FILE__, __LINE__, 0, _(_err_empty_pset));
+
+  if (mode == false) {
+    if (pset->flags & CS_PROBE_AUTO_S)
+      pset->flags -= CS_PROBE_AUTO_S;
+  }
+  else
+    pset->flags |= CS_PROBE_AUTO_S;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set automatic output of curvilinear coordinates.
+ *
+ * \param[in, out] pset     pointer to a cs_probe_set_t structure
+ * \param[in]      mode     true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_probe_set_auto_cartesian_coords(cs_probe_set_t   *pset,
+                                   bool              mode)
+{
+  if (pset == NULL)
+    bft_error(__FILE__, __LINE__, 0, _(_err_empty_pset));
+
+  if (mode == false) {
+    if (pset->flags & CS_PROBE_AUTO_COORD)
+      pset->flags -= CS_PROBE_AUTO_COORD;
+  }
+  else
+    pset->flags |= CS_PROBE_AUTO_COORD;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Set snap mode related to the management of a set of probes.
@@ -920,6 +1312,10 @@ cs_probe_set_snap_mode(cs_probe_set_t   *pset,
  *
  * - \c \b selection_criteria where keyval is selection criteria string
  * - \c \b tolerance  where keyval is for instance "0.05" (default "0.10")
+ * - \c \b interpolation if \ c 0, P0 interpolation (default); if \c 1,
+ *         simple gradient-based interpolation for volume probes (ignored
+ *         for boundaries). Other interpolation options might be added
+ *         in the future.
  *
  * \param[in, out] pset     pointer to a cs_probe_set_t structure to set
  * \param[in]      keyname  name of the keyword related to the parameter to set
@@ -951,7 +1347,7 @@ cs_probe_set_option(cs_probe_set_t   *pset,
     }
     bft_error(__FILE__, __LINE__, 0,
               _(" Invalid key for probe options %s.\n"
-                " Please read listing for more details and"
+                " Please read run_solver.log for more details and"
                 " modify your settings."), pset->name);
 
   } /* Error message */
@@ -992,6 +1388,10 @@ cs_probe_set_option(cs_probe_set_t   *pset,
     pset->tolerance = atof(keyval);
     break;
 
+  case PSETKEY_INTERPOLATION:
+    pset->interpolation = atoi(keyval);
+    break;
+
   default:
     bft_error(__FILE__, __LINE__, 0,
               _(" Key %s is not implemented yet."), keyname);
@@ -1031,11 +1431,17 @@ cs_probe_set_locate(cs_probe_set_t     *pset,
 
   const bool  on_boundary = (pset->flags & CS_PROBE_BOUNDARY) ? true : false;
 
-  /* Build in local case */
+  /* Build in local case, restore saved coordinates if snapped otherwise */
 
   if (pset->p_define_func != NULL)
     _build_local_probe_set(pset);
 
+  else if (pset->coords_ini != NULL) {
+    memcpy(pset->coords,
+           pset->coords_ini,
+           pset->n_probes*3*sizeof(cs_real_t));
+  }
+
   /* Allocate on first pass */
 
   if (pset->located == NULL) {
@@ -1045,7 +1451,7 @@ cs_probe_set_locate(cs_probe_set_t     *pset,
 
   /* Reallocate on all passes, in case local sizes change */
 
-  BFT_REALLOC(pset->loc_id, pset->n_probes, int);
+  BFT_REALLOC(pset->loc_id, pset->n_probes, cs_lnum_t);
   BFT_REALLOC(pset->elt_id, pset->n_probes, cs_lnum_t);
   BFT_FREE(pset->vtx_id);
 
@@ -1185,10 +1591,10 @@ cs_probe_set_locate(cs_probe_set_t     *pset,
 
   if (n_unlocated_probes > 0 && first_location) {
     bft_printf(_("\n Warning: probe set \"%s\"\n"
-                 "   %lu (of %d) probes are not located"
+                 "   %lu (of %ld) probes are not located"
                  " on the associated mesh:\n"),
                pset->name, (unsigned long)n_unlocated_probes,
-               pset->n_probes);
+               (long)pset->n_probes);
     for (int i = 0; i < pset->n_probes; i++) {
       if (pset->located[i] == 0) {
         if (pset->labels == NULL)
@@ -1316,6 +1722,22 @@ cs_probe_set_export_mesh(cs_probe_set_t   *pset,
 
   if (pset->flags & CS_PROBE_BOUNDARY) centers = mq->b_face_cog;
 
+  /* In case of clip to cell center:
+     - Remove redundant probes in case of fixed mesh
+     - save original coordinates in case of variable mesh */
+
+  if (   pset->snap_mode == CS_PROBE_SNAP_ELT_CENTER
+      && pset->p_define_func == NULL) {
+    if (! (pset->flags & CS_PROBE_TRANSIENT))
+      _merge_snapped_to_center(pset, centers);
+    else if (pset->coords_ini == NULL) {
+      BFT_MALLOC(pset->coords_ini, pset->n_probes, cs_real_3_t);
+      memcpy(pset->coords_ini,
+             pset->coords,
+             pset->n_probes*3*sizeof(cs_real_t));
+    }
+  }
+
   BFT_MALLOC(probe_coords, pset->n_loc_probes, cs_coord_3_t);
   BFT_MALLOC(global_num, pset->n_loc_probes, cs_gnum_t);
 
@@ -1346,8 +1768,8 @@ cs_probe_set_export_mesh(cs_probe_set_t   *pset,
 
   if (pset->snap_mode == CS_PROBE_SNAP_ELT_CENTER) {
     for (int i = 0; i < pset->n_loc_probes; i++) {
-      int j = pset->loc_id[i];
       if (pset->elt_id[i] > -1) {
+        const int j = pset->loc_id[i];
         const cs_real_t  *elt_coords = centers + pset->elt_id[i]*3;
         for (int k = 0; k < 3; k++)
           pset->coords[j][k] = elt_coords[k];
@@ -1356,8 +1778,8 @@ cs_probe_set_export_mesh(cs_probe_set_t   *pset,
   }
   else if (pset->snap_mode == CS_PROBE_SNAP_VERTEX) {
     for (int i = 0; i < pset->n_loc_probes; i++) {
-      int j = pset->loc_id[i];
       if (pset->vtx_id[i] > -1) {
+        const int j = pset->loc_id[i];
         const cs_real_t  *vtx_coords = m->vtx_coord + pset->vtx_id[i]*3;
         for (int k = 0; k < 3; k++)
           pset->coords[j][k] = vtx_coords[k];
@@ -1373,14 +1795,9 @@ cs_probe_set_export_mesh(cs_probe_set_t   *pset,
   /* Set a global numbering if needed */
 
   if (pset->p_define_func != NULL) {
-    cs_real_t *s;
-    BFT_MALLOC(s, pset->n_loc_probes, cs_real_t);
-    for (int i = 0; i < pset->n_loc_probes; i++) {
-      int j = pset->loc_id[i];
-      s[i] = pset->s_coords[j];
-    }
+    cs_real_t *s = cs_probe_set_get_loc_curvilinear_abscissa(pset);
     fvm_io_num_t *vtx_io_num
-      = fvm_io_num_create_from_real(pset->s_coords, pset->n_loc_probes);
+      = fvm_io_num_create_from_real(s, pset->n_loc_probes);
     BFT_FREE(s);
     fvm_nodal_transfer_vertex_io_num(exp_mesh, &vtx_io_num);
   }
@@ -1416,7 +1833,6 @@ cs_probe_set_export_mesh(cs_probe_set_t   *pset,
     }
     assert(j == ngl);
     fvm_nodal_transfer_global_vertex_labels(exp_mesh, g_labels);
-
   }
 
   /* probe_coords is managed by exp_mesh */
@@ -1534,7 +1950,8 @@ cs_probe_set_dump(const cs_probe_set_t   *pset)
     bft_printf(" selection:  %s\n", pset->sel_criter);
 
   bft_printf(" n_probes:   %d; %d; %d (locally located; defined; max.)\n",
-             pset->n_loc_probes, pset->n_probes, pset->n_max_probes);
+             (int)pset->n_loc_probes, (int)pset->n_probes,
+             (int)pset->n_max_probes);
 
   for (int i = 0; i < pset->n_probes; i++) {
 
@@ -1544,14 +1961,13 @@ cs_probe_set_dump(const cs_probe_set_t   *pset)
     if (pset->s_coords != NULL)
       bft_printf(" %5.3e |", pset->s_coords[i]);
     if (pset->elt_id != NULL && pset->located != NULL)
-      bft_printf(" %6d | %c |", pset->elt_id[i], pset->located[i]);
+      bft_printf(" %6d | %c |", (int)pset->elt_id[i], pset->located[i]);
     if (pset->labels != NULL)
       if (pset->labels[i] != NULL)
         bft_printf(" %s", pset->labels[i]);
     bft_printf("\n");
 
   }
-
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1628,6 +2044,35 @@ cs_probe_set_get_curvilinear_abscissa(const cs_probe_set_t   *pset)
     return pset->s_coords;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the list of curvilinear abscissa of probes located
+ *         on the local ranks for the given probe set
+ *
+ * The caller is responsible for freeing the returned array.
+ *
+ * \param[in]  pset              pointer to a cs_probe_set_t structure
+ *
+ * \return NULL or the pointer to the array of abscissa
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_real_t *
+cs_probe_set_get_loc_curvilinear_abscissa(const cs_probe_set_t   *pset)
+{
+  if (pset == NULL)
+    return NULL;
+
+  cs_real_t *s;
+  BFT_MALLOC(s, pset->n_loc_probes, cs_real_t);
+  for (int i = 0; i < pset->n_loc_probes; i++) {
+    int j = pset->loc_id[i];
+    s[i] = pset->s_coords[j];
+  }
+
+  return s;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Return the ids of a probe set's local matching elements, relative
