@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -92,11 +92,14 @@ typedef void
                                cs_real_t    **s);
 
 /*============================================================================
- *  Global variables
+ * Global variables
  *============================================================================*/
 
-/*=============================================================================
- * Public function prototypes for Fortran API
+/*============================================================================
+ * Semi-private function prototypes
+ *
+ * The following functions are intended to be used by the postprocessing layer
+ * (cs_post.c), not directly by the user.
  *============================================================================*/
 
 /*----------------------------------------------------------------------------*/
@@ -108,6 +111,29 @@ typedef void
 void
 cs_probe_finalize(void);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Transfer info on associated fields to the caller.
+ *
+ * This function transfert the property of the associated arrays to the caller
+ * and removes it from the probe set info.
+ *
+ * \param[in]   pset        pointer to a cs_probe_set_t structure
+ * \param[out]  n_fields    number of associated fields
+ * \param[out]  field_info  associated field info (array of tuples with
+ *                          writer id, field id, component id)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_probe_set_transfer_associated_field_info(cs_probe_set_t   *pset,
+                                            int              *n_fields,
+                                            int             **field_info);
+
+/*=============================================================================
+ * Public function prototypes.
+ *============================================================================*/
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Retrieve the number of probe sets defined.
@@ -169,6 +195,8 @@ cs_probe_set_get_name(cs_probe_set_t   *pset);
  * \param[out] on_boundary     true if probes are located on boundary
  * \param[out] on_curve        true if the probe set has cuvilinear coordinates
  * \param[out] auto_variables  true if set of variables to output is predefined
+ * \param[out] auto_curve_coo  true if curvilinear coordinates should be output
+ * \param[out] auto_cart_coo   true if cartesian coordinates should be output
  * \param[out] n_writers       number of associated  user-defined writers,
  *                             or -1 if default unchanged
  * \param[out] writer_ids      pointer to a list of writer ids
@@ -181,6 +209,8 @@ cs_probe_set_get_post_info(const cs_probe_set_t   *pset,
                            bool                   *on_boundary,
                            bool                   *on_curve,
                            bool                   *auto_variables,
+                           bool                   *auto_curve_coo,
+                           bool                   *auto_cart_coo,
                            int                    *n_writers,
                            int                    *writer_ids[]);
 
@@ -198,6 +228,21 @@ cs_probe_set_get_post_info(const cs_probe_set_t   *pset,
 const char *
 cs_probe_set_get_location_criteria(cs_probe_set_t   *pset);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the interpolation option for a given probe set
+ *
+ * Interpolation will be applied only where possible.
+ *
+ * \param[in]   pset       pointer to a cs_probe_set_t structure
+ *
+ * \return interpolation option value
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_probe_set_get_interpolation(cs_probe_set_t   *pset);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Create a new set of probes.
@@ -306,6 +351,31 @@ cs_probe_set_create_from_local(const char                   *name,
 void
 cs_probe_set_allow_overwrite(const char  *name);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Assign curvilinear abscissa for the given probe set
+ *
+ * This is effective only when using probe sets to which curvilinear
+ * coordinates have not already been assigned.
+ *
+ * So this may be used following \ref cs_probe_set_create (and probe additions)
+ * or \ref cs_probe_set_create_from_array, but will be ignored for probe sets
+ * defined using \ref cs_probe_set_create_from_segment and
+ * \ref cs_probe_set_create_from_local.
+ *
+ * If provided, the array of curvilinear absicssa must match the size of the
+ * probe set. If set to NULL, it is assumed a unifor spacing is provided
+ * between points.
+ *
+ * \param[in]  pset  pointer to a cs_probe_set_t structure
+ * \param[in]  s     array of curvilinear abscissa, or NULL
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_probe_set_assign_curvilinear_abscissa(cs_probe_set_t   *pset,
+                                         const cs_real_t  *s);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Associate a list of writers to a probe set.
@@ -323,7 +393,31 @@ cs_probe_set_associate_writers(cs_probe_set_t   *pset,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Set to true or false the automatic post-processing of variables
+ * \brief Associate a field to a probe set.
+ *
+ * This function must be called during the postprocessing output definition
+ * stage, before any output actually occurs.
+ *
+ * If the field should already be output automatically based on the mesh
+ * category and field output keywords, it will be filtered at a later stage.
+ *
+ * \param[in]  pset       pointer to a cs_probe_set_t structure
+ * \param[in]  writer_id  id of specified associated writer,
+ *                        or 0 (\ref CS_POST_WRITER_ALL_ASSOCIATED) for all
+ * \param[in]  field_id   id of field to attach
+ * \param[in]  comp_id    id of field component (-1 for all)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_probe_set_associate_field(cs_probe_set_t   *pset,
+                             int               writer_id,
+                             int               field_id,
+                             int               comp_id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set automatic output of variables behavior.
  *
  * \param[in, out] pset     pointer to a cs_probe_set_t structure
  * \param[in]      mode     true or false
@@ -334,6 +428,32 @@ void
 cs_probe_set_auto_var(cs_probe_set_t   *pset,
                       bool              mode);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set automatic output of curvilinear coordinates.
+ *
+ * \param[in, out] pset     pointer to a cs_probe_set_t structure
+ * \param[in]      mode     true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_probe_set_auto_curvilinear_coords(cs_probe_set_t   *pset,
+                                     bool              mode);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set automatic output of curvilinear coordinates.
+ *
+ * \param[in, out] pset     pointer to a cs_probe_set_t structure
+ * \param[in]      mode     true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_probe_set_auto_cartesian_coords(cs_probe_set_t   *pset,
+                                   bool              mode);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Set snap mode related to the management of a set of probes.
@@ -362,6 +482,10 @@ cs_probe_set_snap_mode(cs_probe_set_t   *pset,
  *
  * - \c \b selection_criteria where keyval is selection criteria string
  * - \c \b tolerance  where keyval is for instance "0.05" (default "0.10")
+ * - \c \b interpolation if \ c 0, P0 interpolation (default); if \c 1,
+ *         simple gradient-based interpolation for volume probes (ignored
+ *         for boundaries). Other interpolation options might be added
+ *         in the future.
  *
  * \param[in, out] pset     pointer to a cs_probe_set_t structure to set
  * \param[in]      keyname  name of the keyword related to the parameter to set
@@ -478,6 +602,22 @@ cs_probe_set_get_n_local(const cs_probe_set_t   *pset);
 const cs_real_t *
 cs_probe_set_get_curvilinear_abscissa(const cs_probe_set_t   *pset);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the list of curvilinear abscissa of probes located
+ *         on the local ranks for the given probe set
+ *
+ * The caller is responsible for freeing the returned array.
+ *
+ * \param[in]  pset              pointer to a cs_probe_set_t structure
+ *
+ * \return NULL or the pointer to the array of abscissa
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_real_t *
+cs_probe_set_get_loc_curvilinear_abscissa(const cs_probe_set_t   *pset);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Return the ids of a probe set's local matching elements, relative
