@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -223,8 +223,8 @@ _init_wt_limit(void)
 
 void CS_PROCF (armtps, ARMTPS)
 (
- const cs_int_t  *ntcabs,
-       cs_int_t  *ntmabs
+ const int  *ntcabs,
+ int        *ntmabs
 )
 {
   int ts_max = *ntmabs;
@@ -359,7 +359,7 @@ cs_resource_get_max_timestep(int   ts_cur,
              "===========================================================\n"),
            *ts_max);
 
-        FILE *_f = fopen("status.exceeded_time_limit", "w");
+        FILE *_f = fopen("run_status.exceeded_time_limit", "w");
         if (_f != NULL) {
           fprintf(_f, "%d\n", ts_cur);
           fclose(_f);
@@ -373,7 +373,7 @@ cs_resource_get_max_timestep(int   ts_cur,
 
 #if defined(HAVE_MPI)
     if (cs_glob_n_ranks > 1)
-      MPI_Bcast(ts_max, 1, CS_MPI_INT, 0, cs_glob_mpi_comm);
+      MPI_Bcast(ts_max, 1, MPI_INT, 0, cs_glob_mpi_comm);
 #endif
 
   }
