@@ -1,11 +1,11 @@
 /*============================================================================
- * Syrthes 4 coupling
+ * Syrthes coupling
  *============================================================================*/
 
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -132,23 +132,25 @@ struct _cs_syr4_coupling_t {
 
   /* Mesh-related members */
 
-  int                      dim;            /* Coupled mesh dimension */
-  int                      ref_axis;       /* Selected axis for edge extraction */
+  int                      dim;             /* Coupled mesh dimension */
+  int                      ref_axis;        /* Axis for edge extraction */
 
-  char                    *syr_name;       /* Application name, or -1 */
+  char                    *syr_name;        /* Application name */
 
-  char                    *face_sel;       /* Face selection criteria */
-  char                    *cell_sel;       /* Face selection criteria */
+  int                      n_b_locations;   /* Numbero of boundary locations */
+  int                      n_v_locations;   /* Numbero of volume locations */
+  int                     *b_location_ids;  /* Boundary location ids */
+  int                     *v_location_ids;  /* Volume location ids */
 
-  cs_syr4_coupling_ent_t  *faces;          /* Wall coupling structure */
-  cs_syr4_coupling_ent_t  *cells;          /* Volume coupling structure */
+  cs_syr4_coupling_ent_t  *faces;           /* Wall coupling structure */
+  cs_syr4_coupling_ent_t  *cells;           /* Volume coupling structure */
 
-  bool                     allow_nearest;  /* Allow nearest-neighbor
-                                              mapping beyond basic matching
-                                              tolerance */
-  float                    tolerance;      /* Tolerance */
-  int                      verbosity;      /* Verbosity level */
-  int                      visualization;  /* Visualization output flag */
+  bool                     allow_nearest;   /* Allow nearest-neighbor
+                                               mapping beyond basic matching
+                                               tolerance */
+  float                    tolerance;       /* Tolerance */
+  int                      verbosity;       /* Verbosity level */
+  int                      visualization;   /* Visualization output flag */
 
   /* Communication-related members */
 
@@ -366,9 +368,9 @@ _cs_syr4_coupling_post_function(void                  *coupling,
                             false,
                             false,
                             CS_POST_TYPE_float,
-                            NULL,
-                            NULL,
                             coupling_ent->flux,
+                            NULL,
+                            NULL,
                             ts);
 
       }
@@ -508,7 +510,8 @@ _is_location_complete(cs_syr4_coupling_t      *syr_coupling,
  *
  * parameters:
  *   syr_coupling    <-- partially initialized SYRTHES coupling structure
- *   select_criteria <-- selection criteria
+ *   n_locations     <-- number of associated locations
+ *   location_ids    <-- associated location ids
  *   elt_dim         <-- element dimension
  *
  * returns:
@@ -517,14 +520,14 @@ _is_location_complete(cs_syr4_coupling_t      *syr_coupling,
 
 static cs_syr4_coupling_ent_t *
 _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
-                    const char          *select_criteria,
+                    int                  n_locations,
+                    int                  location_ids[],
                     int                  elt_dim)
 {
   char *coupled_mesh_name = NULL;
   bool      ext_syr = false;
   cs_lnum_t n_exterior = 0;
   cs_gnum_t n_ext = 0;
-  cs_lnum_t *elt_list = NULL;
   cs_coord_t *elt_centers = NULL;
   fvm_nodal_t *location_elts = NULL;
   float *cs_to_syr_dist = NULL;
@@ -560,6 +563,31 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
     bft_printf_flush();
   }
 
+  /* Select elements */
+
+  cs_lnum_t  n_elts = 0;
+  cs_lnum_t *elt_list = NULL;
+
+  for (int l_i = 0; l_i < n_locations; l_i++)
+    n_elts += cs_mesh_location_get_n_elts(location_ids[l_i])[0];
+
+  BFT_MALLOC(elt_list, n_elts, cs_lnum_t);
+
+  n_elts = 0;
+  for (int l_i = 0; l_i < n_locations; l_i++) {
+    int loc_id = location_ids[l_i];
+    const cs_lnum_t n = cs_mesh_location_get_n_elts(loc_id)[0];
+    const cs_lnum_t *ids = cs_mesh_location_get_elt_list(loc_id);
+    if (ids != NULL) {
+      for (cs_lnum_t i = 0; i < n; i++)
+        elt_list[n_elts++] = ids[i] + 1;
+    }
+    else {
+      for (cs_lnum_t i = 0; i < n; i++)
+        elt_list[n_elts++] = i + 1;
+    }
+  }
+
   /* Creation of a new nodal mesh from selected cells */
 
   if (elt_dim == syr_coupling->dim) {
@@ -569,11 +597,7 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                + strlen(syr_coupling->syr_name) + 1, char);
     sprintf(coupled_mesh_name, _("SYRTHES %s cells"), syr_coupling->syr_name);
 
-    BFT_MALLOC(elt_list, cs_glob_mesh->n_cells, cs_lnum_t);
-
-    cs_selector_get_cell_num_list(select_criteria,
-                                  &(coupling_ent->n_elts),
-                                  elt_list);
+    coupling_ent->n_elts = n_elts;
 
     coupling_ent->elts
       = cs_mesh_connect_cells_to_nodal(cs_glob_mesh,
@@ -582,8 +606,6 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                                        coupling_ent->n_elts,
                                        elt_list);
 
-    BFT_FREE(elt_list);
-
     /* Allocate additional buffers */
 
     BFT_MALLOC(coupling_ent->hvol, coupling_ent->n_elts, double);
@@ -600,11 +622,7 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                char);
     sprintf(coupled_mesh_name, _("SYRTHES %s faces"), syr_coupling->syr_name);
 
-    BFT_MALLOC(elt_list, cs_glob_mesh->n_b_faces, cs_lnum_t);
-
-    cs_selector_get_b_face_num_list(select_criteria,
-                                    &(coupling_ent->n_elts),
-                                    elt_list);
+    coupling_ent->n_elts = n_elts;
 
     coupling_ent->elts
       = cs_mesh_connect_faces_to_nodal(cs_glob_mesh,
@@ -615,10 +633,10 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                                        NULL,
                                        elt_list);
 
-    BFT_FREE(elt_list);
-
   }
 
+  BFT_FREE(elt_list);
+
   BFT_FREE(coupled_mesh_name);
 
   if (syr_coupling->verbosity > 0) {
@@ -628,10 +646,10 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
 
   if (fvm_nodal_get_n_g_vertices(coupling_ent->elts) == 0)
     bft_error(__FILE__, __LINE__, 0,
-              _(" Selection criteria:\n"
-                " \"%s\"\n"
-                " leads to an empty mesh for SYRTHES coupling.\n"),
-              select_criteria);
+              _(" Selected mesh locations:\n"
+                " lead to an empty mesh for SYRTHES coupling .\n"
+                " \"%s\"\n"),
+              syr_coupling->syr_name);
 
   /* In case of 2D coupling, project coupled elements to 2D */
 
@@ -809,6 +827,13 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
     cs_post_activate_writer(-1, 1);
     cs_post_write_meshes(cs_glob_time_step);
 
+    const cs_real_t *b_dist = NULL, *v_dist = NULL;
+
+    if (coupling_ent->elt_dim == syr_coupling->dim - 1)
+      b_dist = cs_to_syr_dist;
+    else if (coupling_ent->elt_dim == syr_coupling->dim)
+      v_dist = cs_to_syr_dist;
+
     cs_post_write_var(coupling_ent->post_mesh_id,
                       CS_POST_WRITER_ALL_ASSOCIATED,
                       _("distance_to_solid"),
@@ -816,9 +841,9 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                       false,
                       false, /* use_parent, */
                       CS_POST_TYPE_float,
+                      v_dist,
                       NULL,
-                      NULL,
-                      cs_to_syr_dist,
+                      b_dist,
                       NULL);  /* time-independent variable */
 
     BFT_FREE(cs_to_syr_dist);
@@ -1006,7 +1031,7 @@ _all_comm_times(void)
   cs_log_separator(CS_LOG_PERFORMANCE);
 
   cs_log_printf(CS_LOG_PERFORMANCE,
-                _("\nSYRTHES 4 coupling overheads\n"));
+                _("\nSYRTHES coupling overheads\n"));
 
   for (coupl_id = 0; coupl_id < cs_glob_syr4_n_couplings; coupl_id++) {
 
@@ -1030,7 +1055,7 @@ _all_comm_times(void)
         else
           cs_log_printf(CS_LOG_PERFORMANCE,
                         _("\n  coupling %d (%s):\n\n"),
-                        coupl_id + 1, _(ent_type[ent_id]));
+                        coupl_id, _(ent_type[ent_id]));
 
         ple_locator_get_times(ce->locator,
                               &location_wtime,
@@ -1157,13 +1182,13 @@ _post_var_update(cs_syr4_coupling_ent_t  *coupling_ent,
  * Code_Saturne sent before its computed global flux for this time step.
  *
  * parameters:
- *   syr_coupling    <-- SYRTHES coupling structure
- *   coupl_face_list <-- list of coupled boundary faces
+ *   syr_coupling   <-- SYRTHES coupling structure
+ *   coupl_face_ids <-- ids of coupled boundary faces
  *----------------------------------------------------------------------------*/
 
 static void
 _ensure_conservativity(cs_syr4_coupling_t   *syr_coupling,
-                       const cs_lnum_t       coupl_face_list[])
+                       const cs_lnum_t       coupl_face_ids[])
 {
   cs_lnum_t ii, face_id;
 
@@ -1175,7 +1200,7 @@ _ensure_conservativity(cs_syr4_coupling_t   *syr_coupling,
   /* Sanity checks */
 
   assert(surf != NULL);
-  assert(coupl_face_list != NULL);
+  assert(coupl_face_ids != NULL);
   assert(syr_coupling != NULL);
   coupling_ent = syr_coupling->faces;
   assert(coupling_ent != NULL);
@@ -1183,7 +1208,7 @@ _ensure_conservativity(cs_syr4_coupling_t   *syr_coupling,
   /* Compute Code_Saturne's global flux */
 
   for (ii = 0; ii < coupling_ent->n_elts; ii++) {
-    face_id = coupl_face_list[ii] - 1;
+    face_id = coupl_face_ids[ii];
     _flux += coupling_ent->flux[ii] * surf[face_id];
   }
 
@@ -1210,7 +1235,7 @@ _ensure_conservativity(cs_syr4_coupling_t   *syr_coupling,
              cs_syr4_coupling_tag,
              syr_coupling->comm);
 
-    if (syr_coupling->verbosity > 0)
+    if (syr_coupling->verbosity > 1)
       bft_printf(_(" Global heat flux exchanged with SYRTHES in W: %5.3e\n"),
                  g_flux);
 
@@ -1227,7 +1252,7 @@ _ensure_conservativity(cs_syr4_coupling_t   *syr_coupling,
 
   /* Print message */
 
-  if (syr_coupling->verbosity > 0)
+  if (syr_coupling->verbosity > 1)
     bft_printf(_(" Correction coefficient used to force conservativity during"
                  " coupling with SYRTHES: %5.3e\n"), coef);
 }
@@ -1296,7 +1321,7 @@ _sync_after_location(cs_syr4_coupling_t  *syr_coupling)
  *   number of SYRTHES couplings
  *----------------------------------------------------------------------------*/
 
-cs_lnum_t
+int
 cs_syr4_coupling_n_couplings(void)
 {
   return cs_glob_syr4_n_couplings;
@@ -1313,7 +1338,7 @@ cs_syr4_coupling_n_couplings(void)
  *----------------------------------------------------------------------------*/
 
 cs_syr4_coupling_t *
-cs_syr4_coupling_by_id(cs_lnum_t coupling_id)
+cs_syr4_coupling_by_id(int  coupling_id)
 {
   cs_syr4_coupling_t  *retval = NULL;
 
@@ -1325,13 +1350,13 @@ cs_syr4_coupling_by_id(cs_lnum_t coupling_id)
 }
 
 /*----------------------------------------------------------------------------
- * Create a syr4_coupling_t structure.
+ * Create or redefine a syr4_coupling_t structure.
+ *
+ * If a structure is redefined, associated locations are reset.
  *
  * parameters:
  *   dim                <-- spatial mesh dimension
  *   ref_axis           <-- reference axis
- *   face_sel_criterion <-- criterion for selection of boundary faces
- *   cell_sel_criterion <-- criterion for selection of cells
  *   syr_name           <-- SYRTHES application name
  *   allow_nonmatching  <-- nearest-neighbor search for non-matching faces flag
  *   tolerance          <-- addition to local extents of each element
@@ -1340,24 +1365,46 @@ cs_syr4_coupling_by_id(cs_lnum_t coupling_id)
  *   visualization      <-- visualization output flag
  *----------------------------------------------------------------------------*/
 
-void
-cs_syr4_coupling_add(cs_lnum_t    dim,
-                     cs_lnum_t    ref_axis,
-                     const char  *face_sel_criterion,
-                     const char  *cell_sel_criterion,
-                     const char  *syr_name,
-                     bool         allow_nonmatching,
-                     float        tolerance,
-                     int          verbosity,
-                     int          visualization)
+cs_syr4_coupling_t *
+cs_syr4_coupling_define(int          dim,
+                        int          ref_axis,
+                        const char  *syr_name,
+                        bool         allow_nonmatching,
+                        float        tolerance,
+                        int          verbosity,
+                        int          visualization)
 {
   cs_syr4_coupling_t *syr_coupling = NULL;
 
+  /* Search in existing couplings */
+
+  for (int i = 0;
+       i < cs_glob_syr4_n_couplings && syr_coupling == NULL;
+       i++) {
+
+    if (strcmp(cs_glob_syr4_couplings[i]->syr_name, syr_name) == 0) {
+      syr_coupling = cs_glob_syr4_couplings[i];
+
+      BFT_FREE(syr_coupling->syr_name);
+      BFT_FREE(syr_coupling->b_location_ids);
+      BFT_FREE(syr_coupling->v_location_ids);
+
+      assert(syr_coupling->faces == NULL);  /* not built yet at this stage */
+      assert(syr_coupling->cells == NULL);
+    }
+  }
+
   /* Allocate _cs_syr4_coupling_t structure */
 
-  BFT_REALLOC(cs_glob_syr4_couplings,
-              cs_glob_syr4_n_couplings + 1, cs_syr4_coupling_t *);
-  BFT_MALLOC(syr_coupling, 1, cs_syr4_coupling_t);
+  if (syr_coupling == NULL) {
+    BFT_REALLOC(cs_glob_syr4_couplings,
+                cs_glob_syr4_n_couplings + 1, cs_syr4_coupling_t *);
+    BFT_MALLOC(syr_coupling, 1, cs_syr4_coupling_t);
+
+    cs_glob_syr4_couplings[cs_glob_syr4_n_couplings] = syr_coupling;
+
+    cs_glob_syr4_n_couplings++;
+  }
 
   syr_coupling->dim = dim;
   syr_coupling->ref_axis = ref_axis;
@@ -1375,23 +1422,10 @@ cs_syr4_coupling_add(cs_lnum_t    dim,
 
   /* Selection criteria  */
 
-  syr_coupling->face_sel = NULL;
-  syr_coupling->cell_sel = NULL;
-
-  if (face_sel_criterion != NULL) {
-    BFT_MALLOC(syr_coupling->face_sel, strlen(face_sel_criterion) + 1, char);
-    strcpy(syr_coupling->face_sel, face_sel_criterion);
-  }
-  if (cell_sel_criterion != NULL) {
-    BFT_MALLOC(syr_coupling->cell_sel, strlen(cell_sel_criterion) + 1, char);
-    strcpy(syr_coupling->cell_sel, cell_sel_criterion);
-  }
-
-  if (face_sel_criterion == NULL && cell_sel_criterion == NULL)
-    bft_error(__FILE__, __LINE__, 0,
-              _("Coupling with SYRTHES impossible.\n"
-                "No selection criteria for faces or cells to couple."));
-
+  syr_coupling->n_b_locations = 0;
+  syr_coupling->n_v_locations = 0;
+  syr_coupling->b_location_ids = NULL;
+  syr_coupling->v_location_ids = NULL;
 
   syr_coupling->faces = NULL;
   syr_coupling->cells = NULL;
@@ -1411,10 +1445,38 @@ cs_syr4_coupling_add(cs_lnum_t    dim,
 
 #endif
 
-  /* Update coupling array and return */
+  return  syr_coupling;
+}
 
-  cs_glob_syr4_couplings[cs_glob_syr4_n_couplings] = syr_coupling;
-  cs_glob_syr4_n_couplings++;
+/*----------------------------------------------------------------------------
+ * Add a mesh location to a syr4_coupling_t structure.
+ *
+ * parameters:
+ *   syr_coupling  <-- SYRTHES coupling structure
+ *   location_id   <-- id of mesh location to add (boundary faces or cells)
+ *----------------------------------------------------------------------------*/
+
+void
+cs_syr4_coupling_add_location(cs_syr4_coupling_t  *syr_coupling,
+                              int                  location_id)
+{
+  cs_mesh_location_type_t l_type = cs_mesh_location_get_type(location_id);
+
+  if (l_type == CS_MESH_LOCATION_BOUNDARY_FACES) {
+    int i = syr_coupling->n_b_locations;
+    syr_coupling->n_b_locations += 1;
+    BFT_REALLOC(syr_coupling->b_location_ids, syr_coupling->n_b_locations, int);
+
+    syr_coupling->b_location_ids[i] = location_id;
+  }
+
+  else if (l_type == CS_MESH_LOCATION_CELLS) {
+    int i = syr_coupling->n_v_locations;
+    syr_coupling->n_v_locations += 1;
+    BFT_REALLOC(syr_coupling->v_location_ids, syr_coupling->n_v_locations, int);
+
+    syr_coupling->v_location_ids[i] = location_id;
+  }
 }
 
 /*----------------------------------------------------------------------------
@@ -1438,13 +1500,9 @@ cs_syr4_coupling_all_destroy(void)
 
     /* Free _cs_syr4_coupling structure */
 
-    if (syr_coupling->syr_name != NULL)
-      BFT_FREE(syr_coupling->syr_name);
-
-    if (syr_coupling->face_sel != NULL)
-      BFT_FREE(syr_coupling->face_sel);
-    if (syr_coupling->cell_sel != NULL)
-      BFT_FREE(syr_coupling->cell_sel);
+    BFT_FREE(syr_coupling->syr_name);
+    BFT_FREE(syr_coupling->b_location_ids);
+    BFT_FREE(syr_coupling->v_location_ids);
 
     if (syr_coupling->faces != NULL)
       _destroy_coupled_ent(&(syr_coupling->faces));
@@ -1462,10 +1520,31 @@ cs_syr4_coupling_all_destroy(void)
   cs_glob_syr4_n_couplings = 0;
   BFT_FREE(cs_glob_syr4_couplings);
 
-  bft_printf(_("\nStructures associated with SYRTHES 4 coupling freed.\n"));
+  bft_printf(_("\nStructures associated with SYRTHES coupling freed.\n"));
   bft_printf_flush();
 }
 
+/*----------------------------------------------------------------------------
+ * Get name of SYRTHES coupling.
+ *
+ * parameters:
+ *   syr_coupling <-- SYRTHES coupling structure
+ *
+ * returns:
+ *   pointer to SYRTHES coupling name
+ *----------------------------------------------------------------------------*/
+
+const char *
+cs_syr4_coupling_get_name(cs_syr4_coupling_t  *syr_coupling)
+{
+  const char *retval = cs_empty_string;
+
+  if (syr_coupling->syr_name != NULL)
+    retval = syr_coupling->syr_name;
+
+  return retval;
+}
+
 /*----------------------------------------------------------------------------
  * Set conservativity forcing flag to True (1) or False (0) for all defined
  * SYRTHES couplings
@@ -1522,9 +1601,9 @@ cs_syr4_coupling_init_comm(cs_syr4_coupling_t *syr_coupling,
 
   /* Exchange coupling options */
 
-  if (syr_coupling->face_sel != NULL)
+  if (syr_coupling->n_b_locations > 0)
     boundary_flag = 'b';
-  if (syr_coupling->cell_sel != NULL)
+  if (syr_coupling->n_v_locations > 0)
     volume_flag = 'v';
   if (cs_syr4_coupling_conservativity == 0)
     conservativity_flag = '0';
@@ -1576,16 +1655,18 @@ cs_syr4_coupling_init_mesh(cs_syr4_coupling_t  *syr_coupling)
 
   int match_flag = 0;
 
-  if (syr_coupling->face_sel != NULL) {
+  if (syr_coupling->n_b_locations > 0) {
     syr_coupling->faces = _create_coupled_ent(syr_coupling,
-                                              syr_coupling->face_sel,
+                                              syr_coupling->n_b_locations,
+                                              syr_coupling->b_location_ids,
                                               syr_coupling->dim - 1);
     match_flag += _sync_after_location(syr_coupling);
   }
 
-  if (syr_coupling->cell_sel != NULL) {
+  if (syr_coupling->n_v_locations > 0) {
     syr_coupling->cells = _create_coupled_ent(syr_coupling,
-                                              syr_coupling->cell_sel,
+                                              syr_coupling->n_v_locations,
+                                              syr_coupling->v_location_ids,
                                               syr_coupling->dim);
     match_flag += _sync_after_location(syr_coupling);
   }
@@ -1617,7 +1698,7 @@ cs_syr4_coupling_is_surf(const cs_syr4_coupling_t  *syr_coupling)
 
   assert(syr_coupling != NULL);
 
-  if (syr_coupling->faces != NULL)
+  if (syr_coupling->n_b_locations > 0)
     retval = 1;
 
   return retval;
@@ -1640,7 +1721,7 @@ cs_syr4_coupling_is_vol(const cs_syr4_coupling_t  *syr_coupling)
 
   assert(syr_coupling != NULL);
 
-  if (syr_coupling->cells != NULL)
+  if (syr_coupling->n_v_locations > 0)
     retval = 1;
 
   return retval;
@@ -1685,20 +1766,19 @@ cs_syr4_coupling_get_n_elts(const cs_syr4_coupling_t *syr_coupling,
  *
  * parameters:
  *   syr_coupling  <-- SYRTHES coupling structure
- *   cpl_elt_lst   --> List of coupled elements (1 to n)
+ *   cpl_elt_ids   --> Ids of coupled elements (0 to n-1)
  *   mode          <-- 0 (surface); 1 (volume)
  *----------------------------------------------------------------------------*/
 
 void
-cs_syr4_coupling_get_elt_list(const cs_syr4_coupling_t  *syr_coupling,
-                              cs_int_t                   cpl_elt_lst[],
-                              int                        mode)
+cs_syr4_coupling_get_elt_ids(const cs_syr4_coupling_t  *syr_coupling,
+                             cs_lnum_t                  cpl_elt_ids[],
+                             int                        mode)
 {
   cs_syr4_coupling_ent_t  *coupling_ent = NULL;
 
   /* Sanity checks */
 
-  assert(sizeof(cs_lnum_t) == sizeof(cs_int_t));
   assert(syr_coupling != NULL);
   assert(mode == 0 || mode == 1);
 
@@ -1708,9 +1788,9 @@ cs_syr4_coupling_get_elt_list(const cs_syr4_coupling_t  *syr_coupling,
     coupling_ent = syr_coupling->cells;
 
   if (coupling_ent != NULL)
-    fvm_nodal_get_parent_num(coupling_ent->elts,
-                             coupling_ent->elt_dim,
-                             cpl_elt_lst);
+    fvm_nodal_get_parent_id(coupling_ent->elts,
+                            coupling_ent->elt_dim,
+                            cpl_elt_ids);
 }
 
 /*----------------------------------------------------------------------------
@@ -1766,19 +1846,19 @@ cs_syr4_coupling_recv_tsolid(cs_syr4_coupling_t  *syr_coupling,
  * Send coupling variables to SYRTHES
  *
  * parameters:
- *   syr_coupling    <-- SYRTHES coupling structure
- *   coupl_face_list <-- list of coupled boundary faces
- *   tf              <-- fluid temperature
- *   hf              <-- fluid heat exchange coef. (numerical or user-defined)
- *   mode            <-- 0: surface coupling; 1: volume coupling
+ *   syr_coupling <-- SYRTHES coupling structure
+ *   cpl_elt_ids  <-- ids of coupled elements
+ *   tf           <-- fluid temperature
+ *   hf           <-- fluid heat exchange coef. (numerical or user-defined)
+ *   mode          <-- 0: surface coupling; 1: volume coupling
  *----------------------------------------------------------------------------*/
 
 void
 cs_syr4_coupling_send_tf_hf(cs_syr4_coupling_t  *syr_coupling,
-                            const cs_lnum_t      cpl_elt_lst[],
+                            const cs_lnum_t      cpl_elt_ids[],
                             cs_real_t            tf[],
                             cs_real_t            hf[],
-                            cs_int_t             mode)
+                            int                  mode)
 {
   cs_lnum_t ii;
 
@@ -1837,7 +1917,7 @@ cs_syr4_coupling_send_tf_hf(cs_syr4_coupling_t  *syr_coupling,
   /* Exchange flux and corrector coefficient to ensure conservativity */
 
   if (cs_syr4_coupling_conservativity > 0 && mode == 0)
-    _ensure_conservativity(syr_coupling, cpl_elt_lst);
+    _ensure_conservativity(syr_coupling, cpl_elt_ids);
 }
 
 /*----------------------------------------------------------------------------
@@ -1872,8 +1952,8 @@ cs_syr4_coupling_ts_contrib(const cs_syr4_coupling_t  *syr_coupling,
   hvol = ent->hvol;
   solid_temp = ent->solid_temp;
 
-  assert(hvol != NULL);
-  assert(solid_temp != NULL);
+  assert(hvol != NULL || ent->n_elts == 0);
+  assert(solid_temp != NULL || ent->n_elts == 0);
 
   /* Compute contribution */
 
