@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2015 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -30,6 +30,7 @@
  * Standard C library headers
  *----------------------------------------------------------------------------*/
 
+#include <assert.h>
 #include <string.h>
 #include <time.h>
 
@@ -53,8 +54,8 @@
 #include <pwd.h>
 #endif
 
-#if defined(__bgq__) && defined(__xlc__)
-#include <spi/include/kernel/location.h>
+#if defined(HAVE_CUDA)
+#include "cs_base_cuda.h"
 #endif
 
 /*----------------------------------------------------------------------------
@@ -88,6 +89,38 @@ BEGIN_C_DECLS
  * Private function definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Remove leading and trailing whitespace from a string.
+ *
+ * parameters:
+ *   s <-> string to be cleaned
+ *----------------------------------------------------------------------------*/
+
+static void
+_string_clean(char  *s)
+{
+  assert(s != NULL);
+
+  int l = strlen(s);
+  int i = l - 1;
+  while (i > -1 && (   s[i] == ' ' || s[i] == '\t'
+                    || s[i] == '\n' || s[i] == '\r')) {
+    s[i--] = '\0';
+  }
+
+  i = 0;
+  while (i < l && (   s[i] == ' ' || s[i] == '\t'
+                   || s[i] == '\n' || s[i] == '\r')) {
+    i++;
+  }
+
+  if (i > 0) {
+    int j = 0;
+    while (i <= l)
+      s[j++] = s[i++];
+  }
+}
+
 /*----------------------------------------------------------------------------
  * Return basic available CPU info depending on system.
  *
@@ -102,7 +135,7 @@ _sys_info_cpu(char      *cpu_str,
 {
   strcpy(cpu_str, "");
 
-#if defined(__linux__)
+#if defined(__linux__) && !defined(__ve__)
 
   {
     unsigned _cpu_str_max = (cpu_str_max > 0) ? cpu_str_max - 1 : 0;
@@ -147,32 +180,128 @@ _sys_info_cpu(char      *cpu_str,
 #endif /* HAVE_SYS_UTSNAME_H */
 }
 
-/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
-
-/*============================================================================
- * Public function definitions
- *============================================================================*/
-
 /*----------------------------------------------------------------------------
- * Print available system information.
+ * Return Linux info based on /etc/issue.
+ *
+ * Only the information prior to a first escape sequence is returned.
+ *
+ * parameters:
+ *   issue_str     --> string with system description
+ *   issue_str_max <-- maximum length of string with system description
  *----------------------------------------------------------------------------*/
 
+static void
+_sys_info_issue(char      *issue_str,
+                unsigned   issue_str_max)
+{
+  issue_str[0] = '\0';
+
+#if defined(__linux__)
+
+  {
+    unsigned _issue_str_max = (issue_str_max > 0) ? issue_str_max - 1 : 0;
+
+    FILE *fp;
+    char *s;
+
+    fp = fopen("/etc/issue", "r");
+
+    if (fp != NULL) {
+
+      issue_str[0] = ' ';
+      issue_str[1] = '(';
+
+      s = fgets(issue_str + 2, _issue_str_max - 4, fp);
+
+      if (s != NULL) {
+        int l = strlen(s);
+        for (int i = 0; i < l; i++)
+          if (s[i] == '\\') {
+            s[i] = '\0';
+            l = i;
+          }
+        _string_clean(issue_str + 2);
+        l = strlen(issue_str);
+        if (l > 2) {
+          issue_str[l] = ')';
+          issue_str[l+1] = '\0';
+        }
+        else /* If no info was kept, empty string */
+          issue_str[0] = '\0';
+      }
+
+
+      fclose (fp);
+    }
+
+  }
+
+#endif
+}
+
 #if defined(HAVE_MPI)
 
-void
-cs_system_info(MPI_Comm comm)
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Determine min an max number of ranks per node
+ *
+ * \param[in]  comm            associated MPI communicator
+ * \param[in]  ranks_per_node  number of ranks per node (min and max)
+ */
+/*----------------------------------------------------------------------------*/
+
+static void
+_mpi_ranks_per_node(MPI_Comm  comm,
+                    int       ranks_per_node[2])
+{
+#if (MPI_VERSION < 3)
+  ranks_per_node[0] = -1;
+  ranks_per_node[1] = -1;
+#else
+
+  MPI_Comm sh_comm;
+  MPI_Comm_split_type(comm, MPI_COMM_TYPE_SHARED, 0,
+                      MPI_INFO_NULL, &sh_comm);
+  int sh_ranks;
+  MPI_Comm_rank(sh_comm, &sh_ranks);
+  sh_ranks += 1;
+  MPI_Allreduce(MPI_IN_PLACE, &sh_ranks, 1, MPI_INT, MPI_MAX, sh_comm);
+  MPI_Comm_free(&sh_comm);
+
+  MPI_Allreduce(&sh_ranks, ranks_per_node, 1, MPI_INT, MPI_MIN, comm);
+  MPI_Allreduce(&sh_ranks, ranks_per_node+1, 1, MPI_INT, MPI_MAX, comm);
+
+#endif
+}
+
+#endif /* defined(HAVE_MPI) */
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print available system information.
+ *
+ * \param[in]  comm  associated MPI communicator
+ * \param[in]  log   if true, standard logging; otherwise, single output
+ */
+/*----------------------------------------------------------------------------*/
+
+#if defined(HAVE_MPI)
+
+static void
+_system_info(MPI_Comm comm,
+             bool     log)
 
 #else
 
-void
-cs_system_info(void)
+static void
+_system_info(bool  log)
 
 #endif
 
 {
   time_t          date;
   size_t          ram;
-  int             log_id;
+  int             n_logs = (log) ? 2 : 1;
 
   cs_log_t logs[] = {CS_LOG_DEFAULT, CS_LOG_PERFORMANCE};
 
@@ -189,17 +318,13 @@ cs_system_info(void)
 
   char  str_date[81];
   char  str_cpu[81];
+  char  str_issue[81];
   char  str_directory[PATH_MAX] = "";
 
 # if defined(HAVE_MPI)
   int mpi_flag = 0;
 #endif
 
-#if defined(__bgq__) && defined(__xlc__)
-  Personality_t personality;
-  Kernel_GetPersonality(&personality, sizeof(personality));
-#endif
-
   /* Date */
 
   if (   time(&date) == -1
@@ -216,23 +341,28 @@ cs_system_info(void)
   /* Print local configuration */
   /*---------------------------*/
 
-  for (log_id = 0; log_id < 2; log_id++)
-    cs_log_printf(logs[log_id],
-                  "\n%s\n", _("Local case configuration:\n"));
+  if (log) {
+    for (int log_id = 0; log_id < n_logs; log_id++)
+      cs_log_printf(logs[log_id],
+                    "\n%s\n", _("Local case configuration:\n"));
+  }
 
-  for (log_id = 0; log_id < 2; log_id++)
+  for (int log_id = 0; log_id < n_logs; log_id++)
     cs_log_printf(logs[log_id],
                   "  %s%s\n", _("Date:                "), str_date);
 
   /* System and machine */
 
+  _sys_info_issue(str_issue, 81);
+
 #if defined(HAVE_UNAME)
 
   if (uname(&sys_config) != -1) {
-    for (log_id = 0; log_id < 2; log_id++) {
+    for (int log_id = 0; log_id < n_logs; log_id++) {
       cs_log_printf(logs[log_id],
-                    "  %s%s %s\n", _("System:              "),
-                    sys_config.sysname, sys_config.release);
+                    "  %s%s %s%s\n", _("System:              "),
+                    sys_config.sysname, sys_config.release,
+                    str_issue);
       cs_log_printf(logs[log_id],
                     "  %s%s\n", _("Machine:             "),
                     sys_config.nodename);
@@ -242,15 +372,13 @@ cs_system_info(void)
 
   _sys_info_cpu(str_cpu, 81);
 
-  for (log_id = 0; log_id < 2; log_id++)
+  for (int log_id = 0; log_id < n_logs; log_id++)
     cs_log_printf(logs[log_id],
                   "  %s%s\n", _("Processor:           "), str_cpu);
 
   /* Available memory */
 
-#if defined(__bgq__) && defined(__xlc__)
-  ram = personality.DDR_Config.DDRSizeMB;
-#elif defined(__linux__) \
+#if defined(__linux__) \
    && defined(HAVE_SYS_SYSINFO_H) && defined(HAVE_SYSINFO)
   {
     struct sysinfo info;
@@ -262,7 +390,7 @@ cs_system_info(void)
 #endif
 
   if (ram > 0) {
-    for (log_id = 0; log_id < 2; log_id++)
+    for (int log_id = 0; log_id < n_logs; log_id++)
       cs_log_printf(logs[log_id],
                     "  %s%llu %s\n", _("Memory:              "),
                     (unsigned long long)ram, _("MB"));
@@ -272,10 +400,10 @@ cs_system_info(void)
 
 #if defined(HAVE_GETPWUID) && defined(HAVE_GETEUID)
 
-  /* Functions not available on IBM Blue Gene or Cray XT,
+  /* Functions not available on Cray XT,
      but a stub may exist, so we make sure we ignore it */
-#if   defined(__blrts__) || defined(__bg__) \
-   || defined(__CRAYXT_COMPUTE_LINUX_TARGET)
+#if   defined(_CRAYC) \
+   || defined(__CRAYXT) || defined(__CRAYXE) || defined(__CRAYXC)
   pwd_user = NULL;
 #else
   pwd_user = getpwuid(geteuid());
@@ -306,7 +434,7 @@ cs_system_info(void)
 
   /* Directory info */
 
-  for (log_id = 0; log_id < 2; log_id++)
+  for (int log_id = 0; log_id < n_logs; log_id++)
     cs_log_printf(logs[log_id],
                   "  %s%s\n", _("Directory:           "), str_directory);
 
@@ -323,45 +451,8 @@ cs_system_info(void)
     MPI_Comm_size(comm, &n_ranks);
     MPI_Comm_size(MPI_COMM_WORLD, &n_world_ranks);
 
-#   if defined(__bgq__) && defined(__xlc__)
-
-    {
-      int a_torus, b_torus, c_torus, d_torus, e_torus;
-      int n_flags = personality.Network_Config.NetFlags;
-      int n_hw_threads = Kernel_ProcessorCount();
-
-      for (log_id = 0; log_id < 2; log_id++) {
-        cs_log_printf(logs[log_id],
-                      "  %s%d\n", _("MPI ranks:           "), n_ranks);
-        if (n_world_ranks > n_ranks)
-          cs_log_printf(logs[log_id],
-                        "  %s%d\n", _("MPI_COMM_WORLD size: "),
-                        n_world_ranks);
-        cs_log_printf(logs[log_id],
-                      "  %s%d\n", _("Hardware threads:    "), n_hw_threads);
-      }
-
-      if (n_flags & ND_ENABLE_TORUS_DIM_A) a_torus = 1; else a_torus = 0;
-      if (n_flags & ND_ENABLE_TORUS_DIM_B) b_torus = 1; else b_torus = 0;
-      if (n_flags & ND_ENABLE_TORUS_DIM_C) c_torus = 1; else c_torus = 0;
-      if (n_flags & ND_ENABLE_TORUS_DIM_D) d_torus = 1; else d_torus = 0;
-      if (n_flags & ND_ENABLE_TORUS_DIM_E) e_torus = 1; else e_torus = 0;
-
-      for (log_id = 0; log_id < 2; log_id++) {
-        cs_log_printf(logs[log_id],
-                      "  %s<%d,%d,%d,%d,%d>\n", _("Block shape:         "),
-                      personality.Network_Config.Anodes,
-                      personality.Network_Config.Bnodes,
-                      personality.Network_Config.Cnodes,
-                      personality.Network_Config.Dnodes,
-                      personality.Network_Config.Enodes);
-        cs_log_printf(logs[log_id],
-                      "  %s<%d,%d,%d,%d,%d>\n", _("Torus links enabled: "),
-                      a_torus, b_torus, c_torus, d_torus, e_torus);
-      }
-    }
-
-#   else
+    int ranks_per_node[2];
+    _mpi_ranks_per_node(comm, ranks_per_node);
 
     {
       int appnum = -1;
@@ -374,7 +465,7 @@ cs_system_info(void)
         appnum = *(int *)attp;
 #     endif
 
-      for (log_id = 0; log_id < 2; log_id++) {
+      for (int log_id = 0; log_id < n_logs; log_id++) {
         if (appnum > -1 && log_id == 0)
           cs_log_printf(logs[log_id],
                         "  %s%d (%s %d)\n",
@@ -384,6 +475,16 @@ cs_system_info(void)
           cs_log_printf(logs[log_id],
                         "  %s%d\n", _("MPI ranks:           "), n_ranks);
 
+        if (ranks_per_node[0] > 0 && ranks_per_node[0] < n_ranks) {
+          if (ranks_per_node[0] == ranks_per_node[1])
+            cs_log_printf(logs[log_id],
+                          "  %s%d\n", _("MPI ranks per node:  "),
+                          ranks_per_node[0]);
+          else
+            cs_log_printf(logs[log_id],
+                          "  %s%d - %d\n", _("MPI ranks per node:  "),
+                          ranks_per_node[0], ranks_per_node[1]);
+        }
         if (n_world_ranks > n_ranks)
           cs_log_printf(logs[log_id],
                         "  %s%d\n", _("MPI_COMM_WORLD size: "),
@@ -391,8 +492,6 @@ cs_system_info(void)
       }
     }
 
-#   endif /* defined(HAVE_MPI) */
-
   }
 
 #endif /* defined(HAVE_MPI) */
@@ -401,7 +500,7 @@ cs_system_info(void)
   {
     int t_id = omp_get_thread_num();
     if (t_id == 0) {
-      for (log_id = 0; log_id < 2; log_id++) {
+      for (int log_id = 0; log_id < n_logs; log_id++) {
         cs_log_printf(logs[log_id],
                       "  %s%d\n", _("OpenMP threads:      "),
                       omp_get_max_threads());
@@ -415,6 +514,211 @@ cs_system_info(void)
     }
   }
 #endif
+
+#if defined(HAVE_CUDA)
+  for (int log_id = 0; log_id < n_logs; log_id++)
+    cs_base_cuda_device_info(log_id);
+#endif
+
+#if    defined(CS_CC_VERSION_STRING) || defined(CS_CXX_VERSION_STRING) \
+    || defined(CS_FC_VERSION_STRING) || defined(CS_NVCC_VERSION_STRING)
+  for (int log_id = 0; log_id < n_logs; log_id++) {
+    cs_log_printf(logs[log_id], "\n  Compilers used for build:\n");
+#   if defined(CS_CC_VERSION_STRING)
+    cs_log_printf(logs[log_id],
+                  "    %s%s\n", _("C compiler:        "), CS_CC_VERSION_STRING);
+#   endif
+#   if defined(CS_CXX_VERSION_STRING)
+    cs_log_printf(logs[log_id],
+                  "    %s%s\n", _("C++ compiler:      "), CS_CXX_VERSION_STRING);
+#   endif
+#   if defined(CS_FC_VERSION_STRING)
+    cs_log_printf(logs[log_id],
+                  "    %s%s\n", _("Fortran compiler:  "), CS_FC_VERSION_STRING);
+#   endif
+#   if defined(CS_NVCC_VERSION_STRING)
+    cs_log_printf(logs[log_id],
+                  "    %s%s\n", _("CUDA compiler:     "), CS_NVCC_VERSION_STRING);
+#   endif
+  }
+#endif
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print available MPI library information.
+ *
+ * \param[in]  log   if true, standard logging; otherwise, single output
+ */
+/*----------------------------------------------------------------------------*/
+
+#if defined(HAVE_MPI)
+
+static void
+_mpi_version_info(bool  log)
+{
+#if defined(MPI_SUBVERSION)
+
+  char mpi_vendor_lib[32] = "";
+  char mpi_lib[32] = "";
+
+  /* Base MPI library information */
+
+#if defined(MPI_VENDOR_NAME)
+
+#if defined(OMPI_MAJOR_VERSION)
+  snprintf(mpi_lib, 31, "%s %d.%d.%d",
+           MPI_VENDOR_NAME,
+           OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION, OMPI_RELEASE_VERSION);
+#elif defined(MPICH2_VERSION)
+  snprintf(mpi_lib, 31, "%s %s", MPI_VENDOR_NAME, MPICH2_VERSION);
+#elif defined(MPICH_VERSION)
+  snprintf(mpi_lib, 31, "%s %s", MPI_VENDOR_NAME, MPICH_VERSION);
+#else
+  snprintf(mpi_lib, 31, "%s", MPI_VENDOR_NAME);
+#endif
+
+#elif defined(OPEN_MPI)
+#if defined(OMPI_MAJOR_VERSION)
+  snprintf(mpi_lib, 31, "Open MPI %d.%d.%d",
+           OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION, OMPI_RELEASE_VERSION);
+#else
+  snprintf(mpi_lib, 31, "Open MPI");
+#endif
+
+#elif defined(MPICH2)
+#if defined(MPICH2_VERSION)
+  snprintf(mpi_lib, 31, "MPICH2 %s", MPICH2_VERSION);
+#else
+  snprintf(mpi_lib, 31, "MPICH2");
+#endif
+#elif defined(MPICH_NAME)
+#if defined(MPICH_VERSION)
+  snprintf(mpi_lib, 31, "MPICH %s", MPICH_VERSION);
+#else
+  snprintf(mpi_lib, 31, "MPICH");
+#endif
+#endif
+
+  mpi_lib[31] = '\0';
+
+  /* Possible additional MPI vendor information */
+
+#if defined(MVAPICH2_VERSION)
+  snprintf(mpi_vendor_lib, 31, "MVAPICH2 %s", MVAPICH2_VERSION);
+#elif defined(MSMPI_VER)
+  snprintf(mpi_vendor_lib, 31, "MS-MPI");
+#elif defined(PLATFORM_MPI)
+  {
+    int v, v0, v1, v2, v3;
+    v0 =  PLATFORM_MPI>>24;
+    v =  (PLATFORM_MPI - (v0<<24));
+    v1 =  v>>16;
+    v =  (v - (v1<<16));
+    v2 =  v>>8;
+    v3 =  (v - (v2<<8));
+    snprintf(mpi_vendor_lib, 31, "Platform MPI %x.%x.%x.%x\n",
+             v0, v1, v2, v3);
+  }
+#endif
+
+  mpi_vendor_lib[31] = '\0';
+
+  int  n_logs = (log) ? 2 : 1;
+  cs_log_t logs[] = {CS_LOG_DEFAULT, CS_LOG_PERFORMANCE};
+
+  for (int log_id = 0; log_id < n_logs; log_id++) {
+
+    if (mpi_vendor_lib[0] != '\0') {
+      if (mpi_lib[0] != '\0')
+        cs_log_printf(logs[log_id],
+                      _("\n  MPI version %d.%d (%s, based on %s)\n"),
+                      MPI_VERSION, MPI_SUBVERSION, mpi_vendor_lib, mpi_lib);
+      else
+        cs_log_printf(logs[log_id],
+                      _("\n  MPI version %d.%d (%s)\n"),
+                      MPI_VERSION, MPI_SUBVERSION, mpi_vendor_lib);
+    }
+    else {
+      if (mpi_lib[0] != '\0')
+        cs_log_printf(logs[log_id],
+                      _("\n  MPI version %d.%d (%s)\n"),
+                      MPI_VERSION, MPI_SUBVERSION, mpi_lib);
+      else
+        cs_log_printf(logs[log_id],
+                      _("\n  MPI version %d.%d\n"),
+                      MPI_VERSION, MPI_SUBVERSION);
+    }
+
+  }
+
+#endif /* defined(MPI_SUBVERSION) */
+}
+
+#endif /* (HAVE_MPI) */
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
+/*============================================================================
+ * Public function definitions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print available system information.
+ *
+ * \param[in]  comm  associated MPI communicator
+ */
+/*----------------------------------------------------------------------------*/
+
+#if defined(HAVE_MPI)
+
+void
+cs_system_info(MPI_Comm comm)
+
+#else
+
+void
+cs_system_info(void)
+
+#endif
+
+{
+#if defined(HAVE_MPI)
+  _system_info(comm, true);
+  _mpi_version_info(false);
+#else
+  _system_info(true);
+#endif
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print available system information, without additional logging
+ *
+ * \param[in]  comm  associated MPI communicator
+ */
+/*----------------------------------------------------------------------------*/
+
+#if defined(HAVE_MPI)
+
+void
+cs_system_info_no_log(MPI_Comm comm)
+
+#else
+
+void
+cs_system_info_no_log(void)
+
+#endif
+
+{
+#if defined(HAVE_MPI)
+  _system_info(comm, false);
+  _mpi_version_info(false);
+#else
+  _system_info(false);
+#endif
 }
 
 /*-----------------------------------------------------------------------------*/
