@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -50,6 +50,8 @@
 #include "cs_field.h"
 #include "cs_field_pointer.h"
 #include "cs_log.h"
+#include "cs_halo.h"
+#include "cs_halo_perio.h"
 #include "cs_mesh.h"
 #include "cs_mesh_location.h"
 #include "cs_parall.h"
@@ -182,7 +184,7 @@ typedef struct {
 typedef struct {
 
   int                     nt_prev;        /* Restart time step */
-  int                     t_prev;         /* Restart time */
+  double                  t_prev;         /* Restart time */
 
   int                     n_wa;           /* Number of weight accumulators */
   int                     n_moments;      /* Number of moments */
@@ -289,7 +291,7 @@ _restart_info_read_auxiliary(cs_restart_t  *r)
                                     "time_moments:sizes",
                                     CS_MESH_LOCATION_NONE,
                                     3,
-                                    CS_TYPE_cs_int_t,
+                                    CS_TYPE_int,
                                     sizes);
 
   if (retcode == CS_RESTART_ERR_EXISTS)
@@ -335,7 +337,7 @@ _restart_info_read_auxiliary(cs_restart_t  *r)
                           "time_moments:wa:location_id",
                           CS_MESH_LOCATION_NONE,
                           ri->n_wa,
-                          CS_TYPE_cs_int_t,
+                          CS_TYPE_int,
                           ri->wa_location_id);
   _assert_restart_success(retcode);
 
@@ -349,7 +351,7 @@ _restart_info_read_auxiliary(cs_restart_t  *r)
                           "time_moments:wa:nt_start",
                           CS_MESH_LOCATION_NONE,
                           ri->n_wa,
-                          CS_TYPE_cs_int_t,
+                          CS_TYPE_int,
                           ri->wa_nt_start);
   _assert_restart_success(retcode);
 
@@ -384,7 +386,7 @@ _restart_info_read_auxiliary(cs_restart_t  *r)
                                     "time_moments:type",
                                     CS_MESH_LOCATION_NONE,
                                     ri->n_moments,
-                                    CS_TYPE_cs_int_t,
+                                    CS_TYPE_int,
                                     ri->m_type);
   _assert_restart_success(retcode);
 
@@ -392,7 +394,7 @@ _restart_info_read_auxiliary(cs_restart_t  *r)
                                     "time_moments:location_id",
                                     CS_MESH_LOCATION_NONE,
                                     ri->n_moments,
-                                    CS_TYPE_cs_int_t,
+                                    CS_TYPE_int,
                                     ri->location_id);
   _assert_restart_success(retcode);
 
@@ -400,7 +402,7 @@ _restart_info_read_auxiliary(cs_restart_t  *r)
                                     "time_moments:dimension",
                                     CS_MESH_LOCATION_NONE,
                                     ri->n_moments,
-                                    CS_TYPE_cs_int_t,
+                                    CS_TYPE_int,
                                     ri->dimension);
   _assert_restart_success(retcode);
 
@@ -408,7 +410,7 @@ _restart_info_read_auxiliary(cs_restart_t  *r)
                                     "time_moments:wa_id",
                                     CS_MESH_LOCATION_NONE,
                                     ri->n_moments,
-                                    CS_TYPE_cs_int_t,
+                                    CS_TYPE_int,
                                     ri->wa_id);
   _assert_restart_success(retcode);
 
@@ -416,7 +418,7 @@ _restart_info_read_auxiliary(cs_restart_t  *r)
                                     "time_moments:lower_order_id",
                                     CS_MESH_LOCATION_NONE,
                                     ri->n_moments,
-                                    CS_TYPE_cs_int_t,
+                                    CS_TYPE_int,
                                     ri->l_id);
   _assert_restart_success(retcode);
 }
@@ -438,7 +440,7 @@ _restart_info_read(void)
   /* Read previous time step if not already done */
 
   if (ts->nt_prev < 1) {
-    r = cs_restart_create("main", "restart", CS_RESTART_MODE_READ);
+    r = cs_restart_create("main.csc", "restart", CS_RESTART_MODE_READ);
     cs_restart_read_time_step_info(r);
     if (_restart_uses_main == false)
       cs_restart_destroy(&r);
@@ -448,9 +450,9 @@ _restart_info_read(void)
 
   if (r == NULL) {
     if (_restart_uses_main)
-      r = cs_restart_create("main", NULL, CS_RESTART_MODE_READ);
+      r = cs_restart_create("main.csc", NULL, CS_RESTART_MODE_READ);
     else
-      r = cs_restart_create("auxiliary", NULL, CS_RESTART_MODE_READ);
+      r = cs_restart_create("auxiliary.csc", NULL, CS_RESTART_MODE_READ);
   }
 
   _restart_info_read_auxiliary(r);
@@ -534,7 +536,7 @@ _check_restart(const char                     *name,
   int prev_wa_id = -1;
 
   if (   (*nt_start > -1 && *nt_start >= ri->nt_prev)
-      || (*t_start >= 0 && *t_start >= ri->nt_prev))
+      || (*t_start >= 0 && *t_start >= ri->t_prev))
     return prev_id;
 
   /* Adjust accumulator info if moment should be restarted */
@@ -939,8 +941,8 @@ _sd_moment_data(const void  *input,
 
   const cs_lnum_t n_elts = cs_mesh_location_get_n_elts(location_id)[0];
 
-  cs_lnum_t _f_dim[16*2];
-  cs_lnum_t *f_dim;
+  int _f_dim[16*2];
+  int *f_dim;
 
   const cs_real_t * _f_val[16];
   const cs_real_t **f_val;
@@ -1381,6 +1383,27 @@ _ensure_init_weight_accumulator(cs_time_moment_wa_t  *mwa)
   }
 }
 
+/*----------------------------------------------------------------------------
+ * Reset weight accumulator
+ *
+ * parameters:
+ *   mwa <-- moment weight accumulator
+ *----------------------------------------------------------------------------*/
+
+static void
+_reset_weight_accumulator(cs_time_moment_wa_t  *mwa)
+{
+  if (mwa->location_id == CS_MESH_LOCATION_NONE)
+    mwa->val0 = 0.;
+  else {
+    assert(mwa->val != NULL);
+
+    cs_lnum_t n_w_elts = cs_mesh_location_get_n_elts(mwa->location_id)[0];
+    for (cs_lnum_t i = 0; i < n_w_elts; i++)
+      mwa->val[i] = 0.;
+  }
+}
+
 /*----------------------------------------------------------------------------
  * Update weight accumulator
  *
@@ -1413,7 +1436,7 @@ static void
 _ensure_init_moment(cs_time_moment_t  *mt)
 {
   if (mt->f_id < 0 && mt->val == NULL) {
-    cs_lnum_t n_elts = cs_mesh_location_get_n_elts(mt->location_id)[0];
+    cs_lnum_t n_elts = cs_mesh_location_get_n_elts(mt->location_id)[2];
     cs_lnum_t n_d_elts = n_elts*(cs_lnum_t)(mt->dim);
     BFT_MALLOC(mt->val, n_d_elts, cs_real_t);
     for (cs_lnum_t i = 0; i < n_d_elts; i++)
@@ -1640,16 +1663,14 @@ cs_time_moment_define_by_func(const char                *name,
         break;
       }
     }
+  } else { /* Build field matching moment */
+    f = cs_field_create(name,
+                        CS_FIELD_POSTPROCESS | CS_FIELD_ACCUMULATOR,
+                        location_id,
+                        moment_dim,
+                        false);  /* no previous values */
   }
 
-  /* Build field matching moment */
-
-  f = cs_field_create(name,
-                      CS_FIELD_POSTPROCESS | CS_FIELD_ACCUMULATOR,
-                      location_id,
-                      moment_dim,
-                      false);  /* no previous values */
-
   moment_id = _find_or_add_moment(location_id,
                                   dim,
                                   data_func,
@@ -1858,6 +1879,74 @@ cs_time_moment_is_active(int  moment_id)
   return retval;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Reset a time moment.
+ *        Current iteration is set as starting time step for current moment.
+ *
+ * \param[in]   moment_id  id of associated moment
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_time_moment_reset(int   moment_id)
+{
+  const cs_time_step_t  *ts = cs_glob_time_step;
+
+  cs_time_moment_t *mt = _moment + moment_id;
+  cs_time_moment_wa_t *mwa = _moment_wa + mt->wa_id;
+
+  /* set current iteration as starting time step for current moment */
+  mt->nt_cur = -1;
+  mwa->nt_start = ts->nt_cur;
+  mwa->t_start = -1.;
+
+  cs_lnum_t n_elts = cs_mesh_location_get_n_elts(mt->location_id)[0];
+  cs_lnum_t nd = n_elts * mt->dim;
+
+  cs_real_t *restrict val = mt->val;
+  if (mt->f_id > -1) {
+    cs_field_t *f = cs_field_by_id(mt->f_id);
+    val = f->val;
+  }
+
+  for (cs_lnum_t i = 0; i < nd; i++) {
+    /* reset moment values */
+    val[i] = 0.;
+  }
+
+  _reset_weight_accumulator(mwa);
+
+  /* sub-moments (means for variance) */
+
+  if (mt->l_id > -1) {
+    int l_id = mt->l_id;
+
+    mt = _moment + l_id;
+    mwa = _moment_wa + mt->wa_id;
+
+    mt->nt_cur = -1;
+    mwa->nt_start = ts->nt_cur;
+    mwa->t_start = -1.;
+
+    n_elts = cs_mesh_location_get_n_elts(mt->location_id)[0];
+    nd = n_elts * mt->dim;
+
+    val = mt->val;
+    if (mt->f_id > -1) {
+      cs_field_t *f = cs_field_by_id(mt->f_id);
+      val = f->val;
+    }
+
+    for (cs_lnum_t i = 0; i < nd; i++) {
+      /* reset moment values */
+      val[i] = 0.;
+    }
+
+    _reset_weight_accumulator(mwa);
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Map time step values array for temporal moments.
@@ -2060,6 +2149,25 @@ cs_time_moment_update_all(void)
 
         BFT_FREE(x);
 
+        /* Sync ghost cells so downstream use is safe */
+
+        if (mt->location_id == CS_MESH_LOCATION_CELLS) {
+          const cs_halo_t *halo = cs_glob_mesh->halo;
+          if (halo != NULL) {
+            if (mt->dim == 1)
+              cs_halo_sync_var(halo, CS_HALO_EXTENDED, val);
+            else {
+              cs_halo_sync_var_strided(halo, CS_HALO_EXTENDED, val, mt->dim);
+              if (halo->n_transforms > 0) {
+                if (mt->dim == 3)
+                  cs_halo_perio_sync_var_vect(halo, CS_HALO_EXTENDED, val, 3);
+                else if (mt->dim == 6)
+                  cs_halo_perio_sync_var_sym_tens(halo, CS_HALO_EXTENDED, val);
+              }
+            }
+          }
+        }
+
       } /* End of test if moment is active */
 
     } /* End of loop on moments */
@@ -2630,7 +2738,7 @@ cs_time_moment_restart_write(cs_restart_t  *restart)
                            "time_moments:sizes",
                            CS_MESH_LOCATION_NONE,
                            3,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            sizes);
 
   cs_restart_write_section(restart,
@@ -2668,14 +2776,14 @@ cs_time_moment_restart_write(cs_restart_t  *restart)
                            "time_moments:wa:location_id",
                            CS_MESH_LOCATION_NONE,
                            n_active_wa,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            location_id);
 
   cs_restart_write_section(restart,
                            "time_moments:wa:nt_start",
                            CS_MESH_LOCATION_NONE,
                            n_active_wa,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            nt_start);
 
   cs_restart_write_section(restart,
@@ -2740,35 +2848,35 @@ cs_time_moment_restart_write(cs_restart_t  *restart)
                            "time_moments:type",
                            CS_MESH_LOCATION_NONE,
                            n_active_moments,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            m_type);
 
   cs_restart_write_section(restart,
                            "time_moments:location_id",
                            CS_MESH_LOCATION_NONE,
                            n_active_moments,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            location_id);
 
   cs_restart_write_section(restart,
                            "time_moments:dimension",
                            CS_MESH_LOCATION_NONE,
                            n_active_moments,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            dimension);
 
   cs_restart_write_section(restart,
                            "time_moments:wa_id",
                            CS_MESH_LOCATION_NONE,
                            n_active_moments,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            wa_id);
 
   cs_restart_write_section(restart,
                            "time_moments:lower_order_id",
                            CS_MESH_LOCATION_NONE,
                            n_active_moments,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            l_id);
 
   BFT_FREE(l_id);
