@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -46,6 +46,7 @@
 
 #include "cs_base.h"
 #include "cs_timer.h"
+#include "cs_time_step.h"
 
 /*----------------------------------------------------------------------------
  *  Header for the current file
@@ -810,17 +811,17 @@ _fortran_time_plot_realloc(int                     plot_num,
 
 void CS_PROCF (tpsini, TPSINI)
 (
- const cs_int_t  *tplnum,
+ const int       *tplnum,
  const char      *tplnam,
  const char      *tplpre,
- const cs_int_t  *tplfmt,
- const cs_int_t  *idtvar,
- const cs_int_t  *nstru,
+ const int       *tplfmt,
+ const int       *idtvar,
+ const int       *nstru,
  const cs_real_t *xmstru,
  const cs_real_t *xcstru,
  const cs_real_t *xkstru,
- const cs_int_t  *lnam,
- const cs_int_t  *lpre
+ const int       *lnam,
+ const int       *lpre
  CS_ARGF_SUPP_CHAINE              /*     (possible 'length' arguments added
                                          by many Fortran compilers) */
 )
@@ -832,7 +833,7 @@ void CS_PROCF (tpsini, TPSINI)
   char *file_prefix = cs_base_string_f_to_c_create(tplpre, *lpre);
   bool use_iteration = false;
 
-  if (*idtvar < 0 || *idtvar == 2)
+  if (*idtvar == CS_TIME_STEP_STEADY || *idtvar == CS_TIME_STEP_LOCAL)
     use_iteration = true;
 
   /* Main processing */
@@ -845,16 +846,17 @@ void CS_PROCF (tpsini, TPSINI)
 
       _fortran_time_plot_realloc(*tplnum, plot_name, fmt);
 
-      _plot_files[fmt][*tplnum - 1] = cs_time_plot_init_struct(plot_name,
-                                                               file_prefix,
-                                                               fmt,
-                                                               use_iteration,
-                                                              _flush_wtime_default,
-                                                              _n_buffer_steps_default,
-                                                               *nstru,
-                                                               xmstru,
-                                                               xcstru,
-                                                               xkstru);
+      _plot_files[fmt][*tplnum - 1]
+        = cs_time_plot_init_struct(plot_name,
+                                   file_prefix,
+                                   fmt,
+                                   use_iteration,
+                                   _flush_wtime_default,
+                                   _n_buffer_steps_default,
+                                   *nstru,
+                                   xmstru,
+                                   xcstru,
+                                   xkstru);
     }
 
   }
@@ -880,8 +882,8 @@ void CS_PROCF (tpsini, TPSINI)
 
 void CS_PROCF (tplend, TPLEND)
 (
- const cs_int_t  *tplnum,
- const cs_int_t  *tplfmt
+ const int       *tplnum,
+ const int       *tplfmt
 )
 {
   cs_time_plot_format_t fmt;
@@ -932,10 +934,10 @@ void CS_PROCF (tplend, TPLEND)
 
 void CS_PROCF (tplwri, TPLWRI)
 (
- const cs_int_t  *tplnum,
- const cs_int_t  *tplfmt,
- const cs_int_t  *nprb,
- const cs_int_t  *ntcabs,
+ const int       *tplnum,
+ const int       *tplfmt,
+ const int       *nprb,
+ const int       *ntcabs,
  const cs_real_t *ttcabs,
  const cs_real_t *valprb
 )
@@ -974,7 +976,7 @@ void CS_PROCF (tplwri, TPLWRI)
 
 void CS_PROCF (tplnbr, TPLNBR)
 (
- cs_int_t  *ntpl
+ int       *ntpl
 )
 {
   int fmt;
@@ -1050,6 +1052,10 @@ cs_time_plot_init_probe(const char             *plot_name,
     break;
   }
 
+  cs_time_plot_flush(p); /* Flush after creation so plot tools can start
+                            analyzing file immediately, while computation
+                            is running */
+
   return p;
 }
 
