@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,11 +45,10 @@
 #include "bft_error.h"
 #include "bft_printf.h"
 
+#include "cs_base.h"
 #include "cs_log.h"
 #include "cs_map.h"
 #include "cs_parall.h"
-#include "cs_mesh_location.h"
-#include "cs_stokes_model.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -69,6 +68,19 @@ BEGIN_C_DECLS
   \file cs_time_step.c
         base time step data.
 
+  \enum cs_time_step_type_t
+
+  \brief Time step type
+
+  \var CS_TIME_STEP_STEADY
+       Steady (SIMPLE) algorithm
+  \var CS_TIME_STEP_CONSTANT
+       Unsteady (SIMPLEC) algorithm with constant time step
+  \var CS_TIME_STEP_ADAPTIVE
+       Unsteady (SIMPLEC) algorithm with time-adaptive (varying) time step
+  \var CS_TIME_STEP_LOCAL
+       Pseudo-steady with time-and-space varying time step (SIMPLEC)
+
   \struct cs_time_step_t
 
   \brief time step descriptor
@@ -112,9 +124,13 @@ BEGIN_C_DECLS
         For the restart calculations, \ref t_cur takes
         into account the physical time of the previous calculations.\n
         If the time step is uniform (\ref cs_time_step_options_t::idtvar "idtvar"
-        = 0 or 1), \ref t_cur increases of \ref dt (value of the time step) at each iteration.
-        If the time step is non-uniform (\ref cs_time_step_options_t::idtvar "idtvar"=2),
-        \ref t_cur increases of \ref cs_time_step_options_t::dtref "dtref" at each time step.\n
+        = CS_TIME_STEP_CONSTANT or CS_ADAPTATIVE_TIME_STEP),
+        \ref t_cur increases of \ref dt (value of the time step)
+        at each iteration.
+        If the time step is non-uniform (\ref cs_time_step_options_t::idtvar
+        "idtvar"=CS_TIME_STEP_LOCAL),
+        \ref t_cur increases of \ref cs_time_step_t::dt_ref "dt_ref" at each
+        time step.\n
         \ref t_cur is initialised and updated automatically by the code,
         its value is not to be modified by the user.
   \var  cs_time_step_t::t_max
@@ -127,16 +143,6 @@ BEGIN_C_DECLS
   Members of this time step options descriptor are publicly accessible, to
   allow for concise syntax.
 
-  \var  cs_time_step_options_t::inpdt0
-        Indicator "zero time step"
-        - 0: standard calculation
-        - 1: to simulate no time step
-        - for non-restarted computations: only resolution (Navier-Stokes,
-          turbulence, scalars) is skipped
-        - for restarted computations: resolution, computation of physical
-          properties, and definition of boundary conditions is skipped
-          (values are read from checkpoint file).
-
   \var  cs_time_step_options_t::iptlro
         Clip the time step with respect to the buoyant effects\n
 
@@ -146,58 +152,62 @@ BEGIN_C_DECLS
         lower than this limit, otherwise numerical instabilities may appear.\n
         \ref iptlro indicates whether the time step should be limited to the
         local thermal time step (=1) or not (=0).\n
-        When \ref iptlro=1, the listing shows the number of cells where the
+        When \ref iptlro=1, the log shows the number of cells where the
         time step has been clipped due to the thermal criterion, as well as
         the maximum ratio between the time step and the maximum thermal time
-        step. If \ref idtvar=0, since the time step is fixed and cannot be
-        clipped, this ratio can be greater than 1. When \ref idtvar > 0, this
+        step. If \ref idtvar=CS_TIME_STEP_CONSTANT, since the time step is fixed
+        and cannot be clipped, this ratio can be greater than 1.
+        When \ref idtvar > CS_TIME_STEP_CONSTANT, this
         ratio will be less than 1, except if the constraint \ref dtmin has
         prevented the code from reaching a sufficiently low value for \ref dt.
         Useful when density gradients and gravity are present.
 
   \var  cs_time_step_options_t::idtvar
-        Option for a variable time step
-        - -1: steady algorithm
-        -  0: constant time step
-        -  1: time step constant in space but variable in time
-        -  2: variable time step in space and in time.
+        Time step type: constant, adaptive, or steady algorithm variants.
         If the numerical scheme is a second-order in time, only the
-        option 0 is allowed.
+        option CS_TIME_STEP_CONSTANT is allowed.
+        \anchor idtvar
 
-  \var  cs_time_step_options_t::dtref
+  \var  cs_time_step_t::dt_ref
         Reference time step.\n
 
         This is the time step value used in the case of a calculation run with a
-        uniform and constant time step, i.e. \ref idtvar =0 (restart calculation
-        or not). It is the value used to initialize the time step in the case of
-        an initial calculation run with a non-constant time step(\ref idtvar=1 or
-        2). It is also the value used to initialise the time step in the case of
+        uniform and constant time step, i.e. \ref idtvar =CS_TIME_STEP_CONSTANT
+        (restart calculation or not). It is the value used to initialize the
+        time step in the case of an initial calculation run with a non-constant
+        time step (\ref idtvar=CS_TIME_STEP_ADAPTATIVE or CS_TIME_STEP_LOCAL).
+        It is also the value used to initialise the time step in the case of
         a restart calculation in which the type of time step has been changed
-        (for instance, \ref idtvar=1 in the new calculation and \ref idtvar = 0
-        or 2 in the previous calculation).\n
+        (for instance, \ref idtvar=CS_TIME_STEP_ADAPTATIVE in the new calculation
+        and \ref idtvar = CS_TIME_STEP_CONSTANT or CS_STEP_LOCAL_TIME in the
+        previous calculation).\n
         See \ref user_initialization_time_step for examples.
 
   \var  cs_time_step_options_t::coumax
-        Maximum Courant number (when \ref idtvar is different from 0).
+        Maximum Courant number (when \ref idtvar is different
+        from CS_TIME_STEP_CONSTANT).
 
   \var  cs_time_step_options_t::cflmmx
         Max. Courant number for the continuity equation in compressible model.
 
   \var  cs_time_step_options_t::foumax
-        Maximum Fourier number (when \ref idtvar is different from 0).
+        Maximum Fourier number (when \ref idtvar is different from
+        CS_TIME_STEP_CONSTANT).
 
   \var  cs_time_step_options_t::varrdt
         Maximum allowed relative increase in the calculated time step value
         between two successive time steps (to ensure stability, any decrease
         in the time step is immediate and without limit).\n
-        Useful when idtvar is different from 0.
+        Useful when idtvar is different from CS_TIME_STEP_CONSTANT.
 
   \var  cs_time_step_options_t::dtmin
-        Lower limit for the calculated time step when idtvar is different from 0.
+        Lower limit for the calculated time step when idtvar is different from
+        CS_TIME_STEP_CONSTANT.
         Take \ref dtmin = min (ld/ud, sqrt(lt/(gdelta rho/rho)), ...).
 
   \var  cs_time_step_options_t::dtmax
-        Upper limit for the calculated time step when idtvar is different from 0.
+        Upper limit for the calculated time step when idtvar is different from
+        CS_TIME_STEP_CONSTANT.
         Take \ref dtmax = max (ld/ud, sqrt(lt/(gdelta rho/rho)), ...).
 
   \var  cs_time_step_options_t::relxst
@@ -220,6 +230,18 @@ BEGIN_C_DECLS
  * Static global variables
  *============================================================================*/
 
+static const char *cs_time_step_type_enum_name[]
+  = {"CS_TIME_STEP_STEADY",
+     "CS_TIME_STEP_CONSTANT",
+     "CS_TIME_STEP_ADAPTIVE",
+     "CS_TIME_STEP_LOCAL"};
+
+static const char *cs_time_step_type_name[]
+  = {N_("steady algorithm"),
+     N_("constant"),
+     N_("time-adaptive"),
+     N_("local in time and space (pseudo-steady)")};
+
 /* main time step structure and associated pointer */
 
 static cs_time_step_t  _time_step = {
@@ -231,28 +253,30 @@ static cs_time_step_t  _time_step = {
   .nt_ini = 2,
   .t_prev = 0.,
   .t_cur = 0.,
-  .t_max = -1.
+  .t_max = -1.,
+  .dt = {0.1, 0.1, 0.1},
+  .dt_ref = 0.1,
+  .dt_next = 0.1
 };
 
-static cs_time_step_options_t  _time_step_options =
-{
-  .inpdt0 = 0,
+static cs_time_step_options_t  _time_step_options = {
   .iptlro = 0,
-  .idtvar = 0,
-  .dtref  = -1.e12*10.,
+  .idtvar = CS_TIME_STEP_CONSTANT, /* constant time step by default */
   .coumax = 1.,
   .cflmmx = 0.99,
   .foumax = 10.,
   .varrdt = 0.1,
-  .dtmin  = -1.e12*10.,
-  .dtmax  = -1.e12*10.,
-  .relxst = 0.7
+  .dtmin  = -1.e13,
+  .dtmax  = -1.e13,
+  .relxst = 0.7 /* Not used in CDO schemes */
 };
 
 const cs_time_step_t  *cs_glob_time_step = &_time_step;
 
 const cs_time_step_options_t  *cs_glob_time_step_options = &_time_step_options;
 
+static double _c = 0; /* compensation term for Kahan sum */
+
 /*============================================================================
  * Prototypes for functions intended for use only by Fortran wrappers.
  * (descriptions follow, with function bodies).
@@ -263,15 +287,14 @@ cs_f_time_step_get_pointers(int     **nt_prev,
                             int     **nt_cur,
                             int     **nt_max,
                             int     **nt_ini,
+                            double  **dtref,
                             double  **t_prev,
                             double  **t_cur,
                             double  **t_max);
 
 void
-cs_f_time_step_options_get_pointers(int    **inpdt0,
-                                    int    **iptlro,
+cs_f_time_step_options_get_pointers(int    **iptlro,
                                     int    **idtvar,
-                                    double **dtref,
                                     double **coumax,
                                     double **cflmmx,
                                     double **foumax,
@@ -299,6 +322,7 @@ cs_f_time_step_options_get_pointers(int    **inpdt0,
  *   nt_cur  --> pointer to cs_glob_time_step->nt_cur
  *   nt_max  --> pointer to cs_glob_time_step->nt_max
  *   nt_ini  --> pointer to cs_glob_time_step->nt_ini
+ *   dt_ref  --> pointer to cs_glob_time_step->dt_ref
  *   t_prev  --> pointer to cs_glob_time_step->t_prev
  *   t_cur   --> pointer to cs_glob_time_step->t_cur
  *   t_max   --> pointer to cs_glob_time_step->t_ax
@@ -309,6 +333,7 @@ cs_f_time_step_get_pointers(int      **nt_prev,
                             int      **nt_cur,
                             int      **nt_max,
                             int      **nt_ini,
+                            double   **dtref,
                             double   **t_prev,
                             double   **t_cur,
                             double   **t_max)
@@ -317,6 +342,7 @@ cs_f_time_step_get_pointers(int      **nt_prev,
   *nt_cur = &(_time_step.nt_cur);
   *nt_max = &(_time_step.nt_max);
   *nt_ini = &(_time_step.nt_ini);
+  *dtref  = &(_time_step.dt_ref);
   *t_prev = &(_time_step.t_prev);
   *t_cur = &(_time_step.t_cur);
   *t_max = &(_time_step.t_max);
@@ -329,10 +355,8 @@ cs_f_time_step_get_pointers(int      **nt_prev,
  * enables mapping to Fortran global pointers.
  *
  * parameters:
- *   inpdt0 --> pointer to cs_glob_time_step_options->inpdt0
  *   iptlro --> pointer to cs_glob_time_step_options->iptlro
  *   idtvar --> pointer to cs_glob_time_step_options->idtvar
- *   dtref  --> pointer to cs_glob_time_step_options->dtref
  *   coumax --> pointer to cs_glob_time_step_options->coumax
  *   cflmmx --> pointer to cs_glob_time_step_options->cflmmx
  *   foumax --> pointer to cs_glob_time_step_options->foumax
@@ -343,10 +367,8 @@ cs_f_time_step_get_pointers(int      **nt_prev,
  *----------------------------------------------------------------------------*/
 
 void
-cs_f_time_step_options_get_pointers(int    **inpdt0,
-                                    int    **iptlro,
+cs_f_time_step_options_get_pointers(int    **iptlro,
                                     int    **idtvar,
-                                    double **dtref,
                                     double **coumax,
                                     double **cflmmx,
                                     double **foumax,
@@ -355,10 +377,8 @@ cs_f_time_step_options_get_pointers(int    **inpdt0,
                                     double **dtmax,
                                     double **relxst)
 {
-  *inpdt0 = &(_time_step_options.inpdt0);
   *iptlro = &(_time_step_options.iptlro);
   *idtvar = &(_time_step_options.idtvar);
-  *dtref  = &(_time_step_options.dtref );
   *coumax = &(_time_step_options.coumax);
   *cflmmx = &(_time_step_options.cflmmx);
   *foumax = &(_time_step_options.foumax);
@@ -368,6 +388,33 @@ cs_f_time_step_options_get_pointers(int    **inpdt0,
   *relxst = &(_time_step_options.relxst);
 }
 
+/*=============================================================================
+ * Private function definitions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Update Kahan compensation.
+ *
+ * Called as a function to help avoid compiler optimizating Kahan
+ * summation out (though aggressive or LTO optimizations might).
+ *
+ * new copensation term is (a - b) - c
+ *
+ * \param[in]  a  a
+ * \param[in]  b  b
+ * \param[in]  c  c
+ */
+/*----------------------------------------------------------------------------*/
+
+static void
+_update_kahan_compensation(double  *a,
+                           double  *b,
+                           double  *c)
+{
+  _c = (*a - *b) - *c;
+}
+
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*=============================================================================
@@ -376,10 +423,11 @@ cs_f_time_step_options_get_pointers(int    **inpdt0,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Provide acces to cs_glob_time_step
+ * \brief Provide read/write access to cs_glob_time_step
  *
- * needed to initialize structure with GUI
- *----------------------------------------------------------------------------*/
+ * \return  pointer to global time step structure
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_time_step_t *
 cs_get_glob_time_step(void)
@@ -389,10 +437,11 @@ cs_get_glob_time_step(void)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Provide acces to cs_glob_time_step_options
+ * \brief Provide read/write access to cs_glob_time_step_options
  *
- * needed to initialize structure with GUI
- *----------------------------------------------------------------------------*/
+ * \return  pointer to global time step options structure
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_time_step_options_t *
 cs_get_glob_time_step_options(void)
@@ -446,6 +495,7 @@ void
 cs_time_step_define_nt_max(int  nt_max)
 {
   _time_step.nt_max = nt_max;
+  _time_step.t_max = -1.;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -460,6 +510,7 @@ void
 cs_time_step_define_t_max(double  t_max)
 {
   _time_step.t_max = t_max;
+  _time_step.nt_max = -1;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -492,8 +543,24 @@ cs_time_step_define_prev(int     nt_prev,
 void
 cs_time_step_increment(double  dt)
 {
+  _time_step.dt[2] = _time_step.dt[1];
+  _time_step.dt[1] = _time_step.dt[0];
+  _time_step.dt[0] = dt;
+
+  double z = dt - _c;
+  double t = _time_step.t_cur + z;
+
+  _update_kahan_compensation(&t, &_time_step.t_cur, &z);
+
+  _time_step.t_cur = t;
   _time_step.nt_cur += 1;
-  _time_step.t_cur += dt;
+
+  if (_time_step.is_local)
+    cs_base_update_status("time step: %d\n",
+                          _time_step.nt_cur);
+  else
+    cs_base_update_status("time step: %d; t = %g\n",
+                          _time_step.nt_cur, _time_step.t_cur);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -525,64 +592,78 @@ cs_time_step_redefine_cur(int     nt_cur,
 void
 cs_time_step_log_setup(void)
 {
-
   cs_log_printf
     (CS_LOG_SETUP,
      _("\nTime stepping options\n"
        "---------------------\n\n"));
 
-  /* Steady */
-  if (cs_glob_time_step_options->idtvar < 0) {
-    /* Time step parameters */
-    cs_log_printf
-      (CS_LOG_SETUP,
-       _("  Steady algorithm\n\n"
-         "   Global parameters\n\n"
-         "    idtvar:     %14d (-1: steady algorithm)\n"
-         "    relxst:     %14.5e (Reference relaxation coefficient)\n\n"),
-         cs_glob_time_step_options->idtvar,
-         cs_glob_time_step_options->relxst);
+  cs_time_step_type_t ts_type = cs_glob_time_step_options->idtvar;
+  int ts_id = ts_type + 1; /* (this enum starts at -1) */
 
-    /* Frozen velocity field */
+  if (ts_type == CS_TIME_STEP_STEADY) {
+
+    /* Relaxation parameters */
     cs_log_printf
       (CS_LOG_SETUP,
-       _("   Frozen velocity field\n\n"
-         "    iccvfg:      %14d (1: Frozen velocity field)\n"),
-         cs_glob_stokes_model->iccvfg);
-  } else {
-  /* Unsteady */
-    cs_log_printf
-      (CS_LOG_SETUP,
-       _("  Unsteady algorithm\n\n"
-         "   Time step parameters\n\n"
-         "    idtvar:      %14d (0 cst; 1,2 var (t, t-space)\n"
-         "    iptlro:      %14d (1: rho-related DT clipping)\n"
-         "    coumax:      %14.5e (Maximum target CFL)\n"
-         "    foumax:      %14.5e (Maximum target Fourier)\n"
-         "    varrdt:      %14.5e (For var. DT, max. increase)\n"
-         "    dtmin:       %14.5e (Minimum time step)\n"
-         "    dtmax:       %14.5e (Maximum time step)\n"
-         "    dtref:       %14.5e (Reference time step)\n\n"
-         "    With a non-constant time step (idtvar = 1 or 2)\n"
-         "    when the value of coumax or foumax is negative\n"
-         "    or zero, the associated time step limitation\n"
-         "    (for CFL and Fourier respectively) is ignored.\n\n"),
-         cs_glob_time_step_options->idtvar,
+       _("  Steady (SIMPLE) algorithm\n\n"
+         "    Global parameters\n\n"
+         "      idtvar: %21s (%s)\n"
+         "      relxst:     %17.5g (Reference relaxation coefficient)\n\n"),
+       cs_time_step_type_enum_name[ts_id],
+       cs_time_step_type_name[ts_id],
+       cs_glob_time_step_options->relxst);
+  }
+
+  else {
+
+    if (ts_type ==CS_TIME_STEP_CONSTANT) {
+      cs_log_printf
+        (CS_LOG_SETUP,
+         _("  Unsteady algorithm\n\n"
+           "    Time step parameters\n\n"
+           "      idtvar: %21s (%s)\n"
+           "      dtref:      %17.5g (Reference time step)\n\n"),
+         cs_time_step_type_enum_name[ts_id],
+         cs_time_step_type_name[ts_id],
+         cs_glob_time_step->dt_ref);
+    }
+
+    else {
+      if (cs_glob_time_step_options->idtvar == CS_TIME_STEP_ADAPTIVE) {
+        cs_log_printf
+          (CS_LOG_SETUP,
+           _("  Unsteady algorithm\n\n"));
+      } else if (cs_glob_time_step_options->idtvar == CS_TIME_STEP_LOCAL) {
+        cs_log_printf
+          (CS_LOG_SETUP,
+           _("  Space & time varying time step algorithm (pseudo-steady)\n\n"));
+      }
+
+      cs_log_printf
+        (CS_LOG_SETUP,
+         _("  Time step parameters:\n\n"
+           "    idtvar: %21s (%s)\n"
+           "    iptlro:     %17d (1: rho-related DT clipping)\n"
+           "    coumax:     %17.5g (Maximum target CFL)\n"
+           "    foumax:     %17.5g (Maximum target Fourier)\n"
+           "    varrdt:     %17.5g (For var. DT, max. increase)\n"
+           "    dtmin:      %17.5g (Minimum time step)\n"
+           "    dtmax:      %17.5g (Maximum time step)\n"
+           "    dtref:      %17.5g (Reference time step)\n\n"
+           "  When the value of coumax or foumax is negative\n"
+           "  or zero, the associated time step limitation\n"
+           "  (for CFL and Fourier respectively) is ignored.\n\n"),
+         cs_time_step_type_enum_name[ts_id],
+         cs_time_step_type_name[ts_id],
          cs_glob_time_step_options->iptlro,
          cs_glob_time_step_options->coumax,
          cs_glob_time_step_options->foumax,
          cs_glob_time_step_options->varrdt,
          cs_glob_time_step_options->dtmin,
          cs_glob_time_step_options->dtmax,
-         cs_glob_time_step_options->dtref);
-
-    /* Frozen velocity field */
-    cs_log_printf
-      (CS_LOG_SETUP,
-       _("   Frozen velocity field\n\n"
-         "    iccvfg:      %14d (1: Frozen velocity field)\n"),
-         cs_glob_stokes_model->iccvfg);
+         cs_glob_time_step->dt_ref);
 
+    }
   }
 }
 
