@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,6 +45,19 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Time stepping algorithme
+ *----------------------------------------------------------------------------*/
+
+typedef enum {
+
+  CS_TIME_STEP_STEADY = -1,
+  CS_TIME_STEP_CONSTANT = 0,
+  CS_TIME_STEP_ADAPTIVE = 1,
+  CS_TIME_STEP_LOCAL = 2
+
+} cs_time_step_type_t;
+
 /* time step descriptor */
 /*----------------------*/
 
@@ -67,36 +80,22 @@ typedef struct {
   double        t_cur;        /* current absolute time */
   double        t_max;        /* maximum absolute time */
 
-} cs_time_step_t;
+  double        dt[3];        /* n, n-1, and n-2 time steps */
+  double        dt_ref;       /* reference time step. */
+  double        dt_next;      /* next (predicted) time step. */
 
+} cs_time_step_t;
 
 /* Time step options descriptor */
 /*------------------------------*/
 
 typedef struct {
 
-
-  int       inpdt0; /* Indicator "zero time step"
-                       - 0: standard calculation
-                       - 1: to simulate no time step
-                       - for non-restarted computations: only resolution
-                       (Navier-Stokes, turbulence, scalars) is skipped
-                       - for restarted computations: resolution, computation
-                       of physical properties, and definition of boundary
-                       conditions is skipped (values are read from checkpoint
-                       file). */
-
   int       iptlro; /* Clip the time step with respect to the buoyant effects
                        - 0: false
                        - 1: true. */
 
-  int       idtvar; /* Option for a variable time step
-                       - -1: steady algorithm
-                       -  0: constant time step
-                       -  1: time step constant in space but variable in time
-                       -  2: variable time step in space and in time. */
-
-  double    dtref;  /* Reference time step. */
+  cs_time_step_type_t   idtvar; /* time step type (constant, adaptive, steady) */
 
   double    coumax; /* Maximum Courant number (when idtvar is
                        different from 0). */
@@ -104,16 +103,19 @@ typedef struct {
   double    cflmmx; /* Maximum Courant number for the continuity equation
                        in compressible model. */
 
-  double    foumax; /* Maximum Fourier number (when idtvar is different from 0). */
+  double    foumax; /* Maximum Fourier number
+                       (when idtvar is different from CS_TIME_STEP_CONSTANT). */
 
-  double    varrdt; /* Relative allowed variation of dt (when idtvar is
-                       different from 0). */
+  double    varrdt; /* Relative allowed variation of dt
+                       (when idtvar is different from CS_TIME_STEP_CONSTANT). */
 
-  double    dtmin;  /* Minimum value of dt (when idtvar is different from 0).
+  double    dtmin;  /* Minimum value of dt
+                       (when idtvar is different from CS_TIME_STEP_CONSTANT).
                        Take
                        dtmin = min(ld/ud, sqrt(lt/(gdelta rho/rho)), ...). */
 
-  double    dtmax;  /* Maximum value of dt (when idtvar is different from 0).
+  double    dtmax;  /* Maximum value of dt
+                       (when idtvar is different from CS_TIME_STEP_CONSTANT).
                        Take
                        dtmax = max(ld/ud, sqrt(lt/(gdelta rho/rho)), ...). */
 
@@ -136,18 +138,20 @@ extern const cs_time_step_options_t  *cs_glob_time_step_options;
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- * Provide acces to cs_glob_time_step
+ * Provide read/write access to cs_glob_time_step
  *
- * needed to initialize structure with GUI
+ * returns:
+ *   pointer to global time step structure
  *----------------------------------------------------------------------------*/
 
 cs_time_step_t *
 cs_get_glob_time_step(void);
 
 /*----------------------------------------------------------------------------
- * Provide acces to cs_glob_time_step_options
+ * Provide read/write access to cs_glob_time_step_options
  *
- * needed to initialize structure with GUI
+ * returns:
+ *  pointer to global time step options structure
  *----------------------------------------------------------------------------*/
 
 cs_time_step_options_t *
