@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -179,7 +179,7 @@ _cs_timer_cpu_stdc_clock(cs_timer_t  *timer);
  * Local static variable definitions
  *-----------------------------------------------------------------------------*/
 
-static _Bool _cs_timer_initialized = false;
+static bool  _cs_timer_initialized = false;
 static cs_timer_method_t _cs_timer_wall_method = CS_TIMER_DISABLE;
 static cs_timer_method_t _cs_timer_cpu_method = CS_TIMER_DISABLE;
 static _cs_timer_wall_func_t *_cs_timer_wall = _cs_timer_wall_null;
@@ -646,27 +646,6 @@ cs_timer_diff(const cs_timer_t  *t0,
   return retval;
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Add the the difference between 2 timers to a counter.
- *
- * \param[in, out]  tc  pointer to timer counter
- * \param[in]       t0  oldest timer value
- * \param[in]       t1 most recent timer value
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_timer_counter_add_diff(cs_timer_counter_t  *tc,
-                          const cs_timer_t    *t0,
-                          const cs_timer_t    *t1)
-{
-  tc->wall_nsec +=    (t1->wall_sec - t0->wall_sec) * (long long)1000000000
-                     + t1->wall_nsec - t0->wall_nsec;
-  tc->cpu_nsec +=    (t1->cpu_sec - t0->cpu_sec) * (long long)1000000000
-                    + t1->cpu_nsec - t0->cpu_nsec;
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Return method used to return wall clock time.
