@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -47,6 +47,8 @@
 #include "cs_base.h"
 #include "cs_boundary_zone.h"
 #include "cs_coupling.h"
+#include "cs_cell_to_vertex.h"
+#include "cs_ext_neighborhood.h"
 #include "cs_gradient.h"
 #include "cs_gui.h"
 #include "cs_gui_mesh.h"
@@ -137,10 +139,10 @@ cs_turbomachinery_t  *_turbomachinery = NULL;
  * (descriptions follow, with function bodies).
  *============================================================================*/
 
-void cs_f_map_turbomachinery_model(cs_int_t    *iturbo,
-                                   cs_int_t    *ityint);
+void cs_f_map_turbomachinery_model(int  *iturbo,
+                                   int  *ityint);
 
-void cs_f_map_turbomachinery_rotor(int       **irotce);
+void cs_f_map_turbomachinery_rotor(int  **irotce);
 
 /*============================================================================
  * Private function definitions
@@ -559,7 +561,7 @@ _copy_mesh(const cs_mesh_t  *mesh,
   mesh_copy->n_groups = mesh->n_groups;
 
   if (mesh->n_groups > 0) {
-    BFT_MALLOC(mesh_copy->group_idx, mesh->n_groups + 1, cs_lnum_t);
+    BFT_MALLOC(mesh_copy->group_idx, mesh->n_groups + 1, int);
     memcpy(mesh_copy->group_idx, mesh->group_idx,
            (mesh->n_groups + 1)*sizeof(cs_lnum_t));
     BFT_MALLOC(mesh_copy->group, mesh->group_idx[mesh->n_groups], char);
@@ -572,22 +574,28 @@ _copy_mesh(const cs_mesh_t  *mesh,
 
   n_elts = mesh->n_families*mesh->n_max_family_items;
   if (n_elts > 0) {
-    BFT_MALLOC(mesh_copy->family_item, n_elts , cs_lnum_t);
-    memcpy(mesh_copy->family_item, mesh->family_item, n_elts*sizeof(cs_lnum_t));
+    BFT_MALLOC(mesh_copy->family_item, n_elts , int);
+    memcpy(mesh_copy->family_item, mesh->family_item, n_elts*sizeof(int));
   }
 
-  BFT_MALLOC(mesh_copy->cell_family, mesh->n_cells_with_ghosts, cs_lnum_t);
+  BFT_MALLOC(mesh_copy->cell_family, mesh->n_cells_with_ghosts, int);
   memcpy(mesh_copy->cell_family, mesh->cell_family,
-         mesh->n_cells_with_ghosts*sizeof(cs_lnum_t));
+         mesh->n_cells_with_ghosts*sizeof(int));
 
-  BFT_MALLOC(mesh_copy->i_face_family, mesh->n_i_faces, cs_lnum_t);
+  BFT_MALLOC(mesh_copy->i_face_family, mesh->n_i_faces, int);
   memcpy(mesh_copy->i_face_family, mesh->i_face_family,
-         mesh->n_i_faces*sizeof(cs_lnum_t));
+         mesh->n_i_faces*sizeof(int));
 
   if (mesh->n_b_faces > 0) {
-    BFT_MALLOC(mesh_copy->b_face_family, mesh->n_b_faces, cs_lnum_t);
+    BFT_MALLOC(mesh_copy->b_face_family, mesh->n_b_faces, int);
     memcpy(mesh_copy->b_face_family, mesh->b_face_family,
-           mesh->n_b_faces*sizeof(cs_lnum_t));
+           mesh->n_b_faces*sizeof(int));
+  }
+
+  if (mesh->i_face_r_gen != NULL) {
+    BFT_MALLOC(mesh_copy->i_face_r_gen, mesh->n_i_faces, char);
+    memcpy(mesh_copy->i_face_r_gen, mesh->i_face_r_gen,
+           mesh->n_i_faces);
   }
 }
 
@@ -906,10 +914,12 @@ _update_mesh(bool     restart_mode,
 
   if (restart_mode == false) {
 
-    int n_retry = CS_MIN(tbm->n_max_join_tries, 1);
+    int n_retry = CS_MAX(tbm->n_max_join_tries, 1);
     cs_lnum_t boundary_changed = 0;
     double eps_dt = 0.;
 
+    const cs_time_step_t *ts = cs_glob_time_step;
+
     do {
 
       n_retry -= 1;
@@ -994,8 +1004,7 @@ _update_mesh(bool     restart_mode,
                     (unsigned long long)cs_glob_mesh->n_g_b_faces);
         }
         else {
-          double dt = cs_glob_time_step->t_cur - cs_glob_time_step->t_prev;
-          eps_dt += tbm->dt_retry * dt;
+          eps_dt += tbm->dt_retry * ts->dt[0];
           bft_printf(_(join_err_fmt),
                      (unsigned long long)n_g_b_faces_ref,
                      (unsigned long long)cs_glob_mesh->n_g_b_faces);
@@ -1003,14 +1012,12 @@ _update_mesh(bool     restart_mode,
 
           /* Destroy previous global mesh and related entities */
 
-          cs_mesh_location_finalize();
           cs_mesh_quantities_destroy(cs_glob_mesh_quantities);
 
           cs_mesh_destroy(cs_glob_mesh);
 
           /* Create new global mesh and related entities */
 
-          cs_mesh_location_initialize();
           cs_glob_mesh = cs_mesh_create();
           cs_glob_mesh->verbosity = 0;
           cs_glob_mesh_builder = cs_mesh_builder_create();
@@ -1081,6 +1088,9 @@ _update_mesh(bool     restart_mode,
   cs_mesh_bad_cells_detect(cs_glob_mesh, cs_glob_mesh_quantities);
   cs_user_mesh_bad_cells_tag(cs_glob_mesh, cs_glob_mesh_quantities);
 
+  cs_ext_neighborhood_reduce(cs_glob_mesh,
+                             cs_glob_mesh_quantities);
+
   /* Initialize selectors and locations for the mesh */
 
   cs_mesh_init_selectors();
@@ -1114,6 +1124,8 @@ _update_mesh(bool     restart_mode,
 
   }
 
+  cs_gradient_free_quantities();
+  cs_cell_to_vertex_free();
   cs_mesh_adjacencies_update_mesh();
 
   /* Update linear algebra APIs relative to mesh */
@@ -1141,8 +1153,8 @@ _update_mesh(bool     restart_mode,
  *----------------------------------------------------------------------------*/
 
 void
-cs_f_map_turbomachinery_model(cs_int_t    *iturbo,
-                              cs_int_t    *ityint)
+cs_f_map_turbomachinery_model(int  *iturbo,
+                              int  *ityint)
 {
   if (_turbomachinery != NULL)
     *iturbo = _turbomachinery->model;
@@ -1163,7 +1175,7 @@ cs_f_map_turbomachinery_model(cs_int_t    *iturbo,
  *----------------------------------------------------------------------------*/
 
 void
-cs_f_map_turbomachinery_rotor(cs_int_t   **irotce)
+cs_f_map_turbomachinery_rotor(int  **irotce)
 {
   if (_turbomachinery != NULL)
     *irotce = _turbomachinery->cell_rotor_num;
@@ -1293,12 +1305,20 @@ cs_turbomachinery_join_add(const char  *sel_criteria,
   cs_glob_join_count++; /* Store number of joining (without periodic ones) */
   cs_glob_n_joinings++;
 
+  /* Set mesh modification type */
+
+  if (_turbomachinery != NULL) {
+    if (   _turbomachinery->model == CS_TURBOMACHINERY_TRANSIENT
+        && cs_glob_mesh->time_dep < CS_MESH_TRANSIENT_CONNECT)
+      cs_glob_mesh->time_dep = CS_MESH_TRANSIENT_CONNECT;
+  }
+
   return cs_glob_n_joinings;
 }
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Add a cs_join_t structure to the list of rotor/stator couplings.
+ * \brief Add a boundary coupling to the list of rotor/stator couplings.
  *
  * \param[in]  sel_criteria   boundary face selection criteria
  * \param[in]  tolerance      value of the search tolerance
@@ -1324,6 +1344,12 @@ cs_turbomachinery_coupling_add(const char  *sel_criteria,
 
   _turbomachinery->n_couplings += 1;
 
+  if (_turbomachinery != NULL) {
+    if (   _turbomachinery->model == CS_TURBOMACHINERY_TRANSIENT
+        && cs_glob_mesh->time_dep < CS_MESH_TRANSIENT_COORDS)
+      cs_glob_mesh->time_dep = CS_MESH_TRANSIENT_COORDS;
+  }
+
   return cs_sat_coupling_n_couplings();
 }
 
