@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -58,6 +58,7 @@
 
 #include "cs_log.h"
 #include "cs_mesh.h"
+#include "cs_mesh_quantities.h"
 #include "cs_turbulence_model.h"
 
 /*----------------------------------------------------------------------------
@@ -88,23 +89,32 @@ BEGIN_C_DECLS
   for concise syntax, as it is expected to be used in many places.
 
   \var  cs_wall_functions_t::iwallf
-        Indicates the type of wall function is used for the velocity
+        Indicates the type of wall function used for the velocity
         boundary conditions on a frictional wall.\n
-        - 0: no wall functions
-        - 1: one scale of friction velocities (power law)
-        - 2: one scale of friction velocities (log law)
-        - 3: two scales of friction velocities (log law)
-        - 4: two scales of friction velocities (log law) (scalable wall functions)
-        - 5: two scales of friction velocities (mixing length based on V. Driest analysis)
-        \ref iwallf is initialised to 2 for \ref iturb = 10, 40, 41 or 70
+        - CS_WALL_F_DISABLED: no wall functions
+        - CS_WALL_F_1SCALE_POWER: one scale of friction velocities (power law)
+        - CS_WALL_F_1SCALE_LOG: one scale of friction velocities (log law)
+        - CS_WALL_F_2SCALES_LOG: two scales of friction velocities (log law)
+        - CS_WALL_F_SCALABLE_2SCALES_LOG: two scales of friction velocities
+          (log law) (scalable wall functions)
+        - CS_WALL_F_2SCALES_VDRIEST: two scales of friction velocities
+          (mixing length based on V. Driest analysis)
+        - CS_WALL_F_2SCALES_SMOOTH_ROUGH: wall function unifying rough and smooth
+          friction regimes
+        - CS_WALL_F_2SCALES_CONTINUOUS: All \f$ y^+ \f$  for low Reynolds models\n
+        \ref iwallf is initialised to CS_WALL_F_1SCALE_LOG for \ref iturb = 10,
+          40, 41 or 70
         (mixing length, LES and Spalart Allmaras).\n
-        \ref iwallf is initialised to 0 for \ref iturb = 0, 32, 50 or 51\n
-        \ref iwallf is initialised to 3 for \ref iturb = 20, 21, 30, 31 or 60
-        (\f$k-\epsilon\f$, \f$R_{ij}-\epsilon\f$ LRR, \f$R_{ij}-\epsilon\f$ SSG and
-        \f$k-\omega\f$ SST models).\n
+        \ref iwallf is initialised to CS_WALL_F_DISABLED for \ref iturb = 0, 32,
+          50 or 51\n
+        \ref iwallf is initialised to CS_WALL_F_2SCALES_LOG for \ref iturb = 20,
+          21, 30, 31 or 60
+        (\f$k-\epsilon\f$, \f$R_{ij}-\epsilon\f$ LRR, \f$R_{ij}-\epsilon\f$ SSG
+          and \f$k-\omega\f$ SST models).\n
         The v2f model (\ref iturb=50) is not designed to use wall functions
         (the mesh must be low Reynolds).\n
-        The value \ref iwallf = 3 is not compatible with \ref iturb=0, 10, 40
+        The value \ref iwallf = CS_WALL_F_2SCALES_LOG is not compatible with
+          \ref iturb=0, 10, 40
         or 41 (laminar, mixing length and LES).\n
         Concerning the \f$k-\epsilon\f$ and \f$R_{ij}-\epsilon\f$ models, the
         two-scales model is usually at least as satisfactory as the one-scale
@@ -116,10 +126,15 @@ BEGIN_C_DECLS
         Useful if \ref iturb is different from 50.
   \var  cs_wall_functions_t::iwalfs
         wall functions for scalar
-        - 0: three layers (Arpaci and Larsen) or two layers (Prandtl-Taylor) for
+        - CS_WALL_F_S_ARPACI_LARSEN: three layers (Arpaci and Larsen) or two layers (Prandtl-Taylor) for
              Prandtl number smaller than 0.1
-        - 1: consistant with the 2 scales wall function for velocity using Van
+        - CS_WALL_F_S_VDRIEST: consistant with the 2 scales wall function for velocity using Van
              Driest mixing length
+        - CS_WALL_F_S_LOUIS: default wall function for atmospheric flows for
+             potential temperature. This has influence on the dynamic.
+        - CS_WALL_F_S_MONIN_OBUKHOV: Monin Obukhov wall function for atmospheric flows for
+             potential temperature. This has influence on the dynamic.
+
   \var  cs_wall_functions_t::iwallt
         exchange coefficient correlation
         - 0: not use by default
@@ -128,7 +143,7 @@ BEGIN_C_DECLS
         limit value of \f$y^+\f$ for the viscous sublayer
 
         \ref ypluli depends on the chosen wall function: it is initialized to
-        10.88 for the scalable wall function (\ref iwallf=4), otherwise it is
+        10.88 for the scalable wall function (\ref iwallf=CS_WALL_F_SCALABLE_2SCALES_LOG), otherwise it is
         initialized to \f$1/\kappa\approx 2,38\f$. In LES, \ref ypluli is taken
         by default to be 10.88. Always useful.
 
@@ -144,7 +159,12 @@ BEGIN_C_DECLS
 /* wall functions structure and associated pointer */
 
 static cs_wall_functions_t  _wall_functions =
-  {-999, -999, 0, -1e13};
+{
+  .iwallf = -999,
+  .iwalfs = -999,
+  .iwallt = 0,
+  .ypluli = -1e13
+};
 
 const cs_wall_functions_t  * cs_glob_wall_functions = &_wall_functions;
 
@@ -200,16 +220,16 @@ cs_f_wall_functions_get_pointers(int     **iwallf,
 
 void CS_PROCF (wallfunctions, WALLFUNCTIONS)
 (
- const cs_int_t   *const iwallf,
+ const int        *const iwallf,
  const cs_lnum_t  *const ifac,
  const cs_real_t  *const l_visc,
  const cs_real_t  *const t_visc,
  const cs_real_t  *const vel,
  const cs_real_t  *const y,
- const cs_real_t  *const roughness,
+ const cs_real_t  *const rough_d,
  const cs_real_t  *const rnnb,
  const cs_real_t  *const kinetic_en,
-       cs_int_t         *iuntur,
+       int              *iuntur,
        cs_lnum_t        *nsubla,
        cs_lnum_t        *nlogla,
        cs_real_t        *ustar,
@@ -228,7 +248,7 @@ void CS_PROCF (wallfunctions, WALLFUNCTIONS)
                              *t_visc,
                              *vel,
                              *y,
-                             *roughness,
+                             *rough_d,
                              *rnnb,
                              *kinetic_en,
                              iuntur,
@@ -248,9 +268,12 @@ void CS_PROCF (wallfunctions, WALLFUNCTIONS)
 
 void CS_PROCF (hturbp, HTURBP)
 (
- const cs_int_t   *const iwalfs,
+ const int        *const iwalfs,
+ const cs_real_t  *const l_visc,
  const cs_real_t  *const prl,
  const cs_real_t  *const prt,
+ const cs_real_t  *const rough_t,
+ const cs_real_t  *const uk,
  const cs_real_t  *const yplus,
  const cs_real_t  *const dplus,
        cs_real_t        *htur,
@@ -258,8 +281,11 @@ void CS_PROCF (hturbp, HTURBP)
 )
 {
   cs_wall_functions_scalar((cs_wall_f_s_type_t)*iwalfs,
+                           *l_visc,
                            *prl,
                            *prt,
+                           *rough_t,
+                           *uk,
                            *yplus,
                            *dplus,
                            htur,
@@ -271,7 +297,7 @@ void CS_PROCF (hturbp, HTURBP)
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- *! \brief Provide acces to cs_glob_wall_functions
+ *! \brief Provide access to cs_glob_wall_functions
  *
  * needed to initialize structure with GUI
  *----------------------------------------------------------------------------*/
@@ -283,28 +309,22 @@ cs_get_glob_wall_functions(void)
 }
 
 /*----------------------------------------------------------------------------*/
-
-/*! \brief Compute the friction velocity and \f$y^+\f$ / \f$u^+\f$.
-
-*/
-/*------------------------------------------------------------------------------
-  Arguments
- ______________________________________________________________________________.
-   mode           name          role                                           !
- _____________________________________________________________________________*/
 /*!
+ * \brief Compute the friction velocity and \f$y^+\f$ / \f$u^+\f$.
+ *
  * \param[in]     iwallf        wall function type
  * \param[in]     ifac          face number
  * \param[in]     l_visc        kinematic viscosity
  * \param[in]     t_visc        turbulent kinematic viscosity
  * \param[in]     vel           wall projected cell center velocity
  * \param[in]     y             wall distance
- * \param[in]     roughness     roughness
+ * \param[in]     rough_d       roughness length scale
+ *                              (not sand grain roughness)
  * \param[in]     rnnb          \f$\vec{n}.(\tens{R}\vec{n})\f$
- * \param[in]     kinetic_en    turbulente kinetic energy
- * \param[in]     iuntur        indicator: 0 in the viscous sublayer
- * \param[in]     nsubla        counter of cell in the viscous sublayer
- * \param[in]     nlogla        counter of cell in the log-layer
+ * \param[in]     kinetic_en    turbulent kinetic energy (cell center)
+ * \param[in,out] iuntur        indicator: 0 in the viscous sublayer
+ * \param[in,out] nsubla        counter of cell in the viscous sublayer
+ * \param[in,out] nlogla        counter of cell in the log-layer
  * \param[out]    ustar         friction velocity
  * \param[out]    uk            friction velocity
  * \param[out]    yplus         dimensionless distance to the wall
@@ -323,7 +343,7 @@ cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
                            cs_real_t         t_visc,
                            cs_real_t         vel,
                            cs_real_t         y,
-                           cs_real_t         roughness,
+                           cs_real_t         rough_d,
                            cs_real_t         rnnb,
                            cs_real_t         kinetic_en,
                            int              *iuntur,
@@ -345,6 +365,20 @@ cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
   /* Activation of wall function by default */
   *iuntur = 1;
 
+  /* Get the adjacent border cell and its fluid/solid tag */
+  cs_lnum_t cell_id = cs_glob_mesh->b_face_cells[ifac-1] ;
+
+  /* If the cell is a solid cell, disable wall functions */
+
+  const cs_mesh_quantities_t *mq = cs_glob_mesh_quantities;
+  if (mq->has_disable_flag) {
+    if (mq->c_disable_flag[cell_id])
+      iwallf = CS_WALL_F_DISABLED;
+  }
+
+  /* Sand Grain roughness */
+  cs_real_t sg_rough = rough_d * exp(cs_turb_xkappa*cs_turb_cstlog_rough);
+
   switch (iwallf) {
   case CS_WALL_F_DISABLED:
     cs_wall_functions_disabled(l_visc,
@@ -420,7 +454,7 @@ cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
                                        cofimp);
     break;
   case CS_WALL_F_2SCALES_VDRIEST:
-    cs_wall_functions_2scales_vdriest(rnnb,
+   cs_wall_functions_2scales_vdriest(rnnb,
                                       l_visc,
                                       vel,
                                       y,
@@ -434,7 +468,7 @@ cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
                                       ypup,
                                       cofimp,
                                       &lmk,
-                                      roughness,
+                                      sg_rough,
                                       wf);
     break;
   case CS_WALL_F_2SCALES_SMOOTH_ROUGH:
@@ -442,7 +476,7 @@ cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
                                            t_visc,
                                            vel,
                                            y,
-                                           roughness,
+                                           rough_d,
                                            kinetic_en,
                                            iuntur,
                                            nsubla,
@@ -475,11 +509,10 @@ cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
   }
 }
 
-/*-------------------------------------------------------------------------------*/
-
+/*----------------------------------------------------------------------------*/
 /*!
- *  \brief Compute the correction of the exchange coefficient between the fluid and
- *  the wall for a turbulent flow.
+ *  \brief Compute the correction of the exchange coefficient between the
+ *         fluid and the wall for a turbulent flow.
  *
  *  This is function of the dimensionless
  *  distance to the wall \f$ y^+ = \dfrac{\centip \centf u_\star}{\nu}\f$.
@@ -489,28 +522,27 @@ cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
  *  h_{tur} = Pr \dfrac{y^+}{T^+}
  *  \f]
  *
- */
-/*-------------------------------------------------------------------------------
-  Arguments
- ______________________________________________________________________________.
-   mode           name          role                                           !
- ______________________________________________________________________________*/
-/*!
  * \param[in]     iwalfs        type of wall functions for scalar
+ * \param[in]     l_visc        kinematic viscosity
  * \param[in]     prl           laminar Prandtl number
  * \param[in]     prt           turbulent Prandtl number
+ * \param[in]     rough_t       scalar roughness lenghth scale
+ * \param[in]     uk            velocity scale based on TKE
  * \param[in]     yplus         dimensionless distance to the wall
  * \param[in]     dplus         dimensionless distance for scalable
  *                              wall functions
  * \param[out]    htur          corrected exchange coefficient
  * \param[out]    yplim         value of the limit for \f$ y^+ \f$
  */
-/*-------------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------------*/
 
 void
 cs_wall_functions_scalar(cs_wall_f_s_type_t  iwalfs,
+                         cs_real_t           l_visc,
                          cs_real_t           prl,
                          cs_real_t           prt,
+                         cs_real_t           rough_t,
+                         cs_real_t           uk,
                          cs_real_t           yplus,
                          cs_real_t           dplus,
                          cs_real_t          *htur,
@@ -518,8 +550,11 @@ cs_wall_functions_scalar(cs_wall_f_s_type_t  iwalfs,
 {
   switch (iwalfs) {
   case CS_WALL_F_S_ARPACI_LARSEN:
-    cs_wall_functions_s_arpaci_larsen(prl,
+    cs_wall_functions_s_arpaci_larsen(l_visc,
+                                      prl,
                                       prt,
+                                      rough_t,
+                                      uk,
                                       yplus,
                                       dplus,
                                       htur,
@@ -531,7 +566,31 @@ cs_wall_functions_scalar(cs_wall_f_s_type_t  iwalfs,
                                 yplus,
                                 htur);
     break;
+  case CS_WALL_F_S_SMOOTH_ROUGH:
+    cs_wall_functions_s_smooth_rough(l_visc,
+                                     prl,
+                                     prt,
+                                     rough_t,
+                                     uk,
+                                     yplus,
+                                     dplus,
+                                     htur);
+    break;
   default:
+    /* TODO Monin Obukhov or Louis atmospheric wall function
+     * must be adapted to smooth wall functions.
+     * Arpaci and Larsen walll functions are put as in previous versions of
+     * Code_Saturne.
+     * */
+    cs_wall_functions_s_arpaci_larsen(l_visc,
+                                      prl,
+                                      prt,
+                                      rough_t,
+                                      uk,
+                                      yplus,
+                                      dplus,
+                                      htur,
+                                      yplim);
     break;
   }
 }
