@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -88,6 +88,7 @@ use numvar
 use optcal
 use cstphy
 use cstnum
+use dimens, only: nvar
 use entsor
 use parall
 use period
@@ -105,12 +106,12 @@ implicit none
 integer          nvcp   , nvcpto
 integer          nfbcpl , nfbncp
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          lfbcpl(nfbcpl)  , lfbncp(nfbncp)
 integer          itypfb(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 double precision rvcpfb(nfbcpl,nvcpto), pndcpl(nfbcpl)
 double precision dofcpl(3,nfbcpl)
 
@@ -144,8 +145,11 @@ call field_get_val_v(ivarfl(iu), vel)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
 call field_get_val_s(iflmab, bmasfl)
 
+! No face to face assumption
 if (ifaccp.eq.0) then
   icscp = icscpl
+
+! Face to face mapping assumption
 else
   icscp = icscpd
 endif
@@ -176,13 +180,13 @@ do ivar = 1, nvcp
   if (ivar.ne.iu .and. ivar.ne.iv .and. ivar.ne.iw) then
 
     call field_get_val_s(ivarfl(ivar), cvar_var)
-    call field_gradient_scalar(ivarfl(ivar), iprev, imrgra, inc,   &
-                               iccocg,                             &
+    call field_gradient_scalar(ivarfl(ivar), iprev, 0, inc,   &
+                               iccocg,                        &
                                grad)
 
   else if (ivar.eq.iu) then
 
-    call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc,  &
+    call field_gradient_vector(ivarfl(iu), iprev, 0, inc,  &
                                gradv)
 
   endif
@@ -428,3 +432,145 @@ deallocate(grad)
 
 return
 end subroutine csc2cl
+
+!===============================================================================
+
+!> \brief Initialization of the "itypfb(*, *) = icscpl" condition.
+
+!------------------------------------------------------------------------------
+! Arguments
+!------------------------------------------------------------------------------
+!   mode          name          role
+!------------------------------------------------------------------------------
+!> \param[in]     nvcp
+!> \param[in]     nfbcpl
+!> \param[in]     nfbncp
+!> \param[out]    icodcl        face boundary condition code:
+!>                               - 1 Dirichlet
+!>                               - 2 Radiative outlet
+!>                               - 3 Neumann
+!>                               - 4 sliding and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 5 smooth wall and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 6 rough wall and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 9 free inlet/outlet
+!>                                 (input mass flux blocked to 0)
+!>                               - 13 Dirichlet for the advection operator and
+!>                                    Neumann for the diffusion operator
+!> \param[in]     lfbcpl
+!> \param[in]     lfbncp
+!> \param[out]    itypfb        boundary face types
+!______________________________________________________________________________
+
+subroutine csc2cl_init &
+ (nvcp, nfbcpl, nfbncp, icodcl, itypfb, lfbcpl, lfbncp)
+
+!===============================================================================
+! Module files
+!===============================================================================
+
+use paramx
+use numvar
+use optcal
+use cstphy
+use cstnum
+use dimens, only: nvar
+use entsor
+use parall
+use period
+use cplsat
+use mesh
+use field
+use field_operator
+
+!===============================================================================
+
+implicit none
+
+! Arguments
+
+integer          nvcp
+integer          nfbcpl , nfbncp
+
+integer          icodcl(nfabor,nvar)
+integer          lfbcpl(nfbcpl)  , lfbncp(nfbncp)
+integer          itypfb(nfabor)
+
+! Local variables
+
+integer          ifac, icscp
+integer          ivar
+integer          ipt
+
+!===============================================================================
+
+!===============================================================================
+! 1.  Translation of the coupling to boundary conditions
+!===============================================================================
+
+! No face to face assumption
+if (ifaccp.eq.0) then
+  icscp = icscpl
+
+! Face to face mapping assumption
+else
+  icscp = icscpd
+endif
+
+! Reminder: variables are received in the order of VARPOS;
+! looping on variables is thus sufficient.
+
+do ivar = 1, nvcp
+
+  if (ifaccp.eq.1) then
+
+    do ipt = 1, nfbcpl
+
+      ifac = lfbcpl(ipt)
+
+      itypfb(ifac) = icscp
+      icodcl(ifac,ivar) = 1
+
+    enddo
+
+  else
+
+    do ipt = 1, nfbcpl
+
+      ifac = lfbcpl(ipt)
+
+      itypfb(ifac) = icscp
+
+      if (ivar.ne.ipr) then
+        icodcl(ifac, ivar) = 1
+      else
+        icodcl(ifac, ivar) = 3
+      endif
+
+    enddo
+
+  endif
+
+  ! Non-located boundary faces
+
+  do ipt = 1, nfbncp
+
+    ifac = lfbncp(ipt)
+
+    itypfb(ifac) = icscp
+
+    icodcl(ifac,ivar  ) = 3
+
+  enddo
+
+enddo
+
+!----
+! End
+!----
+
+return
+end subroutine csc2cl_init
+
