@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -59,7 +59,6 @@ use pointe
 use albase
 use parall
 use period
-use ihmpre
 use ppppar
 use ppthch
 use ppincl
@@ -79,15 +78,15 @@ integer          nscal , iterns
 
 integer          ivar  , iel   , ifac  , iscal, f_id0
 integer          nswrgp, imligp, iwarnp
-integer          iccocg, inc
+integer          iccocg, inc, imrgrp
 integer          iconvp, idiffp, ircflp
 integer          ischcp, isstpp
-integer          ifcvsl, iflmas, iflmab
+integer          iscacp, ifcvsl, iflmas, iflmab
 integer          imucpp, idftnp, imasac
 integer          key_buoyant_id, is_buoyant_fld
 double precision epsrgp, climgp, extrap
 double precision blencp, relaxp, thetex
-double precision turb_schmidt
+double precision turb_schmidt, visls_0
 
 integer          icvflb
 integer          ivoid(1)
@@ -118,24 +117,26 @@ do iscal = 1, nscal
   ! Index for variable
   ivar = isca(iscal)
 
-  call field_get_val_s(ivarfl(isca(iscal)), cvar_scal)
+  call field_get_val_s(ivarfl(ivar), cvar_scal)
 
   ! Key id for buoyant field (inside the Navier Stokes loop)
   call field_get_key_id("is_buoyant", key_buoyant_id)
-  call field_get_key_int(ivarfl(isca(iscal)), key_buoyant_id, is_buoyant_fld)
+  call field_get_key_int(ivarfl(ivar), key_buoyant_id, is_buoyant_fld)
 
   ! If the scalar is buoyant, it is inside the Navier Stokes loop, and so iterns >=1
   ! otherwise it is outside of the loop and iterns = -1.
   if (  (is_buoyant_fld.eq. 1 .and. iterns.eq.-1) &
-    .or.(is_buoyant_fld.eq.-1 .and. iterns.ne.-1)) cycle
+    .or.(is_buoyant_fld.eq. 0 .and. iterns.ne.-1)) cycle
 
   imucpp = 0
   if (iscavr(iscal).gt.0) then
-    if (abs(iscacp(iscavr(iscal))).eq.1) then
+    call field_get_key_int(ivarfl(isca(iscavr(iscal))), kscacp, iscacp)
+    if (iscacp.eq.1) then
       imucpp = 1
     endif
   else
-    if (abs(iscacp(iscal)).eq.1) then
+    call field_get_key_int(ivarfl(ivar), kscacp, iscacp)
+    if (iscacp.eq.1) then
       imucpp = 1
     endif
   endif
@@ -170,6 +171,7 @@ do iscal = 1, nscal
   blencp = 0.d0
   imasac = 0
   idiffp = 1
+  imrgrp = vcopt%imrgra
   nswrgp = vcopt%nswrgr
   imligp = vcopt%imligr
   ircflp = vcopt%ircflu
@@ -211,9 +213,10 @@ do iscal = 1, nscal
 
     call field_get_key_double(ivarfl(isca(iscal)), ksigmas, turb_schmidt)
 
-    if(ifcvsl.lt.0)then
+    if (ifcvsl.lt.0)then
+      call field_get_key_double(ivarfl(isca(iscal)), kvisl0, visls_0)
       do iel = 1, ncel
-        vistot(iel) = visls0(iscal)                                     &
+        vistot(iel) = visls_0                                           &
            + vcopt%idifft*xcpp(iel)*max(visct(iel),zero)/turb_schmidt
       enddo
     else
@@ -224,7 +227,6 @@ do iscal = 1, nscal
     endif
 
     call viscfa ( imvisf , vistot , viscf , viscb )
-    !==========
 
   else
 
@@ -250,9 +252,8 @@ do iscal = 1, nscal
   call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
   call bilsca &
-  !==========
   ( idtvar , f_id0  , iconvp , idiffp , nswrgp , imligp , ircflp , &
-    ischcp , isstpp , inc    , imrgra , iccocg ,                   &
+    ischcp , isstpp , inc    , imrgrp , iccocg ,                   &
     iwarnp , imucpp , idftnp , imasac ,                            &
     blencp , epsrgp , climgp , extrap , relaxp , thetex ,          &
     cvar_scal       , cvar_scal       ,                            &
