@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -133,17 +133,6 @@ write(nfecra,1000) dismin, dismax
 ! Formats
 !===============================================================================
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(                                                           &
-'                                                             ',/,&
-' ** DISTANCE A LA PAROI                                      ',/,&
-'    -------------------                                      ',/,&
-'                                                             ',/,&
-'   Distance min = ',E14.5    ,'  Distance max = ',E14.5      ,/)
-
-#else
-
  1000 format(                                                           &
 '                                                             ',/,&
 ' ** WALL DISTANCE                                            ',/,&
@@ -151,8 +140,6 @@ write(nfecra,1000) dismin, dismax
 '                                                             ',/,&
 '  Min distance = ',E14.5    ,' Max distance = ',E14.5         ,/)
 
-#endif
-
 !----
 ! End
 !----
