@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -39,22 +39,21 @@
 !> \param[in]     dt            time step (per cell)
 !> \param[in,out] imasfl        scalar mass flux at interior face centers
 !> \param[in,out] bmasfl        scalar mass flux at boundary face centers
-!> \param[in,out] rovsdt        unsteady term and mass aggregation term
-!> \param[in,out] smbrs         right hand side for the scalar iscal
+!> \param[in,out] divflu        divergence of drift flux
 !______________________________________________________________________________
 
 subroutine driflu &
 ( iflid  ,                                                       &
   dt     ,                                                       &
   imasfl , bmasfl ,                                              &
-  rovsdt , smbrs  )
+  divflu  )
 
 !===============================================================================
 ! Module files
 !===============================================================================
 
 use paramx
-use dimens, only: ndimfb, nvar
+use dimens, only: nvar, ndimfb
 use numvar
 use entsor
 use optcal
@@ -81,8 +80,8 @@ implicit none
 integer          iflid
 
 double precision dt(ncelet)
-double precision imasfl(nfac), bmasfl(ndimfb)
-double precision rovsdt(ncelet), smbrs(ncelet)
+double precision imasfl(nfac), bmasfl(nfabor)
+double precision divflu(ncelet)
 
 ! Local variables
 
@@ -90,7 +89,7 @@ integer          ivar
 integer          ifac  , iel
 integer          init  , inc   , iccocg
 integer          ifcvsl, iflmas, iflmab
-integer          nswrgp, imligp, iwarnp
+integer          imrgrp, nswrgp, imligp, iwarnp
 integer          iconvp, idiffp
 integer          ircflp, ischcp, isstpp
 integer          isou  , jsou
@@ -100,9 +99,10 @@ integer          keysca, iscal, keydri, iscdri, icvflb
 integer          keyccl
 integer          icla, jcla, jvar
 integer          ivoid(1)
-integer          ielup, id_x1, id_vdp_i, imasac
+integer          ielup, id_x1, id_vdp_i, imasac, id_pro, id_drift
 
 double precision epsrgp, climgp, extrap, blencp
+double precision visls_0
 double precision thetap
 double precision rhovdt
 double precision omegaa
@@ -113,7 +113,7 @@ double precision rvoid(1)
 
 character(len=80) :: fname
 
-double precision, dimension(:), allocatable :: w1, viscce
+double precision, dimension(:), allocatable :: w1, viscce, rtrace
 double precision, dimension(:), allocatable :: coefap, coefbp
 double precision, dimension(:), allocatable :: cofafp, cofbfp
 double precision, dimension(:,:), allocatable :: coefa1
@@ -131,14 +131,17 @@ double precision, dimension(:), pointer :: imasfl_mix, bmasfl_mix
 double precision, dimension(:,:), pointer :: vdp_i
 double precision, dimension(:), pointer :: x2
 double precision, dimension(:), pointer :: imasfl_gas, bmasfl_gas
-double precision, dimension(:), pointer :: x1, b_x1
+double precision, dimension(:), pointer :: cpro_x1, bpro_x1
 double precision, dimension(:), pointer :: brom, crom
 double precision, dimension(:,:), pointer :: vel, vela
 double precision, dimension(:), pointer :: cvar_k
 double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
+double precision, dimension(:,:), pointer :: cvar_rij
 double precision, dimension(:), pointer :: visct, cpro_viscls
 double precision, dimension(:), pointer :: cvara_var
 
+type(var_cal_opt) :: vcopt, vcopt_u
+
 !===============================================================================
 
 !===============================================================================
@@ -168,10 +171,10 @@ call field_get_key_int(iflid, keydri, iscdri)
 ! Massic fraction of gas
 call field_get_id_try("x_c", id_x1)
 if (id_x1.ne.-1) then
-  call field_get_val_s(id_x1, x1)
+  call field_get_val_s(id_x1, cpro_x1)
 
   ! Mass fraction of the gas at the boundary
-  call field_get_val_s_by_name("b_x_c", b_x1)
+  call field_get_val_s_by_name("b_x_c", bpro_x1)
 
   ! Get the mass flux of the continuous phase (gas)
   ! that is the negative scalar class
@@ -188,6 +191,9 @@ if (id_x1.ne.-1) then
 
 endif
 
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+
 ! Pointers to the mass fluxes of the mix (based on mix velocity)
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
@@ -205,12 +211,16 @@ call field_get_val_prev_v(ivarfl(iu), vela)
 ! --- Physical properties
 call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(ivisct, visct)
 
 if (itytur.eq.3) then
-  call field_get_val_s(ivarfl(ir11), cvar_r11)
-  call field_get_val_s(ivarfl(ir22), cvar_r22)
-  call field_get_val_s(ivarfl(ir33), cvar_r33)
+  if (irijco.eq.1) then
+    call field_get_val_v(ivarfl(irij), cvar_rij)
+  else
+    call field_get_val_s(ivarfl(ir11), cvar_r11)
+    call field_get_val_s(ivarfl(ir22), cvar_r22)
+    call field_get_val_s(ivarfl(ir33), cvar_r33)
+  endif
 elseif (itytur.eq.2 .or. itytur.eq.5 .or. iturb.eq.60) then
   call field_get_val_s(ivarfl(ik), cvar_k)
 endif
@@ -235,12 +245,12 @@ allocate(flumas(nfac), flumab(nfabor))
 
 if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
   ! Index of the corresponding relaxation time (cpro_taup)
-  call field_get_id('drift_tau_'//trim(fname), f_id)
-  call field_get_val_s(f_id, cpro_taup)
+  call field_get_id_try('drift_tau_'//trim(fname), id_pro)
+  if (id_pro.ne.-1) call field_get_val_s(id_pro, cpro_taup)
 
   ! Index of the corresponding relaxation time (cpro_taup)
-  call field_get_id('drift_vel_'//trim(fname), f_id)
-  call field_get_val_v(f_id, cpro_drift)
+  call field_get_id_try('drift_vel_'//trim(fname), id_drift)
+  if (id_drift.ne.-1) call field_get_val_v(id_drift, cpro_drift)
 
   ! Index of the corresponding interaction time particle--eddies (cpro_taufpt)
   if (btest(iscdri, DRIFT_SCALAR_TURBOPHORESIS)) then
@@ -262,7 +272,7 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
   ! Initialization of the gas "class" convective flux by the
   ! first particle "class":
   !  it is initialized by the mass flux of the bulk
-  if (icla.eq.1) then
+  if (icla.eq.1.and.id_x1.ne.-1) then
     do ifac = 1, nfac
       imasfl_gas(ifac) = imasfl_mix(ifac)
     enddo
@@ -277,23 +287,26 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
 !===============================================================================
 
   ! Test if a deviation velocity of particles class exists
-  write(fname,'(a,i2.2)')'vd_p_' ,icla
-  call field_get_id_try(fname, id_vdp_i)
 
-  if (icla.ge.1.and.id_vdp_i.ne.-1) then
+  if (icla.ge.1) then
 
-    call field_get_val_v(id_vdp_i, vdp_i)
+    write(fname,'(a,i2.2)')'vd_p_' ,icla
+    call field_get_id_try(fname, id_vdp_i)
 
-    do iel = 1, ncel
+    if (id_vdp_i.ne.-1) then
+      call field_get_val_v(id_vdp_i, vdp_i)
 
-      rho = crom(iel)
-      cpro_drift(1, iel) = rho*vdp_i(1, iel)
-      cpro_drift(2, iel) = rho*vdp_i(2, iel)
-      cpro_drift(3, iel) = rho*vdp_i(3, iel)
+      do iel = 1, ncel
 
-    enddo
+        rho = crom(iel)
+        cpro_drift(1, iel) = rho*vdp_i(1, iel)
+        cpro_drift(2, iel) = rho*vdp_i(2, iel)
+        cpro_drift(3, iel) = rho*vdp_i(3, iel)
 
-  else if (icla.ge.0) then
+      enddo
+    endif
+
+  else if (icla.ge.0.and.id_pro.ne.-1.and.id_drift.ne.-1) then
 
     do iel = 1, ncel
 
@@ -318,34 +331,38 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
   if (btest(iscdri, DRIFT_SCALAR_TURBOPHORESIS).and.iturb.ne.0) then
 
     ! The diagonal part is easy to implicit (Grad (K) . n = (K_j - K_i)/IJ)
-
-    ! Compute the K=1/3*trace(K) coefficient (diffusion of Zaichik)
+    ! Compute the K=1/3*trace(R) coefficient (diffusion of Zaichik)
 
     if (itytur.eq.3) then
 
-      do iel = 1, ncel
+      allocate(rtrace(ncel))
 
+      if (irijco.eq.1) then
+        do iel = 1, ncel
+          rtrace(iel) = cvar_rij(1,iel) + cvar_rij(2,iel) + cvar_rij(3,iel)
+        enddo
+      else
+        do iel = 1, ncel
+          rtrace(iel) = cvar_r11(iel) + cvar_r22(iel) + cvar_r33(iel)
+        enddo
+      endif
+
+      do iel = 1, ncel
         ! Correction by Omega
         omegaa = cpro_taup(iel)/cpro_taufpt(iel)
         ! FIXME: use idifft or not?
-        viscce(iel) = 1.d0/3.d0                                        &
-                    * cpro_taup(iel)/(1.d0+omegaa)*( cvar_r11(iel)     &
-                                                   + cvar_r22(iel)     &
-                                                   + cvar_r33(iel) )
+        viscce(iel) = 1.d0/3.d0 * cpro_taup(iel)/(1.d0+omegaa)*rtrace(iel)
       enddo
+      deallocate(rtrace)
 
     elseif (itytur.eq.2 .or. itytur.eq.5 .or. iturb.eq.60) then
 
       do iel = 1, ncel
-
         ! Correction by Omega
         omegaa = cpro_taup(iel)/cpro_taufpt(iel)
-
         viscce(iel) = 2.d0/3.d0*cpro_taup(iel)/(1.d0+omegaa)*cvar_k(iel)
       enddo
 
-    else
-
     endif
 
   endif
@@ -364,8 +381,10 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
 
     else
 
+      call field_get_key_double(ivarfl(ivar), kvisl0, visls_0)
+
       do iel = 1, ncel
-        viscce(iel) = viscce(iel) + visls0(iscal)/crom(iel)
+        viscce(iel) = viscce(iel) + visls_0/crom(iel)
       enddo
 
     endif
@@ -378,12 +397,13 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
     iphydp = 0
     inc    = 1
     iccocg = 1
-    nswrgp = nswrgr(ivar)
-    imligp = imligr(ivar)
-    iwarnp = iwarni(ivar)
-    epsrgp = epsrgr(ivar)
-    climgp = climgr(ivar)
-    extrap = extrag(ivar)
+    imrgrp = vcopt%imrgra
+    nswrgp = vcopt%nswrgr
+    imligp = vcopt%imligr
+    iwarnp = vcopt%iwarni
+    epsrgp = vcopt%epsrgr
+    climgp = vcopt%climgr
+    extrap = vcopt%extrag
 
     ! Face diffusivity of rho to compute rho*(Grad K . n)_face
     do iel = 1, ncel
@@ -413,15 +433,15 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
 
     ! The computed convective flux has the dimension of rho*velocity
     call itrmas &
-   ( f_id0  , init , inc , imrgra , iccocg , nswrgp , imligp , iphydp ,      &
-     iwarnp ,                                                                &
+   ( f_id0  , init , inc , imrgrp , iccocg , nswrgp , imligp , iphydp ,      &
+     0      , iwarnp ,                                                       &
      epsrgp , climgp , extrap ,                                              &
      rvoid  ,                                                                &
      viscce ,                                                                &
      coefap , coefbp ,                                                       &
      cofafp , cofbfp ,                                                       &
      viscf  , viscb  ,                                                       &
-     w1     , w1     , w1     ,                                              &
+     w1     ,                                                                &
      flumas , flumab )
 
   ! TODO add extradiagonal part
@@ -444,20 +464,21 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
 
     iconvp = 1
     idiffp = 0
-    nswrgp = nswrgr(iu)
-    imligp = imligr(iu)
-    ircflp = ircflu(iu)
-    ischcp = ischcv(iu)
-    isstpp = isstpc(iu)
+    imrgrp = vcopt_u%imrgra
+    nswrgp = vcopt_u%nswrgr
+    imligp = vcopt_u%imligr
+    ircflp = vcopt_u%ircflu
+    ischcp = vcopt_u%ischcv
+    isstpp = vcopt_u%isstpc
     inc    = 1
     ivisep = 0
-    iwarnp = iwarni(iu)
-    idftnp = idften(iu)
-    blencp = blencv(iu)
-    epsrgp = epsrgr(iu)
-    climgp = climgr(iu)
-    thetap = thetav(iu)
-    relaxp = relaxv(iu)
+    iwarnp = vcopt_u%iwarni
+    idftnp = vcopt_u%idften
+    blencp = vcopt_u%blencv
+    epsrgp = vcopt_u%epsrgr
+    climgp = vcopt_u%climgr
+    thetap = vcopt_u%thetav
+    relaxp = vcopt_u%relaxv
     icvflb = 0
 
     ! Reset viscf and viscb
@@ -483,15 +504,16 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
 
     ! Warning: bilsc adds "-( grad(u) . rho u)"
     call bilscv &
-   ( idtvar , iu     , iconvp , idiffp , nswrgp , imligp , ircflp , &
-     ischcp , isstpp , inc    , imrgra , ivisep ,                   &
-     iwarnp , idftnp , imasac ,                                     &
-     blencp , epsrgp , climgp , relaxp , thetap ,                   &
-     vel    , vel    ,                                              &
-     coefav , coefbv , cofafv , cofbfv ,                            &
-     imasfl_mix , bmasfl_mix ,                                      &
-     viscf  , viscb  , rvoid  , rvoid  ,                            &
-     icvflb , ivoid  ,                                              &
+   ( idtvar , ivarfl(iu)      , iconvp , idiffp , nswrgp , imligp , ircflp , &
+     ischcp , isstpp , inc    , imrgrp , ivisep ,                            &
+     iwarnp , idftnp , imasac ,                                              &
+     blencp , epsrgp , climgp , relaxp , thetap ,                            &
+     vel    , vel    ,                                                       &
+     coefav , coefbv , cofafv , cofbfv ,                                     &
+     imasfl_mix , bmasfl_mix ,                                               &
+     viscf  , viscb  , rvoid  , rvoid  ,                                     &
+     rvoid  , rvoid  , rvoid  ,                                              &
+     icvflb , ivoid  ,                                                       &
      dudt   )
 
     do iel = 1, ncel
@@ -521,34 +543,58 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
 !===============================================================================
 
   ! For all scalar with a drift excpted the gas phase which is deduced
-  if (icla.ge.0) then
+  ! And for those whom mass flux is imposed elsewhere
+  if (icla.ge.0.and.(.not.btest(iscdri, DRIFT_SCALAR_IMPOSED_MASS_FLUX))) then
 
-    ! Zero additional flux at the boundary
-    do ifac = 1, nfabor
-
-      do isou = 1, 3
-        coefa1(isou, ifac) = 0.d0
-        do jsou = 1, 3
-          coefb1(isou, jsou, ifac) = 0.d0
+    ! Homogeneous Neumann at the boundary
+    if (btest(iscdri, DRIFT_SCALAR_ZERO_BNDY_FLUX)) then
+      do ifac = 1, nfabor
+        do isou = 1, 3
+          coefa1(isou, ifac) = 0.d0
+          do jsou = 1, 3
+            coefb1(isou, jsou, ifac) = 0.d0
+          enddo
         enddo
       enddo
-
-    enddo
+    else if (btest(iscdri, DRIFT_SCALAR_ZERO_BNDY_FLUX_AT_WALLS)) then
+      do ifac = 1, nfabor
+        do isou = 1, 3
+          coefa1(isou, ifac) = 0.d0
+          do jsou = 1, 3
+            coefb1(isou, jsou, ifac) = 0.d0
+          enddo
+          if (itypfb(ifac).ne.iparoi .and. itypfb(ifac).ne.iparug) then
+            coefb1(isou, isou, ifac) = 1.d0
+          endif
+        enddo
+      enddo
+    else
+      do ifac = 1, nfabor
+        do isou = 1, 3
+          coefa1(isou, ifac) = 0.d0
+          do jsou = 1, 3
+            coefb1(isou, jsou, ifac) = 0.d0
+          enddo
+          coefb1(isou, isou, ifac) = 1.d0
+        enddo
+      enddo
+    endif
 
     init   = 0
     inc    = 1
     iflmb0 = 0
     itypfl = 0 ! drift has already been multiplied by rho
-    nswrgp = nswrgr(ivar)
-    imligp = imligr(ivar)
-    iwarnp = iwarni(ivar)
-    epsrgp = epsrgr(ivar)
-    climgp = climgr(ivar)
-    extrap = extrag(ivar)
+    imrgrp = vcopt%imrgra
+    nswrgp = vcopt%nswrgr
+    imligp = vcopt%imligr
+    iwarnp = vcopt%iwarni
+    epsrgp = vcopt%epsrgr
+    climgp = vcopt%climgr
+    extrap = vcopt%extrag
 
     call inimav &
      ( f_id0  , itypfl ,                                              &
-       iflmb0 , init   , inc    , imrgra , nswrgp , imligp ,          &
+       iflmb0 , init   , inc    , imrgrp , nswrgp , imligp ,          &
        iwarnp ,                                                       &
        epsrgp , climgp ,                                              &
        crom   , brom   ,                                              &
@@ -575,32 +621,36 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
     if (icla.ge.1) then
 
       write(fname,'(a,i2.2)')'x_p_' ,icla
-      call field_get_val_s_by_name(fname, x2)
+      call field_get_id_try(fname, f_id)
 
-      do ifac = 1, nfac
+      if (f_id.ne.-1) then
+        call field_get_val_s(f_id, x2)
 
-        ! Upwind value of x2 at the face, consistent with the
-        !  other transport equations
-        ielup = ifacel(2,ifac)
-        if (imasfl(ifac).ge.0.d0) ielup = ifacel(1,ifac)
+        do ifac = 1, nfac
 
-        imasfl_gas(ifac) = imasfl_gas(ifac) - x2(ielup)*imasfl(ifac)
-      enddo
+          ! Upwind value of x2 at the face, consistent with the
+          !  other transport equations
+          ielup = ifacel(2,ifac)
+          if (imasfl(ifac).ge.0.d0) ielup = ifacel(1,ifac)
 
-      do ifac = 1, nfabor
-        ! TODO Upwind value of x2 at the face, consistent with the
-        !  other transport equations
-        !if (bmasfl(ifac).ge.0.d0)
-        ielup = ifabor(ifac)
-        bmasfl_gas(ifac) = bmasfl_gas(ifac) - x2(ielup)*bmasfl(ifac)
-      enddo
+          imasfl_gas(ifac) = imasfl_gas(ifac) - x2(ielup)*imasfl(ifac)
+        enddo
+
+        do ifac = 1, nfabor
+          ! TODO Upwind value of x2 at the face, consistent with the
+          !  other transport equations
+          !if (bmasfl(ifac).ge.0.d0)
+          ielup = ifabor(ifac)
+          bmasfl_gas(ifac) = bmasfl_gas(ifac) - x2(ielup)*bmasfl(ifac)
+        enddo
+      endif
     endif
 
   ! Finalize the convective flux of the gas "class" by scaling by x1
   !  (rho x1 V1)_ij = (rho Vs)_ij - sum_classes (rho x2 V2)_ij
   ! Warning, x1 at the face must be computed so that it is consistent
   ! with an upwind scheme on (rho V1)
-  else if (icla.eq.-1) then
+  else if (icla.eq.-1.and.id_x1.ne.-1) then
 
     do ifac = 1, nfac
 
@@ -609,7 +659,7 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
       ielup = ifacel(2,ifac)
       if (imasfl_gas(ifac).ge.0.d0) ielup = ifacel(1,ifac)
 
-      imasfl_gas(ifac) = imasfl_gas(ifac)/x1(ielup)
+      imasfl_gas(ifac) = imasfl_gas(ifac)/cpro_x1(ielup)
     enddo
 
     do ifac = 1, nfabor
@@ -617,9 +667,9 @@ if (btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
       !  other transport equations
       ielup = ifabor(ifac)
       if (bmasfl_gas(ifac).lt.0.d0) then
-        bmasfl_gas(ifac) = bmasfl_gas(ifac)/b_x1(ifac)
+        bmasfl_gas(ifac) = bmasfl_gas(ifac)/bpro_x1(ifac)
       else
-        bmasfl_gas(ifac) = bmasfl_gas(ifac)/x1(ielup)
+        bmasfl_gas(ifac) = bmasfl_gas(ifac)/cpro_x1(ielup)
       endif
     enddo
   endif
@@ -631,28 +681,29 @@ endif
 !===============================================================================
 
 init = 1
-iconvp = iconv(ivar)
-thetap = thetav(ivar)
+iconvp = vcopt%iconv
+thetap = vcopt%thetav!FIXME not multiplied by theta?
 
 ! recompute the difference between mixture and the class
-do ifac = 1, nfac
-  flumas(ifac) = imasfl(ifac) - imasfl_mix(ifac)
-enddo
+if (btest(iscdri, DRIFT_SCALAR_IMPOSED_MASS_FLUX)) then
+  do ifac = 1, nfac
+    flumas(ifac) = - imasfl_mix(ifac)
+  enddo
 
-do ifac = 1, nfabor
-  flumab(ifac) = bmasfl(ifac) - bmasfl_mix(ifac)
-enddo
+  do ifac = 1, nfabor
+    flumab(ifac) = - bmasfl_mix(ifac)
+  enddo
+else
+  do ifac = 1, nfac
+    flumas(ifac) = imasfl(ifac) - imasfl_mix(ifac)
+  enddo
 
-call divmas(init, flumas, flumab, w1)
+  do ifac = 1, nfabor
+    flumab(ifac) = bmasfl(ifac) - bmasfl_mix(ifac)
+  enddo
+endif
 
-! NB: if the porosity module is swiched on, the the porosity is already
-! taken into account in w1
-
-! --> mass aggregation term
-do iel = 1, ncel
-  rovsdt(iel) = rovsdt(iel) + iconvp*thetap*w1(iel)
-  smbrs(iel) = smbrs(iel) - iconvp*w1(iel)*cvara_var(iel)
-enddo
+call divmas(init, flumas, flumab, divflu)
 
 ! Free memory
 deallocate(viscce)
