@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -67,7 +67,7 @@ use cplsat
 use field
 use atincl, only: init_at_chem
 use atchem, only: ichemistry
-use siream, only: iaerosol
+use sshaerosol, only: iaerosol, CS_ATMO_AEROSOL_OFF
 use mesh
 use cs_c_bindings
 use cs_nz_condensation, only:izzftcd, nztag1d, ztpar
@@ -106,7 +106,6 @@ type(c_ptr) :: rp
 double precision, allocatable, dimension(:,:) :: tmurbf
 double precision, allocatable, dimension(:) :: tparbf
 double precision, pointer, dimension(:) :: dt_s
-double precision, pointer, dimension(:,:) :: disale
 
 type(var_cal_opt) :: vcopt
 
@@ -178,7 +177,7 @@ write(nfecra,1000)
 
 ilecec = 2
 
-ficsui = 'main'
+ficsui = 'main.csc'
 call restart_create(ficsui, '', 1, rp)
 
 !===============================================================================
@@ -233,13 +232,6 @@ nbval  = 1
 ival(1) = iale
 call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
 
-!  ---> Cavitation
-rubriq = 'cavitation'
-itysup = 0
-nbval  = 1
-ival(1) = icavit
-call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
-
 !  ---> VOF
 rubriq = 'vof'
 itysup = 0
@@ -255,7 +247,7 @@ call restart_write_section_real_t(rp,rubriq,itysup,nbval,rval)
 
 call turbomachinery_restart_write(rp)
 
-if (ichemistry.gt.0.or.iaerosol.gt.0) then
+if (ichemistry.gt.0.or.iaerosol.ne.CS_ATMO_AEROSOL_OFF) then
   rubriq = 'atmospheric_chem'
   itysup = 0
   nbval  = 1
@@ -263,11 +255,7 @@ if (ichemistry.gt.0.or.iaerosol.gt.0) then
   call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
 endif
 
-#if defined(_CS_LANG_FR)
-car54 =' Fin de l''ecriture des options                       '
-#else
 car54 =' End writing the options                              '
-#endif
 write(nfecra,1110) car54
 
 ! 3.2 VARIABLES "PRINCIPALES"
@@ -312,7 +300,7 @@ if (iecaux.eq.1) then
   write(nfecra,2000)
 
   ilecec = 2
-  ficsui = 'auxiliary'
+  ficsui = 'auxiliary.csc'
   call restart_create(ficsui, '', 1, rp)
 
   write(nfecra,1100)
@@ -342,13 +330,6 @@ if (iecaux.eq.1) then
   ival(1) = iale
   call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
 
-  !  ---> Cavitation
-  rubriq = 'cavitation'
-  itysup = 0
-  nbval  = 1
-  ival(1) = icavit
-  call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
-
   !  ---> VOF
   rubriq = 'vof'
   itysup = 0
@@ -356,11 +337,7 @@ if (iecaux.eq.1) then
   ival(1) = ivofmt
   call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
 
-#if defined(_CS_LANG_FR)
-  car54 =' Fin de l''ecriture des dimensions et des options     '
-#else
   car54 =' End writing the dimensions and options               '
-#endif
   write(nfecra,1110)car54
 
 ! 5.3 ECRITURE DES VARIABLES
@@ -399,13 +376,13 @@ if (iecaux.eq.1) then
 
   !     Masse volumique si elle est variable uniquement
   !     La masse volumique est egalement ecrite pour l'algo. VOF
-  if (irovar.eq.1.or.ivofmt.ge.0) then
+  if (irovar.eq.1.or.ivofmt.gt.0) then
     ! Masse volumique - cellules
     call restart_write_field_vals(rp, icrom, 0)
 
     ! Masse volumique du pdt precedent - cellules
     ! only for VOF algo. and dilatable models (idilat = 4, 5)
-    if (ivofmt.ge.0.or.idilat.ge.4) then
+    if (ivofmt.gt.0.or.idilat.ge.4) then
       call restart_write_field_vals(rp, icrom, 1)
     endif
 
@@ -434,9 +411,9 @@ if (iecaux.eq.1) then
   !         pouvoir calculer la temperature H/Cp en debut de calcul
 
   call field_get_key_int(iviscl, key_t_ext_id, iviext)
-  if (iviext.gt.0.or.ivofmt.ge.0) then
+  if (iviext.gt.0.or.ivofmt.gt.0) then
     ! Molecular viscosity
-    if (ivivar.eq.1.or.ivofmt.ge.0) then
+    if (ivivar.eq.1.or.ivofmt.gt.0) then
       call restart_write_field_vals(rp, iviscl, 0)
     endif
   endif
@@ -455,13 +432,9 @@ if (iecaux.eq.1) then
     endif
   endif
 
-  call restart_write_linked_fields(rp, "scalar_diffusivity_id", iecr)
+  call restart_write_linked_fields(rp, "diffusivity_id", iecr)
 
-#if defined(_CS_LANG_FR)
-  car54 =' Fin de l''ecriture des proprietes physiques          '
-#else
   car54 =' End writing the physical properties                  '
-#endif
   write(nfecra,1110)car54
 
 ! ---> Pas de temps
@@ -477,11 +450,7 @@ if (iecaux.eq.1) then
     call restart_write_section_real_t(rp,rubriq,itysup,nbval,dt_s)
   endif
 
-#if defined(_CS_LANG_FR)
-  car54 =' Fin de l''ecriture du pas de temps                   '
-#else
   car54 =' End writing the time step                            '
-#endif
   write(nfecra,1110)car54
 
   ! Mass fluxes
@@ -489,14 +458,6 @@ if (iecaux.eq.1) then
   call restart_write_linked_fields(rp, "inner_mass_flux_id", iecr)
   call restart_write_linked_fields(rp, "boundary_mass_flux_id", iecr)
 
-  ! Symmetry flag (used for least-squares gradients,
-  ! with extrapolation at boundary).
-
-  rubriq = 'isympa_fb_phase01'
-  itysup = 3
-  nbval  = 1
-  call restart_write_section_int_t(rp,rubriq,itysup,nbval,isympa)
-
   ! Boundary condition coefficients
 
   call restart_write_bc_coeffs(rp)
@@ -507,11 +468,7 @@ if (iecaux.eq.1) then
   call restart_write_linked_fields(rp, "source_term_prev_id", iecr)
 
   if (iecr.ne.0) then
-#if defined(_CS_LANG_FR)
-    car54=' Fin de l''ecriture des termes sources                '
-#else
     car54=' End writing the source terms                         '
-#endif
     write(nfecra,1110)car54
   endif
 
@@ -534,11 +491,7 @@ if (iecaux.eq.1) then
   endif
 
   if (iecr.ne.0) then
-#if defined(_CS_LANG_FR)
-    car54=' Fin de l''ecriture de la distance a la paroi         '
-#else
     car54=' End writing the wall distance                        '
-#endif
     write(nfecra,1110)car54
   endif
 
@@ -549,16 +502,9 @@ if (iecaux.eq.1) then
     call restart_write_field_vals(rp, f_id, 0)
   endif
 
-! ---> Pression hydrostatique predite
-
-  if (iphydr.eq.2) then
-    call field_get_id('hydrostatic_pressure_prd', f_id)
-    call restart_write_field_vals(rp, f_id, 0)
-  endif
-
 !----------------------------------------------------------------
 ! ---> Wall temperature associated to the condensation model
-!      with or wihtout the 1D thermal model tag1D
+!      with or without the 1D thermal model tag1D
 !----------------------------------------------------------------
 
   if (icondb.eq.0) then
@@ -611,21 +557,12 @@ if (iecaux.eq.1) then
 
 ! ---> Methode ALE
 
-  if (iale.eq.1) then
+  if (iale.ge.1) then
 
-    itysup = 4
-    nbval  = 3
+    call restart_write_field_vals(rp, fdiale, 0)
+    call restart_write_field_vals(rp, fdiale, 1)
 
-    call field_get_val_v(fdiale, disale)
-
-    rubriq = 'vertex_displacement'
-    call restart_write_section_real_t(rp,rubriq,itysup,nbval,disale)
-
-#if defined(_CS_LANG_FR)
-    car54=' Fin de l''ecriture des donnees ALE    '
-#else
     car54=' End writing the ALE data              '
-#endif
     write(nfecra,1110)car54
 
     ngbstr(1) = nbstru
@@ -668,11 +605,7 @@ if (iecaux.eq.1) then
         call restart_write_section_real_t(rp,rubriq,itysup,nbval,tmpstr)
       enddo
 
-#if defined(_CS_LANG_FR)
-      car54=' Fin de l''ecriture des donnees des structures (ALE)'
-#else
       car54=' End writing the structures data (ALE)              '
-#endif
       write(nfecra,1110)car54
 
     endif
@@ -728,11 +661,7 @@ if (iecaux.eq.1) then
     rubriq = 'ientox_zone_bord_cod3p'
     call restart_write_section_int_t(rp,rubriq,itysup,nbval,ientox)
 
-#if defined(_CS_LANG_FR)
-    car54=' Fin de l''ecriture des informations combustion COD3P'
-#else
     car54=' End writing combustion information (COD3P)         '
-#endif
     write(nfecra,1110)car54
 
   endif
@@ -784,11 +713,7 @@ if (iecaux.eq.1) then
     rubriq = 'tkent_zone_bord_ebu'
     call restart_write_section_real_t(rp,rubriq,itysup,nbval,tkent)
 
-#if defined(_CS_LANG_FR)
-    car54=' Fin de l''ecriture des informations combustion EBU '
-#else
     car54=' End writing the combustion information (EBU)      '
-#endif
     write(nfecra,1110)car54
 
   endif
@@ -852,11 +777,7 @@ if (iecaux.eq.1) then
     rubriq = 'tkent_zone_bord_lwc'
     call restart_write_section_real_t(rp,rubriq,itysup,nbval,tkent)
 
-#if defined(_CS_LANG_FR)
-    car54=' Fin de l''ecriture des informations combustion LWC '
-#else
     car54=' End writing combustion information (LWC)          '
-#endif
     write(nfecra,1110)car54
 
   endif
@@ -936,11 +857,7 @@ if (iecaux.eq.1) then
     rubriq = 'timpat_zone_bord_charbon_pulverise'
     call restart_write_section_real_t(rp,rubriq,itysup,nbval,timpat)
 
-#if defined(_CS_LANG_FR)
-    car54=' Fin de l''ecriture des informations combustion CP    '
-#else
     car54=' End writing combustion information (CP)            '
-#endif
     write(nfecra,1110)car54
 
   endif
@@ -992,11 +909,7 @@ if (iecaux.eq.1) then
     rubriq = 'qimpfl_zone_bord_fuel'
     call restart_write_section_real_t(rp,rubriq,itysup,nbval,qimpfl)
 
-#if defined(_CS_LANG_FR)
-    car54=' Fin de l''ecriture des informations combustion FUEL  '
-#else
     car54=' End writing combustion information (FUEL)           '
-#endif
     write(nfecra,1110)car54
 
   endif
@@ -1060,11 +973,7 @@ if (iecaux.eq.1) then
   endif
 
   if (iecr.ne.0) then
-#if defined(_CS_LANG_FR)
-    car54=' Fin de l''ecriture des informations electriques      '
-#else
     car54=' End writing the electric information                '
-#endif
     write(nfecra,1110)car54
   endif
 
@@ -1085,105 +994,6 @@ return
 ! 6. FORMATS
 !===============================================================================
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(3x,'** Ecriture du fichier suite principal',/,       &
-             3x,'   ----------------------------------- ',/)
- 1100 format(' Debut de l''ecriture')
- 1110 format('  ',A54)
- 1200 format(' Fin de l''ecriture')
- 2000 format(/,3x,'** Ecriture du fichier suite auxiliaire',/,    &
-               3x,'   ------------------------------------ ',/)
-
- 7001 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION :       A L''ECRITURE DU FICHIER SUITE        ',/,&
-'@    =========                                               ',/,&
-'@                                                            ',/,&
-'@      Le nombre de scalaires maximal NSCAMX supporte par le ',/,&
-'@        format d''ecriture du fichier suite est             ',/,&
-'@        NFMTSC = ',i10                                       ,/,&
-'@      On a ici un nombre de scalaires maximal superieur     ',/,&
-'@        NSCAMX = ',i10                                       ,/,&
-'@      On ne pourra pas relire les scalaires dont le numero  ',/,&
-'@        est superieur                                       ',/,&
-'@                                                            ',/,&
-'@    Le calcul sera execute.                                 ',/,&
-'@                                                            ',/,&
-'@    Voir le sous-programme ecrava.                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 7002 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION :       A L''ECRITURE DU FICHIER SUITE        ',/,&
-'@    =========                                               ',/,&
-'@                                                            ',/,&
-'@      Le nombre de flux de masse max NVARMX supporte par le ',/,&
-'@        format d''ecriture du fichier suite est             ',/,&
-'@        NFMTFL = ',i10                                       ,/,&
-'@      On a ici un nombre de flux      maximal superieur     ',/,&
-'@        NVARMX = ',i10                                       ,/,&
-'@      On ne pourra pas relire les flux      dont le numero  ',/,&
-'@        est superieur                                       ',/,&
-'@                                                            ',/,&
-'@    Le calcul sera execute.                                 ',/,&
-'@                                                            ',/,&
-'@    Voir le sous-programme ecrava.                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 7004 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION :       A L''ECRITURE DU FICHIER SUITE        ',/,&
-'@    =========                                               ',/,&
-'@                                                            ',/,&
-'@      Le nombre de charbons      max NCHARM supporte par le ',/,&
-'@        format d''ecriture du fichier suite est             ',/,&
-'@        NFMTCH = ',i10                                       ,/,&
-'@      On a ici un nombre de charbons  maximal superieur     ',/,&
-'@        NCHARM = ',i10                                       ,/,&
-'@      On ne pourra pas relire certaines informations        ',/,&
-'@        relatives aux charbons dont le numero               ',/,&
-'@        est superieur                                       ',/,&
-'@                                                            ',/,&
-'@    Le calcul sera execute.                                 ',/,&
-'@                                                            ',/,&
-'@    Voir le sous-programme ecrava.                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 7005 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION :       A L''ECRITURE DU FICHIER SUITE        ',/,&
-'@    =========                                               ',/,&
-'@                                                            ',/,&
-'@      Le nombre de classes par charbon max NCPCMX supporte  ',/,&
-'@        par le format d''ecriture du fichier suite est      ',/,&
-'@        NFMTCL = ',i10                                       ,/,&
-'@      On a ici un nombre de classes par charbon superieur   ',/,&
-'@        NCPCMX = ',i10                                       ,/,&
-'@      On ne pourra pas relire certaines informations        ',/,&
-'@        relatives aux classes  dont le numero               ',/,&
-'@        est superieur                                       ',/,&
-'@                                                            ',/,&
-'@    Le calcul sera execute.                                 ',/,&
-'@                                                            ',/,&
-'@    Voir le sous-programme ecrava.                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
  1000 format(3x,'** Writing the main restart file',/,             &
              3x,'   -----------------------------',/)
  1100 format(' Start writing'                                      )
@@ -1275,7 +1085,4 @@ return
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
-
 end subroutine
