@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -32,21 +32,21 @@ module field
 
   !=============================================================================
 
-  integer :: FIELD_INTENSIVE, FIELD_EXTENSIVE, FIELD_STEADY
+  integer :: FIELD_INTENSIVE, FIELD_EXTENSIVE
   integer :: FIELD_VARIABLE, FIELD_PROPERTY
-  integer :: FIELD_POSTPROCESS, FIELD_ACCUMULATOR, FIELD_USER
+  integer :: FIELD_POSTPROCESS, FIELD_ACCUMULATOR, FIELD_USER, FIELD_CDO
 
   integer :: FIELD_OK, FIELD_INVALID_KEY_NAME, FIELD_INVALID_KEY_ID,   &
              FIELD_INVALID_CATEGORY, FIELD_INVALID_TYPE
 
   parameter (FIELD_INTENSIVE=1)
   parameter (FIELD_EXTENSIVE=2)
-  parameter (FIELD_STEADY=4)
-  parameter (FIELD_VARIABLE=8)
-  parameter (FIELD_PROPERTY=16)
-  parameter (FIELD_POSTPROCESS=32)
-  parameter (FIELD_ACCUMULATOR=64)
-  parameter (FIELD_USER=128)
+  parameter (FIELD_VARIABLE=4)
+  parameter (FIELD_PROPERTY=8)
+  parameter (FIELD_POSTPROCESS=16)
+  parameter (FIELD_ACCUMULATOR=32)
+  parameter (FIELD_USER=64)
+  parameter (FIELD_CDO=128)
 
   parameter (FIELD_OK=0)
   parameter (FIELD_INVALID_KEY_NAME=1)
@@ -349,22 +349,10 @@ module field
 
     !---------------------------------------------------------------------------
 
-    ! Interface to C function mapping field values
-
-    subroutine cs_field_map_values(f, var, var_prev)  &
-      bind(C, name='cs_field_map_values')
-      use, intrinsic :: iso_c_binding
-      implicit none
-      type(c_ptr), value :: f
-      real(kind=c_double), dimension(*) :: var, var_prev
-    end subroutine cs_field_map_values
-
-    !---------------------------------------------------------------------------
-
     ! Interface to C function allocating boundary condition coefficients
 
     subroutine cs_field_allocate_bc_coeffs(f, have_flux_bc, have_mom_bc,  &
-                                           have_conv_bc)                  &
+                                           have_conv_bc, have_exch_bc)    &
       bind(C, name='cs_field_allocate_bc_coeffs')
       use, intrinsic :: iso_c_binding
       implicit none
@@ -372,6 +360,7 @@ module field
       logical(c_bool), value       :: have_flux_bc
       logical(c_bool), value       :: have_mom_bc
       logical(c_bool), value       :: have_conv_bc
+      logical(c_bool), value       :: have_exch_bc
     end subroutine cs_field_allocate_bc_coeffs
 
     !---------------------------------------------------------------------------
@@ -659,7 +648,12 @@ contains
     c_type_flag = type_flag
     c_location_id = location_id
     c_dim = dim
-    c_has_previous = has_previous
+
+    if (has_previous) then
+      c_has_previous = .true.
+    else
+      c_has_previous = .false.
+    endif
 
     f = cs_field_create(c_name, c_type_flag, c_location_id, c_dim, &
                         c_has_previous)
@@ -1130,39 +1124,6 @@ contains
 
   !=============================================================================
 
-  !> \brief  Map existing value arrays to field descriptor.
-
-  !> \param[in]  id       field id
-  !> \param[in]  val      pointer to array of values
-  !> \param[in]  val_pre  pointer to array of previous values (ignored if
-  !>                      field was defined with have_previous = .false.)
-
-  subroutine field_map_values(id, val, val_pre)
-
-    use, intrinsic :: iso_c_binding
-    implicit none
-
-    ! Arguments
-
-    integer, intent(in)                        :: id
-    double precision, intent(in), dimension(*) :: val, val_pre
-
-    ! Local variables
-
-    integer(c_int) :: c_id
-    type(c_ptr)    :: f
-
-    c_id = id
-
-    f = cs_field_by_id(c_id)
-    call cs_field_map_values(f, val, val_pre)
-
-    return
-
-  end subroutine field_map_values
-
-  !=============================================================================
-
   !> \brief Allocate boundary condition coefficient arrays if applicable.
 
   !> \param[in]  id            field id
@@ -1170,10 +1131,15 @@ contains
   !>                           (coefaf and coefbf) are added
   !> \param[in]  have_mom_bc   if .true., BC coefficients used in divergence
   !>                           term (coefad and coefbd) are added
-  !> \param[in]  have_conv_bc   if .true., BC coefficients used in convection
+  !> \param[in]  have_conv_bc  if .true., BC coefficients used in convection
   !>                           term (coefac and coefbc) are added
+  !> \param[in]  have_conv_bc  if .true., BC coefficients used in convection
+  !>                           term (coefac and coefbc) are added
+  !> \param[in]  have_exch_bc  if .true., exchange boundary coefficients
+  !>                           (hint and hext)
 
-  subroutine field_allocate_bc_coeffs(id, have_flux_bc, have_mom_bc, have_conv_bc)
+  subroutine field_allocate_bc_coeffs(id, have_flux_bc, have_mom_bc,           &
+                                          have_conv_bc, have_exch_bc)
 
     use, intrinsic :: iso_c_binding
     implicit none
@@ -1184,6 +1150,7 @@ contains
     logical, intent(in) :: have_flux_bc
     logical, intent(in) :: have_mom_bc
     logical, intent(in) :: have_conv_bc
+    logical, intent(in) :: have_exch_bc
 
     ! Local variables
 
@@ -1191,15 +1158,39 @@ contains
     logical(c_bool) :: c_have_flux_bc
     logical(c_bool) :: c_have_mom_bc
     logical(c_bool) :: c_have_conv_bc
+    logical(c_bool) :: c_have_exch_bc
     type(c_ptr)     :: f
 
     c_id = id
 
     f = cs_field_by_id(c_id)
-    c_have_flux_bc = have_flux_bc
-    c_have_mom_bc = have_mom_bc
-    c_have_conv_bc = have_conv_bc
-    call cs_field_allocate_bc_coeffs(f, c_have_flux_bc, c_have_mom_bc, c_have_conv_bc)
+
+    if (have_flux_bc) then
+      c_have_flux_bc = .true.
+    else
+      c_have_flux_bc = .false.
+    endif
+
+    if (have_mom_bc) then
+      c_have_mom_bc = .true.
+    else
+      c_have_mom_bc = .false.
+    endif
+
+    if (have_conv_bc) then
+      c_have_conv_bc = .true.
+    else
+      c_have_conv_bc = .false.
+    endif
+
+    if (have_exch_bc) then
+      c_have_exch_bc = .true.
+    else
+      c_have_exch_bc = .false.
+    endif
+
+    call cs_field_allocate_bc_coeffs(f, c_have_flux_bc, c_have_mom_bc, &
+                                        c_have_conv_bc, c_have_exch_bc)
 
     return
 
