@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -42,18 +42,18 @@ module field_operator
     !> \param[in]   f_id             field id
     !> \param[in]   use_previous_t   1 if values at previous time step should
     !>                               be used, 0 otherwise
-    !> \param[in]   imrgra           gradient computation mode
+    !> \param[in]   imrgrO           unused
     !> \param[in]   inc              0: increment; 1: do not increment
     !> \param[in]   recompute_cocg   1 or 0: recompute COCG or not
     !> \param[out]  grad             gradient
 
-    subroutine field_gradient_scalar(f_id, use_previous_t, imrgra, inc,        &
+    subroutine field_gradient_scalar(f_id, use_previous_t, imrgr0, inc,        &
                                      recompute_cocg,                           &
                                      grad)                                     &
       bind(C, name='cs_f_field_gradient_scalar')
       use, intrinsic :: iso_c_binding
       implicit none
-      integer(c_int), value             :: f_id, use_previous_t, imrgra, inc
+      integer(c_int), value             :: f_id, use_previous_t, imrgr0, inc
       integer(c_int), value             :: recompute_cocg
       real(kind=c_double), dimension(*) :: grad
     end subroutine field_gradient_scalar
@@ -65,7 +65,7 @@ module field_operator
     !> \param[in]   f_id             field id
     !> \param[in]   use_previous_t   1 if values at previous time step should
     !>                               be used, 0 otherwise
-    !> \param[in]   imrgra           gradient computation mode
+    !> \param[in]   imrgr0           ignored
     !> \param[in]   inc              0: increment; 1: do not increment
     !> \param[in]   recompute_cocg   1 or 0: recompute COCG or not
     !> \param[in]   hyd_p_flag       flag for hydrostatic pressure
@@ -73,14 +73,14 @@ module field_operator
     !>                               the hydrostatic pressure
     !> \param[out]  grad             gradient
 
-    subroutine field_gradient_potential(f_id, use_previous_t, imrgra, inc,     &
+    subroutine field_gradient_potential(f_id, use_previous_t, imrgr0, inc,     &
                                         recompute_cocg,                        &
                                         hyd_p_flag,                            &
                                         f_ext, grad)                           &
       bind(C, name='cs_f_field_gradient_potential')
       use, intrinsic :: iso_c_binding
       implicit none
-      integer(c_int), value                :: f_id, use_previous_t, imrgra, inc
+      integer(c_int), value                :: f_id, use_previous_t, imrgr0, inc
       integer(c_int), value                :: recompute_cocg
       integer(c_int), value                :: hyd_p_flag
       real(kind=c_double), dimension(3, *) :: f_ext
@@ -94,16 +94,16 @@ module field_operator
     !> \param[in]   f_id             field id
     !> \param[in]   use_previous_t   1 if values at previous time step should
     !>                               be used, 0 otherwise
-    !> \param[in]   imrgra           gradient computation mode
+    !> \param[in]   imrgr0           ignored
     !> \param[in]   inc              0: increment; 1: do not increment
     !> \param[out]  grad             gradient
 
-    subroutine field_gradient_vector(f_id, use_previous_t, imrgra, inc,        &
+    subroutine field_gradient_vector(f_id, use_previous_t, imrgr0, inc,        &
                                      grad)                                     &
       bind(C, name='cs_f_field_gradient_vector')
       use, intrinsic :: iso_c_binding
       implicit none
-      integer(c_int), value                 :: f_id, use_previous_t, imrgra, inc
+      integer(c_int), value                 :: f_id, use_previous_t, imrgr0, inc
       real(kind=c_double), dimension(3,3,*) :: grad
     end subroutine field_gradient_vector
 
@@ -114,19 +114,34 @@ module field_operator
     !> \param[in]   f_id             field id
     !> \param[in]   use_previous_t   1 if values at previous time step should
     !>                               be used, 0 otherwise
-    !> \param[in]   imrgra           gradient computation mode
+    !> \param[in]   imrgr0           ignored
     !> \param[in]   inc              0: increment; 1: do not increment
     !> \param[out]  grad             gradient
 
-    subroutine field_gradient_tensor(f_id, use_previous_t, imrgra, inc,        &
+    subroutine field_gradient_tensor(f_id, use_previous_t, imrgr0, inc,        &
                                      grad)                                     &
       bind(C, name='cs_f_field_gradient_tensor')
       use, intrinsic :: iso_c_binding
       implicit none
-      integer(c_int), value                 :: f_id, use_previous_t, imrgra, inc
+      integer(c_int), value                 :: f_id, use_previous_t, imrgr0, inc
       real(kind=c_double), dimension(6,3,*) :: grad
     end subroutine field_gradient_tensor
 
+    !---------------------------------------------------------------------------
+
+    !> \brief  Shift field values in order to set its spatial average to a given
+    !>         value.
+
+    !> \param[in]   f_id  field id
+    !> \param[in]   va    real value of volume average to be set
+
+    subroutine field_set_volume_average(f_id, va)                              &
+      bind(C, name='cs_f_field_set_volume_average')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      integer(c_int), value      :: f_id
+      real(kind=c_double), value :: va
+    end subroutine field_set_volume_average
 
     !---------------------------------------------------------------------------
 
