@@ -3,7 +3,7 @@
 !     This file is part of the Code_Saturne Kernel, element of the
 !     Code_Saturne CFD tool.
 
-!     Copyright (C) 1998-2018 EDF S.A., France
+!     Copyright (C) 1998-2021 EDF S.A., France
 
 !     contact: saturne-support@edf.fr
 
@@ -64,7 +64,6 @@ use ppthch
 use ppincl
 use cfpoin
 use lagran
-use ihmpre
 use radiat
 use cplsat
 use mesh
@@ -83,7 +82,9 @@ implicit none
 
 integer          ii, ivar
 integer          iflid, kcvlim, ifctsl, clip_id
+integer          kdflim
 integer          kturt, kfturt, kislts, keyvar, kclipp, kfturt_alpha
+integer          turb_flux_model, turb_flux_model_type
 integer          itycat, ityloc, idim1, idim3, idim6
 logical          iprev, inoprv
 integer          f_id, kscavr, f_type
@@ -92,7 +93,7 @@ integer          iscdri, icla, iclap
 integer          keyccl, keydri
 integer          idfm, iggafm, nfld
 integer          iflidp, idimf, n_fans
-integer          f_dim
+integer          f_dim, kiflux, kbflux
 
 character(len=80) :: name, f_name
 
@@ -155,8 +156,11 @@ do ii = 1, nscal
     ivar = isca(ii)
     call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
 
-    if (ityturt(ii).gt.0) then
-      if (ityturt(ii).eq.3) then
+    call field_get_key_int(ivarfl(ivar), kturt, turb_flux_model)
+    turb_flux_model_type = turb_flux_model / 10
+
+    if (turb_flux_model_type.gt.0) then
+      if (turb_flux_model_type.eq.3) then
         idfm = 1
       endif
 
@@ -169,9 +173,6 @@ do ii = 1, nscal
       idfm = 1
     endif
 
-    ! Reference diffusivity value
-    call field_set_key_double(ivarfl(ivar), kvisl0, visls0(ii))
-
     ! Additional fields for Drift scalars is done in addfld
 
   endif
@@ -188,12 +189,7 @@ if (iphydr.eq.1) then
                             itycat, ityloc, idim3, f_id)
 
   call field_set_key_int(f_id, keylog, 1)
-  call field_set_key_int(f_id, keyvis, POST_ON_LOCATION)
-endif
-
-if (iphydr.eq.2) then
-  call field_find_or_create('hydrostatic_pressure_prd', &
-                            itycat, ityloc, idim1, f_id)
+  call field_set_key_int(f_id, keyvis, 0)
 endif
 
 ! friction velocity at the wall, in the case of a LES calculation
@@ -361,19 +357,23 @@ do f_id = 0, nfld - 1
   call field_get_type(f_id, f_type)
   ! Is the field of type FIELD_VARIABLE?
   if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    ! Is this field not managed by CDO? Not useful for CDO
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
 
-    call field_get_key_struct_var_cal_opt(f_id, vcopt)
-    if (vcopt%iwgrec.eq.1 .and. vcopt%idiff .ge. 1) then
+      call field_get_key_struct_var_cal_opt(f_id, vcopt)
+      if (vcopt%iwgrec.eq.1 .and. vcopt%idiff .ge. 1) then
+
+        call field_get_name(f_id, name)
+        f_name = 'gradient_weighting_'//trim(name)
+        if (iand(vcopt%idften, ISOTROPIC_DIFFUSION).ne.0) then
+          idimf = 1
+        else if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
+          idimf = 6
+        endif
+        call field_create(f_name, itycat, ityloc, idimf, inoprv, iflid)
+        call field_set_key_int(f_id, kwgrec, iflid)
 
-      call field_get_name(f_id, name)
-      f_name = 'gradient_weighting_'//trim(name)
-      if (iand(vcopt%idften, ISOTROPIC_DIFFUSION).ne.0) then
-        idimf = 1
-      else if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
-        idimf = 6
       endif
-      call field_create(f_name, itycat, ityloc, idimf, inoprv, iflid)
-      call field_set_key_int(f_id, kwgrec, iflid)
 
     endif
   endif
@@ -388,23 +388,28 @@ ityloc = 1 ! cells
 
 do f_id = 0, nfld - 1
   call field_get_type(f_id, f_type)
-  ! Is the field of type FIELD_VARIABLE?
+  ! Is the field of type FIELD_VARIABLE ?
   if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
-    call field_get_key_int(f_id, kislts, ifctsl)
-    if (ifctsl.ge.0) then
-      call field_get_key_struct_var_cal_opt(f_id, vcopt)
+    ! Is this field not managed by CDO ?
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
 
-      ! Now create matching field
-      if (vcopt%iconv.gt.0 .and. vcopt%blencv.gt.0 .and. vcopt%isstpc.eq.0) then
-        ! Build name and label
-        call field_get_name(f_id, f_name)
-        name  = trim(f_name) // '_slope_upwind'
-        call field_create(name, itycat, ityloc, idim1, inoprv, ifctsl)
-        call field_set_key_int(ifctsl, keyvis, POST_ON_LOCATION)
-        call field_set_key_int(f_id, kislts, ifctsl)
+      call field_get_key_int(f_id, kislts, ifctsl)
+      if (ifctsl.ge.0) then
+        call field_get_key_struct_var_cal_opt(f_id, vcopt)
+
+        ! Now create matching field
+        if (vcopt%iconv.gt.0 .and. vcopt%blencv.gt.0 .and. vcopt%isstpc.eq.0) then
+          ! Build name and label
+          call field_get_name(f_id, f_name)
+          name  = trim(f_name) // '_slope_upwind'
+          call field_create(name, itycat, ityloc, idim1, inoprv, ifctsl)
+          call field_set_key_int(ifctsl, keyvis, POST_ON_LOCATION)
+          call field_set_key_int(f_id, kislts, ifctsl)
+        endif
       endif
-    endif
-  endif
+
+    endif ! CDO ?
+  endif ! VARIABLE ?
 enddo
 
 ! Postprocessing of clippings
@@ -416,22 +421,27 @@ ityloc = 1 ! cells
 
 do f_id = 0, nfld - 1
   call field_get_type(f_id, f_type)
-  ! Is the field of type FIELD_VARIABLE?
+  ! Is the field of type FIELD_VARIABLE ?
   if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
-    call field_get_key_int(f_id, kclipp, clip_id)
-    if (clip_id.ge.0) then
+    ! Is this field not managed by CDO ?
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
 
-      ! Now create matching field
-      ! Build name and label
-      call field_get_name(f_id, f_name)
+      call field_get_key_int(f_id, kclipp, clip_id)
+      if (clip_id.ge.0) then
 
-      call field_get_dim(f_id, f_dim)
-      name  = trim(f_name) // '_clipped'
-      call field_create(name, itycat, ityloc, f_dim, inoprv, clip_id)
-      call field_set_key_int(clip_id, keyvis, POST_ON_LOCATION)
-      call field_set_key_int(f_id, kclipp, clip_id)
-    endif
-  endif
+        ! Now create matching field
+        ! Build name and label
+        call field_get_name(f_id, f_name)
+
+        call field_get_dim(f_id, f_dim)
+        name  = trim(f_name) // '_clipped'
+        call field_create(name, itycat, ityloc, f_dim, inoprv, clip_id)
+        call field_set_key_int(clip_id, keyvis, POST_ON_LOCATION)
+        call field_set_key_int(f_id, kclipp, clip_id)
+      endif
+
+    endif ! CDO ?
+  endif ! VARIABLE ?
 enddo
 
 ! Fans output
@@ -465,8 +475,10 @@ do f_id = 0, nfld - 1
 
     call field_get_key_struct_var_cal_opt(f_id, vcopt)
 
+    call field_get_key_int(f_id, kcvlim, ifctsl)
+
     ! Beta limiter or Roe-Sweby limiter
-    if (vcopt%isstpc.eq.2 .or. vcopt%isstpc.eq.3) then
+    if (vcopt%isstpc.eq.2.or.ifctsl.ne.-1) then
       ! Now create matching field
       ! Build name and label
       call field_get_name(f_id, f_name)
@@ -474,11 +486,7 @@ do f_id = 0, nfld - 1
       call field_get_dim(f_id, f_dim)
       name = trim(f_name) // '_conv_lim'
 
-      if (vcopt%isstpc.eq.2) then
-        ityloc = 1 ! cells
-      else
-        ityloc = 2 ! Interior faces
-      endif
+      ityloc = 1 ! cells
 
       call field_create(name, itycat, ityloc, f_dim, inoprv, ifctsl)
       call field_set_key_int(ifctsl, keyvis, POST_ON_LOCATION)
@@ -489,25 +497,85 @@ do f_id = 0, nfld - 1
   endif
 enddo
 
+! Diffusion limiter
+
+call field_get_key_id("diffusion_limiter_id", kdflim)
+
+itycat = FIELD_PROPERTY
+
+do f_id = 0, nfld - 1
+
+  call field_get_type(f_id, f_type)
+
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    ! Is this field not managed by CDO ? Not useful with CDO schemes
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
+
+      call field_get_key_int(f_id, kdflim, ifctsl)
+
+      if (ifctsl.ne.-1) then
+        ! Now create matching field
+        ! Build name and label
+        call field_get_name(f_id, f_name)
+
+        call field_get_dim(f_id, f_dim)
+        name = trim(f_name) // '_diff_lim'
+
+        ityloc = 1 ! cells
+
+        call field_create(name, itycat, ityloc, f_dim, inoprv, ifctsl)
+        call field_set_key_int(ifctsl, keyvis, POST_ON_LOCATION)
+        call field_set_key_int(ifctsl, keylog, 1)
+
+        call field_set_key_int(f_id, kdflim, ifctsl)
+      endif
+
+    endif ! CDO ?
+  endif ! VARIABLE ?
+enddo
+
 !===============================================================================
 
-! VOF algorithm: the void fraction has its spectific convective flux
+! VOF algorithm: the void fraction has its specific convective flux
+!                and void fraction flux needs to be stored
 !-------------------------------------------------------------------
 
-if (ivofmt.ge.0) then
+if (ivofmt.gt.0) then
 
   itycat = FIELD_EXTENSIVE + FIELD_PROPERTY
 
+  call field_get_key_id("inner_flux_id", kiflux)
+  call field_get_key_id("boundary_flux_id", kbflux)
+
   ityloc = 2  ! inner faces
-  f_name = 'inner_void_fraction_flux'
+  f_name = 'inner_volume_flux'
   call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
   call field_set_key_int(ivarfl(ivolf2), kimasf, f_id)
 
+  f_name = 'inner_void_fraction_flux'
+  call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
+  call field_set_key_int(ivarfl(ivolf2), kiflux, f_id)
+
   ityloc = 3 ! boundary faces
-  f_name = 'boundary_void_fraction_flux'
+  f_name = 'boundary_volume_flux'
   call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
   call field_set_key_int(ivarfl(ivolf2), kbmasf, f_id)
 
+  f_name = 'boundary_void_fraction_flux'
+  call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
+  call field_set_key_int(ivarfl(ivolf2), kbflux, f_id)
+
+  if (idrift.gt.0) then
+    ityloc = 2  ! inner faces
+    f_name = 'inner_drift_velocity_flux'
+    call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
+
+    ityloc = 3 ! boundary faces
+    f_name = 'boundary_drift_velocity_flux'
+    call field_create(f_name, itycat, ityloc, idim1, inoprv, f_id)
+  endif
+
 endif
 
 !===============================================================================
@@ -540,7 +608,7 @@ endif
 itycat = FIELD_INTENSIVE + FIELD_PROPERTY
 ityloc = 1 ! cells
 
-if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1 &
+if (idfm.eq.1.or.iggafm.eq.1.or. itytur.eq.3 .and. idirsm.eq.1 &
     .or.darcy_anisotropic_dispersion.eq.1) then
   call field_create('anisotropic_turbulent_viscosity', itycat, ityloc, idim6, &
                     inoprv, ivsten)
@@ -550,6 +618,19 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1 &
   endif
 endif
 
+!===============================================================================
+! Change some field settings
+!===============================================================================
+
+! If ALE, for fluid dtructure interaction, mass fluxes may be needed at the
+! previous iteration
+if (iale.ge.1) then
+  call field_get_key_int(ivarfl(ipr), kimasf, f_id)
+  call field_set_n_previous(f_id, 1)
+  call field_get_key_int(ivarfl(ipr), kbmasf, f_id)
+  call field_set_n_previous(f_id, 1)
+endif
+
 !===============================================================================
 ! Set some field keys
 !===============================================================================
@@ -561,7 +642,7 @@ do f_id = 0, nfld - 1
   ! Is the field of type FIELD_VARIABLE?
   if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
     call field_get_key_struct_var_cal_opt(f_id, vcopt)
-    vcopt%imrgra= imrgra
+    if (vcopt%imrgra .lt. 0) vcopt%imrgra= imrgra
     call field_set_key_struct_var_cal_opt(f_id, vcopt)
   endif
 enddo
@@ -674,11 +755,12 @@ endif
 
 ! Check if scalars are buoyant and set n_buoyant_scal accordingly.
 ! It is then used in tridim to update buoyant scalars and density in U-P loop
-call cs_parameters_set_n_buoyant_scalars
+call cs_velocity_pressure_set_n_buoyant_scalars
 
-! If density is variable, a particular care must be taken when dealing with
-! density in the unsteady term in the velocity pressure loop
-if (irovar.eq.1) then
+! For Low Mach and compressible (increment) algorithms, a particular care
+! must be taken when dealing with density in the unsteady term in the velocity
+! pressure loop
+if (irovar.eq.1.and.(idilat.gt.1.or.ivofmt.gt.0.or.ippmod(icompf).eq.3)) then
   ! EOS density, imposed after the correction step, so we need
   ! to keep the previous one, which is in balance with the mass
   f_name = 'density_mass'
@@ -692,6 +774,7 @@ if (irovar.eq.1) then
 endif
 
 ! Some C mappings
+call cs_field_pointer_map_base
 call cs_field_pointer_map_boundary
 
 return
