@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -55,7 +55,6 @@ use coincl
 use cpincl
 use ppincl
 use radiat
-use ihmpre
 use mesh
 use post
 use field
@@ -74,6 +73,7 @@ character(len=80) :: f_label, f_name, s_name, s_label
 integer           :: ii, ivar, isorb, keysrb, igwfpr, keypre, ischcp
 integer           :: idim1, idim3, idim6, iflid, k_restart_id
 integer           :: type_flag, post_flag, location_id
+integer           :: keypid
 logical           :: has_previous
 
 type(gwf_soilwater_partition) :: sorption_scal
@@ -106,6 +106,12 @@ interface
 
 end interface
 
+!===============================================================================
+! 0. Initialisation
+!===============================================================================
+
+call field_get_key_id("parent_field_id", keypid)
+
 !===============================================================================
 ! 1. PROPRIETES PRINCIPALES
 !===============================================================================
@@ -142,10 +148,15 @@ end interface
 
 call add_property_field_1d('density', 'Density', irom)
 icrom = irom
-! Postprocessed and in the log file by default, hidden in modini if not variable.
+! Postprocessed and in the log file by default, hidden in modini if not variable
 call field_set_key_int(icrom, keylog, 1)
 call field_set_key_int(icrom, keyvis, 1)
 
+! Add pressure increment field; this is not even a true property,
+! as its values at the end of a time step make no sense.
+call field_create('pressure_increment', FIELD_INTENSIVE, 1, 1, .false., iflid)
+call field_set_key_int(iflid, keypid, ivarfl(ipr))
+
 call add_boundary_property_field_owner('boundary_density', 'Boundary Density', &
                                        ibrom)
 
@@ -156,26 +167,26 @@ if (iturb.eq.0) then
   call hide_property(ivisct)
 endif
 
+! Hybrid RANS/LES function f_d is stored for Post Processing in hybrid_blend.
 ! If hybrid spatial scheme is activated for the velocity (ischcv=3)
 ! creation of the field hybrid_blend wihich contains the
 ! local blending factor for each cell
+
 call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
 ischcp = vcopt_u%ischcv
-if (ischcp.eq.3) then
+if (ischcp.eq.3.or.hybrid_turb.gt.0) then
   call add_property_field_1d('hybrid_blend', 'Hybrid blending function', iflid)
 end if
 
+if (hybrid_turb.eq.3) then
+  call add_property_field_1d('hybrid_sas_source_term', 'SAS hybrid source term', iflid)
+endif
+
 if  (iturb.eq.60) then
   call add_property_field_1d('s2', 'S2', is2kw)
   call hide_property(is2kw)
   call add_property_field_1d('vel_gradient_trace', 'Vel. Gradient Trace', idivukw)
   call hide_property(idivukw)
-  ! Hybrid RANS/LES function f_d is stored for Post Processing in hybrid_blend.
-  ! If  hybrid spatial scheme is activated for the velocity (ischcv=3) f_d
-  ! is used as blending factor and this field already exists
-  if (iddes.eq.1.and.ischcp.ne.3) then
-    call add_property_field_1d('hybrid_blend', 'Hybrid blending function', iflid)
-  end if
 endif
 
 call add_property_field_1d('courant_number', 'CFL', icour)
@@ -209,13 +220,15 @@ endif
 
 call ppprop
 
+idim1 = 1
+idim3 = 3
+idim6 = 6
+
 ! --- Properties for Darcy module
 
 if (ippmod(idarcy).eq.1) then
 
   has_previous = .true.
-  idim1 = 1
-  idim6 = 6
   f_name = 'saturation'
   f_label = 'Saturation'
   call add_property_field(f_name, f_label, idim1, has_previous, iflid)
@@ -290,13 +303,20 @@ if (ippmod(idarcy).eq.1) then
 
 endif
 
+if (iand(ivofmt,VOF_FREE_SURFACE).ne.0) then
+  idrift = 2
+  has_previous = .false.
+  call add_property_field('drift_velocity', 'Drift Velocity', idim3, &
+                          has_previous, iflid)
+endif
+
 ! --- Mesh displacement for ALE
 
-if (iale.eq.1) then
+if (iale.ge.1) then
 
   has_previous = .true.
   idim3 = 3
-  f_name = 'disale'
+  f_name = 'mesh_displacement'
   f_label = 'Mesh displacement'
   type_flag = FIELD_PROPERTY
   post_flag = POST_ON_LOCATION
@@ -309,6 +329,15 @@ if (iale.eq.1) then
 
   call field_set_key_str(fdiale, keylbl, trim(f_label))
 
+  has_previous = .false.
+  idim3 = 3
+  f_name = 'vtx_coord0'
+  type_flag = FIELD_PROPERTY
+  location_id = 4 ! variables defined on vertices
+
+  call field_create(f_name, type_flag, location_id, idim3, &
+                    has_previous, iflid)
+
 endif
 
 
@@ -328,15 +357,9 @@ endif
 call cs_field_pointer_map_base
 call cs_field_pointer_map_boundary
 
-return
-
-!===============================================================================
-! 2. Formats
-!===============================================================================
-
-!===============================================================================
-! 5. End
-!===============================================================================
+!====
+! End
+!====
 
 return
 end subroutine fldprp
@@ -347,7 +370,7 @@ end subroutine fldprp
 !>        with no previous time values and with default options
 !
 !> It is recommended not to define property names of more than 16
-!> characters, to get a clear execution listing (some advanced writing
+!> characters, to get a clear execution log (some advanced writing
 !> levels take into account only the first 16 characters).
 !
 !-------------------------------------------------------------------------------
@@ -451,7 +474,7 @@ end subroutine hide_property
 !>        with default options
 !
 !> It is recommended not to define property names of more than 16
-!> characters, to get a clear execution listing (some advanced writing
+!> characters, to get a clear execution log (some advanced writing
 !> levels take into account only the first 16 characters).
 !
 !-------------------------------------------------------------------------------
@@ -480,6 +503,7 @@ use entsor
 use numvar
 use post
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -496,6 +520,11 @@ integer, intent(out)         :: f_id
 
 integer  type_flag, post_flag, location_id
 
+character(len=len_trim(name)+1, kind=c_char) :: c_name
+integer(c_int) :: c_type_flag
+integer(c_int) :: c_location_id
+integer(c_int) :: c_dim
+logical(c_bool) :: c_has_previous
 !===============================================================================
 
 type_flag = FIELD_INTENSIVE + FIELD_PROPERTY
@@ -510,8 +539,21 @@ if (f_id .ge. 0) then
 endif
 
 ! Create field
+c_name = trim(name)//c_null_char
+c_type_flag = type_flag
+c_location_id = location_id
+c_dim = dim
 
-call field_create(name, type_flag, location_id, dim, has_previous, f_id)
+if (has_previous) then
+  c_has_previous = .true.
+else
+  c_has_previous = .false.
+endif
+
+call cs_physical_property_define_from_field(c_name, c_type_flag, &
+  c_location_id, dim, c_has_previous)
+
+f_id = cs_physical_property_field_id_by_name(c_name)
 
 call field_set_key_int(f_id, keyvis, post_flag)
 call field_set_key_int(f_id, keylog, 1)
@@ -526,18 +568,6 @@ return
 ! Formats
 !---
 
-#if defined(_CS_LANG_FR)
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERREUR :    ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    ========                                                ',/,&
-'@     LE CHAMP : ', a, 'EST DEJA DEFINI.                     ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#else
  1000 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -548,7 +578,6 @@ return
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
-#endif
 
 end subroutine add_property_field
 
@@ -558,7 +587,7 @@ end subroutine add_property_field
 !>        with default options
 !
 !> It is recommended not to define property names of more than 16
-!> characters, to get a clear execution listing (some advanced writing
+!> characters, to get a clear execution log (some advanced writing
 !> levels take into account only the first 16 characters).
 !
 !-------------------------------------------------------------------------------
@@ -629,18 +658,6 @@ return
 ! Formats
 !---
 
-#if defined(_CS_LANG_FR)
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERREUR :    ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    ========                                                ',/,&
-'@     LE CHAMP : ', a, 'EST DEJA DEFINI.                     ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#else
  1000 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -651,7 +668,6 @@ return
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
-#endif
 
 end subroutine add_boundary_property_field_owner
 
