@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -55,10 +55,10 @@ use coincl
 use cpincl
 use ppincl
 use radiat
-use ihmpre
 use mesh
 use field
 use cs_c_bindings
+use cfpoin, only:ieos
 
 !===============================================================================
 
@@ -71,7 +71,8 @@ integer       nmodpp
 ! Local variables
 
 integer       ipp
-integer       iok   , keycpl, nmodpp_compatibility
+integer       iok, keycpl, nmodpp_compatibility, vof_mask
+integer       key_lim_id, kscmin, kscmax
 
 type(var_cal_opt) :: vcopt
 
@@ -112,6 +113,9 @@ call field_get_key_id('coupled', keycpl)
 ! ---> Remplissage de ITYTUR
 itytur = iturb/10
 
+! Set type and order of the turbulence model
+call cs_set_type_order_turbulence_model()
+
 ! ---> Coherence modele
 !     Rq : ATTENTION il faudrait renforcer le blindage
 
@@ -134,6 +138,11 @@ if (ippmod(igmix).ne.-1 .and. ippmod(icompf) .ne. -1) then
   nmodpp_compatibility = nmodpp_compatibility - 1
 endif
 
+! Atmo in humid atmosphere et Couling tower (iaeros) coupling
+if (ippmod(iatmos).eq.2 .and. ippmod(iaeros) .ne. -1) then
+  nmodpp_compatibility = nmodpp_compatibility - 1
+endif
+
 if (nmodpp_compatibility.gt.1) then
   write(nfecra,6000)
   iok = iok + 1
@@ -190,29 +199,53 @@ else
   call add_variable_field('hydraulic_head', 'Hydraulic head', 1, ipr)
 endif
 
-! Cavitation: activate VoF
-if (icavit.ge.0.and.ivofmt.lt.0) ivofmt = 0
+! Enabled VoF model if free surface or mass transfer modeling enabled
+vof_mask = ior(VOF_FREE_SURFACE, VOF_MERKLE_MASS_TRANSFER)
+if (iand(ivofmt, vof_mask).ne.0) &
+     ivofmt = ior(VOF_ENABLED, ivofmt)
+
+! Mass balance equation options (pressure)
 
 call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
-if (ippmod(icompf).ge.0) then
+
+! elliptic equation
+vcopt%iconv = 0
+
+if (ippmod(icompf).ge.0) then ! compressible algorithm
   vcopt%istat = 1
 else
   vcopt%istat = 0
 endif
-! VoF algorithm: activate the weighthening for the pressure
-if (ivofmt.ge.0) vcopt%iwgrec = 1
-vcopt%iconv = 0
+
+! VoF algorithm: activate the weighting for the pressure
+if (ivofmt.gt.0) vcopt%iwgrec = 1
+
 call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
 
-! Void fraction (Volume of Fluid algorithm)
-if (ivofmt.ge.0) then
+! void fraction (VoF algorithm)
+
+if (ivofmt.gt.0) then
   call add_variable_field('void_fraction', 'Void Fraction', 1, ivolf2)
   call field_get_key_struct_var_cal_opt(ivarfl(ivolf2), vcopt)
-  vcopt%idiff = 0
+  vcopt%idiff = 0  ! pure convection equation
+
+  ! NVD/TVD scheme
+  vcopt%ischcv = 4
+  call field_get_key_id("limiter_choice", key_lim_id)
+  ! (CICSAM limiter)
+  call field_set_key_int(ivarfl(ivolf2), key_lim_id, 11)
+  ! Beta Limiter
+  vcopt%isstpc = 2
+
   call field_set_key_struct_var_cal_opt(ivarfl(ivolf2), vcopt)
+  ! Bounds for the beta limiter
+  call field_get_key_id("min_scalar", kscmin)
+  call field_get_key_id("max_scalar", kscmax)
+  call field_set_key_double(ivarfl(ivolf2), kscmin, 0.d0)
+  call field_set_key_double(ivarfl(ivolf2), kscmax, 1.d0)
 endif
 
-! --- Turbulence
+! Turbulence
 
 if (itytur.eq.2) then
   call add_variable_field('k', 'Turb Kinetic Energy', 1, ik)
@@ -246,9 +279,9 @@ else if (itytur.eq.3) then
     call field_get_key_struct_var_cal_opt(ivarfl(ial), vcopt)
     vcopt%istat = 0
     vcopt%iconv = 0
-    call field_set_key_struct_var_cal_opt(ivarfl(ial), vcopt)
     ! For alpha, we always have a diagonal term, so do not shift the diagonal
-    idircl(ial) = 0
+    vcopt%idircl = 0
+    call field_set_key_struct_var_cal_opt(ivarfl(ial), vcopt)
   endif
 else if (itytur.eq.5) then
   call add_variable_field('k', 'Turb Kinetic Energy', 1, ik)
@@ -259,17 +292,17 @@ else if (itytur.eq.5) then
     call field_get_key_struct_var_cal_opt(ivarfl(ifb), vcopt)
     vcopt%istat = 0
     vcopt%iconv = 0
-    call field_set_key_struct_var_cal_opt(ivarfl(ifb), vcopt)
     ! For fb, we always have a diagonal term, so do not shift the diagonal
-    idircl(ifb) = 0
+    vcopt%idircl = 0
+    call field_set_key_struct_var_cal_opt(ivarfl(ifb), vcopt)
   else if (iturb.eq.51) then
     call add_variable_field('alpha', 'Alpha', 1, ial)
     call field_get_key_struct_var_cal_opt(ivarfl(ial), vcopt)
     vcopt%istat = 0
     vcopt%iconv = 0
-    call field_set_key_struct_var_cal_opt(ivarfl(ial), vcopt)
     ! For alpha, we always have a diagonal term, so do not shift the diagonal
-    idircl(ial) = 0
+    vcopt%idircl = 0
+    call field_set_key_struct_var_cal_opt(ivarfl(ial), vcopt)
   endif
 else if (iturb.eq.60) then
   call add_variable_field('k', 'Turb Kinetic Energy', 1, ik)
@@ -279,10 +312,16 @@ else if (iturb.eq.70) then
 endif
 
 ! Mesh velocity with ALE
+if (iale.ge.1) then
 
-if (iale.eq.1) then
+  ! field defined on vertices if CDO-Vb scheme is used
+  if (iale.eq.2) then
+    call add_cdo_variable_field('mesh_velocity', 'Mesh Velocity',    &
+                                3, MESH_LOCATION_VERTICES, 1, iuma)
+  else
+    call add_variable_field('mesh_velocity', 'Mesh Velocity', 3, iuma)
+  endif
 
-  call add_variable_field('mesh_velocity', 'Mesh Velocity', 3, iuma)
   call field_set_key_int(ivarfl(iuma), keycpl, 1)
 
   ivma = iuma + 1
@@ -320,10 +359,6 @@ if (nmodpp.eq.0) then
     iscalt = ihm
   endif
 
-  if (itherm.ne.0 .and. iihmpr.eq.1) then
-    call uithsc
-  endif
-
 endif
 
 call add_user_scalar_fields
@@ -364,132 +399,6 @@ return
 ! 5. FORMATS
 !===============================================================================
 
-#if defined(_CS_LANG_FR)
- 6000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@     PLUSIEURS MODELES DE PHYSIQUES PARTICULIERES           ',/,&
-'@     INCOMPATIBLES SONT ACTIVES                             ',/,&
-'@                                                            ',/,&
-'@  Seul les modeles de physiques particulieres compressible  ',/,&
-'@    et melange de gaz parfaits peuvent etre actives         ',/,&
-'@    simultanement.                                          ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Modifier les indicateurs de IPPMOD dans usppmo.           ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 6001 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@     SELECTION INCORRECTE DU MODELE PHYSIQUE PARTICULIERE   ',/,&
-'@                                                            ',/,&
-'@  Les valeurs des indicateurs du tableau IPPMOD ne sont pas ',/,&
-'@    admissibles                                             ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Modifier les indicateurs de IPPMOD dans usppmo.           ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 6002 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION :  A L''ENTREE DES DONNEES                    ',/,&
-'@    =========                                               ',/,&
-'@     EQUATION D''ETAT INCOMPATIBLE AVEC LES PHYSIQUES       ',/,&
-'@     PARTICULIERES SELECTIONNEES                            ',/,&
-'@                                                            ',/,&
-'@  Les modeles de physiques particulieres compressible et    ',/,&
-'@    et melange de gaz parfaits sont actives simultanement   ',/,&
-'@    mais l''equation d''etat selectionnee n''est pas melange',/,&
-'@    de gas parfait (ieos different de 3).                   ',/,&
-'@                                                            ',/,&
-'@  L''indicateur ieos a ete repositionne a 3 et le calcul    ',/,&
-'@  sera execute.                                             ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 6010 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@     NOMBRE DE SCALAIRES ERRONE                             ',/,&
-'@                                                            ',/,&
-'@  Le nombre de scalaires utilisateur doit etre un entier    ',/,&
-'@    positif ou nul. Il vaut ici   NSCAUS  = ',I10            ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 6011 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@     NOMBRE DE SCALAIRES TROP GRAND                         ',/,&
-'@                                                            ',/,&
-'@  Le nombre de scalaires utilisateurs                       ',/,&
-'@    demande                          est  NSCAUS = ',I10     ,/,&
-'@  Le nombre de scalaires total                              ',/,&
-'@    autorise   dans paramx           est  NSCAMX = ',I10     ,/,&
-'@                                                            ',/,&
-'@  La valeur maximale autorisee de NSCAUS                    ',/,&
-'@                          est donc  NSCAMX        = ',I10    ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier NSCAUS.                                          ',/,&
-'@                                                            ',/,&
-'@  NSCAMX doit valoir au moins ',I10                          ,/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 6012 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@     NOMBRE DE SCALAIRES TROP GRAND                         ',/,&
-'@                                                            ',/,&
-'@  Le nombre de scalaires utilisateurs                       ',/,&
-'@    demande                          est  NSCAUS = ',I10     ,/,&
-'@  Le nombre de scalaires pour les physiques particulieres   ',/,&
-'@    necessaire avec le modele choisi est  NSCAPP = ',I10     ,/,&
-'@  Le nombre de scalaires total                              ',/,&
-'@    autorise   dans paramx           est  NSCAMX = ',I10     ,/,&
-'@                                                            ',/,&
-'@  La valeur maximale autorisee de NSCAUS                    ',/,&
-'@    avec le modele choisi est donc NSCAMX-NSCAPP = ',I10     ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier NSCAUS.                                          ',/,&
-'@                                                            ',/,&
-'@  NSCAMX doit valoir au moins ',I10                          ,/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
  6000 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -611,8 +520,6 @@ return
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
 !===============================================================================
 ! 5. FIN
 !===============================================================================
@@ -629,7 +536,7 @@ end subroutine
 !> \brief add field defining a general solved variable, with default options
 !
 !> It is recommended not to define variable names of more than 16
-!> characters, to get a clear execution listing (some advanced writing
+!> characters, to get a clear execution log (some advanced writing
 !> levels take into account only the first 16 characters).
 
 !-------------------------------------------------------------------------------
@@ -637,10 +544,10 @@ end subroutine
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[in]  name           field name
-!> \param[in]  label          field default label, or empty
-!> \param[in]  dim            field dimension
-!> \param[out] ivar           variable number for defined field
+!> \param[in]  name          field name
+!> \param[in]  label         field default label, or empty
+!> \param[in]  dim           field dimension
+!> \param[out] ivar          variable number for defined field
 !_______________________________________________________________________________
 
 subroutine add_variable_field &
@@ -670,15 +577,12 @@ integer, intent(out)         :: ivar
 ! Local variables
 
 integer  id, ii
-integer  location_id
 
 integer, save :: keyvar = -1
 
-location_id = 1         ! variables defined on cells
-
 ! Create field
 
-call variable_field_create(name, label, location_id, dim, id)
+call variable_field_create(name, label, MESH_LOCATION_CELLS, dim, id)
 
 if (keyvar.lt.0) then
   call field_get_key_id("variable_id", keyvar)
@@ -696,8 +600,6 @@ call field_set_key_int(id, keyvar, ivar)
 
 call init_var_cal_opt(id)
 
-call field_set_key_double(id, ksigmas, 1.d0)
-
 if (dim .gt. 1) then
   do ii = 2, dim
     ivarfl(ivar + ii - 1) = id
@@ -710,6 +612,88 @@ end subroutine add_variable_field
 
 !===============================================================================
 
+!> \brief Add a field defining a general solved variable, with default options
+!>        This variable is solved with a CDO scheme.
+!
+!> It is recommended not to define variable names of more than 16
+!> characters, to get a clear execution log (some advanced writing
+!> levels take into account only the first 16 characters).
+
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]  name          field name
+!> \param[in]  label         field default label, or empty
+!> \param[in]  dim           field dimension
+!> \param[in]  location_id   id of the mesh location where the field is defined
+!> \param[in]  has_previous  if greater than 0 then stores previous state
+!> \param[out] ivar          variable number for defined field
+!_______________________________________________________________________________
+
+subroutine add_cdo_variable_field &
+ ( name, label, dim, location_id, has_previous, ivar )
+
+!===============================================================================
+! Module files
+!===============================================================================
+
+use paramx
+use dimens
+use entsor
+use numvar
+use field
+use cs_c_bindings
+
+!===============================================================================
+
+implicit none
+
+! Arguments
+
+character(len=*), intent(in) :: name, label
+integer, intent(in)          :: dim, location_id, has_previous
+integer, intent(out)         :: ivar
+
+! Local variables
+
+integer  id, ii
+
+integer, save :: keyvar = -1
+
+! Create field
+
+call variable_cdo_field_create(name, label, location_id, dim, has_previous, id)
+
+if (keyvar.lt.0) then
+  call field_get_key_id("variable_id", keyvar)
+endif
+
+ivar = nvar + 1
+nvar = nvar + dim
+
+! Check we have enough slots
+call fldvar_check_nvar
+
+ivarfl(ivar) = id
+
+call field_set_key_int(id, keyvar, ivar)
+
+call init_var_cal_opt(id)
+
+if (dim .gt. 1) then
+  do ii = 2, dim
+    ivarfl(ivar + ii - 1) = id
+  enddo
+endif
+
+return
+
+end subroutine add_cdo_variable_field
+
+!===============================================================================
+
 !> \brief add fields defining user solved scalar variables,
 !>        with default options
 !
@@ -803,7 +787,6 @@ do id = nfld1, nfld2 - 1
 
   call field_set_key_int(id, keyvar, ivar)
   call field_set_key_int(id, keysca, iscal)
-  call field_set_key_double(id, ksigmas, 1.d0)
   call init_var_cal_opt(id)
 
   if (dim .gt. 1) then
@@ -828,7 +811,7 @@ end subroutine add_user_scalar_fields
 !>        with default options
 !
 !> It is recommended not to define variable names of more than 16
-!> characters, to get a clear execution listing (some advanced writing
+!> characters, to get a clear execution log (some advanced writing
 !> levels take into account only the first 16 characters).
 !
 !-------------------------------------------------------------------------------
@@ -881,7 +864,7 @@ end subroutine add_model_scalar_field
 !>        with default options
 !
 !> It is recommended not to define variable names of more than 16
-!> characters, to get a clear execution listing (some advanced writing
+!> characters, to get a clear execution log (some advanced writing
 !> levels take into account only the first 16 characters).
 !
 !-------------------------------------------------------------------------------
@@ -1007,7 +990,6 @@ enddo
 
 call field_set_key_int(f_id, keyvar, ivar)
 call field_set_key_int(f_id, keysca, iscal)
-call field_set_key_double(f_id, ksigmas, 1.d0)
 call init_var_cal_opt(f_id)
 
 return
@@ -1055,32 +1037,6 @@ return
 ! Formats
 !---
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERREUR :    ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    ========                                                ',/,&
-'@     NOMBRE DE VARIABLES TROP GRAND                         ',/,&
-'@                                                            ',/,&
-'@  Le type de calcul defini                                  ',/,&
-'@    correspond a un nombre de variables NVAR   >= ', i10     ,/,&
-'@  Le nombre de variables maximal prevu                      ',/,&
-'@                      dans paramx   est NVARMX  = ', i10     ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres                                   ',/,&
-'@                                                            ',/,&
-'@  Si NVARMX est augmente, le code doit etre reinstalle.     ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
  1000 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -1103,8 +1059,6 @@ return
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
 end subroutine fldvar_check_nvar
 
 !===============================================================================
@@ -1144,32 +1098,21 @@ integer id
 ! Local variables
 type(var_cal_opt) :: vcopt
 
+! Most values set by default at in _var_cal_opt default;
+! see cs_parameters.c
+
 call field_get_key_struct_var_cal_opt(id, vcopt)
-vcopt%iwarni = 0
-vcopt%iconv  = 1
-vcopt%istat  = 1
-vcopt%idiff  = 1
-vcopt%idifft = 1
-vcopt%idften = ISOTROPIC_DIFFUSION
-vcopt%iswdyn = 0
-vcopt%ischcv = 1
-vcopt%ibdtso = 1
+
+! Undefined values, may be modified by modini
 vcopt%isstpc = -999
-vcopt%nswrgr = 100
-vcopt%nswrsm = -999
-vcopt%imrgra = 0
+vcopt%nswrsm = -1
 vcopt%imligr = -999
-vcopt%ircflu = 1
-vcopt%iwgrec = 0
-vcopt%thetav = -999.d0
-vcopt%blencv = -999.d0
-vcopt%blend_st = 0.d0
-vcopt%epsilo = -999.d0
-vcopt%epsrsm = -999.d0
-vcopt%epsrgr = 1.d-5
-vcopt%climgr = 1.5d0
-vcopt%extrag = 0.d0
-vcopt%relaxv = -999.d0
+vcopt%thetav = -1.d0
+vcopt%blencv = -1.d0
+vcopt%epsilo = -1.d0
+vcopt%epsrsm = -1.d0
+vcopt%relaxv = -1.d0
+
 call field_set_key_struct_var_cal_opt(id, vcopt)
 
 return
@@ -1217,32 +1160,6 @@ return
 ! Formats
 !---
 
-#if defined(_CS_LANG_FR)
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERREUR :    ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    ========                                                ',/,&
-'@     NOMBRE DE SCALAIRES TROP GRAND                         ',/,&
-'@                                                            ',/,&
-'@  Le nombre de scalaires utilisateurs                       ',/,&
-'@    demande                          est  NSCAUS = ', i10    ,/,&
-'@  Le nombre de scalaires total                              ',/,&
-'@    autorise   dans paramx           est  NSCAMX = ', i10    ,/,&
-'@                                                            ',/,&
-'@  La valeur maximale possible de NSCAUS                     ',/,&
-'@    avec le modele choisi est donc NSCAMX-NSCAUS = ', i10    ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier NSCAUS.                                          ',/,&
-'@                                                            ',/,&
-'@  Si NSCAMX est augmente, le code doit etre reinstalle.     ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#else
  1000 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -1268,8 +1185,6 @@ return
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
 end subroutine fldvar_check_nscapp
 
 !===============================================================================
