@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -31,6 +31,7 @@ subroutine iniini
 ! Module files
 !===============================================================================
 
+use atincl
 use paramx
 use cstnum
 use dimens
@@ -45,7 +46,6 @@ use alstru
 use alaste
 use parall
 use period
-use ihmpre
 use cplsat
 use ppincl
 use ppcpfu
@@ -58,6 +58,9 @@ use radiat
 use turbomachinery
 use cs_nz_condensation, only: nzones
 use ctincl
+use cfpoin
+use vof
+use cs_c_bindings
 
 !===============================================================================
 
@@ -76,21 +79,6 @@ integer          istr
 
 write(nfecra, 900)
 
-#if defined(_CS_LANG_FR)
-
- 900  format(/,                                                   &
-'===============================================================',&
-/,/,                                                        &
-'                   PREPARATION DU CALCUL                     ',/,&
-'                   =====================                     ',/,&
-                                                                /,&
-                                                                /,&
-' =========================================================== ',/,&
-                                                                /,&
-                                                                /)
-
-#else
-
  900  format(/,                                                   &
 '===============================================================',&
 /,/,                                                        &
@@ -102,10 +90,6 @@ write(nfecra, 900)
                                                                 /,&
                                                                 /)
 
-#endif
-
-
-
 !===============================================================================
 ! 0. Global field keys
 !===============================================================================
@@ -117,10 +101,12 @@ call field_get_key_id('post_vis', keyvis)
 call field_get_key_id("inner_mass_flux_id", kimasf)
 call field_get_key_id("boundary_mass_flux_id", kbmasf)
 
-call field_get_key_id("scalar_diffusivity_id", kivisl)
-call field_get_key_id("scalar_diffusivity_ref", kvisl0)
+call field_get_key_id("diffusivity_id", kivisl)
+call field_get_key_id("diffusivity_ref", kvisl0)
 
-call field_get_key_id("scalar_density_id", kromsl)
+call field_get_key_id("is_temperature", kscacp)
+
+call field_get_key_id("density_id", kromsl)
 
 call field_get_key_id("gradient_weighting_id", kwgrec)
 
@@ -139,6 +125,7 @@ iporf = -1
 ! 1. Map Fortran pointers to C global data
 !===============================================================================
 
+call atmo_init
 call time_step_init
 call time_step_options_init
 call thermal_model_init
@@ -147,16 +134,22 @@ call turb_rans_model_init
 call turb_les_model_init
 call turb_model_constants_init
 call wall_functions_init
-call stokes_options_init
 call physical_constants_init
+call porosity_from_scan_init
 call fluid_properties_init
 call space_disc_options_init
-call piso_options_init
+call time_scheme_options_init
+call velocity_pressure_options_init
+call restart_auxiliary_options_init
 call turb_reference_values_init
 call listing_writing_period_init
 call radiat_init
 call gas_mix_options_init
 call ctwr_properties_init
+call map_ale
+call cf_model_init
+call vof_model_init
+call cavitation_model_init
 
 call map_turbomachinery_model(iturbo, ityint)
 
@@ -166,14 +159,6 @@ call map_turbomachinery_model(iturbo, ityint)
 
 ! ---> NFECRA vaut 6 par defaut ou 9 en parallele (CSINIT)
 
-!    Methode des vortex : on utilise la meme unite
-!                  Pour le fichier de donnees, on specifie l'unite ici, mais le
-!                  nom est laisse dans usvort. On utilise 20 et pas 11
-!                  car en cas d'entree multiple, les deux fichiers doivent etre
-!                  ouverts en meme temps dans VORINI
-impmvo = 11
-impdvo = 20
-
 ! ---> Fichier aval
 
 !     NTSUIT : Periode de sauvegarde du fichier suite
@@ -184,11 +169,6 @@ impdvo = 20
 
 ntsuit = 0
 
-!    Methode des vortex : on utilise la meme unite
-!                  et le format ascii obligatoirement
-!                  (pas de detection automatique, fichiers de taille faible)
-impvvo = 20
-
 ! ---> Fichier thermochinie
 !        FPP : utilisateur
 !        JNF : Janaf
@@ -203,7 +183,7 @@ indjon = 1
 
 ! ---> Fichiers module atmospherique
 impmet = 26
-ficmet = 'meteo'
+call atmo_set_meteo_file_name('meteo')
 
 ! ---> Fichiers historiques
 
@@ -238,7 +218,7 @@ do ii = 1, nusrmx
   impusr(ii) = 69+ii
 enddo
 
-! ---> Sorties listing
+! ---> Sorties log
 
 ntlist = 1
 
@@ -250,14 +230,6 @@ ipstdv(ipsttp) = 0
 ipstdv(ipstft) = 1
 ipstdv(ipstnu) = 0
 
-! ---> CPU
-!      TMARUS : marge (Arret du calcul avant limite CPU)
-!        Si TMARUS negatif, le code calcule une marge seul
-!        Sinon, il utilise TMARUS (donnee en secondes)
-
-tmarus = -1.d0
-
-
 ! Ici entsor.f90 est completement initialise
 
 !===============================================================================
@@ -270,7 +242,6 @@ ncel   = 0
 ncelet = 0
 nfac   = 0
 nfabor = 0
-nfml   = 0
 nnod   = 0
 lndfac = 0
 lndfbr = 0
@@ -379,20 +350,11 @@ itempb = -1
 !       Les tests dans l'algo portent sur ces indicateurs
 
 
-!   -- Schema en temps (regroupera les options suivantes)
-!     = 1 Standard ordre 1
-!     = 2 Standard ordre 2
-!     si on veut, on peut en rajouter.
-!     Cette variable conditionne toutes les autres de maniere automatique,
-!     pour definir des schemas coherents dans modini.
-ischtp = -999
-
 !   -- Flux de masse (-999 = non initialise)
-!     = 1 Standard d'ordre 1 (THETFL = -999 inutile)
-!     = 0 Explicite (THETFL = 0)
-!     = 2 Ordre deux (THETFL = 0.5)
+!     = 1 Standard d'ordre 1
+!     = 0 Explicite
+!     = 2 Ordre deux
 istmpf = -999
-thetfl =-999.d0
 
 !   -- Termes sources Navier Stokes
 !     Pour les termes sources explicites en std, I..EXT definit
@@ -411,23 +373,6 @@ thetfl =-999.d0
 !       (il suit bilsc2)
 isno2t = -999
 thetsn =-999.d0
-!   -- Termes sources Grandeurs turbulentes
-!     Pour les termes sources explicites en std, I..EXT definit
-!       l'extrapolation -theta ancien + (1+theta) nouveau
-!     = 0 explicite
-!     = 1 extrapolation avec theta = 1/2
-!     = 2 extrapolation avec theta = 1
-!       0 implique pas de reservation de tableaux
-!       1 et 2 sont deux options equivalentes, la difference etant faite
-!       uniquement au moment de fixer theta
-!     Pour les termes sources implicites en std, I..EXT definit
-!       la mise a l'ordre 2 ou non avec le thetav de la variable associee
-!     = 0 implicite (std)
-!     > 0 utilisation du thetav
-!     Noter cpdt que le TS d'acc. masse n'est pas regi par I..EXT
-!       (il suit bilsc2)
-isto2t = -999
-thetst = -999.d0
 
 !    -- Proprietes physiques
 !     I..EXT definit l'extrapolation -theta ancien + (1+theta) nouveau
@@ -441,15 +386,11 @@ thetst = -999.d0
 !       fichier suite portant les valeurs adaptees)
 
 !     Masse volumique
-iroext = -999
-thetro = -999.d0
 initro = 0
 !     Viscosite totale
-iviext = -999
 thetvi = -999.d0
 initvi = 0
 !     Chaleur specifique
-icpext = -999
 thetcp = -999.d0
 initcp = 0
 
@@ -460,9 +401,6 @@ epsup  = 1.d-5
 xnrmu0 = 0.d0
 xnrmu  = 0.d0
 
-!   -- Nb d'iter point fixe vitesse pression
-nterup = 1
-
 do iscal = 1, nscamx
 
 !   -- Termes sources des scalaires
@@ -494,7 +432,6 @@ do iscal = 1, nscamx
 !     INIT.. =1 indique que la variable a ete proprement initialisee (dans un
 !       fichier suite portant les valeurs adaptees)
 
-  ivsext(iscal) = -999
   thetvs(iscal) = -999.d0
   initvs(iscal) = 0
 
@@ -504,96 +441,24 @@ enddo
 ! default: based on cell center mesh velocity
 iflxmw = 0
 
-! --- Reconstruction des gradients
-!       On donne les valeurs par defaut
-!       Pour la methode de limitation, on decidera plus tard
-!         selon les choix de l'utilisateur
-!       On n'active pas l'extrapolation des gradients par defaut
-!         meme pour la pression (par securite : moins stable sur certains cas)
-
-anomax = -grand*10.d0
-
-! --- Solveurs iteratifs
-!       La methode de resolution sera choisie selon les equations
-!       La valeur de epsilon relatif est tres faible
-!         (1.D-5 pourrait suffire)
-!       On met IDIRCL a 1 pour toutes les variables. Pour toutes les
-!       variables sauf pression et fb (en v2f) on a ISTAT=1, le
-!       decalage de diagonale ne sera pas active. Pour la pression
-!       on decalera la diagonale si necessaire. Pour fb, on sait qu'il
-!       y a un autre terme diagonal (meme si ISTAT=0), donc IDIRCL
-!       sera mis a 0 dans varpos.
-
-do ii = 1, nvarmx
-  idircl(ii) = 1
-  ndircl(ii) = 0
-enddo
-
-! --- Restarted calculation
-!       By default, non-restarted calculation
-!       Write auxiliary restart file by default
-!       Read auxiliary restart file by default (in case of restarted calculation)
-!       The match between new scalars and old scalars will be established later
-!         (GUI, cs_user_parameters.f90, and lecamo)
-!       The restart indicator of the 1D wall thermal model is initialized by default
-!         to -1, to force the user to set it in uspt1d.
-!       The same goes for the vortex method restart indicator.
-!       The same goes for the synthetic turbulence method restart indicator.
+! Restarted calculation
+!   The restart indicator of the 1D wall thermal model is initialized by default
+!   to -1, to force the user to set it in uspt1d.
+!   The same goes for the synthetic turbulence method restart indicator.
 
 isuite = 0
-iecaux = 1
-ileaux = 1
 isuit1 = -1
-isuivo = -1
-isuisy = -1
 
-! --- Reperage du temps
-
-ntpabs = 0
-ntcabs = ntpabs
-ntmabs = 10
-ntinit = 2
-
-inpdt0 = 0
-
-ttpabs = 0.d0
-ttcabs = ttpabs
+! Time stepping (value not already in C)
 
 ttpmob = 0.d0
 ttcmob = ttpmob
 
-! --- Marche en temps
-!       Par defaut pas de temps uniforme et constant,
-!         sans coef multiplicatif
-!       Dans les cas a pas de temps variable, par defaut
-!         COUMAX = FOUMAX = 0.5 et variation max 10%
-!       Les autres grandeurs sont a renseigner par l'utilisateur
-!       Pour DTMIN et DTMAX, si on impose DTMIN > DTMAX,
-!         les bornes sont prises egales a +/-1D12
-
-idtvar = 0
-
-coumax = 1.d0
-foumax = 10.d0
-cflmmx = 0.99d0
-dtmin  = -grand*10.d0
-dtmax  = -grand*10.d0
-varrdt = 0.1d0
-
-dtref  = -grand*10.d0
-
 do ii = 1, nvarmx
   cdtvar(ii) = 1.d0
 enddo
-relxst = 0.7d0
 
-
-!     Par defaut, pas de limitation du pas de temps liee aux
-!     effets de densite
-
-iptlro = 0
-
-! --- Thermique
+! Thermal model
 
 ! No thermal scalar by default
 itherm = 0
@@ -610,17 +475,13 @@ ihgas = -1
 
 iturb  =-999
 itytur =-999
-iddes  = 0
+hybrid_turb  = 0
 
 ! Parfois, IGRHOK=1 donne des vecteurs non physiques en paroi
 !        IGRHOK = 1
 igrhok = 0
 igrake = 1
-iwallf =-999
-iwalfs = -999
-iwallt = 0
 iclkep = 0
-ikecou =-999
 irijnu = 0
 irijrb = 0
 irijec = 0
@@ -629,6 +490,7 @@ idifre = 1
 iclsyr = 1
 iclptr = 0
 idries =-1
+ikwcln = 1
 
 ! --- Rotation/curvature correction of turbulence models
 !     Unactivated by default
@@ -670,17 +532,6 @@ pthermax= -1.d0
 sleak = 0.d0
 kleak = 2.9d0
 
-! --- VOF module (not activated by default)
-!       -1: module not activated
-!        0: module activated
-ivofmt = -1
-
-! --- Cavitation module (not activated by default)
-!       -1: module not activated
-!        0: no vaporization/condensation model
-!        1: Merkle's model
-icavit = -1
-
 ! --- Radiative Transfert (not activated by default)
 iirayo = 0
 
@@ -712,12 +563,6 @@ nzones = -1
 
 itagms = 0
 
-! --- Interpolation face des viscosites
-!     = 0 ARITHMETIQUE
-!     = 1 HARMONIQUE
-
-imvisf = 0
-
 ! --- Type des CL, tables de tri
 !       Sera calcule apres cs_user_boundary_conditions.
 
@@ -792,39 +637,19 @@ nftcdt= 0
 !     Non utilisateur
 
 ! --- Calcul de la distance a la paroi
-!     Seules variables utilisateur : ICDPAR, IWARNY
+!     Seules variables utilisateur : ICDPAR
 
 ineedy = 0
 imajdy = 0
 icdpar = -999
-nitmay = 10000
-nswrsy = 1
-nswrgy = 100
-imligy = -999
-ircfly = 1
-ischcy = 1
-isstpy = 0
-iwarny = -999
-ntcmxy = 1000
 
-
-blency = 0.0d0
-epsily = 1.0d-8
-epsrsy = 1.0d-5
-epsrgy = 1.0d-5
-climgy = 1.5d0
-extray = 0.0d0
-coumxy = 5000.d0
-epscvy = 1.0d-8
-yplmxy = 200.d0
-
-! --- Methode des vortex
-ivrtex = 0
+! --- LES balance
+i_les_balance = 0
 
 ! --- Ici tout optcal.f90 est initialise
 
 !===============================================================================
-! 7. TABLEAUX DE cstphy.f90
+! TABLEAUX DE cstphy.f90
 !===============================================================================
 
 ! --- Gravite
@@ -859,9 +684,15 @@ dpow    = 1.d0/(1.d0+bpow)
 ce1     = 1.44d0
 ce2     = 1.92d0
 ce4     = 1.20d0
-sigmak  = 1.00d0
 
-!   pour le Rij-epsilon standard (et SSG pour CRIJ3)
+!   pour le k-epsilon quadratic (Baglietto)
+cnl1  = 0.8d0
+cnl2  = 11.d0
+cnl3  = 4.5d0
+cnl4  = 1.d3
+cnl5  = 1.d0
+
+!   pour le Rij-epsilon LRR (et SSG pour crij3)
 crij1  = 1.80d0
 crij2  = 0.60d0
 crij3  = 0.55d0
@@ -886,8 +717,6 @@ cebmr2  = 0.80d0
 cebmr3  = 0.65d0
 cebmr4  = 0.625d0
 cebmr5  = 0.20d0
-! cebmr6  is used in the buoyant term
-cebmr6  = 0.6d0
 cebme2  = 1.83d0
 cebmmu  = 0.22d0
 xcl     = 0.122d0
@@ -895,17 +724,6 @@ xa1     = 0.1d0
 xceta   = 80.d0
 xct     = 6.d0
 
-!   pour la LES
-xlesfl = 2.d0
-ales   = 1.d0
-bles   = 1.d0/3.d0
-csmago = 0.065d0
-cwale  = 0.25d0
-xlesfd = 1.5d0
-smagmx = csmago**2
-smagmn = 0.d0
-cdries = 26.d0
-
 !   pour le v2f phi-model
 cv2fa1 = 0.05d0
 cv2fe2 = 1.85d0
@@ -921,7 +739,6 @@ cpale1 = 1.44d0
 cpale2 = 1.83d0
 cpale3 = 2.3d0
 cpale4 = 0.4d0
-cpalse = 1.5d0
 cpalmu = 0.22d0
 cpalct = 4.d0
 cpalcl = 0.164d0
@@ -944,6 +761,10 @@ ckwc1  = 10.d0
 ! pour la ddes
 cddes = 0.65d0
 
+! pour le sas
+csas  = 0.11d0
+csas_eta2 = 3.51d0
+
 !   pour le modele de Spalart Allmaras
 csab1    = 0.1355d0
 csab2    = 0.622d0
@@ -958,29 +779,7 @@ cssr1 = 1.d0
 cssr2 = 2.d0
 cssr3 = 1.d0
 
-! for the Cazalbou rotation/curvature correction
-ccaze2 = 1.83d0
-ccaza  = 4.3d0
-ccazsc = 0.119d0
-ccazb  = 5.130d0
-ccazc  = 0.453d0
-ccazd  = 0.682d0
-
-!   echelle de longueur negative, recalculee par la suite
-!    ou entree par l'utilisateur
-almax   = -999.d0
-
-!   vitesse de reference pour l'initialisation de la turbulence
-!    doit etre entree par l'utilisateur, sauf s'il initialise lui-meme
-!    la turbulence.
-uref    = -grand*10.d0
-
-!   longueur caracteristique pour le modele de longueur de melange
-!    doit etre entree par l'utilisateur
-  xlomlg    = -grand*10.d0
-
 ! --- Scalaires
-!       L'utilisateur devra remplir VISLS0
 !       On remplira plus tard, selon les modifs utilisateur,
 !         ISCSTH
 !       On donne la valeur par defaut pour les autres
@@ -990,17 +789,10 @@ uref    = -grand*10.d0
 !         qu'on ne clippe pas les scalaires (sauf a +/-GRAND)
 
 do iscal = 1, nscamx
-  iscacp(iscal) =-10
   iclvfl(iscal) = -1
-  visls0(iscal) =-grand*10.d0
   rvarfl(iscal) = 0.8d0
 enddo
 
-! --- Turbulent flux for a scalar (Default: SGDH)
-do iscal = 1, nscamx
-  iturt(iscal) = 0
-enddo
-
 ! For the turbulent fluxes of the scalar
 c1trit = 4.15d0
 c2trit = 0.55d0
@@ -1019,31 +811,15 @@ cthebdfm = 0.22d0
 xclt   = 0.305d0
 rhebdfm = 0.5d0
 
-
 ! --- Ici tout cstphy a ete initialise
 
 !===============================================================================
-! 9. INITIALISATION DES PARAMETRES DE IHM de ihmpre.f90
+! INITIALISATION DES PARAMETRES ALE de albase.f90 et alstru.f90
 !===============================================================================
 
-!     Par defaut, pas de fichier IHM consulte (on regarde ensuite si on
-!       en trouve un a partir de la ligne de commande)
-
-iihmpr = 0
-
-!===============================================================================
-! 10. INITIALISATION DES PARAMETRES ALE de albase.f90 et alstru.f90
-!===============================================================================
-
-! --- Methode ALE
-iale = 0
-
 ! --- Iterations d'initialisation fluide seul
 nalinf = 0
 
-! --- Type de viscosite de maillage (isotrope par defaut)
-iortvm = 0
-
 ! --- Nombre de structures internes
 !     (sera relu ou recalcule)
 nbstru = -999
@@ -1096,7 +872,7 @@ betnmk = -grand
 gamnmk = -grand
 
 !===============================================================================
-! 11. INITIALISATION DES PARAMETRES DE COUPLAGE CS/CS
+! INITIALISATION DES PARAMETRES DE COUPLAGE CS/CS
 !===============================================================================
 
 ! --- Nombre de couplage
@@ -1106,21 +882,10 @@ nbrcpl = 0
 ifaccp = 0
 
 !===============================================================================
-! 12. Lagrangian arrays
+! Lagrangian arrays
 !===============================================================================
 
 tslagr => rvoid2
 
-!===============================================================================
-! 13. VOF/Cavitation module
-!===============================================================================
-
-call init_vof
-call init_cavitation
-
-!===============================================================================
-! 14. Exit
-!===============================================================================
-
 return
 end subroutine
