@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,6 @@ subroutine initi1
 use paramx
 use optcal
 use entsor
-use ihmpre
 use ppincl, only: ippmod, nmodmx, iatmos
 use post
 use cs_c_bindings
@@ -54,7 +53,7 @@ implicit none
 
 ! Local variables
 
-integer          iok, ipp, nmodpp
+integer          iok, ipp, nmodpp, have_thermal_model
 
 double precision ttsuit, wtsuit
 
@@ -62,17 +61,24 @@ double precision ttsuit, wtsuit
 
 interface
 
-  subroutine gui_output()  &
+  subroutine cs_gui_boundary_conditions_define(bdy)  &
+    bind(C, name='cs_gui_boundary_conditions_define')
+    use, intrinsic :: iso_c_binding
+    implicit none
+    type(c_ptr), value :: bdy
+  end subroutine cs_gui_boundary_conditions_define
+
+  subroutine cs_gui_output()  &
       bind(C, name='cs_gui_output')
     use, intrinsic :: iso_c_binding
     implicit none
-  end subroutine gui_output
+  end subroutine cs_gui_output
 
-  subroutine user_output()  &
-      bind(C, name='cs_user_output')
+  subroutine user_finalize_setup_wrapper()  &
+      bind(C, name='cs_user_finalize_setup_wrapper')
     use, intrinsic :: iso_c_binding
     implicit none
-  end subroutine user_output
+  end subroutine user_finalize_setup_wrapper
 
   subroutine user_syrthes_coupling()  &
       bind(C, name='cs_user_syrthes_coupling')
@@ -83,14 +89,7 @@ interface
 end interface
 
 !===============================================================================
-! 1. Initialize modules before user.
-!      . entsor
-!      . dimens
-!      . numvar
-!      . pointe
-!      . optcal
-!      . mltgrd
-!      . cstphy
+! Initialize modules before user has access
 !===============================================================================
 
 call iniini
@@ -103,8 +102,7 @@ do ipp = 2, nmodmx
 enddo
 
 !===============================================================================
-! 2. ENTREE DES DONNEES PAR L'UTILISATEUR
-!      ET POSITIONNEMENT DES VARIABLES
+! User input, variable definitions
 !===============================================================================
 
 call iniusi
@@ -121,10 +119,20 @@ call cs_rad_transfer_options
 
 if (ippmod(iatmos).ge.0) call cs_at_data_assim_initialize
 
+! Lagrangian model options
+
+call lagran_init_map
+
+have_thermal_model = 0
+if (iscalt.ge.1) then
+  have_thermal_model = 1
+endif
+call cs_lagr_options_definition(isuite, have_thermal_model, dtref, iccvfg)
+
 ! Additional fields if not in CDO mode only
 
 if (icdo.lt.2) then
-   call addfld
+  call addfld
 endif
 
 ! Time moments
@@ -132,12 +140,6 @@ endif
 call cs_gui_time_moments
 call cs_user_time_moments
 
-! Postprocessing and logging
-
-if (iihmpr.eq.1) then
-  call gui_output
-endif
-
 ! Restart
 
 ttsuit = -1.d0
@@ -145,37 +147,46 @@ wtsuit = -1.d0
 
 call dflsui(ntsuit, ttsuit, wtsuit);
 
-! Lagrangian model options
-
-call lagran_init_map
-
-call lagopt(isuite, iccvfg, iscalt, dtref)
-
 !===============================================================================
-! 3. MODIFS APRES USINI1
+! Changes after user initialization and additional fields dependent on
+! main fields and options.
 !===============================================================================
 
 ! Do not call this routine if CDO mode only (default variables and properties
 ! are not defined anymore)
 if (icdo.lt.2) then
-   call modini
+  call modini
+  call fldini
 endif
 
 !===============================================================================
-! 4. Some additional fields and mappings
+! GUI-based boundary condition definitions
 !===============================================================================
 
+call cs_gui_boundary_conditions_define(c_null_ptr)
+
+!===============================================================================
+! Some final settings
+!===============================================================================
+
+! Postprocessing and logging
+
+call cs_gui_output
+
 ! Do not call this routine if CDO mode only (default variables and properties
 ! are not defined anymore)
 if (icdo.lt.2) then
-   call fldini
+   call usipes(nmodpp)
+   ! Avoid a second spurious call to this function
+   ! Called in the C part if CDO is activated, i.e. when
+   ! additional geometric quantities and connectivities are built
+   if (icdo.lt.0) then
+      call user_finalize_setup_wrapper
+   endif
 endif
 
-call usipes(nmodpp)
-call user_output
-
 !===============================================================================
-! 5. Coherency checks
+! Coherency checks
 !===============================================================================
 
 iok = 0
@@ -193,33 +204,6 @@ else
   write(nfecra,9998)
 endif
 
-#if defined(_CS_LANG_FR)
-
- 9998 format(                                                   /,&
-' Pas d erreur detectee lors de la verification des donnees'   ,/,&
-'               (interface, cs_user_parameters.f90 et autres).',/)
- 9999 format(                                                     &
-'@'                                                            ,/,&
-'@'                                                            ,/,&
-'@'                                                            ,/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@'                                                            ,/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES'               ,/,&
-'@    ========='                                               ,/,&
-'@    LES PARAMETRES DE CALCUL SONT INCOHERENTS OU INCOMPLETS' ,/,&
-'@'                                                            ,/,&
-'@  Le calcul ne sera pas execute (',i10,' erreurs).'          ,/,&
-'@'                                                            ,/,&
-'@  Se reporter aux impressions precedentes pour plus de'      ,/,&
-'@    renseignements.'                                         ,/,&
-'@  Verifier les donnees entrees dans l''interface'            ,/,&
-'@    et dans les sous-programmes utilisateur.'                ,/,&
-'@'                                                            ,/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@'                                                            ,/)
-
-#else
-
  9998 format(                                                   /,&
 ' No error detected during the data verification'              ,/,&
 '                          cs_user_parameters.f90 and others).',/)
@@ -242,10 +226,8 @@ endif
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@'                                                            ,/)
 
-#endif
-
 !===============================================================================
-! 7. Output
+! Output
 !===============================================================================
 
 call impini
