@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -44,10 +44,6 @@ subroutine iniusi
 !__________________!____!_____!________________________________________________!
 !__________________!____!_____!________________________________________________!
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
 !===============================================================================
 
 !===============================================================================
@@ -64,7 +60,6 @@ use entsor
 use albase
 use parall
 use period
-use ihmpre
 use ppppar
 use ppthch
 use coincl
@@ -87,9 +82,9 @@ implicit none
 ! Local variables
 
 integer          nmodpp
-integer          nscmax, nscusi
-integer          iihmpu, l_size
-double precision relaxp, extrap, l_cp(1), l_xmasm(1), l_cv(1)
+integer          nscmax
+integer          l_size, f_id
+double precision relaxp, l_cp(1), l_xmasm(1), l_cv(1)
 
 type(var_cal_opt) :: vcopt
 
@@ -97,12 +92,38 @@ type(var_cal_opt) :: vcopt
 
 interface
 
+  subroutine cs_gui_physical_properties()  &
+       bind(C, name='cs_gui_physical_properties')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_gui_physical_properties
+
+  subroutine cs_gui_porous_model()  &
+       bind(C, name='cs_gui_porous_model')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_gui_porous_model
+
   subroutine cs_gui_radiative_transfer_parameters()  &
-      bind(C, name='cs_gui_radiative_transfer_parameters')
+       bind(C, name='cs_gui_radiative_transfer_parameters')
     use, intrinsic :: iso_c_binding
     implicit none
   end subroutine cs_gui_radiative_transfer_parameters
 
+  subroutine cs_velocity_pressure_set_solid()  &
+       bind(C, name='cs_velocity_pressure_set_solid')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_velocity_pressure_set_solid
+
+  subroutine cs_runaway_check_define_field_max(f_id, value)  &
+       bind(C, name='cs_runaway_check_define_field_max')
+    use, intrinsic :: iso_c_binding
+    implicit none
+    integer(c_int), value :: f_id
+    real(c_double), value :: value
+  end subroutine cs_runaway_check_define_field_max
+
 end interface
 
 !===============================================================================
@@ -112,78 +133,57 @@ end interface
 call parameters_read_restart_info
 
 !===============================================================================
-! 0. INITIALISATION DE L'INFORMATION "FICHIER XML (IHM) REQUIS & EXISTE"
+! 1. Initialize model settings
 !===============================================================================
 
+! Flow model selection through GUI
 
-!   - Interface Code_Saturne
-!     ======================
+call cs_gui_physical_model_select
 
-!     Avec Xml, on regarde si le fichier a ete ouvert (requis et existe,
-!       selon les tests realises dans cs_main)
-!     IIHMPR a ete initialise a 0 juste avant (INIINI)
+! Flow model selection through user Fortran subroutine
 
-call csihmp(iihmpr)
+call usppmo(1)
 
-!===============================================================================
-! 1. INITIALISATION DE PARAMETRES POUR LA PHASE CONTINUE
-!===============================================================================
-
-!     Turbulence
-!     Chaleur massique variable ou non
-
-!   - Interface Code_Saturne
-!     ======================
-
-if (iihmpr.eq.1) then
-
-  call csther()
-
-  call csturb()
-
-  call cscpva()
-
-endif
+! Other models selection through GUI
 
 ! ALE parameters
-!---------------
+call uialin (nalinf, nalimx, epalim)
 
-! GUI
+! thermal model
+call csther
 
-if (iihmpr.eq.1) then
-  call uialin (iale, nalinf, nalimx, epalim, iortvm)
+! turbulence model choice
+call cs_gui_turb_model
+
+! constant or variable specific heat
+call cscpva
+
+! Other models selection through user Fortran subroutine
+
+call usipph(1, iturb, itherm, iale)
+
+! Flow and other models selection through user C function
+call cs_user_model
+
+! Activate CDO for ALE
+if (iale.eq.2) then
+  call cs_ale_activate
 endif
 
-! User sub-routines
-! =================
-
-iihmpu = iihmpr
-call usipph(iihmpu, iturb, itherm, iale, ivofmt, icavit)
-
 ! Other model parameters, including user-defined scalars
-!-------------------------------------------------------
 
-! GUI
+call cs_gui_user_variables
+call cs_gui_user_arrays
 
-if (iihmpr.eq.1) then
-  call cs_gui_user_variables
-endif
+! Solid zones
 
-! User sub-routines
-
-call cs_user_model
+call cs_velocity_pressure_set_solid
 
 !===============================================================================
 ! 2. Initialize parameters for specific physics
 !===============================================================================
 
-! GUI
-
-call cs_gui_physical_model_select(ieos, ieqco2)
-
-if (iihmpr.eq.1) then
-  call cfnmtd(ficfpp, len(ficfpp))
-endif
+call cfnmtd(ficfpp, len(ficfpp))
 
 ! --- Activation du module transferts radiatifs
 
@@ -196,27 +196,18 @@ endif
 
 call cs_gui_radiative_transfer_parameters
 
-! User subroutine
-
-! Initialize specific physics modules not available at the moment
-
-! User initialization
-
-iihmpu = iihmpr
-call usppmo(iihmpu)
-
 ! Define fields for variables, check and build iscapp
+! and computes the number of user scalars (nscaus)
 if (icdo.lt.2) then
-   call fldvar(nmodpp)
+  call fldvar(nmodpp)
 endif
 
-if (iihmpr.eq.1) then
-  call csivis
+if (iale.ge.1) then
+  call uialvm
 endif
+call csivis
 
 nscmax = nscamx
-nscusi = nscaus
-iihmpu = iihmpr
 
 ! ---> Physique particuliere : darcy
 
@@ -224,6 +215,15 @@ if (ippmod(idarcy).ge.0) then
   call daini1
 endif
 
+call field_get_id_try('velocity', f_id)
+if (f_id .ge. 0) then
+  if (ippmod(icompf).ge.0) then
+    call cs_runaway_check_define_field_max(f_id, 1.0d5)
+  else
+    call cs_runaway_check_define_field_max(f_id, 1.0d4)
+  endif
+endif
+
 !===============================================================================
 ! 3. INITIALISATION DE PARAMETRES "GLOBAUX"
 !===============================================================================
@@ -239,17 +239,13 @@ endif
 !   - Interface Code_Saturne
 !     ======================
 
-if (iihmpr.eq.1) then
+call csidtv()
 
-  call csidtv()
+call csiphy()
 
-  call csiphy()
+! Postprocessing
 
-  ! Postprocessing
-
-  call cspstb(ipstdv)
-
-endif
+call cspstb(ipstdv)
 
 ! Define main properties (pointers, checks, ipp) if not in CDO mode only
 if (icdo.lt.2) then
@@ -268,59 +264,58 @@ endif
 !   - Interface Code_Saturne
 !     ======================
 
-if (iihmpr.eq.1) then
+! Restart, read auxiliary file, frozen velocity field
 
-!     Suite de calcul, relecture fichier auxiliaire, champ de vitesse figé
+call csisui(ntsuit, iccvfg)
 
-  call csisui(ntsuit, ileaux, iccvfg)
+! Time step (only ntmabs, dtref)
+call cstime()
 
-!     Pas de temps (seulement NTMABS, DTREF, INPDT0)
-  call cstime()
+! Local numerical options
 
-!      Options numériques locales
+call uinum1(cdtvar)
 
-  call uinum1(cdtvar)
+! If CDO mode only, no pressure is defined at this stage
+if (icdo.lt.2) then
 
-  ! If CDO mode only, no pressure is defined at this stage
-  if (icdo.lt.2) then
+  call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
 
-     call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+  ! Global numeric options
+  relaxp = -1.d0
+  call csnum2 (relaxp, imrgra)
+  if (idtvar.ge.0) vcopt%relaxv = relaxp
 
-     !     Options numériques globales
-     relaxp = -999.d0
-     extrap = 0.d0
-     call csnum2 (relaxp, extrap, imrgra)
-     vcopt%extrag = extrap
-     if (idtvar.ge.0) vcopt%relaxv = relaxp
-
-     call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
-
-  endif
-
-!     Gravite, prop. phys
-  call csphys(viscv0, visls0, itempk)
-
-!     Scamin, scamax, turbulent flux model
-  call cssca2(iturt)
-
-  ! Diffusivites
-  call cssca3(visls0)
-
-  ! Init of reference values (uref, almax)
-  call cstini()
-
-  call uiipsu(iporos)
-
-  ! Init fan
-  call uifans()
-
-  ! Init error estimator
-  call uieres(iescal, iespre, iesder, iescor, iestot)
+  call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
 
 endif
 
-!   - Sous-programme utilisateur
-!     ==========================
+! Gravity, physical properties
+call cs_gui_physical_properties
+
+! Turbulence reference values (uref, almax)
+call cs_gui_turb_ref_values
+
+! Set turbulence constants according to model choices.
+! This can be overwritten by the user in cs_user_parameters()
+call cs_f_turb_complete_constants
+
+! Scamin, scamax, turbulent flux model
+call cssca2()
+
+! Diffusivities
+call cssca3()
+
+! Porosity model
+call cs_gui_porous_model()
+
+! Init fan
+call uifans()
+
+! Init error estimator
+call uieres(iescal, iespre, iesder, iescor, iestot)
+
+!   - User functions
+!     ==============
 
 call usipsu(nmodpp)
 call user_parameters
@@ -340,12 +335,12 @@ call indsui(isuite)
 if (ippmod(icompf).ge.0) then
   ! ieos has been set above in uippmo with the GUI or in usppmo without the GUI.
   ! The variability of the thermal conductivity
-  ! (scalar_diffusivity_id for itempk) and the volume viscosity (iviscv) has
+  ! (diffusivity_id for itempk) and the volume viscosity (iviscv) has
   ! been set in fldprp.
 
-  ! Here call to uscfx2 to get visls0(itempk), viscv0, xmasmr, ivivar and
+  ! Here call to uscfx2 to get visls_0(itempk), viscv0, xmasmr, ivivar and
   ! psginf, gammasg, cv0 in stiffened gas thermodynamic.
-  ! With GUI, visls0(itempk), viscv0, xmasmr and ivivar have already been read
+  ! With GUI, visls_0(itempk), viscv0, xmasmr and ivivar have already been read
   ! above in the call to csphys.
   call uscfx2
 
@@ -357,16 +352,16 @@ if (ippmod(icompf).ge.0) then
   cv0 = l_cv(1)
 endif
 
-! Choose which 3x3 cocg matrixes are computed for gradient algorithms.
-call comcoc(imrgra)
-
 ! Choose the porous model
-call compor(iporos)
+call cs_porous_model_set_model(iporos)
 
 ! --- Varpos
-call varpos
-
+! If CDO mode only, skip this stage
+if (icdo.lt.2) then
+  call varpos
+endif
 ! --- Internal coupling
+call cs_gui_internal_coupling
 call cs_user_internal_coupling
 
 call cs_internal_coupling_setup
