@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -80,13 +80,14 @@ integer          iel   , ifac
 integer          iclip , ii    , jj    , idim, f_dim
 integer          ifcvsl
 integer          iflid, nfld, ifmaip, bfmaip, iflmas, iflmab
-integer          kscmin, kscmax
+integer          kscmin
 integer          f_type, idftnp
 integer          keyvar
+integer          f_id, kdflim
 
 logical          have_previous
 
-double precision xxk, xcmu, trii
+double precision xxk, xcmu, trii, clvfmn, visls_0
 
 double precision, dimension(:), pointer :: dt
 double precision, dimension(:), pointer :: brom, crom
@@ -94,7 +95,7 @@ double precision, dimension(:), pointer :: cofbcp
 double precision, dimension(:), pointer :: porosi
 double precision, dimension(:,:), pointer :: porosf
 double precision, dimension(:), pointer :: field_s_v
-
+double precision, dimension(:), pointer :: cpro_diff_lim
 double precision, dimension(:), pointer :: cvar_pr
 double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_al
 double precision, dimension(:), pointer :: cvar_phi, cvar_fb, cvar_omg, cvar_nusa
@@ -106,6 +107,7 @@ double precision, dimension(:), pointer :: cpro_viscls, cproa_viscls, cvar_tempk
 double precision, dimension(:), pointer :: cpro_visma_s
 double precision, dimension(:), pointer :: mix_mol_mas
 double precision, dimension(:,:), pointer :: cpro_visma_v
+double precision, dimension(:,:), pointer :: xyzno0
 
 type(var_cal_opt) :: vcopt_uma
 
@@ -131,19 +133,28 @@ call field_get_key_id("variable_id", keyvar)
 
 call field_get_val_s_by_name('dt', dt)
 
+! Initialize temperature to reference temperature if present
+call field_get_id_try('temperature', f_id)
+if (f_id .ge. 0) then
+  call field_get_val_s(f_id, field_s_v)
+  do iel = 1, ncelet
+    field_s_v(iel) = t0
+  enddo
+endif
+
+! Initialize boundary temperature to "marker" if present
+call field_get_id_try('boundary_temperature', f_id)
+if (f_id .ge. 0) then
+  call field_get_val_s(f_id, field_s_v)
+  do iel = 1, nfabor
+    field_s_v(iel) = -grand
+  enddo
+endif
+
 ! Initialize variables to avoid compiler warnings
 
 jj = 0
 
-! En compressible, ISYMPA initialise (= 1) car utile dans le calcul
-!     du pas de temps variable avant passage dans les C.L.
-
-if ( ippmod(icompf).ge.0 ) then
-  do ifac = 1, nfabor
-    isympa(ifac) = 1
-  enddo
-endif
-
 !===============================================================================
 ! 2. PAS DE TEMPS
 !===============================================================================
@@ -166,7 +177,7 @@ call field_get_val_s(ibrom, brom)
 
 !     Masse volumique aux cellules (et au pdt precedent si ordre2 ou icalhy
 !     ou algo. VOF)
-do iel = 1, ncel
+do iel = 1, ncelet
   crom(iel)  = ro0
 enddo
 
@@ -182,12 +193,12 @@ call field_current_to_previous(icrom)
 call field_current_to_previous(ibrom)
 call field_current_to_previous(ibrom)
 
-! Moleacular viscosity
+! Molecular viscosity
 call field_get_val_s(iviscl, viscl)
 call field_get_val_s(ivisct, visct)
 
 ! Molecular viscosity at cells (and eventual previous value)
-do iel = 1, ncel
+do iel = 1, ncelet
   viscl(iel) = viscl0
 enddo
 call field_current_to_previous(iviscl)
@@ -195,7 +206,7 @@ call field_current_to_previous(iviscl)
 ! Specific heat at cells (and eventual previous value)
 if(icp.ge.0) then
   call field_get_val_s(icp, cpro_cp)
-  do iel = 1, ncel
+  do iel = 1, ncelet
     cpro_cp(iel) = cp0
   enddo
   call field_current_to_previous(icp)
@@ -210,7 +221,7 @@ endif
 if ((ippmod(icompf).lt.0.and.ippmod(idarcy).lt.0).or.                          &
     (ippmod(idarcy).ge.0.and.darcy_gravity.ge.1)) then
   call field_get_val_s(iprtot, cpro_prtot)
-  do iel = 1, ncel
+  do iel = 1, ncelet
     cpro_prtot(iel) = - rinfin
   enddo
 endif
@@ -219,7 +230,7 @@ endif
 ! (used in cs_cf_thermo_default_init)
 if(ippmod(igmix).ge.0) then
   call field_get_val_s(igmxml, mix_mol_mas)
-  do iel =1, ncel
+  do iel =1, ncelet
     mix_mol_mas(iel) = xmasmr
   enddo
 endif
@@ -229,7 +240,7 @@ if (ippmod(icompf).ge.0) then
   ! In compressible, for now, the temperature is not solved but is a field of
   ! type variable anyway. The reference value has to be taken into account.
   call field_get_val_s(ivarfl(isca(itempk)), cvar_tempk)
-  do iel = 1, ncel
+  do iel = 1, ncelet
     cvar_tempk(iel) = t0
   enddo
 
@@ -238,37 +249,41 @@ if (ippmod(icompf).ge.0) then
   call cs_cf_thermo_default_init
 
   ! Default diffusivity for total energy
-  visls0(ienerg) = visls0(itempk)/cv0
+
+  call field_get_key_double(ivarfl(isca(itempk)), kvisl0, visls_0)
+  visls_0 = visls_0 / cv0
+  call field_set_key_double(ivarfl(isca(ienerg)), kvisl0, visls_0)
 endif
 
 ! Diffusivite des scalaires
 do iscal = 1, nscal
   call field_get_key_int (ivarfl(isca(iscal)), kivisl, ifcvsl)
+  call field_get_key_double(ivarfl(isca(iscal)), kvisl0, visls_0)
   ! Diffusivite aux cellules (et au pdt precedent si ordre2)
   if (ifcvsl.ge.0) then
     call field_get_val_s(ifcvsl, cpro_viscls)
-    do iel = 1, ncel
-      cpro_viscls(iel) = visls0(iscal)
+    do iel = 1, ncelet
+      cpro_viscls(iel) = visls_0
     enddo
     call field_have_previous(ifcvsl, have_previous)
     if (have_previous) then
       call field_get_val_prev_s(ifcvsl, cproa_viscls)
-      do iel = 1, ncel
-        cproa_viscls(iel) = visls0(iscal)
+      do iel = 1, ncelet
+        cproa_viscls(iel) = visls_0
       enddo
     endif
   endif
 enddo
 
 ! Mesh viscosity for ALE
-if (iale.eq.1) then
+if (iale.ge.1) then
 
   call field_get_key_struct_var_cal_opt(ivarfl(iuma), vcopt_uma)
   idftnp = vcopt_uma%idften
 
   if (iand(idftnp, ANISOTROPIC_LEFT_DIFFUSION).ne.0) then
     call field_get_val_v(ivisma, cpro_visma_v)
-    do iel = 1, ncel
+    do iel = 1, ncelet
       do ii = 1, 3
         cpro_visma_v(ii  ,iel) = 1.d0
         cpro_visma_v(ii+3,iel) = 0.d0
@@ -276,13 +291,12 @@ if (iale.eq.1) then
     enddo
   else if (iand(idftnp, ISOTROPIC_DIFFUSION).ne.0) then
     call field_get_val_s(ivisma, cpro_visma_s)
-    do iel = 1, ncel
+    do iel = 1, ncelet
       cpro_visma_s(iel) = 1.d0
     enddo
   endif
 
   call cs_gui_mesh_viscosity
-  call usvima
 
 endif
 
@@ -291,8 +305,6 @@ if (iporos.ge.1) then
   call field_get_val_s(ipori, porosi)
   do iel = 1, ncelet
     porosi(iel) = 1.d0
-    ! No solid cells
-    isolid_0(iel) = 0
   enddo
 
   ! Tensorial porosity
@@ -307,9 +319,6 @@ if (iporos.ge.1) then
       porosf(6, iel) = 0.d0
     enddo
   endif
-
-else
-  isolid_0(1) = 0
 endif
 
 !===============================================================================
@@ -319,21 +328,18 @@ endif
 
 !     On met la pression P* a PRED0
 !$omp parallel do
-do iel = 1, ncel
+do iel = 1, ncelet
   cvar_pr(iel) = pred0
 enddo
 
-! On definit les clipping du taux de vide et on initialize au clipping inf.
-if (ivofmt.ge.0) then
+! initialize void fraction to minimum clipping value
+if (ivofmt.gt.0) then
 
   call field_get_key_id("min_scalar_clipping", kscmin)
-  call field_get_key_id("max_scalar_clipping", kscmax)
-
-  call field_set_key_double(ivarfl(ivolf2), kscmin, clvfmn)
-  call field_set_key_double(ivarfl(ivolf2), kscmax, clvfmx)
+  call field_get_key_double(ivarfl(ivolf2), kscmin, clvfmn)
 
   call field_get_val_s(ivarfl(ivolf2), field_s_v)
-  do iel = 1, ncel
+  do iel = 1, ncelet
     field_s_v(iel) = clvfmn
   enddo
 
@@ -358,16 +364,15 @@ if(itytur.eq.2 .or. itytur.eq.5) then
   if (iturb.eq.51) xcmu = cpalmu
 
   if (uref.ge.0.d0) then
-    do iel = 1, ncel
+    do iel = 1, ncelet
       cvar_k(iel) = 1.5d0*(0.02d0*uref)**2
       cvar_ep(iel) = cvar_k(iel)**1.5d0*xcmu/almax
     enddo
 
-    iclip = 1
-    call clipke(ncelet, ncel, iclip)
+    call clipke(ncel, 1)
 
   else
-    do iel = 1, ncel
+    do iel = 1, ncelet
       cvar_k(iel) = -grand
       cvar_ep(iel) = -grand
     enddo
@@ -376,7 +381,7 @@ if(itytur.eq.2 .or. itytur.eq.5) then
   if (iturb.eq.50) then
     call field_get_val_s(ivarfl(iphi), cvar_phi)
     call field_get_val_s(ivarfl(ifb), cvar_fb)
-    do iel = 1, ncel
+    do iel = 1, ncelet
       cvar_phi(iel) = 2.d0/3.d0
       cvar_fb(iel) = 0.d0
     enddo
@@ -384,7 +389,7 @@ if(itytur.eq.2 .or. itytur.eq.5) then
   if (iturb.eq.51) then
     call field_get_val_s(ivarfl(ial), cvar_al)
     call field_get_val_s(ivarfl(iphi), cvar_phi)
-    do iel = 1, ncel
+    do iel = 1, ncelet
       cvar_phi(iel) = 2.d0/3.d0
       cvar_al(iel) = 1.d0
     enddo
@@ -395,14 +400,13 @@ elseif(itytur.eq.3) then
   call field_get_val_s(ivarfl(iep), cvar_ep)
 
   if (irijco.eq.1) then
-      call field_get_val_v(ivarfl(irij), cvar_rij)
-
+    call field_get_val_v(ivarfl(irij), cvar_rij)
 
     if (uref.ge.0.d0) then
 
       trii   = (0.02d0*uref)**2
 
-      do iel = 1, ncel
+      do iel = 1, ncelet
         cvar_rij(1,iel) = trii
         cvar_rij(2,iel) = trii
         cvar_rij(3,iel) = trii
@@ -418,7 +422,7 @@ elseif(itytur.eq.3) then
 
     else
 
-      do iel = 1, ncel
+      do iel = 1, ncelet
         cvar_rij(1,iel) = -grand
         cvar_rij(2,iel) = -grand
         cvar_rij(3,iel) = -grand
@@ -430,18 +434,18 @@ elseif(itytur.eq.3) then
 
     endif
   else
-      call field_get_val_s(ivarfl(ir11), cvar_r11)
-      call field_get_val_s(ivarfl(ir22), cvar_r22)
-      call field_get_val_s(ivarfl(ir33), cvar_r33)
-      call field_get_val_s(ivarfl(ir12), cvar_r12)
-      call field_get_val_s(ivarfl(ir23), cvar_r23)
-      call field_get_val_s(ivarfl(ir13), cvar_r13)
+    call field_get_val_s(ivarfl(ir11), cvar_r11)
+    call field_get_val_s(ivarfl(ir22), cvar_r22)
+    call field_get_val_s(ivarfl(ir33), cvar_r33)
+    call field_get_val_s(ivarfl(ir12), cvar_r12)
+    call field_get_val_s(ivarfl(ir23), cvar_r23)
+    call field_get_val_s(ivarfl(ir13), cvar_r13)
 
     if (uref.ge.0.d0) then
 
       trii   = (0.02d0*uref)**2
 
-      do iel = 1, ncel
+      do iel = 1, ncelet
         cvar_r11(iel) = trii
         cvar_r22(iel) = trii
         cvar_r33(iel) = trii
@@ -457,7 +461,7 @@ elseif(itytur.eq.3) then
 
     else
 
-      do iel = 1, ncel
+      do iel = 1, ncelet
         cvar_r11(iel) = -grand
         cvar_r22(iel) = -grand
         cvar_r33(iel) = -grand
@@ -470,7 +474,7 @@ elseif(itytur.eq.3) then
   endif
   if(iturb.eq.32)then
     call field_get_val_s(ivarfl(ial), cvar_al)
-    do iel = 1, ncel
+    do iel = 1, ncelet
       cvar_al(iel) = 1.d0
     enddo
   endif
@@ -482,7 +486,7 @@ elseif(iturb.eq.60) then
 
   if (uref.ge.0.d0) then
 
-    do iel = 1, ncel
+    do iel = 1, ncelet
       cvar_k(iel) = 1.5d0*(0.02d0*uref)**2
       !     on utilise la formule classique eps=k**1.5/Cmu/ALMAX et omega=eps/Cmu/k
       cvar_omg(iel) = cvar_k(iel)**0.5d0/almax
@@ -491,7 +495,7 @@ elseif(iturb.eq.60) then
 
   else
 
-    do iel = 1, ncel
+    do iel = 1, ncelet
       cvar_k(iel) = -grand
       cvar_omg(iel) = -grand
     enddo
@@ -504,7 +508,7 @@ elseif(iturb.eq.70) then
 
   if (uref.ge.0.d0) then
 
-    do iel = 1, ncel
+    do iel = 1, ncelet
       cvar_nusa(iel) = sqrt(1.5d0)*(0.02d0*uref)*almax
       !     on utilise la formule classique eps=k**1.5/Cmu/ALMAX
       !     et nusa=Cmu*k**2/eps
@@ -513,7 +517,7 @@ elseif(iturb.eq.70) then
 
   else
 
-    do iel = 1, ncel
+    do iel = 1, ncelet
       cvar_nusa(iel) = -grand
     enddo
 
@@ -577,20 +581,9 @@ endif
 ! Boundary conditions
 
 do ifac = 1, nfabor
-  itypfb(ifac) = 0
   itrifb(ifac) = 0
 enddo
 
-! Type symetrie : on en a besoin dans le cas du calcul des gradients
-!     par moindres carres etendu avec extrapolation du gradient au bord
-!     La valeur 0 permet de ne pas extrapoler le gradient sur les faces.
-!     Habituellement, on evite l'extrapolation sur les faces de symetries
-!     pour ne pas tomber sur une indetermination et une matrice 3*3 non
-!     inversible dans les configurations 2D).
-do ifac = 1, nfabor
-  isympa(ifac) = 0
-enddo
-
 ! Old mass flux. We try not to do the same operation multiple times
 ! (for shared mass fluxes), without doing too complex tests.
 
@@ -600,19 +593,27 @@ ifmaip = -1
 bfmaip = -1
 
 do iflid = 0, nfld - 1
+  call field_get_type(iflid, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    ! Is this field not managed by CDO ? Not useful with CDO
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
 
-  call field_get_key_int(iflid, kimasf, iflmas) ! interior mass flux
-  call field_get_key_int(iflid, kbmasf, iflmab) ! boundary mass flux
+      call field_get_key_int(iflid, kimasf, iflmas) ! interior mass flux
+      call field_get_key_int(iflid, kbmasf, iflmab) ! boundary mass flux
 
-  if (iflmas.ge.0 .and. iflmas.ne.ifmaip) then
-    call field_current_to_previous(iflid)
-    ifmaip = iflmas
-  endif
+      if (iflmas.ge.0 .and. iflmas.ne.ifmaip) then
+        call field_current_to_previous(iflid)
+        ifmaip = iflmas
+      endif
 
-  if (iflmab.ge.0 .and. iflmab.ne.bfmaip) then
-    call field_current_to_previous(iflid)
-    bfmaip = iflmab
-  endif
+      if (iflmab.ge.0 .and. iflmab.ne.bfmaip) then
+        call field_current_to_previous(iflid)
+        bfmaip = iflmab
+      endif
+
+    endif ! CDO ?
+  endif ! VARIABLE ?
 
 enddo
 
@@ -620,13 +621,14 @@ enddo
 ! 8.  INITIALISATIONS EN ALE
 !===============================================================================
 
-if (iale.eq.1) then
+if (iale.ge.1) then
   do ii = 1, nnod
     impale(ii) = 0
   enddo
 endif
 
-if (iale.eq.1) then
+if (iale.ge.1) then
+  call field_get_val_v_by_name("vtx_coord0", xyzno0)
   do ii = 1, nnod
     do idim = 1, 3
       xyzno0(idim,ii) = xyznod(idim,ii)
@@ -642,8 +644,42 @@ do iflid = 0, nfld - 1
   call field_get_type(iflid, f_type)
   ! Is the field of type FIELD_VARIABLE?
   if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
-    call field_current_to_previous(iflid)
-  endif
+    ! Is this field not managed by CDO ? Perfomed elsewhere with CDO
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
+
+      call field_current_to_previous(iflid)
+
+    endif ! CDO ?
+  endif ! VARIABLE ?
+enddo
+
+
+! Diffusion limiter initialization
+call field_get_key_id("diffusion_limiter_id", kdflim)
+
+do f_id = 0, nfld - 1
+
+  call field_get_type(f_id, f_type)
+
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    ! Is this field not managed by CDO ?
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
+
+      call field_get_key_int(f_id, kdflim, iflid)
+
+      if (iflid.ne.-1) then
+
+        call field_get_val_s(iflid, cpro_diff_lim)
+
+        do iel = 1, ncelet
+          cpro_diff_lim(iel) = 1.d0
+        enddo
+
+      endif
+
+    endif ! CDO ?
+  endif ! VARIABLE ?
 enddo
 
 !----
