@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -34,13 +34,10 @@
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[out]    frcxt         external forces making hydrostatic pressure
-!> \param[out]    prhyd         predicted hydrostatic pressure
 !_______________________________________________________________________________
 
 
-subroutine lecamo &
- ( frcxt  , prhyd  )
+subroutine lecamo
 
 !===============================================================================
 
@@ -65,8 +62,6 @@ implicit none
 
 ! Arguments
 
-double precision frcxt(3,ncelet), prhyd(ncelet)
-
 ! Local variables
 
 type(c_ptr)      oflmap
@@ -91,7 +86,7 @@ call lecamp(oflmap)
 !===============================================================================
 
 if (ileaux.eq.1) then
-  call lecamx(oflmap, frcxt, prhyd)
+  call lecamx(oflmap)
 endif
 
 !===============================================================================
@@ -106,19 +101,6 @@ write(nfecra,2000)
 ! 5. FORMATS
 !===============================================================================
 
-
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,                                                   &
-' ----------------------------------------------------------- ',/,&
-                                                                /,&
-     3X,'** LECTURE DES FICHIERS SUITE PRINCIPAL ET AUXILIAIRE',/,&
-     3X,'   ------------------------------------------------- ',/)
- 2000 format(/,                                                   &
-' ----------------------------------------------------------- ',/)
-
-#else
-
  1000 format(/,                                                   &
 ' ----------------------------------------------------------- ',/,&
                                                                 /,&
@@ -127,8 +109,5 @@ write(nfecra,2000)
  2000 format(/,                                                   &
 ' ----------------------------------------------------------- ',/)
 
-#endif
-
-
 return
 end subroutine
