@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -63,7 +63,7 @@ use cplsat
 use field
 use atincl, only: init_at_chem
 use atchem, only: ichemistry
-use siream, only: iaerosol
+use sshaerosol, only: iaerosol, CS_ATMO_AEROSOL_OFF
 use turbomachinery
 use cs_c_bindings
 
@@ -85,13 +85,14 @@ integer          ivar  , ivers(1), f_id
 integer          ierror, itysup, nbval
 integer          nberro, t_id
 integer          nfmtru
-integer          jale, jcavit
+integer          jale, jvolfl
 integer          ival(1)
 double precision rval(1)
 
 logical(kind=c_bool) :: ncelok, nfaiok, nfabok, nsomok
 
 type(c_ptr) :: rp
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
@@ -120,7 +121,7 @@ cindfp='YY'
 ! 1. OUVERTURE DU FICHIER OU STOP
 !===============================================================================
 
-ficsui = 'main'
+ficsui = 'main.csc'
 call restart_create(ficsui, '', 0, rp)
 
 ! ---> Debut de la lecture
@@ -142,8 +143,11 @@ call restart_read_int_t_compat(rp,                                      &
                                itysup, nbval, ivers, ierror)
 
 if (ierror.ne.0) then
-  write(nfecra,9200) ficsui
-  call csexit (1)
+  ierror = cs_restart_check_if_restart_from_ncfd(rp)
+  if (ierror.eq.0) then
+    write(nfecra,9200) ficsui
+    call csexit (1)
+  endif
 endif
 
 !  --->  Tests sur les supports
@@ -180,6 +184,13 @@ itysup = 0
 nbval  = 1
 call restart_read_section_int_t(rp,rubriq,itysup,nbval,ival,ierror)
 ntpabs = ival(1) ! no direct read to avoid pointer issue
+
+! If section doesnt exist, check if it is a restart from neptune:
+if (ierror.ne.0) then
+  rubriq = 'ntcabs'
+  call restart_read_section_int_t(rp,rubriq,itysup,nbval,ival,ierror)
+  ntpabs = ival(1) ! no direct read to avoid pointer issue
+endif
 nberro=nberro+ierror
 
 rubriq = 'instant_precedent'
@@ -187,6 +198,14 @@ itysup = 0
 nbval  = 1
 call restart_read_section_real_t(rp,rubriq,itysup,nbval,rval,ierror)
 ttpabs = rval(1) ! no direct read to avoid pointer issue
+
+! If section doesnt exist, check if it is a restart from neptune:
+if (ierror.ne.0) then
+  rubriq = 'ttcabs'
+  call restart_read_section_real_t(rp,rubriq,itysup,nbval,rval,ierror)
+  ttpabs = rval(1) ! no direct read to avoid pointer issue
+endif
+
 nberro=nberro+ierror
 
 ! --->  Stop si erreur
@@ -207,33 +226,38 @@ jale = ival(1)
 nberro=nberro+ierror
 
 ! --->  Message si erreur (pas de stop pour compatibilite avec les fichiers anterieurs)
-!       -> on n'affiche le message que si IALE=1 (sinon RAS)
+!       -> on n'affiche le message que si IALE>=1 (sinon RAS)
 if (nberro.ne.0) then
-  if (iale.eq.1) write(nfecra,9401)
+  if (iale.ge.1) write(nfecra,9401)
   jale = 0
 endif
 
-!     Cavitation
+!     VOF
 
 nberro = 0
 
-rubriq = 'cavitation'
+rubriq = 'vof'
 itysup = 0
 nbval  = 1
 call restart_read_section_int_t(rp,rubriq,itysup,nbval,ival,ierror)
-jcavit = ival(1)
+jvolfl = ival(1)
 nberro=nberro+ierror
 
-! Message si erreur (pas de stop pour compatibilite avec fichiers anterieurs)
-! -> on n'affiche le message que si ICAVIT>=0 (sinon RAS)
+! warning if error found (backward compatibility)
+! and only if VoF model is enabled
 if (nberro.ne.0) then
-  if (icavit.ge.0) write(nfecra,9401)
-  jcavit = -1
+  if (ivofmt.gt.0) write(nfecra,9405)
+  jvolfl = -1
 endif
 
 ! --->  Stop si pas de temps incoherent
-if (ntpabs.ge.ntmabs.and.inpdt0.eq.0) then
-  write(nfecra,9410)ntpabs,ntmabs
+if (ttmabs.ge.0) then
+  if (ttpabs.gt.ttmabs) then
+    write(nfecra,9411) ttpabs,ttmabs
+    call csexit (1)
+  endif
+else if (ntpabs.gt.ntmabs) then
+  write(nfecra,9410) ntpabs,ntmabs
   call csexit (1)
 endif
 
@@ -243,7 +267,7 @@ write(nfecra,2411) ttpabs
 
 ! --->  Si le calcul precedent etait en ALE, on DOIT relire les
 !         coordonnees des noeuds dans le fichier auxiliaire
-if (iale.eq.1 .and. jale.eq.1) then
+if (iale.ge.1 .and. jale.ge.1) then
   if (ileaux.ne.1) then
     write(nfecra,9402)jale,iale,ileaux
     call csexit(1)
@@ -262,18 +286,21 @@ ttpmob = rval(1) ! no direct read to avoid pointer issue
 nberro=nberro+ierror
 
 ! Message si erreur (pas de stop pour compatibilite avec fichiers anterieurs)
-! -> on n'affiche le message que si imobil=1 ou iturbo=2 (sinon RAS)
+! -> on n'affiche le message que si iturbo=2 (sinon RAS)
 if (nberro.ne.0) then
-  if (imobil.eq.1 .or. iturbo.eq.2) write(nfecra,9403) ttpabs
+  if (iturbo.eq.2) write(nfecra,9403) ttpabs
   ttpmob = ttpabs
 endif
 
-! Information (uniquement si imobil=1 ou iturbo=2
-!              et pas d affichage precedent)
-if (imobil.eq.1 .or. iturbo.eq.2) then
+! Information (uniquement si iturbo=2 et pas d affichage precedent)
+if (iturbo.eq.2) then
   if (nberro.eq.0)  write(nfecra,2412) ttpmob
 endif
 
+if (iturbo.ne.0) then
+  call turbomachinery_restart_read(rp)
+endif
+
 ! Fin de la lecture des options
 write(nfecra,1499)
 
@@ -285,27 +312,35 @@ call restart_read_variables(rp, oflmap, 0)
 
 f_id = -1
 do ivar = 1, nvar
-  if (ibdtso(ivar).gt.1) then
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+  if (vcopt%ibdtso.gt.1) then
     if (f_id.ne.ivarfl(ivar)) then
       ierror = 0
       f_id = ivarfl(ivar)
-      do t_id = 1, ibdtso(ivar) - 1
+      do t_id = 1, vcopt%ibdtso - 1
         call restart_read_field_vals(rp, f_id, t_id, ierror)
         ierror = ierror + 1
       enddo
-      if (ierror.gt.1) ibdtso(ivar) = -ibdtso(ivar)
+      if (ierror.gt.1) then
+        vcopt%ibdtso = -vcopt%ibdtso
+        call field_set_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+      endif
     endif
   endif
 enddo
 
+
+call restart_read_fields(rp, RESTART_MAIN)
+
 !===============================================================================
 ! 6. LECTURE D'INFORMATIONS COMPLEMENTAIRES LEGERES
 !===============================================================================
 
-if (ichemistry.gt.0.or.iaerosol.gt.0) then
+if (ichemistry.gt.0.or.iaerosol.ne.CS_ATMO_AEROSOL_OFF) then
   rubriq = 'atmospheric_chem'
   itysup = 0
   nbval  = 1
+  ival(1) = 1
   call restart_read_section_int_t(rp,rubriq,itysup,nbval,ival,ierror)
   init_at_chem = ival(1)
   if (ierror.eq.0.and.init_at_chem.gt.0) then
@@ -333,40 +368,14 @@ return
 
 ! --- ETAPES
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/, 3x,'   LECTURE DU FICHIER SUITE PRINCIPAL',/)
- 1100 format(' Debut de la lecture')
- 1299 format(' Fin de la lecture des dimensions')
- 1499 format(' Fin de la lecture des options')
- 1799 format(' Fin de la lecture')
-
-#else
-
  1000 format(/, 3x,'   READING THE MAIN RESTART FILE',/)
  1100 format(' Start reading'  )
  1299 format(' Reading dimensions complete'  )
  1499 format(' Reading options complete')
  1799 format(' Reading complete')
 
-#endif
-
 ! --- INFORMATIONS
 
-#if defined(_CS_LANG_FR)
-
- 2410 format                                                            &
- ('  Lecture du pas de temps precedent (suite) ',                &
-                                                  'NTPABS = ',I10)
- 2411 format                                                            &
- ('  Lecture du pas de temps precedent (suite) ',                &
-                                                'TTPABS = ',E12.4)
- 2412 format                                                            &
- ('  Lecture du temps de maillage mobile precedent (suite) ',    &
-                                                'TTPMOB = ',E12.4)
-
-#else
-
  2410 format                                                            &
  ('  Reading the previous time step number ',                    &
                       '(restarting computation)  NTPABS =   ',I10)
@@ -377,149 +386,8 @@ return
  ('  Reading the previous moving mesh moment ',                  &
                       '(restarting computation)  TTPMOB = ',E12.4)
 
-#endif
-
 ! --- ERREURS
 
-#if defined(_CS_LANG_FR)
-
- 9200 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                      PRINCIPAL',/,&
-'@      TYPE DE FICHIER INCORRECT                             ',/,&
-'@                                                            ',/,&
-'@    Le fichier ',A13      ,' ne semble pas etre un fichier  ',/,&
-'@      suite principal.                                      ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne peut etre execute.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier que le fichier suite utilise correspond bien   ',/,&
-'@        a un fichier suite principal.                       ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9201 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                      PRINCIPAL',/,&
-'@      DONNEES AMONT ET ACTUELLES INCOHERENTES               ',/,&
-'@                                                            ',/,&
-'@    Le nombre de cellules a ete modifie                     ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne peut etre execute.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier que le fichier suite utilise correspond bien   ',/,&
-'@        au cas traite.                                      ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9400 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                      PRINCIPAL',/,&
-'@                                                            ',/,&
-'@      ERREUR A LA LECTURE DES INFORMATIONS TEMPORELLES      ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne peut pas etre execute.                     ',/,&
-'@                                                            ',/,&
-'@    Verifier que le fichier suite utilise n''a pas ete      ',/,&
-'@        endommage.                                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9401 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ERREUR A LA LECTURE DU FICHIER SUITE        ',/,&
-'@    =========                                      PRINCIPAL',/,&
-'@                                                            ',/,&
-'@      ERREUR A LA LECTURE DE L''INDICATEUR DE METHODE ALE   ',/,&
-'@                                                            ',/,&
-'@    Il se peut que le fichier suite relu corresponde a une  ',/,&
-'@      version anterieure de Code_Saturne, sans methode ALE. ',/,&
-'@    Le calcul sera execute en reinitialisant toutes les     ',/,&
-'@      donnees ALE.                                          ',/,&
-'@    Verifier neanmoins que le fichier suite utilise n''a    ',/,&
-'@        pas ete endommage.                                  ',/,&
-'@                                                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9402 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                      PRINCIPAL',/,&
-'@      INDICATEUR IALE DU CALCUL PRECEDENT = ',I10            ,/,&
-'@      INDICATEUR IALE DU CALCUL ACTUEL    = ',I10            ,/,&
-'@                                                            ',/,&
-'@    Les coordonnees des noeuds du maillage doivent etre     ',/,&
-'@      relues. Elles sont stockees dans le fichier suite     ',/,&
-'@      auxiliaire.                                           ',/,&
-'@    L''indicateur ILEAUX doit donc etre positionne a 1.     ',/,&
-'@    Il vaut ici ILEAUX = ',I10                               ,/,&
-'@                                                            ',/,&
-'@    Le calcul ne peut pas etre execute.                     ',/,&
-'@                                                            ',/,&
-'@                                                            ',/,&
-'@    Verifier ILEAUX.                                        ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9403 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ERREUR A LA LECTURE DU FICHIER SUITE        ',/,&
-'@    =========                                      PRINCIPAL',/,&
-'@                                                            ',/,&
-'@      ERREUR A LA LECTURE DE L INSTANT DE MAILLAGE MOBILE  ',/,&
-'@                                                   PRECEDENT',/,&
-'@    Il se peut que le fichier suite relu corresponde a une  ',/,&
-'@      version anterieure de Code_Saturne, sans couplage     ',/,&
-'@      rotor/stator instationnaire.                          ',/,&
-'@    Le calcul sera execute en initialisant l instant de     ',/,&
-'@      maillage mobile precedent a TTCMOB = ',E12.4           ,/,&
-'@    Verifier neanmoins que le fichier suite utilise n''a    ',/,&
-'@        pas ete endommage.                                  ',/,&
-'@                                                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9410 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                      PRINCIPAL',/,&
-'@      NUMERO DU PAS DE TEMPS PRECEDENT NTPABS = ',I10        ,/,&
-'@      NUMERO DU PAS DE TEMPS VISE      NTMABS = ',I10        ,/,&
-'@                                                            ',/,&
-'@    Le nombre de pas de temps (absolu) vise, NTMABS,        ',/,&
-'@      doit etre superieur au nombre de pas de temps         ',/,&
-'@      (absolu) deja effectues, NTPABS.                      ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne peut etre execute.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier (augmenter) NTMABS.                            ',/,&
-'@    Verifier que le fichier suite utilise correspond bien   ',/,&
-'@        au cas traite.                                      ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-! NFMTRU = 36 pour A36
-
-#else
-
  9200 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -631,7 +499,26 @@ return
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
- 9410 format(                                                     &
+ 9405 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@ WARNING : ERROR AT THE MAIN RESTART FILE READING        ',/,&
+'@    =========                                               ',/,&
+'@                                                            ',/,&
+'@      ERROR AT READING THE INDICATOR OF THE VOLUME OF FLUID ',/,&
+'@                                                      METHOD',/,&
+'@    The read restart file might come from a previous        ',/,&
+'@      version of Code Saturne, without VOF.                 ',/,&
+'@    The calculation will be executed but                    ',/,&
+'@      Volume of Fluid method data will be reset.            ',/,&
+'@    Please check the integrity of the file used as          ',/,&
+'@        restart file, however.                              ',/,&
+'@                                                            ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+9410 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
@@ -641,8 +528,29 @@ return
 '@      NUMBER OF TIME STEPS WANTED       NTMABS = ',I10       ,/,&
 '@                                                            ',/,&
 '@    The number of time steps (absolute) wanted, NTMABS,     ',/,&
-'@      has to be larger than the number of time steps        ',/,&
-'@      (absolute) already done, NTPABS.                      ',/,&
+'@      has to be greater or equal to than the number of      ',/,&
+'@      time steps (absolute) already run, NTPABS.            ',/,&
+'@                                                            ',/,&
+'@    The calculation cannot be executed.                     ',/,&
+'@                                                            ',/,&
+'@    Please check (increase) NTMABS.                         ',/,&
+'@    Please make sure the file used as restart file does     ',/,&
+'@          correspond to your case                           ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+9411 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@ WARNING : STOP AT THE MAIN RESTART FILE READING         ',/,&
+'@    =========                                               ',/,&
+'@      PREVIOUS TIME TTPABS = ',E12.4                         ,/,&
+'@      TIME WANTED   TTMABS = ',E12.4                         ,/,&
+'@                                                            ',/,&
+'@    The number of time steps (absolute) wanted, NTMABS,     ',/,&
+'@      has to be greater or equal to than the number of      ',/,&
+'@      time steps (absolute) already run, NTPABS.            ',/,&
 '@                                                            ',/,&
 '@    The calculation cannot be executed.                     ',/,&
 '@                                                            ',/,&
@@ -653,6 +561,4 @@ return
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
 end subroutine
