@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -106,35 +106,33 @@ integer          iel   , ifac, ii, istr, nlfld, iscal
 integer          iz, kk
 integer          idecal, iclapc, icha  , icla
 integer          jdtvar
-integer          jortvm
 integer          ierror, itysup, nbval
 integer          nberro, inierr, ivers(1)
 integer          ilu   , ierrch
 integer          nfmtsc, nfmtfl, nfmtch, nfmtcl
 integer          nfmtst
-integer          jale, jcavit, jvolfl
+integer          jale, jvolfl
 integer          f_id, iflmas, iflmab, iflvoi, iflvob
+integer          key_t_ext_id, icpext
+integer          iviext
 integer          ival(1), ngbstr(2)
 double precision rval(1), tmpstr(27)
 
-character(len=80) :: fname
-
 logical(kind=c_bool) :: ncelok, nfaiok, nfabok, nsomok
 
 type(c_ptr) :: rp
 
 double precision, dimension(:), pointer :: sval
-double precision, dimension(:), pointer :: voidfl, cpro_visma_s
-double precision, dimension(:,:), pointer :: disale, cpro_visma_v
+double precision, dimension(:), pointer :: voidfl
+double precision, dimension(:,:), pointer :: disale
 
 double precision, allocatable, dimension(:,:) :: tmurbf
 double precision, allocatable, dimension(:) :: tparbf
-double precision, allocatable, dimension(:) :: vismbf
 
 !===============================================================================
 
 !===============================================================================
-! 0. INITIALISATION
+! 0. Initialisation
 !===============================================================================
 
 !  ---> Banniere
@@ -156,11 +154,14 @@ cindfm='YYYY'
 cindfc='YY'
 cindfl='YYYY'
 
+! Time extrapolation?
+call field_get_key_id("time_extrapolated", key_t_ext_id)
+
 !===============================================================================
 ! 1. OUVERTURE DU FICHIER SUITE AUXILIAIRE
 !===============================================================================
 
-ficsui = 'auxiliary'
+ficsui = 'auxiliary.csc'
 call restart_create(ficsui, '', 0, rp)
 
 ! ---> Debut de la lecture
@@ -213,42 +214,23 @@ jale = ival(1)
 nberro=nberro+ierror
 
 ! --->  Message si erreur (pas de stop pour compatibilite avec les fichiers anterieurs)
-!       -> on n'affiche le message que si IALE=1 (sinon RAS)
+!       -> on n'affiche le message que si IALE>=1 (sinon RAS)
 if (nberro.ne.0) then
-  if (iale.eq.1) write(nfecra,9210)
+  if (iale.ge.1) write(nfecra,9210)
   jale = 0
 endif
 
 ! ---> Pas d'iteration d'initialisation si suite de calcul ALE
 if (italin.eq.-999) then
-  if (iale.eq.1 .and. jale.eq.1) then
+  if (iale.ge.1 .and. jale.ge.1) then
     italin = 0
-  else if (iale.eq.1) then
+  else if (iale.ge.1) then
     italin = 1
   else
     italin = 0
   endif
 endif
 
-!     Cavitation
-
-nberro = 0
-
-rubriq = 'cavitation'
-itysup = 0
-nbval  = 1
-call restart_read_section_int_t(rp,rubriq,itysup,nbval,ival,ierror)
-jcavit = ival(1)
-nberro=nberro+ierror
-
-! --->  Message si erreur (pas de stop pour compatibilite avec les fichiers
-!                          anterieurs)
-!       -> on n'affiche le message que si ICAVIT>=0 (sinon RAS)
-if (nberro.ne.0) then
-  if (icavit.ge.0) write(nfecra,9220)
-  jcavit = -1
-endif
-
 !     VOF
 
 nberro = 0
@@ -260,15 +242,14 @@ call restart_read_section_int_t(rp,rubriq,itysup,nbval,ival,ierror)
 jvolfl = ival(1)
 nberro=nberro+ierror
 
-! --->  Message si erreur (pas de stop pour compatibilite avec les fichiers
-!                          anterieurs)
-!       -> on n'affiche le message que si IVOFMT=1 (sinon RAS)
+! warning if error found (backward compatibility)
+! and only if VoF model is enabled
 if (nberro.ne.0) then
-  if (ivofmt.ge.0) write(nfecra,9221)
+  if (ivofmt.gt.0) write(nfecra,9221)
   jvolfl = 0
 endif
 
-car54 =' Fin de la lecture des options                        '
+car54 =' Finished reading options                             '
 write(nfecra,1110)car54
 
 !===============================================================================
@@ -282,7 +263,7 @@ nberro = 0
 ! ---> Point de reference de pression
 !     On lit les coordonnees si l'utilisateur n'a rien specifie, i.e.
 !       si IXYZP0=-1, et on met IXYZP0 a 1 pour eviter de le changer ensuite.
-if (ixyzp0.eq.1) then
+if (ixyzp0.eq.-1) then
   rubriq = 'ref_presstot01'
   itysup = 0
   nbval  = 3
@@ -327,7 +308,7 @@ endif
 inierr = 0
 
 if (    irovar.eq.1                   &
-    .or.(ivofmt.ge.0.and.jvolfl.ge.0)) then
+    .or.(ivofmt.gt.0.and.jvolfl.ge.0)) then
 
   ! Masse volumique - cellules
   call restart_read_field_vals(rp, icrom, 0, ierror)
@@ -335,7 +316,7 @@ if (    irovar.eq.1                   &
   inierr = inierr+ierror
 
   ! Masse volumique du pdt precedent - cellules
-  if (ivofmt.ge.0.and.jvolfl.ge.0.or.idilat.ge.4) then
+  if (ivofmt.gt.0.and.jvolfl.ge.0.or.idilat.ge.4) then
     call restart_read_field_vals(rp, icrom, 1, ierror)
     nberro = nberro+ierror
     inierr = inierr+ierror
@@ -373,21 +354,24 @@ endif
 !     La viscosite moleculaire est egalement lue pour le modele de cavitation
 !     Si on reussit, on l'indique
 
-if (    iviext.gt.0                   &
-    .or.(ivofmt.ge.0.and.jvolfl.ge.0)) then
+call field_get_key_int(iviscl, key_t_ext_id, iviext)
+if (iviext.gt.0.or.(ivofmt.gt.0.and.jvolfl.ge.0)) then
 
   inierr = 0
 
   !         Viscosite moleculaire - cellules
   !         Uniquement si elle est variable ou pour la methode VOF
   if (    ivivar.eq.1                   &
-      .or.(ivofmt.ge.0.and.jvolfl.ge.0)) then
+      .or.(ivofmt.gt.0.and.jvolfl.ge.0)) then
     call restart_read_field_vals(rp, iviscl, 0, ierror)
     nberro = nberro+ierror
     inierr = inierr+ierror
   endif
+endif
 
-  !         Viscosite turbulente ou de sous-maille - cellules
+call field_get_key_int(ivisct, key_t_ext_id, iviext)
+if (iviext.gt.0.or.(ivofmt.gt.0.and.jvolfl.ge.0)) then
+  ! Viscosite turbulente ou de sous-maille - cellules
   if (iviext.gt.0) then
     call restart_read_field_vals(rp, ivisct, 0, ierror)
     nberro = nberro+ierror
@@ -410,21 +394,25 @@ endif
 !        et quand l'utilisateur peut s'en servir pour passer
 !        de H a T, comme en effet Joule par exemple).
 
-if ((icpext.gt.0.and.icp.ge.0).or.                &
-     (ippmod(ieljou).ge.1.and.icp.ge.0)) then
+if (icp.ge.0) then
 
-  inierr = 0
+  call field_get_key_int(icp, key_t_ext_id, icpext)
 
-  ! Chaleur massique - cellules
-  call restart_read_field_vals(rp, icp, 0, ierror)
-  nberro = nberro+ierror
-  inierr = inierr+ierror
+  if (icpext.gt.0.or.ippmod(ieljou).ge.1) then
+
+    inierr = 0
+
+    ! Chaleur massique - cellules
+    call restart_read_field_vals(rp, icp, 0, ierror)
+    nberro = nberro+ierror
+    inierr = inierr+ierror
+
+    ! Si on a initialise Cp, on l'indique (pour schtmp)
+    if (inierr.eq.0) then
+      initcp = 1
+    endif
 
-  ! Si on a initialise Cp, on l'indique (pour schtmp)
-  if (inierr.eq.0) then
-    initcp = 1
   endif
-
 endif
 
 !     Si on a des scalaires, on lit a diffusivite
@@ -433,7 +421,7 @@ endif
 !       (et qu'elle est variable, et que le scalaire n'est pas une variance)
 
 nlfld = 0
-call restart_read_linked_fields(rp, oflmap, "scalar_diffusivity_id", nlfld)
+call restart_read_linked_fields(rp, oflmap, "diffusivity_id", nlfld)
 
 !     Si erreur, on previent mais pas stop :
 !       auparavant on n'avait pas stocke les prop phy
@@ -534,7 +522,7 @@ if (nfaiok.eqv..true. .or. nfabok.eqv..true.) then
   nberro=0
 
   ! Initialization of the void fraction convective flux, if required
-  if (ivofmt.ge.0.and.jvolfl.lt.0) then
+  if (ivofmt.gt.0.and.jvolfl.lt.0) then
 
     ! Interior faces
 
@@ -617,22 +605,13 @@ if (nfabok.eqv..true.) then
 
   call restart_read_bc_coeffs(rp)
 
-  ! Symmetry type (used for least squares gradients on extended
-  ! neighborhood, with extrapolation of gradient at boundary).
-
-  rubriq = 'isympa_fb_phase01'
-  itysup = 3
-  nbval  = 1
-  call restart_read_section_int_t(rp,rubriq,itysup,nbval,isympa,ierror)
-  nberro = nberro+ierror
-
 endif
 !     fin du test "si les supports des faces de bord sont identiques"
 
 if (ilu.eq.1) then
 
 !     Si erreur, on previent mais pas stop :
-!       (on n'a pas forcement les coefs 2, on n'a pas forcement isympa
+!       (on n'a pas forcement les coefs 2
 !        si on prend les fichiers d'une version anterieure)
   if (nberro.ne.0) then
     car54 = 'Lecture des conditions aux limites                    '
@@ -690,7 +669,7 @@ nberro = 0
 !   Si on arrive a la lire, on note qu'elle est a jour (sauf si ALE).
 
 if (ineedy.eq.1) then
-  if (icdpar.gt.0 .or. inpdt0.eq.1) then
+  if (icdpar.gt.0) then
     if (nfabok.eqv..true.) then
       call field_get_id('wall_distance', f_id)
       call restart_read_field_vals(rp, f_id, 0, ierror)
@@ -728,20 +707,6 @@ if (iphydr.eq.1) then
 
 endif
 
-!===============================================================================
-! 11. PRESSION HYDROSTATIQUE PREDITE
-!===============================================================================
-
-if (iphydr.eq.2) then
-
-  itysup = 1
-  nbval  = 1
-
-  call field_get_id('hydrostatic_pressure_prd', f_id)
-  call restart_read_field_vals(rp, f_id, 0, ierror)
-
-endif
-
 !===============================================================================
 ! 12.  Wall temperature associated to the condensation model
 !      with or wihtout the 1D thermal model tag1D
@@ -763,7 +728,7 @@ if (icondb.eq.0) then
     nberro=nberro+ierror
 
     do ii = 1, nfbpcd
-      ifac= ifbpcd(ii)
+      ifac = ifbpcd(ii)
       iz  = izzftcd(ii)
       do kk = 1, znmur(iz)
         ztmur(ii,kk) =  tmurbf(ifac,kk)
@@ -802,17 +767,28 @@ endif
 ! 13.  DEPLACEMENT AUX NOEUDS EN ALE
 !===============================================================================
 
-if (iale.eq.1 .and. jale.eq.1) then
+if (iale.ge.1 .and. jale.ge.1) then
   nberro = 0
 
   itysup = 4
 
   call field_get_val_v(fdiale, disale)
 
-  call restart_read_real_3_t_compat                       &
-         (rp, 'vertex_displacement',                      &
-         'deplact_x_no', 'deplact_y_no', 'deplact_z_no',  &
-         itysup, disale, ierror)
+  call restart_read_field_vals(rp, fdiale, 0, ierror)
+  if (ierror .ne. 0) then
+    call restart_read_real_3_t_compat                        &
+           (rp, 'vertex_displacement',                       &
+            'deplact_x_no', 'deplact_y_no', 'deplact_z_no',  &
+            itysup, disale, ierror)
+  endif
+
+  if (ierror.eq.0) then
+    call restart_read_field_vals(rp, fdiale, 1, ierror)
+    if (ierror.ne.0) then
+      call field_current_to_previous(fdiale)
+      ierror = 0
+    endif
+  endif
 
   nberro=nberro+ierror
 
@@ -823,59 +799,6 @@ if (iale.eq.1 .and. jale.eq.1) then
     call csexit(1)
   endif
 
-  nberro = 0
-  rubriq = 'type_visc_mail'
-  itysup = 0
-  nbval  = 1
-  call restart_read_section_int_t(rp,rubriq,itysup,nbval,ival,ierror)
-  jortvm = ival(1)
-
-  if (iortvm.eq.1) then
-
-    if (jortvm.ne.1) then
-      call field_get_name(ivisma, fname)
-      rubriq = trim(fname)//'::vals::0'
-      itysup = 1
-      nbval  = 1
-      allocate(vismbf(ncelet))
-      call restart_read_section_real_t(rp,rubriq,itysup,nbval,   &
-                                       vismbf,ierror)
-
-      call field_get_val_v(ivisma, cpro_visma_v)
-      do iel = 1, ncel
-        cpro_visma_v(1,iel) = vismbf(iel)
-        cpro_visma_v(2,iel) = vismbf(iel)
-        cpro_visma_v(3,iel) = vismbf(iel)
-      enddo
-      deallocate(vismbf)
-    else
-      call restart_read_field_vals(rp, ivisma, 0, ierror)
-    endif
-
-  else
-
-    if (jortvm.eq.1) then
-      call field_get_name(ivisma, fname)
-      rubriq = trim(fname)//'::vals::0'
-      itysup = 1
-      nbval  = 3
-      allocate(vismbf(ncelet*3))
-      call restart_read_section_real_t(rp,rubriq,itysup,nbval,   &
-                                       vismbf,ierror)
-
-      call field_get_val_s(ivisma, cpro_visma_s)
-      do iel = 1, ncel
-        cpro_visma_s(iel) = (  vismbf((iel-1)*3 + 1) &
-                             + vismbf((iel-1)*3 + 2) &
-                             + vismbf((iel-1)*3 + 3)) / 3.
-      enddo
-      deallocate(vismbf)
-    else
-      call restart_read_field_vals(rp, ivisma, 0, ierror)
-    endif
-
-  endif
-
   car54 =' Fin de la lecture des donnees ALE                    '
   write(nfecra,1110)car54
 
@@ -1548,170 +1471,21 @@ return
 
 ! --- ETAPES
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,                                                   &
-     3X,'   LECTURE DU FICHIER SUITE AUXILIAIRE               ',/)
- 1100 format(' Debut de la lecture                                    ')
- 1110 format('  ',A54                                                  )
- 1200 format(' Fin de la lecture                                      ')
-
-#else
-
  1000 format(/,                                                   &
      3X,'      READING THE AUXILIARY RESTART FILE             ',/)
  1100 format(' Start reading                                          ')
  1110 format('  ',A54                                                  )
  1200 format(' End reading                                            ')
 
-#endif
-
 ! --- INFORMATIONS
 
-#if defined(_CS_LANG_FR)
-
- 7000 format(/,                                                   &
-'   Mise a jour du point de reference pour la pression totale ',/,&
-'     par relecture du fichier suite                          ',/,&
-'    XYZP0 = ',       E14.5,        E14.5,        E14.5        ,/)
-
-#else
-
  7000 format(/,                                                   &
 '   Apdatation of the reference point for the total pressure  ',/,&
 '       by reading the restart file                           ',/,&
 '    XYZP0 = ',       E14.5,        E14.5,        E14.5        ,/)
 
-#endif
-
 ! --- MISES EN GARDE
 
-#if defined(_CS_LANG_FR)
-
- 8004 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION :       A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@                                                            ',/,&
-'@      Le nombre de structures maximal NSTRMX supporte par le',/,&
-'@        format d''ecriture du fichier suite est             ',/,&
-'@        NFMTST = ',I10                                       ,/,&
-'@      On a ici un nombre de structures maximal superieur    ',/,&
-'@        NSTRMX = ',I10                                       ,/,&
-'@      Si le nombre de structures effectif est superieur,    ',/,&
-'@        elles ne seront pas relues.                         ',/,&
-'@                                                            ',/,&
-'@    Le calcul sera execute.                                 ',/,&
-'@                                                            ',/,&
-'@    Voir le sous-programme lecamx.                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8200 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : LECTURE DU FICHIER SUITE AUXILIAIRE         ',/,&
-'@    =========                                               ',/,&
-'@      DONNEES AMONT ET ACTUELLES DIFFERENTES                ',/,&
-'@                                                            ',/,&
-'@    Le nombre de faces ',A8  ,' a ete modifie.              ',/,&
-'@                                                            ',/,&
-'@    Le calcul peut etre execute mais les donnees            ',/,&
-'@      sur les faces ',A8  ,' ne seront pas relues           ',/,&
-'@      dans le fichier suite.                                ',/,&
-'@    Elles seront initialisees par des valeurs par defaut.   ',/,&
-'@                                                            ',/,&
-'@    Cette situation peut se produire lorsque le fichier     ',/,&
-'@      suite est issu d''un calcul realise avec des options  ',/,&
-'@      de recollement differentes ou lorsque l''on modifie   ',/,&
-'@      la prise en compte de periodicite.                    ',/,&
-'@    Cette situation peut egalement se produire lorsque l''on',/,&
-'@      realise une suite sur une machine de calcul differente',/,&
-'@      et que le jeu de la precision machine modifie le      ',/,&
-'@      nombre de faces issues des recollements.              ',/,&
-'@                                                            ',/,&
-'@    Cette situation peut enfin se produire lorsque le       ',/,&
-'@      fichier suite auxiliaire ne correspond pas au cas     ',/,&
-'@      traite.                                               ',/,&
-'@                                                            ',/,&
-'@    Verifier que le fichier suite auxiliaire utilise        ',/,&
-'@      correspond bien au cas traite                         ',/,&
-'@                                                            ',/,&
-'@    Le calcul se poursuit...                                ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8300 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : LECTURE DU FICHIER SUITE AUXILIAIRE         ',/,&
-'@    =========                                               ',/,&
-'@      ', A54                                                 ,/,&
-'@                                                            ',/,&
-'@    Certaines grandeurs n''ont pas pu etre lues dans le     ',/,&
-'@      fichier suite auxiliaire.                             ',/,&
-'@    Elles seront initialisees par des valeurs par defaut.   ',/,&
-'@                                                            ',/,&
-'@    Cette situation peut se produire lorsque le fichier     ',/,&
-'@      suite est issu d''un calcul realise avec des options  ',/,&
-'@      differentes ou lorsqu''il a ete endommage.            ',/,&
-'@                                                            ',/,&
-'@    Le calcul se poursuit...                                ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8400 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : LECTURE DU FICHIER SUITE AUXILIAIRE         ',/,&
-'@    =========                                               ',/,&
-'@      REPRISE  DE CALCUL           AVEC IDTVAR = ',I10       ,/,&
-'@      A PARTIR D''UN CALCUL REALISE AVEC IDTVAR = ',I10      ,/,&
-'@                                                            ',/,&
-'@    Le mode de marche en temps a ete modifie.               ',/,&
-'@    La valeur (uniforme) du pas de temps est                ',/,&
-'@      DTREF = ',E12.4   ,' fournie.                         ',/,&
-'@                                                            ',/,&
-'@    Il est conseille cependant de                           ',/,&
-'@      verifier la valeur de IDTVAR.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier que le fichier suite auxiliaire utilise        ',/,&
-'@      correspond bien au cas traite                         ',/,&
-'@                                                            ',/,&
-'@    Le calcul se poursuit...                                ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8611 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : LECTURE DU FICHIER SUITE AUXILIAIRE         ',/,&
-'@    =========                                               ',/,&
-'@                                                            ',/,&
-'@      Modele de combustion charbon pulverise.               ',/,&
-'@      On ne trouve pas la masse volumique des charbons dans ',/,&
-'@        le fichier suite. C''est naturel si le calcul       ',/,&
-'@        precedent n''etait pas un calcul charbon pulverise. ',/,&
-'@        La valeur par defaut est utilisee comme valeur      ',/,&
-'@        initiale :                                          ',/,&
-'@         Charbon        rho                                 '  )
- 8612 format(                                                     &
-'@        ',I10   ,'  ',E14.5                                    )
- 8613 format(                                                     &
-'@                                                            ',/,&
-'@    Le calcul peut etre execute.                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
  8004 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -1834,213 +1608,8 @@ return
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
-
 ! --- ERREURS
 
-#if defined(_CS_LANG_FR)
-
- 9100 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@      TYPE DE FICHIER INCORRECT                             ',/,&
-'@                                                            ',/,&
-'@    Le fichier ',A13      ,' ne semble pas etre un fichier  ',/,&
-'@      suite auxiliaire.                                     ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne peut etre execute.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier que le fichier suite utilise correspond bien   ',/,&
-'@        a un fichier suite auxiliaire.                      ',/,&
-'@    Si necessaire, il est possible de desactiver la lecture ',/,&
-'@      du fichier suite auxiliaire par ILEAUX = 0.           ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9101 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@      DONNEES AMONT ET ACTUELLES INCOHERENTES               ',/,&
-'@                                                            ',/,&
-'@    Le nombre de cellules a ete modifie                     ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne peut etre execute.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier que le fichier suite utilise correspond bien   ',/,&
-'@        au cas traite.                                      ',/,&
-'@    Si necessaire, il est possible de desactiver la lecture ',/,&
-'@      du fichier suite auxiliaire par ILEAUX = 0.           ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9200 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION: ARRET A LA LECTURE DU FICHIER SUITE          ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@                                                            ',/,&
-'@      ', A54                                                 ,/,&
-'@                                                            ',/,&
-'@    Le calcul ne peut pas etre execute.                     ',/,&
-'@                                                            ',/,&
-'@    Verifier que le fichier suite utilise n''a pas ete      ',/,&
-'@      endommage.                                            ',/,&
-'@    Si necessaire, il est possible de desactiver la lecture ',/,&
-'@      du fichier suite auxiliaire par ILEAUX = 0.           ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9210 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ERREUR A LA LECTURE DU FICHIER SUITE        ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@                                                            ',/,&
-'@      ERREUR A LA LECTURE DE L''INDICATEUR DE METHODE ALE   ',/,&
-'@                                                            ',/,&
-'@    Il se peut que le fichier suite relu corresponde a une  ',/,&
-'@      version anterieure de Code_Saturne, sans methode ALE. ',/,&
-'@    Le calcul sera execute en reinitialisant toutes les     ',/,&
-'@      donnees ALE.                                          ',/,&
-'@    Verifier neanmoins que le fichier suite utilise n''a    ',/,&
-'@        pas ete endommage.                                  ',/,&
-'@                                                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9220 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ERREUR A LA LECTURE DU FICHIER SUITE        ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@                                                            ',/,&
-'@      ERREUR A LA LECTURE DE L''INDICATEUR DE MODELE DE     ',/,&
-'@                                                  CAVITATION',/,&
-'@                                                            ',/,&
-'@    Il se peut que le fichier suite relu corresponde a une  ',/,&
-'@      version anterieure de Code_Saturne, sans modele de    ',/,&
-'@                                                 cavitation.',/,&
-'@    Le calcul sera execute en reinitialisant toutes les     ',/,&
-'@      donnees du modele de cavitation.                      ',/,&
-'@    Verifier neanmoins que le fichier suite utilise n''a    ',/,&
-'@        pas ete endommage.                                  ',/,&
-'@                                                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9221 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ERREUR A LA LECTURE DU FICHIER SUITE        ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@                                                            ',/,&
-'@      ERREUR A LA LECTURE DE L''INDICATEUR DE MODELE DE     ',/,&
-'@                                                  VOF       ',/,&
-'@                                                            ',/,&
-'@    Il se peut que le fichier suite relu corresponde a une  ',/,&
-'@      version anterieure de Code_Saturne, sans modele de    ',/,&
-'@                                                 VOF.       ',/,&
-'@    Le calcul sera execute en reinitialisant toutes les     ',/,&
-'@      donnees du modele de VOF       .                      ',/,&
-'@    Verifier neanmoins que le fichier suite utilise n''a    ',/,&
-'@        pas ete endommage.                                  ',/,&
-'@                                                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9320 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@                                                            ',/,&
-'@      ERREUR LORS DE LA LECTURE DU DEPLACEMENT AUX NOEUDS   ',/,&
-'@        DU MAILLAGE (METHODE ALE)                           ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne peut pas etre execute.                     ',/,&
-'@                                                            ',/,&
-'@    Verifier que le fichier suite utilise n''a pas ete      ',/,&
-'@        endommage.                                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9321 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@                                                            ',/,&
-'@      ERREUR LORS DE LA LECTURE DES DONNEES DES STRUCTURES  ',/,&
-'@        MOBILES (METHODE ALE)                               ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne peut pas etre execute.                     ',/,&
-'@                                                            ',/,&
-'@    Verifier que le fichier suite utilise n''a pas ete      ',/,&
-'@        endommage.                                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9400 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@                                                            ',/,&
-'@    La relecture des variable CO3DP                         ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne sera pas  execute.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier le fichier suite auxiliaire                    ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9500 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@                                                            ',/,&
-'@    La relecture des variable EBU                           ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne sera pas  execute.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier le fichier suite auxiliaire                    ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9600 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA LECTURE DU FICHIER SUITE         ',/,&
-'@    =========                                     AUXILIAIRE',/,&
-'@                                                            ',/,&
-'@    La relecture des variable LWC                           ',/,&
-'@                                                            ',/,&
-'@    Le calcul ne sera pas  execute.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier le fichier suite auxiliaire                    ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
  9100 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -2115,25 +1684,6 @@ return
 '@    Verify that the restart file used has not been damaged. ',/,&
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9220 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ WARNING: ERROR WHILE READING THE AUXILIARY RESTART FILE ',/,&
-'@    =======                                                 ',/,&
-'@                                                            ',/,&
-'@      ERROR WHEN READING THE INDICATOR OF THE CAVITATION    ',/,&
-'@                                                      MODEL ',/,&
-'@                                                            ',/,&
-'@    It is possible that the file read corresponds to an old ',/,&
-'@      version of Code_Saturne, without the cavitation model.',/,&
-'@    The run will be executed with reinitialising all        ',/,&
-'@      cavitation model data.                                ',/,&
-'@                                                            ',/,&
-'@    Verify that the restart file used has not been damaged. ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
  9221 format(                                                     &
 '@                                                            ',/,&
@@ -2231,6 +1781,4 @@ return
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
 end subroutine
