@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -69,12 +69,13 @@ use pointe
 use albase
 use parall
 use period
-use ihmpre
 use ppppar
 use ppthch
 use ppincl
+use lagran
 use mesh
 use field
+use field_operator
 use cavitation
 use vof
 use darcy_module
@@ -92,18 +93,28 @@ double precision dt(ncelet)
 ! Local variables
 
 character(len=80) :: chaine
-integer          ivar  , iel   , ifac  , iscal
-integer          ii    , iok   , iok1  , iok2  , iisct, idfm, iggafm, iebdfm
-integer          nn    , isou
-integer          mbrom , ifcvsl
+integer          ivar, iel, ifac, iscal, f_id
+integer          ii, jj, iok, iok1, iok2, iisct, idfm, iggafm, iebdfm
+integer          nn, isou
+integer          mbrom, ifcvsl, iscacp
 integer          iclipc, idftnp
+integer          iprev , inc, iccocg
+integer          kturt, turb_flux_model, turb_flux_model_type
+
 double precision xk, xe, xnu, xrom, vismax(nscamx), vismin(nscamx)
+double precision xrij(3,3), xnal(3), xnoral
+double precision xfmu, xmu, xmut
 double precision nusa, xi3, fv1, cv13
-double precision varmn(4), varmx(4), tt, ttmin, ttke, viscto
+double precision varmn(4), varmx(4), tt, ttmin, ttke, viscto, visls_0
 double precision xttkmg, xttdrb
 double precision trrij,rottke
+double precision alpha3, xrnn
+double precision s, s11, s22, s33, delta, c_k, c_epsilon
+double precision dudy, dudz, dvdx, dvdz, dwdx, dwdy
+double precision, dimension(:), pointer :: field_s_v, field_s_b
 double precision, dimension(:), pointer :: brom, crom
 double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_phi, cvar_nusa
+double precision, dimension(:), pointer :: cvar_al
 double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
 double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
 double precision, dimension(:,:), pointer :: cvar_rij
@@ -112,6 +123,9 @@ double precision, dimension(:,:), pointer :: visten, vistes, cpro_visma_v
 double precision, dimension(:), pointer :: viscl, visct, cpro_vis
 double precision, dimension(:), pointer :: cvar_voidf
 double precision, dimension(:), pointer :: cpro_var, cpro_beta, cpro_visma_s
+double precision, allocatable, dimension(:) :: ttmp
+double precision, allocatable, dimension(:,:) :: grad
+double precision, dimension(:,:,:), allocatable :: gradv
 
 integer          ipass
 data             ipass /0/
@@ -120,6 +134,18 @@ save             ipass
 type(var_cal_opt) :: vcopt
 
 !===============================================================================
+! Interfaces
+!===============================================================================
+
+interface
+
+  subroutine cs_ht_convert_h_to_t_cells_solid() &
+    bind(C, name='cs_ht_convert_h_to_t_cells_solid')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_ht_convert_h_to_t_cells_solid
+
+end interface
 
 !===============================================================================
 ! 1. Initializations
@@ -133,7 +159,7 @@ ipass = ipass + 1
 
 if (iperot.gt.0 .and. itytur.eq.3) then
   call field_get_key_struct_var_cal_opt(ivarfl(ir11), vcopt)
-  call perinr(imrgra, vcopt%iwarni, vcopt%epsrgr, vcopt%extrag)
+  call perinr(vcopt%imrgra, vcopt%iwarni, vcopt%epsrgr)
 endif
 
 !===============================================================================
@@ -153,25 +179,35 @@ endif
 ! - Interface Code_Saturne
 !   ======================
 
-if (iihmpr.eq.1) then
-  call uiphyv(iviscv, itempk, visls0, viscv0)
+call uiphyv(iviscv)
 
-  if (ippmod(idarcy).ge.0) then
-    call uidapp                                                           &
-    ( darcy_anisotropic_permeability,                                     &
-      darcy_anisotropic_dispersion,                                       &
-      darcy_gravity,                                                      &
-      darcy_gravity_x, darcy_gravity_y, darcy_gravity_z,                  &
-      darcy_unsaturated)
-  endif
+if (ippmod(idarcy).ge.0) then
+  call uidapp                                                           &
+  ( darcy_anisotropic_permeability,                                     &
+    darcy_anisotropic_dispersion,                                       &
+    darcy_gravity,                                                      &
+    darcy_gravity_x, darcy_gravity_y, darcy_gravity_z,                  &
+    darcy_unsaturated)
 endif
 
+call usphyv(nvar, nscal, mbrom, dt)
 
-call usphyv &
-( nvar   , nscal  ,                                              &
-  mbrom  ,                                                       &
-  dt     )
+! C version
 
+if (mbrom.eq.0 .and. nfabor.gt.0) then
+  call field_get_val_s(ibrom, brom)
+  brom(1) = -grand
+endif
+
+if (itherm.eq.2) then
+  call cs_ht_convert_h_to_t_cells_solid
+endif
+
+call user_physical_properties()
+
+if (mbrom.eq.0 .and. nfabor.gt.0) then
+  if (brom(1) .gt. -grand) mbrom = 1
+endif
 
 ! Finalization of physical properties for specific physics
 ! AFTER the user
@@ -183,7 +219,6 @@ endif
 
 if (mbrom.eq.0) then
   call field_get_val_s(icrom, crom)
-  call field_get_val_s(ibrom, brom)
   do ifac = 1, nfabor
     iel = ifabor(ifac)
     brom(ifac) = crom(iel)
@@ -205,11 +240,11 @@ if (iterns.ge.1) return
 
 !  Au premier pas de temps du calcul
 !     Si on a indique que rho (visc) etait constant
-!       et qu'on l'a modifie dans usphyv, ca ne va pas
+!       et qu'on l'a modifie dans cs_user_physical_properties, ca ne va pas
 !     On se sert de irovar (ivivar) pour ecrire et lire
 !       rho (visc) dans le fichier suite
 
-if (ntcabs.eq.ntpabs+1 .and. ivofmt.lt.0) then
+if (ntcabs.eq.ntpabs+1) then
 
   ! Masse volumique aux cellules et aux faces de bord
   iok1 = 0
@@ -283,15 +318,45 @@ elseif (itytur.eq.2) then
 ! =============
 
   call field_get_val_s(ivisct, visct)
+  call field_get_val_s(iviscl, viscl)
   call field_get_val_s(icrom, crom)
   call field_get_val_s(ivarfl(ik), cvar_k)
   call field_get_val_s(ivarfl(iep), cvar_ep)
 
-  do iel = 1, ncel
-    xk = cvar_k(iel)
-    xe = cvar_ep(iel)
-    visct(iel) = crom(iel)*cmu*xk**2/xe
-  enddo
+  if (iturb.eq.22) then
+
+    ! 4.3.1 Launder-Sharma
+    ! --------------------
+
+    do iel = 1, ncel
+      xk   = cvar_k(iel)
+      xe   = cvar_ep(iel)
+      xrom = crom(iel)
+      xmu  = viscl(iel)
+      xmut = xrom*xk**2/xe
+      xfmu = exp(-3.4d0/(1.d0+xmut/xmu/50.d0)**2.d0)
+      visct(iel) = cmu*xfmu*xmut
+    enddo
+
+  else if (iturb.eq.23) then
+
+    ! 4.3.2 Non-linear quadratic Baglietto
+    ! ------------------------------------
+
+    call visqke
+
+  else
+
+    ! 4.3.3 Standard and Linear Production
+    ! ------------------------------------
+
+    do iel = 1, ncel
+      xk = cvar_k(iel)
+      xe = cvar_ep(iel)
+      visct(iel) = crom(iel)*cmu*xk**2/xe
+    enddo
+
+  endif
 
 elseif (itytur.eq.3) then
 
@@ -301,51 +366,124 @@ elseif (itytur.eq.3) then
   call field_get_val_s(ivisct, visct)
   call field_get_val_s(icrom, crom)
   call field_get_val_s(ivarfl(iep), cvar_ep)
-
   if (irijco.eq.1) then
     call field_get_val_v(ivarfl(irij), cvar_rij)
-
-    do iel = 1, ncel
-      xk = 0.5d0*(cvar_rij(1,iel)+cvar_rij(2,iel)+cvar_rij(3,iel))
-      xe = cvar_ep(iel)
-      visct(iel) = crom(iel)*cmu*xk**2/xe
-    enddo
   else
     call field_get_val_s(ivarfl(ir11), cvar_r11)
     call field_get_val_s(ivarfl(ir22), cvar_r22)
     call field_get_val_s(ivarfl(ir33), cvar_r33)
-
-    do iel = 1, ncel
-      xk = 0.5d0*(cvar_r11(iel)+cvar_r22(iel)+cvar_r33(iel))
-      xe = cvar_ep(iel)
-      visct(iel) = crom(iel)*cmu*xk**2/xe
-    enddo
+    call field_get_val_s(ivarfl(ir12), cvar_r12)
+    call field_get_val_s(ivarfl(ir23), cvar_r23)
+    call field_get_val_s(ivarfl(ir13), cvar_r13)
   endif
+
+  ! In case we are in EB-RSM, we compute the normals
+  if (iturb.eq.32) then
+    call field_get_val_s(ivarfl(ial), cvar_al)
+
+    allocate(grad(3,ncelet))
+
+    ! Compute the gradient of Alpha
+    iprev  = 1
+    inc    = 1
+    iccocg = 1
+
+    call field_gradient_scalar(ivarfl(ial), iprev, 0, inc, iccocg, grad)
+  endif
+
+  do iel = 1, ncel
+
+    if (irijco.eq.1) then
+      xrij(1,1) = cvar_rij(1,iel)
+      xrij(2,2) = cvar_rij(2,iel)
+      xrij(3,3) = cvar_rij(3,iel)
+      xrij(1,2) = cvar_rij(4,iel)
+      xrij(2,3) = cvar_rij(5,iel)
+      xrij(1,3) = cvar_rij(6,iel)
+      xrij(2,1) = xrij(1,2)
+      xrij(3,1) = xrij(1,3)
+      xrij(3,2) = xrij(2,3)
+    else
+      xrij(1,1) = cvar_r11(iel)
+      xrij(2,2) = cvar_r22(iel)
+      xrij(3,3) = cvar_r33(iel)
+      xrij(1,2) = cvar_r12(iel)
+      xrij(1,3) = cvar_r13(iel)
+      xrij(2,3) = cvar_r23(iel)
+      xrij(2,1) = xrij(1,2)
+      xrij(3,1) = xrij(1,3)
+      xrij(3,2) = xrij(2,3)
+    endif
+
+    alpha3 = 1.d0
+    xrnn = 0.d0
+
+    if (iturb.eq.32) then
+      ! Compute the magnitude of the Alpha gradient
+      xnoral = ( grad(1,iel)*grad(1,iel)          &
+             +   grad(2,iel)*grad(2,iel)          &
+             +   grad(3,iel)*grad(3,iel) )
+      xnoral = sqrt(xnoral)
+     ! Compute the unitary vector of Alpha
+      if (xnoral.le.epzero) then
+        xnal(1) = 0.d0
+        xnal(2) = 0.d0
+        xnal(3) = 0.d0
+      else
+        xnal(1) = grad(1,iel)/xnoral
+        xnal(2) = grad(2,iel)/xnoral
+        xnal(3) = grad(3,iel)/xnoral
+      endif
+
+      alpha3 = cvar_al(iel)**3
+
+      ! We compute the normal Reynolds Stresses
+      do ii = 1, 3
+        do jj = 1, 3
+          xrnn = xrnn + xrij(ii,jj)*xnal(ii)*xnal(jj)
+        enddo
+      enddo
+    endif
+
+    xk   = 0.5d0*(xrij(1,1)+xrij(2,2)+xrij(3,3))
+    xrnn = (1.d0-alpha3)*xrnn + alpha3*xk
+    xe = cvar_ep(iel)
+    visct(iel) = crom(iel)*cmu*xrnn*xk/xe
+  enddo
+
+  if (iturb.eq.32) then
+    deallocate(grad)
+  end if
+
 elseif (iturb.eq.40) then
 
 ! 4.5 LES Smagorinsky
 ! ===================
 
-  call vissma
+  allocate(gradv(3, 3, ncelet))
+  call vissma (gradv)
 
 elseif (iturb.eq.41) then
 
 ! 4.6 LES dynamic
 ! ===============
 
+  allocate(gradv(3, 3, ncelet))
+
   call visdyn &
  ( nvar   , nscal  ,                                              &
    ncepdc , ncetsm ,                                              &
    icepdc , icetsm , itypsm ,                                     &
    dt     ,                                                       &
-   ckupdc , smacel )
+   ckupdc , smacel, gradv )
 
 elseif (iturb.eq.42) then
 
 ! 4.7 LES WALE
 ! ============
 
-  call viswal
+  allocate(gradv(3, 3, ncelet))
+  call viswal (gradv)
 
 elseif (itytur.eq.5) then
 
@@ -414,12 +552,17 @@ idfm = 0
 iggafm = 0
 iebdfm = 0
 
+call field_get_key_id('turbulent_flux_model', kturt)
+
 do iscal = 1, nscal
-  if (ityturt(iscal).eq.3) idfm = 1
-  if (iturt(iscal).eq.31) iebdfm = 1
+  call field_get_key_int(ivarfl(isca(iscal)), kturt, turb_flux_model)
+  turb_flux_model_type = turb_flux_model / 10
+
+  if (turb_flux_model_type.eq.3) idfm = 1
+  if (turb_flux_model.eq.31) iebdfm = 1
   ! GGDH or AFM on current scalar
   ! and if DFM, GGDH on the scalar variance
-  if (ityturt(iscal).gt.0) iggafm = 1
+  if (turb_flux_model_type.gt.0) iggafm = 1
 enddo
 
 if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
@@ -443,7 +586,7 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
           ! Durbin scale
           xttkmg = xct*sqrt(viscl(iel)/crom(iel)/cvar_ep(iel))
           xttdrb = max(ttke,xttkmg)
-          rottke  = csrij * crom(iel) * xttdrb
+          rottke  = csrij * crom(iel) * xttdrb * cell_is_active(iel)
 
           do isou = 1, 6
             visten(isou, iel) = rottke*cvar_rij(isou, iel)
@@ -456,7 +599,7 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
 
           do iel = 1, ncel
             trrij = 0.5d0*(cvar_rij(1,iel)+cvar_rij(2,iel)+cvar_rij(3,iel))
-            rottke  = csrij * crom(iel) * trrij / cvar_ep(iel)
+            rottke  = csrij * crom(iel) * trrij / cvar_ep(iel) * cell_is_active(iel)
 
             do isou = 1, 6
               vistes(isou, iel) = rottke*cvar_rij(isou, iel)
@@ -469,7 +612,7 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
 
           do iel = 1, ncel
             trrij = 0.5d0*(cvar_rij(1,iel)+cvar_rij(2,iel)+cvar_rij(3,iel))
-            rottke  = csrij * crom(iel) * trrij / cvar_ep(iel)
+            rottke  = csrij * crom(iel) * trrij / cvar_ep(iel) * cell_is_active(iel)
 
             do isou = 1, 6
               vistes(isou, iel) = rottke*cvar_rij(isou, iel)
@@ -481,7 +624,7 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
       else
         do iel = 1, ncel
           trrij = 0.5d0*(cvar_rij(1,iel)+cvar_rij(2,iel)+cvar_rij(3,iel))
-          rottke  = csrij * crom(iel) * trrij / cvar_ep(iel)
+          rottke  = csrij * crom(iel) * trrij / cvar_ep(iel) * cell_is_active(iel)
 
           do isou = 1, 6
             visten(isou, iel) = rottke*cvar_rij(isou, iel)
@@ -511,7 +654,7 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
           ! Durbin scale
           xttkmg = xct*sqrt(viscl(iel)/crom(iel)/cvar_ep(iel))
           xttdrb = max(ttke,xttkmg)
-          rottke  = csrij * crom(iel) * xttdrb
+          rottke  = csrij * crom(iel) * xttdrb * cell_is_active(iel)
 
           visten(1,iel) = rottke*cvar_r11(iel)
           visten(2,iel) = rottke*cvar_r22(iel)
@@ -532,7 +675,7 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
             xttkmg = xct*sqrt(viscl(iel)/crom(iel)/cvar_ep(iel))
             xttdrb = max(ttke,xttkmg)
             !FIXME xttdrbt = xttdrb*sqrt((1.d0-alpha3)*PR/XRH + alpha3)
-            rottke  = csrij * crom(iel) * xttdrb
+            rottke  = csrij * crom(iel) * xttdrb * cell_is_active(iel)
 
             vistes(1,iel) = rottke*cvar_r11(iel)
             vistes(2,iel) = rottke*cvar_r22(iel)
@@ -548,7 +691,7 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
 
           do iel = 1, ncel
             trrij = 0.5d0*(cvar_r11(iel)+cvar_r22(iel)+cvar_r33(iel))
-            rottke  = csrij * crom(iel) * trrij / cvar_ep(iel)
+            rottke  = csrij * crom(iel) * trrij / cvar_ep(iel) * cell_is_active(iel)
 
             vistes(1,iel) = rottke*cvar_r11(iel)
             vistes(2,iel) = rottke*cvar_r22(iel)
@@ -564,7 +707,7 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
       else
         do iel = 1, ncel
           trrij = 0.5d0*(cvar_r11(iel)+cvar_r22(iel)+cvar_r33(iel))
-          rottke  = csrij * crom(iel) * trrij / cvar_ep(iel)
+          rottke  = csrij * crom(iel) * trrij / cvar_ep(iel) * cell_is_active(iel)
 
           visten(1,iel) = rottke*cvar_r11(iel)
           visten(2,iel) = rottke*cvar_r22(iel)
@@ -593,7 +736,7 @@ endif
 ! 6. Eddy viscosity correction for cavitating flows
 !===============================================================================
 
-if (icavit.ge.0 .and. icvevm.eq.1) then
+if (iand(ivofmt,VOF_MERKLE_MASS_TRANSFER).ne.0.and.icvevm.eq.1) then
   if (itytur.eq.2 .or. itytur.eq.5 .or. iturb.eq.60 .or. iturb.eq.70) then
 
     call field_get_val_s(icrom, crom)
@@ -606,7 +749,48 @@ if (icavit.ge.0 .and. icvevm.eq.1) then
 endif
 
 !===============================================================================
-! 7. User modification of the turbulent viscosity and symmetric tensor
+! 7. Compute subgrid turbulence values for LES if required
+!===============================================================================
+
+if (itytur.eq.4 .and. iilagr.gt.0) then
+
+  call field_get_val_s_by_name("k", cvar_k)
+  call field_get_val_s_by_name("epsilon", cvar_ep)
+
+  c_epsilon = 1.0
+
+  do iel = 1, ncel
+
+    s11  = gradv(1, 1, iel)
+    s22  = gradv(2, 2, iel)
+    s33  = gradv(3, 3, iel)
+
+    dudy = gradv(2, 1, iel)
+    dvdx = gradv(1, 2, iel)
+    dudz = gradv(3, 1, iel)
+    dwdx = gradv(1, 3, iel)
+    dvdz = gradv(3, 2, iel)
+    dwdy = gradv(2, 3, iel)
+
+    s = s11**2 + s22**2 + s33**2         &
+        + 0.5d0 * ((dudy + dvdx)**2      &
+        +          (dudz + dwdx)**2      &
+        +          (dvdz + dwdy)**2)
+    s = sqrt(2.0d0 * s)
+
+    delta = xlesfl * (ales*volume(iel))**bles
+
+    cvar_ep(iel) = (csmago * delta)**2 * s**3
+    cvar_k(iel) =  c_epsilon * (delta * cvar_ep(iel))**(2.0d0/3.0d0)
+
+  enddo
+
+endif
+
+if (allocated(gradv)) deallocate (gradv)
+
+!===============================================================================
+! 8. User modification of the turbulent viscosity and symmetric tensor
 !    diffusivity
 !===============================================================================
 
@@ -618,7 +802,7 @@ call usvist &
   ckupdc , smacel )
 
 !===============================================================================
-! 8. Clipping of the turbulent viscosity in dynamic LES
+! 9. Clipping of the turbulent viscosity in dynamic LES
 !===============================================================================
 
 ! Pour la LES en modele dynamique on clippe la viscosite turbulente de maniere
@@ -648,7 +832,8 @@ if (iturb.eq.41) then
 endif
 
 !===============================================================================
-! 9. Checking of the user values
+! 10. Checking of the user values and put turbulent viscosity to 0 in
+!     disabled cells
 !===============================================================================
 
 ! ---> Calcul des bornes des variables et impressions
@@ -678,7 +863,13 @@ do ii = 1, nn
   else
     ivar = 1
     if (ii.eq.2) call field_get_val_s(iviscl, cpro_var)
-    if (ii.eq.3) call field_get_val_s(ivisct, cpro_var)
+    if (ii.eq.3) then
+      call field_get_val_s(ivisct, cpro_var)
+      ! Set turbulent viscosity to 0 in disabled cells
+      do iel = 1, ncel
+        cpro_var(iel) = cpro_var(iel) * cell_is_active(iel)
+      enddo
+    endif
     if (ii.eq.4) call field_get_val_s(icp, cpro_var)
     varmx(ii) = cpro_var(1)
     varmn(ii) = cpro_var(1)
@@ -761,7 +952,8 @@ if (icp.ge.0) then
     iisct = 0
     if (itherm.ne.0) iisct = 1
     do iscal = 1, nscal
-      if (iscacp(iscal).ne.0) then
+      call field_get_key_int(ivarfl(isca(iscal)), kscacp, iscacp)
+      if (iscacp.ne.0) then
         iisct = 1
       endif
     enddo
@@ -792,12 +984,13 @@ if (nscal.ge.1) then
         vismin(iscal) = min(vismin(iscal),cpro_vis(iel))
       enddo
       if (irangp.ge.0) then
-        call parmax (vismax(iscal))
-        call parmin (vismin(iscal))
+        call parmax(vismax(iscal))
+        call parmin(vismin(iscal))
       endif
     else
-      vismax(iscal) = visls0(iscal)
-      vismin(iscal) = visls0(iscal)
+      call field_get_key_double(ivarfl(isca(iscal)), kvisl0, visls_0)
+      vismax(iscal) = visls_0
+      vismin(iscal) = visls_0
     endif
 
     ivar = isca(iscal)
@@ -842,7 +1035,7 @@ endif
 
 ! ---> Calcul des bornes de viscosite de maillage en ALE
 
-if (iale.eq.1 .and. ntcabs.eq.ntpabs+1) then
+if (iale.ge.1 .and. ntcabs.eq.ntpabs+1) then
 
   call field_get_key_struct_var_cal_opt(ivarfl(iuma), vcopt)
   idftnp = vcopt%idften
@@ -929,212 +1122,51 @@ if (iok.ne.0) then
   call csexit (1)
 endif
 
+! Initialize boundary temperature if present and not initialized yet
+!===================================================================
+
+call field_get_id_try('boundary_temperature', f_id)
+if (f_id .ge. 0) then
+  call field_get_val_s(f_id, field_s_b)
+  call field_get_id_try('temperature', f_id)
+  if (f_id .ge. 0) then
+    call field_get_val_s(f_id, field_s_v)
+    do ifac = 1, nfabor
+      if (field_s_b(ifac) .le. -grand) then
+        iel = ifabor(ifac)
+        field_s_b(ifac) = field_s_v(iel)
+      endif
+    enddo
+  else if (itherm.eq.2) then
+    call field_get_id_try('enthalpy', f_id)
+    if (f_id .ge. 0) then
+      call field_get_val_s(f_id, field_s_v)
+      allocate(ttmp(ncelet))
+      call cs_ht_convert_h_to_t_cells(field_s_v, ttmp);
+      do ifac = 1, nfabor
+        if (field_s_b(ifac) .le. -grand) then
+          iel = ifabor(ifac)
+          field_s_b(ifac) = ttmp(iel)
+        endif
+      enddo
+      deallocate(ttmp)
+    endif
+  endif
+  ! Last resort
+  if (f_id .ge. 0) then
+    do ifac = 1, nfabor
+      if (field_s_b(ifac) .le. -grand) then
+        iel = ifabor(ifac)
+        field_s_b(ifac) = t0
+      endif
+    enddo
+  endif
+endif
+
 !--------
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(                                                     &
-' Nb de clippings de la viscosite totale (mu+mu_t>0) :',i10,/)
- 3010 format(                                                     &
-' -----------------------------------------',                   /,&
-' Propriete          Valeur min  Valeur max',                   /,&
-' -----------------------------------------                   '  )
- 3011 format(                                                     &
- 2x,    a16,      e12.4,      e12.4                              )
- 3012 format(                                                     &
-' -----------------------------------------',                   /)
- 3110 format(                                                     &
-' --- Diffusivite :',                                           /,&
-' -----------------------------------------------',             /,&
-' Scalaire         Numero  Valeur min  Valeur max',             /,&
-' -----------------------------------------------'               )
- 3111 format(                                                     &
- 1x,    a16,   i7,      e12.4,      e12.4                        )
- 3112 format(                                                     &
-' ---------------------------------------',                     /)
- 3210 format(                                                     &
-' --- Viscosite de maillage (methode ALE)',                     /,&
-' -----------------------------------------',                   /,&
-' Propriete          Valeur min  Valeur max',                   /,&
-' -----------------------------------------'                     )
- 3211 format(                                                     &
- 2x,    a16,      e12.4,      e12.4                              )
- 3212 format(                                                     &
-' -----------------------------------------',                   /)
-
- 9001  format(                                                    &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========',                                               /,&
-'@    INCOHERENCE ENTRE LES PARAMETRES ET LA MASSE VOLUMIQUE',  /,&
-'@',                                                            /,&
-'@  On a indique que la masse volumique etait',                 /,&
-'@     constante (IROVAR=0) mais on a modifie ses',             /,&
-'@     valeurs aux cellules ou aux faces de bord.',             /,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute.',                            /,&
-'@',                                                            /,&
-'@  Verifier l''interface, cs_user_parameters.f90, et usphyv.', /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 9002  format(                                                    &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========',                                               /,&
-'@    INCOHERENCE ENTRE LES PARAMETRES ET',                     /,&
-'@                                    LA VISCOSITE MOLECULAIRE',/,&
-'@',                                                            /,&
-'@  On a indique que la viscosite moleculaire',                 /,&
-'@     etait constante (IVIVAR=0) mais on a modifie ses',/,&
-'@     valeurs aux cellules.',                                  /,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute.',                            /,&
-'@',                                                            /,&
-'@  Verifier l''interface, cs_user_parameters.f90, et usphyv.', /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 9003  format(                                                    &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========',                                               /,&
-'@    MODULE COMPRESSIBLE',                                     /,&
-'@    INCOHERENCE ENTRE USCFPV ET USCFX2 POUR',                 /,&
-'@                                    LA VISCOSITE MOLECULAIRE',/,&
-'@',                                                            /,&
-'@  En compressible la viscosite moleculaire est constante par',/,&
-'@     defaut (IVIVAR=0) et la valeur de IVIVAR n''a',   /,&
-'@     pas ete modifiee dans uscfx2. Pourtant, on a modifie',   /,&
-'@     les valeurs de la viscosite moleculaire dans usphyv.',   /,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute.',                            /,&
-'@',                                                            /,&
-'@  Verifier uscfx2 et usphyv.',                                /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 9011  format(                                                    &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========',                                               /,&
-'@    LA PROPRIETE PHYSIQUE ', a16  ,' N A PAS ETE',            /,&
-'@                                       CORRECTEMENT DEFINIE.',/,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute.',                            /,&
-'@',                                                            /,&
-'@  La propriete physique identifiee ci-dessus est variable et',/,&
-'@    le minimum atteint est', e12.4                           ,/,&
-'@  Verifier que cette propriete a ete definie et',             /,&
-'@    que la loi adoptee conduit a des valeurs correctes.',     /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 9012  format(                                                    &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========',                                               /,&
-'@    LA VISCOSITE TURBULENTE',                                 /,&
-'@                           N A PAS ETE CORRECTEMENT DEFINIE.',/,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute.',                            /,&
-'@',                                                            /,&
-'@  Le minimum atteint est', e12.4                             ,/,&
-'@  Verifier le cas echeant la definition de la masse',         /,&
-'@    volumique et la modification de la viscosite turbulente', /,&
-'@    dans usvist.',                                            /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 9013  format(                                                    &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========',                                               /,&
-'@    INCOHERENCE ENTRE LES PARAMETRES ET LE COEFFICIENT'      ,/,&
-'@    de dilatation volumique  Beta'                           ,/,&
-'@',                                                            /,&
-'@  On a indique que la masse volumique etait',                 /,&
-'@     variable (IROVAR=1) mais on n a pas modifie ',           /,&
-'@     la valeur de Beta dans l''interface ou dans usphyv',     /,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute.',                            /,&
-'@',                                                            /,&
-'@  Verifier l''interface et usphyv.'                         , /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
-
- 9111  format(                                                    &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========',                                               /,&
-'@    LA DIFFUSIVITE DU SCALAIRE ', a16                        ,/,&
-'@       (SCALAIRE NUMERO', i10   ,') N A PAS ETE',             /,&
-'@                                       CORRECTEMENT DEFINIE.',/,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute.',                            /,&
-'@',                                                            /,&
-'@  La propriete physique identifiee ci-dessus est variable et',/,&
-'@    le minimum atteint est', e12.4                           ,/,&
-'@  Verifier que cette propriete a bien ete definie et',        /,&
-'@    que la loi adoptee conduit a des valeurs correctes.',     /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 9211  format(                                                    &
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========',                                               /,&
-'@    LA VISCOSITE DE MAILLAGE N A PAS ETE',                    /,&
-'@                                       CORRECTEMENT DEFINIE.',/,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute.',                            /,&
-'@',                                                            /,&
-'@  Le minimum atteint est', e12.4,                             /,&
-'@  Verifier le cas echeant la definition de la viscosite.',    /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
- 9999 format(                                                     &
-'@',                                                            /,&
-'@',                                                            /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /,&
-'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
-'@    =========',                                               /,&
-'@    DES GRANDEURS PHYSIQUES ONT DES VALEURS INCORRECTES',     /,&
-'@',                                                            /,&
-'@  Le calcul ne sera pas execute (',i10,' erreurs).',          /,&
-'@',                                                            /,&
-'@  Se reporter aux impressions precedentes pour plus de',      /,&
-'@    renseignements.',                                         /,&
-'@  Verifier les definitions et lois definies',                 /,&
-'@    dans usphyv ou dans l''interface graphique.',             /,&
-'@',                                                            /,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',                                                            /)
-
-#else
-
  1000 format(                                                     &
 ' Nb of clippings for the effective viscosity (mu+mu_t>0):',i10,/)
  3010 format(                                                     &
@@ -1178,7 +1210,8 @@ endif
 '@',                                                            /,&
 '@  The calculation will not be run.',                          /,&
 '@',                                                            /,&
-'@  Check the interface, cs_user_parameters.f90, and usphyv',   /,&
+'@  Check the interface, cs_user_parameters.f90,',              /,&
+'@     and cs_user_physical_properties',                        /,&
 '@',                                                            /,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',                                                            /)
@@ -1197,7 +1230,8 @@ endif
 '@',                                                            /,&
 '@  The calculation will not be run.',                          /,&
 '@',                                                            /,&
-'@  Check the interface, cs_user_parameters.f90, and usphyv',   /,&
+'@  Check the interface, cs_user_parameters.f90,',              /,&
+'@     and cs_user_physical_properties',                        /,&
 '@',                                                            /,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',                                                            /)
@@ -1213,11 +1247,11 @@ endif
 '@  In the compressible module, the molecular viscosity is',    /,&
 '@     constant by default (IVIVAR=0) and the value',    /,&
 '@     of IVIVAR  has not been modified in uscfx2. Yet, its',   /,&
-'@     value has been modified in usphyv.',                     /,&
+'@     value has been modified in cs_user_physical_properties.',/,&
 '@',                                                            /,&
 '@  The calculation will not be run.',                          /,&
 '@',                                                            /,&
-'@  Verify uscfx2 and usphyv.',                                 /,&
+'@  Verify uscfx2 and cs_user_physical_properties.',            /,&
 '@',                                                            /,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',                                                            /)
@@ -1266,11 +1300,11 @@ endif
 '@',                                                            /,&
 '@  The density has been declared variable (IROVAR=1) but',     /,&
 '@     the value of Beta has not been modified     ',           /,&
-'@     in GUI or usphyv',                                       /,&
+'@     in GUI or cs_user_physical_properties',                  /,&
 '@',                                                            /,&
 '@  The calculation will not be run',                           /,&
 '@',                                                            /,&
-'@  Check the interface or usphyv.'                           , /,&
+'@  Check the interface or cs_user_physical_properties.',       /,&
 '@',                                                            /,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@', /)
@@ -1290,7 +1324,8 @@ endif
 '@',                                                            /,&
 '@  The physical property identified is variable and the',      /,&
 '@    minimum reached is', e12.4                               ,/,&
-'@  Verify that this property has been defined in usphyv and',  /,&
+'@  Verify that this property has been defined in',             /,&
+'@    cs_user_physical_properties and',                         /,&
 '@    that the chosen law leads to correct values.',            /,&
 '@',                                                            /,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -1323,13 +1358,11 @@ endif
 '@  The calculation will not be run (',i10,' errors).',         /,&
 '@',                                                            /,&
 '@  Refer to previous warnings for further information.',       /,&
-'@  Verify usphyv.',                                            /,&
+'@  Verify cs_user_physical_properties.',                       /,&
 '@',                                                            /,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',                                                            /)
 
-#endif
-
 !----
 ! End
 !----
