@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -89,22 +89,6 @@ module pointe
 
   !=============================================================================
 
-  !> \defgroup auxiliary Auxiliary variables
-
-  !> \addtogroup auxiliary
-  !> \{
-
-  !... Auxiliaires
-
-  !> non-dimensional distance \f$y^+\f$ between a given volume and the closest
-  !> wall, when it is necessary (LES with van Driest-wall damping).
-  !> The adimensional distance \f$y^+\f$ between the center of the cell \c iel
-  !> and the closest wall is therefore \c yplpar(iel1)
-  double precision, allocatable, dimension(:)   :: yplpar
-
-  !> \}
-  !=============================================================================
-
   !> \defgroup coupled_case Specific arrays for the coupled case
 
   !> \addtogroup coupled_case
@@ -131,7 +115,6 @@ module pointe
 
   !=============================================================================
 
-  !... Parametres du module thermique 1D
   !> \defgroup thermal_1D Thermal 1D module parameters
 
   !> \addtogroup thermal_1D
@@ -148,6 +131,19 @@ module pointe
 
   !> \}
 
+  !=============================================================================
+
+  !> \defgroup porosity_from_scan Porosity from scan module parameters
+
+  !> \addtogroup porosity_from_scan
+  !> \{
+
+  !> Activate the computation
+  logical(c_bool), pointer, save :: compute_porosity_from_scan
+
+  !> \}
+
+
   !=============================================================================
 
   !... Auxiliaires
@@ -156,22 +152,18 @@ module pointe
 
   !> \anchor ncepdc
   !> number of cells in which a pressure drop is imposed.
-  !> See the user subroutine \ref cs_f_user_head_losses
   integer, save :: ncepdc
 
   !> \anchor icepdc
   !> number of the \c ncepdc cells in which a pressure drop is imposed.
-  !> See \c {iicepd} and the user subroutine \ref cs_f_user_head_losses
+  !> See \c {iicepd}
   integer, allocatable, dimension(:) :: icepdc
 
-  !> zone with head losses
-  integer, allocatable, dimension(:) :: izcpdc
-
   !> \anchor ckupdc
   !> value of the coefficients of the pressure drop tensor of the
   !> \c ncepdc cells in which a pressure drop is imposed.
-  !> Note the 6 values are sorted as follows: (k11, k22, k33, k12, k23, k33).
-  !> See \c ickpdc and the user subroutine \ref cs_f_user_head_losses
+  !> Note the 6 values are interleaved as follows: (k11, k22, k33, k12, k23, k13).
+  !> See \c ickpdc
   double precision, allocatable, dimension(:,:) :: ckupdc
 
   !> Head loss factor of the fluid outside the domain, between infinity and
@@ -189,7 +181,7 @@ module pointe
   !> \anchor icetsm
   !> number of the \c ncetsm cells in which a mass source term is imposed.
   !> See \c iicesm and the user subroutine \ref cs_user_mass_source_terms}}
-  integer, allocatable, dimension(:) :: icetsm
+  integer, allocatable, dimension(:), target :: icetsm
 
   !> zone where a mass source term is imposed.
   integer, allocatable, dimension(:) :: izctsm
@@ -199,15 +191,15 @@ module pointe
   !> - 0 for an injection at ambient value,
   !> - 1 for an injection at imposed value.
   !> See the user subroutine \ref cs_user_mass_source_terms
-  integer, allocatable, dimension(:,:) :: itypsm
+  integer, allocatable, dimension(:,:), target :: itypsm
 
   !> \anchor smacel
   !> value of the mass source term for pressure.
   !> For the other variables, eventual imposed injection value.
   !> See the user subroutine \ref cs_user_mass_source_terms
-  double precision, allocatable, dimension(:,:) :: smacel
+  double precision, allocatable, dimension(:,:), target :: smacel
 
-  !> liquid-vapour mass transfer term for cavitating flows
+  !> liquid-vapor mass transfer term for cavitating flows
   !> and its derivative with respect to pressure
   double precision, allocatable, dimension(:) :: gamcav, dgdpca
 
@@ -319,7 +311,6 @@ contains
     use ppincl
     use lagran
     use albase
-    use ihmpre
     use field
     use cs_c_bindings
 
@@ -339,7 +330,7 @@ contains
 
     ! ALE array for structure definition
 
-    if (iale.eq.1) then
+    if (iale.ge.1) then
       allocate(idfstr(nfabor))
     endif
 
@@ -358,15 +349,9 @@ contains
       call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
     endif
 
-    ! Wall-distance calculation
-
-    if (itytur.eq.4 .and. idries.eq.1) then
-      allocate(yplpar(ncelet))
-    endif
-
-    ! liquid-vapour mass transfer term for cavitating flows
+    ! liquid-vapor mass transfer term for cavitating flows
     ! and its part implicit in pressure
-    if (icavit.ge.0) then
+    if (iand(ivofmt,VOF_MERKLE_MASS_TRANSFER).ne.0) then
       allocate(gamcav(ncelet), dgdpca(ncelet))
     endif
 
@@ -395,19 +380,7 @@ contains
 
     allocate(buffer(ncelet))
 
-    if (allocated(yplpar)) then
-      do iel = 1, ncel
-        buffer(iel) = yplpar(iel)
-      enddo
-      deallocate(yplpar)
-      call synsca (buffer)
-      allocate(yplpar(ncelet))
-      do iel = 1, ncelet
-        yplpar(iel) = buffer(iel)
-      enddo
-    endif
-
-    ! liquid-vapour mass transfer term for cavitating flows
+    ! liquid-vapor mass transfer term for cavitating flows
     ! and its part implicit in pressure
 
     if (allocated(gamcav)) then
@@ -446,9 +419,7 @@ contains
 
     deallocate(itrifb)
     if (allocated(idfstr)) deallocate(idfstr)
-    if (allocated(izcpdc)) deallocate(izcpdc)
     if (allocated(izctsm)) deallocate(izctsm)
-    if (allocated(yplpar)) deallocate(yplpar)
     if (allocated(b_head_loss)) deallocate(b_head_loss)
     if (allocated(gamcav)) deallocate(gamcav, dgdpca)
 
@@ -461,7 +432,7 @@ contains
   subroutine init_kpdc
 
     allocate(icepdc(ncepdc))
-    allocate(ckupdc(ncepdc,6))
+    allocate(ckupdc(6,ncepdc))
 
   end subroutine init_kpdc
 
@@ -476,7 +447,7 @@ contains
 
   !=============================================================================
 
-  subroutine init_tsma ( nvar )
+  subroutine init_tsma(nvar)
 
     implicit none
 
@@ -492,13 +463,13 @@ contains
 
   subroutine finalize_tsma
 
+    ncetsm = 0
     deallocate(icetsm)
     deallocate(itypsm)
     deallocate(smacel)
 
   end subroutine finalize_tsma
 
-  !=============================================================================
   !=============================================================================
 
   subroutine init_pcond ( nvar )
@@ -531,6 +502,7 @@ contains
     deallocate(flthr, dflthr)
 
   end subroutine finalize_pcond
+
   !=============================================================================
 
   subroutine init_vcond ( nvar , ncelet)
@@ -620,6 +592,28 @@ contains
 
   !=============================================================================
 
+  !> \brief Allocate the cs_glob_porosity_from_scan structure.
+
+  subroutine porosity_from_scan_init
+
+    use, intrinsic :: iso_c_binding
+    use cs_c_bindings
+
+    implicit none
+
+    ! Local variables
+    type(c_ptr) :: c_compute_from_scan
+
+    call cs_f_porosity_from_scan_get_pointer(c_compute_from_scan)
+
+    call c_f_pointer(c_compute_from_scan, compute_porosity_from_scan)
+
+    return
+
+  end subroutine porosity_from_scan_init
+
+  !=============================================================================
+
   !> \brief Return pointer to the ifpt1d array for the 1D wall thermal module.
 
   !> \param[out]    ifpt1d         pointer to ifpt1d
@@ -668,4 +662,46 @@ contains
 
   !=============================================================================
 
+  !> \brief Return pointers to the mass source term arrays
+
+  !> \param[in]   var_id   id of associated variables
+  !> \param[out]  ncesmp   number of cells with mass source terms
+  !> \param[out]  icetsm   cell numbers with mass source terms (1 to n)
+  !> \param[out]  itpsmp   mass source types (0: ambient value, 1: smacel value)
+  !> \param[out]  smacel     mass source values
+
+  subroutine cs_f_volume_mass_injection_get_arrays        &
+    (var_id, ncesmp, icetsm_p, itypsm_p, smacel_p)        &
+    bind(C, name='cs_f_volume_mass_injection_get_arrays')
+
+    use, intrinsic :: iso_c_binding
+    implicit none
+
+    integer(c_int), value :: var_id
+    integer(c_int) :: ncesmp
+    type(c_ptr) :: icetsm_p, itypsm_p, smacel_p
+
+    ! Local variables
+
+    integer ivar
+
+    ! Get pointer values
+
+    ivar= var_id
+
+    ncesmp = ncetsm
+    if (ncetsm.gt.0) then
+      icetsm_p = c_loc(icetsm(1))
+      itypsm_p = c_loc(itypsm(1, var_id))
+      smacel_p = c_loc(smacel(1, var_id))
+    else
+      icetsm_p = c_null_ptr
+      itypsm_p = c_null_ptr
+      smacel_p = c_null_ptr
+    endif
+
+  end subroutine cs_f_volume_mass_injection_get_arrays
+
+  !=============================================================================
+
 end module pointe
