@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -76,23 +76,30 @@ double precision, dimension(nfbrps), intent(out)   :: bflux
 
 ! Local variables
 
-integer ::         inc, iccocg, f_id
-integer ::         ifac, iloc, ivar
-integer ::         iel
-integer ::         iflmab
+integer ::         f_id
+integer ::         iloc, ivar
 
-double precision :: cpp   , srfbn , hext  , heq   , hbord2
-double precision :: flumab, diipbx, diipby, diipbz, tcel
+character(len=80) :: f_name
+integer(c_int), dimension(:), allocatable :: c_faces
 
-double precision, allocatable, dimension(:) :: theipb
-double precision, allocatable, dimension(:,:) :: grad
-double precision, dimension(:), pointer :: cpro_cp
-double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
-double precision, dimension(:), pointer :: bmasfl, tscalp, hextp, hintp, dist_theipb
+!===============================================================================
+! Interfaces
+!===============================================================================
 
-type(var_cal_opt) :: vcopt
+interface
 
-logical(c_bool), dimension(:), pointer ::  cpl_faces
+  subroutine cs_post_boundary_flux(scalar_name, n_loc_b_faces, b_face_ids,   &
+                                   b_face_flux)                              &
+    bind(C, name='cs_post_boundary_flux')
+    use, intrinsic :: iso_c_binding
+    implicit none
+    character(kind=c_char, len=1), dimension(*), intent(in) :: scalar_name
+    integer(c_int), value :: n_loc_b_faces
+    integer(c_int), dimension(*) :: b_face_ids
+    real(kind=c_double), dimension(*) :: b_face_flux
+  end subroutine cs_post_boundary_flux
+
+end interface
 
 !===============================================================================
 
@@ -100,125 +107,20 @@ logical(c_bool), dimension(:), pointer ::  cpl_faces
 
 if (iscalt.gt.0) then
 
-  ivar   = isca(iscalt)
-
+  ivar = isca(iscalt)
   f_id = ivarfl(ivar)
 
-  ! Boundary condition pointers for gradients and advection
+  call field_get_name (f_id, f_name)
 
-  call field_get_coefa_s(ivarfl(ivar), coefap)
-  call field_get_coefb_s(ivarfl(ivar), coefbp)
-
-  ! Boundary condition pointers for diffusion
-
-  call field_get_coefaf_s(ivarfl(ivar), cofafp)
-  call field_get_coefbf_s(ivarfl(ivar), cofbfp)
-
-  ! Boundary condition pointers for diffusion with coupling
-
-  call field_get_hext(f_id, hextp)
-  call field_get_hint(f_id, hintp)
-
-  ! Pointers to fields and properties
-
-  call field_get_val_prev_s(ivarfl(ivar), tscalp)
-
-  if (iscacp(iscalt).eq.1 .and. icp.ge.0) then
-    call field_get_val_s(icp, cpro_cp)
-  endif
-
-  call field_get_key_int(ivarfl(ivar), kbmasf, iflmab)
-  call field_get_val_s(iflmab, bmasfl)
-
-  ! Compute variable values at boundary faces
-
-  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
-
-  allocate(theipb(nfabor))
-  theipb = 0.d0
+  allocate(c_faces(nfbrps))
 
   do iloc = 1, nfbrps
-    ifac = lstfbr(iloc)
-    iel = ifabor(ifac)
-    theipb(ifac) = tscalp(iel)
+    c_faces(iloc) = lstfbr(iloc) - 1
   enddo
 
-  ! Reconstructed fluxes
-  if (vcopt%ircflu .gt. 0 .and. itbrrb.eq.1) then
+  call cs_post_boundary_flux(trim(f_name)//c_null_char, nfbrps, c_faces, bflux)
 
-    ! Compute gradient of temperature / enthalpy
-
-    allocate(grad(3,ncelet))
-
-    inc = 1
-    iccocg = 1
-
-    call field_gradient_scalar &
-      (ivarfl(ivar), 0, imrgra, inc,         &
-       iccocg,                               &
-       grad)
-
-    ! Compute reconstructed temperature
-
-    do iloc = 1, nfbrps
-      ifac = lstfbr(iloc)
-      iel = ifabor(ifac)
-
-      diipbx = diipb(1,ifac)
-      diipby = diipb(2,ifac)
-      diipbz = diipb(3,ifac)
-
-      theipb(ifac) = theipb(ifac)       &
-                   + diipbx*grad(1,iel) &
-                   + diipby*grad(2,iel) &
-                   + diipbz*grad(3,iel)
-    enddo
-
-    deallocate(grad)
-  endif
-
-  if (vcopt%icoupl.gt.0) then
-     call field_get_coupled_faces(ivarfl(ivar), cpl_faces)
-    allocate(dist_theipb(nfabor))
-    call cs_ic_field_dist_data_by_face_id(ivarfl(ivar), 1, theipb, dist_theipb)
-  endif
-
-  do iloc = 1, nfbrps
-
-    ifac = lstfbr(iloc)
-    iel = ifabor(ifac)
-
-    if (iscacp(iscalt).eq.1) then
-      if (icp.ge.0) then
-        cpp = cpro_cp(iel)
-      else
-        cpp = cp0
-      endif
-    else
-      cpp = 1.d0
-    endif
-
-    srfbn = max(surfbn(ifac), epzero**2)
-    flumab = bmasfl(ifac)
-
-    bflux(iloc) = (cofafp(ifac) + cofbfp(ifac)*theipb(ifac))   &
-                - flumab*cpp/srfbn*(coefap(ifac) + coefbp(ifac)*theipb(ifac))
-    ! here bflux = 0 if current face is coupled
-
-    if (vcopt%icoupl.gt.0) then
-      if (cpl_faces(ifac)) then
-        heq = hextp(ifac) * hintp(ifac) / ((hextp(ifac) + hintp(ifac))*srfbn)
-        bflux(iloc) = heq*(theipb(ifac)-dist_theipb(ifac))
-      endif
-    endif
-
-  enddo
-
-  if (vcopt%icoupl.gt.0) then
-    deallocate(dist_theipb)
-  endif
-
-  deallocate(theipb)
+  deallocate(c_faces)
 
 else ! if thermal variable is not available
 
@@ -228,10 +130,6 @@ else ! if thermal variable is not available
 
 endif
 
-!--------
-! Formats
-!--------
-
 !----
 ! End
 !----
@@ -295,10 +193,9 @@ integer ::         inc, iccocg
 integer ::         iel, ifac, iloc, ivar
 integer ::         ifcvsl, itplus, itstar
 
-double precision :: xvsl  , srfbn , heq   , phit
+double precision :: xvsl  , srfbn , heq
 double precision :: diipbx, diipby, diipbz
-double precision :: numer, denom, tcel
-double precision :: heqohint, phitohint
+double precision :: numer, denom, visls_0
 
 double precision, allocatable, dimension(:) :: theipb
 double precision, allocatable, dimension(:,:) :: grad
@@ -360,10 +257,7 @@ if (itstar.ge.0 .and. itplus.ge.0) then
     inc = 1
     iccocg = 1
 
-    call field_gradient_scalar &
-      (ivarfl(ivar), 0, imrgra, inc,         &
-       iccocg,                               &
-       grad)
+    call field_gradient_scalar(ivarfl(ivar), 0, 0, inc, iccocg, grad)
 
     ! Compute reconstructed temperature
 
@@ -393,6 +287,9 @@ if (itstar.ge.0 .and. itplus.ge.0) then
   call field_get_key_int (ivarfl(ivar), kivisl, ifcvsl)
   if (ifcvsl .ge. 0) then
     call field_get_val_s(ifcvsl, cviscl)
+    visls_0 = -1
+  else
+    call field_get_key_double(ivarfl(ivar), kvisl0, visls_0)
   endif
 
   ! Boundary condition pointers for gradients and advection
@@ -410,7 +307,7 @@ if (itstar.ge.0 .and. itplus.ge.0) then
     if (ifcvsl.ge.0) then
       xvsl = cviscl(iel)
     else
-      xvsl = visls0(iscalt)
+      xvsl = visls_0
     endif
 
     srfbn = surfbn(ifac)
@@ -418,7 +315,8 @@ if (itstar.ge.0 .and. itplus.ge.0) then
     numer = (cofafp(ifac) + cofbfp(ifac)*theipb(ifac)) * distb(ifac)
     ! here numer = 0 if current face is coupled
 
-    if (vcopt%icoupl.gt.0) then
+    if (vcopt%icoupl.gt.0.and.ntcabs.gt.0) then
+      ! FIXME exchange coefs not computed at start of calculation
       if (cpl_faces(ifac)) then
         heq = hextp(ifac) * hintp(ifac) / ((hextp(ifac) + hintp(ifac))*srfbn)
         numer = heq*(theipb(ifac)-dist_theipb(ifac)) * distb(ifac)
