@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -109,13 +109,9 @@ integer          ifac, ivar
 !===============================================================================
 
 !      ICODCL = 0 INDIQUE QUE LA CL N'A PAS ETE RENSEIGNEE
-!      ITYPFB = 0 INDIQUE QUE LA CL N'A PAS ETE RENSEIGNEE
-
 !      RINFIN : VALEUR INFINIE
 
-do ifac = 1, nfabor
-  itypfb(ifac) = 0
-enddo
+! itypfb has already been initialized with default value, do not modify it here.
 
 ! Pour toutes les variables, on initialise RCODCL(1)a RINFIN
 ! Cette valeur sera reinitialisee a zero dans typecl.F
@@ -131,17 +127,15 @@ do ivar = 1, nvar
 enddo
 
 ! En ALE, on initialise aussi le tableau IALTYB
-if (iale.eq.1) then
+if (iale.ge.1) then
   do ifac = 1, nfabor
     ialtyb(ifac) = 0
   enddo
 endif
 
-! POUR LES PHYSIQUES PARTICULIERES
-
+! Physical module
 if (ippmod(iphpar).ge.1) then
   call ppprcl(nvar, izfppp, rcodcl)
-  !==========
 endif
 
 !----
