@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -105,11 +105,11 @@ integer          iccocg, inc   , init  , isym
 integer          ii, iel   , ifac
 integer          nswmpr
 integer          isweep, niterf
-integer          nswrgp, imligp, iwarnp
+integer          imrgrp, nswrgp, imligp, iwarnp, iwgrp
 integer          iflmas, iflmab
 integer          idiffp, iconvp, ndircp
 integer          ibsize, iesize, iphydp
-integer          imucpp, f_id0
+integer          imucpp, f_id0, f_id
 double precision residu
 double precision thetap
 double precision epsrgp, climgp, extrap, epsilp
@@ -124,6 +124,8 @@ double precision, allocatable, dimension(:) :: cfapot, cfbpot
 double precision, allocatable, dimension(:) :: viscf, viscb
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: brom, crom, croma
+double precision, dimension(:), pointer :: cpro_rho_mass, bpro_rho_mass
+double precision, dimension(:), pointer :: brom_eos, crom_eos
 
 type(var_cal_opt) :: vcopt
 
@@ -334,8 +336,10 @@ do while (isweep.le.nswmpr.and.residu.gt.tcrite)
     iccocg = 1
     init = 1
     inc  = 0
+    imrgrp = vcopt%imrgra
     nswrgp = vcopt%nswrgr
     imligp = vcopt%imligr
+    iwgrp  = vcopt%iwgrec
     iwarnp = vcopt%iwarni
     epsrgp = vcopt%epsrgr
     climgp = vcopt%climgr
@@ -345,8 +349,8 @@ do while (isweep.le.nswmpr.and.residu.gt.tcrite)
 
     call itrgrp &
     !==========
-   ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp ,   &
-     iwarnp ,                                                                  &
+   ( f_id0  , init   , inc    , imrgrp , iccocg , nswrgp , imligp , iphydp ,   &
+     iwgrp  , iwarnp ,                                                         &
      epsrgp , climgp , extrap ,                                                &
      rvoid  ,                                                                  &
      pot    ,                                                                  &
@@ -387,6 +391,7 @@ iccocg = 1
 init = 0
 inc  = 0
 iphydp = 0
+imrgrp = vcopt%imrgra
 nswrgp = vcopt%nswrgr
 imligp = vcopt%imligr
 iwarnp = vcopt%iwarni
@@ -395,7 +400,7 @@ climgp = vcopt%climgr
 extrap = vcopt%extrag
 
 call itrmas &
- ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp ,     &
+ ( f_id0  , init   , inc    , imrgrp , iccocg , nswrgp , imligp , iphydp ,     &
    0      , iwarnp ,                                                           &
    epsrgp , climgp , extrap ,                                                  &
    rvoid  ,                                                                    &
@@ -414,7 +419,7 @@ nswrgp = 0
 inc = 0
 
 call itrmas &
- ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp ,     &
+ ( f_id0  , init   , inc    , imrgrp , iccocg , nswrgp , imligp , iphydp ,     &
    0      , iwarnp ,                                                           &
    epsrgp , climgp , extrap ,                                                  &
    rvoid  ,                                                                    &
@@ -425,6 +430,27 @@ call itrmas &
    dt     ,                                                                    &
    imasfl , bmasfl )
 
+! Update density (which is coherent with the mass)
+!-------------------------------------------------
+
+if (irovar.eq.1) then
+  call field_get_val_s(icrom, crom_eos)
+  call field_get_val_s(ibrom, brom_eos)
+
+  call field_get_id("density_mass", f_id)
+  call field_get_val_s(f_id, cpro_rho_mass)
+  call field_get_id("boundary_density_mass", f_id)
+  call field_get_val_s(f_id, bpro_rho_mass)
+
+  do iel = 1, ncelet
+    cpro_rho_mass(iel) = crom_eos(iel)
+  enddo
+
+  do ifac = 1, nfabor
+    bpro_rho_mass(ifac) = brom_eos(ifac)
+  enddo
+endif
+
 !===============================================================================
 ! 6. Free solver setup
 !===============================================================================
@@ -442,18 +468,6 @@ deallocate(viscf, viscb)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
- 1400 format(1X,A16,' : sweep = ',I5,' norme second membre = ',E14.6,&
-             ', relaxp = ',E14.6)
- 1600 format(                                                     &
-'@                                                            ',/,&
-'@ @@ ATTENTION : ', A16,' Prediction du flux de masse        ',/,&
-'@    =========                                               ',/,&
-'@  Nombre d''iterations maximal ',I10   ,' atteint           ',/,&
-'@                                                            '  )
-
-#else
-
  1400 format(1X,A16,' : sweep = ',I5,' right hand side norm = ',E14.6,&
              ', relaxp = ',E14.6)
  1600 format(                                                     &
@@ -463,8 +477,6 @@ deallocate(viscf, viscb)
 '@  Maximum number of iterations ',I10   ,' reached           ',/,&
 '@'                                                              )
 
-#endif
-
 !----
 ! End
 !----
