@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -102,7 +102,7 @@ contains
     deallocate(buffer)
 
     do ii = 1, size
-      call synsca (array(1,ii))
+      call synsca (array(:,ii))
       !==========
     enddo
 
@@ -143,7 +143,7 @@ contains
     enddo
     deallocate(buffer)
 
-    call synvec (array(1,1), array(1,2), array(1,3))
+    call synvec (array(:,1), array(:,2), array(:,3))
     !==========
 
   end subroutine resize_vec_real_array_ni
@@ -190,6 +190,46 @@ contains
 
   !=============================================================================
 
+  ! Resize a symmetric tensor interleaved array and synchronize halo
+
+  subroutine resize_sym_tens_real_array ( array )
+
+    use mesh, only: ncel, ncelet
+
+    implicit none
+
+    ! Arguments
+
+    double precision, pointer, dimension(:,:) :: array
+
+    ! Local variables
+
+    integer iel, isou
+    double precision, allocatable, dimension(:,:) :: buffer
+
+    allocate(buffer(6,ncel))
+    do iel = 1, ncel
+      do isou = 1, 6
+        buffer(isou,iel) = array(isou,iel)
+      enddo
+    enddo
+    deallocate(array)
+
+    allocate(array(6,ncelet))
+    do iel = 1, ncel
+      do isou = 1, 6
+        array(isou,iel) = buffer(isou,iel)
+      enddo
+    enddo
+    deallocate(buffer)
+
+    call syntis (array)
+    !==========
+
+  end subroutine resize_sym_tens_real_array
+
+  !=============================================================================
+
   ! Resize a tensor non-interleaved array and synchronize halo
 
   subroutine resize_tens_real_array_ni ( array )
@@ -225,9 +265,9 @@ contains
 
     call synten &
     !==========
-   ( array(1,1), array(1,2), array(1,3), &
-     array(1,4), array(1,5), array(1,6), &
-     array(1,7), array(1,8), array(1,9) )
+   ( array(:,1), array(:,2), array(:,3), &
+     array(:,4), array(:,5), array(:,6), &
+     array(:,7), array(:,8), array(:,9) )
 
   end subroutine resize_tens_real_array_ni
 
