@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -33,11 +33,15 @@
 !> \f[
 !> \dfrac{\alpha^n - \alpha^{n-1}}{\Delta t}
 !>     + \divs \left( \alpha^n \vect{u}^n \right)
+!>     + \divs \left( \left[ \alpha^n
+!>                           \left( 1 - \alpha^{n} \right)
+!>                    \right] \vect{u^r}^n \right)
 !>     = \dfrac{\Gamma_V \left( \alpha^{n-1}, p^n \right)}{\rho_v}
 !> \f]
 !> with \f$ \Gamma_V \f$ the eventual vaporization source term (Merkle model) in
-!> case the cavitation model is enabled and \f$ \rho_v \f$ the reference gas
-!> density.
+!> case the cavitation model is enabled, \f$ \rho_v \f$ the reference gas
+!> density and \f$ \vect{u^r} \f$ the drift velocity for the compressed
+!> interface.
 !>
 !-------------------------------------------------------------------------------
 
@@ -71,6 +75,7 @@ use cavitation
 use vof
 use parall
 use cs_c_bindings
+use cs_cf_bindings
 
 !===============================================================================
 
@@ -87,30 +92,31 @@ integer          ivar  , iel, ifac, f_id
 integer          init
 integer          nswrgp, imligp
 integer          iconvp, idiffp, ndircp
-integer          nswrsp, ircflp, ischcp, isstpp, iescap
+integer          imrgrp, nswrsp, ircflp, ischcp, isstpp, iescap
 integer          iflmas, iflmab
-integer          iwarnp
+integer          iwarnp, i_mass_transfer
 integer          imucpp, idftnp, iswdyp
 
-integer          icvflb
+integer          icvflb, kiflux, kbflux, icflux_id, bcflux_id
 integer          ivoid(1)
 integer          kscmin, kscmax, iclmin(1), iclmax(1)
 
-double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp, epsrsp, thetap
+double precision blencp, epsilp, epsrgp, climgp, relaxp, epsrsp, thetap
 
 double precision rvoid(1)
 double precision vmin(1), vmax(1)
 double precision dtmaxl, dtmaxg
 double precision scmaxp, scminp
 double precision thets, thetv, tsexp
+double precision normp
 
 double precision, allocatable, dimension(:) :: viscf, viscb
 double precision, allocatable, dimension(:) :: smbrs, rovsdt
 double precision, allocatable, dimension(:) :: dpvar, divu
-double precision, dimension(:), pointer :: imasfl, bmasfl
+double precision, dimension(:), pointer :: ivolfl, bvolfl
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
 double precision, dimension(:), pointer :: c_st_voidf
-double precision, dimension(:), pointer :: cvar_pr, cvara_pr
+double precision, dimension(:), pointer :: cvar_pr, cvara_pr, icflux, bcflux
 double precision, dimension(:), pointer :: cvar_voidf, cvara_voidf
 
 type(var_cal_opt) :: vcopt
@@ -121,6 +127,8 @@ type(var_cal_opt) :: vcopt
 ! 1. Initialization
 !===============================================================================
 
+i_mass_transfer = iand(ivofmt,VOF_MERKLE_MASS_TRANSFER)
+
 ivar = ivolf2
 
 call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
@@ -129,12 +137,13 @@ call field_get_val_s(ivarfl(ivolf2), cvar_voidf)
 call field_get_val_prev_s(ivarfl(ivolf2), cvara_voidf)
 
 ! implicitation in pressure of the vaporization/condensation model (cavitation)
-if (icavit.ge.0.and.itscvi.eq.1) then
+if (i_mass_transfer.ne.0.and.itscvi.eq.1) then
   call field_get_val_s(ivarfl(ipr), cvar_pr)
   call field_get_val_prev_s(ivarfl(ipr), cvara_pr)
 endif
 
 ! Allocate temporary arrays
+
 allocate(viscf(nfac), viscb(nfabor))
 allocate(smbrs(ncelet),rovsdt(ncelet))
 
@@ -153,8 +162,8 @@ call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 call field_get_key_int(ivarfl(ivar), kimasf, iflmas)
 call field_get_key_int(ivarfl(ivar), kbmasf, iflmab)
-call field_get_val_s(iflmas, imasfl)
-call field_get_val_s(iflmab, bmasfl)
+call field_get_val_s(iflmas, ivolfl)
+call field_get_val_s(iflmab, bvolfl)
 
 ! Key id for clipping
 call field_get_key_id("min_scalar_clipping", kscmin)
@@ -175,19 +184,36 @@ thetv = vcopt%thetav
 ! --- Initialization
 
 do iel = 1, ncel
-   smbrs(iel) = 0.d0
+  smbrs(iel) = 0.d0
 enddo
 
 do iel = 1, ncel
-   rovsdt(iel) = 0.d0
+  rovsdt(iel) = 0.d0
 enddo
 
 ! Arbitrary initialization (no diffusion for void fraction)
 do ifac = 1, nfac
-   viscf(ifac) = 1.d0
+  viscf(ifac) = 1.d0
 enddo
 do ifac = 1, nfabor
-   viscb(ifac) = 1.d0
+  viscb(ifac) = 1.d0
+enddo
+
+! Initialize void fraction convection flux
+call field_get_key_id("inner_flux_id", kiflux)
+call field_get_key_id("boundary_flux_id", kbflux)
+
+call field_get_key_int(ivarfl(ivar), kiflux, icflux_id)
+call field_get_key_int(ivarfl(ivar), kbflux, bcflux_id)
+
+call field_get_val_s(icflux_id, icflux)
+do ifac = 1, nfac
+  icflux(ifac) = 0.d0
+enddo
+
+call field_get_val_s(bcflux_id, bcflux)
+do ifac = 1, nfabor
+  bcflux(ifac) = 0.d0
 enddo
 
 !===============================================================================
@@ -198,7 +224,7 @@ enddo
 !   if it has been implicited in pressure at correction step,
 !   in order to ensure mass conservation.
 
-if (icavit.ge.0.and.itscvi.eq.1) then
+if (i_mass_transfer.ne.0.and.itscvi.eq.1) then
   do iel = 1, ncel
     gamcav(iel) = gamcav(iel) + dgdpca(iel)*(cvar_pr(iel)-cvara_pr(iel))
   enddo
@@ -210,7 +236,7 @@ endif
 dtmaxl = 1.d15
 dtmaxg = 1.d15
 
-if (icavit.gt.0) then
+if (i_mass_transfer.ne.0) then
   do iel = 1, ncel
     if (gamcav(iel).lt.0.d0) then
       dtmaxl = -rho2*cvara_voidf(iel)/gamcav(iel)
@@ -232,7 +258,7 @@ endif
 !-------------
 
 ! Cavitation source term (explicit)
-if (icavit.ge.0) then
+if (i_mass_transfer.ne.0) then
   do iel = 1, ncel
     smbrs(iel) = smbrs(iel) + cell_f_vol(iel)*gamcav(iel)/rho2
   enddo
@@ -240,9 +266,10 @@ endif
 
 ! Source term linked with the non-conservative form of convection term
 ! in codits (always implicited)
-
+! FIXME set imasac per variable? Here it could be set to 0
+! and divu not added
 init = 1
-call divmas (init,imasfl,bmasfl,divu)
+call divmas (init,ivolfl,bvolfl,divu)
 
 do iel = 1, ncel
   rovsdt(iel) = rovsdt(iel) - divu(iel)
@@ -267,6 +294,20 @@ else
   enddo
 endif
 
+if (idrift.gt.0) then
+  imrgrp = vcopt%imrgra
+  nswrgp = vcopt%nswrgr
+  imligp = vcopt%imligr
+  iwarnp = vcopt%iwarni
+  epsrgp = vcopt%epsrgr
+  climgp = vcopt%climgr
+
+  call vof_drift_term &
+  !==========
+( imrgrp , nswrgp , imligp , iwarnp , epsrgp , climgp ,          &
+  cvar_voidf      , cvara_voidf     , smbrs  )
+endif
+
 ! Unteady term
 !-------------
 
@@ -281,8 +322,9 @@ enddo
 ! Solving void fraction
 iconvp = vcopt%iconv
 idiffp = vcopt%idiff
-ndircp = ndircl(ivar)
+ndircp = vcopt%ndircl
 nswrsp = vcopt%nswrsm
+imrgrp = vcopt%imrgra
 nswrgp = vcopt%nswrgr
 imligp = vcopt%imligr
 ircflp = vcopt%ircflu
@@ -298,23 +340,22 @@ epsilp = vcopt%epsilo
 epsrsp = vcopt%epsrsm
 epsrgp = vcopt%epsrgr
 climgp = vcopt%climgr
-extrap = vcopt%extrag
 relaxp = vcopt%relaxv
 thetap = vcopt%thetav
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
 
 call codits &
-!==========
  ( idtvar , iterns , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
-   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   iwarnp ,                                                       &
-   blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
+   iwarnp , normp  ,                                              &
+   blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
    relaxp , thetap ,                                              &
    cvara_voidf     , cvara_voidf     ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
-   imasfl , bmasfl ,                                              &
+   ivolfl , bvolfl ,                                              &
    viscf  , viscb  , viscf  , viscb  , rvoid  ,                   &
    rvoid  , rvoid  ,                                              &
    icvflb , ivoid  ,                                              &
@@ -328,7 +369,7 @@ call codits &
 iclmax(1) = 0
 iclmin(1) = 0
 
-if ((icavit.gt.0.and.dt(1).gt.dtmaxg).or.icavit.le.0) then
+if ((i_mass_transfer.ne.0.and.dt(1).gt.dtmaxg).or.i_mass_transfer.eq.0) then
 
   ! --- Calcul du min et max
   vmin(1) = cvar_voidf(1)
