@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2015 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,7 +26,7 @@ subroutine strdep &
  ( itrale , italim , itrfin ,                                     &
    nvar   ,                                                       &
    dt     ,                                                       &
-   flmalf , flmalb , cofale , xprale )
+   cofale , xprale )
 
 !===============================================================================
 ! FONCTION :
@@ -45,11 +45,9 @@ subroutine strdep &
 !                  !    !     !                    couplage implicite          !
 ! nvar             ! i  ! <-- ! total number of variables                      !
 ! dt(ncelet)       ! ra ! <-- ! time step (per cell)                           !
-! flmalf(nfac)     ! tr ! --> ! sauvegarde du flux de masse faces int          !
-! flmalb(nfabor    ! tr ! --> ! sauvegarde du flux de masse faces brd          !
 ! cofale           ! tr ! --> ! sauvegarde des cl de p et u                    !
 !    (nfabor,8)    !    !     !                                                !
-! xprale(ncelet    ! tr ! --> ! sauvegarde de la pression, si nterup           !
+! xprale(ncelet)   ! tr ! --> ! sauvegarde de la pression, si nterup           !
 !                  !    !     !    est >1                                      !
 !__________________!____!_____!________________________________________________!
 
@@ -64,7 +62,6 @@ subroutine strdep &
 !===============================================================================
 
 use paramx
-use ihmpre
 use cstphy
 use numvar
 use optcal
@@ -77,6 +74,7 @@ use parall
 use period
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -84,12 +82,11 @@ implicit none
 
 ! Arguments
 
-integer          itrale , italim , itrfin
-integer          nvar
+integer :: itrale , italim , itrfin, nvar
 
-double precision dt(ncelet)
-double precision flmalf(nfac), flmalb(nfabor), xprale(ncelet)
-double precision cofale(nfabor,11)
+double precision, dimension(:) :: dt
+double precision, pointer, dimension(:) :: xprale
+double precision, pointer, dimension(:,:) :: cofale
 
 ! Local variables
 
@@ -99,12 +96,11 @@ integer          indast
 integer          icvext, icvint, icved
 integer          f_dim
 
-logical          interleaved
-
 double precision delta
 
 double precision, allocatable, dimension(:,:) :: forast
 double precision, dimension(:), pointer :: imasfl, bmasfl
+double precision, dimension(:), pointer :: imasfl_pre, bmasfl_pre
 double precision, dimension(:,:), pointer :: forbr
 double precision, dimension(:,:), pointer :: coefau
 double precision, dimension(:,:,:), pointer :: coefbu
@@ -112,8 +108,32 @@ double precision, dimension(:), pointer :: coefap, coefbp
 double precision, dimension(:), pointer :: cvar_var, cvara_var
 double precision, dimension(:,:), pointer :: cvar_varv, cvara_varv
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
+!===============================================================================
+! Interfaces
+!===============================================================================
+
+interface
+
+  function cs_ast_coupling_get_ext_cvg() result (icvext) &
+    bind(C, name='cs_ast_coupling_get_ext_cvg')
+    use, intrinsic :: iso_c_binding
+    implicit none
+    integer(c_int) :: icvext
+  end function cs_ast_coupling_get_ext_cvg
+
+  subroutine cs_ast_coupling_send_cvg(icv) &
+    bind(C, name='cs_ast_coupling_send_cvg')
+    use, intrinsic :: iso_c_binding
+    implicit none
+    integer(c_int), value :: icv
+  end subroutine cs_ast_coupling_send_cvg
+
+end interface
+
 !===============================================================================
 ! 1. Initialization
 !===============================================================================
@@ -121,7 +141,9 @@ double precision, dimension(:,:), pointer :: cvar_varv, cvara_varv
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
 call field_get_val_s(iflmas, imasfl)
+call field_get_val_prev_s(iflmas, imasfl_pre)
 call field_get_val_s(iflmab, bmasfl)
+call field_get_val_prev_s(iflmab, bmasfl_pre)
 
 call field_get_val_v(iforbr, forbr)
 
@@ -131,6 +153,8 @@ call field_get_coefb_v(ivarfl(iu), coefbu)
 call field_get_coefa_s(ivarfl(ipr), coefap)
 call field_get_coefb_s(ivarfl(ipr), coefbp)
 
+call field_get_key_struct_var_cal_opt(ivarfl(iuma), vcopt)
+
 !===============================================================================
 ! 2. Computue forces on the structures
 !===============================================================================
@@ -176,8 +200,7 @@ enddo
 
 !     Envoi de l'effort applique aux structures externes
 if (nbaste.gt.0) then
-  call astfor(ntcast, nbfast, forast)
-  !==========
+  call astfor(nbfast, forast)
 endif
 
 ! Free memory
@@ -189,21 +212,9 @@ deallocate(forast)
 
 if (nbstru.gt.0) then
 
-  ! - Interface Code_Saturne
-  !   ======================
-
-  if (iihmpr.eq.1) then
-
-    call uistr2 &
-    !==========
- ( xmstru, xcstru, xkstru,     &
-   forstp,                     &
-   dtref , ttcabs, ntcabs   )
-
-  endif
+  call uistr2(xmstru, xcstru, xkstru, forstp)
 
   call usstr2                                                     &
-  !==========
  ( nbstru ,                                                       &
    idfstr ,                                                       &
    dt     ,                                                       &
@@ -225,7 +236,6 @@ endif
 do istr = 1, nbstru
 
   call newmrk                                                     &
-  !==========
  ( istr  , alpnmk  , betnmk          , gamnmk          ,          &
    xmstru(1,1,istr), xcstru(1,1,istr), xkstru(1,1,istr),          &
    xstreq(1,istr)  ,                                              &
@@ -254,54 +264,55 @@ if (nbstru.gt.0) then
   if (delta.lt.epalim) icvint = 1
 endif
 
-if (nbaste.gt.0) call astcv1(ntcast, icvext)
+if (nbaste.gt.0) icvext = cs_ast_coupling_get_ext_cvg()
 
 if (nbstru.gt.0.and.nbaste.gt.0) then
-   icved = icvext*icvint
+  icved = icvext*icvint
 elseif (nbstru.gt.0.and.nbaste.eq.0) then
-   icved = icvint
+  icved = icvint
 elseif (nbaste.gt.0.and.nbstru.eq.0) then
-   icved = icvext
+  icved = icvext
 endif
 
-if (iwarni(iuma).ge.2) write(nfecra,1000) italim, delta
+if (vcopt%iwarni.ge.2) write(nfecra,1000) italim, delta
 
-!     si convergence
+! if converged
 if (icved.eq.1) then
   if (itrfin.eq.1) then
-!       si ITRFIN=1 on sort
-    if (iwarni(iuma).ge.1) write(nfecra,1001) italim, delta
+    ! si itrfin=1 on sort
+    if (vcopt%iwarni.ge.1) write(nfecra,1001) italim, delta
     itrfin = -1
   else
-!       sinon on refait une derniere iteration pour SYRTHES/T1D/rayonnement
-!        et on remet ICVED a 0 pour que Code_Aster refasse une iteration aussi
+    ! otherwise one last iteration for SYRTHES/T1D/radiation
+    ! and reset icved to 0 so code_aster also runs an iteration.
     itrfin = 1
     icved = 0
   endif
 elseif (itrfin.eq.0 .and. italim.eq.nalimx-1) then
-!       ce sera la derniere iteration
+  ! this will be the lst iteration
   itrfin = 1
 elseif (italim.eq.nalimx) then
-!       on a forcement ITRFIN=1 et on sort
+  ! we have itrfin=1 and are finished
   if (nalimx.gt.1) write(nfecra,1100) italim, delta
   itrfin = -1
-!       On met ICVED a 1 pour que Code_Aster s'arrete lui aussi
+  ! Set icved to 1 so code_aster also stops
   icved = 1
 endif
 
-!     On renvoie l'indicateur de convergence final a Code_Aster
-call astcv2(ntcast, icved)
-!==========
+! Return the final convergence indicator to code_aster
+if (nbaste.gt.0) then
+  call cs_ast_coupling_send_cvg(icved)
+endif
 
 !===============================================================================
 ! 6. Re set previous values if required
 !===============================================================================
 
-! If NTERUP .GT. 1, values at previous time step have been modified after NAVSTO
+! If nterup .gt. 1, values at previous time step have been modified after navstv
 ! We must then go back to a previous value
 if (itrfin.ne.-1) then
   do ii = 1, nvar
-    call field_get_dim (ivarfl(ii), f_dim, interleaved)
+    call field_get_dim (ivarfl(ii), f_dim)
 
     ! Fields of dimension one
     if (f_dim.eq.1) then
@@ -330,10 +341,10 @@ if (itrfin.ne.-1) then
     endif
   enddo
   do ifac = 1, nfac
-    imasfl(ifac) = flmalf(ifac)
+    imasfl(ifac) = imasfl_pre(ifac)
   enddo
   do ifac = 1, nfabor
-    bmasfl(ifac) = flmalb(ifac)
+    bmasfl(ifac) = bmasfl_pre(ifac)
     coefap(ifac) = cofale(ifac,1)
     coefau(1, ifac) = cofale(ifac,2)
     coefau(2, ifac) = cofale(ifac,3)
@@ -356,35 +367,17 @@ endif
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
  1000 format (                                                          &
- '            ALE IMPLICITE : ITER=',I5,' DERIVE=',E12.5     )
+ '            IMPLICIT ALE: ITER=',I5,' DRIFT=',E12.5     )
  1001 format (                                                          &
- 'CONVERGENCE ALE IMPLICITE : ITER=',I5,' DERIVE=',E12.5     )
+ 'CONVERGENCE IMPLICIT ALE: ITER=',I5,' DRIFT=',E12.5     )
  1100 format (                                                          &
-'@                                                            ',/,&
-'@ @@ ATTENTION : COUPLAGE IMPLICITE ALE                      ',/,&
-'@    =========                                               ',/,&
-'@  Nombre d''iterations maximal ',I10   ,' atteint           ',/,&
-'@  Derive normee :',E12.5                                     ,/,&
-'@                                                            '  )
-
-#else
-
- 1000 format (                                                          &
- '            IMPLICIT ALE: ITER=',I5,' DERIVE=',E12.5     )
- 1001 format (                                                          &
- 'CONVERGENCE IMPLICIT ALE: ITER=',I5,' DERIVE=',E12.5     )
- 1100 format (                                                          &
-'@                                                            ',/,&
-'@ @@ WARNING: IMPLICIT ALE                                   ',/,&
-'@    ========                                                ',/,&
-'@  Maximum number of iterations ',I10   ,' reached           ',/,&
-'@  Normed derive :',E12.5                                     ,/,&
-'@                                                            '  )
-
-#endif
+'@'                                                            ,/,&
+'@ @@ WARNING: IMPLICIT ALE'                                   ,/,&
+'@    ========'                                                ,/,&
+'@  Maximum number of iterations ',I10   ,' reached'           ,/,&
+'@  Normed drift:',E12.5                                     ,/,&
+'@'                                                              )
 
 !----
 ! End
