@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -59,7 +59,6 @@ use pointe
 use albase
 use alstru
 use alaste
-use ihmpre
 use parall
 use period
 use mesh
@@ -88,12 +87,26 @@ integer, allocatable, dimension(:) :: lstfac, idfloc, idnloc
 
 !===============================================================================
 
+!===============================================================================
+! Interfaces
+!===============================================================================
+
+interface
+
+  subroutine cs_ast_coupling_initialize(nalimx, epalim) &
+    bind(C, name='cs_ast_coupling_initialize')
+    use, intrinsic :: iso_c_binding
+    implicit none
+    integer(c_int), value :: nalimx
+    real(kind=c_double), value :: epalim
+  end subroutine cs_ast_coupling_initialize
+
+end interface
 
 !===============================================================================
 ! 1. INITIALISATION
 !===============================================================================
 
-
 do istr = 1, nstrmx
   dtstr(istr) = dt(1)
 enddo
@@ -128,22 +141,18 @@ enddo
 
 ! Internal structures
 
-if (iihmpr.eq.1) then
-
-  call uistr1 &
+call uistr1 &
 ( idfstr, mbstru,          &
   aexxst, bexxst, cfopre,  &
   ihistr,                  &
   xstp, xstreq, xpstr )
 
-endif
-
 call usstr1                                                       &
  ( idfstr ,                                                       &
    aexxst , bexxst , cfopre ,                                     &
    xstp   , xpstr  , xstreq )
 
-! External structures: Code_Saturne / Code_Aster coupling
+! External structures: Code_Saturne / code_aster coupling
 
 call uiaste(idfstr, asddlf)
 call usaste(idfstr)
@@ -219,7 +228,7 @@ if (nbaste.gt.0) then
   nbfast = 0
   nbnast = 0
 
-!       Calcul du nombre de faces et noeuds couples avec Code_Aster
+!       Calcul du nombre de faces et noeuds couples avec code_aster
   do ifac = 1, nfabor
     istr = idfstr(ifac)
     if (istr.lt.0) then
@@ -259,12 +268,10 @@ if (nbaste.gt.0) then
   enddo
   nbnast = indast
 
-  ! Free memory
+  ! Exchange code_aster coupling parameters
+  call cs_ast_coupling_initialize(nalimx, epalim)
 
-!       Recuperation des parametres commun du couplage
-  call astpar(ntmabs, nalimx, epalim, ttpabs, dtref)
-
-!       Envoi des donnees geometriques a Code_Aster
+  ! Send geometric information to code_aster
   call astgeo(nbfast, lstfac, idfloc, idnloc, almax)
 
   ! Free memory
@@ -329,138 +336,6 @@ write(nfecra,3000)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@    L''INDICATEUR D''ECRITURE DES FICHIERS HISTORIQUES DES  ',/,&
-'@      STRUCTURES MOBILES NE PEUT VALOIR QUE 0 OU 1.         ',/,&
-'@    IL VAUT ICI ',I10                                        ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne peut etre execute.                           ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres donnes dans usstru.               ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 2000 format( &
-    /,'MODE COUPLAGE DE STRUCTURES NON ACTIVE  ',/,&
-      '                 NBSTRU = ',I10                         ,/)
- 2001 format( &
-      '               NALIMX INUTILE ET POSITIONNE A 1        ',/)
- 2002 format( &
-    /,'MODE COUPLAGE CODE_ASTER NON ACTIVE     ',/,&
-      '                 NBASTE = ',I10                         ,/)
- 2010 format( &
-    /,'MODE COUPLAGE DE STRUCTURES ACTIVE      ',/,&
-      '                 AVEC NBSTRU = ',I10   ,' STRUCTURE(S) ',/,&
-      '                                                       ',/,&
-      '               COEFFICIENTS DE NEWMARK :               ',/,&
-      '                 ALPNMK = ',E12.4                       ,/,&
-      '                 BETNMK = ',E12.4                       ,/,&
-      '                 GAMNMK = ',E12.4                       ,/,&
-      '                                                       ',/,&
-      '               FICHIERS HISTORIQUES DES STRUCTURES :   ',/,&
-      '                 IHISTR = ',I4,' ( 1 : active)         ',/)
- 2012 format( &
-    /,'MODE COUPLAGE CODE_ASTER ACTIVE         ',/,&
-      '                 AVEC NBASTE = ',I10   ,' STRUCTURE(S) ',/)
- 2020 format( &
-    /,'SCHEMA DE COUPLAGE EXPLICITE ACTIVE     ',/,&
-      '                                                       ',/,&
-      '               COEFFICIENTS DU SCHEMA :                ',/,&
-      '                 AEXXST = ',E12.4                       ,/,&
-      '                 BEXXST = ',E12.4                       ,/,&
-      '                 CFOPRE = ',E12.4                       ,/)
- 2030 format( &
-    /,'SCHEMA DE COUPLAGE IMPLICITE ACTIVE     ',/,&
-      '                                                       ',/,&
-      '               NB DE SOUS-ITERATIONS MAX. : ',I10       ,/,&
-      '               SEUIL DE CONVERGENCE       : ',E12.4     ,/)
-
- 3000 format( &
-'-------------------------------------------------------------',/)
- 4000 format( &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA DEFINITION DES STRUCTURES        ',/,&
-'@                INTERNES                                    ',/,&
-'@                                                            ',/,&
-'@    Le nombre de structures definies est superieur au nombre',/,&
-'@    maximum autorise NSTRMX :                               ',/,&
-'@      Nombre de structures definies      : ',I10             ,/,&
-'@      Nombre de structures autorisees    : ',I10             ,/,&
-'@                                                            ',/,&
-'@    Le calcul ne sera pas  execute.                         ',/,&
-'@                                                            ',/,&
-'@    Reduire le nombre de structure                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 4001 format( &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA DEFINITION DES STRUCTURES MOBILES',/,&
-'@                INTERNES                                    ',/,&
-'@                                                            ',/,&
-'@    Le nombre de structures internes definies est           ',/,&
-'@      different de celui du calcul precedent :              ',/,&
-'@      Nombre de structures calcul precedent : ',I10          ,/,&
-'@      Nombre de structures calcul actuel    : ',I10          ,/,&
-'@                                                            ',/,&
-'@    Le calcul ne sera pas  execute.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier le fichier suite auxiliaire ou la specification',/,&
-'@      des structures dans usstru.                           ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 4002 format( &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA DEFINITION DES STRUCTURES        ',/,&
-'@                              EXTERNES (COUPLAGE CODE_ASTER)',/,&
-'@                                                            ',/,&
-'@    Le nombre de structures definies est superieur au nombre',/,&
-'@    maximum autorise NASTMX :                               ',/,&
-'@      Nombre de structures definies      : ',I10             ,/,&
-'@      Nombre de structures autorisees    : ',I10             ,/,&
-'@                                                            ',/,&
-'@    Le calcul ne sera pas  execute.                         ',/,&
-'@                                                            ',/,&
-'@    Reduire le nombre de structure                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 4003 format( &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A LA DEFINITION DES STRUCTURES MOBILES',/,&
-'@                             EXTERNES  (COUPLAGE CODE_ASTER)',/,&
-'@                                                            ',/,&
-'@    Le nombre de structures externes definies est           ',/,&
-'@      different de celui du calcul precedent :              ',/,&
-'@      Nombre de structures calcul precedent : ',I10          ,/,&
-'@      Nombre de structures calcul actuel    : ',I10          ,/,&
-'@                                                            ',/,&
-'@    Le calcul ne sera pas  execute.                         ',/,&
-'@                                                            ',/,&
-'@    Verifier le fichier suite auxiliaire ou la specification',/,&
-'@      des structures dans usaste.                           ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
  1000 format( &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -588,8 +463,6 @@ write(nfecra,3000)
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
 !----
 ! End
 !----
