@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -25,7 +25,7 @@ subroutine strpre &
 
  ( itrale , italim , ineefl ,                                     &
    impale ,                                                       &
-   flmalf , flmalb , xprale , cofale )
+   xprale , cofale )
 
 !===============================================================================
 ! FONCTION :
@@ -43,8 +43,6 @@ subroutine strpre &
 ! ineefl           ! e  ! <-- ! indicateur de sauvegarde des flux              !
 ! nvar             ! i  ! <-- ! total number of variables                      !
 ! nscal            ! i  ! <-- ! total number of scalars                        !
-! flmalf(nfac)     ! tr ! --> ! sauvegarde du flux de masse faces int          !
-! flmalb(nfabor    ! tr ! --> ! sauvegarde du flux de masse faces brd          !
 ! cofale           ! tr ! --> ! sauvegarde des cl de p et u                    !
 !    (nfabor,8)    !    !     !                                                !
 ! xprale(ncelet    ! tr ! --> ! sauvegarde de la pression, si nterup           !
@@ -81,12 +79,12 @@ implicit none
 
 ! Arguments
 
-integer          itrale , italim , ineefl
+integer :: itrale , italim , ineefl
 
-integer          impale(nnod)
+integer, dimension(:) :: impale
 
-double precision flmalf(nfac), flmalb(nfabor), xprale(ncelet)
-double precision cofale(nfabor,11)
+double precision, pointer, dimension(:) :: xprale
+double precision, pointer, dimension(:,:) :: cofale
 
 ! Local variables
 
@@ -206,15 +204,14 @@ if (nbaste.gt.0) then
   enddo
 
 ! Si ITRALE = 0, on ne fait rien pour l'instant, mais il faudrait
-! prevoir la reception de deplacement initiaux venant de Code_Aster
+! prevoir la reception de deplacement initiaux venant de code_aster
   if (itrale.gt.0) then
 
     ntcast = ntcast + 1
 
     ! Reception des deplacements predits et remplissage de disale
 
-    call astcin(ntcast, disale)
-    !==========
+    call astcin(disale)
 
   endif
 
@@ -233,11 +230,7 @@ if (italim.eq.1) then
     enddo
   enddo
   if (ineefl.eq.1) then
-    do ifac = 1, nfac
-      flmalf(ifac) = imasfl(ifac)
-    enddo
     do ifac = 1, nfabor
-      flmalb(ifac) = bmasfl(ifac)
       cofale(ifac,1)  = coefap(ifac)
       cofale(ifac,2)  = coefau(1, ifac)
       cofale(ifac,3)  = coefau(2, ifac)
