@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -55,7 +55,6 @@ use coincl
 use cpincl
 use ppincl
 use radiat
-use ihmpre
 use mesh
 use post
 use field
@@ -73,8 +72,9 @@ integer          iscal , id, ityloc, itycat, pflag
 integer          ii
 integer          iok
 integer          f_id, idftnp
-integer          ivisph, iest
+integer          iest
 integer          key_buoyant_id, is_buoyant_fld
+integer          kturt, turb_flux_model
 
 double precision gravn2
 
@@ -88,6 +88,7 @@ type(var_cal_opt) :: vcopt
 
 ! Key id for buoyant field (inside the Navier Stokes loop)
 call field_get_key_id("is_buoyant", key_buoyant_id)
+call field_get_key_id('turbulent_flux_model', kturt)
 
 ! Determine itycor now that irccor is known (iturb/itytur known much earlier)
 ! type of rotation/curvature correction for turbulent viscosity models
@@ -103,6 +104,12 @@ pflag = POST_ON_LOCATION + POST_MONITOR
 ! Additional physical properties
 !===============================================================================
 
+if (ivofmt.gt.0) then
+  ! variable density
+  irovar = 1
+  ivivar = 1
+endif
+
 ! CP when variable
 if (icp.ge.0) then
   call add_property_field_1d('specific_heat', 'Specific Heat', icp)
@@ -112,7 +119,7 @@ if (icp.ge.0) then
 endif
 
 ! ALE mesh viscosity
-if (iale.eq.1) then
+if (iale.ge.1) then
   call field_get_key_struct_var_cal_opt(ivarfl(iuma), vcopt)
   idftnp = vcopt%idften
 
@@ -155,7 +162,7 @@ endif
 iok = 0
 
 !  Pression hydrostatique
-if (iphydr.eq.0.or.iphydr.eq.2) then
+if (iphydr.eq.0) then
   icalhy = 0
 else if (iphydr.eq.1.and.icalhy.eq.-1) then
   gravn2 = gx**2+gy**2+gz**2
@@ -166,10 +173,10 @@ else if (iphydr.eq.1.and.icalhy.eq.-1) then
   endif
 endif
 
-!   Schemas en temps
-!       en LES : Ordre 2 ; sinon Ordre 1
-!       (en particulier, ordre 2 impossible en k-eps couple)
-if (ischtp.eq.-999) then
+!   Global time stepping
+!       for LES: 2nd order; 1st order otherwise
+!       (2nd order forbidden for "coupled" k-epsilon)
+if (ischtp.eq.-1) then
   if (itytur.eq.4) then
     ischtp = 2
   else
@@ -225,30 +232,16 @@ do iscal = 1, nscal
     endif
   endif
 
-  ! Model for turbulent fluxes u'T' (SGDH, GGDH, AFM, DFM)
-  ityturt(iscal) = iturt(iscal)/10
+  call field_get_key_int(ivarfl(isca(iscal)), kturt, turb_flux_model)
 
   if (iscal.eq.iscalt) then
-    if (iturt(iscalt).gt.0.and.irovar.eq.1) then
+    if (turb_flux_model.gt.0.and.irovar.eq.1) then
       call add_property_field_1d('thermal_expansion', 'Beta', ibeta)
     endif
   endif
 
 enddo
 
-! Pression hydrostatique
-if (iphydr.ne.0.and.iphydr.ne.1.and.iphydr.ne.2) then
-  write(nfecra,8121) 'IPHYDR ',iphydr
-  iok = iok + 1
-endif
-
-! Viscosite secondaire
-ivisph = ivisse
-if (ivisph.ne.0.and.ivisph.ne.1) then
-  write(nfecra,8022) 'IVISSE ',ivisph
-  iok = iok + 1
-endif
-
 ! Schemas en temps
 
 !     Schema en temps global.
@@ -315,6 +308,15 @@ if (iok.gt.0) then
   call csexit(1)
 endif
 
+! add thermal expansion field for Boussinesq approximation
+! if not already added
+if (idilat.eq.0) then
+  call field_get_id_try('thermal_expansion', ibeta)
+  if (ibeta.lt.0) then
+    call add_property_field_1d('thermal_expansion', 'Beta', ibeta)
+  endif
+endif
+
 ! Source term for weakly compressible algorithm (semi analytic scheme)
 if (idilat.ge.4) then
   do iscal = 1, nscal
@@ -338,7 +340,7 @@ endif
 !  de vide pour l'algo. VOF.
 if (isno2t.gt.0) then
   call add_source_term_prev_field(ivarfl(iu))
-  if (ivofmt.ge.0) then
+  if (ivofmt.gt.0) then
     call add_source_term_prev_field(ivarfl(ivolf2))
   endif
 endif
@@ -404,6 +406,15 @@ if (iporos.ge.1) then
   call field_create(f_name, itycat, ityloc, 1, .false., ipori)
   call field_set_key_int(ipori, keylog, 1)
   call field_set_key_int(ipori, keyvis, pflag)
+
+  f_name = 'cell_f_vol'
+  call field_create(f_name,&
+                    itycat,&
+                    1,& ! location: cell
+                    1,& ! dimension
+                    .false.,&
+                    f_id)
+
   if (iporos.eq.2) then
     f_name = 'tensorial_porosity'
     call field_create(f_name, itycat, ityloc, 6, .false., iporf)
@@ -440,8 +451,56 @@ if (iporos.ge.1) then
                       1,& ! dimension
                       .false.,&
                       f_id)
+    f_name = 'i_f_face_normal'
+    call field_create(f_name,&
+                      itycat,&
+                      2,& ! location: inner faces
+                      3,& ! dimension
+                      .false.,&
+                      f_id)
+
+    f_name = 'i_f_face_surf'
+    call field_create(f_name,&
+                      itycat,&
+                      2,& ! location: inner faces
+                      1,& ! dimension
+                      .false.,&
+                      f_id)
+
+    f_name = 'b_f_face_normal'
+    call field_create(f_name,&
+                      itycat,&
+                      3,& ! location: boundary faces
+                      3,& ! dimension
+                      .false.,&
+                      f_id)
+
+    f_name = 'b_f_face_surf'
+    call field_create(f_name,&
+                      itycat,&
+                      3,& ! location: boundary faces
+                      1,& ! dimension
+                      .false.,&
+                      f_id)
+
+    f_name = 'i_f_face_factor'
+    call field_create(f_name,&
+                      itycat,&
+                      2,& ! location: inner faces
+                      2,& ! dimension: 2 per face
+                      .false.,&
+                      f_id)
+
+    f_name = 'b_f_face_factor'
+    call field_create(f_name,&
+                      itycat,&
+                      3,& ! location: boundary faces
+                      1,& ! dimension
+                      .false.,&
+                      f_id)
 
   endif
+
 endif
 
 !===============================================================================
@@ -513,240 +572,6 @@ return
 ! Formats
 !===============================================================================
 
-#if defined(_CS_LANG_FR)
-
- 8022 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@    ',A6,' DOIT ETRE UN ENTIER EGAL A 0 OU 1                ',/,&
-'@    IL VAUT ICI ',I10                                        ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne peut etre execute.                           ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8101 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@    ',A6,' DOIT ETRE UN ENTIER EGAL A 1 ou 2                ',/,&
-'@    IL VAUT ICI ',I10                                        ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne peut etre execute.                           ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8111 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES'               ,/,&
-'@    =========                                               ',/,&
-'@    AVEC UN SCHEMA EN TEMPS D ORDRE 2 : ISCHTP = ', I10      ,/,&
-'@    IL FAUT UTILISER UN PAS DE TEMPS CONSTANT ET UNIFORME   ',/,&
-'@    OR IDTVAR = ',I10                                        ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8112 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES'               ,/,&
-'@    =========                                               ',/,&
-'@    ON IMPOSE UN SCHEMA EN TEMPS D ORDRE 2 (ISCHTP = ',I10   ,/,&
-'@    EN K-EPSILON (ITURB = ',I10,' )'                         ,/,&
-'@                                                            ',/,&
-'@   La version courante ne supporte pas l ordre 2 avec le    ',/,&
-'@   couplage des termes sources du k-epsilon.                ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Modifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8113 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION :       A L''ENTREE DES DONNEES'               ,/,&
-'@    =========                                               ',/,&
-'@    ON IMPOSE UN SCHEMA EN TEMPS D ORDRE 1 (ISCHTP = ',I10   ,/,&
-'@    EN LES (ITURB = ',I10,' )'                               ,/,&
-'@                                                            ',/,&
-'@  Le calcul sera execute.                                   ',/,&
-'@                                                            ',/,&
-'@  Il est conseille de verifier les parametres.              ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8114 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES'               ,/,&
-'@    =========                                               ',/,&
-'@    ON IMPOSE UN SCHEMA EN TEMPS D ORDRE 2 (ISCHTP = ',I10   ,/,&
-'@    EN PHI_FBAR (ITURB = ',I10,' )'                          ,/,&
-'@                                                            ',/,&
-'@   La version courante ne supporte pas l''ordre 2 avec le   ',/,&
-'@   couplage des termes sources du k-epsilon.                ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Modifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8117 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@    ON IMPOSE UN SCHEMA EN TEMPS D ORDRE 2 (ISCHTP = ',I10   ,/,&
-'@    EN BL-V2/K  (ITURB = ',I10,' )'                         ,/,&
-'@                                                            ',/,&
-'@   La version courante ne supporte pas l''ordre 2 avec le   ',/,&
-'@   couplage des termes sources du k-epsilon.                ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Modifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8115 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES'               ,/,&
-'@    =========                                               ',/,&
-'@    ON IMPOSE UN SCHEMA EN TEMPS D ORDRE 2 (ISCHTP = ',I10   ,/,&
-'@    EN K-OMEGA   (ITURB = ',I10,' )'                         ,/,&
-'@                                                            ',/,&
-'@   La version courante ne supporte pas l''ordre 2 avec le   ',/,&
-'@   couplage des termes sources du k-omega.                  ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Modifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8116 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES'               ,/,&
-'@    =========                                               ',/,&
-'@    ON IMPOSE UN SCHEMA EN TEMPS D ORDRE 2 (ISCHTP = ',I10   ,/,&
-'@    EN SPALART   (ITURB = ',I10,' )'                         ,/,&
-'@                                                            ',/,&
-'@   La version courante ne supporte pas l''ordre 2 avec le   ',/,&
-'@   couplage des termes sources de Spalart-Allmaras.         ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Modifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8121 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES'               ,/,&
-'@    =========                                               ',/,&
-'@    ',A6,' DOIT ETRE UN ENTIER EGAL A  0, 1 OU 2            ',/,&
-'@    IL VAUT ICI ',I10                                        ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne peut etre execute.                           ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8131 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES'               ,/,&
-'@    =========                                               ',/,&
-'@    ',A6,' DOIT ETRE UN ENTIER EGAL A 0, 1 OU 2             ',/,&
-'@    IL VAUT ICI ',I10                                        ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne peut etre execute.                           ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8132 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES'               ,/,&
-'@    =========                                               ',/,&
-'@                                                            ',/,&
-'@  Avec le modele de turbulence choisi, ITURB = ',I10         ,/,&
-'@    la valeur de ISTO2T (extrapolation des termes sources   ',/,&
-'@    pour les variables turbulentes) ne doit pas etre modifie',/,&
-'@    or ISTO2T a ete force a ',I10                            ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 8141 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES SCALAIRE ',I10 ,/,&
-'@    =========                                               ',/,&
-'@    ',A6,' DOIT ETRE UN ENTIER EGAL A 0, 1 OU 2             ',/,&
-'@    IL VAUT ICI ',I10                                        ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne peut etre execute.                           ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
- 8022 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ WARNING   : STOP AT THE INITIAL DATA VERIFICATION       ',/,&
-'@    =========                                               ',/,&
-'@    ',A6,' MUST BE AN INTEGER EQUAL TO 0 OR 1               ',/,&
-'@    HERE IT IS  ',I10                                        ,/,&
-'@                                                            ',/,&
-'@  The calculation cannot be executed                        ',/,&
-'@                                                            ',/,&
-'@  Verify   parameters.                                      ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
  8101 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -947,8 +772,6 @@ return
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-#endif
-
 !===============================================================================
 ! 5. End
 !===============================================================================
@@ -1055,21 +878,19 @@ integer, intent(in) :: f_id
 character(len=64) :: f_name
 
 integer :: type_flag, location_id, st_id, f_dim
-logical :: has_previous
 
 !===============================================================================
 
 type_flag = FIELD_EXTENSIVE + FIELD_PROPERTY
 location_id = 1 ! variables defined on cells
-has_previous = .false.
 
 ! Define asscociated field
 
 call field_get_dim(f_id, f_dim)
 call field_get_name (f_id, f_name)
 
-call field_create(trim(f_name)//'_st', type_flag,               &
-                  location_id, f_dim, has_previous, st_id)
+call field_find_or_create(trim(f_name)//'_st', type_flag,       &
+                            location_id, f_dim, st_id)
 
 call field_set_key_int(f_id, kst, st_id)
 
