@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -52,26 +52,11 @@
 !>                                 (input mass flux blocked to 0)
 !>                               - 13 Dirichlet for the advection operator and
 !>                                    Neumann for the diffusion operator
-!> \param[in,out] rcodcl        boundary condition values:
-!>                               - rcodcl(1) value of the dirichlet
-!>                               - rcodcl(2) value of the exterior exchange
-!>                                 coefficient (infinite if no exchange)
-!>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughness
-!>                                 in m if icodcl=6
-!>                                 -# for the velocity \f$ (\mu+\mu_T)
-!>                                    \gradv \vect{u} \cdot \vect{n}  \f$
-!>                                 -# for the pressure \f$ \Delta t
-!>                                    \grad P \cdot \vect{n}  \f$
-!>                                 -# for a scalar \f$ cp \left( K +
-!>                                     \dfrac{K_T}{\sigma_T} \right)
-!>                                     \grad T \cdot \vect{n} \f$
 !_______________________________________________________________________________
 
 subroutine vericl &
  ( nvar   , nscal  ,                                              &
-   itypfb , icodcl ,                                              &
-   rcodcl )
+   itypfb , icodcl )
 
 !===============================================================================
 
@@ -100,26 +85,25 @@ implicit none
 
 ! Arguments
 
-integer          nvar   , nscal
+integer          nvar, nscal
 
 integer          itypfb(nfabor)
 integer          icodcl(nfabor,nvar)
 
-double precision rcodcl(nfabor,nvar,3)
-
 ! Local variables
 
 character(len=80) :: chaine
 
 double precision grav2
-integer          ifac, ivar, icode, f_dim
-integer          nstoni        , nstvit, nstopp
+integer          ifac, ivar, icode, f_dim, f_type
+integer          nstoni, nstvit, nstopp
 integer          nstoke, nstosc, nstovf
 integer          nstuvw, nstoup, nstuke
 integer          nstrij, nsurij, nstov2
 integer          nstuv2, nstokw, nstukw
 integer          nstunu, nstonu
 integer          nstusc
+integer          f_id_rough, f_id_t_rough
 integer          iis, icodcu, icodcv, icodcw, icodck, icodce
 integer          icodcn
 integer          icodcp, icodcf, icodca, icodom
@@ -128,10 +112,13 @@ integer          ipp, iokcod, iok
 integer          icodni(2), icodvi(3), icodpp(2), icodtb(8), icodsc(2)
 integer          icodvf(2), icoduv(3), icodct(11), icodus(4)
 
+double precision, dimension(:), pointer :: bpro_roughness
+double precision, dimension(:), pointer :: bpro_roughness_t
+
 !===============================================================================
 
 !===============================================================================
-! 1.  INITIALISATIONS
+! 1. Initialisations
 !===============================================================================
 
 ! Initialize variables to avoid compiler warnings
@@ -156,6 +143,10 @@ do ipp = 1, 4
   icodus(ipp) = -1
 enddo
 
+call field_get_id_try("boundary_roughness", f_id_rough)
+if (f_id_rough.ge.0) call field_get_val_s(f_id_rough, bpro_roughness)
+call field_get_id_try("boundary_thermal_roughness", f_id_t_rough)
+if (f_id_t_rough.ge.0) call field_get_val_s(f_id_t_rough, bpro_roughness_t)
 
 !===============================================================================
 ! 2.  VERIFICATION DE LA CONSISTANCE DES CL
@@ -170,7 +161,7 @@ enddo
 !  donne, pour l'instant, les contraintes suivantes :
 
 !   - meme type de c.l pour les 3 composantes de vitesse
-!   - pas de conditions de frottemt sur la pression
+!   - pas de conditions de frottement sur la pression
 !   - coherence entre les c.l vitesses et pression
 !   - coherence entre les c.l vitesses et turbulence
 
@@ -200,28 +191,40 @@ nstonu = 0
 ! --- Premiere boucle rapide
 iokcod = 0
 do ivar = 1, nvar
-  do ifac = 1, nfabor
-    icode = icodcl(ifac,ivar)
-    if(icode.eq. 0) then
-      iokcod = 1
-    endif
-  enddo
+  call field_get_type(ivarfl(ivar), f_type)
+  ! Is this field not managed by CDO?
+  if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
+
+    do ifac = 1, nfabor
+      icode = icodcl(ifac,ivar)
+      if(icode.eq. 0) then
+        iokcod = 1
+      endif
+    enddo
+
+  endif ! CDO ?
 enddo
 
 ! --- Seconde boucle lente si pb plus haut
 if (iokcod.ne.0) then
   do ivar = 1, nvar
-    do ifac = 1, nfabor
-      icode = icodcl(ifac,ivar)
-      if(icode.eq. 0) then
-        if (itypfb(ifac).gt.0) then
-          itypfb(ifac) = -itypfb(ifac)
+    call field_get_type(ivarfl(ivar), f_type)
+    ! Is this field not managed by CDO?
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
+
+      do ifac = 1, nfabor
+        icode = icodcl(ifac,ivar)
+        if(icode.eq. 0) then
+          if (itypfb(ifac).gt.0) then
+            itypfb(ifac) = -itypfb(ifac)
+          endif
+          icodni(1) = ivar
+          icodni(2) = ifac
+          nstoni = nstoni + 1
         endif
-        icodni(1) = ivar
-        icodni(2) = ifac
-        nstoni = nstoni + 1
-      endif
-    enddo
+      enddo
+
+    endif ! CDO ?
   enddo
 endif
 
@@ -273,23 +276,24 @@ do ifac = 1, nfabor
     nstvit = nstvit + 1
   endif
 
-  ! --- verification que la rugosite est initialisee si icodl=6
-  if(icodcu.eq.6 .and. rcodcl(ifac,iu,3).lt.epzero)then
-    if (itypfb(ifac).gt.0) then
-      itypfb(ifac) = -itypfb(ifac)
-    endif
-    icodvi(1) = -6
-    icodvi(2) = icodcl(ifac,iw)
-    icodvi(3) = -1
-    nstvit = nstvit + 1
-  endif
+  ! --- Check roughness if rough wall function
+  if (icodcu.eq.6) then
 
-  ! --- on interdit les parois rugueuses en compressible
-  if (icodcu.eq.6 .and. ippmod(icompf).gt.0) then
-    icodvi(1) = iu
-    icodvi(2) = icodcl(ifac,iu)
-    icodvi(3) = 1
-    nstvit = nstvit + 1
+    iok = 0
+    if ((f_id_rough.eq.-1)) then
+      iok = 1
+    else if (bpro_roughness(ifac).le.0.d0) then
+      iok = 1
+    endif
+    if (iok .ge. 1) then
+      if (itypfb(ifac).gt.0) then
+        itypfb(ifac) = -itypfb(ifac)
+      endif
+      icodvi(1) = -6
+      icodvi(2) = icodcl(ifac,iw)
+      icodvi(3) = -1
+      nstvit = nstvit + 1
+    endif
   endif
 
 enddo
@@ -299,7 +303,8 @@ do ifac = 1, nfabor
 
   if (icodcl(ifac,ipr).ne.1 .and. icodcl(ifac,ipr).ne.2 .and.     &
       icodcl(ifac,ipr).ne.3 .and. icodcl(ifac,ipr).ne.11.and.     &
-      icodcl(ifac,ipr).ne.13) then
+      icodcl(ifac,ipr).ne.12.and. icodcl(ifac,ipr).ne.13.and.     &
+      icodcl(ifac,ipr).ne.15) then
     if (itypfb(ifac).gt.0) then
       itypfb(ifac) = -itypfb(ifac)
     endif
@@ -617,77 +622,67 @@ elseif (iturb.eq.70) then
 
 endif
 
-! --- Conditions admissibles pour les scalaires
-if(nscal.ge.1) then
+! Admissible conditions for additional transporter variables
+! (scalars or vectors)
+if (nscal.ge.1) then
   do iis = 1,nscal
     ivar = isca(iis)
     call field_get_dim(ivarfl(ivar), f_dim)
-    if (f_dim.eq.1) then
-      do ifac = 1, nfabor
-        if(icodcl(ifac,ivar).ne. 1.and.                             &
-           icodcl(ifac,ivar).ne. 2.and.                             &
-           icodcl(ifac,ivar).ne. 3.and.                             &
-           icodcl(ifac,ivar).ne. 5.and.                             &
-           icodcl(ifac,ivar).ne.12.and.                             &
-           icodcl(ifac,ivar).ne.13.and.                             &
-           icodcl(ifac,ivar).ne. 6 ) then
-          if (itypfb(ifac).gt.0) then
-            itypfb(ifac) = -itypfb(ifac)
-          endif
-          icodsc(1) = ivar
-          icodsc(2) = icodcl(ifac,ivar)
-          nstosc = nstosc + 1
-        endif
-        if(icodcl(ifac,ivar).eq. 5.and.                             &
-           iscavr(iis).gt.0        ) then
-          if (itypfb(ifac).gt.0) then
-            itypfb(ifac) = -itypfb(ifac)
-          endif
-          icodvf(1) = ivar
-          icodvf(2) = icodcl(ifac,ivar)
-          nstovf = nstovf + 1
-        endif
-        if(icodcl(ifac,ivar).eq. 6.and.                             &
-           iscavr(iis).gt.0        ) then
-          if (itypfb(ifac).gt.0) then
-            itypfb(ifac) = -itypfb(ifac)
-          endif
-          icodvf(1) = ivar
-          icodvf(2) = icodcl(ifac,ivar)
-          nstovf = nstovf + 1
-        endif
-  ! --- verification que la rugosite scalaire est initialisee si icodl=6
-        if(icodcl(ifac,ivar).eq.6.and.                              &
-           rcodcl(ifac,iv,3).lt.epzero)then
-          if (itypfb(ifac).gt.0) then
-            itypfb(ifac) = -itypfb(ifac)
-          endif
-          icodsc(1) = -6
-          icodsc(2) = icodcl(ifac,ivar)
-          nstosc = nstosc + 1
-        endif
-      enddo
-    else ! additional vector variables
-      do ifac = 1, nfabor
-        if(icodcl(ifac,ivar).ne. 1.and.                             &
+    do ifac = 1, nfabor
+      if ((icodcl(ifac,ivar).ne. 1.and.                             &
            icodcl(ifac,ivar).ne. 2.and.                             &
            icodcl(ifac,ivar).ne. 3.and.                             &
            icodcl(ifac,ivar).ne. 4.and.                             &
            icodcl(ifac,ivar).ne. 5.and.                             &
+           icodcl(ifac,ivar).ne. 6.and.                             &
            icodcl(ifac,ivar).ne.11.and.                             &
+           icodcl(ifac,ivar).ne.12.and.                             &
            icodcl(ifac,ivar).ne.13.and.                             &
            icodcl(ifac,ivar).ne.14.and.                             &
-           icodcl(ifac,ivar).ne. 6 ) then
-          if (itypfb(ifac).gt.0) then
-            itypfb(ifac) = -itypfb(ifac)
-          endif
-          icodsc(1) = ivar
-          icodsc(2) = icodcl(ifac,ivar)
-          nstosc = nstosc + 1
+           icodcl(ifac,ivar).ne.15)                                 &
+           ! Only for scalars
+           .or. ( icodcl(ifac,ivar).eq.12.and. f_dim.gt.1)          &
+           ! Only for vectors
+           .or. ( icodcl(ifac,ivar).eq. 4.and. f_dim.eq.1)          &
+           .or. ( icodcl(ifac,ivar).eq.11.and. f_dim.eq.1)          &
+           .or. ( icodcl(ifac,ivar).eq.14.and. f_dim.eq.1)) then
+        if (itypfb(ifac).gt.0) then
+          itypfb(ifac) = -itypfb(ifac)
         endif
-  ! --- verification que la rugosite scalaire est initialisee si icodl=6
-        if(icodcl(ifac,ivar).eq.6.and.                              &
-           rcodcl(ifac,iv,3).lt.epzero)then
+        icodsc(1) = ivar
+        icodsc(2) = icodcl(ifac,ivar)
+        nstosc = nstosc + 1
+      endif
+      if((icodcl(ifac,ivar).eq. 5.or.icodcl(ifac,ivar).eq.15).and.                             &
+         iscavr(iis).gt.0        ) then
+        if (itypfb(ifac).gt.0) then
+          itypfb(ifac) = -itypfb(ifac)
+        endif
+        icodvf(1) = ivar
+        icodvf(2) = icodcl(ifac,ivar)
+        nstovf = nstovf + 1
+      endif
+      if(icodcl(ifac,ivar).eq. 6.and.                             &
+         iscavr(iis).gt.0        ) then
+        if (itypfb(ifac).gt.0) then
+          itypfb(ifac) = -itypfb(ifac)
+        endif
+        icodvf(1) = ivar
+        icodvf(2) = icodcl(ifac,ivar)
+        nstovf = nstovf + 1
+      endif
+
+      ! --- Check roughness if rough wall function
+      if(icodcl(ifac,ivar).eq.6)then
+
+        iok = 0
+        if ((f_id_t_rough.eq.-1)) then
+          iok = 1
+        else if (bpro_roughness_t(ifac).le.0.d0) then
+          iok = 1
+        endif
+        if (iok .ge. 1) then
+
           if (itypfb(ifac).gt.0) then
             itypfb(ifac) = -itypfb(ifac)
           endif
@@ -695,26 +690,26 @@ if(nscal.ge.1) then
           icodsc(2) = icodcl(ifac,ivar)
           nstosc = nstosc + 1
         endif
-      enddo
-    endif ! dim
+      endif
+    enddo
   enddo
 endif
 
 ! --- Check if the gravity is non zero in case of free-surface
 iok = 0
-if (iale.eq.1) then
+if (iale.ge.1) then
+  grav2 = gx**2 + gy**2 + gz**2
   do ifac = 1, nfabor
-    grav2 = gx**2 + gy**2 + gz**2
-    if (ialtyb(ifac).eq.ifresf) then
-      if (grav2.le.epzero**2) then
-        write(nfecra,2001)
-        iok = 1
-      endif
+    if (ialtyb(ifac).eq.ifresf.and.grav2.le.epzero**2) then
+      iok = 1
     endif
   enddo
 endif
 
+if (irangp.ge.0) call parcmx(iok)
+
 if (iok.ne.0) then
+  write(nfecra,2001)
   call csexit (1)
   !==========
 endif
@@ -1277,14 +1272,12 @@ if(iok.ne.0) then
   call sync_bc_err(nstvit, 3, icodvi)
   if (nstvit.ne.0) then
     if (icodvi(1) .eq. -6) then
-      chaine = 'rugosity'
+      chaine = 'roughness'
     else
       call field_get_label(ivarfl(icodvi(1)), chaine)
     endif
     if (icodvi(3).eq.-1) then
       write(nfecra,1010) nstvit, chaine, icodvi(2)
-    elseif (icodvi(3).eq.1) then
-      write(nfecra,1015) nstvit, chaine, icodvi(2), ippmod(icompf)
     endif
   endif
 
@@ -1346,7 +1339,7 @@ if(iok.ne.0) then
   call sync_bc_err(nstosc, 2, icodsc)
   if (nstosc.ne.0) then
     if (icodsc(1) .eq. -6) then
-      chaine = 'rugosity'
+      chaine = 'roughness'
     else
       call field_get_label(ivarfl (icodsc(1)), chaine)
     endif
@@ -1463,207 +1456,52 @@ endif
 ! 3.  FORMATS
 !===============================================================================
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@ COND. LIM. NON INITIALISEES                                ',/,&
-'@   Nombre de faces de bord ',i10   ,'; variable ',a16        ,/,&
-'@     icodcl variable derniere face ', i10                    ,/,&
-'@                                                            '  )
- 1010 format(                                                     &
-'@                                                            ',/,&
-'@ COND. LIM. NON PREVUES                                     ',/,&
-'@   Nombre de faces de bord ',i10   ,'; variable ',a16        ,/,&
-'@     icodcl variable derniere face ', i10                    ,/,&
-'@                                                            '  )
- 1015 format(                                                     &
-'@                                                            ',/,&
-'@ CONDITIONS AUX LIMITES DE PAROI RUGUEUSE INCOMPATIBLES     ',/,&
-'@ AVEC LE MODULE COMPRESSIBLE                                ',/,&
-'@   Nombre de faces de bord ',i10   ,'; variable ',a16        ,/,&
-'@     icodcl variable derniere face ', i10                    ,/,&
-'@     ippmod(icompf)', i10                                    ,/,&
-'@                                                            '  )
- 1020 format(                                                     &
-'@                                                            ',/,&
-'@ INCOHERENCE COND. LIM. COMPOSANTES DE LA VITESSE           ',/,&
-'@   Nombre de faces de bord ',i10                             ,/,&
-'@     icodcl derniere face ', 3i10                            ,/,&
-'@                                                            '  )
- 1030 format(                                                     &
-'@                                                            ',/,&
-'@ INCOHERENCE COND. LIM. VITESSE-VARIABLE                    ',/,&
-'@   Nombre de faces de bord ',i10   ,'; variable ',a16        ,/,&
-'@     icodcl variable ', i10                                  ,/,&
-'@     icodcl vitesse  ',3i10                                  ,/,&
-'@                                                            '  )
- 1040 format(                                                     &
-'@                                                            ',/,&
-'@ INCOHERENCE COND. LIM. VITESSE-RIJ-EPSILON                 ',/,&
-'@   Nombre de faces de bord ',i10                             ,/,&
-'@     icodcl derniere face Rij-eps ', 7i5                     ,/,&
-'@     icodcl               vitesse ', 3i5                     ,/,&
-'@                                                            '  )
- 1041 format(                                                     &
-'@                                                            ',/,&
-'@ INCOHERENCE COND. LIM. VITESSE-RIJ-EPSILON EBRSM           ',/,&
-'@   Nombre de faces de bord ',i10                             ,/,&
-'@     icodcl derniere face Rij-eps ', 8i5                     ,/,&
-'@     icodcl               vitesse ', 3i5                     ,/,&
-'@                                                            '  )
- 1050 format(                                                     &
-'@                                                            ',/,&
-'@ INCOHERENCE COND. LIM. VITESSE-SCALAIRE                    ',/,&
-'@   Nombre de faces de bord ',i10   ,'; variable ',a16        ,/,&
-'@     derniere face : '                                       ,/,&
-'@       scalaire numero ',i10                                 ,/,&
-'@       icodcl scalaire ',i10   ,'; icodcl vitesse ',i10      ,/,&
-'@                                                            '  )
- 1901 format(                                                     &
-'@                                                            ',/,&
-'@                                                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET LORS DE LA VERIFICATION DES COND. LIM.',/,&
-'@    =========                                               ',/,&
-'@                                                            ',/,&
-'@         Conditions aux limites non initialisees  : ',I10    ,/,&
-'@         Conditions aux limites non prevues :               ',/,&
-'@             sur les scalaires                    : ',I10    ,/,&
-'@             sur les scalaires representant                 ',/,&
-'@                                    une variance  : ',I10    ,/,&
-'@         Incoherences :                                     ',/,&
-'@             entre vitesse et scalaires           : ',I10    ,/,&
-'@                                                            ',/,&
-'@         Le calcul ne sera pas execute.                     ',/,&
-'@                                                            ',/,&
-'@         Verifier les parametres donnes via l''interface    ',/,&
-'@           ou cs_user_boundary_conditions.                  ',/,&
-'@                                                            ',/)
- 1902 format(                                                     &
-'@                                                            ',/,&
-'@                                                            ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET LORS DE LA VERIFICATION DES COND. LIM.',/,&
-'@    =========                                               ',/,&
-'@                                                            ',/,&
-'@         Conditions aux limites non prevues :               ',/,&
-'@             sur la vitesse                       : ',I10    ,/,&
-'@             sur la pression                      : ',I10    ,/,&
-'@             sur k et epsilon                     : ',I10    ,/,&
-'@             sur Rij et epsilon                   : ',I10    ,/,&
-'@             sur k, epsilon, phi et f_barre       : ',I10    ,/,&
-'@             sur nu de Spalart Allmaras           : ',I10    ,/,&
-'@         Incoherences :                                     ',/,&
-'@             entre les composantes de la vitesse  : ',I10    ,/,&
-'@             entre vitesse et pression            : ',I10    ,/,&
-'@             entre vitesse et k-epsilon           : ',I10    ,/,&
-'@             entre vitesse et Rij-epsilon         : ',I10    ,/,&
-'@             entre vitesse et v2f                 : ',I10    ,/,&
-'@             entre vitesse et nu                  : ',I10    ,/,&
-'@                                                            ',/,&
-'@         Le calcul ne sera pas execute.                     ',/,&
-'@                                                            ',/,&
-'@         Verifier les parametres donnes via l''interface    ',/,&
-'@           ou cs_user_boundary_conditions.                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 2000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET LORS DE LA VERIFICATION DES COND. LIM.',/,&
-'@    =========                                               ',/,&
-'@                CONDITION DE PAROI RUGUEUSE CHOISIE         ',/,&
-'@   CONDITION LIMITE INCOMPATIBLE AVEC LE MODELE EBRSM       ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute                             ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres donnes via l''interface ou        ',/,&
-'@    cs_user_boundary_conditions :                           ',/,&
-'@      modifier la condition de paroi rugueuse en            ',/,&
-'@               paroi lisse                                  ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 2001 format( &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET LORS DE LA VERIFICATION DES COND. LIM.',/,&
-'@    =========                                               ',/,&
-'@                CONDITION DE SURFACE LIBRE EN ALE.          ',/,&
-'@   CONDITION LIMITE INCOMPATIBLE SANS GRAVITE               ',/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute                             ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres.                                  ',/,&
-'@      modifier la gravite                                   ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
  1000 format(                                                     &
 '@                                                            ',/,&
 '@ UNINITIALIZED BOUNDARY CONDITIONS                          ',/,&
-'@   Number of boundary faces ',i10   ,'; variable ',a16       ,/,&
+'@   Number of boundary faces:',i10   ,'; variable:',a16       ,/,&
 '@     icodcl variable last face ', i10                        ,/,&
 '@                                                            '  )
  1010 format(                                                     &
 '@                                                            ',/,&
 '@ UNEXPECTED BOUNDARY CONDITIONS                             ',/,&
-'@   Number of boundary faces ',i10   ,'; variable ',a16       ,/,&
-'@     icodcl variable last face ', i10                        ,/,&
-'@                                                            '  )
- 1015 format(                                                     &
-'@                                                            ',/,&
-'@ ROUGH WALL BOUNDARY CONDITIONS INCOMPATIBLE WITH THE       ',/,&
-'@ COMPRESSIBLE MODULE                                        ',/,&
-'@   Number of boundary faces ',i10   ,'; variable ',a16       ,/,&
-'@     icodcl variable last face ', i10                        ,/,&
-'@     ippmod(icompf)', i10                                    ,/,&
+'@   Number of boundary faces:',i10   ,'; variable:',a16       ,/,&
+'@     icodcl variable last face:', i10                        ,/,&
 '@                                                            '  )
  1020 format(                                                     &
 '@                                                            ',/,&
 '@ INCOHERENT BOUNDARY CONDITIONS VELOCITY COMPONENT          ',/,&
-'@   Number of boundary faces ',i10                            ,/,&
-'@     icodcl last face ', 3i10                                ,/,&
+'@   Number of boundary faces:',i10                            ,/,&
+'@     icodcl last face:', 3i10                                ,/,&
 '@                                                            '  )
  1030 format(                                                     &
 '@                                                            ',/,&
 '@ INCOHERENCY BOUNDARY CONDITIONS VELOCITY-VARIABLE          ',/,&
-'@   Number of boundary faces ',i10   ,'; variable ',a16       ,/,&
-'@     icodcl variable last face ', i10                        ,/,&
-'@     icodcl velocity ',3i10                                  ,/,&
+'@   Number of boundary faces:',i10   ,'; variable:',a16       ,/,&
+'@     icodcl variable last face:', i10                        ,/,&
+'@     icodcl velocity:',3i10                                  ,/,&
 '@                                                            '  )
  1040 format(                                                     &
 '@                                                            ',/,&
 '@ INCOHERENT BOUNDARY CONDITIONS VELOCITY-RIJ-EPSILON        ',/,&
-'@   Number of boundary faces ',i10                            ,/,&
-'@     icodcl last face Rij-eps  ', 7i5                        ,/,&
-'@     icodcl           velocity ', 3i5                        ,/,&
+'@   Number of boundary faces:',i10                            ,/,&
+'@     icodcl last face Rij-eps: ', 7i5                        ,/,&
+'@     icodcl           velocity:', 3i5                        ,/,&
 '@                                                            '  )
  1041 format(                                                     &
 '@                                                            ',/,&
 '@ INCOHERENT BOUNDARY CONDITIONS VELOCITY-RIJ-EPSILON EBRSM  ',/,&
-'@   Number of boundary faces ',i10                            ,/,&
-'@     icodcl last face Rij-eps  ', 8i5                        ,/,&
-'@     icodcl           velocity ', 3i5                        ,/,&
+'@   Number of boundary faces:',i10                            ,/,&
+'@     icodcl last face Rij-eps: ', 8i5                        ,/,&
+'@     icodcl           velocity:', 3i5                        ,/,&
 '@                                                            '  )
  1050 format(                                                     &
 '@                                                            ',/,&
 '@ INCOHERENT BOUNDARY CONDITIONS VELOCITY-SCALAR             ',/,&
-'@   Number of boundary faces ',i10   ,'; variable ',a16       ,/,&
+'@   Number of boundary faces:',i10   ,'; variable:',a16       ,/,&
 '@     last face:'                                             ,/,&
-'@       scalar number ',i10                                   ,/,&
-'@       icodcl scalar ',i10   ,'; icodcl velocity ',i10       ,/,&
+'@       scalar number:',i10                                   ,/,&
+'@       icodcl scalar:',i10   ,'; icodcl velocity:',i10       ,/,&
 '@                                                            '  )
  1901 format(                                                     &
 '@                                                            ',/,&
@@ -1696,7 +1534,7 @@ endif
 '@ @@ WARNING: ABORT DURING THE BOUNDARY CONDITIONS CHECK     ',/,&
 '@    ========                                                ',/,&
 '@                                                            ',/,&
-'@         Unexpeted boundary conditions:                     ',/,&
+'@         Unexpected boundary conditions:                    ',/,&
 '@             on the velocity                      : ',I10    ,/,&
 '@             on the pressure                      : ',I10    ,/,&
 '@             on k and epsilon                     : ',I10    ,/,&
@@ -1750,9 +1588,6 @@ endif
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
 
-
-#endif
-
 return
 end subroutine vericl
 
