@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -29,7 +29,7 @@
   the correct feature macros are defined first.
 */
 
-#if defined(__linux__) || defined(__blrts__) || defined(__bg__)
+#if defined(__linux__)
 #  define _GNU_SOURCE
 #endif
 
@@ -61,9 +61,26 @@
 
 BEGIN_C_DECLS
 
-/*-------------------------------------------------------------------------------
+/*=============================================================================
  * Additional doxygen documentation
- *-----------------------------------------------------------------------------*/
+ *============================================================================*/
+
+/*!
+  \file bft_mem.c
+        Base memory allocation wrappers with optional tracing.
+
+  The memory managment function provided here provide optional logging,
+  and tracking of non-freed pointers.
+
+  Since in most of the intended applications, failure to allocate memory
+  is considered fatal, failed allocations from these functions are
+  considedered as errors, which are fatal by default but can be handled
+  differently if an appropriate error handler is defined. So additional
+  checking of the return values in the calling code is not needed.
+
+  The functions provided here are otherwise based on the matching C library
+  functions.
+*/
 
 /*-------------------------------------------------------------------------------
  * Local macro documentation
