@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -34,8 +34,9 @@
 #include "cs_cdo_connect.h"
 #include "cs_cdo_local.h"
 #include "cs_cdo_quantities.h"
+#include "cs_field.h"
 #include "cs_mesh_location.h"
-#include "cs_param.h"
+#include "cs_param_types.h"
 #include "cs_property.h"
 #include "cs_xdef.h"
 #include "cs_xdef_eval.h"
@@ -57,8 +58,6 @@ BEGIN_C_DECLS
 
 /*!  1: Perform the computation and post-processing of the Courant number */
 #define CS_ADVECTION_FIELD_POST_COURANT (1 << 0)
-/*!  2: Advection-field is steady-state */
-#define CS_ADVECTION_FIELD_STEADY       (1 << 1)  // steady-state field
 
 /*! @} */
 
@@ -66,50 +65,82 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
-/*! \enum cs_advection_field_key_t
- *  \brief List of available keys for setting an advection field
+typedef cs_flag_t  cs_advection_field_status_t;
+
+/*! \enum cs_advection_field_status_bit_t
+ *  \brief Bit values for specifying the definition/behavior of an advection
+ *         field
  *
- * \var CS_ADVKEY_DEFINE_AT_VERTICES
- * Define a field structure related to the advection field at vertices
- *
- * \var CS_ADVKEY_POST_COURANT
- * Perform the computation (and post-process) of the Courant number
- *
- * \var CS_ADVKEY_STATE_STEADY
- * Advection field is steady-state
- */
-
-typedef enum {
-
-  CS_ADVKEY_DEFINE_AT_VERTICES,
-  CS_ADVKEY_POST_COURANT,
-  CS_ADVKEY_STATE_STEADY,
-  CS_ADVKEY_N_KEYS
-
-} cs_advection_field_key_t;
-
-/*! \enum cs_advection_field_type_t
- *  \brief Type of advection field
+ * @name Category of advection field
+ * @{
  *
  * \var CS_ADVECTION_FIELD_NAVSTO
- * Advection field stemming from the velocity in the (Navier-)Stokes system
+ *      Advection field stemming from the velocity in the (Navier-)Stokes system
  *
  * \var CS_ADVECTION_FIELD_GWF
- * Advection field stemming from the "GroundWater Flows" module. This is the
- * Darcean flux.
+ *      Advection field stemming from the "GroundWater Flows" module. This is the
+ *      Darcean flux.
  *
  * \var CS_ADVECTION_FIELD_USER
- * User-defined advection field.
+ *      User-defined advection field.
+ *
+ * @}
+ * @name Type of definition
+ * @{
+ *
+ * \var CS_ADVECTION_FIELD_TYPE_VELOCITY_VECTOR
+ *      The advection field stands for a velocity.
+ *      This is described by a vector-valued array or function.
+ *
+ * \var CS_ADVECTION_FIELD_TYPE_SCALAR_FLUX
+ *      The advection field stands for a flux.
+ *      This is described by a scalar-valued array or function.
+ *
+ * @}
+ * @name Optional bits
+ * @{
+ *
+ * \var CS_ADVECTION_FIELD_STEADY
+ *      Advection field is steady-state
+ *
+ * \var CS_ADVECTION_FIELD_LEGACY_FV
+ *      Advection field shared with the legacy Finite Volume solver
+ *
+ * \var CS_ADVECTION_FIELD_DEFINE_AT_VERTICES
+ *      Define a field structure related to the advection field at vertices
+ *      Post-processing and log operations are automatically activated.
+ *
+ * \var CS_ADVECTION_FIELD_DEFINE_AT_BOUNDARY_FACES
+ *      Define a field structure related to the advection field at boundary
+ *      faces Post-processing and log operations are automatically activated.
+ *
+ * @}
  */
 
 typedef enum {
 
-  CS_ADVECTION_FIELD_NAVSTO,
-  CS_ADVECTION_FIELD_GWF,
-  CS_ADVECTION_FIELD_USER,
-  CS_N_ADVECTION_FIELD_TYPES
+  /* Category of advection field */
+  /* --------------------------- */
 
-} cs_advection_field_type_t;
+  CS_ADVECTION_FIELD_NAVSTO                            = 1<<0, /* =   1 */
+  CS_ADVECTION_FIELD_GWF                               = 1<<1, /* =   2 */
+  CS_ADVECTION_FIELD_USER                              = 1<<2, /* =   4 */
+
+  /* Type of advection field */
+  /* ----------------------- */
+
+  CS_ADVECTION_FIELD_TYPE_VELOCITY_VECTOR              = 1<<3, /* =   8 */
+  CS_ADVECTION_FIELD_TYPE_SCALAR_FLUX                  = 1<<4, /* =  16 */
+
+  /* Optional */
+  /* -------- */
+
+  CS_ADVECTION_FIELD_STEADY                            = 1<<5, /* =  32 */
+  CS_ADVECTION_FIELD_LEGACY_FV                         = 1<<6, /* =  64 */
+  CS_ADVECTION_FIELD_DEFINE_AT_VERTICES                = 1<<7, /* = 128 */
+  CS_ADVECTION_FIELD_DEFINE_AT_BOUNDARY_FACES          = 1<<8  /* = 256 */
+
+} cs_advection_field_status_bit_t;
 
 /*! \struct cs_advection_field_t
  *  \brief Main structure to handle an advection field
@@ -124,10 +155,11 @@ typedef struct {
    * \var name
    * name of the advection field
    *
-   * \var type
-   * type of advection field
+   * \var status
+   * category (user, navsto, gwf...) and type (velocity, flux...) of the
+   * advection field
    *
-   * \var flag
+   * \var post_flag
    * short descriptor dedicated to postprocessing
    *
    * \var vtx_field_id
@@ -142,35 +174,42 @@ typedef struct {
    * the value of the normal flux across boundary faces. It's always
    * defined since it's used for dealing with the boundary conditions.
    *
+   * \var int_field_id
+   * id to retrieve the related cs_field_t structure corresponding to
+   * the value of the normal flux across interior faces. It's always
+   * defined when the advection field arise from the resolution of the
+   * Navier-Stokes system with the legacy FV solver.
+   *
    * \var definition
    * pointer to the generic structure used to define the advection field. We
-   * assume that only one definition is available (i.e. there is not several
-   * zones).
+   * assume that only one definition is available (i.e. there is a unique
+   * zone related to "cells").
    *
    * \var n_bdy_flux_defs
    * Number of definitions related to the normal flux at the boundary
    *
    * \var bdy_flux_defs
-   * Array of pointers to the definitions of the jormal flux at the boundary
+   * Array of pointers to the definitions of the normal flux at the boundary
    */
 
-  int                         id;
-  char              *restrict name;
-  cs_advection_field_type_t   type;
+  int                           id;
+  char                *restrict name;
+  cs_advection_field_status_t   status;
+  cs_flag_t                     post_flag;
 
-  cs_flag_t                   flag;
-  int                         vtx_field_id;
-  int                         cell_field_id;
-  int                         bdy_field_id;
+  int                           vtx_field_id;
+  int                           cell_field_id;
+  int                           bdy_field_id;
+  int                           int_field_id;
 
   /* We assume that there is only one definition associated to an advection
      field inside the computational domain */
-  cs_xdef_t                  *definition;
+  cs_xdef_t                    *definition;
 
   /* Optional: Definition(s) for the boundary flux */
-  int                         n_bdy_flux_defs;
-  cs_xdef_t                 **bdy_flux_defs;
-  short int                  *bdy_def_ids;
+  int                           n_bdy_flux_defs;
+  cs_xdef_t                   **bdy_flux_defs;
+  short int                    *bdy_def_ids;
 
 } cs_adv_field_t;
 
@@ -182,6 +221,113 @@ typedef struct {
  * Inline public function prototypes
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set a new status for the given advection field structure
+ *
+ * \param[in, out] adv      pointer to an advection field structure
+ * \param[in]      status   status flag to add to the current status
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_advection_field_set_status(cs_adv_field_t               *adv,
+                              cs_advection_field_status_t   status)
+{
+  if (adv == NULL)
+    return;
+
+  adv->status = status;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  returns true if the advection field is uniform, otherwise false
+ *
+ * \param[in]    adv    pointer to an advection field to test
+ *
+ * \return  true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline bool
+cs_advection_field_is_uniform(const cs_adv_field_t   *adv)
+{
+  if (adv == NULL)
+    return false;
+
+  if (adv->definition->state & CS_FLAG_STATE_UNIFORM)
+    return true;
+  else
+    return false;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  returns true if the advection field is uniform in each cell
+ *         otherwise false
+ *
+ * \param[in]    adv    pointer to an advection field to test
+ *
+ * \return  true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline bool
+cs_advection_field_is_cellwise(const cs_adv_field_t   *adv)
+{
+  if (adv == NULL)
+    return false;
+
+  cs_flag_t  state = adv->definition->state;
+
+  if (state & CS_FLAG_STATE_UNIFORM)
+    return true;
+  if (state & CS_FLAG_STATE_CELLWISE)
+    return true;
+  else
+    return false;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve the name of an advection field
+ *
+ * \param[in]    adv    pointer to an advection field structure
+ *
+ * \return  the name of the related advection field
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline const char *
+cs_advection_field_get_name(const cs_adv_field_t   *adv)
+{
+  if (adv == NULL)
+    return NULL;
+
+  return adv->name;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve the type of definition used to set the current advection
+ *         field structure
+ *
+ * \param[in]    adv    pointer to an advection field structure
+ *
+ * \return  the type of definition
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline cs_xdef_type_t
+cs_advection_field_get_deftype(const cs_adv_field_t   *adv)
+{
+  if (adv == NULL)
+    return CS_N_XDEF_TYPES;
+
+  return cs_xdef_get_type(adv->definition);
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Get a cs_field_t structure related to an advection field and a mesh
@@ -203,25 +349,40 @@ cs_advection_field_get_field(const cs_adv_field_t       *adv,
     return f;
 
   switch (ml_type) {
-  case CS_MESH_LOCATION_BOUNDARY_FACES:
-    assert(adv->bdy_field_id > -1);
-    f = cs_field_by_id(adv->bdy_field_id);
-    break;
 
   case CS_MESH_LOCATION_CELLS:
-    assert(adv->cell_field_id > -1);
-    f = cs_field_by_id(adv->cell_field_id);
+    if (adv->cell_field_id > -1)
+      f = cs_field_by_id(adv->cell_field_id);
+    else
+      f = NULL;
+    break;
+
+  case CS_MESH_LOCATION_INTERIOR_FACES:
+    if (adv->int_field_id > -1)
+      f = cs_field_by_id(adv->int_field_id);
+    else
+      f = NULL;
+    break;
+
+  case CS_MESH_LOCATION_BOUNDARY_FACES:
+    if (adv->bdy_field_id > -1)
+      f = cs_field_by_id(adv->bdy_field_id);
+    else
+      f = NULL;
     break;
 
   case CS_MESH_LOCATION_VERTICES:
-    assert(adv->vtx_field_id > -1);
-    f = cs_field_by_id(adv->vtx_field_id);
+    if (adv->vtx_field_id > -1)
+      f = cs_field_by_id(adv->vtx_field_id);
+    else
+      f = NULL;
     break;
 
   default:
     bft_error(__FILE__, __LINE__, 0,
-              " %s: Invalid mesh location type. Stop retrieving the field.\n",
-              __func__);
+              " %s: Invalid mesh location type %d.\n"
+              " Stop retrieving the advection field.\n",
+              __func__, ml_type);
   }
 
   return f;
@@ -301,15 +462,15 @@ cs_advection_field_add_user(const char  *name);
  * \brief  Add and initialize a new advection field structure
  *
  * \param[in]  name        name of the advection field
- * \param[in]  type        type of advection field
+ * \param[in]  status      status of the advection field to create
  *
  * \return a pointer to the new allocated cs_adv_field_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 cs_adv_field_t *
-cs_advection_field_add(const char                  *name,
-                       cs_advection_field_type_t    type);
+cs_advection_field_add(const char                    *name,
+                       cs_advection_field_status_t    status);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -337,60 +498,6 @@ bool
 cs_advection_field_check_name(const cs_adv_field_t   *adv,
                               const char             *ref_name);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  returns true if the advection field is uniform, otherwise false
- *
- * \param[in]    adv    pointer to a property to test
- *
- * \return  true or false
- */
-/*----------------------------------------------------------------------------*/
-
-bool
-cs_advection_field_is_uniform(const cs_adv_field_t   *adv);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  returns true if the advection field is uniform in each cell
- *         otherwise false
- *
- * \param[in]    adv    pointer to a property to test
- *
- * \return  true or false
- */
-/*----------------------------------------------------------------------------*/
-
-bool
-cs_advection_field_is_cellwise(const cs_adv_field_t   *adv);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Retrieve the name of an advection field
- *
- * \param[in]    adv    pointer to an advection field structure
- *
- * \return  the name of the related advection field
- */
-/*----------------------------------------------------------------------------*/
-
-const char *
-cs_advection_field_get_name(const cs_adv_field_t   *adv);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Retrieve the type of definition used to set the current advection
- *         field structure
- *
- * \param[in]    adv    pointer to an advection field structure
- *
- * \return  the type of definition
- */
-/*----------------------------------------------------------------------------*/
-
-cs_xdef_type_t
-cs_advection_field_get_deftype(const cs_adv_field_t   *adv);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Print all setup information related to cs_adv_field_t structures
@@ -402,16 +509,16 @@ cs_advection_field_log_setup(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Set optional parameters related to a cs_adv_field_t structure
+ * \brief  Set optional post-processings
  *
- * \param[in, out]  adv       pointer to a cs_adv_field_t structure
- * \param[in]       key       key related to the member of adv to set
+ * \param[in, out]  adv         pointer to a cs_adv_field_t structure
+ * \param[in]       post_flag   flag to set
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_set_option(cs_adv_field_t            *adv,
-                              cs_advection_field_key_t   key);
+cs_advection_field_set_postprocess(cs_adv_field_t            *adv,
+                                   cs_flag_t                  post_flag);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -441,6 +548,23 @@ cs_advection_field_def_by_analytic(cs_adv_field_t        *adv,
                                    cs_analytic_func_t    *func,
                                    void                  *input);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define a cs_adv_field_t structure using a cs_dof_func_t
+ *
+ * \param[in, out] adv       pointer to a cs_adv_field_t structure
+ * \param[in]      loc_flag  location where values are computed
+ * \param[in]      func      pointer to a cs_dof_func_t function
+ * \param[in]      input     NULL or pointer to a structure cast on-the-fly
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_advection_field_def_by_dof_func(cs_adv_field_t    *adv,
+                                   cs_flag_t          loc_flag,
+                                   cs_dof_func_t     *func,
+                                   void              *input);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Define a cs_adv_field_t structure thanks to an array of values
@@ -448,6 +572,8 @@ cs_advection_field_def_by_analytic(cs_adv_field_t        *adv,
  * \param[in, out]  adv       pointer to a cs_adv_field_t structure
  * \param[in]       loc       information to know where are located values
  * \param[in]       array     pointer to an array
+ * \param[in]       is_owner  transfer the lifecycle to the cs_xdef_t structure
+ *                            (true or false)
  * \param[in]       index     optional pointer to the array index
  */
 /*----------------------------------------------------------------------------*/
@@ -456,11 +582,12 @@ void
 cs_advection_field_def_by_array(cs_adv_field_t    *adv,
                                 cs_flag_t          loc,
                                 cs_real_t         *array,
+                                bool               is_owner,
                                 cs_lnum_t         *index);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Define a cs_adv_field_t structure thanks to an array of values
+ * \brief  Define a cs_adv_field_t structure thanks to a field structure
  *
  * \param[in, out]  adv       pointer to a cs_adv_field_t structure
  * \param[in]       field     pointer to a cs_field_t structure
@@ -514,6 +641,8 @@ cs_advection_field_def_boundary_flux_by_analytic(cs_adv_field_t        *adv,
  * \param[in]       zname     name of the boundary zone to consider
  * \param[in]       loc       information to know where are located values
  * \param[in]       array     pointer to an array
+ * \param[in]       is_owner  transfer the lifecycle to the cs_xdef_t structure
+ *                            (true or false)
  * \param[in]       index     optional pointer to the array index
  */
 /*----------------------------------------------------------------------------*/
@@ -523,8 +652,23 @@ cs_advection_field_def_boundary_flux_by_array(cs_adv_field_t    *adv,
                                               const char        *zname,
                                               cs_flag_t          loc,
                                               cs_real_t         *array,
+                                              bool               is_owner,
                                               cs_lnum_t         *index);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define the value of the boundary normal flux for the given
+ *         cs_adv_field_t structure using a field structure
+ *
+ * \param[in, out]  adv       pointer to a cs_adv_field_t structure
+ * \param[in]       field     pointer to a cs_field_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_advection_field_def_boundary_flux_by_field(cs_adv_field_t    *adv,
+                                              cs_field_t        *field);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Create all needed cs_field_t structures related to an advection
@@ -562,7 +706,28 @@ cs_advection_field_get_cell_vector(cs_lnum_t               c_id,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Compute the mean-value of the advection field inside each cell
+ * \brief  Compute the vector-valued interpolation of the advection field at
+ *         a given location inside a cell
+ *
+ * \param[in]      adv          pointer to a cs_adv_field_t structure
+ * \param[in]      cm           pointer to a cs_cell_mesh_t structure
+ * \param[in]      xyz          location where to perform the evaluation
+ * \param[in]      time_eval    physical time at which one evaluates the term
+ * \param[in, out] eval         pointer to a cs_nvec3_t
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_advection_field_cw_eval_at_xyz(const cs_adv_field_t  *adv,
+                                  const cs_cell_mesh_t  *cm,
+                                  const cs_real_3_t      xyz,
+                                  cs_real_t              time_eval,
+                                  cs_nvec3_t            *eval);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the mean-value of the vector-valued field related to the
+ *         advection field inside each cell
  *
  * \param[in]      adv           pointer to a cs_adv_field_t structure
  * \param[in]      time_eval     physical time at which one evaluates the term
@@ -590,26 +755,6 @@ cs_advection_field_at_vertices(const cs_adv_field_t  *adv,
                                cs_real_t              time_eval,
                                cs_real_t             *vtx_values);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Compute the value of the advection field at a specific location
- *         inside a cell
- *
- * \param[in]      adv          pointer to a cs_adv_field_t structure
- * \param[in]      cm           pointer to a cs_cell_mesh_t structure
- * \param[in]      xyz          location where to perform the evaluation
- * \param[in]      time_eval    physical time at which one evaluates the term
- * \param[in, out] eval         pointer to a cs_nvec3_t
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_advection_field_eval_at_xyz(const cs_adv_field_t  *adv,
-                               const cs_cell_mesh_t  *cm,
-                               const cs_real_3_t      xyz,
-                               cs_real_t              time_eval,
-                               cs_nvec3_t            *eval);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the value of the normal flux of the advection field
@@ -628,8 +773,9 @@ cs_advection_field_across_boundary(const cs_adv_field_t  *adv,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Compute the value of the flux of the advection field across the
- *         the closure of the dual cell related to each vertex
+ * \brief  Compute the value of the normal flux of the advection field across
+ *         the closure of the dual cell related to each vertex belonging to the
+ *         boundary face f
  *
  * \param[in]      cm         pointer to a cs_cell_mesh_t structure
  * \param[in]      adv        pointer to a cs_adv_field_t structure
@@ -640,11 +786,31 @@ cs_advection_field_across_boundary(const cs_adv_field_t  *adv,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_get_f2v_boundary_flux(const cs_cell_mesh_t   *cm,
-                                         const cs_adv_field_t   *adv,
-                                         short int               f,
-                                         cs_real_t               time_eval,
-                                         cs_real_t              *fluxes);
+cs_advection_field_cw_boundary_f2v_flux(const cs_cell_mesh_t   *cm,
+                                        const cs_adv_field_t   *adv,
+                                        short int               f,
+                                        cs_real_t               time_eval,
+                                        cs_real_t              *fluxes);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the value of the normal flux of the advection field across
+ *         a boundary face f (cellwise version)
+ *
+ * \param[in] time_eval   physical time at which one evaluates the term
+ * \param[in] f           face id in the cellwise numbering
+ * \param[in] cm          pointer to a cs_cell_mesh_t structure
+ * \param[in] adv         pointer to a cs_adv_field_t structure
+ *
+ * \return  the normal boundary flux for the face f
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_real_t
+cs_advection_field_cw_boundary_face_flux(const cs_real_t          time_eval,
+                                         const short int          f,
+                                         const cs_cell_mesh_t    *cm,
+                                         const cs_adv_field_t    *adv);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -654,15 +820,15 @@ cs_advection_field_get_f2v_boundary_flux(const cs_cell_mesh_t   *cm,
  * \param[in]      cm         pointer to a cs_cell_mesh_t structure
  * \param[in]      adv        pointer to a cs_adv_field_t structure
  * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in, out] fluxes     array of values attached to dual faces of a cell
+ * \param[in, out] fluxes     array of values attached to primal faces of a cell
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_get_cw_face_flux(const cs_cell_mesh_t       *cm,
-                                    const cs_adv_field_t       *adv,
-                                    cs_real_t                   time_eval,
-                                    cs_real_t                  *fluxes);
+cs_advection_field_cw_face_flux(const cs_cell_mesh_t       *cm,
+                                const cs_adv_field_t       *adv,
+                                cs_real_t                   time_eval,
+                                cs_real_t                  *fluxes);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -677,10 +843,10 @@ cs_advection_field_get_cw_face_flux(const cs_cell_mesh_t       *cm,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_advection_field_get_cw_dface_flux(const cs_cell_mesh_t     *cm,
-                                     const cs_adv_field_t     *adv,
-                                     cs_real_t                 time_eval,
-                                     cs_real_t                *fluxes);
+cs_advection_field_cw_dface_flux(const cs_cell_mesh_t     *cm,
+                                 const cs_adv_field_t     *adv,
+                                 cs_real_t                 time_eval,
+                                 cs_real_t                *fluxes);
 
 /*----------------------------------------------------------------------------*/
 /*!
