@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -89,12 +89,10 @@ static  cs_flag_t  cs_basis_func_hho_cell_flag = 0;
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Compute the x^exp where exp is a positive integer
+ * \brief  Swap two short int
  *
- * \param[in]     x        base
- * \param[in]     exp      exponent
- *
- * \return the result
+ * \param[in, out]     a      first
+ * \param[in, out]     b      second
  */
 /*----------------------------------------------------------------------------*/
 
@@ -316,6 +314,7 @@ _add_contrib(const int           n_gpts,
       } /* wphi_i not zero */
 
     }
+
   }
 }
 
@@ -339,7 +338,7 @@ _symmetrize_and_clean(const int           n_rows,
     const cs_real_t  coef = 1/val_i[i];
     for (short int j = i + 1; j < n_rows; j++) {
       if (fabs(val_i[j]*coef) > _clean_threshold)
-        values[n_rows*j + i] = val_i[j];
+        values[n_rows*j+i] = val_i[j];
       else
         values[n_rows*j+i] = values[i*n_rows + j] = 0.;
     }
@@ -405,9 +404,9 @@ _iner_cell_basis_setup(void                    *pbf,
                        cs_cell_builder_t       *cb)
 {
   cs_basis_func_t  *bf = (cs_basis_func_t *)pbf;
-  assert(cs_flag_test(cm->flag,
-                      CS_CDO_LOCAL_PV | CS_CDO_LOCAL_PFQ | CS_CDO_LOCAL_HFQ |
-                      CS_CDO_LOCAL_FE | CS_CDO_LOCAL_FEQ));
+  assert(cs_eflag_test(cm->flag,
+                       CS_FLAG_COMP_PV | CS_FLAG_COMP_PFQ | CS_FLAG_COMP_HFQ |
+                       CS_FLAG_COMP_FE | CS_FLAG_COMP_FEQ));
 
   /* Advanced parameters for controlling the algorithm */
   const int  n_algo_iters = 20;
@@ -520,8 +519,8 @@ _mono_face_basis_setup(void                    *pbf,
                        cs_cell_builder_t       *cb)
 {
   cs_basis_func_t  *bf = (cs_basis_func_t *)pbf;
-  assert(cs_flag_test(cm->flag, CS_CDO_LOCAL_PV | CS_CDO_LOCAL_PFQ |
-                       CS_CDO_LOCAL_PEQ | CS_CDO_LOCAL_DIAM));
+  assert(cs_eflag_test(cm->flag, CS_FLAG_COMP_PV | CS_FLAG_COMP_PFQ |
+                       CS_FLAG_COMP_PEQ | CS_FLAG_COMP_DIAM));
 
   for (int k = 0; k < 3; k++)
     bf->center[k] = center[k];
@@ -540,7 +539,7 @@ _mono_face_basis_setup(void                    *pbf,
                            &tmp_meas, xexf_vect[idx++]);
 
   cs_real_t min = 1.;
-  short int e1 = 0, e2 = 1; // First selected couple
+  short int e1 = 0, e2 = 1; /* First selected couple */
   const short int n_e = cm->f2e_idx[f+1] - cm->f2e_idx[f];
   for (short int i = 0; i < n_e; i++) {
     for (short int j = i+1; j < n_e; j++) {
@@ -593,8 +592,8 @@ _iner_face_basis_setup(void                    *pbf,
                        cs_cell_builder_t       *cb)
 {
   cs_basis_func_t  *bf = (cs_basis_func_t *)pbf;
-  assert(cs_flag_test(cm->flag, CS_CDO_LOCAL_PV | CS_CDO_LOCAL_PFQ |
-                      CS_CDO_LOCAL_FE | CS_CDO_LOCAL_FEQ));
+  assert(cs_eflag_test(cm->flag, CS_FLAG_COMP_PV | CS_FLAG_COMP_PFQ |
+                       CS_FLAG_COMP_FE | CS_FLAG_COMP_FEQ));
 
   /* Initialization using the monomial basis */
   _mono_face_basis_setup(bf, cm, f, center, cb);
@@ -635,8 +634,8 @@ _iner_face_basis_setup(void                    *pbf,
     c0a0_c1a1[k] =  c0 * bf->axis[0].unitv[k] +
                     c1 * bf->axis[1].unitv[k];
 
-  cs_nvec3(c1a0_c0a1, bf->axis);  // First axis
-  cs_nvec3(c0a0_c1a1, bf->axis + 1);  // Second axis
+  cs_nvec3(c1a0_c0a1, bf->axis);     /* First axis */
+  cs_nvec3(c0a0_c1a1, bf->axis + 1); /* Second axis */
 
   const cs_real_t  odiam = 1. / cm->f_diam[f];
   bf->axis[0].meas = odiam;
@@ -904,9 +903,9 @@ _ck1_compute_projector(void                    *pbf,
 {
   CS_UNUSED(id);
   /* Sanity checks */
-  assert(cs_flag_test(cm->flag,
-                      CS_CDO_LOCAL_PFQ| CS_CDO_LOCAL_HFQ | CS_CDO_LOCAL_FEQ |
-                      CS_CDO_LOCAL_EV | CS_CDO_LOCAL_FE));
+  assert(cs_eflag_test(cm->flag,
+                       CS_FLAG_COMP_PFQ| CS_FLAG_COMP_HFQ | CS_FLAG_COMP_FEQ |
+                       CS_FLAG_COMP_EV | CS_FLAG_COMP_FE));
 
   const int n_rows = CK1_SIZE;
   const int n_gpts = 4;
@@ -957,7 +956,7 @@ _ck1_compute_projector(void                    *pbf,
       const short int n_ef = end - start; /* #vertices (= #edges) */
       const short int *f2e_ids = cm->f2e_ids + start;
       const cs_quant_t  pfq = cm->face[f];
-      const double  hf_coef = cs_math_onethird * cm->hfc[f];
+      const double  hf_coef = cs_math_1ov3 * cm->hfc[f];
 
       switch (n_ef) { /* Optimized version for triangles */
       case CS_TRIANGLE_CASE: /* Triangle */
@@ -980,7 +979,7 @@ _ck1_compute_projector(void                    *pbf,
 
           for (short int e = 0; e < n_ef; e++) { /* Loop on face edges */
 
-            // Edge-related variables
+            /* Edge-related variables */
             const short int e0  = f2e_ids[e];
             const short int v0 = cm->e2v_ids[2*e0];
             const short int v1 = cm->e2v_ids[2*e0+1];
@@ -1077,9 +1076,9 @@ _ck2_compute_projector(void                    *pbf,
 {
   CS_UNUSED(id);
   /* Sanity checks */
-  assert(cs_flag_test(cm->flag,
-                      CS_CDO_LOCAL_PFQ| CS_CDO_LOCAL_HFQ | CS_CDO_LOCAL_FEQ |
-                      CS_CDO_LOCAL_EV | CS_CDO_LOCAL_FE));
+  assert(cs_eflag_test(cm->flag,
+                       CS_FLAG_COMP_PFQ| CS_FLAG_COMP_HFQ | CS_FLAG_COMP_FEQ |
+                       CS_FLAG_COMP_EV | CS_FLAG_COMP_FE));
 
   const int n_rows = CK2_SIZE;
   const int n_gpts = 15;
@@ -1118,7 +1117,7 @@ _ck2_compute_projector(void                    *pbf,
       const short int n_ef = end - start; /* #vertices (= #edges) */
       const short int *f2e_ids = cm->f2e_ids + start;
       const cs_quant_t  pfq = cm->face[f];
-      const double  hf_coef = cs_math_onethird * cm->hfc[f];
+      const double  hf_coef = cs_math_1ov3 * cm->hfc[f];
 
       switch (n_ef) { /* Optimized version for triangles */
       case CS_TRIANGLE_CASE: /* Triangle */
@@ -1141,7 +1140,7 @@ _ck2_compute_projector(void                    *pbf,
 
           for (short int e = 0; e < n_ef; e++) { /* Loop on face edges */
 
-            // Edge-related variables
+            /* Edge-related variables */
             const short int e0  = f2e_ids[e];
             const short int v0 = cm->e2v_ids[2*e0];
             const short int v1 = cm->e2v_ids[2*e0+1];
@@ -1273,9 +1272,9 @@ _cka_compute_projector(void                    *pbf,
 {
   CS_UNUSED(id);
   /* Sanity checks */
-  assert(cs_flag_test(cm->flag,
-                      CS_CDO_LOCAL_PFQ| CS_CDO_LOCAL_HFQ | CS_CDO_LOCAL_FEQ |
-                      CS_CDO_LOCAL_EV | CS_CDO_LOCAL_FE));
+  assert(cs_eflag_test(cm->flag,
+                       CS_FLAG_COMP_PFQ| CS_FLAG_COMP_HFQ | CS_FLAG_COMP_FEQ |
+                       CS_FLAG_COMP_EV | CS_FLAG_COMP_FE));
 
   cs_basis_func_t  *bf = (cs_basis_func_t *)pbf;
 
@@ -1331,7 +1330,7 @@ _cka_compute_projector(void                    *pbf,
       const short int n_ef = end - start; /* #vertices (= #edges) */
       const short int *f2e_ids = cm->f2e_ids + start;
       const cs_quant_t  pfq = cm->face[f];
-      const double  hf_coef = cs_math_onethird * cm->hfc[f];
+      const double  hf_coef = cs_math_1ov3 * cm->hfc[f];
 
       switch (n_ef) { /* Optimized version for triangles */
       case CS_TRIANGLE_CASE: /* Triangle */
@@ -1354,7 +1353,7 @@ _cka_compute_projector(void                    *pbf,
 
           for (short int e = 0; e < n_ef; e++) { /* Loop on face edges */
 
-            // Edge-related variables
+            /* Edge-related variables */
             const short int e0  = f2e_ids[e];
             const short int v0 = cm->e2v_ids[2*e0];
             const short int v1 = cm->e2v_ids[2*e0+1];
@@ -1835,8 +1834,8 @@ _fk1_compute_projector(void                    *pbf,
                        const short int          f)
 {
   /* Sanity checks */
-  assert(cs_flag_test(cm->flag, CS_CDO_LOCAL_PFQ| CS_CDO_LOCAL_FEQ |
-                      CS_CDO_LOCAL_EV | CS_CDO_LOCAL_FE));
+  assert(cs_eflag_test(cm->flag, CS_FLAG_COMP_PFQ | CS_FLAG_COMP_FEQ |
+                       CS_FLAG_COMP_EV | CS_FLAG_COMP_FE));
 
   /* First row (or first column since projector is symmetric) is easy.
      So, 1st row is computed differently */
@@ -1892,7 +1891,7 @@ _fk1_compute_projector(void                    *pbf,
 
       for (short int e = 0; e < n_ef; e++) { /* Loop on face edges */
 
-        // Edge-related variables
+        /* Edge-related variables */
         const short int e0  = f2e_ids[e];
         const short int v0 = cm->e2v_ids[2*e0];
         const short int v1 = cm->e2v_ids[2*e0+1];
@@ -1981,8 +1980,8 @@ _fk2_compute_projector(void                    *pbf,
                        const short int          f)
 {
   /* Sanity checks */
-  assert(cs_flag_test(cm->flag, CS_CDO_LOCAL_PFQ| CS_CDO_LOCAL_FEQ |
-                      CS_CDO_LOCAL_EV | CS_CDO_LOCAL_FE));
+  assert(cs_eflag_test(cm->flag, CS_FLAG_COMP_PFQ | CS_FLAG_COMP_FEQ |
+                       CS_FLAG_COMP_EV | CS_FLAG_COMP_FE));
 
   /* First row (or first column since projector is symmetric) is easy.
      So, 1st row is computed differently */
@@ -2028,7 +2027,7 @@ _fk2_compute_projector(void                    *pbf,
 
       for (short int e = 0; e < n_ef; e++) { /* Loop on face edges */
 
-        // Edge-related variables
+        /* Edge-related variables */
         const short int e0  = f2e_ids[e];
         const short int v0 = cm->e2v_ids[2*e0];
         const short int v1 = cm->e2v_ids[2*e0+1];
@@ -2199,8 +2198,8 @@ _fka_compute_projector(void                    *pbf,
                        const short int          f)
 {
   /* Sanity checks */
-  assert(cs_flag_test(cm->flag, CS_CDO_LOCAL_PFQ| CS_CDO_LOCAL_FEQ |
-                      CS_CDO_LOCAL_EV | CS_CDO_LOCAL_FE));
+  assert(cs_eflag_test(cm->flag, CS_FLAG_COMP_PFQ | CS_FLAG_COMP_FEQ |
+                       CS_FLAG_COMP_EV | CS_FLAG_COMP_FE));
 
   cs_basis_func_t  *bf = (cs_basis_func_t *)pbf;
 
@@ -2261,7 +2260,7 @@ _fka_compute_projector(void                    *pbf,
 
       for (short int e = 0; e < n_ef; e++) { /* Loop on face edges */
 
-        // Edge-related variables
+        /* Edge-related variables */
         const short int e0  = f2e_ids[e];
         const short int v0 = cm->e2v_ids[2*e0];
         const short int v1 = cm->e2v_ids[2*e0+1];
