@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -55,7 +55,10 @@ BEGIN_C_DECLS
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Initialize the computational domain when CDO/HHO schemes are
- *         activated
+ *         activated and cs_user_model() has been called
+ *         At this stage of the settings, mesh quantities and adjacencies are
+ *         not defined. Only the major moddeling options are set. The related
+ *         equations and main properties have been added.
  *
  * \param[in, out]  domain    pointer to a cs_domain_t structure
  */
@@ -92,6 +95,21 @@ cs_cdo_initialize_structures(cs_domain_t           *domain,
 void
 cs_cdo_finalize(cs_domain_t    *domain);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Check if CDO has been initialized.
+ *
+ * \param[in, out]  setup       indicator if setup has been initialized,
+ *                              or NULL if not queried
+ * \param[in, out]  structures  indicator if structures have been initialized,
+ *                              or NULL if not queried
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_cdo_is_initialized(bool  *setup,
+                      bool  *structures);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Main program for running a simulation with the CDO kernel
