@@ -9,7 +9,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -41,12 +41,15 @@
 #include "cs_base.h"
 #include "cs_cdo_connect.h"
 #include "cs_cdo_quantities.h"
+#include "cs_cdofb_priv.h"
+#include "cs_equation_assemble.h"
 #include "cs_equation_common.h"
 #include "cs_equation_param.h"
 #include "cs_field.h"
 #include "cs_matrix.h"
 #include "cs_mesh.h"
 #include "cs_restart.h"
+#include "cs_sles.h"
 #include "cs_source_term.h"
 #include "cs_time_step.h"
 
@@ -69,37 +72,6 @@ typedef struct _cs_cdofb_t cs_cdofb_vecteq_t;
  * Public function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief   Initialize the local structure for the current cell
- *
- * \param[in]      cell_flag   flag related to the current cell
- * \param[in]      cm          pointer to a cellwise view of the mesh
- * \param[in]      eqp         pointer to a cs_equation_param_t structure
- * \param[in]      eqb         pointer to a cs_equation_builder_t structure
- * \param[in]      eqc         pointer to a cs_cdofb_vecteq_t structure
- * \param[in]      dir_values  Dirichlet values associated to each face
- * \param[in]      neu_tags    definition id related to each Neumann face
- * \param[in]      field_tn    values of the field at the last computed time
- * \param[in]      t_eval      time at which one performs the evaluation
- * \param[in, out] csys        pointer to a cellwise view of the system
- * \param[in, out] cb          pointer to a cellwise builder
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_cdofb_vecteq_init_cell_system(const cs_flag_t               cell_flag,
-                                 const cs_cell_mesh_t         *cm,
-                                 const cs_equation_param_t    *eqp,
-                                 const cs_equation_builder_t  *eqb,
-                                 const cs_cdofb_vecteq_t      *eqc,
-                                 const cs_real_t               dir_values[],
-                                 const short int               neu_tags[],
-                                 const cs_real_t               field_tn[],
-                                 cs_real_t                     t_eval,
-                                 cs_cell_sys_t                *csys,
-                                 cs_cell_builder_t            *cb);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief    Check if the generic structures for building a CDO-Fb scheme are
@@ -146,14 +118,14 @@ cs_cdofb_vecteq_matrix_structure(void);
 /*!
  * \brief  Retrieve work buffers used for building a CDO system cellwise
  *
- * \param[out]  csys   pointer to a pointer on a cs_cell_sys_t structure
- * \param[out]  cb     pointer to a pointer on a cs_cell_builder_t structure
+ * \param[out]  csys   double pointer to a \ref cs_cell_sys_t structure
+ * \param[out]  cb     double pointer to a \ref cs_cell_builder_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_vecteq_get(cs_cell_sys_t       **csys,
-                    cs_cell_builder_t   **cb);
+cs_cdofb_vecteq_get(cs_cell_sys_t            **csys,
+                    cs_cell_builder_t        **cb);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -200,112 +172,328 @@ cs_cdofb_vecteq_free_context(void   *data);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Create the matrix of the current algebraic system.
- *         Allocate and initialize the right-hand side associated to the given
- *         data structure
+ * \brief  Set the initial values of the variable field taking into account
+ *         the boundary conditions.
+ *         Case of vector-valued CDO-Fb schemes.
  *
- * \param[in]      eqp            pointer to a cs_equation_param_t structure
- * \param[in, out] eqb            pointer to a cs_equation_builder_t structure
- * \param[in, out] data           pointer to cs_cdofb_vecteq_t structure
- * \param[in, out] system_matrix  pointer of pointer to a cs_matrix_t struct.
- * \param[in, out] system_rhs     pointer of pointer to an array of double
+ * \param[in]      t_eval     time at which one evaluates BCs
+ * \param[in]      field_id   id related to the variable field of this equation
+ * \param[in]      mesh       pointer to a cs_mesh_t structure
+ * \param[in]      eqp        pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
+ * \param[in, out] context    pointer to the scheme context (cast on-the-fly)
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_vecteq_initialize_system(const cs_equation_param_t  *eqp,
-                                  cs_equation_builder_t      *eqb,
-                                  void                       *data,
-                                  cs_matrix_t               **system_matrix,
-                                  cs_real_t                 **system_rhs);
+cs_cdofb_vecteq_init_values(cs_real_t                     t_eval,
+                            const int                     field_id,
+                            const cs_mesh_t              *mesh,
+                            const cs_equation_param_t    *eqp,
+                            cs_equation_builder_t        *eqb,
+                            void                         *context);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Set the boundary conditions known from the settings when the fields
- *         stem from a vector CDO face-based scheme.
+ * \brief   Initialize the local structure for the current cell
+ *          The algebraic system for time t^{n+1} is going to be built knowing
+ *          previous field at time t^{n} and potentially the field at time
+ *          t^{n-1}. Make sure to be consistent between the call to
+ *          current_to_previous and the parameters vel_{f,c}_n/nm1 given
  *
- * \param[in]      mesh        pointer to a cs_mesh_t structure
+ * \param[in]      cm          pointer to a cellwise view of the mesh
  * \param[in]      eqp         pointer to a cs_equation_param_t structure
- * \param[in, out] eqb         pointer to a cs_equation_builder_t structure
- * \param[in]      t_eval      time at which one evaluates BCs
- * \param[in, out] field_val   pointer to the values of the variable field
+ * \param[in]      eqb         pointer to a cs_equation_builder_t structure
+ * \param[in]      dir_values  Dirichlet values associated to each face
+ * \param[in]      forced_ids  indirection in case of internal enforcement
+ * \param[in]      val_f_n     face DoFs at time step n
+ * \param[in]      val_c_n     cell DoFs at time step n
+ * \param[in]      val_f_nm1   face DoFs at time step n-1 or NULL
+ * \param[in]      val_c_nm1   cell DoFs at time step n-1 or NULL
+ * \param[in, out] csys        pointer to a cellwise view of the system
+ * \param[in, out] cb          pointer to a cellwise builder
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_vecteq_set_dir_bc(const cs_mesh_t              *mesh,
-                           const cs_equation_param_t    *eqp,
-                           cs_equation_builder_t        *eqb,
-                           cs_real_t                     t_eval,
-                           cs_real_t                     field_val[]);
+cs_cdofb_vecteq_init_cell_system(const cs_cell_mesh_t         *cm,
+                                 const cs_equation_param_t    *eqp,
+                                 const cs_equation_builder_t  *eqb,
+                                 const cs_real_t               dir_values[],
+                                 const cs_lnum_t               forced_ids[],
+                                 const cs_real_t               val_f_n[],
+                                 const cs_real_t               val_c_n[],
+                                 const cs_real_t               val_f_nm1[],
+                                 const cs_real_t               val_c_nm1[],
+                                 cs_cell_sys_t                *csys,
+                                 cs_cell_builder_t            *cb);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Build the linear system arising from a vector convection/diffusion
- *         equation with a CDO face-based scheme.
+ * \brief  Set the boundary conditions known from the settings
+ *         Define an indirection array for the enforcement of internal DoFs
+ *         only if needed.
+ *
+ * \param[in]      t_eval          time at which one evaluates BCs
+ * \param[in]      mesh            pointer to a cs_mesh_t structure
+ * \param[in]      eqp             pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb             pointer to a cs_equation_builder_t structure
+ * \param[in, out] p_dir_values    pointer to the Dirichlet values to set
+ * \param[in, out] p_enforced_ids  pointer to the list of enforced cells
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_cdofb_vecteq_setup(cs_real_t                     t_eval,
+                      const cs_mesh_t              *mesh,
+                      const cs_equation_param_t    *eqp,
+                      cs_equation_builder_t        *eqb,
+                      cs_real_t                    *p_dir_values[],
+                      cs_lnum_t                    *p_enforced_ids[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Build the local matrices arising from the diffusion term in the
+ *          vector-valued CDO-Fb schemes.
+ *
+ * \param[in]      eqp         pointer to a cs_equation_param_t structure
+ * \param[in]      eqb         pointer to a cs_equation_builder_t structure
+ * \param[in]      eqc         context for this kind of discretization
+ * \param[in]      cm          pointer to a cellwise view of the mesh
+ * \param[in, out] diff_hodge  pointer to a cs_hodge_t structure for diffusion
+ * \param[in, out] csys        pointer to a cellwise view of the system
+ * \param[in, out] cb          pointer to a cellwise builder
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_cdofb_vecteq_diffusion(const cs_equation_param_t     *eqp,
+                          const cs_equation_builder_t   *eqb,
+                          const cs_cdofb_vecteq_t       *eqc,
+                          const cs_cell_mesh_t          *cm,
+                          cs_hodge_t                    *diff_hodge,
+                          cs_cell_sys_t                 *csys,
+                          cs_cell_builder_t             *cb);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Build the local matrices arising from the convection, diffusion,
+ *          reaction terms in vector-valued CDO-Fb schemes.
+ *          mass_hodge could be set to NULL if a Voronoi algo. is used.
+ *          Otherwise, the mass matrix should be pre-computed.
+ *
+ * \param[in]      eqp         pointer to a cs_equation_param_t structure
+ * \param[in]      eqb         pointer to a cs_equation_builder_t structure
+ * \param[in]      eqc         context for this kind of discretization
+ * \param[in]      cm          pointer to a cellwise view of the mesh
+ * \param[in, out] mass_hodge  pointer to a cs_hodge_t structure for reaction
+ * \param[in, out] diff_hodge  pointer to a cs_hodge_t structure for diffusion
+ * \param[in, out] csys        pointer to a cellwise view of the system
+ * \param[in, out] cb          pointer to a cellwise builder
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_cdofb_vecteq_conv_diff_reac(const cs_equation_param_t     *eqp,
+                               const cs_equation_builder_t   *eqb,
+                               const cs_cdofb_vecteq_t       *eqc,
+                               const cs_cell_mesh_t          *cm,
+                               cs_hodge_t                    *mass_hodge,
+                               cs_hodge_t                    *diff_hodge,
+                               cs_cell_sys_t                 *csys,
+                               cs_cell_builder_t             *cb);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the source term for a vector-valued CDO-Fb scheme
+ *         and add it to the local rhs
+ *         Mass matrix is optional. It can be set to NULL.
+ *
+ * \param[in]       cm      pointer to a \ref cs_cell_mesh_t structure
+ * \param[in]       eqp     pointer to a \ref cs_equation_param_t structure
+ * \param[in]       coef    scaling of the time source (for theta schemes)
+ * \param[in]       t_eval  time at which the source term is evaluated
+ * \param[in, out]  hodge   pointer to a \ref cs_hodge_t structure (mass matrix)
+ * \param[in, out]  cb      pointer to a \ref cs_cell_builder_t structure
+ * \param[in, out]  eqb     pointer to a \ref cs_equation_builder_t structure
+ * \param[in, out]  csys    pointer to a \ref cs_cell_sys_t structure
+ *
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_cdofb_vecteq_sourceterm(const cs_cell_mesh_t         *cm,
+                           const cs_equation_param_t    *eqp,
+                           const cs_real_t               t_eval,
+                           const cs_real_t               coef,
+                           cs_hodge_t                   *mass_hodge,
+                           cs_cell_builder_t            *cb,
+                           cs_equation_builder_t        *eqb,
+                           cs_cell_sys_t                *csys)
+{
+  /* Reset the local contribution */
+  memset(csys->source, 0, csys->n_dofs*sizeof(cs_real_t));
+
+  cs_source_term_compute_cellwise(eqp->n_source_terms,
+              (cs_xdef_t *const *)eqp->source_terms,
+                                  cm,
+                                  eqb->source_mask,
+                                  eqb->compute_source,
+                                  t_eval,
+                                  mass_hodge,  /* Mass matrix context */
+                                  cb,
+                                  csys->source);
+
+  /* Only cell-DoFs are involved */
+  const short int _off = 3*cm->n_fc;
+  csys->rhs[_off    ] += coef * csys->source[_off    ];
+  csys->rhs[_off + 1] += coef * csys->source[_off + 1];
+  csys->rhs[_off + 2] += coef * csys->source[_off + 2];
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Perform the assembly stage for a vector-valued system obtained
+ *         with CDO-Fb scheme
+ *
+ * \param[in]      csys              pointer to a cs_cell_sys_t structure
+ * \param[in]      rs                pointer to a cs_range_set_t structure
+ * \param[in]      cm                pointer to a cs_cell_mesh_t structure
+ * \param[in]      has_sourceterm    has the equation a source term?
+ * \param[in, out] eqc               context structure for a vector-valued Fb
+ * \param[in, out] eqa               pointer to cs_equation_assemble_t
+ * \param[in, out] mav               pointer to cs_matrix_assembler_values_t
+ * \param[in, out] rhs               right-end side of the system
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_cdofb_vecteq_assembly(const cs_cell_sys_t            *csys,
+                         const cs_range_set_t           *rs,
+                         const cs_cell_mesh_t           *cm,
+                         const bool                      has_sourceterm,
+                         cs_cdofb_vecteq_t              *eqc,
+                         cs_equation_assemble_t         *eqa,
+                         cs_matrix_assembler_values_t   *mav,
+                         cs_real_t                       rhs[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Update the variables associated to cells in case of a CDO-Fb
+ *         scheme. This has to be done after a resolution.
+ *
+ * \param[in, out] tce       pointer to a timer counter
+ * \param[in, out] fld       pointer to a cs_field_t structure
+ * \param[in, out] eqc       pointer to a context structure
+ * \param[in]      cur2prev  true if one performs "current to previous" op.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_cdofb_vecteq_update_cell_fields(cs_timer_counter_t      *tce,
+                                   cs_field_t              *fld,
+                                   cs_cdofb_vecteq_t       *eqc,
+                                   bool                     cur2prev);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Build and solve the linear system arising from a vector steady-state
+ *         diffusion equation with a CDO-Fb scheme
  *         One works cellwise and then process to the assembly
  *
+ * \param[in]      cur2prev   true="current to previous" operation is performed
  * \param[in]      mesh       pointer to a cs_mesh_t structure
- * \param[in]      field_val  pointer to the current value of the field
- * \param[in]      dt_cur     current value of the time step
+ * \param[in]      field_id   id of the variable field related to this equation
  * \param[in]      eqp        pointer to a cs_equation_param_t structure
  * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
- * \param[in, out] data       pointer to cs_cdofb_vecteq_t structure
- * \param[in, out] rhs        right-hand side
- * \param[in, out] matrix     pointer to cs_matrix_t structure to compute
+ * \param[in, out] context    pointer to cs_cdofb_vecteq_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_vecteq_build_system(const cs_mesh_t            *mesh,
-                             const cs_real_t            *field_val,
-                             double                      dt_cur,
-                             const cs_equation_param_t  *eqp,
-                             cs_equation_builder_t      *eqb,
-                             void                       *data,
-                             cs_real_t                  *rhs,
-                             cs_matrix_t                *matrix);
+cs_cdofb_vecteq_solve_steady_state(bool                        cur2prev,
+                                   const cs_mesh_t            *mesh,
+                                   const int                   field_id,
+                                   const cs_equation_param_t  *eqp,
+                                   cs_equation_builder_t      *eqb,
+                                   void                       *context);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Store solution(s) of the linear system into a field structure
- *         Update extra-field values if required (for hybrid discretization)
+ * \brief  Build and solve the linear system arising from a vector diffusion
+ *         equation with a CDO-Fb scheme and an implicit Euler scheme.
+ *         One works cellwise and then process to the assembly
  *
- * \param[in]      solu       solution array
- * \param[in]      rhs        rhs associated to this solution array
+ * \param[in]      cur2prev   true="current to previous" operation is performed
+ * \param[in]      mesh       pointer to a cs_mesh_t structure
+ * \param[in]      field_id   id of the variable field related to this equation
  * \param[in]      eqp        pointer to a cs_equation_param_t structure
  * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
- * \param[in, out] data       pointer to cs_cdofb_vecteq_t structure
- * \param[in, out] field_val  pointer to the current value of the field
+ * \param[in, out] context    pointer to cs_cdofb_vecteq_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_vecteq_update_field(const cs_real_t              *solu,
-                             const cs_real_t              *rhs,
-                             const cs_equation_param_t    *eqp,
-                             cs_equation_builder_t        *eqb,
-                             void                         *data,
-                             cs_real_t                    *field_val);
+cs_cdofb_vecteq_solve_implicit(bool                        cur2prev,
+                               const cs_mesh_t            *mesh,
+                               const int                   field_id,
+                               const cs_equation_param_t  *eqp,
+                               cs_equation_builder_t      *eqb,
+                               void                       *context);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Build and solve the linear system arising from a vector diffusion
+ *         equation with a CDO-Fb scheme and an implicit/explicit theta scheme.
+ *         One works cellwise and then process to the assembly
+ *
+ * \param[in]      cur2prev   true="current to previous" operation is performed
+ * \param[in]      mesh       pointer to a cs_mesh_t structure
+ * \param[in]      field_id   id of the variable field related to this equation
+ * \param[in]      eqp        pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
+ * \param[in, out] context    pointer to cs_cdofb_vecteq_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_cdofb_vecteq_solve_theta(bool                        cur2prev,
+                            const cs_mesh_t            *mesh,
+                            const int                   field_id,
+                            const cs_equation_param_t  *eqp,
+                            cs_equation_builder_t      *eqb,
+                            void                       *context);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Operate a current to previous operation for the field associated to
+ *         this equation and potentially for related fields/arrays.
+ *
+ * \param[in]       eqp        pointer to a cs_equation_param_t structure
+ * \param[in, out]  eqb        pointer to a cs_equation_builder_t structure
+ * \param[in, out]  context    pointer to cs_cdofb_vecteq_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_cdofb_vecteq_current_to_previous(const cs_equation_param_t  *eqp,
+                                    cs_equation_builder_t      *eqb,
+                                    void                       *context);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Predefined extra-operations related to this equation
  *
- * \param[in]       eqname     name of the equation
- * \param[in]       field      pointer to a field structure
  * \param[in]       eqp        pointer to a cs_equation_param_t structure
  * \param[in, out]  eqb        pointer to a cs_equation_builder_t structure
- * \param[in, out]  data       pointer to cs_cdofb_vecteq_t structure
+ * \param[in, out]  context    pointer to cs_cdofb_vecteq_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_vecteq_extra_op(const char                 *eqname,
-                         const cs_field_t           *field,
-                         const cs_equation_param_t  *eqp,
-                         cs_equation_builder_t      *eqb,
-                         void                       *data);
+cs_cdofb_vecteq_extra_post(const cs_equation_param_t  *eqp,
+                           cs_equation_builder_t      *eqb,
+                           void                       *context);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -316,13 +504,15 @@ cs_cdofb_vecteq_extra_op(const char                 *eqname,
  *         have to free the return pointer.
  *
  * \param[in, out]  context    pointer to a data structure cast on-the-fly
+ * \param[in]       previous   retrieve the previous state (true/false)
  *
- * \return  a pointer to an array of \ref cs_real_t
+ * \return  a pointer to an array of cs_real_t (size 3*n_cells)
  */
 /*----------------------------------------------------------------------------*/
 
 cs_real_t *
-cs_cdofb_vecteq_get_cell_values(void      *context);
+cs_cdofb_vecteq_get_cell_values(void      *context,
+                                bool       previous);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -331,13 +521,15 @@ cs_cdofb_vecteq_get_cell_values(void      *context);
  *         have to free the return pointer.
  *
  * \param[in, out]  context    pointer to a data structure cast on-the-fly
+ * \param[in]       previous   retrieve the previous state (true/false)
  *
- * \return  a pointer to an array of cs_real_t (size n_faces)
+ * \return  a pointer to an array of cs_real_t (size 3*n_faces)
  */
 /*----------------------------------------------------------------------------*/
 
 cs_real_t *
-cs_cdofb_vecteq_get_face_values(void    *context);
+cs_cdofb_vecteq_get_face_values(void    *context,
+                                bool     previous);
 
 /*----------------------------------------------------------------------------*/
 /*!
