@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -38,6 +38,7 @@
  *----------------------------------------------------------------------------*/
 
 #include "bft_error.h"
+#include "bft_mem.h"
 
 #include "cs_log.h"
 
@@ -70,10 +71,65 @@ BEGIN_C_DECLS
 #if defined(DEBUG) && !defined(NDEBUG)
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief   Print a cs_sdm_t structure which is defined by block
+ * \brief   Function used to select which element deserves a dump or specific
+ *          treatment during a debugging stage
+ *
+ * \param[in]  eqp      pointer to a cs_equation_param_t structure
+ * \param[in]  cm       pointer to a cs_cell_mesh_t structure
+ * \param[in]  csys     pointer to a cs_cell_sys_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+bool
+cs_dbg_cw_test(const cs_equation_param_t   *eqp,
+               const cs_cell_mesh_t        *cm,
+               const cs_cell_sys_t         *csys)
+{
+  CS_UNUSED(eqp);
+
+#if 1 /* First example: Look for debug information related the cell number 0 */
+  if (cm != NULL)
+    if (cm->c_id == 0)
+      return true;
+  if (csys != NULL)
+    if (csys->c_id == 0)
+      return true;
+#endif
+
+#if 0 /* Second example: Only search debug information for a given equation and
+         a requested vertex */
+
+  /* Vertex number 441 belongs to the current cell ? */
+  const cs_lnum_t  target_vertex_id = 441;
+  const short int _v = cs_cell_mesh_get_v(target_vertex_id, cm);
+  if (_v < 0)
+    return false;
+
+  if (eqp != NULL)
+    if (strcmp(eqp->name, "Tracer1") == 0)
+      return true;
+#endif
+
+#if 0 /* Third example: Look for the cells which have a previous DoF value
+         greater than 0.06 for the face 1 */
+  const cs_lnum_t target_face_id = 1;
+  const short int _f = cs_cell_mesh_get_f(target_face_id, cm);
+
+  if (csys != NULL && _f > -1) {
+    if (csys->val_n[_f] > 0.06)
+      return true;
+  }
+#endif
+
+  return false; /* Default behavior */
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Print an array.
  *          Print into the file f if given otherwise open a new file named
  *          fname if given otherwise print into the standard output
- *          The usage of threshold allows one to compare more easier matrices
+ *          The usage of threshold allows one to compare more easier arrays
  *          without taking into account numerical roundoff.
  *
  * \param[in]  fp         pointer to a file structure or NULL
@@ -111,9 +167,9 @@ cs_dbg_array_fprintf(FILE             *fp,
   for (cs_lnum_t i = 0; i < n_rows; i++) {
     for (cs_lnum_t j = i*n_cols; j < (i+1)*n_cols; j++) {
       if (fabs(array[j]) < thd)
-        fprintf(fout, "% -8.5e", 0.);
+        fprintf(fout, " % -8.5e", 0.);
       else
-        fprintf(fout, "% -8.5e", array[j]);
+        fprintf(fout, " % -8.5e", array[j]);
     }
     fprintf(fout, "\n");
   }
@@ -121,9 +177,9 @@ cs_dbg_array_fprintf(FILE             *fp,
   if (n_rows*n_cols < n_elts) {
     for (cs_lnum_t j = n_rows*n_cols; j < n_elts; j++) {
       if (fabs(array[j]) < thd)
-        fprintf(fout, "% -8.5e", 0.);
+        fprintf(fout, " % -8.5e", 0.);
       else
-        fprintf(fout, "% -8.5e", array[j]);
+        fprintf(fout, " % -8.5e", array[j]);
     }
     fprintf(fout, "\n");
   }
@@ -134,7 +190,45 @@ cs_dbg_array_fprintf(FILE             *fp,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  In debug mode, dump an array of double into the listing
+ * \brief  In debug mode, print into a file the solution and its right-hand
+ *         side
+ *
+ * \param[in] eqname     name of the related equation
+ * \param[in] nt         number of time step
+ * \param[in] level      level of debug
+ * \param[in] sol        solution array
+ * \param[in] rhs        rhs array
+ * \param[in] size       size of the array to print
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_dbg_fprintf_system(const char        *eqname,
+                      int                nt,
+                      int                level,
+                      const cs_real_t   *sol,
+                      const cs_real_t   *rhs,
+                      cs_lnum_t          size)
+{
+  int  len = strlen(eqname) + strlen("-sol-.log") + 4 + 1;
+  char  *filename = NULL;
+
+  BFT_MALLOC(filename, len, char);
+
+  sprintf(filename, "%s-sol-%04d.log", eqname, nt);
+  if (sol != NULL && level > 4)
+    cs_dbg_array_fprintf(NULL, filename, 1e-16, size, sol, 6);
+
+  sprintf(filename, "%s-rhs-%04d.log", eqname, nt);
+  if (rhs != NULL && level > 5)
+    cs_dbg_array_fprintf(NULL, filename, 1e-16, size, rhs, 6);
+
+  BFT_FREE(filename);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  In debug mode, dump an array of double into the log
  *
  * \param[in] header     header message to write
  * \param[in] size       number of elements in array
@@ -145,9 +239,9 @@ cs_dbg_array_fprintf(FILE             *fp,
 
 void
 cs_dbg_darray_to_listing(const char        *header,
-                        const cs_lnum_t    size,
-                        const cs_real_t    array[],
-                        int                n_cols)
+                         const cs_lnum_t    size,
+                         const cs_real_t    array[],
+                         int                n_cols)
 {
   cs_log_printf(CS_LOG_DEFAULT, "\nDUMP>> %s\n", header);
 
@@ -156,13 +250,13 @@ cs_dbg_darray_to_listing(const char        *header,
 
   for (cs_lnum_t i = 0; i < n_rows; i++) {
     for (cs_lnum_t j = i*n_cols; j < (i+1)*n_cols; j++)
-      cs_log_printf(CS_LOG_DEFAULT, " (%04d) % 6.4e", j, array[j]);
+      cs_log_printf(CS_LOG_DEFAULT, " (%04ld) % 6.4e", (long)j, array[j]);
     cs_log_printf(CS_LOG_DEFAULT, "\n");
   }
 
   if (n_rows*n_cols < size) {
     for (cs_lnum_t j = n_rows*n_cols; j < size; j++)
-      cs_log_printf(CS_LOG_DEFAULT, " (%04d) % 6.4e", j, array[j]);
+      cs_log_printf(CS_LOG_DEFAULT, " (%04ld) % 6.4e", (long)j, array[j]);
     cs_log_printf(CS_LOG_DEFAULT, "\n");
   }
 
@@ -170,7 +264,7 @@ cs_dbg_darray_to_listing(const char        *header,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  In debug mode, dump an array of integer into the listing
+ * \brief  In debug mode, dump an array of integer into the log
  *
  * \param[in] header     header message to write
  * \param[in] size       number of elements in array
@@ -192,18 +286,55 @@ cs_dbg_iarray_to_listing(const char        *header,
 
   for (cs_lnum_t i = 0; i < n_rows; i++) {
     for (cs_lnum_t j = i*n_cols; j < (i+1)*n_cols; j++)
-      cs_log_printf(CS_LOG_DEFAULT, " (%04d) % 6d", j, array[j]);
+      cs_log_printf(CS_LOG_DEFAULT, " (%04ld) % 6ld", (long)j, (long)array[j]);
     cs_log_printf(CS_LOG_DEFAULT, "\n");
   }
 
   if (n_rows*n_cols < size) {
     for (cs_lnum_t j = n_rows*n_cols; j < size; j++)
-      cs_log_printf(CS_LOG_DEFAULT, " (%04d) % 6d", j, array[j]);
+      cs_log_printf(CS_LOG_DEFAULT, " (%04ld) % 6ld", (long)j, (long)array[j]);
     cs_log_printf(CS_LOG_DEFAULT, "\n");
   }
 
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  In debug mode, dump a linear system. Case of scalar-valued entries.
+ *
+ * \param[in] eqname     name of the equation related to the current system
+ * \param[in] matrix     pointer to the matrix to dump
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_dbg_dump_local_scalar_msr_matrix(const char          *name,
+                                    const cs_matrix_t   *matrix)
+{
+  if (cs_matrix_get_type(matrix) != CS_MATRIX_MSR)
+    return;
+
+  cs_log_printf(CS_LOG_DEFAULT, "\nDUMP MSR MATRIX FOR THE EQUATION >> %s\n",
+                name);
+
+  const cs_lnum_t  size = cs_matrix_get_n_rows(matrix);
+  const cs_lnum_t  *row_index, *col_id;
+  const cs_real_t  *d_val, *x_val;
+
+  cs_matrix_get_msr_arrays(matrix, &row_index, &col_id, &d_val, &x_val);
+
+  for (cs_lnum_t i = 0; i < size; i++) {
+
+    const cs_lnum_t  *idx = row_index + i;
+
+    cs_log_printf(CS_LOG_DEFAULT, "%4ld |D|% -6.4e |E", (long)i, d_val[i]);
+    for (cs_lnum_t j = idx[0]; j < idx[1]; j++)
+      cs_log_printf(CS_LOG_DEFAULT, "|% -6.4e c%4ld", x_val[j], (long)col_id[j]);
+    cs_log_printf(CS_LOG_DEFAULT, "\n");
+
+  } /* Loop on rows */
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  In debug mode, dump a linear system
@@ -239,24 +370,24 @@ cs_dbg_dump_linear_system(const char        *eqname,
   cs_log_printf(CS_LOG_DEFAULT, " >> SOLUTION\n");
   for (cs_lnum_t i = 0; i < n_dump_rows; i++) {
     for (cs_lnum_t j = i*n_dump_cols; j < (i+1)*n_dump_cols; j++)
-      cs_log_printf(CS_LOG_DEFAULT, "%4d % -6.4e |", j, x[j]);
+      cs_log_printf(CS_LOG_DEFAULT, "%4ld % -6.4e |", (long)j, x[j]);
     cs_log_printf(CS_LOG_DEFAULT, "\n");
   }
   if (n_dump_rows*n_dump_cols < size) {
     for (cs_lnum_t j = n_dump_rows*n_dump_cols; j < size; j++)
-      cs_log_printf(CS_LOG_DEFAULT, "%4d % -6.4e |", j, x[j]);
+      cs_log_printf(CS_LOG_DEFAULT, "%4ld % -6.4e |", (long)j, x[j]);
     cs_log_printf(CS_LOG_DEFAULT, "\n");
   }
 
   cs_log_printf(CS_LOG_DEFAULT, " >> RIGHT-HAND SIDE\n");
   for (cs_lnum_t i = 0; i < n_dump_rows; i++) {
     for (cs_lnum_t j = i*n_dump_cols; j < (i+1)*n_dump_cols; j++)
-      cs_log_printf(CS_LOG_DEFAULT, "%4d % -6.4e |", j, b[j]);
+      cs_log_printf(CS_LOG_DEFAULT, "%4ld % -6.4e |", (long)j, b[j]);
     cs_log_printf(CS_LOG_DEFAULT, "\n");
   }
   if (n_dump_rows*n_dump_cols < size) {
     for (cs_lnum_t j = n_dump_rows*n_dump_cols; j < size; j++)
-      cs_log_printf(CS_LOG_DEFAULT, "%4d % -6.4e |", j, b[j]);
+      cs_log_printf(CS_LOG_DEFAULT, "%4ld % -6.4e |", (long)j, b[j]);
     cs_log_printf(CS_LOG_DEFAULT, "\n");
   }
 
@@ -265,12 +396,12 @@ cs_dbg_dump_linear_system(const char        *eqname,
     cs_log_printf(CS_LOG_DEFAULT, " >> DIAGONAL ENTRIES\n");
     for (cs_lnum_t i = 0; i < n_dump_rows; i++) {
       for (cs_lnum_t j = i*n_dump_cols; j < (i+1)*n_dump_cols; j++)
-        cs_log_printf(CS_LOG_DEFAULT, "%4d % -6.4e |", j, dval[j]);
+        cs_log_printf(CS_LOG_DEFAULT, "%4ld % -6.4e |", (long)j, dval[j]);
       cs_log_printf(CS_LOG_DEFAULT, "\n");
     }
     if (n_dump_rows*n_dump_cols < size) {
       for (cs_lnum_t j = n_dump_rows*n_dump_cols; j < size; j++)
-        cs_log_printf(CS_LOG_DEFAULT, "%4d % -6.4e |", j, dval[j]);
+        cs_log_printf(CS_LOG_DEFAULT, "%4ld % -6.4e |", (long)j, dval[j]);
       cs_log_printf(CS_LOG_DEFAULT, "\n");
     }
 
@@ -286,15 +417,17 @@ cs_dbg_dump_linear_system(const char        *eqname,
       int  _n_rows = n_entries/_n_cols;
 
       for (cs_lnum_t ii = 0; ii < _n_rows; ii++) {
-        cs_log_printf(CS_LOG_DEFAULT, "ROW%4d >> ", i);
+        cs_log_printf(CS_LOG_DEFAULT, "ROW%4ld >> ", (long)i);
         for (cs_lnum_t jj = ii*_n_cols; jj < (ii+1)*_n_cols; jj++)
-          cs_log_printf(CS_LOG_DEFAULT, "%4d: % -6.4e |", _col[jj], _val[jj]);
+          cs_log_printf(CS_LOG_DEFAULT, "%4ld: % -6.4e |",
+                        (long)_col[jj], _val[jj]);
         cs_log_printf(CS_LOG_DEFAULT, "\n");
       }
       if (_n_rows*_n_cols < n_entries) {
-        cs_log_printf(CS_LOG_DEFAULT, "ROW%4d >> ", i);
+        cs_log_printf(CS_LOG_DEFAULT, "ROW%4ld >> ", (long)i);
         for (cs_lnum_t jj = _n_rows*_n_cols; jj < n_entries; jj++)
-          cs_log_printf(CS_LOG_DEFAULT, "%4d: % -6.4e |", _col[jj], _val[jj]);
+          cs_log_printf(CS_LOG_DEFAULT, "%4ld: % -6.4e |",
+                        (long)_col[jj], _val[jj]);
         cs_log_printf(CS_LOG_DEFAULT, "\n");
       }
 
