@@ -9,7 +9,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -51,7 +51,6 @@
 #include <bft_mem.h>
 
 #include "cs_boundary_zone.h"
-#include "cs_domain_boundary.h"
 #include "cs_log.h"
 #include "cs_math.h"
 #include "cs_mesh_location.h"
@@ -84,7 +83,7 @@ BEGIN_C_DECLS
  *============================================================================*/
 
 cs_domain_t  *cs_glob_domain = NULL; /* Pointer to the main computational
-                                        domain used in CDO/HHO schemes */
+                                        domain */
 
 /*============================================================================
  * Local structure definitions
@@ -103,7 +102,22 @@ static double  cs_domain_kahan_time_compensation = 0.0;
 /*! \cond DOXYGEN_SHOULD_SKIP_THIS */
 
 /*============================================================================
- * Private function prototypes
+ * Prototypes for functions intended for use only by Fortran wrappers.
+ * (descriptions follow, with function bodies).
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------
+ * Set the CDO mode in the Fortran part
+ *
+ * parameters:
+ *   mode <-- -1: no CDO, 1: with CDO, 2: CDO only
+ *----------------------------------------------------------------------------*/
+
+extern void
+cs_f_set_cdo_mode(int  mode);
+
+/*============================================================================
+ * Private function defintitions
  *============================================================================*/
 
 /*----------------------------------------------------------------------------*/
@@ -125,11 +139,10 @@ _create_cdo_context(int     cdo_mode)
 
   cc->mode = cdo_mode;
 
-  cc->force_advfield_update = false;
-
   /* Metadata related to each family of schemes */
   cc->vb_scheme_flag = 0;
   cc->vcb_scheme_flag = 0;
+  cc->eb_scheme_flag = 0;
   cc->fb_scheme_flag = 0;
   cc->hho_scheme_flag = 0;
 
@@ -153,38 +166,42 @@ _create_cdo_context(int     cdo_mode)
 cs_domain_t *
 cs_domain_create(void)
 {
-  cs_real_t  default_time_step = -1e13;
   cs_domain_t  *domain = NULL;
 
+  /* Initialization of several modules */
+  cs_math_set_machine_epsilon(); /* Compute and set machine epsilon */
+  cs_quadrature_setup();         /* Compute constant used in quadrature rules */
+
   BFT_MALLOC(domain, 1, cs_domain_t);
 
   domain->mesh = NULL;
   domain->mesh_quantities = NULL;
   domain->connect = NULL;
   domain->cdo_quantities = NULL;
-  domain->cdo_context = NULL;
+
+  /* By default a wall is defined for the whole boundary of the domain */
+  cs_glob_boundaries = cs_boundary_create(CS_BOUNDARY_CATEGORY_FLOW,
+                                          CS_BOUNDARY_WALL);
+  domain->boundaries = cs_glob_boundaries;
+  domain->ale_boundaries = cs_boundary_create(CS_BOUNDARY_CATEGORY_ALE,
+                                              CS_BOUNDARY_ALE_FIXED);
 
   /* Default initialization of the time step */
   domain->only_steady = true;
   domain->is_last_iter = false;
-  domain->dt_cur = default_time_step;
-  domain->time_step_def = NULL;
 
   /* Global structure for time step management */
   domain->time_step = cs_get_glob_time_step();
 
-  /* Time options () */
-  domain->time_options.inpdt0 = 0; /* standard calculation */
   domain->time_options.iptlro = 0;
   domain->time_options.idtvar = 0; /* constant time step by default */
   domain->time_options.coumax = 1.;
   domain->time_options.cflmmx = 0.99;
   domain->time_options.foumax = 10.;
   domain->time_options.varrdt = 0.1;
-  domain->time_options.dtref = default_time_step;
-  domain->time_options.dtmin = default_time_step;
-  domain->time_options.dtmax = default_time_step;
-  domain->time_options.relxst = 0.7; /* Not useful in CDO schemes */
+  domain->time_options.dtmin  = -1.e13;
+  domain->time_options.dtmax  = -1.e13;
+  domain->time_options.relxst = 0.7; /* Not used in CDO schemes */
 
   /* Other options */
   domain->restart_nt = 0;
@@ -192,19 +209,13 @@ cs_domain_create(void)
   domain->verbosity = 1;
 
   /* By default: CDO-HHO schemes are not activated */
+  domain->cdo_context = NULL;
   cs_domain_set_cdo_mode(domain, CS_DOMAIN_CDO_MODE_OFF);
 
-  /* By default a wall is defined for the whole boundary of the domain */
-  cs_domain_boundary_set_default(CS_DOMAIN_BOUNDARY_WALL);
-
   /* Monitoring */
   CS_TIMER_COUNTER_INIT(domain->tcp); /* domain post */
   CS_TIMER_COUNTER_INIT(domain->tcs); /* domain setup */
 
-  /* Initialization of several modules */
-  cs_math_set_machine_epsilon(); /* Compute and set machine epsilon */
-  cs_quadrature_setup();         /* Compute constant used in quadrature rules */
-
   return domain;
 }
 
@@ -229,18 +240,14 @@ cs_domain_free(cs_domain_t   **p_domain)
   domain->mesh = NULL;
   domain->mesh_quantities = NULL;
 
-  domain->time_step_def = cs_xdef_free(domain->time_step_def);
   domain->time_step = NULL;
 
   if (domain->cdo_context != NULL)
     BFT_FREE(domain->cdo_context);
 
   /* Free arrays related to the domain boundary */
-  cs_domain_boundary_free();
-
-  /* Free CDO structures related to geometric quantities and connectivity */
-  domain->cdo_quantities = cs_cdo_quantities_free(domain->cdo_quantities);
-  domain->connect = cs_cdo_connect_free(domain->connect);
+  cs_boundary_free(&(domain->boundaries));
+  cs_boundary_free(&(domain->ale_boundaries));
 
   BFT_FREE(domain);
   *p_domain = NULL;
@@ -269,7 +276,7 @@ cs_domain_set_cdo_mode(cs_domain_t    *domain,
   else
     domain->cdo_context->mode = mode;
 
-  CS_PROCF(set_cdo_mode, SET_CDO_MODE)(&mode);
+  cs_f_set_cdo_mode(mode);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -310,11 +317,11 @@ cs_domain_needs_iteration(cs_domain_t  *domain)
 
   cs_time_step_t  *ts = domain->time_step;
 
-  if (ts->nt_max > 0) // nt_max has been set
+  if (ts->nt_max > 0) /* nt_max has been set */
     if (ts->nt_cur > ts->nt_max)
       one_more_iter = false;
 
-  if (ts->t_max > 0) // t_max has been set
+  if (ts->t_max > 0) /* t_max has been set */
     if (ts->t_cur >= ts->t_max)
       one_more_iter = false;
 
@@ -329,7 +336,7 @@ cs_domain_needs_iteration(cs_domain_t  *domain)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Check if an ouput is requested according to the domain setting
+ * \brief  Check if an output is requested according to the domain setting
  *
  * \param[in]   domain    pointer to a cs_domain_t structure
  *
@@ -358,70 +365,6 @@ cs_domain_needs_log(const cs_domain_t      *domain)
   return false;
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Set the current time step for this new time iteration
- *
- * \param[in, out]   domain    pointer to a cs_domain_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_domain_define_current_time_step(cs_domain_t   *domain)
-{
-  if (domain == NULL) bft_error(__FILE__, __LINE__, 0, _err_empty_domain);
-
-  if (domain->only_steady)
-    return;
-
-  const cs_time_step_t  *ts = domain->time_step;
-  const double  t_cur = ts->t_cur;
-  const int  nt_cur = ts->nt_cur;
-
-  cs_xdef_t  *ts_def = domain->time_step_def;
-
-  if (ts_def == NULL)
-    bft_error(__FILE__, __LINE__, 0,
-              " Please check your settings: Unsteady computation but no"
-              " current time step defined.\n");
-
-  if (ts_def->type != CS_XDEF_BY_VALUE) { /* dt_cur may change */
-
-    if (ts_def->type == CS_XDEF_BY_TIME_FUNCTION) {
-
-      /* Compute current time step */
-      cs_xdef_timestep_input_t  *param =
-        (cs_xdef_timestep_input_t *)ts_def->input;
-      domain->dt_cur = param->func(nt_cur, t_cur, param->input);
-
-      /* Update time_options */
-      double  dtmin = CS_MIN(domain->time_options.dtmin, domain->dt_cur);
-      double  dtmax = CS_MAX(domain->time_options.dtmax, domain->dt_cur);
-
-      domain->time_options.dtmin = dtmin;
-      domain->time_options.dtmax = dtmax;
-      // TODO: Check how the following value is set in FORTRAN
-      // domain->time_options.dtref = 0.5*(dtmin + dtmax);
-      if (domain->time_options.dtref < 0) // Should be the initial val.
-        domain->time_options.dtref = domain->dt_cur;
-
-    }
-    else
-      bft_error(__FILE__, __LINE__, 0,
-                " Invalid way of defining the current time step.\n"
-                " Please modify your settings.");
-
-  }
-
-  /* Check if this is the last iteration */
-  if (ts->t_max > 0) // t_max has been set
-    if (t_cur + domain->dt_cur > ts->t_max)
-      domain->is_last_iter = true;
-  if (ts->nt_max > 0) // nt_max has been set
-    if (nt_cur + 1 > ts->nt_max)
-      domain->is_last_iter = true;
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Update the simulated time after one temporal iteration
@@ -435,35 +378,14 @@ cs_domain_increment_time(cs_domain_t  *domain)
 {
   cs_time_step_t  *ts = domain->time_step;
 
-  /* Increment time iteration */
-  ts->t_prev = ts->t_cur;
-
   /* Use Kahan's trick to limit the truncation error */
-  double  z = domain->dt_cur - cs_domain_kahan_time_compensation;
+  double  z = ts->dt[0] - cs_domain_kahan_time_compensation;
   double  t = ts->t_cur + z;
 
   cs_domain_kahan_time_compensation = (t - ts->t_cur) - z;
   ts->t_cur = t;
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Update the time step after one temporal iteration
- *
- * \param[in, out]  domain     pointer to a cs_domain_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_domain_increment_time_step(cs_domain_t  *domain)
-{
-  cs_time_step_t  *ts = domain->time_step;
-
-  /* Increment time iteration */
-  ts->nt_prev = ts->nt_cur;
-  ts->nt_cur++;
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief   Print a welcome message indicating which mode of CDO is activated
