@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -50,6 +50,7 @@
 #include "cs_gwf.h"
 #include "cs_log.h"
 #include "cs_log_iteration.h"
+#include "cs_maxwell.h"
 #include "cs_navsto_system.h"
 #include "cs_parall.h"
 #include "cs_post.h"
@@ -57,6 +58,7 @@
 #include "cs_prototypes.h"
 #include "cs_restart.h"
 #include "cs_restart_default.h"
+#include "cs_solidification.h"
 #include "cs_walldistance.h"
 
 /*----------------------------------------------------------------------------
@@ -73,6 +75,14 @@ BEGIN_C_DECLS
  * Local type definitions
  *----------------------------------------------------------------------------*/
 
+/*============================================================================
+ * Prototypes for functions intended for use only by Fortran wrappers.
+ * (descriptions follow, with function bodies).
+ *============================================================================*/
+
+void
+cs_f_cdo_post_domain(void);
+
 /*============================================================================
  * Static global variables
  *============================================================================*/
@@ -96,7 +106,7 @@ _needs_adimensional_numbers(void)
   int n_adv_fields = cs_advection_field_get_n_fields();
   for (int adv_id = 0; adv_id < n_adv_fields; adv_id++) {
     const cs_adv_field_t  *adv = cs_advection_field_by_id(adv_id);
-    if (adv->flag & CS_ADVECTION_FIELD_POST_COURANT)
+    if (adv->post_flag & CS_ADVECTION_FIELD_POST_COURANT)
       return true;
   }
 
@@ -126,7 +136,7 @@ _needs_adimensional_numbers(void)
  *         at cells (scalar-valued)
  *
  * \param[in]  cdoq       pointer to a cs_cdo_quantities_t struct.
- * \param[in]  basename   label for output in the listing
+ * \param[in]  basename   label for output in the log
  * \param[in]  array      pointer to the array to analyze
  */
 /*----------------------------------------------------------------------------*/
@@ -157,8 +167,9 @@ _analyze_cell_array(const cs_cdo_quantities_t   *cdoq,
   else
     min = _min, max = _max, sum = _sum;
 
+
   cs_log_printf(CS_LOG_DEFAULT, "s- %20s  % -6.4e % -6.4e % -6.4e\n",
-                basename, min, max, sum/cdoq->n_cells);
+                basename, min, max, sum/cdoq->n_g_cells);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -168,19 +179,17 @@ _analyze_cell_array(const cs_cdo_quantities_t   *cdoq,
  * \param[in]  adv         pointer to a cs_adv_field_t structure
  * \param[in]  cdoq        pointer to a cs_cdo_quantities_t struct.
  * \param[in]  time_step   pointer to a cs_time_step_t struct.
- * \param[in]  dt_cur      value of the current time step
  */
 /*----------------------------------------------------------------------------*/
 
 static void
 _post_courant_number(const cs_adv_field_t       *adv,
                      const cs_cdo_quantities_t  *cdoq,
-                     const cs_time_step_t       *time_step,
-                     double                      dt_cur)
+                     const cs_time_step_t       *time_step)
 {
   if (adv == NULL)
     return;
-  if (!(adv->flag & CS_ADVECTION_FIELD_POST_COURANT))
+  if (!(adv->post_flag & CS_ADVECTION_FIELD_POST_COURANT))
     return;
 
   int  len = strlen(adv->name) + 8 + 1;
@@ -190,9 +199,9 @@ _post_courant_number(const cs_adv_field_t       *adv,
 
   cs_real_t  *courant = cs_equation_get_tmpbuf();
 
-  cs_advection_get_courant(adv, dt_cur, courant);
+  cs_advection_get_courant(adv, time_step->dt[0], courant);
 
-  /* Brief output for the listing */
+  /* Brief output for the log */
   _analyze_cell_array(cdoq, label, courant);
 
   /* Postprocessing */
@@ -242,7 +251,7 @@ _post_peclet_number(const cs_equation_t        *eq,
   cs_real_t  *peclet = cs_equation_get_tmpbuf();
   cs_equation_compute_peclet(eq, time_step, peclet);
 
-  /* Brief output for the listing */
+  /* Brief output for the log */
   _analyze_cell_array(cdoq, label, peclet);
 
   /* Postprocessing */
@@ -267,15 +276,13 @@ _post_peclet_number(const cs_equation_t        *eq,
  * \param[in]  pty         pointer to a cs_property_t structure
  * \param[in]  cdoq        pointer to a cs_cdo_quantities_t struct.
  * \param[in]  time_step   pointer to a cs_time_step_t struct.
- * \param[in]  dt_cur      value of the current time step
  */
 /*----------------------------------------------------------------------------*/
 
 static void
 _post_fourier_number(const cs_property_t        *pty,
                      const cs_cdo_quantities_t  *cdoq,
-                     const cs_time_step_t       *time_step,
-                     double                      dt_cur)
+                     const cs_time_step_t       *time_step)
 {
   if (pty == NULL)
     return;
@@ -284,14 +291,14 @@ _post_fourier_number(const cs_property_t        *pty,
 
   cs_real_t  *fourier = cs_equation_get_tmpbuf();
 
-  cs_property_get_fourier(pty, time_step->t_cur, dt_cur, fourier);
+  cs_property_get_fourier(pty, time_step->t_cur, time_step->dt[0], fourier);
 
   int  len = strlen(pty->name) + 8 + 1;
   char  *label = NULL;
   BFT_MALLOC(label, len, char);
   sprintf(label, "%s.Fourier", pty->name);
 
-  /* Brief output for the listing */
+  /* Brief output for the log */
   _analyze_cell_array(cdoq, label, fourier);
 
   /* Postprocessing */
@@ -371,6 +378,22 @@ _domain_post(void                      *input,
   cs_equation_extra_post();
 }
 
+/*============================================================================
+ * Fortran wrapper function definitions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Solve only steady-state equations
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_f_cdo_post_domain(void)
+{
+  cs_domain_post(cs_glob_domain);
+}
+
 /*============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -392,9 +415,9 @@ cs_domain_post_init(cs_domain_t   *domain)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Process the computational domain after the resolution
+ * \brief  Post-processing of the computational domain after a resolution
  *
- * \param[in]  domain     pointer to a cs_domain_t structure
+ * \param[in]  domain            pointer to a cs_domain_t structure
  */
 /*----------------------------------------------------------------------------*/
 
@@ -403,35 +426,26 @@ cs_domain_post(cs_domain_t  *domain)
 {
   cs_timer_t  t0 = cs_timer_time();
 
-  /* Pre-stage for post-processing for the current time step */
-  cs_post_time_step_begin(domain->time_step);
+  assert(domain->cdo_context != NULL);
 
   /* Extra-operations */
   /* ================ */
 
   /* Predefined extra-operations related to advection fields */
-  assert(domain->cdo_context != NULL);
-
-  if (domain->cdo_context->force_advfield_update)
-    cs_advection_field_update(domain->time_step->t_cur, true);
-
-  /* User-defined extra operations */
-  cs_user_cdo_extra_op(domain);
+  cs_advection_field_update(domain->time_step->t_cur, true);
 
   /* Log output */
   if (cs_domain_needs_log(domain)) {
 
-    /* Basic statistic related to variables */
-    cs_log_iteration();
-
     /* Post-processing */
     /* =============== */
 
     /* Post-processing of adimensional numbers */
     if (_needs_adimensional_numbers()) {
 
-      cs_log_printf(CS_LOG_DEFAULT,
-                    " ------------------------------------------------------------\n");
+      cs_log_printf
+        (CS_LOG_DEFAULT,
+         " ------------------------------------------------------------\n");
       cs_log_printf(CS_LOG_DEFAULT, "s- %20s %10s %10s %10s\n",
                     "Adim. number", "min", "max", "mean");
 
@@ -440,8 +454,7 @@ cs_domain_post(cs_domain_t  *domain)
       for (int adv_id = 0; adv_id < n_adv_fields; adv_id++)
         _post_courant_number(cs_advection_field_by_id(adv_id),
                              domain->cdo_quantities,
-                             domain->time_step,
-                             domain->dt_cur);
+                             domain->time_step);
 
       /* 2. Peclet numbers */
       int n_equations = cs_equation_get_n_equations();
@@ -455,11 +468,11 @@ cs_domain_post(cs_domain_t  *domain)
       for (int i = 0; i < n_properties; i++)
         _post_fourier_number(cs_property_by_id(i),
                              domain->cdo_quantities,
-                             domain->time_step,
-                             domain->dt_cur);
+                             domain->time_step);
 
-      cs_log_printf(CS_LOG_DEFAULT,
-                    " ------------------------------------------------------------\n");
+      cs_log_printf
+        (CS_LOG_DEFAULT,
+         " ------------------------------------------------------------\n");
 
     } /* Needs to compute adimensional numbers */
 
@@ -467,8 +480,32 @@ cs_domain_post(cs_domain_t  *domain)
     cs_equation_post_balance(domain->mesh,
                              domain->connect,
                              domain->cdo_quantities,
-                             domain->time_step,
-                             domain->dt_cur);
+                             domain->time_step);
+
+    /* 5.a Specific operations for the GWF module */
+    if (cs_gwf_is_activated())
+      cs_gwf_extra_op(domain->connect, domain->cdo_quantities);
+
+    /* 5.b Specific operations for the Maxwell module */
+    if (cs_maxwell_is_activated())
+      cs_maxwell_extra_op(domain->connect, domain->cdo_quantities);
+
+    /* 5.c Specific operations for the Navier-Stokes module */
+    if (cs_navsto_system_is_activated())
+      cs_navsto_system_extra_op(domain->mesh,
+                                domain->connect,
+                                domain->cdo_quantities,
+                                domain->time_step);
+
+    /* 5.c Specific operations for the Solidification module */
+    if (cs_solidification_is_activated())
+      cs_solidification_extra_op(domain->connect,
+                                 domain->cdo_quantities,
+                                 domain->time_step);
+
+    /* Basic statistic related to variables */
+    if (domain->cdo_context->mode == CS_DOMAIN_CDO_MODE_ONLY)
+      cs_log_iteration(); /* Otherwise called from the FORTRAN part */
 
   } /* Needs a new log */
 
@@ -476,12 +513,16 @@ cs_domain_post(cs_domain_t  *domain)
      - the domain (advection fields and properties),
      - equations
      - groundwater flows
+     - Maxwell module
+     - Navier-Stokes
+     - Solidification
      are also handled during the call of this function thanks to
      cs_post_add_time_mesh_dep_output() function pointer
   */
   cs_post_time_step_output(domain->time_step);
 
-  cs_post_time_step_end();
+  /* User-defined extra operations */
+  cs_user_extra_operations(domain);
 
   cs_timer_t  t1 = cs_timer_time();
   cs_timer_counter_add_diff(&(domain->tcp), &t0, &t1);
@@ -506,7 +547,7 @@ cs_domain_read_restart(cs_domain_t  *domain)
     return;
   }
 
-  cs_restart_t  *restart = cs_restart_create("main", /* restart file name */
+  cs_restart_t  *restart = cs_restart_create("main.csc", /* restart file name */
                                              NULL,   /* directory name */
                                              CS_RESTART_MODE_READ);
 
@@ -522,7 +563,7 @@ cs_domain_read_restart(cs_domain_t  *domain)
                           "code_saturne:checkpoint:main:version", // secname
                           CS_MESH_LOCATION_NONE,                  // ml_id
                           1,                                      // nb. values
-                          CS_TYPE_cs_int_t,                       // val. type
+                          CS_TYPE_int,                            // val. type
                           &i_val);                                // value(s)
 
   if (retval != CS_RESTART_SUCCESS)
@@ -544,7 +585,7 @@ cs_domain_read_restart(cs_domain_t  *domain)
                                    "cdo:n_equations",
                                    CS_MESH_LOCATION_NONE,
                                    1,
-                                   CS_TYPE_cs_int_t,
+                                   CS_TYPE_int,
                                    &i_val);
 
   if (retval != CS_RESTART_SUCCESS)
@@ -560,7 +601,7 @@ cs_domain_read_restart(cs_domain_t  *domain)
                                    "cdo:n_properties",
                                    CS_MESH_LOCATION_NONE,
                                    1,
-                                   CS_TYPE_cs_int_t,
+                                   CS_TYPE_int,
                                    &i_val);
 
   if (retval != CS_RESTART_SUCCESS)
@@ -576,7 +617,7 @@ cs_domain_read_restart(cs_domain_t  *domain)
                                    "cdo:n_adv_fields",
                                    CS_MESH_LOCATION_NONE,
                                    1,
-                                   CS_TYPE_cs_int_t,
+                                   CS_TYPE_int,
                                    &i_val);
 
   if (retval != CS_RESTART_SUCCESS)
@@ -593,7 +634,7 @@ cs_domain_read_restart(cs_domain_t  *domain)
                                    "groundwater_flow_module",
                                    CS_MESH_LOCATION_NONE,
                                    1,
-                                   CS_TYPE_cs_int_t,
+                                   CS_TYPE_int,
                                    &i_val);
 
   if (retval != CS_RESTART_SUCCESS)
@@ -610,7 +651,7 @@ cs_domain_read_restart(cs_domain_t  *domain)
                                    "navier_stokes_system",
                                    CS_MESH_LOCATION_NONE,
                                    1,
-                                   CS_TYPE_cs_int_t,
+                                   CS_TYPE_int,
                                    &i_val);
 
   if (retval != CS_RESTART_SUCCESS)
@@ -627,7 +668,7 @@ cs_domain_read_restart(cs_domain_t  *domain)
                                    "wall_distance",
                                    CS_MESH_LOCATION_NONE,
                                    1,
-                                   CS_TYPE_cs_int_t,
+                                   CS_TYPE_int,
                                    &i_val);
 
   if (retval != CS_RESTART_SUCCESS)
@@ -643,7 +684,7 @@ cs_domain_read_restart(cs_domain_t  *domain)
                                    "cur_time_step",
                                    CS_MESH_LOCATION_NONE,
                                    1,
-                                   CS_TYPE_cs_int_t,
+                                   CS_TYPE_int,
                                    &nt_cur);
 
   if (retval != CS_RESTART_SUCCESS)
@@ -710,7 +751,7 @@ cs_domain_write_restart(const cs_domain_t  *domain)
   if (cs_restart_checkpoint_required(domain->time_step) == false)
     return;
 
-  cs_restart_t  *restart = cs_restart_create("main", /* restart file name */
+  cs_restart_t  *restart = cs_restart_create("main.csc", /* restart file name */
                                              NULL,   /* directory name */
                                              CS_RESTART_MODE_WRITE);
 
@@ -720,7 +761,7 @@ cs_domain_write_restart(const cs_domain_t  *domain)
                            "code_saturne:checkpoint:main:version", // secname
                            CS_MESH_LOCATION_NONE,                  // ml_id
                            1,                                      // nb. values
-                           CS_TYPE_cs_int_t,                       // val. type
+                           CS_TYPE_int,                            // val. type
                            &version);                              // value(s)
 
   /* Write a new section: field information */
@@ -732,7 +773,7 @@ cs_domain_write_restart(const cs_domain_t  *domain)
                            "cdo:n_equations",
                            CS_MESH_LOCATION_NONE,
                            1,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            &n_equations);
 
   /* Write a new section */
@@ -741,7 +782,7 @@ cs_domain_write_restart(const cs_domain_t  *domain)
                            "cdo:n_properties",
                            CS_MESH_LOCATION_NONE,
                            1,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            &n_properties);
 
   /* Write a new section */
@@ -750,7 +791,7 @@ cs_domain_write_restart(const cs_domain_t  *domain)
                            "cdo:n_adv_fields",
                            CS_MESH_LOCATION_NONE,
                            1,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            &n_adv_fields);
 
   /* Write a new section: activation or not of the groundwater flow module */
@@ -760,7 +801,7 @@ cs_domain_write_restart(const cs_domain_t  *domain)
                            "groundwater_flow_module",
                            CS_MESH_LOCATION_NONE,
                            1,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            &igwf);
 
   /* Write a new section: activation or not of the Navier-Stokes system */
@@ -770,7 +811,7 @@ cs_domain_write_restart(const cs_domain_t  *domain)
                            "navier_stokes_system",
                            CS_MESH_LOCATION_NONE,
                            1,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            &inss);
 
   /* Write a new section: computation or not of the wall distance */
@@ -780,7 +821,7 @@ cs_domain_write_restart(const cs_domain_t  *domain)
                            "wall_distance",
                            CS_MESH_LOCATION_NONE,
                            1,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            &iwall);
 
   /* Write a new section: number of computed time steps */
@@ -789,7 +830,7 @@ cs_domain_write_restart(const cs_domain_t  *domain)
                            "cur_time_step",
                            CS_MESH_LOCATION_NONE,
                            1,
-                           CS_TYPE_cs_int_t,
+                           CS_TYPE_int,
                            &ntcabs);
 
   /* Read a new section: number of computed time steps */
