@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -37,7 +37,7 @@
 #include "cs_domain.h"
 #include "cs_equation.h"
 #include "cs_gwf.h"
-#include "cs_param.h"
+#include "cs_param_types.h"
 #include "cs_property.h"
 
 /*----------------------------------------------------------------------------*/
@@ -60,17 +60,6 @@ BEGIN_C_DECLS
  * Public function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Set to true the automatic update of all advection fields
- *
- * \param[in, out]  domain    pointer to a \ref cs_domain_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_domain_update_advfield(cs_domain_t       *domain);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Set to true the automatic update of all advection fields
@@ -78,7 +67,7 @@ cs_domain_update_advfield(cs_domain_t       *domain);
  *
  * \param[in, out]  domain       pointer to a cs_domain_t structure
  * \param[in]       nt_interval  frequency for the restart process
- * \param[in]       nt_list      output frequency into the listing
+ * \param[in]       nt_list      output frequency into the log
  * \param[in]       verbosity    level of information displayed
  */
 /*----------------------------------------------------------------------------*/
@@ -107,18 +96,33 @@ cs_domain_set_time_param(cs_domain_t       *domain,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Define the value of the time step thanks to a predefined function
+ * \brief  Set time step parameters for unsteady computations when this is not
+ *         already done. This situation should occur when the GUI is used to
+ *         set a constant time step.
  *
- * \param[in, out] domain      pointer to a cs_domain_t structure
- * \param[in]      func        pointer to a cs_timestep_func_t function
- * \param[in]      func_input  pointer to a structure cast on-the-fly
+ * \param[in, out]  domain    pointer to a \ref cs_domain_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_domain_def_time_step_by_function(cs_domain_t          *domain,
-                                    cs_timestep_func_t   *func,
-                                    void                 *func_input);
+cs_domain_automatic_time_step_settings(cs_domain_t       *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define the value of the time step thanks to a predefined function
+ *
+ * \param[in, out] domain      pointer to a cs_domain_t structure
+ * \param[in]      func        pointer to a cs_time_func_t function
+ * \param[in]      func_input  pointer to a structure cast on-the-fly
+ *
+ * \return a pointer to the created definition (\ref cs_xdef_t structure)
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_xdef_t *
+cs_domain_def_time_step_by_function(cs_domain_t        *domain,
+                                    cs_time_func_t     *func,
+                                    void               *func_input);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -150,18 +154,37 @@ cs_domain_initialize_setup(cs_domain_t                 *domain);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Last setup stage of the cs_domain_t structure
+ * \brief  After having read the mesh and the first setup stage build the
+ *         connectivities and mesh quantities related to CDO/HHO schemes
  *
  * \param[in, out]  domain            pointer to a cs_domain_t struct.
- * \param[in, out]  mesh              pointer to a cs_mesh_t struct.
- * \param[in]       mesh_quantities   pointer to a cs_mesh_quantities_t struct.
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_domain_finalize_setup(cs_domain_t                 *domain,
-                         cs_mesh_t                   *mesh,
-                         const cs_mesh_quantities_t  *mesh_quantities);
+cs_domain_init_cdo_structures(cs_domain_t              *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Last user setup stage of the cs_domain_t structure
+ *
+ * \param[in, out]  domain            pointer to a cs_domain_t struct.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_domain_finalize_user_setup(cs_domain_t         *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Last setup stage of the cs_domain_t structure
+ *
+ * \param[in, out]  domain            pointer to a cs_domain_t struct.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_domain_finalize_module_setup(cs_domain_t         *domain);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -175,6 +198,17 @@ cs_domain_finalize_setup(cs_domain_t                 *domain,
 void
 cs_domain_initialize_systems(cs_domain_t   *domain);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Summary of the main domain settings
+ *
+ * \param[in]   domain    pointer to the cs_domain_t structure to summarize
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_domain_setup_log(const cs_domain_t   *domain);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
