@@ -9,7 +9,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,11 +32,11 @@
 
 #include "cs_cdo_bc.h"
 #include "cs_cdo_connect.h"
-#include "cs_xdef_eval.h"
 #include "cs_cdo_quantities.h"
 #include "cs_cdo_local.h"
 #include "cs_equation_param.h"
 #include "cs_time_step.h"
+#include "cs_xdef_eval.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -50,6 +50,49 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Apply a boundary condition for a given face (inlet, outlet, wall,
+ *         sliding wall, symmetry...)
+ *
+ * \param[in]       f       face id in the cell mesh numbering
+ * \param[in]       eqp     pointer to a \ref cs_equation_param_t struct.
+ * \param[in]       cm      pointer to a \ref cs_cell_mesh_t structure
+ * \param[in]       pty     pointer to a \ref cs_property_data_t structure
+ * \param[in, out]  cb      pointer to a \ref cs_cell_builder_t structure
+ * \param[in, out]  csys    structure storing the cell-wise system
+ */
+/*----------------------------------------------------------------------------*/
+
+typedef void
+(cs_cdo_apply_boundary_t)(short int                     f,
+                          const cs_equation_param_t    *eqp,
+                          const cs_cell_mesh_t         *cm,
+                          const cs_property_data_t     *pty,
+                          cs_cell_builder_t            *cb,
+                          cs_cell_sys_t                *csys);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Enforcement of a boundary condition (Dirichlet, Robin, sliding...)
+ *
+ * \param[in]       eqp       pointer to a \ref cs_equation_param_t struct.
+ * \param[in]       cm        pointer to a \ref cs_cell_mesh_t structure
+ * \param[in, out]  fm        pointer to a \ref cs_face_mesh_t structure
+ * \param[in, out]  hodge     pointer to a \ref cs_hodge_t structure
+ * \param[in, out]  cb        pointer to a \ref cs_cell_builder_t structure
+ * \param[in, out]  csys      structure storing the cell-wise system
+ */
+/*----------------------------------------------------------------------------*/
+
+typedef void
+(cs_cdo_enforce_bc_t)(const cs_equation_param_t      *eqp,
+                      const cs_cell_mesh_t           *cm,
+                      cs_face_mesh_t                 *fm,
+                      cs_hodge_t                     *hodge,
+                      cs_cell_builder_t              *cb,
+                      cs_cell_sys_t                  *csys);
+
 /*============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -78,41 +121,58 @@ cs_equation_init_boundary_flux_from_bc(cs_real_t                   t_eval,
  * \brief   Set the BC into a cellwise view of the current system.
  *          Case of vertex-based schemes
  *
- * \param[in]      cm          pointer to a cellwise view of the mesh
- * \param[in]      connect     pointer to a cs_cdo_connect_t struct.
- * \param[in]      quant       pointer to a cs_cdo_quantities_t structure
- * \param[in]      eqp         pointer to a cs_equation_param_t structure
- * \param[in]      dir_values  Dirichlet values associated to each vertex
- * \param[in]      neu_tags    definition id related to each Neumann face
- * \param[in]      t_eval      time at which one performs the evaluation
- * \param[in, out] csys        pointer to a cellwise view of the system
- * \param[in, out] cb          pointer to a cellwise builder
+ * \param[in]      cm           pointer to a cellwise view of the mesh
+ * \param[in]      eqp          pointer to a cs_equation_param_t structure
+ * \param[in]      face_bc      pointer to a cs_cdo_bc_face_t structure
+ * \param[in]      vtx_bc_flag  BC flags associated to vertices
+ * \param[in]      dir_values   Dirichlet values associated to each vertex
+ * \param[in]      t_eval       time at which one performs the evaluation
+ * \param[in, out] csys         pointer to a cellwise view of the system
+ * \param[in, out] cb           pointer to a cellwise builder
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_equation_vb_set_cell_bc(const cs_cell_mesh_t         *cm,
-                           const cs_cdo_connect_t       *connect,
-                           const cs_cdo_quantities_t    *quant,
                            const cs_equation_param_t    *eqp,
+                           const cs_cdo_bc_face_t       *face_bc,
+                           const cs_flag_t               vtx_bc_flag[],
                            const cs_real_t               dir_values[],
-                           const short int               neu_tags[],
                            cs_real_t                     t_eval,
                            cs_cell_sys_t                *csys,
                            cs_cell_builder_t            *cb);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Set the BC into a cellwise view of the current system.
+ *          Case of edge-based schemes
+ *
+ * \param[in]      cm           pointer to a cellwise view of the mesh
+ * \param[in]      eqp          pointer to a cs_equation_param_t structure
+ * \param[in]      face_bc      pointer to a cs_cdo_bc_face_t structure
+ * \param[in]      dir_values   Dirichlet values associated to each vertex
+ * \param[in, out] csys         pointer to a cellwise view of the system
+ * \param[in, out] cb           pointer to a cellwise builder
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_equation_eb_set_cell_bc(const cs_cell_mesh_t         *cm,
+                           const cs_equation_param_t    *eqp,
+                           const cs_cdo_bc_face_t       *face_bc,
+                           const cs_real_t               dir_values[],
+                           cs_cell_sys_t                *csys,
+                           cs_cell_builder_t            *cb);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief   Set the BC into a cellwise view of the current system.
  *          Case of Face-based schemes
  *
  * \param[in]      cm          pointer to a cellwise view of the mesh
- * \param[in]      connect     pointer to a cs_cdo_connect_t struct.
- * \param[in]      quant       pointer to a cs_cdo_quantities_t structure
  * \param[in]      eqp         pointer to a cs_equation_param_t structure
- * \param[in]      dir_values  Dirichlet values associated to each vertex
- * \param[in]      neu_tags    definition id related to each Neumann face
- * \param[in]      t_eval      time at which one performs the evaluation
+ * \param[in]      face_bc     pointer to a cs_cdo_bc_face_t structure
+ * \param[in]      dir_values  Dirichlet values associated to each face
  * \param[in, out] csys        pointer to a cellwise view of the system
  * \param[in, out] cb          pointer to a cellwise builder
  */
@@ -120,12 +180,9 @@ cs_equation_vb_set_cell_bc(const cs_cell_mesh_t         *cm,
 
 void
 cs_equation_fb_set_cell_bc(const cs_cell_mesh_t         *cm,
-                           const cs_cdo_connect_t       *connect,
-                           const cs_cdo_quantities_t    *quant,
                            const cs_equation_param_t    *eqp,
+                           const cs_cdo_bc_face_t       *face_bc,
                            const cs_real_t               dir_values[],
-                           const short int               neu_tags[],
-                           cs_real_t                     t_eval,
                            cs_cell_sys_t                *csys,
                            cs_cell_builder_t            *cb);
 
@@ -134,25 +191,27 @@ cs_equation_fb_set_cell_bc(const cs_cell_mesh_t         *cm,
  * \brief   Compute the values of the Dirichlet BCs when DoFs are attached to
  *          vertices
  *
+ * \param[in]      t_eval      time at which one performs the evaluation
  * \param[in]      mesh        pointer to a cs_mesh_t structure
  * \param[in]      quant       pointer to a cs_cdo_quantities_t structure
  * \param[in]      connect     pointer to a cs_cdo_connect_t struct.
  * \param[in]      eqp         pointer to a cs_equation_param_t
- * \param[in]      face_bc     pointer to a cs_cdo_bc_t structure
- * \param[in]      t_eval      time at which one performs the evaluation
+ * \param[in]      face_bc     pointer to a cs_cdo_bc_face_t structure
  * \param[in, out] cb          pointer to a cs_cell_builder_t structure
+ * \param[in, out] bcflag      pointer to an array storing type of BC
  * \param[in, out] values      pointer to the array of values to set
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_equation_compute_dirichlet_vb(const cs_mesh_t            *mesh,
+cs_equation_compute_dirichlet_vb(cs_real_t                   t_eval,
+                                 const cs_mesh_t            *mesh,
                                  const cs_cdo_quantities_t  *quant,
                                  const cs_cdo_connect_t     *connect,
                                  const cs_equation_param_t  *eqp,
-                                 const cs_cdo_bc_t          *face_bc,
-                                 cs_real_t                   t_eval,
+                                 const cs_cdo_bc_face_t     *face_bc,
                                  cs_cell_builder_t          *cb,
+                                 cs_flag_t                  *bcflag,
                                  cs_real_t                  *values);
 
 /*----------------------------------------------------------------------------*/
@@ -164,7 +223,7 @@ cs_equation_compute_dirichlet_vb(const cs_mesh_t            *mesh,
  * \param[in]      quant      pointer to a cs_cdo_quantities_t structure
  * \param[in]      connect    pointer to a cs_cdo_connect_t struct.
  * \param[in]      eqp        pointer to a cs_equation_param_t
- * \param[in]      face_bc    pointer to a cs_cdo_bc_t structure
+ * \param[in]      face_bc    pointer to a cs_cdo_bc_face_t structure
  * \param[in]      t_eval     time at which one evaluates the boundary cond.
  * \param[in, out] cb         pointer to a cs_cell_builder_t structure
  * \param[in, out] values     pointer to the array of values to set
@@ -176,7 +235,7 @@ cs_equation_compute_dirichlet_fb(const cs_mesh_t            *mesh,
                                  const cs_cdo_quantities_t  *quant,
                                  const cs_cdo_connect_t     *connect,
                                  const cs_equation_param_t  *eqp,
-                                 const cs_cdo_bc_t          *face_bc,
+                                 const cs_cdo_bc_face_t     *face_bc,
                                  cs_real_t                   t_eval,
                                  cs_cell_builder_t          *cb,
                                  cs_real_t                  *values);
@@ -186,58 +245,55 @@ cs_equation_compute_dirichlet_fb(const cs_mesh_t            *mesh,
  * \brief   Define an array of flags for each vertex collecting the flags
  *          of associated boundary faces
  *
- * \param[in]  connect    pointer to a \ref cs_cdo_connect_t struct.
- * \param[in]  face_bc    pointer to a structure collecting boundary conditions
- *                        applied to faces
- *
- * \return an array collecting bc flags
+ * \param[in]      connect   pointer to a \ref cs_cdo_connect_t struct.
+ * \param[in]      face_bc   pointer to a structure collecting boundary
+ *                           conditions applied to faces
+ * \param[in, out] vflag     BC flag on vertices to define
  */
 /*----------------------------------------------------------------------------*/
 
-cs_flag_t *
+void
 cs_equation_set_vertex_bc_flag(const cs_cdo_connect_t     *connect,
-                               const cs_cdo_bc_t          *face_bc);
+                               const cs_cdo_bc_face_t     *face_bc,
+                               cs_flag_t                  *vflag);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief   Tag each face related to a Neumann BC with its definition id.
- *          Default tag is -1 (not a Neumann face)
+ * \brief   Define an array of flags for each edge collecting the flags
+ *          of associated boundary faces
  *
- * \param[in]      quant       pointer to a cs_cdo_quantities_t structure
- * \param[in]      eqp        pointer to a cs_equation_param_t
-
- * \return an array with prescribed tags
+ * \param[in]      connect     pointer to a \ref cs_cdo_connect_t struct.
+ * \param[in]      face_bc     pointer to a structure collecting boundary
+ *                             conditions applied to faces
+ * \param[in, out] edge_flag   BC flag on edges to define
  */
 /*----------------------------------------------------------------------------*/
 
-short int *
-cs_equation_tag_neumann_face(const cs_cdo_quantities_t    *quant,
-                             const cs_equation_param_t    *eqp);
+void
+cs_equation_set_edge_bc_flag(const cs_cdo_connect_t     *connect,
+                             const cs_cdo_bc_face_t     *face_bc,
+                             cs_flag_t                  *edge_flag);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief   Compute the values of the Neumann BCs when DoFs are scalar-valued
  *          and attached to vertices.
  *
+ * \param[in]      t_eval      time at which one performs the evaluation
  * \param[in]      def_id      id of the definition for setting the Neumann BC
  * \param[in]      f           local face number in the cs_cell_mesh_t
- * \param[in]      quant       pointer to a cs_cdo_quantities_t structure
  * \param[in]      eqp         pointer to a cs_equation_param_t
  * \param[in]      cm          pointer to a cs_cell_mesh_t structure
- * \param[in]      f2v_ids     vertex ids of the face in the cell numbering
- * \param[in]      t_eval      time at which one performs the evaluation
  * \param[in, out] neu_values  array storing the Neumann values
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_equation_compute_neumann_sv(short int                   def_id,
+cs_equation_compute_neumann_sv(cs_real_t                   t_eval,
+                               short int                   def_id,
                                short int                   f,
-                               const cs_cdo_quantities_t  *quant,
                                const cs_equation_param_t  *eqp,
                                const cs_cell_mesh_t       *cm,
-                               const short int            *f2v_ids,
-                               cs_real_t                   t_eval,
                                double                     *neu_values);
 
 /*----------------------------------------------------------------------------*/
@@ -245,25 +301,67 @@ cs_equation_compute_neumann_sv(short int                   def_id,
  * \brief   Compute the values of the Neumann BCs when DoFs are attached to
  *          faces.
  *
+ * \param[in]      t_eval      time at which one performs the evaluation
  * \param[in]      def_id      id of the definition for setting the Neumann BC
  * \param[in]      f           local face number in the cs_cell_mesh_t
- * \param[in]      quant       pointer to a cs_cdo_quantities_t structure
  * \param[in]      eqp         pointer to a cs_equation_param_t
  * \param[in]      cm          pointer to a cs_cell_mesh_t structure
- * \param[in]      t_eval      time at which one performs the evaluation
  * \param[in, out] neu_values  array storing Neumann values to use
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_equation_compute_neumann_fb(short int                    def_id,
+cs_equation_compute_neumann_fb(cs_real_t                    t_eval,
+                               short int                    def_id,
                                short int                    f,
-                               const cs_cdo_quantities_t   *quant,
                                const cs_equation_param_t   *eqp,
                                const cs_cell_mesh_t        *cm,
-                               cs_real_t                    t_eval,
                                double                      *neu_values);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Compute the values of the Robin BCs
+ *
+ * \param[in]      t_eval      time at which one performs the evaluation
+ * \param[in]      def_id      id of the definition for setting the Neumann BC
+ * \param[in]      f           local face number in the cs_cell_mesh_t
+ * \param[in]      eqp         pointer to a cs_equation_param_t
+ * \param[in]      cm          pointer to a cs_cell_mesh_t structure
+ * \param[in, out] rob_values  array storing Robin values to use
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_equation_compute_robin(cs_real_t                    t_eval,
+                          short int                    def_id,
+                          short int                    f,
+                          const cs_equation_param_t   *eqp,
+                          const cs_cell_mesh_t        *cm,
+                          double                      *rob_values);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Compute the values of the tangential component lying on the domain
+ *          boundary. Kind of BCs used when DoFs are attached to CDO (primal)
+ *          edge-based schemes. One sets the values of the circulation.
+ *
+ * \param[in]      t_eval     time at which one evaluates the boundary cond.
+ * \param[in]      mesh       pointer to a cs_mesh_t structure
+ * \param[in]      quant      pointer to a cs_cdo_quantities_t structure
+ * \param[in]      connect    pointer to a cs_cdo_connect_t struct.
+ * \param[in]      eqp        pointer to a cs_equation_param_t
+ * \param[in, out] values     pointer to the array of values to set
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_equation_compute_circulation_eb(cs_real_t                    t_eval,
+                                   const cs_mesh_t             *mesh,
+                                   const cs_cdo_quantities_t   *quant,
+                                   const cs_cdo_connect_t      *connect,
+                                   const cs_equation_param_t   *eqp,
+                                   cs_real_t                   *values);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
