@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,7 +32,6 @@
 #include "cs_equation_param.h"
 #include "cs_equation_common.h"
 #include "cs_field.h"
-#include "cs_param.h"
 #include "cs_restart.h"
 
 /*----------------------------------------------------------------------------*/
@@ -84,6 +83,69 @@ typedef void *
 typedef void *
 (cs_equation_free_context_t)(void  *scheme_context);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Initialize the variable field values related to an equation
+ *
+ * \param[in]      t_eval     time at which one performs the evaluation
+ * \param[in]      field_id   id related to the variable field of this equation
+ * \param[in]      mesh       pointer to a cs_mesh_t structure
+ * \param[in]      eqp        pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
+ * \param[in, out] context    pointer to the scheme context (cast on-the-fly)
+ */
+/*----------------------------------------------------------------------------*/
+
+typedef void
+(cs_equation_init_values_t)(cs_real_t                     t_eval,
+                            const int                     field_id,
+                            const cs_mesh_t              *mesh,
+                            const cs_equation_param_t    *eqp,
+                            cs_equation_builder_t        *eqb,
+                            void                         *context);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Build and solve a linear system within the CDO framework
+ *
+ * \param[in]      cur2prev   true="current to previous" operation is performed
+ * \param[in]      mesh       pointer to a \ref cs_mesh_t structure
+ * \param[in]      field_id   id related to the variable field of this equation
+ * \param[in]      eqp        pointer to a \ref cs_equation_param_t structure
+ * \param[in, out] eqb        pointer to a \ref cs_equation_builder_t structure
+ * \param[in, out] eqc        pointer to a scheme context structure
+ */
+/*----------------------------------------------------------------------------*/
+
+typedef void
+(cs_equation_solve_t)(bool                        cur2prev,
+                      const cs_mesh_t            *mesh,
+                      const int                   field_id,
+                      const cs_equation_param_t  *eqp,
+                      cs_equation_builder_t      *eqb,
+                      void                       *eqc);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set the Dirichlet boundary stemming from the settings.
+ *
+ * \param[in]      t_eval      time at which one evaluates BCs
+ * \param[in]      mesh        pointer to a cs_mesh_t structure
+ * \param[in]      eqp         pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb         pointer to a cs_equation_builder_t structure
+ * \param[in, out] context     pointer to the scheme context (cast on-the-fly)
+ * \param[in, out] field_val   pointer to the values of the variable field
+ */
+/*----------------------------------------------------------------------------*/
+
+typedef void
+(cs_equation_set_dir_bc_t)(cs_real_t                     t_eval,
+                           const cs_mesh_t              *mesh,
+                           const cs_equation_param_t    *eqp,
+                           cs_equation_builder_t        *eqb,
+                           void                         *context,
+                           cs_real_t                     field_val[]);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Create the matrix of the current algebraic system.
@@ -105,32 +167,12 @@ typedef void
                                   cs_matrix_t                **system_matrix,
                                   cs_real_t                  **system_rhs);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Set the Dirichlet boundary stemming from the settings.
- *
- * \param[in]      mesh        pointer to a cs_mesh_t structure
- * \param[in]      eqp         pointer to a cs_equation_param_t structure
- * \param[in, out] eqb         pointer to a cs_equation_builder_t structure
- * \param[in]      t_eval      time at which one evaluates BCs
- * \param[in, out] field_val   pointer to the values of the variable field
- */
-/*----------------------------------------------------------------------------*/
-
-typedef void
-(cs_equation_set_dir_bc_t)(const cs_mesh_t              *mesh,
-                           const cs_equation_param_t    *eqp,
-                           cs_equation_builder_t        *eqb,
-                           cs_real_t                     t_eval,
-                           cs_real_t                     field_val[]);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Build a linear system within the CDO framework
  *
  * \param[in]      m          pointer to a \ref cs_mesh_t structure
  * \param[in]      field_val  pointer to the current value of the field
- * \param[in]      dt_cur     current value of the time step
  * \param[in]      eqp        pointer to a \ref cs_equation_param_t structure
  * \param[in, out] eqb        pointer to a \ref cs_equation_builder_t structure
  * \param[in, out] data       pointer to a scheme builder structure
@@ -142,34 +184,12 @@ typedef void
 typedef void
 (cs_equation_build_system_t)(const cs_mesh_t            *mesh,
                              const cs_real_t            *field_val,
-                             double                      dt_cur,
                              const cs_equation_param_t  *eqp,
                              cs_equation_builder_t      *eqb,
                              void                       *data,
                              cs_real_t                  *rhs,
                              cs_matrix_t                *matrix);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Build and solve a linear system within the CDO framework
- *
- * \param[in]      dt_cur     current value of the time step
- * \param[in]      mesh       pointer to a \ref cs_mesh_t structure
- * \param[in]      field_id   id related to the variable field of this equation
- * \param[in]      eqp        pointer to a \ref cs_equation_param_t structure
- * \param[in, out] eqb        pointer to a \ref cs_equation_builder_t structure
- * \param[in, out] eqc        pointer to a scheme context structure
- */
-/*----------------------------------------------------------------------------*/
-
-typedef void
-(cs_equation_solve_t)(double                      dt_cur,
-                      const cs_mesh_t            *mesh,
-                      const int                   field_id,
-                      const cs_equation_param_t  *eqp,
-                      cs_equation_builder_t      *eqb,
-                      void                       *eqc);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Carry out operations for allocating and/or initializing the solution
@@ -216,8 +236,7 @@ typedef void
  *
  * \param[in]      eqp             pointer to a \ref cs_equation_param_t
  * \param[in, out] eqb             pointer to a \ref cs_equation_builder_t
- * \param[in, out] context         pointer to a scheme builder structure
- * \param[in]      dt_cur          current value of the time step
+ * \param[in, out] context         pointer to a scheme context structure
  *
  * \return a pointer to a cs_equation_balance_t structure
  */
@@ -226,76 +245,36 @@ typedef void
 typedef cs_equation_balance_t *
 (cs_equation_get_balance_t)(const cs_equation_param_t    *eqp,
                             cs_equation_builder_t        *eqb,
-                            void                         *context,
-                            cs_real_t                     dt_cur);
+                            void                         *context);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Compute the diffusive and convective flux across a list of faces
+ * \brief  Generic prototype for extra-operations related to an equation
  *
- * \param[in]       normal     indicate in which direction flux is > 0
- * \param[in]       pdi        pointer to an array of field values
- * \param[in]       ml_id      id related to a cs_mesh_location_t struct.
  * \param[in]       eqp        pointer to a cs_equation_param_t structure
  * \param[in, out]  eqb        pointer to a cs_equation_builder_t structure
- * \param[in, out]  data       pointer to data specific for this scheme
- * \param[in, out]  d_flux     pointer to the value of the diffusive flux
- * \param[in, out]  c_flux     pointer to the value of the convective flux
+ * \param[in, out]  context    pointer to a generic data structure
  */
 /*----------------------------------------------------------------------------*/
 
 typedef void
-(cs_equation_flux_plane_t)(const cs_real_t             normal[],
-                           const cs_real_t            *pdi,
-                           int                         ml_id,
-                           const cs_equation_param_t  *eqp,
-                           cs_equation_builder_t      *eqb,
-                           void                       *data,
-                           double                     *d_flux,
-                           double                     *c_flux);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Cellwise computation of the diffusive flux across all faces.
- *         Primal or dual faces are considered according to the space scheme
- *
- * \param[in]       fvals       pointer to an array of field values
- * \param[in]       eqp         pointer to a cs_equation_param_t structure
- * \param[in]       t_eval      time at which one performs the evaluation
- * \param[in, out]  eqb         pointer to a cs_equation_builder_t structure
- * \param[in, out]  data        pointer to a generic data structure
- * \param[in, out]  location    where the flux is defined
- * \param[in, out]  diff_flux   pointer to the value of the diffusive flux
- */
-/*----------------------------------------------------------------------------*/
-
-typedef void
-(cs_equation_cell_difflux_t)(const cs_real_t            *fvals,
-                             const cs_equation_param_t  *eqp,
-                             cs_real_t                   t_eval,
-                             cs_equation_builder_t      *eqb,
-                             void                       *data,
-                             cs_flag_t                   location,
-                             cs_real_t                  *d_flux);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Extra-operation related to this equation
- *
- * \param[in]       eqname     name of the equation
- * \param[in]       field      pointer to a field structure
- * \param[in]       eqp        pointer to a cs_equation_param_t structure
- * \param[in, out]  eqb        pointer to a cs_equation_builder_t structure
- * \param[in, out]  data       pointer to a generic data structure
- */
-/*----------------------------------------------------------------------------*/
-
-typedef void
-(cs_equation_extra_op_t)(const char                 *eqname,
-                         const cs_field_t           *field,
-                         const cs_equation_param_t  *eqp,
+(cs_equation_extra_op_t)(const cs_equation_param_t  *eqp,
                          cs_equation_builder_t      *eqb,
-                         void                       *data);
+                         void                       *context);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve cellwise structure including work buffers used to build
+ *         a CDO system cellwise. Generic prototype for all CDO schemes.
+ *
+ * \param[out]  csys   pointer to a pointer on a cs_cell_sys_t structure
+ * \param[out]  cb     pointer to a pointer on a cs_cell_builder_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+typedef void
+(cs_equation_get_builders_t)(cs_cell_sys_t       **csys,
+                             cs_cell_builder_t   **cb);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -305,13 +284,15 @@ typedef void
  *         have to free the return pointer.
  *
  * \param[in, out]  scheme_context  pointer to a data structure cast on-the-fly
+ * \param[in]       previous        retrieve the previous state (true/false)
  *
  * \return  a pointer to an array of \ref cs_real_t
  */
 /*----------------------------------------------------------------------------*/
 
 typedef cs_real_t *
-(cs_equation_get_values_t)(void      *scheme_context);
+(cs_equation_get_values_t)(void      *scheme_context,
+                           bool       previous);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -353,7 +334,7 @@ struct _cs_equation_t {
   /* Algebraic system */
   /* ---------------- */
 
-  /* There are possibly two different sizes for the linear system to handle
+  /* There can be two different sizes for the linear system to handle
      - One for "scatter"-type operations based on the number of geometrical
        entities owned by the local instance of the mesh
      - One for "gather"-type operations based on a balance of the number of
@@ -388,28 +369,36 @@ struct _cs_equation_t {
   /* Pointer to functions (see prototypes just above) */
   cs_equation_init_context_t       *init_context;
   cs_equation_free_context_t       *free_context;
+
+  cs_equation_init_values_t        *init_field_values;
+  cs_equation_solve_t              *solve_steady_state;
+  cs_equation_solve_t              *solve;
+
+  cs_equation_get_balance_t        *compute_balance;
+  cs_equation_extra_op_t           *postprocess;
+  cs_equation_extra_op_t           *current_to_previous;
+
+  cs_equation_restart_t            *read_restart;
+  cs_equation_restart_t            *write_restart;
+
+  cs_equation_get_values_t         *get_cell_values;
+  cs_equation_get_values_t         *get_face_values;
+  cs_equation_get_values_t         *get_edge_values;
+  cs_equation_get_values_t         *get_vertex_values;
+
+  cs_equation_get_builders_t       *get_cw_build_structures;
+
+  /* Deprecated functions --> use rather solve() and solve_steady_state() */
+  /* -------------------- */
   cs_equation_initialize_system_t  *initialize_system;
   cs_equation_set_dir_bc_t         *set_dir_bc;
   cs_equation_build_system_t       *build_system;
   cs_equation_prepare_solve_t      *prepare_solving;
   cs_equation_update_field_t       *update_field;
-  cs_equation_solve_t              *solve_steady_state;
-  cs_equation_solve_t              *solve;
 
-  cs_equation_get_balance_t        *compute_balance;
-  cs_equation_flux_plane_t         *compute_flux_across_plane;
-  cs_equation_cell_difflux_t       *compute_cellwise_diff_flux;
-  cs_equation_extra_op_t           *postprocess;
-  cs_equation_restart_t            *read_restart;
-  cs_equation_restart_t            *write_restart;
+  /* Timer statistic for a coarse profiling */
+  int     main_ts_id;   /* Id of the main timer stats for this equation */
 
-  cs_equation_get_values_t         *get_face_values;
-  cs_equation_get_values_t         *get_cell_values;
-  cs_equation_get_values_t         *get_vertex_values;
-
-  /* Timer statistic for a "light" profiling */
-  int     main_ts_id;   /* Id of the main timer states structure related
-                           to this equation */
 };
 
 /*============================================================================
