@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -44,32 +44,21 @@ BEGIN_C_DECLS
  * Macro definitions
  *============================================================================*/
 
+#define CS_GWF_ADV_FIELD_NAME   "darcy_velocity"
+
+typedef cs_flag_t  cs_gwf_model_t;
+
 /*!
  * @name Flags specifying the general behavior of the groundwater flow module
  * @{
  *
- * \def CS_GWF_FORCE_RICHARDS_ITERATIONS
- * \brief Even if the Richards equation is steady-state, this equation is
- *        solved at each iteration.
- *
+ * \enum cs_gwf_model_bit_t
+ * \brief elemental modelling choice either from the physical viewpoint or the
+ * numerical viewpoint
+
  * \def CS_GWF_GRAVITATION
  * \brief Gravitation effects are taken into account in the Richards equation
  *
- * \def CS_GWF_POST_CAPACITY
- * \brief Activate the post-processing of the capacity (property in front of
- *        the unsteady term in Richards equation)
- *
- * \def CS_GWF_POST_MOISTURE
- * \brief Activate the post-processing of the moisture content
- *
- * \def CS_GWF_POST_PERMEABILITY
- * \brief Activate the post-processing of the permeability field
- *
- * \def CS_GWF_RESCALE_HEAD_TO_ZERO_MEAN_VALUE
- * \brief Compute the mean-value of the hydraulic head field and subtract this
- *        mean-value to get a field with zero mean-value. It's important to set
- *        this flag if no boundary condition is given.
- *
  * \def CS_GWF_RICHARDS_UNSTEADY
  * \brief Richards equation is unsteady (unsatured behavior)
  *
@@ -82,17 +71,76 @@ BEGIN_C_DECLS
  *        can be performed in this case)
  *
  *
+ * \def CS_GWF_FORCE_RICHARDS_ITERATIONS
+ * \brief Even if the Richards equation is steady-state, this equation is
+ *        solved at each iteration.
+ *
+ * \def CS_GWF_RESCALE_HEAD_TO_ZERO_MEAN_VALUE
+ * \brief Compute the mean-value of the hydraulic head field and subtract this
+ *        mean-value to get a field with zero mean-value. It's important to set
+ *        this flag if no boundary condition is given.
+ *
+ * \def CS_GWF_ENFORCE_DIVERGENCE_FREE
+ * \brief Activate a treatment to enforce a Darcy flux to be divergence-free
+ *
+ *
  */
 
-#define CS_GWF_FORCE_RICHARDS_ITERATIONS       (1 << 0)
-#define CS_GWF_GRAVITATION                     (1 << 1)
-#define CS_GWF_POST_CAPACITY                   (1 << 2)
-#define CS_GWF_POST_MOISTURE                   (1 << 3)
-#define CS_GWF_POST_PERMEABILITY               (1 << 4)
-#define CS_GWF_RESCALE_HEAD_TO_ZERO_MEAN_VALUE (1 << 5)
-#define CS_GWF_RICHARDS_UNSTEADY               (1 << 6)
-#define CS_GWF_SOIL_PROPERTY_UNSTEADY          (1 << 7)
-#define CS_GWF_SOIL_ALL_SATURATED              (1 << 8)
+typedef enum {
+
+  /* Main physical modelling */
+  /* ----------------------- */
+
+  CS_GWF_GRAVITATION                     = 1<< 0, /* =   1 */
+  CS_GWF_RICHARDS_UNSTEADY               = 1<< 1, /* =   2 */
+  CS_GWF_SOIL_PROPERTY_UNSTEADY          = 1<< 2, /* =   4 */
+  CS_GWF_SOIL_ALL_SATURATED              = 1<< 3, /* =   8 */
+
+
+  /* Main numerical options */
+  /* ---------------------- */
+
+  CS_GWF_FORCE_RICHARDS_ITERATIONS       = 1<< 6, /* =   64 */
+  CS_GWF_RESCALE_HEAD_TO_ZERO_MEAN_VALUE = 1<< 7, /* =  128 */
+  CS_GWF_ENFORCE_DIVERGENCE_FREE         = 1<< 8  /* =  256 */
+
+} cs_gwf_model_bit_t;
+
+/*! @}
+ *!
+ * @name Flags specifying the kind of post-processing to perform in
+ *       the groundwater flow module
+ * @{
+ *
+ * \def CS_GWF_POST_CAPACITY
+ * \brief Activate the post-processing of the capacity (property in front of
+ *        the unsteady term in Richards equation)
+ *
+ * \def CS_GWF_POST_MOISTURE
+ * \brief Activate the post-processing of the moisture content
+ *
+ * \def CS_GWF_POST_PERMEABILITY
+ * \brief Activate the post-processing of the permeability field
+ *
+ * \def CS_GWF_POST_DARCY_FLUX_BALANCE
+ * \brief Compute the overall balance at the different boundaries of
+ *        the Darcy flux
+ *
+ * \def CS_GWF_POST_DARCY_FLUX_DIVERGENCE
+ * \brief Compute in each control volume (vertices or cells w.r.t the space
+ *        scheme) the divergence of the Darcy flux
+ *
+ * \def CS_GWF_POST_DARCY_FLUX_AT_BOUNDARY
+ * \brief Define a field at boundary faces for the Darcy flux and activate the
+ *        post-processing
+ */
+
+#define CS_GWF_POST_CAPACITY                   (1 << 0)
+#define CS_GWF_POST_MOISTURE                   (1 << 1)
+#define CS_GWF_POST_PERMEABILITY               (1 << 2)
+#define CS_GWF_POST_DARCY_FLUX_BALANCE         (1 << 3)
+#define CS_GWF_POST_DARCY_FLUX_DIVERGENCE      (1 << 4)
+#define CS_GWF_POST_DARCY_FLUX_AT_BOUNDARY     (1 << 5)
 
 /*! @} */
 
@@ -154,27 +202,14 @@ cs_gwf_log_setup(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Activate the gravity and set the gravitaty vector
-
- * \param[in]       gvec      values of the gravity vector
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_gwf_set_gravity_vector(const cs_real_3_t      gvec);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Advanced setting: indicate where the darcian flux is stored
- *         cs_flag_primal_cell is the default setting
- *         cs_flag_dual_face_byc is a valid choice for vertex-based schemes
+ * \brief  Set the flag dedicated to the post-processing of the GWF module
  *
- * \param[in] location_flag   where the flux is defined
+ * \param[in]  post_flag             flag to set
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_gwf_set_darcian_flux_location(cs_flag_t      location_flag);
+cs_gwf_set_post_options(cs_flag_t       post_flag);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -183,15 +218,17 @@ cs_gwf_set_darcian_flux_location(cs_flag_t      location_flag);
  *         reaction eq.
  *         Tracer is advected thanks to the darcian velocity and
  *         diffusion/reaction parameters result from a physical modelling.
- *         Terms are activated according to the settings.
+ *         Terms solved in the equation are activated according to the settings.
  *
+ * \param[in]  model      physical modelling to consider (0 = default settings)
  * \param[in]  eq_name    name of the tracer equation
  * \param[in]  var_name   name of the related variable
  */
 /*----------------------------------------------------------------------------*/
 
 cs_gwf_tracer_t *
-cs_gwf_add_tracer(const char               *eq_name,
+cs_gwf_add_tracer(cs_gwf_tracer_model_t     model,
+                  const char               *eq_name,
                   const char               *var_name);
 
 /*----------------------------------------------------------------------------*/
@@ -212,7 +249,7 @@ cs_gwf_add_tracer(const char               *eq_name,
 /*----------------------------------------------------------------------------*/
 
 cs_gwf_tracer_t *
-cs_gwf_add_tracer_user(const char                  *eq_name,
+cs_gwf_add_user_tracer(const char                  *eq_name,
                        const char                  *var_name,
                        cs_gwf_tracer_setup_t       *setup,
                        cs_gwf_tracer_add_terms_t   *add_terms);
@@ -274,7 +311,6 @@ cs_gwf_finalize_setup(const cs_cdo_connect_t     *connect,
  * \param[in]  connect    pointer to a cs_cdo_connect_t structure
  * \param[in]  quant      pointer to a cs_cdo_quantities_t structure
  * \param[in]  ts         pointer to a cs_time_step_t structure
- * \param[in]  dt_cur     current value of the time step
  * \param[in]  cur2prev   true or false
  */
 /*----------------------------------------------------------------------------*/
@@ -284,7 +320,6 @@ cs_gwf_update(const cs_mesh_t             *mesh,
               const cs_cdo_connect_t      *connect,
               const cs_cdo_quantities_t   *quant,
               const cs_time_step_t        *ts,
-              double                       dt_cur,
               bool                         cur2prev);
 
 /*----------------------------------------------------------------------------*/
@@ -311,7 +346,6 @@ cs_gwf_compute_steady_state(const cs_mesh_t              *mesh,
  *
  * \param[in]      mesh       pointer to a cs_mesh_t structure
  * \param[in]      time_step  pointer to a cs_time_step_t structure
- * \param[in]      dt_cur     current value of the time step
  * \param[in]      connect    pointer to a cs_cdo_connect_t structure
  * \param[in]      cdoq       pointer to a cs_cdo_quantities_t structure
  */
@@ -320,7 +354,6 @@ cs_gwf_compute_steady_state(const cs_mesh_t              *mesh,
 void
 cs_gwf_compute(const cs_mesh_t              *mesh,
                const cs_time_step_t         *time_step,
-               double                        dt_cur,
                const cs_cdo_connect_t       *connect,
                const cs_cdo_quantities_t    *cdoq);
 
@@ -346,6 +379,19 @@ cs_gwf_integrate_tracer(const cs_cdo_connect_t     *connect,
                         const cs_gwf_tracer_t      *tracer,
                         const char                 *z_name);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Predefined extra-operations for the groundwater flow module
+ *
+ * \param[in]  connect   pointer to a cs_cdo_connect_t structure
+ * \param[in]  cdoq      pointer to a cs_cdo_quantities_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_gwf_extra_op(const cs_cdo_connect_t      *connect,
+                const cs_cdo_quantities_t   *cdoq);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Predefined post-processing output for the groundwater flow module
