@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -49,7 +49,7 @@
 #include "cs_math.h"
 #include "cs_mesh_location.h"
 #include "cs_parall.h"
-#include "cs_param.h"
+#include "cs_param_types.h"
 #include "cs_post.h"
 #include "cs_prototypes.h"
 #include "cs_reco.h"
@@ -127,10 +127,10 @@ _free_saturated_soil(void       *input)
  *         a soil with a saturated.
  *         Case of an isotropic permeability and an unsteady Richards eq.
  *
+ * \param[in]      t_eval       time at which one performs the evaluation
  * \param[in]      mesh         pointer to a cs_mesh_t structure
  * \param[in]      connect      pointer to a cs_cdo_connect_t structure
  * \param[in]      quant        pointer to a cs_cdo_quantities_t structure
- * \param[in]      ts           pointer to a cs_time_step_t structure
  * \param[in]      head_values  array of values for head used in law
  * \param[in]      zone         pointer to a cs_zone_t
  * \param[in, out] input        pointer to a structure cast on-the-fly
@@ -138,10 +138,10 @@ _free_saturated_soil(void       *input)
 /*----------------------------------------------------------------------------*/
 
 static inline void
-_update_saturated_iso_soil(const cs_mesh_t             *mesh,
+_update_saturated_iso_soil(const cs_real_t              t_eval,
+                           const cs_mesh_t             *mesh,
                            const cs_cdo_connect_t      *connect,
                            const cs_cdo_quantities_t   *quant,
-                           const cs_time_step_t        *ts,
                            const cs_real_t             *head_values,
                            const cs_zone_t             *zone,
                            void                        *input)
@@ -149,7 +149,7 @@ _update_saturated_iso_soil(const cs_mesh_t             *mesh,
   CS_UNUSED(mesh);
   CS_UNUSED(connect);
   CS_UNUSED(quant);
-  CS_UNUSED(ts);
+  CS_UNUSED(t_eval);
   CS_UNUSED(head_values);
 
   /* Retrieve field values associated to properties to update */
@@ -164,8 +164,9 @@ _update_saturated_iso_soil(const cs_mesh_t             *mesh,
 
   const  double  iso_satval = law->saturated_permeability[0][0];
 
-# pragma omp parallel for if (zone->n_elts > CS_THR_MIN) default(none)     \
-  shared(zone, law, permeability_values, moisture_values)
+# pragma omp parallel for if (zone->n_elts > CS_THR_MIN)                \
+  shared(zone, law, permeability_values, moisture_values)               \
+  firstprivate(iso_satval)
   for (cs_lnum_t i = 0; i < zone->n_elts; i++) {
 
     const cs_lnum_t  c_id = zone->elt_ids[i];
@@ -186,10 +187,10 @@ _update_saturated_iso_soil(const cs_mesh_t             *mesh,
  *         a soil with a saturated.
  *         Case of an anisotropic permeability and an unsteady Richards eq.
  *
+ * \param[in]      t_eval       time at which one performs the evaluation
  * \param[in]      mesh         pointer to a cs_mesh_t structure
  * \param[in]      connect      pointer to a cs_cdo_connect_t structure
  * \param[in]      quant        pointer to a cs_cdo_quantities_t structure
- * \param[in]      ts           pointer to a cs_time_step_t structure
  * \param[in]      head_values  array of values for head used in law
  * \param[in]      zone         pointer to a cs_zone_t
  * \param[in, out] input        pointer to a structure cast on-the-fly
@@ -197,18 +198,18 @@ _update_saturated_iso_soil(const cs_mesh_t             *mesh,
 /*----------------------------------------------------------------------------*/
 
 static inline void
-_update_saturated_aniso_soil(const cs_mesh_t             *mesh,
+_update_saturated_aniso_soil(const cs_real_t              t_eval,
+                             const cs_mesh_t             *mesh,
                              const cs_cdo_connect_t      *connect,
                              const cs_cdo_quantities_t   *quant,
-                             const cs_time_step_t        *ts,
                              const cs_real_t             *head_values,
                              const cs_zone_t             *zone,
                              void                        *input)
 {
+  CS_UNUSED(t_eval);
   CS_UNUSED(mesh);
   CS_UNUSED(connect);
   CS_UNUSED(quant);
-  CS_UNUSED(ts);
   CS_UNUSED(head_values);
 
   /* Retrieve field values associated to properties to update */
@@ -221,7 +222,7 @@ _update_saturated_aniso_soil(const cs_mesh_t             *mesh,
   const cs_gwf_soil_saturated_param_t  *law =
     (cs_gwf_soil_saturated_param_t *)input;
 
-# pragma omp parallel for if (zone->n_elts > CS_THR_MIN) default(none)     \
+# pragma omp parallel for if (zone->n_elts > CS_THR_MIN)        \
   shared(zone, law, permeability_values, moisture_values)
   for (cs_lnum_t id = 0; id < zone->n_elts; id++) {
 
@@ -262,10 +263,10 @@ _free_genuchten_soil(void          *input)
  *         a soil with a Van Genuchten-Mualen.
  *         Case of an isotropic permeability and an unsteady Richards eq.
  *
+ * \param[in]      t_eval       time at which one performs the evaluation
  * \param[in]      mesh         pointer to a cs_mesh_t structure
  * \param[in]      connect      pointer to a cs_cdo_connect_t structure
  * \param[in]      quant        pointer to a cs_cdo_quantities_t structure
- * \param[in]      ts           pointer to a cs_time_step_t structure
  * \param[in]      head_values  array of values for head used in law
  * \param[in]      zone         pointer to a cs_zone_t
  * \param[in, out] input        pointer to a structure cast on-the-fly
@@ -273,18 +274,18 @@ _free_genuchten_soil(void          *input)
 /*----------------------------------------------------------------------------*/
 
 static inline void
-_update_genuchten_iso_soil(const cs_mesh_t             *mesh,
+_update_genuchten_iso_soil(const cs_real_t              t_eval,
+                           const cs_mesh_t             *mesh,
                            const cs_cdo_connect_t      *connect,
                            const cs_cdo_quantities_t   *quant,
-                           const cs_time_step_t        *ts,
                            const cs_real_t             *head_values,
                            const cs_zone_t             *zone,
                            void                        *input)
 {
+  CS_UNUSED(t_eval);
   CS_UNUSED(mesh);
   CS_UNUSED(connect);
   CS_UNUSED(quant);
-  CS_UNUSED(ts);
 
   /* Sanity checks */
   assert(head_values != NULL);
@@ -308,9 +309,10 @@ _update_genuchten_iso_soil(const cs_mesh_t             *mesh,
   const  double  delta_moisture =
     law->saturated_moisture - law->residual_moisture;
 
-# pragma omp parallel for if (zone->n_elts > CS_THR_MIN) default(none)     \
-  shared(head_values, zone, law, permeability_values, moisture_values, \
-         capacity_values)
+# pragma omp parallel for if (zone->n_elts > CS_THR_MIN)                \
+  shared(head_values, zone, law, permeability_values, moisture_values,  \
+         capacity_values)                                               \
+  firstprivate(iso_satval, delta_moisture)
   for (cs_lnum_t i = 0; i < zone->n_elts; i++) {
 
     const cs_lnum_t  c_id = zone->elt_ids[i];
@@ -324,8 +326,8 @@ _update_genuchten_iso_soil(const cs_mesh_t             *mesh,
       const double  coef_base = 1 - pow(1 - se_pow_overm, law->m);
 
       /* Set the permeability value */
-      permeability_values[c_id] = iso_satval* pow(se, law->tortuosity) *
-        coef_base*coef_base;
+      permeability_values[c_id] =   iso_satval* pow(se, law->tortuosity)
+                                  * coef_base*coef_base;
 
       /* Set the moisture content */
       moisture_values[c_id] = se*delta_moisture + law->residual_moisture;
@@ -404,31 +406,26 @@ cs_gwf_soil_add(const char                      *z_name,
 
   case CS_GWF_SOIL_SATURATED:
 
-    switch (permeability->type) {
-    case CS_PROPERTY_ISO:
+    if (permeability->type & CS_PROPERTY_ISO)
       soil->update_properties = _update_saturated_iso_soil;
-      break;
-    case CS_PROPERTY_ANISO:
+    else if (permeability->type & CS_PROPERTY_ANISO)
       soil->update_properties = _update_saturated_aniso_soil;
-      break;
-    default:
+    else
       bft_error(__FILE__, __LINE__, 0,
-                " Invalid type of property for the permeability.\n"
-                " Please check your settings.");
-    }
+                "%s: Invalid type of property for the permeability.\n"
+                " Please check your settings.", __func__);
+
     soil->free_input = _free_saturated_soil;
     break;
 
   case CS_GWF_SOIL_GENUCHTEN:
-    switch (permeability->type) {
-    case CS_PROPERTY_ISO:
+
+    if (permeability->type & CS_PROPERTY_ISO)
       soil->update_properties = _update_genuchten_iso_soil;
-      break;
-    default:
+    else
       bft_error(__FILE__, __LINE__, 0,
-                " Invalid type of property for the permeability.\n"
-                " Please check your settings.");
-    }
+                "%s: Invalid type of property for the permeability.\n"
+                " Please check your settings.", __func__);
     soil->free_input = _free_genuchten_soil;
     break;
 
@@ -815,8 +812,8 @@ cs_gwf_soil_set_all_saturated(cs_property_t         *permeability,
 
     if (soil->model != CS_GWF_SOIL_SATURATED)
       bft_error(__FILE__, __LINE__, 0,
-                " Invalid way of setting soil parameter.\n"
-                " All soils are not considered as saturated.");
+                " %s: Invalid way of setting soil parameter.\n"
+                " All soils are not considered as saturated.", __func__);
 
     const cs_zone_t  *z = cs_volume_zone_by_id(soil->zone_id);
 
@@ -824,35 +821,28 @@ cs_gwf_soil_set_all_saturated(cs_property_t         *permeability,
       (cs_gwf_soil_saturated_param_t  *)soil->input;
 
     /* Set the permeability */
-    switch (permeability->type) {
-    case CS_PROPERTY_ISO:
+    if (permeability->type & CS_PROPERTY_ISO) {
       cs_property_def_iso_by_value(permeability,
                                    z->name,
                                    param->saturated_permeability[0][0]);
-      break;
+    }
+    else if (permeability->type & CS_PROPERTY_ORTHO) {
 
-    case CS_PROPERTY_ORTHO:
-      {
-        cs_real_3_t  val = {param->saturated_permeability[0][0],
-                            param->saturated_permeability[1][1],
-                            param->saturated_permeability[2][2]};
+      cs_real_3_t  val = {param->saturated_permeability[0][0],
+                          param->saturated_permeability[1][1],
+                          param->saturated_permeability[2][2]};
 
-        cs_property_def_ortho_by_value(permeability,
-                                       z->name,
-                                       val);
-      }
-      break;
+      cs_property_def_ortho_by_value(permeability, z->name, val);
 
-    case CS_PROPERTY_ANISO:
+    }
+    else if (permeability->type & CS_PROPERTY_ANISO) {
       cs_property_def_aniso_by_value(permeability,
                                      z->name,
                       (double (*)[3])param->saturated_permeability);
-      break;
-
-    default:
+    }
+    else
       bft_error(__FILE__, __LINE__, 0,
                 " %s: Invalid type of property.\n", __func__);
-    }
 
     /* Set the moisture content */
     cs_property_def_iso_by_value(moisture_content,
@@ -907,8 +897,8 @@ cs_gwf_build_cell2soil(cs_lnum_t    n_cells)
     for (cs_lnum_t j = 0; j < n_cells; j++)
       if (_cell2soil_ids[j] == -1)
         bft_error(__FILE__, __LINE__, 0,
-                  " %s: At least cell%d has no related soil.\n",
-                  __func__, j);
+                  " %s: At least cell %ld has no related soil.\n",
+                  __func__, (long)j);
 
   } /* n_soils > 1 */
 
@@ -969,15 +959,18 @@ cs_gwf_soil_set_by_field(cs_property_t     *permeability,
 void
 cs_gwf_soil_log_setup(void)
 {
-  const char  *meta = "  <GWF/Hydraulic Model>";
-  cs_log_printf(CS_LOG_SETUP, "  <GWF/Soils>  n_soils %d", _n_soils);
+  cs_log_printf(CS_LOG_SETUP, "  * GWF | Number of soils: %d\n", _n_soils);
 
+  char  meta[64];
   for (int i = 0; i < _n_soils; i++) {
 
     const cs_gwf_soil_t  *soil = _soils[i];
     const cs_zone_t  *z = cs_volume_zone_by_id(soil->zone_id);
 
-    cs_log_printf(CS_LOG_SETUP, "\n  <GWF/Soil %d> %s\n", soil->id, z->name);
+    cs_log_printf(CS_LOG_SETUP, "\n        Soil.%d | Zone: %s\n",
+                  soil->id, z->name);
+    sprintf(meta, "        Soil.%d | Model |", soil->id);
+
     switch (soil->model) {
 
     case CS_GWF_SOIL_GENUCHTEN:
@@ -986,23 +979,24 @@ cs_gwf_soil_log_setup(void)
           (cs_gwf_soil_genuchten_param_t *)soil->input;
 
         cs_log_printf(CS_LOG_SETUP, "%s VanGenuchten-Mualen\n", meta);
-        cs_log_printf(CS_LOG_SETUP, "    <Soil parameters>");
+        cs_log_printf(CS_LOG_SETUP, "%s Parameters:", meta);
         cs_log_printf(CS_LOG_SETUP,
                       " residual_moisture %5.3e", si->residual_moisture);
         cs_log_printf(CS_LOG_SETUP,
                       " saturated_moisture %5.3e\n", si->saturated_moisture);
-        cs_log_printf(CS_LOG_SETUP, "    <Soil parameters> n= %f, scale= %f,"
-                      "tortuosity= %f\n", si->n, si->scale, si->tortuosity);
-        cs_log_printf(CS_LOG_SETUP, "    <Soil saturated permeability>");
-        cs_log_printf(CS_LOG_SETUP,
-                      " [%-4.2e %4.2e %4.2e; %-4.2e %4.2e %4.2e;"
-                      " %-4.2e %4.2e %4.2e]",
+        cs_log_printf(CS_LOG_SETUP, "%s Parameters:", meta);
+        cs_log_printf(CS_LOG_SETUP, " n= %f, scale= %f, tortuosity= %f\n",
+                      si->n, si->scale, si->tortuosity);
+        cs_log_printf(CS_LOG_SETUP, "%s Saturated permeability\n", meta);
+        cs_log_printf(CS_LOG_SETUP, "%s [%-4.2e %4.2e %4.2e;\n", meta,
                       si->saturated_permeability[0][0],
                       si->saturated_permeability[0][1],
-                      si->saturated_permeability[0][2],
+                      si->saturated_permeability[0][2]);
+        cs_log_printf(CS_LOG_SETUP, "%s  %-4.2e %4.2e %4.2e;\n", meta,
                       si->saturated_permeability[1][0],
                       si->saturated_permeability[1][1],
-                      si->saturated_permeability[1][2],
+                      si->saturated_permeability[1][2]);
+        cs_log_printf(CS_LOG_SETUP, "%s  %-4.2e %4.2e %4.2e]\n", meta,
                       si->saturated_permeability[2][0],
                       si->saturated_permeability[2][1],
                       si->saturated_permeability[2][2]);
@@ -1014,20 +1008,20 @@ cs_gwf_soil_log_setup(void)
         const cs_gwf_soil_saturated_param_t  *si =
           (cs_gwf_soil_saturated_param_t *)soil->input;
 
-        cs_log_printf(CS_LOG_SETUP, "%s saturated\n", meta);
-        cs_log_printf(CS_LOG_SETUP, "    <Soil parameters>");
+        cs_log_printf(CS_LOG_SETUP, "%s Saturated\n", meta);
+        cs_log_printf(CS_LOG_SETUP, "%s Parameters", meta);
         cs_log_printf(CS_LOG_SETUP,
                       " saturated_moisture %5.3e\n", si->saturated_moisture);
-        cs_log_printf(CS_LOG_SETUP, "    <Soil saturated permeability>");
-        cs_log_printf(CS_LOG_SETUP,
-                      " [%-4.2e %4.2e %4.2e; %-4.2e %4.2e %4.2e;"
-                      " %-4.2e %4.2e %4.2e]",
+        cs_log_printf(CS_LOG_SETUP, "%s Saturated permeability\n", meta);
+        cs_log_printf(CS_LOG_SETUP, "%s [%-4.2e %4.2e %4.2e;\n", meta,
                       si->saturated_permeability[0][0],
                       si->saturated_permeability[0][1],
-                      si->saturated_permeability[0][2],
+                      si->saturated_permeability[0][2]);
+        cs_log_printf(CS_LOG_SETUP, "%s  %-4.2e %4.2e %4.2e;\n", meta,
                       si->saturated_permeability[1][0],
                       si->saturated_permeability[1][1],
-                      si->saturated_permeability[1][2],
+                      si->saturated_permeability[1][2]);
+        cs_log_printf(CS_LOG_SETUP, "%s  %-4.2e %4.2e %4.2e]\n", meta,
                       si->saturated_permeability[2][0],
                       si->saturated_permeability[2][1],
                       si->saturated_permeability[2][2]);
@@ -1035,7 +1029,7 @@ cs_gwf_soil_log_setup(void)
       break;
 
     case CS_GWF_SOIL_USER:
-      cs_log_printf(CS_LOG_SETUP, "%s user-defined\n", meta);
+      cs_log_printf(CS_LOG_SETUP, "%s **User-defined**\n", meta);
       break;
 
     default:
@@ -1047,6 +1041,7 @@ cs_gwf_soil_log_setup(void)
 
   } /* Loop on soils */
 
+  cs_log_printf(CS_LOG_SETUP, "\n");
 }
 
 /*----------------------------------------------------------------------------*/
