@@ -9,7 +9,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -41,9 +41,12 @@
 BEGIN_C_DECLS
 
 /*============================================================================
- * Macro definitions
+ * Typedef definition
  *============================================================================*/
 
+typedef cs_flag_t  cs_gwf_tracer_model_t;
+typedef struct _gwf_tracer_t  cs_gwf_tracer_t;
+
 /*============================================================================
  * Public function pointer prototypes
  *============================================================================*/
@@ -53,67 +56,148 @@ BEGIN_C_DECLS
  * \brief  Generic function to update the phisical properties related to a
  *         tracer modelling
  *
- * \param[in, out] input      pointer to a structure cast on-the-fly
+ * \param[in, out] tracer     pointer to a cs_gwf_tracer_structure
+ * \param[in]      t_eval     time at which one performs the evaluation
  * \param[in]      mesh       pointer to a cs_mesh_t structure
  * \param[in]      connect    pointer to a cs_cdo_connect_t structure
  * \param[in]      quant      pointer to a cs_cdo_quantities_t structure
- * \param[in]      t_eval     time at which one performs the evaluation
  */
 /*----------------------------------------------------------------------------*/
 
 typedef void
-(cs_gwf_tracer_update_t) (void                        *input,
+(cs_gwf_tracer_update_t) (cs_gwf_tracer_t             *tracer,
+                          cs_real_t                    t_eval,
                           const cs_mesh_t             *mesh,
                           const cs_cdo_connect_t      *connect,
-                          const cs_cdo_quantities_t   *quant,
-                          cs_real_t                    t_eval);
+                          const cs_cdo_quantities_t   *quant);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Generic function to free the input of a tracer model
  *
- * \param[in, out] input     pointer to a structure cast on-the-fly
+ * \param[in, out] tracer     pointer to a structure cs_gwf_tracer_t
  */
 /*----------------------------------------------------------------------------*/
 
 typedef void
-(cs_gwf_tracer_free_input_t) (void      *input);
+(cs_gwf_tracer_free_input_t) (cs_gwf_tracer_t      *tracer);
 
 /*============================================================================
  * Structure definitions
  *============================================================================*/
 
+
+/*!
+ * @name Flags specifying the general behavior of a tracer associated to
+ *       the groundwater flow module
+ * @{
+ *
+ * \enum cs_gwf_tracer_model_bit_t
+ * \brief elemental modelling choice either from the physical viewpoint or the
+ * numerical viewpoint for the transport of a tracer
+ *
+ * \def CS_GWF_TRACER_USER
+ * \brief user-defined tracer. All terms can be modified with user functions
+ *
+ * \def CS_GWF_TRACER_SORPTION_EK_3_PARAMETERS
+ * \brief Add the sorption phenomena to the default tracer equation. Case of
+ *        the EK model with 3 parameters. Sorption is assumed to be infinite
+ *
+ * \def CS_GWF_TRACER_SORPTION_EK_5_PARAMETERS
+ * \brief Add the sorption phenomena to the default tracer equation. Case of the
+ *        EK model with 5 parameters. Sorption is assumed to be finite.  An
+ *        additional equation related to the concentration of sorpted tracer in
+ *        the second kind of sites.
+ *
+ * \def CS_GWF_TRACER_PRECIPITATION
+ * \brief Add the precipitation phenomena to the default tracer equation
+ */
+
 /* Type of predefined modelling for the groundwater flows */
 typedef enum {
 
-  CS_GWF_TRACER_STANDARD, /* Default behavior of a tracer in groundwater flow
-                            module */
-  CS_GWF_TRACER_USER,     /* User-defined behavior */
-  CS_GWF_N_TRACER_MODELS
+     CS_GWF_TRACER_USER                        = 1<< 0, /* =    1 */
 
-} cs_gwf_tracer_model_t;
+     /* Physical phenomena to consider */
+     /* ------------------------------ */
+
+     CS_GWF_TRACER_SORPTION_EK_3_PARAMETERS    = 1<< 1, /* =    2 */
+     CS_GWF_TRACER_SORPTION_EK_5_PARAMETERS    = 1<< 2, /* =    4 */
+
+
+     CS_GWF_TRACER_PRECIPITATION               = 1<< 4, /* =    16 */
+
+} cs_gwf_tracer_model_bit_t;
+
+/*! @} */
 
 /* Set of parameters related to a tracer equation attached to a standard
    modelling */
 
 typedef struct {
 
-  /* Parameters (array of size: n_soils) */
-  double    *rho_kd;        // Bulk density times the distribution coefficient
-  double    *alpha_l;       // Longitudinal dispersivity
-  double    *alpha_t;       // Transversal dispersivity
-  double    *wmd;           // Water molecular diffusivity
+  /* Parameters to determine the behavior in each soil
+   * (array of size: n_soils)
+   */
+
+  /* Common settings shared by all physical modelling */
+  /* ------------------------------------------------ */
+
+  double    *rho_bulk;      /* bulk density (kg.m^-3) */
+  double    *kd0;           /* reference value of the distribution coefficient
+                               (m^".kg^-1) */
+  double    *rho_kd;        /* Derived quantity: rho_bulk*kd0 */
+
+  double    *alpha_l;       /* Longitudinal dispersivity */
+  double    *alpha_t;       /* Transversal dispersivity */
+
+  double    *wmd;           /* Water molecular diffusivity (m^2.s^-1) */
+
   double    *reaction_rate; /* First order decay coefficient (related to the
                                reaction term) */
 
-  /* Variables used for the update of physical properties */
+  /* Precipitation members (set to NULL if not used) */
+  /* ----------------------------------------------- */
+
+  double       *conc_w_star;    /* maximal value of the concentraction of tracer
+                                   in the liquid phase. Exceeded quantities are
+                                   stored in the solid phase (->
+                                   conc_precip). These values corresponds to the
+                                   user settings */
+
+  cs_real_t    *conc_satura;    /* array storing the value of the saturated
+                                   concentration in the liquid phase at vertices
+                                   (only used in case of CDOVB schemes or CDOVCB
+                                   schemes */
+  cs_real_t    *conc_precip;    /* array storing the concentration in the
+                                   precipitation (solid) storage. The size may
+                                   vary w.r.t. to the discrtization scheme.
+                                */
+
+  cs_field_t   *precip_field;
+
+  /* Sorption members (set to NULL if not used) */
+  /* ------------------------------------------ */
+
+  double       *k0_plus;        /* kinetic coefficient towards site 2 locations
+                                   (m^3.kg^-1.s^-1) */
+  double       *k0_minus;       /* kinetic coefficient from site 2 locations
+                                   (s^-1) */
+
+  cs_real_t    *conc_site2;     /* array allocated to n_cells */
+
+  /* Variables used for the update of physical properties (shared pointers) */
+
   cs_field_t   *darcy_velocity_field;
-  cs_field_t   *moisture_content;
+  cs_field_t   *moisture_content;    /* also called the saturation, denoted by
+                                        \theta (-) */
 
-} cs_gwf_std_tracer_input_t;
+} cs_gwf_tracer_input_t;
 
-/* Set of parameters describing a tracer */
-typedef struct {
+/* Set of parameters describing a tracer structure */
+/* ----------------------------------------------- */
+
+struct _gwf_tracer_t{
 
   int                          id;       /* tracer id */
   cs_equation_t               *eq;       /* related equation */
@@ -130,10 +214,11 @@ typedef struct {
   void                        *input;
 
   /* Pointers to functions */
-  cs_gwf_tracer_update_t      *update_properties;
+  cs_gwf_tracer_update_t      *update_diff_tensor;
+  cs_gwf_tracer_update_t      *update_precipitation;
   cs_gwf_tracer_free_input_t  *free_input;
 
-} cs_gwf_tracer_t;
+};
 
 /*============================================================================
  * Public function pointer prototypes
@@ -212,8 +297,8 @@ cs_gwf_tracer_free(cs_gwf_tracer_t     *tracer);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Set a tracer for a specified soil when the tracer is attached to
- *         the default model
+ * \brief  For a specified soil set the main parameters corresponding to a
+ *         default modelling of a tracer transport
  *
  * \param[in, out] tracer          pointer to a cs_gwf_tracer_t structure
  * \param[in]      soil_name       name of the related soil (or NULL if all
@@ -227,27 +312,45 @@ cs_gwf_tracer_free(cs_gwf_tracer_t     *tracer);
 /*----------------------------------------------------------------------------*/
 
 void
-cs_gwf_set_standard_tracer(cs_gwf_tracer_t   *tracer,
-                           const char        *soil_name,
-                           double             wmd,
-                           double             alpha_l,
-                           double             alpha_t,
-                           double             distrib_coef,
-                           double             reaction_rate);
+cs_gwf_set_main_tracer_param(cs_gwf_tracer_t   *tracer,
+                             const char        *soil_name,
+                             double             wmd,
+                             double             alpha_l,
+                             double             alpha_t,
+                             double             distrib_coef,
+                             double             reaction_rate);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  For a specified soil set the parameters corresponding to a
+ *         precipitation modelling of a tracer transport
+ *
+ * \param[in, out] tracer          pointer to a cs_gwf_tracer_t structure
+ * \param[in]      soil_name       name of the related soil (or NULL if all
+ *                                 soils are selected)
+ * \param[in]      conc_w_star     value of the saturated concentration in the
+ *                                 liquid phase
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_gwf_set_precip_tracer_param(cs_gwf_tracer_t   *tracer,
+                               const char        *soil_name,
+                               double             conc_w_star);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Add terms to the algebraic system related to a tracer equation
  *         according to the settings.
- *         Case of the standar tracer modelling
- *         Rely on the generic functinon: cs_gwf_tracer_add_terms_t
+ *         Case of the standard tracer modelling
+ *         Rely on the generic function: cs_gwf_tracer_add_terms_t
  *
  * \param[in, out] tracer       pointer to a cs_gwf_tracer_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_gwf_tracer_standard_add_terms(cs_gwf_tracer_t     *tracer);
+cs_gwf_tracer_add_terms(cs_gwf_tracer_t     *tracer);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -260,9 +363,20 @@ cs_gwf_tracer_standard_add_terms(cs_gwf_tracer_t     *tracer);
 /*----------------------------------------------------------------------------*/
 
 void
-cs_gwf_tracer_standard_setup(const cs_cdo_connect_t      *connect,
-                             const cs_cdo_quantities_t   *quant,
-                             cs_gwf_tracer_t             *tracer);
+cs_gwf_tracer_setup(const cs_cdo_connect_t      *connect,
+                    const cs_cdo_quantities_t   *quant,
+                    cs_gwf_tracer_t             *tracer);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Display the main features related to a tracer
+ *
+ * \param[in]  tracer   pointer to a cs_gwf_tracer_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_gwf_tracer_log_setup(const cs_gwf_tracer_t     *tracer);
 
 /*----------------------------------------------------------------------------*/
 
