@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -48,6 +48,8 @@
 #include "cs_cdo_advection.h"
 #include "cs_cdo_bc.h"
 #include "cs_cdo_diffusion.h"
+#include "cs_equation_assemble.h"
+#include "cs_equation_bc.h"
 #include "cs_equation_common.h"
 #include "cs_hho_builder.h"
 #include "cs_hodge.h"
@@ -94,45 +96,50 @@ struct _cs_hho_scaleq_t {
   int          bflux_field_id;
 
   /* System size (n_faces * n_face_dofs + n_cells * n_cell_dofs) */
-  cs_lnum_t                      n_dofs;
-  int                            n_max_loc_dofs;
-  int                            n_cell_dofs;
-  int                            n_face_dofs;
+  cs_lnum_t    n_dofs;
+  int          n_max_loc_dofs;
+  int          n_cell_dofs;
+  int          n_face_dofs;
 
   /* Structures related to the algebraic sytem construction (shared) */
-  const cs_matrix_structure_t   *ms;
-  const cs_range_set_t          *rs;
+  const cs_matrix_structure_t    *ms;
+  const cs_range_set_t           *rs;
 
   /* Solution of the algebraic system at the last computed iteration.
      cell_values is different from the values stored in the associated
      field since here it's the values of polynomial coefficients which
      are stored */
-  cs_real_t                     *face_values;  /* DoF unknowns (x) + BCs */
-  cs_real_t                     *cell_values;  /* DoF recomputed after the
-                                                  static condensation */
+  cs_real_t                      *face_values;  /* DoF unknowns (x) + BCs */
+  cs_real_t                      *cell_values;  /* DoF recomputed after the
+                                                   static condensation */
 
   /* Storage of the source term (if one wants to apply a specific time
      discretization) */
-  cs_real_t                     *source_terms;
+  cs_real_t                      *source_terms;
 
   /* Handle the definition of the BCs */
-  short int                     *bf2def_ids;
+  short int                      *bf2def_ids;
 
   /* Pointer of function to build the diffusion term */
-  cs_cdo_diffusion_enforce_dir_t  *enforce_dirichlet;
+  cs_cdo_enforce_bc_t            *enforce_dirichlet;
+
+  /* Assembly process */
+  /* ================ */
+
+  cs_equation_assembly_t         *assemble;
 
   /* Static condensation members */
   /* =========================== */
 
   /* Acc_inv = Inverse of a diagonal matrix (block cell-cell)
      rc_tilda = Acc_inv * rhs_c (perform for each cell) */
-  cs_real_t                     *rc_tilda;
+  cs_real_t                      *rc_tilda;
 
   /* Lower-Left block of the full matrix (block cell-faces).
      Access to the values thanks to the c2f connectivity
      The transposed version of each block is stored for a more efficient
      usage */
-  cs_sdm_t                      *acf_tilda;
+  cs_sdm_t                       *acf_tilda;
 
 };
 
@@ -183,8 +190,8 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
 
   case CS_SPACE_SCHEME_HHO_P0:  /* TODO */
     {
-      BFT_MALLOC(cb->ids, n_fc + 1, short int);
-      memset(cb->ids, 0, (n_fc + 1)*sizeof(short int));
+      BFT_MALLOC(cb->ids, n_fc + 1, int);
+      memset(cb->ids, 0, (n_fc + 1)*sizeof(int));
 
       /* For post-processing errors = 38 */
       size = CS_MAX(38, n_fc*(n_fc+1));
@@ -197,7 +204,6 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
 
       /* Local square dense matrices used during the construction of
          operators */
-      cb->hdg = cs_sdm_square_create(n_fc);
       cb->loc = cs_sdm_square_create(n_fc + 1);
       cb->aux = cs_sdm_square_create(n_fc + 1);
     }
@@ -207,8 +213,8 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
     {
       /* Store the block size description */
       size = n_fc + 1;
-      BFT_MALLOC(cb->ids, size, short int);
-      memset(cb->ids, 0, size*sizeof(short int));
+      BFT_MALLOC(cb->ids, size, int);
+      memset(cb->ids, 0, size*sizeof(int));
 
       /* Store the face, cell and gradient basis function evaluations and
          the Gauss point weights
@@ -230,12 +236,9 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
       memset(cb->vectors, 0, size*sizeof(cs_real_3_t));
 
       /* Local dense matrices used during the construction of operators */
-      const short int g_size = 9;                   /* basis (P_(k+1)) - 1 */
+      const int g_size = 9;                   /* basis (P_(k+1)) - 1 */
       for (int i = 0; i < n_fc; i++) cb->ids[i] = 3;
       cb->ids[n_fc] = 4;
-
-      short int  _sizes[3] = {1, 3, 6}; /* c0, cs-1, cs_kp1 - cs */
-      cb->hdg = cs_sdm_block_create(1, 3, &g_size, _sizes);
       cb->loc = cs_sdm_block_create(n_fc + 1, n_fc + 1, cb->ids, cb->ids);
       cb->aux = cs_sdm_block_create(n_fc + 1, 1, cb->ids, &g_size); /* R_g^T */
     }
@@ -245,8 +248,8 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
     {
       /* Store the block size description */
       size = n_fc + 1;
-      BFT_MALLOC(cb->ids, size, short int);
-      memset(cb->ids, 0, size*sizeof(short int));
+      BFT_MALLOC(cb->ids, size, int);
+      memset(cb->ids, 0, size*sizeof(int));
 
       /* Store the face, cell and gradient basis function evaluations and
          the Gauss point weights */
@@ -268,12 +271,9 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
       memset(cb->vectors, 0, size*sizeof(cs_real_3_t));
 
       /* Local dense matrices used during the construction of operators */
-      const short int g_size = 19; /* basis (P_(k+1)) - 1 */
+      const int g_size = 19; /* basis (P_(k+1)) - 1 */
       for (int i = 0; i < n_fc; i++) cb->ids[i] = 6;
       cb->ids[n_fc] = 10;
-
-      short int  _sizes[3] = {1, 9, 10}; /* c0, cs-1, cs_kp1 - cs */
-      cb->hdg = cs_sdm_block_create(1, 3, &g_size, _sizes);
       cb->loc = cs_sdm_block_create(n_fc + 1, n_fc + 1, cb->ids, cb->ids);
       cb->aux = cs_sdm_block_create(n_fc + 1, 1, cb->ids, &g_size); /* R_g^T */
     }
@@ -289,14 +289,13 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief   Initialize the local structures for the current cell
+ * \brief   Initialize the localstructures for the current cell.
+ *          Case of scalar-valued HHO schemes
  *
- * \param[in]      cell_flag   flag related to the current cell
  * \param[in]      cm          pointer to a cellwise view of the mesh
  * \param[in]      eqp         pointer to a cs_equation_param_t structure
  * \param[in]      eqb         pointer to a cs_equation_builder_t structure
- * \param[in]      eqc         pointer to a cs_hho_scaleq_t structure
- * \param[in]      t_eval      time at which one performs the evaluation
+ * \param[in]      eqc         pointer to a cs_hho_vecteq_t structure
  * \param[in, out] hhob        pointer to a cs_hho_builder_t structure
  * \param[in, out] csys        pointer to a cellwise view of the system
  * \param[in, out] cb          pointer to a cellwise builder
@@ -304,35 +303,31 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
 /*----------------------------------------------------------------------------*/
 
 static void
-_init_cell_system(const cs_flag_t               cell_flag,
-                  const cs_cell_mesh_t         *cm,
-                  const cs_equation_param_t    *eqp,
-                  const cs_equation_builder_t  *eqb,
-                  const cs_hho_scaleq_t        *eqc,
-                  cs_real_t                     t_eval,
-                  cs_hho_builder_t             *hhob,
-                  cs_cell_sys_t                *csys,
-                  cs_cell_builder_t            *cb)
+_shho_init_cell_system(const cs_cell_mesh_t         *cm,
+                       const cs_equation_param_t    *eqp,
+                       const cs_equation_builder_t  *eqb,
+                       const cs_hho_scaleq_t        *eqc,
+                       cs_hho_builder_t             *hhob,
+                       cs_cell_sys_t                *csys,
+                       cs_cell_builder_t            *cb)
 {
   const int  n_dofs = eqc->n_cell_dofs + cm->n_fc*eqc->n_face_dofs;
   const int  n_blocks = cm->n_fc + 1;
-  const cs_cdo_connect_t  *connect = cs_shared_connect;
 
-  short int  *block_sizes = cb->ids;
+  int  *block_sizes = cb->ids;
   for (int i = 0; i < cm->n_fc; i++)
     block_sizes[i] = eqc->n_face_dofs;
   block_sizes[cm->n_fc] = eqc->n_cell_dofs;
 
   csys->c_id = cm->c_id;
   csys->n_dofs = n_dofs;
-  csys->cell_flag = cell_flag;
 
   /* Initialize the local system */
   cs_cell_sys_reset(cm->n_fc, csys);
 
   cs_sdm_block_init(csys->mat, n_blocks, n_blocks, block_sizes, block_sizes);
 
-  for (short int f = 0; f < cm->n_fc; f++) {
+  for (int f = 0; f < cm->n_fc; f++) {
 
     const int _f_shift = eqc->n_face_dofs*f;
     const cs_lnum_t  f_shift = eqc->n_face_dofs*cm->f_ids[f];
@@ -354,12 +349,12 @@ _init_cell_system(const cs_flag_t               cell_flag,
 
   /* Store the local values attached to Dirichlet values if the current cell
      has at least one border face */
-  if (cell_flag & CS_FLAG_BOUNDARY) {
+  if (cb->cell_flag & CS_FLAG_BOUNDARY_CELL_BY_FACE) {
 
     /* Identify which face is a boundary face */
     for (short int f = 0; f < cm->n_fc; f++) {
 
-      const cs_lnum_t  bf_id = cm->f_ids[f] - connect->n_faces[2]; // n_i_faces
+      const cs_lnum_t  bf_id = cm->f_ids[f] - cm->bface_shift;
       if (bf_id > -1) {        /*  Border face */
 
         const cs_flag_t  face_flag = eqb->face_bc->flag[bf_id];
@@ -391,7 +386,7 @@ _init_cell_system(const cs_flag_t               cell_flag,
           cs_hho_builder_compute_dirichlet(eqp->bc_defs[def_id],
                                            f,
                                            cm,
-                                           t_eval,
+                                           cb->t_bc_eval,
                                            cb,
                                            hhob,
                                            dir_reduction);
@@ -403,7 +398,7 @@ _init_cell_system(const cs_flag_t               cell_flag,
 
       } /* Border face id */
 
-    } // Loop on cell faces
+    } /* Loop on cell faces */
 
 #if defined(DEBUG) && !defined(NDEBUG) /* Sanity check */
     for (short int f = 0; f < eqc->n_face_dofs*cm->n_fc; f++) {
@@ -780,7 +775,7 @@ cs_hho_scaleq_init_context(const cs_equation_param_t   *eqp,
     bft_error(__FILE__, __LINE__, 0, " Expected: scalar-valued HHO equation.");
 
   const cs_cdo_connect_t  *connect = cs_shared_connect;
-  const cs_lnum_t  n_faces = connect->n_faces[0];
+  const cs_lnum_t  n_faces = connect->n_faces[CS_ALL_FACES];
   const cs_lnum_t  n_cells = connect->n_cells;
 
   cs_hho_scaleq_t  *eqc = NULL;
@@ -791,34 +786,49 @@ cs_hho_scaleq_init_context(const cs_equation_param_t   *eqp,
   eqc->bflux_field_id = bflux_id;
 
   /* Mesh flag to know what to build */
-  eqb->msh_flag = CS_CDO_LOCAL_PV | CS_CDO_LOCAL_PEQ | CS_CDO_LOCAL_PFQ |
-    CS_CDO_LOCAL_FE | CS_CDO_LOCAL_FEQ | CS_CDO_LOCAL_HFQ |
-    CS_CDO_LOCAL_EV | CS_CDO_LOCAL_DIAM;
+  eqb->msh_flag = CS_FLAG_COMP_PV | CS_FLAG_COMP_PEQ | CS_FLAG_COMP_PFQ |
+    CS_FLAG_COMP_FE | CS_FLAG_COMP_FEQ | CS_FLAG_COMP_HFQ |
+    CS_FLAG_COMP_EV | CS_FLAG_COMP_DIAM;
 
   switch (eqp->space_scheme) {
 
   case CS_SPACE_SCHEME_HHO_P0:
     eqc->n_cell_dofs = CS_N_CELL_DOFS_0TH;
     eqc->n_face_dofs = CS_N_FACE_DOFS_0TH;
+
     /* Not owner; Only shared */
     eqc->ms = cs_shared_ms0;
     eqc->rs = connect->range_sets[CS_CDO_CONNECT_FACE_SP0];
+
+    /* Assembly process */
+    eqc->assemble = cs_equation_assemble_set(CS_SPACE_SCHEME_HHO_P0,
+                                             CS_CDO_CONNECT_FACE_SP0);
     break;
 
   case CS_SPACE_SCHEME_HHO_P1:
     eqc->n_cell_dofs = CS_N_CELL_DOFS_1ST;
     eqc->n_face_dofs = CS_N_FACE_DOFS_1ST;
+
     /* Not owner; Only shared */
     eqc->ms = cs_shared_ms1;
     eqc->rs = connect->range_sets[CS_CDO_CONNECT_FACE_SP1];
+
+    /* Assembly process */
+    eqc->assemble = cs_equation_assemble_set(CS_SPACE_SCHEME_HHO_P1,
+                                             CS_CDO_CONNECT_FACE_SP1);
     break;
 
   case CS_SPACE_SCHEME_HHO_P2:
     eqc->n_cell_dofs = CS_N_CELL_DOFS_2ND;
     eqc->n_face_dofs = CS_N_FACE_DOFS_2ND;
+
     /* Not owner; Only shared */
     eqc->ms = cs_shared_ms2;
     eqc->rs = connect->range_sets[CS_CDO_CONNECT_FACE_SP2];
+
+    /* Assembly process */
+    eqc->assemble = cs_equation_assemble_set(CS_SPACE_SCHEME_HHO_P2,
+                                             CS_CDO_CONNECT_FACE_SP2);
     break;
 
     /* TODO: case CS_SPACE_SCHEME_HHO_PK */
@@ -856,14 +866,14 @@ cs_hho_scaleq_init_context(const cs_equation_param_t   *eqp,
   memset(eqc->rc_tilda, 0, sizeof(cs_real_t)*n_cell_dofs);
 
   cs_lnum_t  n_row_blocks = connect->c2f->idx[n_cells];
-  short int  *row_block_sizes = NULL;
+  int  *row_block_sizes = NULL;
 
-  BFT_MALLOC(row_block_sizes, n_row_blocks, short int);
+  BFT_MALLOC(row_block_sizes, n_row_blocks, int);
 # pragma omp parallel for if (n_cells > CS_THR_MIN)
   for (cs_lnum_t i = 0; i < n_row_blocks; i++)
     row_block_sizes[i] = eqc->n_face_dofs;
 
-  short int  col_block_size = eqc->n_cell_dofs;
+  int  col_block_size = eqc->n_cell_dofs;
   eqc->acf_tilda = cs_sdm_block_create(n_row_blocks, 1,
                                        row_block_sizes, &col_block_size);
   cs_sdm_block_init(eqc->acf_tilda,
@@ -873,7 +883,7 @@ cs_hho_scaleq_init_context(const cs_equation_param_t   *eqp,
   BFT_FREE(row_block_sizes);
 
   /* Handle boundary conditions */
-  const cs_lnum_t  n_b_faces = connect->n_faces[1];
+  const cs_lnum_t  n_b_faces = connect->n_faces[CS_BND_FACES];
   BFT_MALLOC(eqc->bf2def_ids, n_b_faces, short int);
 
 # pragma omp parallel for if (n_b_faces > CS_THR_MIN)
@@ -891,12 +901,12 @@ cs_hho_scaleq_init_context(const cs_equation_param_t   *eqp,
 
   } /* Loop on BC definitions */
 
-  /* DIFFUSION */
+  /* Diffusion */
   eqc->enforce_dirichlet = NULL;
 
   if (cs_equation_param_has_diffusion(eqp)) {
 
-    switch (eqp->enforcement) {
+    switch (eqp->default_enforcement) {
 
     case CS_PARAM_BC_ENFORCE_ALGEBRAIC:
       eqc->enforce_dirichlet = cs_cdo_diffusion_alge_block_dirichlet;
@@ -951,6 +961,49 @@ cs_hho_scaleq_free_context(void   *data)
   return NULL;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set the initial values of the variable field taking into account
+ *         the boundary conditions.
+ *         Case of scalar-valued HHO schemes.
+ *
+ * \param[in]      t_eval     time at which one evaluates BCs
+ * \param[in]      field_id   id related to the variable field of this equation
+ * \param[in]      mesh       pointer to a cs_mesh_t structure
+ * \param[in]      eqp        pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
+ * \param[in, out] context    pointer to the scheme context (cast on-the-fly)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_hho_scaleq_init_values(cs_real_t                     t_eval,
+                          const int                     field_id,
+                          const cs_mesh_t              *mesh,
+                          const cs_equation_param_t    *eqp,
+                          cs_equation_builder_t        *eqb,
+                          void                         *context)
+{
+  /* Unused parameters --> generic function pointer */
+  CS_UNUSED(field_id);
+  CS_UNUSED(eqb);
+  CS_UNUSED(t_eval);
+  CS_UNUSED(mesh);
+  CS_UNUSED(eqp);
+
+  const cs_cdo_quantities_t  *quant = cs_shared_quant;
+
+  cs_hho_scaleq_t  *eqc = (cs_hho_scaleq_t *)context;
+  cs_real_t  *f_vals = eqc->face_values;
+  cs_real_t  *c_vals = eqc->cell_values;
+
+  memset(f_vals, 0, quant->n_faces * eqc->n_face_dofs * sizeof(cs_real_t));
+  memset(c_vals, 0, quant->n_cells * eqc->n_cell_dofs * sizeof(cs_real_t));
+
+  /* TODO */
+
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Create the matrix of the current algebraic system.
@@ -1002,7 +1055,6 @@ cs_hho_scaleq_initialize_system(const cs_equation_param_t  *eqp,
  *
  * \param[in]      mesh       pointer to a cs_mesh_t structure
  * \param[in]      field_val  pointer to the current value of the field
- * \param[in]      dt_cur     current value of the time step
  * \param[in]      eqp        pointer to a cs_equation_param_t structure
  * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
  * \param[in, out] data       pointer to cs_hho_scaleq_t structure
@@ -1014,7 +1066,6 @@ cs_hho_scaleq_initialize_system(const cs_equation_param_t  *eqp,
 void
 cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
                            const cs_real_t            *field_val,
-                           double                      dt_cur,
                            const cs_equation_param_t  *eqp,
                            cs_equation_builder_t      *eqb,
                            void                       *data,
@@ -1027,8 +1078,8 @@ cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
   /* Sanity checks */
   assert(rhs != NULL && matrix != NULL && eqp != NULL && eqb != NULL);
   /* The only way to set a Dirichlet up to now */
-  assert(eqp->enforcement == CS_PARAM_BC_ENFORCE_PENALIZED ||
-         eqp->enforcement == CS_PARAM_BC_ENFORCE_ALGEBRAIC);
+  assert(eqp->default_enforcement == CS_PARAM_BC_ENFORCE_PENALIZED ||
+         eqp->default_enforcement == CS_PARAM_BC_ENFORCE_ALGEBRAIC);
 
   /* Test to remove */
   if (cs_equation_param_has_convection(eqp))
@@ -1042,17 +1093,20 @@ cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
 
   const cs_cdo_quantities_t  *quant = cs_shared_quant;
   const cs_cdo_connect_t  *connect = cs_shared_connect;
-  const cs_real_t  t_cur = cs_shared_time_step->t_cur;
+  const cs_time_step_t  *ts = cs_shared_time_step;
+  const cs_real_t  t_cur = ts->t_cur;
+  const cs_real_t  dt_cur = ts->dt[0];
 
   cs_timer_t  t0 = cs_timer_time();
 
   /* Initialize the structure to assemble values */
-  cs_matrix_assembler_values_t  *mav =
-    cs_matrix_assembler_values_init(matrix, NULL, NULL);
+  cs_matrix_assembler_values_t  *mav
+    = cs_matrix_assembler_values_init(matrix, NULL, NULL);
 
-# pragma omp parallel if (quant->n_cells > CS_THR_MIN) default(none)     \
-  shared(dt_cur, quant, connect, eqp, eqb, eqc, rhs, matrix, mav,        \
-         field_val, cs_hho_cell_sys, cs_hho_cell_bld, cs_hho_builders)
+# pragma omp parallel if (quant->n_cells > CS_THR_MIN)                  \
+  shared(quant, connect, eqp, eqb, eqc, rhs, matrix, mav,               \
+         field_val, cs_hho_cell_sys, cs_hho_cell_bld, cs_hho_builders)  \
+  firstprivate(dt_cur, t_cur)
   {
 #if defined(HAVE_OPENMP) /* Determine default number of OpenMP threads */
     int  t_id = omp_get_thread_num();
@@ -1060,22 +1114,37 @@ cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
     int  t_id = 0;
 #endif
 
-    const cs_real_t  t_eval_pty = t_cur + 0.5*dt_cur;
-
     /* Set inside the OMP section so that each thread has its own value
      * Each thread get back its related structures:
      * Get the cell-wise view of the mesh and the algebraic system */
+    cs_equation_assemble_t  *eqa = cs_equation_assemble_get(t_id);
     cs_cell_mesh_t  *cm = cs_cdo_local_get_cell_mesh(t_id);
     cs_cell_sys_t  *csys = cs_hho_cell_sys[t_id];
     cs_cell_builder_t  *cb = cs_hho_cell_bld[t_id];
     cs_hho_builder_t  *hhob = cs_hho_builders[t_id];
 
-    /* Store the shift to access border faces (first interior faces and
-       then border faces: shift = n_i_faces */
-    csys->face_shift = connect->n_faces[CS_INT_FACES];
+    cb->t_pty_eval = t_cur + eqp->theta*dt_cur;
+    cb->t_bc_eval = t_cur + dt_cur;
+
+    cs_property_data_t  *diff_pty = NULL;
+    BFT_MALLOC(diff_pty, 1, cs_property_data_t);
+    cs_property_data_init(true, true, eqp->diffusion_property, diff_pty);
 
     /* Initialization of the values of properties */
-    cs_equation_init_properties(eqp, eqb, t_eval_pty, cb);
+    cs_equation_init_properties(eqp, eqb, NULL, cb);
+
+    if (cs_equation_param_has_diffusion(eqp)) {
+
+      cs_property_get_cell_tensor(0,
+                                  cb->t_pty_eval,
+                                  eqp->diffusion_property,
+                                  eqp->diffusion_hodgep.inv_pty,
+                                  diff_pty->tensor);
+
+      if (diff_pty->is_iso)
+        diff_pty->value = diff_pty->tensor[0][0];
+
+    }
 
     /* --------------------------------------------- */
     /* Main loop on cells to build the linear system */
@@ -1084,22 +1153,20 @@ cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
 #   pragma omp for CS_CDO_OMP_SCHEDULE
     for (cs_lnum_t c_id = 0; c_id < quant->n_cells; c_id++) {
 
-      const cs_flag_t  cell_flag = connect->cell_flag[c_id];
-      const cs_flag_t  msh_flag = cs_equation_cell_mesh_flag(cell_flag, eqb);
+      cb->cell_flag = connect->cell_flag[c_id];
 
       /* Set the local mesh structure for the current cell */
-      cs_cell_mesh_build(c_id, msh_flag, connect, quant, cm);
+      cs_cell_mesh_build(c_id, cs_equation_cell_mesh_flag(cb->cell_flag, eqb),
+                         connect, quant, cm);
 
       /* Define set of basis functions for cell faces and the current cell */
       cs_hho_builder_cellwise_setup(cm, cb, hhob);
 
       /* Set the local (i.e. cellwise) structures for the current cell */
-      _init_cell_system(cell_flag, cm, eqp, eqb, eqc,
-                        t_cur + dt_cur, /* For Dirichlet up to now */
-                        hhob, csys, cb);
+      _shho_init_cell_system(cm, eqp, eqb, eqc, hhob, csys, cb);
 
 #if defined(DEBUG) && !defined(NDEBUG) && CS_HHO_SCALEQ_DBG > 2
-      if (cs_dbg_cw_test(cm)) cs_cell_mesh_dump(cm);
+      if (cs_dbg_cw_test(eqp, cm, csys)) cs_cell_mesh_dump(cm);
 #endif
 
       const short int  face_offset = cm->n_fc*eqc->n_face_dofs;
@@ -1109,21 +1176,30 @@ cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
 
       if (cs_equation_param_has_diffusion(eqp)) {
 
-        /* Define the local stiffness matrix */
-        if (!(eqb->diff_pty_uniform))
-          cs_equation_set_diffusion_property_cw(eqp, cm, t_eval_pty, cell_flag,
-                                                cb);
+        if (!(eqb->diff_pty_uniform)) {
 
-        cs_hho_builder_compute_grad_reco(cm, cb, hhob);
+          cs_property_tensor_in_cell(cm,
+                                     eqp->diffusion_property,
+                                     cb->t_pty_eval,
+                                     eqp->diffusion_hodgep.inv_pty,
+                                     diff_pty->tensor);
+
+          if (diff_pty->is_iso)
+            diff_pty->value = diff_pty->tensor[0][0];
+
+        }
+
+        /* Define the local stiffness matrix */
+        cs_hho_builder_compute_grad_reco(cm, diff_pty, cb, hhob);
 
         /* Local matrix owned by the cellwise builder (store in cb->loc) */
-        cs_hho_builder_diffusion(cm, cb, hhob);
+        cs_hho_builder_diffusion(cm, diff_pty, cb, hhob);
 
         /* Add the local diffusion operator to the local system */
         cs_sdm_block_add(csys->mat, cb->loc);
 
 #if defined(DEBUG) && !defined(NDEBUG) && CS_HHO_SCALEQ_DBG > 1
-        if (cs_dbg_cw_test(cm))
+        if (cs_dbg_cw_test(eqp, cm, csys))
           cs_cell_sys_dump("\n>> Local system after diffusion", csys);
 #endif
       } /* END OF DIFFUSION */
@@ -1144,7 +1220,7 @@ cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
                                         cm,
                                         eqb->source_mask,
                                         eqb->compute_source,
-                                        t_eval_pty,
+                                        cb->t_bc_eval,
                                         hhob,   /* input structure */
                                         cb,     /* mass matrix is cb->hdg */
                                         csys->source);
@@ -1171,7 +1247,7 @@ cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
       } /* End of term source contribution */
 
 #if defined(DEBUG) && !defined(NDEBUG) && CS_HHO_SCALEQ_DBG > 1
-      if (cs_dbg_cw_test(cm))
+      if (cs_dbg_cw_test(eqp, cm, csys))
         cs_cell_sys_dump(">> Local system matrix before condensation", csys);
 #endif
 
@@ -1185,22 +1261,22 @@ cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
 
       /* TODO: Neumann boundary conditions */
 
-      if (cell_flag & CS_FLAG_BOUNDARY) {
+      if (cb->cell_flag & CS_FLAG_BOUNDARY_CELL_BY_FACE) {
 
         if (cs_equation_param_has_diffusion(eqp)) {
 
           /* Weakly enforced Dirichlet BCs for cells attached to the boundary
              csys is updated inside (matrix and rhs)
-             eqp->diffusion_hodge is a dummy parameter (not used)
+             fm = NULL, hodge = NULL (not useful in the current cntext)
           */
-          eqc->enforce_dirichlet(eqp, cm, NULL, cb, csys);
+          eqc->enforce_dirichlet(eqp, cm, NULL, NULL, cb, csys);
 
         } /* diffusion term */
 
       }
 
 #if defined(DEBUG) && !defined(NDEBUG) && CS_HHO_SCALEQ_DBG > 0
-      if (cs_dbg_cw_test(cm)) {
+      if (cs_dbg_cw_test(eqp, cm, csys)) {
         cs_cell_sys_dump(">> (FINAL) Local system matrix", csys);
         cs_sdm_block_fprintf(NULL, NULL, 1e-16, csys->mat);
       }
@@ -1210,9 +1286,9 @@ cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
       /* ======== */
 
       /* Matrix assembly */
-      cs_equation_assemble_block_matrix(csys, eqc->rs, eqc->n_face_dofs, mav);
+      eqc->assemble(csys->mat, csys->dof_ids, eqc->rs, eqa, mav);
 
-      /* Assemble RHS */
+      /* RHS assembly */
       for (short int i = 0; i < eqc->n_face_dofs*cm->n_fc; i++) {
 #       pragma omp atomic
         rhs[csys->dof_ids[i]] += csys->rhs[i];
@@ -1220,6 +1296,8 @@ cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
 
     } /* Main loop on cells */
 
+    BFT_FREE(diff_pty);
+
   } /* OPENMP Block */
 
   cs_matrix_assembler_values_done(mav); // optional
@@ -1275,7 +1353,7 @@ cs_hho_scaleq_update_field(const cs_real_t            *solu,
   memset(eqc->cell_values, 0,
          sizeof(cs_real_t) * eqc->n_cell_dofs * quant->n_cells);
 
-# pragma omp parallel if (quant->n_cells > CS_THR_MIN) default(none)    \
+# pragma omp parallel if (quant->n_cells > CS_THR_MIN)                  \
   shared(quant, connect, eqp, eqb, eqc, rhs, solu, field_val,           \
          cs_hho_cell_bld, cs_hho_builders)
   {
@@ -1291,8 +1369,6 @@ cs_hho_scaleq_update_field(const cs_real_t            *solu,
     cs_cell_builder_t  *cb = cs_hho_cell_bld[t_id];
     cs_hho_builder_t  *hhob = cs_hho_builders[t_id];
 
-    /* Set inside the OMP section so that each thread has its own value */
-
     /* ----------------------------------------------------------- */
     /* Main loop on cells to reconstruct the field at cell centers */
     /* ----------------------------------------------------------- */
@@ -1301,11 +1377,12 @@ cs_hho_scaleq_update_field(const cs_real_t            *solu,
     for (cs_lnum_t c_id = 0; c_id < quant->n_cells; c_id++) {
 
       const cs_lnum_t  c2f_shift = connect->c2f->idx[c_id];
-      const cs_flag_t  cell_flag = connect->cell_flag[c_id];
-      const cs_flag_t  msh_flag = cs_equation_cell_mesh_flag(cell_flag, eqb);
+
+      cb->cell_flag = connect->cell_flag[c_id];
 
       /* Set the local mesh structure for the current cell */
-      cs_cell_mesh_build(c_id, msh_flag, connect, quant, cm);
+      cs_cell_mesh_build(c_id, cs_equation_cell_mesh_flag(cb->cell_flag, eqb),
+                         connect, quant, cm);
 
       /* Define set of basis functions for cell faces and the current cell */
       cs_hho_builder_cellbasis_setup(cm, cb, hhob);
@@ -1339,9 +1416,9 @@ cs_hho_scaleq_update_field(const cs_real_t            *solu,
         field_val[c_id] += cphi_eval[i] * c_vals[i];
       }
 
-    } // Main loop on cells
+    } /* Main loop on cells */
 
-  } // OPENMP Block
+  } /* OPENMP Block */
 
   /* Set the computed solution in field array */
   memcpy(eqc->face_values, solu, sizeof(cs_real_t)*eqc->n_dofs);
@@ -1350,9 +1427,9 @@ cs_hho_scaleq_update_field(const cs_real_t            *solu,
   cs_dbg_darray_to_listing("FINAL FACE_VALUES",
                            eqc->n_dofs, eqc->face_values, eqc->n_face_dofs);
   cs_dbg_darray_to_listing("FINAL CELL_VALUES", quant->n_cells,
-                             eqc->cell_values, eqc->n_cell_dofs);
+                           eqc->cell_values, eqc->n_cell_dofs);
   cs_dbg_darray_to_listing("FINAL CELL_CENTER_VALUES", quant->n_cells,
-                             field_val, 1);
+                           field_val, 1);
 #endif
 
   cs_timer_t  t1 = cs_timer_time();
@@ -1364,21 +1441,26 @@ cs_hho_scaleq_update_field(const cs_real_t            *solu,
  * \brief  Get the computed values at faces (DoF used in the linear system are
  *         located at primal faces)
  *
- * \param[in, out]  data    pointer to a data structure cast on-the-fly
+ * \param[in, out]  context    pointer to a data structure cast on-the-fly
+ * \param[in]       previous   retrieve the previous state (true/false)
  *
- * \return  a pointer to an array of \ref cs_real_t
+ * \return  a pointer to an array of cs_real_t
  */
 /*----------------------------------------------------------------------------*/
 
 cs_real_t *
-cs_hho_scaleq_get_face_values(void          *data)
+cs_hho_scaleq_get_face_values(void          *context,
+                              bool           previous)
 {
-  cs_hho_scaleq_t  *eqc = (cs_hho_scaleq_t  *)data;
+  cs_hho_scaleq_t  *eqc = (cs_hho_scaleq_t  *)context;
 
   if (eqc == NULL)
     return NULL;
-  else
-    return eqc->face_values;
+
+  if (previous)
+    bft_error(__FILE__, __LINE__, 0, "%s:Case not handle.", __func__);
+
+  return eqc->face_values;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1386,21 +1468,26 @@ cs_hho_scaleq_get_face_values(void          *data)
  * \brief  Get the computed values at cells (DoF used in the linear system are
  *         located at primal faces)
  *
- * \param[in, out]  data    pointer to a data structure cast on the fly
+ * \param[in, out]  context    pointer to a data structure cast on-the-fly
+ * \param[in]       previous   retrieve the previous state (true/false)
  *
- * \return  a pointer to an array of \ref cs_real_t
+ * \return  a pointer to an array of cs_real_t
  */
 /*----------------------------------------------------------------------------*/
 
 cs_real_t *
-cs_hho_scaleq_get_cell_values(void          *data)
+cs_hho_scaleq_get_cell_values(void          *context,
+                              bool           previous)
 {
-  cs_hho_scaleq_t  *eqc = (cs_hho_scaleq_t  *)data;
+  cs_hho_scaleq_t  *eqc = (cs_hho_scaleq_t  *)context;
 
   if (eqc == NULL)
     return NULL;
-  else
-    return eqc->cell_values;
+
+  if (previous)
+    bft_error(__FILE__, __LINE__, 0, "%s:Case not handle.", __func__);
+
+  return eqc->cell_values;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1551,29 +1638,23 @@ cs_hho_scaleq_write_restart(cs_restart_t    *restart,
 /*!
  * \brief  Predefined extra-operations related to this equation
  *
- * \param[in]       eqname     name of the equation
- * \param[in]       field      pointer to a field structure
  * \param[in]       eqp        pointer to a cs_equation_param_t structure
  * \param[in, out]  eqb        pointer to a cs_equation_builder_t structure
- * \param[in, out]  data       pointer to cs_hho_scaleq_t structure
+ * \param[in, out]  context    pointer to cs_hho_scaleq_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_hho_scaleq_extra_op(const char                 *eqname,
-                       const cs_field_t           *field,
-                       const cs_equation_param_t  *eqp,
-                       cs_equation_builder_t      *eqb,
-                       void                       *data)
+cs_hho_scaleq_extra_post(const cs_equation_param_t  *eqp,
+                         cs_equation_builder_t      *eqb,
+                         void                       *context)
 {
-  cs_hho_scaleq_t  *eqc = (cs_hho_scaleq_t  *)data;
+  cs_hho_scaleq_t  *eqc = (cs_hho_scaleq_t  *)context;
 
   // TODO
   CS_UNUSED(eqp);
   CS_UNUSED(eqb);
   CS_UNUSED(eqc);
-  CS_UNUSED(field);
-  CS_UNUSED(eqname);
 }
 
 /*----------------------------------------------------------------------------*/
