@@ -9,7 +9,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -146,6 +146,29 @@ cs_hho_scaleq_init_context(const cs_equation_param_t   *eqp,
 void *
 cs_hho_scaleq_free_context(void   *data);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set the initial values of the variable field taking into account
+ *         the boundary conditions.
+ *         Case of scalar-valued HHO schemes.
+ *
+ * \param[in]      t_eval     time at which one evaluates BCs
+ * \param[in]      field_id   id related to the variable field of this equation
+ * \param[in]      mesh       pointer to a cs_mesh_t structure
+ * \param[in]      eqp        pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
+ * \param[in, out] context    pointer to the scheme context (cast on-the-fly)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_hho_scaleq_init_values(cs_real_t                     t_eval,
+                          const int                     field_id,
+                          const cs_mesh_t              *mesh,
+                          const cs_equation_param_t    *eqp,
+                          cs_equation_builder_t        *eqb,
+                          void                         *context);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Create the matrix of the current algebraic system.
@@ -175,7 +198,6 @@ cs_hho_scaleq_initialize_system(const cs_equation_param_t  *eqp,
  *
  * \param[in]      mesh       pointer to a cs_mesh_t structure
  * \param[in]      field_val  pointer to the current value of the field
- * \param[in]      dt_cur     current value of the time step
  * \param[in]      eqp        pointer to a cs_equation_param_t structure
  * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
  * \param[in, out] data       pointer to cs_hho_scaleq_t structure
@@ -187,7 +209,6 @@ cs_hho_scaleq_initialize_system(const cs_equation_param_t  *eqp,
 void
 cs_hho_scaleq_build_system(const cs_mesh_t            *mesh,
                            const cs_real_t            *field_val,
-                           double                      dt_cur,
                            const cs_equation_param_t  *eqp,
                            cs_equation_builder_t      *eqb,
                            void                       *data,
@@ -221,28 +242,32 @@ cs_hho_scaleq_update_field(const cs_real_t            *solu,
  * \brief  Get the computed values at faces (DoF used in the linear system are
  *         located at primal faces)
  *
- * \param[in, out]  data    pointer to a data structure cast on-the-fly
+ * \param[in, out]  context    pointer to a data structure cast on-the-fly
+ * \param[in]       previous   retrieve the previous state (true/false)
  *
- * \return  a pointer to an array of \ref cs_real_t
+ * \return  a pointer to an array of cs_real_t
  */
 /*----------------------------------------------------------------------------*/
 
 cs_real_t *
-cs_hho_scaleq_get_face_values(void          *data);
+cs_hho_scaleq_get_face_values(void          *context,
+                              bool           previous);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Get the computed values at cells (DoF used in the linear system are
  *         located at primal faces)
  *
- * \param[in, out]  data    pointer to a data structure cast on the fly
+ * \param[in, out]  context    pointer to a data structure cast on-the-fly
+ * \param[in]       previous   retrieve the previous state (true/false)
  *
- * \return  a pointer to an array of \ref cs_real_t
+ * \return  a pointer to an array of cs_real_t
  */
 /*----------------------------------------------------------------------------*/
 
 cs_real_t *
-cs_hho_scaleq_get_cell_values(void          *data);
+cs_hho_scaleq_get_cell_values(void          *context,
+                              bool           previous);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -280,20 +305,16 @@ cs_hho_scaleq_write_restart(cs_restart_t    *restart,
 /*!
  * \brief  Predefined extra-operations related to this equation
  *
- * \param[in]       eqname     name of the equation
- * \param[in]       field      pointer to a field structure
  * \param[in]       eqp        pointer to a cs_equation_param_t structure
  * \param[in, out]  eqb        pointer to a cs_equation_builder_t structure
- * \param[in, out]  data       pointer to cs_hho_scaleq_t structure
+ * \param[in, out]  context    pointer to cs_hho_scaleq_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_hho_scaleq_extra_op(const char                 *eqname,
-                       const cs_field_t           *field,
-                       const cs_equation_param_t  *eqp,
-                       cs_equation_builder_t      *eqb,
-                       void                       *data);
+cs_hho_scaleq_extra_post(const cs_equation_param_t  *eqp,
+                         cs_equation_builder_t      *eqb,
+                         void                       *context);
 
 /*----------------------------------------------------------------------------*/
 
