@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2017 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -174,8 +174,8 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
 
   case CS_SPACE_SCHEME_HHO_P0:  /* TODO */
     {
-      BFT_MALLOC(cb->ids, n_fc + 1, short int);
-      memset(cb->ids, 0, (n_fc + 1)*sizeof(short int));
+      BFT_MALLOC(cb->ids, n_fc + 1, int);
+      memset(cb->ids, 0, (n_fc + 1)*sizeof(int));
 
       /* For post-processing errors = 38 */
       size = CS_MAX(38, n_fc*(n_fc+1));
@@ -188,7 +188,6 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
 
       /* Local square dense matrices used during the construction of
          operators */
-      cb->hdg = cs_sdm_square_create(n_fc);
       cb->loc = cs_sdm_square_create(n_fc + 1);
       cb->aux = cs_sdm_square_create(n_fc + 1);
     }
@@ -198,8 +197,8 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
     {
       /* Store the block size description */
       size = n_fc + 1;
-      BFT_MALLOC(cb->ids, size, short int);
-      memset(cb->ids, 0, size*sizeof(short int));
+      BFT_MALLOC(cb->ids, size, int);
+      memset(cb->ids, 0, size*sizeof(int));
 
       /* Store the face, cell and gradient basis function evaluations and
          the Gauss point weights
@@ -221,12 +220,9 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
       memset(cb->vectors, 0, size*sizeof(cs_real_3_t));
 
       /* Local dense matrices used during the construction of operators */
-      const short int g_size = 9;                   /* basis (P_(k+1)) - 1 */
+      const int g_size = 9;                   /* basis (P_(k+1)) - 1 */
       for (int i = 0; i < n_fc; i++) cb->ids[i] = 3;
       cb->ids[n_fc] = 4;
-
-      short int  _sizes[3] = {1, 3, 6}; /* c0, cs-1, cs_kp1 - cs */
-      cb->hdg = cs_sdm_block_create(1, 3, &g_size, _sizes);
       cb->loc = cs_sdm_block_create(n_fc + 1, n_fc + 1, cb->ids, cb->ids);
       cb->aux = cs_sdm_block_create(n_fc + 1, 1, cb->ids, &g_size); /* R_g^T */
     }
@@ -236,8 +232,8 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
     {
       /* Store the block size description */
       size = n_fc + 1;
-      BFT_MALLOC(cb->ids, size, short int);
-      memset(cb->ids, 0, size*sizeof(short int));
+      BFT_MALLOC(cb->ids, size, int);
+      memset(cb->ids, 0, size*sizeof(int));
 
       /* Store the face, cell and gradient basis function evaluations and
          the Gauss point weights */
@@ -259,12 +255,9 @@ _cell_builder_create(cs_param_space_scheme_t     space_scheme,
       memset(cb->vectors, 0, size*sizeof(cs_real_3_t));
 
       /* Local dense matrices used during the construction of operators */
-      const short int g_size = 19; /* basis (P_(k+1)) - 1 */
+      const int g_size = 19; /* basis (P_(k+1)) - 1 */
       for (int i = 0; i < n_fc; i++) cb->ids[i] = 6;
       cb->ids[n_fc] = 10;
-
-      short int  _sizes[3] = {1, 9, 10}; /* c0, cs-1, cs_kp1 - cs */
-      cb->hdg = cs_sdm_block_create(1, 3, &g_size, _sizes);
       cb->loc = cs_sdm_block_create(n_fc + 1, n_fc + 1, cb->ids, cb->ids);
       cb->aux = cs_sdm_block_create(n_fc + 1, 1, cb->ids, &g_size); /* R_g^T */
     }
@@ -485,7 +478,7 @@ cs_hho_stokes_init_context(const cs_equation_param_t   *eqp,
     bft_error(__FILE__, __LINE__, 0, " Expected: scalar-valued HHO equation.");
 
   const cs_cdo_connect_t  *connect = cs_shared_connect;
-  const cs_lnum_t  n_faces = connect->n_faces[0];
+  const cs_lnum_t  n_faces = connect->n_faces[CS_ALL_FACES];
   const cs_lnum_t  n_cells = connect->n_cells;
 
   cs_hho_stokes_t  *eqc = NULL;
@@ -493,9 +486,9 @@ cs_hho_stokes_init_context(const cs_equation_param_t   *eqp,
   BFT_MALLOC(eqc, 1, cs_hho_stokes_t);
 
   /* Mesh flag to know what to build */
-  eqb->msh_flag = CS_CDO_LOCAL_PV | CS_CDO_LOCAL_PEQ | CS_CDO_LOCAL_PFQ |
-    CS_CDO_LOCAL_FE | CS_CDO_LOCAL_FEQ | CS_CDO_LOCAL_HFQ |
-    CS_CDO_LOCAL_EV | CS_CDO_LOCAL_DIAM;
+  eqb->msh_flag = CS_FLAG_COMP_PV | CS_FLAG_COMP_PEQ | CS_FLAG_COMP_PFQ |
+    CS_FLAG_COMP_FE | CS_FLAG_COMP_FEQ | CS_FLAG_COMP_HFQ |
+    CS_FLAG_COMP_EV | CS_FLAG_COMP_DIAM;
 
   switch (eqp->space_scheme) {
 
@@ -561,14 +554,14 @@ cs_hho_stokes_init_context(const cs_equation_param_t   *eqp,
   memset(eqc->rc_tilda, 0, sizeof(cs_real_t)*n_cell_dofs);
 
   cs_lnum_t  n_row_blocks = connect->c2f->idx[n_cells];
-  short int  *row_block_sizes = NULL;
+  int  *row_block_sizes = NULL;
 
-  BFT_MALLOC(row_block_sizes, n_row_blocks, short int);
+  BFT_MALLOC(row_block_sizes, n_row_blocks, int);
 # pragma omp parallel for if (n_cells > CS_THR_MIN)
   for (cs_lnum_t i = 0; i < n_row_blocks; i++)
     row_block_sizes[i] = eqc->n_face_dofs;
 
-  short int  col_block_size = eqc->n_cell_dofs;
+  int  col_block_size = eqc->n_cell_dofs;
   eqc->acf_tilda = cs_sdm_block_create(n_row_blocks, 1,
                                        row_block_sizes, &col_block_size);
   cs_sdm_block_init(eqc->acf_tilda,
@@ -578,7 +571,7 @@ cs_hho_stokes_init_context(const cs_equation_param_t   *eqp,
   BFT_FREE(row_block_sizes);
 
   /* Handle boundary conditions */
-  const cs_lnum_t  n_b_faces = connect->n_faces[1];
+  const cs_lnum_t  n_b_faces = connect->n_faces[CS_BND_FACES];
   BFT_MALLOC(eqc->bf2def_ids, n_b_faces, short int);
 
 # pragma omp parallel for if (n_b_faces > CS_THR_MIN)
@@ -657,14 +650,12 @@ cs_hho_stokes_compute_source(const cs_equation_param_t  *eqp,
   if (cs_equation_param_has_sourceterm(eqp) == false)
     return;
 
-  const cs_cdo_connect_t  *connect = cs_shared_connect;
-
   cs_timer_t  t0 = cs_timer_time();
 
   /* TODO */
 
   cs_timer_t  t1 = cs_timer_time();
-  cs_timer_counter_add_diff(&(eqb->tcs), &t0, &t1);
+  cs_timer_counter_add_diff(&(eqb->tcb), &t0, &t1);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -744,7 +735,7 @@ cs_hho_stokes_build_system(const cs_mesh_t            *mesh,
   /* Sanity checks */
   assert(rhs != NULL && matrix != NULL && eqp != NULL && eqb != NULL);
   /* The only way to set a Dirichlet up to now */
-  assert(eqp->enforcement == CS_PARAM_BC_ENFORCE_WEAK_PENA);
+  assert(eqp->default_enforcement == CS_PARAM_BC_ENFORCE_PENALIZED);
 
   /* Test to remove */
   if (cs_equation_param_has_convection(eqp))
@@ -762,6 +753,11 @@ cs_hho_stokes_build_system(const cs_mesh_t            *mesh,
   cs_timer_t  t0 = cs_timer_time();
 
   /* TODO */
+  CS_UNUSED(quant);
+  CS_UNUSED(connect);
+  CS_UNUSED(eqc);
+  CS_UNUSED(rhs);
+  CS_UNUSED(matrix);
 
   cs_timer_t  t1 = cs_timer_time();
   cs_timer_counter_add_diff(&(eqb->tcb), &t0, &t1);
@@ -791,6 +787,8 @@ cs_hho_stokes_update_field(const cs_real_t            *solu,
 {
   CS_UNUSED(rhs);
   CS_UNUSED(eqp);
+  CS_UNUSED(solu);
+  CS_UNUSED(field_val);
 
   cs_timer_t  t0 = cs_timer_time();
 
@@ -799,6 +797,11 @@ cs_hho_stokes_update_field(const cs_real_t            *solu,
 
   cs_hho_stokes_t  *eqc = (cs_hho_stokes_t  *)data;
 
+  CS_UNUSED(quant);
+  CS_UNUSED(connect);
+  CS_UNUSED(eqc);
+
+  /* TODO */
 
   cs_timer_t  t1 = cs_timer_time();
   cs_timer_counter_add_diff(&(eqb->tce), &t0, &t1);
