@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -53,7 +53,6 @@
 
 #include "cs_boundary_zone.h"
 #include "cs_domain.h"
-#include "cs_domain_boundary.h"
 #include "cs_domain_setup.h"
 #include "cs_equation.h"
 #include "cs_mesh_builder.h"
@@ -99,6 +98,11 @@ static cs_lnum_t     _vd_size = 0;
 static cs_real_3_t  *_vd = NULL;
 static cs_lnum_t     _cs_comp_id[] = {0, 1, 2};
 
+static bool          _fixed_vertices = false;
+static cs_lnum_t     _n_fixed_vertices = 0;
+static cs_lnum_t    *_fixed_vtx_ids = NULL;
+static cs_real_3_t  *_fixed_vtx_values = NULL;
+
 /*=============================================================================
  * Private function definitions
  *============================================================================*/
@@ -210,13 +214,7 @@ cs_mesh_deform_activate(void)
 
     /* System to solve is SPD by construction */
     cs_equation_set_param(eqp, CS_EQKEY_ITSOL, "cg");
-
-#if defined(HAVE_PETSC)  /* Modify the default settings */
-    cs_equation_set_param(eqp, CS_EQKEY_SOLVER_FAMILY, "petsc");
     cs_equation_set_param(eqp, CS_EQKEY_PRECOND, "amg");
-#else
-    cs_equation_set_param(eqp, CS_EQKEY_PRECOND, "jacobi");
-#endif
 
   }
 
@@ -262,15 +260,11 @@ cs_mesh_deform_setup(cs_domain_t  *domain)
 
   cs_property_t  *conductivity = cs_property_by_name("unity");
 
-  /* Retrieve the equation to set
-     >> cs_equation_t  *eq = cs_equation_by_name("eq_name");  */
+  /* Retrieve the equation parameter to set
+     >> cs_equation_param_t  *eqp = cs_equation_param_by_name("eq_name");  */
 
   const char *eq_name[] = {"mesh_deform_x", "mesh_deform_y", "mesh_deform_z"};
 
-  /* TODO implement a finer control to make mesh deformation
-     compatible with other CDO modules */
-  cs_domain_boundary_set_default(CS_DOMAIN_BOUNDARY_SYMMETRY);
-
   for (int i = 0; i < 3; i++) {
 
     cs_equation_param_t  *eqp = cs_equation_param_by_name(eq_name[i]);
@@ -284,6 +278,29 @@ cs_mesh_deform_setup(cs_domain_t  *domain)
                                      _cs_comp_id + i);
     }
 
+    if (_fixed_vertices) {
+      cs_real_t  *fixed_vtx_values;
+      BFT_MALLOC(fixed_vtx_values, _n_fixed_vertices, cs_real_t);
+      if (_fixed_vtx_values != NULL) {
+#       pragma omp parallel for if (_n_fixed_vertices > CS_THR_MIN)
+        for (cs_lnum_t j = 0; j < _n_fixed_vertices; j++)
+          fixed_vtx_values[j] = _fixed_vtx_values[j][i];
+      }
+      else {
+#       pragma omp parallel for if (_n_fixed_vertices > CS_THR_MIN)
+        for (cs_lnum_t j = 0; j < _n_fixed_vertices; j++)
+          fixed_vtx_values[j] = 0;
+      }
+
+      cs_equation_enforce_vertex_dofs(eqp,
+                                      _n_fixed_vertices,
+                                      _fixed_vtx_ids,
+                                      NULL, /* reference value */
+                                      fixed_vtx_values);
+
+      BFT_FREE(fixed_vtx_values);
+    }
+
     cs_equation_add_diffusion(eqp, conductivity);
 
   }
@@ -297,6 +314,10 @@ cs_mesh_deform_setup(cs_domain_t  *domain)
  * displacement will really be used, as defined by
  * \ref cs_mesh_deform_define_dirichlet_bc_zones.
  *
+ * When calling this function multiple times for different vertex sets,
+ * the most recently defined values are used for vertices belonging to
+ * multiple sets.
+ *
  * \param[in]  n_vertices         number of vertices at which to prescribe
  *                                displacements
  * \param[in]  vertex_ids         ids of vertices at which to prescribe
@@ -341,6 +362,65 @@ cs_mesh_deform_prescribe_displacement(cs_lnum_t          n_vertices,
   }
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define a fixed displacement vector for given vertices.
+ *
+ * This displacement is enforced at all given vertices, including interior
+ * vertices.
+ *
+ * If this function is called multiple times, the previous definitions
+ * are overwritten, so all displacements of this type must be defined
+ * in a single call to this function.
+ *
+ * \param[in]  n_vertices         number of vertices at which to prescribe
+ *                                displacements
+ * \param[in]  vertex_ids         ids of vertices at which to prescribe
+ *                                displacements, or NULL if
+ *                                [0, ... n_vertices-1]
+ * \param[in]  displacement       pointer to prescribed displacements,
+ *                                or NULL for no displacement
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_mesh_deform_force_displacements(cs_lnum_t          n_vertices,
+                                   const cs_lnum_t    vertex_ids[],
+                                   const cs_real_3_t  displacement[])
+{
+  BFT_REALLOC(_fixed_vtx_ids, n_vertices, cs_lnum_t);
+
+  if (displacement != NULL)
+    BFT_REALLOC(_fixed_vtx_values, n_vertices, cs_real_3_t);
+  else
+    BFT_FREE(_fixed_vtx_values);
+
+  _n_fixed_vertices = n_vertices;
+
+  cs_gnum_t n_g_v = n_vertices;
+  cs_parall_counter(&n_g_v, 1);
+  _fixed_vertices = (n_g_v > 0) ? true : false;
+
+  if (vertex_ids != NULL) {
+#   pragma omp parallel for if (n_vertices > CS_THR_MIN)
+    for (cs_lnum_t i = 0; i < n_vertices; i++)
+      _fixed_vtx_ids[i] = vertex_ids[i];
+  }
+  else {
+#   pragma omp parallel for if (n_vertices > CS_THR_MIN)
+    for (cs_lnum_t i = 0; i < n_vertices; i++)
+      _fixed_vtx_ids[i] = i;
+  }
+
+  if (displacement != NULL) {
+#   pragma omp parallel for if (n_vertices > CS_THR_MIN)
+    for (cs_lnum_t i = 0; i < n_vertices; i++) {
+      for (cs_lnum_t j = 0; j < 3; j++)
+        _fixed_vtx_values[i][j] = displacement[i][j];
+    }
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Compute displacement for mesh deformation.
@@ -352,13 +432,6 @@ cs_mesh_deform_prescribe_displacement(cs_lnum_t          n_vertices,
 void
 cs_mesh_deform_solve_displacement(cs_domain_t  *domain)
 {
-  static bool initialized = false;
-
-  if (initialized == false) {
-    cs_domain_initialize_systems(domain);
-    initialized = true;
-  }
-
   /* Build and solve equations related to the deformation */
 
   const char *eq_name[] = {"mesh_deform_x", "mesh_deform_y", "mesh_deform_z"};
@@ -376,10 +449,7 @@ cs_mesh_deform_solve_displacement(cs_domain_t  *domain)
       assert(cs_equation_is_steady(eq));
 
       /* Define the algebraic system */
-      cs_equation_build_system(domain->mesh,
-                               domain->time_step,
-                               domain->dt_cur,
-                               eq);
+      cs_equation_build_system(domain->mesh, eq);
 
       /* Solve the algebraic system */
       cs_equation_solve_deprecated(eq);
@@ -434,6 +504,13 @@ cs_mesh_deform_finalize(void)
 
   BFT_FREE(_vd);
   _vd_size = 0;
+
+  BFT_FREE(_fixed_vtx_ids);
+  BFT_FREE(_fixed_vtx_values);
+  _n_fixed_vertices = 0;
+  _fixed_vertices = false;
+
+  _active = false;
 }
 
 /*---------------------------------------------------------------------------*/
