@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -31,8 +31,9 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
+#include "cs_field.h"
 #include "cs_flag.h"
-#include "cs_param.h"
+#include "cs_param_types.h"
 #include "cs_xdef.h"
 #include "cs_xdef_cw_eval.h"
 #include "cs_xdef_eval.h"
@@ -45,6 +46,10 @@ BEGIN_C_DECLS
  * Macro definitions
  *============================================================================*/
 
+/* Common property names (property which is shared between different module) */
+
+#define CS_PROPERTY_MASS_DENSITY   "mass_density"
+
 /*!
  * @defgroup cdo_property_flags Flags specifying metadata related to the
  *  post-processing for a property
@@ -56,10 +61,46 @@ BEGIN_C_DECLS
 
 /*! @} */
 
+/*!
+ * @defgroup cdo_property_type Flags specifying metadata related to the
+ *  the type of property
+ * @{
+ */
+
+/*! \var CS_PROPERTY_ISO
+ *  1: Isotropic behavior (one real number is sufficient to describe the
+ *  property) */
+#define CS_PROPERTY_ISO           (1 << 0)
+
+/*! \var CS_PROPERTY_ORTHO
+ *  2: Orthotropic behavior (three real numbers describe the behavior assuming
+ *  that the different behavior is aligned with Cartesian axis) */
+#define CS_PROPERTY_ORTHO         (1 << 1)
+
+/*! \var CS_PROPERTY_ANISO
+ *  4: Anisotropic behavior (a 3x3 tensor describe the behavior). This tensor
+ *  should be symmetric positive definite (i.e 6 real numbers describe the
+ *  behavior) but by default a 3x3 tensor is used. */
+#define CS_PROPERTY_ANISO         (1 << 2)
+
+/*! \var CS_PROPERTY_ANISO_SYM
+ *  8: Anisotropic behavior. This tensor is represented with 6 real numbers
+ *  since the tensor is symmetric */
+#define CS_PROPERTY_ANISO_SYM     (1 << 3)
+
+/*! \var CS_PROPERTY_BY_PRODUCT
+ *  16: The property is defined as the product of two other properties
+ */
+#define CS_PROPERTY_BY_PRODUCT    (1 << 4)
+
+/*! @} */
+
 /*============================================================================
  * Type definitions
  *============================================================================*/
 
+typedef cs_flag_t cs_property_type_t;
+
 /*! \enum cs_property_key_t
  *  \brief List of available keys for setting options on a property
  *
@@ -74,50 +115,38 @@ typedef enum {
 
 } cs_property_key_t;
 
-/*! \enum cs_property_type_t
- *  \brief Type of property to consider
- *
- *  \var CS_PROPERTY_ISO
- *  Isotropic behavior (one real number is sufficient to describe the property)
- *
- *  \var CS_PROPERTY_ORTHO
- *  Orthotropic behavior (three real numbers describe the behavior assuming
- *  that the different behavior is aligned with Cartesian axis)
- *
- *  \var CS_PROPERTY_ANISO
- *  Anisotropic behavior (a 3x3 tensor describe the behavior). This tensor
- *  should be symmetric positive definite (i.e 6 real numbers describe the
- *  behavior).
+/* ======================================== */
+/* Set of parameters attached to a property */
+/* ======================================== */
+
+/*!
+ * \struct cs_property_t
+ * \brief Structure associated to the definition of a property relying on the
+ * \ref cs_xdef_t structure
  */
 
-typedef enum {
+typedef struct _cs_property_t cs_property_t;
 
-  CS_PROPERTY_ISO,
-  CS_PROPERTY_ORTHO,
-  CS_PROPERTY_ANISO,
-
-  CS_PROPERTY_N_TYPES
-
-} cs_property_type_t;
-
-/* Set of parameters attached to a property */
-typedef struct {
+struct _cs_property_t {
 
   char  *restrict      name;
   int                  id;
   cs_flag_t            state_flag;
   cs_flag_t            process_flag;
-
-  /* The number of values to set depends on the type of property
-      isotropic   = 1, orthotropic = 3, anisotropic = 9  */
   cs_property_type_t   type;
 
+  /* Reference value wich is used as default when nothing else is set. This
+   * value can also be used to renormalized quantities related to this property
+   * By default, this is set to 1
+   */
+  cs_real_t            ref_value;
+
   /* Property is up to now only defined on the whole domain (volume) */
   int                  n_definitions;  /* Current number of definitions used */
   cs_xdef_t          **defs;           /* List of definitions */
 
-  /* Store the definition id for each cell, NULL is only one
-     definition is set */
+  /* Store the definition id for each cell, NULL if there is only one
+     definition set */
   short int           *def_ids;
 
   /* Function pointers to handle generic tasks related to a property. There
@@ -132,12 +161,175 @@ typedef struct {
      relying on a cs_cell_mesh_t structure */
   cs_xdef_cw_eval_t  **get_eval_at_cell_cw;
 
-} cs_property_t;
+  /* For properties relying on other properties for their definition, one
+   * stores the pointers to these related properties */
+  int                     n_related_properties;
+  const cs_property_t   **related_properties;
+
+};
+
+
+/*!
+ * \struct cs_property_data_t
+ * \brief Structure storing the evaluation of a property and its related
+ *        data
+ */
+
+typedef struct {
+
+  const cs_property_t   *property; /* shared pointer */
+
+  bool                   is_iso;   /* Detect if this an easier case */
+  bool                   is_unity; /* Detect if this a simple case */
+
+  bool                   need_eigen;
+  cs_real_t              eigen_max;
+  cs_real_t              eigen_ratio;
+
+  bool                   need_tensor;
+  cs_real_t              tensor[3][3];
+  cs_real_t              value;
+
+} cs_property_data_t;
 
 /*============================================================================
  * Global variables
  *============================================================================*/
 
+/*============================================================================
+ * Static inline public function prototypes
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  returns true if the property is steady and uniform, otherwise false
+ *
+ * \param[in]    pty    pointer to a property to test
+ *
+ * \return  true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline bool
+cs_property_is_constant(const cs_property_t   *pty)
+{
+  if (pty == NULL)
+    return true; /* Treated as the "unity" property */
+
+  if (pty->state_flag & CS_FLAG_STATE_STEADY) {
+    if (pty->state_flag & CS_FLAG_STATE_UNIFORM)
+      return true;
+    else
+      return false;
+  }
+  else
+    return false;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  returns true if the property is steady, otherwise false
+ *
+ * \param[in]    pty    pointer to a property to test
+ *
+ * \return  true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline bool
+cs_property_is_steady(const cs_property_t   *pty)
+{
+  if (pty == NULL)
+    return true; /* Treated as the "unity" property */
+
+  if (pty->state_flag & CS_FLAG_STATE_STEADY)
+    return true;
+  else
+    return false;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  returns true if the property is uniform, otherwise false
+ *
+ * \param[in]    pty    pointer to a property to test
+ *
+ * \return  true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline bool
+cs_property_is_uniform(const cs_property_t   *pty)
+{
+  if (pty == NULL)
+    return true; /* Treated as the "unity" property */
+
+  if (pty->state_flag & CS_FLAG_STATE_UNIFORM)
+    return true;
+  else
+    return false;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  returns true if the property is isotropic, otherwise false
+ *
+ * \param[in]    pty    pointer to a property to test
+ *
+ * \return  true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline bool
+cs_property_is_isotropic(const cs_property_t   *pty)
+{
+  if (pty == NULL)
+    return false;
+
+  if (pty->type & CS_PROPERTY_ISO)
+    return true;
+  else
+    return false;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve the name of a property
+ *
+ * \param[in]    pty    pointer to a property
+ *
+ * \return  the name of the related property
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline const char *
+cs_property_get_name(const cs_property_t   *pty)
+{
+  if (pty == NULL)
+    return NULL;
+
+  return pty->name;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve the type of a property
+ *
+ * \param[in]    pty    pointer to a property
+ *
+ * \return  the type of the related property
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline cs_property_type_t
+cs_property_get_type(const cs_property_t   *pty)
+{
+  if (pty == NULL)
+    return 0; /* means undefined */
+
+  return pty->type;
+}
+
 /*============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -181,6 +373,27 @@ cs_property_t *
 cs_property_add(const char            *name,
                 cs_property_type_t     type);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define a cs_property_t structure thanks to the product of two
+ *         properties
+ *         The type is infered from that of the related properties
+ *         The value of the property is given as:
+ *         value_ab = value_a * value_b
+ *
+ * \param[in]       name      name of the property
+ * \param[in]       pty_a     pointer to a cs_property_t structure
+ * \param[in]       pty_b     pointer to a cs_property_t structure
+ *
+ * \return a pointer to a new allocated cs_property_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_property_t *
+cs_property_add_as_product(const char             *name,
+                           const cs_property_t    *pty_a,
+                           const cs_property_t    *pty_b);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Find the related property definition from its name
@@ -220,6 +433,20 @@ void
 cs_property_set_option(cs_property_t       *pty,
                        cs_property_key_t    key);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set the reference value associated to a \ref cs_property_t structure
+ *         This is a real number even whatever the type of property is.
+ *
+ * \param[in, out]  pty      pointer to a cs_property_t structure
+ * \param[in]       refval   value to set
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_property_set_reference_value(cs_property_t    *pty,
+                                double            refval);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Free all cs_property_t structures and the array storing all the
@@ -242,85 +469,39 @@ cs_property_finalize_setup(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  returns true if the property is uniform, otherwise false
+ * \brief  Initialize a \ref cs_property_data_t structure. If property is NULL
+ *         then one considers that this is a unitary property
  *
- * \param[in]    pty    pointer to a property to test
- *
- * \return  true or false
+ * \param[in]      need_tensor  true if one needs a tensor-valued evaluation
+ * \param[in]      need_eigen   true if one needs an evaluation of eigen values
+ * \param[in]      property     pointer to the \ref cs_property_t structure
+ * \param[in, out] data         structure to initialize (already allocated)
  */
 /*----------------------------------------------------------------------------*/
 
-static inline bool
-cs_property_is_uniform(const cs_property_t   *pty)
-{
-  if (pty == NULL)
-    return false;
-
-  if (pty->state_flag & CS_FLAG_STATE_UNIFORM)
-    return true;
-  else
-    return false;
-}
+void
+cs_property_data_init(bool                     need_tensor,
+                      bool                     need_eigen,
+                      const cs_property_t     *property,
+                      cs_property_data_t      *data);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  returns true if the property is isotropic, otherwise false
+ * \brief  Define a single uniform and steady isotropic definition for the
+ *         given cs_property_t structure.
+ *         This is a specialized variant of \ref cs_property_def_iso_by_value
+ *         since several assumptions are satisfied.
  *
- * \param[in]    pty    pointer to a property to test
+ * \param[in, out]  pty      pointer to a cs_property_t structure
+ * \param[in]       val      value to set
  *
- * \return  true or false
+ * \return a pointer to the resulting cs_xdef_t structure
  */
 /*----------------------------------------------------------------------------*/
 
-static inline bool
-cs_property_is_isotropic(const cs_property_t   *pty)
-{
-  if (pty == NULL)
-    return false;
-
-  if (pty->type == CS_PROPERTY_ISO)
-    return true;
-  else
-    return false;
-}
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Retrieve the name of a property
- *
- * \param[in]    pty    pointer to a property
- *
- * \return  the name of the related property
- */
-/*----------------------------------------------------------------------------*/
-
-static inline const char *
-cs_property_get_name(const cs_property_t   *pty)
-{
-  if (pty == NULL)
-    return NULL;
-
-  return pty->name;
-}
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Retrieve the type of a property
- *
- * \param[in]    pty    pointer to a property
- *
- * \return  the type of the related property
- */
-/*----------------------------------------------------------------------------*/
-
-static inline cs_property_type_t
-cs_property_get_type(const cs_property_t   *pty)
-{
-  if (pty == NULL)
-    return CS_PROPERTY_N_TYPES;
-
-  return pty->type;
-}
+cs_xdef_t *
+cs_property_def_constant_value(cs_property_t    *pty,
+                               double            val);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -379,6 +560,46 @@ cs_property_def_aniso_by_value(cs_property_t    *pty,
                                const char       *zname,
                                cs_real_t         tens[3][3]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define an anisotropic cs_property_t structure by value for entities
+ *         related to a volume zone
+ *
+ * \param[in, out]  pty      pointer to a cs_property_t structure
+ * \param[in]       zname    name of the associated zone (if NULL or "" all
+ *                           cells are considered)
+ * \param[in]       tens     values to set (3x3 tensor)
+ *
+ * \return a pointer to the resulting cs_xdef_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_xdef_t *
+cs_property_def_aniso_sym_by_value(cs_property_t    *pty,
+                                   const char       *zname,
+                                   cs_real_t         symtens[6]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define a cs_property_t structure thanks to an analytic function in
+ *         a subdomain attached to the mesh location named ml_name
+ *
+ * \param[in, out]  pty      pointer to a cs_property_t structure
+ * \param[in]       zname    name of the associated zone (if NULL or "" all
+ *                           cells are considered)
+ * \param[in]       func     pointer to a cs_analytic_func_t function
+ * \param[in]       input    NULL or pointer to a structure cast on-the-fly
+ *
+ * \return a pointer to the resulting cs_xdef_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_xdef_t *
+cs_property_def_by_time_func(cs_property_t      *pty,
+                             const char         *zname,
+                             cs_time_func_t     *func,
+                             void               *input);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Define a cs_property_t structure thanks to an analytic function in
@@ -431,6 +652,8 @@ cs_property_def_by_func(cs_property_t         *pty,
  * \param[in, out]  pty       pointer to a cs_property_t structure
  * \param[in]       loc       information to know where are located values
  * \param[in]       array     pointer to an array
+ * \param[in]       is_owner  transfer the lifecycle to the cs_xdef_t structure
+ *                            (true or false)
  * \param[in]       index     optional pointer to the array index
  *
  * \return a pointer to the resulting cs_xdef_t structure
@@ -441,11 +664,12 @@ cs_xdef_t *
 cs_property_def_by_array(cs_property_t    *pty,
                          cs_flag_t         loc,
                          cs_real_t        *array,
+                         bool              is_owner,
                          cs_lnum_t        *index);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Define a cs_property_t structure thanks to an array of values
+ * \brief  Define a cs_property_t structure thanks to a field structure
  *
  * \param[in, out]  pty       pointer to a cs_property_t structure
  * \param[in]       field     pointer to a cs_field_t structure
@@ -474,7 +698,7 @@ cs_property_eval_at_cells(cs_real_t               t_eval,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Compute the value of the tensor attached a property at the cell
+ * \brief  Compute the value of the tensor attached to a property at the cell
  *         center
  *
  * \param[in]      c_id          id of the current cell
@@ -490,7 +714,7 @@ cs_property_get_cell_tensor(cs_lnum_t               c_id,
                             cs_real_t               t_eval,
                             const cs_property_t    *pty,
                             bool                    do_inversion,
-                            cs_real_3_t            *tensor);
+                            cs_real_t               tensor[3][3]);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -511,7 +735,7 @@ cs_property_get_cell_value(cs_lnum_t              c_id,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Compute the value of the tensor attached a property at the cell
+ * \brief  Compute the value of the tensor attached to a property at the cell
  *         center
  *         Version using a cs_cell_mesh_t structure
  *
@@ -528,7 +752,7 @@ cs_property_tensor_in_cell(const cs_cell_mesh_t   *cm,
                            const cs_property_t    *pty,
                            cs_real_t               t_eval,
                            bool                    do_inversion,
-                           cs_real_3_t            *tensor);
+                           cs_real_t               tensor[3][3]);
 
 /*----------------------------------------------------------------------------*/
 /*!
