@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -35,6 +35,8 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
+#include "cs_flag.h"
+
 /*----------------------------------------------------------------------------
  * Header for the current file
  *----------------------------------------------------------------------------*/
@@ -169,10 +171,8 @@ cs_quadrature_edge_2pts(const cs_real_3_t  v1,
                         cs_real_3_t        gpts[],
                         double             w[])
 {
-  int  k;
-
   /* Compute quadrature points */
-  for (k = 0; k < 3; k++) {
+  for (int k = 0; k < 3; k++) {
     gpts[0][k] = v1[k]*_edge_quad2c2 + v2[k]*_edge_quad2c1;
     gpts[1][k] = v1[k]*_edge_quad2c1 + v2[k]*_edge_quad2c2;
   }
@@ -201,12 +201,10 @@ cs_quadrature_edge_3pts(const cs_real_3_t  v1,
                         cs_real_3_t        gpts[],
                         double             w[])
 {
-  int  k;
-
   const double  b = len * _quad_over18;
 
   /* Compute quadrature points */
-  for (k = 0; k < 3; k++) {
+  for (int k = 0; k < 3; k++) {
     gpts[0][k] = 0.5*( v1[k] + v2[k] );
     gpts[1][k] = v1[k]*_edge_quad3c1 + v2[k]*_edge_quad3c2;
     gpts[2][k] = v1[k]*_edge_quad3c2 + v2[k]*_edge_quad3c1;
@@ -238,10 +236,8 @@ cs_quadrature_tria_3pts(const cs_real_3_t   v1,
                         cs_real_3_t         gpts[],
                         double              w[])
 {
-  int  k;
-
   /* Compute quadrature points */
-  for (k = 0; k < 3; k++) {
+  for (int k = 0; k < 3; k++) {
     gpts[0][k] = 0.5*( v1[k] + v2[k] );
     gpts[1][k] = 0.5*( v1[k] + v3[k] );
     gpts[2][k] = 0.5*( v2[k] + v3[k] );
@@ -273,10 +269,8 @@ cs_quadrature_tria_4pts(const cs_real_3_t   v1,
                         cs_real_3_t         gpts[],
                         double              w[])
 {
-  int  k;
-
   /* Compute quadrature points */
-  for (k = 0; k < 3; k++) {
+  for (int k = 0; k < 3; k++) {
     gpts[0][k] = _quad_over3*( v1[k] + v2[k] + v3[k] );
     gpts[1][k] = 0.2*( v1[k] + v2[k] ) + 0.6*v3[k];
     gpts[2][k] = 0.2*( v1[k] + v3[k] ) + 0.6*v2[k];
@@ -310,10 +304,8 @@ cs_quadrature_tria_7pts(const cs_real_3_t   v1,
                         cs_real_3_t         gpts[],
                         double              w[])
 {
-  int  k;
-
   /* Compute quadrature points */
-  for (k = 0; k < 3; k++) {
+  for (int k = 0; k < 3; k++) {
     gpts[0][k] = _quad_over3*( v1[k] + v2[k] + v3[k] );
     gpts[1][k] = _tria_quad7c1* (v1[k] + v2[k]) + (1-2*_tria_quad7c1)*v3[k];
     gpts[2][k] = _tria_quad7c1* (v3[k] + v1[k]) + (1-2*_tria_quad7c1)*v2[k];
@@ -478,6 +470,123 @@ cs_quadrature_tet_15pts(const cs_real_3_t   v1,
   weights[14] = vol * _tetr_quad15w4;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Get the flags adapted to the given quadrature type \p qtype and the
+ *         location on which the quadrature should be performed
+ *
+ * \param[in] qtype    \ref cs_quadrature_type_t
+ * \param[in] loc      It could be \ref CS_FLAG_CELL, \ref CS_FLAG_FACE or
+ *                     \ref CS_FLAG_EDGE plus \ref CS_FLAG_PRIMAL or
+ *                     \ref CS_FLAG_DUAL
+ *
+ * \return  metadata stored in a \ref cs_eflag_t to build a \ref cs_cell_mesh_t
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_eflag_t
+cs_quadrature_get_flag(const cs_quadrature_type_t  qtype,
+                       const cs_flag_t             loc)
+{
+  cs_eflag_t ret_flag = 0;
+
+  /* If necessary, enrich the mesh flag to account for the property */
+  switch (qtype) {
+
+  case CS_QUADRATURE_HIGHER:
+  case CS_QUADRATURE_HIGHEST:
+    ret_flag |= CS_FLAG_COMP_PFQ | CS_FLAG_COMP_PEQ;
+    /* No break, pass to the following too */
+  case CS_QUADRATURE_BARY_SUBDIV:
+    ret_flag |= CS_FLAG_COMP_EV | CS_FLAG_COMP_FE | CS_FLAG_COMP_FEQ;
+    break;
+
+  default:
+    /* Nothing to do */
+    break;
+
+  } /* Switch */
+
+  if (cs_flag_test(loc, cs_flag_primal_cell)) {
+
+    switch (qtype) {
+
+    case CS_QUADRATURE_HIGHER:
+    case CS_QUADRATURE_HIGHEST:
+      ret_flag |= CS_FLAG_COMP_PFQ | CS_FLAG_COMP_PEQ | CS_FLAG_COMP_HFQ;
+      /* No break, pass to the following too */
+    case CS_QUADRATURE_BARY_SUBDIV:
+      ret_flag |= CS_FLAG_COMP_EV | CS_FLAG_COMP_FE | CS_FLAG_COMP_FEQ;
+      break;
+
+    default:
+      /* Nothing to do */
+      break;
+
+    } /* Switch */
+
+  } /* Primal cells */
+  else if (cs_flag_test(loc, cs_flag_primal_face)) {
+
+    switch (qtype) {
+
+    case CS_QUADRATURE_HIGHER:
+    case CS_QUADRATURE_HIGHEST:
+      ret_flag |= CS_FLAG_COMP_PFQ | CS_FLAG_COMP_PEQ;
+      /* No break, pass to the following too */
+    case CS_QUADRATURE_BARY_SUBDIV:
+      ret_flag |= CS_FLAG_COMP_EV | CS_FLAG_COMP_FE | CS_FLAG_COMP_FEQ |
+        CS_FLAG_COMP_PF;
+      break;
+
+    default:
+      /* Nothing to do */
+      break;
+
+    } /* Switch */
+
+  } /* Primal faces */
+  else if (cs_flag_test(loc, cs_flag_dual_face)) {
+
+    switch (qtype) {
+
+    case CS_QUADRATURE_HIGHER:
+    case CS_QUADRATURE_HIGHEST:
+      ret_flag |= CS_FLAG_COMP_DFQ | CS_FLAG_COMP_SEF;
+      /* No break, pass to the following too */
+    case CS_QUADRATURE_BARY_SUBDIV:
+      ret_flag |= CS_FLAG_COMP_EF | CS_FLAG_COMP_DFQ | CS_FLAG_COMP_PEQ |
+        CS_FLAG_COMP_PV | CS_FLAG_COMP_PE;
+      break;
+
+    default:
+      /* Nothing to do */
+      break;
+
+    } /* Switch */
+
+  } /* Dual faces */
+  else if (cs_flag_test(loc, cs_flag_primal_edge)) {
+
+    switch (qtype) {
+
+    case CS_QUADRATURE_HIGHER:
+    case CS_QUADRATURE_HIGHEST:
+    case CS_QUADRATURE_BARY_SUBDIV:
+      ret_flag |= CS_FLAG_COMP_PEQ;
+      break;
+
+    default:
+      /* Nothing to do */
+      break;
+
+    } /* Switch */
+
+  } /* Primal edge */
+
+  return ret_flag;
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
