@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -33,8 +33,9 @@
 
 #include "cs_base.h"
 #include "cs_defs.h"
+#include "cs_flag.h"
 #include "cs_math.h"
-#include "cs_param.h"
+#include "cs_param_types.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -317,9 +318,9 @@ cs_quadrature_tria_1pt(const cs_real_3_t   v1,
                        cs_real_3_t         gpts[],
                        double             *w)
 {
-  gpts[0][0] = cs_math_onethird * (v1[0] + v2[0] + v3[0]);
-  gpts[0][1] = cs_math_onethird * (v1[1] + v2[1] + v3[1]);
-  gpts[0][2] = cs_math_onethird * (v1[2] + v2[2] + v3[2]);
+  gpts[0][0] = cs_math_1ov3 * (v1[0] + v2[0] + v3[0]);
+  gpts[0][1] = cs_math_1ov3 * (v1[1] + v2[1] + v3[1]);
+  gpts[0][2] = cs_math_1ov3 * (v1[2] + v2[2] + v3[2]);
   w[0] = area;
 }
 
@@ -517,16 +518,18 @@ cs_quadrature_edge_1pt_scal(double                 tcur,
                             double                 results[])
 {
   cs_real_3_t  xg;
-  double  evaluation;
+  double  feval;
 
-  // Copied from cs_quadrature_1pt
+  /* Copied from cs_quadrature_1pt */
   xg[0] = .5 * (v1[0] + v2[0]);
   xg[1] = .5 * (v1[1] + v2[1]);
   xg[2] = .5 * (v1[2] + v2[2]);
 
-  ana(tcur, 1, NULL, xg, false, input, &evaluation);
+  /* Evaluate the function at the Gauss points */
+  ana(tcur, 1, NULL, xg, false, input, &feval);
 
-  *results += len * evaluation;
+  /* Update the result with the quadrature rule */
+  *results += len * feval;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -555,15 +558,16 @@ cs_quadrature_edge_2pts_scal(double                tcur,
                              double                results[])
 {
   cs_real_3_t  gauss_pts[2];
-  double  evaluation[2], weights[2];
+  double  feval[2], weights[2];
 
   /* Compute Gauss points and its unique weight */
   cs_quadrature_edge_2pts(v1, v2, len, gauss_pts, weights);
 
-  ana(tcur, 2, NULL, (const cs_real_t *)gauss_pts, false, input, evaluation);
+  /* Evaluate the function at the Gauss points */
+  ana(tcur, 2, NULL, (const cs_real_t *)gauss_pts, false, input, feval);
 
-  /* Return results */
-  *results += weights[0] * evaluation[0] + weights[1] * evaluation[1];
+  /* Update the result with the quadrature rule */
+  *results += weights[0] * feval[0] + weights[1] * feval[1];
 }
 
 /*----------------------------------------------------------------------------*/
@@ -592,17 +596,141 @@ cs_quadrature_edge_3pts_scal(double                tcur,
                              double                results[])
 {
   cs_real_3_t  gauss_pts[3];
-  double  evaluation[3], weights[3];
+  double  feval[3], weights[3];
 
   /* Compute Gauss points and its weights */
   cs_quadrature_edge_3pts(v1, v2, len, gauss_pts, weights);
 
-  ana(tcur, 3, NULL, (const cs_real_t *)gauss_pts, false, input, evaluation);
+  /* Evaluate the function at the Gauss points */
+  ana(tcur, 3, NULL, (const cs_real_t *)gauss_pts, false, input, feval);
 
-  *results += weights[0] * evaluation[0] + weights[1] * evaluation[1] +
-              weights[2] * evaluation[2];
+  /* Update the result with the quadrature rule */
+  *results += weights[0]*feval[0] + weights[1]*feval[1] + weights[2]*feval[2];
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the integral over an edge using a barycentric quadrature
+ *         rule and add it to \p results
+ *         Case of a vector-valued function.
+ *
+ * \param[in]      tcur        current physical time of the simulation
+ * \param[in]      v1          1st point of the triangle
+ * \param[in]      v2          2nd point of the triangle
+ * \param[in]      len         length of the edge
+ * \param[in]      ana         pointer to the analytic function
+ * \param[in]      input       NULL or pointer to a structure cast on-the-fly
+ * \param[in, out] results     array of double
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_quadrature_edge_1pt_vect(double                 tcur,
+                            const cs_real_3_t      v1,
+                            const cs_real_3_t      v2,
+                            double                 len,
+                            cs_analytic_func_t    *ana,
+                            void                  *input,
+                            double                 results[])
+{
+  cs_real_3_t  xg;
+  double  feval[3];
+
+  /* Copied from cs_quadrature_1pt */
+  xg[0] = .5 * (v1[0] + v2[0]);
+  xg[1] = .5 * (v1[1] + v2[1]);
+  xg[2] = .5 * (v1[2] + v2[2]);
+
+  /* Evaluate the function at the Gauss points */
+  ana(tcur, 1, NULL, xg, false, input, feval);
+
+  /* Update the result with the quadrature rule */
+  results[0] += len * feval[0];
+  results[1] += len * feval[1];
+  results[2] += len * feval[2];
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the integral over an edge with a quadrature rule using 2
+ *         Gauss points and a unique weight and add it to \p results
+ *         Case of a vector-valued function.
+ *
+ * \param[in]      tcur        current physical time of the simulation
+ * \param[in]      v1          1st point of the triangle
+ * \param[in]      v2          2nd point of the triangle
+ * \param[in]      len         length of the edge
+ * \param[in]      ana         pointer to the analytic function
+ * \param[in]      input       NULL or pointer to a structure cast on-the-fly
+ * \param[in, out] results     array of double
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_quadrature_edge_2pts_vect(double                 tcur,
+                             const cs_real_3_t      v1,
+                             const cs_real_3_t      v2,
+                             double                 len,
+                             cs_analytic_func_t    *ana,
+                             void                  *input,
+                             double                 results[])
+{
+  cs_real_3_t  gauss_pts[2];
+  double  feval[6], weights[2];
+
+  /* Compute Gauss points and its unique weight */
+  cs_quadrature_edge_2pts(v1, v2, len, gauss_pts, weights);
+
+  /* Evaluate the function at the Gauss points */
+  ana(tcur, 2, NULL, (const cs_real_t *)gauss_pts, false, input, feval);
+
+  /* Update the result with the quadrature rule */
+  results[0] += weights[0] * feval[0] + weights[1] * feval[3];
+  results[1] += weights[0] * feval[1] + weights[1] * feval[4];
+  results[2] += weights[0] * feval[2] + weights[1] * feval[5];
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the integral over an edge with a quadrature rule using 3
+ *         Gauss points and weights and add it to \p results
+ *         Case of a vector-valued function.
+ *
+ * \param[in]      tcur         current physical time of the simulation
+ * \param[in]      v1           first point of the edge
+ * \param[in]      v2           second point of the edge
+ * \param[in]      len          length of the edge
+ * \param[in]      ana          pointer to the analytic function
+ * \param[in]      input        NULL or pointer to a structure cast on-the-fly
+ * \param[in, out] results      array of double
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_quadrature_edge_3pts_vect(double                tcur,
+                             const cs_real_3_t     v1,
+                             const cs_real_3_t     v2,
+                             double                len,
+                             cs_analytic_func_t   *ana,
+                             void                 *input,
+                             double                results[])
+{
+  cs_real_3_t  gauss_pts[3];
+  double  feval[9], weights[3];
+
+  /* Compute Gauss points and its weights */
+  cs_quadrature_edge_3pts(v1, v2, len, gauss_pts, weights);
+
+  /* Evaluate the function at the Gauss points */
+  ana(tcur, 3, NULL, (const cs_real_t *)gauss_pts, false, input, feval);
+
+  /* Update the result with the quadrature rule */
+  for (int p = 0; p < 3; p++) {
+    results[0] += weights[p] * feval[3*p  ];
+    results[1] += weights[p] * feval[3*p+1];
+    results[2] += weights[p] * feval[3*p+2];
+  }
+}
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -635,9 +763,9 @@ cs_quadrature_tria_1pt_scal(double                 tcur,
   double  evaluation;
 
   /* Copied from cs_quadrature_1pt */
-  xg[0] = cs_math_onethird * (v1[0] + v2[0] + v3[0]);
-  xg[1] = cs_math_onethird * (v1[1] + v2[1] + v3[1]);
-  xg[2] = cs_math_onethird * (v1[2] + v2[2] + v3[2]);
+  xg[0] = cs_math_1ov3 * (v1[0] + v2[0] + v3[0]);
+  xg[1] = cs_math_1ov3 * (v1[1] + v2[1] + v3[1]);
+  xg[2] = cs_math_1ov3 * (v1[2] + v2[2] + v3[2]);
 
   ana(tcur, 1, NULL, xg, false, input, &evaluation);
 
@@ -723,6 +851,47 @@ cs_quadrature_tria_4pts_scal(double                tcur,
               weights[2] * evaluation[2] + weights[3] * evaluation[3];
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the integral over a triangle with a quadrature rule using
+ *         7 Gauss points and 7 weights and add it to \p results
+ *         Case of a scalar-valued function.
+ *
+ * \param[in]      tcur         current physical time of the simulation
+ * \param[in]      v1           first point of the triangle
+ * \param[in]      v2           second point of the triangle
+ * \param[in]      v3           third point of the triangle
+ * \param[in]      area         area of the triangle
+ * \param[in]      ana          pointer to the analytic function
+ * \param[in]      input        NULL or pointer to a structure cast on-the-fly
+ * \param[in, out] results      array of double
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_quadrature_tria_7pts_scal(double                tcur,
+                             const cs_real_3_t     v1,
+                             const cs_real_3_t     v2,
+                             const cs_real_3_t     v3,
+                             double                area,
+                             cs_analytic_func_t   *ana,
+                             void                 *input,
+                             double                results[])
+{
+  cs_real_3_t  gauss_pts[7];
+  double  evaluation[7], weights[7];
+
+  /* Compute Gauss points and its weights */
+  cs_quadrature_tria_7pts(v1, v2, v3, area, gauss_pts, weights);
+
+  ana(tcur, 7, NULL, (const cs_real_t *)gauss_pts, false, input, evaluation);
+
+  *results += weights[0] * evaluation[0] + weights[1] * evaluation[1] +
+              weights[2] * evaluation[2] + weights[3] * evaluation[3] +
+              weights[4] * evaluation[4] + weights[5] * evaluation[5] +
+              weights[6] * evaluation[6] ;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the integral over a triangle using a barycentric quadrature
@@ -754,9 +923,9 @@ cs_quadrature_tria_1pt_vect(double                 tcur,
   double evaluation[3];
 
   /* Copied from cs_quadrature_1pt */
-  xg[0] = cs_math_onethird * (v1[0] + v2[0] + v3[0]);
-  xg[1] = cs_math_onethird * (v1[1] + v2[1] + v3[1]);
-  xg[2] = cs_math_onethird * (v1[2] + v2[2] + v3[2]);
+  xg[0] = cs_math_1ov3 * (v1[0] + v2[0] + v3[0]);
+  xg[1] = cs_math_1ov3 * (v1[1] + v2[1] + v3[1]);
+  xg[2] = cs_math_1ov3 * (v1[2] + v2[2] + v3[2]);
 
   ana(tcur, 1, NULL, xg, false, input, evaluation);
 
@@ -849,6 +1018,48 @@ cs_quadrature_tria_4pts_vect(double                tcur,
   }
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the integral over a triangle with a quadrature rule using
+ *         7 Gauss points and 7 weights and add it to \p results.
+ *         Case of a vector-valued function.
+ *
+ * \param[in]      tcur         current physical time of the simulation
+ * \param[in]      v1           first point of the triangle
+ * \param[in]      v2           second point of the triangle
+ * \param[in]      v3           third point of the triangle
+ * \param[in]      area         area of the triangle
+ * \param[in]      ana          pointer to the analytic function
+ * \param[in]      input        NULL or pointer to a structure cast on-the-fly
+ * \param[in, out] results      array of double
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_quadrature_tria_7pts_vect(double                tcur,
+                             const cs_real_3_t     v1,
+                             const cs_real_3_t     v2,
+                             const cs_real_3_t     v3,
+                             double                area,
+                             cs_analytic_func_t   *ana,
+                             void                 *input,
+                             double                results[])
+{
+  cs_real_3_t  gauss_pts[7];
+  double  evaluation[3*7], weights[7];
+
+  /* Compute Gauss points and its weights */
+  cs_quadrature_tria_7pts(v1, v2, v3, area, gauss_pts, weights);
+
+  ana(tcur, 7, NULL, (const cs_real_t *)gauss_pts, false, input, evaluation);
+
+  for (int p = 0; p < 7; p++) {
+    results[0] += weights[p] * evaluation[3*p  ];
+    results[1] += weights[p] * evaluation[3*p+1];
+    results[2] += weights[p] * evaluation[3*p+2];
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the integral over a triangle using a barycentric quadrature
@@ -880,9 +1091,9 @@ cs_quadrature_tria_1pt_tens(double                 tcur,
   double evaluation[9];
 
   /* Copied from cs_quadrature_1pt */
-  xg[0] = cs_math_onethird * (v1[0] + v2[0] + v3[0]);
-  xg[1] = cs_math_onethird * (v1[1] + v2[1] + v3[1]);
-  xg[2] = cs_math_onethird * (v1[2] + v2[2] + v3[2]);
+  xg[0] = cs_math_1ov3 * (v1[0] + v2[0] + v3[0]);
+  xg[1] = cs_math_1ov3 * (v1[1] + v2[1] + v3[1]);
+  xg[2] = cs_math_1ov3 * (v1[2] + v2[2] + v3[2]);
 
   ana(tcur, 1, NULL, xg, false, input, evaluation);
 
@@ -976,6 +1187,49 @@ cs_quadrature_tria_4pts_tens(double                tcur,
   }
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the integral over a triangle with a quadrature rule using
+ *         7 Gauss points and 7 weights and add it to \p results.
+ *         Case of a tensor-valued function.
+ *
+ * \param[in]      tcur         current physical time of the simulation
+ * \param[in]      v1           first point of the triangle
+ * \param[in]      v2           second point of the triangle
+ * \param[in]      v3           third point of the triangle
+ * \param[in]      area         area of the triangle
+ * \param[in]      ana          pointer to the analytic function
+ * \param[in]      input        NULL or pointer to a structure cast on-the-fly
+ * \param[in, out] results      array of double
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_quadrature_tria_7pts_tens(double                tcur,
+                             const cs_real_3_t     v1,
+                             const cs_real_3_t     v2,
+                             const cs_real_3_t     v3,
+                             double                area,
+                             cs_analytic_func_t   *ana,
+                             void                 *input,
+                             double                results[])
+{
+  cs_real_3_t  gauss_pts[7];
+  double  evaluation[9*7], weights[7];
+
+  /* Compute Gauss points and its weights */
+  cs_quadrature_tria_7pts(v1, v2, v3, area, gauss_pts, weights);
+
+  ana(tcur, 7, NULL, (const cs_real_t *)gauss_pts, false, input, evaluation);
+
+  for (int p = 0; p < 7; p++) {
+    const  double wp = weights[p];
+    double  *eval_p = evaluation + 9*p;
+    for (short int ij = 0; ij < 9; ij++)
+      results[ij] += wp * eval_p[ij];
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the integral over a tetrahedron using a barycentric
@@ -1132,7 +1386,7 @@ cs_quadrature_tet_1pt_vect(double                 tcur,
                            double                 results[])
 {
   cs_real_3_t  xg;
-  double evaluation[3];
+  double  evaluation[3];
 
   /* Copied from cs_quadrature_tet_1pt */
   xg[0] = 0.25 * (v1[0] + v2[0] + v3[0] + v4[0]);
@@ -1367,6 +1621,71 @@ cs_quadrature_tet_5pts_tens(double                tcur,
   }
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve the integral function according to the quadrature type
+ *         and the stride provided
+ *         Case of integral along edges.
+ *
+ * \param[in]     dim          dimension of the function to integrate
+ * \param[in]     qtype        quadrature type
+ *
+ * \return a pointer to the integral function
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline cs_quadrature_edge_integral_t *
+cs_quadrature_get_edge_integral(int                   dim,
+                                cs_quadrature_type_t  qtype)
+{
+  switch (dim) {
+
+  case 1: /* Scalar-valued integral */
+
+    switch (qtype) {
+
+    case CS_QUADRATURE_BARY:
+    case CS_QUADRATURE_BARY_SUBDIV:
+      return cs_quadrature_edge_1pt_scal;
+    case CS_QUADRATURE_HIGHER:
+      return cs_quadrature_edge_2pts_scal;
+    case CS_QUADRATURE_HIGHEST:
+      return cs_quadrature_edge_3pts_scal;
+
+    default:
+      bft_error(__FILE__, __LINE__, 0,
+                " %s: Invalid quadrature type\n", __func__);
+    }
+    break;
+
+  case 3: /* Vector-valued case */
+
+    switch (qtype) {
+
+    case CS_QUADRATURE_BARY:
+    case CS_QUADRATURE_BARY_SUBDIV:
+      return cs_quadrature_edge_1pt_vect;
+    case CS_QUADRATURE_HIGHER:
+      return cs_quadrature_edge_2pts_vect;
+    case CS_QUADRATURE_HIGHEST:
+      return cs_quadrature_edge_3pts_vect;
+
+    default:
+      bft_error(__FILE__, __LINE__, 0,
+                " %s: Invalid quadrature type\n", __func__);
+    }
+    break;
+
+  default:
+    bft_error(__FILE__, __LINE__, 0,
+              " %s: Invalid dimension value %d. Only 1 and 3 are valid.\n",
+              __func__, dim);
+
+  } /* switch on dim */
+
+  return NULL; /* Should not go to this stage */
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Retrieve the integral function according to the quadrature type
@@ -1393,9 +1712,9 @@ cs_quadrature_get_tria_integral(int                   dim,
     case CS_QUADRATURE_BARY_SUBDIV:
       return cs_quadrature_tria_1pt_scal;
     case CS_QUADRATURE_HIGHER:
-      return cs_quadrature_tria_3pts_scal;
-    case CS_QUADRATURE_HIGHEST:
       return cs_quadrature_tria_4pts_scal;
+    case CS_QUADRATURE_HIGHEST:
+      return cs_quadrature_tria_7pts_scal;
 
     default:
       bft_error(__FILE__, __LINE__, 0,
@@ -1411,9 +1730,9 @@ cs_quadrature_get_tria_integral(int                   dim,
     case CS_QUADRATURE_BARY_SUBDIV:
       return cs_quadrature_tria_1pt_vect;
     case CS_QUADRATURE_HIGHER:
-      return cs_quadrature_tria_3pts_vect;
-    case CS_QUADRATURE_HIGHEST:
       return cs_quadrature_tria_4pts_vect;
+    case CS_QUADRATURE_HIGHEST:
+      return cs_quadrature_tria_7pts_vect;
 
     default:
       bft_error(__FILE__, __LINE__, 0,
@@ -1429,9 +1748,9 @@ cs_quadrature_get_tria_integral(int                   dim,
     case CS_QUADRATURE_BARY_SUBDIV:
       return cs_quadrature_tria_1pt_tens;
     case CS_QUADRATURE_HIGHER:
-      return cs_quadrature_tria_3pts_tens;
-    case CS_QUADRATURE_HIGHEST:
       return cs_quadrature_tria_4pts_tens;
+    case CS_QUADRATURE_HIGHEST:
+      return cs_quadrature_tria_7pts_tens;
 
     default:
       bft_error(__FILE__, __LINE__, 0,
@@ -1532,6 +1851,24 @@ cs_quadrature_get_tetra_integral(int                   dim,
   return NULL;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Get the flags adapted to the given quadrature type \p qtype and the
+ *         location on which the quadrature should be performed
+ *
+ * \param[in] qtype    \ref cs_quadrature_type_t
+ * \param[in] loc      It could be \ref CS_FLAG_CELL, \ref CS_FLAG_FACE or
+ *                     \ref CS_FLAG_EDGE plus \ref CS_FLAG_PRIMAL or
+ *                     \ref CS_FLAG_DUAL
+ *
+ * \return  metadata stored in a \ref cs_eflag_t to build a \ref cs_cell_mesh_t
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_eflag_t
+cs_quadrature_get_flag(const cs_quadrature_type_t  qtype,
+                       const cs_flag_t             loc);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
