@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -184,8 +184,7 @@ cs_sdm_create_copy(const cs_sdm_t   *m)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief   Define a new matrix by adding the given matrix with its transpose.
- *          Keep the transposed matrix for a future use.
+ * \brief   Define a new matrix which is its transpose.
  *
  * \param[in] mat   local matrix to transpose
  *
@@ -229,10 +228,10 @@ cs_sdm_create_transpose(cs_sdm_t  *mat)
 /*----------------------------------------------------------------------------*/
 
 cs_sdm_t *
-cs_sdm_block_create(int                n_max_blocks_by_row,
-                    int                n_max_blocks_by_col,
-                    const short int    max_row_block_sizes[],
-                    const short int    max_col_block_sizes[])
+cs_sdm_block_create(int          n_max_blocks_by_row,
+                    int          n_max_blocks_by_col,
+                    const int    max_row_block_sizes[],
+                    const int    max_col_block_sizes[])
 {
   cs_sdm_t  *m = NULL;
 
@@ -276,6 +275,48 @@ cs_sdm_block_create(int                n_max_blocks_by_row,
   return m;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Allocate and initialize a cs_sdm_t structure by block when the
+ *          block size is constant and equal to 3
+ *
+ * \param[in]  n_max_blocks_by_row    max number of blocks in a row
+ * \param[in]  n_max_blocks_by_col    max number of blocks in a column
+ *
+ * \return  a new allocated cs_sdm_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_sdm_t *
+cs_sdm_block33_create(int      n_max_blocks_by_row,
+                      int      n_max_blocks_by_col)
+{
+  cs_sdm_t  *m = NULL;
+
+  if (n_max_blocks_by_row < 1 || n_max_blocks_by_col < 1)
+    return m;
+
+  m = _create_sdm(CS_SDM_BY_BLOCK,
+                  3*n_max_blocks_by_row,
+                  3*n_max_blocks_by_col);
+
+  /* Define the block description */
+  m->block_desc->n_max_blocks_by_row = n_max_blocks_by_row;
+  m->block_desc->n_max_blocks_by_col = n_max_blocks_by_col;
+  m->block_desc->n_row_blocks = n_max_blocks_by_row;
+  m->block_desc->n_col_blocks = n_max_blocks_by_col;
+  BFT_MALLOC(m->block_desc->blocks,
+             n_max_blocks_by_row*n_max_blocks_by_col, cs_sdm_t);
+
+  cs_real_t  *p_val = m->val;
+  for (int i = 0; i < n_max_blocks_by_row*n_max_blocks_by_col; i++) {
+    cs_sdm_map_array(3, 3, m->block_desc->blocks + i, p_val);
+      p_val += 9;
+  }
+
+  return m;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief   Free a cs_sdm_t structure
@@ -319,11 +360,11 @@ cs_sdm_free(cs_sdm_t  *mat)
 /*----------------------------------------------------------------------------*/
 
 void
-cs_sdm_block_init(cs_sdm_t          *m,
-                  int                n_row_blocks,
-                  int                n_col_blocks,
-                  const short int    row_block_sizes[],
-                  const short int    col_block_sizes[])
+cs_sdm_block_init(cs_sdm_t      *m,
+                  int            n_row_blocks,
+                  int            n_col_blocks,
+                  const int      row_block_sizes[],
+                  const int      col_block_sizes[])
 {
   /* Sanity checks */
   assert(m != NULL && row_block_sizes != NULL && col_block_sizes != NULL);
@@ -363,6 +404,93 @@ cs_sdm_block_init(cs_sdm_t          *m,
   }
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Initialize the pattern of cs_sdm_t structure defined by 3x3 block
+ *          The matrix should have been allocated before calling this function
+ *
+ * \param[in, out] m
+ * \param[in]      n_row_blocks      number of blocks in a row
+ * \param[in]      n_col_blocks      number of blocks in a column
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_block33_init(cs_sdm_t     *m,
+                    int           n_row_blocks,
+                    int           n_col_blocks)
+{
+  /* Sanity checks */
+  assert(m != NULL);
+  assert(m->flag & CS_SDM_BY_BLOCK);
+  assert(m->block_desc != NULL);
+  assert(n_row_blocks <= m->block_desc->n_max_blocks_by_row);
+  assert(n_col_blocks <= m->block_desc->n_max_blocks_by_col);
+
+  cs_sdm_block_t  *bd = m->block_desc;
+
+  bd->n_row_blocks = n_row_blocks;
+  bd->n_col_blocks = n_col_blocks;
+  m->n_rows = 3*n_row_blocks;
+  m->n_cols = 3*n_col_blocks;
+
+  memset(m->val, 0, m->n_rows*m->n_cols*sizeof(cs_real_t));
+
+  cs_real_t  *p_val = m->val;
+  for (int i = 0; i < bd->n_row_blocks*bd->n_col_blocks; i++) {
+    cs_sdm_map_array(3, 3, bd->blocks + i, p_val);
+    p_val += 9; /* Each 3x3 block has 9 entries */
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Convert a matrix with each entry is a 3x3 block into a matrix
+ *          with a block for each component x,y,z.
+ *
+ * \param[in]      mb33        pointer to a matrix
+ * \param[in, out] mbxyz       pointer to a matrix to build (but allocated)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_block_33_to_xyz(const cs_sdm_t   *mb33,
+                       cs_sdm_t         *mbxyz)
+{
+  if (mb33 == NULL)
+    return;
+
+  assert(mb33->flag & CS_SDM_BY_BLOCK);
+  assert(mb33->block_desc != NULL);
+
+  const cs_sdm_block_t  *mb33_desc = mb33->block_desc;
+  const int  n_cols = mb33_desc->n_col_blocks;
+  assert(n_cols == mb33_desc->n_row_blocks);
+
+  /* Reshape the block matrix to fit the shape requested by mb33 */
+  int block_sizes[3];
+  for (int i = 0; i < 3; i++) block_sizes[i] = n_cols;
+  cs_sdm_block_init(mbxyz, 3, 3, block_sizes, block_sizes);
+
+  cs_sdm_t  *mxyz[3][3];
+  for (int i = 0; i < 3; i++)
+    for (int j = 0; j < 3; j++)
+      mxyz[i][j] = cs_sdm_get_block(mbxyz, i, j);
+
+  for (int bi = 0; bi < n_cols; bi++) {
+    for (int bj = 0; bj < n_cols; bj++) {
+
+      cs_sdm_t  *m33_ij = cs_sdm_get_block(mb33, bi, bj);
+
+      for (int _i = 0; _i < 3; _i++)
+        for (int _j = 0; _j < 3; _j++)
+          mxyz[_i][_j]->val[bi*n_cols+bj] = m33_ij->val[3*_i+_j];
+
+    } /* Loop on column blocks */
+  } /* Loop on row blocks */
+
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief   Allocate and initialize a cs_sdm_t structure w.r.t. to a given
@@ -600,8 +728,9 @@ cs_sdm_block_multiply_rowrow(const cs_sdm_t   *a,
 
   const cs_sdm_block_t  *a_desc = a->block_desc;
   const cs_sdm_block_t  *b_desc = b->block_desc;
-  const cs_sdm_block_t  *c_desc = c->block_desc; /* used in assert */
+  const cs_sdm_block_t  *c_desc = c->block_desc;
 
+  CS_UNUSED(c_desc);            /* Only in debug mode */
   assert(a_desc->n_col_blocks == b_desc->n_col_blocks &&
          a_desc->n_row_blocks == c_desc->n_row_blocks &&
          c_desc->n_col_blocks == b_desc->n_row_blocks);
@@ -714,9 +843,9 @@ cs_sdm_square_matvec(const cs_sdm_t    *mat,
 
   /* Increment mv */
   for (short int i = 0; i < n; i++) {
-    cs_real_t *m_i = mat->val + i*n;
+    const cs_real_t *m_i = mat->val + i*n;
     for (short int j = 1; j < n; j++)
-      mv[i] +=  m_i[j] * vec[j];
+      mv[i] += m_i[j] * vec[j];
   }
 
 }
@@ -881,9 +1010,10 @@ cs_sdm_block_add_mult(cs_sdm_t        *mat,
   if (mat == NULL || add == NULL)
     return;
 
-  const cs_sdm_block_t  *add_desc = add->block_desc; /* used in assert */
+  const cs_sdm_block_t  *add_desc = add->block_desc;
   const cs_sdm_block_t  *mat_desc = mat->block_desc;
 
+  CS_UNUSED(add_desc);          /* Only in debug mode */
   assert(add_desc != NULL && mat_desc != NULL);
   assert(add_desc->n_row_blocks == mat_desc->n_row_blocks);
   assert(add_desc->n_col_blocks == mat_desc->n_col_blocks);
@@ -1020,7 +1150,8 @@ cs_sdm_square_add_transpose(cs_sdm_t  *mat,
                             cs_sdm_t  *tr)
 {
   /* Sanity check */
-  assert(mat != NULL && tr != NULL && tr->n_max_rows == mat->n_max_rows);
+  assert(mat != NULL && tr != NULL);
+  assert(mat->n_rows <= tr->n_max_cols && mat->n_cols <= tr->n_max_rows);
   assert(mat->n_rows == mat->n_cols);
 
   if (mat->n_rows < 1 || mat->n_cols < 1)
@@ -1050,6 +1181,38 @@ cs_sdm_square_add_transpose(cs_sdm_t  *mat,
 
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Set the given matrix to two times its symmetric part
+ *          mat --> mat + mat_tr = 2*symm(mat)
+ *
+ * \param[in, out] mat   small dense matrix to transform
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_square_2symm(cs_sdm_t   *mat)
+{
+  /* Sanity check */
+  assert(mat != NULL);
+  assert(mat->n_rows == mat->n_cols);
+
+  if (mat->n_rows < 1)
+    return;
+
+  for (short int i = 0; i < mat->n_rows; i++) {
+
+    cs_real_t  *mi = mat->val + i*mat->n_cols;
+    for (short int j = i; j < mat->n_cols; j++) {
+      int  ji = j*mat->n_rows + i;
+      mi[j] += mat->val[ji];
+      mat->val[ji] = mi[j];
+    } /* col j */
+
+  } /* row i */
+
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief   Set the given matrix into its anti-symmetric part
@@ -1210,6 +1373,184 @@ cs_sdm_33_sym_qr_compute(const cs_real_t   m[9],
     q2[k] = tmp.unitv[k];
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  LU factorization of a small dense 3x3 matrix.
+ *
+ * \param[in]      m        pointer to a cs_sdm_t structure
+ * \param[in, out] facto    compact storage of coefficients for the LU
+ *                          factorization
+ *
+ * \note: facto stores L the lower triangular matrix (without its diagonal
+ *        entries assumed to be equal to 1) and U the upper triangular matrix.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_33_lu_compute(const cs_sdm_t   *m,
+                     cs_real_t         facto[9])
+{
+  /* Sanity check */
+  assert(m != NULL && facto != NULL);
+  assert(m->n_cols == m->n_rows);
+
+  /* j=0: first row */
+  const cs_real_t  d00 = m->val[0];
+  if (fabs(d00) < cs_math_zero_threshold)
+    bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
+  const cs_real_t  invd00 = 1./d00;
+
+  facto[0] = d00, facto[1] = m->val[1], facto[2] = m->val[2];     /* 1st row */
+  facto[3] =  m->val[3]*invd00;                                   /* L10 */
+  facto[4] =  m->val[4] - facto[3]*facto[1];                      /* U11 */
+  facto[5] =  m->val[5] - facto[3]*facto[2];                      /* U12 */
+  facto[6] =  m->val[6]*invd00;                                   /* L20 */
+  facto[7] = (m->val[7] - facto[6]*m->val[1])/facto[4];           /* L21 */
+  facto[8] =  m->val[8] - facto[6]*m->val[2] - facto[7]*facto[5]; /* U22 */
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  LU factorization of a small dense matrix. Small means that the
+ *         number m->n_rows is less than 100 for instance.
+ *
+ * \param[in]      m        pointer to a cs_sdm_t structure
+ * \param[in, out] facto    compact storage of coefficients for the LU
+ *                          factorization (should be allocated to the right
+ *                          size, i.e. m->n_rows*m->n_rows)
+ *
+ * \note: facto stores L the lower triangular matrix (without its diagonal
+ *        entries assumed to be equal to 1) and U the upper triangular matrix.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_lu_compute(const cs_sdm_t   *m,
+                  cs_real_t         facto[])
+{
+  /* Sanity check */
+  assert(m != NULL && facto != NULL);
+  assert(m->n_cols == m->n_rows);
+
+  const cs_lnum_t  n = m->n_rows;
+
+  /* Initialization */
+  memcpy(facto, m->val, n*n*sizeof(cs_real_t));
+
+  /* Each step work on a smaller block */
+  for (cs_lnum_t k = 0; k < n-1; k++) {
+
+    /* Pivot */
+    cs_real_t  pivot = facto[k*(n+1)];
+    if (fabs(pivot) < cs_math_zero_threshold)
+      bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
+    const cs_real_t  invp = 1./pivot;
+
+    for (cs_lnum_t i = k+1; i < m->n_rows; i++) { /* Loop on rows */
+
+      cs_real_t  *pr_fact = facto + (i-1)*n;
+      cs_real_t  *cr_fact = pr_fact + n;
+
+      /* L-part: lower part of the (i,i-1) entry */
+      cr_fact[k] *= invp;
+      const double  lval = cr_fact[k];
+
+      /* U-part */
+      for (cs_lnum_t j = k+1; j < n; j++) {
+        cr_fact[j] -= lval*pr_fact[j];
+
+      } /* Loop on j (columns) */
+
+    } /* Loop on i (rows) */
+
+  } /* Loop on k (block size) */
+
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Solve a system A.sol = rhs using a LU factorization of A (a small
+ *         3x3 dense matrix).
+ *
+ * \param[in]       facto    compact storage of coefficients for the LU
+ *                           factorization (should be allocated to the right
+ *                           size, i.e. n_rows*n_rows)
+ * \param[in]       rhs      right-hand side
+ * \param[in, out]  sol      solution
+ *
+ * \note: facto stores L the lower triangular matrix (without its diagonal
+ *        entries assumed to be equal to 1) and U the upper triangular matrix.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_33_lu_solve(const cs_real_t    facto[9],
+                   const cs_real_t    rhs[3],
+                   cs_real_t          sol[3])
+{
+  /* Sanity check */
+  assert(facto != NULL && rhs != NULL && sol != NULL);
+
+  /* Forward pass */
+  sol[0] = rhs[0];
+  sol[1] = rhs[1] - facto[3]*sol[0];
+  sol[2] = rhs[2] - facto[6]*sol[0] - facto[7]*sol[1];
+
+  /* Backward pass */
+  sol[2] = sol[2]/facto[8];
+  sol[1] = (sol[1] - facto[5]*sol[2])/facto[4];
+  sol[0] = (sol[0] - facto[2]*sol[2] - facto[1]*sol[1])/facto[0];
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Solve a system A.sol = rhs using a LU factorization of A (a small
+ *         dense matrix).
+ *
+ * \param[in]       n_rows   dimension of the system to solve
+ * \param[in]       facto    compact storage of coefficients for the LU
+ *                           factorization (should be allocated to the right
+ *                           size, i.e. n_rows*n_rows)
+ * \param[in]       rhs      right-hand side
+ * \param[in, out]  sol      solution
+ *
+ * \note: facto stores L the lower triangular matrix (without its diagonal
+ *        entries assumed to be equal to 1) and U the upper triangular matrix.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_lu_solve(cs_lnum_t          n_rows,
+                const cs_real_t    facto[],
+                const cs_real_t   *rhs,
+                cs_real_t         *sol)
+{
+  /* Sanity check */
+  assert(facto != NULL && rhs != NULL && sol != NULL);
+
+  /* Forward pass: L.y = rhs (sol stores the values for y) */
+  for (cs_lnum_t  i = 0; i < n_rows; i++) {
+
+    sol[i] = rhs[i];
+    const cs_real_t  *_fact = facto + i*n_rows;
+    for (cs_lnum_t j = 0; j < i; j++) {
+      sol[i] -= sol[j]*_fact[j];
+    }
+
+  }
+
+  /* Backward pass: U.sol = y */
+  for (cs_lnum_t i = n_rows-1; i >= 0; i--) { /* Loop on rows */
+    for (cs_lnum_t j = n_rows-1; j > i; j--) { /* Loop on columns */
+
+      sol[i] -= sol[j]*facto[i*n_rows + j];
+
+    }
+    sol[i] /= facto[i*(n_rows + 1)];
+  }
+
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  LDL^T: Modified Cholesky decomposition of a 3x3 SPD matrix.
@@ -1234,7 +1575,7 @@ cs_sdm_33_ldlt_compute(const cs_sdm_t   *m,
   assert(m != NULL && facto != NULL);
   assert(m->n_cols == m->n_rows && m->n_cols == 3);
 
-  // j=0: first row
+  /* j=0: first row */
   const cs_real_t  d00 = m->val[0];
   if (fabs(d00) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
@@ -1243,14 +1584,14 @@ cs_sdm_33_ldlt_compute(const cs_sdm_t   *m,
   const cs_real_t  l10 = facto[1] = m->val[1] * facto[0];
   const cs_real_t  l20 = facto[3] = m->val[2] * facto[0];
 
-  // j=1: second row
+  /* j=1: second row */
   const cs_real_t  d11 = m->val[4] - l10*l10 * d00;
   if (fabs(d11) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
   facto[2] = 1. / d11;
   const cs_real_t l21 = facto[4] = (m->val[5] - l20*d00*l10) * facto[2];
 
-  // j=2: third row
+  /* j=2: third row */
   const cs_real_t  d22 = m->val[8] - l20*l20*d00 - l21*l21*d11;
   if (fabs(d22) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
@@ -1281,7 +1622,7 @@ cs_sdm_44_ldlt_compute(const cs_sdm_t   *m,
   assert(m != NULL && facto != NULL);
   assert(m->n_cols == m->n_rows && m->n_cols == 4);
 
-  // j=0: first row
+  /* j=0: first row */
   const cs_real_t  d00 = m->val[0];
   if (fabs(d00) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
@@ -1291,7 +1632,7 @@ cs_sdm_44_ldlt_compute(const cs_sdm_t   *m,
   const cs_real_t  l20 = facto[3] = m->val[2] * facto[0];
   const cs_real_t  l30 = facto[6] = m->val[3] * facto[0];
 
-  // j=1: second row
+  /* j=1: second row */
   const cs_real_t  d11 = m->val[5] - l10*l10 * d00;
   if (fabs(d11) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
@@ -1299,7 +1640,7 @@ cs_sdm_44_ldlt_compute(const cs_sdm_t   *m,
   const cs_real_t  l21 = facto[4] = (m->val[6] - l20*d00*l10) * facto[2];
   const cs_real_t  l31 = facto[7] = (m->val[7] - l30*d00*l10) * facto[2];
 
-  // j=2: third row
+  /* j=2: third row */
   const cs_real_t  d22 = m->val[10] - l20*l20*d00 - l21*l21*d11;
   if (fabs(d22) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
@@ -1307,7 +1648,7 @@ cs_sdm_44_ldlt_compute(const cs_sdm_t   *m,
   const cs_real_t  l32 = facto[8] =
     (m->val[11] - l30*d00*l20 - l31*d11*l21) * facto[5];
 
-  // j=3: row 4
+  /* j=3: row 4 */
   const cs_real_t  d33 = m->val[15] - l30*l30*d00 - l31*l31*d11 - l32*l32*d22;
   if (fabs(d33) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
@@ -1338,7 +1679,7 @@ cs_sdm_66_ldlt_compute(const cs_sdm_t   *m,
   assert(m != NULL && facto != NULL);
   assert(m->n_cols == m->n_rows && m->n_cols == 6);
 
-  // j=0: first row
+  /* j=0: first row */
   const cs_real_t  d00 = m->val[0];
   if (fabs(d00) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
@@ -1350,7 +1691,7 @@ cs_sdm_66_ldlt_compute(const cs_sdm_t   *m,
   const cs_real_t  l40 = facto[10] = m->val[4] * facto[0];
   const cs_real_t  l50 = facto[15] = m->val[5] * facto[0];
 
-  // j=1: second row
+  /* j=1: second row */
   const cs_real_t  d11 = m->val[7] - l10*l10 * d00;
   if (fabs(d11) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
@@ -1361,7 +1702,7 @@ cs_sdm_66_ldlt_compute(const cs_sdm_t   *m,
   const cs_real_t  l41 = facto[11] = (m->val[10] - l40*d0l10) * facto[2];
   const cs_real_t  l51 = facto[16] = (m->val[11] - l50*d0l10) * facto[2];
 
-  // j=2: third row
+  /* j=2: third row */
   const cs_real_t  d22 = m->val[14] - l20*l20*d00 - l21*l21*d11;
   if (fabs(d22) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
@@ -1374,7 +1715,7 @@ cs_sdm_66_ldlt_compute(const cs_sdm_t   *m,
   const cs_real_t  l52 = facto[17] =
     (m->val[17] - l30*d0l20 - l31*d1l21) * facto[5];
 
-  // j=3: row 4
+  /* j=3: row 4 */
   const cs_real_t  d33 = m->val[21] - l30*l30*d00 - l31*l31*d11 - l32*l32*d22;
   if (fabs(d33) < cs_math_zero_threshold)
     bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
@@ -1385,7 +1726,7 @@ cs_sdm_66_ldlt_compute(const cs_sdm_t   *m,
   const cs_real_t  l53 = facto[18] =
     (m->val[23] - l50*d0l30 - l51*d1l31 - l52*d2l32) * facto[9];
 
-  // j=4: row 5
+  /* j=4: row 5 */
   const cs_real_t  d44 =
     m->val[28] - l40*l40*d00 - l41*l41*d11 - l42*l42*d22 - l43*l43*d33;
   if (fabs(d44) < cs_math_zero_threshold)
@@ -1394,7 +1735,7 @@ cs_sdm_66_ldlt_compute(const cs_sdm_t   *m,
   const cs_real_t  l54 = facto[19] = facto[14] *
     (m->val[29] - l50*d00*l40 - l51*d11*l41 - l52*d22*l42 - l53*d33*l43);
 
-  // j=5: row 6
+  /* j=5: row 6 */
   const cs_real_t  d55 = m->val[35]
     - l50*l50*d00 - l51*l51*d11 - l52*l52*d22 - l53*l53*d33 - l54*l54*d44;
   if (fabs(d55) < cs_math_zero_threshold)
@@ -1448,13 +1789,13 @@ cs_sdm_ldlt_compute(const cs_sdm_t     *m,
 
     case 0:  /* Optimization for the first colum */
       {
-        dkk[0] = m->val[0]; // d00
+        dkk[0] = m->val[0]; /* d00 */
 
         if (fabs(dkk[0]) < cs_math_zero_threshold)
           bft_error(__FILE__, __LINE__, 0, _msg_small_p, __func__);
         const cs_real_t  inv_d00 = facto[0] = 1. / dkk[0];
 
-        // l_i0 = a_i0 / d_00
+        /* l_i0 = a_i0 / d_00 */
         short int rowi_idx = rowj_idx;
         const cs_real_t  *a_0 = m->val;  /* a_ij = a_ji */
         for (short int i = j+1; i < n; i++) { /* Loop on rows */
@@ -1470,7 +1811,7 @@ cs_sdm_ldlt_compute(const cs_sdm_t     *m,
 
     case 1:  /* Optimization for the second colum */
       {
-        // d_11 = a_11 - l_10^2 * d_00
+        /* d_11 = a_11 - l_10^2 * d_00 */
         cs_real_t  *l_1 = facto + rowj_idx;
 
         const cs_real_t  d11 = dkk[1] = m->val[n+1] - l_1[0]*l_1[0]*dkk[0];
@@ -1479,7 +1820,7 @@ cs_sdm_ldlt_compute(const cs_sdm_t     *m,
 
         const cs_real_t  inv_d11 = facto[djj_idx] = 1. / d11;
 
-        // l_i1 = (a_i1 - l_i0 * d_00 * l_10 ) / d_11
+        /* l_i1 = (a_i1 - l_i0 * d_00 * l_10 ) / d_11 */
         short int rowi_idx = rowj_idx;
         const cs_real_t  *a_1 = m->val + n;  /* a_i1 = a_1i */
         for (short int i = 2; i < n; i++) { /* Loop on rows */
@@ -1495,7 +1836,7 @@ cs_sdm_ldlt_compute(const cs_sdm_t     *m,
 
     default:
       {
-        // d_jj = a_jj - \sum_{k=0}^{j-1} l_jk^2 * d_kk
+        /* d_jj = a_jj - \sum_{k=0}^{j-1} l_jk^2 * d_kk */
         cs_real_t  *l_j = facto + rowj_idx;
 
         cs_real_t  sum = 0.;
@@ -1508,7 +1849,7 @@ cs_sdm_ldlt_compute(const cs_sdm_t     *m,
 
         const cs_real_t  inv_djj = facto[djj_idx] = 1. / djj;
 
-        // l_ij = (a_ij - \sum_{k=1}^{j-1} l_ik * d_kk * l_jk ) / d_jj
+        /* l_ij = (a_ij - \sum_{k=1}^{j-1} l_ik * d_kk * l_jk ) / d_jj */
         short int rowi_idx = rowj_idx;
         const cs_real_t  *a_j = m->val + j*n;  /* a_ij = a_ji */
         for (short int i = j+1; i < n; i++) { /* Loop on rows */
@@ -1676,9 +2017,9 @@ cs_sdm_ldlt_solve(int                n_rows,
    */
 
   const short int  last_row_id = n_rows - 1;
-  const int  shift = n_rows*(last_row_id)/2;   // idx with n_rows - 1
-  int  diagi_idx = shift + last_row_id;        // last entry of the facto.
-  sol[last_row_id] *= facto[diagi_idx];         // 1 / d_nn
+  const int  shift = n_rows*(last_row_id)/2;   /* idx with n_rows - 1 */
+  int  diagi_idx = shift + last_row_id;        /* last entry of the facto. */
+  sol[last_row_id] *= facto[diagi_idx];        /* 1 / d_nn */
 
   for (short int i = last_row_id - 1; i >= 0; i--) {
 
@@ -1802,10 +2143,15 @@ cs_sdm_dump(cs_lnum_t           parent_id,
             const cs_lnum_t    *col_ids,
             const cs_sdm_t     *mat)
 {
-  if (mat == NULL)
+  if (mat == NULL) {
+    cs_log_printf(CS_LOG_DEFAULT,
+                  "<< MATRIX is set to NULL (parent id: %ld)>>\n",
+                  (long)parent_id);
     return;
+  }
 
-  cs_log_printf(CS_LOG_DEFAULT, "<< MATRIX parent id: %d >>\n", parent_id);
+  cs_log_printf(CS_LOG_DEFAULT, "<< MATRIX parent id: %ld >>\n",
+                (long)parent_id);
 
   if (mat->n_rows < 1 || mat->n_cols < 1) {
     cs_log_printf(CS_LOG_DEFAULT, " No value.\n");
@@ -1817,13 +2163,13 @@ cs_sdm_dump(cs_lnum_t           parent_id,
 
   else {
 
-    cs_log_printf(CS_LOG_DEFAULT, " %8s %11d", " ", col_ids[0]);
+    cs_log_printf(CS_LOG_DEFAULT, " %8s %11ld", " ", (long)col_ids[0]);
     for (short int i = 1; i < mat->n_cols; i++)
-      cs_log_printf(CS_LOG_DEFAULT, " %11d", col_ids[i]);
+      cs_log_printf(CS_LOG_DEFAULT, " %11ld", (long)col_ids[i]);
     cs_log_printf(CS_LOG_DEFAULT, "\n");
 
     for (short int i = 0; i < mat->n_rows; i++) {
-      cs_log_printf(CS_LOG_DEFAULT, " %8d ", row_ids[i]);
+      cs_log_printf(CS_LOG_DEFAULT, " %8ld ", (long)row_ids[i]);
       for (short int j = 0; j < mat->n_cols; j++)
         cs_log_printf(CS_LOG_DEFAULT, " % .4e", mat->val[i*mat->n_cols+j]);
       cs_log_printf(CS_LOG_DEFAULT, "\n");
@@ -1910,8 +2256,8 @@ cs_sdm_block_dump(cs_lnum_t           parent_id,
   }
   assert(mat->block_desc != NULL);
 
-  cs_log_printf(CS_LOG_DEFAULT, "\n << BLOCK MATRIX parent id: %d >>\n",
-                parent_id);
+  cs_log_printf(CS_LOG_DEFAULT, "\n << BLOCK MATRIX parent id: %ld >>\n",
+                (long)parent_id);
 
   const int  n_b_rows = mat->block_desc->n_row_blocks;
   const int  n_b_cols = mat->block_desc->n_col_blocks;
@@ -1924,7 +2270,7 @@ cs_sdm_block_dump(cs_lnum_t           parent_id,
   cs_log_printf(CS_LOG_DEFAULT, " n_row_blocks: %d; n_col_blocks: %d\n",
                 n_b_rows, n_b_cols);
 
-  const char _sep[] = ".............";
+  const char _sep[] = "------------------------------------------------------";
   for (short int bi = 0; bi < n_b_rows; bi++) {
 
     const cs_sdm_t  *bi0 = blocks + bi*n_b_cols;
