@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -212,6 +212,26 @@ cs_sdm_add_scalvect(int               n,
     y[i] += s * x[i];
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Set the lower left according to the upper right part in order to
+ *          get a symmetric matrix.
+ *
+ * \param[in, out]      mat    pointer to a cs_sdm_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_sdm_symm_ur(cs_sdm_t      *mat)
+{
+  assert(mat != NULL);
+  for (int i = 1; i < mat->n_rows; i++) {
+    cs_real_t  *m_i = mat->val + i*mat->n_rows;
+    for (int j = 0; j < i; j++)
+      m_i[j] = mat->val[j*mat->n_rows + i];
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief   Allocate and initialize a cs_sdm_t structure
@@ -260,8 +280,7 @@ cs_sdm_create_copy(const cs_sdm_t   *m);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief   Define a new matrix by adding the given matrix with its transpose.
- *          Keep the transposed matrix for a future use.
+ * \brief   Define a new matrix which is its transpose.
  *
  * \param[in] mat   local matrix to transpose
  *
@@ -286,10 +305,26 @@ cs_sdm_create_transpose(cs_sdm_t  *mat);
 /*----------------------------------------------------------------------------*/
 
 cs_sdm_t *
-cs_sdm_block_create(int                n_max_blocks_by_row,
-                    int                n_max_blocks_by_col,
-                    const short int    max_row_block_sizes[],
-                    const short int    max_col_block_sizes[]);
+cs_sdm_block_create(int          n_max_blocks_by_row,
+                    int          n_max_blocks_by_col,
+                    const int    max_row_block_sizes[],
+                    const int    max_col_block_sizes[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Allocate and initialize a cs_sdm_t structure by block when the
+ *          block size is constant and equal to 3
+ *
+ * \param[in]  n_max_blocks_by_row    max number of blocks in a row
+ * \param[in]  n_max_blocks_by_col    max number of blocks in a column
+ *
+ * \return  a new allocated cs_sdm_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_sdm_t *
+cs_sdm_block33_create(int      n_max_blocks_by_row,
+                      int      n_max_blocks_by_col);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -390,11 +425,42 @@ cs_sdm_square_init(int         n_rows,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_sdm_block_init(cs_sdm_t          *m,
-                  int                n_row_blocks,
-                  int                n_col_blocks,
-                  const short int    row_block_sizes[],
-                  const short int    col_block_sizes[]);
+cs_sdm_block_init(cs_sdm_t      *m,
+                  int            n_row_blocks,
+                  int            n_col_blocks,
+                  const int      row_block_sizes[],
+                  const int      col_block_sizes[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Initialize the pattern of cs_sdm_t structure defined by 3x3 block
+ *          The matrix should have been allocated before calling this function
+ *
+ * \param[in, out] m
+ * \param[in]      n_row_blocks      number of blocks in a row
+ * \param[in]      n_col_blocks      number of blocks in a column
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_block33_init(cs_sdm_t     *m,
+                    int           n_row_blocks,
+                    int           n_col_blocks);
+
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Convert a matrix with each entry is a 3x3 block into a matrix
+ *          with a block for each component x,y,z.
+ *
+ * \param[in]      mb33        pointer to a matrix
+ * \param[in, out] mbxyz       pointer to a matrix to build (but allocated)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_block_33_to_xyz(const cs_sdm_t   *mb33,
+                       cs_sdm_t         *mbxyz);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -449,9 +515,9 @@ cs_sdm_block_create_copy(const cs_sdm_t   *mref);
 /*----------------------------------------------------------------------------*/
 
 static inline cs_sdm_t *
-cs_sdm_get_block(const cs_sdm_t    *m,
-                 int                row_block_id,
-                 int                col_block_id)
+cs_sdm_get_block(const cs_sdm_t    *const m,
+                 int                      row_block_id,
+                 int                      col_block_id)
 {
   /* Sanity checks */
   assert(m != NULL);
@@ -459,9 +525,8 @@ cs_sdm_get_block(const cs_sdm_t    *m,
   assert(col_block_id < m->block_desc->n_col_blocks);
   assert(row_block_id < m->block_desc->n_row_blocks);
 
-  const cs_sdm_block_t  *bd = m->block_desc;
-
-  return  bd->blocks + row_block_id*bd->n_col_blocks + col_block_id;
+  return  m->block_desc->blocks
+    + row_block_id*m->block_desc->n_col_blocks + col_block_id;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -523,10 +588,30 @@ cs_sdm_copy_block(const cs_sdm_t       *m,
     memcpy(b->val + i*nc, _start, sizeof(cs_real_t)*nc);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Multiply a matrix with the scaling factor given as parameter
+ *
+ * \param[in]      scaling
+ * \param[in, out] m      pointer to cs_sdm_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_sdm_scale(cs_real_t       scaling,
+             cs_sdm_t       *m)
+{
+  /* Sanity checks */
+  assert(m != NULL);
+
+  for (int i = 0; i < m->n_rows*m->n_cols; i++)
+    m->val[i] *= scaling;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  transpose and copy a matrix into another one already shaped
- *          sub-matrix starting from (r_id, c_id)
+ *         sub-matrix starting from (r_id, c_id)
  *
  * \param[in]      m      pointer to cs_sdm_t structure
  * \param[in, out] mt     matrix to update with the transposed of m
@@ -565,6 +650,31 @@ cs_sdm_multiply(const cs_sdm_t   *a,
                 const cs_sdm_t   *b,
                 cs_sdm_t         *c);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Compute a dense 3x3 matrix-vector product
+ *          mv has been previously allocated
+ *
+ * \param[in]      mat    local matrix to use
+ * \param[in]      vec    local vector to use
+ * \param[in, out] mv result of the local matrix-vector product
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_sdm_33_matvec(const cs_sdm_t    *mat,
+                 const cs_real_t   *vec,
+                 cs_real_t         *mv)
+{
+  /* Sanity checks */
+  assert(mat != NULL && vec != NULL && mv != NULL);
+  assert(mat->n_rows == 3 && mat->n_cols == 3);
+
+  mv[0] = vec[0]*mat->val[0] + vec[1]*mat->val[1] + vec[2]*mat->val[2];
+  mv[1] = vec[0]*mat->val[3] + vec[1]*mat->val[4] + vec[2]*mat->val[5];
+  mv[2] = vec[0]*mat->val[6] + vec[1]*mat->val[7] + vec[2]*mat->val[8];
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief    Compute a row-row matrix product of a and b. It is basically equal
@@ -825,6 +935,18 @@ void
 cs_sdm_square_add_transpose(cs_sdm_t  *mat,
                             cs_sdm_t  *tr);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Set the given matrix to two times its symmetric part
+ *          mat --> mat + mat_tr = 2*symm(mat)
+ *
+ * \param[in, out] mat   small dense matrix to transform
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_square_2symm(cs_sdm_t   *mat);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief   Set the given matrix into its anti-symmetric part
@@ -871,6 +993,86 @@ cs_sdm_33_sym_qr_compute(const cs_real_t   m[9],
                          cs_real_t         Qt[9],
                          cs_real_t         R[6]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  LU factorization of a small dense 3x3 matrix.
+ *
+ * \param[in]      m        pointer to a cs_sdm_t structure
+ * \param[in, out] facto    compact storage of coefficients for the LU
+ *                          factorization
+ *
+ * \note: facto stores L the lower triangular matrix (without its diagonal
+ *        entries assumed to be equal to 1) and U the upper triangular matrix.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_33_lu_compute(const cs_sdm_t   *m,
+                     cs_real_t         facto[9]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  LU factorization of a small dense matrix. Small means that the
+ *         number m->n_rows is less than 100 for instance.
+ *
+ * \param[in]      m        pointer to a cs_sdm_t structure
+ * \param[in, out] facto    compact storage of coefficients for the LU
+ *                          factorization (should be allocated to the right
+ *                          size, i.e. m->n_rows*m->n_rows)
+ *
+ * \note: facto stores L the lower triangular matrix (without its diagonal
+ *        entries assumed to be equal to 1) and U the upper triangular matrix.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_lu_compute(const cs_sdm_t   *m,
+                  cs_real_t         facto[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Solve a system A.sol = rhs using a LU factorization of A (a small
+ *         3x3 dense matrix).
+ *
+ * \param[in]       facto    compact storage of coefficients for the LU
+ *                           factorization (should be allocated to the right
+ *                           size, i.e. n_rows*n_rows)
+ * \param[in]       rhs      right-hand side
+ * \param[in, out]  sol      solution
+ *
+ * \note: facto stores L the lower triangular matrix (without its diagonal
+ *        entries assumed to be equal to 1) and U the upper triangular matrix.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_33_lu_solve(const cs_real_t    facto[9],
+                   const cs_real_t    rhs[3],
+                   cs_real_t          sol[3]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Solve a system A.sol = rhs using a LU factorization of A (a small
+ *         dense matrix).
+ *
+ * \param[in]       n_rows   dimension of the system to solve
+ * \param[in]       facto    compact storage of coefficients for the LU
+ *                           factorization (should be allocated to the right
+ *                           size, i.e. n_rows*n_rows)
+ * \param[in]       rhs      right-hand side
+ * \param[in, out]  sol      solution
+ *
+ * \note: facto stores L the lower triangular matrix (without its diagonal
+ *        entries assumed to be equal to 1) and U the upper triangular matrix.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sdm_lu_solve(cs_lnum_t          n_rows,
+                const cs_real_t    facto[],
+                const cs_real_t   *rhs,
+                cs_real_t         *sol);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  LDL^T: Modified Cholesky decomposition of a 3x3 SPD matrix.
