@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,7 @@
 #include "cs_base.h"
 #include "cs_cdo_quantities.h"
 #include "cs_cdo_local.h"
-#include "cs_param.h"
+#include "cs_param_types.h"
 #include "cs_quadrature.h"
 #include "cs_xdef.h"
 
@@ -52,10 +52,25 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
+/*! \struct cs_source_term_boussinesq_t
+ *  \brief Structure storing the parameters related to the Boussinesq source
+ *         term
+ */
+
+typedef struct {
+
+  cs_real_t    g[3];      /* gravity vector */
+  cs_real_t    rho0;      /* reference value for the mass density */
+  cs_real_t    beta;      /* dilatation coefficient */
+  cs_real_t    var0;      /* reference value of the variable */
+  cs_real_t   *var;       /* array of values of the variable */
+
+} cs_source_term_boussinesq_t;
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *
  * \param[in]      source     pointer to a cs_xdef_term_t structure
  * \param[in]      cm         pointer to a cs_cell_mesh_t structure
@@ -97,14 +112,16 @@ cs_source_term_set_shared_pointers(const cs_cdo_quantities_t    *quant,
  * \brief  Set the default flag related to a source term according to the
  *         numerical scheme chosen for discretizing an equation
  *
- * \param[in]       scheme    numerical scheme used for the discretization
- *
- * \return a default flag
+ * \param[in]   scheme      numerical scheme used for the discretization
+ * \param[out]  state_flag  flag describing the status of the source term
+ * \param[out]  meta_flag   additional flags associated to a source term
  */
 /*----------------------------------------------------------------------------*/
 
-cs_flag_t
-cs_source_term_set_default_flag(cs_param_space_scheme_t  scheme);
+void
+cs_source_term_set_default_flag(cs_param_space_scheme_t    scheme,
+                                cs_flag_t                 *state_flag,
+                                cs_flag_t                 *meta_flag);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -149,7 +166,7 @@ cs_source_term_get_flag(const cs_xdef_t  *st);
  */
 /*----------------------------------------------------------------------------*/
 
-cs_flag_t
+cs_eflag_t
 cs_source_term_init(cs_param_space_scheme_t      space_scheme,
                     const int                    n_source_terms,
                     cs_xdef_t            *const *source_terms,
@@ -185,50 +202,14 @@ cs_source_term_compute_cellwise(const int                    n_source_terms,
                                 cs_cell_builder_t           *cb,
                                 cs_real_t                   *result);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Compute the contribution related to a source term in the case of
- *         an input data which is a density
- *
- * \param[in]      loc        describe where is located the associated DoF
- * \param[in]      source     pointer to a cs_xdef_t structure
- * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in, out] p_values   pointer to the computed values (allocated if NULL)
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_source_term_compute_from_density(cs_flag_t                loc,
-                                    const cs_xdef_t         *source,
-                                    cs_real_t                time_eval,
-                                    double                  *p_values[]);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Compute the contribution related to a source term in the case of
- *         an input data which is a potential
- *
- * \param[in]      loc        describe where is located the associated DoF
- * \param[in]      source     pointer to a cs_xdef_t structure
- * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in, out] p_values   pointer to the computed values (allocated if NULL)
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_source_term_compute_from_potential(cs_flag_t                loc,
-                                      const cs_xdef_t         *source,
-                                      cs_real_t                time_eval,
-                                      double                  *p_values[]);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar potential defined at primal vertices by a constant
  *         value.
  *         A discrete Hodge operator has to be computed before this call and
- *         stored inside a cs_cell_builder_t structure
+ *         given as input parameter
  *
  * \param[in]      source     pointer to a cs_xdef_t structure
  * \param[in]      cm         pointer to a cs_cell_mesh_t structure
@@ -250,11 +231,11 @@ cs_source_term_pvsp_by_value(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar potential defined at primal vertices by an
  *         analytical function.
  *         A discrete Hodge operator has to be computed before this call and
- *         stored inside a cs_cell_builder_t structure
+ *         given as input parameter
  *
  * \param[in]      source     pointer to a cs_xdef_t structure
  * \param[in]      cm         pointer to a cs_cell_mesh_t structure
@@ -276,7 +257,7 @@ cs_source_term_pvsp_by_analytic(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar density defined at dual cells by a value.
  *
  * \param[in]      source     pointer to a cs_xdef_t structure
@@ -299,7 +280,30 @@ cs_source_term_dcsd_by_value(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
+ *         Case of a vector-valued density defined at dual cells by a value.
+ *
+ * \param[in]      source     pointer to a cs_xdef_t structure
+ * \param[in]      cm         pointer to a cs_cell_mesh_t structure
+ * \param[in]      time_eval  physical time at which one evaluates the term
+ * \param[in, out] cb         pointer to a cs_cell_builder_t structure
+ * \param[in, out] input      pointer to an element cast on-the-fly (or NULL)
+ * \param[in, out] values     pointer to the computed values
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_source_term_dcvd_by_value(const cs_xdef_t           *source,
+                             const cs_cell_mesh_t      *cm,
+                             cs_real_t                  time_eval,
+                             cs_cell_builder_t         *cb,
+                             void                      *input,
+                             double                    *values);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the contribution for a cell related to a source term and
+ *         add it to the given array of values.
  *         Case of a scalar density defined at dual cells by an array.
  *
  * \param[in]      source     pointer to a cs_xdef_t structure
@@ -322,7 +326,30 @@ cs_source_term_dcsd_by_array(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
+ *         Case of a scalar density defined at dual cells by a DoF function.
+ *
+ * \param[in]      source     pointer to a cs_xdef_t structure
+ * \param[in]      cm         pointer to a cs_cell_mesh_t structure
+ * \param[in]      time_eval  physical time at which one evaluates the term
+ * \param[in, out] cb         pointer to a cs_cell_builder_t structure
+ * \param[in, out] input      pointer to an element cast on-the-fly (or NULL)
+ * \param[in, out] values     pointer to the computed values
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_source_term_dcsd_by_dof_func(const cs_xdef_t           *source,
+                                const cs_cell_mesh_t      *cm,
+                                cs_real_t                  time_eval,
+                                cs_cell_builder_t         *cb,
+                                void                      *input,
+                                double                    *values);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the contribution for a cell related to a source term and
+ *         add it to the given array of values.
  *         Case of a scalar density defined at dual cells by an analytical
  *         function.
  *         Use the barycentric approximation as quadrature to evaluate the
@@ -348,7 +375,7 @@ cs_source_term_dcsd_bary_by_analytic(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar density defined at dual cells by an analytical
  *         function.
  *         Use a the barycentric approximation as quadrature to evaluate the
@@ -374,7 +401,7 @@ cs_source_term_dcsd_q1o1_by_analytic(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar density defined at dual cells by an analytical
  *         function.
  *         Use a ten-point quadrature rule to evaluate the integral.
@@ -400,7 +427,7 @@ cs_source_term_dcsd_q10o2_by_analytic(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar density defined at dual cells by an analytical
  *         function.
  *         Use a five-point quadrature rule to evaluate the integral.
@@ -428,11 +455,11 @@ cs_source_term_dcsd_q5o3_by_analytic(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar potential defined at primal vertices and cells
  *         by a constant value.
  *         A discrete Hodge operator has to be computed before this call and
- *         stored inside a cs_cell_builder_t structure
+ *         given as input parameter
  *
  * \param[in]      source     pointer to a cs_xdef_t structure
  * \param[in]      cm         pointer to a cs_cell_mesh_t structure
@@ -454,11 +481,11 @@ cs_source_term_vcsp_by_value(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar potential defined at primal vertices and cells by
  *         an analytical function.
  *         A discrete Hodge operator has to be computed before this call and
- *         stored inside a cs_cell_builder_t structure
+ *         given as input parameter
  *
  * \param[in]      source     pointer to a cs_xdef_t structure
  * \param[in]      cm         pointer to a cs_cell_mesh_t structure
@@ -480,7 +507,7 @@ cs_source_term_vcsp_by_analytic(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar density (sd) defined on primal cells by a value.
  *         Case of face-based schemes
  *
@@ -501,6 +528,54 @@ cs_source_term_pcsd_by_value(const cs_xdef_t           *source,
                              void                      *input,
                              double                    *values);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the contribution for a cell related to a source term and
+ *         add it to the given array of values.
+ *         Case of a density defined on primal cells by a DoF function.
+ *         Case of face-based schemes
+ *
+ * \param[in]      source     pointer to a cs_xdef_t structure
+ * \param[in]      cm         pointer to a cs_cell_mesh_t structure
+ * \param[in]      time_eval  physical time at which one evaluates the term
+ * \param[in, out] cb         pointer to a cs_cell_builder_t structure
+ * \param[in, out] input      pointer to an element cast on-the-fly (or NULL)
+ * \param[in, out] values     pointer to the computed value
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_source_term_pcsd_by_dof_func(const cs_xdef_t           *source,
+                                const cs_cell_mesh_t      *cm,
+                                cs_real_t                  time_eval,
+                                cs_cell_builder_t         *cb,
+                                void                      *input,
+                                double                    *values);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the contribution for a cell related to a source term and
+ *         add it to the given array of values.
+ *         Case of a density defined on primal cells by a DoF function.
+ *         Case of vector-valued face-based schemes
+ *
+ * \param[in]      source     pointer to a cs_xdef_t structure
+ * \param[in]      cm         pointer to a cs_cell_mesh_t structure
+ * \param[in]      time_eval  physical time at which one evaluates the term
+ * \param[in, out] cb         pointer to a cs_cell_builder_t structure
+ * \param[in, out] input      pointer to an element cast on-the-fly (or NULL)
+ * \param[in, out] values     pointer to the computed value
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_source_term_pcvd_by_dof_func(const cs_xdef_t           *source,
+                                const cs_cell_mesh_t      *cm,
+                                cs_real_t                  time_eval,
+                                cs_cell_builder_t         *cb,
+                                void                      *input,
+                                double                    *values);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
@@ -529,7 +604,7 @@ cs_source_term_pcvd_by_value(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar density defined at primal cells by an analytical
  *         function.
  *         Use the barycentric approximation as quadrature to evaluate the
@@ -580,7 +655,7 @@ cs_source_term_pcsd_by_analytic(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a scalar density defined at primal cells by an array.
  *
  * \param[in]      source     pointer to a cs_xdef_t structure
@@ -603,7 +678,7 @@ cs_source_term_pcsd_by_array(const cs_xdef_t           *source,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the contribution for a cell related to a source term and
- *         add it the given array of values.
+ *         add it to the given array of values.
  *         Case of a vector-valued density defined at primal cells by an
  *         analytical function.
  *         Use the barycentric approximation as quadrature to evaluate the
@@ -749,6 +824,29 @@ cs_source_term_hhovd_by_analytic(const cs_xdef_t           *source,
                                  double                    *values);
 
 /*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the contribution for a cell related to a source term and
+ *         add it to the given array of values.
+ *         Case of a scalar flux defined at dual faces by a constant value.
+ *
+ * \param[in]      source     pointer to a cs_xdef_t structure
+ * \param[in]      cm         pointer to a cs_cell_mesh_t structure
+ * \param[in]      time_eval  physical time at which one evaluates the term
+ * \param[in, out] cb         pointer to a cs_cell_builder_t structure
+ * \param[in, out] input      pointer to an element cast on-the-fly (or NULL)
+ * \param[in, out] values     pointer to the computed values
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_source_term_dfsf_by_value(const cs_xdef_t           *source,
+                             const cs_cell_mesh_t      *cm,
+                             cs_real_t                  time_eval,
+                             cs_cell_builder_t         *cb,
+                             void                      *input,
+                             double                    *values);
+
+/*----------------------------------------------------------------------------*/
 
 
 
