@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -49,13 +49,13 @@
 #include "bft_printf.h"
 
 #include "cs_blas.h"
+#include "cs_boundary.h"
 #include "cs_equation.h"
 #include "cs_field.h"
 #include "cs_log.h"
 #include "cs_math.h"
 #include "cs_mesh.h"
 #include "cs_mesh_location.h"
-#include "cs_param.h"
 #include "cs_post.h"
 #include "cs_reco.h"
 
@@ -121,22 +121,22 @@ _compute_poisson_cdovcb(const cs_cdo_connect_t     *connect,
   cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
                            CS_POST_WRITER_ALL_ASSOCIATED,
                            "Poisson_Sol",
-                           1,               // dim
-                           true,            // interlace
-                           true,            // true = original mesh
+                           1,               /* dim */
+                           true,            /* interlace */
+                           true,            /* true = original mesh */
                            CS_POST_TYPE_cs_real_t,
-                           field->val,      // values on vertices
-                           NULL);           // time step management structure
+                           field->val,      /* values on vertices */
+                           NULL);           /* time step management structure */
 
   cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
                            CS_POST_WRITER_ALL_ASSOCIATED,
                            "GrdVtx",
-                           3,               // dim
-                           true,           // interlace
-                           true,            // true = original mesh
+                           3,               /* dim */
+                           true,            /* interlace */
+                           true,            /* true = original mesh */
                            CS_POST_TYPE_cs_real_t,
-                           vtx_gradient,             // values on vertices
-                           NULL);           // time step management structure
+                           vtx_gradient,    /* values on vertices */
+                           NULL);           /* time step management structure */
 #endif
 
   /* Compute now wall distance at each vertex */
@@ -168,12 +168,12 @@ _compute_poisson_cdovcb(const cs_cdo_connect_t     *connect,
   cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
                            CS_POST_WRITER_ALL_ASSOCIATED,
                            field->name,
-                           1,               // dim
-                           false,           // interlace
-                           true,            // true = original mesh
+                           1,               /* dim */
+                           false,           /* interlace */
+                           true,            /* true = original mesh */
                            CS_POST_TYPE_cs_real_t,
-                           dist,            // values on vertices
-                           NULL);           // time step management structure
+                           dist,            /* values on vertices */
+                           NULL);           /* time step management structure */
 
   /* Free memory */
   BFT_FREE(vtx_gradient);
@@ -227,23 +227,23 @@ _compute_poisson_cdovb(const cs_cdo_connect_t     *connect,
       for (int k = 0; k < 3; k++)
         vtx_gradient[v_id][k] += cdoq->dcell_vol[i]*cell_gradient[k];
 
-    } // Loop on cell vertices
+    } /* Loop on cell vertices */
 
-  } // Loop on cells
+  } /* Loop on cells */
 
-  if (cs_glob_n_ranks > 1) {
+  if (connect->interfaces[CS_CDO_CONNECT_VTX_SCAL] != NULL) {
 
     cs_interface_set_sum(connect->interfaces[CS_CDO_CONNECT_VTX_SCAL],
                          connect->n_vertices,
                          1,
-                         true, // interlace
+                         true, /* interlace */
                          CS_REAL_TYPE,
                          dualcell_vol);
 
     cs_interface_set_sum(connect->interfaces[CS_CDO_CONNECT_VTX_SCAL],
                          connect->n_vertices,
                          3,
-                         true, // interlace
+                         true, /* interlace */
                          CS_REAL_TYPE,
                          vtx_gradient);
 
@@ -261,22 +261,22 @@ _compute_poisson_cdovb(const cs_cdo_connect_t     *connect,
   cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
                            CS_POST_WRITER_ALL_ASSOCIATED,
                            "Poisson_Sol",
-                           1,               // dim
-                           true,            // interlace
-                           true,            // true = original mesh
+                           1,               /* dim */
+                           true,            /* interlace */
+                           true,            /* true = original mesh */
                            CS_POST_TYPE_cs_real_t,
-                           var,             // values on vertices
-                           NULL);           // time step management structure
+                           var,             /* values on vertices */
+                           NULL);           /* time step management structure */
 
   cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
                            CS_POST_WRITER_ALL_ASSOCIATED,
                            "GrdVtx",
-                           3,               // dim
-                           true,            // interlace
-                           true,            // true = original mesh
+                           3,               /* dim */
+                           true,            /* interlace */
+                           true,            /* true = original mesh */
                            CS_POST_TYPE_cs_real_t,
-                           vtx_gradient,    // values on vertices
-                           NULL);           // time step management structure
+                           vtx_gradient,    /* values on vertices */
+                           NULL);           /* time step management structure */
 #endif
 
   /* Compute now the wall distance at each vertex */
@@ -296,21 +296,21 @@ _compute_poisson_cdovb(const cs_cdo_connect_t     *connect,
 
   if (count > 0) {
     cs_base_warn(__FILE__, __LINE__);
-    bft_printf(" %d degree(s) of freedom have a negative value and have been"
+    bft_printf(" %s: %d degrees of freedom have a negative value and have been"
                " modified\n"
-               " This may result from a bad mesh quality.", count);
+               " This may result from a bad mesh quality.", __func__, count);
   }
 
   /* Post-processing */
   cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
                            CS_POST_WRITER_ALL_ASSOCIATED,
                            field->name,
-                           1,               // dim
-                           false,           // interlace
-                           true,            // true = original mesh
+                           1,               /* dim */
+                           false,           /* interlace */
+                           true,            /* true = original mesh */
                            CS_POST_TYPE_cs_real_t,
-                           dist,            // values on vertices
-                           NULL);           // time step management structure
+                           dist,            /* values on vertices */
+                           NULL);           /* time step management structure */
 
   /* Free memory */
   BFT_FREE(dualcell_vol);
@@ -340,7 +340,7 @@ _compute_poisson_cdofb(const cs_cdo_connect_t     *connect,
   cs_lnum_t  i, k;
 
   const cs_real_t  *c_var = field->val;
-  const cs_real_t  *f_var = cs_equation_get_face_values(eq);
+  const cs_real_t  *f_var = cs_equation_get_face_values(eq, false);
 
   /* Loop on cells */
   for (cs_lnum_t c_id = 0; c_id < cdoq->n_cells; c_id++) {
@@ -358,30 +358,30 @@ _compute_poisson_cdofb(const cs_cdo_connect_t     *connect,
       for (k = 0; k < 3; k++)
         cell_gradient[k] += dualedge_contrib*fq.unitv[k];
 
-    } // Loop on cell faces
+    } /* Loop on cell faces */
 
     for (k = 0; k < 3; k++)
       cell_gradient[k] *= inv_cell_vol;
 
     /* Compute the distance from the wall at this cell center */
     cs_real_t  tmp = _dp3(cell_gradient, cell_gradient) + 2*c_var[c_id];
-    assert(tmp >= 0.); // Sanity check
+    assert(tmp >= 0.); /* Sanity check */
 
     dist[c_id] = sqrt(tmp) - cs_math_3_norm(cell_gradient);
 
-  } // Loop on cells
+  } /* Loop on cells */
 
   cs_post_write_var(CS_POST_MESH_VOLUME,
                     CS_POST_WRITER_ALL_ASSOCIATED,
                     field->name,
-                    1,               // dim
-                    false,           // interlace
-                    true,            // true = original mesh
+                    1,               /* dim */
+                    false,           /* interlace */
+                    true,            /* true = original mesh */
                     CS_POST_TYPE_cs_real_t,
-                    dist,            // values on cells
-                    NULL,            // values at internal faces
-                    NULL,            // values at border faces
-                    NULL);           // time step management structure
+                    dist,            /* values on cells */
+                    NULL,            /* values at internal faces */
+                    NULL,            /* values at border faces */
+                    NULL);           /* time step management structure */
 
 }
 
@@ -463,7 +463,7 @@ cs_walldistance_setup(void)
 
   /* Add boundary conditions */
   cs_real_t  zero_value = 0.;
-  const char  bc_zone_name[] = "cs_domain_boundary_walls";
+  const char  bc_zone_name[] = CS_BOUNDARY_WALLS_NAME;
 
   cs_equation_add_bc_by_value(eqp,
                               CS_PARAM_BC_DIRICHLET,
@@ -517,6 +517,8 @@ cs_walldistance_compute(const cs_mesh_t              *mesh,
                         const cs_cdo_connect_t       *connect,
                         const cs_cdo_quantities_t    *cdoq)
 {
+  CS_UNUSED(time_step);
+
   /* First step:
      Solve the equation related to the definition of the wall distance. */
 
@@ -527,13 +529,11 @@ cs_walldistance_compute(const cs_mesh_t              *mesh,
 
   else { /* Deprecated */
 
-    double  dt_cur = 0.;  /* Wall distance is a steady-stae equation */
-
     /* Sanity check */
     assert(cs_equation_is_steady(eq));
 
     /* Define the algebraic system */
-    cs_equation_build_system(mesh, time_step, dt_cur, eq);
+    cs_equation_build_system(mesh, eq);
 
     /* Solve the algebraic system */
     cs_equation_solve_deprecated(eq);
