@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -39,12 +39,10 @@
 
 #include "bft_mem.h"
 
-#include "cs_boundary_zone.h"
 #include "cs_field.h"
 #include "cs_flag.h"
 #include "cs_log.h"
 #include "cs_mesh_location.h"
-#include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -66,6 +64,18 @@ BEGIN_C_DECLS
  * Global static variables
  *============================================================================*/
 
+static const char *_xdef_type_name[]
+  = {"CS_XDEF_BY_ANALYTIC_FUNCTION",
+     "CS_XDEF_BY_ARRAY",
+     "CS_XDEF_BY_DOF_FUNCTION",
+     "CS_XDEF_BY_FIELD",
+     "CS_XDEF_BY_FUNCTION",
+     "CS_XDEF_BY_QOV",
+     "CS_XDEF_BY_SUB_DEFINITIONS",
+     "CS_XDEF_BY_TIME_FUNCTION",
+     "CS_XDEF_BY_VALUE",
+     "out of range"};
+
 /*============================================================================
  * Private function prototypes
  *============================================================================*/
@@ -79,24 +89,24 @@ BEGIN_C_DECLS
  * \brief  Allocate and initialize a new cs_xdef_t structure based on volumic
  *         elements
  *
- * \param[in]  type       type of definition
- * \param[in]  dim        dimension of the values to define
- * \param[in]  z_id       volume zone id
- * \param[in]  state      flag to know if this uniform, cellwise, steady...
- * \param[in]  meta       metadata associated to this description
- * \param[in]  input      pointer to a structure
+ * \param[in]  type        type of definition
+ * \param[in]  dim         dimension of the values to define
+ * \param[in]  z_id        volume zone id
+ * \param[in]  state       flag to know if this uniform, cellwise, steady...
+ * \param[in]  meta        metadata associated to this description
+ * \param[in]  context     pointer to a structure
  *
  * \return a pointer to the new cs_xdef_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 cs_xdef_t *
-cs_xdef_volume_create(cs_xdef_type_t    type,
-                      int               dim,
-                      int               z_id,
-                      cs_flag_t         state,
-                      cs_flag_t         meta,
-                      void             *input)
+cs_xdef_volume_create(cs_xdef_type_t           type,
+                      int                      dim,
+                      int                      z_id,
+                      cs_flag_t                state,
+                      cs_flag_t                meta,
+                      void                    *context)
 {
   cs_xdef_t  *d = NULL;
 
@@ -108,17 +118,18 @@ cs_xdef_volume_create(cs_xdef_type_t    type,
   d->z_id = z_id;
   d->state = state;
   d->meta = meta;
-  d->qtype = CS_QUADRATURE_BARY; // default value
+  d->qtype = CS_QUADRATURE_BARY; /* default value */
 
+  /* Now define the context pointer */
   switch (type) {
 
   case CS_XDEF_BY_VALUE:
     {
-      double  *_input = (double *)input;
-      BFT_MALLOC(d->input, dim, double);
-      double  *_input_cpy = (double *)d->input;
+      double  *_context = (double *)context;
+      BFT_MALLOC(d->context, dim, double);
 
-      for (int i = 0; i < dim; i++) _input_cpy[i] = _input[i];
+      double  *_context_cpy = (double *)d->context;
+      for (int i = 0; i < dim; i++) _context_cpy[i] = _context[i];
 
       /* Update state flag */
       d->state |= CS_FLAG_STATE_UNIFORM | CS_FLAG_STATE_CELLWISE;
@@ -127,26 +138,63 @@ cs_xdef_volume_create(cs_xdef_type_t    type,
 
   case CS_XDEF_BY_ANALYTIC_FUNCTION:
     {
-      cs_xdef_analytic_input_t  *a = (cs_xdef_analytic_input_t *)input;
-      cs_xdef_analytic_input_t  *b = NULL;
+      cs_xdef_analytic_context_t  *a = (cs_xdef_analytic_context_t *)context;
+      cs_xdef_analytic_context_t  *b = NULL;
 
-      BFT_MALLOC(b, 1, cs_xdef_analytic_input_t);
+      BFT_MALLOC(b, 1, cs_xdef_analytic_context_t);
+      assert(a->z_id == z_id);
+      b->z_id = a->z_id;
       b->func = a->func;
       b->input = a->input;
+      b->free_input = a->free_input;
 
-      d->input = b;
+      d->context = b;
+    }
+    break;
+
+  case CS_XDEF_BY_DOF_FUNCTION:
+    {
+      cs_xdef_dof_context_t  *a = (cs_xdef_dof_context_t *)context;
+      cs_xdef_dof_context_t  *b = NULL;
+
+      BFT_MALLOC(b, 1, cs_xdef_dof_context_t);
+      assert(a->z_id == z_id);
+      b->z_id = a->z_id;
+      b->func = a->func;
+      b->loc = a->loc;
+      b->input = a->input;
+      b->free_input = a->free_input;
+
+      d->context = b;
+    }
+    break;
+
+  case CS_XDEF_BY_TIME_FUNCTION:
+    {
+      cs_xdef_time_func_context_t  *a = (cs_xdef_time_func_context_t *)context;
+      cs_xdef_time_func_context_t  *b = NULL;
+
+      BFT_MALLOC(b, 1, cs_xdef_time_func_context_t);
+      b->func = a->func;
+      b->input = a->input;
+      b->free_input = a->free_input;
+
+      d->context = b;
     }
     break;
 
   case CS_XDEF_BY_ARRAY:
     {
-      cs_xdef_array_input_t  *a = (cs_xdef_array_input_t *)input;
-      cs_xdef_array_input_t  *b = NULL;
+      cs_xdef_array_context_t  *a = (cs_xdef_array_context_t *)context;
+      cs_xdef_array_context_t  *b = NULL;
 
-      BFT_MALLOC(b, 1, cs_xdef_array_input_t);
+      BFT_MALLOC(b, 1, cs_xdef_array_context_t);
+      assert(a->z_id == z_id);
+      b->z_id = a->z_id;
       b->stride = a->stride;
       b->loc = a->loc;
       b->values = a->values;
+      b->is_owner = a->is_owner;
       b->index = a->index;
 
       /* Update state flag */
@@ -154,33 +202,51 @@ cs_xdef_volume_create(cs_xdef_type_t    type,
           cs_flag_test(b->loc, cs_flag_dual_face_byc))
         d->state |= CS_FLAG_STATE_CELLWISE;
 
-      d->input = b;
+      d->context = b;
     }
     break;
 
   case CS_XDEF_BY_FIELD:
     {
-      cs_field_t  *f = (cs_field_t *)input;
+      cs_field_t  *f = (cs_field_t *)context;
 
-      d->input = f;
+      d->context = f;
+      assert(f != NULL);
+
+      const cs_mesh_location_type_t  loc_type =
+        cs_mesh_location_get_type(f->location_id);
 
       /* Update state flag */
-      if (f->location_id == cs_mesh_location_get_id_by_name(N_("cells")))
+      switch(loc_type) {
+
+      case CS_MESH_LOCATION_CELLS:
         d->state |= CS_FLAG_STATE_CELLWISE;
+        d->meta |= CS_FLAG_FULL_LOC;
+        break;
+      case CS_MESH_LOCATION_VERTICES:
+        d->meta |= CS_FLAG_FULL_LOC;
+        break;
+
+      default:
+        break; /* Nothing to do */
+      }
+
     }
     break;
 
   case CS_XDEF_BY_QOV:
     {
-      double  *_input = (double *)input;
-      BFT_MALLOC(d->input, 1, double);
-      double *_input_cpy = (double *)d->input;
-      _input_cpy[0] = _input[0];
+      double  *_context = (double *)context;
+
+      BFT_MALLOC(d->context, 1, double);
+
+      double *_context_cpy = (double *)d->context;
+      _context_cpy[0] = _context[0];
     }
     break;
 
   default:
-    d->input = input;
+    d->context = context;
     break;
   }
 
@@ -197,7 +263,7 @@ cs_xdef_volume_create(cs_xdef_type_t    type,
  * \param[in]  z_id       volume zone id
  * \param[in]  state      flag to know if this uniform, cellwise, steady...
  * \param[in]  meta       metadata associated to this description
- * \param[in]  input      pointer to a structure
+ * \param[in]  context    pointer to a structure
  *
  * \return a pointer to the new cs_xdef_t structure
  */
@@ -209,7 +275,7 @@ cs_xdef_boundary_create(cs_xdef_type_t    type,
                         int               z_id,
                         cs_flag_t         state,
                         cs_flag_t         meta,
-                        void             *input)
+                        void             *context)
 {
   cs_xdef_t  *d = NULL;
 
@@ -221,16 +287,18 @@ cs_xdef_boundary_create(cs_xdef_type_t    type,
   d->z_id = z_id;
   d->state = state;
   d->meta = meta;
-  d->qtype = CS_QUADRATURE_BARY; // default value
+  d->qtype = CS_QUADRATURE_BARY; /* default value */
 
   switch (type) {
 
   case CS_XDEF_BY_VALUE:
     {
-      double  *_input = (double *)input;
-      BFT_MALLOC(d->input, dim, double);
-      double  *_input_cpy = (double *)d->input;
-      for (int i = 0; i < dim; i++) _input_cpy[i] = _input[i];
+      double  *_context = (double *)context;
+
+      BFT_MALLOC(d->context, dim, double);
+
+      double  *_context_cpy = (double *)d->context;
+      for (int i = 0; i < dim; i++) _context_cpy[i] = _context[i];
 
       /* Update state flag */
       d->state |= CS_FLAG_STATE_UNIFORM | CS_FLAG_STATE_FACEWISE;
@@ -239,28 +307,48 @@ cs_xdef_boundary_create(cs_xdef_type_t    type,
 
   case CS_XDEF_BY_ANALYTIC_FUNCTION:
     {
-      cs_xdef_analytic_input_t  *a = (cs_xdef_analytic_input_t *)input;
-      cs_xdef_analytic_input_t  *b = NULL;
+      cs_xdef_analytic_context_t  *a = (cs_xdef_analytic_context_t *)context;
+      cs_xdef_analytic_context_t  *b = NULL;
 
-      BFT_MALLOC(b, 1, cs_xdef_analytic_input_t);
+      BFT_MALLOC(b, 1, cs_xdef_analytic_context_t);
+      assert(a->z_id == z_id);
+      b->z_id = a->z_id;
       b->func = a->func;
       b->input = a->input;
+      b->free_input = a->free_input;
 
-      d->input = b;
+      d->context = b;
+    }
+    break;
+
+  case CS_XDEF_BY_DOF_FUNCTION:
+    {
+      cs_xdef_dof_context_t  *a = (cs_xdef_dof_context_t *)context;
+      cs_xdef_dof_context_t  *b = NULL;
+
+      BFT_MALLOC(b, 1, cs_xdef_dof_context_t);
+      b->func = a->func;
+      b->loc = a->loc;
+      b->input = a->input;
+      b->free_input = a->free_input;
+
+      d->context = b;
     }
     break;
 
   case CS_XDEF_BY_ARRAY:
     {
-      cs_xdef_array_input_t  *a = (cs_xdef_array_input_t *)input;
-      cs_xdef_array_input_t  *b = NULL;
+      cs_xdef_array_context_t  *a = (cs_xdef_array_context_t *)context;
+      cs_xdef_array_context_t  *b = NULL;
 
-      BFT_MALLOC(b, 1, cs_xdef_array_input_t);
+      BFT_MALLOC(b, 1, cs_xdef_array_context_t);
       b->stride = a->stride;
       b->loc = a->loc;
       b->values = a->values;
+      b->is_owner = a->is_owner;
+      b->index = a->index;
 
-      d->input = b;
+      d->context = b;
 
       /* Update state flag */
       if (cs_flag_test(b->loc, cs_flag_primal_face))
@@ -270,26 +358,41 @@ cs_xdef_boundary_create(cs_xdef_type_t    type,
 
   case CS_XDEF_BY_FIELD:
     {
-      cs_field_t  *f = (cs_field_t *)input;
+      cs_field_t  *f = (cs_field_t *)context;
+      assert(f != NULL);
+      d->context = f;
 
-      d->input = &(f->id);
+      const cs_mesh_location_type_t  loc_type =
+        cs_mesh_location_get_type(f->location_id);
+
+      /* Update flags */
+      if (loc_type == CS_MESH_LOCATION_BOUNDARY_FACES) {
+        d->meta |= CS_FLAG_FULL_LOC;
+        d->state |= CS_FLAG_STATE_FACEWISE;
+      }
+      else
+        bft_error(__FILE__, __LINE__, 0,
+                  " %s: Definition by field on the boundary rely on a mesh"
+                  " location defined at boundary faces.", __func__);
     }
     break;
 
   case CS_XDEF_BY_QOV:
     {
-      double  *_input = (double *)input;
-      BFT_MALLOC(d->input, 1, double);
-      double  *_input_cpy = (double *)d->input;
-      _input_cpy[0] = _input[0];
+      double  *_context = (double *)context;
+
+      BFT_MALLOC(d->context, 1, double);
+
+      double  *_context_cpy = (double *)d->context;
+      _context_cpy[0] = _context[0];
 
       /* Update state flag */
       d->state |= CS_FLAG_STATE_UNIFORM | CS_FLAG_STATE_FACEWISE;
     }
     break;
 
-  default: // analytic functions or more generic functions
-    d->input = input;
+  default: /* analytic functions or more generic functions */
+    d->context = context;
     break;
 
   }
@@ -305,7 +408,7 @@ cs_xdef_boundary_create(cs_xdef_type_t    type,
  * \param[in]  type       type of definition
  * \param[in]  state      flag to know if this uniform, cellwise, steady...
  * \param[in]  meta       metadata associated to this description
- * \param[in]  input      pointer to a structure storing the parameters (cast
+ * \param[in]  context    pointer to a structure storing the parameters (cast
  *                        on-the-fly according to the type of definition)
  *
  * \return a pointer to the new cs_xdef_t structure
@@ -313,10 +416,10 @@ cs_xdef_boundary_create(cs_xdef_type_t    type,
 /*----------------------------------------------------------------------------*/
 
 cs_xdef_t *
-cs_xdef_timestep_create(cs_xdef_type_t             type,
-                        cs_flag_t                  state,
-                        cs_flag_t                  meta,
-                        void                      *input)
+cs_xdef_timestep_create(cs_xdef_type_t       type,
+                        cs_flag_t            state,
+                        cs_flag_t            meta,
+                        void                *context)
 {
   cs_xdef_t  *d = NULL;
 
@@ -325,40 +428,44 @@ cs_xdef_timestep_create(cs_xdef_type_t             type,
   d->type = type;
   d->support = CS_XDEF_SUPPORT_TIME;
   d->dim = 1;
-  d->z_id = -1; // No associated zone
+  d->z_id = -1;                  /* no associated zone */
   d->state = state;
   d->meta = meta;
-  d->qtype = CS_QUADRATURE_NONE; // default value
+  d->qtype = CS_QUADRATURE_NONE; /* default value */
 
   switch (type) {
 
   case CS_XDEF_BY_VALUE:
     {
-      double  *_input = (double *)input;
-      BFT_MALLOC(d->input, 1, double);
-      double  *_input_cpy = (double *)d->input;
-      _input_cpy[0] = _input[0];
+      double  *_context = (double *)context;
+
+      BFT_MALLOC(d->context, 1, double);
+
+      double  *_context_cpy = (double *)d->context;
+      _context_cpy[0] = _context[0];
 
       /* Update state flag */
       d->state |= CS_FLAG_STATE_UNIFORM | CS_FLAG_STATE_STEADY;
     }
     break;
 
-  case  CS_XDEF_BY_TIME_FUNCTION:
+  case CS_XDEF_BY_TIME_FUNCTION:
     {
-      cs_xdef_timestep_input_t  *a = (cs_xdef_timestep_input_t *)input;
-      cs_xdef_timestep_input_t  *b = NULL;
+      cs_xdef_time_func_context_t *a = (cs_xdef_time_func_context_t *)context;
+      cs_xdef_time_func_context_t *b = NULL;
 
-      BFT_MALLOC(b, 1, cs_xdef_timestep_input_t);
+      BFT_MALLOC(b, 1, cs_xdef_time_func_context_t);
       b->func = a->func;
       b->input = a->input;
+      b->free_input = a->free_input;
 
-      d->input = b;
+      d->state |= CS_FLAG_STATE_UNIFORM;
+      d->context = b;
     }
     break;
 
   default:
-    d->input = input;
+    d->context = context;
     break;
   }
 
@@ -381,37 +488,251 @@ cs_xdef_free(cs_xdef_t     *d)
   if (d == NULL)
     return d;
 
-  if (d->type == CS_XDEF_BY_ARRAY) {
-    if (d->state & CS_FLAG_STATE_OWNER) {
+  switch (d->type) {
 
-      cs_xdef_array_input_t  *a = (cs_xdef_array_input_t *)d->input;
-      BFT_FREE(a->values);
-      BFT_FREE(d->input);
+  case CS_XDEF_BY_ARRAY:
+    {
+      cs_xdef_array_context_t  *a = (cs_xdef_array_context_t *)d->context;
+      if (a->is_owner)
+        BFT_FREE(a->values);
+      BFT_FREE(d->context);
     }
-  }
+    break;
 
-  if (d->type == CS_XDEF_BY_VALUE ||
-      d->type == CS_XDEF_BY_ANALYTIC_FUNCTION ||
-      d->type == CS_XDEF_BY_QOV)
-    BFT_FREE(d->input);
+  case CS_XDEF_BY_ANALYTIC_FUNCTION:
+    {
+      cs_xdef_analytic_context_t *c = (cs_xdef_analytic_context_t *)d->context;
+
+      if (c->free_input != NULL)
+        c->input = c->free_input(c->input);
+
+      BFT_FREE(d->context);
+    }
+    break;
+
+  case CS_XDEF_BY_DOF_FUNCTION:
+    {
+      cs_xdef_dof_context_t *c = (cs_xdef_dof_context_t *)d->context;
+
+      if (c->free_input != NULL)
+        c->input = c->free_input(c->input);
+
+      BFT_FREE(d->context);
+    }
+    break;
+
+  case CS_XDEF_BY_TIME_FUNCTION:
+    {
+      cs_xdef_time_func_context_t *c =
+        (cs_xdef_time_func_context_t *)d->context;
+
+      if (c->free_input != NULL)
+        c->input = c->free_input(c->input);
+
+      BFT_FREE(d->context);
+    }
+    break;
+
+  case CS_XDEF_BY_VALUE:
+  case CS_XDEF_BY_QOV:
+    BFT_FREE(d->context);
+    break;
+
+  default:
+    break; /* Nothing special to do */
+  }
 
   BFT_FREE(d);
 
   return NULL;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  copy a cs_xdef_t structure
+ *
+ * \param[in]  src    pointer to a cs_xdef_t structure to copy
+ *
+ * \return a pointer to a new allocated cs_xdef_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_xdef_t *
+cs_xdef_copy(cs_xdef_t     *src)
+{
+  cs_xdef_t  *cpy = NULL;
+  if (src == NULL)
+    return cpy;
+
+  /* In the case of a definition by array where the structure is not owner
+     one sets the copy to be owner of the array in order to avoid a memory
+     leak */
+
+  switch (src->support) {
+
+  case CS_XDEF_SUPPORT_VOLUME:
+    cpy = cs_xdef_volume_create(src->type,
+                                src->dim,
+                                src->z_id,
+                                src->state,
+                                src->meta,
+                                src->context);
+    break;
+
+  case CS_XDEF_SUPPORT_TIME:
+    cpy = cs_xdef_timestep_create(src->type,
+                                  src->state,
+                                  src->meta,
+                                  src->context);
+    break;
+
+  case CS_XDEF_SUPPORT_BOUNDARY:
+    cpy = cs_xdef_boundary_create(src->type,
+                                  src->dim,
+                                  src->z_id,
+                                  src->state,
+                                  src->meta,
+                                  src->context);
+    break;
+
+  default:
+    bft_error(__FILE__, __LINE__, 0, " %s: Invalid case", __func__);
+
+  }
+
+  cpy->qtype = src->qtype;
+
+  return cpy;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief In the case of a definition by an analytic function, a time function
+ *        or a function relying on degrees of freedom (DoFs), this function
+ *        allows one to set a more or less complex input data structure.  This
+ *        call should be done before the first evaluation call of the
+ *        associated cs_xdef_t structure.
+ *
+ * \param[in, out]  d         pointer to a cs_xdef_t structure
+ * \param[in]       input     pointer to an input structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_xdef_set_input_context(cs_xdef_t       *d,
+                          void            *input)
+{
+  if (d == NULL)
+    return;
+
+  switch (d->type) {
+
+  case CS_XDEF_BY_ANALYTIC_FUNCTION:
+    {
+      cs_xdef_analytic_context_t *c = (cs_xdef_analytic_context_t *)d->context;
+
+      c->input = input;
+    }
+    break;
+
+  case CS_XDEF_BY_DOF_FUNCTION:
+    {
+      cs_xdef_dof_context_t *c = (cs_xdef_dof_context_t *)d->context;
+
+      c->input = input;
+    }
+    break;
+
+  case CS_XDEF_BY_TIME_FUNCTION:
+    {
+      cs_xdef_time_func_context_t *c =
+        (cs_xdef_time_func_context_t *)d->context;
+
+      c->input = input;
+    }
+    break;
+
+  default:
+    cs_base_warn(__FILE__, __LINE__);
+    cs_log_printf(CS_LOG_DEFAULT,
+                  " %s: Setting a free input function is ignored.\n"
+                  " The type of definition is not compatible.", __func__);
+    break; /* Nothing special to do */
+
+  } /* End of switch */
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief In case of a definition by an analytic function, a time function or a
+ *        function relying on degrees of freedom (DoFs). One can set a function
+ *        to free a complex input data structure (please refer to \ref
+ *        cs_xdef_free_input_t) for more details.
+ *
+ * \param[in, out]  d             pointer to a cs_xdef_t structure
+ * \param[in]       free_input    pointer to a function which free the input
+ *                                structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_xdef_set_free_input_function(cs_xdef_t               *d,
+                                cs_xdef_free_input_t    *free_input)
+{
+  if (d == NULL)
+    return;
+
+  switch (d->type) {
+
+  case CS_XDEF_BY_ANALYTIC_FUNCTION:
+    {
+      cs_xdef_analytic_context_t *c = (cs_xdef_analytic_context_t *)d->context;
+
+      c->free_input = free_input;
+    }
+    break;
+
+  case CS_XDEF_BY_DOF_FUNCTION:
+    {
+      cs_xdef_dof_context_t *c = (cs_xdef_dof_context_t *)d->context;
+
+      c->free_input = free_input;
+    }
+    break;
+
+  case CS_XDEF_BY_TIME_FUNCTION:
+    {
+      cs_xdef_time_func_context_t *c =
+        (cs_xdef_time_func_context_t *)d->context;
+
+      c->free_input = free_input;
+    }
+    break;
+
+  default:
+    cs_base_warn(__FILE__, __LINE__);
+    cs_log_printf(CS_LOG_DEFAULT,
+                  " %s: Setting a free input function is ignored.\n"
+                  " The type of definition is not compatible.", __func__);
+    break; /* Nothing special to do */
+
+  } /* End of switch */
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  In case of definition by array, set the array after having added
  *         this definition
  *
- * \param[in, out]  d       pointer to a cs_xdef_t structure
- * \param[in]       array   values
+ * \param[in, out]  d          pointer to a cs_xdef_t structure
+ * \param[in]       is_owner   manage or not the lifecycle of the array values
+ * \param[in]       array      values
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_xdef_set_array(cs_xdef_t     *d,
+                  bool           is_owner,
                   cs_real_t     *array)
 {
   if (d == NULL)
@@ -422,9 +743,15 @@ cs_xdef_set_array(cs_xdef_t     *d,
               "%s: The given cs_xdef_t structure should be defined by array.",
               __func__);
 
-  cs_xdef_array_input_t  *ai = (cs_xdef_array_input_t *)d->input;
+  cs_xdef_array_context_t  *a = (cs_xdef_array_context_t *)d->context;
 
-  ai->values = array;
+  /* An array is already assigned and one manages the lifecycle */
+  if (a->is_owner && a->values != NULL)
+    BFT_FREE(a->values);
+
+  /* Set the new values */
+  a->is_owner = is_owner;
+  a->values = array;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -449,7 +776,7 @@ cs_xdef_set_array_index(cs_xdef_t     *d,
               "%s: The given cs_xdef_t structure should be defined by array.",
               __func__);
 
-  cs_xdef_array_input_t  *ai = (cs_xdef_array_input_t *)d->input;
+  cs_xdef_array_context_t  *ai = (cs_xdef_array_context_t *)d->context;
 
   ai->index = array_index;
 }
@@ -534,14 +861,16 @@ cs_xdef_get_state_flag(const cs_xdef_t     *d)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Output the settings related to a a cs_xdef_t structure
+ * \brief  Output the settings related to a cs_xdef_t structure
  *
- * \param[in] d    pointer to a cs_xdef_t structure
+ * \param[in] prefix    optional string
+ * \param[in] d         pointer to a cs_xdef_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_xdef_log(cs_xdef_t     *d)
+cs_xdef_log(const char          *prefix,
+            const cs_xdef_t     *d)
 {
   if (d == NULL)
     return;
@@ -551,103 +880,143 @@ cs_xdef_log(cs_xdef_t     *d)
   if (d->state & CS_FLAG_STATE_STEADY)   is_steady = true;
   if (d->state & CS_FLAG_STATE_CELLWISE) is_cellwise = true;
 
+  const char  *_p;
+  const char _empty_prefix[2] = "";
+  if (prefix == NULL)
+    _p = _empty_prefix;
+  else
+    _p = prefix;
+
   cs_log_printf(CS_LOG_SETUP,
-                " <Definition> uniform [%s], cellwise [%s], steady [%s],"
-                " meta: %u\n",
-                cs_base_strtf(is_uniform), cs_base_strtf(is_cellwise),
+                "%s | Uniform %s Cellwise %s Steady %s Meta: %u\n",
+                _p, cs_base_strtf(is_uniform), cs_base_strtf(is_cellwise),
                 cs_base_strtf(is_steady), d->meta);
 
+  /* Which support */
+  /* ============= */
+
   if (d->support == CS_XDEF_SUPPORT_VOLUME) {
 
     const cs_zone_t  *z = cs_volume_zone_by_id(d->z_id);
     assert(z != NULL);
-    cs_log_printf(CS_LOG_SETUP,
-                  " <Definition> support: volume, zone: %d, %s,"
-                  " mesh_location: %s\n",
-                  z->id, z->name, cs_mesh_location_get_name(z->location_id));
+    cs_log_printf(CS_LOG_SETUP, "%s | Support:   volume | Zone: %s (id:%5d)\n",
+                  _p, z->name, z->id);
 
   }
   else if (d->support == CS_XDEF_SUPPORT_BOUNDARY) {
 
     const cs_zone_t  *z = cs_boundary_zone_by_id(d->z_id);
     assert(z != NULL);
-    cs_log_printf(CS_LOG_SETUP,
-                  " <Definition> support: boundary, zone: %d, %s,"
-                  " mesh_location: %s\n",
-                  z->id, z->name, cs_mesh_location_get_name(z->location_id));
+    cs_log_printf(CS_LOG_SETUP, "%s | Support: boundary | Zone: %s (id:%5d)\n",
+                  _p, z->name, z->id);
 
   }
   else if (d->support == CS_XDEF_SUPPORT_TIME)
-    cs_log_printf(CS_LOG_SETUP, " <Definition> support: time\n");
+    cs_log_printf(CS_LOG_SETUP, "%s | Support: time\n", _p);
+
+  /* Type of definition */
+  /* ================== */
 
   switch (d->type) {
 
   case CS_XDEF_BY_ANALYTIC_FUNCTION:
-    cs_log_printf(CS_LOG_SETUP, "              by an analytical function\n");
+    cs_log_printf(CS_LOG_SETUP, "%s | Definition by an analytical function\n",
+                  _p);
+    break;
+
+  case CS_XDEF_BY_DOF_FUNCTION:
+    cs_log_printf(CS_LOG_SETUP, "%s | Definition by a DoF function\n", _p);
     break;
 
   case CS_XDEF_BY_ARRAY:
-    cs_log_printf(CS_LOG_SETUP, "              by an array\n");
+    cs_log_printf(CS_LOG_SETUP, "%s | Definition by an array\n", _p);
     break;
 
   case CS_XDEF_BY_FIELD:
     {
-      cs_field_t  *f = (cs_field_t *)d->input;
+      cs_field_t  *f = (cs_field_t *)d->context;
 
-      cs_log_printf(CS_LOG_SETUP, "              by the field %s\n",
-                    f->name);
+      if (f == NULL)
+        bft_error(__FILE__, __LINE__, 0,
+                  " Field pointer is set to NULL in a definition by field");
+
+      cs_log_printf(CS_LOG_SETUP, "%s | Definition by the field %s\n",
+                    _p, f->name);
     }
     break;
 
   case CS_XDEF_BY_FUNCTION:
-    cs_log_printf(CS_LOG_SETUP, "              by function\n");
+    cs_log_printf(CS_LOG_SETUP, "%s | Definition by function\n", _p);
     break;
 
   case CS_XDEF_BY_QOV:
-    cs_log_printf(CS_LOG_SETUP, "              by quantity over a volume\n");
+    cs_log_printf(CS_LOG_SETUP,
+                  "%s | Definition by a quantity over a volume\n", _p);
+    break;
+
+  case CS_XDEF_BY_SUB_DEFINITIONS:
+    cs_log_printf(CS_LOG_SETUP, "%s | Definition by sub-definitions\n", _p);
     break;
 
   case CS_XDEF_BY_TIME_FUNCTION:
-    cs_log_printf(CS_LOG_SETUP, "              by time function\n");
+    cs_log_printf(CS_LOG_SETUP, "%s | Definition by a time function\n", _p);
     break;
 
   case CS_XDEF_BY_VALUE:
     {
-      cs_real_t *values = (cs_real_t *)d->input;
+      cs_real_t *values = (cs_real_t *)d->context;
 
       if (d->dim == 1)
-        cs_log_printf(CS_LOG_SETUP, "              by_value, % 5.3e\n",
-                      values[0]);
+        cs_log_printf(CS_LOG_SETUP, "%s | Definition by_value: % 5.3e\n",
+                      _p, values[0]);
       else if (d->dim == 3)
-        cs_log_printf(CS_LOG_SETUP, "              by_value,"
-                      " (% 5.3e, % 5.3e, % 5.3e)\n",
-                      values[0], values[1], values[2]);
+        cs_log_printf(CS_LOG_SETUP, "%s | Definition by_value:"
+                      " [% 5.3e, % 5.3e, % 5.3e]\n",
+                      _p, values[0], values[1], values[2]);
       else if (d->dim == 9)
-        cs_log_printf(CS_LOG_SETUP, "              by_value,"
-                      " ((% 4.2e, % 4.2e, % 4.2e) (% 4.2e, % 4.2e, % 4.2e)"
-                      " (% 4.2e, % 4.2e, % 4.2e))\n",
-                      values[0], values[1], values[2],
-                      values[3], values[4], values[5],
-                      values[6], values[7], values[8]);
+        cs_log_printf(CS_LOG_SETUP, "%s | Definition by_value:"
+                      " [[% 4.2e, % 4.2e, % 4.2e], [% 4.2e, % 4.2e, % 4.2e],"
+                      " [% 4.2e, % 4.2e, % 4.2e]]\n",
+                      _p, values[0], values[1], values[2], values[3], values[4],
+                      values[5], values[6], values[7], values[8]);
       else
         bft_error(__FILE__, __LINE__, 0,
-                  " Invalid case. dim = %d (expected 3, 6 or 9)\n", d->dim);
+                  " %s: Invalid case. dim = %d (expected 3, 6 or 9)\n",
+                  __func__, d->dim);
     }
-    break; // BY_VALUE
-
+    break; /* BY_VALUE */
 
   default:
-    bft_error(__FILE__, __LINE__, 0,
-              _(" Invalid type of description."));
+    bft_error(__FILE__, __LINE__, 0, _("%s: Invalid type of description."),
+              __func__);
     break;
 
   } /* switch on def_type */
 
-  cs_log_printf(CS_LOG_SETUP, " <Definition/Quadrature> %s\n",
-                cs_quadrature_get_type_name(d->qtype));
+  cs_log_printf(CS_LOG_SETUP, "%s | Quadrature: %s\n",
+                _p, cs_quadrature_get_type_name(d->qtype));
 
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve a pointer to the cs_xdef_type's name string
+ *
+ * \param[in] xdef_type  type to query
+ *
+ * \return a pointer to mathing name string
+ */
+/*----------------------------------------------------------------------------*/
+
+const char *
+cs_xdef_type_get_name(cs_xdef_type_t  xdef_type)
+{
+  if (xdef_type < 0 || xdef_type >= CS_N_XDEF_TYPES)
+    xdef_type = CS_N_XDEF_TYPES;
+
+  return _xdef_type_name[xdef_type];
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
