@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -35,7 +35,7 @@
 
 #include "cs_base.h"
 #include "cs_boundary_zone.h"
-#include "cs_param.h"
+#include "cs_param_types.h"
 #include "cs_quadrature.h"
 #include "cs_volume_zone.h"
 
@@ -51,15 +51,36 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Destroy an input data structure.
+ *         Complex data structure can be used when a \ref cs_xdef_t structure
+ *         is defined by an analytic function, a DoF function or a time
+ *         function. Please refer to \ref cs_xdef_analytic_context_t,
+ *         \ref cs_xdef_time_func_context_t or \ref cs_xdef_dof_context_t
+ *
+ * \param[in, out]  input   pointer to an input structure associated to a
+ *                          context structure
+ *
+ * \return a NULL pointer
+ */
+/*----------------------------------------------------------------------------*/
+
+typedef void *
+(cs_xdef_free_input_t)(void   *input);
+
 /*!
  * \enum cs_xdef_type_t
  *
  * \var CS_XDEF_BY_ANALYTIC_FUNCTION
- * Definition relying on an analytic function (see \ref cs_analytic_func_t)
+ * Definition relying on a \ref cs_analytic_func_t function pointer
  *
  * \var CS_XDEF_BY_ARRAY
  * Definition based on an array
  *
+ * \var CS_XDEF_BY_DOF_FUNCTION
+ * Definition relying on a \ref cs_dof_func_t function pointer
+ *
  * \var CS_XDEF_BY_FIELD
  * Definition based on a field (see \ref cs_field_t)
  *
@@ -71,9 +92,14 @@ BEGIN_C_DECLS
  * Definition which enables to spread a given quantity inside a volume.
  * Useful to initialized a tracer in a subdomain for instance.
  *
+ * \var CS_XDEF_BY_SUB_DEFINITIONS
+ * Definition relying on a combination of other CS_XDEF_***
+ * This kind of definition is useful for the definition of a property
+ * as the product of two existing ones.
+ *
  * \var CS_XDEF_BY_TIME_FUNCTION
  * Definition relying on a function for setting the time step (see
- * \ref cs_timestep_func_t)
+ * \ref cs_time_func_t)
  *
  * \var CS_XDEF_BY_VALUE
  * Simple definition by a constant value
@@ -83,9 +109,11 @@ typedef enum {
 
   CS_XDEF_BY_ANALYTIC_FUNCTION,
   CS_XDEF_BY_ARRAY,
+  CS_XDEF_BY_DOF_FUNCTION,
   CS_XDEF_BY_FIELD,
   CS_XDEF_BY_FUNCTION,
   CS_XDEF_BY_QOV,
+  CS_XDEF_BY_SUB_DEFINITIONS,
   CS_XDEF_BY_TIME_FUNCTION,
   CS_XDEF_BY_VALUE,
 
@@ -150,10 +178,11 @@ typedef struct {
    * type of quadrature to use for evaluating the description (see
    * \ref cs_quadrature_type_t)
    *
-   * \var input
+   * \var context
    * Pointer to a structure cast on-the-fly according to the type of description
-   * May be set to NULL or \ref cs_xdef_array_input_t or
-   * \ref cs_xdef_analytic_input_t or \ref cs_xdef_timestep_input_t
+   * May be set to NULL or \ref cs_xdef_array_context_t or
+   * \ref cs_xdef_analytic_context_t or \ref cs_xdef_time_func_context_t or
+   * \ref cs_xdef_dof_context_t
    */
 
   int                    dim;
@@ -166,75 +195,139 @@ typedef struct {
 
   cs_quadrature_type_t   qtype;
 
-  void                  *input;
+  void                  *context;
 
 } cs_xdef_t;
 
 /*!
- * \struct cs_xdef_array_input_t
- * \brief Input structure when an array is used for the definition
+ * \struct cs_xdef_array_context_t
+ * \brief Context structure when an array is used for the definition
  */
 
 typedef struct {
 
-  /*! * \var stride
-   * stride to access the array values
+  /*!
+   * \var z_id
+   * id related to a zone (volume or boundary) for this definition
+   *
+   * \var stride
+   * Stride to access the array values
    *
    * \var loc
-   * flag to know where are defined array values
+   * Flag to know where are defined array values
    *
    * \var values
-   * array values
+   * Array values
    *
    * \var index
-   * optional index for accessing to the values
+   * Optional index for accessing to the values. One assumes that the lifecycle
+   * of this buffer is managed outside (pointer to a cs_adjacency_t stored
+   * either in the \ref cs_cdo_connect_t struct. or the \ref cs_mesh_t struct.
+   *
+   * \var is_owner
+   * If true the lifecycle of the values is managed by the cs_xdef_t structure.
+   * Otherwise, the lifecycle is managed by the calling code.
    */
 
+  int           z_id;
   int           stride;
   cs_flag_t     loc;
   cs_real_t    *values;
   cs_lnum_t    *index;
+  bool          is_owner;
 
-} cs_xdef_array_input_t;
+} cs_xdef_array_context_t;
 
 /*!
- * \struct cs_xdef_analytic_input_t
- * \brief Input structure when an analytic function is used for the definition
+ * \struct cs_xdef_analytic_context_t
+ * \brief Context structure when a definition by analytic function is used
  */
 
 typedef struct {
 
-  /*! \var input
-   * NULL or pointer to a structure cast on-the-fly for additional information
-   * used in the function
+  /*! \var z_id
+   * id related to a zone (volume or boundary) for this definition
    */
-  void                *input;
+  int                    z_id;
 
   /*! \var func
-   * \ref cs_analytic_func_t to call
+   * pointer to a \ref cs_analytic_func_t to call
    */
-  cs_analytic_func_t  *func;
-
-} cs_xdef_analytic_input_t;
-
-/*!
- * \struct cs_xdef_timestep_input_t
- * \brief Input structure when a time step function is used for the definition
- */
-
-typedef struct {
+  cs_analytic_func_t    *func;
 
   /*! \var input
    * NULL or pointer to a structure cast on-the-fly for additional information
    * used in the function
-   *
-   * \var func
-   * \ref cs_timestep_func_t to call
    */
-  void                *input;
-  cs_timestep_func_t  *func;
+  void                  *input;
 
-} cs_xdef_timestep_input_t;
+  /*! \var free_input
+   * NULL or pointer to a function to free a given input structure
+   */
+  cs_xdef_free_input_t  *free_input;
+
+} cs_xdef_analytic_context_t;
+
+/*!
+ * \struct cs_xdef_dof_context_t
+ * \brief Context structure when a definition by DoF function is used
+ */
+
+typedef struct {
+
+  /*! \var z_id
+   * id related to a zone (volume or boundary) for this definition
+   */
+  int                    z_id;
+
+  /*! \var loc
+   *  Flag to know which type of entities are given as parameter in the
+   *  \ref cs_dof_func_t
+   */
+  cs_flag_t              loc;
+
+  /*! \var func
+   * pointer to a \ref cs_dof_func_t to call
+   */
+  cs_dof_func_t         *func;
+
+  /*! \var input
+   * NULL or pointer to a structure cast on-the-fly for additional information
+   * used in the function
+   */
+  void                  *input;
+
+  /*! \var free_input
+   * NULL or pointer to a function to free a given input structure
+   */
+  cs_xdef_free_input_t  *free_input;
+
+} cs_xdef_dof_context_t;
+
+/*!
+ * \struct cs_xdef_time_func_context_t
+ * \brief Context structure when a time step function is used for the definition
+ */
+
+typedef struct {
+
+  /*! \var func
+   * pointer to a \ref cs_time_func_t to call
+   */
+  cs_time_func_t        *func;
+
+  /*! \var input
+   * NULL or pointer to a structure cast on-the-fly for additional information
+   * used in the function
+   */
+  void                  *input;
+
+  /*! \var free_input
+   * NULL or pointer to a function to free a given input structure
+   */
+  cs_xdef_free_input_t  *free_input;
+
+} cs_xdef_time_func_context_t;
 
 /*============================================================================
  * Public function prototypes
@@ -288,6 +381,52 @@ cs_get_bdy_zone_id(const char   *z_name)
   return z_id;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve the value associated to the given definition.
+ *         This should be a definition by value and the dimension should be
+ *         equal to one.
+ *
+ * \param[in]  def    pointer to a cs_xdef_t structure
+ *
+ * \return the value of the definition
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_real_t
+cs_xdef_get_scalar_value(cs_xdef_t     *def)
+{
+  assert(def != NULL);
+  assert(def->dim == 1);
+  assert(def->type == CS_XDEF_BY_VALUE);
+
+  cs_real_t  *value = (cs_real_t *)def->context;
+
+  return value[0];
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve the values associated to the given definition.
+ *         This should be a definition by array
+ *
+ * \param[in]  def    pointer to a cs_xdef_t structure
+ *
+ * \return the pointer to the array of values
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_real_t *
+cs_xdef_get_array(cs_xdef_t     *def)
+{
+  assert(def != NULL);
+  assert(def->type == CS_XDEF_BY_ARRAY);
+
+  cs_xdef_array_context_t  *ai = (cs_xdef_array_context_t *)def->context;
+
+  return ai->values;
+}
+
 /*============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -297,24 +436,24 @@ cs_get_bdy_zone_id(const char   *z_name)
  * \brief  Allocate and initialize a new cs_xdef_t structure based on volumic
  *         elements
  *
- * \param[in]  type       type of definition
- * \param[in]  dim        dimension of the values to define
- * \param[in]  z_id       volume zone id
- * \param[in]  state      flag to know if this uniform, cellwise, steady...
- * \param[in]  meta       metadata associated to this description
- * \param[in]  input      pointer to a structure
+ * \param[in]  type        type of definition
+ * \param[in]  dim         dimension of the values to define
+ * \param[in]  z_id        volume zone id
+ * \param[in]  state       flag to know if this uniform, cellwise, steady...
+ * \param[in]  meta        metadata associated to this description
+ * \param[in]  context     pointer to a structure
  *
  * \return a pointer to the new cs_xdef_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 cs_xdef_t *
-cs_xdef_volume_create(cs_xdef_type_t    type,
-                      int               dim,
-                      int               z_id,
-                      cs_flag_t         state,
-                      cs_flag_t         meta,
-                      void             *input);
+cs_xdef_volume_create(cs_xdef_type_t           type,
+                      int                      dim,
+                      int                      z_id,
+                      cs_flag_t                state,
+                      cs_flag_t                meta,
+                      void                    *context);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -326,7 +465,7 @@ cs_xdef_volume_create(cs_xdef_type_t    type,
  * \param[in]  z_id       volume zone id
  * \param[in]  state      flag to know if this uniform, cellwise, steady...
  * \param[in]  meta       metadata associated to this description
- * \param[in]  input      pointer to a structure
+ * \param[in]  context    pointer to a structure
  *
  * \return a pointer to the new cs_xdef_t structure
  */
@@ -338,7 +477,7 @@ cs_xdef_boundary_create(cs_xdef_type_t    type,
                         int               z_id,
                         cs_flag_t         state,
                         cs_flag_t         meta,
-                        void             *input);
+                        void             *context);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -348,7 +487,7 @@ cs_xdef_boundary_create(cs_xdef_type_t    type,
  * \param[in]  type       type of definition
  * \param[in]  state      flag to know if this uniform, cellwise, steady...
  * \param[in]  meta       metadata associated to this description
- * \param[in]  input      pointer to a structure storing the parameters (cast
+ * \param[in]  context    pointer to a structure storing the parameters (cast
  *                        on-the-fly according to the type of definition)
  *
  * \return a pointer to the new cs_xdef_t structure
@@ -356,10 +495,10 @@ cs_xdef_boundary_create(cs_xdef_type_t    type,
 /*----------------------------------------------------------------------------*/
 
 cs_xdef_t *
-cs_xdef_timestep_create(cs_xdef_type_t             type,
-                        cs_flag_t                  state,
-                        cs_flag_t                  meta,
-                        void                      *input);
+cs_xdef_timestep_create(cs_xdef_type_t       type,
+                        cs_flag_t            state,
+                        cs_flag_t            meta,
+                        void                *context);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -387,6 +526,40 @@ cs_xdef_free(cs_xdef_t     *d);
 cs_xdef_t *
 cs_xdef_copy(cs_xdef_t     *src);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief In the case of a definition by an analytic function, a time function
+ *        or a function relying on degrees of freedom (DoFs), this function
+ *        allows one to set a more or less complex input data structure.  This
+ *        call should be done before the first evaluation call of the
+ *        associated cs_xdef_t structure.
+ *
+ * \param[in, out]  d         pointer to a cs_xdef_t structure
+ * \param[in]       input     pointer to an input structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_xdef_set_input_context(cs_xdef_t       *d,
+                          void            *input);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief In case of a definition by an analytic function, a time function or a
+ *        function relying on degrees of freedom (DoFs). One can set a function
+ *        to free a complex input data structure (please refer to \ref
+ *        cs_xdef_free_input_t) for more details.
+ *
+ * \param[in, out]  d             pointer to a cs_xdef_t structure
+ * \param[in]       free_input    pointer to a function which free the input
+ *                                structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_xdef_set_free_input_function(cs_xdef_t               *d,
+                                cs_xdef_free_input_t    *free_input);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  In case of definition by array, set the array after having added
@@ -473,14 +646,29 @@ cs_xdef_get_state_flag(const cs_xdef_t     *d);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Output the settings related to a a cs_xdef_t structure
+ * \brief  Output the settings related to a cs_xdef_t structure
  *
- * \param[in] d    pointer to a cs_xdef_t structure
+ * \param[in] prefix    optional string
+ * \param[in] d         pointer to a cs_xdef_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_xdef_log(cs_xdef_t     *d);
+cs_xdef_log(const char          *prefix,
+            const cs_xdef_t     *d);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Retrieve a pointer to the cs_xdef_type's name string
+ *
+ * \param[in] xdef_type  type to query
+ *
+ * \return a pointer to mathing name string
+ */
+/*----------------------------------------------------------------------------*/
+
+const char *
+cs_xdef_type_get_name(cs_xdef_type_t  xdef_type);
 
 /*----------------------------------------------------------------------------*/
 
