@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -61,7 +61,7 @@ BEGIN_C_DECLS
  *
  * \param[in]  cm         pointer to a \ref cs_cell_mesh_t structure
  * \param[in]  time_eval  physical time at which one evaluates the term
- * \param[in]  input      pointer to an input structure
+ * \param[in]  context    pointer to a context structure
  * \param[out] eval       result of the evaluation
  */
 /*----------------------------------------------------------------------------*/
@@ -194,6 +194,32 @@ cs_xdef_cw_eval_vector_by_val(const cs_cell_mesh_t     *cm,
   eval[2] = constant_val[2];
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Evaluate a tensor-valued quantity with a symmetric storage by a
+ *         cellwise process
+ *
+ * \param[in]  cm         pointer to a \ref cs_cell_mesh_t structure
+ * \param[in]  time_eval  physical time at which one evaluates the term
+ * \param[in]  input      pointer to an input structure
+ * \param[out] eval       result of the evaluation
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline void
+cs_xdef_cw_eval_symtens_by_val(const cs_cell_mesh_t     *cm,
+                               cs_real_t                 time_eval,
+                               void                     *input,
+                               cs_real_t                *eval)
+{
+  CS_UNUSED(cm);
+  CS_UNUSED(time_eval);
+
+  const cs_real_t  *constant_val = (const cs_real_t *)input;
+  for (int k = 0; k < 6; k++)
+    eval[k] = constant_val[k];
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Evaluate a tensor-valued quantity by a cellwise process
@@ -281,7 +307,7 @@ cs_xdef_cw_eval_flux_by_val(const cs_cell_mesh_t     *cm,
   CS_UNUSED(time_eval);
 
   /* Sanity check */
-  assert(cs_flag_test(cm->flag, CS_CDO_LOCAL_PFQ));
+  assert(cs_eflag_test(cm->flag, CS_FLAG_COMP_PFQ));
 
   const cs_real_t  *flux = (cs_real_t *)input;
   const cs_quant_t  fq = cm->face[f];
@@ -388,8 +414,8 @@ cs_xdef_cw_eval_scalar_face_avg_by_array(const cs_cell_mesh_t       *cm,
               " %s: Array storing the evaluation should be allocated before"
               " the call to this function.", __func__);
 
-  const cs_xdef_array_input_t *array_input =
-    (const cs_xdef_array_input_t *)input;
+  const cs_xdef_array_context_t *array_input
+    = (const cs_xdef_array_context_t *)input;
 
   assert(input != NULL);
   assert(cs_flag_test(array_input->loc, cs_flag_primal_face));
@@ -423,7 +449,7 @@ cs_xdef_cw_eval_face_drhm_by_analytic(const cs_cell_mesh_t       *cm,
                                       cs_real_t                  *eval)
 {
   CS_UNUSED(qtype);
-  cs_xdef_analytic_input_t *anai = (cs_xdef_analytic_input_t *)input;
+  cs_xdef_analytic_context_t *anai = (cs_xdef_analytic_context_t *)input;
 
   anai->func(t_eval, 1, NULL, cm->face[f].center, false, anai->input, eval);
 }
@@ -497,8 +523,8 @@ cs_xdef_cw_eval_vector_face_avg_by_array(const cs_cell_mesh_t     *cm,
               " %s: Array storing the evaluation should be allocated before"
               " the call to this function.", __func__);
 
-  const cs_xdef_array_input_t *array_input =
-    (const cs_xdef_array_input_t *)input;
+  const cs_xdef_array_context_t *array_input
+    = (const cs_xdef_array_context_t *)input;
 
   assert(input != NULL);
   assert(cs_flag_test(array_input->loc, cs_flag_primal_face));
@@ -577,8 +603,8 @@ cs_xdef_cw_eval_tensor_face_avg_by_array(const cs_cell_mesh_t     *cm,
               " %s: Array storing the evaluation should be allocated before"
               " the call to this function.", __func__);
 
-  const cs_xdef_array_input_t *array_input =
-    (const cs_xdef_array_input_t *)input;
+  const cs_xdef_array_context_t *array_input
+    = (const cs_xdef_array_context_t *)input;
 
   assert(input != NULL);
   assert(cs_flag_test(array_input->loc, cs_flag_primal_face));
@@ -594,8 +620,9 @@ cs_xdef_cw_eval_tensor_face_avg_by_array(const cs_cell_mesh_t     *cm,
 /*!
  * \brief  Integrate an analytic function over a face
  *
- * \param[in]      cm       pointer to \ref cs_cell_mesh_t structure
+ * \param[in]      cm       pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      t_eval   time at which the function is evaluated
+ * \param[in]      f        face id in the local cell numbering
  * \param[in]      ana      analytic function to integrate
  * \param[in]      input    pointer to an input structure
  * \param[in]      qfunc    quadrature function to use
@@ -692,7 +719,7 @@ cs_xdef_cw_eval_scalar_face_avg_by_analytic(const cs_cell_mesh_t   *cm,
  * \param[in]      cm       pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      f        local face id
  * \param[in]      t_eval   physical time at which one evaluates the term
- * \param[in]      input    pointer to an input structure
+ * \param[in]      context  pointer to a context structure
  * \param[in]      qtype    level of quadrature to use
  * \param[in, out] eval     result of the evaluation
  */
@@ -702,7 +729,7 @@ void
 cs_xdef_cw_eval_vector_face_avg_by_analytic(const cs_cell_mesh_t    *cm,
                                             short int                f,
                                             cs_real_t                t_eval,
-                                            void                    *input,
+                                            void                    *context,
                                             cs_quadrature_type_t     qtype,
                                             cs_real_t               *eval);
 
@@ -715,7 +742,7 @@ cs_xdef_cw_eval_vector_face_avg_by_analytic(const cs_cell_mesh_t    *cm,
  * \param[in]      cm       pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      f        local face id
  * \param[in]      t_eval   physical time at which one evaluates the term
- * \param[in]      input    pointer to an input structure
+ * \param[in]      context  pointer to a context structure
  * \param[in]      qtype    level of quadrature to use
  * \param[in, out] eval     result of the evaluation
  */
@@ -725,7 +752,7 @@ void
 cs_xdef_cw_eval_tensor_face_avg_by_analytic(const cs_cell_mesh_t    *cm,
                                             short int                f,
                                             cs_real_t                t_eval,
-                                            void                    *input,
+                                            void                    *context,
                                             cs_quadrature_type_t     qtype,
                                             cs_real_t               *eval);
 
@@ -739,7 +766,7 @@ cs_xdef_cw_eval_tensor_face_avg_by_analytic(const cs_cell_mesh_t    *cm,
  * \param[in]      cm       pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      t_eval   physical time at which one evaluates the term
  * \param[in]      qtype    quadrature type
- * \param[in]      input    pointer to an input structure
+ * \param[in]      context  pointer to a context structure
  * \param[in, out] eval     result of the evaluation
  */
 /*----------------------------------------------------------------------------*/
@@ -747,7 +774,7 @@ cs_xdef_cw_eval_tensor_face_avg_by_analytic(const cs_cell_mesh_t    *cm,
 void
 cs_xdef_cw_eval_scalar_avg_by_analytic(const cs_cell_mesh_t     *cm,
                                        cs_real_t                 t_eval,
-                                       void                     *input,
+                                       void                     *context,
                                        cs_quadrature_type_t      qtype,
                                        cs_real_t                *eval);
 
@@ -759,18 +786,18 @@ cs_xdef_cw_eval_scalar_avg_by_analytic(const cs_cell_mesh_t     *cm,
  *         This evaluation hinges on the computation of integrals
  *         Vector-valued case.
  *
- * \param[in]      cm       pointer to a \ref cs_cell_mesh_t structure
- * \param[in]      t_eval   physical time at which one evaluates the term
- * \param[in]      qtype    quadrature type
- * \param[in]      input    pointer to an input structure
- * \param[in, out] eval     result of the evaluation
+ * \param[in]      cm        pointer to a \ref cs_cell_mesh_t structure
+ * \param[in]      t_eval    physical time at which one evaluates the term
+ * \param[in]      qtype     quadrature type
+ * \param[in]      context   pointer to a context structure
+ * \param[in, out] eval      result of the evaluation
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_xdef_cw_eval_vector_avg_by_analytic(const cs_cell_mesh_t     *cm,
                                        cs_real_t                 t_eval,
-                                       void                     *input,
+                                       void                     *context,
                                        cs_quadrature_type_t      qtype,
                                        cs_real_t                *eval);
 
@@ -782,37 +809,55 @@ cs_xdef_cw_eval_vector_avg_by_analytic(const cs_cell_mesh_t     *cm,
  *         This evaluation hinges on the computation of integrals
  *         Tensor-valued case.
  *
- * \param[in]      cm       pointer to a \ref cs_cell_mesh_t structure
- * \param[in]      t_eval   physical time at which one evaluates the term
- * \param[in]      qtype    quadrature type
- * \param[in]      input    pointer to an input structure
- * \param[in, out] eval     result of the evaluation
+ * \param[in]      cm        pointer to a \ref cs_cell_mesh_t structure
+ * \param[in]      t_eval    physical time at which one evaluates the term
+ * \param[in]      qtype     quadrature type
+ * \param[in]      context   pointer to a context structure
+ * \param[in, out] eval      result of the evaluation
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_xdef_cw_eval_tensor_avg_by_analytic(const cs_cell_mesh_t     *cm,
                                        cs_real_t                 t_eval,
-                                       void                     *input,
+                                       void                     *context,
                                        cs_quadrature_type_t      qtype,
                                        cs_real_t                *eval);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Evaluate a  quantity by a cellwise process using a definition by
+ *         time function
+ *
+ * \param[in]  cm          pointer to a \ref cs_cell_mesh_t structure
+ * \param[in]  time_eval   physical time at which one evaluates the term
+ * \param[in]  context     pointer to a context structure
+ * \param[out] eval        result of the evaluation
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_xdef_cw_eval_by_time_func(const cs_cell_mesh_t     *cm,
+                             cs_real_t                 time_eval,
+                             void                     *context,
+                             cs_real_t                *eval);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Evaluate a quantity defined using an analytic function by a
  *         cellwise process (usage of a \ref cs_cell_mesh_t structure)
  *
- * \param[in]      cm         pointer to a \ref cs_cell_mesh_t structure
- * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in]      input      pointer to an input structure
- * \param[in, out] eval       result of the evaluation at cell center
+ * \param[in]      cm          pointer to a \ref cs_cell_mesh_t structure
+ * \param[in]      time_eval   physical time at which one evaluates the term
+ * \param[in]      context     pointer to a context structure
+ * \param[in, out] eval        result of the evaluation at cell center
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_xdef_cw_eval_by_analytic(const cs_cell_mesh_t       *cm,
                             cs_real_t                   time_eval,
-                            void                       *input,
+                            void                       *context,
                             cs_real_t                  *eval);
 
 /*----------------------------------------------------------------------------*/
@@ -823,7 +868,7 @@ cs_xdef_cw_eval_by_analytic(const cs_cell_mesh_t       *cm,
  *
  * \param[in]      cm         pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in]      input      pointer to an input structure
+ * \param[in]      context    pointer to a context structure
  * \param[in, out] eval       result of the evaluation at cell center
  */
 /*----------------------------------------------------------------------------*/
@@ -831,7 +876,7 @@ cs_xdef_cw_eval_by_analytic(const cs_cell_mesh_t       *cm,
 void
 cs_xdef_cw_eval_by_array(const cs_cell_mesh_t      *cm,
                          cs_real_t                  time_eval,
-                         void                      *input,
+                         void                      *context,
                          cs_real_t                 *eval);
 
 /*----------------------------------------------------------------------------*/
@@ -841,16 +886,16 @@ cs_xdef_cw_eval_by_array(const cs_cell_mesh_t      *cm,
  *
  * \param[in]      cm         pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in]      input      pointer to an input structure
+ * \param[in]      context    pointer to a context structure
  * \param[in, out] eval       value of the property at the cell center
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_xdef_cw_eval_by_field(const cs_cell_mesh_t        *cm,
-                         cs_real_t                    time_eval,
-                         void                        *input,
-                         cs_real_t                   *eval);
+cs_xdef_cw_eval_by_field(const cs_cell_mesh_t      *cm,
+                         cs_real_t                  time_eval,
+                         void                      *context,
+                         cs_real_t                 *eval);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -862,7 +907,7 @@ cs_xdef_cw_eval_by_field(const cs_cell_mesh_t        *cm,
  * \param[in]      n_points   number of points where to compute the evaluation
  * \param[in]      xyz        where to compute the evaluation
  * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in]      input      pointer to an input structure
+ * \param[in]      context    pointer to a context structure
  * \param[in, out] eval       result of the evaluation
  */
 /*----------------------------------------------------------------------------*/
@@ -872,7 +917,7 @@ cs_xdef_cw_eval_at_xyz_by_analytic(const cs_cell_mesh_t       *cm,
                                    cs_lnum_t                   n_points,
                                    const cs_real_t            *xyz,
                                    cs_real_t                   time_eval,
-                                   void                       *input,
+                                   void                       *context,
                                    cs_real_t                  *eval);
 
 /*----------------------------------------------------------------------------*/
@@ -886,18 +931,18 @@ cs_xdef_cw_eval_at_xyz_by_analytic(const cs_cell_mesh_t       *cm,
  * \param[in]      n_points   number of points where to compute the evaluation
  * \param[in]      xyz        where to compute the evaluation
  * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in]      input      pointer to an input structure
+ * \param[in]      context    pointer to a context structure
  * \param[in, out] eval       result of the evaluation
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_xdef_cw_eval_vector_at_xyz_by_array(const cs_cell_mesh_t       *cm,
-                                       cs_lnum_t                   n_points,
-                                       const cs_real_t            *xyz,
-                                       cs_real_t                   time_eval,
-                                       void                       *input,
-                                       cs_real_t                  *eval);
+cs_xdef_cw_eval_vector_at_xyz_by_array(const cs_cell_mesh_t     *cm,
+                                       cs_lnum_t                 n_points,
+                                       const cs_real_t          *xyz,
+                                       cs_real_t                 time_eval,
+                                       void                     *context,
+                                       cs_real_t                *eval);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -910,7 +955,7 @@ cs_xdef_cw_eval_vector_at_xyz_by_array(const cs_cell_mesh_t       *cm,
  * \param[in]      n_points   number of points where to compute the evaluation
  * \param[in]      xyz        where to compute the evaluation
  * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in]      input      pointer to an input structure
+ * \param[in]      context    pointer to a context structure
  * \param[in, out] eval       result of the evaluation
  */
 /*----------------------------------------------------------------------------*/
@@ -920,7 +965,7 @@ cs_xdef_cw_eval_vector_at_xyz_by_field(const cs_cell_mesh_t    *cm,
                                        cs_lnum_t                n_points,
                                        const cs_real_t         *xyz,
                                        cs_real_t                time_eval,
-                                       void                    *input,
+                                       void                    *context,
                                        cs_real_t               *eval);
 
 /*----------------------------------------------------------------------------*/
@@ -933,7 +978,7 @@ cs_xdef_cw_eval_vector_at_xyz_by_field(const cs_cell_mesh_t    *cm,
  * \param[in]      cm         pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      f          local face id
  * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in]      input      pointer to an input structure
+ * \param[in]      context    pointer to a context structure
  * \param[in, out] eval       result of the evaluation (updated inside)
  */
 /*----------------------------------------------------------------------------*/
@@ -942,7 +987,7 @@ void
 cs_xdef_cw_eval_flux_at_vtx_by_val(const cs_cell_mesh_t     *cm,
                                    short int                 f,
                                    cs_real_t                 time_eval,
-                                   void                     *input,
+                                   void                     *context,
                                    cs_real_t                *eval);
 
 /*----------------------------------------------------------------------------*/
@@ -952,10 +997,10 @@ cs_xdef_cw_eval_flux_at_vtx_by_val(const cs_cell_mesh_t     *cm,
  *         portion of face related to a vertex.
  *         Use of a \ref cs_cell_mesh_t structure.
  *
- * \param[in]      cm         pointer to a \ref cs_cell_mesh_t structure
+ * \param[in]      cm         pointer to a cs_cell_mesh_t structure
  * \param[in]      f          local face id
  * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in]      input      pointer to an input structure
+ * \param[in]      context    pointer to a context structure
  * \param[in]      qtype      level of quadrature to use
  * \param[in, out] eval       result of the evaluation (updated inside)
  */
@@ -965,7 +1010,7 @@ void
 cs_xdef_cw_eval_flux_at_vtx_by_analytic(const cs_cell_mesh_t      *cm,
                                         short int                  f,
                                         cs_real_t                  time_eval,
-                                        void                      *input,
+                                        void                      *context,
                                         cs_quadrature_type_t       qtype,
                                         cs_real_t                 *eval);
 
@@ -978,7 +1023,7 @@ cs_xdef_cw_eval_flux_at_vtx_by_analytic(const cs_cell_mesh_t      *cm,
  * \param[in]      cm         pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      f          local face id
  * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in]      input      pointer to an input structure
+ * \param[in]      context    pointer to a context structure
  * \param[in]      qtype      level of quadrature to use
  * \param[in, out] eval       result of the evaluation (set inside)
  */
@@ -988,7 +1033,7 @@ void
 cs_xdef_cw_eval_flux_by_analytic(const cs_cell_mesh_t      *cm,
                                  short int                  f,
                                  cs_real_t                  time_eval,
-                                 void                      *input,
+                                 void                      *context,
                                  cs_quadrature_type_t       qtype,
                                  cs_real_t                 *eval);
 
@@ -1002,7 +1047,7 @@ cs_xdef_cw_eval_flux_by_analytic(const cs_cell_mesh_t      *cm,
  * \param[in]      cm         pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      f          local face id
  * \param[in]      time_eval  physical time at which one evaluates the term
- * \param[in]      input      pointer to an input structure
+ * \param[in]      context    pointer to a context structure
  * \param[in]      qtype      level of quadrature to use
  * \param[in, out] eval       result of the evaluation (set inside)
  */
@@ -1012,7 +1057,7 @@ void
 cs_xdef_cw_eval_tensor_flux_by_analytic(const cs_cell_mesh_t      *cm,
                                         short int                  f,
                                         cs_real_t                  time_eval,
-                                        void                      *input,
+                                        void                      *context,
                                         cs_quadrature_type_t       qtype,
                                         cs_real_t                 *eval);
 
@@ -1028,17 +1073,17 @@ cs_xdef_cw_eval_tensor_flux_by_analytic(const cs_cell_mesh_t      *cm,
  * \param[in]      cm       pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      t_eval   physical time at which one evaluates the term
  * \param[in]      qtype    quadrature type
- * \param[in]      input    pointer to an input structure
+ * \param[in]      context  pointer to a context structure
  * \param[in, out] eval     result of the evaluation
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_xdef_cw_eval_scal_avg_reduction_by_analytic(const cs_cell_mesh_t     *cm,
-                                               cs_real_t                 t_eval,
-                                               void                     *input,
-                                               cs_quadrature_type_t      qtype,
-                                               cs_real_t                *eval);
+cs_xdef_cw_eval_scal_avg_reduction_by_analytic(const cs_cell_mesh_t    *cm,
+                                               cs_real_t                t_eval,
+                                               void                    *context,
+                                               cs_quadrature_type_t     qtype,
+                                               cs_real_t               *eval);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -1052,17 +1097,17 @@ cs_xdef_cw_eval_scal_avg_reduction_by_analytic(const cs_cell_mesh_t     *cm,
  * \param[in]      cm       pointer to a \ref cs_cell_mesh_t structure
  * \param[in]      t_eval   physical time at which one evaluates the term
  * \param[in]      qtype    quadrature type
- * \param[in]      input    pointer to an input structure
+ * \param[in]      context  pointer to a context structure
  * \param[in, out] eval     result of the evaluation
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_xdef_cw_eval_vect_avg_reduction_by_analytic(const cs_cell_mesh_t     *cm,
-                                               cs_real_t                 t_eval,
-                                               void                     *input,
-                                               cs_quadrature_type_t      qtype,
-                                               cs_real_t                *eval);
+cs_xdef_cw_eval_vect_avg_reduction_by_analytic(const cs_cell_mesh_t    *cm,
+                                               cs_real_t                t_eval,
+                                               void                    *context,
+                                               cs_quadrature_type_t     qtype,
+                                               cs_real_t               *eval);
 
 /*----------------------------------------------------------------------------*/
 
