@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -48,13 +48,9 @@
 !------------------------------------------------------------------------------
 !   mode          name          role
 !------------------------------------------------------------------------------
-!> \param[in]     nvar          total number of variables
-!> \param[in]     nscal         total number of scalars
-!> \param[in]     dt            time step (per cell)
 !______________________________________________________________________________
 
-subroutine cfiniv &
- ( nvar   , nscal  , dt     )
+subroutine cfiniv
 
 !===============================================================================
 ! Module files
@@ -79,25 +75,12 @@ use cs_cf_bindings
 
 implicit none
 
-integer          nvar   , nscal
-
-double precision dt(ncelet)
-
 ! Local variables
 
 double precision, dimension(:), pointer :: cpro_cp, cpro_cv, mix_mol_mas
 
 !===============================================================================
 
-!===============================================================================
-! User initialisation of the variables only if this not a resumed computation
-! and if it has not already been done in gas mix initialization
-!===============================================================================
-
-if (isuite.eq.0.and.ippmod(igmix).lt.0) then
-  call cs_user_f_initialization(nvar, nscal, dt)
-endif
-
 !===============================================================================
 ! Computation of variable Cv in order to have a correct initialization
 ! of the total energy (computed in inivar by a call to a thermodynamic
