@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -74,7 +74,6 @@ use ppppar
 use ppthch
 use ppincl
 use mesh
-use pointe, only: itypfb
 use field
 use cs_c_bindings
 use cs_cf_bindings
@@ -102,19 +101,20 @@ integer          ifac  , iel
 integer          init  , inc   , iccocg, ii, jj
 integer          iflmas, iflmab
 integer          iphydp, icvflb
-integer          nswrgp, imligp, iwarnp
+integer          imrgrp, nswrgp, imligp, iwarnp
 integer          istatp, iconvp, idiffp, ndircp
 integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          ivoid(1)
 double precision epsrgp, climgp, extrap, blencp, epsilp
 double precision epsrsp
-double precision sclnor
+double precision sclnor, hint
 
 integer          imucpp, idftnp, iswdyp
-integer          imvis1, f_id0, idtcfl
+integer          imvis1, f_id0
 integer          f_id
 
-double precision thetv, relaxp, hint
+double precision thetv, relaxp
+double precision normp
 
 double precision rvoid(1)
 
@@ -136,14 +136,16 @@ double precision, dimension(:), pointer :: cvar_pr, cvara_pr, cpro_cp, cpro_cv
 double precision, dimension(:,:), pointer :: coefau
 double precision, dimension(:,:,:), pointer :: coefbu
 double precision, dimension(:), pointer :: cpro_divq
+double precision, dimension(:), pointer :: cvar_fracv, cvar_fracm, cvar_frace
 
 type(var_cal_opt) :: vcopt_p
 
-!===============================================================================
 !===============================================================================
 ! 1. INITIALISATION
 !===============================================================================
 
+extrap = 0
+
 ! Allocate temporary arrays for the mass resolution
 allocate(viscf(nfac), viscb(nfabor))
 allocate(smbrs(ncelet), rovsdt(ncelet))
@@ -178,10 +180,21 @@ call field_get_val_prev_s(icrom, rhopre)
 call field_get_val_s(ivarfl(ipr), cvar_pr)
 call field_get_val_prev_s(ivarfl(ipr), cvara_pr)
 
+
 call field_get_label(ivarfl(ipr), chaine)
 
 call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
 
+if (ippmod(icompf).gt.1) then
+  call field_get_val_s(ivarfl(isca(ifracv)), cvar_fracv)
+  call field_get_val_s(ivarfl(isca(ifracm)), cvar_fracm)
+  call field_get_val_s(ivarfl(isca(ifrace)), cvar_frace)
+else
+  cvar_fracv => rvoid1
+  cvar_fracm => rvoid1
+  cvar_frace => rvoid1
+endif
+
 if(vcopt_p%iwarni.ge.1) then
   write(nfecra,1000) chaine(1:8)
 endif
@@ -202,14 +215,17 @@ else
 endif
 
 ! Computation of the boundary coefficients for the pressure gradient
-! recontruction in accordance with the diffusion boundary coefficients (coefaf_p,
-! coefbf_p). Always a homogeneous Neumann except at walls where the hydrostatic
-! pressure is taken into account (icfgrp option).
+! recontruction in accordance with the diffusion boundary coefficients
+! (coefaf_p, coefbf_p)
+
 do ifac = 1, nfabor
   iel = ifabor(ifac)
   hint = dt(iel) / distb(ifac)
-  wbfa(ifac) = -coefaf_p(ifac) / hint
-  wbfb(ifac) = 1.d0
+
+  call set_neumann_scalar                 &
+     ( wbfa(ifac)    , coefaf_p(ifac),    &
+       wbfb(ifac)    , rvoid(1)      ,    &
+       coefaf_p(ifac), hint )
 enddo
 
 !===============================================================================
@@ -246,7 +262,8 @@ endif
 
 allocate(c2(ncelet))
 
-call cs_cf_thermo_c_square(cpro_cp, cpro_cv, cvar_pr, crom, c2, ncel)
+call cs_cf_thermo_c_square(cpro_cp, cpro_cv, cvar_pr, crom, &
+                           cvar_fracv, cvar_fracm, cvar_frace, c2, ncel)
 
 do iel = 1, ncel
   rovsdt(iel) = rovsdt(iel) + vcopt_p%istat*(cell_f_vol(iel)/(dt(iel)*c2(iel)))
@@ -259,9 +276,8 @@ enddo
 ! computation of the "convective flux" for the density
 
 ! volumic flux (u + dt f)
-idtcfl = 0
 call cfmsfp                                                                    &
-( nvar   , nscal  , idtcfl , iterns , ncepdp , ncesmp ,                        &
+( nvar   , nscal  , iterns , ncepdp , ncesmp ,                                 &
   icepdc , icetsm , itypsm ,                                                   &
   dt     , vela   ,                                                            &
   ckupdc , smacel ,                                                            &
@@ -284,24 +300,9 @@ do ifac = 1, nfabor
   wflmab(ifac) = -brom(ifac)*bvolfl(ifac)
 enddo
 
-if (icfgrp.eq.1) then
-  ! The hydrostatic pressure gradient contribution has to be added to the mass
-  ! flux if it has been taken into account in the pressure B.C. at walls.
-  do ifac = 1, nfabor
-    iel = ifabor(ifac)
-    if (itypfb(ifac).eq.iparoi .or. itypfb(ifac).eq.iparug) then
-      wflmab(ifac) = wflmab(ifac)                                              &
-                     -dt(iel)*crom(iel)*(  gx*surfbo(1,ifac)                   &
-                                         + gy*surfbo(2,ifac)                   &
-                                         + gz*surfbo(3,ifac))
-    endif
-  enddo
-endif
-
 init = 0
 call divmas(init,wflmas,wflmab,smbrs)
 
-
 call field_get_id_try("predicted_vel_divergence", f_id)
 if (f_id.ge.0) then
   call field_get_val_s(f_id, cpro_divq)
@@ -325,8 +326,9 @@ call viscfa                                                                    &
 istatp = vcopt_p%istat
 iconvp = vcopt_p%iconv
 idiffp = vcopt_p%idiff
-ndircp = ndircl(ipr)
+ndircp = vcopt_p%ndircl
 nswrsp = vcopt_p%nswrsm
+imrgrp = vcopt_p%imrgra
 nswrgp = vcopt_p%nswrgr
 imligp = vcopt_p%imligr
 ircflp = vcopt_p%ircflu
@@ -342,17 +344,17 @@ epsilp = vcopt_p%epsilo
 epsrsp = vcopt_p%epsrsm
 epsrgp = vcopt_p%epsrgr
 climgp = vcopt_p%climgr
-extrap = vcopt_p%extrag
 relaxp = vcopt_p%relaxv
 thetv  = vcopt_p%thetav
 icvflb = 0
+normp = -1.d0
 
 call codits                                                                    &
 ( idtvar , init   , ivarfl(ipr)     , iconvp , idiffp , ndircp ,               &
-  imrgra , nswrsp , nswrgp , imligp , ircflp ,                                 &
+  imrgrp , nswrsp , nswrgp , imligp , ircflp ,                                 &
   ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,                        &
-  iwarnp ,                                                                     &
-  blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,                        &
+  iwarnp , normp  ,                                                            &
+  blencp , epsilp , epsrsp , epsrgp , climgp ,                                 &
   relaxp , thetv  ,                                                            &
   cvara_pr        , cvara_pr        ,                                          &
   wbfa   , wbfb   ,                                                            &
@@ -411,7 +413,7 @@ iphydp = 0
 ! This flux is stored as the mass flux of the energy
 
 call itrmas                                                                    &
-( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp ,               &
+( f_id0  , init   , inc    , imrgrp , iccocg , nswrgp , imligp ,               &
   iphydp , 0      , iwarnp ,                                                   &
   epsrgp , climgp , extrap ,                                                   &
   rvoid  ,                                                                     &
@@ -456,10 +458,6 @@ if (igrdpp.gt.0) then
 
 endif
 
-! There are no clippings on density because we consider that pressure
-! and energy have been checked and are correct so that the density
-! is also correct through the state law or the linearized law.
-
 deallocate(c2)
 deallocate(viscf, viscb)
 deallocate(smbrs, rovsdt)
