@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -106,45 +106,48 @@ double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
-integer          ivar  , ifac  , iel, l_size
-integer          ii    , iii   , iccfth
+integer          ifac  , iel
+integer          ii    , iccfth
 integer          icalep
-integer          ien   , itk
+integer          ien   , itk, niv
 integer          nvarcf
 
 integer          nvcfmx
 parameter       (nvcfmx=6)
 integer          ivarcf(nvcfmx)
 
-double precision hint, cpb(1), cvb(1), bmasfl
+double precision hint, bmasfl, drom
 
-double precision, allocatable, dimension(:) :: w1, w2
-double precision, allocatable, dimension(:) :: w4, w5, w6
+double precision, allocatable, dimension(:) :: w5
 double precision, allocatable, dimension(:) :: w7
 double precision, allocatable, dimension(:) :: wbfb, wbfa
 double precision, allocatable, dimension(:) :: bc_en, bc_pr, bc_tk
+double precision, allocatable, dimension(:) :: bc_fracv, bc_fracm, bc_frace
 double precision, allocatable, dimension(:,:) :: bc_vel
 
 double precision, dimension(:), pointer :: coefbp
-double precision, dimension(:), pointer :: crom, brom, cpro_cp, cpro_cv, cvar_en
+double precision, dimension(:), pointer :: crom, brom, cpro_cv, cvar_en
 double precision, dimension(:,:), pointer :: vel
+double precision, dimension(:), pointer :: cvar_fracv, cvar_fracm, cvar_frace
 
 !===============================================================================
 
 ! Map field arrays
-call field_get_val_v(ivarfl(iu), vel)
+ call field_get_val_v(ivarfl(iu), vel)
 
 !===============================================================================
 ! 1. Initializations
 !===============================================================================
 
 ! Allocate temporary arrays
-allocate(w1(ncelet), w2(ncelet))
-allocate(w4(ncelet), w5(ncelet), w6(ncelet))
+allocate(w5(ncelet))
 allocate(w7(nfabor), wbfa(nfabor), wbfb(nfabor))
 allocate(bc_en(nfabor))
 allocate(bc_pr(nfabor))
 allocate(bc_tk(nfabor))
+allocate(bc_fracv(nfabor))
+allocate(bc_fracm(nfabor))
+allocate(bc_frace(nfabor))
 allocate(bc_vel(3,nfabor))
 
 ien = isca(ienerg)
@@ -155,9 +158,15 @@ call field_get_val_s(ibrom, brom)
 
 call field_get_val_s(ivarfl(ien), cvar_en)
 
-if (icp.ge.0) call field_get_val_s(icp, cpro_cp)
 if (icv.ge.0) call field_get_val_s(icv, cpro_cv)
 
+! mixture fractions for the homogeneous two-phase flows
+if (ippmod(icompf).eq.2) then
+  call field_get_val_s(ivarfl(isca(ifracv)), cvar_fracv)
+  call field_get_val_s(ivarfl(isca(ifracm)), cvar_fracm)
+  call field_get_val_s(ivarfl(isca(ifrace)), cvar_frace)
+endif
+
 ! list of the variables of the compressible model
 ivarcf(1) = ipr
 ivarcf(2) = iu
@@ -206,7 +215,7 @@ do ifac = 1, nfabor
 
     if(icfgrp.eq.1) then
 
-      icodcl(ifac,ipr) = 3
+      icodcl(ifac,ipr) = 15
       hint = dt(iel)/distb(ifac)
       rcodcl(ifac,ipr,3) = -hint                                  &
            * ( gx*(cdgfbo(1,ifac)-xyzcen(1,iel))                  &
@@ -222,12 +231,12 @@ do ifac = 1, nfabor
       call cs_cf_thermo_wall_bc(wbfa, wbfb, ifac-1)
 
       if (wbfb(ifac).lt.rinfin*0.5d0.and.wbfb(ifac).gt.0.d0) then
-        icodcl(ifac,ipr) = 11
+        icodcl(ifac,ipr) = 12
         rcodcl(ifac,ipr,1) = wbfa(ifac)
         rcodcl(ifac,ipr,2) = wbfb(ifac)
       else
         ! If rarefaction is too strong : homogeneous Dirichlet
-        icodcl(ifac,ipr) = 1
+        icodcl(ifac,ipr) = 13
         rcodcl(ifac,ipr,1) = 0.d0
       endif
 
@@ -318,15 +327,34 @@ do ifac = 1, nfabor
     ! we look for the variable to be initialized
     ! (if a zero value has been given, it is not adapted, so it will
     ! be considered as not initialized and the computation will stop
-    ! displaying an error message
-    iccfth = 10000
-    if(rcodcl(ifac,ipr,1).lt.rinfin*0.5d0) iccfth = 2*iccfth
-    if(brom(ifac).gt.0.d0                ) iccfth = 3*iccfth
-    if(rcodcl(ifac,itk,1).lt.rinfin*0.5d0) iccfth = 5*iccfth
-    if(rcodcl(ifac,ien,1).lt.rinfin*0.5d0) iccfth = 7*iccfth
+    ! displaying an error message. The boundary density may
+    ! be pre-initialized to the cell density also, so is tested last.
 
-    if(iccfth.le.70000.and.iccfth.ne.60000.or.iccfth.eq.350000) then
-      write(nfecra,1000)iccfth
+    iel = ifabor(ifac)
+    drom = abs(crom(iel) - brom(ifac))
+
+    niv = 0
+    iccfth = 10000
+    if (rcodcl(ifac,ipr,1).lt.rinfin*0.5d0) then
+      iccfth = 2*iccfth
+      niv = niv + 1
+    endif
+    if (rcodcl(ifac,itk,1).lt.rinfin*0.5d0) then
+      iccfth = 5*iccfth
+      niv = niv + 1
+    endif
+    if (rcodcl(ifac,ien,1).lt.rinfin*0.5d0) then
+      iccfth = 7*iccfth
+      niv = niv + 1
+    endif
+
+    if (brom(ifac).gt.0.d0 .and. (niv.lt.2 .or. drom.gt.epzero)) then
+      iccfth = 3*iccfth
+      niv = niv + 1
+    endif
+
+    if (niv .ne. 2) then
+      write(nfecra,1000) iccfth
       call csexit (1)
     endif
     iccfth = iccfth + 900
@@ -341,9 +369,6 @@ do ifac = 1, nfabor
 
     call cs_cf_thermo(iccfth, ifac-1, bc_en, bc_pr, bc_tk, bc_vel)
 
-    ! Rusanov fluxes, mass flux and boundary conditions types (icodcl) are
-    ! dealt with further below
-
 !===============================================================================
 ! 3.2 Outlet with imposed pressure
 !===============================================================================
@@ -367,8 +392,6 @@ do ifac = 1, nfabor
 
     call cs_cf_thermo_subsonic_outlet_bc(bc_en, bc_pr, bc_vel, ifac-1)
 
-    ! mass fluxes and boundary conditions codes, see further below.
-
 !===============================================================================
 ! 3.3 Inlet with Ptot, Htot imposed (reservoir boundary conditions)
 !===============================================================================
@@ -395,8 +418,6 @@ do ifac = 1, nfabor
 
     call cs_cf_thermo_ph_inlet_bc(bc_en, bc_pr, bc_vel, ifac-1)
 
-    ! mass fluxes and boundary conditions codes, see further below.
-
 !===============================================================================
 ! 3.4 Inlet with imposed rho*U and rho*U*H
 !===============================================================================
@@ -422,9 +443,9 @@ do ifac = 1, nfabor
 
 !===============================================================================
 ! 4. Complete the treatment for inlets and outlets:
-!    - mass fluxes computation
 !    - boundary convective fluxes computation (analytical or Rusanov) if needed
 !    - B.C. code (Dirichlet or Neumann)
+!    - Dirichlet values
 !===============================================================================
 
   if (itypfb(ifac).eq.iesicf.or.                    &
@@ -434,15 +455,16 @@ do ifac = 1, nfabor
       itypfb(ifac).eq.ieqhcf) then
 
 !===============================================================================
-! 4.1 Mass fluxes computation and
-!     boundary convective fluxes computation (analytical or Rusanov) if needed
+! 4.1 Boundary convective fluxes computation (analytical or Rusanov) if needed
 !     (gamma should already have been computed if Rusanov fluxes are computed)
 !===============================================================================
 
     ! Rusanov fluxes are computed only for the imposed inlet for stability
-    ! reasons (the mass flux computation is concluded)
+    ! reasons.
     if (itypfb(ifac).eq.iesicf) then
 
+    ! Dirichlet for velocity and pressure are computed in order to
+    ! impose the Rusanov fluxes in mass, momentum and energy balance.
       call cfrusb(ifac, bc_en, bc_pr, bc_vel)
 
     ! For the other types of inlets/outlets (subsonic outlet, QH inlet,
@@ -466,6 +488,11 @@ do ifac = 1, nfabor
       rcodcl(ifac,iu,1)  = bc_vel(1,ifac)
       rcodcl(ifac,iv,1)  = bc_vel(2,ifac)
       rcodcl(ifac,iw,1)  = bc_vel(3,ifac)
+      if (ippmod(icompf).eq.2) then ! FIXME fill bc_frac...
+        rcodcl(ifac,isca(ifracv),1) = bc_fracv(ifac)
+        rcodcl(ifac,isca(ifracm),1) = bc_fracm(ifac)
+        rcodcl(ifac,isca(ifrace),1) = bc_frace(ifac)
+      endif
     else ! supersonic outlet
       rcodcl(ifac,ien,3) = 0.d0
       rcodcl(ifac,ipr,3) = 0.d0
@@ -473,6 +500,11 @@ do ifac = 1, nfabor
       rcodcl(ifac,iu,3)  = 0.d0
       rcodcl(ifac,iv,3)  = 0.d0
       rcodcl(ifac,iw,3)  = 0.d0
+      if (ippmod(icompf).eq.2) then
+        rcodcl(ifac,isca(ifracv),3) = 0.d0
+        rcodcl(ifac,isca(ifracm),3) = 0.d0
+        rcodcl(ifac,isca(ifrace),3) = 0.d0
+      endif
     endif
 
 !===============================================================================
@@ -497,6 +529,12 @@ do ifac = 1, nfabor
       icodcl(ifac,ien)   = 1
       ! temperature
       icodcl(ifac,itk)   = 1
+      ! mixture fractions
+      if (ippmod(icompf).eq.2) then
+        icodcl(ifac,isca(ifracv))   = 1
+        icodcl(ifac,isca(ifracm))   = 1
+        icodcl(ifac,isca(ifrace))   = 1
+      endif
     else ! supersonic outlet
       icodcl(ifac,ipr)   = 3
       icodcl(ifac,iu)    = 3
@@ -504,6 +542,12 @@ do ifac = 1, nfabor
       icodcl(ifac,iw)    = 3
       icodcl(ifac,ien)   = 3
       icodcl(ifac,itk)   = 3
+      ! mixture fractions
+      if (ippmod(icompf).eq.2) then
+        icodcl(ifac,isca(ifracv))   = 3
+        icodcl(ifac,isca(ifracm))   = 3
+        icodcl(ifac,isca(ifrace))   = 3
+      endif
     endif
 
 !-------------------------------------------------------------------------------
@@ -602,50 +646,49 @@ do ifac = 1, nfabor
           icodcl(ifac,ifb ) = 3
         endif
       elseif(iturb.eq.60) then
-         if(rcodcl(ifac,ik  ,1).lt.rinfin*0.5d0  .and.      &
-            rcodcl(ifac,iomg,1).lt.rinfin*0.5d0) then
-           icodcl(ifac,ik  ) = 1
-           icodcl(ifac,iomg) = 1
-         else
-           icodcl(ifac,ik  ) = 3
-           icodcl(ifac,iomg) = 3
-         endif
-       elseif(iturb.eq.70) then
-         if(rcodcl(ifac,inusa,1).gt.0.d0) then
-           icodcl(ifac,inusa) = 1
-         else
-           icodcl(ifac,inusa) = 3
-         endif
-       endif
-       if (nscaus.gt.0) then
-         do ii = 1, nscaus
-           if(rcodcl(ifac,isca(ii),1).lt.rinfin*0.5d0) then
-             icodcl(ifac,isca(ii)) = 1
-           else
-             icodcl(ifac,isca(ii)) = 3
-           endif
-         enddo
-       endif
-       if (nscasp.gt.0) then
-         do ii = 1, nscasp
-           if(rcodcl(ifac,iscasp(ii),1).lt.rinfin*0.5d0) then
-             icodcl(ifac,iscasp(ii)) = 1
-           else
-             icodcl(ifac,iscasp(ii)) = 3
-           endif
-         enddo
-       endif
-     endif
+        if(rcodcl(ifac,ik  ,1).lt.rinfin*0.5d0  .and.      &
+           rcodcl(ifac,iomg,1).lt.rinfin*0.5d0) then
+          icodcl(ifac,ik  ) = 1
+          icodcl(ifac,iomg) = 1
+        else
+          icodcl(ifac,ik  ) = 3
+          icodcl(ifac,iomg) = 3
+        endif
+      elseif(iturb.eq.70) then
+        if(rcodcl(ifac,inusa,1).gt.0.d0) then
+          icodcl(ifac,inusa) = 1
+        else
+          icodcl(ifac,inusa) = 3
+        endif
+      endif
+      if (nscaus.gt.0) then
+        do ii = 1, nscaus
+          if(rcodcl(ifac,isca(ii),1).lt.rinfin*0.5d0) then
+            icodcl(ifac,isca(ii)) = 1
+          else
+            icodcl(ifac,isca(ii)) = 3
+          endif
+        enddo
+      endif
+      if (nscasp.gt.0) then
+        do ii = 1, nscasp
+          if(rcodcl(ifac,iscasp(ii),1).lt.rinfin*0.5d0) then
+            icodcl(ifac,iscasp(ii)) = 1
+          else
+            icodcl(ifac,iscasp(ii)) = 3
+          endif
+        enddo
+      endif
+    endif
 
-   endif ! end of test on inlet/outlet faces
+  endif ! end of test on inlet/outlet faces
 
- enddo ! end of loop on boundary faces
+enddo ! end of loop on boundary faces
 
 ! Free memory
-deallocate(w1, w2)
-deallocate(w4, w5, w6)
+deallocate(w5)
 deallocate(w7, wbfb, wbfa)
-deallocate(bc_en, bc_pr, bc_tk, bc_vel)
+deallocate(bc_en, bc_pr, bc_tk, bc_fracv, bc_fracm, bc_frace, bc_vel)
 
 !----
 ! FORMATS
@@ -659,12 +702,11 @@ deallocate(bc_en, bc_pr, bc_tk, bc_vel)
 '@    =========                                               ',/,&
 '@    two and only two independant variables among            ',/,&
 '@    P, rho, T and E have to be imposed at boundaries of type',/,&
-'@    iesicf in uscfcl (iccfth = ',I10,').                  ',/,&
+'@    iesicf in uscfcl (iccfth = ',i10,').                  ',/,&
 '@                                                            ',/,&
 '@    The computation will stop.                              ',/,&
 '@                                                            ',/,&
-'@    Check the boundary conditions in                        ',/,&
-'@    cs_user_boundary_conditions                             ',/,&
+'@    Check the boundary condition definitions.               ',/,&
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
