@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -27,7 +27,6 @@
 
 /*----------------------------------------------------------------------------*/
 
-
 #include "cs_defs.h"
 
 /*----------------------------------------------------------------------------
@@ -38,14 +37,256 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
+#include "cs_physical_constants.h"
+#include "cs_cf_model.h"
+
 /*----------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
 
 /*============================================================================
- * Private function definitions
+ * Type definitions
  *============================================================================*/
 
+/* Stiffened gas thermodynamic parameters */
+/*---------------------------------------*/
+
+typedef struct {
+
+  cs_real_t cv;
+  cs_real_t gamma;
+  cs_real_t pinf;
+  cs_real_t qprim;
+  cs_real_t q; /* FIXME to be taken into account in equilibrium functions.
+                  Should be set to 0 until it is fully taken into account. */
+
+} cs_stiffened_gas_t;
+
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
+/*============================================================================
+ * Static variables
+ *============================================================================*/
+
+static cs_real_t _eps_log_lim = 1.e-11;
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
+/*============================================================================
+ * Public inlined function
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Computation of entropy in plane (v,e) with a stiffened gas law.
+ *
+ * \f\[
+ * s = Q^{\prime} + C_v * log(e-v P_{\infty})
+              + C_v * (\gamma-1) log(v)
+ * \f\]
+ * where \f$Q^{\prime}\f$, \f$C_v\f$, \f$P_{\infty}\f$ and \f$\gamma\f$
+ * are constants.
+ *
+ * \param[in]     vol   specific volume
+ * \param[in]     energ specific energy
+ * \param[in]     sg    stiffened gas parameters structure
+ *
+ * \return entropy
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_real_t
+cs_cf_thermo_entropy_sg_ve(cs_real_t          vol,
+                           cs_real_t          energ,
+                           cs_stiffened_gas_t sg)
+{
+  cs_real_t CV = sg.cv;
+  cs_real_t gamma = sg.gamma;
+  cs_real_t Pinf = sg.pinf;
+  cs_real_t Qprim = sg.qprim;
+  cs_real_t Q = sg.q;
+
+  cs_real_t eps = _eps_log_lim;
+  cs_real_t s, tmp1, tmp2;
+  tmp1 = energ - Q - vol*Pinf;
+  tmp2 = vol;
+  s = Qprim;
+  if (tmp1 > eps) s += CV*log(tmp1);
+  if (tmp2 > eps) s += CV*(gamma-1.)*log(tmp2);
+
+  return s;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Computation of the temperature
+ *
+ *        The routine concerns the stiffened gas law :
+ *        \f$T_1=(e_1-P_{\infty,1}*v_1)/CV_1\f$,
+ *        where \f$CV_1\f$ and \f$P_{\infty,1}\f$ are constants.
+ *
+ * \param[in]     vol   the specific volume of phase iph
+ * \param[in]     energ the specific energy of phase iph
+ * \param[in]     sg    stiffened gas parameters structure
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_real_t
+cs_cf_thermo_temperature_sg_ve(cs_real_t          vol,
+                               cs_real_t          energ,
+                               cs_stiffened_gas_t sg)
+{
+  cs_real_t CV = sg.cv;
+  cs_real_t Pinf = sg.pinf;
+  cs_real_t Q = sg.q;
+
+  cs_real_t T = (energ-Q-Pinf*vol) / CV;
+
+  return T;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Computation of the pressure
+ *
+ *        The routine concerns the stiffened gas law :
+ *        \f$P_1=(\gamma_1-1.0)*e_1/v_1-P_{\infty,1}*\gamma_1\f$,
+ *        where \f$P_{\infty,1}\f$ and \f$\gamma_1\f$ are constants.
+ *
+ * \param[in]     vol   the specific volume of phase iph
+ * \param[in]     energ the specific energy of phase iph
+ * \param[in]     sg    stiffened gas parameters structure
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_real_t
+cs_cf_thermo_pressure_sg_ve(cs_real_t          vol,
+                            cs_real_t          energ,
+                            cs_stiffened_gas_t sg)
+{
+  cs_real_t gamma = sg.gamma;
+  cs_real_t Pinf = sg.pinf;
+  cs_real_t Q = sg.q;
+
+  cs_real_t P = (gamma-1.)*(energ-Q) / vol-Pinf*gamma;
+
+  return P;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Computation of internal energy in plane (T,P).
+ *
+ * \param[in]     T    temperature
+ * \param[in]     P    pressure
+ * \param[in]     sg    stiffened gas parameters structure
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_real_t
+cs_cf_thermo_internal_energy_sg_tp(cs_real_t          T,
+                                   cs_real_t          P,
+                                   cs_stiffened_gas_t sg)
+{
+  cs_real_t CV = sg.cv;
+  cs_real_t gamma = sg.gamma;
+  cs_real_t Pinf = sg.pinf;
+  cs_real_t Q = sg.q;
+
+  cs_real_t e, tmp;
+  tmp = P + Pinf;
+  e = Q + CV*T*(P+gamma*Pinf) / tmp;
+
+  return e;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Computation of specific volume in plane (T,P).
+ *
+ * \param[in]     T    temperature
+ * \param[in]     P    pressure
+ * \param[in]     sg    stiffened gas parameters structure
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_real_t
+cs_cf_thermo_specific_volume_sg_tp(cs_real_t          T,
+                                   cs_real_t          P,
+                                   cs_stiffened_gas_t sg)
+{
+  cs_real_t CV = sg.cv;
+  cs_real_t gamma = sg.gamma;
+  cs_real_t Pinf = sg.pinf;
+
+  cs_real_t tau,tmp;
+  tmp = P + Pinf;
+  tau = (gamma-1.)*CV*T / tmp;
+
+  return tau;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Computation of entropy in plane (T,P).
+ *
+ * \param[in]     T    temperature
+ * \param[in]     P    pressure
+ * \param[in]     sg    stiffened gas parameters structure
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_real_t
+cs_cf_thermo_entropy_sg_tp(cs_real_t          T,
+                           cs_real_t          P,
+                           cs_stiffened_gas_t sg)
+{
+  cs_real_t CV = sg.cv;
+  cs_real_t gamma = sg.gamma;
+  cs_real_t Pinf = sg.pinf;
+  cs_real_t Qprim = sg.qprim;
+
+  cs_real_t s;
+  cs_real_t tmp = P + Pinf;
+
+  s =   CV*gamma*log(CV*T)  +  CV*(gamma-1.)*log(gamma-1.)
+     -  CV*(gamma-1.)*log(tmp) + Qprim;
+
+ return s;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Computation of internal energy in plane (s,v).
+ *
+ * \param[in]     s    entropy
+ * \param[in]     v    specific volume
+ * \param[in]     sg    stiffened gas parameters structure
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_real_t
+cs_cf_thermo_internal_energy_sg_sv(cs_real_t          s,
+                                   cs_real_t          v,
+                                   cs_stiffened_gas_t sg)
+{
+  cs_real_t CV = sg.cv;
+  cs_real_t gamma = sg.gamma;
+  cs_real_t Pinf = sg.pinf;
+  cs_real_t Qprim = sg.qprim;
+  cs_real_t Q = sg.q;
+
+  cs_real_t tmp1, tmp2;
+  cs_real_t e = -1.;
+
+  tmp1 = pow(v, 1.-gamma);
+
+  tmp2 = exp((s - Qprim) / CV);
+  e = Q + v * Pinf + tmp1*tmp2;
+
+  return e;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Compute \f$\gamma\f$.
@@ -64,7 +305,7 @@ cs_cf_thermo_gamma(cs_real_t *cp,
                    cs_lnum_t l_size)
 {
   /*  Local variables */
-  int ieos = cs_glob_fluid_properties->ieos;
+  int ieos = cs_glob_cf_model->ieos;
 
   /*  Gamma is supposed to be superior or equal to 1.
       It is computed at each call, even if this may seem costly,
@@ -73,7 +314,7 @@ cs_cf_thermo_gamma(cs_real_t *cp,
 
   /* single ideal gas - constant gamma
      or ideal gas mix - gamma for the mixture */
-  if (ieos == 1 || ieos == 3) {
+  if (ieos == CS_EOS_IDEAL_GAS || ieos == CS_EOS_GAS_MIX) {
     for (cs_lnum_t ii = 0; ii < l_size; ii++) {
       gamma[ii] = cp[ii]/cv[ii];
       if (gamma[ii] < 1.)
@@ -86,9 +327,9 @@ cs_cf_thermo_gamma(cs_real_t *cp,
     }
   }
   /* stiffened gas - constant gamma (parameter of the law) */
-  else if (ieos == 2) {
+  else if (ieos == CS_EOS_STIFFENED_GAS) {
     for (cs_lnum_t ii = 0; ii < l_size; ii++)
-      gamma[ii] = cs_glob_fluid_properties->gammasg;
+      gamma[ii] = cs_glob_cf_model->gammasg;
   }
 }
 
@@ -271,6 +512,9 @@ cs_cf_thermo_pe_from_dt(cs_real_t   *cp,
  *   pres   --> array of pressure values
  *   temp   --> array of temperature values
  *   vel    <-- array of velocity component values
+ *   fracv  <-- array of volume fraction values
+ *   fracm  <-- array of mass fraction values
+ *   frace  <-- array of energy fraction values
  *   l_size <-- l_size of the array
  *----------------------------------------------------------------------------*/
 
@@ -282,6 +526,9 @@ cs_cf_thermo_pt_from_de(cs_real_t   *cp,
                         cs_real_t   *pres,
                         cs_real_t   *temp,
                         cs_real_3_t *vel,
+                        cs_real_t   *fracv,
+                        cs_real_t   *fracm,
+                        cs_real_t   *frace,
                         cs_lnum_t    l_size);
 
 /*----------------------------------------------------------------------------
@@ -301,6 +548,9 @@ cs_cf_thermo_c_square(cs_real_t *cp,
                       cs_real_t *cv,
                       cs_real_t *pres,
                       cs_real_t *dens,
+                      cs_real_t *fracv,
+                      cs_real_t *fracm,
+                      cs_real_t *frace,
                       cs_real_t *c2,
                       cs_lnum_t  l_size);
 
@@ -415,9 +665,9 @@ cs_cf_thermo_ph_inlet_bc(cs_real_t   *bc_en,
  *----------------------------------------------------------------------------*/
 
 void
-cs_cf_thermo_eps_sup(cs_real_t *dens,
-                     cs_real_t *eps_sup,
-                     cs_lnum_t  l_size);
+cs_cf_thermo_eps_sup(const cs_real_t  *dens,
+                     cs_real_t        *eps_sup,
+                     cs_lnum_t         l_size);
 
 /*----------------------------------------------------------------------------
  * This subroutine is a driver allowing to call the appropriate
@@ -468,6 +718,9 @@ cs_cf_thermo_eps_sup(cs_real_t *dens,
  *   bc_pr   <-- pressure values at boundary faces
  *   bc_tk   <-- temperature values at boundary faces
  *   bc_vel  <-- velocity values at boundary faces
+ *   bc_fracv <--  volume fraction values at boundary faces
+ *   bc_fracm <--  mass fraction values at boundary faces
+ *   bc_frace <--  energy fraction values at boundary faces
  *----------------------------------------------------------------------------*/
 
 void
@@ -476,7 +729,10 @@ cs_cf_thermo(const int    iccfth,
              cs_real_t   *bc_en,
              cs_real_t   *bc_pr,
              cs_real_t   *bc_tk,
-             cs_real_3_t *bc_vel);
+             cs_real_3_t *bc_vel,
+             cs_real_t   *bc_fracv,
+             cs_real_t   *bc_fracm,
+             cs_real_t   *bc_frace);
 
 /*----------------------------------------------------------------------------*/
 
