@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -63,7 +63,8 @@ use coincl
 use cpincl
 use ppincl
 use radiat
-use ihmpre
+use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -73,7 +74,7 @@ implicit none
 
 integer          ii , jj, iok
 integer          isc
-double precision wmolme
+double precision wmolme, turb_schmidt
 
 !===============================================================================
 ! 1. VARIABLES TRANSPORTEES
@@ -84,12 +85,13 @@ double precision wmolme
 
 ! --> Flamme de diffusion : chimie 3 points
 
-if ( ippmod(icod3p).ge.0 ) then
+if (ippmod(icod3p).ge.0) then
 
   ! ---- Variance du taux de melange
   !        Type de clipping superieur pour la variance
   !        0 pas de clipping, 1 clipping var max de fm, 2 clipping a SCAMAX
-  iclvfl(ifp2m) = 1
+  !iclvfl(ifp2m) = 1
+  iclvfl(ifp2m) = 2
   !        scamin(ifp2m) = 0.d0
   !        scamax(ifp2m) = 0.25D0
 
@@ -97,7 +99,7 @@ endif
 
 ! --> Flamme de premelange : modele LWC
 
-if ( ippmod(icolwc).ge.0 ) then
+if (ippmod(icolwc).ge.0) then
   iclvfl(ifp2m) = 0
   iclvfl(iyfp2m) = 0
 endif
@@ -109,56 +111,24 @@ do isc = 1, nscapp
 
   jj = iscapp(isc)
 
-  if ( iscavr(jj).le.0 ) then
+  if (iscavr(jj).le.0) then
 
 ! ---- En combustion on considere que la viscosite turbulente domine
 !      ON S'INTERDIT DONC LE CALCUL DES FLAMMES LAMINAIRES AVEC Le =/= 1
 
-    visls0(jj) = viscl0
+    call field_set_key_double(ivarfl(isca(jj)), kvisl0, viscl0)
 
   endif
 
-! ---- Schmidt ou Prandtl turbulent
+  ! Schmidt ou Prandtl turbulent
 
-  sigmas(jj) = 0.7d0
+  turb_schmidt = 0.7d0
+  call field_set_key_double(ivarfl(isca(jj)), ksigmas, turb_schmidt)
 
-! ---- Coeff dissipation des fluctuations
+  ! Coeff dissipation des fluctuations
 
   rvarfl(jj) = 0.8d0
 
-  ii = isca(iscapp(isc))
-
-! ------ Niveau de detail des impressions pour les variables et
-!          donc les scalaires (valeurs 0 ou 1)
-!          Si = -10000 non modifie par l'utilisateur -> niveau 1
-  if(iwarni(ii).eq.-10000) then
-    iwarni(ii) = 1
-  endif
-
-! ---- Informations relatives a la resolution des scalaires
-
-!       - Facteur multiplicatif du pas de temps
-  cdtvar(ii) = 1.d0
-
-!         - Schema convectif % schema 2ieme ordre
-!           = 0 : upwind
-!           = 1 : second ordre
-  blencv(ii) = 1.d0
-
-!         - Type de schema convetif second ordre (utile si BLENCV > 0)
-!           = 0 : Second Order Linear Upwind
-!           = 1 : Centre
-  ischcv(ii) = 1
-
-!         - Test de pente pour basculer d'un schema centre vers l'upwind
-!           = 0 : utilisation automatique du test de pente
-!           = 1 : calcul sans test de pente
-  isstpc(ii) = 0
-
-!         - Reconstruction des flux de convetion et de diffusion aux faces
-!           = 0 : pas de reconstruction
-  ircflu(ii) = 1
-
 enddo
 
 !===============================================================================
@@ -167,9 +137,9 @@ enddo
 
 ! --> Calcul de RO0 a partir de T0 et P0
 
- if ( ippmod(icod3p).ne.-1 .or.                                   &
-      ippmod(icoebu).ne.-1 .or.                                   &
-      ippmod(icolwc).ne.-1     ) then
+ if (ippmod(icod3p).ne.-1 .or.                                   &
+     ippmod(icoebu).ne.-1 .or.                                   &
+     ippmod(icolwc).ne.-1) then
    wmolme = wmolg(2)
    ro0 = pther*wmolme / (cs_physical_constants_r*t0)
    roref = ro0
@@ -204,9 +174,8 @@ tinoxy = -grand
 nmaxf = 9
 nmaxh = 9
 
-! ---> Masse volumique variable et viscosite constante (pour les suites)
+! ---> Masse volumique variable
 irovar = 1
-ivivar = 0
 
 !===============================================================================
 ! 3. ON REDONNE LA MAIN A L'UTLISATEUR
@@ -214,19 +183,13 @@ ivivar = 0
 
 ! GUI
 if (ippmod(icoebu).ge.0) then
-  if (iihmpr.eq.1) then
-    call uicpi1 (srrom, diftl0)
-    cebu   = 2.5d0
-  endif
+  call uicpi1(srrom, diftl0)
+  cebu   = 2.5d0
 else if (ippmod(icod3p).ge.0) then
-  if (iihmpr.eq.1) then
-    call uicpi1 (srrom, diftl0)
-    call uicpi2 (tinoxy, tinfue)
-  endif
-else if (ippmod(icolwc).ge.0) then
-  if (iihmpr.eq.1) then
-    !TODO no GUI yet
-  endif
+  call uicpi1(srrom, diftl0)
+  call uicpi2(tinoxy, tinfue)
+! else if (ippmod(icolwc).ge.0) then
+  !TODO no GUI yet
 endif
 
 ! User subroutines
@@ -237,34 +200,30 @@ call cs_user_combustion
 !===============================================================================
 
 iok = 0
-if ( ippmod(icoebu).ge.0 ) then
+if (ippmod(icoebu).ge.0) then
 
-  call ebuver (iok)
-  !==========
-  if(iok.gt.0) then
+  call ebuver(iok)
+  if (iok.gt.0) then
     write(nfecra,9999)iok
-    call csexit (1)
-    !==========
+    call csexit(1)
   else
     write(nfecra,9998)
   endif
 
-else if( ippmod(icod3p).ge.0 ) then
-  call d3pver (iok)
-  !==========
-  if(iok.gt.0) then
+else if (ippmod(icod3p).ge.0) then
+  call d3pver(iok)
+  if (iok.gt.0) then
     write(nfecra,9991)iok
-    call csexit (1)
+    call csexit(1)
   else
     write(nfecra,9990)
   endif
 
-else if( ippmod(icolwc).ge.0 ) then
-  call lwcver (iok)
-  !==========
-  if(iok.gt.0) then
+else if (ippmod(icolwc).ge.0) then
+  call lwcver(iok)
+  if (iok.gt.0) then
     write(nfecra,9993)iok
-    call csexit (1)
+    call csexit(1)
   else
     write(nfecra,9992)
   endif
