@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -61,8 +61,8 @@ use ppthch
 use coincl
 use cpincl
 use ppincl
-use ihmpre
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -70,9 +70,11 @@ implicit none
 
 ! Local variables
 
-integer        f_id
+integer        f_id, isc, ii, jj
 integer        kscmin, kscmax, kscavr
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
@@ -176,8 +178,8 @@ if (ippmod(icoebu).ge.0) then
   endif
 endif
 
-! 1.3 Flamme de premelange : modele BML A DEVELOPPER
-! ==================================================
+! 1.3 Flamme de premelange : modele BML A DEVELOPER
+! =================================================
 
 ! 1.4 Flamme de premelange : modele LWC
 ! =====================================
@@ -242,5 +244,35 @@ if (ippmod(icod3p).eq.1 .or.   &
 
 endif
 
+!===============================================================================
+! Default numerical options
+!===============================================================================
+
+do isc = 1, nscapp
+
+  jj = iscapp(isc)
+
+  ii = isca(iscapp(isc))
+
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+
+  ! Second order convective scheme
+  vcopt%blencv = 1.d0
+
+  ! Centered convective scheme
+  vcopt%ischcv = 1
+
+  ! Automatic slope test
+  vcopt%isstpc = 0
+
+  ! Reconstruct convection and diffusion fluxes at faces
+  vcopt%ircflu = 1
+
+  call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+
+enddo
+
+!===============================================================================
+
 return
 end subroutine
