@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,11 +20,7 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine ebuini &
-!================
-
- ( nvar   , nscal  ,                                              &
-   dt     )
+subroutine ebuini
 
 !===============================================================================
 ! FONCTION :
@@ -56,9 +52,6 @@ subroutine ebuini &
 !__________________.____._____.________________________________________________.
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
-! nvar             ! i  ! <-- ! total number of variables                      !
-! nscal            ! i  ! <-- ! total number of scalars                        !
-! dt(ncelet)       ! tr ! <-- ! valeur du pas de temps                         !
 !__________________!____!_____!________________________________________________!
 
 !     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
@@ -91,10 +84,6 @@ use field
 
 implicit none
 
-integer          nvar   , nscal
-
-double precision dt(ncelet)
-
 ! Local variables
 
 character(len=80) :: chaine
@@ -126,7 +115,6 @@ save             ipass
 
 ipass = ipass + 1
 
-
 do igg = 1, ngazgm
   coefg(igg) = zero
 enddo
@@ -315,12 +303,6 @@ if ( isuite.eq.0 ) then
 
     enddo
 
-! ----- On donne la main a l'utilisateur
-    call cs_user_f_initialization &
-    !==========================
-  ( nvar   , nscal  ,                                            &
-    dt     )
-
 ! ----- En periodique et en parallele,
 !       il faut echanger ces initialisations
 
@@ -337,7 +319,6 @@ if ( isuite.eq.0 ) then
       endif
     endif
 
-
 !      Impressions de controle
 
     write(nfecra,2000)
