@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -242,9 +242,7 @@ if (ivar.eq.isca(icoyfp)) then
   inc = 1
   iccocg = 1
 
-  call field_gradient_scalar(ivarfl(ii), iprev, imrgra, inc,              &
-                             iccocg,                                      &
-                             gradf)
+  call field_gradient_scalar(ivarfl(ii), iprev, 0, inc, iccocg, gradf)
 
 ! --- Calcul du gradient de Yfuel
 !     ===========================
@@ -258,9 +256,7 @@ if (ivar.eq.isca(icoyfp)) then
   inc = 1
   iccocg = 1
 
-  call field_gradient_scalar(ivarfl(ii), iprev, imrgra, inc,              &
-                             iccocg,                                      &
-                             grady)
+  call field_gradient_scalar(ivarfl(ii), iprev, 0, inc, iccocg, grady)
 
 ! --- Calcul du terme source
 !     ======================
