@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -100,10 +100,10 @@ double precision smbrs(ncelet), rovsdt(ncelet)
 integer          ivar   , ivarsc , ivarut, f_id0
 integer          iel, ifac
 integer          icha
-integer          inc , iccocg , nswrgp , imligp , iwarnp
+integer          inc , iccocg , imrgrp , nswrgp , imligp , iwarnp
 
 double precision xk , xe , rhovst
-double precision epsrgp , climgp , extrap
+double precision epsrgp , climgp
 double precision turb_schmidt
 
 double precision, allocatable, dimension(:) :: coefap, coefbp
@@ -180,12 +180,12 @@ if ( itytur.eq.2 .or. itytur.eq.3                   &
     ivarut = isca(if3m)
   endif
   call field_get_key_struct_var_cal_opt(ivarfl(ivarut), vcopt)
+  imrgrp = vcopt%imrgra
   nswrgp = vcopt%nswrgr
   imligp = vcopt%imligr
   iwarnp = vcopt%iwarni
   epsrgp = vcopt%epsrgr
   climgp = vcopt%climgr
-  extrap = vcopt%extrag
 
   ! Allocate work arrays
   allocate(w7(ncelet))
@@ -245,8 +245,8 @@ if ( itytur.eq.2 .or. itytur.eq.3                   &
   f_id0 = -1
 
   call gradient_s                                                 &
- ( f_id0  , imrgra , inc    , iccocg , nswrgp , imligp ,          &
-   iwarnp , epsrgp , climgp , extrap ,                            &
+ ( f_id0  , imrgrp , inc    , iccocg , nswrgp , imligp ,          &
+   iwarnp , epsrgp , climgp ,                                     &
    w7     , coefap , coefbp ,                                     &
    grad   )
 
