@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -86,11 +86,11 @@ double precision smbrs(ncelet), rovsdt(ncelet)
 
 integer           iel    , ifac   , f_id0
 integer           icla   , icha
-integer           inc    , iccocg , nswrgp , imligp , iwarnp
+integer           inc    , iccocg , imrgrp , nswrgp , imligp , iwarnp
 integer           iold
 
 double precision xk     , xe     , rhovst
-double precision epsrgp , climgp , extrap
+double precision epsrgp , climgp
 double precision aux
 double precision gdev1 , gdev2
 double precision fsd   , fdev  , diamdv , gdev
@@ -160,12 +160,12 @@ if ( itytur.eq.2 .or. iturb.eq.50 .or.             &
   iccocg = 1
 ! A defaut de savoir pour F1M+F2M on prend comme pour F1M(1)
   call field_get_key_struct_var_cal_opt(ivarfl(isca(if1m(1))), vcopt)
+  imrgrp = vcopt%imrgra
   nswrgp = vcopt%nswrgr
   imligp = vcopt%imligr
   iwarnp = vcopt%iwarni
   epsrgp = vcopt%epsrgr
   climgp = vcopt%climgr
-  extrap = vcopt%extrag
 
 ! --> calcul de X1
 
@@ -224,8 +224,8 @@ if ( itytur.eq.2 .or. iturb.eq.50 .or.             &
 !              n'est pas Rij)
   f_id0  = -1
   call gradient_s                                                 &
- ( f_id0  , imrgra , inc    , iccocg , nswrgp , imligp ,          &
-   iwarnp , epsrgp , climgp , extrap ,                            &
+ ( f_id0  , imrgrp , inc    , iccocg , nswrgp , imligp ,          &
+   iwarnp , epsrgp , climgp ,                                     &
    f1f2   , coefap , coefbp ,                                     &
    grad   )
 
