@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -85,7 +85,7 @@ integer          icha, ifcvsl
 double precision xx2     , xch    , xck    , xash   , xnp , xuash
 double precision pparo2 , xdfchi , xdfext , xdftot0 , xdftot1
 double precision devto1(ncharm)  , devto2(ncharm) , coxck ,  den
-double precision diacka
+double precision diacka, visls_0
 double precision dp , lv, yvs, yv , tebl , shrd , xmeau, xmgaz
 double precision xnuss, tlimit , tmini
 double precision tmin, tmax, yvmin, yvmax, yymax
@@ -100,7 +100,7 @@ double precision, dimension(:), pointer :: cvara_xchcl, cvara_xckcl, cvara_xnpcl
 double precision, dimension(:), pointer :: cvara_xwtcl
 double precision, dimension(:), pointer :: cpro_cgd1, cpro_cgd2, cpro_cgch
 double precision, dimension(:), pointer :: cpro_cght, cpro_rom2, cpro_temp2
-double precision, dimension(:), pointer :: cpro_diam2, cpro_temp1, cpro_mmel
+double precision, dimension(:), pointer :: cpro_diam2, cpro_temp, cpro_mmel
 double precision, dimension(:), pointer :: cpro_yox, cpro_yco2, cpro_yh2o
 double precision, dimension(:), pointer :: cpro_rom1, cpro_csec
 
@@ -257,9 +257,9 @@ enddo
 ! 4. Mass transfert by heterogeneous combustion with O2
 !===============================================================================
 
-call field_get_val_s(iym1(io2),cpro_yox)
-call field_get_val_s(itemp1,cpro_temp1)
-call field_get_val_s(irom1,cpro_rom1)
+call field_get_val_s(iym1(io2), cpro_yox)
+call field_get_val_s(itemp, cpro_temp)
+call field_get_val_s(irom1, cpro_rom1)
 
 do icla = 1, nclacp
 
@@ -279,7 +279,7 @@ do icla = 1, nclacp
     !                                                 ---
     !       PO2 = RHO1*CS_PHYSICAL_CONSTANTS_R*T*YO2/MO2
 
-    pparo2 = rho1(iel)*cs_physical_constants_r*cpro_temp1(iel) &
+    pparo2 = rho1(iel)*cs_physical_constants_r*cpro_temp(iel) &
             *cpro_yox(iel)/wmole(io2)
     pparo2 = pparo2 / prefth
 
@@ -356,7 +356,7 @@ if ( ihtco2 .eq. 1) then
       !                                                 ---
       !       PCO2 = RHO1*CS_PHYSICAL_CONSTANTS_R*T*YCO2/MCO2
 
-      pprco2 = rho1(iel)*cs_physical_constants_r*cpro_temp1(iel)  &
+      pprco2 = rho1(iel)*cs_physical_constants_r*cpro_temp(iel)  &
                            *cpro_yco2(iel)/wmole(ico2)
       pprco2 = pprco2 / prefth
 
@@ -435,7 +435,7 @@ if ( ihth2o .eq. 1) then
       !                                                 ---
       !       PH2O = RHO1*CS_PHYSICAL_CONSTANTS_R*T*YH2O/MH2O
 
-      pprh2o = rho1(iel)*cs_physical_constants_r*cpro_temp1(iel) &
+      pprh2o = rho1(iel)*cs_physical_constants_r*cpro_temp(iel) &
               *cpro_yh2o(iel)/wmole(ih2o)
       pprh2o = pprh2o/ prefth
 
@@ -512,6 +512,8 @@ if ( ippmod(iccoal) .ge. 1 ) then
 
   if (icp.ge.0) call field_get_val_s(icp, cpro_cp)
 
+  call field_get_key_double(ivarfl(isca(iscalt)), kvisl0, visls_0)
+
   do iel = 1, ncel
     if (ifcvsl.ge.0) then
       if (icp.ge.0) then
@@ -521,14 +523,14 @@ if ( ippmod(iccoal) .ge. 1 ) then
       endif
     else
       if (icp.ge.0) then
-        w1(iel) = visls0(iscalt) * cpro_cp(iel)
+        w1(iel) = visls_0 * cpro_cp(iel)
       else
-        w1(iel) = visls0(iscalt) * cp0
+        w1(iel) = visls_0 * cp0
       endif
     endif
   enddo
 
-  if(ntlist.gt.0) then
+  if (ntlist.gt.0) then
     modntl = mod(ntcabs,ntlist)
   elseif(ntlist.eq.-1.and.ntcabs.eq.ntmabs) then
     modntl = 0
