@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -147,7 +147,7 @@ double precision , dimension ( : , : ) , allocatable :: af1    , af2
 double precision , dimension ( : )     , allocatable :: fs3no  , fs4no
 double precision , dimension ( : , : ) , allocatable :: yfs4no
 double precision, allocatable, dimension(:) :: tpdf
-double precision, dimension(:), pointer :: cpro_x1, cpro_temp1, cpro_mmel
+double precision, dimension(:), pointer :: cpro_x1, cpro_temp, cpro_mmel
 double precision, dimension(:), pointer :: cvar_xchcl, cvar_xckcl, cvar_xnpcl
 double precision, dimension(:), pointer :: cvar_xwtcl
 double precision, dimension(:), pointer :: cpro_cyf1, cpro_cyf2, cpro_cyf3
@@ -167,6 +167,23 @@ type(pmapper_double_r1), dimension(:), allocatable :: cvar_f1m, cvar_f2m
 integer          ipass
 data ipass / 0 /
 
+!===============================================================================
+! Interfaces
+!===============================================================================
+
+interface
+
+  subroutine cs_coal_thfieldconv1(location_id, eh, tp) &
+    bind(C, name='cs_coal_thfieldconv1')
+
+    use, intrinsic :: iso_c_binding
+    implicit none
+    integer(c_int), value :: location_id
+    real(kind=c_double), dimension(*) :: eh, tp
+  end subroutine cs_coal_thfieldconv1
+
+end interface
+
 !===============================================================================
 ! 0. Memory allocation
 !===============================================================================
@@ -248,7 +265,6 @@ enddo
 allocate(tpdf(ncelet))
 
 call pppdfr &
-!==========
  ( ncelet , ncel  , intpdf ,                                      &
    tpdf   ,                                                       &
    ffuel  , fvp2m ,                                               &
@@ -361,8 +377,7 @@ enddo
 deallocate(cvar_f1m, cvar_f2m)
 
 call cs_gascomb &
-!==============
- ( ncelet , ncel   , ichx1 , ichx2 ,                              &
+ ( ncel   , ichx1 , ichx2 ,                                       &
    intpdf ,                                                       &
    f1m    , f2m , f3m , f4m , f5m , f6m , f7m , f8m , f9m ,       &
    pdfm1  , pdfm2  , doxyd    , dfuel  , hrec ,                   &
@@ -402,10 +417,10 @@ enddo
 ! 4. Calculation of temperature and density
 !===============================================================================
 
-call field_get_val_s(itemp1,cpro_temp1)
+call field_get_val_s(itemp,cpro_temp)
 call field_get_val_s(immel,cpro_mmel)
 
-call cs_coal_thfieldconv1(MESH_LOCATION_CELLS, enth, cpro_temp1)
+call cs_coal_thfieldconv1(MESH_LOCATION_CELLS, enth, cpro_temp)
 
 do iel = 1, ncel
   wmolme = cpro_cyf1(iel)/wmchx1(iel)                         &
@@ -428,7 +443,7 @@ do iel = 1, ncel
   ! ---- We do not include the mecanical pressure IPR
   !      but P0
 
-  rom1(iel) = p0 / (wmolme*cs_physical_constants_r*cpro_temp1(iel))
+  rom1(iel) = p0 / (wmolme*cs_physical_constants_r*cpro_temp(iel))
 enddo
 
 !===============================================================================
@@ -441,8 +456,7 @@ if ( ieqnox .eq. 1 .and. ipass .gt. 1 ) then
 
   call cs_coal_noxst &
 
- ( ncelet , ncel   ,                                              &
-   intpdf ,                                                       &
+ ( ncel   , intpdf ,                                              &
    pdfm1  , pdfm2  , doxyd  , dfuel  , hrec ,                     &
    f3m    , f4m    , f5m    , f6m    , f7m  , f8m , f9m ,         &
    fs3no  , fs4no  , yfs4no , enthox )
