@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -42,12 +42,10 @@
 !______________________________________________________________________________.
 !  mode           name          role
 !______________________________________________________________________________!
-!> \param[in]     ncelet        number of extended (real + ghost) cells
 !> \param[in]     ncel          number of cells
 !______________________________________________________________________________!
 
-subroutine cs_coal_physprop2 &
- ( ncelet , ncel )
+subroutine cs_coal_physprop2(ncel)
 
 !===============================================================================
 ! Module files
@@ -147,9 +145,9 @@ do icla = 1, nclacp
     call field_get_val_s(ivarfl(isca(ixwt(icla))), cvar_xwtcl)
   endif
 
-  call field_get_val_s(iprpfl(ix2(icla)),cpro_x2)
-  call field_get_val_s(iprpfl(irom2(icla)),cpro_rom2)
-  call field_get_val_s(iprpfl(idiam2(icla)),cpro_diam2)
+  call field_get_val_s(ix2(icla),cpro_x2)
+  call field_get_val_s(irom2(icla),cpro_rom2)
+  call field_get_val_s(idiam2(icla),cpro_diam2)
   xashcl = xashch(ichcor(icla))
 
   do iel = 1, ncel
