@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -62,8 +62,7 @@
 !______________________________________________________________________________!
 
 subroutine cs_coal_scast &
- ( iscal  ,                                                       &
-   smbrs  , rovsdt )
+ (iscal, smbrs, rovsdt)
 
 !===============================================================================
 ! Module files
@@ -130,7 +129,7 @@ double precision err1mx,err2mx
 double precision errch,xden
 double precision fn0,fn1,fn2,anmr0,anmr1,anmr2
 double precision lnk0p,l10k0e,lnk0m,t0e,xco2eq,xcoeq,xo2eq
-double precision xcom,xo2m,xkcequ,xkpequ
+double precision xcom,xo2m,xkcequ,xkpequ,visls_0
 
 double precision xw1,char_formation,exp_st,imp_st
 double precision xo2,wmel,wmhcn,wmno,wmo2,wmnh3
@@ -160,7 +159,7 @@ double precision, dimension(:), pointer :: cvar_xwtcl, cvara_xwtcl
 double precision, dimension(:), pointer :: cvar_yno, cvara_yno
 double precision, dimension(:), pointer :: cvara_yhcn, cvara_ynh3
 double precision, dimension(:), pointer :: cvara_var
-double precision, dimension(:), pointer :: cpro_temp1, cpro_temp2, cpro_rom2
+double precision, dimension(:), pointer :: cpro_temp, cpro_temp2, cpro_rom2
 double precision, dimension(:), pointer :: cpro_diam2, cpro_cgd1, cpro_cgd2
 double precision, dimension(:), pointer :: cpro_cght, cpro_yox, cpro_yco2
 double precision, dimension(:), pointer :: cpro_yco, cpro_yh2o, cpro_rom1
@@ -196,9 +195,8 @@ double precision mckcl1, mckcl2
 double precision, dimension(:), pointer :: dt
 
 type(var_cal_opt) :: vcopt
+logical(kind=c_bool) :: log_active
 
-!===============================================================================
-!
 !===============================================================================
 ! 1. Initialization
 !===============================================================================
@@ -214,7 +212,7 @@ call field_get_label(ivarfl(ivar), chaine)
 call field_get_val_s(icrom, crom)
 if (icp.ge.0) call field_get_val_s(icp, cpro_cp)
 
-call field_get_val_s(itemp1, cpro_temp1)
+call field_get_val_s(itemp, cpro_temp)
 call field_get_val_s(irom1, cpro_rom1)
 
 call field_get_val_s(iym1(io2), cpro_yox)
@@ -241,6 +239,8 @@ if ( ieqnox .eq. 1 .and. ntcabs .gt. 1) then
   call field_get_val_prev_s(ivarfl(isca(iynh3)), cvara_ynh3)
 endif
 
+log_active = cs_log_default_is_active()
+
 !===============================================================================
 ! Deallocation dynamic arrays
 !----
@@ -266,7 +266,7 @@ call field_get_val_s_by_name('dt', dt)
 
 ! --> Source term for the mass fraction of reactive coal
 
-if ( ivar.ge.isca(ixch(1)) .and. ivar.le.isca(ixch(nclacp)) ) then
+if (ivar.ge.isca(ixch(1)) .and. ivar.le.isca(ixch(nclacp))) then
 
   if (vcopt%iwarni.ge.1) then
     write(nfecra,1000) chaine(1:8)
@@ -296,7 +296,7 @@ endif
 ! 2.1 Source term for the mass fraction of coke
 !===============================================================================
 
-if ( ivar.ge.isca(ixck(1)) .and. ivar.le.isca(ixck(nclacp)) ) then
+if (ivar.ge.isca(ixck(1)) .and. ivar.le.isca(ixck(nclacp))) then
 
   if (vcopt%iwarni.ge.1) then
     write(nfecra,1000) chaine(1:8)
@@ -320,14 +320,13 @@ if ( ivar.ge.isca(ixck(1)) .and. ivar.le.isca(ixck(nclacp)) ) then
     call field_get_val_s(ighh2o(numcla), cpro_ghh2o)
   endif
 
-!
   do iel = 1, ncel
     exp_st = 0.d0
     ! volatile formation minus coal consuming = char formation
     ! (Coke formation in French)
     ! NB: we take values at current and not previous time step
     !     to be conservative in mass
-    char_formation = crom(iel)*cvar_xchcl(iel)*cell_f_vol(iel)                     &
+    char_formation = crom(iel)*cvar_xchcl(iel)*cell_f_vol(iel)      &
                    *(cpro_cgd1(iel)+cpro_cgd2(iel)-cpro_cgch(iel))
 
     ! Compute the implict part of the Source term
@@ -342,7 +341,8 @@ if ( ivar.ge.isca(ixck(1)) .and. ivar.le.isca(ixck(nclacp)) ) then
       ! Reaction C(s) + H2O ---> CO + H2
       if (ihth2o .eq. 1) exp_st = exp_st + cpro_ghh2o(iel)
 
-      exp_st = -2.d0/3.d0 * crom(iel) * exp_st * cell_f_vol(iel) / cvara_xckcl(iel)**(1.d0/3.d0)
+      exp_st = -2.d0/3.d0 * crom(iel) * exp_st   &
+                          * cell_f_vol(iel) / cvara_xckcl(iel)**(1.d0/3.d0)
     endif
 
     ! Compute the explicit part of the Source term
@@ -360,7 +360,7 @@ endif
 ! 2.2 Source term for the mass fraction of water
 !===============================================================================
 
-if ( ippmod(iccoal) .eq. 1 ) then
+if (ippmod(iccoal) .eq. 1) then
 
   if (ivar.ge.isca(ixwt(1)) .and. ivar.le.isca(ixwt(nclacp))) then
 
@@ -382,7 +382,7 @@ if ( ippmod(iccoal) .eq. 1 ) then
 
      if ( cvara_var(iel).gt. epsicp .and.                         &
           xwatch(numcha).gt. epsicp       ) then
-       xw1 = crom(iel)*cpro_csec(iel)*cell_f_vol(iel)                         &
+       xw1 = crom(iel)*cpro_csec(iel)*cell_f_vol(iel)             &
             *(1.d0/cpro_x2(iel))*(1.d0/xwatch(numcha))
 
        rovsdt(iel) = rovsdt(iel) + max(xw1,zero)
@@ -480,7 +480,7 @@ if (i_comb_drift.eq.1) then
         ! For H2O gasification, one H2O comes in and one CO and one H2 get out
         smbrs1 = 0.d0
         if (cvara_coke(iel).gt.epsicp) then
-          smbrs1                 = smbrs1 + wmole(io2 )/wmolat(iatc)*cpro_cght(iel)
+          smbrs1                  = smbrs1 + wmole(io2 )/wmolat(iatc)*cpro_cght(iel)
           if (ihtco2.eq.1) smbrs1 = smbrs1 + wmole(ico2)/wmolat(iatc)*cpro_ghco2(iel)
           if (ihth2o.eq.1) smbrs1 = smbrs1 + wmole(ih2o)/wmolat(iatc)*cpro_ghh2o(iel)
           smbrs1                 = smbrs1 * cvara_coke(iel)**(2.d0/3.d0)
@@ -621,6 +621,8 @@ if ((ivar.ge.isca(ih2(1)) .and. ivar.le.isca(ih2(nclacp)))) then
     call field_get_val_s(ifcvsl, cpro_viscls)
   endif
 
+  call field_get_key_double(ivarfl(isca(iscalt)), kvisl0, visls_0)
+
   do iel = 1, ncel
     if (ifcvsl.ge.0) then
       if (icp.ge.0) then
@@ -630,9 +632,9 @@ if ((ivar.ge.isca(ih2(1)) .and. ivar.le.isca(ih2(nclacp)))) then
       endif
     else
       if (icp.ge.0) then
-        w1(iel) = visls0(iscalt) * cpro_cp(iel)
+        w1(iel) = visls_0 * cpro_cp(iel)
       else
-        w1(iel) = visls0(iscalt) * cp0
+        w1(iel) = visls_0 * cp0
       endif
     endif
   enddo
@@ -651,8 +653,8 @@ if ((ivar.ge.isca(ih2(1)) .and. ivar.le.isca(ih2(nclacp)))) then
         / cpro_rom2(iel) * crom(iel)       &
         * cell_f_vol(iel)
 
-    smbrs(iel)  = smbrs(iel)-aux*(cpro_temp2(iel)-cpro_temp1(iel))*cpro_x2(iel)
-    smbrsh1(iel) = smbrsh1(iel)+aux*(cpro_temp2(iel)-cpro_temp1(iel))*cpro_x2(iel)
+    smbrs(iel)  = smbrs(iel)-aux*(cpro_temp2(iel)-cpro_temp(iel))*cpro_x2(iel)
+    smbrsh1(iel) = smbrsh1(iel)+aux*(cpro_temp2(iel)-cpro_temp(iel))*cpro_x2(iel)
 
     ! Store the implicite part of the exchange so that we can compute a
     ! conservative exhcange term when computing the gas enthalpy
@@ -768,7 +770,7 @@ if ((ivar.ge.isca(ih2(1)) .and. ivar.le.isca(ih2(nclacp)))) then
       f2mc(icha) = zero
     enddo
 
-    t1        = cpro_temp1(iel)
+    t1        = cpro_temp(iel)
     mode      = -1
     call cs_coal_htconvers1 &
     ( mode , xho2 , coefe , f1mc , f2mc , t1 )
@@ -825,7 +827,7 @@ if ((ivar.ge.isca(ih2(1)) .and. ivar.le.isca(ih2(nclacp)))) then
         f2mc(icha) = zero
       enddo
 
-      t1        = cpro_temp1(iel)
+      t1        = cpro_temp(iel)
       mode      = -1
       call cs_coal_htconvers1 &
       !======================
@@ -900,7 +902,7 @@ if ((ivar.ge.isca(ih2(1)) .and. ivar.le.isca(ih2(nclacp)))) then
         f2mc(icha) = zero
       enddo
 
-      t1        = cpro_temp1(iel)
+      t1        = cpro_temp(iel)
       mode      = -1
       call cs_coal_htconvers1 &
       ( mode , xhh2o , coefe , f1mc , f2mc , t1    )
@@ -1282,7 +1284,7 @@ if ( ieqco2 .eq. 1 ) then
       call field_get_val_s(ix2(icla),cpro_x2c(icla)%p)
     enddo
 
-    ! ---- Contribution of interfacial source term to explicit and implicit balances
+    ! Contribution of interfacial source term to explicit and implicit balances
 
     ! Oxydation of CO
     ! ===============
@@ -1356,12 +1358,12 @@ if ( ieqco2 .eq. 1 ) then
      xxh2o = max(xxh2o,zero)
      sqh2o = sqrt(xxh2o)
 
-     xkp = exp(lnk0p-t0p/cpro_temp1(iel))
-     xkm = exp(lnk0m-t0m/cpro_temp1(iel))
+     xkp = exp(lnk0p-t0p/cpro_temp(iel))
+     xkm = exp(lnk0m-t0m/cpro_temp(iel))
 
-     xkpequ = 10.d0**(l10k0e-t0e/cpro_temp1(iel))
+     xkpequ = 10.d0**(l10k0e-t0e/cpro_temp(iel))
      xkcequ = xkpequ                                              &
-             /sqrt(8.32d0*cpro_temp1(iel)/1.015d5)
+             /sqrt(8.32d0*cpro_temp(iel)/1.015d5)
 
      !        initialization by the transported state
 
@@ -1369,7 +1371,7 @@ if ( ieqco2 .eq. 1 ) then
      xcom  = xxco + xxco2
      xo2m  = xxo2 + 0.5d0*xxco2
 
-     if ( cpro_temp1(iel) .gt. 1200.d0 ) then
+     if ( cpro_temp(iel) .gt. 1200.d0 ) then
 
       !        Search for the equilibrum state
       !        Iterative search without control of convergence
@@ -1493,9 +1495,11 @@ if ( ieqco2 .eq. 1 ) then
      call parcmx(nbimax)
    endif
 
-   write(nfecra,*) ' Max Error = ',ERR1MX
-   write(nfecra,*) ' no Points   ',NBERIC,NBARRE,NBPASS
-   write(nfecra,*) ' Iter max number ',NBIMAX
+   if (log_active .eqv. .true.) then
+     write(nfecra,*) ' Max Error = ', err1mx
+     write(nfecra,*) ' no Points   ', nberic, nbarre, nbpass
+     write(nfecra,*) ' Iter max number ', nbimax
+   endif
 
    !     Source term: heterogeneous combustion by CO2
 
@@ -1618,7 +1622,7 @@ if ( ieqnox .eq. 1 .and. ntcabs .gt. 1) then
         tfuel(iel) = tfuel(iel)/xmx2
 
       else
-        tfuel(iel) = cpro_temp1(iel)
+        tfuel(iel) = cpro_temp(iel)
       endif
       tfuelmin = min(tfuel(iel),tfuelmin)
       tfuelmax = max(tfuel(iel),tfuelmax)
@@ -1627,7 +1631,9 @@ if ( ieqnox .eq. 1 .and. ntcabs .gt. 1) then
       call parmin(tfuelmin)
       call parmax(tfuelmax)
     endif
-    write(nfecra,*) ' Min max de Tfuel pour Hoxy ',tfuelmin,tfuelmax
+    if (log_active .eqv. .true.) then
+      write(nfecra,*) ' Min max of Tfuel for Hoxy ', tfuelmin, tfuelmax
+    endif
 
     ! Heterogeneous combustion: C + O2 ---> 0.5 CO
 
@@ -1658,7 +1664,7 @@ if ( ieqnox .eq. 1 .and. ntcabs .gt. 1) then
         f1mc(icha) = zero
         f2mc(icha) = zero
       enddo
-      t1        = cpro_temp1(iel)
+      t1        = cpro_temp(iel)
       mode      = -1
       call cs_coal_htconvers1 &
       ( mode  , xho2    , coefe  , f1mc   , f2mc   , t1    )
@@ -1714,7 +1720,7 @@ if ( ieqnox .eq. 1 .and. ntcabs .gt. 1) then
           f1mc(icha) = zero
           f2mc(icha) = zero
         enddo
-        t1        = cpro_temp1(iel)
+        t1        = cpro_temp(iel)
         mode      = -1
         call cs_coal_htconvers1 &
         ( mode  , xhco2    , coefe  , f1mc   , f2mc   , t1    )
@@ -1788,7 +1794,7 @@ if ( ieqnox .eq. 1 .and. ntcabs .gt. 1) then
           f1mc(icha) = zero
           f2mc(icha) = zero
         enddo
-        t1        = cpro_temp1(iel)
+        t1        = cpro_temp(iel)
         mode      = -1
         call cs_coal_htconvers1 &
       ( mode  , xhh2o    , coefe  , f1mc   , f2mc   , t1    )
@@ -2132,7 +2138,7 @@ if ( ieqnox .eq. 1 .and. imdnox.eq.1 .and. ntcabs .gt. 1) then
               do ii = 1,7
 
                 !  We look for the interval teno(ii) < Tgaz < teno(ii+1)
-                if(cpro_temp1(iel).ge.teno(ii).and.cpro_temp1(iel).lt.   &
+                if(cpro_temp(iel).ge.teno(ii).and.cpro_temp(iel).lt.   &
                    teno(ii+1)) then
 
                   !  JJ indicates the quotient H/C of the fuel (4=CH4;3=CH3,etc.)
@@ -2142,31 +2148,31 @@ if ( ieqnox .eq. 1 .and. imdnox.eq.1 .and. ntcabs .gt. 1) then
                     if(chx1(icha).ge.4.d0) then
 
                     core1 = ka(4,ii) + ( (ka(4,ii+1)- ka(4,ii))/(teno(ii+1)-   &
-                            teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                            teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                     core2 = kb(4,ii) + ( (kb(4,ii+1)- kb(4,ii))/(teno(ii+1)-   &
-                            teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                            teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                     para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii))/(teno(ii+1) -   &
-                            teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                            teno(ii)) ) * (cpro_temp(iel) - teno(ii))
 
                     elseif(chx1(icha).le.1.d0) then
 
                     core1 = ka(1,ii) + ( (ka(1,ii+1)- ka(1,ii))/(teno(ii+1)-   &
-                            teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                            teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                     core2 = kb(1,ii) + ( (kb(1,ii+1)- kb(1,ii))/(teno(ii+1)-   &
-                            teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                            teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                     para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii))/(teno(ii+1) -   &
-                            teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                            teno(ii)) ) * (cpro_temp(iel) - teno(ii))
 
                     elseif (chx1(icha).ge.jj.and.chx1(icha).lt.jj+1) then
 
                     core1 = ka(jj,ii) + ( (ka(jj+1,ii+1)- ka(jj,ii))/          &
-                            (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel)      &
+                            (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel)      &
                             - teno(ii))
                     core2 = kb(jj,ii) + ( (kb(jj+1,ii+1)- kb(jj,ii))/          &
-                            (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                            (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                             teno(ii))
                     para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii))/(teno(ii+1)-    &
-                            teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                            teno(ii)) ) * (cpro_temp(iel) - teno(ii))
 
                     endif
 
@@ -2204,7 +2210,7 @@ if ( ieqnox .eq. 1 .and. imdnox.eq.1 .and. ntcabs .gt. 1) then
               do ii = 1,7
 
                 !  We look for the interval teno(ii) < Tgaz < teno(ii+1)
-                if (cpro_temp1(iel).ge.teno(ii).and.cpro_temp1(iel).lt.   &
+                if (cpro_temp(iel).ge.teno(ii).and.cpro_temp(iel).lt.   &
                    teno(ii+1)) then
 
                   !  JJ indicates the quotient H/C of
@@ -2215,31 +2221,31 @@ if ( ieqnox .eq. 1 .and. imdnox.eq.1 .and. ntcabs .gt. 1) then
                     if(chx2(icha).ge.4.d0) then
 
                       core1 = ka(4,ii) + ( (ka(4,ii+1)- ka(4,ii))/(teno(ii+1)-   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                       core2 = kb(4,ii) + ( (kb(4,ii+1)- kb(4,ii))/(teno(ii+1)-   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                       para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii))/(teno(ii+1) -   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
 
                     elseif(chx2(icha).le.1.d0) then
 
                       core1 = ka(1,ii) + ( (ka(1,ii+1)- ka(1,ii))/(teno(ii+1)-   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                       core2 = kb(1,ii) + ( (kb(1,ii+1)- kb(1,ii))/(teno(ii+1)-   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                       para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii))/(teno(ii+1) -   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
 
                     elseif (chx2(icha).ge.jj.and.chx2(icha).lt.jj+1) then
 
                       core1 = ka(jj,ii) + ( (ka(jj+1,ii+1)- ka(jj,ii))/          &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       core2 = kb(jj,ii) + ( (kb(jj+1,ii+1)- kb(jj,ii))/          &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii))/(teno(ii+1) -   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
 
                     endif
 
@@ -2508,7 +2514,7 @@ if ( ieqnox .eq. 1 .and. imdnox.eq.1 .and. ntcabs .gt. 1) then
               do ii = 1,7
 
                 !  We look for the interval teno(ii) < Tgaz < teno(ii+1)
-                if(cpro_temp1(iel).ge.teno(ii).and.cpro_temp1(iel).lt.   &
+                if(cpro_temp(iel).ge.teno(ii).and.cpro_temp(iel).lt.   &
                    teno(ii+1)) then
 
                   !  JJ indicates the quotient H/C of the fuel (4=CH4;3=CH3,etc.)
@@ -2518,41 +2524,41 @@ if ( ieqnox .eq. 1 .and. imdnox.eq.1 .and. ntcabs .gt. 1) then
                     if(chx1(icha).ge.4.d0) then
 
                       core1 = ka(4,ii) + ( (ka(4,ii+1)- ka(4,ii))/(teno(ii+1)-   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                       core2 = kb(4,ii) + ( (kb(4,ii+1)- kb(4,ii))/(teno(ii+1)-   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                       core3 = kc(4,ii) + ( (kc(4,ii+1)- kc(4,ii))/(teno(ii+1)-   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                       para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii))/(teno(ii+1) -   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
 
                     elseif(chx1(icha).le.1.d0) then
 
                       core1 = ka(1,ii) + ( (ka(1,ii+1)- ka(1,ii))/               &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel)  -   &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel)  -   &
                               teno(ii))
                       core2 = kb(1,ii) + ( (kb(1,ii+1)- kb(1,ii))/(teno(ii+1)-   &
-                              teno(ii)) ) * (cpro_temp1(iel) - teno(ii))
+                              teno(ii)) ) * (cpro_temp(iel) - teno(ii))
                       core3 = kc(1,ii) + ( (kc(1,ii+1)- kc(1,ii))/               &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii)) /               &
-                              (teno(ii+1) - teno(ii)) ) * (cpro_temp1(iel) -  &
+                              (teno(ii+1) - teno(ii)) ) * (cpro_temp(iel) -  &
                               teno(ii))
 
                     elseif (chx1(icha).ge.jj.and.chx1(icha).lt.jj+1) then
 
                       core1 = ka(jj,ii) + ( (ka(jj+1,ii+1)- ka(jj,ii))/          &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       core2 = kb(jj,ii) + ( (kb(jj+1,ii+1)- kb(jj,ii))/          &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       core3 = kc(jj,ii) + ( (kc(jj+1,ii+1)- kc(jj,ii))/          &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii))/                &
-                              (teno(ii+1) - teno(ii)) ) * (cpro_temp1(iel) -  &
+                              (teno(ii+1) - teno(ii)) ) * (cpro_temp(iel) -  &
                               teno(ii))
 
                     endif
@@ -2590,7 +2596,7 @@ if ( ieqnox .eq. 1 .and. imdnox.eq.1 .and. ntcabs .gt. 1) then
               do ii = 1,7
 
                 !  We look for the interval teno(ii) < Tgaz < teno(ii+1)
-                if(cpro_temp1(iel).ge.teno(ii).and.cpro_temp1(iel).lt.   &
+                if(cpro_temp(iel).ge.teno(ii).and.cpro_temp(iel).lt.   &
                    teno(ii+1)) then
 
                   !  JJ incates the quotient H/C of the fuel (4=CH4;3=CH3,etc.)
@@ -2600,45 +2606,45 @@ if ( ieqnox .eq. 1 .and. imdnox.eq.1 .and. ntcabs .gt. 1) then
                     if(chx2(icha).ge.4.d0) then
 
                       core1 = ka(4,ii) + ( (ka(4,ii+1)- ka(4,ii))/               &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       core2 = kb(4,ii) + ( (kb(4,ii+1)- kb(4,ii))/               &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       core3 = kc(4,ii) + ( (kc(4,ii+1)- kc(4,ii))/               &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii))/                &
-                              (teno(ii+1) - teno(ii)) ) * (cpro_temp1(iel) -  &
+                              (teno(ii+1) - teno(ii)) ) * (cpro_temp(iel) -  &
                               teno(ii))
 
                     elseif(chx2(icha).le.1.d0) then
 
                       core1 = ka(1,ii) + ( (ka(1,ii+1)- ka(1,ii))/(teno(ii+1)-   &
-                              teno(ii)) ) * (cpro_temp1(iel) -                &
+                              teno(ii)) ) * (cpro_temp(iel) -                &
                               teno(ii))
                       core2 = kb(1,ii) + ( (kb(1,ii+1)- kb(1,ii))/               &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       core3 = kc(1,ii) + ( (kc(1,ii+1)- kc(1,ii))/               &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii)) /               &
-                              (teno(ii+1) - teno(ii)) ) * (cpro_temp1(iel) -  &
+                              (teno(ii+1) - teno(ii)) ) * (cpro_temp(iel) -  &
                               teno(ii))
                     elseif (chx2(icha).ge.jj.and.chx2(icha).lt.jj+1) then
 
                       core1 = ka(jj,ii) + ( (ka(jj+1,ii+1)- ka(jj,ii))/          &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       core2 = kb(jj,ii) + ( (kb(jj+1,ii+1)- kb(jj,ii))/          &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       core3 = kc(jj,ii) + ( (kc(jj+1,ii+1)- kc(jj,ii))/          &
-                              (teno(ii+1)-teno(ii)) ) * (cpro_temp1(iel) -    &
+                              (teno(ii+1)-teno(ii)) ) * (cpro_temp(iel) -    &
                               teno(ii))
                       para2 = chi2(ii) + ( (chi2(ii+1)-chi2(ii))/                &
-                              (teno(ii+1) - teno(ii)) ) * (cpro_temp1(iel) -  &
+                              (teno(ii+1) - teno(ii)) ) * (cpro_temp(iel) -  &
                               teno(ii))
 
                     endif
@@ -2745,22 +2751,12 @@ endif
 ! End
 !----
 
-
-! Deallocation dynamic arrays
-deallocate(w1,w3,w4,w5,stat=iok1)
+! Deallocate work arrays
+deallocate(w1, w3, w4, w5, tfuel, stat=iok1)
 
 if (iok1 > 0) then
-  write(nfecra,*) ' Memory deallocation error inside: '
-  write(nfecra,*) '      cs_coal_scast                '
+  write(nfecra,*) ' cs_coal_scast: memory deallocation error'
   call csexit(1)
 endif
-deallocate(tfuel, stat=iok1)
-if (iok1 > 0) then
-  write(nfecra,*) ' Memory deallocation error inside: '
-  write(nfecra,*) '      cs_coal_scast                '
-  call csexit(1)
-endif
-
-return
 
 end subroutine
