@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,11 +20,7 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine cs_coal_varini &
-!========================
-
- ( nvar   , nscal  ,                                            &
-   dt     )
+subroutine cs_coal_varini
 
 !===============================================================================
 ! FONCTION :
@@ -56,15 +52,8 @@ subroutine cs_coal_varini &
 !__________________.____._____.________________________________________________.
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
-! nvar             ! i  ! <-- ! total number of variables                      !
-! nscal            ! i  ! <-- ! total number of scalars                        !
-! dt(ncelet)       ! tr ! <-- ! valeur du pas de temps                         !
 !__________________!____!_____!________________________________________________!
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
 !===============================================================================
 
 !===============================================================================
@@ -91,10 +80,6 @@ use field
 
 implicit none
 
-integer          nvar   , nscal
-
-double precision dt(ncelet)
-
 ! Local variables
 
 integer          iel, ige, mode, icla, icha, ifac
@@ -383,15 +368,6 @@ if (ipass.eq.1) then
     bpro_x1(ifac) = 1.d0
   enddo
 
-!===============================================================================
-! 3. User initialization
-!===============================================================================
-
-  call cs_user_f_initialization &
-  !==========================
-( nvar   , nscal  ,                                            &
-  dt     )
-
 endif
 
 !--------
