@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -56,7 +56,6 @@ use coincl
 use cpincl
 use ppincl
 use ppcpfu
-use ihmpre
 use cs_coal_incl
 use field
 
@@ -83,13 +82,6 @@ interface
     integer(c_int), value        :: n_coals, n_classes
   end subroutine cs_field_pointer_map_coal_combustion
 
-  subroutine cs_gui_labels_coal_combustion(n_coals, n_classes)  &
-    bind(C, name='cs_gui_labels_coal_combustion')
-    use, intrinsic :: iso_c_binding
-    implicit none
-    integer(c_int), value        :: n_coals, n_classes
-  end subroutine cs_gui_labels_coal_combustion
-
 end interface
 
 !===============================================================================
@@ -676,7 +668,7 @@ if (ieqnox.eq.1) then
 endif
 
 ! Age of bulk
-!FIXME give the possibility of having the age seperately
+!FIXME give the possibility of having the age separately
 if (i_comb_drift.ge.1) then
 
   f_name = 'age'
@@ -696,12 +688,6 @@ n_classes = nclacp
 
 call cs_field_pointer_map_coal_combustion(n_coals, n_classes)
 
-! Map labels for GUI
-
-if (iihmpr.eq.1) then
-  call cs_gui_labels_coal_combustion(n_coals, n_classes)
-endif
-
 !===============================================================================
 ! 2. PROPRIETES PHYSIQUES
 !    A RENSEIGNER OBLIGATOIREMENT (sinon pb dans varpos)
