@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -86,10 +86,10 @@ double precision smbrs(ncelet), rovsdt(ncelet)
 
 integer           iel    , ifac   , f_id0
 integer           icla
-integer           inc    , iccocg , nswrgp , imligp , iwarnp
+integer           inc    , iccocg , imrgrp , nswrgp , imligp , iwarnp
 
 double precision xk     , xe     , rhovst
-double precision epsrgp , climgp , extrap
+double precision epsrgp , climgp
 double precision aux
 double precision gvap , t2mt1
 double precision turb_schmidt
@@ -99,7 +99,7 @@ double precision, dimension(:), allocatable :: x1,f1f2
 double precision, dimension(:), allocatable :: coefap , coefbp
 double precision, allocatable, dimension(:,:) :: grad
 double precision, dimension(:), pointer ::  crom, cpro_rom1
-double precision, dimension(:), pointer ::  cpro_temp1, cpro_temp2, cpro_gmeva
+double precision, dimension(:), pointer ::  cpro_temp, cpro_temp2, cpro_gmeva
 double precision, dimension(:), pointer :: visct
 double precision, dimension(:), pointer :: cvara_k, cvara_ep, cvara_omg
 double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
@@ -159,12 +159,12 @@ if ( itytur.eq.2 .or. iturb.eq.50 .or.             &
 !
 
   call field_get_key_struct_var_cal_opt(ivarfl(isca(ifvap)), vcopt)
+  imrgrp = vcopt%imrgra
   nswrgp = vcopt%nswrgr
   imligp = vcopt%imligr
   iwarnp = vcopt%iwarni
   epsrgp = vcopt%epsrgr
   climgp = vcopt%climgr
-  extrap = vcopt%extrag
   call field_set_key_struct_var_cal_opt(ivarfl(isca(ifvap)), vcopt)
 
 ! --> calcul de X1
@@ -207,8 +207,8 @@ if ( itytur.eq.2 .or. iturb.eq.50 .or.             &
 !              n'est pas Rij)
   f_id0  = -1
   call gradient_s                                                 &
- ( f_id0  , imrgra , inc    , iccocg , nswrgp , imligp ,          &
-   iwarnp , epsrgp , climgp , extrap ,                            &
+ ( f_id0  , imrgrp , inc    , iccocg , nswrgp , imligp ,          &
+   iwarnp , epsrgp , climgp ,                                     &
    f1f2   , coefap , coefbp ,                                     &
    grad   )
 
@@ -241,14 +241,14 @@ endif
 !==============================================================================
 
 call field_get_val_s(icrom, crom)
-call field_get_val_s(itemp1,cpro_temp1)
+call field_get_val_s(itemp, cpro_temp)
 do icla=1,nclafu
 !
   call field_get_val_s(itemp2(icla),cpro_temp2)
   call field_get_val_s(igmeva(icla),cpro_gmeva)
   do iel = 1, ncel
 !
-    t2mt1  = cpro_temp2(iel)-cpro_temp1(iel)
+    t2mt1  = cpro_temp2(iel)-cpro_temp(iel)
     gvap = -cpro_gmeva(iel)*t2mt1*crom(iel)
 !
     aux  = gvap*(1.d0-f1f2(iel))**2.d0
