@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -191,8 +191,7 @@ endif
 !    Mass density
 !===============================================================================
 
-call cs_fuel_physprop2 ( ncelet , ncel )
-!=====================
+call cs_fuel_physprop2(ncel)
 
 !===============================================================================
 ! 3. Calculation of the physical properties of the gaseous phase
@@ -225,7 +224,7 @@ f2m( : ) = 0.d0
 f3m( : ) = 0.d0
 f4m( : ) = 0.d0
 f5m( : ) = 0.d0
-! Water vapour
+! Water vapor
 f6m( : ) = 0.d0
 ! Heterogeneous combustion
 f7m( : ) = 0.d0
@@ -361,8 +360,7 @@ call cs_fuel_physprop1 &
 
 ! --- Transport of H2
 
-call  cs_fuel_thfieldconv2 ( ncelet , ncel )
-!=========================
+call  cs_fuel_thfieldconv2(ncel)
 
 !===============================================================================
 ! 5. Calculation of the physical properties of the mixture
