@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -148,7 +148,7 @@ double precision , dimension ( : , : ) , allocatable :: af1    , af2
 double precision , dimension ( : )     , allocatable :: fs3no  , fs4no
 double precision , dimension ( : , : ) , allocatable :: yfs4no
 double precision, allocatable, dimension(:) :: tpdf
-double precision, dimension(:), pointer :: cpro_x1, cpro_mmel, cpro_temp1
+double precision, dimension(:), pointer :: cpro_x1, cpro_mmel, cpro_temp
 double precision, dimension(:), pointer :: cvar_yfolcl
 double precision, dimension(:), pointer :: cpro_cyf1, cpro_cyf2, cpro_cyf3
 double precision, dimension(:), pointer :: cpro_cyf4, cpro_cyf5, cpro_cyf6
@@ -161,6 +161,23 @@ type(pmapper_double_r1), dimension(:), allocatable :: cpro_ym1
 integer          ipass
 data ipass / 0 /
 
+!===============================================================================
+! Interfaces
+!===============================================================================
+
+interface
+
+  subroutine cs_fuel_thfieldconv1(location_id, eh, tp) &
+    bind(C, name='cs_fuel_thfieldconv1')
+
+    use, intrinsic :: iso_c_binding
+    implicit none
+    integer(c_int), value :: location_id
+    real(kind=c_double), dimension(*) :: eh, tp
+  end subroutine cs_fuel_thfieldconv1
+
+end interface
+
 !===============================================================================
 ! 0. Memory allocation
 !===============================================================================
@@ -204,18 +221,18 @@ endif
 !===============================================================================
 
 ! pointer
-call field_get_val_s(iprpfl(iym1(ifo0 )),cpro_cyf1)
-call field_get_val_s(iprpfl(iym1(ifov )),cpro_cyf2)
-call field_get_val_s(iprpfl(iym1(ico  )),cpro_cyf3)
-call field_get_val_s(iprpfl(iym1(ih2s )),cpro_cyf4)
-call field_get_val_s(iprpfl(iym1(ihy  )),cpro_cyf5)
-call field_get_val_s(iprpfl(iym1(ihcn )),cpro_cyf6)
-call field_get_val_s(iprpfl(iym1(inh3 )),cpro_cyf7)
-call field_get_val_s(iprpfl(iym1(io2  )),cpro_cyox)
-call field_get_val_s(iprpfl(iym1(ico2 )),cpro_cyp1)
-call field_get_val_s(iprpfl(iym1(ih2o )),cpro_cyp2)
-call field_get_val_s(iprpfl(iym1(iso2 )),cpro_cyp3)
-call field_get_val_s(iprpfl(iym1(in2  )),cpro_cyin)
+call field_get_val_s(iym1(ifo0 ),cpro_cyf1)
+call field_get_val_s(iym1(ifov ),cpro_cyf2)
+call field_get_val_s(iym1(ico  ),cpro_cyf3)
+call field_get_val_s(iym1(ih2s ),cpro_cyf4)
+call field_get_val_s(iym1(ihy  ),cpro_cyf5)
+call field_get_val_s(iym1(ihcn ),cpro_cyf6)
+call field_get_val_s(iym1(inh3 ),cpro_cyf7)
+call field_get_val_s(iym1(io2  ),cpro_cyox)
+call field_get_val_s(iym1(ico2 ),cpro_cyp1)
+call field_get_val_s(iym1(ih2o ),cpro_cyp2)
+call field_get_val_s(iym1(iso2 ),cpro_cyp3)
+call field_get_val_s(iym1(in2  ),cpro_cyin)
 
 ipass = ipass + 1
 
@@ -234,7 +251,6 @@ enddo
 allocate(tpdf(ncelet))
 
 call pppdfr &
-!==========
  ( ncelet , ncel  , intpdf ,                                      &
    tpdf   ,                                                       &
    ffuel  , fvp2m ,                                               &
@@ -272,8 +288,7 @@ do iel = 1, ncel
 enddo
 
 call cs_gascomb &
-!==============
- ( ncelet , ncel   , ifo0 , ifov ,                                &
+ ( ncel   , ifo0 , ifov ,                                         &
    intpdf ,                                                       &
    f1m    , f2m , f3m , f4m , f5m , f6m , f7m , f8m , f9m ,       &
    pdfm1  , pdfm2  , doxyd    , dfuel  , hrec ,                   &
@@ -288,7 +303,7 @@ call cs_gascomb &
 
 allocate(cpro_ym1(ngazg))
 do ice = 1, ngazg
-  call field_get_val_s(iprpfl(iym1(ice)), cpro_ym1(ice)%p)
+  call field_get_val_s(iym1(ice), cpro_ym1(ice)%p)
 enddo
 
 do iel = 1, ncel
@@ -312,10 +327,10 @@ enddo
 ! 4. Calculation of temperature and density
 !===============================================================================
 
-call field_get_val_s(iprpfl(itemp1),cpro_temp1)
-call field_get_val_s(iprpfl(immel),cpro_mmel)
+call field_get_val_s(itemp, cpro_temp)
+call field_get_val_s(immel, cpro_mmel)
 
-call cs_fuel_thfieldconv1(MESH_LOCATION_CELLS, enth, cpro_temp1)
+call cs_fuel_thfieldconv1(MESH_LOCATION_CELLS, enth, cpro_temp)
 
 do iel = 1, ncel
   wmolme = cpro_cyf1(iel)/wmole(ifo0)                         &
@@ -338,7 +353,7 @@ do iel = 1, ncel
   ! ---- We do not include the mecanical pressure IPR
   !      but P0
 
-  rom1(iel) = p0 / (wmolme*cs_physical_constants_r*cpro_temp1(iel))
+  rom1(iel) = p0 / (wmolme*cs_physical_constants_r*cpro_temp(iel))
 enddo
 
 !===============================================================================
@@ -351,18 +366,16 @@ enddo
 if ( ieqnox .eq. 1 .and. ipass .gt. 1 ) then
 
   call cs_fuel_noxst &
- !==================
- ( ncelet , ncel   ,                                              &
-   intpdf ,                                                       &
+ ( ncel   , intpdf ,                                              &
    pdfm1  , pdfm2  , doxyd  , dfuel  , hrec ,                     &
    f3m    , f4m    , f5m    , f6m    , f7m  , f8m , f9m ,         &
    fs3no  , fs4no  , yfs4no , enthox )
 
 else if ( ieqnox .eq. 1 ) then
 
-  call field_get_val_s(iprpfl(ighcn1),cpro_ghcn1)
-  call field_get_val_s(iprpfl(ighcn2),cpro_ghcn2)
-  call field_get_val_s(iprpfl(ignoth),cpro_gnoth)
+  call field_get_val_s(ighcn1,cpro_ghcn1)
+  call field_get_val_s(ighcn2,cpro_ghcn2)
+  call field_get_val_s(ignoth,cpro_gnoth)
 
   do iel = 1, ncel
     cpro_ghcn1(iel) = 0.d0
@@ -376,9 +389,9 @@ endif
 ! 6. Calculation of balances in C, O et H
 !===============================================================================
 
-call field_get_val_s(iprpfl(ibcarbone),cpro_bcarbone)
-call field_get_val_s(iprpfl(iboxygen),cpro_boxygen)
-call field_get_val_s(iprpfl(ibhydrogen),cpro_bhydrogen)
+call field_get_val_s(ibcarbone,cpro_bcarbone)
+call field_get_val_s(iboxygen,cpro_boxygen)
+call field_get_val_s(ibhydrogen,cpro_bhydrogen)
 
 do iel=1,ncel
   cpro_bcarbone(iel) = cpro_x1(iel)                                &
