@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -41,15 +41,15 @@
 !> \param[in,out] tp            gas temperature (in kelvin)
 !______________________________________________________________________________!
 
-subroutine cs_fuel_thfieldconv1 &
- ( location_id     ,                                              &
-   eh     ,                                                       &
-   tp     )
+subroutine cs_fuel_thfieldconv1(location_id, eh, tp)  &
+ bind(C, name='cs_fuel_thfieldconv1')
 
 !==============================================================================
 ! Module files
 !==============================================================================
 
+use, intrinsic :: iso_c_binding
+
 use paramx
 use numvar
 use optcal
@@ -73,10 +73,8 @@ implicit none
 
 ! Arguments
 
-integer          location_id
-
-double precision eh(*)
-double precision tp(*)
+integer(c_int), value :: location_id
+real(kind=c_double), dimension(*) :: eh, tp
 
 ! Local variables
 
