@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -58,10 +58,11 @@
 #include "fvm_nodal_extract.h"
 
 #include "cs_base.h"
-#include "cs_ctwr_air_props.h"
+#include "cs_air_props.h"
 #include "cs_field.h"
 #include "cs_field_pointer.h"
 #include "cs_halo.h"
+#include "cs_halo_perio.h"
 #include "cs_log.h"
 #include "cs_math.h"
 #include "cs_mesh.h"
@@ -98,6 +99,8 @@ static cs_ctwr_option_t  _ctwr_option = {
   .evap_model = CS_CTWR_NONE,
   .has_rain = false};
 
+const cs_ctwr_option_t *cs_glob_ctwr_option = &_ctwr_option;
+
 /* Cooling tower exchange zone structure definition */
 /*--------------------------------------------------*/
 
@@ -129,9 +132,9 @@ struct _cs_ctwr_zone_t {
   cs_real_t  xleak_fac;          /* Leakage factor (ratio of outlet/inlet
                                     flow rate) */
 
-  cs_int_t   n_cells;            /* Number of air cells belonging to the zone */
+  cs_lnum_t  n_cells;            /* Number of air cells belonging to the zone */
 
-  cs_int_t   up_ct_id;           /* Id of upstream exchange zone (if any) */
+  int        up_ct_id;           /* Id of upstream exchange zone (if any) */
 
   cs_lnum_t  n_inlet_faces;      /* Number of inlet faces */
   cs_lnum_t  n_outlet_faces;     /* Number of outlet faces */
@@ -317,7 +320,7 @@ _lewis_factor(const int        evap_model,
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- * Provide acces to cs_ctwr_option
+ * Provide access to cs_ctwr_option
  *----------------------------------------------------------------------------*/
 
 cs_ctwr_option_t *
@@ -499,7 +502,8 @@ cs_ctwr_field_pointer_map(void)
      have already been defined in 'cs_field_pointer_map',
      which comes after 'ctvarp' */
   cs_field_pointer_map(CS_ENUMF_(humid), cs_field_by_name_try("humidity"));
-  cs_field_pointer_map(CS_ENUMF_(ym_w), cs_field_by_name_try("ym_water"));
+  cs_field_pointer_map(CS_ENUMF_(ym_w),
+                       cs_field_by_name_try("ym_water"));
   cs_field_pointer_map(CS_ENUMF_(t_l), cs_field_by_name_try("temperature_liquid"));
   cs_field_pointer_map(CS_ENUMF_(h_l), cs_field_by_name_try("enthalpy_liquid"));
   cs_field_pointer_map(CS_ENUMF_(y_l_pack), cs_field_by_name_try("y_l_packing"));
@@ -530,7 +534,6 @@ cs_ctwr_build_zones(void)
   /* Define the zones with source terms */
   if (ct_opt->has_rain) {
     /* Phase change may take place in the entire computational domain */
-    void *input;
     cs_volume_zone_define("rain_zone",
                           "all[]",
                           CS_VOLUME_ZONE_MASS_SOURCE_TERM);
@@ -656,7 +659,7 @@ cs_ctwr_log_setup(void)
                   "--------------\n"
                   "  Droplet diameter: %f\n"
                   "  Evaporation model: %s\n"),
-                cs_glob_ctwr_props->droplet_diam,
+                cs_glob_air_props->droplet_diam,
                 model_type
                 );
 
@@ -703,19 +706,14 @@ cs_ctwr_log_balance(void)
 
   const cs_lnum_2_t *i_face_cells
     = (const cs_lnum_2_t *)(cs_glob_mesh->i_face_cells);
-  cs_real_t *rho_h = (cs_real_t *)CS_F_(rho)->val;  /* humid air bulk density */
   cs_real_t *t_h = (cs_real_t *)CS_F_(t)->val;      /* humid air temperature */
   cs_real_t *h_h = (cs_real_t *)CS_F_(h)->val;      /* humid air enthalpy */
-  cs_real_t *y_w = (cs_real_t *)CS_F_(ym_w)->val;   /* Water mass fraction
-                                                       in humid air */
-  cs_real_t *x = (cs_real_t *)CS_F_(humid)->val;    /* humid air bulk humidity */
-
   cs_real_t *t_l = (cs_real_t *)CS_F_(t_l)->val;    /* liquid temperature */
   cs_real_t *h_l = (cs_real_t *)CS_F_(h_l)->val;    /* liquid enthalpy */
   cs_real_t *y_l = (cs_real_t *)CS_F_(y_l_pack)->val;   /* liquid mass per unit
                                                        cell volume */
 
-  cs_real_t *liq_mass_flow = cs_field_by_name("inner_mass_flux_y_l_packing")->val;//FIXME take the good one... for y_p
+  cs_real_t *liq_mass_flow = cs_field_by_name("inner_mass_flux_y_l_packing")->val; //FIXME take the good one... for y_p
   cs_real_t *mass_flow = cs_field_by_name("inner_mass_flux")->val;
 
   FILE *f;
@@ -919,10 +917,10 @@ cs_ctwr_init_field_vars(cs_real_t  rho0,
   else
     BFT_MALLOC(cpro_taup, n_cells_with_ghosts, cs_real_t);
 
-  const cs_ctwr_fluid_props_t  *ct_prop = cs_glob_ctwr_props;
-  cs_real_t rho_l = ct_prop->rho_l;
+  const cs_air_fluid_props_t  *air_prop = cs_glob_air_props;
+  cs_real_t rho_l = air_prop->rho_l;
   cs_real_t visc = cs_glob_fluid_properties->viscl0;
-  cs_real_t droplet_diam = ct_prop->droplet_diam;
+  cs_real_t droplet_diam = air_prop->droplet_diam;
 
   cs_real_t gravity[] = {cs_glob_physical_constants->gravity[0],
                          cs_glob_physical_constants->gravity[1],
@@ -950,18 +948,19 @@ cs_ctwr_init_field_vars(cs_real_t  rho0,
     t_h_a[cell_id] = t_h[cell_id];
 
     /* Update the humid air density */
-    rho_h[cell_id] = cs_ctwr_rho_humidair(x[cell_id],
-                                          rho0,
-                                          p0,
-                                          t0,
-                                          molmassrat,
-                                          t_h[cell_id]);
+    rho_h[cell_id] = cs_air_rho_humidair(x[cell_id],
+                                         rho0,
+                                         p0,
+                                         t0,
+                                         molmassrat,
+                                         t_h[cell_id]);
 
     /* Update the humid air enthalpy */
-    x_s[cell_id] = cs_ctwr_xsath(t_h[cell_id],p0);
-    cs_real_t cp_h = cs_ctwr_cp_humidair(x[cell_id], x_s[cell_id]);
+    x_s[cell_id] = cs_air_x_sat(t_h[cell_id],p0);
 
-    h_h[cell_id] = cs_ctwr_h_humidair(cp_h,
+    cs_real_t cp_h = cs_air_cp_humidair(x[cell_id], x_s[cell_id]);
+
+    h_h[cell_id] = cs_air_h_humidair(cp_h,
                                       x[cell_id],
                                       x_s[cell_id],
                                       t_h[cell_id]);
@@ -1010,7 +1009,7 @@ cs_ctwr_init_field_vars(cs_real_t  rho0,
       t_l[cell_id] = ct->t_l_bc;
 
       /* Update the injected liquid enthalpy */
-      h_l[cell_id] = cs_ctwr_h_liqwater(t_l[cell_id]);
+      h_l[cell_id] = cs_liq_t_to_h(t_l[cell_id]);
 
       /* Initialise the liquid vertical velocity component
        * this is correct for droplet and extended for other packing zones */
@@ -1038,7 +1037,8 @@ cs_ctwr_init_field_vars(cs_real_t  rho0,
     if (cfld_drift_vel != NULL) {
       cs_halo_sync_var_strided(halo, CS_HALO_STANDARD, cfld_drift_vel->val, 3);
       if (m->n_init_perio > 0)
-        cs_halo_perio_sync_var_vect(halo, CS_HALO_STANDARD, cfld_drift_vel->val);
+        cs_halo_perio_sync_var_vect(halo, CS_HALO_STANDARD,
+                                    cfld_drift_vel->val, 3);
     }
   }
 
@@ -1059,8 +1059,6 @@ cs_ctwr_init_field_vars(cs_real_t  rho0,
 void
 cs_ctwr_init_flow_vars(cs_real_t  liq_mass_flow[])
 {
-
-  const cs_real_t  *rho_h = (cs_real_t *)CS_F_(rho)->val; /* humid air (bulk) density */
   cs_real_t *y_l = (cs_real_t *)CS_F_(y_l_pack)->val; /* mass fraction of liquidus */
   cs_real_t *h_l = (cs_real_t *)CS_F_(h_l)->val;    /*liquid enthalpy */
   cs_real_t *t_l = (cs_real_t *)CS_F_(t_l)->val;    /*liquid temperature */
@@ -1070,7 +1068,6 @@ cs_ctwr_init_flow_vars(cs_real_t  liq_mass_flow[])
   const cs_lnum_2_t *i_face_cells =
     (const cs_lnum_2_t *)(cs_glob_mesh->i_face_cells);
 
-  const cs_lnum_t n_cells = cs_glob_mesh->n_cells;
   const cs_lnum_t n_cells_with_ghosts = cs_glob_mesh->n_cells_with_ghosts;
   const cs_lnum_t n_i_faces = cs_glob_mesh->n_i_faces;
 
@@ -1157,7 +1154,7 @@ cs_ctwr_init_flow_vars(cs_real_t  liq_mass_flow[])
           ct->surface_in += liq_surf;
           y_l[cell_id_2] = ct->y_l_bc;
           t_l[cell_id_2] = ct->t_l_bc;
-          h_l[cell_id_2] = cs_ctwr_h_liqwater(ct->t_l_bc);
+          h_l[cell_id_2] = cs_liq_t_to_h(ct->t_l_bc);
           /* The transported value is (y_l.h_l) and not (h_l) */
           h_l[cell_id_2] *= y_l[cell_id_2];
         }
@@ -1185,7 +1182,7 @@ cs_ctwr_init_flow_vars(cs_real_t  liq_mass_flow[])
           ct->surface_in += liq_surf;
           y_l[cell_id_1] = ct->y_l_bc;
           t_l[cell_id_1] = ct->t_l_bc;
-          h_l[cell_id_1] = cs_ctwr_h_liqwater(ct->t_l_bc);
+          h_l[cell_id_1] = cs_liq_t_to_h(ct->t_l_bc);
           /* The transported value is (y_l.h_l) and not (h_l) */
           h_l[cell_id_1] *= y_l[cell_id_1];
         }
@@ -1293,7 +1290,6 @@ cs_ctwr_restart_field_vars(cs_real_t  rho0,
   const cs_lnum_t n_cells_with_ghosts = m->n_cells_with_ghosts;
 
   // Initialise the fields - based on map
-  cs_real_t *rho_h = (cs_real_t *)CS_F_(rho)->val;   /* humid air (bulk) density */
   cs_real_t *cp_h = (cs_real_t *)CS_F_(cp)->val;     /* humid air (bulk) Cp */
   cs_real_t *t_h = (cs_real_t *)CS_F_(t)->val;       /* humid air temperature */
   cs_real_t *t_h_a = (cs_real_t *)CS_F_(t)->val_pre; /* humid air temperature */
@@ -1328,10 +1324,10 @@ cs_ctwr_restart_field_vars(cs_real_t  rho0,
       ct_opt->has_rain = true;
   }
 
-  const cs_ctwr_fluid_props_t  *ct_prop = cs_glob_ctwr_props;
-  cs_real_t rho_l = ct_prop->rho_l;
+  const cs_air_fluid_props_t  *air_prop = cs_glob_air_props;
+  cs_real_t rho_l = air_prop->rho_l;
   cs_real_t visc = cs_glob_fluid_properties->viscl0;
-  cs_real_t droplet_diam = ct_prop->droplet_diam;
+  cs_real_t droplet_diam = air_prop->droplet_diam;
 
   cs_real_t gravity[] = {cs_glob_physical_constants->gravity[0],
                          cs_glob_physical_constants->gravity[1],
@@ -1351,12 +1347,12 @@ cs_ctwr_restart_field_vars(cs_real_t  rho0,
 
   cs_real_t t_h_ini = t0 - cs_physical_constants_celsius_to_kelvin;
 
-  cs_real_t rho_h_ini = cs_ctwr_rho_humidair(x_ini,
-                                             rho0,
-                                             p0,
-                                             t0,
-                                             molmassrat,
-                                             t_h_ini);
+  cs_real_t rho_h_ini = cs_air_rho_humidair(x_ini,
+                                            rho0,
+                                            p0,
+                                            t0,
+                                            molmassrat,
+                                            t_h_ini);
 
   /* Initialise the cooling towers variables */
 
@@ -1383,11 +1379,11 @@ cs_ctwr_restart_field_vars(cs_real_t  rho0,
     /* Update the humid air enthalpy based on the solved value of T_h */
     //FIXME Need to use the method of 'cs_ctwr_phyvar_update'
 
-    x_s[cell_id] = cs_ctwr_xsath(t_h[cell_id],p0);
+    x_s[cell_id] = cs_air_x_sat(t_h[cell_id],p0);
 
-    cp_h[cell_id] = cs_ctwr_cp_humidair(x[cell_id], x_s[cell_id]);
+    cp_h[cell_id] = cs_air_cp_humidair(x[cell_id], x_s[cell_id]);
 
-    h_h[cell_id] = cs_ctwr_h_humidair(cp_h[cell_id],
+    h_h[cell_id] = cs_air_h_humidair(cp_h[cell_id],
                                       x[cell_id],
                                       x_s[cell_id],
                                       t_h[cell_id]);
@@ -1399,7 +1395,7 @@ cs_ctwr_restart_field_vars(cs_real_t  rho0,
     t_l[cell_id] = t0 - cs_physical_constants_celsius_to_kelvin;
     if (y_l[cell_id] > 0.) {
       cs_real_t h_liq = h_l[cell_id] / y_l[cell_id];
-      t_l[cell_id] = cs_ctwr_t_liqwater(h_liq);
+      t_l[cell_id] = cs_liq_h_to_t(h_liq);
     }
 
     /* Initialise the liquid vertical velocity component
@@ -1496,7 +1492,8 @@ cs_ctwr_restart_field_vars(cs_real_t  rho0,
     if (cfld_drift_vel != NULL) {
       cs_halo_sync_var_strided(halo, CS_HALO_STANDARD, cfld_drift_vel->val, 3);
       if (m->n_init_perio > 0)
-        cs_halo_perio_sync_var_vect(halo, CS_HALO_STANDARD, cfld_drift_vel->val);
+        cs_halo_perio_sync_var_vect(halo, CS_HALO_STANDARD,
+                                    cfld_drift_vel->val, 3);
     }
   }
 
@@ -1505,7 +1502,6 @@ cs_ctwr_restart_field_vars(cs_real_t  rho0,
     BFT_FREE(cpro_taup);
 }
 
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Update the thermo physical properties fields for the humid air and
@@ -1530,24 +1526,24 @@ cs_ctwr_phyvar_update(cs_real_t  rho0,
     = (const cs_lnum_t *)(cs_glob_mesh->b_face_cells);
   const cs_halo_t *halo = cs_glob_mesh->halo;
 
-  cs_real_t *rho_h = (cs_real_t *)CS_F_(rho)->val;      /* humid air (bulk) density */
-  cs_real_t *cp_h = (cs_real_t *)CS_F_(cp)->val;        /* humid air (bulk) Cp */
+  cs_real_t *rho_h = (cs_real_t *)CS_F_(rho)->val;    /* humid air (bulk) density */
+  cs_real_t *cp_h = (cs_real_t *)CS_F_(cp)->val;      /* humid air (bulk) Cp */
 
   // Fields based on maps
-  cs_real_t *t_h = (cs_real_t *)CS_F_(t)->val;       /* humid air temperature */
+  cs_real_t *t_h = (cs_real_t *)CS_F_(t)->val;        /* humid air temperature */
   cs_real_t *t_h_a = (cs_real_t *)CS_F_(t)->val_pre;  /* humid air temperature */
-  cs_real_t *h_h = (cs_real_t *)CS_F_(h)->val;       /* humid air enthalpy */
-  cs_real_t *therm_diff_h = cs_field_by_name_try("thermal_conductivity")->val;
+  cs_real_t *h_h = (cs_real_t *)CS_F_(h)->val;        /* humid air enthalpy */
+  cs_real_t *therm_diff_h = cs_field_by_name("thermal_conductivity")->val;
   cs_real_t *cpro_x1 = cs_field_by_name("x_c")->val;
   cs_real_t *bpro_x1 = cs_field_by_name("b_x_c")->val;
-  cs_real_t *y_w = (cs_real_t *)CS_F_(ym_w)->val; /* Water mass fraction
-                                                     in humid air */
-  cs_real_t *x = (cs_real_t *)CS_F_(humid)->val; /* humidity in humid air (bulk) */
+  cs_real_t *y_w = (cs_real_t *)CS_F_(ym_w)->val;     /* Water mass fraction
+                                                         in humid air */
+  cs_real_t *x = (cs_real_t *)CS_F_(humid)->val;      /* humidity in humid air (bulk) */
   cs_real_t *x_s = cs_field_by_name("x_s")->val;
 
-  cs_real_t *t_l = (cs_real_t *)CS_F_(t_l)->val;    /*liquid temperature */
-  cs_real_t *h_l = (cs_real_t *)CS_F_(h_l)->val;    /*liquid enthalpy */
-  cs_real_t *y_l = (cs_real_t *)CS_F_(y_l_pack)->val;      /*liquid mass per unit cell volume*/
+  cs_real_t *t_l = (cs_real_t *)CS_F_(t_l)->val;      /*liquid temperature */
+  cs_real_t *h_l = (cs_real_t *)CS_F_(h_l)->val;      /*liquid enthalpy */
+  cs_real_t *y_l = (cs_real_t *)CS_F_(y_l_pack)->val; /*liquid mass per unit cell volume*/
 
   cs_real_t *liq_mass_flow = cs_field_by_name("inner_mass_flux_y_l_packing")->val;//FIXME
 
@@ -1561,9 +1557,7 @@ cs_ctwr_phyvar_update(cs_real_t  rho0,
   cs_lnum_t n_cells = cs_glob_mesh->n_cells;
   cs_lnum_t n_b_faces = cs_glob_mesh->n_b_faces;
 
-  cs_real_t lambda_h = cs_glob_ctwr_props->lambda_h;
-  cs_real_t cp_l = cs_glob_ctwr_props->cp_l;
-  cs_real_t lambda_l = cs_glob_ctwr_props->lambda_l;
+  cs_real_t lambda_h = cs_glob_air_props->lambda_h;
 
   for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
 
@@ -1582,43 +1576,46 @@ cs_ctwr_phyvar_update(cs_real_t  rho0,
         y_p[cell_id] = 1. - y_w[cell_id] - cs_math_epzero; //TODO count it
 
       /* Continuous phase mass fraction */
-      cpro_x1[cell_id] = 1. - y_p[cell_id];//TODO not one for rain zones - Why not?
+      cpro_x1[cell_id] = 1. - y_p[cell_id];
+      //TODO not one for rain zones - Why not?
       //If it represents the humid air, then it should be one?  If it represents
       //the dry air, then it should account for both y_p and y_w
     }
 
     /* Update humidity field */
-    x[cell_id] = y_w[cell_id]/(1.0-y_w[cell_id]); //FIXME for drops - This should be the
-    //the proportion of 'gaseous' water (dissolved and condensate) in the humid air:
-    // Y(dry air)+ Y(gasesous water) + Y(drops) = 1 in all computational cells
-    //If we do that, then the density needs to be revised as well and the temperatures of both
-    //the bulk (dry air + gaseos water +drops) and the humid air must be solved for
-    //Here, the approximation is that Y(drops) is negligible
+    x[cell_id] = y_w[cell_id]/(1.0-y_w[cell_id]);
+    // FIXME for drops - This should be the proportion of 'gaseous' water
+    // (dissolved and condensate) in the humid air:
+    //   Y(dry air)+ Y(gasesous water) + Y(drops) = 1 in all computational cells
+    //   If we do that, then the density needs to be revised as well and the
+    //   temperatures of both the bulk (dry air + gaseos water +drops) and the
+    //   humid air must be solved for.
+    // Here, the approximation is that Y(drops) is negligible
 
     /* Saturated humidity */
-    x_s[cell_id] = cs_ctwr_xsath(t_h[cell_id], p0);
+    x_s[cell_id] = cs_air_x_sat(t_h[cell_id], p0);
 
     /* Update the humid air temperature using new enthalpy but old
      * Specific heat */
 
-    cp_h[cell_id] = cs_ctwr_cp_humidair(x[cell_id], x_s[cell_id]);
+    cp_h[cell_id] = cs_air_cp_humidair(x[cell_id], x_s[cell_id]);
 
     //FIXME - What is the formula below - Inconsistent with taking into
-    //account the saturated phase in the enthalpy in 'cs_ctwr_h_humidair'
+    //account the saturated phase in the enthalpy in 'cs_air_h_humidair'
     h_h[cell_id] += (t_h[cell_id] - t_h_a[cell_id]) * cp_h[cell_id];
 
     // Udate the humid air enthalpy diffusivity lambda_h if solve for T_h?
     // Need to update since a_0 is variable as a function of T and humidity
     therm_diff_h[cell_id] = lambda_h;
 
-    /* Update the humid air density */  // Again, formally this should be the bulk density,
-                                        // including the rain drops
-    rho_h[cell_id] = cs_ctwr_rho_humidair(x[cell_id],
-                                          rho0,
-                                          p0,
-                                          t0,
-                                          molmassrat,
-                                          t_h[cell_id]);
+    /* Update the humid air density */  // Again, formally this should be the
+                                        // bulk density, including the rain drops
+    rho_h[cell_id] = cs_air_rho_humidair(x[cell_id],
+                                         rho0,
+                                         p0,
+                                         t0,
+                                         molmassrat,
+                                         t_h[cell_id]);
 
   }
 
@@ -1636,7 +1633,7 @@ cs_ctwr_phyvar_update(cs_real_t  rho0,
        * NB: (y_l.h_l) is transported and not (h_l) */
       if (y_l[cell_id] > 0.) {
         cs_real_t h_liq = h_l[cell_id] / y_l[cell_id];
-        t_l[cell_id] = cs_ctwr_t_liqwater(h_liq);
+        t_l[cell_id] = cs_liq_h_to_t(h_liq);
       }
     }
 
@@ -1650,7 +1647,7 @@ cs_ctwr_phyvar_update(cs_real_t  rho0,
       for (cs_lnum_t i = 0; i < ct->n_outlet_faces; i++) {
 
         cs_lnum_t face_id = ct->outlet_faces_ids[i];
-        cs_lnum_t cell_id_l, cell_id_h;
+        cs_lnum_t cell_id_l;
 
         /* Convention: outlet is positive mass flux
          * Then upwind cell for liquid is i_face_cells[][0] */
@@ -1658,10 +1655,8 @@ cs_ctwr_phyvar_update(cs_real_t  rho0,
         if (liq_mass_flow[face_id] < 0) {
           sign = -1;
           cell_id_l = i_face_cells[face_id][1];
-          cell_id_h = i_face_cells[face_id][0];
         } else {
           cell_id_l = i_face_cells[face_id][0];
-          cell_id_h = i_face_cells[face_id][1];
         }
 
         /* h_l is in fact (y_l. h_l),
@@ -1724,7 +1719,6 @@ cs_ctwr_source_term(int              f_id,
                     cs_real_t        exp_st[],
                     cs_real_t        imp_st[])
 {
-
   const cs_mesh_t *m = cs_glob_mesh;
   const cs_lnum_2_t *i_face_cells
     = (const cs_lnum_2_t *)(m->i_face_cells);
@@ -1732,7 +1726,7 @@ cs_ctwr_source_term(int              f_id,
   const cs_real_t *cell_f_vol = cs_glob_mesh_quantities->cell_f_vol;
 
   cs_real_t  *rho_h = (cs_real_t *)CS_F_(rho)->val; /* humid air (bulk) density */
-  cs_real_3_t *vel_h = (cs_real_3_t *)CS_F_(u)->val;   /* humid air (bulk) */
+  cs_real_3_t *vel_h = (cs_real_3_t *)CS_F_(vel)->val;   /* humid air (bulk) */
 
   cs_real_t *y_w = (cs_real_t *)CS_F_(ym_w)->val; /* Water mass fraction
                                                      in humid air */
@@ -1748,15 +1742,10 @@ cs_ctwr_source_term(int              f_id,
   cs_field_t *cfld_yp = cs_field_by_name_try("y_p");         /* Rain drops mass fraction */
   cs_field_t *cfld_tp = cs_field_by_name_try("y_p_t_l");     /* Rain drops temperature */
   cs_field_t *cfld_drift_vel = cs_field_by_name_try("drift_vel_y_p");
-  cs_field_t *cfld_taup = cs_field_by_name_try("drift_tau_y_p");
 
-  cs_real_t vertical[3], horizontal[3], norme_g;
+  cs_real_t vertical[3], horizontal[3];
 
-  cs_real_t gravity[] = {cs_glob_physical_constants->gravity[0],
-                         cs_glob_physical_constants->gravity[1],
-                         cs_glob_physical_constants->gravity[2]};
-
-  cs_ctwr_option_t *ct_opt = cs_get_glob_ctwr_option();
+  const cs_ctwr_option_t *ct_opt = cs_glob_ctwr_option;
 
   int evap_model = ct_opt->evap_model;
 
@@ -1764,7 +1753,7 @@ cs_ctwr_source_term(int              f_id,
      In the old code, it seems to be taken as the value of the
      face mass flux upstream of the cell */
 
-  cs_ctwr_fluid_props_t *ct_prop = cs_glob_ctwr_props;
+  cs_air_fluid_props_t *air_prop = cs_glob_air_props;
 
   cs_real_t v_air = 0.;
 
@@ -1778,13 +1767,13 @@ cs_ctwr_source_term(int              f_id,
 
   /* Compute the source terms */
 
-  cs_real_t cp_v = ct_prop->cp_v;
-  cs_real_t cp_l = ct_prop->cp_l;
-  cs_real_t hv0 = ct_prop->hv0;
-  cs_real_t rho_l = ct_prop->rho_l;
+  cs_real_t cp_v = air_prop->cp_v;
+  cs_real_t cp_l = air_prop->cp_l;
+  cs_real_t hv0 = air_prop->hv0;
+  cs_real_t rho_l = air_prop->rho_l;
   cs_real_t visc = cs_glob_fluid_properties->viscl0;
-  cs_real_t lambda_h = ct_prop->lambda_h;
-  cs_real_t droplet_diam  = ct_prop->droplet_diam;
+  cs_real_t lambda_h = air_prop->lambda_h;
+  cs_real_t droplet_diam  = air_prop->droplet_diam;
 
   if (evap_model != CS_CTWR_NONE) {
 
@@ -1792,15 +1781,13 @@ cs_ctwr_source_term(int              f_id,
        Phase change source terms
        ========================= */
 
-    vertical[0] = -gravity[0];
-    vertical[1] = -gravity[1];
-    vertical[2] = -gravity[2];
+    cs_math_3_normalize(cs_glob_physical_constants->gravity,
+                        vertical);
 
-    norme_g = cs_math_3_norm(vertical);
+    vertical[0] *= -1;
+    vertical[1] *= -1;
+    vertical[2] *= -1;
 
-    vertical[0] /= norme_g;
-    vertical[1] /= norme_g;
-    vertical[2] /= norme_g;
     horizontal[0] = vertical[0] -1.;
     horizontal[1] = vertical[1] -1.;
     horizontal[2] = vertical[2] -1.;
@@ -1828,10 +1815,10 @@ cs_ctwr_source_term(int              f_id,
         cs_real_t temp_h = CS_MIN(t_h[cell_id], t_l[cell_id]);
 
         /* saturation humidity at humid air temperature */
-        cs_real_t x_s_th = cs_ctwr_xsath(temp_h, p0);
+        cs_real_t x_s_th = cs_air_x_sat(temp_h, p0);
 
         /* saturation humidity at injected liquid temperature */
-        cs_real_t x_s_tl = cs_ctwr_xsath(t_l[cell_id], p0);
+        cs_real_t x_s_tl = cs_air_x_sat(t_l[cell_id], p0);
 
         /*--------------------------------------------*
          * Counter or cross flow packing zone         *
@@ -1890,9 +1877,9 @@ cs_ctwr_source_term(int              f_id,
         }
 
         /* Humid air temperature equation */
-        else if (f_id == (CS_F_(t)->id)) {
+        else if (f_id == (CS_F_(t)->id)) {//FIXME source term for theta_l instead...
           /* Because the writing is in a non-conservative form */
-          cs_real_t cp_h = cs_ctwr_cp_humidair(x[cell_id], x_s[cell_id]);
+          cs_real_t cp_h = cs_air_cp_humidair(x[cell_id], x_s[cell_id]);
           cs_real_t l_imp_st = vol_mass_source * cp_h;
           cs_real_t xlew = _lewis_factor(evap_model, molmassrat,
                                          x[cell_id], x_s_tl);
@@ -1920,7 +1907,7 @@ cs_ctwr_source_term(int              f_id,
          * NB: it is in fact "y_l x h_l" */
         else if (f_id == (CS_F_(h_l)->id)) {
           /* Implicit term */
-          cs_real_t cp_h = cs_ctwr_cp_humidair(x[cell_id], x_s[cell_id]);
+          cs_real_t cp_h = cs_air_cp_humidair(x[cell_id], x_s[cell_id]);
           cs_real_t l_imp_st = vol_mass_source;
           cs_real_t xlew = _lewis_factor(evap_model,molmassrat,x[cell_id],x_s_tl);
           /* Under saturated */
@@ -1963,15 +1950,15 @@ cs_ctwr_source_term(int              f_id,
           cs_real_t temp_h = CS_MIN(t_h[cell_id], temp_rain[cell_id]);
 
           /* saturation humidity at the temperature of the humid air */
-          cs_real_t x_s_th = cs_ctwr_xsath(temp_h, p0);
+          cs_real_t x_s_th = cs_air_x_sat(temp_h, p0);
 
           /* saturation humidity at the temperature of the rain drop  */
-          cs_real_t x_s_tl = cs_ctwr_xsath(temp_rain[cell_id], p0);
+          cs_real_t x_s_tl = cs_air_x_sat(temp_rain[cell_id], p0);
 
           cs_real_3_t *drift_vel_rain = (cs_real_3_t *restrict)(cfld_drift_vel->val);
           cs_real_t drift_vel_mag = cs_math_3_norm(drift_vel_rain[cell_id]);
           cs_real_t xlew = _lewis_factor(evap_model, molmassrat, x[cell_id], x_s_tl);
-          cs_real_t cp_h = cs_ctwr_cp_humidair(x[cell_id], x_s[cell_id]);
+          cs_real_t cp_h = cs_air_cp_humidair(x[cell_id], x_s[cell_id]);
 
           /* Drop diameter based Reynolds number */
           cs_real_t rey = rho_h[cell_id] * drift_vel_mag * droplet_diam / visc;
@@ -2026,14 +2013,16 @@ cs_ctwr_source_term(int              f_id,
 
           /* Global mass source term for continuity (pressure) equation
            * Note that if rain were already considered in the bulk, then inner
-           * mass transfer between liquid and vapor would disappeard */
-          //FIXME: Consider putting rain in the bulk --> implication on 'c' variables different from 'h'
+           * mass transfer between liquid and vapor would disappear */
+
+          //FIXME: Consider putting rain in the bulk
+          //       --> implication on 'c' variables different from 'h'
           if (f_id == (CS_F_(p)->id)) {
             /* Warning: not multiplied by Cell volume! no addition neither */
             exp_st[cell_id] = +mass_source;
           }
 
-          /* Water (vapour + condensate) in gas mass fraction equation except rain */
+          /* Water (vapor + condensate) in gas mass fraction equation except rain */
           else if (f_id == (CS_F_(ym_w)->id)) {
             exp_st[cell_id] += vol_mass_source*(1. - f_var[cell_id]);
             imp_st[cell_id] += vol_mass_source;
@@ -2111,12 +2100,9 @@ cs_ctwr_source_term(int              f_id,
        ==================================================== */
 
     cs_real_t *liq_mass_frac = CS_F_(y_l_pack)->val;   /* liquid mass fraction */
-    cs_real_t *h_l = (cs_real_t *)CS_F_(h_l)->val;     /* liquid enthalpy x liquid mass fraction */
     cs_real_t *liq_mass_flow
       = cs_field_by_name("inner_mass_flux_y_l_packing")->val; /* Inner mass flux of liquids
                                                                  (in the packing) */
-    cs_real_t *y_rain = (cs_real_t *)cfld_yp->val;
-
     for (int ict = 0; ict < _n_ct_zones; ict++) {
 
       cs_ctwr_zone_t *ct = _ct_zone[ict];
@@ -2163,10 +2149,13 @@ cs_ctwr_source_term(int              f_id,
           }
           /* Rain temperature */
           else if (f_id == cfld_tp->id) {
-            //FIXME: There should be a y_p factor in there so that mass and enthalpy are compatible
+            // FIXME: There should be a y_p factor in there so that
+            // mass and enthalpy are compatible
             //The transported variable is y_rain * H_rain
-            vol_mass_source = vol_mass_source * cp_l;  // For temperature equation of the rain
-            exp_st[cell_id_rain] += vol_mass_source * (t_l[cell_id_leak] - f_var[cell_id_rain]);
+            vol_mass_source = vol_mass_source * cp_l;
+            // For temperature equation of the rain
+            exp_st[cell_id_rain] +=   vol_mass_source
+                                    * (t_l[cell_id_leak] - f_var[cell_id_rain]);
             imp_st[cell_id_rain] += vol_mass_source;
           }
 
@@ -2190,7 +2179,7 @@ cs_ctwr_source_term(int              f_id,
  *
  * \param[in]   p0              Reference pressure
  * \param[in]   molmassrat      Dry air to water vapor molecular mass ratio
- * \param[in]   mass_source     Mass source term
+ * \param[out]  mass_source     Mass source term
  */
 /*----------------------------------------------------------------------------*/
 
@@ -2207,15 +2196,17 @@ cs_ctwr_bulk_mass_source_term(const cs_real_t   p0,
   BFT_MALLOC(imp_st, n_cells_with_ghosts, cs_real_t);
 
   for (cs_lnum_t cell_id = 0; cell_id < n_cells_with_ghosts; cell_id++) {
+    mass_source[cell_id] = 0.0;
     imp_st[cell_id] = 0.0;
   }
 
-  cs_ctwr_source_term(CS_F_(p)->id, /* Bulk mass source term is
-                                       stored for pressure */
-      p0,
-      molmassrat,
-      mass_source,
-      imp_st);
+  /* Bulk mass source term is stored for pressure */
+
+  cs_ctwr_source_term(CS_F_(p)->id,
+                      p0,
+                      molmassrat,
+                      mass_source,
+                      imp_st);
 
   BFT_FREE(imp_st);
 }
