@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -58,8 +58,8 @@ use cstnum
 use ppppar
 use ppthch
 use ppincl
-use ihmpre
 use darcy_module
+use cs_c_bindings
 
 !===============================================================================
 
@@ -67,14 +67,39 @@ implicit none
 
 ! Local variables
 
+type(var_cal_opt) :: vcopt
+
+!=============================================================================
+
+interface
+
+  !---------------------------------------------------------------------------
+
+  ! Interface to C function defining field keys for Ground water flow module.
+
+  subroutine cs_gwf_parameters_define_field_keys()  &
+    bind(C, name='cs_gwf_parameters_define_field_keys')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_gwf_parameters_define_field_keys
+
+  !---------------------------------------------------------------------------
+
+end interface
+
 !===============================================================================
 
 !===============================================================================
 ! 1. Set options
 !===============================================================================
 
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+
 ! Standard gradient calculation option
-iwgrec(ipr) = 0
+vcopt%iwgrec = 0
+
+call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+
 imrgra = 1
 imvisf = 1
 
@@ -103,12 +128,17 @@ darcy_gravity_x = 0.d0
 darcy_gravity_y = 0.d0
 darcy_gravity_z = 1.d0
 
-if (iihmpr.eq.1) then
-  call uidai1(darcy_anisotropic_permeability,     &
-              darcy_anisotropic_dispersion,       &
-              darcy_unsteady,                     &
-              darcy_gravity)
-endif
+! Unsaturated zone is taken into account by default
+darcy_unsaturated = 1
+
+! Definition of sorption parameters
+call cs_gwf_parameters_define_field_keys
+
+call uidai1(darcy_anisotropic_permeability,     &
+            darcy_anisotropic_dispersion,       &
+            darcy_unsteady,                     &
+            darcy_gravity,                      &
+            darcy_unsaturated)
 
 !===============================================================================
 ! 2. Define user settings
