@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -114,7 +114,7 @@ double precision, pointer, dimension(:)   :: dt
 ! Local variables
 
 integer iccocg, inc   , iel, isou, init
-integer nswrgp, imligp, iwarnp
+integer imrgrp, nswrgp, imligp, iwarnp
 integer imucpp, ircflp, isweep, isym, lchain
 integer ndircp, niterf, nswmpr
 integer iflmas, iflmab, iesize, idiffp, iconvp, ibsize
@@ -309,7 +309,7 @@ endif
 
 iconvp = 0 ! no convection
 idiffp = 1 ! diffusion
-ndircp = ndircl(ipr) ! no diagonal stepped aside
+ndircp = vcopt_p%ndircl ! no diagonal stepped aside
 thetap = 1.d0 ! implicit scheme
 imucpp = 0 ! do not multiply the convectiv term by anything
 
@@ -346,8 +346,10 @@ isweep = 1 ! counter for non-orthogonalities
 iccocg = 1 ! no calculation of cocg. What does it mean?
 init = 1 ! it is an initialization of the residue
 inc  = 1 ! 0 increment, 1 otherwise
+imrgrp = vcopt_p%imrgra
 nswrgp = vcopt_p%nswrgr ! number of sweeps for gradients reconstruction
 imligp = vcopt_p%imligr
+iwgrp  = vcopt_p%iwgrec
 iwarnp = vcopt_p%iwarni
 epsrgp = vcopt_p%epsrgr
 climgp = vcopt_p%climgr
@@ -357,8 +359,8 @@ ircflp = vcopt_p%ircflu
 if (darcy_anisotropic_permeability.eq.0) then
 
   call itrgrp &
-( ivarfl(ipr), init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydr , &
-  iwarnp ,                                                                    &
+( ivarfl(ipr), init   , inc    , imrgrp , iccocg , nswrgp , imligp , iphydr , &
+  iwgrp  , iwarnp ,                                                           &
   epsrgp , climgp , extrap ,                                                  &
   rvoid  ,                                                                    &
   cvar_pr   ,                                                                 &
@@ -371,8 +373,8 @@ if (darcy_anisotropic_permeability.eq.0) then
 else if (darcy_anisotropic_permeability.eq.1) then
 
   call itrgrv &
-( ivarfl(ipr), init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,        &
-  iphydr , iwarnp ,                                                                  &
+( ivarfl(ipr), init   , inc    , imrgrp , iccocg , nswrgp , imligp , ircflp ,        &
+  iphydr , iwgrp  , iwarnp ,                                                         &
   epsrgp , climgp , extrap ,                                                         &
   rvoid  ,                                                                           &
   cvar_pr   ,                                                                        &
@@ -475,8 +477,8 @@ do while ( (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp) &
   if (darcy_anisotropic_permeability.eq.0) then
 
     call itrgrp &
-  ( ivarfl(ipr), init  , inc , imrgra , iccocg , nswrgp , imligp , iphydr ,  &
-    iwarnp ,                                                                 &
+  ( ivarfl(ipr), init  , inc , imrgrp , iccocg , nswrgp , imligp , iphydr ,  &
+    iwgrp  , iwarnp ,                                                        &
     epsrgp , climgp , extrap ,                                               &
     rvoid  ,                                                                 &
     cvar_pr   ,                                                              &
@@ -489,17 +491,11 @@ do while ( (isweep.le.nswmpr.and.residu.gt.vcopt_p%epsrsm*rnormp) &
   else if (darcy_anisotropic_permeability.eq.1) then
 
     call itrgrv &
-    ( ivarfl(ipr), init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,        &
-    iphydr , iwarnp ,                                                                    &
-    epsrgp , climgp , extrap ,                                                           &
-    rvoid  ,                                                                             &
-    cvar_pr   ,                                                                          &
-    coefa_p  , coefb_p  ,                                                                &
-    coefaf_p , coefbf_p ,                                                                &
-    viscf  , viscb  ,                                                                    &
-    cpro_permeability_6 ,                                                                &
-    weighf , weighb ,                                                                    &
-    rhs   )
+    ( ivarfl(ipr), init, inc, imrgrp, iccocg, nswrgp, imligp, ircflp,        &
+      iphydr, iwgrp, iwarnp, epsrgp , climgp, extrap, rvoid,                 &
+      cvar_pr, coefa_p, coefb_p, coefaf_p, coefbf_p,                         &
+      viscf, viscb,                                                          &
+      cpro_permeability_6, weighf, weighb, rhs)
 
   endif
 
@@ -556,6 +552,7 @@ deallocate(dam, xam, rhs)
 iccocg = 1
 init = 1
 inc = 1
+imrgrp = vcopt_p%imrgra
 nswrgp = vcopt_p%nswrgr
 imligp = vcopt_p%imligr
 iwarnp = vcopt_p%iwarni
@@ -570,7 +567,7 @@ extrap = vcopt_p%extrag
 if (darcy_anisotropic_permeability.eq.0) then
 
   call itrmas &
- ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydr , &
+ ( f_id0  , init   , inc    , imrgrp , iccocg , nswrgp , imligp , iphydr , &
    iwgrp  , iwarnp ,                                              &
    epsrgp , climgp , extrap ,                                     &
    rvoid  ,                                                       &
@@ -588,7 +585,7 @@ if (darcy_anisotropic_permeability.eq.0) then
   init = 0
 
   call itrmas &
- ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydr ,     &
+ ( f_id0  , init   , inc    , imrgrp , iccocg , nswrgp , imligp , iphydr ,     &
    iwgrp  , iwarnp ,                                                           &
    epsrgp , climgp , extrap ,                                                  &
    rvoid  ,                                                                    &
@@ -601,7 +598,7 @@ if (darcy_anisotropic_permeability.eq.0) then
 else if (darcy_anisotropic_permeability.eq.1) then
 
   call itrmav &
- ( f_id0, init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp , &
+ ( f_id0, init   , inc    , imrgrp , iccocg , nswrgp , imligp , ircflp , &
    iphydr , iwgrp  , iwarnp ,                                            &
    epsrgp , climgp , extrap ,                                            &
    rvoid  ,                                                              &
@@ -621,7 +618,7 @@ else if (darcy_anisotropic_permeability.eq.1) then
   ircflp = 0
 
   call itrmav &
- ( f_id0, init   , inc    , imrgra , iccocg , nswrgp , imligp , ircflp ,  &
+ ( f_id0, init   , inc    , imrgrp , iccocg , nswrgp , imligp , ircflp ,  &
    iphydr , iwgrp  , iwarnp ,                                             &
    epsrgp , climgp , extrap ,                                             &
    rvoid  ,                                                               &
@@ -657,7 +654,7 @@ allocate(gradp(3,ncelet))
 
 ! We use gradient_scalar instead of potential because iphydr is
 ! not taken into account in case of Darcy calculation.
-call field_gradient_scalar(ivarfl(ipr), iprev, imrgra, inc,    &
+call field_gradient_scalar(ivarfl(ipr), iprev, 0, inc,    &
                            iccocg, gradp)
 
 ! Computation of velocity
@@ -665,13 +662,13 @@ if (darcy_anisotropic_permeability.eq.1) then
 
   !$omp parallel do
   do iel = 1, ncel
-    cvar_vel(1,iel) = - ( cpro_permeability_6(1,iel)*gradp(1,iel)          &
+    cvar_vel(1,iel) = - ( cpro_permeability_6(1,iel)*gradp(1,iel)      &
                          + cpro_permeability_6(4,iel)*gradp(2,iel)     &
                          + cpro_permeability_6(6,iel)*gradp(3,iel))
-    cvar_vel(2,iel) = - ( cpro_permeability_6(4,iel)*gradp(1,iel)          &
+    cvar_vel(2,iel) = - ( cpro_permeability_6(4,iel)*gradp(1,iel)      &
                          + cpro_permeability_6(2,iel)*gradp(2,iel)     &
                          + cpro_permeability_6(5,iel)*gradp(3,iel))
-    cvar_vel(3,iel) = - (cpro_permeability_6(6,iel)*gradp(1,iel)           &
+    cvar_vel(3,iel) = - (cpro_permeability_6(6,iel)*gradp(1,iel)       &
                          + cpro_permeability_6(5,iel)*gradp(2,iel)     &
                          + cpro_permeability_6(3,iel)*gradp(3,iel))
   enddo
@@ -749,20 +746,6 @@ deallocate(viscf, viscb)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
- 1400 format(1X,A16,' : SWEEP = ',I5,' NORME SECOND MEMBRE = ',E14.6,  &
-             ', RELAXP = ',E14.6)
- 1500 format ( &
- 1X,A16,' : Current reconstruction sweep = ',I5                     ,/,&
-'           sweep residual = ',E12.5,', norm = ',E12.5              ,/,&
-'           number of sweeps for solver = ',I5)
- 1600 format( &
-'@'                                                                 ,/,&
-'@ @@ ATTENTION : ', A16,' RICHARDS'                                ,/,&
-'@    ========='                                                    ,/,&
-'@  Nombre d''iterations maximal ',I10   ,' atteint'                ,/,&
-'@' )
-#else
  1400 format(1X,A16,' : SWEEP = ',I5,' RIGHT HAND SIDE NORM = ',E14.6, &
              ', RELAXP = ',E14.6)
  1500 format ( &
@@ -775,7 +758,6 @@ deallocate(viscf, viscb)
 '@    ========'                                                     ,/,&
 '@  Maximum number of iterations ',I10   ,' reached'                ,/,&
 '@'                                                              )
-#endif
 
 !--------
 ! End
