@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -263,13 +263,13 @@ const cs_data_joule_effect_t  *cs_glob_transformer     = NULL;
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*!
- * vaccum magnetic permeability constant (H/m). (= 1.2566e-6)
+ * vacuum magnetic permeability constant (H/m). (= 1.2566e-6)
  *
  */
 const cs_real_t cs_elec_permvi = 1.2566e-6;
 
 /*!
- * vaccum permittivity constant (F/m). (= 8.854e-12)
+ * vacuum permittivity constant (F/m). (= 8.854e-12)
  *
  */
 const cs_real_t cs_elec_epszer = 8.854e-12;
@@ -459,6 +459,102 @@ _field_pointer_properties_map_electric_arcs(void)
                        cs_field_by_name_try("electric_field"));
 }
 
+/*----------------------------------------------------------------------------
+ * convert temperature to enthalpy
+ *----------------------------------------------------------------------------*/
+
+static cs_real_t
+_cs_elec_convert_t_to_h(const cs_real_t ym[],
+                        cs_real_t       temp)
+{
+  int ngaz = cs_glob_elec_properties->ngaz;
+  int it   = cs_glob_elec_properties->npoint;
+
+  cs_real_t enthal = 0.;
+
+  if (temp >= cs_glob_elec_properties->th[it - 1]) {
+    for (int iesp = 0; iesp < ngaz; iesp++)
+      enthal += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it-1) + it-1];
+  }
+  else if (temp <= cs_glob_elec_properties->th[0]) {
+    for (int iesp = 0; iesp < ngaz; iesp++)
+      enthal += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it-1) + 0];
+  }
+  else {
+    for (int itt = 0; itt < cs_glob_elec_properties->npoint - 1; itt++) {
+      if (   temp > cs_glob_elec_properties->th[itt]
+          && temp <= cs_glob_elec_properties->th[itt + 1]) {
+        cs_real_t eh0 = 0.;
+        cs_real_t eh1 = 0.;
+
+        for (int iesp = 0; iesp < ngaz; iesp++) {
+          eh0 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it-1) + itt];
+          eh1 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it-1) + itt+1];
+        }
+
+        enthal = eh0 + (eh1 - eh0) * (temp - cs_glob_elec_properties->th[itt]) /
+                       (  cs_glob_elec_properties->th[itt + 1]
+                        - cs_glob_elec_properties->th[itt]);
+
+        break;
+      }
+    }
+  }
+
+  return enthal;
+}
+
+/*----------------------------------------------------------------------------
+ * Convert enthalpy to temperature
+ *----------------------------------------------------------------------------*/
+
+static cs_real_t
+_cs_elec_convert_h_to_t(const cs_real_t  ym[],
+                        cs_real_t        enthal)
+{
+  int ngaz = cs_glob_elec_properties->ngaz;
+  int it   = cs_glob_elec_properties->npoint;
+
+  cs_real_t eh1 = 0.;
+
+  for (int iesp = 0; iesp < ngaz; iesp++)
+    eh1 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it-1) + it - 1];
+
+  if (enthal >= eh1) {
+    return cs_glob_elec_properties->th[it - 1];
+  }
+
+  eh1 = 0.;
+
+  for (int iesp = 0; iesp < ngaz; iesp++)
+    eh1 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it-1) + 0];
+
+  if (enthal <= eh1) {
+    return cs_glob_elec_properties->th[0];
+  }
+
+  for (int itt = 0; itt < cs_glob_elec_properties->npoint - 1; itt++) {
+    cs_real_t eh0 = 0.;
+    eh1 = 0.;
+
+    for (int iesp = 0; iesp < ngaz; iesp++) {
+      eh0 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it-1) + itt];
+      eh1 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it-1) + itt+1];
+    }
+
+    if (enthal > eh0 && enthal <= eh1) {
+      cs_real_t temp = cs_glob_elec_properties->th[itt]
+                       + (enthal - eh0) * (  cs_glob_elec_properties->th[itt+1]
+                                           - cs_glob_elec_properties->th[itt])
+                                        / (eh1 - eh0);
+      return temp;
+    }
+  }
+
+  assert(0);  /* Should not arrive here */
+  return 0;
+}
+
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*=============================================================================
@@ -468,28 +564,33 @@ _field_pointer_properties_map_electric_arcs(void)
 /*----------------------------------------------------------------------------*/
 
 void
-CS_PROCF (elini1, ELINI1) (cs_real_t *visls0,
-                           cs_real_t *diftl0,
-                           cs_int_t  *idircl,
-                           cs_int_t  *isca)
+CS_PROCF (elini1, ELINI1) (cs_real_t *diftl0)
 {
-  cs_electrical_model_specific_initialization(visls0, diftl0, idircl, isca);
+  cs_electrical_model_specific_initialization(diftl0);
 }
 
 void
-CS_PROCF (elflux, ELFLUX) (cs_int_t *iappel)
+CS_PROCF (elflux, ELFLUX) (int      *iappel)
 {
   cs_elec_compute_fields(cs_glob_mesh,
                          *iappel);
 }
 
 void
-CS_PROCF (elthht, ELTHHT) (cs_int_t  *mode,
+CS_PROCF (elthht, ELTHHT) (int       *mode,
                            cs_real_t *ym,
                            cs_real_t *enthal,
                            cs_real_t *temp)
 {
-  cs_elec_convert_h_t(*mode, ym, enthal, temp);
+  if (*mode == -1)
+    *enthal = _cs_elec_convert_t_to_h(ym, *temp);
+  else if (*mode == 1)
+    *temp = _cs_elec_convert_h_to_t(ym, *enthal);
+  else
+    bft_error(__FILE__, __LINE__, 0,
+              _("electric module:\n"
+                "bad value for mode (integer equal to -1 or 1 : %i here.\n"),
+              *mode);
 }
 
 void
@@ -502,12 +603,11 @@ CS_PROCF (ellecd, ELLECD) (void)
 void
 CS_PROCF (elphyv, ELPHYV) (void)
 {
-  cs_elec_physical_properties(cs_glob_mesh,
-                              cs_glob_mesh_quantities);
+  cs_elec_physical_properties(cs_glob_domain);
 }
 
 void
-CS_PROCF (eltssc, ELTSSC) (const cs_int_t  *isca,
+CS_PROCF (eltssc, ELTSSC) (const int       *isca,
                            cs_real_t       *smbrs)
 {
   const cs_mesh_t *mesh = cs_glob_mesh;
@@ -532,7 +632,7 @@ CS_PROCF (eltsvv, ELTSVV) (const int       *f_id,
 }
 
 void
-CS_PROCF (eliniv, ELINIV) (cs_int_t  *isuite)
+CS_PROCF (eliniv, ELINIV) (int       *isuite)
 {
   cs_elec_fields_initialize(cs_glob_mesh,  *isuite);
 }
@@ -548,7 +648,7 @@ CS_PROCF (elreca, ELRECA) (cs_real_t *dt)
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- * Provide acces to cs_elec_option
+ * Provide access to cs_elec_option
  *----------------------------------------------------------------------------*/
 
 cs_elec_option_t *
@@ -558,7 +658,7 @@ cs_get_glob_elec_option(void)
 }
 
 /*----------------------------------------------------------------------------
- * Provide acces to cs_glob_transformer
+ * Provide access to cs_glob_transformer
  *----------------------------------------------------------------------------*/
 
 cs_data_joule_effect_t *
@@ -651,26 +751,21 @@ cs_electrical_model_finalize(void)
  *----------------------------------------------------------------------------*/
 
 void
-cs_electrical_model_specific_initialization(cs_real_t  *visls0,
-                                            cs_real_t  *diftl0,
-                                            int        *idircl,
-                                            int        *isca)
+cs_electrical_model_specific_initialization(cs_real_t  *diftl0)
 {
   cs_field_t *f = NULL;
   int key_cal_opt_id = cs_field_key_id("var_cal_opt");
-  const int keysca = cs_field_key_id("scalar_id");
+  const int kvisls0 = cs_field_key_id("diffusivity_ref");
   const int ksigmas = cs_field_key_id("turbulent_schmidt");
   cs_var_cal_opt_t var_cal_opt;
 
   /* specific initialization for field */
   f = CS_F_(potr);
   cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-  int id = cs_field_get_key_int(f, keysca) - 1;
   var_cal_opt.iconv  = 0;
   var_cal_opt.istat  = 0;
   var_cal_opt.idiff  = 1;
   var_cal_opt.idifft = 0;
-  idircl[isca[id] - 1] = 1;
 
   cs_field_set_key_struct(f, key_cal_opt_id, &var_cal_opt);
 
@@ -680,39 +775,33 @@ cs_electrical_model_specific_initialization(cs_real_t  *visls0,
   if (ieljou == 2 || ieljou == 4) {
     f = CS_F_(poti);
     cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-    id = cs_field_get_key_int(f, keysca) - 1;
     var_cal_opt.iconv  = 0;
     var_cal_opt.istat  = 0;
     var_cal_opt.idiff  = 1;
     var_cal_opt.idifft = 0;
-    idircl[isca[id] - 1] = 1;
     cs_field_set_key_struct(f, key_cal_opt_id, &var_cal_opt);
   }
 
   if (ielarc > 1) {
     cs_field_t  *fp = cs_field_by_name_try("vec_potential");
     cs_field_get_key_struct(fp, key_cal_opt_id, &var_cal_opt);
-    id = cs_field_get_key_int(fp, keysca) - 1;
     var_cal_opt.iconv  = 0;
     var_cal_opt.istat  = 0;
     var_cal_opt.idiff  = 1;
     var_cal_opt.idifft = 0;
-    idircl[isca[id] - 1] = 1;
-    visls0[id  ] = 1.;
     cs_field_set_key_struct(fp, key_cal_opt_id, &var_cal_opt);
+    cs_field_set_key_double(fp, kvisls0, 1.0);
   }
 
   /* for all specific field */
   f = CS_F_(h);
   cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-  id = cs_field_get_key_int(f, keysca) - 1;
   var_cal_opt.blencv = 1.;
   cs_field_set_key_double(f, ksigmas, 0.7);
   cs_field_set_key_struct(f, key_cal_opt_id, &var_cal_opt);
 
   f = CS_F_(potr);
   cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-  id = cs_field_get_key_int(f, keysca) - 1;
   var_cal_opt.blencv = 1.;
   cs_field_set_key_double(f, ksigmas, 0.7);
   cs_field_set_key_struct(f, key_cal_opt_id, &var_cal_opt);
@@ -720,7 +809,6 @@ cs_electrical_model_specific_initialization(cs_real_t  *visls0,
   if (ieljou == 2 || ieljou == 4) {
     f = CS_F_(poti);
     cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-    id = cs_field_get_key_int(f, keysca) - 1;
     var_cal_opt.blencv = 1.;
     cs_field_set_key_double(f, ksigmas, 0.7);
     cs_field_set_key_struct(f, key_cal_opt_id, &var_cal_opt);
@@ -729,7 +817,6 @@ cs_electrical_model_specific_initialization(cs_real_t  *visls0,
   if (ielarc > 1) {
     cs_field_t  *fp = cs_field_by_name_try("vec_potential");
     cs_field_get_key_struct(fp, key_cal_opt_id, &var_cal_opt);
-    id = cs_field_get_key_int(fp, keysca) - 1;
     var_cal_opt.blencv = 1.;
     cs_field_set_key_double(f, ksigmas, 0.7);
     cs_field_set_key_struct(fp, key_cal_opt_id, &var_cal_opt);
@@ -739,22 +826,17 @@ cs_electrical_model_specific_initialization(cs_real_t  *visls0,
     for (int igaz = 0; igaz < cs_glob_elec_properties->ngaz - 1; igaz++) {
       f = CS_FI_(ycoel, igaz);
       cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
-      id = cs_field_get_key_int(f, keysca) - 1;
       var_cal_opt.blencv = 1.;
       cs_field_set_key_double(f, ksigmas, 0.7);
       cs_field_set_key_struct(f, key_cal_opt_id, &var_cal_opt);
     }
   }
 
-  if (cs_gui_file_is_loaded()) {
-    CS_PROCF(uicpi1,UICPI1) (&(_elec_option.srrom), diftl0);
-    cs_gui_elec_model();
-    _elec_option.pot_diff = 1000.;//FIXME
-  }
+  CS_PROCF(uicpi1,UICPI1) (&(_elec_option.srrom), diftl0);
+  cs_gui_elec_model();
+  _elec_option.pot_diff = 1000.;//FIXME
 
   _cs_electrical_model_verify();
-
-  return;
 }
 
 /*----------------------------------------------------------------------------
@@ -949,115 +1031,18 @@ cs_electrical_properties_read(void)
   }
 }
 
-/*----------------------------------------------------------------------------
- * convert enthalpy-temperature
- *----------------------------------------------------------------------------*/
-
-void
-cs_elec_convert_h_t(int        mode,
-                    cs_real_t *ym,
-                    cs_real_t *enthal,
-                    cs_real_t *temp)
-{
-  int ngaz = cs_glob_elec_properties->ngaz;
-  int it   = cs_glob_elec_properties->npoint;
-
-  /* convert temperature to enthalpy */
-  if (mode == -1) {
-    *enthal = 0.;
-
-    if (*temp >= cs_glob_elec_properties->th[it - 1]) {
-      for (int iesp = 0; iesp < ngaz; iesp++)
-        *enthal += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it - 1) + it - 1];
-    }
-    else if (*temp <= cs_glob_elec_properties->th[0]) {
-      for (int iesp = 0; iesp < ngaz; iesp++)
-        *enthal += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it - 1) + 0];
-    }
-    else {
-      for (int itt = 0; itt < cs_glob_elec_properties->npoint - 1; itt++) {
-        if (*temp > cs_glob_elec_properties->th[itt] &&
-            *temp <= cs_glob_elec_properties->th[itt + 1]) {
-          cs_real_t eh0 = 0.;
-          cs_real_t eh1 = 0.;
-
-          for (int iesp = 0; iesp < ngaz; iesp++) {
-            eh0 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it - 1) + itt    ];
-            eh1 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it - 1) + itt + 1];
-          }
-
-          *enthal = eh0 + (eh1 - eh0) * (*temp - cs_glob_elec_properties->th[itt]) /
-                          (  cs_glob_elec_properties->th[itt + 1]
-                           - cs_glob_elec_properties->th[itt]);
-
-          break;
-        }
-      }
-    }
-    return;
-  }
-  /* convert enthalpy to temperature */
-  else if (mode == 1) {
-    cs_real_t eh1 = 0.;
-
-    for (int iesp = 0; iesp < ngaz; iesp++)
-      eh1 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it - 1) + it - 1];
-
-    if (*enthal >= eh1) {
-      *temp = cs_glob_elec_properties->th[it - 1];
-      return;
-    }
-
-    eh1 = 0.;
-
-    for (int iesp = 0; iesp < ngaz; iesp++)
-      eh1 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it - 1) + 0];
-
-    if (*enthal <= eh1) {
-      *temp = cs_glob_elec_properties->th[0];
-      return;
-    }
-
-    for (int itt = 0; itt < cs_glob_elec_properties->npoint - 1; itt++) {
-      cs_real_t eh0 = 0.;
-      eh1 = 0.;
-
-      for (int iesp = 0; iesp < ngaz; iesp++) {
-        eh0 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it - 1) + itt    ];
-        eh1 += ym[iesp] * cs_glob_elec_properties->ehgaz[iesp * (it - 1) + itt + 1];
-      }
-
-      if (*enthal > eh0 && *enthal <= eh1) {
-        *temp = cs_glob_elec_properties->th[itt] +
-                  (*enthal - eh0) *
-                  (  cs_glob_elec_properties->th[itt + 1]
-                   - cs_glob_elec_properties->th[itt]) /
-                  (eh1 - eh0);
-        break;
-      }
-    }
-    return;
-  }
-  else
-    bft_error(__FILE__, __LINE__, 0,
-              _("electric module:\n"
-                "bad value for mode (integer equal to -1 or 1 : %i here.\n"),
-              mode);
-}
-
 /*----------------------------------------------------------------------------
  * compute physical properties
  *----------------------------------------------------------------------------*/
 
 void
-cs_elec_physical_properties(const cs_mesh_t             *mesh,
-                            const cs_mesh_quantities_t  *mesh_quantities)
+cs_elec_physical_properties(cs_domain_t  *domain)
 {
   static long ipass = 0;
   int nt_cur = cs_glob_time_step->nt_cur;
   int isrrom = 0;
-  cs_lnum_t  n_cells = mesh->n_cells;
-  const int keysca = cs_field_key_id("scalar_diffusivity_id");
+  const cs_lnum_t  n_cells = domain->mesh->n_cells;
+  const int keysca = cs_field_key_id("diffusivity_id");
   int diff_id = cs_field_get_key_int(CS_F_(potr), keysca);
   cs_field_t *c_prop = NULL;
   if (diff_id > -1)
@@ -1072,8 +1057,8 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
   /* Joule effect (law must be specified by user) */
 
   int ifcvsl = cs_field_get_key_int(CS_F_(h), keysca);
-  cs_field_t *diff_th;
-  if (ifcvsl > 0)
+  cs_field_t *diff_th = NULL;
+  if (ifcvsl >= 0)
     diff_th = cs_field_by_id(ifcvsl);
 
   int ielarc = cs_glob_physical_model_flag[CS_ELECTRIC_ARCS];
@@ -1085,7 +1070,6 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
       bft_printf("electric arc module: properties read on file.\n");
 
     /* compute temperature from enthalpy */
-    int mode = 1;
     int ngaz = e_props->ngaz;
     int npt  = e_props->npoint;
 
@@ -1107,9 +1091,7 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
       ym[0] = 1.;
 
       for (cs_lnum_t iel = 0; iel < n_cells; iel++)
-        cs_elec_convert_h_t(mode, ym,
-                          &(CS_F_(h)->val[iel]),
-                          &(CS_F_(t)->val[iel]));
+        CS_F_(t)->val[iel] = _cs_elec_convert_h_to_t(ym, CS_F_(h)->val[iel]);
     }
     else {
 
@@ -1121,9 +1103,7 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
           ym[ngaz - 1] -= ym[ii];
         }
 
-        cs_elec_convert_h_t(mode, ym,
-                          &(CS_F_(h)->val[iel]),
-                          &(CS_F_(t)->val[iel]));
+        CS_F_(t)->val[iel] = _cs_elec_convert_h_to_t(ym, CS_F_(h)->val[iel]);
       }
     }
 
@@ -1132,11 +1112,13 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
     cs_real_t *cpro_absco = NULL;
 
     if (cs_glob_elec_option->ixkabe == 1) {
-      if (cs_glob_field_pointers[CS_ENUMF_(rad_cak)].a != NULL)
+      if (CS_FI_(rad_cak, 0) != NULL)
         cpro_absco = CS_FI_(rad_cak, 0)->val;
     }
 
-    /* interpolate properties */
+    /* Interpolate properties */
+
+#   pragma omp parallel for
     for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
       // temperature
       cs_real_t tp = CS_F_(t)->val[iel];
@@ -1254,24 +1236,30 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
       }
 
       /* compute molecular viscosity : kg/(m s) */
-      for (int iesp1 = 0; iesp1 < ngaz; iesp1++)
+      for (int iesp1 = 0; iesp1 < ngaz; iesp1++) {
         for (int iesp2 = 0; iesp2 < ngaz; iesp2++) {
-          coef[iesp1 * (ngaz - 1) + iesp2] = 1. + sqrt(visesp[iesp1] / visesp[iesp2]) *
-                                             sqrt(sqrt(roesp[iesp2] / roesp[iesp1]));
+          coef[iesp1 * (ngaz - 1) + iesp2]
+            = 1. +   sqrt(visesp[iesp1] / visesp[iesp2])
+                   * sqrt(sqrt(roesp[iesp2] / roesp[iesp1]));
           coef[iesp1 * (ngaz - 1) + iesp2] *= coef[iesp1 * (ngaz - 1) + iesp2];
-          coef[iesp1 * (ngaz - 1) + iesp2] /= (sqrt(1. + roesp[iesp1] / roesp[iesp2]) * sqrt(8.));
+          coef[iesp1 * (ngaz - 1) + iesp2] /=    (sqrt(1. + roesp[iesp1]
+                                               / roesp[iesp2]) * sqrt(8.));
         }
+      }
 
       CS_F_(mu)->val[iel] = 0.;
 
       for (int iesp1 = 0; iesp1 < ngaz; iesp1++) {
-        double somphi = 0.;
-        for (int iesp2 = 0; iesp2 < ngaz; iesp2++) {
-          if (iesp1 != iesp2)
-            somphi += coef[iesp1 * (ngaz - 1) + iesp2] * yvol[iesp2] / yvol[iesp1];
-        }
+        if (yvol[iesp1] > 1e-30) {
+          double somphi = 0.;
+          for (int iesp2 = 0; iesp2 < ngaz; iesp2++) {
+            if (iesp1 != iesp2)
+              somphi +=   coef[iesp1 * (ngaz - 1) + iesp2]
+                        * yvol[iesp2] / yvol[iesp1];
+          }
 
-        CS_F_(mu)->val[iel] += visesp[iesp1] / (1. + somphi);
+          CS_F_(mu)->val[iel] += visesp[iesp1] / (1. + somphi);
+        }
       }
 
       /* compute specific heat : J/(kg degres) */
@@ -1282,8 +1270,9 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
       }
 
       /* compute Lambda/Cp : kg/(m s) */
-      if (ifcvsl >= 0) {
-        for (int iesp1 = 0; iesp1 < ngaz; iesp1++)
+      if (diff_th != NULL) {
+
+        for (int iesp1 = 0; iesp1 < ngaz; iesp1++) {
           for (int iesp2 = 0; iesp2 < ngaz; iesp2++) {
             coef[iesp1 * (ngaz - 1) + iesp2]
               = 1. +   sqrt(xlabes[iesp1] / xlabes[iesp2])
@@ -1293,18 +1282,21 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
             coef[iesp1 * (ngaz - 1) + iesp2]
               /= (sqrt(1. + roesp[iesp1] / roesp[iesp2]) * sqrt(8.));
           }
+        }
         /* Lambda */
         diff_th->val[iel] = 0.;
 
         for (int iesp1 = 0; iesp1 < ngaz; iesp1++) {
-          double somphi = 0.;
-          for (int iesp2 = 0; iesp2 < ngaz; iesp2++) {
-            if (iesp1 != iesp2)
-              somphi += coef[  iesp1 * (ngaz - 1) + iesp2]
-                             * yvol[iesp2] / yvol[iesp1];
-          }
+          if (yvol[iesp1] > 1e-30) {
+            double somphi = 0.;
+            for (int iesp2 = 0; iesp2 < ngaz; iesp2++) {
+              if (iesp1 != iesp2)
+                somphi +=   coef[iesp1 * (ngaz - 1) + iesp2]
+                          * yvol[iesp2] / yvol[iesp1];
+            }
 
-          diff_th->val[iel] += xlabes[iesp1] / (1. + 1.065 * somphi);
+            diff_th->val[iel] += xlabes[iesp1] / (1. + 1.065 * somphi);
+          }
         }
 
         /* Lambda/Cp */
@@ -1314,7 +1306,7 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
           diff_th->val[iel] /= CS_F_(cp)->val[iel];
       }
 
-      /* compute electric conductivity : S/m */
+      /* compute electric conductivity: S/m */
       if (ifcsig >= 0) {
         c_prop->val[iel] = 0.;
         double val = 0.;
@@ -1347,7 +1339,8 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
       /* diffusivity for other properties
        * nothing to do
        * no other properties in this case */
-    }
+
+    } /* End of loop on cells */
 
     BFT_FREE(ym);
     BFT_FREE(yvol);
@@ -1361,7 +1354,7 @@ cs_elec_physical_properties(const cs_mesh_t             *mesh,
   }
 
   /* now user properties (for joule effect particulary) */
-  cs_user_physical_properties(mesh, mesh_quantities);
+  cs_user_physical_properties(domain);
 }
 
 /*----------------------------------------------------------------------------
@@ -1374,17 +1367,12 @@ cs_elec_compute_fields(const cs_mesh_t  *mesh,
 {
   cs_lnum_t  n_cells   = mesh->n_cells;
   cs_lnum_t  n_cells_ext = mesh->n_cells_with_ghosts;
-  const int keysca  = cs_field_key_id("scalar_diffusivity_id");
-
-  cs_halo_type_t halo_type;
-  cs_gradient_type_t gradient_type;
+  const int keysca  = cs_field_key_id("diffusivity_id");
 
   int ieljou = cs_glob_physical_model_flag[CS_JOULE_EFFECT];
   int ielarc = cs_glob_physical_model_flag[CS_ELECTRIC_ARCS];
 
-  /* if listing printing is needed */
-  int modntl = 0;
-  //TODO : control listing output
+  bool log_active = cs_log_default_is_active();
 
   /* Reconstructed value */
   cs_real_3_t *grad;
@@ -1437,52 +1425,55 @@ cs_elec_compute_fields(const cs_mesh_t  *mesh,
     }
 
     /* compute min max for E and J */
-    if (modntl == 0) {
+    if (log_active) {
       bft_printf("-----------------------------------------\n"
                  "   Variable         Minimum       Maximum\n"
                  "-----------------------------------------\n");
 
       /* Grad PotR = -E */
-      double vrmin, vrmax;
+      double vrmin[3], vrmax[3];
 
       for (int i = 0; i < 3; i++) {
-        vrmin = grad[0][i];
-        vrmax = grad[0][i];
+        vrmin[i] = HUGE_VAL;
+        vrmax[i] = -HUGE_VAL;
+      }
 
-        for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
-          vrmin = CS_MIN(vrmin, grad[iel][i]);
-          vrmax = CS_MAX(vrmax, grad[iel][i]);
+      for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
+        for (int i = 0; i < 3; i++) {
+          vrmin[i] = CS_MIN(vrmin[i], grad[iel][i]);
+          vrmax[i] = CS_MAX(vrmax[i], grad[iel][i]);
         }
+      }
 
-        cs_parall_min(1, CS_DOUBLE, &vrmin);
-        cs_parall_max(1, CS_DOUBLE, &vrmax);
-        if (i == 0)
-          bft_printf("v  Gr_PotRX    %12.5E  %12.5E\n", vrmin, vrmax);
-        else if (i == 1)
-          bft_printf("v  Gr_PotRY    %12.5E  %12.5E\n", vrmin, vrmax);
-        else if (i == 2)
-          bft_printf("v  Gr_PotRZ    %12.5E  %12.5E\n", vrmin, vrmax);
+      cs_parall_min(3, CS_DOUBLE, vrmin);
+      cs_parall_max(3, CS_DOUBLE, vrmax);
+
+      for (int i = 0; i < 3; i++) {
+        bft_printf("v  Gr_PotR%s    %12.5e  %12.5e\n",
+                   cs_glob_field_comp_name_3[i],
+                   vrmin[i], vrmax[i]);
       }
 
       /* current real */
       for (int i = 0; i < 3; i++) {
-        vrmin = -c_prop->val[0] * grad[0][i];
-        vrmax = -c_prop->val[0] * grad[0][i];
+        vrmin[i] = HUGE_VAL;
+        vrmax[i] = -HUGE_VAL;
+      }
 
-        for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
-          vrmin = CS_MIN(vrmin, -c_prop->val[iel] * grad[iel][i]);
-          vrmax = CS_MAX(vrmax, -c_prop->val[iel] * grad[iel][i]);
+      for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
+        for (int i = 0; i < 3; i++) {
+          vrmin[i] = CS_MIN(vrmin[i], -c_prop->val[iel] * grad[iel][i]);
+          vrmax[i] = CS_MAX(vrmax[i], -c_prop->val[iel] * grad[iel][i]);
         }
+      }
 
-        cs_parall_min(1, CS_DOUBLE, &vrmin);
-        cs_parall_max(1, CS_DOUBLE, &vrmax);
+      cs_parall_min(3, CS_DOUBLE, vrmin);
+      cs_parall_max(3, CS_DOUBLE, vrmax);
 
-        if (i == 0)
-          bft_printf("v  Cour_ReX    %12.5E  %12.5E\n", vrmin, vrmax);
-        else if (i == 1)
-          bft_printf("v  Cour_ReY    %12.5E  %12.5E\n", vrmin, vrmax);
-        else if (i == 2)
-          bft_printf("v  Cour_ReZ    %12.5E  %12.5E\n", vrmin, vrmax);
+      for (int i = 0; i < 3; i++) {
+        bft_printf("v  Cour_Re%s    %12.5E  %12.5E\n",
+                   cs_glob_field_comp_name_3[i],
+                   vrmin[i], vrmax[i]);
       }
       bft_printf("-----------------------------------------\n");
     }
@@ -1515,57 +1506,59 @@ cs_elec_compute_fields(const cs_mesh_t  *mesh,
 
       /* compute joule effect : j . E */
       for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
-        CS_F_(joulp)->val[iel] +=  c_propi->val[iel] *
-                                  (grad[iel][0] * grad[iel][0] +
-                                   grad[iel][1] * grad[iel][1] +
-                                   grad[iel][2] * grad[iel][2]);
+        CS_F_(joulp)->val[iel] +=   c_propi->val[iel]
+                                  * cs_math_3_square_norm(grad[iel]);
       }
 
       /* compute min max for E and J */
-      if (modntl == 0) {
+      if (log_active) {
+
+        double vrmin[3], vrmax[3];
+
         /* Grad PotR = -Ei */
-        double vrmin, vrmax;
 
         for (int i = 0; i < 3; i++) {
-          vrmin = grad[0][i];
-          vrmax = grad[0][i];
-
-          for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
-            vrmin = CS_MIN(vrmin, grad[iel][0]);
-            vrmax = CS_MAX(vrmax, grad[iel][0]);
-          }
-
-          cs_parall_min(1, CS_DOUBLE, &vrmin);
-          cs_parall_max(1, CS_DOUBLE, &vrmax);
-
-          if (i == 0)
-            bft_printf("v  Gr_PotIX    %12.5E  %12.5E\n", vrmin, vrmax);
-          else if (i == 1)
-            bft_printf("v  Gr_PotIY    %12.5E  %12.5E\n", vrmin, vrmax);
-          else if (i == 2)
-            bft_printf("v  Gr_PotIZ    %12.5E  %12.5E\n", vrmin, vrmax);
+          vrmin[i] = HUGE_VAL;
+          vrmax[i] = -HUGE_VAL;
         }
 
-        /* current imaginary */
-        for (int i = 0; i < 3; i++) {
-          vrmin = -c_propi->val[0] * grad[0][i];
-          vrmax = -c_propi->val[0] * grad[0][i];
-
-          for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
-            vrmin = CS_MIN(vrmin, -c_propi->val[iel] * grad[iel][i]);
-            vrmax = CS_MAX(vrmax, -c_propi->val[iel] * grad[iel][i]);
+        for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
+          for (int i = 0; i < 3; i++) {
+            vrmin[i] = CS_MIN(vrmin[i], grad[iel][0]);
+            vrmax[i] = CS_MAX(vrmax[i], grad[iel][0]);
           }
+        }
 
-          cs_parall_min(1, CS_DOUBLE, &vrmin);
-          cs_parall_max(1, CS_DOUBLE, &vrmax);
+        cs_parall_min(3, CS_DOUBLE, vrmin);
+        cs_parall_max(3, CS_DOUBLE, vrmax);
 
-          if (i == 0)
-            bft_printf("v  Cour_ImX    %12.5E  %12.5E\n", vrmin, vrmax);
-          else if (i == 1)
-            bft_printf("v  Cour_ImY    %12.5E  %12.5E\n", vrmin, vrmax);
-          else if (i == 2)
-            bft_printf("v  Cour_ImZ    %12.5E  %12.5E\n", vrmin, vrmax);
+        for (int i = 0; i < 3; i++) {
+          bft_printf("v  Gr_PotI%s    %12.5E  %12.5E\n",
+                     cs_glob_field_comp_name_3[i],
+                     vrmin[i], vrmax[i]);
+        }
 
+        /* Imaginary current */
+
+        for (int i = 0; i < 3; i++) {
+          vrmin[i] = HUGE_VAL;
+          vrmax[i] = -HUGE_VAL;
+        }
+
+        for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
+          for (int i = 0; i < 3; i++) {
+            vrmin[i] = CS_MIN(vrmin[i], -c_propi->val[iel] * grad[iel][i]);
+            vrmax[i] = CS_MAX(vrmax[i], -c_propi->val[iel] * grad[iel][i]);
+          }
+        }
+
+        cs_parall_min(3, CS_DOUBLE, vrmin);
+        cs_parall_max(3, CS_DOUBLE, vrmax);
+
+        for (int i = 0; i < 3; i++) {
+          bft_printf("v  Cour_Im%s    %12.5E  %12.5E\n",
+                     cs_glob_field_comp_name_3[i],
+                     vrmin[i], vrmax[i]);
         }
       }
     }
@@ -1616,48 +1609,28 @@ cs_elec_compute_fields(const cs_mesh_t  *mesh,
     }
 
     /* compute min max for B */
-    if (ielarc > 1) {
-      if (modntl == 0) {
-        /* Grad PotR = -E */
-        double vrmin, vrmax;
-        vrmin = cpro_magfl[0][0];
-        vrmax = cpro_magfl[0][0];
+    if (ielarc > 1 && log_active) {
+      /* Grad PotR = -E */
+      double vrmin[3], vrmax[3];
 
-        for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
-          vrmin = CS_MIN(vrmin, cpro_magfl[iel][0]);
-          vrmax = CS_MAX(vrmax, cpro_magfl[iel][0]);
+      for (int i = 0; i < 3; i++) {
+        vrmin[i] = HUGE_VAL;
+        vrmax[i] = -HUGE_VAL;
+      }
+
+      for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
+        for (int i = 0; i < 3; i++) {
+          vrmin[i] = CS_MIN(vrmin[i], cpro_magfl[iel][i]);
+          vrmax[i] = CS_MAX(vrmax[i], cpro_magfl[iel][i]);
         }
+      }
 
-        cs_parall_min(1, CS_DOUBLE, &vrmin);
-        cs_parall_max(1, CS_DOUBLE, &vrmax);
+      cs_parall_min(3, CS_DOUBLE, vrmin);
+      cs_parall_max(3, CS_DOUBLE, vrmax);
 
-        bft_printf("v  Magnetic_fieldX    %12.5E  %12.5E\n", vrmin, vrmax);
-
-        vrmin = cpro_magfl[0][1];
-        vrmax = cpro_magfl[0][1];
-
-        for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
-          vrmin = CS_MIN(vrmin, cpro_magfl[iel][1]);
-          vrmax = CS_MAX(vrmax, cpro_magfl[iel][1]);
-        }
-
-        cs_parall_min(1, CS_DOUBLE, &vrmin);
-        cs_parall_max(1, CS_DOUBLE, &vrmax);
-
-        bft_printf("v  Magnetic_fieldY    %12.5E  %12.5E\n", vrmin, vrmax);
-
-        vrmin = cpro_magfl[0][2];
-        vrmax = cpro_magfl[0][2];
-
-        for (cs_lnum_t iel = 0; iel < n_cells; iel++) {
-          vrmin = CS_MIN(vrmin, cpro_magfl[iel][2]);
-          vrmax = CS_MAX(vrmax, cpro_magfl[iel][2]);
-        }
-
-        cs_parall_min(1, CS_DOUBLE, &vrmin);
-        cs_parall_max(1, CS_DOUBLE, &vrmax);
-
-        bft_printf("v  Magnetic_fieldZ    %12.5E  %12.5E\n", vrmin, vrmax);
+      for (int i = 0; i < 3; i++) {
+        bft_printf("v  Magnetic_field%s    %12.5E  %12.5E\n",
+                   cs_glob_field_comp_name_3[i], vrmin[i], vrmax[i]);
       }
     }
   }
@@ -1693,7 +1666,7 @@ cs_elec_source_terms(const cs_mesh_t             *mesh,
 
   /* enthalpy source term */
   if (strcmp(name, "enthalpy") == 0) {
-    if (var_cal_opt.iwarni > 0)
+    if (var_cal_opt.verbosity > 0)
       bft_printf("compute source terms for variable : %s\n", name);
 
     if (cs_glob_time_step->nt_cur > 2) {
@@ -1708,7 +1681,7 @@ cs_elec_source_terms(const cs_mesh_t             *mesh,
       for (cs_lnum_t iel = 0; iel < n_cells; iel++)
         smbrs[iel] += w1[iel];
 
-      if (var_cal_opt.iwarni > 0) {
+      if (var_cal_opt.verbosity > 0) {
         double valmin = w1[0];
         double valmax = w1[0];
 
@@ -1754,8 +1727,8 @@ cs_elec_source_terms_v(const cs_mesh_t             *mesh,
   if (ielarc >= 2 && f_id == (CS_F_(potva)->id)) {
     cs_real_3_t *cpro_curre = (cs_real_3_t *)(CS_F_(curre)->val);
 
-    if (var_cal_opt.iwarni > 0)
-      bft_printf("compute source terms for variable : %s\n", f->name);
+    if (var_cal_opt.verbosity > 0)
+      bft_printf("compute source terms for variable: %s\n", f->name);
 
     for (cs_lnum_t iel = 0; iel < n_cells; iel++)
       for (int isou = 0; isou < 3; isou++)
@@ -1777,7 +1750,7 @@ cs_elec_add_variable_fields(void)
 
   const int kscmin = cs_field_key_id("min_scalar_clipping");
   const int kscmax = cs_field_key_id("max_scalar_clipping");
-  const int kivisl = cs_field_key_id("scalar_diffusivity_id");
+  const int kivisl = cs_field_key_id("diffusivity_id");
 
   const cs_data_elec_t  *e_props = cs_glob_elec_properties; /* local name */
 
@@ -1895,7 +1868,7 @@ cs_elec_add_property_fields(void)
                         has_previous);
     cs_field_set_key_int(f, keyvis, post_flag);
     cs_field_set_key_int(f, keylog, 1);
-    cs_field_set_key_str(f, klbl, "PuisJoul");
+    cs_field_set_key_str(f, klbl, "PowJoul");
   }
 
   {
@@ -1929,7 +1902,7 @@ cs_elec_add_property_fields(void)
                         has_previous);
     cs_field_set_key_int(f, keyvis, post_flag);
     cs_field_set_key_int(f, keylog, 1);
-    cs_field_set_key_str(f, klbl, "Curent_Imag");
+    cs_field_set_key_str(f, klbl, "Current_Imag");
   }
 
   /* specific for electric arcs */
@@ -1971,7 +1944,7 @@ cs_elec_add_property_fields(void)
                         has_previous);
     cs_field_set_key_int(f, keyvis, post_flag);
     cs_field_set_key_int(f, keylog, 1);
-    cs_field_set_key_str(f, klbl, "TS_radia");
+    cs_field_set_key_str(f, klbl, "ST_radia");
   }
 
 
@@ -2009,7 +1982,7 @@ cs_elec_fields_initialize(const cs_mesh_t   *mesh,
           ym[i] = 0.;
 
       cs_real_t tinit = cs_glob_fluid_properties->t0;
-      cs_elec_convert_h_t(-1, ym, &hinit, &tinit);
+      hinit = _cs_elec_convert_t_to_h(ym, tinit);
       BFT_FREE(ym);
     }
 
@@ -2063,7 +2036,7 @@ cs_elec_scaling_function(const cs_mesh_t             *mesh,
       if (coepot < 0.75)
         coepot = 0.75;
 
-      bft_printf("imposed current / current %14.5E, scaling coef. %14.5E\n",
+      bft_printf("imposed current / current %14.5e, scaling coef. %14.5e\n",
                  coepoa, coepot);
     }
     else if (cs_glob_elec_option->modrec == 2) {
@@ -2071,6 +2044,8 @@ cs_elec_scaling_function(const cs_mesh_t             *mesh,
       cs_gui_elec_model_rec();
       double elcou = 0.;
       cs_real_3_t *cpro_curre = (cs_real_3_t *)(CS_F_(curre)->val);
+      if (mesh->halo != NULL)
+        cs_halo_sync_var_strided(mesh->halo, CS_HALO_STANDARD, cpro_curre, 3);
       for (cs_lnum_t ifac = 0; ifac < nfac; ifac++) {
         if (cs_glob_elec_option->izreca[ifac] > 0) {
           bool ok = true;
@@ -2200,6 +2175,174 @@ cs_elec_scaling_function(const cs_mesh_t             *mesh,
   cs_user_scaling_elec(mesh, mesh_quantities, dt);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Convert enthalpy to temperature at all boundary faces.
+ *
+ * This handles both user and model enthalpy conversions, so can be used
+ * safely whenever conversion is needed.
+ *
+ * \param[in]   h   enthalpy values
+ * \param[out]  t   temperature values
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_elec_convert_h_to_t_faces(const cs_real_t  h[],
+                             cs_real_t        t[])
+{
+  const cs_mesh_t *m = cs_glob_mesh;
+  const cs_lnum_t n_b_faces = m->n_b_faces;
+
+  const cs_data_elec_t  *el_p = cs_glob_elec_properties;
+  const int n_gasses = el_p->ngaz;
+
+  if (n_gasses == 1) {
+
+    cs_real_t ym[1] = {1.};
+
+    for (cs_lnum_t i = 0; i < n_b_faces; i++)
+      t[i] = _cs_elec_convert_h_to_t(ym, h[i]);
+
+  }
+  else {
+
+    const cs_lnum_t *b_face_cells = m->b_face_cells;
+
+    cs_real_t *ym;
+    BFT_MALLOC(ym, n_gasses, cs_real_t);
+
+    for (cs_lnum_t f_id = 0; f_id < n_b_faces; f_id++) {
+
+      cs_lnum_t c_id = b_face_cells[f_id];
+
+      ym[n_gasses - 1] = 1.;
+      for (int gas_id = 0; gas_id < n_gasses - 1; gas_id++) {
+        ym[gas_id] = CS_FI_(ycoel, gas_id)->val[c_id];
+        ym[n_gasses - 1] -= ym[gas_id];
+      }
+
+      t[f_id] = _cs_elec_convert_h_to_t(ym, h[f_id]);
+
+    }
+
+    BFT_FREE(ym);
+
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Convert temperature to enthalpy at all cells.
+ *
+ * This handles both user and model temperature conversions, so can be used
+ * safely whenever conversion is needed.
+ *
+ * \param[in]   t   temperature values
+ * \param[out]  h   enthalpy values
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_elec_convert_t_to_h_cells(const cs_real_t  t[],
+                             cs_real_t        h[])
+{
+  const cs_mesh_t *m = cs_glob_mesh;
+  const cs_lnum_t n_cells = m->n_cells;
+
+  const cs_data_elec_t  *el_p = cs_glob_elec_properties;
+  const int n_gasses = el_p->ngaz;
+
+  if (n_gasses == 1) {
+
+    cs_real_t ym[1] = {1.};
+
+    for (cs_lnum_t i = 0; i < n_cells; i++)
+      h[i] = _cs_elec_convert_t_to_h(ym, t[i]);
+
+  }
+  else {
+
+    cs_real_t *ym;
+    BFT_MALLOC(ym, n_gasses, cs_real_t);
+
+    for (cs_lnum_t c_id = 0; c_id < n_cells; c_id++) {
+
+      ym[n_gasses - 1] = 1.;
+      for (int gas_id = 0; gas_id < n_gasses - 1; gas_id++) {
+        ym[gas_id] = CS_FI_(ycoel, gas_id)->val[c_id];
+        ym[n_gasses - 1] -= ym[gas_id];
+      }
+
+      h[c_id] = _cs_elec_convert_t_to_h(ym, t[c_id]);
+
+    }
+
+    BFT_FREE(ym);
+
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Convert temperature to enthalpy at selected boundary faces.
+ *
+ * This handles both user and model temperature conversions, so can be used
+ * safely whenever conversion is needed.
+ *
+ * \param[in]   n_faces   number of selected faces
+ * \param[in]   face_ids  ids of selected faces
+ * \param[in]   t         temperature values (defined on all boundary faces)
+ * \param[out]  h         enthalpy values (defined on all boundary faces)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_elec_convert_t_to_h_faces(const cs_lnum_t  n_faces,
+                             const cs_lnum_t  face_ids[],
+                             const cs_real_t  t[],
+                             cs_real_t        h[])
+{
+  const cs_mesh_t *m = cs_glob_mesh;
+
+  const cs_data_elec_t  *el_p = cs_glob_elec_properties;
+  const int n_gasses = el_p->ngaz;
+
+  if (n_gasses == 1) {
+
+    cs_real_t ym[1] = {1.};
+
+    for (cs_lnum_t i = 0; i < n_faces; i++) {
+      cs_lnum_t f_id = face_ids[i];
+      h[f_id] = _cs_elec_convert_t_to_h(ym, t[f_id]);
+    }
+
+  }
+  else {
+
+    const cs_lnum_t *b_face_cells = m->b_face_cells;
+
+    cs_real_t *ym;
+    BFT_MALLOC(ym, n_gasses, cs_real_t);
+
+    for (cs_lnum_t i = 0; i < n_faces; i++) {
+
+      cs_lnum_t f_id = face_ids[i];
+      cs_lnum_t c_id = b_face_cells[c_id];
+      for (int gas_id = 0; gas_id < n_gasses - 1; gas_id++) {
+        ym[gas_id] = CS_FI_(ycoel, gas_id)->val[c_id];
+        ym[n_gasses - 1] -= ym[gas_id];
+      }
+
+      h[f_id] = _cs_elec_convert_t_to_h(ym, t[f_id]);
+
+    }
+
+    BFT_FREE(ym);
+
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
