@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -75,7 +75,7 @@ BEGIN_C_DECLS
 
 typedef struct {
 
-  _Bool              is_leaf;      /* True for leaf nodes */
+  bool               is_leaf;      /* True for leaf nodes */
 
   fvm_morton_code_t  morton_code;  /* Level and coordinates in the grid
                                       according to Morton encoding */
@@ -188,7 +188,7 @@ _box_max(const fvm_box_set_t   *boxes,
  *   true or false
  *---------------------------------------------------------------------------*/
 
-inline static _Bool
+inline static bool
 _boxes_intersect_3d(const cs_coord_t  *extents,
                     cs_lnum_t          id_0,
                     cs_lnum_t          id_1)
@@ -204,7 +204,7 @@ _boxes_intersect_3d(const cs_coord_t  *extents,
     return true;
 }
 
-inline static _Bool
+inline static bool
 _boxes_intersect_2d(const cs_coord_t  *extents,
                     cs_lnum_t          id_0,
                     cs_lnum_t          id_1)
@@ -219,7 +219,7 @@ _boxes_intersect_2d(const cs_coord_t  *extents,
     return true;
 }
 
-inline static _Bool
+inline static bool
 _boxes_intersect_1d(const cs_coord_t  *extents,
                     cs_lnum_t          id_0,
                     cs_lnum_t          id_1)
@@ -358,7 +358,7 @@ _get_grid_coords_1d(fvm_morton_int_t  level,
  *   true or false
  *----------------------------------------------------------------------------*/
 
-inline static _Bool
+inline static bool
 _node_intersect_box_3d(fvm_morton_code_t  morton_code,
                        const cs_coord_t   min_box[3],
                        const cs_coord_t   max_box[3])
@@ -379,7 +379,7 @@ _node_intersect_box_3d(fvm_morton_code_t  morton_code,
     return true;
 }
 
-inline static _Bool
+inline static bool
 _node_intersect_box_2d(fvm_morton_code_t  morton_code,
                        const cs_coord_t   min_box[2],
                        const cs_coord_t   max_box[2])
@@ -399,7 +399,7 @@ _node_intersect_box_2d(fvm_morton_code_t  morton_code,
     return true;
 }
 
-inline static _Bool
+inline static bool
 _node_intersect_box_1d(fvm_morton_code_t  morton_code,
                        const cs_coord_t   min_box[1],
                        const cs_coord_t   max_box[1])
@@ -687,7 +687,7 @@ _count_next_level(fvm_box_tree_t           *bt,
  *   true if we should continue, false otherwise.
  *----------------------------------------------------------------------------*/
 
-static _Bool
+static bool
 _recurse_tree_build(fvm_box_tree_t       *bt,
                     const fvm_box_set_t  *boxes,
                     fvm_box_tree_sync_t   build_type,
@@ -696,7 +696,7 @@ _recurse_tree_build(fvm_box_tree_t       *bt,
   int  state = 0;
   cs_lnum_t   _next_size = 0;
 
-  _Bool retval = false;
+  bool retval = false;
 
 #if defined(HAVE_MPI)
 
@@ -1907,13 +1907,13 @@ _dump_node(const fvm_box_tree_t  *bt,
   const fvm_morton_code_t  m_code = node->morton_code;
 
   bft_printf("\n"
-             "  node %10d (%s)\n"
+             "  node %10ld (%s)\n"
              "    level:   %3u - anchor: [ %10u %10u %10u ]\n"
-             "    n_boxes: %3d - start_id: %u\n"
+             "    n_boxes: %3ld - start_id: %ld\n"
              "    boxes:\n",
-             node_id, node_type[(int)(node->is_leaf)],
+             (long)node_id, node_type[(int)(node->is_leaf)],
              m_code.L, m_code.X[0], m_code.X[1], m_code.X[2],
-             node->n_boxes, node->start_id);
+             (long)node->n_boxes, (long)node->start_id);
 
   for (i = 0; i < node->n_boxes; i++)
     bft_printf("        %d\n", (int)(bt->box_ids[node->start_id + i]));
@@ -2293,7 +2293,7 @@ fvm_box_tree_get_distrib(fvm_box_tree_t        *bt,
     distrib->index[i+1] += distrib->index[i];
 
   BFT_MALLOC(distrib->list,
-             distrib->index[distrib->n_ranks], int);
+             distrib->index[distrib->n_ranks], cs_lnum_t);
 
   BFT_MALLOC(counter, distrib->n_ranks, cs_lnum_t);
 
