@@ -18,7 +18,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -223,7 +223,7 @@ _reorder_coords_lexicographic(int               dim,
                               cs_lnum_t         order[])
 {
   size_t  i;
-  _Bool g_swap;
+  bool g_swap;
 
   do {
 
@@ -232,7 +232,7 @@ _reorder_coords_lexicographic(int               dim,
     for (i = start_id + 1; i < end_id; i++) {
 
       size_t j_prev = order[i-1], j = order[i];
-      _Bool l_swap = false;
+      bool l_swap = false;
 
       if (dim == 3) {
         if (coords[j_prev*3] < coords[j*3])
@@ -371,7 +371,7 @@ _fvm_io_num_local_max(const fvm_io_num_t  *this_io_num)
 static void
 _fvm_io_num_order_finalize(fvm_io_num_t     *this_io_num,
                            const cs_lnum_t   n_sub_entities[],
-                           _Bool             may_be_shared)
+                           bool              may_be_shared)
 {
   if (n_sub_entities != NULL) {
 
@@ -438,7 +438,7 @@ _fvm_io_num_local_order(fvm_io_num_t     *this_io_num,
 {
   cs_gnum_t   num_prev, num_cur;
 
-  _Bool       may_be_shared = false;
+  bool        may_be_shared = false;
 
   cs_gnum_t   current_gnum = 0;
 
@@ -657,7 +657,7 @@ _fvm_io_num_global_order(fvm_io_num_t       *this_io_num,
 {
   cs_gnum_t   num_prev, num_cur;
 
-  _Bool       may_be_shared = false;
+  bool        may_be_shared = false;
 
   cs_lnum_t  *b_nsub = NULL;
   int         have_sub_loc = 0, have_sub_glob = 0;
@@ -993,7 +993,7 @@ _fvm_io_num_global_order_s(fvm_io_num_t       *this_io_num,
  *  true or false
  *----------------------------------------------------------------------------*/
 
-inline static _Bool
+inline static bool
 _indexed_is_greater(size_t            i1,
                     size_t            i2,
                     const cs_lnum_t   index[],
@@ -2467,16 +2467,24 @@ fvm_io_num_create_from_real(const cs_real_t  val[],
   cs_parall_min(1, CS_DOUBLE, &v_min);
   cs_parall_max(1, CS_DOUBLE, &v_max);
 
-  if (v_max <= v_min)
-    bft_error(__FILE__, __LINE__, 0,
-              _("%s: point set is empty or contains identical values."),
-              __func__);
+  if (v_max <= v_min) {
+    cs_gnum_t n_g_entities = n_entities;
+    cs_parall_counter(&n_g_entities, 1);
+    if (n_g_entities > 1)
+      bft_error(__FILE__, __LINE__, 0,
+                _("%s: point set contains identical values."),
+                __func__);
+  }
 
 #if defined(HAVE_MPI)
 
   if (n_ranks > 1) {
 
-    double scale = (1.0 - 1.e12) / (v_max - v_min);
+    double scale;
+    if (v_max > v_min)
+      scale = (1.0 - 1.e-12) / (v_max - v_min);
+    else
+      scale = 0;
 
     cs_real_t *s_val;
     BFT_MALLOC(s_val, n_entities, cs_real_t);
@@ -2624,34 +2632,28 @@ fvm_io_num_create_from_scan(size_t  n_entities)
 {
   fvm_io_num_t  *this_io_num = NULL;
 
-  /* Initial checks */
+  /* Create structure */
 
-  if (cs_glob_n_ranks < 2)
-    return NULL;
+  BFT_MALLOC(this_io_num, 1, fvm_io_num_t);
+
+  BFT_MALLOC(this_io_num->_global_num, n_entities, cs_gnum_t);
+  this_io_num->global_num = this_io_num->_global_num;
+
+  this_io_num->global_num_size = n_entities;
 
 #if defined(HAVE_MPI)
-  {
-    size_t  i;
+  if (cs_glob_n_ranks > 1) {
     cs_gnum_t gnum_base = n_entities;
     cs_gnum_t gnum_sum = n_entities;
     cs_gnum_t gnum_shift = 0;
 
     MPI_Comm comm = cs_glob_mpi_comm;
 
-    /* Create structure */
-
-    BFT_MALLOC(this_io_num, 1, fvm_io_num_t);
-
-    BFT_MALLOC(this_io_num->_global_num, n_entities, cs_gnum_t);
-    this_io_num->global_num = this_io_num->_global_num;
-
-    this_io_num->global_num_size = n_entities;
-
     MPI_Scan(&gnum_base, &gnum_shift, 1, CS_MPI_GNUM, MPI_SUM, comm);
 
     gnum_base = gnum_shift - gnum_base + 1;
 
-    for (i = 0; i < n_entities; i++)
+    for (size_t i = 0; i < n_entities; i++)
       this_io_num->_global_num[i] = gnum_base + i;
 
     gnum_base = n_entities;
@@ -2662,6 +2664,13 @@ fvm_io_num_create_from_scan(size_t  n_entities)
   }
 #endif
 
+  if (cs_glob_n_ranks < 2) {
+    for (size_t i = 0; i < n_entities; i++)
+      this_io_num->_global_num[i] = i+1;
+
+    this_io_num->global_count = n_entities;
+  }
+
   return this_io_num;
 }
 
