@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -136,7 +136,7 @@ _local_to_global_extents(int         dim,
  *  true or false
  *----------------------------------------------------------------------------*/
 
-inline static _Bool
+inline static bool
 _a_ge_b(fvm_morton_code_t  code_a,
         fvm_morton_code_t  code_b)
 {
@@ -149,15 +149,15 @@ _a_ge_b(fvm_morton_code_t  code_a,
   if (a_diff > 0) {
     code_a.L = l;
     code_a.X[0] = code_a.X[0] << a_diff;
-    code_a.X[1] = code_a.X[2] << a_diff;
-    code_a.X[1] = code_a.X[2] << a_diff;
+    code_a.X[1] = code_a.X[1] << a_diff;
+    code_a.X[2] = code_a.X[2] << a_diff;
   }
 
   if (b_diff > 0) {
     code_b.L = l;
     code_b.X[0] = code_b.X[0] << b_diff;
-    code_b.X[1] = code_b.X[2] << b_diff;
-    code_b.X[1] = code_b.X[2] << b_diff;
+    code_b.X[1] = code_b.X[1] << b_diff;
+    code_b.X[2] = code_b.X[2] << b_diff;
   }
 
   i = l - 1;
@@ -190,7 +190,7 @@ _a_ge_b(fvm_morton_code_t  code_a,
  *  true or false
  *----------------------------------------------------------------------------*/
 
-inline static _Bool
+inline static bool
 _a_gt_b(fvm_morton_code_t  code_a,
         fvm_morton_code_t  code_b)
 {
@@ -203,15 +203,15 @@ _a_gt_b(fvm_morton_code_t  code_a,
   if (a_diff > 0) {
     code_a.L = l;
     code_a.X[0] = code_a.X[0] << a_diff;
-    code_a.X[1] = code_a.X[2] << a_diff;
-    code_a.X[1] = code_a.X[2] << a_diff;
+    code_a.X[1] = code_a.X[1] << a_diff;
+    code_a.X[2] = code_a.X[2] << a_diff;
   }
 
   if (b_diff > 0) {
     code_b.L = l;
     code_b.X[0] = code_b.X[0] << b_diff;
-    code_b.X[1] = code_b.X[2] << b_diff;
-    code_b.X[1] = code_b.X[2] << b_diff;
+    code_b.X[1] = code_b.X[1] << b_diff;
+    code_b.X[2] = code_b.X[2] << b_diff;
   }
 
   i = l - 1;
@@ -1268,7 +1268,7 @@ fvm_morton_compare(int                dim,
  *  true or false
  *----------------------------------------------------------------------------*/
 
-_Bool
+bool
 fvm_morton_a_gt_b(fvm_morton_code_t  a,
                   fvm_morton_code_t  b)
 {
@@ -1286,7 +1286,7 @@ fvm_morton_a_gt_b(fvm_morton_code_t  a,
  *  true or false
  *----------------------------------------------------------------------------*/
 
-_Bool
+bool
 fvm_morton_a_ge_b(fvm_morton_code_t  a,
                   fvm_morton_code_t  b)
 {
