@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -465,6 +465,9 @@ fvm_nodal_get_n_elements(const fvm_nodal_t  *this_nodal,
  * with the parent entity numbers of those entities, in order (i.e. in
  * local section order, section by section).
  *
+ * This function is similar to fvm_nodal_get_parent_num(), but returns
+ * numbers (1 to n) instead of ids (0 to n-1).
+ *
  * parameters:
  *   this_nodal <-- pointer to nodal mesh structure
  *   entity_dim <-- dimension of entities we are interested in (0 to 3)
@@ -476,6 +479,26 @@ fvm_nodal_get_parent_num(const fvm_nodal_t  *this_nodal,
                          int                 entity_dim,
                          cs_lnum_t           parent_num[]);
 
+/*----------------------------------------------------------------------------
+ * Return local parent id array for all entities of a given
+ * dimension in a nodal mesh.
+ *
+ * The number of entities of the given dimension may be obtained
+ * through fvm_nodal_get_n_entities(), the parent_num[] array is populated
+ * with the parent entity numbers of those entities, in order (i.e. in
+ * local section order, section by section).
+ *
+ * parameters:
+ *   this_nodal <-- pointer to nodal mesh structure
+ *   entity_dim <-- dimension of entities we are interested in (0 to 3)
+ *   parent_id --> entity parent id (array must be pre-allocated)
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_get_parent_id(const fvm_nodal_t  *this_nodal,
+                        int                 entity_dim,
+                        cs_lnum_t           parent_id[]);
+
 /*----------------------------------------------------------------------------
  * Return pointer to global vertex labels of a nodal mesh.
  *
