@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -66,6 +66,11 @@ typedef struct _fvm_nodal_section_t {
 
   fvm_element_t  type;             /* Element types */
 
+  int         boundary_flag;       /* Is section on boundary ?
+                                      -1: does not apply or unspecified;
+                                      0: no
+                                      1: yes */
+
   /* Connectivity */
   /*--------------*/
 
