@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -88,7 +88,7 @@ fvm_point_location_nodal(const fvm_nodal_t  *this_nodal,
                          float               tolerance_fraction,
                          int                 locate_on_parents,
                          cs_lnum_t           n_points,
-                         const cs_lnum_t    *point_tag,
+                         const int          *point_tag,
                          const cs_coord_t    point_coords[],
                          cs_lnum_t           location[],
                          float               distance[]);
