@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -108,7 +108,7 @@ fvm_selector_postfix_get_infix(const fvm_selector_postfix_t  *pf);
  *   true if expression depends on coordinates, false otherwise
  *----------------------------------------------------------------------------*/
 
-_Bool
+bool
 fvm_selector_postfix_coords_dep(const fvm_selector_postfix_t  *pf);
 
 /*----------------------------------------------------------------------------
@@ -121,7 +121,7 @@ fvm_selector_postfix_coords_dep(const fvm_selector_postfix_t  *pf);
  *   true if expression depends on normals, false otherwise
  *----------------------------------------------------------------------------*/
 
-_Bool
+bool
 fvm_selector_postfix_normals_dep(const fvm_selector_postfix_t  *pf);
 
 /*----------------------------------------------------------------------------
@@ -170,7 +170,7 @@ fvm_selector_postfix_get_missing(const fvm_selector_postfix_t  *pf,
  *   true or false base on expression evaluation
  *----------------------------------------------------------------------------*/
 
-_Bool
+bool
 fvm_selector_postfix_eval(const fvm_selector_postfix_t  *pf,
                           int                            n_groups,
                           int                            n_attributes,
