@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -75,6 +75,9 @@
 #include <vtkCPInputDataDescription.h>
 #include <vtkCPProcessor.h>
 #include <vtkCPPythonScriptPipeline.h>
+#if defined(HAVE_VTKCPPYTHONSCRIPTV2PIPELINE_H)
+#include <vtkCPPythonScriptV2Pipeline.h>
+#endif
 #include <vtkSmartPointer.h>
 
 #include <vtkXMLUnstructuredGridWriter.h>
@@ -82,6 +85,8 @@
 #include <vtkGenericDataObjectWriter.h>
 #include <vtkDoubleArray.h>
 
+#include <vtkFileOutputWindow.h>
+
 /*----------------------------------------------------------------------------
  *  Local headers
  *----------------------------------------------------------------------------*/
@@ -154,13 +159,8 @@ typedef struct {
   int                         time_step;       /* Latest time step */
   double                      time_value;      /* Latest time value */
 
-#if defined(HAVE_MPI)
-  MPI_Comm                    comm;            /* Associated communicator */
-#endif
-
-  vtkCPProcessor             *processor;       /* Co processor */
-  vtkCPDataDescription       *datadesc;        /* Data description */
-
+  char                       *input_name;      /* input name, or NULL for
+                                                  default */
   bool                        private_comm;    /* Use private communicator */
   bool                        ensight_names;   /* Use EnSight rules for
                                                   field names */
@@ -174,10 +174,459 @@ typedef struct {
  * Static global variables
  *============================================================================*/
 
+int _n_writers = 0;
+
+int _n_scripts = 0;                         /* Number of scripts */
+char **_scripts = NULL;                     /* List of scripts */
+
+vtkCPProcessor  *_processor = NULL;         /* Co processor */
+
+#if defined(HAVE_MPI)
+MPI_Comm  _comm = MPI_COMM_NULL;            /* Associated communicator */
+MPI_Comm  _reference_comm = MPI_COMM_NULL;  /* Reference communicator */
+#endif
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Check if a script is a Catalyst script
+ *
+ * The script only does cursory checks, so may return false positives
+ * in some cases, but using "incorrect" scripts for Catalyst only
+ * leads to extra warnings, which should be ok pending a more general
+ * file checker.
+ *
+ * parameters:
+ *   path <-- scripts path
+ *
+ * returns:
+ *   1 if script is a Catalyst V1, 2 if it seems to be a V2 script,
+ *   script, 0 otherwise
+ *----------------------------------------------------------------------------*/
+
+static int
+_check_script_is_catalyst(const char  *path)
+{
+  assert(path != NULL);
+  int retval = 0;
+
+  FILE *fp = fopen(path, "r");
+
+  if (fp == NULL)
+    return retval;
+
+  int checks[] = {false, false, false};
+  int n_checks = 0;
+  bool import_catalyst = false;
+
+  const char *check_strings[]
+    = {"CreateCoProcessor(",
+       "RequestDataDescription(",
+       "DoCoProcessing("};
+
+  /* Note: we could simply check for "from paraview import coprocessing"
+     in most cases but in cases of non-autogenerated scripts, this might
+     appear in a different form, so we limit ourselves to the main
+     required methods */
+
+  while (true) {
+    char buffer[1024];
+    char *e = buffer+1024;
+
+    char *s = fgets(buffer, 1024, fp);
+    if (s == NULL) break;
+
+    while (s < e && *s == ' ' && *s == '\t')  /* skip initial whitespace */
+      s++;
+
+    if (strncmp(s, "def", 3) == 0) {
+      s += 3;
+      if (*s == ' ' || *s == '\t') {
+        while (*s == ' ' && *s == '\t' && *s != '\0')
+          s++;
+        /* Remove whitespace */
+        size_t l = strlen(s);
+        size_t i, j;
+        for (i = 0, j = 0 ; i < l ; i++) {
+          if (s[i] != ' ' && s[i] != '\t')
+            s[j++] = s[i];
+        }
+
+        for (i = 0; i < 3; i++) {
+          if (strncmp(s, check_strings[i], strlen(check_strings[i])) == 0) {
+            if (checks[i] == false) {
+              checks[i] = true;
+              n_checks += 1;
+            }
+          }
+        }
+
+      }
+    }
+    else if (strncmp(s, "from", 4) == 0) {
+
+      /* cleanup whitespace */
+      int n_space = 0;
+      int i = 0;
+      for (int j = 0; j < 1024 && s+j < e && s[j] != '\0'; j++) {
+        if (s[j] == ' ' || s[j] == '\t') {
+          if (n_space < 1)
+            s[i++] = ' ';
+          n_space += 1;
+        }
+        else {
+          s[i++] = s[j];
+          n_space = 0;
+        }
+      }
+
+      if (strncmp(s, "from paraview import catalyst", 29) == 0) {
+        import_catalyst = true;
+      }
+
+    }
+
+    if (n_checks == 3) {
+      retval = 1;
+      break;
+    }
+    else if (import_catalyst) {
+      retval = 2;
+      break;
+    }
+
+    /* Check for end of line; if not present, continue reading from buffer */
+
+    while (s < e && *s != '\0' && *s != '\n')
+      s++;
+    while (s >= e) {
+      s = fgets(buffer, 1024, fp);
+      if (s == NULL) break;
+      while (s < e && *s != '\0' && *s != '\n')
+        s++;
+    }
+  }
+
+  fclose(fp);
+
+  return retval;
+}
+
+/*----------------------------------------------------------------------------
+ * Add a Catalyst V2 pipeline if not already present.
+ *
+ * parameters:
+ *   path <-- V2 pipeline file path
+ *
+ * returns:
+ *   id of pipeline script in list, or -1 if not valid
+ *----------------------------------------------------------------------------*/
+
+static int
+_add_v2_pipeline(const char  *path)
+{
+  assert(path != NULL);
+
+  /* Check that we did not already add this file */
+
+  for (int i = 0; i < _n_scripts; i++) {
+    if (strcmp(_scripts[i], path) == 0)
+      return i;
+  }
+
+#if defined(HAVE_VTKCPPYTHONSCRIPTV2PIPELINE_H)
+
+  /* Create Catalyst pipeline and check the file is valid */
+
+  vtkNew<vtkCPPythonScriptV2Pipeline> pipeline;
+  if (pipeline->Initialize(path) == false) {
+    bft_printf(_("\nFile \"%s\"\n"
+                 "does not seem to contain a Catalyst Python pipeline.\n"),
+               path);
+    return -1;
+  }
+
+  int id = _n_scripts;
+  _n_scripts += 1;
+
+  BFT_REALLOC(_scripts, _n_scripts, char *);
+
+  size_t l = strlen(path);
+  BFT_MALLOC(_scripts[id], l+1, char);
+  strncpy(_scripts[id], path, l);
+  _scripts[id][l] = '\0';
+
+  /* pipeline->SetGhostLevel(1); */
+  _processor->AddPipeline(pipeline);
+
+  return id;
+
+#else
+
+  bft_printf
+    (_("\nFile \"%s\"\n"
+       "seems to contain a Catalyst V2 Python pipeline but the version.\n"
+       "of Catalyst linked with only supports version 1 pipelines.\n\n"
+       "You may need to use a more recent Catalyst version or downgrade\n"
+       "to earlier V1 pipelines.\n"),
+     path);
+
+  return -1;
+
+#endif /* defined(HAVE_VTKCPPYTHONSCRIPTV2PIPELINE_H) */
+
+}
+
+/*----------------------------------------------------------------------------
+ * Add a Catalyst script if not already present.
+ *
+ * parameters:
+ *   path <-- scripts path
+ *
+ * returns:
+ *   id of script in list, or -1 if not valid
+ *----------------------------------------------------------------------------*/
+
+static int
+_add_script(const char         *path)
+{
+  assert(path != NULL);
+
+  int is_catalyst = 0;
+  int rank = 0, n_ranks = 1;
+
+#if defined(HAVE_MPI)
+  if (_comm != MPI_COMM_NULL) {
+    MPI_Comm_rank(_comm, &rank);
+    MPI_Comm_size(_comm, &n_ranks);
+  }
+#endif
+
+  if (rank < 1)
+    is_catalyst = _check_script_is_catalyst(path);
+
+#if defined(HAVE_MPI)
+  if (n_ranks > 1)
+    MPI_Bcast(&is_catalyst, 1, MPI_INT, 0, _comm);
+#endif
+
+  if (is_catalyst < 1)
+    return -1;
+
+  if (is_catalyst == 2) {
+    _add_v2_pipeline(path);
+    return 2;
+  }
+
+  for (int i = 0; i < _n_scripts; i++) {
+    if (strcmp(_scripts[i], path) == 0)
+      return i;
+  }
+
+  int id = _n_scripts;
+  _n_scripts += 1;
+
+  BFT_REALLOC(_scripts, _n_scripts, char *);
+
+  size_t l = strlen(path);
+  BFT_MALLOC(_scripts[id], l+1, char);
+  strncpy(_scripts[id], path, l);
+  _scripts[id][l] = '\0';
+
+  /* Add Catalyst pipeline */
+
+  vtkNew<vtkCPPythonScriptPipeline> pipeline;
+  if (pipeline->Initialize(path) == 0)
+    bft_error(__FILE__, __LINE__, 0,
+              _("Error initializing pipeline from \"%s\""), path);
+
+  /* pipeline->SetGhostLevel(1); */
+  _processor->AddPipeline(pipeline);
+
+  return id;
+}
+
+/*----------------------------------------------------------------------------
+ * Add Catalyst scripts from directoty if not already present.
+ *
+ * Currently assumes all Python files in the given directory
+ * are Catalyst scripts.
+ *
+ * parameters:
+ *   dir_path <-- directory path
+ *----------------------------------------------------------------------------*/
+
+static void
+_add_dir_scripts(const char  *dir_path)
+{
+  char **dir_files = cs_file_listdir(dir_path);
+
+  for (int i = 0; dir_files[i] != NULL; i++) {
+
+    const char *file_name = dir_files[i];
+    const char *ext = NULL;
+    int l_ext = 0;
+
+    /* Find extension */
+    for (int j = strlen(file_name) - 1; j > -1; j--) {
+      l_ext++;
+      if (file_name[j] == '.') {
+        ext = file_name + j;
+        break;
+      }
+    }
+    if (ext == NULL)
+      continue;
+
+    /* Filter: Python files only */
+    if (l_ext == 3 && strncmp(ext, ".py", 3) == 0) {
+      char *tmp_name = NULL;
+      BFT_MALLOC(tmp_name,
+                 strlen(dir_path) + 1 + strlen(file_name) + 1,
+                 char);
+      sprintf(tmp_name, "%s/%s", dir_path, file_name);
+      _add_script(tmp_name);
+      BFT_FREE(tmp_name);
+    }
+
+#if defined(HAVE_VTKCPPYTHONSCRIPTV2PIPELINE_H)
+
+    /* Filter: Catalyst V2 pipeline might be in ".zip" form */
+    else if (l_ext == 4 && strncmp(ext, ".zip", 4) == 0) {
+      char *tmp_name = NULL;
+      BFT_MALLOC(tmp_name,
+                 strlen(dir_path) + 1 + strlen(file_name) + 1,
+                 char);
+      sprintf(tmp_name, "%s/%s", dir_path, file_name);
+      _add_v2_pipeline(tmp_name);
+      BFT_FREE(tmp_name);
+    }
+#endif /* defined(HAVE_VTKCPPYTHONSCRIPTV2PIPELINE_H) */
+
+    BFT_FREE(dir_files[i]);
+  }
+
+  BFT_FREE(dir_files);
+}
+
+/*----------------------------------------------------------------------------
+ * Initialize coprocessor.
+ *
+ * parameters:
+ *   private_comm <-- if true, use dedicated communicator
+ *   comm         <-- associated MPI communicator.
+ *----------------------------------------------------------------------------*/
+
+#if defined(HAVE_MPI)
+static void
+_init_coprocessor(bool      private_comm,
+                  MPI_Comm  comm)
+#else
+_init_coprocessor(void)
+#endif
+{
+  int mpi_flag = 0;
+  int mpi_rank = -1;
+
+#if defined(HAVE_MPI)
+
+  MPI_Initialized(&mpi_flag);
+
+  if (mpi_flag) {
+    if (comm != _reference_comm) {
+      if (comm != MPI_COMM_NULL && _reference_comm != MPI_COMM_NULL)
+        bft_error(__FILE__, __LINE__, 0,
+                  _("All Catalyst writers must use the same MPI communicator"));
+    }
+  }
+
+#endif
+
+  if (_processor == NULL) {
+
+    _processor = vtkCPProcessor::New();
+
+#if defined(HAVE_MPI)
+
+    if (mpi_flag) {
+
+      _reference_comm = comm;
+      if (private_comm && comm != MPI_COMM_NULL)
+        MPI_Comm_dup(comm, &(_comm));
+      else
+        _comm = comm;
+
+      if (comm != MPI_COMM_NULL)
+        MPI_Comm_rank(_comm, &mpi_rank);
+
+      vtkMPICommunicatorOpaqueComm vtk_comm
+        = vtkMPICommunicatorOpaqueComm(&_comm);
+      _processor->Initialize(vtk_comm);
+
+    }
+
+#endif
+
+    if (!mpi_flag)
+      _processor->Initialize();
+
+    vtkFileOutputWindow *log_output = vtkFileOutputWindow::New();
+    if (mpi_rank < 1)
+      log_output->SetFileName("./catalyst.log");
+    else
+      log_output->SetFileName("/dev/null");
+    vtkFileOutputWindow::SetInstance(log_output);
+
+  }
+}
+
+/*----------------------------------------------------------------------------
+ * Finalize coprocessor.
+ *
+ * parameters:
+ *   private_comm <-- if true, use dedicated communicator
+ *   comm         <-- associated MPI communicator.
+ *----------------------------------------------------------------------------*/
+
+static void
+_free_coprocessor(void)
+{
+  if (_processor != NULL && _n_writers < 2) {
+
+    _processor->Finalize();
+
+    /* Workaround for segmentation fault observed on a Red Hat 8.3 system
+       with gcc 8.3.1, at coprocessor destruction, not observed on other
+       machines. */
+
+    bool cp_delete = true;
+
+    const char *s = getenv("CS_PV_CP_DELETE_CRASH_WORKAROUND");
+    if (s != NULL) {
+      if (atoi(s) > 0)
+        cp_delete = false;
+    }
+    if (cp_delete)
+      _processor->Delete();
+
+    _processor = NULL;
+
+    for (int i = 0; i < _n_scripts; i++)
+      BFT_FREE(_scripts[i]);
+    BFT_FREE(_scripts);
+
+#if defined(HAVE_MPI)
+    {
+      if (_comm != _reference_comm && _comm != MPI_COMM_NULL)
+        MPI_Comm_free(&_comm);
+    }
+#endif
+
+  }
+}
+
 /*----------------------------------------------------------------------------
  * Return the Catalyst mesh id associated with a given mesh name,
  * or -1 if no association found.
@@ -315,7 +764,6 @@ _get_norm_elt_type(const fvm_element_t fvm_elt_type)
  * parameters:
  *   norm_elt_type  <-- Catalyst element type.
  *   vertex_order  --> Pointer to vertex order array (0 to n-1).
- *
  *----------------------------------------------------------------------------*/
 
 static void
@@ -479,7 +927,6 @@ _add_catalyst_field(fvm_to_catalyst_t         *writer,
               fvm_catalyst_field_t *);
 
   vtkUnstructuredGrid *f = NULL;
-  vtkDoubleArray *tmp = NULL;
 
   const int dest_dim = (dim == 6) ? 9 : dim;
 
@@ -488,7 +935,7 @@ _add_catalyst_field(fvm_to_catalyst_t         *writer,
     f = vtkUnstructuredGrid::SafeDownCast(vtkDataSet::SafeDownCast
           (writer->mb->GetBlock(mesh_id)));
 
-    tmp = vtkDoubleArray::New();
+    vtkNew<vtkDoubleArray> tmp;
     tmp->SetName(fieldname);
 
     tmp->SetNumberOfComponents(dest_dim);
@@ -506,7 +953,6 @@ _add_catalyst_field(fvm_to_catalyst_t         *writer,
       vtkDataSetAttributes::SafeDownCast
         (f->GetCellData())->AddArray(vtkAbstractArray::SafeDownCast(tmp));
     }
-    tmp->Delete();
 
   }
 
@@ -551,7 +997,7 @@ _export_vertex_coords(const fvm_nodal_t        *mesh,
 
   stride = (size_t)(mesh->dim);
 
-  vtkPoints *points = vtkPoints::New();
+  vtkNew<vtkPoints> points;
 
   points->Allocate(mesh->n_vertices);
 
@@ -580,7 +1026,7 @@ _export_vertex_coords(const fvm_nodal_t        *mesh,
     const cs_gnum_t *g_vtx_num
       = fvm_io_num_get_global_num(mesh->global_vertex_num);
 
-    vtkIdTypeArray *g_vtx_id = vtkIdTypeArray::New();
+    vtkNew<vtkIdTypeArray> g_vtx_id;
 
     g_vtx_id->SetNumberOfComponents(1);
     g_vtx_id->SetName("GlobalNodeIds");
@@ -597,12 +1043,9 @@ _export_vertex_coords(const fvm_nodal_t        *mesh,
 
     vtk_mesh->GetPointData()->SetGlobalIds(g_vtx_id);
 
-    g_vtx_id->Delete();
-
   }
 
   vtk_mesh->SetPoints(points);
-  points->Delete();
 }
 
 /*----------------------------------------------------------------------------
@@ -892,8 +1335,6 @@ _export_field_values_e(const fvm_nodal_t         *mesh,
     (f->GetCellData()->GetArray(fieldname))
     ->WritePointer(0, dest_dim*f->GetNumberOfCells());
 
-  assert(values != NULL);
-
   /* Distribute partition to block values */
 
   cs_lnum_t start_id = 0;
@@ -910,6 +1351,8 @@ _export_field_values_e(const fvm_nodal_t         *mesh,
     if (section->entity_dim < elt_dim)
       continue;
 
+    assert(values != NULL || section->n_elements == 0);
+
     fvm_convert_array(dim,
                       0,
                       dest_dim,
@@ -935,6 +1378,9 @@ _export_field_values_e(const fvm_nodal_t         *mesh,
   if (dim == 6) {
 
     cs_lnum_t n_elts = f->GetNumberOfCells();
+
+    assert(values != NULL || n_elts == 0);
+
     for (cs_lnum_t i = 0; i < n_elts; i++) {
       values[9*i + 8] = values[9*i + 2];
       values[9*i + 7] = values[9*i + 4];
@@ -960,10 +1406,11 @@ BEGIN_C_DECLS
  *   private_comm        use private MPI communicator (default: false)
  *   names=<fmt>         use same naming rules as <fmt> format
  *                       (default: ensight)
+ *   input_name=<name>   define input name (default: writer name)
  *
  * parameters:
  *   name           <-- base output case name.
- *   options        <-- whitespace separated, lowercase options list
+ *   options        <-- whitespace separaed, lowercase options list
  *   time_dependecy <-- indicates if and how meshes will change with time
  *   comm           <-- associated MPI communicator.
  *
@@ -990,6 +1437,8 @@ fvm_to_catalyst_init_writer(const char             *name,
 
   fvm_to_catalyst_t  *w = NULL;
 
+  bool private_comm = false;
+
   /* Initialize writer */
 
   BFT_MALLOC(w, 1, fvm_to_catalyst_t);
@@ -1007,8 +1456,8 @@ fvm_to_catalyst_init_writer(const char             *name,
   w->time_step  = -1;
   w->time_value = 0.0;
 
-  w->private_comm = false;
   w->ensight_names = true;
+  w->input_name = NULL;
 
   /* Writer name */
 
@@ -1017,7 +1466,7 @@ fvm_to_catalyst_init_writer(const char             *name,
     strcpy(w->name, name);
   }
   else {
-    const char _name[] = "catalyst_writer";
+    const char _name[] = "catalyst";
     BFT_MALLOC(w->name, strlen(_name) + 1, char);
     strcpy(w->name, _name);
   }
@@ -1036,13 +1485,19 @@ fvm_to_catalyst_init_writer(const char             *name,
       l_opt = i2 - i1;
 
       if ((l_opt == 12) && (strncmp(options + i1, "private_comm", l_opt) == 0))
-        w->private_comm = true;
-      else if ((l_opt == 6) && (strncmp(options + i1, "names=", l_opt) == 0)) {
+        private_comm = true;
+      else if ((l_opt > 6) && (strncmp(options + i1, "names=", 6) == 0)) {
         if ((l_opt == 6+7) && (strncmp(options + i1 + 6, "ensight", 7) == 0))
           w->ensight_names = true;
         else
           w->ensight_names = false;
       }
+      else if ((l_opt > 11) && (strncmp(options + i1, "input_name=", 11) == 0)) {
+        int l = strlen(options + i1 + 11);
+        BFT_MALLOC(w->input_name, l+1, char);
+        strncpy(w->input_name, options + i1 + 11, l);
+        w->input_name[l] = '\0';
+      }
 
       for (i1 = i2 + 1; i1 < l_tot && options[i1] == ' '; i1++);
 
@@ -1050,81 +1505,35 @@ fvm_to_catalyst_init_writer(const char             *name,
 
   }
 
+#if CS_PV_VERSION < 55
+  if (w->input_name == NULL) {
+    char n[] = "input";
+    BFT_MALLOC(w->input_name, strlen(n)+1, char);
+    strcpy(w->input_name, n);
+  }
+#endif
+
+  /* Ensure coprocessor is built */
+
+  _n_writers += 1;
+
+#if defined(HAVE_MPI)
+  _init_coprocessor(private_comm, comm);
+#else
+  _init_coprocessor();
+#endif
+
   /* Parallel parameters */
 
 #if defined(HAVE_MPI)
-  if (w->private_comm) {
-    int mpi_flag;
-    MPI_Initialized(&mpi_flag);
-
-    if (mpi_flag && comm != MPI_COMM_NULL) {
-      MPI_Comm_dup(comm, &(w->comm));
-      MPI_Comm_rank(w->comm, &(w->rank));
-      MPI_Comm_size(w->comm, &(w->n_ranks));
-    }
-    else
-      w->comm = MPI_COMM_NULL;
+  if (_comm != MPI_COMM_NULL) {
+    MPI_Comm_rank(_comm, &(w->rank));
+    MPI_Comm_size(_comm, &(w->n_ranks));
   }
-  else
-    w->comm = comm;
-#endif /* defined(HAVE_MPI) */
-
-  /* Catalyst pipeline */
-
-  w->processor = vtkCPProcessor::New();
-
-#if defined(HAVE_MPI)
-  if (w->comm != MPI_COMM_NULL) {
-    vtkMPICommunicatorOpaqueComm vtk_comm
-      = vtkMPICommunicatorOpaqueComm(&(w->comm));
-    w->processor->Initialize(vtk_comm);
-  }
-  else
-    w->processor->Initialize();
-#else
-  w->processor->Initialize();
 #endif
 
-  vtkCPPythonScriptPipeline  *pipeline = vtkCPPythonScriptPipeline::New();
-
-  /* read the coprocessing Python file */
-
-  char *script_path = NULL;
-
-  int l = strlen(w->name);
-  BFT_MALLOC(script_path, strlen(w->name) + 4, char);
-  strcpy(script_path, w->name);
-  for (int i = 0; i < l; i++) {
-    if (script_path[i] == ' ')
-      script_path[i] = '_';
-  }
-  strcat(script_path, ".py");
-
-  /* check file's existence */
-
-  if (cs_file_isreg(script_path)) {
-
-    if (pipeline->Initialize(script_path) == 0)
-      bft_error(__FILE__, __LINE__, 0,
-                _("Error initializing pipeline from \"%s\":\n\n"), script_path);
-
-    /* pipeline->SetGhostLevel(1); */
-    w->processor->AddPipeline(pipeline);
-    pipeline->Delete();
-
-  }
-  else
-    bft_error(__FILE__, __LINE__, 0,
-              _("Missing Catalyst pipeline file \"%s\":\n\n"), script_path);
-
-  BFT_FREE(script_path);
-
-  w->datadesc = vtkCPDataDescription::New();
-#if CS_PV_VERSION < 55
-  w->datadesc->AddInput("input");
-#else
-  w->datadesc->AddInput(w->name);
-#endif
+  if (_n_scripts < 1)
+    _add_dir_scripts(".");
 
   w->modified = true;
 
@@ -1161,9 +1570,10 @@ fvm_to_catalyst_finalize_writer(void  *this_writer_p)
   /* Free vtkUnstructuredGrid and field structures
      (reference counters should go to 0) */
 
-  w->processor->Finalize();
-  w->processor->Delete();
-  w->datadesc->Delete();
+  _free_coprocessor();
+
+  _n_writers -= 1;
+
   w->mb->Delete();
 
   for (i = 0; i < w->n_fields; i++) {
@@ -1173,13 +1583,6 @@ fvm_to_catalyst_finalize_writer(void  *this_writer_p)
 
   BFT_FREE(w->fields);
 
-#if defined(HAVE_MPI)
-  {
-    if (w->private_comm && w->comm != MPI_COMM_NULL)
-      MPI_Comm_free(&(w->comm));
-  }
-#endif /* defined(HAVE_MPI) */
-
   /* Free fvm_to_catalyst_t structure */
 
   BFT_FREE(w);
@@ -1221,7 +1624,6 @@ fvm_to_catalyst_set_mesh_time(void    *this_writer_p,
     w->time_step = _time_step;
     assert(time_value >= w->time_value);
     w->time_value = _time_value;
-    w->datadesc->SetTimeData(w->time_value, w->time_step);
   }
 }
 
@@ -1392,6 +1794,14 @@ fvm_to_catalyst_export_field(void                  *this_writer_p,
     fvm_to_catalyst_export_nodal(w, mesh);
   }
 
+  int _time_step = (time_step > -1) ? time_step : 0;
+  double _time_value = (time_value > 0.0) ? time_value : 0.0;
+  if (_time_step > w->time_step) {
+    w->time_step = _time_step;
+    assert(time_value >= w->time_value);
+    w->time_value = _time_value;
+  }
+
   /* Get field id */
 
   field_id = _get_catalyst_field_id(w,
@@ -1462,14 +1872,25 @@ fvm_to_catalyst_flush(void  *this_writer_p)
 {
   fvm_to_catalyst_t *w = (fvm_to_catalyst_t *)this_writer_p;
 
-  if (w->processor->RequestDataDescription(w->datadesc) != 0 && w->modified) {
-#if CS_PV_VERSION < 55
-    w->datadesc->GetInputDescriptionByName("input")->SetGrid(w->mb);
-#else
-    w->datadesc->GetInputDescriptionByName(w->name)->SetGrid(w->mb);
-#endif
+  vtkNew<vtkCPDataDescription> dataDescription;
+  if (w->input_name != NULL)
+    dataDescription->AddInput(w->input_name);
+  else
+    dataDescription->AddInput(w->name);
+  dataDescription->SetTimeData(w->time_value, w->time_step);
 
-    w->processor->CoProcess(w->datadesc);
+  if (_processor->RequestDataDescription(dataDescription) != 0 && w->modified) {
+    int n = dataDescription->GetNumberOfInputDescriptions();
+    if (n == 1)
+      dataDescription->GetInputDescription(0)->SetGrid(w->mb);
+    else {
+      if (w->input_name != NULL)
+        dataDescription->GetInputDescriptionByName(w->input_name)->SetGrid(w->mb);
+      else
+        dataDescription->GetInputDescriptionByName(w->name)->SetGrid(w->mb);
+    }
+
+    _processor->CoProcess(dataDescription);
     w->modified = false;
   }
 }
