@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -480,7 +480,7 @@ _field_output(void           *context,
     strcpy(var_name, c->name);
 
   /* File name */
-  char t_stamp[32];
+  char t_stamp[141];
   sprintf(t_stamp, "_%s_%.4i", var_name, w->nt);
   size_t l =   strlen(w->path) + strlen(w->name)
              + strlen(t_stamp) + 4 + 1;
@@ -759,9 +759,13 @@ fvm_to_histogram_export_field(void                  *writer,
   if (datatype >= CS_INT32 && datatype <= CS_UINT64)
     dest_datatype = CS_INT64;
 
+  int export_dim = fvm_nodal_get_max_entity_dim(mesh);
+
   fvm_writer_section_t  *export_list
     = fvm_writer_export_list(mesh,
-                             fvm_nodal_get_max_entity_dim(mesh),
+                             export_dim,
+                             export_dim,
+                             -1,
                              true,
                              true,
                              false,
