@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -95,6 +95,8 @@ typedef struct {
 
   cs_plot_format_t  format;        /* Plot format */
 
+  bool              no_time_step;  /* Do not append time step to file name */
+
   int               nt;            /* Time step */
   double            t;             /* Time value */
 
@@ -158,6 +160,7 @@ _field_output(void           *context,
               void           *buffer)
 {
   CS_UNUSED(datatype);
+  CS_UNUSED(component_id);
 
   _plot_context_t *c = context;
 
@@ -184,10 +187,10 @@ _field_output(void           *context,
   if (w->f == NULL) {
 
     char t_stamp[32];
-    if (w->nt >= 0)
-      sprintf(t_stamp, "_%.4i", w->nt);
-    else
+    if (w->no_time_step || w->nt < 0)
       t_stamp[0] = '\0';
+    else
+      sprintf(t_stamp, "_%.4i", w->nt);
     size_t l =   strlen(w->path) + strlen(w->name)
                + strlen(t_stamp) + 4 + 1;
     BFT_REALLOC(w->file_name, l, char);
@@ -289,6 +292,7 @@ _field_output(void           *context,
  * Options are:
  *   csv                 output CSV (comma-separated-values) files
  *   dat                 output dat (space-separated) files
+ *   no_time_step        do not add time step id to file name
  *
  * parameters:
  *   name           <-- base output case name.
@@ -350,6 +354,7 @@ fvm_to_plot_init_writer(const char             *name,
   /* Defaults */
 
   w->format = CS_PLOT_CSV;
+  w->no_time_step = false;
 
   w->nt = -1;
   w->t = -1;
@@ -380,6 +385,8 @@ fvm_to_plot_init_writer(const char             *name,
         w->format = CS_PLOT_CSV;
       else if ((l_opt == 3) && (strncmp(options + i1, "dat", l_opt) == 0))
         w->format = CS_PLOT_DAT;
+      else if ((l_opt == 12) && (strncmp(options + i1, "no_time_step", l_opt) == 0))
+        w->no_time_step = true;
 
       for (i1 = i2 + 1 ; i1 < l_tot && options[i1] == ' ' ; i1++);
 
@@ -455,6 +462,9 @@ void
 fvm_to_plot_export_nodal(void               *writer,
                          const fvm_nodal_t  *mesh)
 {
+  CS_UNUSED(writer);
+  CS_UNUSED(mesh);
+
 #if 0
   if (fvm_nodal_get_max_entity_dim(mesh) == 0) {
 
