@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -174,7 +174,7 @@ static fvm_writer_format_t _fvm_writer_format_list[10] = {
     "3.1 +",
     (  FVM_WRITER_FORMAT_USE_EXTERNAL
      | FVM_WRITER_FORMAT_HAS_POLYGON),
-    FVM_WRITER_FIXED_MESH,
+    FVM_WRITER_TRANSIENT_COORDS,
     0,                                 /* dynamic library count */
     NULL,                              /* dynamic library */
     NULL,                              /* dynamic library name */
@@ -208,7 +208,8 @@ static fvm_writer_format_t _fvm_writer_format_list[10] = {
     "4.2 +",
     (  FVM_WRITER_FORMAT_USE_EXTERNAL
      | FVM_WRITER_FORMAT_HAS_POLYGON
-     | FVM_WRITER_FORMAT_HAS_POLYHEDRON),
+     | FVM_WRITER_FORMAT_HAS_POLYHEDRON
+     | FVM_WRITER_FORMAT_NO_SEPARATE_MESHES),
     FVM_WRITER_TRANSIENT_CONNECT,
 #if !defined(HAVE_CATALYST) || defined(HAVE_PLUGIN_CATALYST)
     0,                                 /* dynamic library count */
@@ -280,7 +281,7 @@ static fvm_writer_format_t _fvm_writer_format_list[10] = {
     (  FVM_WRITER_FORMAT_USE_EXTERNAL
      | FVM_WRITER_FORMAT_HAS_POLYGON
      | FVM_WRITER_FORMAT_HAS_POLYHEDRON),
-    FVM_WRITER_TRANSIENT_CONNECT,
+    FVM_WRITER_TRANSIENT_COORDS,
 #if !defined(HAVE_MELISSA) || defined(HAVE_PLUGIN_MELISSA)
     0,                                 /* dynamic library count */
     NULL,                              /* dynamic library */
@@ -661,7 +662,14 @@ _format_writer_init(fvm_writer_t  *this_writer,
       strcpy(tmp_path, this_writer->path);
       if (tmp_path[l - 1] == DIR_SEPARATOR)
         tmp_path[l - 1] = '\0';
-      if (cs_file_mkdir_default(this_writer->path) == 1)
+      int dir_err = 0;
+      if (cs_glob_rank_id < 1)
+        dir_err = cs_file_mkdir_default(this_writer->path);
+#if defined(HAVE_MPI)
+      if (cs_glob_n_ranks > 1)
+        MPI_Bcast(&dir_err, 1, MPI_INT, 0, cs_glob_mpi_comm);
+#endif
+      if (dir_err == 1)
         tmp_path[0] = '\0';
       else {
         l = strlen(tmp_path);
@@ -838,6 +846,8 @@ fvm_writer_get_format_id(const char  *format_name)
     strcpy(closest_name, "Catalyst");
   else if (strncmp(tmp_name, "ccm", 3) == 0)
     strcpy(closest_name, "CCM-IO");
+  else if (strncmp(tmp_name, "melissa", 7) == 0)
+    strcpy(closest_name, "Melissa");
   else
     strcpy(closest_name, tmp_name);
 
@@ -1190,9 +1200,12 @@ fvm_writer_init(const char             *name,
 
   if (this_writer->format->info_mask & FVM_WRITER_FORMAT_SEPARATE_MESHES)
     separate_meshes = true;
+  else if (  this_writer->format->info_mask
+           & FVM_WRITER_FORMAT_NO_SEPARATE_MESHES)
+    separate_meshes = false;
 
   if (separate_meshes)
-    this_writer->n_format_writers = 0; /* Delay contruction */
+    this_writer->n_format_writers = 0; /* Delay construction */
   else
     this_writer->n_format_writers = 1;
 
