@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -123,9 +123,12 @@ typedef void
  *
  * parameters:
  *   mesh                 <-- pointer to nodal mesh structure
+ *   min_export_dim       <-- minimum dimension of sections to export
+ *   max_export_dim       <-- maximum dimension of sections to export
+ *   boundary_flag        <-- -1 if ignored, otherwise only sections
+ *                            with -1 or matching boundary flag
  *   group_by_type        <-- if true, group sections of same type
  *   group_all            <-- if true, all sections continue previous ones
- *   min_export_dim       <-- minimum dimension of sections to export
  *   discard_polygons     <-- ignore polygonal sections
  *   discard_polyhedra    <-- ignore polyhedral sections
  *   divide_polygons      <-- tesselate polygonal sections
@@ -139,6 +142,8 @@ typedef void
 fvm_writer_section_t *
 fvm_writer_export_list(const fvm_nodal_t          *mesh,
                        int                         min_export_dim,
+                       int                         max_export_dim,
+                       int                         boundary_flag,
                        bool                        group_by_type,
                        bool                        group_all,
                        bool                        discard_polygons,
