@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -56,20 +56,20 @@ BEGIN_C_DECLS
  *
  *----------------------------------------------------------------------------*/
 
-
 void CS_PROCF (csther, CSTHER) (void);
 
 /*----------------------------------------------------------------------------
- * Turbulence model.
- *
- * Fortran Interface:
- *
- * SUBROUTINE CSTURB
- * *****************
- *
+ * Turbulence model
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (csturb, CSTURB) (void);
+void cs_gui_turb_model(void);
+
+/*----------------------------------------------------------------------------
+ * Define reference length and reference velocity for the initialization of
+ * the turbulence variables
+ *----------------------------------------------------------------------------*/
+
+void cs_gui_turb_ref_values(void);
 
 /*----------------------------------------------------------------------------
  * Specific heat variable or constant indicator.
@@ -96,17 +96,6 @@ void CS_PROCF (cscpva, CSCPVA) (void);
 
 void CS_PROCF (csvvva, CSVVVA) (int *iviscv);
 
-/*----------------------------------------------------------------------------
- * User thermal scalar.
- *
- * Fortran Interface:
- *
- * SUBROUTINE UITHSC
- * *****************
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uithsc, UITHSC) (void);
-
 /*----------------------------------------------------------------------------
  * Constant or variable indicator for the user scalar laminar viscosity.
  *
@@ -164,13 +153,11 @@ void CS_PROCF (cscfgp, CSCFGP) (int *icfgrp);
  * *****************
  *
  * INTEGER          NTSUIT  -->   checkpoint frequency
- * INTEGER          ILEAUX  -->   restart with auxiliary
  * INTEGER          ICCFVG  -->   restart with frozen field
  *----------------------------------------------------------------------------*/
 
 
 void CS_PROCF (csisui, CSISUI) (int *ntsuit,
-                                int *ileaux,
                                 int *iccvfg);
 
 /*----------------------------------------------------------------------------
@@ -205,18 +192,11 @@ void CS_PROCF (uinum1, UINUM1) (double *cdtvar);
  * *****************
  *
  * INTEGER          RELAXP  -->   pressure relaxation
- * INTEGER          EXTRAG  -->   wall pressure extrapolation
  * INTEGER          IMRGRA  -->   gradient reconstruction
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (csnum2, CSNUM2) (double *relaxp,
-                                double *extrag,
-                                   int *imrgra);
-
-void CS_PROCF (csphys, CSPHYS) (
-                                double       *viscv0,
-                                double       *visls0,
-                                const    int *itempk);
+void CS_PROCF (csnum2, CSNUM2) (double  *relaxp,
+                                int     *imrgra);
 
 /*----------------------------------------------------------------------------
  * User scalar min and max values for clipping.
@@ -226,12 +206,15 @@ void CS_PROCF (csphys, CSPHYS) (
  * subroutine cssca2
  * *****************
  *
- * integer          iturt    -->  turbulent flux model
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (cssca2, CSSCA2) (int        *iturt);
+void CS_PROCF (cssca2, CSSCA2) (void);
 
-void CS_PROCF (cssca3, CSSCA3) (double     *visls0);
+/*----------------------------------------------------------------------------
+ * Read reference dynamic and user scalar viscosity
+ *----------------------------------------------------------------------------*/
+
+void CS_PROCF (cssca3, CSSCA3) (void);
 
 /*----------------------------------------------------------------------------
  * Turbulence initialization parameters.
@@ -245,19 +228,6 @@ void CS_PROCF (cssca3, CSSCA3) (double     *visls0);
 
 void CS_PROCF (cstini, CSTINI) (void);
 
-/*----------------------------------------------------------------------------
- * Solver taking a scalar porosity into account
- *
- * Fortran Interface:
- *
- * SUBROUTINE UIIPSU
- * *****************
- *
- * INTEGER          IPOROS     -->   porosity
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uiipsu, UIIPSU) (int *iporos);
-
 /*----------------------------------------------------------------------------
  * Define porosity.
  *
@@ -352,27 +322,9 @@ void CS_PROCF(uiiniv, UIINIV)(const int          *isuite,
  * *****************
  *
  * integer          iviscv   <--  pointer for volumic viscosity viscv
- * integer          itempk   <--  pointer for temperature (in K)
- * double precision visls0   <--  diffusion coefficient of the scalars
- * double precision viscv0   <--  volumic viscosity
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF(uiphyv, UIPHYV)(const cs_int_t  *iviscv,
-                              const cs_int_t  *itempk,
-                              const cs_real_t *visls0,
-                              const cs_real_t *viscv0);
-
-/*----------------------------------------------------------------------------
- * 1D profile postprocessing
- *
- * Fortran Interface:
- *
- * SUBROUTINE UIPROF
- * *****************
- *
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uiprof, UIPROF)(void);
+void CS_PROCF(uiphyv, UIPHYV)(const int       *iviscv);
 
 /*----------------------------------------------------------------------------
  * extra operations
@@ -410,19 +362,6 @@ void CS_PROCF (uidapp, UIDAPP) (const int       *permeability,
                                 const cs_real_t *gravity_z,
                                 const int       *unsaturated);
 
-/*----------------------------------------------------------------------------
- * Free memory: clean global private variables and libxml2 variables.
- *
- * Fortran Interface:
- *
- * SUBROUTINE MEMUI1
- * *****************
- *
- * INTEGER          NCHARB  <-- number of coal
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (memui1, MEMUI1) (const int *ncharb);
-
 /*----------------------------------------------------------------------------
  * Define fans with GUI
  *
@@ -455,15 +394,8 @@ void CS_PROCF (uieres, UIERES) (int *iescal,
  * Public function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Initialize GUI reader structures.
- *----------------------------------------------------------------------------*/
-
-void
-cs_gui_init(void);
-
 /*-----------------------------------------------------------------------------
- * Free memory: clean global private variables and libxml2 variables
+ * Free memory: clean global private variables.
  *----------------------------------------------------------------------------*/
 
 void
@@ -476,14 +408,16 @@ cs_gui_finalize(void);
  * Head loss tensor coefficients for each cell are organized as follows:
  * cku11, cku22, cku33, cku12, cku13, cku23.
  *
- * \param[in]       zone  pointer to zone structure
- * \param[in, out]  cku   head loss coefficients
+ * \param[in]       zone       pointer to zone structure
+ * \param[in]       cvara_vel  pointer to the velocity values of the previous time step
+ * \param[in, out]  cku        head loss coefficients
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_gui_head_losses(const cs_zone_t  *zone,
-                   cs_real_t         cku[][6]);
+cs_gui_head_losses(const cs_zone_t   *zone,
+                   const cs_real_3_t *cvara_vel,
+                   cs_real_t          cku[][6]);
 
 /*-----------------------------------------------------------------------------
  * Selection of linear solvers.
@@ -506,6 +440,28 @@ cs_gui_parallel_io(void);
 void
 cs_gui_partition(void);
 
+/*-----------------------------------------------------------------------------
+ * Set MPI related algorithm options
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_mpi_algorithms(void);
+
+/*----------------------------------------------------------------------------
+ * Treatment of gravity and fluid physical properties
+ * Initialize reference pressure and temperature if present
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_physical_properties(void);
+
+/*----------------------------------------------------------------------------
+ * Determine porosity model type
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_porous_model(void);
+
 /*-----------------------------------------------------------------------------
  * Get initial value from property markup.
  *
@@ -519,7 +475,21 @@ cs_gui_properties_value(const char  *property_name,
                         double      *value);
 
 /*-----------------------------------------------------------------------------
- * Initialization choice of the reference variables parameters.
+ * Get value of property markup for fluid of given id
+ *
+ * parameters:
+ *   fluid_id           <--   fluid index
+ *   property_name      <--   name of the property
+ *   value              -->   new initial value of the property
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_properties_value_by_fluid_id(const int    fluid_id,
+                                    const char  *property_name,
+                                    double      *value);
+
+/*-----------------------------------------------------------------------------
+ * Get value of reference fluid properties parameter.
  *
  * parameters:
  *   name            <--   parameter name
@@ -527,14 +497,14 @@ cs_gui_properties_value(const char  *property_name,
  *----------------------------------------------------------------------------*/
 
 void
-cs_gui_reference_initialization(const char  *param,
-                                double      *value);
+cs_gui_fluid_properties_value(const char  *param,
+                              double      *value);
 
 /*----------------------------------------------------------------------------
  * Get thermal scalar model.
  *
  * return:
- *   value of itherm
+ *   value of itherm*10 + (temperature variant flag), or -1 if not defined
  *----------------------------------------------------------------------------*/
 
 int
@@ -568,13 +538,6 @@ cs_gui_turbomachinery_rotor(void);
 void
 cs_gui_usage_log(void);
 
-/*----------------------------------------------------------------------------
- * Set GUI-defined user scalar labels.
- *----------------------------------------------------------------------------*/
-
-void
-cs_gui_user_scalar_labels(void);
-
 /*----------------------------------------------------------------------------
  * Define user variables through the GUI.
  *----------------------------------------------------------------------------*/
@@ -582,6 +545,13 @@ cs_gui_user_scalar_labels(void);
 void
 cs_gui_user_variables(void);
 
+/*----------------------------------------------------------------------------
+ * Define user arrays through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_user_arrays(void);
+
 /*----------------------------------------------------------------------------
  * Define balance by zone through the GUI.
  *----------------------------------------------------------------------------*/
@@ -621,6 +591,13 @@ cs_gui_error_estimator(int *iescal,
 void
 cs_gui_zones(void);
 
+/*----------------------------------------------------------------------------
+ * Define internal coupling through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_internal_coupling(void);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
