@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
+#include "cs_boundary.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -60,11 +61,7 @@ BEGIN_C_DECLS
  * *****************
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uiclim, UICLIM)(const int  *idarcy,
-                               const int  *nozppm,
-                               const int  *ncharm,
-                               const int  *ncharb,
-                               const int  *nclpch,
+void CS_PROCF (uiclim, UICLIM)(const int  *nozppm,
                                int        *iqimp,
                                int        *icalke,
                                int        *ientat,
@@ -101,13 +98,11 @@ void CS_PROCF (uiclim, UICLIM)(const int  *idarcy,
  * *****************
  *
  * integer          nozppm  <-- max number of boundary conditions zone
- * integer          iale    <-- ale module activated
  * integer          itypfb  <-- type of boundary for each face
  * integer          izfppp  <-- zone number
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (uiclve, UICLVE)(const int  *nozppm,
-                               const int  *iale,
                                int        *itypfb,
                                int        *izfppp);
 
@@ -115,12 +110,23 @@ void CS_PROCF (uiclve, UICLVE)(const int  *nozppm,
  * Public function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Free boundary conditions structures
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define boundary conditions based on setup file.
  *
- * parameters:
- *   ncharb  <-- number of coals
- *----------------------------------------------------------------------------*/
+ * \param[in, out]  bdy   boundaries structure to update
+ *                        (if NULL, default to cs_glob_domain->boundaries)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_gui_boundary_conditions_define(cs_boundary_t  *bdy);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Free GUI boundary condition structures.
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_gui_boundary_conditions_free_memory(void);
