@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -46,20 +46,20 @@
 #include "bft_error.h"
 #include "bft_printf.h"
 
-#include "mei_evaluate.h"
-
+#include "cs_ale.h"
 #include "cs_base.h"
+#include "cs_boundary.h"
 #include "cs_boundary_zone.h"
 #include "cs_convection_diffusion.h"
 #include "cs_field_pointer.h"
 #include "cs_gui.h"
 #include "cs_gui_util.h"
-#include "cs_gui_variables.h"
 #include "cs_gui_boundary_conditions.h"
 #include "cs_mesh.h"
 #include "cs_prototypes.h"
 #include "cs_timer.h"
 #include "cs_time_step.h"
+#include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -96,7 +96,8 @@ enum ale_boundary_nature
   ale_boundary_nature_internal_coupling,
   ale_boundary_nature_external_coupling,
   ale_boundary_nature_fixed_velocity,
-  ale_boundary_nature_fixed_displacement
+  ale_boundary_nature_fixed_displacement,
+  ale_boundary_nature_free_surface
 };
 
 /*============================================================================
@@ -104,7 +105,7 @@ enum ale_boundary_nature
  *============================================================================*/
 
 /*-----------------------------------------------------------------------------
- * Return value for iale method
+ * Return value for ALE method
  *
  * parameters:
  *   tn_ale    <-- tree node associated with ALE
@@ -114,7 +115,7 @@ enum ale_boundary_nature
  *----------------------------------------------------------------------------*/
 
 static int
-_iale_visc_type(cs_tree_node_t  *tn_ale)
+_ale_visc_type(cs_tree_node_t  *tn_ale)
 {
   int mvisc_type = 0;
 
@@ -134,77 +135,11 @@ _iale_visc_type(cs_tree_node_t  *tn_ale)
   return mvisc_type;
 }
 
-/*-----------------------------------------------------------------------------
- * Initialize mei tree and check for symbols existence
- *
- * parameters:
- *   formula        <-- mei formula
- *   symbols        <-- array of symbol to check
- *   symbol_nbr     <-- number of symbol in symbols
- *   variables      <-- variables required in the formula
- *   variable_nbr   <-- number of variable in variables
- *   dtref          <-- time step
- *   ttcabs         <-- current time
- *   ntcabs         <-- current iteration number
- *----------------------------------------------------------------------------*/
-
-static mei_tree_t *
-_init_mei_tree(const char    *formula,
-               const char   **symbols,
-               int            symbol_nbr,
-               const char   **variables,
-               const double  *variables_value,
-               int            variable_nbr,
-               const double   dtref,
-               const double   ttcabs,
-               const int      ntcabs)
-{
-  int i = 0;
-
-  /* return an empty interpreter */
-  mei_tree_t *tree = mei_tree_new(formula);
-
-  /* Insert variables into mei_tree */
-  for (i = 0; i < variable_nbr; ++i) {
-    double value = 0;
-
-    /* Read value from variables_value if it is not null 0 otherwise */
-    if (variables_value)
-      value = variables_value[i];
-    mei_tree_insert(tree, variables[i], value);
-  }
-
-  /* Add commun variables: dt, t, nbIter */
-  mei_tree_insert(tree, "dt",   dtref);
-  mei_tree_insert(tree, "t",    ttcabs);
-  mei_tree_insert(tree, "iter", ntcabs);
-
-  /* add variable from notebook */
-  cs_gui_add_notebook_variables(tree);
-
-  /* try to build the interpreter */
-  if (mei_tree_builder(tree))
-    bft_error(__FILE__, __LINE__, 0,
-              _("Error: can not interpret expression: %s\n"), tree->string);
-
-  /* Check for symbols */
-  for (i = 0; i < symbol_nbr; ++i) {
-    const char* symbol = symbols[i];
-
-    if (mei_tree_find_symbol(tree, symbol)) {
-      bft_error(__FILE__, __LINE__, 0,
-                _("Error: can not find the required symbol: %s\n"), symbol);
-    }
-  }
-
-  return tree;
-}
-
 /*-----------------------------------------------------------------------------
  * Get the ale boundary formula
  *
  * parameters:
- *   tn_w    <-- pointer to tree node for a given wall BC
+ *   tn_w    <-- pointer to tree node for a given BC
  *   choice  <-- nature: "fixed_velocity" or "fixed_displacement"
  *----------------------------------------------------------------------------*/
 
@@ -224,31 +159,20 @@ _get_ale_boundary_formula(cs_tree_node_t  *tn_w,
  * Get uialcl data for fixed displacement
  *
  * parameters:
- *   tn_w           <-- pointer to tree node for a given wall BC
- *   begin          <-- begin index for nodfbr
- *   end            <-- end index for nodfbr
- *   nnod           <-> number of nodes
- *   b_face_vtx_lst <-- nodfbr
+ *   tn_w           <-- pointer to tree node for a given mobile_boundary BC
+ *   z              <-- selected boundary zone
  *   impale         --> impale
  *   disale         --> disale
- *   dtref          <-- time step
- *   ttcabs         <-- current time
- *   ntcabs         <-- current iteration number
  *----------------------------------------------------------------------------*/
 
 static void
 _uialcl_fixed_displacement(cs_tree_node_t   *tn_w,
-                           const cs_lnum_t   begin,
-                           const cs_lnum_t   end,
-                           const cs_lnum_t   b_face_vtx_lst[],
+                           const cs_zone_t  *z,
                            int              *impale,
-                           cs_real_3_t      *disale,
-                           const double      dtref,
-                           const double      ttcabs,
-                           const int         ntcabs)
+                           cs_real_t         disale[][3])
 {
-  const char*  variables[3] = {"mesh_x", "mesh_y", "mesh_z"};
-  int variable_nbr = 3;
+
+  const cs_mesh_t *m = cs_glob_mesh;
 
   /* Get formula */
   const char *formula = _get_ale_boundary_formula(tn_w, "fixed_displacement");
@@ -258,27 +182,30 @@ _uialcl_fixed_displacement(cs_tree_node_t   *tn_w,
               _("Boundary nature formula is null for %s."),
               cs_gui_node_get_tag(tn_w, "label"));
 
-  /* Init mei */
-  mei_tree_t *ev = _init_mei_tree(formula, variables, variable_nbr,
-                                  0, 0, 0, dtref, ttcabs, ntcabs);
+  /* Evaluate formula using meg */
+  cs_real_t *bc_vals = cs_meg_boundary_function(z,
+                                                "mesh_velocity",
+                                                "fixed_displacement");
 
-  mei_evaluate(ev);
+  /* Loop over boundary faces */
+  for (cs_lnum_t elt_id = 0; elt_id < z->n_elts; elt_id++) {
+    const cs_lnum_t face_id = z->elt_ids[elt_id];
 
-  /* Get mei results */
-  cs_real_t x_mesh = mei_tree_lookup(ev, "mesh_x");
-  cs_real_t y_mesh = mei_tree_lookup(ev, "mesh_y");
-  cs_real_t z_mesh = mei_tree_lookup(ev, "mesh_z");
+    /* ALE BC on vertices */
+    const cs_lnum_t s = m->b_face_vtx_idx[face_id];
+    const cs_lnum_t e = m->b_face_vtx_idx[face_id+1];
 
-  mei_tree_destroy(ev);
+    /* Compute the portion of surface associated to v_id_1 */
+    for (cs_lnum_t k = s; k < e; k++) {
+
+      const cs_lnum_t v_id = m->b_face_vtx_lst[k];
+      cs_real_t *_val = disale[v_id];
+
+      impale[v_id] = 1;
+      //FIXME prorata
+      for (int d = 0; d < 3; d++)
+        _val[d] = bc_vals[elt_id + d * z->n_elts];
 
-  /* Set disale and impale */
-  for (cs_lnum_t ii = begin; ii < end; ++ii) {
-    cs_lnum_t inod = b_face_vtx_lst[ii];
-    if (impale[inod] == 0) {
-      disale[inod][0] = x_mesh;
-      disale[inod][1] = y_mesh;
-      disale[inod][2] = z_mesh;
-      impale[inod] = 1;
     }
   }
 }
@@ -287,17 +214,13 @@ _uialcl_fixed_displacement(cs_tree_node_t   *tn_w,
  * Get uialcl data for fixed velocity
  *
  * parameters:
- *   tn_w         <-- pointer to tree node for a given wall BC
+ *   tn_w         <-- pointer to tree node for a given mobile_boundary BC
  *   iuma         <-- iuma
  *   ivma         <-- ivma
  *   iwma         <-- iwma
  *   nfabor       <-- Number of boundary faces
- *   n_faces      <-- number of selected faces
- *   faces_list   <-- listof selected faces
+ *   z            <-- selected boundary zone
  *   rcodcl       --> rcodcl
- *   dtref        <-- time step
- *   ttcabs       <-- current time
- *   ntcabs       <-- current iteration number
  *----------------------------------------------------------------------------*/
 
 static void
@@ -307,18 +230,10 @@ _uialcl_fixed_velocity(cs_tree_node_t  *tn_w,
                        int              iwma,
                        int              ivimpo,
                        cs_lnum_t        nfabor,
-                       cs_lnum_t        n_faces,
-                       const cs_lnum_t  faces_list[],
+                       const cs_zone_t *z,
                        int              ialtyb[],
-                       double          *rcodcl,
-                       double           dtref,
-                       double           ttcabs,
-                       int              ntcabs)
+                       double          *rcodcl)
 {
-  const char*  variables[3] = {"mesh_velocity_U",
-                               "mesh_velocity_V",
-                               "mesh_velocity_W" };
-
   /* Get formula */
   const char *formula = _get_ale_boundary_formula(tn_w, "fixed_velocity");
 
@@ -327,60 +242,128 @@ _uialcl_fixed_velocity(cs_tree_node_t  *tn_w,
               _("Boundary nature formula is null for %s."),
               cs_gui_node_get_tag(tn_w, "label"));
 
-  /* Init MEI */
-  mei_tree_t *ev = _init_mei_tree(formula, variables, 3, 0, 0, 0,
-                                  dtref, ttcabs, ntcabs);
+  /* Evaluate formula using meg */
+  cs_real_t *bc_vals = cs_meg_boundary_function(z,
+                                                "mesh_velocity",
+                                                "fixed_velocity");
 
-  for (cs_lnum_t ifac = 0; ifac < n_faces; ifac++) {
+  /* Loop over boundary faces */
+  for (cs_lnum_t elt_id = 0; elt_id < z->n_elts; elt_id++) {
+    const cs_lnum_t face_id = z->elt_ids[elt_id];
 
-    cs_lnum_t ifbr = faces_list[ifac];
-
-    mei_evaluate(ev);
 
     /* Fill  rcodcl */
-    rcodcl[(iuma-1) * nfabor + ifbr] = mei_tree_lookup(ev, "mesh_velocity_U");
-    rcodcl[(ivma-1) * nfabor + ifbr] = mei_tree_lookup(ev, "mesh_velocity_V");
-    rcodcl[(iwma-1) * nfabor + ifbr] = mei_tree_lookup(ev, "mesh_velocity_W");
+    rcodcl[(iuma-1) * nfabor + face_id] = bc_vals[elt_id + 0 * z->n_elts];
+    rcodcl[(ivma-1) * nfabor + face_id] = bc_vals[elt_id + 1 * z->n_elts];
+    rcodcl[(iwma-1) * nfabor + face_id] = bc_vals[elt_id + 2 * z->n_elts];
 
-    ialtyb[ifbr]  = ivimpo;
+    ialtyb[face_id]  = ivimpo;
 
   }
 
-  mei_tree_destroy(ev);
+  /* Free memory */
+  BFT_FREE(bc_vals);
+
 }
 
 /*-----------------------------------------------------------------------------
  * Return the ale boundary nature of a given boundary condition
  *
  * parameters:
- *   tn_w <-- pointer to tree node for a given wall BC
+ *   tn <-> pointer to tree node for a given mobile_boundary BC
  *
  * return:
  *   associated nature
  *----------------------------------------------------------------------------*/
 
 static enum  ale_boundary_nature
-_get_ale_boundary_nature(cs_tree_node_t  *tn_w)
+_get_ale_boundary_nature(cs_tree_node_t  *tn)
 {
-  enum ale_boundary_nature nature = ale_boundary_nature_none;
+  const char *nat_bndy = cs_tree_node_get_tag(tn, "nature");
 
-  cs_tree_node_t *tn = cs_tree_get_node(tn_w, "ale/choice");
-  const char *nat = cs_tree_node_get_value_str(tn);
+  if (cs_gui_strcmp(nat_bndy, "free_surface"))
+    return ale_boundary_nature_free_surface;
 
-  if (cs_gui_strcmp(nat, "fixed_boundary"))
-    nature = ale_boundary_nature_fixed_wall;
-  if (cs_gui_strcmp(nat, "sliding_boundary"))
-    nature = ale_boundary_nature_sliding_wall;
-  else if (cs_gui_strcmp(nat, "internal_coupling"))
-    nature = ale_boundary_nature_internal_coupling;
-  else if (cs_gui_strcmp(nat, "external_coupling"))
-    nature = ale_boundary_nature_external_coupling;
-  else if (cs_gui_strcmp(nat, "fixed_velocity"))
-    nature = ale_boundary_nature_fixed_velocity;
-  else if (cs_gui_strcmp(nat, "fixed_displacement"))
-    nature = ale_boundary_nature_fixed_displacement;
+  else {
 
-  return nature;
+    const char *label = cs_tree_node_get_tag(tn, "label");
+
+    /* get the matching BC node */
+    tn = cs_tree_node_get_child(tn->parent, nat_bndy);
+
+    /* Now searh from siblings */
+    tn = cs_tree_node_get_sibling_with_tag(tn, "label", label);
+
+    /* Finaly get child node ALE */
+    cs_tree_node_t *tn_ale = cs_tree_get_node(tn, "ale/choice");
+    const char *nat_ale = cs_tree_node_get_value_str(tn_ale);
+
+    if (cs_gui_strcmp(nat_ale, "fixed_boundary"))
+      return ale_boundary_nature_fixed_wall;
+    if (cs_gui_strcmp(nat_ale, "sliding_boundary"))
+      return ale_boundary_nature_sliding_wall;
+    else if (cs_gui_strcmp(nat_ale, "internal_coupling"))
+      return ale_boundary_nature_internal_coupling;
+    else if (cs_gui_strcmp(nat_ale, "external_coupling"))
+      return ale_boundary_nature_external_coupling;
+    else if (cs_gui_strcmp(nat_ale, "fixed_velocity"))
+      return ale_boundary_nature_fixed_velocity;
+    else if (cs_gui_strcmp(nat_ale, "fixed_displacement"))
+      return ale_boundary_nature_fixed_displacement;
+    else
+      return ale_boundary_nature_none;
+  }
+}
+
+/*-----------------------------------------------------------------------------
+ * Return the ale boundary type of a given boundary condition
+ *
+ * parameters:
+ *   tn_bndy <-- pointer to tree node for a given BC
+ *
+ * return:
+ *   associated boundary type
+ *----------------------------------------------------------------------------*/
+
+static cs_boundary_type_t
+_get_ale_boundary_type(cs_tree_node_t  *tn_bndy)
+{
+  const char *nat_bndy = cs_tree_node_get_tag(tn_bndy, "nature");
+
+  if (cs_gui_strcmp(nat_bndy, "free_surface"))
+    return CS_BOUNDARY_ALE_FREE_SURFACE;
+
+  else {
+
+    const char *label = cs_tree_node_get_tag(tn_bndy, "label");
+
+    /* get the matching BC node */
+    cs_tree_node_t *tn = cs_tree_node_get_child(tn_bndy->parent, nat_bndy);
+
+    /* Now search from siblings */
+    tn = cs_tree_node_get_sibling_with_tag(tn, "label", label);
+
+    /* Finally get child node ALE */
+    tn = cs_tree_get_node(tn, "ale/choice");
+    const char *nat = cs_tree_node_get_value_str(tn);
+
+    if (cs_gui_strcmp(nat, "fixed_boundary"))
+      return CS_BOUNDARY_ALE_FIXED;
+    else if (cs_gui_strcmp(nat, "sliding_boundary"))
+      return CS_BOUNDARY_ALE_SLIDING;
+    else if (cs_gui_strcmp(nat, "internal_coupling"))
+      return CS_BOUNDARY_ALE_INTERNAL_COUPLING;
+    else if (cs_gui_strcmp(nat, "external_coupling"))
+      return CS_BOUNDARY_ALE_EXTERNAL_COUPLING;
+    else if (cs_gui_strcmp(nat, "fixed_velocity"))
+      return CS_BOUNDARY_ALE_IMPOSED_VEL;
+    else if (cs_gui_strcmp(nat, "fixed_displacement"))
+      return CS_BOUNDARY_ALE_IMPOSED_DISP;
+    else if (cs_gui_strcmp(nat, "free_surface"))
+      return CS_BOUNDARY_ALE_FREE_SURFACE;
+    else
+      return CS_BOUNDARY_UNDEFINED;
+  }
 }
 
 /*-----------------------------------------------------------------------------
@@ -404,63 +387,6 @@ _get_internal_coupling_xyz_values(cs_tree_node_t  *tn_ic,
   cs_gui_node_get_child_real(tn, "Z", xyz+2);
 }
 
-/*-----------------------------------------------------------------------------
- * Retrieve the internal coupling matrices
- *
- * parameters:
- *   tn_ic            <-- node for a given BC's internal coupling definitions
- *   name             <-- matrix name
- *   symbols          <-- see _init_mei_tree
- *   symbol_nbr       <-- see _init_mei_tree
- *   variables        <-- see _init_mei_tree
- *   variables_value  <-- see _init_mei_tree
- *   variable_nbr     <-- see _init_mei_tree
- *   output_matrix,   --> result matrix
- *   dtref            <-- time step
- *   ttcabs           <-- current time
- *   ntcabs           <-- current iteration number
- *----------------------------------------------------------------------------*/
-
-static void
-_get_internal_coupling_matrix(cs_tree_node_t  *tn_ic,
-                              const char      *name,
-                              const char      *symbols[],
-                              int              symbol_nbr,
-                              const char     **variables,
-                              const double    *variables_value,
-                              int              variable_nbr,
-                              double          *output_matrix,
-                              double           dtref,
-                              double           ttcabs,
-                              int              ntcabs)
-{
-  /* Get the formula */
-  mei_tree_t *tree;
-
-  int i = 0;
-  cs_tree_node_t  *tn = cs_tree_node_get_child(tn_ic, name);
-
-  const char *matrix = cs_tree_node_get_child_value_str(tn, "formula");
-
-  if (!matrix)
-    bft_error(__FILE__, __LINE__, 0,
-              _("Formula is null for %s %s"), tn_ic->name, name);
-
-  /* Initialize mei */
-  tree = _init_mei_tree(matrix, symbols, symbol_nbr,
-                        variables, variables_value, variable_nbr,
-                        dtref, ttcabs, ntcabs);
-  mei_evaluate(tree);
-
-  /* Read matrix values */
-  for (i = 0; i < symbol_nbr; ++i) {
-    const char *symbol = symbols[i];
-    output_matrix[i] = mei_tree_lookup(tree, symbol);
-  }
-
-  mei_tree_destroy(tree);
-}
-
 /*-----------------------------------------------------------------------------
  * Retrieve data for internal coupling for a specific boundary
  *
@@ -471,69 +397,33 @@ _get_internal_coupling_matrix(cs_tree_node_t  *tn_ic,
  *   xkstru   --> Stiffness matrix
  *   forstr   --> Fluid force matrix
  *   istruc   <-- internal coupling boundary index
- *   dtref    <-- time step
- *   ttcabs   <-- current time
- *   ntcabs   <-- current iteration number
  *----------------------------------------------------------------------------*/
 
 static void
-_get_uistr2_data(cs_tree_node_t  *tn_ic,
+_get_uistr2_data(const char      *label,
                  double          *xmstru,
                  double          *xcstru,
                  double          *xkstru,
                  double          *forstr,
-                 int              istruc,
-                 double           dtref,
-                 double           ttcabs,
-                 int              ntcabs)
+                 int              istruc)
 {
-  const char  *m_symbols[] = {"m11", "m12", "m13",
-                              "m21", "m22", "m23",
-                              "m31", "m32", "m33"};
-  const char  *c_symbols[] = {"c11", "c12", "c13",
-                              "c21", "c22", "c23",
-                              "c31", "c32", "c33"};
-  const char  *k_symbols[] = {"k11", "k12", "k13",
-                              "k21", "k22", "k23",
-                              "k31", "k32", "k33"};
-
-  int symbol_nbr = sizeof(m_symbols) / sizeof(m_symbols[0]);
-
-  const char   *force_symbols[] = {"fx", "fy", "fz"};
-  int force_symbol_nbr = sizeof(force_symbols) / sizeof(force_symbols[0]);
-
-  const int  variable_nbr = 3;
-  const char *variables[3] = {"fluid_fx", "fluid_fy", "fluid_fz"};
-  double variable_values[3];
-
   /* Get mass matrix, damping matrix and stiffness matrix */
 
-  _get_internal_coupling_matrix(tn_ic, "mass_matrix", m_symbols,
-                                symbol_nbr, 0, 0, 0,
-                                &xmstru[istruc * symbol_nbr],
-                                dtref, ttcabs, ntcabs);
+  cs_meg_fsi_struct("mass_matrix", label, NULL,
+                    &xmstru[istruc * 9]);
+  cs_meg_fsi_struct("damping_matrix", label, NULL,
+                    &xcstru[istruc * 9]);
+  cs_meg_fsi_struct("stiffness_matrix", label, NULL,
+                    &xkstru[istruc * 9]);
 
-  _get_internal_coupling_matrix(tn_ic, "damping_matrix", c_symbols,
-                                symbol_nbr, 0, 0, 0,
-                                &xcstru[istruc * symbol_nbr],
-                                dtref, ttcabs, ntcabs);
-
-  _get_internal_coupling_matrix(tn_ic, "stiffness_matrix", k_symbols,
-                                symbol_nbr, 0, 0, 0,
-                                &xkstru[istruc * symbol_nbr],
-                                dtref, ttcabs, ntcabs);
-
-  /* Set variable for fluid force matrix */
-  variable_values[0] = forstr[istruc * force_symbol_nbr + 0];
-  variable_values[1] = forstr[istruc * force_symbol_nbr + 1];
-  variable_values[2] = forstr[istruc * force_symbol_nbr + 2];
+  /* Set variable for fluid force vector */
+  const cs_real_t fluid_f[3] = {forstr[istruc*3],
+                                forstr[istruc*3 + 1],
+                                forstr[istruc*3 + 2]};
 
   /* Get fluid force matrix */
-  _get_internal_coupling_matrix(tn_ic, "fluid_force_matrix",
-                                force_symbols, force_symbol_nbr,
-                                variables, variable_values, variable_nbr,
-                                &forstr[istruc * force_symbol_nbr],
-                                dtref, ttcabs, ntcabs);
+  cs_meg_fsi_struct("fluid_force", label, fluid_f,
+                    &forstr[istruc * 3]);
 }
 
 /*-----------------------------------------------------------------------------
@@ -575,27 +465,25 @@ _get_external_coupling_dof(cs_tree_node_t  *tn_ec,
  * SUBROUTINE UIALIN
  * *****************
  *
- * INTEGER          IALE    <--  iale method activation
- * INTEGER          NALINF  <--  number of sub iteration of initialization
- *                               of fluid
- * INTEGER          NALIMX  <--  max number of iterations of implicitation of
- *                               the displacement of the structures
- * DOUBLE           EPALIM  <--  realtive precision of implicitation of
- *                               the displacement of the structures
+ * nalinf  <->  number of sub iteration of initialization
+ *              of fluid
+ * nalimx  <->  max number of iterations of implicitation of
+ *              the displacement of the structures
+ * epalim  <->  realtive precision of implicitation of
+ *              the displacement of the structures
  *
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uialin, UIALIN) (int     *iale,
-                                int     *nalinf,
+void CS_PROCF (uialin, UIALIN) (int     *nalinf,
                                 int     *nalimx,
                                 double  *epalim)
 {
   cs_tree_node_t *tn
     = cs_tree_get_node(cs_glob_tree, "thermophysical_models/ale_method");
 
-  cs_gui_node_get_status_int(tn, iale);
+  cs_gui_node_get_status_int(tn, &cs_glob_ale);
 
-  if (*iale) {
+  if (cs_glob_ale) {
     cs_gui_node_get_child_int(tn, "fluid_initialization_sub_iterations",
                               nalinf);
     cs_gui_node_get_child_int(tn, "max_iterations_implicitation",
@@ -606,8 +494,8 @@ void CS_PROCF (uialin, UIALIN) (int     *iale,
 
 #if _XML_DEBUG_
   bft_printf("==> %s\n", __func__);
-  bft_printf("--iale = %i\n", *iale);
-  if (*iale) {
+  bft_printf("--cs_glob_ale = %i\n", cs_glob_ale);
+  if (cs_glob_ale > 0) {
     bft_printf("--nalinf = %i\n", *nalinf);
     bft_printf("--nalimx = %i\n", *nalimx);
     bft_printf("--epalim = %g\n", *epalim);
@@ -629,7 +517,7 @@ void CS_PROCF (uialvm, UIALVM) ()
   cs_tree_node_t *tn
     = cs_tree_get_node(cs_glob_tree, "thermophysical_models/ale_method");
 
-  int iortvm = _iale_visc_type(tn);
+  int iortvm = _ale_visc_type(tn);
 
   cs_var_cal_opt_t vcopt;
   int key_cal_opt_id = cs_field_key_id("var_cal_opt");
@@ -641,6 +529,7 @@ void CS_PROCF (uialvm, UIALVM) ()
   } else { /* isotropic viscosity */
     vcopt.idften = CS_ISOTROPIC_DIFFUSION;
   }
+  cs_field_set_key_struct(f_mesh_u, key_cal_opt_id, &vcopt);
 
 #if _XML_DEBUG_
   bft_printf("==> %s\n", __func__);
@@ -674,19 +563,17 @@ void CS_PROCF (uialcl, UIALCL) (const int *const    ibfixe,
                                 const int *const    iwma,
                                 double    *const    rcodcl)
 {
-  double t0;
-
   const cs_mesh_t *m = cs_glob_mesh;
 
   cs_tree_node_t *tn_b0 = cs_tree_get_node(cs_glob_tree, "boundary_conditions");
 
-  /* Loop on wall boundary zones */
+  /* Loop on boundary zones */
 
-  for (cs_tree_node_t *tn_w = cs_tree_node_get_child(tn_b0, "wall");
-       tn_w != NULL;
-       tn_w = cs_tree_node_get_next_of_name(tn_w)) {
+  for (cs_tree_node_t *tn_bndy = cs_tree_node_get_child(tn_b0, "boundary");
+       tn_bndy != NULL;
+       tn_bndy = cs_tree_node_get_next_of_name(tn_bndy)) {
 
-    const char *label = cs_tree_node_get_tag(tn_w, "label");
+    const char *label = cs_tree_node_get_tag(tn_bndy, "label");
 
     const cs_zone_t *z = cs_boundary_zone_by_name_try(label);
     if (z == NULL)  /* possible in case of old XML file with "dead" nodes */
@@ -695,11 +582,17 @@ void CS_PROCF (uialcl, UIALCL) (const int *const    ibfixe,
     cs_lnum_t n_faces = z->n_elts;
     const cs_lnum_t *faces_list = z->elt_ids;
 
-    enum ale_boundary_nature nature = _get_ale_boundary_nature(tn_w);
+    /* Get ALE nature and get sibling tn */
+    enum ale_boundary_nature nature = _get_ale_boundary_nature(tn_bndy);
 
     if (nature == ale_boundary_nature_none)
       continue;
 
+    /* get the matching BC node */
+    const char *nat_bndy = cs_tree_node_get_tag(tn_bndy, "nature");
+    cs_tree_node_t *tn_bc = cs_tree_node_get_child(tn_bndy->parent, nat_bndy);
+    tn_bc = cs_tree_node_get_sibling_with_tag(tn_bc, "label", label);
+
     if (nature ==  ale_boundary_nature_fixed_wall) {
       for (cs_lnum_t ifac = 0; ifac < n_faces; ifac++) {
         cs_lnum_t ifbr = faces_list[ifac];
@@ -712,53 +605,25 @@ void CS_PROCF (uialcl, UIALCL) (const int *const    ibfixe,
         ialtyb[ifbr] = *igliss;
       }
     }
-    else if (nature == ale_boundary_nature_fixed_displacement) {
-      t0 = cs_timer_wtime();
+    else if (nature ==  ale_boundary_nature_free_surface) {
       for (cs_lnum_t ifac = 0; ifac < n_faces; ifac++) {
         cs_lnum_t ifbr = faces_list[ifac];
-        _uialcl_fixed_displacement(tn_w,
-                                   m->b_face_vtx_idx[ifbr],
-                                   m->b_face_vtx_idx[ifbr+1],
-                                   m->b_face_vtx_lst, impale, disale,
-                                   cs_glob_time_step_options->dtref,
-                                   cs_glob_time_step->t_cur,
-                                   cs_glob_time_step->nt_cur);
+        ialtyb[ifbr] = *ifresf;
       }
-      cs_gui_add_mei_time(cs_timer_wtime() - t0);
+    }
+    else if (nature == ale_boundary_nature_fixed_displacement) {
+      _uialcl_fixed_displacement(tn_bc,
+                                 z,
+                                 impale,
+                                 disale);
     }
     else if (nature == ale_boundary_nature_fixed_velocity) {
-      t0 = cs_timer_wtime();
-      _uialcl_fixed_velocity(tn_w, *iuma, *ivma, *iwma, *ivimpo,
-                             m->n_b_faces, n_faces, faces_list,
-                             ialtyb, rcodcl,
-                             cs_glob_time_step_options->dtref,
-                             cs_glob_time_step->t_cur,
-                             cs_glob_time_step->nt_cur);
-      cs_gui_add_mei_time(cs_timer_wtime() - t0);
+      _uialcl_fixed_velocity(tn_bc, *iuma, *ivma, *iwma, *ivimpo,
+                             m->n_b_faces,
+                             z,
+                             ialtyb, rcodcl);
     }
   }
-
-  /* Loop on free surface boundary zones */
-
-  for (cs_tree_node_t *tn = cs_tree_node_get_child(tn_b0, "free_surface");
-       tn != NULL;
-       tn = cs_tree_node_get_next_of_name(tn)) {
-
-    const char *label = cs_tree_node_get_tag(tn, "label");
-
-    const cs_zone_t *z = cs_boundary_zone_by_name_try(label);
-    if (z == NULL)  /* possible in case of old XML file with "dead" nodes */
-      continue;
-
-    cs_lnum_t n_faces = z->n_elts;
-    const cs_lnum_t *faces_list = z->elt_ids;
-
-    for (cs_lnum_t ifac = 0; ifac < n_faces; ifac++) {
-      cs_lnum_t ifbr = faces_list[ifac];
-      ialtyb[ifbr]  = *ifresf;
-    }
-
-  }
 }
 
 /*-----------------------------------------------------------------------------
@@ -800,7 +665,7 @@ void CS_PROCF (uistr1, UISTR1) (cs_lnum_t        *idfstr,
   cs_tree_node_t *tn = cs_tree_get_node(cs_glob_tree, "boundary_conditions");
 
   cs_tree_node_t *tn_b0 = cs_tree_node_get_child(tn, "boundary");
-  cs_tree_node_t *tn_w0 = cs_tree_node_get_child(tn, "wall");
+  cs_tree_node_t *tn_w0 = cs_tree_node_get_child(tn, "boundary");//FIXME
 
   /* At each time-step, loop on boundary faces */
 
@@ -859,54 +724,46 @@ void CS_PROCF (uistr1, UISTR1) (cs_lnum_t        *idfstr,
  *   xcstr        --> Damping matrix
  *   xkstru       --> Stiffness matrix
  *   forstr       --> Fluid force matrix
- *   dtref        <--  time step
- *   ttcabs       <-- current time
- *   ntcabs       <-- current iteration number
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (uistr2, UISTR2) (double *const  xmstru,
                                 double *const  xcstru,
                                 double *const  xkstru,
-                                double *const  forstr,
-                                double *const  dtref,
-                                double *const  ttcabs,
-                                int    *const  ntcabs)
+                                double *const  forstr)
 {
-  int istru   = 0;
+  int istru = 0;
 
-  cs_tree_node_t *tn = cs_tree_get_node(cs_glob_tree, "boundary_conditions");
-
-  cs_tree_node_t *tn_b0 = cs_tree_node_get_child(tn, "boundary");
-  cs_tree_node_t *tn_w0 = cs_tree_node_get_child(tn, "wall");
+  cs_tree_node_t *tn_b0 = cs_tree_get_node(cs_glob_tree, "boundary_conditions");
 
   /* At each time-step, loop on boundary faces */
 
-  for (tn = tn_b0; tn != NULL; tn = cs_tree_node_get_next_of_name(tn)) {
+  for (cs_tree_node_t *tn_bndy = cs_tree_node_get_child(tn_b0, "boundary");
+       tn_bndy != NULL;
+       tn_bndy = cs_tree_node_get_next_of_name(tn_bndy)) {
 
-    const char *label = cs_tree_node_get_tag(tn, "label");
+    const char *label = cs_tree_node_get_tag(tn_bndy, "label");
 
-    cs_tree_node_t *tn_w
-      = cs_tree_node_get_sibling_with_tag(tn_w0, "label", label);
-
-    enum ale_boundary_nature nature =_get_ale_boundary_nature(tn_w);
+    enum ale_boundary_nature nature =_get_ale_boundary_nature(tn_bndy);
 
     /* Keep only internal coupling */
     if (nature == ale_boundary_nature_internal_coupling) {
 
-      cs_tree_node_t *tn_ic = cs_tree_get_node(tn_w, "ale");
+      /* get the matching BC node */
+      const char *nat_bndy = cs_tree_node_get_tag(tn_bndy, "nature");
+      cs_tree_node_t *tn_bc = cs_tree_node_get_child(tn_bndy->parent, nat_bndy);
+      tn_bc = cs_tree_node_get_sibling_with_tag(tn_bc, "label", label);
+
+      cs_tree_node_t *tn_ic = cs_tree_get_node(tn_bc, "ale");
       tn_ic = cs_tree_node_get_sibling_with_tag(tn_ic,
                                                 "choice",
                                                 "internal_coupling");
 
-      _get_uistr2_data(tn_ic,
+      _get_uistr2_data(label,
                        xmstru,
                        xcstru,
                        xkstru,
                        forstr,
-                       istru,
-                       *dtref,
-                       *ttcabs,
-                       *ntcabs);
+                       istru);
       ++istru;
     }
   }
@@ -921,18 +778,15 @@ void CS_PROCF (uistr2, UISTR2) (double *const  xmstru,
  *----------------------------------------------------------------------------*/
 
 void
-CS_PROCF(uiaste, UIASTE)(int       *idfstr,
-                         cs_int_t  *asddlf)
+CS_PROCF(uiaste, UIASTE)(int  *idfstr,
+                         int  *asddlf)
 {
-  if (!cs_gui_file_is_loaded())
-    return;
-
   int istruct     = 0;
 
   cs_tree_node_t *tn = cs_tree_get_node(cs_glob_tree, "boundary_conditions");
 
   cs_tree_node_t *tn_b0 = cs_tree_node_get_child(tn, "boundary");
-  cs_tree_node_t *tn_w0 = cs_tree_node_get_child(tn, "wall");
+  cs_tree_node_t *tn_w0 = cs_tree_node_get_child(tn, "boundary");//FIXME
 
   /* Loop on boundary faces */
 
@@ -993,7 +847,7 @@ cs_gui_get_ale_viscosity_type(int  *type)
     = cs_tree_get_node(cs_glob_tree,
                        "thermophysical_models/ale_method");
 
-  *type = _iale_visc_type(tn);
+  *type = _ale_visc_type(tn);
 }
 
 /*----------------------------------------------------------------------------
@@ -1003,9 +857,6 @@ cs_gui_get_ale_viscosity_type(int  *type)
 void
 cs_gui_mesh_viscosity(void)
 {
-  if (!cs_gui_file_is_loaded())
-    return;
-
   cs_tree_node_t *tn0
     = cs_tree_get_node(cs_glob_tree, "thermophysical_models/ale_method");
 
@@ -1015,44 +866,118 @@ cs_gui_mesh_viscosity(void)
   if (mvisc_expr == NULL)
     return;
 
-  const cs_real_3_t *restrict cell_cen
-    = (const cs_real_3_t *restrict)cs_glob_mesh_quantities->cell_cen;
-  const cs_lnum_t n_cells   = cs_glob_mesh->n_cells;
-  const char * symbols[3]   = {"x", "y", "z" };
-  const char * variables[3] = {"mesh_viscosity_1",
-                               "mesh_viscosity_2",
-                               "mesh_viscosity_3" };
+  /* Compute mesh viscosity using the MEG function.
+   * The function accepts a field of arbitrary dimension, hence no need to
+   * check here. The formula is generated by the code.
+   */
+  const cs_zone_t *z_all = cs_volume_zone_by_name("all_cells");
+  cs_field_t *fmeg[1] = {CS_F_(vism)};
+  cs_meg_volume_function(z_all, fmeg);
 
-  int orthotropic = _iale_visc_type(tn0);
+}
 
-  cs_lnum_t nd = (orthotropic) ? 3 : 1;
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Translate the user settings for the domain boundaries into a
+ *         structure storing the ALE boundaries (New mechanism used in CDO)
+ *
+ * \param[in, out]  domain   pointer to a \ref cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
 
-  /* Init mei */
-  mei_tree_t *ev = _init_mei_tree(mvisc_expr, variables,
-                                  nd, symbols, 0, 3,
-                                  cs_glob_time_step_options->dtref,
-                                  cs_glob_time_step->t_cur,
-                                  cs_glob_time_step->nt_cur);
+void
+cs_gui_mobile_mesh_get_boundaries(cs_domain_t  *domain)
+{
+  assert(domain != NULL);
 
-  /* for each cell, update the value of the table of symbols for each scalar
-     (including the thermal scalar), and evaluate the interpreter */
+  cs_tree_node_t *tn_b0 = cs_tree_get_node(cs_glob_tree, "boundary_conditions");
 
-  for (cs_lnum_t c_id = 0; c_id < n_cells; c_id++) {
-    /* insert symbols */
-    mei_tree_insert(ev, "x", cell_cen[c_id][0]);
-    mei_tree_insert(ev, "y", cell_cen[c_id][1]);
-    mei_tree_insert(ev, "z", cell_cen[c_id][2]);
+  /* Loop on boundary zones */
 
-    mei_evaluate(ev);
+  for (cs_tree_node_t *tn_bndy = cs_tree_node_get_child(tn_b0, "boundary");
+       tn_bndy != NULL;
+       tn_bndy = cs_tree_node_get_next_of_name(tn_bndy)) {
+
+    const char *label = cs_tree_node_get_tag(tn_bndy, "label");
+    const cs_zone_t *z = cs_boundary_zone_by_name_try(label);
+    if (z == NULL)  /* possible in case of old XML file with "dead" nodes */
+      continue;
+
+    cs_boundary_type_t ale_bdy = _get_ale_boundary_type(tn_bndy);
+
+    if (ale_bdy == CS_BOUNDARY_UNDEFINED)
+      continue;
+
+    cs_boundary_add(domain->ale_boundaries,
+                    ale_bdy,
+                    z->name);
+
+  } /* Loop on mobile_boundary zones */
+
+    /* TODO */
+    /* else if (nature == ale_boundary_nature_fixed_displacement) { */
+    /*   _uialcl_fixed_displacement(tn_bndy, z, */
+    /*                              impale, disale); */
+    /* } */
+    /* else if (nature == ale_boundary_nature_fixed_velocity) { */
+    /*   _uialcl_fixed_velocity(tn_bndy, *iuma, *ivma, *iwma, *ivimpo, */
+    /*                          m->n_b_faces, z, */
+    /*                          ialtyb, rcodcl); */
+    /* } */
+
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return the fixed velocity for a boundary
+ *
+ * \param[in]  label boundary condition label
+ *
+ * \return a pointer to an array of cs_real_t values
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_real_t *
+cs_gui_mobile_mesh_get_fixed_velocity(const char    *label)
+{
+  cs_tree_node_t *tn_b0 = cs_tree_get_node(cs_glob_tree, "boundary_conditions");
+
+  /* Loop on boundary zones */
+
+  for (cs_tree_node_t *tn_bndy = cs_tree_node_get_child(tn_b0, "boundary");
+       tn_bndy != NULL;
+       tn_bndy = cs_tree_node_get_next_of_name(tn_bndy)) {
+
+    const char *nat_bndy = cs_tree_node_get_tag(tn_bndy, "nature");
+    const char *label_bndy = cs_tree_node_get_tag(tn_bndy, "label");
+
+    /* get the matching BC node */
+    cs_tree_node_t *tn = cs_tree_node_get_child(tn_bndy->parent, nat_bndy);
+
+    /* Now searh from siblings */
+    tn = cs_tree_node_get_sibling_with_tag(tn, "label", label_bndy);
+
+    if (strcmp(label_bndy, label) == 0) {
+
+      /* Get formula */
+      const char *formula = _get_ale_boundary_formula(tn, "fixed_velocity");
+
+      if (!formula)
+        bft_error(__FILE__, __LINE__, 0,
+                  _("Boundary nature formula is null for %s."),
+                  cs_gui_node_get_tag(tn, "label"));
+
+      const cs_zone_t *bz = cs_boundary_zone_by_name(label);
+
+      /* Evaluate formula using meg */
+      return cs_meg_boundary_function(bz,
+                                      "mesh_velocity",
+                                      "fixed_velocity");
 
-    /* Set mesh_u components */
-    CS_F_(vism)->val[nd*c_id] = mei_tree_lookup(ev, "mesh_viscosity_1");
-    if (orthotropic) {
-      CS_F_(vism)->val[nd*c_id + 1] = mei_tree_lookup(ev, "mesh_viscosity_2");
-      CS_F_(vism)->val[nd*c_id + 2] = mei_tree_lookup(ev, "mesh_viscosity_3");
     }
   }
-  mei_tree_destroy(ev);
+
+  return NULL; /* avoid a compilation warning */
 }
 
 /*----------------------------------------------------------------------------*/
