@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -48,22 +48,23 @@
 
 #include "fvm_selector.h"
 
-#include "mei_evaluate.h"
-
 #include "cs_base.h"
+#include "cs_boundary_zone.h"
 #include "cs_gui.h"
 #include "cs_gui_util.h"
-#include "cs_gui_variables.h"
 #include "cs_log.h"
+#include "cs_math.h"
 #include "cs_mesh_location.h"
 #include "cs_selector.h"
+#include "cs_parameters.h"
 #include "cs_physical_model.h"
 #include "cs_post.h"
 #include "cs_field.h"
 #include "cs_field_pointer.h"
-#include "cs_parameters.h"
+#include "cs_prototypes.h"
 #include "cs_thermal_model.h"
 #include "cs_time_moment.h"
+#include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -97,9 +98,11 @@ BEGIN_C_DECLS
  *============================================================================*/
 
 /*============================================================================
- * Static global variables
+ * Static local variables
  *============================================================================*/
 
+static char _rij_c_names[6][4] = {"r11", "r22", "r33", "r12", "r23", "r13"};
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -154,7 +157,7 @@ _field_post(const char  *field_type,
   const int k_lbl = cs_field_key_id("label");
   const int k_post = cs_field_key_id("post_vis");
 
-  cs_tree_node_t *tn =_get_node(field_type, f->name);
+  cs_tree_node_t *tn = _get_node(field_type, f->name);
 
   if (tn == NULL)
     return;
@@ -229,44 +232,388 @@ _surfacic_variable_post(const char  *name,
   return active;
 }
 
-/*-----------------------------------------------------------------------------
- * Initialize mei tree and check for symbols existence.
+/*----------------------------------------------------------------------------
+ * Return a string value associated with a child node for a profile.
+ *
+ * If the matching child node is not present, an error is produced
  *
  * parameters:
- *   formula        <--  mei formula
- *   nt_cur         <--  current time step
- *   t_cur          <--  current time value
+ *   tn <-- tree node associated with profile
  *
  * return:
- *   MEI tree object
+ *   pointer to matching child string
  *----------------------------------------------------------------------------*/
 
-static mei_tree_t *
-_init_mei_tree(const char      *formula,
-               const int        nt_cur,
-               const cs_real_t  t_cur)
+static const char *
+_get_profile_child_str(cs_tree_node_t  *tn,
+                       const char      *child_name)
 {
-  /* return an empty interpreter */
+  const char *name = cs_tree_node_get_child_value_str(tn, child_name);
 
-  mei_tree_t *tree = mei_tree_new(formula);
-
-  /* add commun variables */
-  mei_tree_insert(tree, "niter", nt_cur );
-  mei_tree_insert(tree, "t", t_cur);
-
-  /* add variable from notebook */
-  cs_gui_add_notebook_variables(tree);
-
-  /* try to build the interpreter */
-  if (mei_tree_builder(tree))
+  if (name == NULL) {
+    cs_base_warn(__FILE__, __LINE__);
+    bft_printf(_("Incorrect setup tree definition for the following node:\n"));
+    cs_tree_dump(CS_LOG_DEFAULT, 2, tn);
     bft_error(__FILE__, __LINE__, 0,
-              _("Error: can not interpret expression: %s\n"), tree->string);
-  /* check for symbols */
-  if (mei_tree_find_symbol(tree, "iactive"))
-    bft_error(__FILE__, __LINE__, 0,
-              _("Error: can not find the required symbol: %s\n"), "iactive");
+              _("Missing child node: %s"), child_name);
+  }
 
-  return tree;
+  return name;
+}
+
+/*----------------------------------------------------------------------------
+ * Return an array of integers associated with a child node for a profile.
+ *
+ * If the matching child node is not present, an error is produced
+ *
+ * parameters:
+ *   tn <-- tree node associated with profile
+ *
+ * return:
+ *   pointer to matching child values
+ *----------------------------------------------------------------------------*/
+
+static const int *
+_get_profile_child_int(cs_tree_node_t  *tn,
+                       const char      *child_name)
+{
+  const int *v = cs_tree_node_get_child_values_int(tn, child_name);
+
+  if (v == NULL) {
+    cs_base_warn(__FILE__, __LINE__);
+    bft_printf(_("Incorrect setup tree definition for the following node:\n"));
+    cs_tree_dump(CS_LOG_DEFAULT, 2, tn);
+    bft_error(__FILE__, __LINE__, 0,
+              _("Missing child node: %s"), child_name);
+  }
+
+  return v;
+}
+
+/*----------------------------------------------------------------------------
+ * Get output format for 1D profile
+ *
+ * parameters:
+ *   tn <-- tree node associated with profile
+ *
+ * return:
+ *   1 for CSV, 0 for DAT
+ *----------------------------------------------------------------------------*/
+
+static int
+_get_profile_format(cs_tree_node_t  *tn)
+{
+  int format = 0;
+
+  const char *format_s
+    = cs_tree_node_get_tag(cs_tree_node_get_child(tn, "format"),
+                           "name");
+
+  if (format_s != NULL) {
+    if (cs_gui_strcmp(format_s, "CSV"))
+      format = 1;
+    else if (cs_gui_strcmp(format_s, "DAT"))
+      format = 0;
+    else
+      bft_error(__FILE__, __LINE__, 0,
+                _("Invalid profile format: %s"), format_s);
+  }
+
+  return format;
+}
+
+/*----------------------------------------------------------------------------
+ * Return the component of variables or properties or scalar for 1D profile
+ *
+ * parameters:
+ *   tn <-- tree node associated with profile variable
+ *----------------------------------------------------------------------------*/
+
+static int
+_get_profile_v_component(cs_tree_node_t  *tn)
+{
+  int comp_id = -1;
+
+  const char *c_name = cs_tree_node_get_tag(tn, "component");
+
+  if (c_name != NULL) {
+    int n = sscanf(c_name, "%d", &comp_id);
+    if (n != 1)
+      bft_error(__FILE__, __LINE__, 0,
+                _("Error converting profile component tag %s to integer."),
+                c_name);
+  }
+
+  return comp_id;
+}
+
+/*----------------------------------------------------------------------------
+ * Return associated field for a node referring to a field.
+ *
+ * A node referring to a field must have a "name" tag (i.e. child with a
+ * string value).
+ *
+ * In the case of NEPTUNE_CFD, an additional "field_id" tag requiring
+ * addition of the "_<field_id>" extension to match the field's name
+ * may be present, so this is tested also.
+ *
+ * If the category and associated name are defined and a field matching
+ * a tag is not present, a message is logged. If those are not defined
+ * and a matching field is not found, an error is produced.
+ *
+ * parameters:
+ *   tn            <-- tree node associated with profile variable
+ *   category      <-- category for which this function is used, or NULL
+ *   category_name <-- name of object in category
+ *
+ * return:
+ *   pointer to field if match, NULL otherwise
+ *----------------------------------------------------------------------------*/
+
+static const cs_field_t *
+_tree_node_get_field(cs_tree_node_t  *tn,
+                     const char      *category,
+                     const char      *category_name)
+{
+  const cs_field_t *f = NULL;
+
+  const char *name = cs_gui_node_get_tag(tn, "name");
+  const char *id_name = cs_tree_node_get_tag(tn, "field_id");
+
+  /* Handle phase id (field_id tag in xml) for NEPTUNE_CFD */
+
+  if (id_name != NULL) {
+    if (strcmp(id_name, "none") != 0) {
+      char buffer[128];
+      snprintf(buffer, 127, "%s_%s", name, id_name);
+      buffer[127] = '\0';
+      if (strlen(buffer) >= 127)
+        bft_error(__FILE__, __LINE__, 0,
+                  "Local buffer too small to assemble field name with:\n"
+                  "name: %s\n"
+                  "field_id: %s\n", name, id_name);
+      f = cs_field_by_name_try(buffer);
+    }
+  }
+
+  if (f == NULL) {
+
+    /* Fix time step output */
+    if (strcmp(name, "local_time_step") == 0)
+      f = CS_F_(dt);
+
+    /* General case */
+    else
+      f = cs_field_by_name_try(name);
+
+    /* Handle segregated Reynolds tensor solver */
+    if (f == NULL) {
+      if (strcmp(name, "rij") == 0) {
+        int idim = _get_profile_v_component(tn);
+        f = cs_field_by_name_try(_rij_c_names[idim]);
+      }
+    }
+
+  }
+
+  if (f == NULL) {
+    if (category != NULL && category_name != NULL)
+      bft_printf(_("  For %s \"%s\", field with name \"%s\" not found\n"),
+                 category, category_name,  name);
+    else
+      bft_error(__FILE__, __LINE__, 0,
+                _("Field with name \"%s\" not found"), name);
+  }
+
+  return f;
+}
+
+/*----------------------------------------------------------------------------
+ * Profile definitions
+ *----------------------------------------------------------------------------*/
+
+static void
+_define_profiles(void)
+{
+  /* Loop on 1D profile definitions */
+
+  int profile_id = 0;
+
+  const char path0[] = "analysis_control/profiles/profile";
+
+  int n_writers = 0;
+
+  int        *w_i_vals = NULL;
+  cs_real_t  *w_r_vals = NULL;
+
+  int writer_id_start = cs_post_get_free_writer_id();
+
+  const char *format_name[] = {"dat", "csv"};
+
+  for (cs_tree_node_t *tn = cs_tree_get_node(cs_glob_tree, path0);
+       tn != NULL;
+       tn = cs_tree_node_get_next_of_name(tn), profile_id++) {
+
+    const char *name = cs_gui_node_get_tag(tn, "label");
+
+    /* for each profile, check the output frequency */
+
+    int writer_id = 0;
+    int output_format = _get_profile_format(tn);
+    int output_at_end = 0;
+    int output_frequency = -1;
+    cs_real_t time_output = -1.;
+
+    const char *output_type = _get_profile_child_str(tn, "output_type");
+    if (cs_gui_strcmp(output_type, "time_value")) {
+      const cs_real_t *v
+        = cs_tree_node_get_child_values_real(tn, "output_frequency");
+      if (v != NULL)
+        time_output = v[0];
+    }
+    else if (cs_gui_strcmp(output_type, "frequency")) {
+      const int *v
+        = cs_tree_node_get_child_values_int(tn, "output_frequency");
+      if (v != NULL)
+        output_frequency = v[0];
+      else
+        output_frequency = 1;
+      output_at_end = true;  /* Debatable, but consistent with v6.0 behavior */
+    }
+    else if (cs_gui_strcmp(output_type, "end")) {
+      output_at_end = true;
+    }
+
+    /* Check if the output frequency matches an existing writer id;
+       create a writer if necessary */
+
+    for (int i = 0; i < n_writers; i++) {
+      if (   w_i_vals[i*3]   == output_format
+          && w_i_vals[i*3+1] == output_at_end
+          && w_i_vals[i*3+2] == output_frequency
+          && fabs(w_r_vals[i] - time_output) < cs_math_epzero) {
+        writer_id = writer_id_start - i;
+        break;
+      }
+    }
+
+    if (writer_id == 0) {  /* Add new writer */
+
+      int i = n_writers;
+
+      n_writers += 1;
+      BFT_REALLOC(w_i_vals, n_writers*3, int);
+      BFT_REALLOC(w_r_vals, n_writers, cs_real_t);
+      w_i_vals[i*3] = output_format;
+      w_i_vals[i*3+1] = output_at_end;
+      w_i_vals[i*3+2] = output_frequency;
+      w_r_vals[i] = time_output;
+      writer_id = writer_id_start - i;
+
+      bool _output_at_end = (output_at_end) ? true : false;
+
+      char format_options[64];
+      strncpy(format_options, format_name[output_format], 63);
+      format_options[63] = '\0';
+      if (_output_at_end && output_frequency < 0 && time_output < 0) {
+        size_t l = strlen(format_options);
+        strncat(format_options, ", no_time_step", 63-l);
+        format_options[63] = '\0';
+      }
+
+      cs_post_define_writer(writer_id,
+                            "",
+                            "profiles",
+                            "plot",
+                            format_options,
+                            FVM_WRITER_FIXED_MESH,
+                            false, /* output_at_start */
+                            _output_at_end,
+                            output_frequency,
+                            time_output);
+
+    }
+
+    int n_coords = 0;
+    const int *v_np = _get_profile_child_int(tn, "points");
+    if (v_np != NULL)
+      n_coords = v_np[0];
+
+    cs_real_3_t *coords;
+    BFT_MALLOC(coords, n_coords, cs_real_3_t);
+
+    /* Be debugger-friendly in case cs_meg_post_profiles does not handle
+       this well (if generation is wrong or missing) */
+    for (cs_lnum_t i = 0; i < n_coords; i++) {
+      coords[i][0] = -1.e30;
+      coords[i][1] = -1.e30;
+      coords[i][2] = -1.e30;
+    }
+
+    cs_meg_post_profiles(name, n_coords, coords);
+
+    cs_probe_set_t *pset
+      = cs_probe_set_create_from_array(name,
+                                       n_coords,
+                                       coords,
+                                       NULL);
+
+    BFT_FREE(coords);
+
+    cs_probe_set_assign_curvilinear_abscissa(pset, NULL);
+
+    int writer_ids[1] = {writer_id};
+    cs_probe_set_associate_writers(pset, 1, writer_ids);
+
+    cs_probe_set_auto_var(pset, false);
+
+    /* Set snap mode. Default is "SNAP_TO_CENTER" */
+    const char *snap_mode = cs_tree_node_get_child_value_str(tn, "snap_mode");
+    if (cs_gui_strcmp(snap_mode, "snap_to_vertex"))
+      cs_probe_set_snap_mode(pset, CS_PROBE_SNAP_VERTEX);
+    else if (cs_gui_strcmp(snap_mode, "none"))
+      cs_probe_set_snap_mode(pset, CS_PROBE_SNAP_NONE);
+    else
+      cs_probe_set_snap_mode(pset, CS_PROBE_SNAP_ELT_CENTER);
+
+    /* Activate interpolation if needed. Default is no */
+    const char *activate_interpolation
+      = cs_tree_node_get_child_value_str(tn, "interpolation");
+    if (cs_gui_strcmp(activate_interpolation, "yes"))
+      cs_probe_set_option(pset, "interpolation", "1");
+
+    if (cs_glob_mesh->time_dep > CS_MESH_FIXED)
+      cs_probe_set_option(pset, "transient_location", "true");
+
+    /* Output coordinates */
+
+    cs_probe_set_auto_curvilinear_coords(pset, true);
+    cs_probe_set_auto_cartesian_coords(pset, true);
+
+    /* Associate fields */
+
+    for (cs_tree_node_t *tn_vp = cs_tree_node_get_child(tn, "var_prop");
+         tn_vp != NULL;
+         tn_vp = cs_tree_node_get_next_of_name(tn_vp)) {
+
+      const cs_field_t *f = _tree_node_get_field(tn_vp, "profile", name);
+
+      if (f == NULL)
+        continue;
+
+      int comp_id = _get_profile_v_component(tn_vp);
+
+      cs_probe_set_associate_field(pset,
+                                   writer_id,
+                                   f->id,
+                                   comp_id);
+
+    }
+
+  }
+
+  BFT_FREE(w_i_vals);
+  BFT_FREE(w_r_vals);
 }
 
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
@@ -279,11 +626,8 @@ _init_mei_tree(const char      *formula,
  * Determine output boundary fields
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (cspstb, CSPSTB) (cs_int_t        *ipstdv)
+void CS_PROCF (cspstb, CSPSTB) (int  *ipstdv)
 {
-  if (!cs_gui_file_is_loaded())
-    return;
-
   /* Surfacic variables output */
 
   for (int i = 0; i < 5; i++)
@@ -303,7 +647,18 @@ void CS_PROCF (cspstb, CSPSTB) (cs_int_t        *ipstdv)
       ipstdv[2] = 1;
     if (_surfacic_variable_post("thermal_flux", true))
       ipstdv[3] = 1;
-    if (_surfacic_variable_post("boundary_temperature", true)) {
+    bool post_b_temp = _surfacic_variable_post("boundary_temperature", true);
+    /* activate by default using GUI; ignore for non-temperature variable
+       when properties not present in GUI, or the thermal model is not
+       set in the GUI, as this implies the GUI was probably not used
+       and we cannot determine easily whether enthalpy to temperature
+       conversion is available */
+    if (cs_glob_thermal_model->itherm != CS_THERMAL_MODEL_TEMPERATURE) {
+      if (   cs_tree_find_node_simple(cs_glob_tree, "property") == NULL
+          || cs_gui_thermal_model() <= 0)
+        post_b_temp = false;
+    }
+    if (post_b_temp) {
       cs_field_t *bf = cs_parameters_add_boundary_temperature();
       if (bf != NULL) {
         int k_vis = cs_field_key_id("post_vis");
@@ -357,11 +712,15 @@ cs_gui_output(void)
     const cs_field_t  *f = cs_field_by_id(f_id);
     if (f->type & CS_FIELD_VARIABLE)
       _field_post("variable", f->id);
-    else if (f->type & CS_FIELD_PROPERTY)
+    else if (   (f->type & CS_FIELD_PROPERTY)
+             || (f->type & CS_FIELD_POSTPROCESS)) {
+      if (moment_id != NULL) {
+        if (moment_id[f_id] > -1) {
+          _field_post("time_average", f->id);
+          continue;
+        }
+      }
       _field_post("property", f->id);
-    else if (moment_id != NULL) {
-      if (moment_id[f_id] > -1)
-        _field_post("time_average", f->id);
     }
   }
 
@@ -444,8 +803,18 @@ cs_gui_postprocess_meshes(void)
                                   add_groups, auto_vars,
                                   n_writers, writer_ids);
     }
-    else if(cs_gui_strcmp(type, "boundary_faces")) {
-      cs_post_define_surface_mesh(id, label, NULL, location,
+    else if(cs_gui_strcmp(type, "VolumicZone")) {
+      const cs_zone_t *z = cs_volume_zone_by_name(location);
+      const char *criteria =
+        cs_mesh_location_get_selection_string(z->location_id);
+      cs_post_define_volume_mesh(id, label, criteria, add_groups, auto_vars,
+                                 n_writers, writer_ids);
+    }
+    else if(cs_gui_strcmp(type, "BoundaryZone")) {
+      const cs_zone_t *z = cs_boundary_zone_by_name(location);
+      const char *criteria =
+        cs_mesh_location_get_selection_string(z->location_id);
+      cs_post_define_surface_mesh(id, label, NULL, criteria,
                                   add_groups, auto_vars,
                                   n_writers, writer_ids);
     }
@@ -468,6 +837,11 @@ cs_gui_postprocess_meshes(void)
 
   if (n_probes > 0) {
 
+    char **probe_labels;
+    BFT_MALLOC(probe_labels, n_probes, char *);
+    for (int ii = 0; ii < n_probes; ii++)
+      BFT_MALLOC(probe_labels[ii], 128, char);
+
     const char *coord_node_name[] = {"probe_x", "probe_y", "probe_z"};
 
     cs_real_3_t *p_coords;
@@ -478,42 +852,57 @@ cs_gui_postprocess_meshes(void)
          tn != NULL;
          tn = cs_tree_node_get_next_of_name(tn), i++) {
 
+      /* Probe coordinates */
       for (int j = 0; j < 3; j++) {
         v_r = cs_tree_node_get_child_values_real(tn, coord_node_name[j]);
         p_coords[i][j] = (v_r != NULL) ? v_r[0] : 0;
       }
+
+      /* Probe name */
+      const char *pn = cs_tree_node_get_child_value_str(tn, "name");
+      strcpy(probe_labels[i], pn);
     }
 
-    cs_probe_set_create_from_array("probes",
-                                   n_probes,
-                                   (const cs_real_3_t *)p_coords,
-                                   NULL);
+    cs_probe_set_t *pset =
+      cs_probe_set_create_from_array("probes",
+                                     n_probes,
+                                     (const cs_real_3_t *)p_coords,
+                                     (const char **)probe_labels);
+
+    /* Set snap mode. Default is "SNAP_TO_CENTER" */
+    const char *snap_mode
+      = cs_tree_node_get_tag(cs_tree_node_get_child(tn_o, "probes_snap"),
+                             "choice");
+    if (cs_gui_strcmp(snap_mode, "snap_to_vertex"))
+      cs_probe_set_snap_mode(pset, CS_PROBE_SNAP_VERTEX);
+    else if (cs_gui_strcmp(snap_mode, "none"))
+      cs_probe_set_snap_mode(pset, CS_PROBE_SNAP_NONE);
+    else
+      cs_probe_set_snap_mode(pset, CS_PROBE_SNAP_ELT_CENTER);
+
+    /* Activate interpolation if needed. Default is no */
+    const char *activate_interpolation
+      = cs_tree_node_get_tag(cs_tree_node_get_child(tn_o,
+                                                    "probes_interpolation"),
+                             "choice");
+    if (cs_gui_strcmp(activate_interpolation, "yes"))
+      cs_probe_set_option(pset, "interpolation", "1");
+
 
     BFT_FREE(p_coords);
 
-    v_i = cs_tree_node_get_child_values_int(tn_o, "probe_recording_frequency");
-    int frequency_n = (v_i != NULL) ? v_i[0] : 1;
+    for (int ii = 0; ii < n_probes; ii++)
+      BFT_FREE(probe_labels[ii]);
+    BFT_FREE(probe_labels);
 
-    v_r = cs_tree_node_get_child_values_real
-           (tn_o, "probe_recording_frequency_time");
-    cs_real_t frequency_t = (v_r != NULL) ? v_r[0] : -1.;
-
-    /* Time plot (probe) format string */
-    const char *fmt_opts
-      = cs_tree_node_get_tag(cs_tree_node_get_child(tn_o, "probe_format"),
-                             "choice");
-
-    cs_post_define_writer(CS_POST_WRITER_PROBES,   /* writer_id */
-                          "",                      /* case_name */
-                          "monitoring",            /* dir_name */
-                          "time_plot",
-                          fmt_opts,
-                          FVM_WRITER_FIXED_MESH,
-                          false,                   /* output_at_start */
-                          false,                   /* output_at_end */
-                          frequency_n,
-                          frequency_t);
   }
+
+  /* Profile definitions;
+     note that this may lead to additional writer definitions, as
+     the GUI does not yet present profiles in a consistent
+     writer/mesh logic (TODO) */
+
+  _define_profiles();
 }
 
 /*----------------------------------------------------------------------------
@@ -610,46 +999,33 @@ cs_gui_postprocess_writers(void)
                           time_step,
                           time_value);
   }
-}
 
-/*----------------------------------------------------------------------------
- * Activate writers depending on a formula
- *----------------------------------------------------------------------------*/
+  /* Probes default writer */
 
-void
-cs_gui_postprocess_activate(void)
-{
-  const char path_o[] = "analysis_control/output";
-  cs_tree_node_t *tn_o = cs_tree_get_node(cs_glob_tree, path_o);
+  const int *v_i
+    = cs_tree_node_get_child_values_int(tn_o, "probe_recording_frequency");
+  int frequency_n = (v_i != NULL) ? v_i[0] : 1;
 
-  for (cs_tree_node_t *tn = cs_tree_get_node(tn_o, "writer");
-       tn != NULL;
-       tn = cs_tree_node_get_next_of_name(tn)) {
+  const cs_real_t *v_r
+    = cs_tree_node_get_child_values_real(tn_o, "probe_recording_frequency_time");
+  cs_real_t frequency_t = (v_r != NULL) ? v_r[0] : -1.;
 
-    const int *v_i = cs_tree_node_get_child_values_int(tn, "id");
-    if (v_i == NULL) continue;
+  /* Time plot (probe) format string */
+  const char *fmt_opts
+    = cs_tree_node_get_tag(cs_tree_node_get_child(tn_o, "probe_format"),
+                           "choice");
 
-    int id = v_i[0];
+  cs_post_define_writer(CS_POST_WRITER_PROBES,   /* writer_id */
+                        "",                      /* case_name */
+                        "monitoring",            /* dir_name */
+                        "time_plot",
+                        fmt_opts,
+                        FVM_WRITER_FIXED_MESH,
+                        false,                   /* output_at_start */
+                        false,                   /* output_at_end */
+                        frequency_n,
+                        frequency_t);
 
-    const char *frequency_choice
-      = cs_tree_node_get_tag(cs_tree_node_get_child(tn, "frequency"),
-                             "perio");
-
-    if (cs_gui_strcmp(frequency_choice, "formula")) {
-      const char *formula = cs_tree_node_get_child_value_str(tn, "frequency");
-      assert(formula != NULL);
-      const cs_time_step_t *ts = cs_glob_time_step;
-      mei_tree_t *ev_formula = _init_mei_tree(formula, ts->nt_cur, ts->t_cur);
-      mei_evaluate(ev_formula);
-      int iactive =  mei_tree_lookup(ev_formula, "iactive");
-      mei_tree_destroy(ev_formula);
-      if (iactive == 1)
-        cs_post_activate_writer(id, true);
-      else
-        cs_post_activate_writer(id, false);
-    }
-
-  }
 }
 
 /*----------------------------------------------------------------------------*/
