@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -49,7 +49,7 @@ BEGIN_C_DECLS
  * Determine output boundary fields
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (cspstb, CSPSTB) (cs_int_t        *ipstdv);
+void CS_PROCF (cspstb, CSPSTB) (int  *ipstdv);
 
 /*============================================================================
  * Public function prototypes
@@ -76,13 +76,6 @@ cs_gui_postprocess_meshes(void);
 void
 cs_gui_postprocess_writers(void);
 
-/*----------------------------------------------------------------------------
- * Activate writers depending on a formula
- *----------------------------------------------------------------------------*/
-
-void
-cs_gui_postprocess_activate(void);
-
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
