@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -47,13 +47,14 @@
 #include "cs_gui.h"
 #include "cs_gui_util.h"
 #include "cs_gui_boundary_conditions.h"
-#include "cs_prototypes.h"
 
 #include "cs_lagr.h"
 #include "cs_lagr_post.h"
 #include "cs_lagr_stat.h"
 #include "cs_lagr_tracking.h"
 
+#include "cs_parameters.h"
+
 /*----------------------------------------------------------------------------
  * Header for the current file
  *----------------------------------------------------------------------------*/
@@ -148,18 +149,14 @@ _attr_post_status(cs_tree_node_t       *tn_o,
 }
 
 /*-----------------------------------------------------------------------------
- * Activate statistics and postprocessing for volume or boundary statistics
+ * Activate statistics
  *
  * parameters:
  *   tn_s     <--  parent tree node ("volume" or "boundary")
- *   name     <--  name of statistics in tree
- *   status   <->  associated postprocessing status
  *----------------------------------------------------------------------------*/
 
 static void
-_get_stats_post(cs_tree_node_t  *tn_s,
-                const char      *name,
-                int             *status)
+_get_stats_post(cs_tree_node_t  *tn_s)
 {
   cs_tree_node_t *tn = NULL;
 
@@ -168,12 +165,14 @@ _get_stats_post(cs_tree_node_t  *tn_s,
        tn = cs_tree_node_get_next_of_name(tn)) {
     const char *_name = cs_tree_node_get_tag(tn, "name");
     if (_name != NULL) {
-      if (! strcmp(_name, name)) {
-        *status = 1; /* default if active */
-        cs_gui_node_get_status_int
-          (cs_tree_node_get_child(tn, "postprocessing_recording"),
-           status);
-        break;
+      int stat_type = cs_lagr_stat_type_by_name(_name);
+      if (stat_type > -1) {
+        int status = 1; /* default if active */
+        cs_gui_node_get_status_int(tn, &status);
+        if (status < 1)
+          cs_lagr_stat_deactivate(stat_type);
+        else
+          cs_lagr_stat_activate(stat_type);
       }
     }
   }
@@ -196,14 +195,14 @@ cs_gui_particles_model(void)
 
   const char *model = cs_tree_node_get_tag(tn_lagr, "model");
 
-  cs_glob_lagr_time_scheme->iilagr = 0;
+  cs_glob_lagr_time_scheme->iilagr = CS_LAGR_OFF;
   if (model != NULL) {
     if (! strcmp(model, "one_way"))
-      cs_glob_lagr_time_scheme->iilagr = 1;
+      cs_glob_lagr_time_scheme->iilagr = CS_LAGR_ONEWAY_COUPLING;
     else if (! strcmp(model, "two_way"))
-      cs_glob_lagr_time_scheme->iilagr = 2;
+      cs_glob_lagr_time_scheme->iilagr = CS_LAGR_TWOWAY_COUPLING;
     else if (! strcmp(model, "frozen"))
-      cs_glob_lagr_time_scheme->iilagr = 3;
+      cs_glob_lagr_time_scheme->iilagr = CS_LAGR_FROZEN_CONTINUOUS_PHASE;
   }
 
 #if _XML_DEBUG_
@@ -211,7 +210,7 @@ cs_gui_particles_model(void)
   bft_printf("--iilagr = %i\n", cs_glob_lagr_time_scheme->iilagr);
 #endif
 
-  if (cs_glob_lagr_time_scheme->iilagr == 0)
+  if (cs_glob_lagr_time_scheme->iilagr == CS_LAGR_OFF)
     return;
 
   /* Global settings */
@@ -234,7 +233,7 @@ cs_gui_particles_model(void)
   cs_glob_lagr_model->physical_model = _get_particles_model(tn_pm);
 
   switch (cs_glob_lagr_model->physical_model) {
-  case 1:
+  case CS_LAGR_PHYS_HEAT:
     {
       cs_gui_node_get_status_int(cs_tree_node_get_child(tn_pm, "break_up"),
                                  &(cs_glob_lagr_specific_physics->idpvar));
@@ -244,7 +243,7 @@ cs_gui_particles_model(void)
                                  &(cs_glob_lagr_specific_physics->itpvar));
     }
     break;
-  case 2:
+  case CS_LAGR_PHYS_COAL:
     {
       cs_tree_node_t *tn_cf = cs_tree_node_get_child(tn_pm, "coal_fouling");
 
@@ -283,7 +282,7 @@ cs_gui_particles_model(void)
 
   /* Two-way coupling */
 
-  if (cs_glob_lagr_time_scheme->iilagr == 2) {
+  if (cs_glob_lagr_time_scheme->iilagr == CS_LAGR_TWOWAY_COUPLING) {
     cs_tree_node_t *tn = cs_tree_node_get_child(tn_lagr, "two_way_coupling");
 
     cs_gui_node_get_child_int(tn, "iteration_start",
@@ -308,26 +307,16 @@ cs_gui_particles_model(void)
   if (choice != NULL)
     cs_glob_lagr_time_scheme->t_order = atoi(choice);
 
-  cs_gui_node_get_status_int
-    (cs_tree_node_get_child(tn_lagr, "turbulent_dispersion"),
-     &(cs_glob_lagr_time_scheme->idistu));
-
   cs_gui_node_get_status_int
     (cs_tree_node_get_child(tn_lagr, "fluid_particles_turbulent_diffusion"),
-     &(cs_glob_lagr_time_scheme->idiffl));
+     &(cs_glob_lagr_model->idiffl));
 
   cs_gui_node_get_status_int(cs_tree_node_get_child(tn_lagr,
                                                     "deposition_submodel"),
                              &(cs_glob_lagr_model->deposition));
 
-  cs_gui_node_get_child_int(tn_lagr, "complete_model",
-                            (&cs_glob_lagr_time_scheme->modcpl));
-
-  choice = cs_tree_node_get_tag(cs_tree_node_get_child
-                                  (tn_lagr, "complete_model_direction"),
-                                "choice");
-  if (choice != NULL)
-    cs_glob_lagr_time_scheme->idirla = atoi(choice);
+  cs_gui_node_get_child_int(tn_lagr, "regular_particles",
+                            (&cs_glob_lagr_model->modcpl));
 
   /* Output */
 
@@ -339,8 +328,10 @@ cs_gui_particles_model(void)
     _attr_post_status(tn_o, CS_LAGR_DIAMETER, "diameter");
     _attr_post_status(tn_o, CS_LAGR_TEMPERATURE, "temperature");
     _attr_post_status(tn_o, CS_LAGR_MASS, "mass");
+    _attr_post_status(tn_o, CS_LAGR_AGGLO_CLASS_ID, "parcel_class");
+    _attr_post_status(tn_o, CS_LAGR_STAT_WEIGHT, "stat_weight");
 
-    if (cs_glob_lagr_model->physical_model == 2) {
+    if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
       _attr_post_status(tn_o, CS_LAGR_SHRINKING_DIAMETER,
                         "shrinking_core_diameter");
       _attr_post_status(tn_o, CS_LAGR_WATER_MASS, "moisture_mass_fraction");
@@ -365,6 +356,7 @@ cs_gui_particles_model(void)
 
     cs_gui_node_get_child_int(tn_s, "iteration_start",
                               &cs_glob_lagr_stat_options->idstnt);
+
     cs_gui_node_get_child_int(tn_s, "iteration_steady_start",
                               &cs_glob_lagr_stat_options->nstist);
 
@@ -381,26 +373,31 @@ cs_gui_particles_model(void)
 
     if (volume_stats) {
 
-      int flag = 0;
-      _get_stats_post(tn_vs, "Part_vol_frac", &flag);
-      if (flag)
-        cs_lagr_stat_activate(CS_LAGR_STAT_VOLUME_FRACTION);
+      /* Default statistics */
 
-      flag = 0;
-      _get_stats_post(tn_vs, "Part_velocity", &flag);
-      if (flag)
-        cs_lagr_stat_activate_attr(CS_LAGR_VELOCITY);
+      cs_lagr_stat_activate(CS_LAGR_STAT_CUMULATIVE_WEIGHT);
+      cs_lagr_stat_activate(CS_LAGR_STAT_VOLUME_FRACTION);
+      cs_lagr_stat_activate_attr(CS_LAGR_RESIDENCE_TIME);
+      cs_lagr_stat_activate_attr(CS_LAGR_DIAMETER);
+      cs_lagr_stat_activate_attr(CS_LAGR_MASS);
+      cs_lagr_stat_activate_attr(CS_LAGR_VELOCITY);
 
-      flag = 0;
-      _get_stats_post(tn_vs, "Part_resid_time", &flag);
-      if (flag)
-        cs_lagr_stat_activate_attr(CS_LAGR_RESIDENCE_TIME);
-
-      flag = 0;
-      _get_stats_post(tn_vs, "Part_stat_weight", &flag);
-      if (flag)
-        cs_lagr_stat_activate(CS_LAGR_STAT_CUMULATIVE_WEIGHT);
+      switch(cs_glob_lagr_model->physical_model) {
+      case CS_LAGR_PHYS_HEAT:
+        cs_lagr_stat_activate_attr(CS_LAGR_TEMPERATURE);
+        break;
+      case CS_LAGR_PHYS_COAL:
+        cs_lagr_stat_activate_attr(CS_LAGR_WATER_MASS);
+        cs_lagr_stat_activate_attr(CS_LAGR_TEMPERATURE);
+        cs_lagr_stat_activate_attr(CS_LAGR_COAL_MASS);
+        cs_lagr_stat_activate_attr(CS_LAGR_COKE_MASS);
+        cs_lagr_stat_activate_attr(CS_LAGR_COAL_DENSITY);
+      default:
+        break;
+      }
 
+      /* XML-defined */
+      _get_stats_post(tn_vs);
     }
 
     cs_tree_node_t *tn_bs = cs_tree_node_get_child(tn_s, "boundary");
@@ -409,28 +406,33 @@ cs_gui_particles_model(void)
 
     if (boundary_stats) {
 
-      _get_stats_post(tn_bs, "Part_impact_number",
-                      &cs_glob_lagr_boundary_interactions->has_part_impact_nbr);
-      _get_stats_post(tn_bs, "Part_bndy_mass_flux",
-                      &cs_glob_lagr_boundary_interactions->iflmbd);
-      _get_stats_post(tn_bs, "Part_impact_angle",
-                      &cs_glob_lagr_boundary_interactions->iangbd);
-      _get_stats_post(tn_bs, "Part_impact_velocity",
-                      &cs_glob_lagr_boundary_interactions->ivitbd);
+      /* Default statistics */
 
-      /* Coal fouling statistics*/
-      _get_stats_post(tn_bs, "Part_fouled_impact_number",
-                      &cs_glob_lagr_boundary_interactions->iencnbbd);
-      _get_stats_post(tn_bs, "Part_fouled_mass_flux",
-                      &cs_glob_lagr_boundary_interactions->iencmabd);
-      _get_stats_post(tn_bs, "Part_fouled_diam",
-                      &cs_glob_lagr_boundary_interactions->iencdibd);
-      _get_stats_post(tn_bs, "Part_fouled_Xck",
-                      &cs_glob_lagr_boundary_interactions->iencckbd);
+      cs_lagr_stat_activate(CS_LAGR_STAT_E_CUMULATIVE_WEIGHT);
+      cs_lagr_stat_activate(CS_LAGR_STAT_MASS_FLUX);
+      cs_lagr_stat_activate(CS_LAGR_STAT_IMPACT_ANGLE);
+      cs_lagr_stat_activate(CS_LAGR_STAT_IMPACT_VELOCITY);
 
+      if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
+        cs_lagr_stat_activate(CS_LAGR_STAT_FOULING_CUMULATIVE_WEIGHT);
+        cs_lagr_stat_activate(CS_LAGR_STAT_FOULING_MASS_FLUX);
+        cs_lagr_stat_activate(CS_LAGR_STAT_FOULING_DIAMETER);
+        cs_lagr_stat_activate(CS_LAGR_STAT_FOULING_COKE_FRACTION);
+      }
+
+      _get_stats_post(tn_bs);
     }
+
   }
 
+  /* When activating the complete turbulent dispersion model,
+   * statistics are required, so activate it after the start time of
+   * statistics.
+   */
+  if (cs_glob_lagr_model->modcpl > 0)
+    cs_glob_lagr_model->modcpl
+      = CS_MAX(cs_glob_lagr_model->modcpl, cs_glob_lagr_stat_options->idstnt);
+
 #if _XML_DEBUG_
   bft_printf("==> %s\n", __func__);
   bft_printf("--iilagr = %i\n", cs_glob_lagr_time_scheme->iilagr);
@@ -439,14 +441,14 @@ cs_gui_particles_model(void)
   bft_printf("--idepst = %i\n", cs_glob_lagr_model->deposition);
   bft_printf("--iphyla = %i\n", cs_glob_lagr_model->physical_model);
   switch(cs_glob_lagr_model->physical_model) {
-  case 0:
+  case CS_LAGR_PHYS_OFF:
     break;
-  case 1:
+  case CS_LAGR_PHYS_HEAT:
     bft_printf("--idpvar = %i\n", cs_glob_lagr_specific_physics->idpvar);
     bft_printf("--impvar = %i\n", cs_glob_lagr_specific_physics->impvar);
     bft_printf("--itpvar = %i\n", cs_glob_lagr_specific_physics->itpvar);
     break;
-  case 2:
+  case CS_LAGR_PHYS_COAL:
     bft_printf("--iencra = %i\n", cs_glob_lagr_model->fouling);
     const cs_lagr_extra_module_t *extra = cs_get_lagr_extra_module();
     for (int icoal = 0; icoal < extra->ncharb; icoal++) {
@@ -462,7 +464,7 @@ cs_gui_particles_model(void)
     break;
   }
 
-  if (cs_glob_lagr_time_scheme->iilagr == 2) {
+  if (cs_glob_lagr_time_scheme->iilagr == CS_LAGR_TWOWAY_COUPLING) {
     bft_printf("--nstits = %i\n", cs_glob_lagr_source_terms->nstits);
     bft_printf("--ltsdyn = %i\n", cs_glob_lagr_source_terms->ltsdyn);
     bft_printf("--ltsmas = %i\n", cs_glob_lagr_source_terms->ltsmas);
@@ -470,10 +472,9 @@ cs_gui_particles_model(void)
   }
 
   bft_printf("--nordre = %i\n", cs_glob_lagr_time_scheme->t_order);
-  bft_printf("--idistu = %i\n", cs_glob_lagr_time_scheme->idistu);
-  bft_printf("--idiffl = %i\n", cs_glob_lagr_time_scheme->idiffl);
-  bft_printf("--modcpl = %i\n", cs_glob_lagr_time_scheme->modcpl);
-  bft_printf("--idirla = %i\n", cs_glob_lagr_time_scheme->idirla);
+  bft_printf("--idistu = %i\n", cs_glob_lagr_model->idistu);
+  bft_printf("--idiffl = %i\n", cs_glob_lagr_model->idiffl);
+  bft_printf("--modcpl = %i\n", cs_glob_lagr_model->modcpl);
 
   bft_printf("--isuist = %i\n", cs_glob_lagr_stat_options->isuist);
   bft_printf("--nbclst = %i\n", cs_glob_lagr_model->n_stat_classes);
@@ -488,12 +489,7 @@ cs_gui_particles_model(void)
   if (b_stats) {
     bft_printf("--has_part_impact_nbr   = %i\n",
                cs_glob_lagr_boundary_interactions->has_part_impact_nbr);
-    bft_printf("--iflmbd   = %i\n", cs_glob_lagr_boundary_interactions->iflmbd);
-    bft_printf("--iangbd   = %i\n", cs_glob_lagr_boundary_interactions->iangbd);
-    bft_printf("--ivitbd   = %i\n", cs_glob_lagr_boundary_interactions->ivitbd);
     bft_printf("--iencnbbd = %i\n", cs_glob_lagr_boundary_interactions->iencnbbd);
-    bft_printf("--iencmabd = %i\n", cs_glob_lagr_boundary_interactions->iencmabd);
-    bft_printf("--iencdibd = %i\n", cs_glob_lagr_boundary_interactions->iencdibd);
     bft_printf("--iencckbd = %i\n", cs_glob_lagr_boundary_interactions->iencckbd);
   }
 
@@ -572,11 +568,11 @@ cs_gui_particles_bcs(void)
     else if (! strcmp(interaction, "deposit2"))
       bdy_cond->zone_type[zone_id] = CS_LAGR_DEPO2;
 
-    else if (! strcmp(interaction, "fouling") && iphyla == 2)
+    else if (! strcmp(interaction, "fouling") && iphyla == CS_LAGR_PHYS_COAL)
       bdy_cond->zone_type[zone_id] = CS_LAGR_FOULING;
 
     else if (   ! strcmp(interaction, "fouling")
-             && (iphyla == 0  || iphyla == 1))
+             && (iphyla == CS_LAGR_PHYS_OFF  || iphyla == CS_LAGR_PHYS_HEAT))
       bdy_cond->zone_type[zone_id] = CS_LAGR_DEPO_DLVO;
 
 #if _XML_DEBUG_
@@ -631,35 +627,33 @@ cs_gui_particles_bcs(void)
 
           choice = cs_tree_node_get_tag(tn_c, "choice");
 
-          if (cs_gui_strcmp(choice, "fluid"))
-            zis->velocity_profile = -1;
+          if (choice == NULL || cs_gui_strcmp(choice, "fluid"))
+            zis->velocity_profile = CS_LAGR_IN_IMPOSED_FLUID_VALUE;
 
           else if (cs_gui_strcmp(choice, "norm")) {
-            zis->velocity_profile = 0;
+            zis->velocity_profile = CS_LAGR_IN_IMPOSED_NORM;
 
             v_r = cs_tree_node_get_child_values_real(tn_c, "norm");
             if (v_r != NULL) zis->velocity_magnitude = v_r[0];
           }
           else if (cs_gui_strcmp(choice, "components")) {
             const char *cname[] = {"velocity_x", "velocity_y", "velocity_z"};
-            zis->velocity_profile = 1;
+            zis->velocity_profile = CS_LAGR_IN_IMPOSED_COMPONENTS;
             for (int i = 0; i < 3; i++) {
               v_r = cs_tree_node_get_child_values_real(tn_c, cname[i]);
               if (v_r != NULL) zis->velocity[i] = v_r[0];
             }
           }
-          else if (cs_gui_strcmp(choice, "subroutine"))
-            zis->velocity_profile = 2;
 
 #if _XML_DEBUG_
           bft_printf("---velocity choice: %i "
                      " (-1: fluid, 0: norm, 1: components, 2: subroutine)\n",
                      zis->velocity_profile);
 
-          if (zis->velocity_profile == 0)
+          if (zis->velocity_profile == CS_LAGR_IN_IMPOSED_NORM)
             bft_printf("----norm = %f \n", zis->velocity_magnitude);
 
-          else if (zis->velocity_profile == 1) {
+          else if (zis->velocity_profile == CS_LAGR_IN_IMPOSED_COMPONENTS) {
             bft_printf("----u = %f \n", zis->velocity[0]);
             bft_printf("----v = %f \n", zis->velocity[1]);
             bft_printf("----w = %f \n", zis->velocity[2]);
@@ -679,7 +673,7 @@ cs_gui_particles_bcs(void)
             zis->stat_weight = 0;
             if (v_r != NULL) zis->flow_rate = v_r[0];
           }
-          else if (cs_gui_strcmp(choice, "prescribed")) {
+          else if (cs_gui_strcmp(choice, "prescribed") || choice == NULL) {
             v_r = cs_tree_node_get_values_real(tn_c);
             if (v_r != NULL) zis->stat_weight = v_r[0];
             zis->flow_rate = 0;
@@ -709,7 +703,7 @@ cs_gui_particles_bcs(void)
 #endif
 
         /* density */
-        if (iphyla != 2) {
+        if (iphyla != CS_LAGR_PHYS_COAL) {
           v_r = cs_tree_node_get_child_values_real(tn_i, "density");
           if (v_r != NULL) zis->density = v_r[0];
 
@@ -726,7 +720,7 @@ cs_gui_particles_bcs(void)
           bft_printf("---fouling_index = %f \n", zis->fouling_index);
 #endif
 
-        if (iphyla == 1) {
+        if (iphyla == CS_LAGR_PHYS_HEAT) {
 
           /* temperature, specific_heat, emissivity */
 
@@ -740,7 +734,7 @@ cs_gui_particles_bcs(void)
               v_r = cs_tree_node_get_values_real(tn_c);
               if (v_r != NULL) zis->temperature = v_r[0];
             }
-            else if (cs_gui_strcmp(choice, "fluid")) {
+            else if (cs_gui_strcmp(choice, "fluid") || choice == NULL) {
               zis->temperature_profile = 0;
               zis->temperature = 0;
             }
@@ -766,7 +760,7 @@ cs_gui_particles_bcs(void)
         }
 
         /* coal */
-        else if (iphyla == 2) {
+        else if (iphyla == CS_LAGR_PHYS_COAL) {
 
           /* Read the coal number */
           v_i = cs_tree_node_get_child_values_int(tn_i, "coal_number");
