@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -50,7 +50,6 @@
 
 #include "cs_base.h"
 #include "cs_boundary_zone.h"
-#include "cs_gui_variables.h"
 #include "cs_gui_util.h"
 #include "cs_gui_boundary_conditions.h"
 #include "cs_gui_specific_physics.h"
@@ -58,6 +57,7 @@
 #include "cs_mesh.h"
 #include "cs_post.h"
 #include "cs_parameters.h"
+#include "cs_physical_model.h"
 #include "cs_restart.h"
 #include "cs_field.h"
 #include "cs_field_pointer.h"
@@ -142,7 +142,7 @@ _radiative_transfer_char_post(cs_tree_node_t  *tn_rt,
   cs_gui_node_get_child_status_int(tn, "listing_printing", f_log);
   cs_gui_node_get_child_status_int(tn, "postprocessing_recording", f_post_vis);
 
-  if (*f_post_vis == -1) /* align with defaults in GUI */
+  if (tn != NULL && *f_post_vis == -1) /* align with defaults in GUI */
     *f_post_vis = 1;
 
   return label;
@@ -163,7 +163,7 @@ _radiative_transfer_type(cs_tree_node_t  *tn_rt,
                          int             *keyword)
 {
   cs_tree_node_t *tn = cs_tree_get_node(tn_rt, param);
-  const char *type = cs_gui_node_get_tag(tn, "type");
+  const char *type = (tn != NULL) ? cs_gui_node_get_tag(tn, "type") : NULL;
 
   if (type != NULL) {
     if (cs_gui_strcmp(type, "constant"))
@@ -199,15 +199,15 @@ _radiative_boundary_type(cs_tree_node_t  *tn_bc)
   const char *type = cs_tree_node_get_child_value_str(tn_bc, "choice");
 
   if (cs_gui_strcmp(type, "itpimp"))
-    result = cs_glob_rad_transfer_params->itpimp;
+    result = CS_BOUNDARY_RAD_WALL_GRAY;
   else if (cs_gui_strcmp(type, "ipgrno"))
-    result = cs_glob_rad_transfer_params->ipgrno;
+    result = CS_BOUNDARY_RAD_WALL_GRAY_EXTERIOR_T;
   else if (cs_gui_strcmp(type, "iprefl"))
-    result = cs_glob_rad_transfer_params->iprefl;
+    result = CS_BOUNDARY_RAD_WALL_REFL_EXTERIOR_T;
   else if (cs_gui_strcmp(type, "ifgrno"))
-    result = cs_glob_rad_transfer_params->ifgrno;
+    result = CS_BOUNDARY_RAD_WALL_GRAY_COND_FLUX;
   else if (cs_gui_strcmp(type, "ifrefl"))
-    result = cs_glob_rad_transfer_params->ifrefl;
+    result = CS_BOUNDARY_RAD_WALL_REFL_COND_FLUX;
 
   if (result == -999)
     bft_error (__FILE__, __LINE__, 0,
@@ -255,12 +255,8 @@ cs_gui_radiative_transfers_finalize(void)
 void
 cs_gui_radiative_transfer_parameters(void)
 {
-  if (!cs_gui_file_is_loaded())
-    return;
-
   const char *model = cs_gui_get_thermophysical_model("radiative_transfer");
 
-  int isuird = 0;
   int ac_type = 0;
 
   if (cs_gui_strcmp(model, "off"))
@@ -276,28 +272,32 @@ cs_gui_radiative_transfer_parameters(void)
       = cs_tree_get_node(cs_glob_tree,
                          "thermophysical_models/radiative_transfer");
 
+    int isuird = -1;
     cs_gui_node_get_child_status_int(tn0, "restart", &isuird);
-    if (isuird) {
-      if (cs_restart_present())
-        cs_glob_rad_transfer_params->restart = true;
-    }
+    if (! cs_restart_present() || isuird == 0)
+      cs_glob_rad_transfer_params->restart = false;
+    else if (isuird == 1)
+      cs_glob_rad_transfer_params->restart = true;
+
     cs_gui_node_get_child_int(tn0, "quadrature",
                               &cs_glob_rad_transfer_params->i_quadrature);
     cs_gui_node_get_child_int(tn0, "directions_number",
                               &cs_glob_rad_transfer_params->ndirec);
-    cs_gui_node_get_child_int(tn0, "frequency",
-                              &cs_glob_rad_transfer_params->nfreqr);
     cs_gui_node_get_child_int(tn0, "thermal_radiative_source_term",
                               &cs_glob_rad_transfer_params->idiver);
     cs_gui_node_get_child_int(tn0, "temperature_listing_printing",
                               &cs_glob_rad_transfer_params->iimpar);
     cs_gui_node_get_child_int(tn0, "intensity_resolution_listing_printing",
-                              &cs_glob_rad_transfer_params->iimlum);
-    if (!cs_gui_get_activ_thermophysical_model()) {
+                              &cs_glob_rad_transfer_params->verbosity);
+    if (cs_glob_physical_model_flag[CS_PHYSICAL_MODEL_FLAG] >= 2) {
       _radiative_transfer_type(tn0, "absorption_coefficient", &ac_type);
       if (ac_type == 3)
         cs_glob_rad_transfer_params->imodak = 1;
     }
+    cs_gui_node_get_child_int
+      (tn0, "frequency",
+       &(cs_glob_rad_transfer_params->time_control.interval_nt));
+
   }
 #if _XML_DEBUG_
   bft_printf("==> cs_gui_radiative_transfer_parameters\n");
@@ -307,10 +307,11 @@ cs_gui_radiative_transfer_parameters(void)
     bft_printf("--isuird = %d\n", isuird);
     bft_printf("--quadra = %d\n", cs_glob_rad_transfer_params->i_quadrature);
     bft_printf("--ndirec = %d\n", cs_glob_rad_transfer_params->ndirec);
-    bft_printf("--nfreqr = %d\n", cs_glob_rad_transfer_params->nfreqr);
+    bft_printf("--interval_nt = %d\n",
+               cs_glob_rad_transfer_params->time_control.interval_nt);
     bft_printf("--idiver = %d\n", cs_glob_rad_transfer_params->idiver);
     bft_printf("--iimpar = %d\n", cs_glob_rad_transfer_params->iimpar);
-    bft_printf("--iimlum = %d\n", cs_glob_rad_transfer_params->iimlum);
+    bft_printf("--verbosity = %d\n", cs_glob_rad_transfer_params->verbosity);
     bft_printf("--absorption coefficient type: %d\n", ac_type);
     bft_printf("--absorption coefficient by modak: %i\n",
                cs_glob_rad_transfer_params->imodak);
@@ -333,8 +334,7 @@ cs_gui_rad_transfer_absorption(cs_real_t  ck[])
 
   const cs_lnum_t n_cells = cs_glob_mesh->n_cells;
 
-  if (!cs_gui_get_activ_thermophysical_model()) {
-
+  if (cs_glob_physical_model_flag[CS_PHYSICAL_MODEL_FLAG] < 2) {
     cs_tree_node_t *tn
       = cs_tree_get_node(cs_glob_tree,
                          "thermophysical_models/radiative_transfer");
@@ -361,9 +361,6 @@ cs_gui_rad_transfer_absorption(cs_real_t  ck[])
 void
 cs_gui_radiative_transfer_postprocess(void)
 {
-  if (!cs_gui_file_is_loaded())
-    return;
-
   const int n_rad_b_f = 8;
 
   const char  *b_rad_names[8] = {
@@ -445,14 +442,10 @@ cs_gui_radiative_transfer_bcs(const    int   itypfb[],
                               int           *isothp,
                               double        *epsp,
                               double        *epap,
-                              double        *tintp,
                               double        *textp,
                               double        *xlamp,
                               double        *rcodcl)
 {
-  if (!cs_gui_file_is_loaded())
-    return;
-
   const cs_lnum_t  n_b_faces = cs_glob_mesh->n_b_faces;
 
   cs_tree_node_t *tn_b0 = cs_tree_get_node(cs_glob_tree,
@@ -571,27 +564,23 @@ cs_gui_radiative_transfer_bcs(const    int   itypfb[],
                "with these new natures.\n"));
 
         isothp[ifbr] = _boundary->type[izone];
-        if (isothp[ifbr] == cs_glob_rad_transfer_params->itpimp) {
+        if (isothp[ifbr] == CS_BOUNDARY_RAD_WALL_GRAY) {
           epsp[ifbr] = _boundary->emissivity[izone];
-          tintp[ifbr] = _boundary->internal_temp[izone];
         }
-        else if (isothp[ifbr] == cs_glob_rad_transfer_params->ipgrno) {
+        else if (isothp[ifbr] == CS_BOUNDARY_RAD_WALL_GRAY_EXTERIOR_T) {
           xlamp[ifbr] = _boundary->thermal_conductivity[izone];
           epap[ifbr] = _boundary->thickness[izone];
           textp[ifbr] = _boundary->external_temp[izone];
-          tintp[ifbr] = _boundary->internal_temp[izone];
           epsp[ifbr] = _boundary->emissivity[izone];
           if (cs_gui_is_equal_real(_boundary->emissivity[izone], 0.))
-            isothp[ifbr] = cs_glob_rad_transfer_params->iprefl;
+            isothp[ifbr] = CS_BOUNDARY_RAD_WALL_REFL_EXTERIOR_T;
         }
-        else if (isothp[ifbr] == cs_glob_rad_transfer_params->ifgrno) {
+        else if (isothp[ifbr] == CS_BOUNDARY_RAD_WALL_GRAY_COND_FLUX) {
           rcodcl[2 * n_b_faces*_nvar + _ivart*n_b_faces + ifbr]
             = _boundary->conduction_flux[izone];
-          tintp[ifbr] = _boundary->internal_temp[izone];
+          epsp[ifbr] = _boundary->emissivity[izone];
           if (cs_gui_is_equal_real(_boundary->emissivity[izone], 0.))
-            isothp[ifbr] = cs_glob_rad_transfer_params->ifrefl;
-          else
-            epsp[ifbr] = _boundary->emissivity[izone];
+            isothp[ifbr] = CS_BOUNDARY_RAD_WALL_REFL_COND_FLUX;
         }
       }
 
