@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -48,20 +48,23 @@
 
 #include "fvm_selector.h"
 
-#include "mei_evaluate.h"
-
+#include "cs_atmo.h"
 #include "cs_base.h"
+#include "cs_cf_model.h"
 #include "cs_gui_util.h"
 #include "cs_gui.h"
-#include "cs_gui_variables.h"
 #include "cs_mesh.h"
 #include "cs_field.h"
 #include "cs_field_pointer.h"
+#include "cs_parameters.h"
 #include "cs_physical_model.h"
-#include "cs_prototypes.h"
 #include "cs_selector.h"
+#include "cs_physical_properties.h"
+#include "cs_physical_constants.h"
 #include "cs_elec_model.h"
 #include "cs_gwf_physical_properties.h"
+#include "cs_vof.h"
+#include "cs_combustion_model.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -523,6 +526,113 @@ _get_nox_reburning(cs_tree_node_t  *tn_nox,
     *keyword = 2;
 }
 
+/*----------------------------------------------------------------------------
+ * Atmospheric flows: read of meteorological file of data
+ *----------------------------------------------------------------------------*/
+
+static void
+_gui_atmo_get_set_meteo_profile(void)
+{
+  const char path_af[] = "thermophysical_models/atmospheric_flows";
+
+  cs_tree_node_t *tn = cs_tree_get_node(cs_glob_tree, path_af);
+
+  if (tn == NULL)
+    return;
+
+  int is_meteo_file = 0;
+  int is_large_scale_meteo = 0;
+
+  cs_gui_node_get_child_status_int(tn, "read_meteo_data",
+                                   &is_meteo_file);
+  cs_gui_node_get_child_status_int(tn, "large_scale_meteo",
+                                   &is_large_scale_meteo);
+
+  if (is_meteo_file && !is_large_scale_meteo) {
+    cs_glob_atmo_option->meteo_profile = 1;
+    const char *cstr = cs_tree_node_get_child_value_str(tn, "meteo_data");
+
+    /* Copy string */
+    if (cstr != NULL)
+      cs_atmo_set_meteo_file_name(cstr);
+
+  }
+  else if (is_large_scale_meteo && !is_meteo_file) {
+    cs_glob_atmo_option->meteo_profile = 2;
+    const char *str_latitude
+      = cs_tree_node_get_child_value_str(tn, "latitude");
+    const char *str_longitude
+      = cs_tree_node_get_child_value_str(tn, "longitude");
+    const char *str_domain_orient
+      = cs_tree_node_get_child_value_str(tn, "domain_orientation");
+    const char *str_wind_dir
+      = cs_tree_node_get_child_value_str(tn, "wind_direction");
+
+    const char *str_meteo_z0
+      = cs_tree_node_get_child_value_str(tn, "meteo_z0");
+    const char *str_meteo_uref
+      = cs_tree_node_get_child_value_str(tn, "meteo_uref");
+    const char *str_meteo_ustar
+      = cs_tree_node_get_child_value_str(tn, "meteo_ustar");
+    const char *str_meteo_dlmo
+      = cs_tree_node_get_child_value_str(tn, "meteo_dlmo");
+    const char *str_meteo_zref
+      = cs_tree_node_get_child_value_str(tn, "meteo_zref");
+    const char *str_meteo_psea
+      = cs_tree_node_get_child_value_str(tn, "meteo_psea");
+    const char *str_meteo_t0
+      = cs_tree_node_get_child_value_str(tn, "meteo_t0");
+
+    const char *str_syear = cs_tree_node_get_child_value_str(tn, "start_year");
+    const char *str_sday = cs_tree_node_get_child_value_str(tn, "start_day");
+    const char *str_shour = cs_tree_node_get_child_value_str(tn, "start_hour");
+    const char *str_smin = cs_tree_node_get_child_value_str(tn, "start_min");
+    const char *str_ssec = cs_tree_node_get_child_value_str(tn, "start_sec");
+
+    if (str_latitude != NULL)
+      cs_glob_atmo_option->latitude = atof(str_latitude);
+    if (str_longitude != NULL)
+      cs_glob_atmo_option->longitude = atof(str_longitude);
+    if (str_domain_orient != NULL)
+      cs_glob_atmo_option->domain_orientation = atof(str_domain_orient);
+    if (str_wind_dir != NULL)
+      cs_glob_atmo_option->meteo_angle = atof(str_wind_dir);
+
+    if (str_meteo_z0 != NULL)
+      cs_glob_atmo_option->meteo_z0  = atof(str_meteo_z0);
+    if (str_meteo_uref != NULL)
+      cs_glob_atmo_option->meteo_uref  = atof(str_meteo_uref);
+    if (str_meteo_ustar != NULL)
+      cs_glob_atmo_option->meteo_ustar0  = atof(str_meteo_ustar);
+    if (str_meteo_dlmo != NULL)
+      cs_glob_atmo_option->meteo_dlmo  = atof(str_meteo_dlmo);
+    if (str_meteo_zref != NULL)
+      cs_glob_atmo_option->meteo_zref  = atof(str_meteo_zref);
+    if (str_meteo_psea != NULL)
+      cs_glob_atmo_option->meteo_psea  = atof(str_meteo_psea);
+    if (str_meteo_t0 != NULL)
+      cs_glob_atmo_option->meteo_t0  = atof(str_meteo_t0);
+
+    if(str_syear != NULL)
+      cs_glob_atmo_option->syear = atoi(str_syear);
+    if(str_sday != NULL)
+      cs_glob_atmo_option->squant = atoi(str_sday);
+    if(str_shour != NULL)
+      cs_glob_atmo_option->shour = atoi(str_shour);
+    if(str_smin != NULL)
+      cs_glob_atmo_option->smin = atoi(str_smin);
+    if(str_ssec != NULL)
+      cs_glob_atmo_option->ssec = atof(str_ssec);
+    /*TODO how to split which way profile definition we are using ? */
+
+  }
+
+#if _XML_DEBUG_
+  bft_printf("==> %s\n", __func__);
+  bft_printf("--meteo_profile  = %i\n", cs_glob_atmo_option->meteo_profile);
+#endif
+}
+
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*============================================================================
@@ -543,22 +653,26 @@ _get_nox_reburning(cs_tree_node_t  *tn_nox,
 void CS_PROCF (uicpi1, UICPI1) (double *const srrom,
                                 double *const diftl0)
 {
-  cs_var_t  *vars = cs_glob_var;
-
   cs_tree_node_t *tn
     = cs_tree_get_node(cs_glob_tree, "numerical_parameters/density_relaxation");
 
   cs_gui_node_get_real(tn, srrom);
 
-  if (cs_gui_strcmp(vars->model, "gas_combustion") ||
-      cs_gui_strcmp(vars->model, "solid_fuels")) {
-    cs_gui_properties_value("dynamic_diffusion", diftl0);
+  bool gas_combustion = false;
+  for (cs_physical_model_type_t m_type = CS_COMBUSTION_3PT;
+       m_type <= CS_COMBUSTION_FUEL;
+       m_type++) {
+    if (cs_glob_physical_model_flag[m_type] > -1)
+      gas_combustion = true;
   }
 
+  if (gas_combustion)
+    cs_gui_properties_value("dynamic_diffusion", diftl0);
+
 #if _XML_DEBUG_
   bft_printf("==> %s\n", __func__);
   bft_printf("--srrom  = %f\n", *srrom);
-  if (cs_gui_strcmp(vars->model, "gas_combustion")) {
+  if (gas_combustion) {
     bft_printf("--diftl0  = %f\n", *diftl0);
   }
 #endif
@@ -569,17 +683,15 @@ void CS_PROCF (uicpi1, UICPI1) (double *const srrom,
  *
  * Fortran Interface:
  *
- * SUBROUTINE UICPI2 (Toxy, Tfuel)
- * *****************
- * DOUBLE PRECISION Toxy   <--   Oxidant temperature
- * DOUBLE PRECISION Tfuel  <--   Fuel temperature
+ * Toxy   <--   Oxidant temperature
+ * Tfuel  <--   Fuel temperature
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (uicpi2, UICPI2) (double *const toxy,
                                 double *const tfuel)
 {
-  cs_gui_reference_initialization("oxydant_temperature", toxy);
-  cs_gui_reference_initialization("fuel_temperature", tfuel);
+  cs_gui_fluid_properties_value("reference_oxydant_temperature", toxy);
+  cs_gui_fluid_properties_value("reference_fuel_temperature", tfuel);
 #if _XML_DEBUG_
   bft_printf("==> %s\n", __func__);
   bft_printf("--toxy  = %f\n", *toxy);
@@ -587,65 +699,6 @@ void CS_PROCF (uicpi2, UICPI2) (double *const toxy,
 #endif
 }
 
-
-/*----------------------------------------------------------------------------
- * Atmospheric flows: read of meteorological file of data
- *
- * Fortran Interface:
- *
- * subroutine uiati1
- * *****************
- * integer         imeteo   <--   on/off index
- * char(*)         fmeteo   <--   meteo file name
- * int             len      <--   meteo file name destination string length
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uiati1, UIATI1) (int           *imeteo,
-                                char          *fmeteo,
-                                int           *len
-                                CS_ARGF_SUPP_CHAINE)
-{
-  const char path_af[] = "thermophysical_models/atmospheric_flows";
-
-  cs_tree_node_t *tn = cs_tree_get_node(cs_glob_tree, path_af);
-
-  if (tn == NULL)
-    return;
-
-  cs_gui_node_get_child_status_int(tn, "read_meteo_data", imeteo);
-
-  if (*imeteo) {
-
-    const char *cstr = cs_tree_node_get_child_value_str(tn, "meteo_data");
-
-    /* Copy string */
-
-    if (cstr != NULL) {
-
-      /* Compute string length (removing start or end blanks) */
-
-      int l = strlen(cstr);
-      if (l > *len)
-        l = *len;
-
-      for (int i = 0; i < l; i++)
-        fmeteo[i] = cstr[i];
-
-      /* Pad with blanks if necessary */
-
-      for (int i = l; i < *len; i++)
-        fmeteo[i] = ' ';
-
-    }
-
-  }
-
-#if _XML_DEBUG_
-  bft_printf("==> %s\n", __func__);
-  bft_printf("--imeteo  = %i\n", *imeteo);
-#endif
-}
-
 /*----------------------------------------------------------------------------
  * Indirection between the solver numbering and the XML one
  * for physical properties of the activated specific physics
@@ -653,7 +706,6 @@ void CS_PROCF (uiati1, UIATI1) (int           *imeteo,
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (uisofu, UISOFU) (const int    *iirayo,
-                                const int    *iihmpr,
                                 const int    *ncharm,
                                 int          *ncharb,
                                 int          *nclpch,
@@ -718,11 +770,6 @@ void CS_PROCF (uisofu, UISOFU) (const int    *iirayo,
                                 double       *repnle,
                                 double       *repnlo)
 {
-  cs_var_t  *vars = cs_glob_var;
-
-  if (*iihmpr != 1)
-    cs_gui_load_file("dp_FCP.xml");
-
   /* Read gas mix absorption coefficient */
   if (*iirayo > 0)
     *ckabs1 = _get_absorption_coefficient();
@@ -740,7 +787,6 @@ void CS_PROCF (uisofu, UISOFU) (const int    *iirayo,
   cs_gui_node_get_child_status_int(tn_sf, "CO2_kinetics", ihtco2);
   cs_gui_node_get_child_status_int(tn_sf, "H2O_kinetics", ihth2o);
 
-
   /* Kinetic model (CO2 or CO transport) */
   _get_kinetic_model(tn_sf, ieqco2);
 
@@ -924,12 +970,12 @@ void CS_PROCF (uisofu, UISOFU) (const int    *iirayo,
     rho0ch[icha] = _get_solid_fuel_child_real(tn, "density");
 
     /* ---- Thermal conductivity of the coal (W/m/K) */
-    if (vars != NULL && thcdch != NULL)
-      if (cs_gui_strcmp(vars->model_value, "homogeneous_fuel_moisture")) {
-        thcdch[icha] = 1e-5; /* default */
-        cs_gui_node_get_child_real(tn, "thermal_conductivity",
-                                   &(thcdch[icha]));
-      }
+    if (   thcdch != NULL
+        && cs_glob_physical_model_flag[CS_COMBUSTION_COAL] == 1) {
+      thcdch[icha] = 1e-5; /* default */
+      cs_gui_node_get_child_real(tn, "thermal_conductivity",
+                                 &(thcdch[icha]));
+    }
 
     /* Ash characteristics */
 
@@ -1249,44 +1295,109 @@ void CS_PROCF (uidai1, UIDAI1) (int  *permeability,
 #endif
 }
 
+/*-----------------------------------------------------------------------------
+ * Return 1 if a specific physics model is activated, 0 otherwise.
+ *----------------------------------------------------------------------------*/
+
+static int
+_get_active_thermophysical_model(char  **model_name,
+                                 char  **model_value)
+{
+  int isactiv = 0;
+
+  if (*model_name != NULL && *model_value != NULL) {
+    isactiv = 1;
+    return isactiv;
+  }
+  else {
+    BFT_FREE(*model_name);
+    BFT_FREE(*model_value);
+  }
+
+  const char *model = NULL, *name = NULL;
+
+  const char *name_m[] = {"solid_fuels",
+                          "joule_effect",
+                          "atmospheric_flows",
+                          "compressible_model",
+                          "groundwater_model",
+                          "hgn_model"};
+  const char *name_o[] = {"gas_combustion"};
+
+  int n_names_m = sizeof(name_m) / sizeof(name_m[0]);
+  int n_names_o = sizeof(name_o) / sizeof(name_o[0]);
+
+  cs_tree_node_t *tn0 = cs_tree_get_node(cs_glob_tree, "thermophysical_models");
+
+  /* Loop on model nodes to compare to the listed ones */
+
+  if (tn0 != NULL) {
+    for (cs_tree_node_t *tn = tn0->children;
+         tn != NULL && name == NULL;
+         tn = tn->next) {
+      for (int j = 0; j < n_names_m && name == NULL; j++) {
+        if (! strcmp(tn->name, name_m[j])) {
+          model = cs_tree_node_get_tag(tn, "model");
+          if (model != NULL && !cs_gui_strcmp(model, "off"))
+            name = name_m[j];
+        }
+      }
+      for (int j = 0; j < n_names_o && name == NULL; j++) {
+        if (! strcmp(tn->name, name_o[j])) {
+          model = cs_tree_node_get_tag(tn, "option");
+          if (model != NULL && !cs_gui_strcmp(model, "off"))
+            name = name_o[j];
+        }
+      }
+    }
+  }
+
+  if (name != NULL) {
+    BFT_MALLOC(*model_name, strlen(name)+1, char);
+    strcpy(*model_name, name);
+
+    BFT_MALLOC(*model_value, strlen(model)+1, char);
+    strcpy(*model_value, model);
+
+    isactiv = 1;
+  }
+
+  return isactiv;
+}
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
 
 /*-----------------------------------------------------------------------------
  * Activate specific physical models based on XML settings.
- *
- * parameters:
- *   ieos    --> compressible
  *----------------------------------------------------------------------------*/
 
 void
-cs_gui_physical_model_select(cs_int_t  *ieos)
+cs_gui_physical_model_select(void)
 {
-  if (!cs_gui_file_is_loaded())
-    return;
-
   int isactiv = 0;
 
-  cs_var_t  *vars = cs_glob_var;
-
   /* Look for the active specific physics and give the value of the associated
      model attribute */
-  isactiv = cs_gui_get_activ_thermophysical_model();
+
+  char *model_name = NULL, *model_value = NULL;
+
+  isactiv = _get_active_thermophysical_model(&model_name, &model_value);
 
   if (isactiv)  {
 
-    if (cs_gui_strcmp(vars->model, "solid_fuels")) {
-      if (cs_gui_strcmp(vars->model_value, "homogeneous_fuel"))
+    if (cs_gui_strcmp(model_name, "solid_fuels")) {
+      if (cs_gui_strcmp(model_value, "homogeneous_fuel"))
         cs_glob_physical_model_flag[CS_COMBUSTION_COAL] = 0;
-      else if (cs_gui_strcmp(vars->model_value,
+      else if (cs_gui_strcmp(model_value,
                              "homogeneous_fuel_moisture"))
         cs_glob_physical_model_flag[CS_COMBUSTION_COAL] = 1;
       else
         bft_error(__FILE__, __LINE__, 0,
-                  _("Invalid coal model: %s."), vars->model_value);
+                  _("Invalid coal model: %s."), model_value);
     }
-    else if (cs_gui_strcmp(vars->model, "gas_combustion")) {
+    else if (cs_gui_strcmp(model_name, "gas_combustion")) {
 
       cs_tree_node_t *tn
         = cs_tree_get_node(cs_glob_tree, "thermophysical_models/gas_combustion");
@@ -1297,50 +1408,81 @@ cs_gui_physical_model_select(cs_int_t  *ieos)
 
       if (model != NULL && !cs_gui_strcmp(model, "off")) {
 
-        if (cs_gui_strcmp(vars->model_value, "adiabatic"))
+        if (cs_gui_strcmp(model_value, "adiabatic"))
           cs_glob_physical_model_flag[CS_COMBUSTION_3PT] = 0;
-        else if (cs_gui_strcmp(vars->model_value, "extended"))
+        else if (cs_gui_strcmp(model_value, "extended"))
           cs_glob_physical_model_flag[CS_COMBUSTION_3PT] = 1;
-        else if (cs_gui_strcmp(vars->model_value, "spalding"))
+        else if (cs_gui_strcmp(model_value, "spalding"))
           cs_glob_physical_model_flag[CS_COMBUSTION_EBU] = 0;
-        else if (cs_gui_strcmp(vars->model_value, "enthalpy_st"))
+        else if (cs_gui_strcmp(model_value, "enthalpy_st"))
           cs_glob_physical_model_flag[CS_COMBUSTION_EBU] = 1;
-        else if (cs_gui_strcmp(vars->model_value, "mixture_st"))
+        else if (cs_gui_strcmp(model_value, "mixture_st"))
           cs_glob_physical_model_flag[CS_COMBUSTION_EBU] = 2;
-        else if (cs_gui_strcmp(vars->model_value, "enthalpy_mixture_st"))
+        else if (cs_gui_strcmp(model_value, "enthalpy_mixture_st"))
           cs_glob_physical_model_flag[CS_COMBUSTION_EBU] = 3;
-        else if (cs_gui_strcmp(vars->model_value, "2-peak_adiabatic"))
+        else if (cs_gui_strcmp(model_value, "2-peak_adiabatic"))
           cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 0;
-        else if (cs_gui_strcmp(vars->model_value, "2-peak_enthalpy"))
+        else if (cs_gui_strcmp(model_value, "2-peak_enthalpy"))
           cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 1;
-        else if (cs_gui_strcmp(vars->model_value, "3-peak_adiabatic"))
+        else if (cs_gui_strcmp(model_value, "3-peak_adiabatic"))
           cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 2;
-        else if (cs_gui_strcmp(vars->model_value, "3-peak_enthalpy"))
+        else if (cs_gui_strcmp(model_value, "3-peak_enthalpy"))
           cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 3;
-        else if (cs_gui_strcmp(vars->model_value, "4-peak_adiabatic"))
+        else if (cs_gui_strcmp(model_value, "4-peak_adiabatic"))
           cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 4;
-        else if (cs_gui_strcmp(vars->model_value, "4-peak_enthalpy"))
+        else if (cs_gui_strcmp(model_value, "4-peak_enthalpy"))
           cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 5;
         else
           bft_error(__FILE__, __LINE__, 0,
                     _("Invalid gas combustion flow model: %s."),
-                    vars->model_value);
+                    model_value);
+
+        /* Read uniform variable thermodynamical pressure (ipthrm) */
+        cs_fluid_properties_t *phys_pp = cs_get_glob_fluid_properties();
+        cs_gui_node_get_child_status_int(tn, "thermodynamical_pressure",
+                                         &(phys_pp->ipthrm));
+
+        /* Read the soot model (isoot, rosoot, xsoot) */
+        cs_combustion_model_t   *cm = cs_glob_combustion_model;
+        cs_tree_node_t *tn_soot
+        = cs_tree_get_node(cs_glob_tree,
+                           "thermophysical_models/gas_combustion/soot_model");
+
+        const char *model_soot = cs_tree_node_get_child_value_str(tn_soot,
+                                                                  "model");
+
+        if (model_soot != NULL && !cs_gui_strcmp(model_soot, "off")) {
+          if (cs_gui_strcmp(model_soot, "constant_soot_yield")) {
+            cm->isoot = 0;
+            cs_gui_node_get_child_real(tn_soot, "soot_density",
+                                       &(cm->gas.rosoot));
+            cs_gui_node_get_child_real(tn_soot, "soot_fraction",
+                                       &(cm->gas.xsoot));}
+          else if (cs_gui_strcmp(model_soot, "moss")) {
+            cm->isoot = 1;
+            cs_gui_node_get_child_real(tn_soot, "soot_density",
+                                       &(cm->gas.rosoot));}
+        }
       }
     }
-    else if (cs_gui_strcmp(vars->model, "atmospheric_flows")) {
-      if (cs_gui_strcmp(vars->model_value, "constant"))
-        cs_glob_physical_model_flag[CS_ATMOSPHERIC] = 0;
-      else if (cs_gui_strcmp(vars->model_value, "dry"))
-        cs_glob_physical_model_flag[CS_ATMOSPHERIC] = 1;
-      else if (cs_gui_strcmp(vars->model_value, "humid"))
-        cs_glob_physical_model_flag[CS_ATMOSPHERIC] = 2;
+    else if (cs_gui_strcmp(model_name, "atmospheric_flows")) {
+      if (cs_gui_strcmp(model_value, "constant"))
+        cs_glob_physical_model_flag[CS_ATMOSPHERIC] = CS_ATMO_CONSTANT_DENSITY;
+      else if (cs_gui_strcmp(model_value, "dry"))
+        cs_glob_physical_model_flag[CS_ATMOSPHERIC] = CS_ATMO_DRY;
+      else if (cs_gui_strcmp(model_value, "humid"))
+        cs_glob_physical_model_flag[CS_ATMOSPHERIC] = CS_ATMO_HUMID;
       else
         bft_error(__FILE__, __LINE__, 0,
                   _("Invalid atmospheric flow model: %s."),
-                  vars->model_value);
+                  model_value);
+
+      /* Get and set meteo file if given */
+      _gui_atmo_get_set_meteo_profile();
+
     }
-    else if (cs_gui_strcmp(vars->model, "joule_effect")) {
-      if (cs_gui_strcmp(vars->model_value, "joule")) {
+    else if (cs_gui_strcmp(model_name, "joule_effect")) {
+      if (cs_gui_strcmp(model_value, "joule")) {
 
         cs_tree_node_t *tn
           = cs_tree_get_node(cs_glob_tree,
@@ -1358,39 +1500,51 @@ cs_gui_physical_model_select(cs_int_t  *ieos)
         else
           bft_error(__FILE__, __LINE__, 0,
                     _("Invalid joule model: %s."),
-                    vars->model_value);
+                    model_value);
 
       }
-      else if (cs_gui_strcmp(vars->model_value, "arc"))
+      else if (cs_gui_strcmp(model_value, "arc"))
         cs_glob_physical_model_flag[CS_ELECTRIC_ARCS] = 2;
       else
         bft_error(__FILE__, __LINE__, 0,
                   _("Invalid electrical model: %s."),
-                  vars->model_value);
+                  model_value);
     }
-    else if (cs_gui_strcmp(vars->model, "compressible_model")) {
-      if (cs_gui_strcmp(vars->model_value, "constant_gamma")) {
+    else if (cs_gui_strcmp(model_name, "compressible_model")) {
+      if (cs_gui_strcmp(model_value, "constant_gamma")) {
         cs_glob_physical_model_flag[CS_COMPRESSIBLE] = 0;
-        *ieos = 1;
+        cs_cf_model_t *cf_mdl = cs_get_glob_cf_model();
+        cf_mdl->ieos = 1;
       }
       else
         bft_error(__FILE__, __LINE__, 0,
                   _("Invalid compressible model: %s."),
-                  vars->model_value);
+                  model_value);
     }
-    else if (cs_gui_strcmp(vars->model, "groundwater_model")) {
-      if (cs_gui_strcmp(vars->model_value, "groundwater"))
+    else if (cs_gui_strcmp(model_name, "groundwater_model")) {
+      if (cs_gui_strcmp(model_value, "groundwater"))
         cs_glob_physical_model_flag[CS_GROUNDWATER] = 1;
     }
+    else if (cs_gui_strcmp(model_name, "hgn_model")) {
+      cs_vof_parameters_t *vof_param = cs_get_glob_vof_parameters();
+      if (cs_gui_strcmp(model_value, "merkle_model")) {
+        vof_param->vof_model = CS_VOF_ENABLED | CS_VOF_MERKLE_MASS_TRANSFER;
+      } else {
+        vof_param->vof_model = CS_VOF_ENABLED + CS_VOF_FREE_SURFACE;
+      }
+    }
   }
 
 #if _XML_DEBUG_
   bft_printf("%s\n", __func__);
   if (isactiv) {
-    bft_printf("--thermophysical model: %s\n", vars->model);
-    bft_printf("--thermophysical value: %s\n", vars->model_value);
+    bft_printf("--thermophysical model: %s\n", model_name);
+    bft_printf("--thermophysical value: %s\n", model_value);
   }
 #endif
+
+  BFT_FREE(model_name);
+  BFT_FREE(model_value);
 }
 
 /*----------------------------------------------------------------------------
@@ -1528,81 +1682,6 @@ cs_gui_get_thermophysical_model(const char  *model_thermo)
   return retval;
 }
 
-/*-----------------------------------------------------------------------------
- * Return 1 if a specific physics model is activated, 0 othewise.
- *
- * Updates the cs_glob_vars global structure.
- *----------------------------------------------------------------------------*/
-
-int
-cs_gui_get_activ_thermophysical_model(void)
-{
-  int isactiv = 0;
-
-  if (cs_glob_var == NULL)
-    cs_gui_init();
-
-  cs_var_t  *vars = cs_glob_var;
-
-  if (vars->model != NULL && vars->model_value != NULL) {
-    isactiv = 1;
-    return isactiv;
-  }
-  else {
-    BFT_FREE(vars->model);
-    vars->model_value = NULL;
-  }
-
-  const char *model = NULL, *name = NULL;
-
-  const char *name_m[] = {"solid_fuels",
-                          "joule_effect",
-                          "atmospheric_flows",
-                          "compressible_model",
-                          "groundwater_model"};
-  const char *name_o[] = {"gas_combustion"};
-
-  int n_names_m = sizeof(name_m) / sizeof(name_m[0]);
-  int n_names_o = sizeof(name_o) / sizeof(name_o[0]);
-
-  cs_tree_node_t *tn0 = cs_tree_get_node(cs_glob_tree, "thermophysical_models");
-
-  /* Loop on model nodes to compare to the listed ones */
-
-  if (tn0 != NULL) {
-    for (cs_tree_node_t *tn = tn0->children;
-         tn != NULL && name == NULL;
-         tn = tn->next) {
-      for (int j = 0; j < n_names_m && name == NULL; j++) {
-        if (! strcmp(tn->name, name_m[j])) {
-          model = cs_tree_node_get_tag(tn, "model");
-          if (model != NULL && !cs_gui_strcmp(model, "off"))
-            name = name_m[j];
-        }
-      }
-      for (int j = 0; j < n_names_o && name == NULL; j++) {
-        if (! strcmp(tn->name, name_o[j])) {
-          model = cs_tree_node_get_tag(tn, "option");
-          if (model != NULL && !cs_gui_strcmp(model, "off"))
-            name = name_o[j];
-        }
-      }
-    }
-  }
-
-  if (name != NULL) {
-    BFT_MALLOC(vars->model, strlen(name)+1, char);
-    strcpy(vars->model, name);
-
-    BFT_MALLOC(vars->model_value, strlen(model)+1, char);
-    strcpy(vars->model_value, model);
-
-    isactiv = 1;
-  }
-
-  return isactiv;
-}
-
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
