@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -80,17 +80,14 @@ void CS_PROCF (uicpi2, UICPI2) (double *const toxy,
  *
  * subroutine uiati1
  * *****************
- * integer         imeteo   <--   on/off index
  * char(*)         fmeteo   <--   meteo file name
  * int             len      <--   meteo file name destination string length
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uiati1, UIATI1) (int           *imeteo,
-                                char          *fmeteo,
+void CS_PROCF (uiati1, UIATI1) (char          *fmeteo,
                                 int           *len
                                 CS_ARGF_SUPP_CHAINE);
 
-
 /*----------------------------------------------------------------------------
  * Indirection between the solver numbering and the XML one
  * for physical properties of the activated specific physics
@@ -98,7 +95,6 @@ void CS_PROCF (uiati1, UIATI1) (int           *imeteo,
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (uisofu, UISOFU) (const int    *iirayo,
-                                const int    *iihmpr,
                                 const int    *ncharm,
                                 int          *ncharb,
                                 int          *nclpch,
@@ -171,7 +167,6 @@ void CS_PROCF(cfnmtd, CFNMTD) (char          *fstr,    /* --> Fortran string */
                                int           *len      /* --> String Length  */
                                CS_ARGF_SUPP_CHAINE);
 
-
 /*----------------------------------------------------------------------------
  * groundwater model : read parameters
  *
@@ -198,13 +193,10 @@ void CS_PROCF (uidai1, UIDAI1) (int    *permeability,
 
 /*-----------------------------------------------------------------------------
  * Activate specific physical models based on XML settings.
- *
- * parameters:
- *   ieos    --> compressible
  *----------------------------------------------------------------------------*/
 
 void
-cs_gui_physical_model_select(cs_int_t  *ieos);
+cs_gui_physical_model_select(void);
 
 /*----------------------------------------------------------------------------
  * Electrical model: read parameters
@@ -230,15 +222,6 @@ cs_gui_elec_model_rec(void);
 const char *
 cs_gui_get_thermophysical_model(const char  *model_thermo);
 
-/*-----------------------------------------------------------------------------
- * Return 1 if a specific physics model is activated, 0 othewise.
- *
- * Updates the cs_glob_vars global structure.
- *----------------------------------------------------------------------------*/
-
-int
-cs_gui_get_activ_thermophysical_model(void);
-
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
