@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -89,23 +89,6 @@ static bool _setup_read = false;
  * Public function definitions
  *============================================================================*/
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Indicate if an XML file has been loaded.
- *
- * \return  1 if an XML file has been loaded, 0 otherwise
- */
-/*----------------------------------------------------------------------------*/
-
-int
-cs_gui_file_is_loaded(void)
-{
-  int retval = 0;
-  if (_setup_read)
-    retval = 1;
-  return retval;
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Load the XML file in memory.
@@ -260,7 +243,7 @@ cs_gui_is_equal_real(cs_real_t v1,
  * \brief  Update an integer value based on a tree node
  *
  * If no node is present, the initial value is unchanged.
- * If the node is present but the value missing, an error is returne.
+ * If the node is present but the value missing, an error is returned.
  *
  * \param[in]       node    node whose value is queried
  * \param[in, out]  value   queried value
@@ -293,7 +276,7 @@ cs_gui_node_get_int(cs_tree_node_t  *node,
  * \brief  Update an real value based on a tree node
  *
  * If no node is present, the initial value is unchanged.
- * If the node is present but the value missing, an error is returne.
+ * If the node is present but the value missing, an error is returned.
  *
  * \param[in]       node    node whose value is queried
  * \param[in, out]  value   queried value
@@ -328,20 +311,20 @@ cs_gui_node_get_real(cs_tree_node_t  *node,
  * The status is defined in a string-valued child (tag) node. If no such
  * tag is present, the initial status is unchanged.
  *
- * \param[in]       tn      node whose status is queried
+ * \param[in]       node    node whose status is queried
  * \param[in, out]  status  status value (0 or 1)
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_gui_node_get_status_int(cs_tree_node_t  *tn,
+cs_gui_node_get_status_int(cs_tree_node_t  *node,
                            int             *status)
 {
-  const char  *value = cs_tree_node_get_tag(tn, "status");
+  const char  *value = cs_tree_node_get_tag(node, "status");
 
   if (cs_gui_strcmp(value, "on"))
     *status = 1;
-  else if (cs_gui_strcmp(value, "off"))
+  else if (cs_gui_strcmp(value, "off") || cs_gui_strcmp(value, ""))
     *status = 0;
   else if (value != NULL)
     bft_error(__FILE__, __LINE__, 0,
@@ -368,7 +351,7 @@ cs_gui_node_get_status_bool(cs_tree_node_t  *node,
 
   if (cs_gui_strcmp(value, "on"))
     *status = true;
-  else if (cs_gui_strcmp(value, "off"))
+  else if (cs_gui_strcmp(value, "off") || cs_gui_strcmp(value, ""))
     *status = false;
   else if (value != NULL)
     bft_error(__FILE__, __LINE__, 0,
@@ -411,7 +394,7 @@ cs_gui_node_get_tag(cs_tree_node_t  *node,
  * \brief  Update an integer value based on a tree's child node
  *
  * If no node is present, the initial value is unchanged.
- * If the node is present but the value missing, an error is returne.
+ * If the node is present but the value missing, an error is returned.
  *
  * \param[in]       node        node whose value is queried
  * \param[in]       child_name  name of child node
@@ -448,7 +431,7 @@ cs_gui_node_get_child_int(cs_tree_node_t  *node,
  * \brief  Update an integer value based on a tree's child node
  *
  * If no node is present, the initial value is unchanged.
- * If the node is present but the value missing, an error is returne.
+ * If the node is present but the value missing, an error is returned.
  *
  * \param[in]       node        node whose value is queried
  * \param[in]       child_name  name of child node
@@ -490,16 +473,16 @@ cs_gui_node_get_child_real(cs_tree_node_t  *node,
  *
  * \param[in]       node        node whose value is queried
  * \param[in]       child_name  name of child node
- * \param[in, out]  value       queried value
+ * \param[in, out]  status      queried value
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_gui_node_get_child_status_int(cs_tree_node_t  *tn,
+cs_gui_node_get_child_status_int(cs_tree_node_t  *node,
                                  const char      *child_name,
                                  int             *status)
 {
-  cs_tree_node_t *tn_c = cs_tree_node_get_child(tn, child_name);
+  cs_tree_node_t *tn_c = cs_tree_node_get_child(node, child_name);
 
   const char  *value = cs_tree_node_get_tag(tn_c, "status");
 
@@ -524,7 +507,7 @@ cs_gui_node_get_child_status_int(cs_tree_node_t  *tn,
  *
  * \param[in]       node        node whose value is queried
  * \param[in]       child_name  name of child node
- * \param[in, out]  value       queried value
+ * \param[in, out]  status      queried value
  */
 /*----------------------------------------------------------------------------*/
 
