@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -82,35 +82,74 @@ typedef void
 
 typedef enum {
 
-  CS_LAGR_BC_UNDEFINED,
-  CS_LAGR_INLET,
-  CS_LAGR_OUTLET,
-  CS_LAGR_REBOUND,
-  CS_LAGR_DEPO1,
-  CS_LAGR_DEPO2,
-  CS_LAGR_FOULING,
-  CS_LAGR_JBORD1,
-  CS_LAGR_JBORD2,
-  CS_LAGR_JBORD3,
-  CS_LAGR_JBORD4,
-  CS_LAGR_JBORD5,
-  CS_LAGR_DEPO_DLVO,
-  CS_LAGR_SYM
+  CS_LAGR_BC_UNDEFINED,  /*!< undefined conditions */
+  CS_LAGR_SYM,           /*!< symmetry */
+  CS_LAGR_INLET,         /*!< inlet */
+  CS_LAGR_OUTLET,        /*!< outlet */
+  CS_LAGR_REBOUND,       /*!< elastic rebound */
+  CS_LAGR_DEPO1,         /*!< immediate deposition and elimination */
+  CS_LAGR_DEPO2,         /*!< deposition */
+  CS_LAGR_DEPO_DLVO,     /*!< deposition based on DLVO theory */
+  CS_LAGR_FOULING,       /*!< fouling (combustion) */
+  CS_LAGR_BC_USER        /*!< user-defined */
 
-} cs_lagr_bc_type_t;
+} cs_lagr_bc_type;
 
-/*! Lagrangian deposition state */
+/*! Lagrangian injection condition types */
+/*---------------------------------------*/
+
+enum {
+  CS_LAGR_IN_IMPOSED_FLUID_VALUE = -1,  /*!< impose fluid value
+                                          for the injected particles */
+  CS_LAGR_IN_IMPOSED_NORM = 0,          /*!< impose norm */
+  CS_LAGR_IN_IMPOSED_COMPONENTS = 1     /*!< impose components (for vectors) */
+};
+
+/*! Lagrangian module status.
+   the different values correspond to the following coupling:
+   - CS_LAGR_OFF: Lagrangian module off
+   - CS_LAGR_ONEWAY_COUPLING: Lagrangian two-phase flow in one-way coupling
+       (no influence of the particles on the continuous phase)
+   - CS_LAGR_TWOWAY_COUPLING: Lagrangian two-phase flow with two-way coupling
+       (influence of the particles on the dynamics of the continuous phase).
+       Dynamics, temperature and mass may be coupled independently.
+   - CS_LAGR_FROZEN_CONTINUOUS_PHASE: Lagrangian two-phase flow on frozen i
+       continuous phase. This option
+       only only be used in case of a calculation restart. All the
+       Eulerian fields are frozen (including the scalar fields).
+       This option automatically implies \ref iccvfg = 1 */
+
+typedef enum {
+  CS_LAGR_OFF = 0,
+  CS_LAGR_ONEWAY_COUPLING = 1,
+  CS_LAGR_TWOWAY_COUPLING = 2,
+  CS_LAGR_FROZEN_CONTINUOUS_PHASE = 3
+} cs_lagr_module_status_t;
+
+/*! Particle shape condition types */
+/*---------------------------------*/
 
 typedef enum {
 
-  CS_LAGR_PART_IN_FLOW        = 0,
-  CS_LAGR_PART_DEPOSITED      = 1,
-  CS_LAGR_PART_ROLLING        = 2,
-  CS_LAGR_PART_TO_DELETE      = 3,
-  CS_LAGR_PART_NO_MOTION      = 10,
-  CS_LAGR_PART_IMPOSED_MOTION = 11
+  /*! Impose spherical particles */
+  CS_LAGR_SHAPE_SPHERE_MODEL = 0,
 
-} cs_lagr_deposition_state_t;
+  /*! Impose spheroids (stochastic model for transport) */
+  CS_LAGR_SHAPE_SPHEROID_STOC_MODEL = 1,
+
+  /*< Impose spheroids (Jeffery equations for transport) */
+  CS_LAGR_SHAPE_SPHEROID_JEFFERY_MODEL = 2,
+
+} cs_lagr_module_shape_t;
+
+/*! Lagrangian additional physical model */
+/*---------------------------------------*/
+
+enum {
+  CS_LAGR_PHYS_OFF = 0,
+  CS_LAGR_PHYS_HEAT = 1,
+  CS_LAGR_PHYS_COAL = 2
+};
 
 /*! Fixed maximum sizes */
 /*----------------------*/
@@ -143,15 +182,15 @@ typedef struct {
 typedef struct {
 
   /*! Lagrangian module status.
-     the different values correspond to the following coupling:
-     - 0: Lagrangian module off
-     - 1: Lagrangian two-phase flow in one-way coupling (no influence of
-          the particles on the continuous phase)
-     - 2 Lagrangian two-phase flow with two-way coupling (influence of
-         the particles on the dynamics of the continuous phase). Dynamics,
-         temperature and mass may be coupled independently.
-     - 3 Lagrangian two-phase flow on frozen continuous phase. This option
-         only only be used in case of a calculation restart. All the
+     - CS_LAGR_OFF: Lagrangian module off
+     - CS_LAGR_ONEWAY_COUPLING: Lagrangian two-phase flow in one-way coupling
+         (no influence of the particles on the continuous phase)
+     - CS_LAGR_TWOWAY_COUPLING: Lagrangian two-phase flow with two-way coupling
+         (influence of the particles on the dynamics of the continuous phase).
+         Dynamics, temperature and mass may be coupled independently.
+     - CS_LAGR_FROZEN_CONTINUOUS_PHASE: Lagrangian two-phase flow on frozen i
+         continuous phase. This option may
+         only be used in the case of a calculation restart. All the
          Eulerian fields are frozen (including the scalar fields).
          This option automatically implies \ref iccvfg = 1 */
   int  iilagr;
@@ -160,16 +199,19 @@ typedef struct {
       continuous phase flow
       in particular, \ref isttio = 1 is needed in order to:
       calculate steady statistics in the volume or at the boundaries
-      (starting respectively from the iterations \ref nstist)
+      (starting respectively from the iterations
+      \ref cs_lagr_stat_options_t::nstist "nstist")
       and calculate time-averaged two-way coupling source terms (from the
       time step \ref nstits).
-      Useful if \ref iilagr=1 or \ref iilagr=2 (if \ref iilagr=3,
+      Useful if \ref iilagr = CS_LAGR_ONEWAY_COUPLING
+      or \ref iilagr = CS_LAGR_TWOWAY_COUPLING
+      (if \ref iilagr = CS_LAGR_FROZEN_CONTINUOUS_PHASE,
       then \ref isttio=1 automatically) */
   int  isttio;
 
   /*! activation (=1) or not (=0) of a Lagrangian calculation restart.
     The calculation restart file read when this option is activated
-    only contains the data related to the particles (see also \ref isuist)
+    only contains the data related to the particles;
     the global calculation must also be a restart calculation
   */
   int  isuila;
@@ -177,31 +219,6 @@ typedef struct {
   /*! trajectory algorithm order in time */
   int  t_order;
 
-  /*! activates (>0) or not (=0) the complete turbulent dispersion model.
-    When \ref modcpl is strictly positive, its value is interpreted as the
-    absolute Lagrangian time step number (including restarts) after which the
-    complete model is applied.
-    Since the complete model uses volume statistics, \ref modcpl must
-    either be 0 or be larger than \ref idstnt. */
-  int     modcpl;
-
-  /*!  direction (1=x, 2=y, 3=z) of the complete model.
-    it corresponds to the main directions of the flow.
-    Useful if \ref modcpl > 0 */
-  int     idirla;
-
-  /*!  activation (=1) or not (=0) of the particle turbulent dispersion.
-    The turbulent dispersion is compatible only with the RANS turbulent models
-    (\f$k-\varepsilon\f$, \f$R_{ij}-\varepsilon\f$, v2f or \f$k-\omega\f$).
-    (\ref iturb=20, 21, 30, 31, 50 or 60). */
-  int     idistu;
-
-  /*! \ref idiffl=1 suppresses the crossing trajectory effect, making
-    turbulent dispersion for the particles identical to the turbulent
-    diffusion of fluid particles.
-    Useful if \ref idistu=1 */
-  int     idiffl;
-
   /*! activation (=1) or not (=0) of the solution of a Poisson's equation for
     the correction of the particle instantaneous velocities
     (in order to obtain a null divergence).
@@ -238,20 +255,41 @@ typedef struct {
 
   /*! activates (>0) or deactivates (=0) the physical models associated to the
     particles:
-    - 1: allows to associate with the particles evolution equations on
-         their temperature (in degrees Celsius), their diameter and
-         their mass
-    - = 2: the particles are pulverised coal particles.
-    Evolution equations on temperature (in degree Celsius), mass of
-    reactive coal, mass of char and diameter of the shrinking core are
-    associated with the particles. This option is available only if the
-    continuous phase represents a pulverised coal flame. */
-  int  physical_model;  /* FIXME: => enum: CS_LAGR_PHYS_STD,
-                                           CS_LAGR_PHYS_COAL,
-                                           CS_LAGR_PHYS_HEAT... */
+    - CS_LAGR_PHYS_HEAT: allows to associate with the particles evolution
+      equations on  their temperature (in degrees Celsius), their diameter and
+      their mass
+    - CS_LAGR_PHYS_COAL: the particles are pulverised coal particles.
+      Evolution equations on temperature (in degree Celsius), mass of
+      reactive coal, mass of char and diameter of the shrinking core are
+      associated with the particles. This option is available only if the
+      continuous phase represents a pulverised coal flame. */
+  int  physical_model;
   int  n_temperature_layers;
 
+  /*! Activates (1) or not (0) the assumption that we have regular particles.
+    When set to, 0 then the particles are assumed to be fluid particles
+    (and the turbulence dispersion model is disabled by default).
+    Since this model uses volume statistics, is use forces their
+    activation even if not otherwise requested
+    (see \ref cs_lagr_stat_options_t::idstnt "idstnt"). */
+  int modcpl;
+
+  /*! Activation of the turbulent dispersion (on: 1; off).
+     Default is on if \ref modcpl = 1, off if \ref modcpl = 0);
+     This is compatible only with the RANS turbulent models
+     (\f$k-\varepsilon\f$, \f$R_{ij}-\varepsilon\f$, v2f or \f$k-\omega\f$). */
+  int idistu;
+
+  /*! Suppress the crossing trajectory effect (if set to 1), making
+    turbulent dispersion for the particles identical to the turbulent
+    diffusion of fluid particles.
+    Default is off if \ref modcpl = 1, on if \ref modcpl = 0). */
+  int idiffl;
+
+  /*!- 0: no deposition model
+     - 1: depositionion model */
   int  deposition;
+
   int  dlvo;
 
   /*! - 0: no DLVO conditions with roughness surface
@@ -262,18 +300,32 @@ typedef struct {
      - 1: resuspension model */
   int  resuspension;
 
-  /* - 0: no clogging model
+  /*!- 0: no clogging model
      - 1: clogging model */
   int  clogging;
 
-  /* - 0: no consolidation model
-     - 1: consolidation model */
+  /*!- 0: spherical particles (default)
+     - 1: spheroid particles
+     - 2: ellipsoids */
+  int  shape;
+
+  /*! - 0: no consolidation model
+      - 1: consolidation model */
   int  consolidation;
 
   int  precipitation;
   int  fouling;
 
+  /*! - 0: no agglomeration model
+      - 1: agglomeration model used */
+  int agglomeration;
+
+  /*! - 0: no fragmentation model
+      - 1: fragmentation model used */
+  int fragmentation;
+
   int  n_stat_classes;
+
   int  n_user_variables;
 
 } cs_lagr_model_t;
@@ -300,6 +352,9 @@ typedef struct {
   /*! number of exited particles*/
   cs_gnum_t   n_g_exit;
 
+  /*! number of merged particles*/
+  cs_gnum_t   n_g_merged;
+
   /*! number of deposited particles */
   cs_gnum_t   n_g_deposited;
 
@@ -321,6 +376,9 @@ typedef struct {
   /*! weight of exited particles*/
   cs_real_t   w_exit;
 
+  /*! weight of merged particles*/
+  cs_real_t   w_merged;
+
   /*! weight of deposited particles */
   cs_real_t   w_deposited;
 
@@ -427,6 +485,39 @@ typedef struct {
 
 } cs_lagr_clogging_model_t;
 
+/*! Parameters of model for non-spherical particles */
+/* ------------------------------------------------ */
+
+typedef struct {
+
+  cs_real_t          param_chmb;
+
+} cs_lagr_shape_model_t;
+
+/*! Parameters of the particle agglomeration model */
+/* ------------------------------------------ */
+
+typedef struct {
+
+  cs_lnum_t          n_max_classes;
+  cs_real_t          min_stat_weight;
+  cs_real_t          max_stat_weight;
+  cs_real_t          scalar_kernel;
+  cs_real_t          base_diameter;
+
+} cs_lagr_agglomeration_model_t;
+
+/*! Parameters of the particle fragmentation model */
+/* ------------------------------------------ */
+
+typedef struct {
+
+  cs_real_t          scalar_kernel;
+  cs_real_t          base_diameter;
+  cs_real_t          (*function_kernel)(cs_lnum_t);
+
+} cs_lagr_fragmentation_model_t;
+
 /*! Parameters of the particle consolidation model */
 /* ----------------------------------------------- */
 
@@ -487,11 +578,15 @@ typedef struct {
     - 1 imposed temperature */
   int         temperature_profile;
 
-  int         coal_number;          /*!< particle coal number
-                                      (if \ref physical_model=2) */
+  int         coal_number;          /*!< particle coal number (if
+                                      \ref cs_lagr_model_t::physical_model "physical_model"
+                                      =2) */
 
   int         cluster;              /*!< statistical cluster id */
 
+  int         aggregat_class_id;    /*!< aggregate class id */
+  cs_real_t   aggregat_fractal_dim; /*!< aggregate fractal dimension */
+
   cs_real_t   velocity_magnitude;   /*!< particle velocity magnitude */
   cs_real_t   velocity[3];          /*!< particle velocity components */
 
@@ -500,6 +595,20 @@ typedef struct {
   cs_real_t   diameter;             /*!< particle diameter */
   cs_real_t   diameter_variance;    /*!< particle diameter variance */
 
+  cs_real_t   shape;                /*!< particle shape for spheroids
+                                        (if shape model is activated */
+  cs_real_t   orientation[3];       /*!< particle orintation for spheroids */
+  cs_real_t   radii[3];             /*!< particle radii for ellispoids */
+  cs_real_t   angular_vel[3];       /*!< particle angular velocity
+                                         (if shape model is activated */
+
+  cs_real_t   euler[4];             /*!< particle four Euler parameters
+                                         (if shape model is activated */
+  cs_real_t   shape_param[4];       /*!< particle shape parameters
+                                         for ellispoids
+                                         (alpha_0, beta_0, gamma_0, chi _0)
+                                         in Brenner 1964
+                                         (if shape model is activated */
   cs_real_t   density;              /*!< particle density */
 
   cs_real_t   fouling_index;        /*!< fouling index */
@@ -521,11 +630,13 @@ typedef struct {
 
   /*! activation (=1) or not (=0) of the two-way coupling on the dynamics
     of the continuous phase.
-    Useful if \ref iilagr = 2 and \ref iccvfg = 0 */
+    Useful if \ref iilagr = CS_LAGR_TWOWAY_COUPLING and \ref iccvfg = 0 */
   int  ltsdyn;
 
   /*! activation (=1) or not (=0) of the two-way coupling on the mass.
-    Useful if \ref iilagr = 2, \ref physical_model = 1 and \ref impvar = 1 */
+    Useful if \ref iilagr = CS_LAGR_TWOWAY_COUPLING,
+    \ref cs_lagr_model_t::physical_model "physical_model" = 1 and
+    \ref cs_lagr_specific_physics_t::impvar "impvar" = 1 */
   int  ltsmas;
 
   /*  if \ref physical_model = 1 and \ref itpvar = 1, \ref ltsthe
@@ -533,7 +644,7 @@ typedef struct {
    if \ref physical_model = 2, \ref ltsthe activates (=1) or not (=0) the
    two-way coupling on the eulerian variables related to pulverised
    coal combustion.
-   Useful if \ref iilagr = 2 */
+   Useful if \ref iilagr = CS_LAGR_TWOWAY_COUPLING */
   int  ltsthe;
 
   /*! implicit source term for the continuous phase velocity and
@@ -571,15 +682,16 @@ typedef struct {
   /*! number of absolute time steps (including the restarts)
     after which a time-average of the two-way coupling source terms is
     calculated.
-    indeed, if the flow is steady (\ref isttio=1), the average quantities
-    that appear in the two-way coupling source terms can be calculated over
-    different time steps, in order to get a better precision.
+    Indeed, if the flow is steady (\ref cs_lagr_time_scheme_t::isttio "isttio"=1),
+    the average quantities that appear in the two-way coupling source terms can
+    be calculated over different time steps, in order to get a better precision.
     if the number of absolute time steps is strictly inferior to
     \ref nstits, the code considers that the flow has not yet reached its
     steady state (transition period) and the averages appearing in the source
     terms are reinitialized at each time step, as it is the case for unsteady
-    flows (\ref isttio=0).
-    Useful if \ref iilagr = 2 and \ref isttio = 1 */
+    flows (\ref cs_lagr_time_scheme_t::isttio "isttio"=0).
+    Useful if \ref iilagr = CS_LAGR_TWOWAY_COUPLING and
+    \ref cs_lagr_time_scheme_t::isttio "isttio"=1 */
   int  nstits;
 
   /*! number of time steps for source terms accumulations */
@@ -638,35 +750,31 @@ typedef struct {
 
 typedef struct {
 
-  /*  activates (=1) or not (=0) the option of coal particle fouling.
-   It then is necessary to specify the domain boundaries
-   on which fouling may take place. Useful if \ref physical_model = 2*/
+  /* Activates (=1) or not (=0) the option of coal particle fouling.
+     It then is necessary to specify the domain boundaries
+     on which fouling may take place. Useful if \ref physical_model = 2*/
   int  iencra;
 
-  /*  encrustation data*/
+  /* encrustation data*/
   int  npencr;
   // TODO cf particles->n_part_fou in cs_lagr_tracking.c
 
-  /*  encrustation data*/
-//TODO
-  cs_real_t  *enc1;//ncharm2
-  /*  encrustation data*/
-//TODO
-  cs_real_t  *enc2;//ncharm2
+  /* encrustation data*/
+  cs_real_t  *enc1;  // size: ncharm2
+  /* encrustation data*/
+  cs_real_t  *enc2;  // size: ncharm2
 
-  /*  limit temperature (in degree Celsius) below which the coal particles do
-   not cause any fouling (if the fouling model is activated).
-   Useful if \ref physical_model = 2 and \ref iencra = 1*/
-//TODO
-  cs_real_t  *tprenc;//ncharm2
+  /* Limit temperature (in degree Celsius) below which the coal particles do
+     not cause any fouling (if the fouling model is activated).
+     Useful if \ref physical_model = 2 and \ref iencra = 1*/
+  cs_real_t  *tprenc; // size: ncharm2
 
-  /*  ash critical viscosity in \f$ kg.m^{-1}.s^{-1} \f$, in the fouling model
-   cf J.D. Watt et T. Fereday (J.Inst.Fuel, Vol.42-p99).
-   Useful if \ref physical_model = 2 and \ref iencra = 1*/
-//TODO
-  cs_real_t  *visref;//ncharm2
+  /* Ash critical viscosity in \f$ kg.m^{-1}.s^{-1} \f$, in the fouling model
+     cf J.D. Watt et T. Fereday (J.Inst.Fuel, Vol.42-p99).
+     Useful if \ref physical_model = 2 and \ref iencra = 1*/
+  cs_real_t  *visref;  // size: ncharm2
 
-  /*  encrustation data */
+  /* encrustation data */
   cs_real_t  dnpenc;
 
 } cs_lagr_encrustation_t;
@@ -714,13 +822,10 @@ typedef struct {
 
 typedef struct {
 
-  /*! number of additional user data to record for the calculation
-    of additional boundary statistics in \ref bound_stat */
-  int  nusbor;
-
-  /*! number of iterations during which steady boundary statistics have
+  /*! Number of iterations during which steady boundary statistics have
     been accumulated.
-    Useful if \ref isttio=1 and \ref nstist inferior
+    Useful if \ref cs_lagr_time_scheme_t::isttio "isttio"=1 and
+    \ref cs_lagr_stat_options_t::nstist "nstist" inferior
     or equal to the current time step.
     \ref npstf is initialized and updated automatically by the code,
     its value is not to be modified by the user */
@@ -736,96 +841,17 @@ typedef struct {
 
   /*! activation (=1) or not (=0) of the recording of the number of
     particle/boundary interactions, and of the calculation of the associated
-    boundary statistics.
-    \ref has_part_impact_nbr is set to 1 when using the particulate average
-    \ref imoybr = 2. */
+    boundary statistics. */
   int  has_part_impact_nbr;
 
-  /*!  activation (=1) or not (=0) of the recording of the particulate mass flow
-    related to the particle/boundary interactions, and of the calculation of
-    the associated boundary statistics.
-    \ref has_part_impact_nbr is set to 1 when using \ref iflmbd=1.
-   */
-  int  iflmbd;
-
-  /*!  activation (=1) or not (=0) of the recording of the angle between a
-    particle trajectory and a boundary face involved in a particle/boundary
-    interaction, and of the calculation of the associated boundary statistics. */
-  int  iangbd;
-
-  /*!  activation (=1) or not (=0) of the recording of the velocity of a particle
-    involved in a particle/boundary interaction, and of the calculation of
-    the associated boundary statistics. */
-  int  ivitbd;
-
   /*!  activation (=1) or not (=0) of the recording of clogging parameters
     involved in a particle/boundary interaction, and of the calculation of
     the associated boundary statistics. */
   int  iclgst;
 
-  /*! flag for number of recorded particle/boundary interactions with fouling */
-  int  iencnbbd;
-
-  /*! flag for mass of fouled coal particles */
-  int  iencmabd;
-
-  /*! flag for diameter of fouled coal particles */
-  int  iencdibd;
-
-  /*! flag for coke fraction of fouled coal particles */
-  int  iencckbd;
-
   /*!  id for number of particle/boundary interactions */
   int  inbr;
 
-  /*!  id for particle mass flow at the boundary faces */
-  int  iflm;
-
-  /*!  id for mean interaction angle with the boundary faces */
-  int  iang;
-
-  /*!  id for mean interaction velocity with the boundary faces */
-  int  ivit;
-
-  /*!  id for number of resuspended particles */
-  int  ires;
-
-  /*!  id for mass flow of resuspended particles at the boundary faces */
-  int  iflres;
-
-  /*! flag for number of recorded particle/boundary interactions with fouling */
-  int  iencnb;
-
-  /*! id for mass of fouled coal particles */
-  int  iencma;
-
-  /*! id for diameter of fouled coal particles */
-  int  iencdi;
-
-  /*! id for coke fraction of fouled coal particles */
-  int  iencck;
-
-  /*!  supplementary user boundary statistics */
-  int  *iusb;
-
-  /*! the recordings in \ref bound_stat at every particle/boundary interaction are
-      cumulated values (possibly reset to zero at every iteration in the
-      unsteady case). They must therefore be divided by a quantity to
-      get boundary statistics. The user can choose between two average types:
-      - = 0: no average is applied to the recorded cumulated values.
-      - = 1: a time-average is calculated. The cumulated value
-             is divided by the physical duration in the case of steady
-             averages (\ref isttio=1). The cumulated value is divided by the
-             value of the last time step in the case of unsteady averages
-             (\ref isttio=0), and also in the case of steady averages while the
-             absolute iteration number is inferior to \ref nstist.
-      - = 2: a particulate average is calculated. The cumulated value is divided
-             by the number of particle/boundary interactions (in terms of
-             statistical weight) recorded in \ref bound_stat "bound_stat"(nfabor,inbr).
-             This average can only be calculated when \ref has_part_impact_nbr = 1.
-             Only the cumulated value is recorded in the restart file. */
-  int  *imoybr;
-
   /*! id for number of deposited particles */
   int  inclg;
 
@@ -864,10 +890,10 @@ typedef struct {
   /*!  name of the boundary statistics, displayed in the log
     and the post-processing files.
     Warning: this name is also used to reference information in the restart
-    file (\ref isuist =1). If the name of a variable is changed between two
+    file. If the name of a variable is changed between two
     calculations, it will not be possible to read its value from the restart
     file */
-  char           **nombrd;
+  char  **nombrd;
 
 } cs_lagr_boundary_interactions_t;
 
@@ -905,7 +931,7 @@ typedef struct {
      ----------------- */
 
   /* wall ustar */
-  cs_real_t *uetbor;
+  cs_field_t *ustar;
 
   /* Fluid density */
   cs_field_t *cromf;
@@ -916,7 +942,6 @@ typedef struct {
   /* Fluid temparature */
   cs_field_t *scal_t;
   cs_field_t *temperature;
-  cs_field_t *t_gaz;
 
   /* Fluid velocity */
   cs_field_t *vel;
@@ -958,6 +983,12 @@ typedef struct {
   /* Reynolds Stress Tensor */
   cs_field_t *cvar_rij;
 
+  /* Total pressure gradient */
+  cs_real_3_t *grad_pr;
+
+  /* velocity gradient */
+  cs_real_33_t *grad_vel;
+
 } cs_lagr_extra_module_t;
 
 /*! External data relative to coal combustion */
@@ -1031,6 +1062,11 @@ extern cs_lagr_specific_physics_t            *cs_glob_lagr_specific_physics;
 extern cs_lagr_reentrained_model_t           *cs_glob_lagr_reentrained_model;
 extern cs_lagr_precipitation_model_t         *cs_glob_lagr_precipitation_model;
 extern cs_lagr_clogging_model_t              *cs_glob_lagr_clogging_model;
+extern cs_lagr_shape_model_t                 *cs_glob_lagr_shape_model;
+
+extern cs_lagr_agglomeration_model_t         *cs_glob_lagr_agglomeration_model;
+extern cs_lagr_fragmentation_model_t         *cs_glob_lagr_fragmentation_model;
+
 extern cs_lagr_consolidation_model_t         *cs_glob_lagr_consolidation_model;
 extern cs_lagr_time_step_t                   *cs_glob_lagr_time_step;
 extern cs_lagr_source_terms_t                *cs_glob_lagr_source_terms;
@@ -1046,9 +1082,6 @@ extern const cs_lagr_zone_data_t             *cs_glob_lagr_boundary_conditions;
 extern const cs_lagr_zone_data_t             *cs_glob_lagr_volume_conditions;
 extern cs_lagr_internal_condition_t          *cs_glob_lagr_internal_conditions;
 
-/* Unit normals and offsets of boundary faces */
-extern cs_real_4_t   *cs_glob_lagr_b_u_normal;
-
 /* Projection matrices for global to local coordinates on boundary faces */
 extern cs_real_33_t  *cs_glob_lagr_b_face_proj;
 
@@ -1148,6 +1181,24 @@ cs_get_lagr_precipitation_model(void);
 cs_lagr_clogging_model_t *
 cs_get_lagr_clogging_model(void);
 
+/*----------------------------------------------------------------------------
+ * Provide access to cs_lagr_shape_model_t
+ *
+ * needed to initialize structure with GUI
+ *----------------------------------------------------------------------------*/
+
+cs_lagr_shape_model_t *
+cs_get_lagr_shape_model(void);
+
+/*----------------------------------------------------------------------------
+ * Provide access to cs_lagr_agglomeration_model_t
+ *
+ * needed to initialize structure with GUI
+ *----------------------------------------------------------------------------*/
+
+cs_lagr_agglomeration_model_t *
+cs_get_lagr_agglomeration_model(void);
+
 /*----------------------------------------------------------------------------
  * Provide access to cs_lagr_consolidation_model_t
  *
