@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -55,7 +55,6 @@
 #include "cs_mesh.h"
 #include "cs_mesh_quantities.h"
 #include "cs_parall.h"
-#include "cs_prototypes.h"
 #include "cs_random.h"
 #include "cs_search.h"
 #include "cs_halo.h"
@@ -232,7 +231,8 @@ cs_lagr_clogging_barrier(const void                     *particle,
 
   cs_real_t mean_nb_cont;
 
-  cs_lnum_t  dim_aux = 1, contact_count[1];
+  cs_lnum_t  dim_aux = 1;
+  int        contact_count[1];
   cs_real_t  value;
   cs_lnum_t  i;
 
