@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -50,12 +50,14 @@
 
 #include "cs_base.h"
 #include "cs_math.h"
-#include "cs_prototypes.h"
+#include "cs_mesh.h"
+#include "cs_mesh_quantities.h"
 
 #include "bft_mem.h"
 #include "bft_error.h"
 
 #include "cs_physical_constants.h"
+#include "cs_time_step.h"
 
 #include "cs_lagr.h"
 #include "cs_lagr_particle.h"
@@ -83,56 +85,43 @@ static const cs_real_t  _c_stephan = 5.6703e-8;
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*============================================================================
- * Public function definitions for Fortran API
+ * Public function definitions
  *============================================================================*/
 
-/* --------------------------------------------------------------------
- *     CALCUL DES TERMES SOURCES DU COUPLAGE RETOUR
- *     Remarque : les termes sources sont calcules pour
- *                la cellule de depart de la particule
- *                lors de l'iteration courante. Attention, meme
- *                si la particule est sortante du domaine de
- *                calcul (peu importe la maniere) on doit calculer
- *                un terme source qui correspond a ce qu'echange le
- *                fluide porteur et la particule au debut du pas de
- *                temps. Si NORDRE = 2 et que la particule est en
- *                interaction avec la frontiere, alors les termes
- *                source sont calcules comme si NORDRE=1
- *                (on oublie le pre-remplissage de TSFEXT dans
- * ONFC                 LAGES2).
- * --------------------------------------------------------------------
- * Arguments
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute source terms for Lagrangian 2-way coupling.
  *
- *   ntersl            <--  nbr termes sources de couplage retour
+ * \remark  Source terms are computed for the starting cell of a particle
+ *          during a given iteration. Even if particle exits the domain,
+ *          it s necessary to compute a source term matching the exchange
+ *          between the carrier fluid and the particle at the beginning
+ *          of the time step. If cs_glob_lagr_time_step->nor == 2 and the
+ *          particle interacts with a boundary, then the source terms
+ *          are computed as if nor == 1.
  *
- *   taup(nbpart)      <--  temps caracteristique dynamique
- *   tsfext(nbpart)    <--  forces externes
- *   tempct            <--  temps caracteristique thermique
- *    (nbpart,2)
- *   cpgd1,cpgd2,      <--  termes de devolatilisation 1 et 2 et
- *    cpght(nbpart)           de combusion heterogene (charbon
- *                            avec couplage retour thermique)
- *   volp(ncelet)      ---  fraction volumique des particules
- *   volm(ncelet)      ---  fraction massique des particules
- * --------------------------------------------------------------------   */
+ * \param[in]   taup    dynamic characteristic time
+ * \param[in]   tempct  thermal charactersitic time
+ * \param[out]  tsfext  external forces
+ * \param[in]   cpgd1   devolatization term 1 for heterogeneous coal
+ * \param[in]   cpgd2   devolatization term 2 for heterogeneous coal
+ * \param[in]   cpght   combustion term for heterogeneous coal
+ */
+/*----------------------------------------------------------------------------*/
 
 void
-cs_lagr_coupling(cs_real_t taup[],
-                 cs_real_t tempct[],
-                 cs_real_t tsfext[],
-                 cs_real_t cpgd1[],
-                 cs_real_t cpgd2[],
-                 cs_real_t cpght[],
-                 cs_real_t volp[],
-                 cs_real_t volm[])
+cs_lagr_coupling(const cs_real_t  taup[],
+                 const cs_real_t  tempct[],
+                 cs_real_t        tsfext[],
+                 const cs_real_t  cpgd1[],
+                 const cs_real_t  cpgd2[],
+                 const cs_real_t  cpght[])
 {
   cs_real_3_t *st_vel = NULL, *t_st_vel = NULL;
   cs_real_6_t *st_rij = NULL, *t_st_rij = NULL;
 
-  /* ====================================================================   */
-  /* 1. INITIALISATION    */
-  /* ====================================================================   */
-
+  /* Initialization
+     ============== */
   {
     cs_field_t *f = cs_field_by_name_try("velocity_st_lagr");
     if (f != NULL)
@@ -164,12 +153,12 @@ cs_lagr_coupling(cs_real_t taup[],
   cs_lnum_t ntersl = cs_glob_lagr_dim->ntersl;
 
   cs_real_t *tslag, *auxl1, *auxl2, *auxl3;
-  BFT_MALLOC (tslag, ncelet * ntersl, cs_real_t);
-  BFT_MALLOC (auxl1, nbpart, cs_real_t);
-  BFT_MALLOC (auxl2, nbpart, cs_real_t);
-  BFT_MALLOC (auxl3, nbpart, cs_real_t);
+  BFT_MALLOC(tslag, ncelet * ntersl, cs_real_t);
+  BFT_MALLOC(auxl1, nbpart, cs_real_t);
+  BFT_MALLOC(auxl2, nbpart, cs_real_t);
+  BFT_MALLOC(auxl3, nbpart, cs_real_t);
 
-  /*   Nombre de passage pour les termes sources en stationnaire  */
+  /* Number of passes for steady source terms */
   if (   cs_glob_lagr_time_scheme->isttio == 1
       && cs_glob_time_step->nt_cur >= cs_glob_lagr_source_terms->nstits)
     lag_st->npts += 1;
@@ -178,9 +167,12 @@ cs_lagr_coupling(cs_real_t taup[],
   cs_glob_lagr_source_terms->vmax = 0.0;
   cs_glob_lagr_source_terms->tmamax = 0.0;
 
+  cs_real_t *volp = NULL, *volm = NULL;
+  BFT_MALLOC(volp, ncel, cs_real_t);
+  BFT_MALLOC(volm, ncel, cs_real_t);
   for (cs_lnum_t iel = 0; iel < ncel; iel++) {
-    volp[iel]      = 0.0;
-    volm[iel]      = 0.0;
+    volp[iel] = 0.0;
+    volm[iel] = 0.0;
   }
 
   for (cs_lnum_t ivar = 0; ivar < ntersl; ivar++) {
@@ -190,11 +182,11 @@ cs_lagr_coupling(cs_real_t taup[],
 
   }
 
-  /* ====================================================================   */
-  /* 2. CALCULS PRELIMINAIRES  */
-  /* ====================================================================   */
-  /* Finalisation des forces externes (Si la particule a interagit avec     */
-  /* une frontiere du domaine de calcul, on degenere a l'ordre 1).     */
+  /* Preliminary computations
+     ======================== */
+
+  /* Finalization of external forces (if the particle interacts with a
+     domain boundary, revert to order 1). */
 
   for (cs_lnum_t npt = 0; npt < nbpart; npt++) {
 
@@ -213,12 +205,15 @@ cs_lagr_coupling(cs_real_t taup[],
 
   for (cs_lnum_t npt = 0; npt < nbpart; npt++) {
 
-    cs_real_t  p_stat_w = cs_lagr_particles_get_real(p_set, npt, CS_LAGR_STAT_WEIGHT);
+    cs_real_t  p_stat_w = cs_lagr_particles_get_real(p_set, npt,
+                                                     CS_LAGR_STAT_WEIGHT);
     cs_real_t  p_mass   = cs_lagr_particles_get_real(p_set, npt, CS_LAGR_MASS);
     cs_real_t *p_vel    = cs_lagr_particles_attr(p_set, npt, CS_LAGR_VELOCITY);
 
-    cs_real_t  prev_p_mass = cs_lagr_particles_get_real_n(p_set, npt, 1, CS_LAGR_MASS);
-    cs_real_t *prev_p_vel  = cs_lagr_particles_attr_n(p_set, npt, 1, CS_LAGR_VELOCITY);
+    cs_real_t  prev_p_mass = cs_lagr_particles_get_real_n(p_set, npt, 1,
+                                                          CS_LAGR_MASS);
+    cs_real_t *prev_p_vel  = cs_lagr_particles_attr_n(p_set, npt, 1,
+                                                      CS_LAGR_VELOCITY);
 
     auxl1[npt] = p_stat_w * (p_mass * p_vel[0] - prev_p_mass * prev_p_vel[0]
                                                - grav[0] * tsfext[npt]) / dtp;
@@ -229,9 +224,8 @@ cs_lagr_coupling(cs_real_t taup[],
 
   }
 
-  /* ====================================================================   */
-  /* 3. TERMES SOURCES DE QUANTITE DE MOUVEMENT    */
-  /* ====================================================================   */
+  /* Momentum source terms
+     ===================== */
 
   if (cs_glob_lagr_source_terms->ltsdyn == 1) {
 
@@ -259,35 +253,37 @@ cs_lagr_coupling(cs_real_t taup[],
       cs_real_t  p_mass = cs_lagr_particle_get_real(particle, p_am,
                                                     CS_LAGR_MASS);
 
-      cs_lnum_t iel = cs_lagr_particle_get_cell_id(particle, p_am);
+      cs_lnum_t iel = cs_lagr_particle_get_lnum(particle, p_am, CS_LAGR_CELL_ID);
 
-      /* Volume et masse des particules dans la maille */
+      /* Volume and mass of particles in cell */
       volp[iel] += p_stat_w * cs_math_pi * pow(prev_p_diam, 3) / 6.0;
       volm[iel] += p_stat_w * prev_p_mass;
 
-      /* TS de QM   */
+      /* Momentum source term */
       t_st_vel[iel][0] += - auxl1[npt];
       t_st_vel[iel][1] += - auxl2[npt];
       t_st_vel[iel][2] += - auxl3[npt];
-      tslag[iel + (lag_st->itsli-1) * ncelet] += - 2.0 * p_stat_w * p_mass / taup[npt];
+      tslag[iel + (lag_st->itsli-1) * ncelet]
+        += - 2.0 * p_stat_w * p_mass / taup[npt];
 
     }
 
-    /* ====================================================================   */
-    /* 4. TERMES SOURCES SUR LA TURBULENCE */
-    /* ====================================================================   */
+  /* Turbulence source terms
+     ======================= */
 
-    if (extra->itytur == 2 || extra->iturb == 50 || extra->iturb == 60) {
+    if (extra->itytur == 2 || extra->itytur == 4 ||
+        extra->iturb == 50 || extra->iturb == 60) {
 
-      /* En v2f (ITURB=50) les TS lagrangiens influent uniquement sur k et eps  */
-      /* (difficile d'ecrire quoi que ce soit sur v2, qui perd son sens de */
-      /*  "composante de Rij")     */
+      /* In v2f (ITURB=50) the Lagrangian STs only influence k and epsilon
+         (difficult to write something for v2, which loses its meaning as
+         "Rij comonent") */
 
       for (cs_lnum_t npt = 0; npt < nbpart; npt++) {
 
         unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
 
-        cs_lnum_t  iel         = cs_lagr_particle_get_cell_id(particle, p_am);
+        cs_lnum_t  iel         = cs_lagr_particle_get_lnum(particle, p_am,
+                                                           CS_LAGR_CELL_ID);
         cs_real_t *prev_f_vel  = cs_lagr_particle_attr_n(particle, p_am, 1,
                                                          CS_LAGR_VELOCITY_SEEN);
         cs_real_t *f_vel       = cs_lagr_particle_attr(particle, p_am,
@@ -326,7 +322,8 @@ cs_lagr_coupling(cs_real_t taup[],
 
         unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
 
-        cs_lnum_t  iel         = cs_lagr_particle_get_cell_id(particle, p_am);
+        cs_lnum_t  iel         = cs_lagr_particle_get_lnum(particle, p_am,
+                                                           CS_LAGR_CELL_ID);
 
         cs_real_t *prev_f_vel  = cs_lagr_particle_attr_n(particle, p_am, 1,
                                                          CS_LAGR_VELOCITY_SEEN);
@@ -371,9 +368,8 @@ cs_lagr_coupling(cs_real_t taup[],
 
   }
 
-  /* ====================================================================   */
-  /* 5. TERME SOURCE MASSIQUES */
-  /* ====================================================================   */
+  /* Mass source terms
+     ================= */
 
   if (    cs_glob_lagr_source_terms->ltsmas == 1
       && (   cs_glob_lagr_specific_physics->impvar == 1
@@ -383,43 +379,57 @@ cs_lagr_coupling(cs_real_t taup[],
 
       unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
 
-      cs_real_t  p_stat_w = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_STAT_WEIGHT);
-      cs_real_t  prev_p_mass = cs_lagr_particle_get_real_n(particle, p_am, 1, CS_LAGR_MASS);
-      cs_real_t  p_mass = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_MASS);
+      cs_real_t  p_stat_w
+        = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_STAT_WEIGHT);
+      cs_real_t  prev_p_mass
+        = cs_lagr_particle_get_real_n(particle, p_am, 1, CS_LAGR_MASS);
+      cs_real_t  p_mass
+        = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_MASS);
 
-      /* Dans saturne TSmasse > 0 ===> Apport de masse sur le fluide  */
-      cs_lnum_t iel = cs_lagr_particle_get_cell_id(particle, p_am);
+      /* Fluid mass source term > 0 -> add mass to fluid */
+      cs_lnum_t cell_id = cs_lagr_particle_get_lnum(particle, p_am,
+                                                    CS_LAGR_CELL_ID);
 
-      tslag[iel + (lag_st->itsmas-1) * ncelet] += - p_stat_w * (p_mass - prev_p_mass) / dtp;
+      tslag[cell_id + (lag_st->itsmas-1) * ncelet]
+        += - p_stat_w * (p_mass - prev_p_mass) / dtp;
 
     }
 
   }
 
-  /* ====================================================================   */
-  /* 6. TERMES SOURCES THERMIQUE    */
-  /* ====================================================================   */
+  /* Thermal source terms
+     ==================== */
 
   if (cs_glob_lagr_source_terms->ltsthe == 1) {
 
-    if (   cs_glob_lagr_model->physical_model == 1
+    if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
         && cs_glob_lagr_specific_physics->itpvar == 1) {
 
       for (cs_lnum_t npt = 0; npt < nbpart; npt++) {
 
         unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
-        cs_lnum_t iel = cs_lagr_particle_get_cell_id(particle, p_am);
-        cs_real_t  p_mass = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_MASS);
-        cs_real_t  prev_p_mass = cs_lagr_particle_get_real_n(particle, p_am, 1, CS_LAGR_MASS);
-        cs_real_t  p_cp = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_CP);
-        cs_real_t  prev_p_cp = cs_lagr_particle_get_real_n(particle, p_am, 1, CS_LAGR_CP);
-        cs_real_t  p_tmp = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_TEMPERATURE);
-        cs_real_t  prev_p_tmp = cs_lagr_particle_get_real_n(particle, p_am, 1, CS_LAGR_TEMPERATURE);
-        cs_real_t  p_stat_w = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_STAT_WEIGHT);
+        cs_lnum_t  iel = cs_lagr_particle_get_lnum(particle, p_am,
+                                                   CS_LAGR_CELL_ID);
+        cs_real_t  p_mass = cs_lagr_particle_get_real_n(particle, p_am, 0,
+                                                        CS_LAGR_MASS);
+        cs_real_t  prev_p_mass = cs_lagr_particle_get_real_n(particle, p_am, 1,
+                                                             CS_LAGR_MASS);
+        cs_real_t  p_cp = cs_lagr_particle_get_real_n(particle, p_am, 0,
+                                                      CS_LAGR_CP);
+        cs_real_t  prev_p_cp = cs_lagr_particle_get_real_n(particle, p_am, 1,
+                                                           CS_LAGR_CP);
+        cs_real_t  p_tmp = cs_lagr_particle_get_real_n(particle, p_am, 0,
+                                                       CS_LAGR_TEMPERATURE);
+        cs_real_t  prev_p_tmp = cs_lagr_particle_get_real_n(particle, p_am, 1,
+                                                            CS_LAGR_TEMPERATURE);
+        cs_real_t  p_stat_w = cs_lagr_particle_get_real(particle, p_am,
+                                                        CS_LAGR_STAT_WEIGHT);
 
-        tslag[iel + (lag_st->itste-1) * ncelet] += - (p_mass * p_tmp * p_cp
-                                - prev_p_mass * prev_p_tmp * prev_p_cp) / dtp * p_stat_w;
-        tslag[iel + (lag_st->itsti-1) * ncelet] += tempct[nbpart + npt] * p_stat_w;
+        tslag[iel + (lag_st->itste-1) * ncelet]
+          += - (p_mass * p_tmp * p_cp
+             - prev_p_mass * prev_p_tmp * prev_p_cp) / dtp * p_stat_w;
+        tslag[iel + (lag_st->itsti-1) * ncelet]
+          += tempct[nbpart + npt] * p_stat_w;
 
       }
       if (extra->radiative_model > 0) {
@@ -427,14 +437,20 @@ cs_lagr_coupling(cs_real_t taup[],
         for (cs_lnum_t npt = 0; npt < nbpart; npt++) {
 
           unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
-          cs_lnum_t iel = cs_lagr_particle_get_cell_id(particle, p_am);
-          cs_real_t  p_diam = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_DIAMETER);
-          cs_real_t  p_eps = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_EMISSIVITY);
-          cs_real_t  p_tmp = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_TEMPERATURE);
-          cs_real_t  p_stat_w = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_STAT_WEIGHT);
+          cs_lnum_t  iel = cs_lagr_particle_get_lnum(particle, p_am,
+                                                     CS_LAGR_CELL_ID);
+          cs_real_t  p_diam = cs_lagr_particle_get_real_n(particle, p_am, 0,
+                                                          CS_LAGR_DIAMETER);
+          cs_real_t  p_eps = cs_lagr_particle_get_real_n(particle, p_am, 0,
+                                                         CS_LAGR_EMISSIVITY);
+          cs_real_t  p_tmp = cs_lagr_particle_get_real_n(particle, p_am, 0,
+                                                         CS_LAGR_TEMPERATURE);
+          cs_real_t  p_stat_w = cs_lagr_particle_get_real(particle, p_am,
+                                                          CS_LAGR_STAT_WEIGHT);
 
           cs_real_t aux1 = cs_math_pi * p_diam * p_diam * p_eps
-                          * (extra->luminance->val[iel] - 4.0 * _c_stephan * pow (p_tmp, 4));
+                          * (extra->luminance->val[iel]
+                             - 4.0 * _c_stephan * cs_math_pow4(p_tmp));
 
           tslag[iel + (lag_st->itste-1) * ncelet] += aux1 * p_stat_w;
 
@@ -443,10 +459,10 @@ cs_lagr_coupling(cs_real_t taup[],
       }
 
     }
-    else if (cs_glob_lagr_model->physical_model == 2) {
+    else if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
       if (cs_glob_lagr_const_dim->nlayer > 1)
         bft_error(__FILE__, __LINE__, 0,
-                  _("Couplage thermique non-fonctionnel en multi-layer"));
+                  _("Thermal coupling not implemented in multi-layer case"));
 
       else {
 
@@ -454,25 +470,39 @@ cs_lagr_coupling(cs_real_t taup[],
 
           unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
 
-          cs_lnum_t iel  = cs_lagr_particle_get_cell_id(particle, p_am);
-          cs_lnum_t icha = cs_lagr_particle_get_lnum(particle, p_am, CS_LAGR_COAL_ID);
+          cs_lnum_t  iel = cs_lagr_particle_get_lnum(particle, p_am,
+                                                     CS_LAGR_CELL_ID);
+          cs_lnum_t icha = cs_lagr_particle_get_lnum(particle, p_am,
+                                                     CS_LAGR_COAL_ID);
 
-          cs_real_t  p_mass = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_MASS);
-          cs_real_t  p_tmp = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_TEMPERATURE);
-          cs_real_t  p_cp = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_CP);
+          cs_real_t  p_mass = cs_lagr_particle_get_real_n(particle, p_am, 0,
+                                                          CS_LAGR_MASS);
+          cs_real_t  p_tmp = cs_lagr_particle_get_real(particle, p_am,
+                                                       CS_LAGR_TEMPERATURE);
+          cs_real_t  p_cp = cs_lagr_particle_get_real_n(particle, p_am, 0,
+                                                        CS_LAGR_CP);
 
-          cs_real_t  prev_p_mass = cs_lagr_particle_get_real_n(particle, p_am, 1, CS_LAGR_MASS);
-          cs_real_t  prev_p_tmp  = cs_lagr_particle_get_real_n(particle, p_am, 1, CS_LAGR_TEMPERATURE);
-          cs_real_t  prev_p_cp   = cs_lagr_particle_get_real_n(particle, p_am, 1, CS_LAGR_CP);
+          cs_real_t  prev_p_mass = cs_lagr_particle_get_real_n
+                                     (particle, p_am, 1, CS_LAGR_MASS);
+          cs_real_t  prev_p_tmp  = cs_lagr_particle_get_real_n
+                                     (particle, p_am, 1, CS_LAGR_TEMPERATURE);
+          cs_real_t  prev_p_cp   = cs_lagr_particle_get_real_n
+                                     (particle, p_am, 1, CS_LAGR_CP);
 
-          cs_real_t  p_stat_w = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_STAT_WEIGHT);
+          cs_real_t  p_stat_w = cs_lagr_particle_get_real
+                                  (particle, p_am, CS_LAGR_STAT_WEIGHT);
 
-          tslag[iel + (lag_st->itste-1) * ncelet]  += - (  p_mass * p_tmp * p_cp
-                                             - prev_p_mass * prev_p_tmp * prev_p_cp) / dtp * p_stat_w;
-          tslag[iel + (lag_st->itsti-1) * ncelet] += tempct[nbpart + npt] * p_stat_w;
-          tslag[iel + (lag_st->itsmv1[icha]-1) * ncelet] += p_stat_w * cpgd1[npt];
-          tslag[iel + (lag_st->itsmv2[icha]-1) * ncelet] += p_stat_w * cpgd2[npt];
-          tslag[iel + (lag_st->itsco-1) * ncelet] += p_stat_w * cpght[npt];
+          tslag[iel + (lag_st->itste-1) * ncelet]
+            += - (  p_mass * p_tmp * p_cp
+               - prev_p_mass * prev_p_tmp * prev_p_cp) / dtp * p_stat_w;
+          tslag[iel + (lag_st->itsti-1) * ncelet]
+            += tempct[nbpart + npt] * p_stat_w;
+          tslag[iel + (lag_st->itsmv1[icha]-1) * ncelet]
+            += p_stat_w * cpgd1[npt];
+          tslag[iel + (lag_st->itsmv2[icha]-1) * ncelet]
+            += p_stat_w * cpgd2[npt];
+          tslag[iel + (lag_st->itsco-1) * ncelet]
+            += p_stat_w * cpght[npt];
           tslag[iel + (lag_st->itsfp4-1) * ncelet] = 0.0;
 
         }
@@ -483,10 +513,9 @@ cs_lagr_coupling(cs_real_t taup[],
 
   }
 
-  /* ====================================================================   */
-  /* 7. Verif que le taux volumique maximal TVMAX admissible de particules  */
-  /*    ne soit pas depasse dans quelques cellules.     */
-  /* ====================================================================   */
+  /* Check that the maximum admissible volume fraction of particles
+     is not exceeded in some cells.
+     ============================================================== */
 
   cs_real_t *st_val = cs_glob_lagr_source_terms->st_val; /* short alias */
 
@@ -525,9 +554,8 @@ cs_lagr_coupling(cs_real_t taup[],
 
   }
 
-  /* ====================================================================   */
-  /* 8. MOYENNE TEMPORELLE DES TERMES SOURCES */
-  /* ====================================================================   */
+  /* Time average of source terms
+     ============================ */
 
   if (cs_glob_lagr_time_scheme->isttio == 1 && lag_st->npts > 0) {
 
@@ -577,6 +605,9 @@ cs_lagr_coupling(cs_real_t taup[],
   if (t_st_rij != st_rij)
     BFT_FREE(t_st_rij);
 
+  BFT_FREE(volp);
+  BFT_FREE(volm);
+
   BFT_FREE(auxl1);
   BFT_FREE(auxl2);
   BFT_FREE(auxl3);
