@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,6 +54,7 @@
 #include "cs_boundary_zone.h"
 #include "cs_volume_zone.h"
 
+#include "cs_ht_convert.h"
 #include "cs_mesh.h"
 #include "cs_mesh_quantities.h"
 #include "cs_parall.h"
@@ -61,19 +62,16 @@
 #include "cs_parameters.h"
 #include "cs_physical_model.h"
 #include "cs_physical_constants.h"
+#include "cs_prototypes.h"
 #include "cs_time_step.h"
 
 #include "cs_field.h"
 #include "cs_field_pointer.h"
 #include "cs_random.h"
-#include "cs_prototypes.h"
-
-#include "cs_gui_particles.h"
-#include "cs_gui_util.h"
 
 #include "cs_lagr.h"
+#include "cs_lagr_gradients.h"
 #include "cs_lagr_tracking.h"
-#include "cs_lagr_geom.h"
 #include "cs_lagr_new.h"
 #include "cs_lagr_precipitation_model.h"
 #include "cs_lagr_prototypes.h"
@@ -352,12 +350,36 @@ _injection_check(const cs_lagr_injection_set_t  *zis)
                 set_id);
   }
 
+  if (cs_glob_lagr_model->agglomeration == 1) {
+    if (zis->aggregat_class_id < 1)
+      bft_error(__FILE__, __LINE__, 0,
+                _("Lagrangian module: \n"
+                  "  id of the class of aggregates = %d is \n"
+                  " either not defined (negative)\n"
+                  "  or smaller than 1 \n"
+                  "  for zone %d and set %d."),
+                (int)zis->aggregat_class_id,
+                z_id,
+                set_id);
+    if (zis->aggregat_fractal_dim > 3.)
+      bft_error(__FILE__, __LINE__, 0,
+                _("Lagrangian module: \n"
+                  "  value of fractal dimension = %d for aggregates \n"
+                  " is either not defined (negative) \n"
+                  "  or greater than 3 \n"
+                  "  for zone %d and set %d."),
+                (int)zis->aggregat_fractal_dim,
+                z_id,
+                set_id);
+
+  }
+
   /* temperature */
-  if (   cs_glob_lagr_model->physical_model == 1
+  if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
       && (   cs_glob_lagr_specific_physics->itpvar == 1
           || cs_glob_lagr_specific_physics->idpvar == 1
           || cs_glob_lagr_specific_physics->impvar == 1)) {
-    if (zis->temperature_profile < 1 || zis->temperature_profile > 1)
+    if (zis->temperature_profile < 0 || zis->temperature_profile > 1)
       bft_error(__FILE__, __LINE__, 0, _profile_err_fmt_i,
                 z_type_name, z_id, set_id,
                 _("temperature"), (int)zis->temperature_profile);
@@ -365,13 +387,14 @@ _injection_check(const cs_lagr_injection_set_t  *zis)
 
   /* velocity */
   if (   zis->location_id != CS_MESH_LOCATION_BOUNDARY_FACES
-      && zis->velocity_profile == 0)
+      && zis->velocity_profile == CS_LAGR_IN_IMPOSED_NORM)
     bft_error(__FILE__, __LINE__, 0,
               _("Lagrangian %s zone %d, set %d:\n"
-                " velocity profile type 0 may not be used\n"
+                " velocity profile type CS_LAGR_IN_IMPOSED_NORM may not be used\n"
                 " for volume zones, as it requires surface normals."),
               z_type_name, z_id, set_id);
-  else if (zis->velocity_profile <  -1 || zis->velocity_profile > 1)
+  else if (zis->velocity_profile < CS_LAGR_IN_IMPOSED_FLUID_VALUE
+      ||   zis->velocity_profile > CS_LAGR_IN_IMPOSED_COMPONENTS)
     bft_error(__FILE__, __LINE__, 0, _profile_err_fmt_i,
               z_type_name, z_id, set_id,
               _("velocity"), (int)zis->velocity_profile);
@@ -392,7 +415,7 @@ _injection_check(const cs_lagr_injection_set_t  *zis)
               (double)zis->flow_rate);
 
   /* particle properties: diameter, variance, and rho */
-  if (cs_glob_lagr_model->physical_model != 2) {
+  if (cs_glob_lagr_model->physical_model != CS_LAGR_PHYS_COAL) {
     if (   zis->density  < 0.0
         || zis->diameter < 0.0
         || zis->diameter_variance < 0.0)
@@ -418,23 +441,44 @@ _injection_check(const cs_lagr_injection_set_t  *zis)
               (double)zis->diameter,
               (double)zis->diameter_variance);
 
+  /* Ellipsoidal particle properties: radii */
+  const int shape = cs_glob_lagr_model->shape;
+  if (shape == CS_LAGR_SHAPE_SPHEROID_JEFFERY_MODEL) {
+    if (   zis->radii[0] < 0.0
+        || zis->radii[1] < 0.0
+        || zis->radii[2] < 0.0)
+      bft_error
+        (__FILE__, __LINE__, 0,
+         _("Lagrangian %s zone %d, set %d:\n"
+           "  error on particle properties definition:\n"
+           "  Ellispoid radii = %g, %g, %g\n"
+           "This may lead to injection of  particles with negative radii."),
+         z_type_name, z_id, set_id,
+         (double)zis->radii[0],
+         (double)zis->radii[1],
+         (double)zis->radii[2]);
+  }
+
   /* temperature and Cp */
-  if (   cs_glob_lagr_model->physical_model == 1
+  if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
       && cs_glob_lagr_specific_physics->itpvar == 1) {
     cs_real_t tkelvn = -cs_physical_constants_celsius_to_kelvin;
-    if (zis->cp < 0.0 || zis->temperature < tkelvn)
+    if (zis->cp < 0.0)
       bft_error(__FILE__, __LINE__, 0,
                 _("Lagrangian %s zone %d, set %d:\n"
-                  "  specific heat capacity (%g) is negative\n"
-                  "  or temperature (%g) is lower than %g."),
+                  "  specific heat capacity (%g) is negative."),
+                z_type_name, z_id, set_id, (double)zis->cp);
+    if (zis->temperature_profile > 0 && zis->temperature < tkelvn)
+      bft_error(__FILE__, __LINE__, 0,
+                _("Lagrangian %s zone %d, set %d:\n"
+                  "  temperature (%g) is lower than %g."),
                 z_type_name, z_id, set_id,
-                (double)zis->cp,
                 (double)zis->temperature,
                 (double)tkelvn);
   }
 
   /* emissivity */
-  if (   cs_glob_lagr_model->physical_model == 1
+  if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
       && cs_glob_lagr_specific_physics->itpvar == 1
       && extra->radiative_model > 0) {
 
@@ -448,8 +492,7 @@ _injection_check(const cs_lagr_injection_set_t  *zis)
   }
 
   /* Coal */
-
-  if (cs_glob_lagr_model->physical_model == 2) {
+  if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
 
     cs_real_t tkelvi = cs_physical_constants_celsius_to_kelvin;
 
@@ -566,7 +609,7 @@ _get_particle_face_ids(cs_lnum_t         n_faces,
  * \brief Initialize particle values
  *
  * \param[in,out]  p_set             particle set
- * \param[in]      zis               injection data this zone and set
+ * \param[in]      zis               injection data for this zone and set
  * \param[in]      time_id           time step indicator for fields
  *                                     0: use fields at current time step
  *                                     1: use fields at previous time step
@@ -600,33 +643,41 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
 
   const cs_real_t *vela = extra->vel->vals[time_id];
   const cs_real_t *cval_h = NULL, *cval_t = NULL;
+  cs_real_t *_cval_t = NULL;
+
   cs_real_t tscl_shift = 0;
 
   /* Initialize pointers (used to simplify future tests) */
 
-  if (   (   cs_glob_lagr_model->physical_model == 1
+  if (   (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
           && cs_glob_lagr_specific_physics->itpvar == 1)
-      || cs_glob_lagr_model->physical_model == 2) {
+      || cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
 
-    if (   cs_glob_physical_model_flag[CS_COMBUSTION_COAL] >= 0
-        || cs_glob_physical_model_flag[CS_COMBUSTION_PCLC] >= 0
-        || cs_glob_physical_model_flag[CS_COMBUSTION_FUEL] >= 0)
-      cval_t = cs_field_by_name("t_gas")->val;
+    const cs_field_t *f = cs_field_by_name_try("temperature");
+    if (f != NULL)
+      cval_t = f->val;
+    else if (cs_glob_thermal_model->itherm == CS_THERMAL_MODEL_ENTHALPY)
+      cval_h = cs_field_by_name("enthalpy")->val;
 
-    else {
-      const cs_field_t *f = cs_field_by_name_try("temperature");
-      if (f != NULL)
-        cval_t = f->val;
-      else if (   cs_glob_thermal_model->itherm
-               == CS_THERMAL_MODEL_ENTHALPY)
-        cval_h = cs_field_by_name("enthalpy")->val;
-    }
-
-    if (cs_glob_thermal_model->itpscl == 1) /* Kelvin */
+    if (cs_glob_thermal_model->itpscl == CS_TEMPERATURE_SCALE_KELVIN)
       tscl_shift = - cs_physical_constants_celsius_to_kelvin;
   }
 
   const cs_real_t pis6 = cs_math_pi / 6.0;
+  const int shape = cs_glob_lagr_model->shape;
+
+  /* Prepare  enthalpy to temperature conversion if needed */
+
+  if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
+      && cs_glob_lagr_specific_physics->itpvar == 1
+      && cval_t == NULL
+      && cval_h != NULL) {
+
+    BFT_MALLOC(_cval_t, cs_glob_mesh->n_cells_with_ghosts, cs_real_t);
+    cs_ht_convert_h_to_t_cells(cval_h, _cval_t);
+    cval_t = _cval_t;
+
+  }
 
   /* Loop on zone elements where particles are injected */
 
@@ -648,7 +699,8 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
 
       unsigned char *particle = p_set->p_buffer + p_am->extents * p_id;
 
-      cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
+      cs_lnum_t cell_id = cs_lagr_particles_get_lnum(p_set, p_id,
+                                                     CS_LAGR_CELL_ID);
 
       /* Random value associated with each particle */
 
@@ -663,13 +715,13 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
                                                   CS_LAGR_VELOCITY);
 
       /* prescribed components */
-      if (zis->velocity_profile == 1) {
+      if (zis->velocity_profile == CS_LAGR_IN_IMPOSED_COMPONENTS) {
         for (cs_lnum_t i = 0; i < 3; i++)
           part_vel[i] = zis->velocity[i];
       }
 
       /* prescribed norm */
-      else if (zis->velocity_profile == 0) {
+      else if (zis->velocity_profile == CS_LAGR_IN_IMPOSED_NORM) {
         assert(face_id >= 0);
         for (cs_lnum_t i = 0; i < 3; i++)
           part_vel[i] = -   fvq->b_face_normal[face_id * 3 + i]
@@ -678,7 +730,7 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
       }
 
       /* velocity as seen from fluid */
-      else if (zis->velocity_profile ==  -1) {
+      else if (zis->velocity_profile == CS_LAGR_IN_IMPOSED_FLUID_VALUE) {
         for (cs_lnum_t i = 0; i < 3; i++)
           part_vel[i] = vela[cell_id * 3  + i];
       }
@@ -686,7 +738,7 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
       /* fluid velocity seen */
       cs_real_t *part_seen_vel = cs_lagr_particle_attr(particle, p_am,
                                                        CS_LAGR_VELOCITY_SEEN);
-      for (int i = 0; i < 3; i++)
+      for (cs_lnum_t i = 0; i < 3; i++)
         part_seen_vel[i] = vela[cell_id * 3 + i];
 
       /* Residence time (may be negative to ensure continuous injection) */
@@ -704,6 +756,177 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
       cs_lagr_particle_set_real(particle, p_am, CS_LAGR_DIAMETER,
                                 zis->diameter);
 
+      /* Shape for spheroids without inertia */
+      if (   shape == CS_LAGR_SHAPE_SPHEROID_STOC_MODEL
+          || shape == CS_LAGR_SHAPE_SPHEROID_JEFFERY_MODEL) {
+
+        /* Spherical radii a b c */
+        cs_real_t *radii = cs_lagr_particle_attr(particle, p_am,
+                                                 CS_LAGR_RADII);
+
+        for (cs_lnum_t i = 0; i < 3; i++) {
+          radii[i] = zis->radii[i];
+        }
+
+        /* Shape parameters */
+        cs_real_t *shape_param = cs_lagr_particle_attr(particle, p_am,
+                                                       CS_LAGR_SHAPE_PARAM);
+
+        /* Compute shape parameters from radii */
+        /* FIXME valid for all spheroids only (a = b, c > a,b ) */
+        cs_real_t lamb = radii[2] / radii[1];  /* FIXME do not divide by 0... */
+        cs_real_t lamb_m1 = (radii[2] - radii[1]) / radii[1];
+        cs_real_t _a2 = radii[0] * radii[0];
+        /* TODO MF shape_param check development in series */
+        cs_real_t aux1 = lamb * lamb;
+        cs_real_t aux2 = aux1 -1;
+        if (lamb_m1 > 1e-10) {
+          cs_real_t aux3 = sqrt(aux2 - 1);
+          cs_real_t kappa = -log(lamb + aux3);
+          shape_param[0] = aux1/aux2 + lamb*kappa/(aux2*aux3);
+          shape_param[1] = shape_param[0];
+          shape_param[2] = -2./aux2 - 2.*lamb*kappa/(aux2*aux3);
+          shape_param[3] = -2. * _a2 *lamb*kappa/aux3;
+        }
+        else if (lamb_m1 < -1e-10) {
+          cs_real_t aux3 = sqrt(1. - aux2);
+          cs_real_t kappa = acos(lamb);
+          shape_param[0] = aux1/aux2+lamb*kappa/(-aux2*aux3);
+          shape_param[1] = shape_param[0];
+          shape_param[2] = -2./aux2 - 2.*lamb*kappa/(-aux2*aux3);
+          shape_param[3] = 2. * _a2 * lamb*kappa/aux3;
+        }
+        else {
+          shape_param[0] = 2.0/3.0;
+          shape_param[1] = 2.0/3.0;
+          shape_param[2] = 2.0/3.0;
+          shape_param[3] = 2. * _a2;
+        }
+
+        if (shape == CS_LAGR_SHAPE_SPHEROID_STOC_MODEL) {
+          /* Orientation */
+          cs_real_t *orientation = cs_lagr_particle_attr(particle, p_am,
+                                                          CS_LAGR_ORIENTATION);
+          cs_real_t *quaternion = cs_lagr_particle_attr(particle, p_am,
+                                                          CS_LAGR_QUATERNION);
+          for (cs_lnum_t i = 0; i < 3; i++) {
+            orientation[i] = zis->orientation[i];
+          }
+
+          /* Compute orientation from uniform orientation on a unit-sphere */
+          cs_real_t theta0;
+          cs_real_t phi0;
+          cs_random_uniform(1, &theta0);
+          cs_random_uniform(1, &phi0);
+          theta0   = acos(2.0*theta0-1.0);
+          phi0     = phi0*2.0*cs_math_pi;
+          orientation[0] = sin(theta0)*cos(phi0);
+          orientation[1] = sin(theta0)*sin(phi0);
+          orientation[2] = cos(theta0);
+          /* Initialize quaternions */
+          quaternion[0] = 1.;
+          quaternion[1] = 0.;
+          quaternion[2] = 0.;
+          quaternion[3] = 0.;
+          /* TODO initialize other things */
+        }
+
+        if (shape == CS_LAGR_SHAPE_SPHEROID_JEFFERY_MODEL) {
+
+          /* Euler parameters */
+          cs_real_t *euler = cs_lagr_particle_attr(particle, p_am,
+                                                   CS_LAGR_EULER);
+
+          for (cs_lnum_t i = 0; i < 4; i++)
+            euler[i] = zis->euler[i];
+
+          /* Compute Euler angles
+             (random orientation with a uniform distribution in [-1;1]) */
+          cs_real_t trans_m[3][3];
+          /* Generate the first two vectors */
+          for (cs_lnum_t id = 0; id < 3; id++) {
+            cs_random_uniform(1, &trans_m[id][0]); /* (?,0) */
+            cs_random_uniform(1, &trans_m[id][1]); /* (?,1) */
+            cs_random_uniform(1, &trans_m[id][2]); /* (?,2) */
+            cs_real_3_t loc_vector =  {-1.+2*trans_m[id][0],
+              -1.+2*trans_m[id][1],
+              -1.+2*trans_m[id][2]};
+            cs_real_t norm_trans_m = cs_math_3_norm( loc_vector );
+            while ( norm_trans_m > 1 )
+            {
+              cs_random_uniform(1, &trans_m[id][0]); /* (?,0) */
+              cs_random_uniform(1, &trans_m[id][1]); /* (?,1) */
+              cs_random_uniform(1, &trans_m[id][2]); /* (?,2) */
+              loc_vector[0] = -1.+2*trans_m[id][0];
+              loc_vector[1] = -1.+2*trans_m[id][1];
+              loc_vector[2] = -1.+2*trans_m[id][2];
+              norm_trans_m = cs_math_3_norm( loc_vector );
+            }
+            for (cs_lnum_t id1 = 0; id1 < 3; id1++)
+              trans_m[id][id1] = (-1.+2*trans_m[id][id1]) / norm_trans_m;
+          }
+          /* Correct 2nd vector (for perpendicularity to the 1st) */
+          cs_real_3_t loc_vector0 =  {trans_m[0][0],
+            trans_m[0][1],
+            trans_m[0][2]};
+          cs_real_3_t loc_vector1 =  {trans_m[1][0],
+            trans_m[1][1],
+            trans_m[1][2]};
+          cs_real_t scal_prod = cs_math_3_dot_product(loc_vector0, loc_vector1);
+          for (cs_lnum_t id = 0; id < 3; id++)
+            trans_m[1][id] -= scal_prod * trans_m[0][id];
+          /* Re-normalize */
+          loc_vector1[0] = trans_m[1][0];
+          loc_vector1[1] = trans_m[1][1];
+          loc_vector1[2] = trans_m[1][2];
+          cs_real_t norm_trans_m = cs_math_3_norm( loc_vector1 );
+          for (cs_lnum_t id = 0; id < 3; id++)
+            trans_m[1][id] /= norm_trans_m;
+
+          /* Compute last vector (cross product of the two others) */
+          loc_vector1[0] = trans_m[1][0];
+          loc_vector1[1] = trans_m[1][1];
+          loc_vector1[2] = trans_m[1][2];
+          cs_real_3_t loc_vector2 =  {trans_m[2][0],
+            trans_m[2][1],
+            trans_m[2][2]};
+          cs_math_3_cross_product( loc_vector0, loc_vector1, loc_vector2);
+          for (cs_lnum_t id = 0; id < 3; id++)
+            trans_m[2][id] = loc_vector2[id];
+
+          /* Write Euler angles */
+          cs_real_t random;
+          cs_random_uniform(1, &random);
+          if (random >= 0.5)
+            euler[0] = pow(0.25*(trans_m[0][0]+trans_m[1][1]+trans_m[2][2]+1.),
+                           0.5);
+          else
+            euler[0] = -pow(0.25*(trans_m[0][0]+trans_m[1][1]+trans_m[2][2]+1.),
+                            0.5);
+          euler[1] = 0.25 * (trans_m[2][1] - trans_m[1][2]) / euler[0];
+          euler[2] = 0.25 * (trans_m[0][2] - trans_m[2][0]) / euler[0];
+          euler[3] = 0.25 * (trans_m[1][0] - trans_m[0][1]) / euler[0];
+
+          /* Compute initial angular velocity */
+          /* Get velocity gradient */
+          cs_lagr_gradients(0, extra->grad_pr, extra->grad_vel);
+
+          /* Local reference frame */
+          cs_real_t grad_vf_r[3][3];
+          cs_math_33_transform_a_to_r(extra->grad_vel[cell_id],
+                                      trans_m,
+                                      grad_vf_r);
+
+          cs_real_t *ang_vel = cs_lagr_particle_attr(particle, p_am,
+              CS_LAGR_ANGULAR_VEL);
+
+          ang_vel[0] = 0.5*(grad_vf_r[2][1] - grad_vf_r[1][2]);
+          ang_vel[1] = 0.5*(grad_vf_r[0][2] - grad_vf_r[2][0]);
+          ang_vel[2] = 0.5*(grad_vf_r[0][1] - grad_vf_r[1][0]);
+        }
+
+      }
+
       if (zis->diameter_variance > 0.0) {
 
         /* Randomize diameter, ensuring we obtain a
@@ -731,7 +954,8 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
       }
 
       /* Other parameters */
-      cs_real_t diam = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_DIAMETER);
+      cs_real_t diam = cs_lagr_particle_get_real(particle, p_am,
+                                                 CS_LAGR_DIAMETER);
       cs_real_t mporos = cs_glob_lagr_clogging_model->mporos;
       if (cs_glob_lagr_model->clogging == 1) {
         cs_lagr_particle_set_real(particle, p_am, CS_LAGR_DIAMETER,
@@ -746,12 +970,19 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
         cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_STAT_CLASS,
                                   zis->cluster);
 
+      if (cs_glob_lagr_model->agglomeration == 1) {
+        cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_AGGLO_CLASS_ID,
+                                  zis->aggregat_class_id);
+        cs_lagr_particle_set_real(particle, p_am, CS_LAGR_AGGLO_FRACTAL_DIM,
+                                  zis->aggregat_fractal_dim);
+      }
+
       /* used for 2nd order only */
       if (p_am->displ[0][CS_LAGR_TAUP_AUX] > 0)
         cs_lagr_particle_set_real(particle, p_am, CS_LAGR_TAUP_AUX, 0.0);
 
-      if (   cs_glob_lagr_model->physical_model == 0
-          || cs_glob_lagr_model->physical_model == 1) {
+      if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_OFF
+          || cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT) {
 
         if (cs_glob_lagr_model->clogging == 0)
           cs_lagr_particle_set_real(particle, p_am, CS_LAGR_MASS,
@@ -761,27 +992,18 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
                                     zis->density * pis6 * d3
                                     * pow(1.0-mporos, 3));
 
-        if (   cs_glob_lagr_model->physical_model == 1
+        if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
             && cs_glob_lagr_specific_physics->itpvar == 1) {
 
-          if (cval_t != NULL)
-            cs_lagr_particle_set_real(particle, p_am,
-                                      CS_LAGR_FLUID_TEMPERATURE,
-                                      cval_t[cell_id] + tscl_shift);
-
-          else if (cval_h != NULL) {
-
-            int mode = 1;
-            cs_real_t temp[1];
-            CS_PROCF(usthht, USTHHT)(&mode, &(cval_h[cell_id]), temp);
-            cs_lagr_particle_set_real(particle, p_am,
-                                      CS_LAGR_FLUID_TEMPERATURE,
-                                      temp[0]);
-
+          if (zis->temperature_profile < 1) {
+            if (cval_t != NULL)
+              cs_lagr_particle_set_real(particle, p_am,
+                                        CS_LAGR_FLUID_TEMPERATURE,
+                                        cval_t[cell_id] + tscl_shift);
           }
 
           /* constant temperature set, may be modified later by user function */
-          if (zis->temperature_profile == 1)
+          else if (zis->temperature_profile == 1)
             cs_lagr_particle_set_real(particle, p_am, CS_LAGR_TEMPERATURE,
                                       zis->temperature);
 
@@ -795,7 +1017,7 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
 
       }
 
-      else if (cs_glob_lagr_model->physical_model == 2) {
+      else if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
 
         int coal_id = zis->coal_number - 1;
 
@@ -892,11 +1114,14 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
 
       /* Initialize the additional user variables */
 
-      for (int i = 0;
-           i < cs_glob_lagr_model->n_user_variables;
-           i++)
-        cs_lagr_particle_set_real(particle, p_am, CS_LAGR_USER + i, 0.0);
-
+      if (cs_glob_lagr_model->n_user_variables > 0) {
+        cs_real_t  *user_attr
+          = cs_lagr_particle_attr(particle, p_am, CS_LAGR_USER);
+        for (int i = 0;
+             i < cs_glob_lagr_model->n_user_variables;
+             i++)
+          user_attr[i] = 0.0;
+      }
     }
 
   }
@@ -944,6 +1169,8 @@ _init_particles(cs_lagr_particle_set_t         *p_set,
     }
 
   }
+
+  BFT_FREE(_cval_t);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -990,9 +1217,9 @@ _check_particles(cs_lagr_particle_set_t         *p_set,
         if (val <= 0.0)
           bft_error(__FILE__, __LINE__, 0,
                     _("Lagrangian %s zone %d, set %d:\n"
-                      "  particle %d has a negative %s: %g"),
+                      "  particle %ld has a negative %s: %g"),
                     z_type_name, zis->zone_id, zis->set_id,
-                    p_id, cs_lagr_attribute_name[attr], (double)val);
+                    (long)p_id, cs_lagr_attribute_name[attr], (double)val);
 
       }
 
@@ -1000,7 +1227,7 @@ _check_particles(cs_lagr_particle_set_t         *p_set,
 
   }
 
-  if (cs_glob_lagr_model->physical_model == 2) {
+  if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
 
     int r01_attrs[] = {CS_LAGR_WATER_MASS, CS_LAGR_COAL_MASS, CS_LAGR_COKE_MASS,
                        CS_LAGR_COAL_DENSITY};
@@ -1019,16 +1246,18 @@ _check_particles(cs_lagr_particle_set_t         *p_set,
             if (n_vals == 1)
               bft_error(__FILE__, __LINE__, 0,
                         _("Lagrangian %s zone %d, set %d:\n"
-                          "  particle %d has a negative %s: %g"),
+                          "  particle %ld has a negative %s: %g"),
                         z_type_name, zis->zone_id, zis->set_id,
-                        p_id, cs_lagr_attribute_name[attr], (double)vals[0]);
+                        (long)p_id, cs_lagr_attribute_name[attr],
+                        (double)vals[0]);
             else
               bft_error(__FILE__, __LINE__, 0,
                         _("Lagrangian %s zone %d, set %d:\n"
-                          "  particle %d has a negative %s\n"
+                          "  particle %ld has a negative %s\n"
                           "  in layer %d: %g"),
                         z_type_name, zis->zone_id, zis->set_id,
-                        p_id, cs_lagr_attribute_name[attr], l_id, (double)vals[l_id]);
+                        (long)p_id, cs_lagr_attribute_name[attr], l_id,
+                        (double)vals[l_id]);
           }
 
         }
@@ -1043,9 +1272,9 @@ _check_particles(cs_lagr_particle_set_t         *p_set,
         if (val < 0) {
           bft_error(__FILE__, __LINE__, 0,
                     _("Lagrangian %s zone %d, set %d:\n"
-                      "  particle %d has a negative %s: %g"),
+                      "  particle %ld has a negative %s: %g"),
                     z_type_name, zis->zone_id, zis->set_id,
-                    p_id, cs_lagr_attribute_name[attr], (double)val);
+                    (long)p_id, cs_lagr_attribute_name[attr], (double)val);
 
         }
 
@@ -1064,21 +1293,28 @@ _check_particles(cs_lagr_particle_set_t         *p_set,
 /*!
  * \brief Inject particles in the computational domain.
  *
- * \param[in] time_id     time step indicator for fields
- *                         0: use fields at current time step
- *                         1: use fields at previous time step
- * \param[in] itypfb      boundary face types
+ * \param[in]  time_id      time step indicator for fields
+ *                           0: use fields at current time step
+ *                           1: use fields at previous time step
+ * \param[in]  itypfb       boundary face types
+ * \param[in]  visc_length  viscous layer thickness
+ *                          (size: number of mesh boundary faces)
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_lagr_injection(int        time_id,
                   const int  itypfb[],
-                  cs_real_t  vislen[])
+                  cs_real_t  visc_length[])
 {
+  CS_UNUSED(itypfb);
+
+  /* We may be mapped to an auxiliary field with no previous time id */
+
   cs_real_t dnbpnw_preci = 0.;
 
   cs_lagr_extra_module_t *extra = cs_get_lagr_extra_module();
+  time_id = CS_MIN(time_id, extra->vel->n_time_vals -1);
 
   const cs_mesh_t  *mesh = cs_glob_mesh;
   const cs_mesh_quantities_t *fvq = cs_glob_mesh_quantities;
@@ -1086,7 +1322,7 @@ cs_lagr_injection(int        time_id,
   /* Particles management */
   cs_lagr_particle_set_t  *p_set = cs_glob_lagr_particle_set;
 
-  /* Non-lagrangian fields */
+  /* Mean fluid velocity field */
   cs_real_t *vela = extra->vel->vals[time_id];
 
   cs_lagr_particle_counter_t *pc = cs_lagr_get_particle_counter();
@@ -1108,18 +1344,18 @@ cs_lagr_injection(int        time_id,
 
     for (int z_id = 0; z_id < zd->n_zones; z_id++) {
 
-      if (zd->zone_type[z_id] > CS_LAGR_SYM)
+      if (zd->zone_type[z_id] > CS_LAGR_BC_USER)
         bft_error(__FILE__, __LINE__, 0,
                   _("Lagrangian boundary zone %d nature %d is unknown."),
                   z_id + 1,
                   (int)zd->zone_type[z_id]);
 
       if (   zd->zone_type[z_id] == CS_LAGR_FOULING
-          && cs_glob_lagr_model->physical_model != 2)
+          && cs_glob_lagr_model->physical_model != CS_LAGR_PHYS_COAL)
         bft_error
           (__FILE__, __LINE__, 0,
            _("Lagrangian boundary zone %d nature is of type CS_LAGR_FOULING,\n"
-             "but cs_glob_lagr_model->physical_model is not equal to 2."),
+             "but cs_glob_lagr_model->physical_model is not equal to CS_LAGR_PHYS_COAL."),
            z_id);
       if (   zd->zone_type[z_id] == CS_LAGR_FOULING
           && cs_glob_lagr_model->fouling != 1)
@@ -1320,9 +1556,17 @@ cs_lagr_injection(int        time_id,
 
         cs_lagr_new_particle_init(particle_range,
                                   time_id,
-                                  vislen);
+                                  visc_length);
 
-        /* advanced user modification */
+        /* Advanced user modification:
+
+           WARNING: the user may change the particle coordinates but is
+           prevented from changing the previous location (otherwise, if
+           the particle is not in the same cell anymore, it would be lost).
+
+           Moreover, a precaution has to be taken when calling
+           "current to previous" in the tracking stage.
+        */
 
         {
           cs_lnum_t *particle_face_ids = NULL;
@@ -1332,22 +1576,127 @@ cs_lagr_injection(int        time_id,
                                                        z_elt_ids,
                                                        elt_particle_idx);
 
+          cs_lnum_t *saved_cell_id;
+          cs_real_3_t *saved_coords;
+          BFT_MALLOC(saved_cell_id, n_inject, cs_lnum_t);
+          BFT_MALLOC(saved_coords, n_inject, cs_real_3_t);
+
+          for (cs_lnum_t i = 0; i < n_inject; i++) {
+            cs_lnum_t p_id = particle_range[0] + i;
+
+            saved_cell_id[i] = cs_lagr_particles_get_lnum(p_set,
+                                                           p_id,
+                                                           CS_LAGR_CELL_ID);
+            const cs_real_t *p_coords
+              = cs_lagr_particles_attr_const(p_set,
+                                             p_id,
+                                             CS_LAGR_COORDS);
+            for (cs_lnum_t j = 0; j < 3; j++)
+              saved_coords[i][j] = p_coords[j];
+          }
+
           cs_user_lagr_in(p_set,
                           zis,
                           particle_range,
                           particle_face_ids,
-                          vislen);
+                          visc_length);
+
+          /* For safety, build values at previous time step, but reset saved values
+             for previous cell number and particle coordinates */
+
+          for (cs_lnum_t i = 0; i < n_inject; i++) {
+            cs_lnum_t p_id = particle_range[0] + i;
+
+            cs_lagr_particles_current_to_previous(p_set, p_id);
+
+            cs_lagr_particles_set_lnum_n(p_set,
+                                         p_id,
+                                         1,
+                                         CS_LAGR_CELL_ID,
+                                         saved_cell_id[i]);
+            cs_real_t *p_coords_prev
+              = cs_lagr_particles_attr_n(p_set,
+                                         p_id,
+                                         1,
+                                         CS_LAGR_COORDS);
+            for (cs_lnum_t j = 0; j < 3; j++)
+              p_coords_prev[j] = saved_coords[i][j];
+
+            /* Just after injection, compute the next particle position with
+             * a reduce integration time so as to simulate continuous injection
+             */
+            cs_real_t res_time = cs_lagr_particles_get_real(p_set, p_id,
+                                                            CS_LAGR_RESIDENCE_TIME);
+
+            if (res_time < 0) {
+
+              cs_real_t *p_coords
+                = cs_lagr_particles_attr(p_set,
+                                         p_id,
+                                         CS_LAGR_COORDS);
+              cs_real_t *p_vel =
+                cs_lagr_particles_attr(p_set, p_id, CS_LAGR_VELOCITY);
+              cs_real_t t_fraction = (cs_glob_lagr_time_step->dtp + res_time);
+
+              for (cs_lnum_t j = 0; j < 3; j++)
+                p_coords[j] = p_coords_prev[j] + t_fraction * p_vel[j];
+
+            }
+
+          }
+
+          BFT_FREE(saved_coords);
+          BFT_FREE(saved_cell_id);
+
+          /* Add particle tracking events for boundary injection */
+
+          if (   particle_face_ids != NULL
+              && cs_lagr_stat_is_active(CS_LAGR_STAT_GROUP_TRACKING_EVENT)) {
+
+            cs_lagr_event_set_t  *events
+              = cs_lagr_event_set_boundary_interaction();
+
+            /* Event set "expected" size: n boundary faces*2 */
+            cs_lnum_t events_min_size = mesh->n_b_faces * 2;
+            if (events->n_events_max < events_min_size)
+              cs_lagr_event_set_resize(events, events_min_size);
+
+            for (cs_lnum_t i = 0; i < n_inject; i++) {
+              cs_lnum_t p_id = particle_range[0] + i;
+
+              cs_lnum_t event_id = events->n_events;
+              events->n_events += 1;
+
+              if (event_id >= events->n_events_max) {
+                /* flush events */
+                cs_lagr_stat_update_event(events,
+                                          CS_LAGR_STAT_GROUP_TRACKING_EVENT);
+                events->n_events = 0;
+                event_id = 0;
+              }
+
+              cs_lagr_event_init_from_particle(events, p_set, event_id, p_id);
+
+              cs_lnum_t face_id = particle_face_ids[i];
+              cs_lagr_events_set_lnum(events,
+                                      event_id,
+                                      CS_LAGR_E_FACE_ID,
+                                      face_id);
+
+              cs_lnum_t *e_flag = cs_lagr_events_attr(events,
+                                                      event_id,
+                                                      CS_LAGR_E_FLAG);
+
+              *e_flag = *e_flag | CS_EVENT_INFLOW;
+
+            }
+
+          }
 
           BFT_FREE(particle_face_ids);
+
         }
 
-        /* for safety, build values at previous time step */
-
-        for (cs_lnum_t p_id = particle_range[0];
-             p_id < particle_range[1];
-             p_id++)
-          cs_lagr_particles_current_to_previous(p_set, p_id);
-
         /* check some particle attributes consistency */
 
         _check_particles(p_set, zis, n_z_elts, elt_particle_idx);
