@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,17 +45,19 @@ BEGIN_C_DECLS
 /*!
  * \brief Inject particles in the computational domain.
  *
- * \param[in] time_id     time step indicator for fields
- *                         0: use fields at current time step
- *                         1: use fields at previous time step
- * \param[in] itypfb      boundary face types
+ * \param[in]  time_id      time step indicator for fields
+ *                           0: use fields at current time step
+ *                           1: use fields at previous time step
+ * \param[in]  itypfb       boundary face types
+ * \param[in]  visc_length  viscous layer thickness
+ *                          (size: number of mesh boundary faces)
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_lagr_injection(int        time_id,
                   const int  itypfb[],
-                  cs_real_t  vislen[]);
+                  cs_real_t  visc_length[]);
 
 /*----------------------------------------------------------------------------*/
 
