@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -44,28 +44,6 @@ BEGIN_C_DECLS
  * Global variables
  *============================================================================*/
 
-/*============================================================================
- * Fortran wrapper function prototypes
- *============================================================================*/
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Fortran wrapper for restart files readings
- */
-/*----------------------------------------------------------------------------*/
-
-void
-CS_PROCF(laglec, LAGLEC)(void);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Fortran wrapper for restart files output.
- */
-/*----------------------------------------------------------------------------*/
-
-void
-CS_PROCF(lagout, LAGOUT)(void);
-
 /*=============================================================================
  * Public function prototypes
  *============================================================================*/
