@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,7 +45,6 @@
  *----------------------------------------------------------------------------*/
 
 #include "bft_printf.h"
-#include "bft_error.h"
 #include "bft_mem.h"
 
 #include "cs_log.h"
@@ -58,6 +57,7 @@
 #include "cs_field.h"
 
 #include "cs_lagr.h"
+#include "cs_lagr_particle.h"
 #include "cs_lagr_tracking.h"
 #include "cs_lagr_post.h"
 #include "cs_lagr_stat.h"
@@ -120,77 +120,36 @@ _status(int i)
  *
  * Parameters:
  * \param[in]  s_id      stat id
+ * \param[out] nbrfac    number of particles used for the statistics
  * \param[out] gmin      min value
  * \param[out] gmax      max value
- * \param[in]  unsnbr    inverse of the number of particles impacting
- *                       the boundary
- * \param[in]  unsnbrfou inverse of the number of particles impacting
- *                       the boundary taking fooling into account.
  */
  /*----------------------------------------------------------------*/
 
 static void
 _lagr_min_max_boundary_stats(int         s_id,
+                             cs_lnum_t  *nbrfac,
                              cs_real_t  *gmin,
-                             cs_real_t  *gmax,
-                             cs_real_t   unsnbr[],
-                             cs_real_t   unsnbrfou[])
+                             cs_real_t  *gmax)
 {
-  cs_lagr_boundary_interactions_t *lagr_bd_i = cs_glob_lagr_boundary_interactions;
+  cs_lagr_boundary_interactions_t *lagr_bd_i
+    = cs_glob_lagr_boundary_interactions;
   cs_lnum_t n_b_faces = cs_glob_mesh->n_b_faces;
 
   /* Initializations */
-  cs_lnum_t nbrfac = 0;
+  *nbrfac = 0;
   *gmax = -cs_math_big_r;
   *gmin =  cs_math_big_r;
 
   cs_real_t threshold = cs_glob_lagr_stat_options->threshold;
 
-  if (lagr_bd_i->imoybr[s_id] == 3) {
-
-    for (cs_lnum_t ifac = 0; ifac < n_b_faces; ifac++) {
-      if (bound_stat[ifac + n_b_faces * lagr_bd_i->iencnb] > threshold) {
-        nbrfac++;
-        *gmax = CS_MAX(*gmax, bound_stat[ifac + n_b_faces * s_id] * unsnbrfou[ifac]);
-        *gmin = CS_MIN(*gmin, bound_stat[ifac + n_b_faces * s_id] * unsnbrfou[ifac]);
-      }
+  for (cs_lnum_t ifac = 0; ifac < n_b_faces; ifac++) {
+    if (bound_stat[ifac + n_b_faces * lagr_bd_i->inbr] > threshold) {
+      *nbrfac = *nbrfac + 1;
+      *gmax = CS_MAX(*gmax, bound_stat[ifac + n_b_faces * s_id]);
+      *gmin = CS_MIN(*gmin, bound_stat[ifac + n_b_faces * s_id]);
     }
-
   }
-  else if (lagr_bd_i->imoybr[s_id] == 2) {
-
-    for (cs_lnum_t ifac = 0; ifac < n_b_faces; ifac++) {
-      if (bound_stat[ifac + n_b_faces * lagr_bd_i->inbr] > threshold) {
-        nbrfac++;
-        *gmax = CS_MAX(*gmax, bound_stat[ifac + n_b_faces * s_id] * unsnbr[ifac]);
-        *gmin = CS_MIN(*gmin, bound_stat[ifac + n_b_faces * s_id] * unsnbr[ifac]);
-      }
-    }
-
-  }
-  else if (lagr_bd_i->imoybr[s_id] == 1) {
-
-    for (cs_lnum_t ifac = 0; ifac < n_b_faces; ifac++) {
-      if (bound_stat[ifac + n_b_faces * lagr_bd_i->inbr] > threshold) {
-        nbrfac++;
-        *gmax = CS_MAX(*gmax, bound_stat[ifac + n_b_faces * s_id] / lagr_bd_i->tstatp);
-        *gmin = CS_MIN(*gmin, bound_stat[ifac + n_b_faces * s_id] / lagr_bd_i->tstatp);
-      }
-    }
-
-  }
-  else if (lagr_bd_i->imoybr[s_id] == 0) {
-
-    for (cs_lnum_t ifac = 0; ifac < n_b_faces; ifac++) {
-      if (bound_stat[ifac + n_b_faces * lagr_bd_i->inbr] > threshold) {
-        nbrfac++;
-        *gmax = CS_MAX(*gmax, bound_stat[ifac + n_b_faces * s_id]);
-        *gmin = CS_MIN(*gmin, bound_stat[ifac + n_b_faces * s_id]);
-      }
-    }
-
-  }
-
 }
 
 /*----------------------------------------------------------------------------*/
@@ -209,7 +168,7 @@ _log_setup_injection(cs_log_t  log)
   if (cs_glob_lagr_time_scheme == NULL)
     return;
 
-  if (cs_glob_lagr_time_scheme->iilagr < 1)
+  if (cs_glob_lagr_time_scheme->iilagr == CS_LAGR_OFF)
     return;
 
   cs_lagr_extra_module_t *extra = cs_get_lagr_extra_module();
@@ -275,6 +234,19 @@ _log_setup_injection(cs_log_t  log)
                         "    density: %g\n"),
                       zis->diameter, zis->diameter_variance, zis->density);
 
+
+        if (cs_glob_lagr_model->shape == CS_LAGR_SHAPE_SPHEROID_STOC_MODEL) {
+          cs_log_printf(log,
+                        _("    shape parameter: %g\n"),
+                        zis->shape);
+        }
+
+        if (cs_glob_lagr_model->shape == CS_LAGR_SHAPE_SPHEROID_JEFFERY_MODEL) {
+          cs_log_printf(log,
+                        _("    ellipsoid radii: [%g, %g, %g]\n"),
+                        zis->radii[0], zis->radii[1], zis->radii[2]);
+        }
+
         if (zis->flow_rate > 0)
           cs_log_printf(log,
                         _("    flow rate: %g\n"),
@@ -291,7 +263,7 @@ _log_setup_injection(cs_log_t  log)
                         _("    fouling index: %g\n"),
                         zis->fouling_index);
 
-        if (   cs_glob_lagr_model->physical_model == 1
+        if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
             && cs_glob_lagr_specific_physics->itpvar == 1) {
           if (zis->temperature_profile == 0)
             cs_log_printf(log,
@@ -308,7 +280,7 @@ _log_setup_injection(cs_log_t  log)
                           _("    emissivity: %g\n"),
                           zis->emissivity);
         }
-        else if (cs_glob_lagr_model->physical_model == 2) {
+        else if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
           cs_log_printf(log,
                         _("    coal number: %d\n"),
                         zis->coal_number);
@@ -340,7 +312,7 @@ cs_lagr_log_setup(void)
   if (cs_glob_lagr_time_scheme == NULL)
     return;
 
-  if (cs_glob_lagr_time_scheme->iilagr < 1)
+  if (cs_glob_lagr_time_scheme->iilagr == CS_LAGR_OFF)
     return;
 
   /* Now add Lagrangian setup info */
@@ -349,44 +321,90 @@ cs_lagr_log_setup(void)
                 _("\n"
                   "Lagrangian model options\n"
                   "------------------------\n"));
+  cs_log_printf(CS_LOG_SETUP,
+                ("  Continuous phase:\n"));
 
-  cs_log_printf
-    (CS_LOG_SETUP,
-     _("  Continuous phase:\n"
-       "    iilagr:                 %3d  (0: Lagrangian deactivated\n"
-       "                                  1: one way coupling\n"
-       "                                  2: two way coupling\n"
-       "                                  3: on frozen fields)\n"
-       "    restart: %s\n"
-       "    statistics/return source terms restart: %s\n\n"
-       "  Specific physics associated with particles\n"
-       "    physical_model:         %3d  (0: no additional equations\n"
-       "                                  1: equations on Dp Tp Mp\n"
-       "                                  2: coal particles)\n"),
-     cs_glob_lagr_time_scheme->iilagr,
-     _status(cs_glob_lagr_time_scheme->isuila),
-     _status(cs_glob_lagr_stat_options->isuist),
-     cs_glob_lagr_model->physical_model);
+  const char *iilagr_value_str[]
+    = {N_("  CS_LAGR_OFF (no Lagrangian model)"),
+       N_("  CS_LAGR_ONEWAY_COUPLING (one way coupling)"),
+       N_("  CS_LAGR_TOWAY_COUPLING (two way coupling)"),
+       N_("  CS_LAGR_FROZEN_CONTINUOUS_PHASE (on frozen fields)")};
 
-  if (cs_glob_lagr_model->physical_model == 1)
-    cs_log_printf
-      (CS_LOG_SETUP,
-       _("    idpvar:                 %3d  (1: eqn diameter Dp,    or 0)\n"
-         "    itpvar:                 %3d  (1: eqn temperature Tp, or 0)\n"
-         "    impvar:                 %3d  (1: eqn mass Mp,        or 0)\n"),
-       cs_glob_lagr_specific_physics->idpvar,
-       cs_glob_lagr_specific_physics->itpvar,
-       cs_glob_lagr_specific_physics->impvar);
+  const char *isuila_value_str[] = {N_("  off (restart not activated)"),
+                                    N_("  on (restart activated)")};
 
-  cs_log_printf
-    (CS_LOG_SETUP,
-     _("\n  Global parameters:\n"
-       "    user particle variables: %2d\n"
-       "    isttio:                 %3d  (1: steady carrier phase)\n"),
-     cs_glob_lagr_model->n_user_variables,
-     cs_glob_lagr_time_scheme->isttio);
+  const char *isuist_value_str[] = {N_("  off (reinitialized)"),
+                                    N_("  on (read from restart file)")};
 
-  if (cs_glob_lagr_model->physical_model == 2) {
+  const char *physical_model_value_str[]
+    = {N_("  0 (no additional equations)"),
+       N_("  1 (equations on Dp Tp Mp)"),
+       N_("  2 (coal particles)")};
+  cs_log_printf(CS_LOG_SETUP,
+                _("    iilagr:    %s\n"),
+                _(iilagr_value_str[cs_glob_lagr_time_scheme->iilagr]));
+
+  cs_log_printf(CS_LOG_SETUP,
+                _("    Restart options\n"));
+  cs_log_printf(CS_LOG_SETUP,
+                _("    isuila:    %s\n"),
+                _(isuila_value_str[cs_glob_lagr_time_scheme->isuila]));
+
+  cs_log_printf(CS_LOG_SETUP,
+                _("    Statistics/return source terms restart\n"));
+  cs_log_printf(CS_LOG_SETUP,
+                _("    isuist:    %s\n"),
+                _(isuist_value_str[cs_glob_lagr_stat_options->isuist]));
+
+  cs_log_printf(CS_LOG_SETUP,
+                ("    Additional models associated with particles\n"));
+
+  cs_log_printf(CS_LOG_SETUP,
+                _("    physical_model:    %s\n"),
+                _(physical_model_value_str[cs_glob_lagr_model->physical_model]));
+
+  if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT) {
+    const char *idpvar_value_str[]
+      = {N_("    0 (no evolution equation on particle diameter)"),
+         N_("    1 (solve the particle diameter evolution)")};
+
+    const char *itpvar_value_str[]
+      = {N_("    0 (equation on the particle temperature)"),
+         N_("    1 (solve the particle temperature)")};
+
+    const char *impvar_value_str[]
+      = {N_("    0 (no evolution equation on particle mass)"),
+         N_("    1 (solve the particle mass)")};
+
+    cs_log_printf(CS_LOG_SETUP,
+                  _("    idpvar:    %s\n"),
+                  _(idpvar_value_str[cs_glob_lagr_specific_physics->idpvar]));
+
+    cs_log_printf(CS_LOG_SETUP,
+                  _("    itpvar:    %s\n"),
+                  _(itpvar_value_str[cs_glob_lagr_specific_physics->itpvar]));
+
+    cs_log_printf(CS_LOG_SETUP,
+                  _("    impvar:    %s\n"),
+                  _(impvar_value_str[cs_glob_lagr_specific_physics->impvar]));
+  }
+
+  const char *isttio_value_str[]
+    = {N_("  0 (unsteady the continuous phase flow)"),
+       N_("  1 (steady continuous phase flow)")};
+
+  cs_log_printf(CS_LOG_SETUP,
+                ("\n  Global parameters:\n"));
+
+  cs_log_printf(CS_LOG_SETUP,
+                _("    user particle variables: %22d\n"),
+                cs_glob_lagr_model->n_user_variables);
+
+  cs_log_printf(CS_LOG_SETUP,
+                _("    isttio:    %s\n"),
+                _(isttio_value_str[cs_glob_lagr_time_scheme->isttio]));
+
+  if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
 
     cs_log_printf
       (CS_LOG_SETUP,
@@ -422,7 +440,7 @@ cs_lagr_log_setup(void)
 
   }
 
-  if (cs_glob_lagr_model->physical_model == 2) {
+  if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
 
     cs_log_printf
       (CS_LOG_SETUP,
@@ -451,21 +469,12 @@ cs_lagr_log_setup(void)
   cs_log_printf
     (CS_LOG_SETUP,
      _("\n  Turbulent dispersion options:\n"
-       "    lagrangian turbulent dispersion:              %s\n"
+       "    Lagrangian turbulent dispersion:              %s\n"
        "      identical to fluid turbulent diffusion:     %s\n"
-       "    apply complete model from time step:          %d\n"),
-     _status(cs_glob_lagr_time_scheme->idistu),
-     _status(cs_glob_lagr_time_scheme->idiffl),
-     cs_glob_lagr_time_scheme->modcpl);
-
-  if (cs_glob_lagr_time_scheme->modcpl) {
-    const char c_dir[] = "xyze";
-    int _idirla = cs_glob_lagr_time_scheme->modcpl;
-    cs_log_printf
-      (CS_LOG_SETUP,
-       _("    complete model main flow direction: %c\n"),
-       c_dir[_idirla]);
-  }
+       "    apply model from time step:                   %d\n"),
+     _status(cs_glob_lagr_model->idistu),
+     _status(cs_glob_lagr_model->idiffl),
+     cs_glob_lagr_model->modcpl);
 
   cs_log_printf
     (CS_LOG_SETUP,
@@ -492,26 +501,8 @@ cs_lagr_log_setup(void)
     cs_log_printf(CS_LOG_SETUP, "    %s\n", "none");
   if (cs_glob_lagr_boundary_interactions->has_part_impact_nbr)
     cs_log_printf(CS_LOG_SETUP, "    %s\n", "particle impact number");
-  if (cs_glob_lagr_boundary_interactions->iflmbd)
-    cs_log_printf(CS_LOG_SETUP, "    %s\n", "particle mass flow");
-  if (cs_glob_lagr_boundary_interactions->iangbd)
-    cs_log_printf(CS_LOG_SETUP, "    %s\n", "impact angle");
-  if (cs_glob_lagr_boundary_interactions->ivitbd)
-    cs_log_printf(CS_LOG_SETUP, "    %s\n", "impact velocity");
-  if (cs_glob_lagr_boundary_interactions->iencnbbd)
-    cs_log_printf(CS_LOG_SETUP, "    %s\n", "interactions with fouling");
-  if (cs_glob_lagr_boundary_interactions->iencmabd)
-    cs_log_printf(CS_LOG_SETUP, "    %s\n", "fouling coal mass flux");
-  if (cs_glob_lagr_boundary_interactions->iencdibd)
-    cs_log_printf(CS_LOG_SETUP, "    %s\n", "fouling coal diameter");
-  if (cs_glob_lagr_boundary_interactions->iencckbd)
-    cs_log_printf(CS_LOG_SETUP, "    %s\n", "fouling coal coke fraction");
-  if (cs_glob_lagr_boundary_interactions->nusbor)
-    cs_log_printf(CS_LOG_SETUP,
-                  _("    number of additional user statistics: %d\n"),
-                  cs_glob_lagr_boundary_interactions->nusbor);
 
-  /* Volumic statistics   */
+  /* Volume statistics   */
 
   cs_log_printf(CS_LOG_SETUP,
                 _("\n"
@@ -537,11 +528,12 @@ cs_lagr_log_setup(void)
 
   }
 
+  cs_lagr_stat_log_setup();
 }
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Log Lagrangian module output in the main listing file.
+ * \brief  Log Lagrangian module output in the main log file.
  */
 /*----------------------------------------------------------------------------*/
 
@@ -553,10 +545,12 @@ cs_lagr_log_iteration(void)
   if (cs_glob_lagr_time_scheme == NULL)
     return;
 
-  if (cs_glob_lagr_time_scheme->iilagr < 1)
+  if (cs_glob_lagr_time_scheme->iilagr == CS_LAGR_OFF)
     return;
 
-  const cs_real_t  *b_stats = bound_stat;
+  /* Return silently if called before Lagrangian start */
+  if (cs_glob_lagr_particle_set == NULL)
+    return;
 
   cs_log_printf(CS_LOG_DEFAULT,
                 _("   ** INFORMATION ON THE LAGRANGIAN CALCULATION\n"));
@@ -585,7 +579,7 @@ cs_lagr_log_iteration(void)
      (unsigned long long)(pc->n_g_new),
      pc->w_new);
 
-  if (cs_glob_lagr_model->physical_model == 2 && cs_glob_lagr_model->fouling == 1)
+  if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL && cs_glob_lagr_model->fouling == 1)
     cs_log_printf
       (CS_LOG_DEFAULT,
        _("ln  coal particles fouled                    %8llu   %14.5E\n"),
@@ -644,7 +638,7 @@ cs_lagr_log_iteration(void)
 
   cs_parall_sum(flow_rate_size, CS_REAL_TYPE, flow_rate);
 
-  for (cs_lnum_t z_id = 0; z_id < bdy_cond->n_zones; z_id++) {
+  for (int z_id = 0; z_id < bdy_cond->n_zones; z_id++) {
 
     if (CS_ABS(flow_rate[z_id*n_stats]) > 0.) {
 
@@ -660,15 +654,17 @@ cs_lagr_log_iteration(void)
       else if (bdy_cond->zone_type[z_id] == CS_LAGR_OUTLET)
         chcond = _("outlet");
 
-      else if (   bdy_cond->zone_type[z_id] == CS_LAGR_DEPO1
-               || bdy_cond->zone_type[z_id] == CS_LAGR_DEPO2)
+      else if (bdy_cond->zone_type[z_id] == CS_LAGR_DEPO1)
+        chcond = _("deposition and elimination");
+
+      else if (bdy_cond->zone_type[z_id] == CS_LAGR_DEPO2)
         chcond = _("deposition");
 
       else if (bdy_cond->zone_type[z_id] == CS_LAGR_FOULING)
         chcond = _("fouling");
 
       else if (bdy_cond->zone_type[z_id] == CS_LAGR_DEPO_DLVO)
-        chcond = _("dlvo conditions");
+        chcond = _("DLVO conditions");
 
       else if (bdy_cond->zone_type[z_id] == CS_LAGR_SYM)
         chcond = _("symmetry");
@@ -730,76 +726,39 @@ cs_lagr_log_iteration(void)
     cs_log_printf(CS_LOG_DEFAULT,
                   _("                           Min value    Max value    \n"));
 
-    const cs_real_t _threshold = 1.e-30;
-
-    cs_real_t *tabvr = NULL;
-    cs_real_t *tabvrfou = NULL;
-
-    if (cs_glob_lagr_boundary_interactions->has_part_impact_nbr == 1) {
-
-      /* Allocate a work array */
-      BFT_MALLOC(tabvr, cs_glob_mesh->n_b_faces, cs_real_t);
-
-      int s_id = cs_glob_lagr_boundary_interactions->inbr;
-
-      for (cs_lnum_t ifac = 0; ifac < cs_glob_mesh->n_b_faces; ifac++) {
-
-        if (b_stats[ifac + cs_glob_mesh->n_b_faces * s_id] > _threshold)
-          tabvr[ifac] = 1.0 / b_stats[ifac + cs_glob_mesh->n_b_faces * s_id];
-        else
-          tabvr[ifac] = 0.0;
-
-      }
-
-    }
-
-    if (cs_glob_lagr_boundary_interactions->iencnbbd == 1) {
-
-      BFT_MALLOC(tabvrfou, cs_glob_mesh->n_b_faces, cs_real_t);
-
-      int s_id = cs_glob_lagr_boundary_interactions->iencnb;
-
-      for (cs_lnum_t ifac = 0; ifac < cs_glob_mesh->n_b_faces; ifac++) {
-
-        if (b_stats[ifac + cs_glob_mesh->n_b_faces * s_id] > _threshold)
-          tabvrfou[ifac] = 1.0 / b_stats[ifac + cs_glob_mesh->n_b_faces * s_id];
-        else
-          tabvrfou[ifac] = 0.0;
-
-      }
-    }
-
     for (int s_id = 0; s_id < cs_glob_lagr_dim->n_boundary_stats; s_id++) {
 
-      cs_real_t gmin = cs_math_big_r;
-      cs_real_t gmax =-cs_math_big_r;
+      cs_real_t gmin;
+      cs_real_t gmax;
+      cs_lnum_t nbrfac;
 
-      _lagr_min_max_boundary_stats(s_id, &gmin, &gmax, tabvr, tabvrfou);
+      _lagr_min_max_boundary_stats(s_id, &nbrfac, &gmin, &gmax);
 
+      cs_parall_max(1, CS_INT_TYPE, &nbrfac);
       cs_parall_min(1, CS_REAL_TYPE, &gmin);
       cs_parall_max(1, CS_REAL_TYPE, &gmax);
 
-      cs_log_printf(CS_LOG_DEFAULT,
-                    "lp  %20s  %12.5E  %12.5E\n",
-                    cs_glob_lagr_boundary_interactions->nombrd[s_id],
-                    gmin,
-                    gmax);
+      /* If there is no particles, no statistics */
+      if (nbrfac > 0)
+        cs_log_printf(CS_LOG_DEFAULT,
+                      "lp  %20s  %12.5E  %12.5E\n",
+                      cs_glob_lagr_boundary_interactions->nombrd[s_id],
+                      gmin,
+                      gmax);
+      else
+        cs_log_printf(CS_LOG_DEFAULT,
+                      "lp  %20s\n",
+                      cs_glob_lagr_boundary_interactions->nombrd[s_id]);
 
     }
 
-    /* Free memory */
-    if (tabvr != NULL)
-      BFT_FREE(tabvr);
-    if (tabvrfou != NULL)
-      BFT_FREE(tabvrfou);
-
     cs_log_separator(CS_LOG_DEFAULT);
 
   }
 
   /* Information about two-way coupling  */
 
-  if (cs_glob_lagr_time_scheme->iilagr == 2) {
+  if (cs_glob_lagr_time_scheme->iilagr == CS_LAGR_TWOWAY_COUPLING) {
 
     if (cs_glob_lagr_time_scheme->isttio == 0) {
 
