@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -51,38 +51,37 @@
 #include "fvm_periodicity.h"
 
 #include "cs_base.h"
-#include "cs_halo.h"
 #include "cs_math.h"
-#include "cs_mesh.h"
-#include "cs_mesh_quantities.h"
 #include "cs_order.h"
 #include "cs_parall.h"
-#include "cs_prototypes.h"
 #include "cs_random.h"
-#include "cs_search.h"
 #include "cs_timer_stats.h"
 
-#include "cs_field.h"
-#include "cs_field_pointer.h"
-
 #include "cs_lagr.h"
 #include "cs_lagr_post.h"
 #include "cs_lagr_clogging.h"
 #include "cs_lagr_roughness.h"
 #include "cs_lagr_dlvo.h"
-#include "cs_lagr_stat.h"
-#include "cs_lagr_geom.h"
 
 /*----------------------------------------------------------------------------
  *  Header for the current file
  *----------------------------------------------------------------------------*/
 
-#include "cs_lagr_tracking.h"
+#include "cs_lagr_particle.h"
 
 /*----------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
 
+/*=============================================================================
+ * Additional doxygen documentation
+ *============================================================================*/
+
+/*!
+  \file cs_lagr_particle.c
+        Particle structure.
+*/
+
 /*! \cond DOXYGEN_SHOULD_SKIP_THIS */
 
 /*=============================================================================
@@ -98,11 +97,11 @@ BEGIN_C_DECLS
 
 /* keys to sort attributes by type.
 
-   ieptp/ieptpa integer values at current and previous time steps
-   pepa real values at current time step
-   _int_loc local integer values at current time step
-   ipepa integer values at current time step
-   iprkid values are for rank ids, useful and valid only for previous
+   rvar real values at current and previous time steps
+   ivar integer values at current and previous time steps
+   rprp real properties at current an previous time steps
+   iprp integer properties at current and previous time steps
+   rkid values are for rank ids, useful and valid only for previous
    time steps */
 
 typedef enum {
@@ -155,7 +154,8 @@ union cs_lagr_value_t {
 /* Enumerator names */
 
 const char *cs_lagr_attribute_name[] = {
-  "cell_num",
+  "state_flag",
+  "cell_id",
   "rank_id",
   "rebound_id",
   "random_value",
@@ -169,29 +169,44 @@ const char *cs_lagr_attribute_name[] = {
   "velocity_seen",
   "tr_truncate",
   "tr_reposition",
+
+  /* Arrays for 2nd order scheme */
   "turb_state_1",
   "pred_velocity",
   "pred_velocity_seen",
   "v_gauss",
   "br_gauss",
+
+  /* Non-spherical particles submodel additoinal parameters */
+  "shape",
+  "orientation",
+  "radii",
+  "angular_vel",
+  "euler",
+  "shape_param",
+
+  /* Deposition submodel additional parameters */
   "yplus",
   "interf",
   "neighbor_face_id",
   "marko_value",
-  "deposition_flag",
   "fouling_index",
+
   "n_large_asperities",
   "n_small_asperities",
   "adhesion_force",
   "adhesion_torque",
   "displacement_norm",
+
   "height",
   "cluster_nb_part",
   "depo_time",
   "consol_height",
+
   "temperature",
   "fluid_temperature",
   "cp",
+
   "water_mass",
   "coal_mass",
   "coke_mass",
@@ -199,8 +214,14 @@ const char *cs_lagr_attribute_name[] = {
   "initial_diameter",
   "coal_id",
   "coal_density",
+
   "emissivity",
+
   "stat_class",
+
+  "agglo_class_id",
+  "agglo_fractal_dim",
+
   "user",
   "<none>"};
 
@@ -476,6 +497,7 @@ _create_particle_set(cs_lnum_t                       n_particles_max,
 
   new_set->n_particles = 0;
   new_set->n_part_new = 0;
+  new_set->n_part_merged = 0;
   new_set->n_part_out = 0;
   new_set->n_part_dep = 0;
   new_set->n_part_fou = 0;
@@ -484,6 +506,7 @@ _create_particle_set(cs_lnum_t                       n_particles_max,
 
   new_set->weight = 0.0;
   new_set->weight_new = 0.0;
+  new_set->weight_merged = 0.0;
   new_set->weight_out = 0.0;
   new_set->weight_dep = 0.0;
   new_set->weight_fou = 0.0;
@@ -653,7 +676,7 @@ cs_lagr_particle_attr_initialize(void)
 
   /*  cs_glob_lagr_brownian->lamvbr = *lamvbr; */
 
-  if (lagr_model->physical_model == 2)
+  if (lagr_model->physical_model == CS_LAGR_PHYS_COAL)
     lagr_model->n_temperature_layers = cs_glob_lagr_const_dim->nlayer;
   else
     lagr_model->n_temperature_layers = 1;
@@ -667,17 +690,20 @@ cs_lagr_particle_attr_initialize(void)
   }
 
   /* Special case:
-     cell number is also needed in for previous time step.
+     cell id is also needed in for previous time step.
   */
 
-  attr_keys[CS_LAGR_CELL_NUM][0] = CS_LAGR_P_IVAR;
-  attr_keys[CS_LAGR_CELL_NUM][1] = ++loc_count;
+  attr_keys[CS_LAGR_CELL_ID][0] = CS_LAGR_P_IVAR;
+  attr_keys[CS_LAGR_CELL_ID][1] = ++loc_count;
 
   attr_keys[CS_LAGR_RANK_ID][0] = CS_LAGR_P_RKID;
   attr_keys[CS_LAGR_RANK_ID][1] = 1;
 
   /* Other attributes */
 
+  attr_keys[CS_LAGR_P_FLAG][0] = CS_LAGR_P_IPRP;
+  attr_keys[CS_LAGR_P_FLAG][1] = ++loc_count;
+
   attr_keys[CS_LAGR_REBOUND_ID][0] = CS_LAGR_P_IPRP;
   attr_keys[CS_LAGR_REBOUND_ID][1] = ++loc_count;
 
@@ -699,6 +725,33 @@ cs_lagr_particle_attr_initialize(void)
   attr_keys[CS_LAGR_DIAMETER][0] = CS_LAGR_P_RVAR_TS;
   attr_keys[CS_LAGR_DIAMETER][1] = ++loc_count;
 
+  /* Non-sphere model
+   * TODO activate only required arrays */
+  if (lagr_model->shape != CS_LAGR_SHAPE_SPHERE_MODEL) {
+    attr_keys[CS_LAGR_SHAPE][0] = CS_LAGR_P_RPRP;
+    attr_keys[CS_LAGR_SHAPE][1] = ++loc_count;
+
+    attr_keys[CS_LAGR_ORIENTATION][1] = ++loc_count;
+    attr_keys[CS_LAGR_ORIENTATION][2] = 3;
+
+    attr_keys[CS_LAGR_QUATERNION][1] = ++loc_count;
+    attr_keys[CS_LAGR_QUATERNION][2] = 4;
+
+    attr_keys[CS_LAGR_RADII][0] = CS_LAGR_P_RPRP;
+    attr_keys[CS_LAGR_RADII][1] = ++loc_count;
+    attr_keys[CS_LAGR_RADII][2] = 3;
+
+    attr_keys[CS_LAGR_ANGULAR_VEL][1] = ++loc_count;
+    attr_keys[CS_LAGR_ANGULAR_VEL][2] = 3;
+
+    attr_keys[CS_LAGR_EULER][1] = ++loc_count;
+    attr_keys[CS_LAGR_EULER][2] = 4;
+
+    attr_keys[CS_LAGR_SHAPE_PARAM][0] = CS_LAGR_P_RPRP;
+    attr_keys[CS_LAGR_SHAPE_PARAM][1] = ++loc_count;
+    attr_keys[CS_LAGR_SHAPE_PARAM][2] = 4;
+  }
+
   attr_keys[CS_LAGR_COORDS][1] = ++loc_count;
   attr_keys[CS_LAGR_COORDS][2] = 3;
 
@@ -730,7 +783,7 @@ cs_lagr_particle_attr_initialize(void)
     attr_keys[CS_LAGR_PRED_VELOCITY_SEEN][1] = ++pepa_loc_add;
     attr_keys[CS_LAGR_PRED_VELOCITY_SEEN][2] = 3;
 
-    if (cs_glob_lagr_time_scheme->idistu == 1) {
+    if (cs_glob_lagr_model->idistu == 1) {
       attr_keys[CS_LAGR_V_GAUSS][0] = CS_LAGR_P_RPRP;
       attr_keys[CS_LAGR_V_GAUSS][1] = ++pepa_loc_add;
       attr_keys[CS_LAGR_V_GAUSS][2] = 9;
@@ -751,9 +804,6 @@ cs_lagr_particle_attr_initialize(void)
     attr_keys[CS_LAGR_INTERF][0] = CS_LAGR_P_RPRP;
     attr_keys[CS_LAGR_INTERF][1] = ++loc_count;
 
-    attr_keys[CS_LAGR_DEPOSITION_FLAG][0] = CS_LAGR_P_IPRP;
-    attr_keys[CS_LAGR_DEPOSITION_FLAG][1] = ++loc_count;
-
     attr_keys[CS_LAGR_NEIGHBOR_FACE_ID][0] = CS_LAGR_P_IPRP;
     attr_keys[CS_LAGR_NEIGHBOR_FACE_ID][1] = ++loc_count;
 
@@ -797,7 +847,7 @@ cs_lagr_particle_attr_initialize(void)
 
   }
 
-  if (lagr_model->physical_model == 1) {
+  if (lagr_model->physical_model == CS_LAGR_PHYS_HEAT) {
 
     if (cs_glob_lagr_specific_physics->itpvar == 1) {
 
@@ -816,7 +866,7 @@ cs_lagr_particle_attr_initialize(void)
 
   }
 
-  if (lagr_model->physical_model == 2) {
+  if (lagr_model->physical_model == CS_LAGR_PHYS_COAL) {
 
     attr_keys[CS_LAGR_CP][1] = ++loc_count;
 
@@ -858,6 +908,15 @@ cs_lagr_particle_attr_initialize(void)
     attr_keys[CS_LAGR_STAT_CLASS][1] = ++loc_count;
   }
 
+  if (lagr_model->agglomeration == 1 ||
+      lagr_model->fragmentation == 1 ) {
+    attr_keys[CS_LAGR_AGGLO_CLASS_ID][0] = CS_LAGR_P_IPRP;
+    attr_keys[CS_LAGR_AGGLO_CLASS_ID][1] = ++loc_count;
+
+    attr_keys[CS_LAGR_AGGLO_FRACTAL_DIM][0] = CS_LAGR_P_RPRP;
+    attr_keys[CS_LAGR_AGGLO_FRACTAL_DIM][1] = ++loc_count;
+  }
+
   if (lagr_model->n_user_variables > 0) {
     attr_keys[CS_LAGR_USER][1] = ++loc_count;
     attr_keys[CS_LAGR_USER][2] = lagr_model->n_user_variables;
@@ -1225,8 +1284,8 @@ cs_lagr_particle_set_dump(const cs_lagr_particle_set_t  *particles)
 
     bft_printf("Particle set\n");
     bft_printf("------------\n");
-    bft_printf("  n_particles:      %10d\n", particles->n_particles);
-    bft_printf("  n_particles_max:  %10d\n", particles->n_particles_max);
+    bft_printf("  n_particles:      %10ld\n", (long)particles->n_particles);
+    bft_printf("  n_particles_max:  %10ld\n", (long)particles->n_particles_max);
 
     bft_printf_flush();
 
