@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -29,8 +29,6 @@
 
 #include "cs_defs.h"
 
-#include "cs_interface.h"
-
 #include "assert.h"
 
 /*----------------------------------------------------------------------------*/
@@ -41,6 +39,36 @@ BEGIN_C_DECLS
  * Macro definitions
  *============================================================================*/
 
+/*=============================================================================
+ * Macro definitions
+ *============================================================================*/
+
+/*!
+ * Flags specifying general event attributes
+ */
+
+/*! particle will be deleted */
+#define CS_LAGR_PART_TO_DELETE         (1 << 0)
+
+/*! particle is fixed, no resuspension possible */
+#define CS_LAGR_PART_FIXED             (1 << 1)
+
+/*! particle is deposited */
+#define CS_LAGR_PART_DEPOSITED         (1 << 2)
+
+/*! particle is rolling */
+#define CS_LAGR_PART_ROLLING           (1 << 3)
+
+/*! particle motion is prescribed */
+#define CS_LAGR_PART_IMPOSED_MOTION    (1 << 4)
+
+/*! Flag sets (useful for cancelling nay flag of a set) */
+
+/*! particle is in flow */
+#define CS_LAGR_PART_DEPOSITION_FLAGS \
+  (  CS_LAGR_PART_TO_DELETE | CS_LAGR_PART_FIXED | CS_LAGR_PART_DEPOSITED \
+   | CS_LAGR_PART_ROLLING | CS_LAGR_PART_IMPOSED_MOTION)
+
 /*============================================================================
  * Type definitions
  *============================================================================*/
@@ -50,7 +78,9 @@ BEGIN_C_DECLS
 
 typedef enum {
 
-  CS_LAGR_CELL_NUM,            /*!< local cell number (1 to n) */
+  CS_LAGR_P_FLAG,             /*!< local metadata flag */
+
+  CS_LAGR_CELL_ID,             /*!< local cell id (0 to n-1) */
   CS_LAGR_RANK_ID,             /*!< local parallel rank id */
 
   CS_LAGR_REBOUND_ID,          /*!< number of time steps since rebound, or -1 */
@@ -79,13 +109,22 @@ typedef enum {
   CS_LAGR_V_GAUSS,             /* 1st step Gaussian variable */
   CS_LAGR_BR_GAUSS,            /* 1st step Brownian motion Gaussian variable */
 
+  /* Non-spherical particles submodel additoinal parameters */
+
+  CS_LAGR_SHAPE,               /*!< Shape of spheroids */
+  CS_LAGR_ORIENTATION,         /*!< Orientation of spheroids */
+  CS_LAGR_QUATERNION,          /*!< Quaternion for spheroids */
+  CS_LAGR_RADII,               /*!< Radii a, b, c for ellispoids */
+  CS_LAGR_ANGULAR_VEL,         /*!< Angular velocity */
+  CS_LAGR_EULER,               /*!< Euler four parameters */
+  CS_LAGR_SHAPE_PARAM,         /*!< Shape parameters computed from a b c */
+
   /* Deposition submodel additional parameters */
 
   CS_LAGR_YPLUS,
   CS_LAGR_INTERF,
   CS_LAGR_NEIGHBOR_FACE_ID,
   CS_LAGR_MARKO_VALUE,
-  CS_LAGR_DEPOSITION_FLAG,
   CS_LAGR_FOULING_INDEX,
 
   /* Resuspension model additional parameters */
@@ -118,7 +157,7 @@ typedef enum {
   CS_LAGR_SHRINKING_DIAMETER,
   CS_LAGR_INITIAL_DIAMETER,
 
-  CS_LAGR_COAL_NUM,
+  CS_LAGR_COAL_ID,
   CS_LAGR_COAL_DENSITY,
 
   /* Radiative model additional parameters */
@@ -129,6 +168,10 @@ typedef enum {
 
   CS_LAGR_STAT_CLASS,
 
+  /* Agglomeration/fragmentation model parameters */
+  CS_LAGR_AGGLO_CLASS_ID,
+  CS_LAGR_AGGLO_FRACTAL_DIM,
+
   /* User attributes */
 
   CS_LAGR_USER,
@@ -181,6 +224,7 @@ typedef struct {
   cs_lnum_t  n_particles;                     /* number of particle in domain */
   cs_lnum_t  n_part_new;
   cs_lnum_t  n_part_out;
+  cs_lnum_t  n_part_merged;                   /* number of merged particles */
   cs_lnum_t  n_part_dep;
   cs_lnum_t  n_part_fou;
   cs_lnum_t  n_part_resusp;
@@ -189,6 +233,7 @@ typedef struct {
   cs_real_t  weight;
   cs_real_t  weight_new;
   cs_real_t  weight_out;
+  cs_real_t  weight_merged;
   cs_real_t  weight_dep;
   cs_real_t  weight_fou;
   cs_real_t  weight_resusp;
@@ -454,6 +499,85 @@ cs_lagr_particles_attr_n_const(const cs_lagr_particle_set_t  *particle_set,
          + particle_set->p_am->displ[time_id][attr];
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get flag value with a selected mask for a given particle in a set.
+ *
+ * \param[in]  particle_set  pointer to particle set
+ * \param[in]  particle_id   particle id
+ * \param[in]  mask          requested attribute flag value
+ *
+ * \return  attribute value
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static int
+cs_lagr_particles_get_flag(const cs_lagr_particle_set_t  *particle_set,
+                           cs_lnum_t                      particle_id,
+                           int                            mask)
+{
+  int flag
+    = *((const cs_lnum_t *)(  particle_set->p_buffer
+                            + particle_set->p_am->extents*particle_id
+                            + particle_set->p_am->displ[0][CS_LAGR_P_FLAG]));
+
+  return (flag & mask);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set flag value with a selected mask for a given particle in a set.
+ *
+ * \param[in]  particle_set  pointer to particle set
+ * \param[in]  particle_id   particle id
+ * \param[in]  mask          attribute flag value to set
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static void
+cs_lagr_particles_set_flag(const cs_lagr_particle_set_t  *particle_set,
+                           cs_lnum_t                      particle_id,
+                           int                            mask)
+{
+  int flag
+    = *((const cs_lnum_t *)(  particle_set->p_buffer
+                            + particle_set->p_am->extents*particle_id
+                            + particle_set->p_am->displ[0][CS_LAGR_P_FLAG]));
+
+  flag = flag | mask;
+
+  *((cs_lnum_t *)(  particle_set->p_buffer
+                  + particle_set->p_am->extents*particle_id
+                  + particle_set->p_am->displ[0][CS_LAGR_P_FLAG])) = flag;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Unset flag value with a selected mask for a given particle in a set.
+ *
+ * \param[in]  particle_set  pointer to particle set
+ * \param[in]  particle_id   particle id
+ * \param[in]  mask          attribute flag value to set
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static void
+cs_lagr_particles_unset_flag(const cs_lagr_particle_set_t  *particle_set,
+                             cs_lnum_t                      particle_id,
+                             int                            mask)
+{
+  int flag
+    = *((const cs_lnum_t *)(  particle_set->p_buffer
+                            + particle_set->p_am->extents*particle_id
+                            + particle_set->p_am->displ[0][CS_LAGR_P_FLAG]));
+
+  flag = (flag | mask) - mask;
+
+  *((cs_lnum_t *)(  particle_set->p_buffer
+                  + particle_set->p_am->extents*particle_id
+                  + particle_set->p_am->displ[0][CS_LAGR_P_FLAG])) = flag;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Get attribute value of type cs_lnum_t of a given particle in a set.
@@ -928,52 +1052,6 @@ cs_lagr_particle_get_lnum(const void                     *particle,
                                 + attr_map->displ[0][attr]));
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Get cell id of a given particle in a set
- *
- * \param[in]  particle  pointer to particle data
- * \param[in]  attr_map  pointer to attribute map
- *
- * \return cell id
- */
-/*----------------------------------------------------------------------------*/
-
-inline static cs_lnum_t
-cs_lagr_particle_get_cell_id(const void                     *particle,
-                             const cs_lagr_attribute_map_t  *attr_map)
-{
-  assert(attr_map->count[0][CS_LAGR_CELL_NUM] > 0);
-
-  const cs_lnum_t cell_num
-    = *((const cs_lnum_t *)(  (const unsigned char *)particle
-                            + attr_map->displ[0][CS_LAGR_CELL_NUM]));
-
-  return CS_ABS(cell_num) - 1;
-}
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Set cell id of a given particle in a set
- *
- * \param[in]  particle  pointer to particle data
- * \param[in]  attr_map  pointer to attribute map
- * \param[in]  cell_id   containing cell (0-based)
- */
-/*----------------------------------------------------------------------------*/
-
-inline static void
-cs_lagr_particle_set_cell_id(void                           *particle,
-                             const cs_lagr_attribute_map_t  *attr_map,
-                             cs_lnum_t                       cell_id)
-{
-  assert(attr_map->count[0][CS_LAGR_CELL_NUM] > 0);
-
-  *((cs_lnum_t *)(  (unsigned char *)particle
-                  + attr_map->displ[0][CS_LAGR_CELL_NUM]))
-    = cell_id + 1;
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Get attribute value of type cs_lnum_t of a given particle
