@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -62,7 +62,6 @@
 #include "cs_mesh_quantities.h"
 #include "cs_order.h"
 #include "cs_parall.h"
-#include "cs_prototypes.h"
 #include "cs_random.h"
 #include "cs_search.h"
 #include "cs_timer_stats.h"
@@ -74,7 +73,6 @@
 #include "cs_lagr_roughness.h"
 #include "cs_lagr_dlvo.h"
 #include "cs_lagr_stat.h"
-#include "cs_lagr_geom.h"
 #include "cs_lagr.h"
 #include "cs_lagr_tracking.h"
 #include "cs_lagr_prototypes.h"
@@ -95,8 +93,10 @@ BEGIN_C_DECLS
  * File variables
  *============================================================================*/
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
- * Public function definitions for Fortran API
+ * Public function definitions
  *============================================================================*/
 
 /*----------------------------------------------------------------------------*/
@@ -109,11 +109,12 @@ BEGIN_C_DECLS
  * \param[out]  crvexp     explicit part of the source term
  */
 /*----------------------------------------------------------------------------*/
+
 void
-CS_PROCF (precst,PRECST) (cs_real_t *dtref,
-                          cs_real_t *crom,
-                          cs_real_t *cvar_scal,
-                          cs_real_t  crvexp[])
+cs_lagr_precipitation_mass_st(cs_real_t        dtref,
+                              const cs_real_t  crom[],
+                              const cs_real_t  cvar_scal[],
+                              cs_real_t        crvexp[])
 {
   cs_real_t pis6 = cs_math_pi / 6.0;
 
@@ -178,7 +179,8 @@ CS_PROCF (precst,PRECST) (cs_real_t *dtref,
               * pow(cs_lagr_particle_get_real(particle, p_am,
                                               CS_LAGR_DIAMETER),3.0);
 
-          if (   cs_lagr_particle_get_cell_id(particle, p_am) == iel
+          if (   cs_lagr_particle_get_lnum(particle, p_am,
+                                           CS_LAGR_CELL_ID) == iel
               &&   cs_lagr_particle_get_real(particle, p_am, CS_LAGR_MASS)
                  - part_mass < 1e-12)
 
@@ -204,7 +206,7 @@ CS_PROCF (precst,PRECST) (cs_real_t *dtref,
         preci->nbprec[iel] =   (cvar_scal[iel] - solub[iel])
                              * fvq->cell_vol[iel] / mass;
         mp_preci[iel] =  preci->nbprec[iel] * mass;
-        crvexp[iel]   = -crom[iel] * mp_preci[iel] / *dtref;
+        crvexp[iel]   = -crom[iel] * mp_preci[iel] / dtref;
 
       }
 
@@ -224,7 +226,8 @@ CS_PROCF (precst,PRECST) (cs_real_t *dtref,
                                                            CS_LAGR_DIAMETER);
               cs_real_t p_mass = cs_lagr_particle_get_real(particle, p_am,
                                                            CS_LAGR_MASS);
-              cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
+              cs_lnum_t cell_id = cs_lagr_particle_get_lnum(particle, p_am,
+                                                            CS_LAGR_CELL_ID);
               cs_real_t mass = preci->rho * pis6 * pow(p_diam,3.0);
               if (   cell_id == iel
                   && p_diam - ref_diameter < 1e-12
@@ -234,7 +237,7 @@ CS_PROCF (precst,PRECST) (cs_real_t *dtref,
                                                                CS_LAGR_STAT_WEIGHT);
 
                 if (   ((solub[iel] - cvar_scal[iel]) * fvq->cell_vol[iel])
-                    >= (mp_diss[iel * preci->nbrclas + iclas] + p_weight * mass) )
+                    >= (mp_diss[iel * preci->nbrclas + iclas] + p_weight * mass))
                   mp_diss[iel * preci->nbrclas + iclas] += p_weight * mass;
 
               }
@@ -246,7 +249,7 @@ CS_PROCF (precst,PRECST) (cs_real_t *dtref,
         }
 
         for (cs_lnum_t iclas = 0; iclas < preci->nbrclas; iclas++)
-          crvexp[iel] += crom[iel] * mp_diss[iel * preci->nbrclas + iclas] / *dtref;
+          crvexp[iel] += crom[iel] * mp_diss[iel*preci->nbrclas + iclas] / dtref;
 
       }
 
@@ -258,12 +261,6 @@ CS_PROCF (precst,PRECST) (cs_real_t *dtref,
   BFT_FREE(part_tot);
 }
 
-/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
-
-/*============================================================================
- * Public function definitions
- *============================================================================*/
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Management of the injection of particles formed by precipitation.
@@ -343,7 +340,7 @@ cs_lagr_precipitation_injection(cs_real_t   *vela,
     if (preci->nbprec[iel] > 0) {
 
       for (cs_lnum_t i = 0; i < preci->nbprec[iel]; i++)
-        cell[nbprec_tot + i]      = iel;
+        cell[nbprec_tot + i] = iel;
 
       nbprec_tot += preci->nbprec[iel];
 
@@ -364,14 +361,15 @@ cs_lagr_precipitation_injection(cs_real_t   *vela,
 
         for (cs_lnum_t iclas = 0; iclas < preci->nbrclas; iclas++) {
 
-          if (   cs_lagr_particle_get_cell_id(particle, p_am) == iel
+          if (   cs_lagr_particle_get_lnum(particle, p_am,
+                                           CS_LAGR_CELL_ID) == iel
               && (  cs_lagr_particle_get_real(particle, p_am, CS_LAGR_DIAMETER)
                   - ref_diameter < 1e-12)
               && (mp[iclas] < mp_diss[iel * preci->nbrclas + iclas])) {
 
             /* Removing of particles due to dissolution */
 
-            cs_lagr_particle_set_cell_id(particle, p_am, -1);
+            cs_lagr_particles_set_flag(p_set, npt, CS_LAGR_PART_TO_DELETE);
             cs_real_t d3 = pow (cs_lagr_particle_get_real(particle, p_am,
                                                           CS_LAGR_DIAMETER), 3);
             mp[iclas] += cs_lagr_particle_get_real(particle, p_am,
@@ -416,7 +414,7 @@ cs_lagr_precipitation_injection(cs_real_t   *vela,
       for (cs_lnum_t i = 0; i <  3; i++)
         part_coord[i] = fvq->cell_cen[cell[ip - npt] * 3 + i];
 
-      cs_lagr_particle_set_cell_id(particle, p_am, cell[ip - npt]);
+      cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_CELL_ID, cell[ip - npt]);
 
       cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_REBOUND_ID, -1);
 
@@ -453,8 +451,8 @@ cs_lagr_precipitation_injection(cs_real_t   *vela,
                                   CS_LAGR_MARKO_VALUE, -1);
         cs_lagr_particle_set_lnum(particle, p_am,
                                   CS_LAGR_NEIGHBOR_FACE_ID, -1);
-        cs_lagr_particle_set_lnum(particle, p_am,
-                                  CS_LAGR_DEPOSITION_FLAG, CS_LAGR_PART_IN_FLOW);
+        cs_lagr_particles_unset_flag(p_set, ip,
+                                     CS_LAGR_PART_DEPOSITION_FLAGS);
 
       }
 
