@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -107,12 +107,9 @@ cs_lagr_print(cs_real_t ttcabs)
 
   if (   cs_glob_rank_id <= 0
       && flal == NULL && _ipass == 1)
-    flal = fopen("listla","w");
+    flal = fopen("lagrangian.log","w");
 
-  /* ====================================================================   */
-  /* 2. OUVERTURE DU FICHIER DE STOCKAGE */
-  /* ====================================================================   */
-  /*     Seul le premier processeur ecrit les informations   */
+  /* Open log file on rank 0 only */
 
   if (flal != NULL) {
 
@@ -135,66 +132,65 @@ cs_lagr_print(cs_real_t ttcabs)
                 "# column  9: inst. number of deposited particles\n"
                 "# column 10: inst. number of deposited particles (weighted)\n");
 
-        if (   lagr_model->physical_model == 2
-            && lagr_model->fouling == 1)
+        int col_id = 11;
+        if (cs_glob_lagr_model->agglomeration) {
           fprintf(flal,
-                  "# column 11: inst. number of fouled particles (coal)\n"
-                  "# column 12: inst. number of fouled particles (coal, weighted)\n"
-                  "# column 13: inst. number of lost particles\n"
-                  "#\n");
+                  "# column %2d: inst. number of merged particles\n"
+                  "# column %2d: inst. number of merged particles (weighted)\n",
+                  col_id, col_id+1);
+          col_id += 2;
+        }
 
-        else if (lagr_model->resuspension > 0)
+        if (   lagr_model->physical_model == CS_LAGR_PHYS_COAL
+            && lagr_model->fouling == 1) {
           fprintf(flal,
-                  "# column 11: inst. number of resuspended particles\n"
-                  "# column 12: inst. number of resuspended particles (weighted)\n"
-                  "# column 13: inst. number of lost particles\n"
-                  "#\n");
+                  "# column %2d: inst. number of fouled particles (coal)\n"
+                  "# column %2d: inst. number of fouled particles (coal, weighted)\n",
+                  col_id, col_id+1);
+          col_id += 2;
+        }
 
-        else
+        else if (lagr_model->resuspension > 0) {
           fprintf(flal,
-                  "# column 11: inst. number of lost particles\n"
-                  "#\n");
+                  "# column %2d: inst. number of resuspended particles\n"
+                  "# column %2d: inst. number of resuspended particles (weighted)\n",
+                  col_id, col_id+1);
+          col_id += 2;
+        }
+
+        fprintf(flal,
+                "# column %2d: inst. number of lost particles\n"
+                "#\n",
+                col_id);
 
       }
 
-      /* ====================================================================
-       * 2 - Ecriture des INFORMATIONS
-       * ====================================================================   */
+      /* Write output */
 
-      if (   lagr_model->physical_model == 2
+      fprintf(flal, " %8d %11.4e %8llu %11.4e %8llu %11.4e %8llu %11.4e %8llu %11.4e",
+              cs_glob_time_step->nt_cur,
+              ttcabs,
+              (unsigned long long)(pc->n_g_total), pc->w_total,
+              (unsigned long long)(pc->n_g_new), pc->w_new,
+              (unsigned long long)(pc->n_g_exit - pc->n_g_fouling),
+              pc->w_exit - pc->w_fouling,
+              (unsigned long long)(pc->n_g_deposited), pc->w_deposited);
+
+      if (cs_glob_lagr_model->agglomeration)
+        fprintf(flal, " %8llu %11.4e",
+                (unsigned long long)(pc->n_g_merged), pc->w_merged);
+
+      if (   lagr_model->physical_model == CS_LAGR_PHYS_COAL
           && lagr_model->fouling == 1)
-        fprintf(flal, " %8d %11.4E %8llu %11.4E %8llu %11.4E %8llu %11.4E %8llu %11.4E %8llu %11.4E %8llu\n",
-                cs_glob_time_step->nt_cur,
-                ttcabs,
-                (unsigned long long)(pc->n_g_total), pc->w_total,
-                (unsigned long long)(pc->n_g_new), pc->w_new,
-                (unsigned long long)(pc->n_g_exit - pc->n_g_fouling),
-                pc->w_exit - pc->w_fouling,
-                (unsigned long long)(pc->n_g_deposited), pc->w_deposited,
-                (unsigned long long)(pc->n_g_fouling), pc->w_fouling,
-                (unsigned long long)(pc->n_g_failed));
+        fprintf(flal, " %8llu %11.4e",
+                (unsigned long long)(pc->n_g_fouling), pc->w_fouling);
 
       else if (lagr_model->resuspension > 0)
-        fprintf(flal, " %8d %11.4E %8llu %11.4E %8llu %11.4E %8llu %11.4E %8llu %11.4E %8llu %11.4E %8llu\n",
-                cs_glob_time_step->nt_cur,
-                ttcabs,
-                (unsigned long long)(pc->n_g_total), pc->w_total,
-                (unsigned long long)(pc->n_g_new), pc->w_new,
-                (unsigned long long)(pc->n_g_exit), pc->w_exit,
-                (unsigned long long)(pc->n_g_deposited), pc->w_deposited,
-                (unsigned long long)(pc->n_g_resuspended), pc->w_resuspended,
-                (unsigned long long)(pc->n_g_failed));
+        fprintf(flal, " %8llu %11.4e",
+                (unsigned long long)(pc->n_g_resuspended), pc->w_resuspended);
 
-      else
-        fprintf(flal, " %8d %11.4E %8llu %11.4E %8llu %11.4E %8llu %11.4E %8llu %11.4E %8llu\n",
-                cs_glob_time_step->nt_cur,
-                ttcabs,
-                (unsigned long long)(pc->n_g_total), pc->w_total,
-                (unsigned long long)(pc->n_g_new), pc->w_new,
-                (unsigned long long)(pc->n_g_exit),
-                pc->w_exit,
-                (unsigned long long)(pc->n_g_deposited), pc->w_deposited,
-                (unsigned long long)(pc->n_g_failed));
+      fprintf(flal, " %8llu\n",
+              (unsigned long long)(pc->n_g_failed));
 
     }
 
