@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -48,11 +48,13 @@
 
 #include "cs_field.h"
 #include "cs_field_pointer.h"
+#include "cs_lagr.h"
 #include "cs_lagr_extract.h"
 #include "cs_lagr_tracking.h"
 #include "cs_log.h"
 #include "cs_map.h"
 #include "cs_parall.h"
+#include "cs_physical_constants.h"
 #include "cs_prototypes.h"
 #include "cs_mesh_location.h"
 #include "cs_restart.h"
@@ -123,150 +125,22 @@ _lagr_section_name(cs_lagr_attribute_t  attr,
 }
 
 /*----------------------------------------------------------------------------
- * Build particle values section name
+ * Build legacy particle values section name or formats for names
  *
  * parameters:
- *   base     <-- base name
- *   comp_id  <-- component id, or -1 for all
- *   sec_name --> associated section name
+ *   attr     <-- attribute
+ *   sec_name --> associated section name, component 0
  *----------------------------------------------------------------------------*/
 
 static void
-_section_name(const char  *base,
-              int          comp_id,
-              char         sec_name[128])
+_legacy_section_name(cs_lagr_attribute_t  attr,
+                     char                 sec_name[128])
 {
-  size_t l;
-
-  if (comp_id < 0)
-    l= snprintf(sec_name, 127, "%s::vals::0", base);
-  else
-    l= snprintf(sec_name, 127, "%s::vals::%d::0", base, comp_id);
-
-  sec_name[127] = '\0';
-  for (size_t i = 0; i < l; i++)
-    sec_name[i] = tolower(sec_name[i]);
-}
-
-/*----------------------------------------------------------------------------
- * Build legacy particle values section names or formats for names
- *
- * parameters:
- *   attr       <-- attribute
- *   comp_id    <-- component id, or -1 for all
- *   sec_name_0 --> associated section name, component 0
- *   sec_name_0 --> associated section name, component 1
- *   sec_name_0 --> associated section name, component 2
- *----------------------------------------------------------------------------*/
-
-static void
-_legacy_section_names(cs_lagr_attribute_t  attr,
-                      int                  comp_id,
-                      char                 sec_name_0[128],
-                      char                 sec_name_1[128],
-                      char                 sec_name_2[128])
-{
-  sec_name_0[0] = '\0';
-  sec_name_1[0] = '\0';
-  sec_name_2[0] = '\0';
+  sec_name[0] = '\0';
 
   switch(attr) {
   case CS_LAGR_RANDOM_VALUE:
-    strcpy(sec_name_0, "random_value");
-    break;
-  case CS_LAGR_STAT_WEIGHT:
-    strcpy(sec_name_0, "poids_statistiques_particules");
-    break;
-  case CS_LAGR_RESIDENCE_TIME:
-    strcpy(sec_name_0, "temps_sejour_particules");
-    break;
-  case CS_LAGR_MASS:
-    strcpy(sec_name_0, "variable_masse_particule");
-    break;
-  case CS_LAGR_DIAMETER:
-    strcpy(sec_name_0, "variable_diametre_particule");
-    break;
-  case CS_LAGR_VELOCITY:
-    strcpy(sec_name_0, "variable_vitesseU_particule");
-    strcpy(sec_name_1, "variable_vitesseV_particule");
-    strcpy(sec_name_2, "variable_vitesseW_particule");
-    break;
-  case CS_LAGR_VELOCITY_SEEN:
-    strcpy(sec_name_0, "variable_vitesseU_fluide_vu");
-    strcpy(sec_name_1, "variable_vitesseV_fluide_vu");
-    strcpy(sec_name_2, "variable_vitesseW_fluide_vu");
-    break;
-  case CS_LAGR_YPLUS:
-    strcpy(sec_name_0, "yplus_particules");
-    break;
-  case CS_LAGR_INTERF:
-    strcpy(sec_name_0, "dx_particules");
-    break;
-  case CS_LAGR_NEIGHBOR_FACE_ID:
-    strcpy(sec_name_0, "dfac_particules");
-    break;
-  case CS_LAGR_MARKO_VALUE:
-    strcpy(sec_name_0, "indicateur_de_saut");
-    break;
-  case CS_LAGR_DEPOSITION_FLAG:
-    strcpy(sec_name_0, "part_depo");
-    break;
-  case CS_LAGR_N_LARGE_ASPERITIES:
-    strcpy(sec_name_0, "nb_ls_aspe");
-    break;
-  case CS_LAGR_N_SMALL_ASPERITIES:
-    strcpy(sec_name_0, "nb_sms_aspe");
-    break;
-  case CS_LAGR_ADHESION_FORCE:
-    strcpy(sec_name_0, "force_adhesion");
-    break;
-  case CS_LAGR_ADHESION_TORQUE:
-    strcpy(sec_name_0, "moment_adhesion");
-    break;
-  case CS_LAGR_DISPLACEMENT_NORM:
-    strcpy(sec_name_0, "disp_norm");
-    break;
-  case CS_LAGR_TEMPERATURE:
-    if (comp_id < 0)
-      strcpy(sec_name_0, "variable_temperature_particule");
-    else
-      sprintf(sec_name_0, "variable_temperature_particule_couche_%04d", comp_id+1);
-    break;
-  case CS_LAGR_FLUID_TEMPERATURE:
-    strcpy(sec_name_0, "variable_temperature_fluide_vu");
-    break;
-  case CS_LAGR_CP:
-    strcpy(sec_name_0, "variable_chaleur_specifique_particule");
-    break;
-  case CS_LAGR_WATER_MASS:
-    strcpy(sec_name_0, "variable_masse_humidite");
-    break;
-  case CS_LAGR_COAL_MASS:
-    sprintf(sec_name_0, "variable_masse_charbon_reactif_%04d", comp_id+1);
-    break;
-  case CS_LAGR_COKE_MASS:
-    sprintf(sec_name_0, "variable_masse_coke_couche_%04d", comp_id+1);
-    break;
-  case CS_LAGR_SHRINKING_DIAMETER:
-    strcpy(sec_name_0, "diametre_coeur_retrecissant_charbon");
-    break;
-  case CS_LAGR_INITIAL_DIAMETER:
-    strcpy(sec_name_0, "diametre_initial_charbon");
-    break;
-  case CS_LAGR_COAL_NUM:
-    strcpy(sec_name_0, "numero_charbon");
-    break;
-  case CS_LAGR_COAL_DENSITY:
-    sprintf(sec_name_0, "masse_volumique_coke_couche_%04d", comp_id+1);
-    break;
-  case CS_LAGR_EMISSIVITY:
-    strcpy(sec_name_0, "emissivite_particules");
-    break;
-  case CS_LAGR_STAT_CLASS:
-    strcpy(sec_name_0, "numero_groupe_statistiques");
-    break;
-  case CS_LAGR_USER:
-    sprintf(sec_name_0, "variable_supplementaire_%04d", comp_id+1);
+    strcpy(sec_name, "random_value");
     break;
   default:
     break;
@@ -422,44 +296,22 @@ _init_particle_values(cs_lagr_particle_set_t  *particles,
   case CS_LAGR_TEMPERATURE:
   case CS_LAGR_FLUID_TEMPERATURE:
     {
-      /* Determine temperature or enthalpy */
+      cs_lagr_extra_module_t *extra = cs_glob_lagr_extra_module;
       double c_kelvin = 0;
       const double *t = NULL;
-      double *h = NULL;
-      if (t == NULL) {
-        const cs_field_t *f = cs_field_by_name_try("t_gas");
-        if (f != NULL) {
-          t = f->val;
-          c_kelvin = 273.15;
-        }
-      }
-      if (t == NULL) {
-        const cs_field_t *f = cs_field_by_name_try("t_celcius");
-        if (f != NULL)
-          t = f->val;
-      }
-      if (t == NULL) {
-        const cs_field_t *f = cs_field_by_name_try("temperature");
-        if (f != NULL) {
-          t = f->val;
-          if (cs_glob_thermal_model->itpscl == 1)
-            c_kelvin = 273.15;
-        }
-      }
-      if (t == NULL) {
-        const cs_field_t *f = cs_field_by_name_try("enthalpy");
-        if (f != NULL)
-          h = f->val;
-      }
+
+      if (extra->temperature != NULL)
+        t = extra->temperature->val;
+      if (cs_glob_thermal_model->itpscl == CS_TEMPERATURE_SCALE_KELVIN)
+        c_kelvin = 273.15;
 
       assert(datatype == CS_REAL_TYPE);
 
       /* initialize particle temperature with fluid temperature */
       if (t != NULL) {
         for (cs_lnum_t i = 0; i < n_particles; i++) {
-          cs_lnum_t cell_num
-            = cs_lagr_particles_get_lnum(particles, i, CS_LAGR_CELL_NUM);
-          cs_lnum_t cell_id = CS_ABS(cell_num) - 1;
+          cs_lnum_t cell_id
+            = cs_lagr_particles_get_lnum(particles, i, CS_LAGR_CELL_ID);
           cs_real_t *part_val
             = cs_lagr_particles_attr_n(particles, i, 0, attr);
           for (cs_lnum_t j = 0; j < stride; j++)
@@ -467,27 +319,13 @@ _init_particle_values(cs_lagr_particle_set_t  *particles,
         }
       }
 
-      /* initialize particle temperature with enthalpy */
-      else if (h != NULL) {
-        int mode = 1;
-        for (cs_lnum_t i = 0; i < n_particles; i++) {
-          cs_lnum_t cell_num
-            = cs_lagr_particles_get_lnum(particles, i, CS_LAGR_CELL_NUM);
-          cs_lnum_t cell_id = CS_ABS(cell_num) - 1;
-          cs_real_t *part_val
-            = cs_lagr_particles_attr_n(particles, i, 0, attr);
-          for (cs_lnum_t j = 0; j < stride; j++)
-            CS_PROCF(usthht, USTHHT)(&mode, h + cell_id, part_val + j);
-        }
-      }
-
-      /* set to zero */
+      /* set to reference temperature */
       else {
         for (cs_lnum_t i = 0; i < n_particles; i++) {
           cs_real_t *part_val
             = cs_lagr_particles_attr_n(particles, i, 0, attr);
           for (cs_lnum_t j = 0; j < stride; j++)
-            part_val[j] = 0.0;
+            part_val[j] = cs_glob_fluid_properties->t0 - c_kelvin;
         }
       }
     }
@@ -557,7 +395,7 @@ cs_lagr_restart_read_particle_data(cs_restart_t  *r)
   unsigned char *vals = NULL;
   size_t max_size = 0;
 
-  char sec_name[128], old_name_0[128], old_name_1[128], old_name_2[128];
+  char sec_name[128], old_name[128];
 
   /* Initialization */
 
@@ -592,15 +430,15 @@ cs_lagr_restart_read_particle_data(cs_restart_t  *r)
   /* Read coordinates and get mesh location */
   /*-----------------------------------------*/
 
-  cs_lnum_t  *p_cell_num;
+  cs_lnum_t  *p_cell_id;
   cs_real_t  *p_coords;
 
-  BFT_MALLOC(p_cell_num, n_particles, cs_lnum_t);
+  BFT_MALLOC(p_cell_id, n_particles, cs_lnum_t);
   BFT_MALLOC(p_coords, n_particles*3, cs_real_t);
 
   sec_code = cs_restart_read_particles(r,
                                        particles_location_id,
-                                       p_cell_num,
+                                       p_cell_id,
                                        p_coords);
 
   if (sec_code == CS_RESTART_SUCCESS) {
@@ -613,12 +451,12 @@ cs_lagr_restart_read_particle_data(cs_restart_t  *r)
     _set_particle_values(p_set, CS_LAGR_COORDS, CS_REAL_TYPE,
                          3, -1, p_coords);
 
-    _set_particle_values(p_set, CS_LAGR_CELL_NUM, CS_LNUM_TYPE,
-                         1, -1, p_cell_num);
+    _set_particle_values(p_set, CS_LAGR_CELL_ID, CS_LNUM_TYPE,
+                         1, -1, p_cell_id);
 
   }
 
-  BFT_FREE(p_cell_num);
+  BFT_FREE(p_cell_id);
   BFT_FREE(p_coords);
 
   if (sec_code == CS_RESTART_SUCCESS)
@@ -642,7 +480,7 @@ cs_lagr_restart_read_particle_data(cs_restart_t  *r)
       continue;
 
     if (datatype == CS_LNUM_TYPE)
-      restart_type = CS_TYPE_cs_int_t;
+      restart_type = CS_TYPE_int;
     else if (datatype == CS_GNUM_TYPE)
       restart_type = CS_TYPE_cs_gnum_t;
     else {
@@ -653,33 +491,101 @@ cs_lagr_restart_read_particle_data(cs_restart_t  *r)
     switch(attr) {
 
     case CS_LAGR_COORDS:
+    case CS_LAGR_CELL_ID:
     case CS_LAGR_RANK_ID:
       break;
 
     case CS_LAGR_NEIGHBOR_FACE_ID:
-      _lagr_section_name(attr, -1, sec_name);
-      sec_code = cs_restart_read_ids(r,
-                                     sec_name,
-                                     particles_location_id,
-                                     CS_MESH_LOCATION_BOUNDARY_FACES,
-                                     1, /* numbering base */
-                                     (cs_lnum_t *)vals);
-      for (cs_lnum_t i = 0; i < p_set->n_particles; i++)
-        ((cs_lnum_t *)vals)[i] -= 1;
-      if (sec_code == CS_RESTART_SUCCESS) {
+      {
+        cs_lnum_t *face_id = (cs_lnum_t *)vals;
+
+        /* Initialize to default */
+
+        for (cs_lnum_t i = 0; i < p_set->n_particles; i++)
+          face_id[i] = -1;
+
+        /* Now read global numbers (1 to n, 0 for none) */
+
+        cs_gnum_t *gnum_read = NULL;
+        BFT_MALLOC(gnum_read, p_set->n_particles, cs_gnum_t);
+
+        snprintf(sec_name, 127, "particle_%s::vals::0", "neighbor_face_num");
+        _legacy_section_name(attr, old_name);
+
+        sec_code = cs_restart_read_section_compat(r,
+                                                  sec_name,
+                                                  old_name,
+                                                  particles_location_id,
+                                                  stride,
+                                                  CS_TYPE_cs_gnum_t,
+                                                  gnum_read);
+
+        if (sec_code == CS_RESTART_SUCCESS)
+          retval += 1;
+        else {
+          for (cs_lnum_t i = 0; i < p_set->n_particles; i++)
+            gnum_read[i] = 0;
+        }
+
+        /* Now assign values */
+
+        const cs_lnum_t   *cell_b_faces_idx = NULL;
+        const cs_lnum_t   *cell_b_faces = NULL;
+        const cs_mesh_adjacencies_t *ma = cs_glob_mesh_adjacencies;
+
+        if (ma != NULL) {
+          cell_b_faces_idx = ma->cell_b_faces_idx;
+          cell_b_faces = ma->cell_b_faces;
+        }
+
+        if (cell_b_faces_idx != NULL) {
+          const cs_gnum_t *g_b_face_num = cs_glob_mesh->global_b_face_num;
+          if (g_b_face_num != NULL) {
+            for (cs_lnum_t i = 0; i < p_set->n_particles; i++) {
+              if (gnum_read[i] > 0) {
+                cs_lnum_t cell_id = cs_lagr_particles_get_lnum(p_set, i,
+                                                               CS_LAGR_CELL_ID);
+                cs_lnum_t s_id = cell_b_faces_idx[cell_id];
+                cs_lnum_t e_id = cell_b_faces_idx[cell_id+1];
+                for (cs_lnum_t j = s_id; j < e_id; j++) {
+                  if (g_b_face_num[cell_b_faces[j]] == gnum_read[i]) {
+                    face_id[i] = cell_b_faces[j];
+                    break;
+                  }
+                }
+              }
+            }
+          }
+          else {
+            for (cs_lnum_t i = 0; i < p_set->n_particles; i++) {
+              if (gnum_read[i] > 0) {
+                cs_lnum_t cell_id = cs_lagr_particles_get_lnum(p_set, i,
+                                                               CS_LAGR_CELL_ID);
+                cs_lnum_t s_id = cell_b_faces_idx[cell_id];
+                cs_lnum_t e_id = cell_b_faces_idx[cell_id+1];
+                for (cs_lnum_t j = s_id; j < e_id; j++) {
+                  if ((cs_gnum_t)(j+1) == gnum_read[i]) {
+                    face_id[i] = cell_b_faces[j];
+                    break;
+                  }
+                }
+              }
+            }
+          }
+        }
+        else {
+          assert(cell_b_faces != NULL);
+        }
+
+        BFT_FREE(gnum_read);
+
         _set_particle_values(p_set,
                              attr,
                              CS_LNUM_TYPE,
                              1,
                              -1,
                              vals);
-        if (attr == CS_LAGR_STAT_WEIGHT) {
-          cs_real_t *w = (cs_real_t *)vals;
-          for (cs_lnum_t i = 0; i < p_set->n_particles; i++)
-            p_set->weight += w[i];
-        }
       }
-      retval += 1;
       break;
 
     default:
@@ -698,58 +604,31 @@ cs_lagr_restart_read_particle_data(cs_restart_t  *r)
       for (int s_id = 0; s_id < n_sections; s_id++) {
 
         int comp_id = (n_sections == 1) ? -1 : s_id;
+        int c_stride = (n_sections == 1) ? stride : 1;
 
         _lagr_section_name(attr, comp_id, sec_name);
-        _legacy_section_names(attr, comp_id,
-                              old_name_0, old_name_1, old_name_2);
-
-        /* Special case for cell number: sign used for deposition */
-
-        if (attr == CS_LAGR_CELL_NUM) {
-          _section_name("particle_status_flag", comp_id, sec_name);
-          strcpy(old_name_0, "particle_status_flag");
-        }
+        _legacy_section_name(attr, old_name);
 
         /* Try to read data */
 
-        if (   attr == CS_LAGR_VELOCITY
-            || attr == CS_LAGR_VELOCITY_SEEN) {
-          sec_code = cs_restart_read_real_3_t_compat(r,
-                                                     sec_name,
-                                                     old_name_0,
-                                                     old_name_1,
-                                                     old_name_2,
-                                                     particles_location_id,
-                                                     (cs_real_3_t *)vals);
-        }
-        else
-          sec_code = cs_restart_read_section_compat(r,
-                                                    sec_name,
-                                                    old_name_0,
-                                                    particles_location_id,
-                                                    stride,
-                                                    restart_type,
-                                                    vals);
+        sec_code = cs_restart_read_section_compat(r,
+                                                  sec_name,
+                                                  old_name,
+                                                  particles_location_id,
+                                                  c_stride,
+                                                  restart_type,
+                                                  vals);
 
         /* Finish setting values */
 
-        if (sec_code == CS_RESTART_SUCCESS && attr == CS_LAGR_CELL_NUM) {
+        if (sec_code == CS_RESTART_SUCCESS) {
 
-          cs_lnum_t  *flag = (cs_lnum_t *)vals;
-          for (cs_lnum_t j = 0; j < n_particles; j++) {
-            if (flag[j] == 1) {
-              cs_lnum_t cell_num
-                = cs_lagr_particles_get_lnum(p_set, j, CS_LAGR_CELL_NUM);
-              cs_lagr_particles_set_lnum(p_set, j, CS_LAGR_CELL_NUM, -cell_num);
-            }
+          if (attr == CS_LAGR_STAT_WEIGHT) {
+            cs_real_t *w = (cs_real_t *)vals;
+            for (cs_lnum_t i = 0; i < p_set->n_particles; i++)
+              p_set->weight += w[i];
           }
 
-          retval += 1;
-
-        }
-
-        else if (sec_code == CS_RESTART_SUCCESS) {
-
           _set_particle_values(p_set,
                                attr,
                                datatype,
@@ -812,27 +691,27 @@ cs_lagr_restart_write_particle_data(cs_restart_t  *r)
 
   _lagr_section_name(CS_LAGR_COORDS, -1, sec_name);
 
-  cs_lnum_t  *p_cell_num;
+  cs_lnum_t  *p_cell_id;
   cs_real_t  *p_coords;
 
-  BFT_MALLOC(p_cell_num, n_particles, cs_lnum_t);
+  BFT_MALLOC(p_cell_id, n_particles, cs_lnum_t);
   BFT_MALLOC(p_coords, n_particles*3, cs_real_t);
 
   cs_lagr_get_particle_values(p_set, CS_LAGR_COORDS, CS_REAL_TYPE,
                               3, -1, n_particles, NULL, p_coords);
 
-  cs_lagr_get_particle_values(p_set, CS_LAGR_CELL_NUM, CS_LNUM_TYPE,
-                              1, -1, n_particles, NULL, p_cell_num);
+  cs_lagr_get_particle_values(p_set, CS_LAGR_CELL_ID, CS_LNUM_TYPE,
+                              1, -1, n_particles, NULL, p_cell_id);
 
   const int particles_location_id
     = cs_restart_write_particles(r,
                                  sec_name,
                                  false, /* number_by_coords */
                                  n_particles,
-                                 p_cell_num,
+                                 p_cell_id,
                                  p_coords);
 
-  BFT_FREE(p_cell_num);
+  BFT_FREE(p_cell_id);
   BFT_FREE(p_coords);
 
   retval = 1;
@@ -849,7 +728,7 @@ cs_lagr_restart_write_particle_data(cs_restart_t  *r)
       continue;
 
     if (datatype == CS_LNUM_TYPE)
-      restart_type = CS_TYPE_cs_int_t;
+      restart_type = CS_TYPE_int;
     else if (datatype == CS_GNUM_TYPE)
       restart_type = CS_TYPE_cs_gnum_t;
     else {
@@ -860,28 +739,57 @@ cs_lagr_restart_write_particle_data(cs_restart_t  *r)
     switch(attr) {
 
     case CS_LAGR_COORDS:
+    case CS_LAGR_CELL_ID:
     case CS_LAGR_RANK_ID:
       break;
 
     case CS_LAGR_NEIGHBOR_FACE_ID:
-      cs_lagr_get_particle_values(p_set,
-                                  attr,
-                                  CS_LNUM_TYPE,
-                                  1,
-                                  -1,
-                                  n_particles,
-                                  NULL,
-                                  vals);
-      _lagr_section_name(attr, -1, sec_name);
-      for (cs_lnum_t i = 0; i < p_set->n_particles; i++)
-        ((cs_lnum_t *)vals)[i] += 1;
-      cs_restart_write_ids(r,
-                           sec_name,
-                           particles_location_id,
-                           CS_MESH_LOCATION_BOUNDARY_FACES,
-                           1, /* numbering base */
-                           (const cs_lnum_t *)vals);
-      retval += 1;
+      {
+        cs_lagr_get_particle_values(p_set,
+                                    attr,
+                                    CS_LNUM_TYPE,
+                                    1,
+                                    -1,
+                                    n_particles,
+                                    NULL,
+                                    vals);
+
+        cs_gnum_t *gnum_write = NULL;
+        BFT_MALLOC(gnum_write, p_set->n_particles, cs_gnum_t);
+
+        const cs_gnum_t *g_b_face_num = cs_glob_mesh->global_b_face_num;
+        if (g_b_face_num != NULL) {
+          for (cs_lnum_t i = 0; i < p_set->n_particles; i++) {
+            cs_lnum_t face_id = ((cs_lnum_t *)vals)[i];
+            if (face_id < 0)
+              gnum_write[i] = 0;
+            else
+              gnum_write[i] = g_b_face_num[face_id];
+          }
+        }
+        else {
+          for (cs_lnum_t i = 0; i < p_set->n_particles; i++) {
+            cs_lnum_t face_id = ((cs_lnum_t *)vals)[i];
+            if (face_id < 0)
+              gnum_write[i] = 0;
+            else
+              gnum_write[i] = face_id + 1;
+          }
+        }
+
+        snprintf(sec_name, 127, "particle_%s::vals::0", "neighbor_face_num");
+
+        cs_restart_write_section(r,
+                                 sec_name,
+                                 particles_location_id,
+                                 1,
+                                 CS_TYPE_cs_gnum_t,
+                                 gnum_write);
+
+        BFT_FREE(gnum_write);
+
+        retval += 1;
+      }
       break;
 
     default:
@@ -913,19 +821,6 @@ cs_lagr_restart_write_particle_data(cs_restart_t  *r)
 
         _lagr_section_name(attr, comp_id, sec_name);
 
-        /* Special case for cell number: sign used for deposition */
-
-        if (attr == CS_LAGR_CELL_NUM) {
-          cs_lnum_t  *flag = (cs_lnum_t *)vals;
-          for (cs_lnum_t j = 0; j < n_particles; j++) {
-            if (flag[j] < 0)
-              flag[j] = 1;
-            else
-              flag[j] = 0;
-          }
-          _section_name("particle_status_flag", comp_id, sec_name);
-        }
-
         cs_restart_write_section(r,
                                  sec_name,
                                  particles_location_id,
