@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -55,7 +55,6 @@
 #include "cs_math.h"
 #include "cs_physical_constants.h"
 #include "cs_physical_model.h"
-#include "cs_prototypes.h"
 
 #include "cs_mesh.h"
 #include "cs_thermal_model.h"
@@ -66,6 +65,8 @@
 
 #include "cs_lagr_sde.h"
 
+#include "cs_prototypes.h"
+
 /*----------------------------------------------------------------------------
  *  Header for the current file
  *----------------------------------------------------------------------------*/
@@ -159,12 +160,12 @@ _lagtmp(cs_lnum_t        npt,
         const cs_real_t  phith[],
         cs_real_t        temp[])
 {
-  cs_lnum_t  l_id;
+  int l_id;
 
   cs_lagr_extra_module_t *extra = cs_glob_lagr_extra_module;
 
-  cs_lnum_t nlayer = cs_glob_lagr_const_dim->nlayer;
-  cs_lnum_t l_id_max = nlayer - 1;
+  int nlayer = cs_glob_lagr_const_dim->nlayer;
+  int l_id_max = nlayer - 1;
 
   cs_real_t delray[nlayer], radiusd[nlayer], rho[nlayer];
 
@@ -202,9 +203,8 @@ _lagtmp(cs_lnum_t        npt,
 
   cs_real_t dd2 = cs_math_sq(p_diam);
 
-  cs_lnum_t cell_id  = cs_lagr_particle_get_cell_id(particle, p_am);
-  cs_lnum_t co_id = cs_lagr_particle_get_lnum(particle, p_am,
-                                              CS_LAGR_COAL_ID);
+  cs_lnum_t cell_id  = cs_lagr_particle_get_lnum(particle, p_am, CS_LAGR_CELL_ID);
+  cs_lnum_t co_id = cs_lagr_particle_get_lnum(particle, p_am, CS_LAGR_COAL_ID);
 
   /* Multiple-layer resolution
      ------------------------- */
@@ -479,7 +479,7 @@ _lagsec(cs_lnum_t         npt,
     fwatsat[l_id] = 0.0;
   }
 
-  cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
+  cs_lnum_t cell_id  = cs_lagr_particle_get_lnum(particle, p_am, CS_LAGR_CELL_ID);
 
   /* find layer */
 
@@ -687,7 +687,7 @@ _lagsec(cs_lnum_t         npt,
 }
 
 /*----------------------------------------------------------------------------
- * Integrate stochastic differential equations par particles mass.
+ * Integrate stochastic differential equations by particles mass.
  *----------------------------------------------------------------------------*/
 
 static void
@@ -696,25 +696,20 @@ _lagimp(void)
   /* Particles management */
 
   cs_lagr_particle_set_t  *p_set = cs_glob_lagr_particle_set;
-  const cs_lagr_attribute_map_t  *p_am = p_set->p_am;
 
   cs_real_t *tcarac, *pip;
 
   BFT_MALLOC(tcarac, p_set->n_particles, cs_real_t);
   BFT_MALLOC(pip,    p_set->n_particles, cs_real_t);
 
-  for (cs_lnum_t npt = 0; npt < p_set->n_particles; npt++) {
+  for (cs_lnum_t ip = 0; ip < p_set->n_particles; ip++) {
 
-    unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
+    /* FIXME check this: a fixed particle's mass might still evolve */
+    if (cs_lagr_particles_get_flag(p_set, ip, CS_LAGR_PART_FIXED))
+      continue;
 
-    cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
-
-    if (cell_id >= 0) {
-
-      tcarac[npt] = 1.0;
-      pip[npt]    = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_MASS);
-
-    }
+    tcarac[ip] = 1.0;
+    pip[ip]    = cs_lagr_particles_get_real(p_set, ip, CS_LAGR_MASS);
 
   }
 
@@ -734,25 +729,20 @@ _lagidp(void)
   /* Particles management */
 
   cs_lagr_particle_set_t  *p_set = cs_glob_lagr_particle_set;
-  const cs_lagr_attribute_map_t  *p_am = p_set->p_am;
 
   cs_real_t *tcarac, *pip ;
 
   BFT_MALLOC(tcarac, p_set->n_particles, cs_real_t);
   BFT_MALLOC(pip,    p_set->n_particles, cs_real_t);
 
-  for (cs_lnum_t npt = 0; npt < p_set->n_particles; npt++) {
+  for (cs_lnum_t ip = 0; ip < p_set->n_particles; ip++) {
 
-    unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
+    /* FIXME check this: a fixed particle's diameter might still evolve */
+    if (cs_lagr_particles_get_flag(p_set, ip, CS_LAGR_PART_FIXED))
+      continue;
 
-    cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
-
-    if (cell_id >= 0) {
-
-      tcarac[npt] = 1.0;
-      pip[npt]    = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_DIAMETER);
-
-    }
+    tcarac[ip] = 1.0;
+    pip[ip]    = cs_lagr_particles_get_real(p_set, ip, CS_LAGR_DIAMETER);
 
   }
 
@@ -786,20 +776,16 @@ _lagitp(const cs_real_t  tempct[])
    * REMPLISSAGE DU TEMPS CARACTERISTIQUE ET DU "PSEUDO SECOND MEMBRE"
    *=========================================================================== */
 
-  for (cs_lnum_t npt = 0; npt < p_set->n_particles; npt++) {
+  for (cs_lnum_t ip = 0; ip < p_set->n_particles; ip++) {
 
-    unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
+    /* FIXME check this: a fixed particle'ss temperature might still evolve */
+    if (cs_lagr_particles_get_flag(p_set, ip, CS_LAGR_PART_FIXED))
+      continue;
 
-    cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
+    tcarac[ip] = tempct[ip];
 
-    if (cell_id >= 0) {
-
-      tcarac[npt] = tempct[npt];
-
-      pip[npt] = cs_lagr_particle_get_real_n(particle, p_am, 2 - nor,
-                                             CS_LAGR_FLUID_TEMPERATURE);
-
-    }
+    pip[ip] = cs_lagr_particles_get_real_n(p_set, ip, 2 - nor,
+                                           CS_LAGR_FLUID_TEMPERATURE);
 
   }
 
@@ -809,42 +795,47 @@ _lagitp(const cs_real_t  tempct[])
 
   if (extra->radiative_model > 0) {
 
-    for (cs_lnum_t npt = 0; npt < p_set->n_particles; npt++) {
+    for (cs_lnum_t ip = 0; ip < p_set->n_particles; ip++) {
 
-      unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
+      /* FIXME check this: a fixed particle might still radiate */
+      if (cs_lagr_particles_get_flag(p_set, ip, CS_LAGR_PART_FIXED))
+        continue;
 
-      cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
+      unsigned char *particle = p_set->p_buffer + p_am->extents * ip;
 
-      if (cell_id >= 0) {
+      cs_lnum_t cell_id = cs_lagr_particle_get_lnum(particle, p_am,
+                                                    CS_LAGR_CELL_ID);
 
-        cs_real_t p_mass = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_MASS);
-        cs_real_t p_cp   = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_CP);
-        cs_real_t p_eps  = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_EMISSIVITY);
+      cs_real_t p_mass = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_MASS);
+      cs_real_t p_cp   = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_CP);
+      cs_real_t p_eps  = cs_lagr_particle_get_real(particle, p_am, CS_LAGR_EMISSIVITY);
 
-        if (nor == 1) {
+      if (nor == 1) {
 
-          cs_real_t prev_p_diam = cs_lagr_particle_get_real_n(particle, p_am,
-                                                              1, CS_LAGR_DIAMETER);
-          cs_real_t prev_p_temp = cs_lagr_particle_get_real_n(particle, p_am,
-                                                              1, CS_LAGR_TEMPERATURE);
+        cs_real_t prev_p_diam = cs_lagr_particle_get_real_n(particle, p_am,
+                                                            1, CS_LAGR_DIAMETER);
+        cs_real_t prev_p_temp = cs_lagr_particle_get_real_n(particle, p_am,
+                                                            1, CS_LAGR_TEMPERATURE);
 
-          cs_real_t srad =  cs_math_pi * pow(prev_p_diam, 2.0) * p_eps
-                          * (extra->luminance->val[cell_id] - 4.0 * _c_stephan * pow (prev_p_temp,4));
-          pip[npt] =  cs_lagr_particle_get_real_n(particle, p_am, 1, CS_LAGR_FLUID_TEMPERATURE)
-                    + tcarac[npt] * srad / p_cp / p_mass;
+        cs_real_t srad =    cs_math_pi * pow(prev_p_diam, 2.0) * p_eps
+                          * (extra->luminance->val[cell_id]
+                        - 4.0 * _c_stephan * pow (prev_p_temp,4));
+        pip[ip] =   cs_lagr_particle_get_real_n(particle, p_am, 1,
+                                                 CS_LAGR_FLUID_TEMPERATURE)
+                   + tcarac[ip] * srad / p_cp / p_mass;
 
-        }
-        else{
+      }
+      else {
 
-          cs_real_t p_diam = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_DIAMETER);
-          cs_real_t p_temp = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_TEMPERATURE);
+        cs_real_t p_diam = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_DIAMETER);
+        cs_real_t p_temp = cs_lagr_particle_get_real_n(particle, p_am, 0, CS_LAGR_TEMPERATURE);
 
-          cs_real_t srad =  cs_math_pi * pow(p_diam, 2.0) * p_eps
-                          * (extra->luminance->val[cell_id] - 4.0 * _c_stephan *  pow(p_temp , 4));
-          pip[npt] =  cs_lagr_particle_get_real(particle, p_am, CS_LAGR_FLUID_TEMPERATURE)
-                    + tcarac[npt] * srad / p_cp /p_mass;
-
-        }
+        cs_real_t srad =    cs_math_pi * pow(p_diam, 2.0) * p_eps
+                          * (extra->luminance->val[cell_id]
+                        - 4.0 * _c_stephan *  pow(p_temp , 4));
+        pip[ip] =  cs_lagr_particle_get_real(particle, p_am,
+                                              CS_LAGR_FLUID_TEMPERATURE)
+                  + tcarac[ip] * srad / p_cp /p_mass;
 
       }
 
@@ -883,10 +874,9 @@ _lagitf(cs_lagr_attribute_t  *iattr)
   BFT_MALLOC(auxl1, p_set->n_particles, cs_real_t);
   BFT_MALLOC(tempf, mesh->n_cells_with_ghosts, cs_real_t);
 
-  /* Initialize variables to avoid compiler warnings    */
+  /* Initialize variables to avoid compiler warnings */
 
   cs_real_t ct   = 1.0;
-  cs_lnum_t mode = 1;
 
   int ltsvar = 0;
 
@@ -895,101 +885,82 @@ _lagitf(cs_lagr_attribute_t  *iattr)
       ltsvar   = 1;
   }
 
-  /* =========================================================================
-   * Mean fluid temperature in degrees C
-   * =========================================================================*/
+  /* Mean fluid temperature in degrees C
+   * =================================== */
 
-  if (   cs_glob_physical_model_flag[CS_COMBUSTION_COAL] >= 0
-      || cs_glob_physical_model_flag[CS_COMBUSTION_PCLC] >= 0
-      || cs_glob_physical_model_flag[CS_COMBUSTION_FUEL] >= 0) {
+  if (   extra->temperature != NULL
+      && cs_glob_thermal_model->itpscl == CS_TEMPERATURE_SCALE_CELSIUS) {
 
     for (cs_lnum_t cell_id = 0; cell_id < mesh->n_cells; cell_id++)
-      tempf[cell_id]  = extra->t_gaz->val[cell_id] - _tkelvi;
+      tempf[cell_id] = extra->temperature->val[cell_id];
 
   }
-  else if (   cs_glob_physical_model_flag[CS_COMBUSTION_3PT] >= 0
-           || cs_glob_physical_model_flag[CS_COMBUSTION_EBU] >= 0
-           || cs_glob_physical_model_flag[CS_ELECTRIC_ARCS] >= 0
-           || cs_glob_physical_model_flag[CS_JOULE_EFFECT] >= 0) {
-
-    for (cs_lnum_t cell_id = 0; cell_id < mesh->n_cells; cell_id++)
-      tempf[cell_id]  = extra->temperature->val[cell_id] - _tkelvi;
-
-  }
-  else if (   cs_glob_thermal_model->itherm == CS_THERMAL_MODEL_TEMPERATURE
-           && cs_glob_thermal_model->itpscl == CS_TEMPERATURE_SCALE_CELSIUS) {
-
-    for (cs_lnum_t cell_id = 0; cell_id < mesh->n_cells; cell_id++)
-      tempf[cell_id]  = extra->scal_t->val[cell_id];
-
-  }
-  else if (   cs_glob_thermal_model->itherm == CS_THERMAL_MODEL_TEMPERATURE
+  else if (   extra->temperature != NULL
            && cs_glob_thermal_model->itpscl == CS_TEMPERATURE_SCALE_KELVIN) {
 
     for (cs_lnum_t cell_id = 0; cell_id < mesh->n_cells; cell_id++)
-      tempf[cell_id]  = extra->scal_t->val[cell_id] - _tkelvi;
+      tempf[cell_id] = extra->temperature->val[cell_id] - _tkelvi;
 
   }
-  else if (cs_glob_thermal_model->itherm == CS_THERMAL_MODEL_ENTHALPY) {
+  else {
 
-    for (cs_lnum_t cell_id = 0; cell_id < mesh->n_cells; cell_id++)
-      CS_PROCF(usthht,USTHHT) (&mode,
-                               &extra->scal_t->val[cell_id],
-                               &tempf[cell_id]);
+    bft_error(__FILE__, __LINE__, 0,
+              _("%s (Lagrangian module):\n\n"
+                "Temperature field is not defined or mapped."), __func__);
 
   }
 
-  /* =========================================================================
-   * 3. INTEGRATION DE L'EDS SUR LES PARTICULES
-   * =========================================================================*/
+  /* Integration of the SDE over particles
+   * ===================================== */
 
-  for (cs_lnum_t npt = 0; npt < p_set->n_particles; npt++) {
+  for (cs_lnum_t ip = 0; ip < p_set->n_particles; ip++) {
 
-    unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
-    cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
+    /* FIXME check this: a fixed particle's temperature might still evolve */
+    if (cs_lagr_particles_get_flag(p_set, ip, CS_LAGR_PART_FIXED))
+      continue;
 
-    if (cell_id >= 0){
+    unsigned char *particle = p_set->p_buffer + p_am->extents * ip;
+    cs_lnum_t cell_id = cs_lagr_particle_get_lnum(particle, p_am,
+                                                  CS_LAGR_CELL_ID);
 
-      if (   extra->itytur == 2 || extra->itytur == 3
-          || extra->iturb == 50 || extra->iturb == 60) {
+    if (   extra->itytur == 2 || extra->itytur == 4 || extra->itytur == 3
+        || extra->iturb == 50 || extra->iturb == 60) {
 
-        if (extra->itytur == 2 || extra->iturb == 50) {
+      if (extra->itytur == 2 || extra->itytur == 4 || extra->iturb == 50) {
 
-          energ    = extra->cvar_k->val[cell_id];
-          dissip   = extra->cvar_ep->val[cell_id];
-
-        }
-        else if (extra->itytur == 3) {
-
-          if (extra->cvar_rij == NULL) {
-            energ    = 0.5 * (  extra->cvar_r11->val[cell_id]
-                              + extra->cvar_r22->val[cell_id]
-                              + extra->cvar_r33->val[cell_id]);
-          } else {
-            energ    = 0.5 * (  extra->cvar_rij->val[6*cell_id    ]
-                              + extra->cvar_rij->val[6*cell_id + 1]
-                              + extra->cvar_rij->val[6*cell_id + 2]);
-          }
-          dissip   = extra->cvar_ep->val[cell_id];
-
-        }
-        else if (extra->iturb == 60) {
-
-          energ    = extra->cvar_k->val[cell_id];
-          dissip   = extra->cmu * extra->cvar_k->val[cell_id]
-                                * extra->cvar_omg->val[cell_id];
-
-        }
-
-        auxl1[npt] = energ / (ct * dissip);
-        auxl1[npt] = CS_MAX(auxl1[npt], cs_math_epzero);
+        energ    = extra->cvar_k->val[cell_id];
+        dissip   = extra->cvar_ep->val[cell_id];
 
       }
-      else {
+      else if (extra->itytur == 3) {
 
-        auxl1[npt] = cs_math_epzero;
+        if (extra->cvar_rij == NULL) {
+          energ    = 0.5 * (  extra->cvar_r11->val[cell_id]
+                            + extra->cvar_r22->val[cell_id]
+                            + extra->cvar_r33->val[cell_id]);
+        } else {
+          energ    = 0.5 * (  extra->cvar_rij->val[6*cell_id    ]
+                            + extra->cvar_rij->val[6*cell_id + 1]
+                            + extra->cvar_rij->val[6*cell_id + 2]);
+        }
+        dissip   = extra->cvar_ep->val[cell_id];
 
       }
+      else if (extra->iturb == 60) {
+
+        energ    = extra->cvar_k->val[cell_id];
+        dissip   = extra->cmu * extra->cvar_k->val[cell_id]
+                              * extra->cvar_omg->val[cell_id];
+
+      }
+
+      auxl1[ip] = energ / (ct * dissip);
+      auxl1[ip] = CS_MAX(auxl1[ip], cs_math_epzero);
+
+    }
+    else {
+
+      auxl1[ip] = cs_math_epzero;
 
     }
 
@@ -997,32 +968,33 @@ _lagitf(cs_lagr_attribute_t  *iattr)
 
   if (nor == 1) {
 
-    for (cs_lnum_t npt = 0; npt < p_set->n_particles; npt++) {
+    for (cs_lnum_t ip = 0; ip < p_set->n_particles; ip++) {
 
-      unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
-      cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
+      /* FIXME check this: a fixed particle's temperature might still evolve */
+      if (cs_lagr_particles_get_flag(p_set, ip, CS_LAGR_PART_FIXED))
+        continue;
 
-      if (cell_id >= 0) {
+      unsigned char *particle = p_set->p_buffer + p_am->extents * ip;
+      cs_lnum_t cell_id = cs_lagr_particle_get_lnum(particle, p_am,
+                                                    CS_LAGR_CELL_ID);
 
-        cs_real_t aux1 = -cs_glob_lagr_time_step->dtp / auxl1[npt];
-        cs_real_t aux2 = exp(aux1);
+      cs_real_t aux1 = -cs_glob_lagr_time_step->dtp / auxl1[ip];
+      cs_real_t aux2 = exp(aux1);
 
-        cs_real_t ter1 = cs_lagr_particle_get_real_n(particle, p_am, 1,
-                                                     CS_LAGR_FLUID_TEMPERATURE) * aux2;
-        cs_real_t ter2 = tempf[cell_id] * (1.0 - aux2);
+      cs_real_t ter1 = cs_lagr_particle_get_real_n(particle, p_am, 1,
+                                                   CS_LAGR_FLUID_TEMPERATURE) * aux2;
+      cs_real_t ter2 = tempf[cell_id] * (1.0 - aux2);
 
-        cs_lagr_particle_set_real(particle, p_am, CS_LAGR_FLUID_TEMPERATURE, ter1 + ter2);
+      cs_lagr_particle_set_real(particle, p_am, CS_LAGR_FLUID_TEMPERATURE, ter1 + ter2);
 
-        /* Pour le cas NORDRE= 2, on calcule en plus TSVAR pour NOR= 2  */
-        if (ltsvar) {
+      /* Pour le cas NORDRE= 2, on calcule en plus TSVAR pour NOR= 2  */
+      if (ltsvar) {
 
-          cs_real_t *part_ts_fluid_t
-            = cs_lagr_particles_source_terms(p_set, npt,
-                                             CS_LAGR_FLUID_TEMPERATURE);
-          *part_ts_fluid_t = 0.5 * ter1 + tempf[cell_id]
-                                 * (-aux2 + (aux2 - 1.0) / aux1);
-
-        }
+        cs_real_t *part_ts_fluid_t
+          = cs_lagr_particles_source_terms(p_set, ip,
+                                           CS_LAGR_FLUID_TEMPERATURE);
+        *part_ts_fluid_t = 0.5 * ter1 + tempf[cell_id]
+                               * (-aux2 + (aux2 - 1.0) / aux1);
 
       }
 
@@ -1031,22 +1003,26 @@ _lagitf(cs_lagr_attribute_t  *iattr)
   }
   else if (nor == 2) {
 
-    for (cs_lnum_t npt = 0; p_set->n_particles; npt++) {
+    for (cs_lnum_t ip = 0; p_set->n_particles; ip++) {
 
-      unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
-      cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
+      /* FIXME check this: a fixed particle's temperature might still evolve */
+      if (cs_lagr_particles_get_flag(p_set, ip, CS_LAGR_PART_FIXED))
+        continue;
 
-      if (   cell_id  >= 0
-          && cs_lagr_particle_get_lnum(particle, p_am, CS_LAGR_REBOUND_ID) != 0 ) {
+      unsigned char *particle = p_set->p_buffer + p_am->extents * ip;
 
-        cs_real_t aux1   = -cs_glob_lagr_time_step->dtp / auxl1[npt];
+      if (cs_lagr_particle_get_lnum(particle, p_am, CS_LAGR_REBOUND_ID) != 0 ) {
+
+        cs_lnum_t cell_id = cs_lagr_particle_get_lnum(particle, p_am,
+                                                      CS_LAGR_CELL_ID);
+        cs_real_t aux1   = -cs_glob_lagr_time_step->dtp / auxl1[ip];
         cs_real_t aux2   = exp(aux1);
         cs_real_t ter1
           = 0.5 * cs_lagr_particle_get_real_n(particle, p_am, 1,
                                               CS_LAGR_FLUID_TEMPERATURE) * aux2;
         cs_real_t ter2   = tempf[cell_id] * (1.0 - (aux2 - 1.0) / aux1);
         cs_real_t *part_ts_fluid_t
-          = cs_lagr_particles_source_terms(p_set, npt, CS_LAGR_FLUID_TEMPERATURE);
+          = cs_lagr_particles_source_terms(p_set, ip, CS_LAGR_FLUID_TEMPERATURE);
 
         cs_lagr_particle_set_real(particle, p_am, CS_LAGR_FLUID_TEMPERATURE,
                                   *part_ts_fluid_t + ter1 + ter2);
@@ -1057,7 +1033,7 @@ _lagitf(cs_lagr_attribute_t  *iattr)
 
   }
 
-  /* Free memory     */
+  /* Free memory */
   BFT_FREE(auxl1);
   BFT_FREE(tempf);
 }
@@ -1123,10 +1099,10 @@ _lagich(const cs_real_t   tempct[],
     coef = 1.0 / cs_glob_lagr_time_scheme->t_order;
 
     if (nor == 1) {
-      for (cs_lnum_t npt = 0; npt < p_set->n_particles; npt++) {
-        cpgd1[npt] = 0.0;
-        cpgd2[npt] = 0.0;
-        cpght[npt] = 0.0;
+      for (cs_lnum_t ip = 0; ip < p_set->n_particles; ip++) {
+        cpgd1[ip] = 0.0;
+        cpgd2[ip] = 0.0;
+        cpght[ip] = 0.0;
       }
     }
 
@@ -1138,15 +1114,17 @@ _lagich(const cs_real_t   tempct[],
   cs_lnum_t nlayer = cs_glob_lagr_const_dim->nlayer;
   cs_real_t f_nlayer = nlayer;
 
-  for (cs_lnum_t npt = 0; npt < p_set->n_particles; npt++) {
+  for (cs_lnum_t ip = 0; ip < p_set->n_particles; ip++) {
 
-    unsigned char *particle = p_set->p_buffer + p_am->extents * npt;
-
-    cs_lnum_t cell_id = cs_lagr_particle_get_cell_id(particle, p_am);
-
-    if (cell_id < 0)
+    /* FIXME check this: a fixed particle's attributes might still evolve */
+    if (cs_lagr_particles_get_flag(p_set, ip, CS_LAGR_PART_FIXED))
       continue;
 
+    unsigned char *particle = p_set->p_buffer + p_am->extents * ip;
+
+    cs_lnum_t cell_id = cs_lagr_particle_get_lnum(particle, p_am,
+                                                  CS_LAGR_CELL_ID);
+
     /* local variables*/
     cs_real_t aux1, aux2, aux3, aux4, aux5;
 
@@ -1244,7 +1222,7 @@ _lagich(const cs_real_t   tempct[],
     /* Vapor flux for particle */
 
     cs_real_t fwat[nlayer];
-    _lagsec(npt, layer_vol, mwat_max, sherw, radius, tempct, mlayer, mwater, fwat);
+    _lagsec(ip, layer_vol, mwat_max, sherw, radius, tempct, mlayer, mwater, fwat);
 
     /* Compute velocity constants SPK1 of SPK2 of the mass transfer by
        devolatilization with the Arrhenius laws */
@@ -1264,8 +1242,8 @@ _lagich(const cs_real_t   tempct[],
       /* Thermal return coupling */
 
       if (cs_glob_lagr_source_terms->ltsthe == 1) {
-        cpgd1[npt] = cpgd1[npt] + coef * aux1;
-        cpgd2[npt] = cpgd2[npt] + coef * aux2;
+        cpgd1[ip] = cpgd1[ip] + coef * aux1;
+        cpgd2[ip] = cpgd2[ip] + coef * aux2;
       }
 
     }
@@ -1302,7 +1280,8 @@ _lagich(const cs_real_t   tempct[],
 
       /* Constant 2.53e-7 is explained in tome 5 of report on Code_Saturne
          specific physics (HI-81/04/003/A) equation 80 */
-      aux3 = sherw * 2.53e-07 * (pow (extra->t_gaz->val[cell_id], 0.75)) / shrink_diam;
+      aux3 = sherw * 2.53e-07 * (pow (extra->temperature->val[cell_id], 0.75))
+                              / shrink_diam;
       skglob = (aux2 * aux3) / (aux2 + aux3);
 
     }
@@ -1317,7 +1296,7 @@ _lagich(const cs_real_t   tempct[],
      *     PO2 = RHO1*cs_physical_constants_r*T*YO2/MO2
      *                                                      */
     aux1 =   extra->cromf->val[cell_id] * cs_physical_constants_r
-           * extra->t_gaz->val[cell_id]
+           * extra->temperature->val[cell_id]
            * extra->x_oxyd->val[cell_id] / lag_cc->wmole[lag_cc->io2] / lag_cc->prefth;
 
     /* Compute working surface: SE */
@@ -1325,7 +1304,7 @@ _lagich(const cs_real_t   tempct[],
 
     /* No heterogeneous combustion if Mch/Mp >= 1.e-3 */
     cs_real_t gamhet;
-    if (prev_part_coal_mass[0] >= (0.001 * mlayer[0]))
+    if (prev_part_coal_mass[0] <= (0.001 * mlayer[0]))
       gamhet = 0.0;
     else
       /* Compute GamHET */
@@ -1333,7 +1312,7 @@ _lagich(const cs_real_t   tempct[],
 
     /* Thermal return coupling */
     if (cs_glob_lagr_source_terms->ltsthe == 1)
-      cpght[npt] = cpght[npt] + coef * gamhet;
+      cpght[ip] = cpght[ip] + coef * gamhet;
 
     /* Compute 0.5(MO2/MC)*(HO2(Tp)-HO2(TF))
      * ------------------------------------- */
@@ -1359,7 +1338,7 @@ _lagich(const cs_real_t   tempct[],
         f2mc[iii] = 0.0;
     }
 
-    cs_lnum_t mode = -1;
+    int mode = -1;
     CS_PROCF(cpthp1, CPTHP1) (&mode, &aux2, coefe, f1mc, f2mc, &part_temp[l_id_het]);
 
     /* Compute MO2/MC/2. HO2(TF)    */
@@ -1468,7 +1447,7 @@ _lagich(const cs_real_t   tempct[],
         fcoke[l_id]  = 0.0;
 
       /* Loop on all cells which have reactive coke or coal */
-      for (cs_lnum_t l_id = l_id_het; l_id < 0; l_id--) {
+      for (cs_lnum_t l_id = 0; l_id < l_id_het; l_id++) {
 
         aux1 =  (skp1[l_id] *    (1.0 - lag_cc->y1ch[co_id]) + skp2[l_id]
                             * (1.0 - lag_cc->y2ch[co_id]))
@@ -1536,7 +1515,7 @@ _lagich(const cs_real_t   tempct[],
       phith[l_id] = (-fcoke[l_id] * deltah) - fwat[l_id] * lv;
 
     cs_real_t temp[nlayer];
-    _lagtmp(npt, layer_vol, tempct, radius, mlayer, phith, temp);
+    _lagtmp(ip, layer_vol, tempct, radius, mlayer, phith, temp);
 
     for (cs_lnum_t l_id = 0; l_id < nlayer; l_id++)
       part_temp[l_id]  = temp[l_id];
@@ -1567,14 +1546,14 @@ _lagich(const cs_real_t   tempct[],
 
     l_id_het = 0;
     for (cs_lnum_t l_id = 0; l_id < nlayer; l_id++) {
-      if (prev_part_coal_mass[l_id] > 0.0)
+      if (part_coal_mass[l_id] > 0.0)
         l_id_het = l_id;
     }
 
     /* Check for remaining coke on a more external layer */
 
     for (cs_lnum_t l_id = l_id_het; l_id < nlayer; l_id++) {
-      if (prev_part_coke_mass[l_id] > 0.0)
+      if (part_coke_mass[l_id] > 0.0)
         l_id_het = l_id;
     }
 
@@ -1605,10 +1584,13 @@ _lagich(const cs_real_t   tempct[],
       }
       else {
 
-        aux5 = pow(  pow(2.0 * radius[l_id_het - 1], 3)
+        cs_real_t f1 = part_coal_mass[l_id_het] / lag_cc->rho0ch[co_id];
+        cs_real_t f2 = 0;
+        if (part_coal_density[l_id_het] > 0.0)
+          f2 = part_coke_mass[l_id_het] / part_coal_density[l_id_het];
+        aux5 = pow(  cs_math_pow3(2.0 * radius[l_id_het - 1])
                    + (  d6spi / (1.0 - lag_cc->xashch[co_id])
-                      * (  part_coal_mass[l_id_het] / lag_cc->rho0ch[co_id]
-                         + part_coke_mass[l_id_het] / part_coal_density[l_id_het])), d1s3);
+                      * (f1 + f2)), d1s3);
 
         /* Clipping   */
         if (aux5 > 2.0 * radius[l_id_het])
@@ -1684,32 +1666,32 @@ cs_lagr_sde_model(const cs_real_t  tempct[],
 
   /* Integration of fluid temperature seen by particles */
 
-  if (   cs_glob_lagr_model->physical_model == 2
-      || (   cs_glob_lagr_model->physical_model == 1
+  if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL
+      || (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
           && cs_glob_lagr_specific_physics->itpvar == 1))
     _lagitf(&fluid_temp);
 
   /* Integration of particles temperature */
 
-  if (   cs_glob_lagr_model->physical_model == 1
+  if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
       && cs_glob_lagr_specific_physics->itpvar == 1)
     _lagitp(tempct);
 
   /* Integration of particles diameter */
 
-  if (   cs_glob_lagr_model->physical_model == 1
+  if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
       && cs_glob_lagr_specific_physics-> idpvar == 1)
     _lagidp();
 
   /* Integration of particles mass */
 
-  if (   cs_glob_lagr_model->physical_model == 1
+  if (   cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_HEAT
       && cs_glob_lagr_specific_physics->impvar == 1)
     _lagimp();
 
   /* Integration of coal equations: hp, mch, mck */
 
-  if (cs_glob_lagr_model->physical_model == 2)
+  if (cs_glob_lagr_model->physical_model == CS_LAGR_PHYS_COAL)
     _lagich(tempct, cpgd1, cpgd2, cpght);
 }
 
