@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -89,7 +89,7 @@ module lagran
 
   !> - 0: no head losses calculation for influence of the deposit on the flow
   !> - 1: head losses calculation for influence of the deposit on the flow
-  integer, save :: iflow
+  integer(c_int), pointer, save :: iflow
 
   !> - 0: no precipitation/dissolution model
   !> - 1: precipitation/dissolution model
@@ -110,14 +110,17 @@ module lagran
   integer(c_int), pointer, save ::  ltsdyn
 
   !> activation (=1) or not (=0) of the two-way coupling on the mass.
-  !> Useful if \ref iilagr = 2, \ref physical_model = 1 and \ref impvar = 1
+  !> Useful if \ref iilagr = 2, \ref cs_lagr_model_t::physical_model = 1
+  !>and \ref cs_lagr_specific_physics_t::impvar = 1
   integer(c_int), pointer, save ::  ltsmas
 
-  !> if \ref physical_model = 1 and \ref itpvar = 1, \ref ltsthe
-  !> activates (=1) or not (=0) the two-way coupling on temperature.
-  !> if \ref physical_model = 2, \ref ltsthe activates (=1) or not (=0) the
-  !> two-way coupling on the eulerian variables related to pulverised
-  !> coal combustion.
+  !> if \ref cs_lagr_model_t::physical_model = 1 and
+  !> \ref cs_lagr_specific_physics_t::itpvar = 1,
+  !> \ref ltsthe !> activates (=1) or not (=0) the two-way coupling on
+  !> temperature.
+  !> if \ref cs_lagr_model_t::physical_model = 2, \ref ltsthe
+  !> activates (=1) or not (=0) the two-way coupling on the eulerian
+  !> variables related to pulverized coal combustion.
   !> Useful if \ref iilagr = 2
   integer(c_int), pointer, save ::  ltsthe
 
@@ -168,11 +171,11 @@ module lagran
 
     ! Interface to C function returning particle attribute pointers
 
-    subroutine cs_f_lagr_params_pointers(p_iilagr, p_idepst, p_ipreci)         &
+    subroutine cs_f_lagr_params_pointers(p_iilagr, p_idepst, p_iflow, p_ipreci) &
       bind(C, name='cs_f_lagr_params_pointers')
       use, intrinsic :: iso_c_binding
       implicit none
-      type(c_ptr), intent(out) :: p_iilagr, p_idepst, p_ipreci
+      type(c_ptr), intent(out) :: p_iilagr, p_idepst, p_iflow, p_ipreci
     end subroutine cs_f_lagr_params_pointers
 
     subroutine cs_f_lagr_source_terms_pointers(p_ltsdyn, p_ltsmas,             &
@@ -226,6 +229,33 @@ module lagran
 
     !---------------------------------------------------------------------------
 
+    !> Execute one time step of the Lagrangian model.
+
+    subroutine cs_lagr_options_definition(isuite, have_thermal_model,  &
+                                          dtref, iccvfg)               &
+      bind(C, name='cs_lagr_options_definition')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      integer(c_int), value :: isuite, have_thermal_model
+      real(kind=c_double), value :: dtref
+      integer(c_int), intent(inout) :: iccvfg
+    end subroutine cs_lagr_options_definition
+
+    !---------------------------------------------------------------------------
+
+    !> Mass source term due to precipitation.
+
+    subroutine cs_lagr_precipitation_mass_st(dtref, crom, cvar_scal, crvexp)  &
+      bind(C, name='cs_lagr_precipitation_mass_st')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      real(kind=c_double), value :: dtref
+      real(kind=c_double), dimension(*), intent(in) :: crom, cvar_scal
+      real(kind=c_double), dimension(*), intent(out) :: crvexp
+    end subroutine cs_lagr_precipitation_mass_st
+
+    !---------------------------------------------------------------------------
+
     !> Prepare for execution of the Lagrangian model.
 
     subroutine cs_lagr_solve_initialize(dt)  &
@@ -299,7 +329,7 @@ contains
     type(c_ptr) :: p_iilagr
 
     ! lagr_params
-    type(c_ptr) :: p_idepst, p_ipreci
+    type(c_ptr) :: p_idepst, p_iflow, p_ipreci
 
     ! lagr_option_source_terms
     type(c_ptr) :: p_ltsdyn, p_ltsmas, p_ltsthe, p_itsli, p_itske,     &
@@ -307,10 +337,11 @@ contains
                    p_itsmas, p_itsmv1, p_itsmv2, p_itsco
     integer(c_int) :: dim_itsmv1, dim_itsmv2
 
-    call cs_f_lagr_params_pointers(p_iilagr, p_idepst, p_ipreci)
+    call cs_f_lagr_params_pointers(p_iilagr, p_idepst, p_iflow, p_ipreci)
     call c_f_pointer(p_iilagr, iilagr)
     call c_f_pointer(p_idepst, idepst)
-    call c_f_pointer(p_ipreci , ipreci)
+    call c_f_pointer(p_iflow , iflow)
+    call c_f_pointer(p_ipreci, ipreci)
 
     call cs_f_lagr_source_terms_pointers(p_ltsdyn, p_ltsmas,           &
                                          p_ltsthe, p_itsli,            &
