@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -211,9 +211,9 @@ cs_geom_closest_point(cs_lnum_t         n_points,
  * \param[in]      face_cog       coordinates of face center
  * \param[in]      sx0            segment start coordinates
  * \param[in]      sx1            segment end coordinates
- * \param[out]     n_crossings    number sub_face crossings
+ * \param[out]     n_inout        number sub_face crossings
  *                                 [0: in; 1: out]
- * \param[in, out] face_norm      local face unite noraml of the crossed sub
+ * \param[in, out] face_norm      local face unit normal of the crossed sub
  *                                 triangle (if entering with something
  *                                 different from NULL)
  *
@@ -225,19 +225,19 @@ cs_geom_closest_point(cs_lnum_t         n_points,
 /*----------------------------------------------------------------------------*/
 
 double
-cs_geom_segment_intersect_face(int              orient,
-                               cs_lnum_t        n_vertices,
-                               const cs_lnum_t  vertex_ids[],
-                               const cs_real_t  vtx_coord[][3],
-                               const cs_real_t  face_cog[3],
-                               const cs_real_t  sx0[3],
-                               const cs_real_t  sx1[3],
-                               int              n_crossings[2],
+cs_geom_segment_intersect_face(int               orient,
+                               cs_lnum_t         n_vertices,
+                               const cs_lnum_t   vertex_ids[],
+                               const cs_real_t   vtx_coord[][3],
+                               const cs_real_t   face_cog[3],
+                               const cs_real_t   sx0[3],
+                               const cs_real_t   sx1[3],
+                               int               n_inout[2],
                                cs_real_t        *face_norm)
 {
   const double epsilon = 1.e-15;
 
-  /* Initialization of retval to unity*/
+  /* Initialization of retval to two */
   double retval = 2.;
 
   assert(sizeof(cs_real_t) == 8);
@@ -252,6 +252,7 @@ cs_geom_segment_intersect_face(int              orient,
                             sx0[2] - face_cog[2]};
 
   int n_intersects = 0;
+  bool in_cog = true;
 
   /* Principle:
    *  - loop on sub-triangles of the face
@@ -286,13 +287,23 @@ cs_geom_segment_intersect_face(int              orient,
   cs_real_3_t e0, e1;
   int pip1, p0;
 
-  /* 1st vertex: vector e0, p0 = e0 ^ vgo  */
+  /* 1st vertex: test edge e0 with respect to (OD)
+   * in an orthogonal plane of (OD)
+   * vector e0 = [G, v0], p0 = e0 ^ OG
+   * return "p0 . OD > 0 ?" */
   cs_lnum_t vtx_id_0 = vertex_ids[0];
   const cs_real_t *vtx_0 = vtx_coord[vtx_id_0];
 
   p0 = _test_edge(sx0, sx1, face_cog, vtx_0);
   pip1 = p0;
 
+  /* Sign of "[OD].p" for the first sub-triangle.
+   * We need to store this to check if all e_out are around
+   * (OD), and for that the multiplier "[OD].p" must be the same
+   * Otherwise we can have troubles when [OD] is orthogonal to the face normal
+   * (it can switch sign from one triangle to an other...) */
+  int sign_od_p0 = 0;
+
   /* Loop on vertices of the face */
   for (cs_lnum_t i = 0; i < n_vertices; i++) {
 
@@ -306,7 +317,8 @@ cs_geom_segment_intersect_face(int              orient,
       e1[j] = vtx_1[j] - face_cog[j];
     }
 
-    /* P = e1^e0: same value for the two neighbooring cells */
+    /* P = e1^e0: same value for the two neighbouring cells
+     * NB: in the other direction to the face normal */
 
     const cs_real_3_t pvec = {e1[1]*e0[2] - e1[2]*e0[1],
                               e1[2]*e0[0] - e1[0]*e0[2],
@@ -314,12 +326,15 @@ cs_geom_segment_intersect_face(int              orient,
 
     double od_p = cs_math_3_dot_product(disp, pvec);
 
-    /* This sign is absolute (ie same result is obtained if the face is seen i
-     * from the other neighbooring cell).
+    /* This sign is absolute (ie same result is obtained if the face is seen
+     * from the other neighbouring cell).
      */
     int sign_od_p = (od_p > 0 ? 1 : -1);
 
-    /* 2nd edge: vector ei+1, pi+1 = ei+1 ^ vgo */
+    if (sign_od_p0 == 0)
+      sign_od_p0 = sign_od_p;
+
+    /* 2nd edge: vector ei+1, pi+1 = ei+1 ^ OG */
 
     int pi = pip1;
     if (i == n_vertices - 1)
@@ -327,10 +342,10 @@ cs_geom_segment_intersect_face(int              orient,
     else
       pip1 = _test_edge(sx0, sx1, face_cog, vtx_1);
 
-    const int u_sign = pip1 * sign_od_p;
+    int u_sign = pip1 * sign_od_p;
 
-    /* 1st edge: vector ei, pi = ei ^ vgo */
-    const int v_sign = - pi * sign_od_p;
+    /* 1st edge: vector ei, pi = ei ^ OG */
+    int v_sign = - pi * sign_od_p;
 
     /* 3rd edge: vector e_out */
 
@@ -351,6 +366,24 @@ cs_geom_segment_intersect_face(int              orient,
     int w_sign = _test_edge(sx0, sx1, vtx_0, vtx_1)
                  * reorient_edge * sign_od_p;
 
+    /* Special treatment if the intersection point is
+     * in G exactly (so w_sign is < 0 for all e_out)
+     * Not: all e_out should be talen with the same noram p.
+     * We take the first sign_od_p
+     */
+    if (w_sign * sign_od_p *sign_od_p0 > 0)
+      in_cog = false;
+
+    /* Last vertex for the face,
+     * in_cog is true for all e_out edges which means that
+     * w_sign <= 0 for all
+     * AND no triangle was selected (because the line is passing exactly
+     * in the cog G) */
+    if ((i == (n_vertices - 1)) && in_cog) {
+      u_sign = 1;
+      v_sign = 1;
+    }
+
     /* The projection of point O along displacement (OD) is outside of the
      * triangle then no intersection */
     if (w_sign > 0 || u_sign  < 0 || v_sign < 0)
@@ -359,20 +392,57 @@ cs_geom_segment_intersect_face(int              orient,
     /* Line (OD) intersects the triangle because
      * u_sign >= 0, v_sign >= 0 and w_sign <= 0
      */
+    /* No need to deal with the special case where the intersection is in G */
+    in_cog = false;
 
     double og_p = - cs_math_3_dot_product(vgo, pvec);
 
     /* This sign is absolute (ie same result is obtained if the face is seen from
-     * the other neighbooring cell.
+     * the other neighbouring cell.
      */
     int sign_og_p = (og_p > 0 ? 1 : -1);
 
     /* Same sign (meaning there is a possible intersection with t > 0). */
     if (sign_od_p == sign_og_p) {
-      /* The line (OD) enters (n_crossings[0]++)
-       * or leaves (n_crossings[1]++) the cell */
-      if (orient != sign_od_p) {
-        n_crossings[1]++;
+      /* The line (OD) enters (n_inout[0]++)
+       * (it means OD points toward cell i)
+       * or leaves (n_inout[1]++) the cell
+       * (it means OD points toward cell j)
+       * */
+
+      /* We don't care if it is entering or outgoing
+       * but we store this information correctly
+       * Convention
+       * t > 0: entering i
+       * t < 0: outgoing i */
+      if (orient == 0) {
+        /* entering cell i */
+        if (sign_od_p == 1)
+          n_inout[0]++;
+        /* going out cell i */
+        if (sign_od_p == -1)
+          n_inout[1]++;
+
+        if (fabs(og_p) < fabs(od_p)) {
+          /* There is a real intersection (inward or outward) with 0 <= t < 1 */
+          double t = 0.;
+          n_intersects++;
+
+          const double det = cs_math_3_norm(e0)*cs_math_3_norm(pvec);
+          if (fabs(od_p) > epsilon * fabs(det)) {
+            t = og_p / od_p;
+          }
+
+          if (t < retval) {
+            retval = t;
+            /* Store the normal if needed */
+            if (face_norm != NULL)
+              cs_math_3_normalise(pvec, face_norm);
+          }
+        }
+      }
+      else if (orient != sign_od_p) {
+        n_inout[1]++;
         if (fabs(og_p) < fabs(od_p)) {
           /* There is a real intersection (outward) with 0 <= t < 1 */
           double t = 0.;
@@ -391,8 +461,8 @@ cs_geom_segment_intersect_face(int              orient,
           }
         }
       } else {
-        n_crossings[0]++;
-        /* Incomming intersection on segment [OD] */
+        n_inout[0]++;
+        /* Incoming intersection on segment [OD] */
         if (fabs(og_p) < fabs(od_p))
           n_intersects--;
       }
@@ -400,10 +470,31 @@ cs_geom_segment_intersect_face(int              orient,
     } else {
       /* Opposite sign (meaning there is a possible intersection of the line
        * with t<0).  */
-      if (orient != sign_od_p)
-        n_crossings[1]++;
+
+      /* We don't care if it is entering or outgoing
+       * but we store this information correctly
+       * Convention
+       * t > 0: entering i
+       * t < 0: outgoing i */
+      if (orient == 0) {
+        cs_real_t t = -1;
+        const double det = cs_math_3_norm(e0)*cs_math_3_norm(pvec);
+        if (fabs(od_p) > epsilon * fabs(det))
+          t = og_p / od_p;
+
+        if (t < retval)
+          retval = t;
+
+        if (sign_od_p == -1)
+          n_inout[1]++;
+        else
+          n_inout[0]++;
+
+      }
+      else if (orient != sign_od_p)
+        n_inout[1]++;
       else
-        n_crossings[0]++;
+        n_inout[0]++;
     }
 
   }
@@ -411,7 +502,7 @@ cs_geom_segment_intersect_face(int              orient,
    *  (i.e.  n_intersects < 1 , but retval < 1),
    *  the retval value is forced to 2
    *  (no intersection since the particle entered and left from this face). */
-  if ((n_intersects < 1) && retval < 1.) {
+  if ((n_intersects < 1) && retval < 1. && retval >= 0) {
     retval = 2.;
   }
 
