@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -129,52 +129,6 @@ typedef struct {
  * Public function prototypes
  *===========================================================================*/
 
-#if defined(HAVE_MPI)
-
-/*----------------------------------------------------------------------------
- * Create a MPI_Datatype for the cs_join_vertex_t structure.
- *
- * returns:
- *   an MPI_Datatype associated to the cs_join_vertex_t structure.
- *---------------------------------------------------------------------------*/
-
-MPI_Datatype
-cs_join_mesh_create_vtx_datatype(void);
-
-/*----------------------------------------------------------------------------
- * Create a function to define an operator for MPI reduction operation
- *
- * parameters:
- *   in        <--  input vertices
- *   inout     <->  in/out vertices (vertex with the min. toelrance)
- *   len       <--  size of input array
- *   datatype  <--  MPI_datatype associated to cs_join_vertex_t
- *---------------------------------------------------------------------------*/
-
-void
-cs_join_mesh_mpi_vertex_min(cs_join_vertex_t   *in,
-                            cs_join_vertex_t   *inout,
-                            int                *len,
-                            MPI_Datatype       *datatype);
-
-/*----------------------------------------------------------------------------
- * Create a function to define an operator for MPI reduction operation
- *
- * parameters:
- *   in        <--  input vertices
- *   inout     <->  in/out vertices (vertex with the max. toelrance)
- *   len       <--  size of input array
- *   datatype  <--  MPI_datatype associated to cs_join_vertex_t
- *---------------------------------------------------------------------------*/
-
-void
-cs_join_mesh_mpi_vertex_max(cs_join_vertex_t   *in,
-                            cs_join_vertex_t   *inout,
-                            int                *len,
-                            MPI_Datatype       *datatype);
-
-#endif /* HAVE_MPI */
-
 /*----------------------------------------------------------------------------
  * Allocate and initialize a new cs_join_mesh_t structure.
  *
@@ -328,7 +282,7 @@ cs_join_mesh_minmax_tol(cs_join_param_t    param,
 
 void
 cs_join_mesh_exchange(cs_lnum_t              n_send,
-                      const cs_lnum_t        send_rank[],
+                      const int              send_rank[],
                       const cs_lnum_t        send_faces[],
                       const cs_join_mesh_t  *send_mesh,
                       cs_join_mesh_t        *recv_mesh,
@@ -420,7 +374,7 @@ cs_join_mesh_define_edges(const cs_join_mesh_t  *mesh);
  *   an edge number relative to the couple of vertices
  *---------------------------------------------------------------------------*/
 
-cs_int_t
+cs_lnum_t
 cs_join_mesh_get_edge(cs_lnum_t               v1_num,
                       cs_lnum_t               v2_num,
                       const cs_join_edges_t  *edges);
