@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -55,7 +55,6 @@
  *  Header for the current file
  *---------------------------------------------------------------------------*/
 
-#include "cs_prototypes.h"
 #include "cs_join_post.h"
 
 /*---------------------------------------------------------------------------*/
@@ -311,6 +310,7 @@ cs_join_post_mesh(const char            *mesh_name,
   /* Define fvm_nodal_t structure */
 
   fvm_nodal_from_desc_add_faces(post_mesh,
+                                -1,
                                 join_mesh->n_faces,
                                 NULL,
                                 1,
