@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -1541,372 +1541,13 @@ cs_join_gset_merge_elts(cs_join_gset_t  *set,
 
 #if defined(HAVE_MPI)
 
-/*----------------------------------------------------------------------------
- * Synchronize a cs_join_gset_t structure and distribute the resulting set
- * over the rank using a round-robin distribution. Elements in sync_set
- * are ordered and there is no redundancy but list may have redundancies.
- * Use cs_join_gset_clean() to remove redundancies in g_list.
- *
- * parameters:
- *   loc_set  <-> pointer to the local structure to work with
- *   comm     <-- mpi_comm on which synchro. and distribution take place
- *
- * returns:
- *   a synchronized and distributed cs_join_gset_t structure.
- *---------------------------------------------------------------------------*/
-
-cs_join_gset_t *
-cs_join_gset_robin_sync(cs_join_gset_t  *loc_set,
-                        MPI_Comm         comm)
-{
-  int local_rank, n_ranks;
-  cs_join_gset_t  *sync_set = NULL;
-
-  assert(loc_set != NULL);
-
-  MPI_Comm_rank(comm, &local_rank);
-  MPI_Comm_size(comm, &n_ranks);
-
-  /* Assign rank in a round-robin manner */
-
-  int *dest_rank;
-  BFT_MALLOC(dest_rank, loc_set->n_elts, int);
-
-  cs_gnum_t _n_ranks = n_ranks;
-  for (cs_lnum_t i = 0; i < loc_set->n_elts; i++)
-    dest_rank[i] = (loc_set->g_elts[i] - 1) % _n_ranks;
-
-  cs_all_to_all_t
-    *d = cs_all_to_all_create(loc_set->n_elts,
-                              0, /* flags */
-                              NULL, /* dest_id */
-                              dest_rank,
-                              comm);
-
-  cs_lnum_t n_recv_elts = cs_all_to_all_n_elts_dest(d);
-
-  /* Define sync_set: a distributed cs_join_gset_t structure which
-     synchronize data over the ranks */
-
-  sync_set = cs_join_gset_create(n_recv_elts);
-
-  /* Synchronize list definition for each global element */
-
-  cs_lnum_t *index;
-  BFT_MALLOC(index, loc_set->n_elts+1, cs_lnum_t);
-
-  index[0] = 0;
-  for (cs_lnum_t i = 0; i < loc_set->n_elts; i++) {
-    cs_lnum_t n_sub_elts = loc_set->index[i+1] - loc_set->index[i];
-    index[i+1] = index[i] + 1 + n_sub_elts;
-  }
-
-  cs_lnum_t *block_index = cs_all_to_all_copy_index(d,
-                                                    false, /* reverse */
-                                                    index,
-                                                    NULL);
-
-  /* Fill send_buffer: global number and number of elements in index */
-
-  cs_gnum_t *send_buffer;
-  BFT_MALLOC(send_buffer, index[loc_set->n_elts], cs_gnum_t);
-
-  for (cs_lnum_t i = 0, j = 0; i < loc_set->n_elts; i++) {
-
-    cs_gnum_t g_ent_num = loc_set->g_elts[i];
-    cs_lnum_t n_sub_elts = loc_set->index[i+1] - loc_set->index[i];
-
-    send_buffer[j++] = g_ent_num;
-
-    for (cs_lnum_t k = 0; k < n_sub_elts; k++)
-      send_buffer[j++] = loc_set->g_list[loc_set->index[i] + (cs_gnum_t)k];
-
-  }
-
-  cs_gnum_t *recv_buffer = cs_all_to_all_copy_indexed(d,
-                                                      CS_GNUM_TYPE,
-                                                      false, /* reverse */
-                                                      index,
-                                                      send_buffer,
-                                                      block_index,
-                                                      NULL);
-
-  /* Partial free memory */
-
-  BFT_FREE(index);
-  BFT_FREE(send_buffer);
-
-  /* Build index on elements of sync_set */
-
-  sync_set->index[0] = 0;
-  for (cs_lnum_t i = 0; i < n_recv_elts; i++) {
-    cs_lnum_t n_sub_elts = block_index[i+1] - block_index[i] - 1;
-    sync_set->index[i+1] = sync_set->index[i] + n_sub_elts;
-  }
-
-  BFT_FREE(block_index);
-
-  /* Fill g_list of sync_set */
-
-  BFT_MALLOC(sync_set->g_list,
-             sync_set->index[sync_set->n_elts],
-             cs_gnum_t);
-
-  for (cs_lnum_t i = 0, j = 0; i < n_recv_elts; i++) {
-
-    cs_lnum_t shift = sync_set->index[i];
-
-    sync_set->g_elts[i] = recv_buffer[j++];
-
-    cs_lnum_t n_sub_elts = sync_set->index[i+1] - sync_set->index[i];
-    for (cs_lnum_t k = 0; k < n_sub_elts; k++)
-      sync_set->g_list[k + shift] = recv_buffer[j++];
-
-  }
-
-  BFT_FREE(recv_buffer);
-
-  /* Return pointer */
-
-  cs_join_gset_merge_elts(sync_set, 0); /* sync_set elts are not ordered */
-
-  return  sync_set;
-}
-
-/*----------------------------------------------------------------------------
- * Update a local cs_join_gset_t structure from a distributed and
- * synchronized cs_join_gset_t structure. Round-robin distribution is used
- * to store synchronized elements.
- *
- * loc_set should not have redundant elements.
- *
- * parameters:
- *   sync_set <-- pointer to the structure which holds a synchronized block
- *   loc_set  <-> pointer to a local structure holding elements to update
- *   comm     <-- comm on which synchronization and distribution take place
- *---------------------------------------------------------------------------*/
-
-void
-cs_join_gset_robin_update(const cs_join_gset_t  *sync_set,
-                          cs_join_gset_t        *loc_set,
-                          MPI_Comm               comm)
-{
-  int  rank, local_rank, n_ranks, n_sub_elts, n_recv_elts, shift;
-  cs_lnum_t  i, j, k, elt_id, start, end;
-  cs_gnum_t  g_ent_num, _n_ranks;
-
-  cs_lnum_t  *send_count = NULL, *recv_count = NULL;
-  cs_lnum_t  *send_shift = NULL, *recv_shift = NULL, *wanted_rank_index = NULL;
-  cs_gnum_t  *send_buffer = NULL, *recv_buffer = NULL, *wanted_elts = NULL;
-
-  /* Sanity checks */
-
-  assert(sync_set != NULL);
-  assert(loc_set != NULL);
-
-  /* Build a cs_join_block_info_t structure */
-
-  MPI_Comm_rank(comm, &local_rank);
-  MPI_Comm_size(comm, &n_ranks);
-  _n_ranks = n_ranks;
-
-  /* Allocate parameters for MPI functions */
-
-  BFT_MALLOC(send_count, n_ranks, cs_lnum_t);
-  BFT_MALLOC(recv_count, n_ranks, cs_lnum_t);
-  BFT_MALLOC(send_shift, n_ranks + 1, cs_lnum_t);
-  BFT_MALLOC(recv_shift, n_ranks + 1, cs_lnum_t);
-  BFT_MALLOC(wanted_rank_index, n_ranks + 1, cs_lnum_t);
-
-  /* Initialization */
-
-  for (i = 0; i < n_ranks; i++)
-    send_count[i] = 0;
-
-  /* Get a synchronized list definition for each global element */
-
-  for (i = 0; i < loc_set->n_elts; i++) {
-    rank = (loc_set->g_elts[i] - 1) % _n_ranks;
-    send_count[rank] += 1;
-  }
-
-  MPI_Alltoall(send_count, 1, MPI_INT, recv_count, 1, MPI_INT, comm);
-
-  send_shift[0] = 0;
-  wanted_rank_index[0] = 0;
-
-  for (rank = 0; rank < n_ranks; rank++) {
-    send_shift[rank + 1] = send_shift[rank] + send_count[rank];
-    wanted_rank_index[rank + 1] = wanted_rank_index[rank] + recv_count[rank];
-  }
-
-  /* Fill send_buffer: global number */
-
-  BFT_MALLOC(send_buffer, send_shift[n_ranks], cs_gnum_t);
-  BFT_MALLOC(wanted_elts, wanted_rank_index[n_ranks], cs_gnum_t);
-
-  for (i = 0; i < n_ranks; i++)
-    send_count[i] = 0;
-
-  for (i = 0; i < loc_set->n_elts; i++) {
-
-    g_ent_num = loc_set->g_elts[i];
-    rank = (g_ent_num - 1) % _n_ranks;
-    shift = send_shift[rank] + send_count[rank];
-
-    send_buffer[shift] = g_ent_num;
-    send_count[rank] += 1;
-
-  }
-
-  MPI_Alltoallv(send_buffer, send_count, send_shift, CS_MPI_GNUM,
-                wanted_elts, recv_count, wanted_rank_index, CS_MPI_GNUM,
-                comm);
-
-  /* Send new list definition holding by sync_set to ranks which have
-     request it */
-
-  for (i = 0; i < n_ranks; i++)
-    send_count[i] = 0;
-
-  for (rank = 0; rank < n_ranks; rank++) {
-
-    for (i = wanted_rank_index[rank]; i < wanted_rank_index[rank+1]; i++) {
-
-      elt_id = cs_search_g_binary(sync_set->n_elts,
-                                  wanted_elts[i],
-                                  sync_set->g_elts);
-
-      assert(elt_id != -1);
-
-      wanted_elts[i] = elt_id;
-      n_sub_elts = sync_set->index[elt_id+1] - sync_set->index[elt_id];
-      send_count[rank] +=  2 + n_sub_elts; /* glob. num,
-                                              n_sub_elts,
-                                              g_list */
-
-    }
-
-  } /* End of loop on ranks */
-
-  MPI_Alltoall(send_count, 1, MPI_INT, recv_count, 1, MPI_INT, comm);
-
-  send_shift[0] = 0;
-  recv_shift[0] = 0;
-
-  for (rank = 0; rank < n_ranks; rank++) {
-    send_shift[rank + 1] = send_shift[rank] + send_count[rank];
-    recv_shift[rank + 1] = recv_shift[rank] + recv_count[rank];
-  }
-
-  BFT_REALLOC(send_buffer, send_shift[n_ranks], cs_gnum_t);
-  BFT_MALLOC(recv_buffer, recv_shift[n_ranks], cs_gnum_t);
-
-  for (i = 0; i < n_ranks; i++)
-    send_count[i] = 0;
-
-  for (rank = 0; rank < n_ranks; rank++) {
-
-    for (i = wanted_rank_index[rank]; i < wanted_rank_index[rank+1]; i++) {
-
-      shift = send_shift[rank] + send_count[rank];
-      elt_id = wanted_elts[i];
-
-      start = sync_set->index[elt_id];
-      end = sync_set->index[elt_id+1];
-      n_sub_elts = end - start;
-
-      send_buffer[shift++] = sync_set->g_elts[elt_id];
-      send_buffer[shift++] = n_sub_elts;
-
-      for (j = start, k = 0; j < end; j++, k++)
-        send_buffer[shift+k] = sync_set->g_list[j];
-
-      send_count[rank] +=  2 + n_sub_elts; /* glob. num, n_sub_elts, g_list */
-
-    }
-
-  } /* End of loop on ranks */
-
-  MPI_Alltoallv(send_buffer, send_count, send_shift, CS_MPI_GNUM,
-                recv_buffer, recv_count, recv_shift, CS_MPI_GNUM,
-                comm);
-
-  n_recv_elts = recv_shift[n_ranks];
-
-  /* Partial memory free */
-
-  BFT_FREE(send_buffer);
-  BFT_FREE(send_count);
-  BFT_FREE(send_shift);
-  BFT_FREE(recv_count);
-  BFT_FREE(recv_shift);
-
-  /* Re-initialize loc_set
-     As loc_set->g_elts and sync_set->g_elts are ordered, it's easier.
-     We can take values as they come */
-
-  /* First redefine index */
-
-  assert(loc_set->index[0] == 0);
-
-  for (i = 0; i < loc_set->n_elts; i++)
-    loc_set->index[i+1] = 0;
-
-  i = 0; /* id in recv_buffer */
-  j = 0; /* id in g_elts */
-
-  while (i < n_recv_elts) {
-
-    assert(loc_set->g_elts[j] = recv_buffer[i]);
-
-    i++;
-    n_sub_elts = recv_buffer[i++];
-    loc_set->index[j+1] = n_sub_elts;
-
-    i += n_sub_elts;
-    j++; /* go to the next elements */
-
-  } /* End of loop on elements of recv_buffer */
-
-  /* Define the new index */
-
-  for (i = 0; i < loc_set->n_elts; i++)
-    loc_set->index[i+1] += loc_set->index[i];
-
-  BFT_REALLOC(loc_set->g_list, loc_set->index[loc_set->n_elts], cs_gnum_t);
-
-  i = 0; /* id in recv_buffer */
-  j = 0; /* id in g_elts */
-
-  while (i < n_recv_elts) {
-
-    assert(loc_set->g_elts[j] = recv_buffer[i]);
-
-    i++;
-    n_sub_elts = recv_buffer[i++];
-
-    for (k = 0; k < n_sub_elts; k++)
-      loc_set->g_list[loc_set->index[j] + k] = recv_buffer[i + k];
-
-    i += n_sub_elts;
-    j++; /* go to the next elements */
-
-  } /* End of loop on elements of recv_buffer */
-
-  BFT_FREE(recv_buffer);
-  BFT_FREE(wanted_elts);
-  BFT_FREE(wanted_rank_index);
-
-}
-
 /*----------------------------------------------------------------------------
  * Synchronize a cs_join_gset_t structure and distribute the resulting set
  * over the rank by block
  *
  * parameters:
  *   max_gnum <-- max global number in global element numbering
- *   loc_set  <-> pointer to the local structure to work with
+ *   loc_set  <-- pointer to the local structure to work with
  *   comm     <-- mpi_comm on which synchro. and distribution take place
  *
  * returns:
@@ -1918,19 +1559,10 @@ cs_join_gset_block_sync(cs_gnum_t        max_gnum,
                         cs_join_gset_t  *loc_set,
                         MPI_Comm         comm)
 {
-  int  block_id;
-  int  rank, local_rank, n_ranks, n_recv_elts, shift;
-  cs_lnum_t i, j, n_sub_elts;
-  cs_gnum_t g_ent_num;
-  cs_block_dist_info_t  bi;
-
-  cs_lnum_t block_size = 0;
-
-  cs_lnum_t  *send_count = NULL, *recv_count = NULL, *counter = NULL;
-  cs_lnum_t  *send_shift = NULL, *recv_shift = NULL;
-  cs_gnum_t  *send_buffer = NULL, *recv_buffer = NULL;
   cs_join_gset_t  *sync_set = NULL;
 
+  int  local_rank, n_ranks;
+
   assert(loc_set != NULL);
 
   if (max_gnum == 0)
@@ -1939,146 +1571,113 @@ cs_join_gset_block_sync(cs_gnum_t        max_gnum,
   MPI_Comm_rank(comm, &local_rank);
   MPI_Comm_size(comm, &n_ranks);
 
-  bi = cs_block_dist_compute_sizes(local_rank,
-                                   n_ranks,
-                                   1,
-                                   0,
-                                   max_gnum);
+  cs_block_dist_info_t  bi
+    = cs_block_dist_compute_sizes(local_rank,
+                                  n_ranks,
+                                  1,
+                                  0,
+                                  max_gnum);
 
+  cs_lnum_t block_size = 0;
   if (bi.gnum_range[1] > bi.gnum_range[0])
     block_size = bi.gnum_range[1] - bi.gnum_range[0];
 
-  /* Allocate parameters for MPI functions */
-
-  BFT_MALLOC(send_count, n_ranks, cs_lnum_t);
-  BFT_MALLOC(recv_count, n_ranks, cs_lnum_t);
-  BFT_MALLOC(send_shift, n_ranks + 1, cs_lnum_t);
-  BFT_MALLOC(recv_shift, n_ranks + 1, cs_lnum_t);
-
-  /* Initialization */
-
-  for (i = 0; i < n_ranks; i++)
-    send_count[i] = 0;
+  cs_all_to_all_t *
+    d = cs_all_to_all_create_from_block(loc_set->n_elts,
+                                        0,  /* flags */
+                                        loc_set->g_elts,
+                                        bi,
+                                        comm);
 
   /* Synchronize list definition for each global element */
 
-  for (i = 0; i < loc_set->n_elts; i++) {
+  cs_lnum_t *p_index;
+  BFT_MALLOC(p_index, loc_set->n_elts+1, cs_lnum_t);
+  cs_gnum_t *p_buffer;
+  BFT_MALLOC(p_buffer,
+             loc_set->index[loc_set->n_elts] + loc_set->n_elts,
+             cs_gnum_t);
 
-    rank = (loc_set->g_elts[i] - 1)/bi.block_size * bi.rank_step;
-    n_sub_elts = loc_set->index[i+1] - loc_set->index[i];
-    send_count[rank] += 2 + n_sub_elts;
+  p_index[0] = 0;
+  for (cs_lnum_t i = 0; i < loc_set->n_elts; i++) {
+
+    cs_lnum_t shift = p_index[i];
+    p_buffer[shift++] = loc_set->g_elts[i];
+    cs_lnum_t s_id = loc_set->index[i], e_id = loc_set->index[i+1];
+    for (cs_lnum_t j = s_id; j < e_id; j++)
+      p_buffer[shift++] = loc_set->g_list[j];
+
+    p_index[i+1] = shift;
 
   }
 
-  MPI_Alltoall(send_count, 1, MPI_INT, recv_count, 1, MPI_INT, comm);
+  cs_lnum_t *r_index
+    = cs_all_to_all_copy_index(d,
+                               false,  /* reverse */
+                               p_index,
+                               NULL);
 
-  send_shift[0] = 0;
-  recv_shift[0] = 0;
+  cs_gnum_t *r_buffer
+    = cs_all_to_all_copy_indexed(d,
+                                 CS_GNUM_TYPE,
+                                 false,  /* reverse */
+                                 p_index,
+                                 p_buffer,
+                                 r_index,
+                                 NULL);
 
-  for (rank = 0; rank < n_ranks; rank++) {
-    send_shift[rank + 1] = send_shift[rank] + send_count[rank];
-    recv_shift[rank + 1] = recv_shift[rank] + recv_count[rank];
-  }
+  BFT_FREE(p_index);
+  BFT_FREE(p_buffer);
 
-  /* Fill send_buffer: global number and number of elements in index */
+  cs_lnum_t n_r_elts = cs_all_to_all_n_elts_dest(d);
 
-  BFT_MALLOC(send_buffer, send_shift[n_ranks], cs_gnum_t);
-  BFT_MALLOC(recv_buffer, recv_shift[n_ranks], cs_gnum_t);
-
-  for (i = 0; i < n_ranks; i++)
-    send_count[i] = 0;
-
-  for (i = 0; i < loc_set->n_elts; i++) {
-
-    g_ent_num = loc_set->g_elts[i];
-    rank = (g_ent_num - 1)/(cs_gnum_t)(bi.block_size) * bi.rank_step;
-    shift = send_shift[rank] + send_count[rank];
-    n_sub_elts = loc_set->index[i+1] - loc_set->index[i];
-
-    send_buffer[shift++] = g_ent_num;
-    send_buffer[shift++] = n_sub_elts;
-
-    for (j = 0; j < n_sub_elts; j++)
-      send_buffer[shift + j] = loc_set->g_list[loc_set->index[i] + j];
-
-    send_count[rank] += 2 + n_sub_elts;
-
-  }
-
-  MPI_Alltoallv(send_buffer, send_count, send_shift, CS_MPI_GNUM,
-                recv_buffer, recv_count, recv_shift, CS_MPI_GNUM,
-                comm);
-
-  n_recv_elts = recv_shift[n_ranks];
-
-  /* Partial free memory */
-
-  BFT_FREE(send_buffer);
-  BFT_FREE(send_count);
-  BFT_FREE(send_shift);
-  BFT_FREE(recv_count);
-  BFT_FREE(recv_shift);
+  cs_all_to_all_destroy(&d);
 
   /* Define sync_set: a distributed cs_join_gset_t structure which
-     synchronize data over the ranks */
+     synchronizes data over the ranks */
 
   sync_set = cs_join_gset_create(block_size);
 
-  for (i = 0; i < sync_set->n_elts; i++) {
-    cs_gnum_t g_id = i;
-    sync_set->g_elts[i] = bi.gnum_range[0] + g_id;
+  for (cs_lnum_t i = 0; i < sync_set->n_elts; i++) {
+    sync_set->g_elts[i] = bi.gnum_range[0] + (cs_gnum_t)i;
   }
 
-  /* Fill sync_set->index and define a new recv_count for the next comm. */
+  /* Build index */
 
-  i = 0; /* position in recv_buffer */
+  for (cs_lnum_t i = 0; i < n_r_elts; i++) {
+    cs_lnum_t j = r_buffer[r_index[i]] - bi.gnum_range[0];
+    sync_set->index[j+1] += r_index[i+1] - r_index[i] - 1;
+  }
 
-  while (i < n_recv_elts) {
-
-    block_id = recv_buffer[i++] - bi.gnum_range[0];
-
-    assert(sync_set->g_elts[block_id] == recv_buffer[i-1]);
-
-    n_sub_elts = recv_buffer[i++];
-    sync_set->index[block_id+1] += n_sub_elts;
-    i += n_sub_elts;
-
-  } /* End of loop on ranks */
-
-  /* Build index on elements of sync_set */
-
-  for (i = 0; i < sync_set->n_elts; i++)
+  for (cs_lnum_t i = 0; i < sync_set->n_elts; i++) {
     sync_set->index[i+1] += sync_set->index[i];
+  }
 
   BFT_MALLOC(sync_set->g_list,
              sync_set->index[sync_set->n_elts],
              cs_gnum_t);
 
-  /* Fill g_list of sync_set */
+  /* Now build set */
 
-  BFT_MALLOC(counter, sync_set->n_elts, cs_lnum_t);
+  cs_lnum_t *count;
+  BFT_MALLOC(count, sync_set->n_elts, cs_lnum_t);
+  for (cs_lnum_t i = 0; i < sync_set->n_elts; i++)
+    count[i] = 0;
 
-  for (i = 0; i < sync_set->n_elts; i++)
-    counter[i] = 0;
+  for (cs_lnum_t i = 0; i < n_r_elts; i++) {
+    cs_lnum_t r_shift = r_index[i];
+    cs_lnum_t j = r_buffer[r_shift] - bi.gnum_range[0];
+    cs_lnum_t w_shift = sync_set->index[j] + count[j];
 
-  i = 0; /* position in recv_buffer */
+    cs_lnum_t n_sub_elts = r_index[i+1] - r_index[i] - 1;
+    for (cs_lnum_t k = 0; k < n_sub_elts; k++)
+      sync_set->g_list[w_shift + k] = r_buffer[r_shift + 1 + k];
+    count[j] += n_sub_elts;
+  }
 
-  while (i < n_recv_elts) {
-
-    block_id = recv_buffer[i++] - bi.gnum_range[0];
-    shift = sync_set->index[block_id] + counter[block_id];
-
-    n_sub_elts = recv_buffer[i++];
-
-    for (j = 0; j < n_sub_elts; j++)
-      sync_set->g_list[j + shift] = recv_buffer[i++];
-
-    counter[block_id] += n_sub_elts;
-
-  } /* End of loop on ranks */
-
-  BFT_FREE(recv_buffer);
-  BFT_FREE(counter);
+  BFT_FREE(count);
+  BFT_FREE(r_buffer);
+  BFT_FREE(r_index);
 
   /* Return pointer */
 
@@ -2295,8 +1894,8 @@ cs_join_gset_dump(FILE                  *f,
     f = stdout;
 
   fprintf(f, "\nDump cs_join_gset_t structure: %p\n", (const void *)set);
-  fprintf(f, "number of elements: %10d\n", set->n_elts);
-  fprintf(f, "size of the list  : %10d\n\n", set->index[set->n_elts]);
+  fprintf(f, "number of elements: %10ld\n", (long)set->n_elts);
+  fprintf(f, "size of the list  : %10ld\n\n", (long)set->index[set->n_elts]);
 
   for (i = 0; i < set->n_elts; i++) {
 
