@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -306,20 +306,24 @@ _renumber_local_ordered_i(cs_lnum_t         n_elts,
  * parameters:
  *   o2n_hist        <-- old -> new global face numbering
  *   gnum_rank_index <-- index on ranks for the old global face numbering
- *   send_rank_index --> index on ranks for sending face
+ *   n_send          --> number of faces/rank couples to send
+ *   send_rank       --> ids of ranks for sending faces
  *   send_faces      --> list of face ids to send
  *---------------------------------------------------------------------------*/
 
 static void
 _get_faces_to_send(const cs_join_gset_t  *o2n_hist,
                    const cs_gnum_t        gnum_rank_index[],
-                   cs_lnum_t             *send_rank_index[],
+                   cs_lnum_t             *n_send,
+                   int                   *send_rank[],
                    cs_lnum_t             *send_faces[])
 {
   cs_lnum_t  i, j, rank, start, end;
 
+  cs_lnum_t _n_send = 0;
   cs_lnum_t  reduce_size = 0;
-  cs_lnum_t  *_send_rank_index = NULL, *_send_faces = NULL, *reduce_ids = NULL;
+  int  *_send_rank = NULL;
+  cs_lnum_t  *_send_faces = NULL, *reduce_ids = NULL;
   cs_gnum_t  *reduce_index = NULL;
   cs_join_gset_t  *new_face_rank = NULL;
 
@@ -418,14 +422,18 @@ _get_faces_to_send(const cs_join_gset_t  *o2n_hist,
 
   /* Define arrays to return */
 
-  BFT_MALLOC(_send_rank_index, n_ranks + 1, cs_lnum_t);
+  _n_send = new_face_rank->index[n_ranks];
 
-  for (i = 0; i < n_ranks + 1; i++)
-    _send_rank_index[i] = new_face_rank->index[i];
+  BFT_MALLOC(_send_rank, _n_send, int);
 
-  BFT_MALLOC(_send_faces, _send_rank_index[n_ranks], cs_lnum_t);
+  for (i = 0; i < n_ranks; i++) {
+    for (j = new_face_rank->index[i]; j < new_face_rank->index[i+1]; j++)
+      _send_rank[j] = i;
+  }
 
-  for (i = 0; i < _send_rank_index[n_ranks]; i++)
+  BFT_MALLOC(_send_faces, _n_send, cs_lnum_t);
+
+  for (i = 0; i < _n_send; i++)
     _send_faces[i] = new_face_rank->g_list[i];
 
   cs_join_gset_destroy(&new_face_rank);
@@ -434,21 +442,16 @@ _get_faces_to_send(const cs_join_gset_t  *o2n_hist,
   if (cs_glob_join_log != NULL) {
     fprintf(cs_glob_join_log,
             "\n Exchange to do after the splitting operation:\n");
-    for (i = 0; i < n_ranks; i++) {
-      start = _send_rank_index[i];
-      end = _send_rank_index[i+1];
+    for (i = 0; i < _n_send; i++)
       fprintf(cs_glob_join_log,
-              " Send to rank %5d (n = %10d):", i, end - start);
-      for (j = start; j < end; j++)
-        fprintf(cs_glob_join_log, " %d ", _send_faces[j]);
-      fprintf(cs_glob_join_log, "\n");
-    }
+              " send %10d to rank %5d\n", _send_faces[i], _send_rank[i]);
   }
 #endif
 
   /* Return pointers */
 
-  *send_rank_index = _send_rank_index;
+  *n_send = _n_send;
+  *send_rank = _send_rank;
   *send_faces = _send_faces;
 }
 
@@ -618,7 +621,8 @@ _dump_face_builder(cs_lnum_t              face_id,
   cs_lnum_t  face_e = builder->face_index[face_id+1];
   cs_lnum_t  n_subfaces = face_e - face_s;
 
-  fprintf(logfile, " Face %9d (n_subfaces: %d):\n", face_id+1, n_subfaces);
+  fprintf(logfile, " Face %9ld (n_subfaces: %ld):\n",
+          (long)face_id+1, (long)n_subfaces);
 
   for (i = face_s; i <face_e; i++, subface_id++) {
 
@@ -626,17 +630,17 @@ _dump_face_builder(cs_lnum_t              face_id,
     cs_lnum_t  subface_e = builder->subface_index->array[i+1];
 
     if (builder->subface_gnum == NULL)
-      fprintf(logfile, "   subface %4d: (%d, %d) -",
-              subface_id, subface_s, subface_e);
+      fprintf(logfile, "   subface %4ld: (%ld, %ld) -",
+              (long)subface_id, (long)subface_s, (long)subface_e);
     else
-      fprintf(logfile, "   subface %4d (%10llu) - (%d, %d) -",
-              subface_id,
+      fprintf(logfile, "   subface %4ld (%10llu) - (%ld, %ld) -",
+              (long)subface_id,
               (unsigned long long)builder->subface_gnum[face_s + subface_id],
-              subface_s, subface_e);
+              (long)subface_s, (long)subface_e);
 
     if (builder->subface_gconnect == NULL) {
       for (j = subface_s; j < subface_e; j++)
-        fprintf(logfile, " %d ", builder->subface_connect->array[j]);
+        fprintf(logfile, " %ld ", (long)builder->subface_connect->array[j]);
     }
     else {
       for (j = subface_s; j < subface_e; j++)
@@ -1036,9 +1040,10 @@ _find_next(cs_join_param_t         param,
     assert(n_connect_vertices < 2);
 
     bft_error(__FILE__, __LINE__, 0,
-              _(" Joining operation : split face %d\n"
+              _(" Joining operation : split face %ld\n"
                 " Problem in the connectivity. Could not find a "
-                "connection with the vertex %d\n"), fid, vid1+1);
+                "connection with the vertex %ld\n"),
+              (long)fid, (long)vid1+1);
 
 
   } /* End of test on the number of vertices connected to vid2 */
@@ -1352,7 +1357,7 @@ _split_face(cs_lnum_t               fid,
 
   builder->face_index[block_id+1] = builder->face_index[block_id] + n_subfaces;
 
-  assert(builder->face_index[block_id+1] = subface_shift);
+  assert(builder->face_index[block_id+1] == subface_shift);
 
   /* Returns pointers */
 
@@ -1905,6 +1910,7 @@ cs_join_split_faces(cs_join_param_t          param,
   for (fid = 0, block_id = 0; fid < n_init_faces; fid++) {
 
     int  block_rank = (w->face_gnum[fid] - 1)/(cs_gnum_t)(bi.block_size);
+    block_rank *= bi.rank_step;
 
     if (block_rank == local_rank) { /* This face is a "main" face for the
                                        local rank */
@@ -2051,8 +2057,8 @@ cs_join_split_faces(cs_join_param_t          param,
 
           if (param.verbosity > 2) {
             fprintf(cs_glob_join_log,
-                    "\n Keep initial connectivity for face %d [%llu]:\n",
-                    fid+1, (unsigned long long)w->face_gnum[fid]);
+                    "\n Keep initial connectivity for face %ld [%llu]:\n",
+                    (long)fid+1, (unsigned long long)w->face_gnum[fid]);
             _dump_face_builder(block_id, builder, cs_glob_join_log);
           }
 
@@ -2276,7 +2282,9 @@ cs_join_split_update_struct(const cs_join_param_t   param,
   if (n_ranks > 1) { /* Parallel mode */
 
     cs_lnum_t  j, subface_id;
-    cs_lnum_t  *send_rank_index = NULL, *send_faces = NULL;
+    cs_lnum_t   n_send = 0;
+    int        *send_rank = NULL;
+    cs_lnum_t  *send_faces = NULL;
     cs_gnum_t  *init_face_gnum = NULL;
     cs_join_gset_t  *distrib_sync_hist = NULL;
     cs_lnum_t  n_init_faces = _local_mesh->n_faces;
@@ -2317,20 +2325,21 @@ cs_join_split_update_struct(const cs_join_param_t   param,
 
     _get_faces_to_send(_o2n_hist, /* Local subface num is stored up to now */
                        gnum_rank_index,
-                       &send_rank_index,
+                       &n_send,
+                       &send_rank,
                        &send_faces);
 
     /* Get the new face connectivity from the distributed work_mesh */
 
-    cs_join_mesh_exchange(n_ranks,
-                          send_rank_index,
+    cs_join_mesh_exchange(n_send,
+                          send_rank,
                           send_faces,
                           work_mesh,
                           _local_mesh,
                           mpi_comm);
 
     BFT_FREE(send_faces);
-    BFT_FREE(send_rank_index);
+    BFT_FREE(send_rank);
 
     /* Order face by increasing global number */
 
