@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -528,7 +528,7 @@ _bad_cells_post(const cs_mesh_t             *mesh,
 {
   int i;
 
-  cs_lnum_t  *bad_cells_v = NULL;
+  int  *bad_cells_v = NULL;
 
   const cs_lnum_t  n_cells       = mesh->n_cells;
   const unsigned  *bad_cell_flag = mesh_quantities->bad_cell_flag;
@@ -643,7 +643,7 @@ _to_regularize(const cs_mesh_t             *mesh,
   cs_lnum_t n_i_faces = mesh->n_i_faces;
   cs_lnum_t n_b_faces = mesh->n_b_faces;
   const cs_lnum_2_t *i_face_cells = (const cs_lnum_2_t *)mesh->i_face_cells;
-  const int *b_face_cells = mesh->b_face_cells;
+  const cs_lnum_t *b_face_cells = mesh->b_face_cells;
 
   const cs_real_3_t *cdgfac = (const cs_real_3_t *) mq->i_face_cog;
   const cs_real_3_t *cdgfbo = (const cs_real_3_t *) mq->b_face_cog;
@@ -880,7 +880,7 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
   for (i = 0; i < n_cells_wghosts; i++)
     bad_cell_flag[i] = 0;
 
-  /* Possible warning printed in the listing --> flag initialization */
+  /* Possible warning printed in the log --> flag initialization */
 
   iwarning = 0;
   n_cells_tot = mesh->n_g_cells;
@@ -945,7 +945,7 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
       cs_parall_counter(&iwarning, 1);
     }
 
-    /* Display listing output */
+    /* Display log output */
     bft_printf(_("\n  Criterion 2: Offset:\n"));
     bft_printf(_("    Number of bad cells detected: %llu --> %3.0f %%\n"),
                (unsigned long long)ibad,
@@ -1007,7 +1007,7 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
       cs_parall_counter(&iwarning, 1);
     }
 
-    /* Display listing output */
+    /* Display log output */
     bft_printf(_("\n  Criterion 4: Cells Volume Ratio:\n"));
     bft_printf(_("    Number of bad cells detected: %llu --> %3.0f %%\n"),
                (unsigned long long)ibad,
@@ -1057,7 +1057,7 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
         cs_parall_counter(&iwarning, 1);
       }
 
-      /* Display listing output */
+      /* Display log output */
       bft_printf(_("\n  Criterion 5: Guilt by Association:\n"));
       bft_printf(_("    Number of bad cells detected: %llu --> %3.0f %%\n"),
                  (unsigned long long)ibad,
@@ -1091,7 +1091,7 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
         cs_parall_counter(&iwarning, 1);
       }
 
-      /* Display listing output */
+      /* Display log output */
       bft_printf(_("\n  Regularisation:\n"));
       bft_printf(_("    Number of bad cells detected: %llu --> %3.0f %%\n"),
                  (unsigned long long)ibad,
