@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,6 +54,7 @@
 
 #include "cs_math.h"
 #include "cs_mesh.h"
+#include "cs_mesh_boundary.h"
 #include "cs_mesh_quantities.h"
 #include "cs_order.h"
 #include "cs_parall.h"
@@ -120,12 +121,18 @@ _angle(const cs_lnum_t  v_ids[3],
   const cs_real_t lsq0 = cs_math_3_square_norm(u);
   const cs_real_t lsq1 = cs_math_3_square_norm(v);
 
+  double r = d/sqrt(lsq0*lsq1);
+  if (r > 1)
+    r = 1;
+  else if (r < -1)
+    r = -1;
+
+  double theta = acos(r);
+
   cs_real_t uv[3];
 
   cs_math_3_cross_product(u, v, uv);
 
-  double theta = acos(d/sqrt(lsq0*lsq1));
-
   /* Check the sign */
   if (cs_math_3_dot_product(uv, f_n) < 0)
     theta = 2.*cs_math_pi - theta;
@@ -285,6 +292,8 @@ _compact_indexed_data(size_t      elt_size,
  * \param[in]       n_vertices    number of selected vertices
  * \param[in]       faces         list of selected boundary faces (0 to n-1)
  * \param[in]       vertices      ids of selected vertices (0 to n-1)
+ * \param[in]       n_layers      number of layers per selected vertex
+ * \param[in]       vertices      ids of selected vertices (0 to n-1)
  * \param[out]      n_i_edges     number of interior edges
  * \param[out]      n_b_edges     number of boundary edges
  * \param[out]      i_e2sf        interior edges to selected faces
@@ -296,15 +305,19 @@ _compact_indexed_data(size_t      elt_size,
  * \param[out]      b_e2v         boundary edge to vertices connectivity
  * \param[out]      b_e_gc        boundary edge group class
  * \param[out]      b_e_io_num    boundary edge global number (or NULL)
+ *
+ * \return  pointer modified number of layers par selected vertex array if
+*           locking of some vertices is required, NULL otherwise
  */
 /*----------------------------------------------------------------------------*/
 
-static void
+static cs_lnum_t *
 _build_face_edges(cs_mesh_t         *m,
                   cs_lnum_t          n_faces,
                   cs_lnum_t          n_vertices,
                   const cs_lnum_t    faces[],
                   const cs_lnum_t    vertices[],
+                  const cs_lnum_t    n_layers[],
                   cs_lnum_t         *n_i_edges,
                   cs_lnum_t         *n_b_edges,
                   cs_lnum_2_t       *i_e2sf[],
@@ -568,6 +581,8 @@ _build_face_edges(cs_mesh_t         *m,
 
   /* Counting loop */
 
+  cs_lnum_t *lock_mult = NULL;
+
   cs_lnum_t _n_i_edges = 0, _n_b_edges = 0;
 
   for (cs_lnum_t i = 0; i < _n_edges; i++) {
@@ -581,15 +596,14 @@ _build_face_edges(cs_mesh_t         *m,
         _n_b_edges += 1;
     }
     else if (e_nf[i] > 2) {
-      cs_real_t *cv0 = m->vtx_coord + (e2v[i][0] * 3);
-      cs_real_t *cv1 = m->vtx_coord + (e2v[i][1] * 3);
-      bft_error(__FILE__, __LINE__, 0,
-                _("Edge for extrusion joining vertices at coordinates\n"
-                  " [%12.3g, %12.3g, %12.3g] and\n"
-                  " [%12.3g, %12.3g, %12.3g] has %d adjacent faces."),
-                (double)cv0[0], (double)cv0[1], (double)cv0[2],
-                (double)cv1[0], (double)cv1[1], (double)cv1[2],
-                (int)e_nf[i]);
+      /* Too many incident faces: no extrusion along this edge */
+      if (lock_mult == NULL) {
+        BFT_MALLOC(lock_mult,  m->n_vertices, cs_lnum_t);
+        for (cs_lnum_t j = 0; j < m->n_vertices; j++)
+          lock_mult[j] = 1;
+      }
+      lock_mult[e2v[i][0]] = 0;
+      lock_mult[e2v[i][1]] = 0;
     }
   }
 
@@ -684,6 +698,20 @@ _build_face_edges(cs_mesh_t         *m,
   *b_e2sf = _b_e2sf;
   *b_e2v = _b_e2v;
   *b_e_gc = _b_e_gc;
+
+  /* Restrict _n_layers to selected vertices */
+
+  cs_lnum_t  *_n_layers = NULL;
+
+  if (lock_mult != NULL) {
+    BFT_MALLOC(_n_layers, n_vertices, cs_lnum_t);
+    for (cs_lnum_t j = 0; j < n_vertices; j++) {
+      _n_layers[j] = n_layers[j] * lock_mult[vertices[j]];
+    }
+    BFT_FREE(lock_mult);
+  }
+
+  return _n_layers;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -699,6 +727,7 @@ _build_face_edges(cs_mesh_t         *m,
  * \param[in]       n_vertices    number of selected vertices
  * \param[in]       vertices      ids of selected vertices (0 to n-1)
  * \param[in]       n_layers      number of layers for each vertex
+ * \param[in]       n_layers_ini  predicted number of layers for each vertex
  * \param[in]       coord_shift   extrusion vector for each vertex
  * \param[in]       distribution  optional distribution of resulting vertices
  *                                along each extrusion vector
@@ -715,6 +744,7 @@ _add_extruded_vertices(cs_mesh_t          *m,
                        cs_lnum_t           n_vertices,
                        const cs_lnum_t     vertices[],
                        const cs_lnum_t     n_layers[],
+                       const cs_lnum_t     n_layers_ini[],
                        const cs_coord_3_t  coord_shift[],
                        const float         distribution[])
 {
@@ -755,7 +785,7 @@ _add_extruded_vertices(cs_mesh_t          *m,
       const cs_real_t *s_coo = m->vtx_coord + 3*v_id;
       const cs_lnum_t s_id = v_shift[i];
       cs_real_t *d_coo = m->vtx_coord + (n_vertices_ini + s_id)*3;
-      cs_real_t d_f = 1./n_layers[i];
+      cs_real_t d_f = 1./n_layers_ini[i];
       for (cs_lnum_t j = 0; j < n_layers[i]; j++) {
         for (cs_lnum_t k = 0; k < 3; k++)
           d_coo[j*3 + k] = s_coo[k] + d_f*(j+1)*coord_shift[i][k];
@@ -1008,7 +1038,8 @@ _add_layer_faces(cs_mesh_t        *m,
                 m->i_face_vtx_idx[m->n_i_faces] + connect_add_size,
                 cs_lnum_t);
 
-    BFT_REALLOC(m->i_face_family, m->n_i_faces + n_add_faces, cs_lnum_t);
+    BFT_REALLOC(m->i_face_family, m->n_i_faces + n_add_faces, int);
+    BFT_REALLOC(m->i_face_r_gen, m->n_i_faces + n_add_faces, char);
 
     bool have_g_i_face_num = (m->global_i_face_num != NULL) ? true : false;
     bool have_g_cell_num = (m->global_cell_num != NULL) ? true : false;
@@ -1118,6 +1149,7 @@ _add_layer_faces(cs_mesh_t        *m,
         else
           m->i_face_family[n_i_faces_ini + s_id]
             = m->b_face_family[f_id];
+        m->i_face_r_gen[n_i_faces_ini + s_id] = 0;
 
         /* Other faces shifted */
 
@@ -1126,6 +1158,7 @@ _add_layer_faces(cs_mesh_t        *m,
           m->i_face_cells[l][0] = n_cells_ini + s_id + j - 1;
           m->i_face_cells[l][1] = n_cells_ini + s_id + j;
           m->i_face_family[l] = default_family_id;
+          m->i_face_r_gen[l] = 0;
         }
 
         /* Boundary face updated to new boundary cell */
@@ -1292,8 +1325,9 @@ _add_side_faces(cs_mesh_t           *m,
     cs_lnum_t *a_face_cell = NULL;
     cs_lnum_t *p_face_vtx_idx = NULL;
     cs_lnum_t *p_face_vtx_lst = NULL;
-    cs_lnum_t *a_face_gc = NULL;
+    int *a_face_gc = NULL;
     cs_gnum_t *a_face_gnum = NULL;
+    char *a_face_r_gen = NULL;
 
     if (e2f_stride == 2) {
       cs_lnum_t f2v_size_ini = m->i_face_vtx_idx[m->n_i_faces];
@@ -1301,11 +1335,13 @@ _add_side_faces(cs_mesh_t           *m,
       BFT_REALLOC(m->i_face_cells, m->n_i_faces + n_faces_add, cs_lnum_2_t);
       BFT_REALLOC(m->i_face_vtx_idx, m->n_i_faces + n_faces_add + 1, cs_lnum_t);
       BFT_REALLOC(m->i_face_vtx_lst, f2v_size_ini + f2v_size_add, cs_lnum_t);
-      BFT_REALLOC(m->i_face_family, m->n_i_faces + n_faces_add, cs_lnum_t);
+      BFT_REALLOC(m->i_face_family, m->n_i_faces + n_faces_add, int);
+      BFT_REALLOC(m->i_face_r_gen, m->n_i_faces + n_faces_add, char);
       a_face_cell = (cs_lnum_t *)(m->i_face_cells + m->n_i_faces);
       p_face_vtx_idx = m->i_face_vtx_idx + m->n_i_faces;
       p_face_vtx_lst = m->i_face_vtx_lst + f2v_size_ini;
       a_face_gc = m->i_face_family + m->n_i_faces;
+      a_face_r_gen = m->i_face_r_gen + m->n_i_faces;
       if (e_io_num != NULL) {
         BFT_REALLOC(m->global_i_face_num, m->n_i_faces + n_faces_add, cs_gnum_t);
         a_face_gnum = m->global_i_face_num +  m->n_i_faces;
@@ -1317,7 +1353,7 @@ _add_side_faces(cs_mesh_t           *m,
       BFT_REALLOC(m->b_face_cells, m->n_b_faces + n_faces_add, cs_lnum_t);
       BFT_REALLOC(m->b_face_vtx_idx, m->n_b_faces + n_faces_add + 1, cs_lnum_t);
       BFT_REALLOC(m->b_face_vtx_lst, f2v_size_ini + f2v_size_add, cs_lnum_t);
-      BFT_REALLOC(m->b_face_family, m->n_b_faces + n_faces_add, cs_lnum_t);
+      BFT_REALLOC(m->b_face_family, m->n_b_faces + n_faces_add, int);
       a_face_cell = m->b_face_cells + m->n_b_faces;
       p_face_vtx_idx = m->b_face_vtx_idx + m->n_b_faces;
       p_face_vtx_lst = m->b_face_vtx_lst + f2v_size_ini;
@@ -1442,6 +1478,8 @@ _add_side_faces(cs_mesh_t           *m,
       if (e2f_stride != 1) {
         for (cs_lnum_t k = 0; k < n_f_sub; k++)
           a_face_gc[f_shift[i] + k] = default_family_id;
+        for (cs_lnum_t k = 0; k < n_f_sub; k++)
+          a_face_r_gen[f_shift[i] + k] = 0;
       }
       else {
         for (cs_lnum_t k = 0; k < n_f_sub; k++)
@@ -1939,6 +1977,15 @@ cs_mesh_extrude(cs_mesh_t                        *m,
   if (n_g_sel_faces < 1)
     return;
 
+  if (m->n_init_perio > 0) {
+    cs_mesh_boundary_remove_periodicity(m);
+    bft_printf
+      (_("\n Warning:\n"
+         " --------\n"
+         "  Extruding a mesh removes periodicity information.\n"
+         "  Periodicity may need to be rebuild.\n"));
+  }
+
   cs_mesh_free_rebuildable(m, false);
 
   /* Local names for parameters */
@@ -1947,7 +1994,7 @@ cs_mesh_extrude(cs_mesh_t                        *m,
   const cs_lnum_t      n_vertices = e->n_vertices;
   const cs_lnum_t     *faces = e->face_ids;
   const cs_lnum_t     *vertices = e->vertex_ids;
-  const cs_lnum_t     *n_layers = e->n_layers;
+  const cs_lnum_t     *n_layers_ini = e->n_layers;
   const cs_coord_3_t  *coord_shift = (const cs_coord_3_t *)e->coord_shift;
   const float         *distribution = e->distribution;
 
@@ -1975,26 +2022,33 @@ cs_mesh_extrude(cs_mesh_t                        *m,
   int  *b_e_gc = NULL;
   fvm_io_num_t  *i_e_io_num = NULL, *b_e_io_num = NULL;
 
-  _build_face_edges(m,
-                    n_faces,
-                    n_vertices,
-                    _faces,
-                    _vertices,
-                    &n_i_edges,
-                    &n_b_edges,
-                    &i_e2sf,
-                    &i_e2v,
-                    &i_e_io_num,
-                    &b_e2sf,
-                    &b_e2v,
-                    &b_e_gc,
-                    &b_e_io_num);
+  cs_lnum_t  *_n_layers
+    = _build_face_edges(m,
+                        n_faces,
+                        n_vertices,
+                        _faces,
+                        _vertices,
+                        n_layers_ini,
+                        &n_i_edges,
+                        &n_b_edges,
+                        &i_e2sf,
+                        &i_e2v,
+                        &i_e_io_num,
+                        &b_e2sf,
+                        &b_e2v,
+                        &b_e_gc,
+                        &b_e_io_num);
+
+  const cs_lnum_t *n_layers = (const cs_lnum_t *)_n_layers;
+  if (_n_layers == NULL)
+    n_layers = n_layers_ini;
 
   /* Now generate added vertices */
 
   cs_lnum_t *n_v_shift = _add_extruded_vertices(m,
                                                 n_vertices,
                                                 _vertices,
+                                                n_layers_ini,
                                                 n_layers,
                                                 coord_shift,
                                                 distribution);
@@ -2080,6 +2134,9 @@ cs_mesh_extrude(cs_mesh_t                        *m,
 
   /* Free local arrays */
 
+  n_layers = NULL;
+  BFT_FREE(_n_layers);
+
   BFT_FREE(v_s_id);
 
   BFT_FREE(n_c_shift);
@@ -2094,16 +2151,14 @@ cs_mesh_extrude(cs_mesh_t                        *m,
   BFT_FREE(l_faces);
   BFT_FREE(l_vertices);
 
-  m->modified = CS_MAX(m->modified, 1);
+  m->modified |= CS_MESH_MODIFIED;
 
   /* Rebuild ghosts */
 
   if (m->halo != NULL || m->halo_type == CS_HALO_EXTENDED) {
-    cs_halo_type_t halo_type = m->halo_type;
     cs_halo_destroy(&(m->halo));
-    assert(m == cs_glob_mesh);
     cs_mesh_builder_t *mb = (m == cs_glob_mesh) ? cs_glob_mesh_builder : NULL;
-    cs_mesh_init_halo(m, mb, halo_type);
+    cs_mesh_init_halo(m, mb, m->halo_type);
   }
 
   cs_mesh_update_auxiliary(cs_glob_mesh);
@@ -2181,7 +2236,7 @@ cs_mesh_extrude_face_info_create(const cs_mesh_t  *m)
 
   BFT_MALLOC(efi->n_layers, n_faces, cs_lnum_t);
   BFT_MALLOC(efi->distance, n_faces, cs_real_t);
-  BFT_MALLOC(efi->expansion_factor, n_faces, float_t);
+  BFT_MALLOC(efi->expansion_factor, n_faces, float);
   BFT_MALLOC(efi->thickness_s, n_faces, cs_real_t);
   BFT_MALLOC(efi->thickness_e, n_faces, cs_real_t);
 
