@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -658,7 +658,8 @@ _mesh_group_add(cs_mesh_t        *mesh,
   /* Build index on entities (previous group class for elements
      not selected, previous + new for those selected) */
 
-  cs_lnum_t *gc_tmp_idx = NULL, *gc_tmp = NULL;
+  cs_lnum_t *gc_tmp_idx = NULL;
+  int *gc_tmp = NULL;
 
   BFT_MALLOC(gc_tmp_idx, n_elts + 1, cs_lnum_t);
   gc_tmp_idx[0] = 0;
@@ -762,7 +763,7 @@ cs_mesh_group_clean(cs_mesh_t  *mesh)
 
   BFT_FREE(order);
 
-  BFT_REALLOC(mesh->group_idx, n_groups + 1, cs_int_t);
+  BFT_REALLOC(mesh->group_idx, n_groups + 1, int);
   BFT_REALLOC(mesh->group, size_tot, char);
 
   mesh->n_groups = n_groups;
@@ -873,7 +874,7 @@ cs_mesh_group_combine_classes(cs_mesh_t   *mesh,
 
     for (i = 0; i < n_elts; i++) {
       cs_lnum_t k, l, n;
-      _Bool is_same = true;
+      bool is_same = true;
       j = order[i];
       n = gc_id_idx[j+1] - gc_id_idx[j];
       if (n != n_prev)
@@ -959,7 +960,7 @@ cs_mesh_group_combine_classes(cs_mesh_t   *mesh,
     if (n_max_family_items > mesh->n_max_family_items) {
       BFT_REALLOC(mesh->family_item,
                   mesh->n_families*n_max_family_items,
-                  cs_lnum_t);
+                  int);
       for (i = mesh->n_max_family_items;
            i < n_max_family_items;
            i++) {
