@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -899,7 +899,7 @@ _compute_warp_error(const cs_mesh_t              *mesh,
         tens[ki][kj] *= invvol_c;
 
     warp_error[c_id] =
-      fabs(cs_math_33_determinant((const cs_real_t (*)[3])tens) - 1);
+      fabs(cs_math_33_determinant((const cs_real_t (*)[3])tens) - 1.);
 
   } // Loop on cells
 
@@ -1442,7 +1442,9 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                                warp_error,
                                warp_error);
 
-    bft_printf(" L2-error norm induced by warping : %5.3e\n", sqrt(l2_error));
+    if (l2_error > 0) /* May be < 0 in case of negative volumes */
+      bft_printf(" L2-error norm induced by warping : %5.3e\n", sqrt(l2_error));
+
     BFT_FREE(warp_error);
 
   } /* End of cell volume treatment */
