@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -33,7 +33,6 @@
 
 #include "cs_base.h"
 #include "cs_mesh.h"
-#include "cs_internal_coupling.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -50,7 +49,7 @@ BEGIN_C_DECLS
  */
 
 /*
- * Field property type
+ * Cell quantities correction types
  */
 
 /*! Correct bad cells warping for gradients */
@@ -71,9 +70,14 @@ BEGIN_C_DECLS
 /*! Clip geometrical quantities used in flux reconstuction */
 #define CS_FACE_RECONSTRUCTION_CLIP (1 << 5)
 
-/*! Limit cells volumle ratio */
+/*! Limit cells volume ratio */
 #define CS_CELL_VOLUME_RATIO_CORRECTION (1 << 6)
 
+/*! Refine face center computation for warped cells
+    (iteratively compute center using previous position instead
+    of using only the initial estimate based on the vertices center) */
+#define CS_FACE_CENTER_REFINE (1 << 7)
+
 /*! @} */
 
 /*============================================================================
@@ -105,6 +109,9 @@ typedef struct {
   cs_real_t     *i_f_face_surf;  /* Fluid surface of interior faces. */
   cs_real_t     *b_f_face_surf;  /* Fluid surface of boundary faces. */
 
+  cs_real_2_t   *i_f_face_factor;/* Fluid surface factor of interior faces. */
+  cs_real_t     *b_f_face_factor;/* Fluid surface factor of boundary faces. */
+
   cs_real_t     *dijpf;          /* Vector I'J' for interior faces */
   cs_real_t     *diipb;          /* Vector II'  for border faces */
   cs_real_t     *dofij;          /* Vector OF   for interior faces */
@@ -126,31 +133,23 @@ typedef struct {
   cs_real_t      max_f_vol;        /* Maximum cell volume */
   cs_real_t      tot_f_vol;        /* Total volume */
 
-  cs_real_33_t  *cocgb_s_it;     /* coupling of gradient components for
-                                    iterative reconstruction at boundary */
-  cs_real_33_t  *cocg_s_it;      /* coupling of gradient components for
-                                    iterative reconstruction */
-  cs_real_33_t  *cocgb_s_lsq;    /* coupling of gradient components for
-                                    least-square reconstruction at boundary */
-
-  cs_real_33_t  *cocg_it;        /* Interleaved cocg matrix
-                                    for iterative gradients */
-  cs_real_33_t  *cocg_lsq;       /* Interleaved cocg matrix
-                                    for least square gradients */
-
   cs_real_t     *corr_grad_lin_det;  /* Determinant of geometrical matrix
                                         linear gradient correction */
-  cs_real_33_t  *corr_grad_lin;      /* Geometrical matrix
+  cs_real_33_t  *corr_grad_lin;    /* Geometrical matrix
                                         linear gradient correction */
 
-  cs_int_t      *b_sym_flag;     /* Symmetry flag for boundary faces */
-  cs_int_t      *c_solid_flag;   /* Is the fluid volume 0 flag */
-  unsigned      *bad_cell_flag;  /* Flag (mask) for bad cells detected */
+  int          *b_sym_flag;        /* Symmetry flag for boundary faces */
+  int           has_disable_flag;  /* Is the cell disabled?
+                                      0: unactivated
+                                      1: activated */
+  int          *c_disable_flag;    /* Is the cell disabled?
+                                      used for fluid solid and porous modelling */
+  unsigned     *bad_cell_flag;     /* Flag (mask) for bad cells detected */
 
 } cs_mesh_quantities_t ;
 
 /*============================================================================
- * Static global variables
+ * Global variables
  *============================================================================*/
 
 /* Pointer to mesh quantities structure associated to the main mesh */
@@ -160,160 +159,101 @@ extern cs_mesh_quantities_t  *cs_glob_mesh_quantities;
 /* Flag (mask) to activate bad cells correction */
 extern unsigned cs_glob_mesh_quantities_flag;
 
-/* Choice of the porous model */
-extern int cs_glob_porous_model;
-
-/*============================================================================
- * Public function prototypes for API Fortran
- *============================================================================*/
-
-/*----------------------------------------------------------------------------
- * Query or modification of the option for computing cell centers.
- *
- * This function returns 1 or 2 according to the selected algorithm.
- *
- * Fortran interface :
- *
- * SUBROUTINE ALGCEN (IOPT)
- * *****************
- *
- * INTEGER          IOPT        : <-> : Choice of the algorithm
- *                                      < 0 : query
- *                                        0 : computation based
- *                                            on faces (default choice)
- *                                        1 : computation based
- *                                            on vertices
- *----------------------------------------------------------------------------*/
-
-void
-CS_PROCF (algcen, ALGCEN) (cs_int_t  *const iopt);
-
-/*----------------------------------------------------------------------------
- * Set behavior for computing the cocg matrixes for the iterative algo
- * and for the Least square method for scalar and vector gradients.
- *
- * Fortran interface :
- *
- * subroutine comcoc (imrgra)
- * *****************
- *
- * integer          imrgra        : <-- : gradient reconstruction option
- *----------------------------------------------------------------------------*/
-
-void
-CS_PROCF (comcoc, COMCOC) (const cs_int_t  *const imrgra);
-
-/*----------------------------------------------------------------------------
- * Set porous model
- *
- * Fortran interface :
- *
- * subroutine compor (iporos)
- * *****************
- *
- * integer          iporos        : <-- : porous model
- *----------------------------------------------------------------------------*/
-
-void
-CS_PROCF (compor, COMPOR) (const cs_int_t  *const iporos);
-
 /*=============================================================================
  * Public function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Query or modification of the option for computing cell centers.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Query or modification of the option for computing cell centers.
  *
- *  < 0 : query
- *    0 : computation based on faces (default choice)
- *    1 : computation based on vertices
+ * \param[in]  algo_choice  < 0 : query
+ *                            0 : computation based on face centers (default)
+ *                            1 : computation by cell sub-volumes
  *
- * algo_choice  <--  choice of algorithm to compute cell centers.
- *
- * returns:
- *  1 or 2 according to the selected algorithm.
- *----------------------------------------------------------------------------*/
+ * \return  0 or 1 according to the selected algorithm
+ */
+/*----------------------------------------------------------------------------*/
 
 int
-cs_mesh_quantities_cell_cen_choice(const int algo_choice);
+cs_mesh_quantities_cell_cen_choice(int  algo_choice);
 
-/*----------------------------------------------------------------------------
- * Compute cocg for iterative gradient reconstruction for scalars.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Query or modification of the option for computing face centers.
  *
- * parameters:
- *   gradient_option <-- gradient option (Fortran IMRGRA)
- *----------------------------------------------------------------------------*/
-
-void
-cs_mesh_quantities_set_cocg_options(int  gradient_option);
-
-/*----------------------------------------------------------------------------
- * Compute Fluid volumes and fluid surface in addition to cell volume and surfaces.
+ * \param[in]  algo_choice  < 0 : query
+ *                            0 : standard computation
+ *                            1 : use adjustment for volume
+ *                                from versions 1.1 to 5.3
  *
- * parameters:
- *   porous_model <-- gradient option (Fortran iporos)
- *----------------------------------------------------------------------------*/
+ * \return  0 or 1 according to the selected algorithm
+ */
+/*----------------------------------------------------------------------------*/
 
-void
-cs_mesh_quantities_set_porous_model(int  porous_model);
+int
+cs_mesh_quantities_face_cog_choice(int  algo_choice);
 
-/*----------------------------------------------------------------------------
- * Create a mesh quantities structure.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Create a mesh quantities structure.
  *
- * returns:
- *   pointer to created cs_mesh_quantities_t structure
- *----------------------------------------------------------------------------*/
-
-cs_mesh_quantities_t  *
-cs_mesh_quantities_create(void);
-
-/*----------------------------------------------------------------------------
- * Destroy a mesh quantities structure
- *
- * parameters:
- *   mesh_quantities <-- pointer to a cs_mesh_quantities_t structure
- *
- * returns:
- *  NULL
- *----------------------------------------------------------------------------*/
+ * \return  pointer to created cs_mesh_quantities_t structure
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_mesh_quantities_t *
-cs_mesh_quantities_destroy(cs_mesh_quantities_t  *mesh_quantities);
+cs_mesh_quantities_create(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Destroy a mesh quantities structure.
+ *
+ * \param[in]  mq  pointer to mesh quantities structures
+ *
+ * \return  NULL
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_mesh_quantities_t *
+cs_mesh_quantities_destroy(cs_mesh_quantities_t  *mq);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Reset a mesh quantities structure to its empty initial state.
  *
- * \param[in]   mq           pointer to mesh quantities structures.
+ * \param[in]  mq  pointer to mesh quantities structures
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_mesh_quantities_free_all(cs_mesh_quantities_t  *mq);
 
-/*----------------------------------------------------------------------------
- * Compute mesh quantities needed fo preprocessing
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute mesh quantities needed for preprocessing.
  *
- * parameters:
- *   mesh            <-- pointer to a cs_mesh_t structure
- *   mesh_quantities <-> pointer to a cs_mesh_quantities_t structure
- *----------------------------------------------------------------------------*/
+ * \param[in]       m   pointer to mesh structure
+ * \param[in, out]  mq  pointer to mesh quantities structures
+ */
+/*----------------------------------------------------------------------------*/
 
 void
-cs_mesh_quantities_compute_preprocess(const cs_mesh_t       *mesh,
-                                      cs_mesh_quantities_t  *mesh_quantities);
+cs_mesh_quantities_compute_preprocess(const cs_mesh_t       *m,
+                                      cs_mesh_quantities_t  *mq);
 
-/*----------------------------------------------------------------------------
- * Compute mesh quantities
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute mesh quantities.
  *
- * parameters:
- *   mesh            <-- pointer to a cs_mesh_t structure
- *   mesh_quantities <-> pointer to a cs_mesh_quantities_t structure
- *----------------------------------------------------------------------------*/
+ * \param[in]       m   pointer to mesh structure
+ * \param[in, out]  mq  pointer to mesh quantities structures.
+ */
+/*----------------------------------------------------------------------------*/
 
 void
-cs_mesh_quantities_compute(const cs_mesh_t       *mesh,
-                           cs_mesh_quantities_t  *mesh_quantities);
+cs_mesh_quantities_compute(const cs_mesh_t       *m,
+                           cs_mesh_quantities_t  *mq);
 
 /*----------------------------------------------------------------------------
  * Compute fluid mesh quantities
@@ -327,6 +267,18 @@ void
 cs_mesh_quantities_fluid_compute(const cs_mesh_t       *mesh,
                                  cs_mesh_quantities_t  *mesh_quantities);
 
+/*----------------------------------------------------------------------------
+ * Compute the total, min, and max fluid volumes of cells
+ *
+ * parameters:
+ *   mesh            <-- pointer to mesh structure
+ *   mesh_quantities <-> pointer to a mesh quantities structure
+ *----------------------------------------------------------------------------*/
+
+void
+cs_mesh_quantities_fluid_vol_reductions(const cs_mesh_t       *mesh,
+                                        cs_mesh_quantities_t  *mesh_quantities);
+
 /*----------------------------------------------------------------------------
  * Compute fluid section mesh quantities at the initial step
  *
@@ -399,20 +351,51 @@ cs_mesh_quantities_b_faces(const cs_mesh_t   *mesh,
                            cs_real_t         *p_b_face_cog[],
                            cs_real_t         *p_b_face_normal[]);
 
-/*----------------------------------------------------------------------------
- * Compute cell centers.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute approximate cells centers as the mean of the given face
+ *         centers weighted by the associated surfaces.
  *
- * The corresponding array is allocated by this function, and it is the
- * caller's responsibility to free it when they are no longer needed.
+ *           n-1
+ *           Sum  Surf(Fi) G(Fi)
+ *           i=0
+ *  G(C) = -----------------------
+ *           n-1
+ *           Sum  Surf(Fi)
+ *           i=0
  *
- * parameters:
- *   mesh       <-- pointer to a cs_mesh_t structure
- *   p_cell_cen <-> pointer to the cell centers array
- *----------------------------------------------------------------------------*/
+ * \param[in]   mesh         pointer to mesh structure
+ * \param[in]   i_face_norm  surface normal of internal faces
+ * \param[in]   i_face_cog   center of gravity of internal faces
+ * \param[in]   b_face_norm  surface normal of border faces
+ * \param[in]   b_face_cog   center of gravity of border faces
+ * \param[out]  cell_cen     cell centers
+ */
+/*----------------------------------------------------------------------------*/
 
 void
-cs_mesh_quantities_cell_cen(const cs_mesh_t  *mesh,
-                            cs_real_t        *cell_cen[]);
+cs_mesh_quantities_cell_faces_cog(const cs_mesh_t  *mesh,
+                                  const cs_real_t   i_face_norm[],
+                                  const cs_real_t   i_face_cog[],
+                                  const cs_real_t   b_face_norm[],
+                                  const cs_real_t   b_face_cog[],
+                                  cs_real_t         cell_cen[]);
+
+/*----------------------------------------------------------------------------
+ * Compute cell volumes.
+ *
+ * The corresponding array is allocated by this function, and it is the
+ * caller's responsability to free it when they are no longer needed.
+ *
+ * parameters:
+ *   mesh     <-- pointer to a cs_mesh_t structure
+ *
+ * return:
+ *   pointer to newly allocated cell volumes array
+ *----------------------------------------------------------------------------*/
+
+cs_real_t *
+cs_mesh_quantities_cell_volume(const cs_mesh_t  *mesh);
 
 /*----------------------------------------------------------------------------
  * Check that no negative volumes are present, and exit on error otherwise.
@@ -428,18 +411,24 @@ cs_mesh_quantities_check_vol(const cs_mesh_t             *mesh,
                              const cs_mesh_quantities_t  *mesh_quantities,
                              int                          allow_error);
 
-/*----------------------------------------------------------------------------
- * Update mesh quantities relative to extended ghost cells when the
- * neighborhood is reduced.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the bounding box for cells.
  *
- * parameters:
- *   mesh            <-- pointer to a cs_mesh_t structure
- *   mesh_quantities <-> pointer to a cs_mesh_quantities_t structure
- *----------------------------------------------------------------------------*/
+ * The corresponding array is allocated by this function, and it is the
+ * caller's responsability to free it when they are no longer needed.
+ *
+ * \param[in]   m          pointer to mesh structure
+ * \param[in]   tolerance  addition to local extents of each element:
+ *                         extent = base_extent * (1 + tolerance)
+ *
+ * \return  pointer to newly allocated cell volumes array
+ */
+/*----------------------------------------------------------------------------*/
 
-void
-cs_mesh_quantities_reduce_extended(const cs_mesh_t       *mesh,
-                                   cs_mesh_quantities_t  *mesh_quantities);
+cs_real_6_t *
+cs_mesh_quantities_cell_extents(const cs_mesh_t  *m,
+                                cs_real_t         tolerance);
 
 /*----------------------------------------------------------------------------
  * Return the number of times mesh quantities have been computed.
@@ -486,6 +475,15 @@ cs_mesh_quantities_b_thickness_f(const cs_mesh_t             *m,
                                  int                          n_passes,
                                  cs_real_t                    b_thickness[]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Log mesh quantities options to setup file.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_mesh_quantities_log_setup(void);
+
 /*----------------------------------------------------------------------------
  * Dump a cs_mesh_quantities_t structure
  *
@@ -498,36 +496,6 @@ void
 cs_mesh_quantities_dump(const cs_mesh_t             *mesh,
                         const cs_mesh_quantities_t  *mesh_quantities);
 
-/*----------------------------------------------------------------------------
- * Compute 3x3 matrix cocg for the scalar gradient least squares algorithm
- * adapted for internal coupling.
- *
- * parameters:
- *   m    <--  mesh
- *   fvq  <->  mesh quantities
- *   ce   <->  coupling
- *----------------------------------------------------------------------------*/
-
-void
-cs_compute_cell_cocg_lsq_coupling(const cs_mesh_t         *m,
-                                  cs_mesh_quantities_t    *fvq,
-                                  cs_internal_coupling_t  *ce);
-
-/*----------------------------------------------------------------------------
- * Compute 3x3 matrix cocg for the scalar gradient iterative algorithm
- * adapted for internal coupling.
- *
- * parameters:
- *   m    <--  mesh
- *   fvq  <->  mesh quantities
- *   ce   <->  coupling
- *----------------------------------------------------------------------------*/
-
-void
-cs_compute_cell_cocg_it_coupling(const cs_mesh_t         *m,
-                                 cs_mesh_quantities_t    *fvq,
-                                 cs_internal_coupling_t  *ce);
-
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
