@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -118,15 +118,14 @@ _compute_distance(const double  a[3],
  *----------------------------------------------------------------------------*/
 
 static void
-_compute_minmax(cs_int_t            n_vals,
+_compute_minmax(cs_lnum_t           n_vals,
                 const cs_real_t     var[],
                 cs_real_t          *min,
                 cs_real_t          *max)
 {
-  cs_int_t  i;
   cs_real_t  _min = DBL_MAX, _max = -DBL_MAX;
 
-  for (i = 0; i < n_vals; i++) {
+  for (cs_lnum_t i = 0; i < n_vals; i++) {
     _min = CS_MIN(_min, var[i]);
     _max = CS_MAX(_max, var[i]);
   }
@@ -375,9 +374,9 @@ _move_vertices(cs_mesh_t  *mesh,
 static void
 _get_local_tolerance(const cs_real_t   vtx_coords[],
                      double            vtx_tolerance[],
-                     const cs_int_t    n_faces,
-                     const cs_int_t    face_vtx_idx[],
-                     const cs_int_t    face_vtx_lst[],
+                     const cs_lnum_t   n_faces,
+                     const cs_lnum_t   face_vtx_idx[],
+                     const cs_lnum_t   face_vtx_lst[],
                      double            fraction)
 {
   cs_lnum_t  j, k, start, end, face_id, vtx_id1, vtx_id2;
@@ -464,7 +463,7 @@ _get_global_tolerance(cs_mesh_t            *mesh,
                                      0,
                                      n_g_vertices);
 
-  cs_int_t block_size = bi.block_size;
+  cs_lnum_t block_size = bi.block_size;
 
   cs_all_to_all_t
     *d = cs_all_to_all_create_from_block(n_vertices,
@@ -775,14 +774,22 @@ _compute_vtx_normals(cs_mesh_t           *mesh,
  *
  * <a name="fix_by_feature"></a>
  *
+ * This function locks a vertex in place if one of its feature angles is
+ * less than the maximum feature angle (in degrees) defined by the user.
+ *
+ * Vertex normals are based on the average of the normals of the adjacent
+ * boundary faces.
+ * The feature angle between a vertex and one of its adjacent faces is defined
+ * as the angle between the vertex normal and the face normal.
+ *
  * Please refer to the
  * <a href="../../theory.pdf#fixbyfeature"><b>specific treatment for boundary faces</b></a>
- * section of the theory guide for more informations.
+ * section of the theory guide for more information.
  *
- * parameters:
  *  \param[in]  mesh           pointer to a cs_mesh_t structure
- *  \param[in]  feature_angle  feature angle (bounded between 0 and 90 degrees)
- *  \param[out] vtx_is_fixed   array to define vertices mobility (1: fixed, 0: free)
+ *  \param[in]  feature_angle  feature angle (between 0 and 90 degrees)
+ *  \param[out] vtx_is_fixed   array to define vertices mobility
+ *                             (1: fixed, 0: free)
  */
 /*----------------------------------------------------------------------------*/
 
