@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -52,6 +52,7 @@
 #include "cs_mesh_quality.h"
 #include "cs_mesh_connect.h"
 #include "cs_order.h"
+#include "cs_parall.h"
 #include "cs_post.h"
 
 /*----------------------------------------------------------------------------
@@ -89,16 +90,16 @@ static double cs_glob_mesh_warping_threshold = -1.0;
  *----------------------------------------------------------------------------*/
 
 static void
-_select_warped_faces(cs_int_t        n_faces,
+_select_warped_faces(cs_lnum_t       n_faces,
                      double          max_warp_angle,
                      double          face_warping[],
-                     cs_int_t        *p_n_warp_faces,
-                     cs_int_t        *p_warp_face_lst[])
+                     cs_lnum_t      *p_n_warp_faces,
+                     cs_lnum_t      *p_warp_face_lst[])
 {
-  cs_int_t  face_id;
+  cs_lnum_t  face_id;
 
-  cs_int_t  n_warp_faces = 0;
-  cs_int_t  *warp_face_lst = NULL;
+  cs_lnum_t  n_warp_faces = 0;
+  cs_lnum_t  *warp_face_lst = NULL;
 
   if (n_faces > 0) {
 
@@ -106,7 +107,7 @@ _select_warped_faces(cs_int_t        n_faces,
       if (face_warping[face_id] >= max_warp_angle)
         n_warp_faces++;
 
-    BFT_MALLOC(warp_face_lst, n_warp_faces, cs_int_t);
+    BFT_MALLOC(warp_face_lst, n_warp_faces, cs_lnum_t);
 
     n_warp_faces = 0;
 
@@ -392,6 +393,7 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
   cs_lnum_t  *new_face_vtx_idx = NULL, *new_face_vtx_lst = NULL;
   cs_lnum_2_t  *new_face_cells = NULL;
   int        *new_face_family = NULL;
+  char       *new_face_r_gen = NULL;
   cs_lnum_t  *cut_face_lst = NULL;
   cs_lnum_t  *new_face_shift = NULL;
   cs_lnum_t  *n_sub_elt_lst = NULL;
@@ -489,22 +491,11 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
 
   *p_n_sub_elt_lst = n_sub_elt_lst;
 
-  /* If no faces are cut, we are sure at this stage that we do
-     need to communicate face cutting info on the interfaces for
-     this rank, and do not otherwise call collective MPI operations,
-     so we may return immediately. */
-
-  if (n_cut_faces == 0) {
-    BFT_FREE(cut_flag);
-    BFT_FREE(new_face_shift);
-    cs_interface_set_destroy(&face_ifs);
-    return;
-  }
-
   BFT_MALLOC(new_face_vtx_idx, n_new_faces + 1, cs_lnum_t);
   BFT_MALLOC(new_face_vtx_lst, connect_size, cs_lnum_t);
   BFT_MALLOC(new_face_cells, n_new_faces, cs_lnum_2_t);
   BFT_MALLOC(new_face_family, n_new_faces, int);
+  BFT_MALLOC(new_face_r_gen, n_new_faces, char);
 
   BFT_MALLOC(cut_face_lst, n_cut_faces, cs_lnum_t);
 
@@ -528,23 +519,15 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
 
     if (cut_flag[face_id] != 0) {
 
-      if (n_face_vertices == 4)
-        n_triangles = fvm_triangulate_quadrangle(dim,
-                                                 0,
-                                                 mesh->vtx_coord,
-                                                 NULL,
-                                                 mesh->i_face_vtx_lst + idx_start,
-                                                 new_face_vtx_lst + connect_size);
-      else
-        n_triangles = fvm_triangulate_polygon(dim,
-                                              0,
-                                              n_face_vertices,
-                                              mesh->vtx_coord,
-                                              NULL,
-                                              mesh->i_face_vtx_lst + idx_start,
-                                              FVM_TRIANGULATE_ELT_DEF,
-                                              new_face_vtx_lst + connect_size,
-                                              triangle_state);
+      n_triangles = fvm_triangulate_polygon(dim,
+                                            0,
+                                            n_face_vertices,
+                                            mesh->vtx_coord,
+                                            NULL,
+                                            mesh->i_face_vtx_lst + idx_start,
+                                            FVM_TRIANGULATE_ELT_DEF,
+                                            new_face_vtx_lst + connect_size,
+                                            triangle_state);
 
       assert(n_triangles == n_face_vertices - 2);
 
@@ -559,17 +542,17 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
         for (j = 0; j < 2; j++)
           new_face_cells[n_new_faces][j] = mesh->i_face_cells[face_id][j];
 
-        /* Update family for each face */
+        /* Update family and refinement generation for each face */
 
         new_face_family[n_new_faces] = mesh->i_face_family[face_id];
+        new_face_r_gen[n_new_faces] = mesh->i_face_r_gen[face_id];
 
         /* Update "face -> vertices" connectivity index
            (list has alread been defined by fvm_triangulate_polygon) */
 
         n_new_faces++;
         connect_size += 3;
-        new_face_vtx_idx[n_new_faces]
-          = new_face_vtx_idx[n_new_faces-1] + 3;
+        new_face_vtx_idx[n_new_faces] = new_face_vtx_idx[n_new_faces-1] + 3;
 
       } /* End of loop on triangles */
 
@@ -581,9 +564,10 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
       for (j = 0; j < 2; j++)
         new_face_cells[n_new_faces][j] = mesh->i_face_cells[face_id][j];
 
-      /* Update family for each face */
+      /* Update family and refinement generation for each face */
 
       new_face_family[n_new_faces] = mesh->i_face_family[face_id];
+      new_face_r_gen[n_new_faces] = mesh->i_face_r_gen[face_id];
 
       /* Update "face -> vertices" connectivity */
 
@@ -605,9 +589,11 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
 
   BFT_FREE(mesh->i_face_cells);
   BFT_FREE(mesh->i_face_family);
+  BFT_FREE(mesh->i_face_r_gen);
 
   mesh->i_face_cells = new_face_cells;
   mesh->i_face_family = new_face_family;
+  mesh->i_face_r_gen = new_face_r_gen;
 
   new_face_cells = NULL;
   new_face_family = NULL;
@@ -680,15 +666,16 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
  *
  * parameters:
  *   mesh                    <-> pointer to a mesh structure
- *   face_type               <-- internal or border faces
+ *   stride                  <-- 2 or 1 for internal or border faces
  *   p_n_cut_faces           <-> in:  number of faces to cut
  *                               out: number of cut faces
  *   p_cut_face_lst          <-> pointer to the cut face list
  *   p_n_sub_elt_lst         <-> pointer to the sub-elt count list
  *   p_n_faces               <-> pointer to the number of faces
- *   p_face_num              <-> pointer to the global face numbers
  *   p_face_vtx_connect_size <-> size of the "face -> vertex" connectivity
  *   p_face_cells            <-> "face -> cells" connectivity
+ *   p_face_family           <-> face family
+ *   p_face_r_gen            <-> face refinement generation
  *   p_face_vtx_idx          <-> pointer on "face -> vertices" connect. index
  *   p_face_vtx_lst          <-> pointer on "face -> vertices" connect. list
  *----------------------------------------------------------------------------*/
@@ -702,7 +689,8 @@ _cut_warped_faces(cs_mesh_t      *mesh,
                   cs_lnum_t      *p_n_faces,
                   cs_lnum_t      *p_face_vtx_connect_size,
                   cs_lnum_t      *p_face_cells[],
-                  cs_lnum_t      *p_face_family[],
+                  int            *p_face_family[],
+                  char           *p_face_r_gen[],
                   cs_lnum_t      *p_face_vtx_idx[],
                   cs_lnum_t      *p_face_vtx_lst[])
 {
@@ -714,7 +702,9 @@ _cut_warped_faces(cs_mesh_t      *mesh,
 
   fvm_triangulate_state_t  *triangle_state = NULL;
   cs_lnum_t  *new_face_vtx_idx = NULL, *new_face_vtx_lst = NULL;
-  cs_lnum_t  *new_face_cells = NULL, *new_face_family = NULL;
+  cs_lnum_t  *new_face_cells = NULL;
+  int        *new_face_family = NULL;
+  char       *new_face_r_gen = NULL;
   cs_lnum_t  *cut_face_lst = NULL;
   cs_lnum_t  *n_sub_elt_lst = NULL;
   char *cut_flag = NULL;
@@ -770,19 +760,23 @@ _cut_warped_faces(cs_mesh_t      *mesh,
 
   *p_n_sub_elt_lst = n_sub_elt_lst;
 
-  if (n_cut_faces == 0)
+  if (n_cut_faces == 0) {
+    BFT_FREE(cut_flag);
     return;
+  }
 
   BFT_MALLOC(new_face_vtx_idx, n_new_faces + 1, cs_lnum_t);
   BFT_MALLOC(new_face_vtx_lst, connect_size, cs_lnum_t);
   BFT_MALLOC(new_face_cells, n_new_faces*stride, cs_lnum_t);
-  BFT_MALLOC(new_face_family, n_new_faces, cs_lnum_t);
+  BFT_MALLOC(new_face_family, n_new_faces, int);
+  if (p_face_r_gen != NULL)
+    BFT_MALLOC(new_face_r_gen, n_new_faces, char);
 
   BFT_MALLOC(cut_face_lst, n_cut_faces, cs_lnum_t);
 
   triangle_state = fvm_triangulate_state_create(n_max_face_vertices);
 
-  /* Second loop : define the new connectivity after triangulation */
+  /* Second loop: define the new connectivity after triangulation */
 
   new_face_vtx_idx[0] = 0;
   connect_size = 0;
@@ -797,23 +791,15 @@ _cut_warped_faces(cs_mesh_t      *mesh,
 
     if (cut_flag[face_id] != 0) {
 
-      if (n_face_vertices == 4)
-        n_triangles = fvm_triangulate_quadrangle(dim,
-                                                 0,
-                                                 mesh->vtx_coord,
-                                                 NULL,
-                                                 (*p_face_vtx_lst) + idx_start,
-                                                 new_face_vtx_lst + connect_size);
-      else
-        n_triangles = fvm_triangulate_polygon(dim,
-                                              0,
-                                              n_face_vertices,
-                                              mesh->vtx_coord,
-                                              NULL,
-                                              (*p_face_vtx_lst) + idx_start,
-                                              FVM_TRIANGULATE_MESH_DEF,
-                                              new_face_vtx_lst + connect_size,
-                                              triangle_state);
+      n_triangles = fvm_triangulate_polygon(dim,
+                                            0,
+                                            n_face_vertices,
+                                            mesh->vtx_coord,
+                                            NULL,
+                                            (*p_face_vtx_lst) + idx_start,
+                                            FVM_TRIANGULATE_MESH_DEF,
+                                            new_face_vtx_lst + connect_size,
+                                            triangle_state);
 
       assert(n_triangles == n_face_vertices - 2);
 
@@ -829,12 +815,14 @@ _cut_warped_faces(cs_mesh_t      *mesh,
           new_face_cells[stride*n_new_faces + j]
             = (*p_face_cells)[stride*face_id + j];
 
-        /* Update family for each face */
+        /* Update family and refinement generation for each face */
 
         new_face_family[n_new_faces] = (*p_face_family)[face_id];
+        if (p_face_r_gen != NULL)
+          new_face_r_gen[n_new_faces] = (*p_face_r_gen)[face_id];
 
         /* Update "face -> vertices" connectivity index
-           (list has alread been defined by fvm_triangulate_polygon) */
+           (list has already been defined by fvm_triangulate_polygon) */
 
         n_new_faces++;
         connect_size += 3;
@@ -851,9 +839,11 @@ _cut_warped_faces(cs_mesh_t      *mesh,
         new_face_cells[stride*n_new_faces + j]
           = (*p_face_cells)[stride*face_id + j];
 
-      /* Update family for each faces */
+      /* Update family and refinemen generation for each face */
 
       new_face_family[n_new_faces] = (*p_face_family)[face_id];
+      if (p_face_r_gen != NULL)
+        new_face_r_gen[n_new_faces] = (*p_face_r_gen)[face_id];
 
       /* Update "face -> vertices" connectivity */
 
@@ -877,6 +867,8 @@ _cut_warped_faces(cs_mesh_t      *mesh,
   BFT_FREE(*p_face_vtx_lst);
   BFT_FREE(*p_face_cells);
   BFT_FREE(*p_face_family);
+  if (p_face_r_gen != NULL)
+    BFT_FREE(*p_face_r_gen);
 
   /* Define returned pointers */
 
@@ -884,6 +876,7 @@ _cut_warped_faces(cs_mesh_t      *mesh,
   *p_face_vtx_lst = new_face_vtx_lst;
   *p_face_cells = new_face_cells;
   *p_face_family = new_face_family;
+  *p_face_r_gen = new_face_r_gen;
   *p_face_vtx_connect_size = connect_size;
   *p_n_faces = n_new_faces;
   *p_n_cut_faces = n_cut_faces;
@@ -907,8 +900,8 @@ _cut_warped_faces(cs_mesh_t      *mesh,
 
 static void
 _update_cut_faces_num(cs_mesh_t      *mesh,
-                      cs_int_t        n_faces,
-                      cs_int_t        n_init_faces,
+                      cs_lnum_t       n_faces,
+                      cs_lnum_t       n_init_faces,
                       cs_lnum_t       n_sub_elt_lst[],
                       cs_gnum_t      *n_g_faces,
                       cs_gnum_t     **p_global_face_num)
@@ -972,10 +965,10 @@ _update_cut_faces_num(cs_mesh_t      *mesh,
  *----------------------------------------------------------------------------*/
 
 static void
-_post_before_cutting(cs_int_t        n_i_warp_faces,
-                     cs_int_t        n_b_warp_faces,
-                     cs_int_t        i_warp_face_lst[],
-                     cs_int_t        b_warp_face_lst[],
+_post_before_cutting(cs_lnum_t       n_i_warp_faces,
+                     cs_lnum_t       n_b_warp_faces,
+                     cs_lnum_t       i_warp_face_lst[],
+                     cs_lnum_t       b_warp_face_lst[],
                      double          i_face_warping[],
                      double          b_face_warping[])
 {
@@ -985,7 +978,7 @@ _post_before_cutting(cs_int_t        n_i_warp_faces,
   fvm_nodal_t  *fvm_mesh = NULL;
   fvm_writer_t  *writer = NULL;
 
-  const cs_int_t  writer_id = -1; /* default writer */
+  const cs_lnum_t  writer_id = -1; /* default writer */
   const void  *var_ptr[2] = {NULL, NULL};
 
   parent_num_shift[0] = 0;
@@ -1044,15 +1037,15 @@ _post_before_cutting(cs_int_t        n_i_warp_faces,
  *----------------------------------------------------------------------------*/
 
 static void
-_post_after_cutting(cs_int_t       n_i_cut_faces,
-                    cs_int_t       n_b_cut_faces,
-                    cs_int_t       i_cut_face_lst[],
-                    cs_int_t       b_cut_face_lst[])
+_post_after_cutting(cs_lnum_t   n_i_cut_faces,
+                    cs_lnum_t   n_b_cut_faces,
+                    cs_lnum_t   i_cut_face_lst[],
+                    cs_lnum_t   b_cut_face_lst[])
 {
   fvm_nodal_t  *fvm_mesh = NULL;
   fvm_writer_t  *writer = NULL;
 
-  const cs_int_t  writer_id = -1; /* default writer */
+  const int  writer_id = -1; /* default writer */
 
   if (cs_post_writer_exists(writer_id) == false)
     return;
@@ -1078,33 +1071,6 @@ _post_after_cutting(cs_int_t       n_i_cut_faces,
 
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
-/*============================================================================
- * Public function definitions for Fortran API
- *============================================================================*/
-
-/*----------------------------------------------------------------------------
- * Set the threshold to cut warped faces.
- *
- * Fortran interface :
- *
- * subroutine setcwf (cwfthr)
- * *****************
- *
- * integer          cwfpst      : <-> : if 1, activate postprocessing when
- *                                      cutting warped faces (default 0)
- * double precision cwfthr      : <-> : threshold angle (in degrees) if
- *                                      positive, do not cut warped faces
- *                                      if negative (default -1)
- *----------------------------------------------------------------------------*/
-
-void
-CS_PROCF (setcwf, SETCWF) (const cs_int_t   *cwfpst,
-                           const cs_real_t  *cwfthr)
-{
-  cs_mesh_warping_set_defaults((double)(*cwfthr),
-                               (int)(*cwfpst));
-}
-
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -1125,18 +1091,16 @@ cs_mesh_warping_cut_faces(cs_mesh_t  *mesh,
                           double      max_warp_angle,
                           bool        post_flag)
 {
-  cs_int_t  i;
-
-  cs_int_t  n_i_cut_faces = 0, n_b_cut_faces = 0;
-  cs_int_t  *i_face_lst = NULL, *b_face_lst = NULL;
+  cs_lnum_t  n_i_cut_faces = 0, n_b_cut_faces = 0;
+  cs_lnum_t  *i_face_lst = NULL, *b_face_lst = NULL;
   cs_real_t  *i_face_normal = NULL, *b_face_normal = NULL;
   double  *working_array = NULL, *i_face_warping = NULL, *b_face_warping = NULL;
   cs_lnum_t  *n_i_sub_elt_lst = NULL, *n_b_sub_elt_lst = NULL;
   cs_gnum_t  n_g_faces_ini = 0;
   cs_gnum_t  n_g_i_cut_faces = 0, n_g_b_cut_faces = 0;
 
-  const cs_int_t  n_init_i_faces = mesh->n_i_faces;
-  const cs_int_t  n_init_b_faces = mesh->n_b_faces;
+  const cs_lnum_t  n_init_i_faces = mesh->n_i_faces;
+  const cs_lnum_t  n_init_b_faces = mesh->n_b_faces;
 
 #if 0   /* DEBUG */
   cs_mesh_dump(mesh);
@@ -1150,7 +1114,7 @@ cs_mesh_warping_cut_faces(cs_mesh_t  *mesh,
 
   BFT_MALLOC(working_array, n_init_i_faces + n_init_b_faces, double);
 
-  for (i = 0; i < n_init_i_faces + n_init_b_faces; i++)
+  for (cs_lnum_t i = 0; i < n_init_i_faces + n_init_b_faces; i++)
     working_array[i] = 0.;
 
   i_face_warping = working_array;
@@ -1183,18 +1147,11 @@ cs_mesh_warping_cut_faces(cs_mesh_t  *mesh,
 
   /* Define the global number of faces which need to be cut */
 
+  n_g_i_cut_faces = n_i_cut_faces;
+  n_g_b_cut_faces = n_b_cut_faces;
   if (mesh->n_domains > 1) {
-#if defined(HAVE_MPI)
-    MPI_Allreduce(&n_i_cut_faces, &n_g_i_cut_faces, 1, CS_MPI_INT,
-                  MPI_SUM, cs_glob_mpi_comm);
-
-    MPI_Allreduce(&n_b_cut_faces, &n_g_b_cut_faces, 1, CS_MPI_INT,
-                  MPI_SUM, cs_glob_mpi_comm);
-#endif
-  }
-  else {
-    n_g_i_cut_faces = n_i_cut_faces;
-    n_g_b_cut_faces = n_b_cut_faces;
+    cs_parall_counter(&n_g_i_cut_faces, 1);
+    cs_parall_counter(&n_g_b_cut_faces, 1);
   }
 
   /* Test if there are faces to cut to continue */
@@ -1236,6 +1193,7 @@ cs_mesh_warping_cut_faces(cs_mesh_t  *mesh,
                       &mesh->i_face_vtx_connect_size,
                       (cs_lnum_t **)(&mesh->i_face_cells),
                       &mesh->i_face_family,
+                      &mesh->i_face_r_gen,
                       &mesh->i_face_vtx_idx,
                       &mesh->i_face_vtx_lst);
 
@@ -1278,6 +1236,7 @@ cs_mesh_warping_cut_faces(cs_mesh_t  *mesh,
                     &mesh->b_face_vtx_connect_size,
                     &mesh->b_face_cells,
                     &mesh->b_face_family,
+                    NULL,
                     &mesh->b_face_vtx_idx,
                     &mesh->b_face_vtx_lst);
 
@@ -1315,7 +1274,7 @@ cs_mesh_warping_cut_faces(cs_mesh_t  *mesh,
 
   /* Set mesh modification flag */
 
-  mesh->modified = 1;
+  mesh->modified |= CS_MESH_MODIFIED;
 }
 
 /*----------------------------------------------------------------------------
