@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,6 +45,9 @@
 #include "bft_mem.h"
 #include "bft_error.h"
 #include "bft_printf.h"
+#include "cs_log.h"
+#include "cs_physical_constants.h"
+#include "cs_physical_model.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -61,8 +64,8 @@ BEGIN_C_DECLS
  *============================================================================*/
 
 /*!
-  \file cs_physical_model.c
-        Specific physical models selection.
+  \file cs_combustion_model.c
+        Combustion  model selection parameters.
 */
 
 /*----------------------------------------------------------------------------*/
@@ -95,6 +98,10 @@ cs_combustion_model_t
                                .rosoot = 0.},
                        .coal = {.nclacp = 0},
                        .fuel = {.nclafu = 0},
+                       .n_gas_el_comp = 0,
+                       .n_gas_species = 0,
+                       .n_atomic_species = 0,
+                       .n_reactions = 0,
                        .isoot = -1,
                        .ckabs0 = 0,
                        .xco2 = -1,
@@ -114,7 +121,11 @@ void
 cs_f_combustion_model_get_pointers(int  **isoot);
 
 void
-cs_f_ppthch_get_pointers(int     **iic,
+cs_f_ppthch_get_pointers(int     **ngaze,
+                         int     **ngazg,
+                         int     **nato,
+                         int     **nrgaz,
+                         int     **iic,
                          double  **wmole,
                          double  **wmolg,
                          double  **xco2,
@@ -130,7 +141,9 @@ cs_f_coincl_get_pointers(double  **coefeg,
 void
 cs_f_cpincl_get_pointers(int     **ico2,
                          int     **ih2o,
+                         int     **ncharb,
                          int     **nclacp,
+                         int     **nclpch,
                          int     **ichcor,
                          double  **xashch,
                          double  **diam20,
@@ -158,7 +171,7 @@ cs_f_fuel_get_pointers(int     **nclafu);
  * enables mapping to Fortran global pointers.
  *
  * parameters:
- *   isoot  --> pointer to issot in combustion model
+ *   isoot  --> pointer to isoot in combustion model
  *----------------------------------------------------------------------------*/
 
 void
@@ -174,6 +187,9 @@ cs_f_combustion_model_get_pointers(int  **isoot)
  * enables mapping to Fortran global pointers.
  *
  * parameters:
+ *   ngaze  --> pointer to number of elementary species
+ *   ngazg  --> pointer to number of global species
+ *   nato   --> pointer to number of atomic species
  *   iic    --> pointer to rank of C in gas composition
  *   wmole  --> pointer to molar mass of elementary gas components
  *   wmolg  --> pointer to molar mass of global species
@@ -183,13 +199,22 @@ cs_f_combustion_model_get_pointers(int  **isoot)
  *----------------------------------------------------------------------------*/
 
 void
-cs_f_ppthch_get_pointers(int     **iic,
+cs_f_ppthch_get_pointers(int     **ngaze,
+                         int     **ngazg,
+                         int     **nato,
+                         int     **nrgaz,
+                         int     **iic,
                          double  **wmole,
                          double  **wmolg,
                          double  **xco2,
                          double  **xh2o,
                          double  **ckabs1)
 {
+  *ngaze  = &(cs_glob_combustion_model->n_gas_el_comp);
+  *ngazg  = &(cs_glob_combustion_model->n_gas_species);
+  *nato   = &(cs_glob_combustion_model->n_atomic_species);
+  *nrgaz  = &(cs_glob_combustion_model->n_reactions);
+
   *iic    = &(cs_glob_combustion_model->gas.iic);
   *wmole  = cs_glob_combustion_model->wmole;
   *wmolg  = cs_glob_combustion_model->gas.wmolg;
@@ -231,6 +256,7 @@ cs_f_coincl_get_pointers(double  **coefeg,
  *   ico2   --> pointer to cs_glob_combustion_model->ico2
  *   ih2o   --> pointer to cs_glob_combustion_model->ih2o
  *   nclacp --> pointer to cs_glob_combustion_model->coal.nclacp
+ *   nclacp --> pointer to cs_glob_combustion_model->coal.n_classes_per_coal
  *   ichcor --> pointer to cs_glob_combustion_model->coal.ichcor
  *   xashch --> pointer to cs_glob_combustion_model->coal.xashch
  *   diam20 --> pointer to cs_glob_combustion_model->coal.diam20
@@ -244,7 +270,9 @@ cs_f_coincl_get_pointers(double  **coefeg,
 void
 cs_f_cpincl_get_pointers(int     **ico2,
                          int     **ih2o,
+                         int     **ncharb,
                          int     **nclacp,
+                         int     **nclpch,
                          int     **ichcor,
                          double  **xashch,
                          double  **diam20,
@@ -256,7 +284,9 @@ cs_f_cpincl_get_pointers(int     **ico2,
 {
   *ico2   = &(cs_glob_combustion_model->ico2);
   *ih2o   = &(cs_glob_combustion_model->ih2o);
+  *ncharb = &(cs_glob_combustion_model->coal.n_coals);
   *nclacp = &(cs_glob_combustion_model->coal.nclacp);
+  *nclpch = &(cs_glob_combustion_model->coal.n_classes_per_coal);
   *ichcor = cs_glob_combustion_model->coal.ichcor;
   *xashch = cs_glob_combustion_model->coal.xashch;
   *diam20 = cs_glob_combustion_model->coal.diam20;
@@ -283,6 +313,65 @@ cs_f_fuel_get_pointers(int     **nclafu)
   *nclafu = &(cs_glob_combustion_model->fuel.nclafu);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print the combustion module options to setup.log.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_combustion_log_setup(void)
+{
+  if (  cs_glob_physical_model_flag[CS_COMBUSTION_3PT] >= 0
+      || cs_glob_physical_model_flag[CS_COMBUSTION_EBU] >= 0
+      || cs_glob_physical_model_flag[CS_COMBUSTION_LW]  >= 0) {
+
+    cs_log_printf(CS_LOG_SETUP,
+                  _("\n"
+                    "Combustion module options\n"
+                    "-------------------------\n\n"));
+
+    switch(cs_glob_combustion_model->isoot) {
+    case -1:
+      /* No Soot model */
+      cs_log_printf(CS_LOG_SETUP,
+                    _("    isoot:    -1 (No Soot model)\n\n"));
+      break;
+    case 0:
+      /* constant fraction of product Xsoot */
+      cs_log_printf(CS_LOG_SETUP,
+                    _("    isoot:     0 (Constant soot yield)\n\n"));
+      cs_log_printf(CS_LOG_SETUP,
+                    _("  Parameters for the soot model:\n"
+                      "    xsoot:  %14.5e (Fraction of product - Used only if\n"
+                      "            the soot yield is not defined in the\n"
+                      "            thermochemistry data file)\n"
+                      "    rosoot: %14.5e (Soot density)\n\n"),
+                    cs_glob_combustion_model->gas.xsoot,
+                    cs_glob_combustion_model->gas.rosoot);
+      break;
+    case 1:
+      /* 2 equations model of Moss et al. */
+      cs_log_printf
+        (CS_LOG_SETUP,
+         _("    isoot:     1 (2 equations model of Moss et al.)\n\n"));
+      cs_log_printf(CS_LOG_SETUP,
+                    _("  Parameter for the soot model:\n"
+                      "    rosoot: %14.5e (Soot density)\n\n"),
+                      cs_glob_combustion_model->gas.rosoot);
+      break;
+    default:
+      break;
+    }
+
+    const char *ipthrm_value_str[] = {N_("0 (no mean pressure computation)"),
+                                      N_("1 (mean pressure computation)")};
+    cs_log_printf(CS_LOG_SETUP,
+                  _("    ipthrm:    %s\n"),
+                  _(ipthrm_value_str[cs_glob_fluid_properties->ipthrm]));
+
+  }
+}
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*=============================================================================
