@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -108,8 +108,10 @@ module ppincl
   !>      - ippmod(ieljou) = 4 use of complex potential and specific boundary conditions
   !>  for transformers.
   !>      - ippmod(ieljou) =-1 module not activated
-  !>   - Compressible module: indicator ippmod(icompf)
-  !>      - ippmod(icompf) = 0 module activated
+  !>   - compressible flow module: indicator ippmod(icompf)
+  !>      - ippmod(icompf) = 2 module activated: homogeneous two phase model
+  !>      - ippmod(icompf) = 1 module activated: single phase model
+  !>      - ippmod(icompf) = 0 module activated: single phase barotropic model
   !>      - ippmod(icompf) =-1 module not activated
   !>   - atmospheric flow module: indicator ippmod(iatmos)
   !>      - ippmod(iatmos) =-1 module not activated
@@ -202,8 +204,10 @@ module ppincl
   integer ::  icfuel
 
   !> pointer to specify compressible module with indicator ippmod(icompf)
-  !> - ippmod(icompf) = 0 module activated
-  !> - ippmod(icompf) =-1 module not activated
+  !>      - ippmod(icompf) = 2 module activated: homogeneous two phase model
+  !>      - ippmod(icompf) = 1 module activated: single phase model
+  !>      - ippmod(icompf) = 0 module activated: single phase barotropic model
+  !>      - ippmod(icompf) =-1 module not activated
   integer ::  icompf
 
   !> pointer to specify atmospheric flow module with indicator ippmod(iatmos)
@@ -437,9 +441,6 @@ module ppincl
   !>  - iym1(7): mass fraction of \f$N_2\f$ in the gas mixture
   integer, save :: iym1(ngazem)
 
-  !> temperature of the gas mixture
-  integer, save :: itemp1
-
   ! TODO absent de la doc utilisateur
   !> State variables of continuous phase (gas mixture)
   integer, save :: irom1
@@ -555,6 +556,25 @@ module ppincl
   !> temperature deduced from the specific total energy
   integer, save :: itempk
 
+  !> \defgroup comp_homogeneous Homogeneous two-phase flow model
+
+  !> \addtogroup comp_homogeneous
+  !> \{
+
+  !> \anchor ifracv
+  !> homogeneous model, volume fraction \f$ \alpha \f$
+  integer, save :: ifracv
+
+  !> \anchor ifracm
+  !> homogeneous model, mass fraction \f$ y \f$
+  integer, save :: ifracm
+
+  !> \anchor ifrace
+  !> homogeneous model, energy fraction \f$ z \f$
+  integer, save :: ifrace
+
+  !> \}
+
   !> \defgroup comp_properties Physical properties
 
   !> \addtogroup comp_properties
@@ -564,7 +584,7 @@ module ppincl
   !>  - 0 indicates that the volume viscosity is constant and equal to
   !> the reference volume viscosity \ref viscv0.
   !>  - 1 indicates that the volume viscosity is variable: its
-  !> variation law must be specified in the user subroutine \ref usphyv.
+  !> variation law must be specified in the user subroutine \ref cs_user_physical_properties.
   !>
   !> Always useful.
   !> The volume viscosity \f$\kappa\f$ is defined by the formula expressing the stress:
@@ -593,11 +613,8 @@ module ppincl
   !> alias for boundary conditions
   integer, save :: irunh
 
-  !> reference volume viscosity (noted \f$\kappa\f$ in the equation
-  !> expressing \f$\tens{\sigma}\f$ in the paragraph dedicated to \ref iviscv)
-  !> always useful, it is the used value, unless the user specifies the volume
-  !> viscosity in the user subroutine \ref usphyv
-  double precision, save :: viscv0
+  !> reference volume viscosity
+  real(c_double), pointer, save :: viscv0
 
   !> pressure predicion by an evolution equation
   integer, save :: ippred
@@ -656,9 +673,6 @@ module ppincl
   !> \addtogroup cool_transported
   !> \{
 
-  !> Pointer to define the mass fraction of the water in array isca(iymw)
-  integer, save :: iymw
-
   !> Pointer to define mass fraction of liquid water which is injected in the packing
   integer, save :: iyml
 
@@ -763,6 +777,16 @@ module ppincl
 
     !> \cond DOXYGEN_SHOULD_SKIP_THIS
 
+    ! Interface to C function retrieving pointers to members of the
+    ! global fluid properties structure
+
+    subroutine cs_f_fluid_properties_pp_get_pointers(viscv0)   &
+      bind(C, name='cs_f_fluid_properties_pp_get_pointers')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      type(c_ptr), intent(out) :: viscv0
+    end subroutine cs_f_fluid_properties_pp_get_pointers
+
     !---------------------------------------------------------------------------
 
     ! Interface to C function retrieving pointers to members of the
@@ -811,7 +835,10 @@ contains
 
     ! Local variables
 
-    type(c_ptr) :: p_ippmod, p_isoot
+    type(c_ptr) :: p_viscv0, p_ippmod, p_isoot
+
+    call cs_f_fluid_properties_pp_get_pointers(p_viscv0)
+    call c_f_pointer(p_viscv0, viscv0)
 
     call cs_f_physical_model_get_pointers(p_ippmod)
     call cs_f_combustion_model_get_pointers(p_isoot)
