@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,27 +20,16 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine ppinii
-!================
-
-
-!===============================================================================
-!  FONCTION  :
-!  ---------
-
-! INITIALISATION PAR DEFAUT DES COMMONS PHYSIQUE PARTICULIERE
-
 !-------------------------------------------------------------------------------
-! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!__________________!____!_____!________________________________________________!
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
+!> \file ppinii.f90
+!> \brief Default initialization of specific modules
+!> (only non-map fortran common variables of modules)
+!>
+!------------------------------------------------------------------------------
+
+subroutine ppinii
+
 !===============================================================================
 
 !===============================================================================
@@ -59,8 +48,11 @@ use cs_fuel_incl
 use ppincl
 use ppcpfu
 use atincl
+use atimbr
 use atchem
-use siream
+use atsoil
+use field
+use sshaerosol
 
 !===============================================================================
 
@@ -69,7 +61,7 @@ implicit none
 ! Local variables
 
 integer         icla, icha, igg, it, ir, ih, if, izone
-integer         isol, ige, iat , iii
+integer         isol, ige, iat
 integer         idirac
 
 
@@ -157,7 +149,6 @@ do icla = 1, nclcpm
 enddo
 !
 !       Variables algebriques ou d'etat
-itemp1 = 0
 do ige = 1, ngazem
   iym1(ige) = 0
 enddo
@@ -203,7 +194,6 @@ itaire  = 0
 
 !       Variables algebriques ou d'etat
 
-itemp1 = 0
 do ige = 1, ngazem
   iym1(ige) = 0
 enddo
@@ -237,12 +227,24 @@ ieqnox = 1
 imdnox = 0
 irb = 0
 
-ieqco2 = 0
+! Kinetic model
+! solve transport equation of CO2 mass fraction by default
+! (for coal or fuel combustion)
+ieqco2 = 1
 
 ! ---> Coefficient de relation de la masse volumique
 !      RHO(n+1) = SRROM * RHO(n) + (1-SRROM) * RHO(n+1)
 srrom = 1.d0
 
+! Initialization for compressible module
+
+! Standard compressible module scalars
+ienerg = 0
+itempk = 0
+! Compressible homogeneous two-phase flow scalars
+ifracv = 0
+ifracm = 0
+ifrace = 0
 
 !===============================================================================
 ! 2. REMPLISSAGE INCLUDE ppthch.h
@@ -253,10 +255,6 @@ srrom = 1.d0
 ! ---> Initialisation Common / TCHPPI /
 
 npo   = 0
-ngaze = 0
-ngazg = 0
-nato  = 0
-nrgaz = 0
 
 ! ---> Initialisation Common / TCHPPR /
 
@@ -637,52 +635,114 @@ do izone = 1, nozppm
 enddo
 
 !===============================================================================
-! 6. REMPLISSAGE INCLUDE atincl.h
-!                INCLUDE POUR LA VERSION ATMOSPHERIQUE
+! 6. Global variables for atmospheric flows (module atincl.f90)
 !===============================================================================
 
-!--> Initialisation for the meteo profile
-imeteo = 0
-nbmetd = 0
-nbmett = 0
-nbmetm = 0
-nbmaxt = 0
-do izone = 1, nozppm
-  iprofm(izone) = 0
-enddo
-ihpm=0
+! Space and time reference of the run:
+! ------------------------------------
 
-!--> Initialisation for the time and space reference of the run
-syear=-999
-squant=-999
-shour=-999
-smin=-999
-ssec=-999
+! Option for the meteo profile computation
+!ihpm   --> flag to compute the hydrostastic pressure by Laplace integration
+!           in the meteo profiles
+!       = 0 : bottom to top Laplace integration, based on P(sea level) (default)
+!       = 1 : top to bottom Laplace integration based on P computed for
+!            the standard atmosphere at z(nbmaxt)
+ihpm = 0
 
-xlon=rinfin
-xlat=rinfin
+! 1d radiative transfer model:
+! ----------------------------
 
-! --> Initialisation for the 1d radiative model:
+! iatra1 -->  flag for the use of the 1d atmo radiative model
+! nfatr1 --> 1d radiative model pass frequency
+! ivert  --> flag for the definition of the vertical grid
+!            -1: no vertical grid
+!            0 : automatic definition !!!!!!!!!MM 2do
+!            1 : definition by the user in usatdv
+! iqv0   --> flag for humidity above the domain (0 : no humidity; 1 : decreasing)
 
 iatra1 = 0
 nfatr1 = 1
+ivert = 1  ! if iatra1=1 then ivert=1
+iqv0 = 0
+
+! --> Initialisation for the 1d radiative model:
 nvert = 1
 kvert = 20
-ivert = 0
-iqv0 = 0
+
+! flag to use the soil model (if humid atmosphere)
 iatsoil = 0
+! Initial values for soil variables
+tsini  = 20.d0   !TSINI  : Surface ground temperature
+tprini = 20.d0   !TPRINI : Deep ground temperature
+qvsini = 0.d0    !QVSINI : Ground humidity
+tmer   = 20.d0   !Sea temperature
+
+!  -------------------------------------------------------------------------------
+!  Microphysics parameterization options
+!  -------------------------------------------------------------------------------
+
+! logaritmic standard deviation of the log-normal law of the droplet spectrum
+! adimensional
+sigc = 0.53 ! other referenced values are 0.28, 0.15
+
+!  -----------------------------------------------------------------------------
+!  Atmospheric imbrication on large scale meteo (atimbr module)
+!  -----------------------------------------------------------------------------
+
+! activation flag
+imbrication_flag = .false.
+imbrication_verbose = .false.
+
+! ------------------------------------------------------------------------------
+! flags for activating the cressman interpolation for the boundary conditions
+! ------------------------------------------------------------------------------
+cressman_u = .false.
+cressman_v = .false.
+cressman_tke = .false.
+cressman_eps = .false.
+cressman_theta = .false.
+cressman_qw = .false.
+cressman_nc = .false.
+
+! --------------------------------------------------------------
+! numerical parameters for the cressman interpolation formulas
+! --------------------------------------------------------------
+horizontal_influence_radius = 8500.d0
+vertical_influence_radius = 100.d0
+
+! key id for optimal interpolation
+
+call field_get_key_id("opt_interp_id", kopint)
+
+! -------------------------------------
+! flags for 1d radiative transfer model
+! -------------------------------------
+
+! no computation / storage of downward and upward infrared radiative fluxes
+irdu = 1
+
+! initmeteo --> use meteo profile for variables initialization
+!               (0: not used; 1: used )
+! NB : should eventually be set by interface and by zone (as BCs)
+
+initmeteo = 1
+
+theo_interp = 0
+
+!--> Initialisation for the meteo profile
+
+do izone = 1, nozppm
+  iprofm(izone) = 0
+enddo
 
 ! --> Initialisation for the chemistry models:
 
 init_at_chem = 1
 
 ! --> Initialisation for the gaseous chemistry model:
-ichemistry = 0
+
 ifilechemistry = 0
 isepchemistry = 2
-iphotolysis = 1
-nespg = 0
-nrg = 0
 nbchim = 0
 nbchmz = 0
 nespgi = 0
@@ -691,44 +751,22 @@ do izone = 1, nozppm
   iprofc(izone) = 0
 enddo
 
-
 ! --> Initialisation for the aerosol chemistry model:
-iaerosol = 0
-inogaseouschemistry = 0
-nespg_siream = 0
-! Number of iterations for time splitting
-ncycle_aer = 1
-! Flag to activate or not coagulation (1=yes, 0=no)
-icoag_siream = 1
-! Flag to activate or not condensation/evaporation (1=yes, 0=no)
-icond_siream = 1
-! Flag to activate or not nucleation (1=yes, 0=no)
-inucl_siream = 1
-! Flag to consider or not kelvin effect (1=yes, 0=no)
-ikelv_siream = 1
-! Cutting bin between equilibrium (1 to icut_siream)
-! and dynamic bins (icut_siream to nbin_aer)
-icut_siream = nbin_aer
-! Sulfate condensation computation method (1=dynamic, 0=equilibrium)
-isulfcond_siream = 1
-! Solver for dynamic condensation (1='etr', 2='ros2', 3='ebi')
-kdslv_siream = 2
-! Redistribution method of lagrangian bins
-! (1=number conserving, 2=interpolation)
-iredist_siream = 1
-! Nucleation model (0=binary, 1=ternary)
-itern_siream = 0
-! Method used for estimation of wet diameter (0=isoropia, 1=gerber)
-ithrm_siream = 1
-! m
-bin_bound_aer = (/0.01d-6, 0.0398d-6, 0.1585d-6, 0.6310d-6, 2.5119d-6, 10.d-6/)
-fixed_density_aer = 1400.0d0 ! kg/m3
-do iii = 1, nbin_aer
-  density_aer(iii) = fixed_density_aer
-enddo
+
 do izone = 1, nozppm
   iprofa(izone) = 0
 enddo
 
+! Default values (climatic ones) for radiative transfer and
+! aerosols
+aod_o3_tot=0.20d0
+aod_h2o_tot=0.10d0
+gaero_o3=0.66d0
+gaero_h2o=0.64d0
+piaero_o3=0.84d0
+piaero_h2o=0.84d0
+black_carbon_frac=0.d0
+zaero = 6000d0
+
 return
 end subroutine ppinii
