@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -59,7 +59,6 @@ use cs_coal_incl
 use cs_fuel_incl
 use ppcpfu
 use atincl
-use ihmpre
 use field
 use cs_c_bindings
 
@@ -185,7 +184,7 @@ endif
 !------------------------
 
 if (ippmod(iaeros).ge.0) then
-  call ctvarp(ippmod(iatmos))
+  call ctvarp
 endif
 
 ! 8. Gas mixtures modelling
