@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -106,16 +106,18 @@ BEGIN_C_DECLS
  * For the P-1 model, this function also checks whether the medium's optical
  * length is at least of the order of unity.
  *
- * \param[in]   tempk  gas phase temperature at cells (in Kelvin)
- * \param[out]  kgas   radiation coefficients of the gray gases at cells
- *                      (per gas)
- * \param[out]  agas   weights of the gray gases at cells (per gas)
- * \param[out]  agasb  weights of the gray gases at boundary faces (per gas)
+ * \param[in]   tempk      gas phase temperature at cells (in Kelvin)
+ * \param[out]  cpro_cak0  medium (gas) Absorption coefficient
+ * \param[out]  kgas       radiation coefficients of the gray gases at cells
+ *                         (per gas)
+ * \param[out]  agas       weights of the gray gases at cells (per gas)
+ * \param[out]  agasb      weights of the gray gases at boundary faces (per gas)
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_rad_transfer_absorption(const cs_real_t  tempk[],
+                           cs_real_t        cpro_cak0[],
                            cs_real_t        kgas[],
                            cs_real_t        agas[],
                            cs_real_t        agasb[])
@@ -142,8 +144,6 @@ cs_rad_transfer_absorption(const cs_real_t  tempk[],
 
   cs_real_t *crom = CS_F_(rho)->val;
 
-  cs_real_t *cpro_cak0 = CS_FI_(rad_cak, 0)->val;
-
   /* Absorption coefficient of gas mix (m-1)
      --------------------------------------- */
 
@@ -163,7 +163,12 @@ cs_rad_transfer_absorption(const cs_real_t  tempk[],
       if (cm->isoot >= 1)
         cvar_fsm = CS_F_(fsm)->val;
 
-      const cs_real_t *cpro_temp = CS_F_(t)->val;
+      // If we are in multiphase, we get the first temperature field
+      const cs_real_t *cpro_temp;
+      if (CS_F_(t) != NULL)
+        cpro_temp = CS_F_(t)->val;
+      else
+        cpro_temp = CS_FI_(t,0)->val;
 
       const cs_real_t *cpro_ym1 = cs_field_by_name("ym_fuel")->val;
       const cs_real_t *cpro_ym2 = cs_field_by_name("ym_oxyd")->val;
@@ -210,13 +215,13 @@ cs_rad_transfer_absorption(const cs_real_t  tempk[],
 
   /* Coal or fuel combustion */
 
-  else if (   pm_flag[CS_COMBUSTION_COAL] >=  0
+  else if (   pm_flag[CS_COMBUSTION_COAL] >= 0
            || pm_flag[CS_COMBUSTION_FUEL] >= 0) {
 
-    cs_real_t *cpro_temp1 = cs_field_by_name("t_gas")->val;
-    cs_real_t *cpro_yco2  = cs_field_by_name("ym_co2")->val;
-    cs_real_t *cpro_yh2o  = cs_field_by_name("ym_h2o")->val;
-    cs_real_t *cpro_mmel  = cs_field_by_name("xm")->val;
+    cs_real_t *cpro_temp = cs_field_by_name("temperature")->val;
+    cs_real_t *cpro_yco2 = cs_field_by_name("ym_co2")->val;
+    cs_real_t *cpro_yh2o = cs_field_by_name("ym_h2o")->val;
+    cs_real_t *cpro_mmel = cs_field_by_name("xm")->val;
 
     const double *restrict wmole = cm->wmole;
     const int ico2 = cm->ico2 - 1;
@@ -236,7 +241,7 @@ cs_rad_transfer_absorption(const cs_real_t  tempk[],
       }
 
       if (rt_params->imodak == 1)
-        cs_rad_transfer_modak(cpro_cak0, w1, w2, w3, cpro_temp1);
+        cs_rad_transfer_modak(cpro_cak0, w1, w2, w3, cpro_temp);
 
       else if (rt_params->imoadf == 1)
         cs_rad_transfer_adf08(w1, w2, tempk, kgas, agas, agasb);
@@ -433,7 +438,7 @@ cs_rad_transfer_absorption_check_p1(const cs_real_t  cpro_cak[])
 
   cs_rad_transfer_params_t *rt_params = cs_glob_rad_transfer_params;
 
-  cs_real_t s[3] = {0, 0, 0};
+  cs_real_t s[2] = {0, 0};
 
   /* Compute the characteristic length of the computational domain */
 
@@ -476,9 +481,9 @@ cs_rad_transfer_absorption_check_p1(const cs_real_t  cpro_cak[])
            "   this optical length is xkmin = %11.4e.\n"
            "   This value is not reached for %11.4e%% of mesh cells.\n\n"
            "   The percentage of cells for which we allow this condition\n"
-           "   is not reached is currently set to:\n"
+           "   not to be reached is currently set to:\n"
            "   \"cs_glob_rad_transfer_params->xnp1mx\" = %11.4e.\n\n"),
-         __func__, xkmin, iok/n_cells*100.,
+         __func__, xkmin, iok/cs_glob_mesh->n_g_cells*100.,
          rt_params->xnp1mx);
     rt_params->iwrp1t += 1;
 
