@@ -4,7 +4,7 @@
 
 /* This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -59,8 +59,6 @@
 #include "cs_thermal_model.h"
 #include "cs_physical_constants.h"
 
-#include "cs_prototypes.h"
-
 #include "cs_rad_transfer.h"
 
 /*----------------------------------------------------------------------------
@@ -86,18 +84,18 @@ BEGIN_C_DECLS
  *============================================================================*/
 
 /* Common statics */
-static cs_int_t ntsto = 0;
-static cs_int_t ipass = 0;
+static int  ntsto = 0;
+static int  ipass = 0;
 static cs_real_t *tsto = NULL;
 static cs_real_t *asto = NULL;
 static cs_real_t *ksto2 = NULL;
 
 /* ADF08 model specifics */
-static cs_int_t nysto = 0;
+static int        nysto = 0;
 static cs_real_t *ysto = NULL;
 
 /* ADF50 model specifics */
-static cs_int_t nxh2osto = 0;
+static int        nxh2osto = 0;
 static cs_real_t *xh2osto = NULL;
 static cs_real_t *ksto1 = NULL;
 
@@ -116,7 +114,7 @@ static cs_real_t *ksto1 = NULL;
 static inline void
 _line_to_array(FILE      *radfile,
                cs_real_t  values[],
-               cs_int_t  *nvalues)
+               int       *nvalues)
 {
   char line[256];
   fgets(line, 256, radfile);
@@ -176,11 +174,10 @@ cs_rad_transfer_adf08(const cs_real_t  pco2[],
   /* Memory allocation and initialization */
 
   cs_real_t *y, *tpaadf;
-  cs_field_t *f_b_temp = NULL;
-  f_b_temp = cs_field_by_name_try("boundary_temperature");
+  cs_field_t *f_b_temp = cs_field_by_name_try("boundary_temperature");
   BFT_MALLOC(y, cs_glob_mesh->n_cells_with_ghosts, cs_real_t);
 
-  if (cs_glob_thermal_model->itpscl == 2) {
+  if (cs_glob_thermal_model->itpscl == CS_TEMPERATURE_SCALE_CELSIUS) {
 
     BFT_MALLOC(tpaadf, nfabor, cs_real_t );
     for (cs_lnum_t ifac = 0; ifac < nfabor; ifac++)
@@ -327,7 +324,7 @@ cs_rad_transfer_adf08(const cs_real_t  pco2[],
 
     /* Absortion Coefficient */
 
-    for (cs_int_t i = 0; i < nwsgg; i++) {
+    for (int i = 0; i < nwsgg; i++) {
       cs_real_t kmloc =    (1.0 - rt) * (1.0 - rx)
                          * ksto2[i + ix * nwsgg + it * nysto * nwsgg]
                        +   (1.0 - rt) * (rx)
@@ -447,10 +444,9 @@ cs_rad_transfer_adf50(const cs_real_t  pco2[],
   /* Memory allocation and initialization */
 
   cs_real_t *tpaadf;
-  cs_field_t *f_b_temp = NULL;
-  f_b_temp = cs_field_by_name_try("boundary_temperature");
+  cs_field_t *f_b_temp = cs_field_by_name_try("boundary_temperature");
 
-  if (cs_glob_thermal_model->itpscl == 2) {
+  if (cs_glob_thermal_model->itpscl == CS_TEMPERATURE_SCALE_CELSIUS) {
 
     BFT_MALLOC(tpaadf, nfabor, cs_real_t );
     for (cs_lnum_t ifac = 0; ifac < nfabor; ifac++)
@@ -480,7 +476,7 @@ cs_rad_transfer_adf50(const cs_real_t  pco2[],
     fgets(line, 256, radfile);
     fscanf(radfile, "%d", &ntsto);
 
-    /* Number of tabulated gas phase temperatures    */
+    /* Number of tabulated gas phase temperatures */
     BFT_MALLOC(tsto, ntsto, cs_real_t);
 
     fgets(line, 256, radfile);
@@ -597,7 +593,7 @@ cs_rad_transfer_adf50(const cs_real_t  pco2[],
     }
 
     /* Absortion Coefficient */
-    for (cs_int_t i = 0; i < nwsgg; i++) {
+    for (int i = 0; i < nwsgg; i++) {
       cs_real_t kco2loc =  ksto1[i + it * nwsgg]
                          + rt * (  ksto1[i + (it+1) * nwsgg]
                                  - ksto1[i + it * nwsgg]);
