@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -108,48 +108,45 @@ cs_rad_transfer_bcs(int         nvar,
 /*!
  * \brief Boundary conditions for DO and P-1 models
  *
- * 1. Boundary conditions for the radiative intensity (DO model)
- * --------------------------------------------------------------
- *      The array coefap stores the intensity for each boundary faces,
- *        depending of the natur of the boundary (Dirichlet condition).
- *      The intensity of radiation is defined as the rate of emitted
- *        energy from unit surface area through unit solid angle.
+ *  The coefap array stores the intensity for each boundary face,
+ *  depending of the nature of the boundary (Dirichlet condition).
+ *  The intensity of radiation is defined as the rate of emitted
+ *  energy from unit surface area through a unit solid angle.
  *
  *   1/ Gray wall: isotropic radiation field.
- *                                   4
- *                     eps.sig.twall         (1-eps).qincid
- *       coefap   =    --------------    +    --------------
- *                           pi                     pi
- *   wall intensity     wall emission           reflecting flux.
- *      (eps=1: black wall; eps=0: reflecting wall)
- *   2/ Free boundary: condition to mimic infinite domain
  *
- * 2. Boundary conditions for the P-1 model
- * ----------------------------------------
+ *   \f$ coefap =  \epsilon.\sigma.twall^4 / \pi + (1-\epsilon).qincid / \pi \f$
+ *
+ *   which is the sum of the wall emission and reflecting flux
+ *   (eps=1: black wall; eps=0: reflecting wall).
+ *
+ *   2/ Free boundary: condition to mimic infinite domain
  *
  * \param[in]  bc_type         boundary face types
  * \param[in]  vect_s          direction vector or NULL
+ * \param[in]  ckmel           Absoprtion coefficcient of the mixture
+ *                               gas-particules of coal
+ * \param[in]  bpro_eps        Boundary emissivity, or NULL for solar radiation
+ * \param[in]  w_gg            Weights of the i-th gray gas at boundaries
+ * \param[in]  gg_id           number of the i-th grey gas
  * \param[out] coefap          boundary conditions for intensity or P-1 model
  * \param[out] coefbp          boundary conditions for intensity or P-1 model
  * \param[out] cofafp          boundary conditions for intensity or P-1 model
  * \param[out] cofbfp          boundary conditions for intensity or P-1 model
- * \param[in]  ckmel           coeff d'absorption du melange
- *                               gaz-particules de charbon
- * \param[in]  w_gg            Weights of the i-th gray gas at boundaries
- * \param[in]  gg_id           number of the i-th grey gas
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_rad_transfer_bc_coeffs(int        bc_type[],
                           cs_real_t  vect_s[3],
+                          cs_real_t  ckmel[],
+                          cs_real_t  bpro_eps[],
+                          cs_real_t  w_gg[],
+                          int        gg_id,
                           cs_real_t  coefap[],
                           cs_real_t  coefbp[],
                           cs_real_t  cofafp[],
-                          cs_real_t  cofbfp[],
-                          cs_real_t  ckmel[],
-                          cs_real_t  w_gg[],
-                          int        gg_id);
+                          cs_real_t  cofbfp[]);
 
 /*----------------------------------------------------------------------------*/
 
