@@ -4,7 +4,7 @@
 
 /* This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -97,8 +97,8 @@ const int z = 2;
 static void
 _init_quadrature(void)
 {
-  if (cs_glob_rad_transfer_params->sxyz == NULL)
-    BFT_MALLOC(cs_glob_rad_transfer_params->sxyz,
+  if (cs_glob_rad_transfer_params->vect_s == NULL)
+    BFT_MALLOC(cs_glob_rad_transfer_params->vect_s,
                cs_glob_rad_transfer_params->ndirs,
                cs_real_3_t);
 
@@ -187,15 +187,15 @@ cs_rad_transfer_dir(void)
 
   switch(cs_rp->i_quadrature){
 
-  case 1: /* quadrature S4 : 24 directions  */
+  case CS_RAD_QUADRATURE_S4: /* quadrature S4 : 24 directions  */
     cs_rp->ndirs = 3;
     break;
 
-  case 2: /* quadrature S6 : 48 directions  */
+  case CS_RAD_QUADRATURE_S6: /* quadrature S6 : 48 directions  */
     cs_rp->ndirs = 6;
     break;
 
-  case 3: /* quadrature S8 : 80 directions  */
+  case CS_RAD_QUADRATURE_S8: /* quadrature S8 : 80 directions  */
     cs_rp->ndirs = 10;
     break;
 
@@ -208,7 +208,7 @@ cs_rad_transfer_dir(void)
     break;
 
   case 6: /* Quadrature Tn : 8 n^2 directions    */
-    cs_rp->ndirs = pow(cs_rp->ndirec, 2.0);
+    cs_rp->ndirs = cs_rp->ndirec * cs_rp->ndirec;
     break;
 
   case 7: /* Quadrature 120 directions (LC11)    */
@@ -240,15 +240,15 @@ cs_rad_transfer_dir(void)
     vec[0]    = 0.2958759;
     vec[1]    = 0.9082483;
     weight[0]  = 0.5235987;
-    cs_rp->sxyz[0][0] = vec[0];
-    cs_rp->sxyz[1][0] = vec[0];
-    cs_rp->sxyz[2][0] = vec[1];
-    cs_rp->sxyz[0][1] = vec[0];
-    cs_rp->sxyz[1][1] = vec[1];
-    cs_rp->sxyz[2][1] = vec[0];
-    cs_rp->sxyz[0][2] = vec[1];
-    cs_rp->sxyz[1][2] = vec[0];
-    cs_rp->sxyz[2][2] = vec[0];
+    cs_rp->vect_s[0][0] = vec[0];
+    cs_rp->vect_s[1][0] = vec[0];
+    cs_rp->vect_s[2][0] = vec[1];
+    cs_rp->vect_s[0][1] = vec[0];
+    cs_rp->vect_s[1][1] = vec[1];
+    cs_rp->vect_s[2][1] = vec[0];
+    cs_rp->vect_s[0][2] = vec[1];
+    cs_rp->vect_s[1][2] = vec[0];
+    cs_rp->vect_s[2][2] = vec[0];
     cs_rp->angsol[0]  = weight[0];
     cs_rp->angsol[1]  = weight[0];
     cs_rp->angsol[2]  = weight[0];
@@ -263,24 +263,24 @@ cs_rad_transfer_dir(void)
     weight[0]  = 0.1609517;
     weight[1]  = 0.3626469;
 
-    cs_rp->sxyz[0][0] = vec[0];
-    cs_rp->sxyz[1][0] = vec[0];
-    cs_rp->sxyz[2][0] = vec[0];
-    cs_rp->sxyz[3][0] = vec[1];
-    cs_rp->sxyz[4][0] = vec[1];
-    cs_rp->sxyz[5][0] = vec[2];
-    cs_rp->sxyz[0][1] = vec[0];
-    cs_rp->sxyz[1][1] = vec[1];
-    cs_rp->sxyz[2][1] = vec[2];
-    cs_rp->sxyz[3][1] = vec[0];
-    cs_rp->sxyz[4][1] = vec[1];
-    cs_rp->sxyz[5][1] = vec[0];
-    cs_rp->sxyz[0][2] = vec[2];
-    cs_rp->sxyz[1][2] = vec[1];
-    cs_rp->sxyz[2][2] = vec[0];
-    cs_rp->sxyz[3][2] = vec[1];
-    cs_rp->sxyz[4][2] = vec[0];
-    cs_rp->sxyz[5][2] = vec[0];
+    cs_rp->vect_s[0][0] = vec[0];
+    cs_rp->vect_s[1][0] = vec[0];
+    cs_rp->vect_s[2][0] = vec[0];
+    cs_rp->vect_s[3][0] = vec[1];
+    cs_rp->vect_s[4][0] = vec[1];
+    cs_rp->vect_s[5][0] = vec[2];
+    cs_rp->vect_s[0][1] = vec[0];
+    cs_rp->vect_s[1][1] = vec[1];
+    cs_rp->vect_s[2][1] = vec[2];
+    cs_rp->vect_s[3][1] = vec[0];
+    cs_rp->vect_s[4][1] = vec[1];
+    cs_rp->vect_s[5][1] = vec[0];
+    cs_rp->vect_s[0][2] = vec[2];
+    cs_rp->vect_s[1][2] = vec[1];
+    cs_rp->vect_s[2][2] = vec[0];
+    cs_rp->vect_s[3][2] = vec[1];
+    cs_rp->vect_s[4][2] = vec[0];
+    cs_rp->vect_s[5][2] = vec[0];
     cs_rp->angsol[0]  = weight[0];
     cs_rp->angsol[1]  = weight[1];
     cs_rp->angsol[2]  = weight[0];
@@ -300,36 +300,36 @@ cs_rad_transfer_dir(void)
     weight[1]   = 0.1712359;
     weight[2]   = 0.4617179;
 
-    cs_rp->sxyz[0][0] = vec[0];
-    cs_rp->sxyz[1][0] = vec[0];
-    cs_rp->sxyz[2][0] = vec[0];
-    cs_rp->sxyz[3][0] = vec[0];
-    cs_rp->sxyz[4][0] = vec[1];
-    cs_rp->sxyz[5][0] = vec[1];
-    cs_rp->sxyz[6][0] = vec[1];
-    cs_rp->sxyz[7][0] = vec[2];
-    cs_rp->sxyz[8][0] = vec[2];
-    cs_rp->sxyz[9][0] = vec[3];
-    cs_rp->sxyz[0][1] = vec[0];
-    cs_rp->sxyz[1][1] = vec[1];
-    cs_rp->sxyz[2][1] = vec[2];
-    cs_rp->sxyz[3][1] = vec[3];
-    cs_rp->sxyz[4][1] = vec[0];
-    cs_rp->sxyz[5][1] = vec[1];
-    cs_rp->sxyz[6][1] = vec[2];
-    cs_rp->sxyz[7][1] = vec[0];
-    cs_rp->sxyz[8][1] = vec[1];
-    cs_rp->sxyz[9][1] = vec[0];
-    cs_rp->sxyz[0][2] = vec[3];
-    cs_rp->sxyz[1][2] = vec[2];
-    cs_rp->sxyz[2][2] = vec[1];
-    cs_rp->sxyz[3][2] = vec[0];
-    cs_rp->sxyz[4][2] = vec[2];
-    cs_rp->sxyz[5][2] = vec[1];
-    cs_rp->sxyz[6][2] = vec[0];
-    cs_rp->sxyz[7][2] = vec[1];
-    cs_rp->sxyz[8][2] = vec[0];
-    cs_rp->sxyz[9][2] = vec[0];
+    cs_rp->vect_s[0][0] = vec[0];
+    cs_rp->vect_s[1][0] = vec[0];
+    cs_rp->vect_s[2][0] = vec[0];
+    cs_rp->vect_s[3][0] = vec[0];
+    cs_rp->vect_s[4][0] = vec[1];
+    cs_rp->vect_s[5][0] = vec[1];
+    cs_rp->vect_s[6][0] = vec[1];
+    cs_rp->vect_s[7][0] = vec[2];
+    cs_rp->vect_s[8][0] = vec[2];
+    cs_rp->vect_s[9][0] = vec[3];
+    cs_rp->vect_s[0][1] = vec[0];
+    cs_rp->vect_s[1][1] = vec[1];
+    cs_rp->vect_s[2][1] = vec[2];
+    cs_rp->vect_s[3][1] = vec[3];
+    cs_rp->vect_s[4][1] = vec[0];
+    cs_rp->vect_s[5][1] = vec[1];
+    cs_rp->vect_s[6][1] = vec[2];
+    cs_rp->vect_s[7][1] = vec[0];
+    cs_rp->vect_s[8][1] = vec[1];
+    cs_rp->vect_s[9][1] = vec[0];
+    cs_rp->vect_s[0][2] = vec[3];
+    cs_rp->vect_s[1][2] = vec[2];
+    cs_rp->vect_s[2][2] = vec[1];
+    cs_rp->vect_s[3][2] = vec[0];
+    cs_rp->vect_s[4][2] = vec[2];
+    cs_rp->vect_s[5][2] = vec[1];
+    cs_rp->vect_s[6][2] = vec[0];
+    cs_rp->vect_s[7][2] = vec[1];
+    cs_rp->vect_s[8][2] = vec[0];
+    cs_rp->vect_s[9][2] = vec[0];
     cs_rp->angsol[0] = weight[1];
     cs_rp->angsol[1] = weight[0];
     cs_rp->angsol[2] = weight[0];
@@ -355,18 +355,18 @@ cs_rad_transfer_dir(void)
     weight[0] = 0.5512855984;
     weight[1] = 0.3398369095;
 
-    cs_rp->sxyz[0][0] = vec[0];
-    cs_rp->sxyz[1][0] = vec[1];
-    cs_rp->sxyz[2][0] = vec[2];
-    cs_rp->sxyz[3][0] = vec[0];
-    cs_rp->sxyz[0][1] = vec[0];
-    cs_rp->sxyz[1][1] = vec[0];
-    cs_rp->sxyz[2][1] = vec[2];
-    cs_rp->sxyz[3][1] = vec[1];
-    cs_rp->sxyz[0][2] = vec[1];
-    cs_rp->sxyz[1][2] = vec[0];
-    cs_rp->sxyz[2][2] = vec[2];
-    cs_rp->sxyz[3][2] = vec[0];
+    cs_rp->vect_s[0][0] = vec[0];
+    cs_rp->vect_s[1][0] = vec[1];
+    cs_rp->vect_s[2][0] = vec[2];
+    cs_rp->vect_s[3][0] = vec[0];
+    cs_rp->vect_s[0][1] = vec[0];
+    cs_rp->vect_s[1][1] = vec[0];
+    cs_rp->vect_s[2][1] = vec[2];
+    cs_rp->vect_s[3][1] = vec[1];
+    cs_rp->vect_s[0][2] = vec[1];
+    cs_rp->vect_s[1][2] = vec[0];
+    cs_rp->vect_s[2][2] = vec[2];
+    cs_rp->vect_s[3][2] = vec[0];
     cs_rp->angsol[0] = weight[1];
     cs_rp->angsol[1] = weight[1];
     cs_rp->angsol[2] = weight[0];
@@ -392,54 +392,54 @@ cs_rad_transfer_dir(void)
     weight[3]  = 0.1320249278;
     weight[4]  = 0.155210815;
 
-    cs_rp->sxyz[0][0]  = vec[0];
-    cs_rp->sxyz[1][0]  = vec[1];
-    cs_rp->sxyz[2][0]  = vec[2];
-    cs_rp->sxyz[3][0]  = vec[3];
-    cs_rp->sxyz[4][0]  = vec[4];
-    cs_rp->sxyz[5][0]  = vec[5];
-    cs_rp->sxyz[6][0]  = vec[6];
-    cs_rp->sxyz[7][0]  = vec[7];
-    cs_rp->sxyz[8][0]  = vec[3];
-    cs_rp->sxyz[9][0]  = vec[8];
-    cs_rp->sxyz[10][0] = vec[9];
-    cs_rp->sxyz[11][0] = vec[4];
-    cs_rp->sxyz[12][0] = vec[5];
-    cs_rp->sxyz[13][0] = vec[1];
-    cs_rp->sxyz[14][0] = vec[2];
-    cs_rp->sxyz[15][0] = vec[0];
-    cs_rp->sxyz[0][1]  = vec[0];
-    cs_rp->sxyz[1][1]  = vec[3];
-    cs_rp->sxyz[2][1]  = vec[2];
-    cs_rp->sxyz[3][1]  = vec[1];
-    cs_rp->sxyz[4][1]  = vec[3];
-    cs_rp->sxyz[5][1]  = vec[7];
-    cs_rp->sxyz[6][1]  = vec[6];
-    cs_rp->sxyz[7][1]  = vec[5];
-    cs_rp->sxyz[8][1]  = vec[4];
-    cs_rp->sxyz[9][1]  = vec[0];
-    cs_rp->sxyz[10][1] = vec[2];
-    cs_rp->sxyz[11][1] = vec[1];
-    cs_rp->sxyz[12][1] = vec[5];
-    cs_rp->sxyz[13][1] = vec[4];
-    cs_rp->sxyz[14][1] = vec[9];
-    cs_rp->sxyz[15][1] = vec[8];
-    cs_rp->sxyz[0][2]  = vec[8];
-    cs_rp->sxyz[1][2]  = vec[4];
-    cs_rp->sxyz[2][2]  = vec[9];
-    cs_rp->sxyz[3][2]  = vec[4];
-    cs_rp->sxyz[4][2]  = vec[1];
-    cs_rp->sxyz[5][2]  = vec[5];
-    cs_rp->sxyz[6][2]  = vec[6];
-    cs_rp->sxyz[7][2]  = vec[5];
-    cs_rp->sxyz[8][2]  = vec[1];
-    cs_rp->sxyz[9][2]  = vec[0];
-    cs_rp->sxyz[10][2] = vec[2];
-    cs_rp->sxyz[11][2] = vec[3];
-    cs_rp->sxyz[12][2] = vec[7];
-    cs_rp->sxyz[13][2] = vec[3];
-    cs_rp->sxyz[14][2] = vec[2];
-    cs_rp->sxyz[15][2] = vec[0];
+    cs_rp->vect_s[0][0]  = vec[0];
+    cs_rp->vect_s[1][0]  = vec[1];
+    cs_rp->vect_s[2][0]  = vec[2];
+    cs_rp->vect_s[3][0]  = vec[3];
+    cs_rp->vect_s[4][0]  = vec[4];
+    cs_rp->vect_s[5][0]  = vec[5];
+    cs_rp->vect_s[6][0]  = vec[6];
+    cs_rp->vect_s[7][0]  = vec[7];
+    cs_rp->vect_s[8][0]  = vec[3];
+    cs_rp->vect_s[9][0]  = vec[8];
+    cs_rp->vect_s[10][0] = vec[9];
+    cs_rp->vect_s[11][0] = vec[4];
+    cs_rp->vect_s[12][0] = vec[5];
+    cs_rp->vect_s[13][0] = vec[1];
+    cs_rp->vect_s[14][0] = vec[2];
+    cs_rp->vect_s[15][0] = vec[0];
+    cs_rp->vect_s[0][1]  = vec[0];
+    cs_rp->vect_s[1][1]  = vec[3];
+    cs_rp->vect_s[2][1]  = vec[2];
+    cs_rp->vect_s[3][1]  = vec[1];
+    cs_rp->vect_s[4][1]  = vec[3];
+    cs_rp->vect_s[5][1]  = vec[7];
+    cs_rp->vect_s[6][1]  = vec[6];
+    cs_rp->vect_s[7][1]  = vec[5];
+    cs_rp->vect_s[8][1]  = vec[4];
+    cs_rp->vect_s[9][1]  = vec[0];
+    cs_rp->vect_s[10][1] = vec[2];
+    cs_rp->vect_s[11][1] = vec[1];
+    cs_rp->vect_s[12][1] = vec[5];
+    cs_rp->vect_s[13][1] = vec[4];
+    cs_rp->vect_s[14][1] = vec[9];
+    cs_rp->vect_s[15][1] = vec[8];
+    cs_rp->vect_s[0][2]  = vec[8];
+    cs_rp->vect_s[1][2]  = vec[4];
+    cs_rp->vect_s[2][2]  = vec[9];
+    cs_rp->vect_s[3][2]  = vec[4];
+    cs_rp->vect_s[4][2]  = vec[1];
+    cs_rp->vect_s[5][2]  = vec[5];
+    cs_rp->vect_s[6][2]  = vec[6];
+    cs_rp->vect_s[7][2]  = vec[5];
+    cs_rp->vect_s[8][2]  = vec[1];
+    cs_rp->vect_s[9][2]  = vec[0];
+    cs_rp->vect_s[10][2] = vec[2];
+    cs_rp->vect_s[11][2] = vec[3];
+    cs_rp->vect_s[12][2] = vec[7];
+    cs_rp->vect_s[13][2] = vec[3];
+    cs_rp->vect_s[14][2] = vec[2];
+    cs_rp->vect_s[15][2] = vec[0];
     cs_rp->angsol[0]   = weight[0];
     cs_rp->angsol[1]   = weight[1];
     cs_rp->angsol[2]   = weight[2];
@@ -528,7 +528,7 @@ cs_rad_transfer_dir(void)
     for (int ii = 0; ii < nquad * max_npt; ii++)
       xyz3d[ii][0]  = 1.0 - xyz3d[ii][2] - xyz3d[ii][1];
 
-    /* write xyz3d in cs_rp->sxyz   */
+    /* write xyz3d in cs_rp->vect_s   */
 
     jj = 0;
 
@@ -537,9 +537,9 @@ cs_rad_transfer_dir(void)
       int lev_npt = 2 * lev - 1;
 
       for (int ii = 0; ii < lev_npt; ii++) {
-        cs_rp->sxyz[jj][0] = xyz3d[(lev - 1) * max_npt + ii][0];
-        cs_rp->sxyz[jj][1] = xyz3d[(lev - 1) * max_npt + ii][1];
-        cs_rp->sxyz[jj][2] = xyz3d[(lev - 1) * max_npt + ii][2];
+        cs_rp->vect_s[jj][0] = xyz3d[(lev - 1) * max_npt + ii][0];
+        cs_rp->vect_s[jj][1] = xyz3d[(lev - 1) * max_npt + ii][1];
+        cs_rp->vect_s[jj][2] = xyz3d[(lev - 1) * max_npt + ii][2];
         jj++;
       }
 
@@ -548,7 +548,7 @@ cs_rad_transfer_dir(void)
     /* Vectors normalisation     */
 
     for (int ii = 0; ii < cs_rp->ndirs; ii++)
-      _normve(cs_rp->sxyz[ii]);
+      _normve(cs_rp->vect_s[ii]);
 
     BFT_FREE(xyz3d);
     xyz3d = NULL;
@@ -678,51 +678,51 @@ cs_rad_transfer_dir(void)
     vec[8]    = 0.0;
     weight[0]  = 16 * atan(1.0) / 96.0;
     weight[1]  = 8  * atan(1.0) / 96.0;
-    cs_rp->sxyz[0][0]  = vec[0];
-    cs_rp->sxyz[1][0]  = vec[1];
-    cs_rp->sxyz[2][0]  = vec[1];
-    cs_rp->sxyz[3][0]  = vec[2];
-    cs_rp->sxyz[4][0]  = vec[3];
-    cs_rp->sxyz[5][0]  = vec[3];
-    cs_rp->sxyz[6][0]  = vec[4];
-    cs_rp->sxyz[7][0]  = vec[4];
-    cs_rp->sxyz[8][0]  = vec[5];
-    cs_rp->sxyz[9][0]  = vec[6];
-    cs_rp->sxyz[10][0] = vec[7];
-    cs_rp->sxyz[11][0] = vec[8];
-    cs_rp->sxyz[12][0] = vec[8];
-    cs_rp->sxyz[13][0] = vec[7];
-    cs_rp->sxyz[14][0] = vec[6];
-    cs_rp->sxyz[0][1]  = vec[1];
-    cs_rp->sxyz[1][1]  = vec[0];
-    cs_rp->sxyz[2][1]  = vec[1];
-    cs_rp->sxyz[3][1]  = vec[3];
-    cs_rp->sxyz[4][1]  = vec[2];
-    cs_rp->sxyz[5][1]  = vec[2];
-    cs_rp->sxyz[6][1]  = vec[5];
-    cs_rp->sxyz[7][1]  = vec[4];
-    cs_rp->sxyz[8][1]  = vec[4];
-    cs_rp->sxyz[9][1]  = vec[7];
-    cs_rp->sxyz[10][1] = vec[6];
-    cs_rp->sxyz[11][1] = vec[6];
-    cs_rp->sxyz[12][1] = vec[7];
-    cs_rp->sxyz[13][1] = vec[8];
-    cs_rp->sxyz[14][1] = vec[8];
-    cs_rp->sxyz[0][2]  = vec[1];
-    cs_rp->sxyz[1][2]  = vec[1];
-    cs_rp->sxyz[2][2]  = vec[0];
-    cs_rp->sxyz[3][2]  = vec[3];
-    cs_rp->sxyz[4][2]  = vec[3];
-    cs_rp->sxyz[5][2]  = vec[2];
-    cs_rp->sxyz[6][2]  = vec[4];
-    cs_rp->sxyz[7][2]  = vec[5];
-    cs_rp->sxyz[8][2]  = vec[4];
-    cs_rp->sxyz[9][2]  = vec[8];
-    cs_rp->sxyz[10][2] = vec[8];
-    cs_rp->sxyz[11][2] = vec[7];
-    cs_rp->sxyz[12][2] = vec[6];
-    cs_rp->sxyz[13][2] = vec[6];
-    cs_rp->sxyz[14][2] = vec[7];
+    cs_rp->vect_s[0][0]  = vec[0];
+    cs_rp->vect_s[1][0]  = vec[1];
+    cs_rp->vect_s[2][0]  = vec[1];
+    cs_rp->vect_s[3][0]  = vec[2];
+    cs_rp->vect_s[4][0]  = vec[3];
+    cs_rp->vect_s[5][0]  = vec[3];
+    cs_rp->vect_s[6][0]  = vec[4];
+    cs_rp->vect_s[7][0]  = vec[4];
+    cs_rp->vect_s[8][0]  = vec[5];
+    cs_rp->vect_s[9][0]  = vec[6];
+    cs_rp->vect_s[10][0] = vec[7];
+    cs_rp->vect_s[11][0] = vec[8];
+    cs_rp->vect_s[12][0] = vec[8];
+    cs_rp->vect_s[13][0] = vec[7];
+    cs_rp->vect_s[14][0] = vec[6];
+    cs_rp->vect_s[0][1]  = vec[1];
+    cs_rp->vect_s[1][1]  = vec[0];
+    cs_rp->vect_s[2][1]  = vec[1];
+    cs_rp->vect_s[3][1]  = vec[3];
+    cs_rp->vect_s[4][1]  = vec[2];
+    cs_rp->vect_s[5][1]  = vec[2];
+    cs_rp->vect_s[6][1]  = vec[5];
+    cs_rp->vect_s[7][1]  = vec[4];
+    cs_rp->vect_s[8][1]  = vec[4];
+    cs_rp->vect_s[9][1]  = vec[7];
+    cs_rp->vect_s[10][1] = vec[6];
+    cs_rp->vect_s[11][1] = vec[6];
+    cs_rp->vect_s[12][1] = vec[7];
+    cs_rp->vect_s[13][1] = vec[8];
+    cs_rp->vect_s[14][1] = vec[8];
+    cs_rp->vect_s[0][2]  = vec[1];
+    cs_rp->vect_s[1][2]  = vec[1];
+    cs_rp->vect_s[2][2]  = vec[0];
+    cs_rp->vect_s[3][2]  = vec[3];
+    cs_rp->vect_s[4][2]  = vec[3];
+    cs_rp->vect_s[5][2]  = vec[2];
+    cs_rp->vect_s[6][2]  = vec[4];
+    cs_rp->vect_s[7][2]  = vec[5];
+    cs_rp->vect_s[8][2]  = vec[4];
+    cs_rp->vect_s[9][2]  = vec[8];
+    cs_rp->vect_s[10][2] = vec[8];
+    cs_rp->vect_s[11][2] = vec[7];
+    cs_rp->vect_s[12][2] = vec[6];
+    cs_rp->vect_s[13][2] = vec[6];
+    cs_rp->vect_s[14][2] = vec[7];
     for (int ii = 0; ii < 9; ii++)
       cs_rp->angsol[ii]   = weight[0];
     for (int ii = 9; ii < 15; ii++)
@@ -737,24 +737,24 @@ cs_rad_transfer_dir(void)
     weight[0] = 0.2437531319;
     weight[1] = 0.2798456437;
 
-    cs_rp->sxyz[0][0] = vec[0];
-    cs_rp->sxyz[1][0] = vec[2];
-    cs_rp->sxyz[2][0] = vec[2];
-    cs_rp->sxyz[3][0] = vec[1];
-    cs_rp->sxyz[4][0] = vec[3];
-    cs_rp->sxyz[5][0] = vec[1];
-    cs_rp->sxyz[0][1] = vec[1];
-    cs_rp->sxyz[1][1] = vec[3];
-    cs_rp->sxyz[2][1] = vec[2];
-    cs_rp->sxyz[3][1] = vec[1];
-    cs_rp->sxyz[4][1] = vec[2];
-    cs_rp->sxyz[5][1] = vec[0];
-    cs_rp->sxyz[0][2] = vec[1];
-    cs_rp->sxyz[1][2] = vec[2];
-    cs_rp->sxyz[2][2] = vec[3];
-    cs_rp->sxyz[3][2] = vec[0];
-    cs_rp->sxyz[4][2] = vec[2];
-    cs_rp->sxyz[5][2] = vec[1];
+    cs_rp->vect_s[0][0] = vec[0];
+    cs_rp->vect_s[1][0] = vec[2];
+    cs_rp->vect_s[2][0] = vec[2];
+    cs_rp->vect_s[3][0] = vec[1];
+    cs_rp->vect_s[4][0] = vec[3];
+    cs_rp->vect_s[5][0] = vec[1];
+    cs_rp->vect_s[0][1] = vec[1];
+    cs_rp->vect_s[1][1] = vec[3];
+    cs_rp->vect_s[2][1] = vec[2];
+    cs_rp->vect_s[3][1] = vec[1];
+    cs_rp->vect_s[4][1] = vec[2];
+    cs_rp->vect_s[5][1] = vec[0];
+    cs_rp->vect_s[0][2] = vec[1];
+    cs_rp->vect_s[1][2] = vec[2];
+    cs_rp->vect_s[2][2] = vec[3];
+    cs_rp->vect_s[3][2] = vec[0];
+    cs_rp->vect_s[4][2] = vec[2];
+    cs_rp->vect_s[5][2] = vec[1];
     cs_rp->angsol[0] = weight[0];
     cs_rp->angsol[1] = weight[1];
     cs_rp->angsol[2] = weight[1];
