@@ -4,7 +4,7 @@
 
 /* This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -57,8 +57,7 @@
 #include "cs_sles.h"
 #include "cs_sles_it.h"
 #include "cs_timer.h"
-
-#include "cs_prototypes.h"
+#include "cs_time_step.h"
 
 #include "cs_rad_transfer.h"
 
@@ -127,9 +126,9 @@ const cs_real_t x_kg[5] = {0.0001, 0.25, 0.5, 0.75, 1.0};
 /*----------------------------------------------------------------------------*/
 
 static inline void
-_line_to_array(FILE      *radfile,
-               cs_real_t  values[],
-               cs_int_t  *nvalues)
+_line_to_array(FILE       *radfile,
+               cs_real_t   values[],
+               int        *nvalues)
 {
   char line[256];
   fgets(line, 256, radfile);
@@ -154,9 +153,6 @@ _line_to_array(FILE      *radfile,
   }
 
   *nvalues = index;
-
-  return;
-
 }
 
 /*----------------------------------------------------------------------------*/
@@ -181,7 +177,6 @@ _gridposnbsg1(cs_real_t  trad,
               int        interp_method,
               int        itx[4][4])
 {
-
   int itrad[4] = {0};
   int ita[4]   = {0};
   int ico2a[4] = {0};
@@ -438,7 +433,6 @@ _seval(int       n,
   cs_real_t dx = u - x[i];
 
   return y[i] + dx * (b[i] + dx * (c[i] + dx * d[i]));
-
 }
 
 /*----------------------------------------------------------------------------*/
@@ -537,7 +531,8 @@ _splmi(int        n,
           && 2.0 * al[i] + be[i] > 3.0
           && al[i] + 2.0 * be[i] > 3.0) {
 
-        cs_real_t phi = al[i] - pow((2.0 * al[i] + be[i] - 3.0), 2.0) / (al[i] + be[i] - 2.0) / 3.0;
+        cs_real_t phi = al[i] -   pow((2.0 * al[i] + be[i] - 3.0), 2.0)
+                                / (al[i] + be[i] - 2.0) / 3.0;
 
         if (phi < 0.0) {
           cs_real_t ti = 3.0 / sqrt (pow (al[i], 2.0) + pow (be[i], 2.0));
@@ -1243,7 +1238,8 @@ cs_rad_transfer_fsck(const cs_real_t  *restrict pco2,
                       kfsk);
     /* [m^-1] */
     for (int i = 0; i < ng; i++)
-      kfsk[i] /= 100.0;
+      kfsk[i] *= 100.0;
+
     _simple_interpg(ng,
                     gfsk,
                     kfsk,
@@ -1253,7 +1249,6 @@ cs_rad_transfer_fsck(const cs_real_t  *restrict pco2,
     for (int i = 0; i < cs_glob_rad_transfer_params->nwsgg; i++)
       kloc[i * cs_glob_mesh->n_cells + iel] = kloctmp[i];
 
-
     /* Determination of the local weights */
     for (int i = 0; i < ng; i++) {
       kfsk[i] = 0.;
@@ -1269,6 +1264,7 @@ cs_rad_transfer_fsck(const cs_real_t  *restrict pco2,
     /* [m^-1] */
     for (int i = 0; i < ng; i++)
       kg1[i] *= 100.0;
+
     _simple_interpg(ng,
                     kg1,
                     gg1,
@@ -1300,6 +1296,7 @@ cs_rad_transfer_fsck(const cs_real_t  *restrict pco2,
                      kg1);
     for (int i = 0; i < ng; i++)
       kg1[i] *= 100.0;
+
     _simple_interpg(ng,
                     kg1,
                     gg1,
