@@ -4,7 +4,7 @@
 
 /* This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -592,10 +592,10 @@ _absorb(cs_real_t  ts,
  *        composition of products in co2, h2o and soot
  *        using linear regressions established by Modak.
  *
- * \param[out] ck     coefficient d'absorption du milieu (nul si transparent)
- * \param[in]  pco2   pression partielle de co2
- * \param[in]  pco2   pression partielle de h2o
- * \param[in]  fv     fraction volumique de suies
+ * \param[out] ck     medium's absorbtion coefficient (0 if transparent)
+ * \param[in]  pco2   CO2 partial pressure
+ * \param[in]  ph2o   H2O partial pressure
+ * \param[in]  fv     soot volume fraction
  * \param[in]  temp   temperature
  */
 /*----------------------------------------------------------------------------*/
@@ -643,7 +643,7 @@ cs_rad_transfer_modak(cs_real_t        ck[],
     if ((1.0 - alpha) <= cs_math_epzero)
       bft_error(__FILE__, __LINE__, 0,
                 _("Error in %s: absorptivity computation\n"
-                  "  cell_id = %10d\n"
+                  "  cell_id = %10ld\n"
                   "  alpha = %15.7e\n"
                   "  pco2  = %15.7e\n"
                   "  ph2o  = %15.7e\n"
@@ -652,7 +652,7 @@ cs_rad_transfer_modak(cs_real_t        ck[],
                   "  path  = %15.7e\n"
                   "  fv    = %15.7E\n"),
                 __func__,
-                cell_id,
+                (long)cell_id,
                 alpha,
                 pco2[cell_id],
                 ph2o[cell_id],
