@@ -4,7 +4,7 @@
 
 /* This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -92,6 +92,9 @@ BEGIN_C_DECLS
  * \param[in]  nvar     number of variable BC's
  * \param[in]  ivart    variable id of thermal variable
  * \param[in]  isothp   list of isothermal boundaries
+ * \param[in]  tmin     minimum allowed temperature (clip to this value)
+ * \param[in]  tmax     maximum allowed temperature (clip to this value)
+ * \param[in]  tx       temperature relaxtion parameter
  * \param[in]  rcodcl   boundary condition values
  *                        rcodcl[0] = Dirichlet value
  *                        rcodcl[1] = exchange coefficient value. (infinite
@@ -104,9 +107,8 @@ BEGIN_C_DECLS
  *                                    - for scalars,
  *                                      cp*(viscls+visct/turb_schmidt)*gradt
  * \param[out] tparop   wall temperature in Kelvin
- * \param[in]  qincip   dradiative flux density at boundaries
+ * \param[in]  qincip   radiative flux density at boundaries
  * \param[in]  textp    exterior boundary temperature in degrees C
- * \param[in]  tintp    interior boundary temperature in degrees C
  * \param[in]  xlamp    thermal conductivity coefficient of wall faces (w/m/k)
  * \param[in]  epap     wall thickness (m)
  * \param[in]  epsp     wall emissivity
@@ -120,14 +122,13 @@ void
 cs_rad_transfer_wall_flux(int         nvar,
                           int         ivart,
                           int         isothp[],
-                          cs_real_t  *tmin,
-                          cs_real_t  *tmax,
-                          cs_real_t  *tx,
+                          cs_real_t   tmin,
+                          cs_real_t   tmax,
+                          cs_real_t   tx,
                           cs_real_t  *rcodcl,
                           cs_real_t   tparop[],
                           cs_real_t   qincip[],
                           cs_real_t   textp[],
-                          cs_real_t   tintp[],
                           cs_real_t   xlamp[],
                           cs_real_t   epap[],
                           cs_real_t   epsp[],
@@ -205,18 +206,20 @@ cs_rad_transfer_wall_flux(int         nvar,
     cs_real_t qconv = 0.0;
     cs_real_t qrayt = 0.0;
 
+    cs_real_t detep = 0.0;
+
     int log_z_id = b_face_class_id[ifac];
 
+    int rad_bc_code = isothp[ifac];
+
     /* Isotherms */
-    if (isothp[ifac] == cs_glob_rad_transfer_params->itpimp) {
+    if (rad_bc_code == CS_BOUNDARY_RAD_WALL_GRAY) {
 
       /* Mark for logging */
       iitpim = 1;
-      indtp[log_z_id] = cs_glob_rad_transfer_params->itpimp;
+      indtp[log_z_id] = CS_BOUNDARY_RAD_WALL_GRAY;
 
       /* Computation */
-      tparop[ifac] = tintp[ifac];
-
       qconv = flconp[ifac];
       cs_real_t qinci = qincip[ifac];
       cs_real_t sigt4 = stephn * cs_math_pow4(tparop[ifac]);
@@ -224,117 +227,56 @@ cs_rad_transfer_wall_flux(int         nvar,
       qrayt = epp * (qinci - sigt4);
     }
 
-    /* Grey or black boundaries (non reflecting) */
-    else if (isothp[ifac] == cs_glob_rad_transfer_params->ipgrno) {
+    /* Grey or black boundaries (reflecting or not,
+       with fixed exterior temperature or conduction flux) */
+    else if (   rad_bc_code == CS_BOUNDARY_RAD_WALL_GRAY_EXTERIOR_T
+             || rad_bc_code == CS_BOUNDARY_RAD_WALL_REFL_EXTERIOR_T
+             || rad_bc_code == CS_BOUNDARY_RAD_WALL_GRAY_COND_FLUX) {
 
       /* Mark for logging */
-      iipgrn = 1;
-      indtp[log_z_id] = cs_glob_rad_transfer_params->ipgrno;
-
-      /* Computation */
-      cs_real_t esl    = epap[ifac] / xlamp[ifac];
-      qconv  = flconp[ifac];
-      cs_real_t qinci  = qincip[ifac];
-      cs_real_t epp    = epsp[ifac];
-      cs_real_t sigt3  = stephn * cs_math_pow3(tparop[ifac]);
-      qrayt  = epp * (qinci - sigt3 * tparop[ifac]);
-      cs_real_t detep  =   (esl * (qconv + qrayt) - (tparop[ifac] - textp[ifac]))
-                         / (1.0 + 4.0 * esl * epp * sigt3 + esl * hfconp[ifac]);
-      cs_real_t rapp   = detep / tparop[ifac];
-      cs_real_t abrapp = CS_ABS(rapp);
-
-      /* Relaxation */
-      if (abrapp >= *tx) {
-        nrelax++;
-        tparop[ifac] *= 1.0 + *tx * rapp / abrapp;
-      }
-      else
-        tparop[ifac] += detep;
-
-      rapmax = CS_MAX(rapmax, abrapp);
-      if (rapp <= 0.0)
-        nmoins++;
-      else
-        nplus++;
-
-      /* Clipping */
-      if (tparop[ifac] < *tmin) {
-        n1min++;
-        tparop[ifac] = *tmin;
-      }
-      if (tparop[ifac] > *tmax) {
-        n1max++;
-        tparop[ifac] = *tmax;
-      }
-
-    }
-
-    /* Reflecting walls */
-    else if (isothp[ifac] == cs_glob_rad_transfer_params->iprefl) {
-
-      /* Mark for logging */
-      iipref = 1;
-      indtp[log_z_id] = cs_glob_rad_transfer_params->iprefl;
-
-      /* Computation */
-      cs_real_t esl    = epap[ifac] / xlamp[ifac];
-      qconv  = flconp[ifac];
-      cs_real_t detep  =  (esl * qconv - (tparop[ifac] - textp[ifac]))
-                        / (1.0 + esl * hfconp[ifac]);
-      cs_real_t rapp   = detep / tparop[ifac];
-      cs_real_t abrapp = CS_ABS(rapp);
-      qrayt = 0.0;
-
-      /* Relaxation */
-      if (abrapp >= *tx) {
-        nrelax++;
-        tparop[ifac] *= 1.0 + *tx * rapp / abrapp;
-      }
-      else
-        tparop[ifac] += detep;
-
-      rapmax = CS_MAX(rapmax, abrapp);
-      if (rapp <= 0.0)
-        nmoins++;
-      else
-        nplus++;
-
-      /* Clipping */
-      if (tparop[ifac] < *tmin) {
-        n1min++;
-        tparop[ifac] = *tmin;
-      }
-      if (tparop[ifac] > *tmax) {
-        n1max++;
-        tparop[ifac] = *tmax;
-      }
-
-    }
-
-    /* Wall with non-reflecting conduction flux:
-     *   if the flux is zero, the wall is adiabatic
-     *   (radiative transfer is balanced by convection) */
-    else if (isothp[ifac] == cs_glob_rad_transfer_params->ifgrno) {
-
-      /* Mark for logging */
-      iifgrn = 1;
-      indtp[log_z_id] = cs_glob_rad_transfer_params->ifgrno;
+      indtp[log_z_id] = rad_bc_code;
 
       /* Computation */
       qconv  = flconp[ifac];
       cs_real_t qinci  = qincip[ifac];
-      cs_real_t epp    = epsp[ifac];
-      cs_real_t sigt3  = stephn * cs_math_pow3(tparop[ifac]);
-      qrayt  = epp * (qinci - sigt3 * tparop[ifac]);
-      cs_real_t detep  =  (qconv + qrayt - rcodcl[ifac + ircodcl])
-                        / (4.0 * epp * sigt3 + hfconp[ifac]);
+
+      if (rad_bc_code == CS_BOUNDARY_RAD_WALL_GRAY_EXTERIOR_T) {
+        /* Non-reflecting, fixed exterior temperature */
+        iipgrn = 1;
+
+        cs_real_t esl    = epap[ifac] / xlamp[ifac];
+        cs_real_t epp    = epsp[ifac];
+        cs_real_t sigt3  = stephn * cs_math_pow3(tparop[ifac]);
+        qrayt  = epp * (qinci - sigt3 * tparop[ifac]);
+        detep  =   (esl * (qconv + qrayt) - (tparop[ifac] - textp[ifac]))
+                 / (1.0 + 4.0*esl*epp*sigt3 + esl*hfconp[ifac]);
+      }
+      else if (rad_bc_code == CS_BOUNDARY_RAD_WALL_REFL_EXTERIOR_T) {
+        /* Reflecting, fixed exterior temperature */
+        iipref = 1;
+
+        cs_real_t esl    = epap[ifac] / xlamp[ifac];
+        detep  =   (esl * qconv - (tparop[ifac] - textp[ifac]))
+                 / (1.0 + esl*hfconp[ifac]);
+      }
+      else if (rad_bc_code == CS_BOUNDARY_RAD_WALL_GRAY_COND_FLUX) {
+        /* Non-reflecting, fixed conduction flux */
+        iifgrn = 1;
+
+        cs_real_t epp    = epsp[ifac];
+        cs_real_t sigt3  = stephn * cs_math_pow3(tparop[ifac]);
+        qrayt  = epp * (qinci - sigt3 * tparop[ifac]);
+        detep  =   (qconv + qrayt - rcodcl[ifac + ircodcl])
+                 / (4.0 * epp * sigt3 + hfconp[ifac]);
+      }
+
       cs_real_t rapp   = detep / tparop[ifac];
       cs_real_t abrapp = CS_ABS(rapp);
 
       /* Relaxation */
-      if (abrapp >= *tx) {
+      if (abrapp >= tx) {
         nrelax++;
-        tparop[ifac] *= 1.0 + *tx * rapp / abrapp;
+        tparop[ifac] *= 1.0 + tx * rapp / abrapp;
       }
       else
         tparop[ifac] += detep;
@@ -346,51 +288,51 @@ cs_rad_transfer_wall_flux(int         nvar,
         nplus++;
 
       /* Clipping */
-      if (tparop[ifac] < *tmin) {
+      if (tparop[ifac] < tmin) {
         n1min++;
-        tparop[ifac] = *tmin;
+        tparop[ifac] = tmin;
       }
-      if (tparop[ifac] > *tmax) {
+      if (tparop[ifac] > tmax) {
         n1max++;
-        tparop[ifac] = *tmax;
+        tparop[ifac] = tmax;
       }
 
     }
 
-    /* 5) parois a flux de conduction imposee et reflechissante
-     *    equivalent a impose un flux total au fluide    */
-    else if (isothp[ifac] == cs_glob_rad_transfer_params->ifrefl) {
+    /* Reflecting wall with fixed conduction flux
+     * equivalent to total flux for fluid */
+    else if (rad_bc_code == CS_BOUNDARY_RAD_WALL_REFL_COND_FLUX) {
 
       /* Mark for logging */
       iifref = 1;
-      indtp[log_z_id] = cs_glob_rad_transfer_params->ifrefl;
+      indtp[log_z_id] = rad_bc_code;
 
       /* Computation */
       cs_lnum_t iel = cs_glob_mesh->b_face_cells[ifac];
-      tparop[ifac] = hfconp[ifac] * tempkp[iel] - rcodcl[ifac + ircodcl];
-      tparop[ifac] = tparop[ifac] / CS_MAX(hfconp[ifac], cs_math_epzero);
+      tparop[ifac] =  (hfconp[ifac] * tempkp[iel] - rcodcl[ifac + ircodcl])
+                     / CS_MAX(hfconp[ifac], cs_math_epzero);
 
       qconv = flconp[ifac];
       qrayt = 0.0;
 
       /* Clipping */
-      if (tparop[ifac] < *tmin) {
+      if (tparop[ifac] < tmin) {
         n1min++;
-        tparop[ifac] = *tmin;
+        tparop[ifac] = tmin;
       }
-      if (tparop[ifac] > *tmax) {
+      if (tparop[ifac] > tmax) {
         n1max++;
-        tparop[ifac] = *tmax;
+        tparop[ifac] = tmax;
       }
 
     }
 
     /* Max-Min */
-    if (   isothp[ifac] == cs_glob_rad_transfer_params->itpimp
-        || isothp[ifac] == cs_glob_rad_transfer_params->ipgrno
-        || isothp[ifac] == cs_glob_rad_transfer_params->iprefl
-        || isothp[ifac] == cs_glob_rad_transfer_params->ifgrno
-        || isothp[ifac] == cs_glob_rad_transfer_params->ifrefl) {
+    if (   rad_bc_code == CS_BOUNDARY_RAD_WALL_GRAY
+        || rad_bc_code == CS_BOUNDARY_RAD_WALL_GRAY_EXTERIOR_T
+        || rad_bc_code == CS_BOUNDARY_RAD_WALL_REFL_EXTERIOR_T
+        || rad_bc_code == CS_BOUNDARY_RAD_WALL_GRAY_COND_FLUX
+        || rad_bc_code == CS_BOUNDARY_RAD_WALL_REFL_COND_FLUX) {
       if (tpmax <= tparop[ifac]) {
         ifacmx = ifac;
         tpmax  = tparop[ifac];
@@ -412,6 +354,10 @@ cs_rad_transfer_wall_flux(int         nvar,
   /* Logging
    * ======= */
 
+  int verbosity = cs_glob_rad_transfer_params->iimpar;
+  if (cs_log_default_is_active() == false)
+    verbosity -= 2;
+
   /* Check if there are any wall zones */
 
   if (cs_glob_rank_id >= 0)
@@ -429,7 +375,7 @@ cs_rad_transfer_wall_flux(int         nvar,
   if (indtpm > 0) {
 
     /* Level 1 verbosity */
-    if (cs_glob_rad_transfer_params->iimpar >= 1) {
+    if (verbosity >= 1) {
 
       /* Calcul de TZOMOY FLUNET RADIOS SURFT */
       for (int log_z_id = 0; log_z_id < n_zones; log_z_id++) {
@@ -566,7 +512,7 @@ cs_rad_transfer_wall_flux(int         nvar,
                       _("\n"
                         " Warning: wall temperature relaxed to %7.2f "
                         "at (%llu points)\n"),
-                      *tx * 100.0, (unsigned long long)nrelax);
+                      tx * 100.0, (unsigned long long)nrelax);
 
       if (n1min > 0 || n1max > 0)
         cs_log_printf(CS_LOG_DEFAULT,
@@ -595,7 +541,7 @@ cs_rad_transfer_wall_flux(int         nvar,
                         " ------------------------------------------------"
                         "---------------------------\n"));
         for (int log_z_id = 0; log_z_id < n_zones; log_z_id++)
-          if (indtp[log_z_id] == cs_glob_rad_transfer_params->itpimp)
+          if (indtp[log_z_id] == CS_BOUNDARY_RAD_WALL_GRAY)
             cs_log_printf(CS_LOG_DEFAULT,
                           "%10d        %11.4e    %11.4e    %11.4e    %11.4e\n",
                           log_z_id,
@@ -613,7 +559,7 @@ cs_rad_transfer_wall_flux(int         nvar,
                         "------------------------------------------------"
                         "---------------------------\n"));
         for (int log_z_id = 0; log_z_id < n_zones; log_z_id++)
-          if (indtp[log_z_id] == cs_glob_rad_transfer_params->ipgrno)
+          if (indtp[log_z_id] == CS_BOUNDARY_RAD_WALL_GRAY_EXTERIOR_T)
             cs_log_printf(CS_LOG_DEFAULT,
                           "%10d        %11.4e    %11.4e    %11.4e    %11.4e\n",
                           log_z_id,
@@ -631,7 +577,7 @@ cs_rad_transfer_wall_flux(int         nvar,
                         "------------------------------------------------"
                         "---------------------------\n"));
         for (int log_z_id = 0; log_z_id < n_zones; log_z_id++) {
-          if (indtp[log_z_id] == cs_glob_rad_transfer_params->iprefl)
+          if (indtp[log_z_id] == CS_BOUNDARY_RAD_WALL_REFL_EXTERIOR_T)
             cs_log_printf(CS_LOG_DEFAULT,
                           "%10d        %11.4e    %11.4e    %11.4e    %11.4e\n",
                           log_z_id,
@@ -650,7 +596,7 @@ cs_rad_transfer_wall_flux(int         nvar,
                         "------------------------------------------------"
                         "---------------------------\n"));
         for (int log_z_id = 0; log_z_id < n_zones; log_z_id++) {
-          if (indtp[log_z_id] == cs_glob_rad_transfer_params->ifgrno)
+          if (indtp[log_z_id] == CS_BOUNDARY_RAD_WALL_GRAY_COND_FLUX)
             cs_log_printf(CS_LOG_DEFAULT,
                           "%10d        %11.4e    %11.4e    %11.4e    %11.4e\n",
                           log_z_id,
@@ -670,7 +616,7 @@ cs_rad_transfer_wall_flux(int         nvar,
                         "------------------------------------------------"
                         "---------------------------\n"));
         for (int log_z_id = 0; log_z_id < n_zones; log_z_id++) {
-          if (indtp[log_z_id] == cs_glob_rad_transfer_params->ifrefl)
+          if (indtp[log_z_id] == CS_BOUNDARY_RAD_WALL_REFL_COND_FLUX)
             cs_log_printf(CS_LOG_DEFAULT,
                           "%10d        %11.4e    %11.4e    %11.4e    %11.4e\n",
                           log_z_id,
@@ -685,7 +631,7 @@ cs_rad_transfer_wall_flux(int         nvar,
 
     /* If we need additional verbosity */
 
-    if (cs_glob_rad_transfer_params->iimpar >= 2) {
+    if (verbosity >= 2) {
       const char fmt[]
         = N_("\n"
              " %s wall temperature (degrees Celsius) = %15.7f\n"
