@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -44,14 +44,11 @@ module radiat
   !>  - 2: P1
   integer(c_int), pointer, save :: iirayo
 
-  !> Period of the radiation module.
-  !> The radiation module is called every \ref nfreqr time steps (more precisely,
-  !> every time \ref optcal::ntcabs "ntcabs" is a multiple of \ref nfreqr).
-  !> Also, in order to have proper initialization of the variables, whatever
-  !> the value of \ref nfreqr, the radiation module is called at
-  !> the first time step of a calculation (restart or not).
-  !> Useful if and only if the radiation module is activated}
-  integer(c_int), pointer, save ::           nfreqr
+  !> Atmospheric radiation model:
+  !> - Direct Solar the first bit
+  !> - diFfuse Solar for the second bit
+  !> - InfraRed for the third bitPeriod of the radiation module.
+  integer(c_int), pointer, save :: rad_atmo_model
 
   !> \}
 
@@ -71,12 +68,12 @@ module radiat
     !---------------------------------------------------------------------------
 
     ! Interface to C function to retrieve pointers
-    subroutine cs_rad_transfer_get_pointers(p_iirayo, p_nfreqr)  &
+    subroutine cs_rad_transfer_get_pointers(p_iirayo, p_rad_atmo_model)  &
       bind(C, name='cs_rad_transfer_get_pointers')
       use, intrinsic :: iso_c_binding
       implicit none
 
-      type(c_ptr), intent(out) :: p_iirayo, p_nfreqr
+      type(c_ptr), intent(out) :: p_iirayo, p_rad_atmo_model
 
     end subroutine cs_rad_transfer_get_pointers
 
@@ -127,15 +124,14 @@ module radiat
 
     ! Interface to C function handling resolution
 
-    subroutine cs_rad_transfer_solve(bc_type, nclacp, nclafu,         &
-                                     dt, cp2fol, cp2ch, ichcor)       &
+    subroutine cs_rad_transfer_solve(bc_type,     &
+                                     cp2fol, cp2ch, ichcor)  &
       bind(C, name='cs_rad_transfer_solve')
       use, intrinsic :: iso_c_binding
       implicit none
-      integer(kind=c_int), value :: nclacp, nclafu
       integer(kind=c_int), dimension(*) :: bc_type, ichcor
       real(kind=c_double), value :: cp2fol
-      real(kind=c_double), dimension(*) :: dt, cp2ch
+      real(kind=c_double), dimension(*) :: cp2ch
     end subroutine cs_rad_transfer_solve
 
     !---------------------------------------------------------------------------
@@ -167,12 +163,12 @@ contains
     use ppcpfu
     use numvar
 
-    type(c_ptr) :: p_iirayo, p_nfreqr
+    type(c_ptr) :: p_iirayo, p_rad_atmo_model
 
-    call cs_rad_transfer_get_pointers(p_iirayo, p_nfreqr)
+    call cs_rad_transfer_get_pointers(p_iirayo, p_rad_atmo_model)
 
     call c_f_pointer(p_iirayo, iirayo)
-    call c_f_pointer(p_nfreqr, nfreqr)
+    call c_f_pointer(p_rad_atmo_model, rad_atmo_model)
 
   end subroutine radiat_init
 
