@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -71,18 +71,19 @@ double precision c_durbin_l
 integer          iel
 integer          ii    , jj    , ifac
 integer          iflmas, iflmab
-integer          nswrgp, imligp, iwarnp
+integer          imrgrp, nswrgp, imligp, iwarnp
 integer          iconvp, idiffp, ndircp
 integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          imucpp, idftnp, iswdyp
 integer          icvflb
 integer          init
 integer          ivoid(1)
-double precision blencp, epsilp, epsrsp, epsrgp, climgp, extrap, relaxp
+double precision blencp, epsilp, epsrsp, epsrgp, climgp, relaxp
 double precision thetv , thetap
 double precision d1s4, d3s2, d1s2
 double precision xk, xnu, l2
 double precision xllke, xllkmg, xlldrb
+double precision normp
 
 double precision rvoid(1)
 
@@ -225,9 +226,10 @@ call viscfa                                                       &
 
 iconvp = vcopt%iconv
 idiffp = vcopt%idiff
-ndircp = ndircl(ial) ! Diagonal reinforcement: same as dynamic Alpha
+ndircp = vcopt%ndircl ! Diagonal reinforcement: same as dynamic Alpha
 nswrsp = vcopt%nswrsm
 nswrgp = vcopt%nswrgr
+imrgrp = vcopt%imrgra
 imligp = vcopt%imligr
 ircflp = vcopt%ircflu
 ischcp = vcopt%ischcv
@@ -242,20 +244,20 @@ epsilp = vcopt%epsilo
 epsrsp = vcopt%epsrsm
 epsrgp = vcopt%epsrgr
 climgp = vcopt%climgr
-extrap = vcopt%extrag
 relaxp = vcopt%relaxv
 
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
 
 init   = 1
 
 call codits &
  ( idtvar , init   , f_id   , iconvp , idiffp , ndircp ,          &
-   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   iwarnp ,                                                       &
-   blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
+   iwarnp , normp  ,                                              &
+   blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
    relaxp , thetv  ,                                              &
    cvara_al        , cvara_al        ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
