@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -38,7 +38,6 @@
 !______________________________________________________________________________!
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
-!> \param[in]     ncepdp        number of cells with head loss
 !> \param[in]     ncesmp        number of cells with mass source term
 !> \param[in]     icepdc        index of cells with head loss
 !> \param[in]     icetsm        index of cells with mass source term
@@ -61,11 +60,11 @@
 !_______________________________________________________________________________
 
 subroutine reseps &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   icepdc , icetsm , itypsm ,                                     &
+ ( nvar   , nscal  , ncesmp ,                                     &
+   icetsm , itypsm ,                                              &
    dt     ,                                                       &
    gradv  , produc , gradro ,                                     &
-   ckupdc , smacel ,                                              &
+   smacel ,                                                       &
    viscf  , viscb  ,                                              &
    smbr   , rovsdt )
 
@@ -97,15 +96,14 @@ implicit none
 ! Arguments
 
 integer          nvar   , nscal
-integer          ncepdp , ncesmp
+integer          ncesmp
 
-integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
 double precision produc(6,ncelet), gradv(3, 3, ncelet)
 double precision gradro(3,ncelet)
-double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
+double precision smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
 double precision smbr(ncelet), rovsdt(ncelet)
 
@@ -114,9 +112,8 @@ double precision smbr(ncelet), rovsdt(ncelet)
 integer          init
 integer          ivar
 integer          iel
-integer          iiun
 integer          iflmas, iflmab
-integer          nswrgp, imligp, iwarnp
+integer          imrgrp, nswrgp, imligp, iwarnp
 integer          iconvp, idiffp, ndircp
 integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          st_prv_id
@@ -125,12 +122,14 @@ integer          icvflb
 integer          ivoid(1)
 integer          key_t_ext_id
 integer          iroext
-double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
+double precision blencp, epsilp, epsrgp, climgp, relaxp
 double precision epsrsp, alpha3
 double precision trprod , trrij
 double precision tseps , kseps , ceps2
 double precision tuexpe, thets , thetv , thetap, thetp1
 double precision prdeps, xttdrb, xttke , xttkmg
+double precision normp
+double precision sigmae
 
 double precision rvoid(1)
 
@@ -188,6 +187,7 @@ call field_get_val_s(ivisct, visct)
 
 call field_get_val_s(ivarfl(iep), cvar_ep)
 call field_get_val_prev_s(ivarfl(iep), cvara_ep)
+call field_get_key_double(ivarfl(iep), ksigmas, sigmae)
 if (iturb.eq.32) call field_get_val_s(ivarfl(ial), cvar_al)
 if(irijco.eq.1) then
   call field_get_val_prev_v(ivarfl(irij), cvara_rij)
@@ -246,12 +246,7 @@ enddo
 ! 2. User source terms
 !===============================================================================
 
-call cs_user_turbulence_source_terms &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivarfl(ivar)    ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
-   ckupdc , smacel ,                                              &
-   smbr   , rovsdt )
+call user_source_terms(ivarfl(ivar), smbr, rovsdt)
 
 !     If we extrapolate the source terms
 if (st_prv_id.ge.0) then
@@ -312,15 +307,10 @@ endif
 
 if (ncesmp.gt.0) then
 
-  ! Integer equal to 1 (forr navsto: nb of sur-iter)
-  iiun = 1
-
   ! We increment smbr with -Gamma.var_prev. and rovsdt with Gamma
-  call catsma &
- ( ncelet , ncel   , ncesmp , iiun   , isto2t ,                   &
-   icetsm , itypsm(:,ivar)  ,                                     &
-   volume , cvara_ep        , smacel(:,ivar)   , smacel(:,ipr) ,  &
-   smbr   , rovsdt , w1 )
+  call catsma(ncesmp, 1, icetsm, itypsm(:,ivar),                    &
+              cell_f_vol, cvara_ep, smacel(:,ivar), smacel(:,ipr),  &
+              smbr, rovsdt, w1)
 
   ! If we extrapolate the source terms, we put Gamma Pinj in c_st_prv
   if (st_prv_id.ge.0) then
@@ -550,8 +540,9 @@ endif
 
 iconvp = vcopt%iconv
 idiffp = vcopt%idiff
-ndircp = ndircl(ivar)
+ndircp = vcopt%ndircl
 nswrsp = vcopt%nswrsm
+imrgrp = vcopt%imrgra
 nswrgp = vcopt%nswrgr
 imligp = vcopt%imligr
 ircflp = vcopt%ircflu
@@ -567,18 +558,18 @@ epsilp = vcopt%epsilo
 epsrsp = vcopt%epsrsm
 epsrgp = vcopt%epsrgr
 climgp = vcopt%climgr
-extrap = vcopt%extrag
 relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
 init   = 1
 
 call codits &
  ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
-   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   iwarnp ,                                                       &
-   blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
+   iwarnp , normp  ,                                              &
+   blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
    relaxp , thetv  ,                                              &
    cvara_ep        , cvara_ep        ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
@@ -599,15 +590,7 @@ deallocate(weighf, weighb)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,'           Resolution de la variable ',A8,/)
-
-#else
-
- 1000 format(/,'           Solving variable ',A8           ,/)
-
-#endif
+ 1000 format(/,'           Solving variable ', a8          ,/)
 
 !----
 ! End
