@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -45,11 +45,9 @@
 !______________________________________________________________________________!
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
-!> \param[in]     ncepdp        number of cells with head loss
 !> \param[in]     ncesmp        number of cells with mass source term
 !> \param[in]     ivar          variable number
 !> \param[in]     isou          local variable number (7 here)
-!> \param[in]     icepdc        index of cells with head loss
 !> \param[in]     icetsm        index of cells with mass source term
 !> \param[in]     itypsm        type of mass source term for each variable
 !>                               (see \ref cs_user_mass_source_terms)
@@ -57,7 +55,6 @@
 !> \param[in]     produc        work array for production
 !> \param[in]     gradro        work array for grad rom
 !>                              (without rho volume) only for iturb=30
-!> \param[in]     ckupdc        work array for the head loss
 !> \param[in]     smacel        value associated to each variable in the mass
 !>                              source terms or mass rate (see
 !>                              \ref cs_user_mass_source_terms)
@@ -69,12 +66,12 @@
 !______________________________________________________________________________!
 
 subroutine resrij &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
+ ( nvar   , nscal  , ncesmp ,                                     &
    ivar   , isou   ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
+   icetsm , itypsm ,                                              &
    dt     ,                                                       &
    produc , gradro ,                                              &
-   ckupdc , smacel ,                                              &
+   smacel ,                                                       &
    viscf  , viscb  ,                                              &
    tslagi ,                                                       &
    smbr   , rovsdt )
@@ -108,16 +105,15 @@ implicit none
 ! Arguments
 
 integer          nvar   , nscal
-integer          ncepdp , ncesmp
+integer          ncesmp
 integer          ivar   , isou
 
-integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
 double precision produc(6,ncelet)
 double precision gradro(3,ncelet)
-double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
+double precision smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
 double precision tslagi(ncelet)
 double precision smbr(ncelet), rovsdt(ncelet)
@@ -125,9 +121,9 @@ double precision smbr(ncelet), rovsdt(ncelet)
 ! Local variables
 
 integer          iel
-integer          ii    , jj    , kk    , iiun, comp_id
+integer          ii    , jj    , kk    , comp_id
 integer          iflmas, iflmab
-integer          nswrgp, imligp, iwarnp
+integer          imrgrp, nswrgp, imligp, iwarnp
 integer          iconvp, idiffp, ndircp
 integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          st_prv_id
@@ -140,13 +136,14 @@ integer          ivoid(1)
 integer          key_t_ext_id
 integer          iroext
 
-double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
+double precision blencp, epsilp, epsrgp, climgp, relaxp
 double precision epsrsp
 double precision trprod, trrij , deltij
 double precision tuexpr, thets , thetv , thetp1
 double precision d1s3  , d2s3
 double precision ccorio, matrot(3,3)
 double precision rctse
+double precision normp
 
 double precision rvoid(1)
 
@@ -262,12 +259,7 @@ endif
 ! 2. User source terms
 !===============================================================================
 
-call cs_user_turbulence_source_terms &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivarfl(ivar)    ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
-   ckupdc , smacel ,                                              &
-   smbr   , rovsdt )
+call user_source_terms(ivarfl(ivar), smbr, rovsdt)
 
 !     If we extrapolate the source terms
 if (st_prv_id.ge.0) then
@@ -315,22 +307,17 @@ endif
 
 if (ncesmp.gt.0) then
 
-!       Integer equal to 1 (for navsto: nb of sur-iter)
-  iiun = 1
+  ! We increment smbr with -Gamma.var_prev. and rovsdt with Gamma
+  call catsma(ncesmp, 1, icetsm, itypsm(:,ivar),                     &
+              cell_f_vol, cvara_var, smacel(:,ivar), smacel(:,ipr),  &
+              smbr,  rovsdt, w1)
 
-!       We increment smbr with -Gamma.var_prev. and rovsdt with Gamma
-  call catsma &
- ( ncelet , ncel   , ncesmp , iiun   , isto2t ,                   &
-   icetsm , itypsm(:,ivar)  ,                                     &
-   cell_f_vol , cvara_var   , smacel(:,ivar)   , smacel(:,ipr) ,  &
-   smbr   ,  rovsdt , w1 )
-
-!       If we extrapolate the source terms we put Gamma Pinj in c_st_prv
+  ! If we extrapolate the source terms we put Gamma Pinj in c_st_prv
   if (st_prv_id.ge.0) then
     do iel = 1, ncel
       c_st_prv(iel) = c_st_prv(iel) + w1(iel)
     enddo
-!       Otherwise we put it directly in smbr
+  ! Otherwise we put it directly in smbr
   else
     do iel = 1, ncel
       smbr(iel) = smbr(iel) + w1(iel)
@@ -645,8 +632,9 @@ endif
 
 iconvp = vcopt%iconv
 idiffp = vcopt%idiff
-ndircp = ndircl(ivar)
+ndircp = vcopt%ndircl
 nswrsp = vcopt%nswrsm
+imrgrp = vcopt%imrgra
 nswrgp = vcopt%nswrgr
 imligp = vcopt%imligr
 ircflp = vcopt%ircflu
@@ -662,18 +650,18 @@ epsilp = vcopt%epsilo
 epsrsp = vcopt%epsrsm
 epsrgp = vcopt%epsrgr
 climgp = vcopt%climgr
-extrap = vcopt%extrag
 relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
 init   = 1
 
 call codits &
  ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
-   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   iwarnp ,                                                       &
-   blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
+   iwarnp , normp  ,                                              &
+   blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
    relaxp , thetv  ,                                              &
    cvara_var       , cvara_var       ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
@@ -695,15 +683,7 @@ deallocate(weighf, weighb)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,'           Resolution pour la variable ',A8,/)
-
-#else
-
- 1000 format(/,'           Solving variable ',A8           ,/)
-
-#endif
+ 1000 format(/,'           Solving variable ', a8          ,/)
 
 !----
 ! End
