@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -88,7 +88,7 @@ double precision grad_al(3,ncelet)
 integer          iel
 integer          ii, ivar
 integer          iflmas, iflmab
-integer          nswrgp, imligp, iwarnp
+integer          imrgrp, nswrgp, imligp, iwarnp
 integer          iconvp, idiffp, ndircp
 integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          st_prv_id
@@ -99,6 +99,7 @@ integer          idftnp, iswdyp, icvflb
 integer          f_id
 integer          keyvar, iut
 integer          init
+integer          kturt, turb_flux_model
 
 integer          ivoid(1)
 
@@ -112,7 +113,7 @@ double precision xrij(3,3),phiith(3), phiitw(3)
 double precision xnal(3), xnoral
 double precision alpha, xttdrbt, xttdrbw
 double precision pk, gk, xxc1, xxc2, xxc3, imp_term
-double precision rctse
+double precision rctse, visls_0
 
 double precision rvoid(1)
 
@@ -208,6 +209,10 @@ if (vcopt%iwarni.ge.1) then
   write(nfecra,1000) trim(name)//'_turbulent_flux'
 endif
 
+! Get the turbulent flux model
+call field_get_key_id('turbulent_flux_model', kturt)
+call field_get_key_int(ivarfl(isca(iscal)), kturt, turb_flux_model)
+
 ! S pour Source, V pour Variable
 thets  = thetst
 thetv  = vcopt%thetav
@@ -222,6 +227,8 @@ endif
 call field_get_key_int (ivarfl(ivar), kivisl, ifcvsl)
 if (ifcvsl .ge. 0) then
   call field_get_val_s(ifcvsl, viscls)
+else
+  call field_get_key_double(ivarfl(ivar), kvisl0, visls_0)
 endif
 
 do iel = 1, ncelet
@@ -274,7 +281,7 @@ endif
 do iel = 1, ncel
   do isou = 1, 3
     fimp(isou,isou,iel) = fimp(isou,isou,iel)                                  &
-                        + vcopt%istat*(crom(iel)/dt(iel))*volume(iel)
+                        + vcopt%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)
   enddo
 enddo
 
@@ -287,7 +294,7 @@ if (itt.gt.0) then
   call field_get_val_s(ivarfl(isca(itt)), cvar_tt)
   call field_get_val_prev_s(ivarfl(isca(itt)), cvara_tt)
 endif
-if (iturt(iscal).eq.31) then
+if (turb_flux_model.eq.31) then
   ! Name of the scalar
   call field_get_name(ivarfl(isca(iscal)), fname)
 
@@ -321,14 +328,14 @@ do iel = 1, ncel
   if (ifcvsl.ge.0) then
     prdtl = viscl(iel)*xcpp(iel)/viscls(iel)
   else
-    prdtl = viscl(iel)*xcpp(iel)/visls0(iscal)
+    prdtl = viscl(iel)*xcpp(iel)/visls_0
   endif
 
   trrij  = 0.5d0*(xrij(1,1) + xrij(2,2) + xrij(3,3))
   ! --- Compute Durbin time scheme
   xttke  = trrij/cvar_ep(iel)
 
-  if (iturt(iscal).eq.31) then
+  if (turb_flux_model.eq.31) then
     alpha = cvar_al(iel)
     !FIXME Warning / rhebdfm**0.5 compared to F Dehoux
     xttdrbt = xttke * sqrt( (1.d0-alpha)*prdtl/ rhebdfm + alpha )
@@ -405,10 +412,10 @@ do iel = 1, ncel
     ! Pressure/thermal fluctuation correlation term
     !----------------------------------------------
     smbrut(isou,iel) = smbrut(isou,iel) +                                      &
-                volume(iel)*crom(iel)*(  alpha *         phiith(isou)          &
+                cell_f_vol(iel)*crom(iel)*(  alpha *         phiith(isou)          &
                                       + (1.d0 - alpha) * phiitw(isou))
 
-    imp_term = max(volume(iel)*crom(iel)*(                                     &
+    imp_term = max(cell_f_vol(iel)*crom(iel)*(                                     &
               alpha        * (c1trit/xttdrbt-c2trit*gradv(isou,isou,iel))      &
             + (1.d0-alpha) * (xxc1*xnal(isou)*xnal(isou) / xttdrbw) &
             ), 0.d0)
@@ -418,34 +425,34 @@ do iel = 1, ncel
     ! Production terms
     !-----------------
     smbrut(isou,iel) = smbrut(isou,iel)                              &
-                     + volume(iel)*crom(iel)                         &
+                     + cell_f_vol(iel)*crom(iel)                         &
                        ! Production term due to the mean velcoity
-                       *( -xuta(1,iel)*gradv(1,isou,iel)             &
-                          -xuta(2,iel)*gradv(2,isou,iel)             &
-                          -xuta(3,iel)*gradv(3,isou,iel)             &
+                       *( -gradv(1,isou,iel)*xuta(1,iel)             &
+                          -gradv(2,isou,iel)*xuta(2,iel)             &
+                          -gradv(3,isou,iel)*xuta(3,iel)             &
                        ! Production term due to the mean temperature
-                         -xrij(isou,1)*gradt(1,iel)                  &
-                         -xrij(isou,2)*gradt(2,iel)                  &
-                         -xrij(isou,3)*gradt(3,iel)                  &
+                         -xrij(1,isou)*gradt(1,iel)                  &
+                         -xrij(2,isou)*gradt(2,iel)                  &
+                         -xrij(3,isou)*gradt(3,iel)                  &
                         )
 
     ! Production term due to the gravity
     if (itt.gt.0.and.ibeta.ge.0) then
       smbrut(isou,iel) = smbrut(isou,iel)                            &
-                       + volume(iel)*crom(iel)*(            &
+                       + cell_f_vol(iel)*crom(iel)*(            &
                -grav(isou)*cpro_beta(iel)*cvara_tt(iel))
     endif
 
     ! Dissipation (Wall term only because "h" term is zero
     smbrut(isou,iel) = smbrut(isou,iel) -                                      &
-                volume(iel)*crom(iel)*(1.d0 - alpha) / xttdrbw *               &
+                cell_f_vol(iel)*crom(iel)*(1.d0 - alpha) / xttdrbw *               &
                        ( xxc2 * xuta(isou, iel)                                &
                        + xxc3*( xuta(1,iel)*xnal(1)*xnal(isou)                 &
                               + xuta(2,iel)*xnal(2)*xnal(isou)                 &
                               + xuta(3,iel)*xnal(3)*xnal(isou)))
 
     ! TODO we can implicite more terms
-    imp_term = max(volume(iel)*crom(iel)*(1.d0 - alpha) / xttdrbw *            &
+    imp_term = max(cell_f_vol(iel)*crom(iel)*(1.d0 - alpha) / xttdrbw *            &
                   ( xxc2 + xxc3 * xnal(isou)*xnal(isou)), 0.d0)
     fimp(isou,isou,iel) = fimp(isou,isou,iel) + imp_term
 
@@ -457,12 +464,15 @@ enddo
 !===============================================================================
 ! Symmetric tensor diffusivity (GGDH)
 if (iand(vcopt_ut%idften, ANISOTROPIC_RIGHT_DIFFUSION).ne.0) then
+
+  call field_get_key_double(ivarfl(isca(iscal)), kvisl0, visls_0)
+
   do iel = 1, ncel
 
     if (ifcvsl.ge.0) then
       prdtl = viscl(iel)*xcpp(iel)/viscls(iel)
     else
-      prdtl = viscl(iel)*xcpp(iel)/visls0(iscal)
+      prdtl = viscl(iel)*xcpp(iel)/visls_0
     endif
 
     do isou = 1, 6
@@ -486,12 +496,7 @@ if (iand(vcopt_ut%idften, ANISOTROPIC_RIGHT_DIFFUSION).ne.0) then
 else
 
   do iel = 1, ncel
-    if (irijco.eq.1) then
-      trrij = 0.5d0 * (cvar_rij(1,iel) + cvar_rij(2,iel) + cvar_rij(3,iel))
-    else
-      trrij = 0.5d0 * (cvar_r11(iel) + cvar_r22(iel) + cvar_r33(iel))
-    end if
-    rctse = crom(iel) * csrij * trrij**2 / cvar_ep(iel)
+    rctse = ctheta(iscal) * visct(iel) / cmu
     w1(iel) = viscl(iel) + vcopt%idifft*rctse
   enddo
 
@@ -528,8 +533,9 @@ call field_get_coefbf_v(f_id,cofbfv)
 
 iconvp = vcopt%iconv
 idiffp = vcopt%idiff
-ndircp = ndircl(ivar)
+ndircp = vcopt%ndircl
 nswrsp = vcopt%nswrsm
+imrgrp = vcopt%imrgra
 nswrgp = vcopt%nswrgr
 imligp = vcopt%imligr
 ircflp = vcopt%ircflu
@@ -555,7 +561,7 @@ init   = 1
 
 call coditv &
 (idtvar , init   , f_id   , iconvp , idiffp , ndircp ,          &
- imrgra , nswrsp , nswrgp , imligp , ircflp , ivisep ,          &
+ imrgrp , nswrsp , nswrgp , imligp , ircflp , ivisep ,          &
  ischcp , isstpp , iescap , idftnp , iswdyp ,                   &
  iwarnp ,                                                       &
  blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
@@ -586,14 +592,7 @@ deallocate(w1)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,'           Resolution pour la variable ',A23,/)
-
-#else
-
  1000 format(/,'           Solving variable ',A23           ,/)
-#endif
 
 !----
 ! End
