@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -48,7 +48,6 @@
 !> \param[in]     ncepdp        number of cells with head loss
 !> \param[in]     ncesmp        number of cells with mass source term
 !> \param[in]     ivar          variable number
-!> \param[in]     icepdc        index of cells with head loss
 !> \param[in]     icetsm        index of cells with mass source term
 !> \param[in]     itypsm        type of mass source term for each variable
 !>                               (see \ref cs_user_mass_source_terms)
@@ -58,7 +57,6 @@
 !> \param[in]     produc        work array for production
 !> \param[in]     gradro        work array for grad rom
 !>                              (without rho volume) only for iturb=30
-!> \param[in]     ckupdc        work array for the head loss
 !> \param[in]     smacel        value associated to each variable in the mass
 !>                               source terms or mass rate
 !>                               (see \ref cs_user_mass_source_terms)
@@ -72,10 +70,10 @@
 subroutine resssg2 &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    ivar   ,                                                       &
-   icepdc , icetsm , itypsm ,                                     &
+   icetsm , itypsm ,                                              &
    dt     ,                                                       &
    gradv  , produc , gradro ,                                     &
-   ckupdc , smacel ,                                              &
+   smacel ,                                                       &
    viscf  , viscb  ,                                              &
    tslagi ,                                                       &
    smbr   , rovsdt )
@@ -113,14 +111,13 @@ integer          nvar   , nscal
 integer          ncepdp , ncesmp
 integer          ivar
 
-integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
 double precision gradv(3, 3, ncelet)
 double precision produc(6,ncelet)
 double precision gradro(3,ncelet)
-double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
+double precision smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
 double precision tslagi(ncelet)
 double precision smbr(6,ncelet)
@@ -129,11 +126,11 @@ double precision rovsdt(6,6,ncelet)
 ! Local variables
 
 integer          iel, isou, jsou
-integer          ii    , jj    , kk    , iiun  , iii   , jjj
+integer          ii    , jj    , kk    , iii   , jjj
 integer          iflmas, iflmab
 integer          nswrgp, imligp, iwarnp
 integer          iconvp, idiffp, ndircp
-integer          nswrsp, ircflp, ischcp, isstpp
+integer          imrgrp, nswrsp, ircflp, ischcp, isstpp
 integer          st_prv_id
 integer          iprev , inc, iccocg, ll
 integer          idftnp, iswdyp
@@ -164,13 +161,15 @@ double precision rctse
 double precision eigen_max
 double precision eigen_vals(3)
 double precision turb_schmidt
-double precision gradchk, gradro_impl
+double precision gradchk, gradro_impl, const
 double precision kseps
 double precision matrn(6), oo_matrn(6)
 double precision ceps_impl, cphi3impl, cphiw_impl
 double precision impl_drsm(6,6)
 double precision implmat2add(3,3)
 double precision impl_lin_cst, impl_id_cst
+double precision grav(3)
+double precision gkks3
 
 character(len=80) :: label
 double precision, allocatable, dimension(:,:) :: grad
@@ -187,7 +186,7 @@ double precision, dimension(:,:,:), pointer :: coefbp, cofbfp
 double precision, dimension(:,:), pointer :: visten
 double precision, dimension(:), pointer :: cvara_ep, cvar_al
 double precision, dimension(:,:), pointer :: cvar_var, cvara_var
-double precision, dimension(:), pointer :: viscl
+double precision, dimension(:), pointer :: viscl, visct
 double precision, dimension(:,:), pointer:: c_st_prv, lagr_st_rij
 double precision, dimension(:,:), pointer :: cpro_buoyancy
 
@@ -242,6 +241,7 @@ endif
 
 call field_get_val_s(icrom, crom)
 call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 
 call field_get_val_prev_s(ivarfl(iep), cvara_ep)
 if (iturb.ne.31) call field_get_val_s(ivarfl(ial), cvar_al)
@@ -283,19 +283,6 @@ else
   call field_get_val_s(icrom, cromo)
 endif
 
-do isou = 1, 6
-  do iel = 1, ncel
-    smbr(isou,iel) = 0.d0
-  enddo
-enddo
-do iel = 1, ncel
-  do isou = 1, 6
-    do jsou = 1, 6
-      rovsdt(isou,jsou,iel) = 0.d0
-    enddo
-  enddo
-enddo
-
 if (icorio.eq.1 .or. iturbo.eq.1) then
 allocate(cvara_r(3,3))
 
@@ -313,17 +300,11 @@ endif
 !===============================================================================
 ! 2. User source terms
 !===============================================================================
-call cs_user_turbulence_source_terms2 &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivarfl(ivar)    ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
-   ckupdc , smacel ,                                              &
-   smbr   , rovsdt )
 
-  !     If we extrapolate the source terms
-do isou = 1, dimrij
-  if (st_prv_id.ge.0) then
-    do iel = 1, ncel
+! If we extrapolate the source terms
+if (st_prv_id.ge.0) then
+  do iel = 1, ncel
+    do isou = 1, dimrij
       ! Save for exchange
       tuexpr = c_st_prv(isou,iel)
       ! For continuation and the next time step
@@ -331,17 +312,23 @@ do isou = 1, dimrij
       ! Second member of the previous time step
       ! We suppose -rovsdt > 0: we implicite
       !    the user source term (the rest)
-      smbr(isou,iel) = rovsdt(isou,isou,iel)*cvara_var(isou,iel)  - thets*tuexpr
-      ! Diagonal
-      rovsdt(isou,isou,iel) = - thetv*rovsdt(isou,isou,iel)
+      do jsou = 1, dimrij
+        smbr(isou,iel) = rovsdt(jsou,isou,iel)*cvara_var(jsou,iel)  - thets*tuexpr
+        ! Diagonal
+        rovsdt(jsou,isou,iel) = - thetv*rovsdt(jsou,isou,iel)
+      enddo
     enddo
-  else
-    do iel = 1, ncel
-      smbr(isou,iel)   = rovsdt(isou,isou,iel)*cvara_var(isou,iel) + smbr(isou,iel)
-      rovsdt(isou,isou,iel) = max(-rovsdt(isou,isou,iel),zero)
+  enddo
+else
+  do iel = 1, ncel
+    do isou = 1, dimrij
+      do jsou = 1, dimrij
+        smbr(isou,iel)   = rovsdt(jsou,isou,iel)*cvara_var(jsou,iel) + smbr(isou,iel)
+      enddo
+      rovsdt(isou,isou,iel) = max(-rovsdt(isou,isou,iel), 0.d0)
     enddo
-  endif
-enddo
+  enddo
+endif
 
 !===============================================================================
 ! 3. Lagrangian source terms
@@ -365,15 +352,11 @@ endif
 if (ncesmp.gt.0) then
 
   do isou = 1, dimrij
-    !       Integer equal to 1 (for navsto: nb of sur-iter)
-    iiun = 1
 
     ! We increment smbr with -Gamma.var_prev and rovsdr with Gamma
-    call catsmt &
-   ( ncelet , ncel   , ncesmp , iiun     , isto2t ,                   &
-     icetsm , itypsm(:,ivar + isou - 1)  ,                            &
-     cell_f_vol , cvara_var  , smacel(:,ivar+isou-1)  , smacel(:,ipr) ,   &
-     smbr   ,  rovsdt    , gatinj )
+    call catsmt(ncesmp, 1, icetsm, itypsm(:,ivar+isou-1),                     &
+                cell_f_vol, cvara_var, smacel(:,ivar+isou-1), smacel(:,ipr),  &
+                smbr, rovsdt, gatinj)
 
    ! If we extrapolate the source terms we put Gamma Pinj in the previous st
     if (st_prv_id.ge.0) then
@@ -424,9 +407,7 @@ if (iturb.eq.32) then
   inc    = 1
   iccocg = 1
 
-  call field_gradient_scalar(ivarfl(ial), iprev, imrgra, inc,     &
-                             iccocg,                              &
-                             grad)
+  call field_gradient_scalar(ivarfl(ial), iprev, 0, inc, iccocg, grad)
 
 endif
 
@@ -841,6 +822,9 @@ endif
 
 if (igrari.eq.1) then
 
+  grav(1) = gx
+  grav(2) = gy
+  grav(3) = gz
   call field_get_id_try("rij_buoyancy", f_id)
   if (f_id.ge.0) then
     call field_get_val_v(f_id, cpro_buoyancy)
@@ -872,17 +856,9 @@ if (igrari.eq.1) then
   ! Implicit buoyancy term
   if (iscalt.gt.0 .and. nscal.ge.iscalt) then
     call field_get_key_double(ivarfl(isca(iscalt)), ksigmas, turb_schmidt)
-    if (iturb .eq. 32) then
-      gradro_impl = -1.5d0*cmu/turb_schmidt * (1.0d0-cebmr6)
-    else
-      gradro_impl = -1.5d0*cmu/turb_schmidt * (1.0d0-crij3)
-    end if
+    const = -1.5d0 * cmu / turb_schmidt
   else
-    if (iturb .eq. 32) then
-      gradro_impl = -1.5d0*cmu*(1.0d0-cebmr6)
-    else
-      gradro_impl = -1.5d0*cmu*(1.0d0-crij3)
-    end if
+    const = -1.5d0 * cmu
   end if
 
   do iel = 1, ncel
@@ -894,17 +870,65 @@ if (igrari.eq.1) then
     end do
     implmat2add(:,:) = 0.0d0
 
-    gradchk = gx*gradro(1,iel) + gy*gradro(2,iel) + gz*gradro(3,iel)
-    if (gradchk .gt. 0.0d0) then
-      kseps = (cvara_var(1,iel) + cvara_var(2,iel) + cvara_var(3,iel)) &
-              / (2.0d0*cvara_ep(iel))
+    kseps = (cvara_var(1,iel) + cvara_var(2,iel) + cvara_var(3,iel)) &
+      / (2.0d0*cvara_ep(iel))
 
-      do jsou = 1, 3
-        implmat2add(1,jsou) = kseps * gradro_impl * gx * gradro(jsou,iel)
-        implmat2add(2,jsou) = kseps * gradro_impl * gy * gradro(jsou,iel)
-        implmat2add(3,jsou) = kseps * gradro_impl * gz * gradro(jsou,iel)
+    xrij(1,1) = cvara_var(1,iel)
+    xrij(2,2) = cvara_var(2,iel)
+    xrij(3,3) = cvara_var(3,iel)
+    xrij(1,2) = cvara_var(4,iel)
+    xrij(2,3) = cvara_var(5,iel)
+    xrij(1,3) = cvara_var(6,iel)
+    xrij(2,1) = xrij(1,2)
+    xrij(3,1) = xrij(1,3)
+    xrij(3,2) = xrij(2,3)
+
+    trrij = 0.5d0 * (xrij(1,1) + xrij(2,2) + xrij(3,3))
+
+    gkks3 = 0.d0
+    do jsou = 1, 3
+      do isou = 1, 3
+        gkks3 = gkks3 + grav(isou) * gradro(jsou,iel) * xrij(isou, jsou)
+      enddo
+    enddo
+    gkks3 = const * kseps * gkks3 * crij3 * 2.d0 / 3.d0
+
+    if (gkks3.le.0.d0) then
+      ! Term "C3 tr(G) Id"
+      ! Computing the inverse matrix of R^n
+      ! Scaling by tr(R) in order to dodge inversion errors
+      do isou = 1, 6
+        matrn(isou) = cvara_var(isou,iel)/trrij
+        oo_matrn(isou) = 0.0d0
       end do
 
+      ! Inversing the matrix
+      call symmetric_matrix_inverse(matrn, oo_matrn)
+      do isou = 1, 6
+        oo_matrn(isou) = oo_matrn(isou)/trrij
+      end do
+
+      do jsou = 1, 3
+        do isou = 1, 3
+          iii = t2v(isou,jsou)
+          implmat2add(isou,jsou) = - 0.5d0 * gkks3 * oo_matrn(iii)
+        end do
+      end do
+
+    endif
+
+    gradchk = grav(1)*gradro(1,iel) + grav(2)*gradro(2,iel) + grav(3)*gradro(3,iel)
+    if (gradchk .gt. 0.0d0) then
+
+      ! Implicit term written as:
+      !   Po . R^n+1 + R^n+1 . Po^t
+      ! with Po proportional to "g (x) Grad rho"
+      gradro_impl = const * (1.0d0-crij3) * kseps
+      do jsou = 1, 3
+        do isou = 1, 3
+          implmat2add(isou,jsou) = implmat2add(isou,jsou) - gradro_impl * grav(isou) * gradro(jsou,iel)
+        end do
+      end do
     end if
 
     ! Compute the 6x6 matrix A which verifies
@@ -913,7 +937,9 @@ if (igrari.eq.1) then
 
     do isou = 1, dimrij
       do jsou = 1, dimrij
-        rovsdt(jsou,isou,iel) = rovsdt(jsou,isou,iel) - cell_f_vol(iel) &
+        ! Carefull ! Inversion of the order of the coefficients since
+        ! rovsdt matrix is then used by a c function for the linear solving
+        rovsdt(jsou,isou,iel) = rovsdt(jsou,isou,iel) + cell_f_vol(iel) &
                                 * impl_drsm(isou,jsou)
       end do
     end do
@@ -951,8 +977,7 @@ if (iand(vcopt%idften, ANISOTROPIC_RIGHT_DIFFUSION).ne.0) then
 else
 
   do iel = 1, ncel
-    trrij = 0.5d0 * (cvara_var(1,iel) + cvara_var(2,iel) + cvara_var(3,iel))
-    rctse = crom(iel) * csrij * trrij**2 / cvara_ep(iel)
+    rctse = csrij * visct(iel) / cmu
     w1(iel) = viscl(iel) + vcopt%idifft*rctse
   enddo
 
@@ -978,7 +1003,8 @@ endif
 
 iconvp = vcopt%iconv
 idiffp = vcopt%idiff
-ndircp = ndircl(ivar)
+ndircp = vcopt%ndircl
+imrgrp = vcopt%imrgra
 nswrsp = vcopt%nswrsm
 nswrgp = vcopt%nswrgr
 imligp = vcopt%imligr
@@ -1005,7 +1031,7 @@ call field_get_coefbf_v(ivarfl(ivar), cofbfp)
 
 call coditts &
  ( idtvar , ivarfl(ivar)    , iconvp , idiffp , ndircp ,          &
-   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , idftnp , iswdyp ,                            &
    iwarnp ,                                                       &
    blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
@@ -1028,15 +1054,7 @@ deallocate(weighf, weighb)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,'           Resolution pour la variable ',A8,/)
-
-#else
-
- 1000 format(/,'           Solving variable ',A8           ,/)
-
-#endif
+ 1000 format(/,'           Solving variable ', a8          ,/)
 
 !----
 ! End
