@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,15 +36,11 @@
 !  mode           name          role
 !______________________________________________________________________________!
 !> \param[in]     nvar          total number of variables
-!> \param[in]     nscal         total number of scalars
-!> \param[in]     ncepdp        number of cells with head loss
 !> \param[in]     ncesmp        number of cells with mass source term
-!> \param[in]     icepdc        number of ncepdp cells with head losses
 !> \param[in]     icetsm        number of cells with mass source
 !> \param[in]     itypsm        type of masss source for the
 !>                              variables (cf. cs_user_mass_source_terms)
 !> \param[in]     dt            time step (per cell)
-!> \param[in]     ckupdc        work array for head losses
 !> \param[in]     smacel        value of variables associated to the
 !>                              mass source
 !>                              for ivar=ipr,smacel=flux of masse
@@ -54,10 +50,10 @@
 
 
 subroutine resv2f &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   icepdc , icetsm , itypsm ,                                     &
+ ( nvar   , ncesmp ,                                              &
+   icetsm , itypsm ,                                              &
    dt     ,                                                       &
-   ckupdc , smacel ,                                              &
+   smacel ,                                                       &
    prdv2f )
 
 !===============================================================================
@@ -83,23 +79,21 @@ implicit none
 
 ! Arguments
 
-integer          nvar   , nscal
-integer          ncepdp , ncesmp
+integer          nvar
+integer          ncesmp
 
-integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
-double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
+double precision smacel(ncesmp,nvar)
 double precision prdv2f(ncelet)
 
 ! Local variables
 
 integer          init  , ifac  , iel   , inc   , iprev , iccocg
 integer          ivar
-integer          iiun
 integer          iflmas, iflmab
-integer          nswrgp, imligp, iwarnp, iphydp
+integer          imrgrp, nswrgp, imligp, iwarnp, iwgrp, iphydp
 integer          iconvp, idiffp, ndircp
 integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          istprv
@@ -109,7 +103,7 @@ integer          ivoid(1)
 integer          key_t_ext_id
 integer          iroext
 integer          iviext
-double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
+double precision blencp, epsilp, epsrgp, climgp, relaxp, extrap
 double precision epsrsp
 double precision tuexpe, thets , thetv , thetap, thetp1
 double precision d2s3
@@ -117,6 +111,8 @@ double precision xk, xe, xnu, xrom, ttke, ttmin, llke, llmin, tt
 double precision fhomog
 double precision hint
 double precision l2, time_scale
+double precision normp
+double precision sigmak
 
 double precision rvoid(1)
 
@@ -146,6 +142,8 @@ type(var_cal_opt) :: vcopt
 ! 1. Initialization
 !===============================================================================
 
+extrap = 0
+
 ! Allocate temporary arrays for the turbulence resolution
 allocate(viscf(nfac), viscb(nfabor))
 allocate(smbr(ncelet), rovsdt(ncelet))
@@ -164,6 +162,7 @@ call field_get_val_s(iflmas, imasfl)
 call field_get_val_s(iflmab, bmasfl)
 
 call field_get_val_prev_s(ivarfl(ik), cvara_k)
+call field_get_key_double(ivarfl(ik), ksigmas, sigmak)
 call field_get_val_prev_s(ivarfl(iep), cvara_ep)
 call field_get_val_prev_s(ivarfl(iphi), cvara_phi)
 if (iturb.eq.50) then
@@ -219,17 +218,13 @@ inc = 1
 iprev = 1
 ivar = iphi
 
-call field_gradient_scalar(ivarfl(ivar), iprev, imrgra, inc,      &
-                           iccocg,                                &
-                           gradp)
+call field_gradient_scalar(ivarfl(ivar), iprev, 0, inc, iccocg, gradp)
 
 iccocg = 1
 inc = 1
 ivar = ik
 
-call field_gradient_scalar(ivarfl(ik), iprev, imrgra, inc,        &
-                           iccocg,                                &
-                           gradk)
+call field_gradient_scalar(ivarfl(ik), iprev, 0, inc, iccocg, gradk)
 
 do iel = 1, ncel
   w1(iel) = gradp(1,iel)*gradk(1,iel) &
@@ -290,12 +285,7 @@ enddo
 ! 3.1 User source terms
 !===============================================================================
 
-call cs_user_turbulence_source_terms &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivarfl(ivar)    ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
-   ckupdc , smacel ,                                              &
-   smbr   , rovsdt )
+call user_source_terms(ivarfl(ivar), smbr, rovsdt)
 
 !     If we extrapolate the source terms
 if (istprv.ge.0) then
@@ -374,18 +364,19 @@ init = 1
 
 call field_get_key_struct_var_cal_opt(ivarfl(iphi), vcopt)
 
+imrgrp = vcopt%imrgra
 nswrgp = vcopt%nswrgr
 imligp = vcopt%imligr
+iwgrp  = vcopt%iwgrec
 iwarnp = vcopt%iwarni
 epsrgp = vcopt%epsrgr
 climgp = vcopt%climgr
-extrap = vcopt%extrag
 iphydp = 0
 
 call itrgrp &
- ( ivarfl(iphi)    , init   , inc    , imrgra ,                      &
+ ( ivarfl(iphi)    , init   , inc    , imrgrp ,                      &
    iccocg , nswrgp , imligp , iphydp ,                               &
-   iwarnp ,                                                          &
+   iwgrp  , iwarnp ,                                                 &
    epsrgp , climgp , extrap ,                                        &
    rvoid  ,                                                          &
    cvara_phi       ,                                                 &
@@ -472,7 +463,8 @@ call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
 
 iconvp = vcopt%iconv
 idiffp = vcopt%idiff
-ndircp = ndircl(ivar)
+ndircp = vcopt%ndircl
+imrgrp = vcopt%imrgra
 nswrsp = vcopt%nswrsm
 nswrgp = vcopt%nswrgr
 imligp = vcopt%imligr
@@ -489,10 +481,10 @@ epsilp = vcopt%epsilo
 epsrsp = vcopt%epsrsm
 epsrgp = vcopt%epsrgr
 climgp = vcopt%climgr
-extrap = vcopt%extrag
 relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
 
 call field_get_coefa_s(ivarfl(ivar), coefap)
 call field_get_coefb_s(ivarfl(ivar), coefbp)
@@ -501,10 +493,10 @@ call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 call codits &
  ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
-   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   iwarnp ,                                                       &
-   blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
+   iwarnp , normp  ,                                              &
+   blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
    relaxp , thetv  ,                                              &
    cvara_var       , cvara_var       ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
@@ -554,12 +546,7 @@ enddo
 ! 4.1 User source terms
 !===============================================================================
 
-call cs_user_turbulence_source_terms &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivarfl(ivar)    ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
-   ckupdc , smacel ,                                              &
-   smbr   , rovsdt )
+call user_source_terms(ivarfl(ivar), smbr, rovsdt)
 
 !     If we extrapolate the source terms
 if (istprv.ge.0) then
@@ -589,15 +576,10 @@ endif
 
 if (ncesmp.gt.0) then
 
-  !       Integer equal to 1 (for navsto: nb of over-iter)
-  iiun = 1
-
   !       We increment smbr by -Gamma.var_prev and rovsdt by Gamma
-  call catsma                                                     &
- ( ncelet , ncel   , ncesmp , iiun   , isto2t ,                   &
-   icetsm , itypsm(1,ivar) ,                                      &
-   cell_f_vol , cvara_var  , smacel(1,ivar) , smacel(1,ipr) ,     &
-   smbr   ,  rovsdt , w2 )
+  call catsma(ncesmp, 1, icetsm, itypsm(:,ivar),                     &
+              cell_f_vol, cvara_var, smacel(:,ivar), smacel(:,ipr),  &
+              smbr, rovsdt, w2)
 
   ! If we extrapolate the source term we put Gamma Pinj in the prev. TS
   if (istprv.ge.0) then
@@ -790,9 +772,10 @@ endif
 
 iconvp = vcopt%iconv
 idiffp = vcopt%idiff
-ndircp = ndircl(ivar)
+ndircp = vcopt%ndircl
 nswrsp = vcopt%nswrsm
 nswrgp = vcopt%nswrgr
+imrgrp = vcopt%imrgra
 imligp = vcopt%imligr
 ircflp = vcopt%ircflu
 ischcp = vcopt%ischcv
@@ -807,10 +790,10 @@ epsilp = vcopt%epsilo
 epsrsp = vcopt%epsrsm
 epsrgp = vcopt%epsrgr
 climgp = vcopt%climgr
-extrap = vcopt%extrag
 relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
 
 call field_get_coefa_s(ivarfl(ivar), coefap)
 call field_get_coefb_s(ivarfl(ivar), coefbp)
@@ -819,10 +802,10 @@ call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 call codits &
  ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
-   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   iwarnp ,                                                       &
-   blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
+   iwarnp , normp  ,                                              &
+   blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
    relaxp , thetv  ,                                              &
    cvara_var       , cvara_var       ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
@@ -852,22 +835,11 @@ deallocate(dpvar)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000    format(/,                                         &
-'   ** Resolution pour V2F (phi et f_bar/alpha)               ',/,&
-'      ----------------------------------------        ',/)
- 1100    format(/,'           Resolution pour la variable ',A8,/)
-
-#else
-
  1000    format(/,                                         &
 '   ** Solving V2F (phi and f_bar/alpha)'               ,/,&
 '      ---------------------------------'               ,/)
  1100    format(/,'           Solving variable ',A8                  ,/)
 
-#endif
-
 !----
 ! End
 !----
