@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -37,13 +37,13 @@
 !  mode           name          role
 !______________________________________________________________________________!
 !> \param[in]     nscal         total number of scalars
-!> \param[in]     gradro        work array for \f$ \grad{rom} \f$
-!> \param[in,out] smbr          work array for second member
+!> \param[in]     gradro        work array for \f$ \grad{\rho} \f$
+!> \param[in,out] buoyancy      Buoyancy term for the Reynolds stress model
 !______________________________________________________________________________!
 
 subroutine rijthe2 &
  ( nscal  ,                                                       &
-   gradro , smbr   )
+   gradro , buoyancy   )
 
 !===============================================================================
 ! Module files
@@ -66,15 +66,14 @@ implicit none
 integer          nscal
 
 double precision gradro(3,ncelet)
-double precision smbr(6,ncelet)
+double precision buoyancy(6,ncelet)
 
 ! Local variables
 
 integer          iel, dimrij, isou, i, j
 double precision dij
 
-double precision uns3, const, kseps, csttmp
-double precision prdtur
+double precision uns3, const, kseps
 double precision rit(3), gij(3, 3), grav(3)
 double precision gkks3
 double precision turb_schmidt
@@ -88,21 +87,13 @@ double precision, dimension(:,:), pointer :: cvara_rij
 ! 1. Initialization
 !===============================================================================
 
-! ebrsm
-if (iturb.eq.32) then
-  csttmp = cebmr6
-else
-  csttmp = crij3
-endif
-
 if(iscalt.gt.0.and.nscal.ge.iscalt) then
   call field_get_key_double(ivarfl(isca(iscalt)), ksigmas, turb_schmidt)
-  prdtur = turb_schmidt
+  const = -1.5d0 * cmu / turb_schmidt
 else
-  prdtur = 1.d0
+  const = -1.5d0 * cmu
 endif
 
-const = -1.5d0*cmu/prdtur
 uns3  = 1.d0/3.d0
 
 grav(1) = gx
@@ -119,7 +110,7 @@ call field_get_dim(ivarfl(irij), dimrij)
 ! 2. Terms for Rij:
 !      rom*volume*dRij/dt =
 !                     ... + (Gij - crij3*(Gij-Delta ij Gkk/3))*volume
-!            With Gij = -(1.5 cmu/prdtur) (k/eps) (Rit Gj + Rjt Gi)
+!            With Gij = -(1.5 cmu/PrT) (k/eps) (Rit Gj + Rjt Gi)
 !                 Rit = Rik drom/dxk (sum on k)
 !            Note that in tensorial notation Gij is:
 !                 G = a [R.( Grho x g) + (g x Grho).R]
@@ -174,9 +165,7 @@ do iel = 1, ncel
       dij = 0.0d0
     endif
 
-    smbr(isou,iel) = smbr(isou,iel)+ ( gij(i,j) * (1.d0 - csttmp) &
-                                     + csttmp * dij * gkks3       &
-                                     )* volume(iel)
+    buoyancy(isou,iel) = gij(i,j) * (1.d0 - crij3) + crij3 * dij * gkks3
 
   enddo
 enddo
@@ -184,28 +173,6 @@ enddo
 return
 
 end subroutine rijthe2
-!-------------------------------------------------------------------------------
-
-! This file is part of Code_Saturne, a general-purpose CFD tool.
-!
-! Copyright (C) 1998-2018 EDF S.A.
-!
-! This program is free software; you can redistribute it and/or modify it under
-! the terms of the GNU General Public License as published by the Free Software
-! Foundation; either version 2 of the License, or (at your option) any later
-! version.
-!
-! This program is distributed in the hope that it will be useful, but WITHOUT
-! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-! details.
-!
-! You should have received a copy of the GNU General Public License along with
-! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
-! Street, Fifth Floor, Boston, MA 02110-1301, USA.
-
-!-------------------------------------------------------------------------------
-
 
 !===============================================================================
 ! Function:
@@ -221,7 +188,7 @@ end subroutine rijthe2
 !  mode           name          role
 !______________________________________________________________________________!
 !> \param[in]     nscal         total number of scalars
-!> \param[in]     gradro        work array for \f$ \grad{rom} \f$
+!> \param[in]     gradro        work array for \f$ \grad{\rho} \f$
 !> \param[in,out] smbr          work array for second member
 !______________________________________________________________________________!
 
@@ -256,8 +223,8 @@ double precision smbr(ncelet)
 
 integer          iel
 
-double precision uns3, const, csttmp
-double precision prdtur, r1t, r2t, r3t
+double precision uns3, const
+double precision r1t, r2t, r3t
 double precision g11p, g22p, g33p
 double precision aa, bb
 double precision turb_schmidt
@@ -271,21 +238,13 @@ double precision, dimension(:,:), pointer :: cvara_rij
 ! 1. Initialization
 !===============================================================================
 
-! ebrsm
-if (iturb.eq.32) then
-  csttmp = cebmr6
-else
-  csttmp = crij3
-endif
-
 if(iscalt.gt.0.and.nscal.ge.iscalt) then
   call field_get_key_double(ivarfl(isca(iscalt)), ksigmas, turb_schmidt)
-  prdtur = turb_schmidt
+  const = -1.5d0 * cmu / turb_schmidt
 else
-  prdtur = 1.d0
+  const = -1.5d0 * cmu
 endif
 
-const = -1.5d0*cmu/prdtur
 uns3  = 1.d0/3.d0
 
 call field_get_val_prev_s(ivarfl(iep), cvara_ep)
@@ -296,15 +255,14 @@ call field_get_val_prev_v(ivarfl(irij), cvara_rij)
 ! 2. Terms for epsilon:
 !      rom*volumr*deps/dt =
 !                     ... + CEPS1*(EPS/K)*Max(0,(Gkk/2))*volume
-!            With Gij = -(1.5 cmu/prdtur) (k/eps) (Rit Gj + Rjt Gi)
+!            With Gij = -(1.5 cmu/PrT) (k/eps) (Rit Gj + Rjt Gi)
 !                 Rit = Rik drom/dxk (sum on k)
 !            We simplify (eps/k) by noting
-!                GijP = -(1.5 cmu/prdtur)         (Rit Gj + Rjt Gi)
+!                GijP = -(1.5 cmu/PrT)         (Rit Gj + Rjt Gi)
 !      rom*volume*deps/dt =
 !                     ... + CEPS1*        Max(0,(GkkP/2))*volume
 !===============================================================================
 
-
 do iel = 1, ncel
 
   r1t = cvara_rij(1,iel)*gradro(1,iel)                            &
@@ -321,9 +279,10 @@ do iel = 1, ncel
   g22p = const*2.d0*(r2t*gy)
   g33p = const*2.d0*(r3t*gz)
 
+  !FIXME for EB-DFM and EBRSM
   aa = 0.d0
   bb = 0.5d0*(g11p+g22p+g33p)
-  smbr(iel) = smbr(iel) + ce1*max(aa,bb)*volume(iel)
+  smbr(iel) = ce1*max(aa,bb)
 
 enddo
 
