@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -39,15 +39,11 @@
 !  mode           name          role
 !______________________________________________________________________________!
 !> \param[in]     nvar          total number of variables
-!> \param[in]     nscal         total number of scalars
-!> \param[in]     ncepdp        number of cells with head loss
 !> \param[in]     ncesmp        number of cells with mass source term
-!> \param[in]     icepdc        number of ncepdp cells with head losses
 !> \param[in]     icetsm        number of cells with mass source
 !> \param[in]     itypsm        type of mass source for the variables
 !>                               (cf. cs_user_mass_source_terms)
 !> \param[in]     dt            time step (per cell)
-!> \param[in]     ckupdc        work array for head losses
 !> \param[in]     smacel        value of variables associated to the
 !                               mass source
 !                               for ivar = ipr, smacel = mass flux
@@ -55,10 +51,10 @@
 !______________________________________________________________________________!
 
 subroutine turbsa &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   icepdc , icetsm , itypsm ,                                     &
+ ( nvar   , ncesmp ,                                              &
+   icetsm , itypsm ,                                              &
    dt     ,                                                       &
-   ckupdc , smacel ,                                              &
+   smacel ,                                                       &
    itypfb )
 
 !===============================================================================
@@ -83,23 +79,21 @@ implicit none
 
 ! Arguments
 
-integer          nvar   , nscal
-integer          ncepdp , ncesmp
+integer          nvar
+integer          ncesmp
 
-integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 integer          itypfb(nfabor)
 integer          icvflb
 integer          ivoid(1)
 
 double precision dt(ncelet)
-double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
+double precision smacel(ncesmp,nvar)
 
 ! Local variables
 
 integer          iel   , ifac  , inc   , iccocg, iprev, ivar
-integer          iiun
-integer          nswrgp, imligp
+integer          imrgrp, nswrgp, imligp
 integer          iconvp, idiffp, ndircp
 integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          iflmas, iflmab
@@ -123,6 +117,7 @@ double precision cofbnu
 double precision chi  , chi3, taussa, nusa, distbf, fw, fv1, fv2
 double precision gsa , rsa , dsigma, cv13
 double precision nu0, dsa0, hssa, omega, sbar, cst2, cst3
+double precision normp
 
 double precision rvoid(1)
 
@@ -146,8 +141,6 @@ double precision, dimension(:), pointer :: w_dist
 
 type(var_cal_opt) :: vcopt_nusa
 
-!===============================================================================
-
 !===============================================================================
 ! 1. Initialization
 !===============================================================================
@@ -231,8 +224,7 @@ allocate(gradv(3, 3, ncelet))
 inc = 1
 iprev = 1
 
-call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc,       &
-                           gradv)
+call field_gradient_vector(ivarfl(iu), iprev, 0, inc, gradv)
 
 ! vort = omega**2 = dudy**2 + dvdx**2 + dudz**2 + dwdx**2 + dvdz**2 + dwdy**2
 !                - 2*dudy*dvdx - 2*dudz*dwdx - 2*dvdz*dwdy
@@ -257,9 +249,7 @@ allocate(grad(3,ncelet))
 
 iccocg = 1
 
-call field_gradient_scalar(ivarfl(inusa), iprev, imrgra, inc,       &
-                           iccocg,                                  &
-                           grad)
+call field_gradient_scalar(ivarfl(inusa), iprev, 0, inc, iccocg, grad)
 
 ! trgrdn = GRAD(nusa)**2
 do iel = 1, ncel
@@ -321,7 +311,7 @@ if (irccor.eq.1) then
   allocate(rotfct(ncel))
 
   ! Compute the rotation function (w1 array not used)
-  call rotcor(dt, rotfct, w1)
+  call cs_turbulence_rotation_correction(dt, rotfct, w1)
 
   do iel = 1, ncel
     csab1r(iel) = csab1*rotfct(iel)
@@ -379,13 +369,13 @@ do iel = 1, ncel
   gsa   = rsa + csaw2*(rsa**6-rsa)
   fw    = gsa*( (1.d0+csaw3**6)/(gsa**6+csaw3**6))**(1.d0/6.d0)
 
-  rhssa(iel) = volume(iel)*rom*(                                 &
+  rhssa(iel) = cell_f_vol(iel)*rom*(                                 &
      dsigma * csab2*trgrdn(iel)+csab1r(iel)*taussa*nusa-csaw1*fw*(nusa/distbf)**2)
 
   ! Implicitation of the negative source terms of the SA equation.
   ! NB: this term could be negative, and if so, then we explicit it.
   tinssa(iel) = (max(csaw1*fw*nusa/distbf**2-csab1r(iel)*taussa,0.d0)         &
-                      )*rom*volume(iel)
+                      )*rom*cell_f_vol(iel)
 
 enddo
 
@@ -405,12 +395,7 @@ do iel = 1, ncel
   tsexp (iel) = 0.d0
 enddo
 
-call cs_user_turbulence_source_terms &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivarfl(inusa)   ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
-   ckupdc , smacel ,                                              &
-   tsexp  , tsimp )
+call user_source_terms(ivarfl(inusa), tsexp, tsimp)
 
 !===============================================================================
 ! 6. User source terms and d/dt(rho) and div(rho u) are taken into account
@@ -456,7 +441,7 @@ endif
 !     Extrapolated or not in coherence with bilsc2
 do iel = 1, ncel
   rom = crom(iel)
-  romvsd = rom*volume(iel)/dt(iel)
+  romvsd = rom*cell_f_vol(iel)/dt(iel)
 
   ! tinssa already contains the negativ implicited source term
   tinssa(iel) = tinssa(iel)                                        &
@@ -478,19 +463,13 @@ enddo
 
 if (ncesmp.gt.0) then
 
-  ! Integer equal to 1. (in navsto: nb of sub-iteration)
-  iiun = 1
-
   ! --- Explicit and Implicit part
   !     -Gamma.var_prev is added to the RHS and Gamma to tinssa
   ivar = inusa
 
-  call catsma &
- ( ncelet , ncel   , ncesmp , iiun   ,                            &
-   isto2t ,                                                       &
-   icetsm , itypsm(1,ivar) ,                                      &
-   volume , cvara_nusa     , smacel(1,ivar) , smacel(1,ipr) ,     &
-   rhssa  , tinssa , w1 )
+  call catsma(ncesmp, 1, icetsm, itypsm(:,ivar),                      &
+              cell_f_vol, cvara_nusa, smacel(:,ivar), smacel(:,ipr),  &
+              rhssa, tinssa, w1)
 
   ! --- Explicit part: Gamma Pinj
   !     (if we extrapolate source terms, Gamma.var_prev is stored in prev. TS)
@@ -557,8 +536,9 @@ endif
 
 iconvp = vcopt_nusa%iconv
 idiffp = vcopt_nusa%idiff
-ndircp = ndircl(ivar)
+ndircp = vcopt_nusa%ndircl
 nswrsp = vcopt_nusa%nswrsm
+imrgrp = vcopt_nusa%imrgra
 nswrgp = vcopt_nusa%nswrgr
 imligp = vcopt_nusa%imligr
 ircflp = vcopt_nusa%ircflu
@@ -579,14 +559,15 @@ relaxp = vcopt_nusa%relaxv
 thetap = vcopt_nusa%thetav
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
 init   = 1
 
 call codits &
  ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
-   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+   imrgrp , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   iwarnp ,                                                       &
-   blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
+   iwarnp , normp  ,                                              &
+   blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
    relaxp , thetap ,                                              &
    cvara_nusa      , cvara_nusa      ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
@@ -617,17 +598,9 @@ deallocate(dpvar)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,                                                   &
-'   ** Resolution de Spalart-Allmaras                         ',/,&
-'      ------------------------------------                   ',/)
-#else
-
  1000 format(/,                                                   &
 '   ** Solving Spalart-Allmaras      '                         ,/,&
 '      ------------------------------'                         ,/)
-#endif
 
 !----
 ! End
