@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -98,7 +98,7 @@ integer          icetsm(ncesmp)
 integer, dimension(ncesmp,nvar), target :: itypsm
 
 double precision dt(ncelet)
-double precision ckupdc(ncepdp,6)
+double precision ckupdc(6,ncepdp)
 
 double precision, dimension(ncesmp,nvar), target ::  smacel
 double precision, dimension(ncelet,ntersl), target :: tslagr
@@ -106,21 +106,26 @@ double precision, dimension(:), pointer :: bromo, cromo
 
 ! Local variables
 
-integer          ifac  , iel   , ivar  , isou  , ii
+integer          ifac  , iel   , ivar  , isou, jsou
 integer          inc   , iccocg
 integer          iwarnp, iclip
-integer          nswrgp, imligp
-integer          f_id0
+integer          imrgrp, nswrgp, imligp
+integer          f_id0 , f_id
 integer          iprev
+integer          key_t_ext_id
+integer          iroext
+integer          f_id_phij
 double precision epsrgp, climgp, extrap
 double precision rhothe
 double precision utaurf,ut2,ypa,ya,tke,xunorm, limiter, nu0,alpha
 double precision xnoral, xnal(3)
+double precision k,P
+double precision d1s3,d2s3
 
 double precision, allocatable, dimension(:) :: viscf, viscb
 double precision, allocatable, dimension(:) :: smbr, rovsdt
 double precision, allocatable, dimension(:,:,:) :: gradv
-double precision, allocatable, dimension(:,:) :: produc
+double precision, allocatable, dimension(:,:), target :: produc
 double precision, allocatable, dimension(:,:) :: gradro
 double precision, allocatable, dimension(:,:) :: grad
 double precision, allocatable, dimension(:,:) :: smbrts
@@ -138,6 +143,9 @@ double precision, dimension(:), pointer :: cvara_scalt
 double precision, dimension(:), pointer :: cvar_ep, cvar_al
 double precision, dimension(:,:), pointer :: cvara_rij, cvar_rij, vel
 double precision, dimension(:,:), pointer :: lagr_st_rij
+double precision, dimension(:,:), pointer :: cpro_produc
+double precision, dimension(:,:), pointer :: cpro_press_correl
+double precision, dimension(:), pointer :: cpro_beta
 
 type(var_cal_opt) :: vcopt
 
@@ -162,24 +170,32 @@ if (irijco.eq.1) then
 endif
 
 ! Allocate other arrays, depending on user options
-if (iturb.eq.30) then
+call field_get_id_try("rij_production", f_id)
+if (f_id.ge.0) then
+  call field_get_val_v(f_id, cpro_produc)
+else
   allocate(produc(6,ncelet))
+  cpro_produc => produc
 endif
 
+call field_get_id_try("rij_pressure_strain_correlation", f_id_phij)
+if (f_id_phij.ge.0) then
+  call field_get_val_v(f_id_phij, cpro_press_correl)
+endif
+call field_get_val_s(ivarfl(iep), cvar_ep)
+
 call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
 
-if (iturb.eq.30) then
-  if (irijco.eq.1) then
-    call field_get_val_prev_v(ivarfl(irij), cvara_rij)
-  else
-    call field_get_val_prev_s(ivarfl(ir11), cvara_r11)
-    call field_get_val_prev_s(ivarfl(ir22), cvara_r22)
-    call field_get_val_prev_s(ivarfl(ir33), cvara_r33)
-    call field_get_val_prev_s(ivarfl(ir12), cvara_r12)
-    call field_get_val_prev_s(ivarfl(ir13), cvara_r13)
-    call field_get_val_prev_s(ivarfl(ir23), cvara_r23)
-  endif
+if (irijco.eq.1) then
+  call field_get_val_prev_v(ivarfl(irij), cvara_rij)
+else
+  call field_get_val_prev_s(ivarfl(ir11), cvara_r11)
+  call field_get_val_prev_s(ivarfl(ir22), cvara_r22)
+  call field_get_val_prev_s(ivarfl(ir33), cvara_r33)
+  call field_get_val_prev_s(ivarfl(ir12), cvara_r12)
+  call field_get_val_prev_s(ivarfl(ir13), cvara_r13)
+  call field_get_val_prev_s(ivarfl(ir23), cvara_r23)
 endif
 
 call field_get_key_struct_var_cal_opt(ivarfl(iep), vcopt)
@@ -194,6 +210,39 @@ if(vcopt%iwarni.ge.1) then
   endif
 endif
 
+! Time extrapolation?
+call field_get_key_id("time_extrapolated", key_t_ext_id)
+
+!===============================================================================
+! 1.1 Call source terms for Rij
+!===============================================================================
+
+if (irijco.eq.1) then
+  do iel = 1, ncel
+    do isou = 1 ,6
+      smbrts(isou,iel) = 0.d0
+    enddo
+  enddo
+  do iel = 1, ncel
+    do isou = 1, 6
+      do jsou = 1, 6
+        rovsdtts(isou,jsou,iel) = 0.d0
+      enddo
+    enddo
+  enddo
+
+  call cs_user_turbulence_source_terms2 &
+   ( nvar   , nscal  , ncepdp , ncesmp ,                            &
+     ivarfl(irij)    ,                                              &
+     icepdc , icetsm , itypsm ,                                     &
+     ckupdc , smacel ,                                              &
+     smbrts , rovsdtts)
+
+  ! C version
+  call user_source_terms(ivarfl(irij), smbrts, rovsdtts)
+
+endif
+
 !===============================================================================
 ! 1.1 Advanced init for EBRSM
 !===============================================================================
@@ -217,13 +266,10 @@ if (ntcabs.eq.1.and.reinit_turb.eq.1.and.iturb.eq.32) then
   inc    = 1
   iccocg = 1
 
-  call field_gradient_scalar(ivarfl(ial), iprev, imrgra, inc,     &
-                             iccocg,                              &
-                             grad)
+  call field_gradient_scalar(ivarfl(ial), iprev, 0, inc, iccocg, grad)
 
   if (irijco.eq.1) then
     call field_get_val_v(ivarfl(irij), cvar_rij)
-    call field_get_val_prev_v(ivarfl(irij), cvara_rij)
   else
     call field_get_val_s(ivarfl(ir11), cvar_r11)
     call field_get_val_s(ivarfl(ir22), cvar_r22)
@@ -231,12 +277,6 @@ if (ntcabs.eq.1.and.reinit_turb.eq.1.and.iturb.eq.32) then
     call field_get_val_s(ivarfl(ir12), cvar_r12)
     call field_get_val_s(ivarfl(ir13), cvar_r13)
     call field_get_val_s(ivarfl(ir23), cvar_r23)
-    call field_get_val_prev_s(ivarfl(ir11), cvara_r11)
-    call field_get_val_prev_s(ivarfl(ir22), cvara_r22)
-    call field_get_val_prev_s(ivarfl(ir33), cvara_r33)
-    call field_get_val_prev_s(ivarfl(ir12), cvara_r12)
-    call field_get_val_prev_s(ivarfl(ir13), cvara_r13)
-    call field_get_val_prev_s(ivarfl(ir23), cvara_r23)
   endif
 
   utaurf=0.05d0*uref
@@ -345,128 +385,149 @@ endif
 inc = 1
 iprev = 1
 
-call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc,    &
-                           gradv)
+call field_gradient_vector(ivarfl(iu), iprev, 0, inc, gradv)
 
 !===============================================================================
-! 2.2 Compute the production term for Rij LRR (iturb =30)
+! 2.2 Compute the production term for Rij
 !===============================================================================
 
-if (iturb.eq.30) then
-    do ii = 1 , 6
-      do iel = 1, ncel
-        produc(ii,iel) = 0.0d0
+if (irijco.eq.1) then
+  do iel = 1, ncel
+
+    ! Pij = - (Rik dUk/dXj + dUk/dXi Rkj)
+    ! Pij is stored as (P11, P22, P33, P12, P23, P13)
+    cpro_produc(1,iel) = &
+                  - 2.0d0*(cvara_rij(1,iel)*gradv(1, 1, iel) +           &
+                           cvara_rij(4,iel)*gradv(2, 1, iel) +           &
+                           cvara_rij(6,iel)*gradv(3, 1, iel) )
+
+    cpro_produc(4,iel) = &
+                  - (cvara_rij(4,iel)*gradv(1, 1, iel) +                 &
+                     cvara_rij(2,iel)*gradv(2, 1, iel) +                 &
+                     cvara_rij(5,iel)*gradv(3, 1, iel) )                 &
+                  - (cvara_rij(1,iel)*gradv(1, 2, iel) +                 &
+                     cvara_rij(4,iel)*gradv(2, 2, iel) +                 &
+                     cvara_rij(6,iel)*gradv(3, 2, iel) )
+
+    cpro_produc(6,iel) = &
+                  - (cvara_rij(6,iel)*gradv(1, 1, iel) +                 &
+                     cvara_rij(5,iel)*gradv(2, 1, iel) +                 &
+                     cvara_rij(3,iel)*gradv(3, 1, iel) )                 &
+                  - (cvara_rij(1,iel)*gradv(1, 3, iel) +                 &
+                     cvara_rij(4,iel)*gradv(2, 3, iel) +                 &
+                     cvara_rij(6,iel)*gradv(3, 3, iel) )
+
+    cpro_produc(2,iel) = &
+                  - 2.0d0*(cvara_rij(4,iel)*gradv(1, 2, iel) +           &
+                           cvara_rij(2,iel)*gradv(2, 2, iel) +           &
+                           cvara_rij(5,iel)*gradv(3, 2, iel) )
+
+    cpro_produc(5,iel) = &
+                  - (cvara_rij(6,iel)*gradv(1, 2, iel) +                 &
+                     cvara_rij(5,iel)*gradv(2, 2, iel) +                 &
+                     cvara_rij(3,iel)*gradv(3, 2, iel) )                 &
+                  - (cvara_rij(4,iel)*gradv(1, 3, iel) +                 &
+                     cvara_rij(2,iel)*gradv(2, 3, iel) +                 &
+                     cvara_rij(5,iel)*gradv(3, 3, iel) )
+
+    cpro_produc(3,iel) = &
+                  - 2.0d0*(cvara_rij(6,iel)*gradv(1, 3, iel) +           &
+                           cvara_rij(5,iel)*gradv(2, 3, iel) +           &
+                           cvara_rij(3,iel)*gradv(3, 3, iel) )
+
+  enddo
+else
+  do iel = 1 , ncel
+
+    ! Pij = - (Rik dUk/dXj + dUk/dXi Rkj)
+    cpro_produc(1,iel) = &
+                  - 2.0d0*(cvara_r11(iel)*gradv(1, 1, iel) +           &
+                           cvara_r12(iel)*gradv(2, 1, iel) +           &
+                           cvara_r13(iel)*gradv(3, 1, iel) )
+
+    cpro_produc(4,iel) = &
+                  - (cvara_r12(iel)*gradv(1, 1, iel) +                 &
+                     cvara_r22(iel)*gradv(2, 1, iel) +                 &
+                     cvara_r23(iel)*gradv(3, 1, iel) )                 &
+                  - (cvara_r11(iel)*gradv(1, 2, iel) +                 &
+                     cvara_r12(iel)*gradv(2, 2, iel) +                 &
+                     cvara_r13(iel)*gradv(3, 2, iel) )
+
+    cpro_produc(6,iel) = &
+                  - (cvara_r13(iel)*gradv(1, 1, iel) +                 &
+                     cvara_r23(iel)*gradv(2, 1, iel) +                 &
+                     cvara_r33(iel)*gradv(3, 1, iel) )                 &
+                  - (cvara_r11(iel)*gradv(1, 3, iel) +                 &
+                     cvara_r12(iel)*gradv(2, 3, iel) +                 &
+                     cvara_r13(iel)*gradv(3, 3, iel) )
+
+    cpro_produc(2,iel) = &
+                  - 2.0d0*(cvara_r12(iel)*gradv(1, 2, iel) +           &
+                           cvara_r22(iel)*gradv(2, 2, iel) +           &
+                           cvara_r23(iel)*gradv(3, 2, iel) )
+
+    cpro_produc(5,iel) = &
+                  - (cvara_r13(iel)*gradv(1, 2, iel) +                 &
+                     cvara_r23(iel)*gradv(2, 2, iel) +                 &
+                     cvara_r33(iel)*gradv(3, 2, iel) )                 &
+                  - (cvara_r12(iel)*gradv(1, 3, iel) +                 &
+                     cvara_r22(iel)*gradv(2, 3, iel) +                 &
+                     cvara_r23(iel)*gradv(3, 3, iel) )
+
+    cpro_produc(3,iel) = &
+                  - 2.0d0*(cvara_r13(iel)*gradv(1, 3, iel) +           &
+                           cvara_r23(iel)*gradv(2, 3, iel) +           &
+                           cvara_r33(iel)*gradv(3, 3, iel) )
+
+  enddo
+endif
+
+!===============================================================================
+! 2.3 Compute the pressure correlation  term for Rij
+!===============================================================================
+! Phi,ij = Phi1,ij+Phi2,ij
+! Phi,ij = -C1 k/eps (Rij-2/3k dij) - C2 (Pij-2/3P dij)
+! Phi,ij is stored as (Phi11, Phi22, Phi33, Phi12, Phi23, Phi13)
+
+! TODO : coherent with the model
+if (f_id_phij.ge.0) then
+  d1s3 = 1.0d0/3.0d0
+  d2s3 = 2.0d0/3.0d0
+  if (irijco.eq.1) then
+    do iel = 1, ncel
+      k=0.5*(cvara_rij(1,iel)+cvara_rij(2,iel)+cvara_rij(3,iel))
+      P=0.5*(cpro_produc(1,iel)+cpro_produc(2,iel)+cpro_produc(3,iel))
+      do isou=1,3
+        cpro_press_correl(isou, iel) = -crij1*cvar_ep(iel)/k*(cvara_rij(isou,iel)-d2s3*k)  &
+                                       -crij2*(cpro_produc(isou,iel)-d2s3*P)
+      enddo
+      do isou=4,6
+        cpro_press_correl(isou, iel) = -crij1*cvar_ep(iel)/k*(cvara_rij(isou,iel))  &
+                                       -crij2*(cpro_produc(isou,iel))
       enddo
     enddo
-  if (irijco.eq.1) then
-    do iel = 1 , ncel
-
-      ! grad u
-
-      produc(1,iel) = produc(1,iel)                                  &
-                    - 2.0d0*(cvara_rij(1,iel)*gradv(1, 1, iel) +           &
-                             cvara_rij(4,iel)*gradv(2, 1, iel) +           &
-                             cvara_rij(6,iel)*gradv(3, 1, iel) )
-
-      produc(4,iel) = produc(4,iel)                                  &
-                    - (cvara_rij(4,iel)*gradv(1, 1, iel) +                 &
-                       cvara_rij(2,iel)*gradv(2, 1, iel) +                 &
-                       cvara_rij(5,iel)*gradv(3, 1, iel) )
-
-      produc(6,iel) = produc(6,iel)                                  &
-                    - (cvara_rij(6,iel)*gradv(1, 1, iel) +                 &
-                       cvara_rij(5,iel)*gradv(2, 1, iel) +                 &
-                       cvara_rij(3,iel)*gradv(3, 1, iel) )
-
-      ! grad v
-
-      produc(2,iel) = produc(2,iel)                                  &
-                    - 2.0d0*(cvara_rij(4,iel)*gradv(1, 2, iel) +           &
-                             cvara_rij(2,iel)*gradv(2, 2, iel) +           &
-                             cvara_rij(5,iel)*gradv(3, 2, iel) )
-
-      produc(4,iel) = produc(4,iel)                                  &
-                    - (cvara_rij(1,iel)*gradv(1, 2, iel) +                 &
-                       cvara_rij(4,iel)*gradv(2, 2, iel) +                 &
-                       cvara_rij(6,iel)*gradv(3, 2, iel) )
-
-      produc(5,iel) = produc(5,iel)                                  &
-                    - (cvara_rij(6,iel)*gradv(1, 2, iel) +                 &
-                       cvara_rij(5,iel)*gradv(2, 2, iel) +                 &
-                       cvara_rij(3,iel)*gradv(3, 2, iel) )
-
-      ! grad w
-
-      produc(3,iel) = produc(3,iel)                                  &
-                    - 2.0d0*(cvara_rij(6,iel)*gradv(1, 3, iel) +           &
-                             cvara_rij(5,iel)*gradv(2, 3, iel) +           &
-                             cvara_rij(3,iel)*gradv(3, 3, iel) )
-
-      produc(6,iel) = produc(6,iel)                                  &
-                    - (cvara_rij(1,iel)*gradv(1, 3, iel) +                 &
-                       cvara_rij(4,iel)*gradv(2, 3, iel) +                 &
-                       cvara_rij(6,iel)*gradv(3, 3, iel) )
-
-      produc(5,iel) = produc(5,iel)                                  &
-                    - (cvara_rij(4,iel)*gradv(1, 3, iel) +                 &
-                       cvara_rij(2,iel)*gradv(2, 3, iel) +                 &
-                       cvara_rij(5,iel)*gradv(3, 3, iel) )
-
-    enddo
   else
-    do iel = 1 , ncel
+    do iel = 1, ncel
+      k=0.5*(cvara_r11(iel)+cvara_r22(iel)+cvara_r33(iel))
+      P=0.5*(cpro_produc(1,iel)+cpro_produc(2,iel)+cpro_produc(3,iel))
 
-      ! grad u
+      cpro_press_correl(1, iel)= -crij1*cvar_ep(iel)/k*(cvara_r11(iel)-d2s3*k)  &
+                                 -crij2*(cpro_produc(1,iel)-d2s3*P)
 
-      produc(1,iel) = produc(1,iel)                                  &
-                    - 2.0d0*(cvara_r11(iel)*gradv(1, 1, iel) +           &
-                             cvara_r12(iel)*gradv(2, 1, iel) +           &
-                             cvara_r13(iel)*gradv(3, 1, iel) )
+      cpro_press_correl(2, iel)= -crij1*cvar_ep(iel)/k*(cvara_r22(iel)-d2s3*k)  &
+                                 -crij2*(cpro_produc(2,iel)-d2s3*P)
 
-      produc(4,iel) = produc(4,iel)                                  &
-                    - (cvara_r12(iel)*gradv(1, 1, iel) +                 &
-                       cvara_r22(iel)*gradv(2, 1, iel) +                 &
-                       cvara_r23(iel)*gradv(3, 1, iel) )
+      cpro_press_correl(3, iel)= -crij1*cvar_ep(iel)/k*(cvara_r33(iel)-d2s3*k)  &
+                                 -crij2*(cpro_produc(3,iel)-d2s3*P)
 
-      produc(6,iel) = produc(6,iel)                                  &
-                    - (cvara_r13(iel)*gradv(1, 1, iel) +                 &
-                       cvara_r23(iel)*gradv(2, 1, iel) +                 &
-                       cvara_r33(iel)*gradv(3, 1, iel) )
+      cpro_press_correl(4, iel)= -crij1*cvar_ep(iel)/k*cvara_r12(iel)  &
+                                 -crij2*cpro_produc(4,iel)
 
-      ! grad v
-
-      produc(2,iel) = produc(2,iel)                                  &
-                    - 2.0d0*(cvara_r12(iel)*gradv(1, 2, iel) +           &
-                             cvara_r22(iel)*gradv(2, 2, iel) +           &
-                             cvara_r23(iel)*gradv(3, 2, iel) )
-
-      produc(4,iel) = produc(4,iel)                                  &
-                    - (cvara_r11(iel)*gradv(1, 2, iel) +                 &
-                       cvara_r12(iel)*gradv(2, 2, iel) +                 &
-                       cvara_r13(iel)*gradv(3, 2, iel) )
-
-      produc(5,iel) = produc(5,iel)                                  &
-                    - (cvara_r13(iel)*gradv(1, 2, iel) +                 &
-                       cvara_r23(iel)*gradv(2, 2, iel) +                 &
-                       cvara_r33(iel)*gradv(3, 2, iel) )
-
-      ! grad w
-
-      produc(3,iel) = produc(3,iel)                                  &
-                    - 2.0d0*(cvara_r13(iel)*gradv(1, 3, iel) +           &
-                             cvara_r23(iel)*gradv(2, 3, iel) +           &
-                             cvara_r33(iel)*gradv(3, 3, iel) )
-
-      produc(6,iel) = produc(6,iel)                                  &
-                    - (cvara_r11(iel)*gradv(1, 3, iel) +                 &
-                       cvara_r12(iel)*gradv(2, 3, iel) +                 &
-                       cvara_r13(iel)*gradv(3, 3, iel) )
-
-      produc(5,iel) = produc(5,iel)                                  &
-                    - (cvara_r12(iel)*gradv(1, 3, iel) +                 &
-                       cvara_r22(iel)*gradv(2, 3, iel) +                 &
-                       cvara_r23(iel)*gradv(3, 3, iel) )
+      cpro_press_correl(5, iel)= -crij1*cvar_ep(iel)/k*cvara_r22(iel)  &
+                                 -crij2*cpro_produc(5,iel)
 
+      cpro_press_correl(6, iel)= -crij1*cvar_ep(iel)/k*cvara_r13(iel)  &
+                                 -crij2*cpro_produc(6,iel)
     enddo
   endif
 endif
@@ -490,16 +551,13 @@ if (igrari.eq.1 .and. ippmod(iatmos).ge.1) then
 
   call field_get_val_prev_s(ivarfl(isca(iscalt)), cvara_scalt)
 
-  iprev = 1
   inc = 1
   iccocg = 1
 
-  call field_gradient_scalar(ivarfl(isca(iscalt)), 1, imrgra, inc, &
-                             iccocg,                               &
-                             gradro)
+  call field_gradient_scalar(ivarfl(isca(iscalt)), 1, 0, inc, iccocg, gradro)
 
   ! gradro stores: - rho grad(theta)/theta
-  ! grad(ro) and grad(teta) have opposite sign
+  ! grad(rho) and grad(theta) have opposite signs
   do iel = 1, ncel
     rhothe = cromo(iel)/cvara_scalt(iel)
     gradro(1, iel) = -rhothe*gradro(1, iel)
@@ -511,47 +569,72 @@ else if (igrari.eq.1) then
   ! Allocate a temporary array for the gradient calculation
   allocate(gradro(3,ncelet))
 
-! Boundary conditions: Dirichlet romb
-!   We use viscb to store the relative coefficient of rom
-!   We impose in Dirichlet (coefa) the value romb
+  ! Boussinesq approximation, only for the thermal scalar for the moment
+  if (idilat.eq.0)  then
 
-  do ifac = 1, nfabor
-    viscb(ifac) = 0.d0
-  enddo
+    iccocg = 1
+    inc = 1
 
-! The choice below has the advantage to be simple
-  call field_get_key_struct_var_cal_opt(ivarfl(ir11), vcopt)
+    ! Use the current value...
+    call field_gradient_scalar(ivarfl(isca(iscalt)), 0, 0, inc, iccocg, gradro)
 
-  nswrgp = vcopt%nswrgr
-  imligp = vcopt%imligr
-  iwarnp = vcopt%iwarni
-  epsrgp = vcopt%epsrgr
-  climgp = vcopt%climgr
-  extrap = vcopt%extrag
+    !FIXME make it dependant on the scalar and use is_buoyant field
+    call field_get_val_s(ibeta, cpro_beta)
 
-  f_id0 = -1
-  iccocg = 1
+    ! gradro stores: - beta grad(T)
+    ! grad(rho) and grad(T) have opposite signs
+    do iel = 1, ncel
+      gradro(1, iel) = - ro0 * cpro_beta(iel) * gradro(1, iel)
+      gradro(2, iel) = - ro0 * cpro_beta(iel) * gradro(2, iel)
+      gradro(3, iel) = - ro0 * cpro_beta(iel) * gradro(3, iel)
+    enddo
 
-  ! If we extrapolate the source terms and rho, we use cpdt rho^n
-  if(isto2t.gt.0.and.iroext.gt.0) then
-    call field_get_val_prev_s(icrom, cromo)
-    call field_get_val_prev_s(ibrom, bromo)
   else
-    call field_get_val_s(icrom, cromo)
-    call field_get_val_s(ibrom, bromo)
+
+    ! Boundary conditions: Dirichlet romb
+    ! We use viscb to store the relative coefficient of rom
+    ! We impose in Dirichlet (coefa) the value romb
+
+    do ifac = 1, nfabor
+      viscb(ifac) = 0.d0
+    enddo
+
+    ! The choice below has the advantage to be simple
+    call field_get_key_struct_var_cal_opt(ivarfl(irij), vcopt)
+
+    imrgrp = vcopt%imrgra
+    nswrgp = vcopt%nswrgr
+    imligp = vcopt%imligr
+    iwarnp = vcopt%iwarni
+    epsrgp = vcopt%epsrgr
+    climgp = vcopt%climgr
+    extrap = vcopt%extrag
+
+    f_id0 = -1
+    iccocg = 1
+
+    call field_get_key_int(icrom, key_t_ext_id, iroext)
+    ! If we extrapolate the source terms and rho, we use cpdt rho^n
+    if(isto2t.gt.0.and.iroext.gt.0) then
+      call field_get_val_prev_s(icrom, cromo)
+      call field_get_val_prev_s(ibrom, bromo)
+    else
+      call field_get_val_s(icrom, cromo)
+      call field_get_val_s(ibrom, bromo)
+    endif
+
+    call gradient_s                                                 &
+      ( f_id0  , imrgrp , inc    , iccocg , nswrgp , imligp ,       &
+      iwarnp , epsrgp , climgp ,                                    &
+      cromo  , bromo  , viscb           ,                           &
+      gradro )
+
   endif
-
-  call gradient_s                                                 &
- ( f_id0  , imrgra , inc    , iccocg , nswrgp , imligp ,          &
-   iwarnp , epsrgp , climgp , extrap ,                            &
-   cromo  , bromo  , viscb           ,                            &
-   gradro )
-
 endif
 
 !===============================================================================
 ! 4.  Loop on the variables Rij (6 variables)
-!     The order is R11 R22 R33 R12 R13 R23 (The place of those variables
+!     The order is R11 R22 R33 R12 R23 R13 (The place of those variables
 !      is IR11.    ..
 !     We solve the equation in a routine similar to covofi.f90
 !===============================================================================
@@ -571,15 +654,16 @@ if (irijco.eq.1) then
 
   ! Rij-epsilon standard (LRR)
   if (iturb.eq.30) then !TODO
+
     call resrij2 &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    icepdc , icetsm , itypsm ,                                     &
    dt     ,                                                       &
-   produc , gradro ,                                              &
+   gradv  , cpro_produc , gradro ,                                &
    ckupdc , smacel ,                                              &
    viscf  , viscb  ,                                              &
    tslagi ,                                                       &
-   smbrts   , rovsdtts )
+   smbrts , rovsdtts )
 
   ! Rij-epsilon SSG or EBRSM
   elseif (iturb.eq.31.or.iturb.eq.32) then
@@ -587,10 +671,10 @@ if (irijco.eq.1) then
     call resssg2 &
   ( nvar    , nscal  , ncepdp , ncesmp ,                            &
     ivar    ,                                                       &
-    icepdc  , icetsm , itypsm ,                                     &
+    icetsm , itypsm ,                                               &
     dt      ,                                                       &
-    gradv   , gradro ,                                              &
-    ckupdc  , smacel ,                                              &
+    gradv   , cpro_produc, gradro ,                                 &
+    smacel ,                                                        &
     viscf   , viscb  ,                                              &
     tslagi ,                                                        &
     smbrts  , rovsdtts )
@@ -618,12 +702,12 @@ else
     ! Rij-epsilon standard (LRR)
     if (iturb.eq.30) then
       call resrij &
-   ( nvar   , nscal  , ncepdp , ncesmp ,                            &
+   ( nvar   , nscal  , ncesmp ,                                     &
      ivar   , isou   ,                                              &
-     icepdc , icetsm , itypsm ,                                     &
+     icetsm , itypsm ,                                              &
      dt     ,                                                       &
-     produc , gradro ,                                              &
-     ckupdc , smacel ,                                              &
+     cpro_produc , gradro ,                                         &
+     smacel ,                                                       &
      viscf  , viscb  ,                                              &
      tslagi ,                                                       &
      smbr   , rovsdt )
@@ -631,12 +715,12 @@ else
     ! Rij-epsilon SSG or EBRSM
     elseif (iturb.eq.31.or.iturb.eq.32) then
       call resssg &
-      ( nvar   , nscal  , ncepdp , ncesmp ,                            &
+      ( nvar   , nscal  , ncesmp ,                                     &
         ivar   , isou   ,                                              &
-        icepdc , icetsm , itypsm ,                                     &
+        icetsm , itypsm ,                                              &
         dt     ,                                                       &
         gradv  , gradro ,                                              &
-        ckupdc , smacel ,                                              &
+        smacel ,                                                       &
         viscf  , viscb  ,                                              &
         tslagi ,                                                       &
         smbr   , rovsdt )
@@ -650,11 +734,11 @@ endif
 !===============================================================================
 
 call reseps &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   icepdc , icetsm , itypsm ,                                     &
+ ( nvar   , nscal  , ncesmp ,                                     &
+   icetsm , itypsm ,                                              &
    dt     ,                                                       &
-   gradv  , produc , gradro ,                                     &
-   ckupdc , smacel ,                                              &
+   gradv  , cpro_produc , gradro ,                                &
+   smacel ,                                                       &
    viscf  , viscb  ,                                              &
    smbr   , rovsdt )
 
@@ -686,20 +770,6 @@ deallocate(gradv)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(/,                                      &
-'   ** Resolution du Rij-EPSILON LRR'                   ,/,&
-'      -----------------------------'             ,/)
- 1001 format(/,                                      &
-'   ** Resolution du Rij-EPSILON SSG'                   ,/,&
-'      -----------------------------'             ,/)
- 1002 format(/,                                      &
-'   ** Resolution du Rij-EPSILON EBRSM'                 ,/,&
-'      -------------------------------'           ,/)
-
-#else
-
  1000 format(/,                                      &
 '   ** Solving Rij-EPSILON LRR'                   ,/,&
 '      -----------------------'                   ,/)
@@ -710,8 +780,6 @@ deallocate(gradv)
 '   ** Solving Rij-EPSILON EBRSM'                 ,/,&
 '      -------------------------'                 ,/)
 
-#endif
-
 !----
 ! End
 !----
