@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -60,13 +60,14 @@
 !> \param[in]     smacel        value of variables associated to the
 !>                               mass source
 !>                               for ivar = ipr, smacel = mass flux
+!> \param[out]    gradv         the computed velocity gradients
 !______________________________________________________________________________!
 
 subroutine visdyn &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    icepdc , icetsm , itypsm ,                                     &
    dt     ,                                                       &
-   ckupdc , smacel )
+   ckupdc , smacel, gradv )
 
 !===============================================================================
 ! Module files
@@ -99,6 +100,7 @@ integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
 double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
+double precision, intent(inout) :: gradv(3,3,ncelet)
 
 ! Local variables
 
@@ -120,7 +122,7 @@ double precision, allocatable, dimension(:) :: w4, w5, w6
 double precision, allocatable, dimension(:) :: w7, w8, w9
 double precision, allocatable, dimension(:) :: w10, w0
 double precision, allocatable, dimension(:,:) :: xmij, w61, w62
-double precision, dimension(:,:,:), allocatable :: gradv, gradvf
+double precision, dimension(:,:,:), allocatable :: gradvf
 double precision, dimension(:,:), pointer :: coefau
 double precision, dimension(:,:,:), pointer :: coefbu
 double precision, dimension(:), pointer :: crom
@@ -167,12 +169,12 @@ allocate(xmij(6,ncelet))
 !===============================================================================
 
 ! Allocate temporary arrays for gradients calculation
-allocate(gradv(3,3,ncelet), gradvf(3,3,ncelet))
+allocate(gradvf(3,3,ncelet))
 
 inc = 1
 iprev = 0
 
-call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc, gradv)
+call field_gradient_vector(ivarfl(iu), iprev, 0, inc, gradv)
 
 ! Filter the velocity gradient on the extended neighborhood
 
@@ -222,7 +224,7 @@ do iel = 1, ncel
 enddo
 
 ! Free memory
-deallocate(gradv, gradvf)
+deallocate(gradvf)
 
 !     Here XMIJ contains Sij
 !         VISCT contains ||S||
@@ -399,7 +401,7 @@ do iel = 1, ncel
 enddo
 
 !===============================================================================
-! 3.  Calculation of (dynamic) velocity
+! 3.  Calculation of (dynamic) viscosity
 !===============================================================================
 
 ! Clipping in (mu + mu_t)>0 in phyvar
@@ -447,22 +449,6 @@ deallocate(w1, w0)
 ! Formats
 !----
 
-#if defined(_CS_LANG_FR)
-
- 1000 format(                                                           &
-' Nb Clipping Constante  Smagorinsky par valeurs maximales',I10,/)
- 2001 format(                                                           &
-' --- Informations sur la constante de Smagorinsky^2             ',/,&
-' ----------------------------------                          ',/,&
-' Val. moy.      Val. min   Val. max                               ',/,&
-' ----------------------------------                          '  )
- 2002 format(                                                           &
- e12.4    ,      e12.4,      e12.4                               )
- 2003 format(                                                           &
-' ----------------------------------                          ',/)
-
-#else
-
  1000 format(                                                           &
 ' Nb of clipping of the Smagorinsky constant by max values',I10,/)
  2001 format(                                                           &
@@ -475,8 +461,6 @@ deallocate(w1, w0)
  2003 format(                                                           &
 ' --------------------------------'                            ,/)
 
-#endif
-
 !----
 ! End
 !----
