@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -31,7 +31,7 @@
 !> \brief User subroutine which fills boundary conditions arrays
 !> (\c icodcl, \c rcodcl) for unknown variables.
 !>
-!> See \subpage cs_user_boundary_conditions_examples for examples.
+!> See \ref cs_user_boundary_conditions_examples for examples.
 !>
 !> \section cs_user_boundary_conditions_intro Introduction
 !>
@@ -123,9 +123,9 @@
 !>                  - \c rcodcl(ifac, iv, 1) = fluid velocity in the y direction
 !>                  - \c rcodcl(ifac, iw, 1) = fluid velocity in the z direction
 !>    - Value of the dynamic roughness height to specify in
-!>                  - \c rcodcl(ifac, iu, 3)
+!>                  - \c boundary_roughness field
 !>    - Value of the scalar roughness height (if required) to specify in
-!>                  - \c rcodcl(ifac, iv, 3) (values for iw are not used)
+!>                  - \c boundary_thermal_roughness
 !>    - Specific code and prescribed temperature value at wall if applicable:
 !>                  - \c icodcl(ifac, ivar)    = 6
 !>                  - \c rcodcl(ifac, ivar, 1) = prescribed temperature
@@ -235,16 +235,17 @@
 !>        \c rcodcl(ifac, ivar, 3) = -(viscls+visct/turb_schmidt) * (grad F).n
 !>
 !>  - \c rcodcl(ifac, ivar, 3) if \c icodcl(ifac, ivar) = 6:
-!>      Roughness for the rough wall law
+!>      Roughness length scale for the rough wall law
 !>     - For velocities U, dynamic roughness
-!>         \c rcodcl(ifac, iu, 3) = roughd
+!>         \c boundary_roughness field contains the dynamic roughness
+!>            length scale
 !>     - For other scalars, thermal roughness
-!>         \c rcodcl(ifac, iv, 3) = rough
+!>         \c boundary_thermal_roughness field contains the scalar roughness
+!>            length scale
 !>
-!>
-!> Note that if the user assigns a value to \c itypfb equal to \c ientre, \c isolib,
-!> \c isymet, \c iparoi, or \c iparug and does not modify \c icodcl (zero value by
-!>  default), \c itypfb will define the boundary condition type.
+!> Note that if the user assigns a value to \c itypfb equal to \c ientre,
+!> \c isolib, \c isymet, \c iparoi, or \c iparug and does not modify \c icodcl
+!> (zero value by default), \c itypfb will define the boundary condition type.
 !>
 !> To the contrary, if the user prescribes \c icodcl(ifac, ivar) (nonzero),
 !> the values assigned to \c rcodcl will be used for the considered face
@@ -270,6 +271,13 @@
 !> the code).
 !>
 !>
+!> \subsection comp_bcs Boundary condition types for enthalpy
+!>
+!> For enthalpy, prescribed values in rcodcl(ifac, ivar, 1) may
+!> be defined using the temperature instead. In this case, icodcl(ifac, ivar)
+!> must be replaced by -icodcl(ifac, ivar) to mark the face for automatic
+!> conversion.
+!>
 !> \subsection comp_bcs Boundary condition types for compressible flows
 !>
 !> For compressible flows, only predefined boundary conditions may
@@ -330,7 +338,7 @@
 !>
 !> Cell value field ids
 !>
-!> - Density:                        \c irom
+!> - Density:                        \c icrom
 !> - Dynamic molecular viscosity:    \c iviscl
 !> - Turbulent viscosity:            \c ivisct
 !> - Specific heat:                  \c icp
@@ -382,8 +390,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughness
-!>                                 in m if icodcl=6
+!>                                 (negative if gain) in w/m2
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
 !>                                 -# for the pressure \f$ \Delta t
@@ -413,13 +420,13 @@ use cstnum
 use entsor
 use parall
 use period
-use ihmpre
 use ppppar
 use ppthch
 use coincl
 use cpincl
 use ppincl
 use ppcpfu
+use atchem
 use atincl
 use atsoil
 use ctincl
@@ -453,38 +460,6 @@ integer, allocatable, dimension(:) :: lstelt
 
 !===============================================================================
 
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-
-!===============================================================================
-! 0.  This test allows the user to ensure that the version of this subroutine
-!       used is that from his case definition, and not that from the library.
-!     If a file from the GUI is used, this subroutine may not be mandatory,
-!       thus the default (library reference) version returns immediately.
-!===============================================================================
-
-if (iihmpr.eq.1) then
-  return
-else if (nfabor.gt.0) then
-  write(nfecra,9000)
-  call csexit (1)
-endif
-
- 9000 format(                                                     &
-'@',/,                                                            &
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',/,                                                            &
-'@ @@ WARNING:    stop in definition of boundary conditions',   /,&
-'@    =======',/,                                                 &
-'@  The user subroutine ''cs_f_user_boundary_conditions'       ,/,&
-'@  must be completed.'                                        ,/,&
-'@'                                                            ,/,&
-'@  The calculation will not be run.'                          ,/,&
-'@'                                                            ,/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',/)
-
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
 !===============================================================================
 ! Initialization
 !===============================================================================
