@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -33,7 +33,7 @@
 !>  - Fills boundary conditions (ialtyb, icodcl, rcodcl) for mesh velocity.
 !>  - This subroutine also enables one to fix displacement on nodes.
 !>
-!> See \subpage cs_user_boundary_conditions_ale for examples.
+!> See \ref cs_user_boundary_conditions_ale for examples.
 !>
 !> \section intro_usa Introduction
 !>
@@ -347,7 +347,6 @@ use cstnum
 use entsor
 use parall
 use period
-use ihmpre
 use mesh
 
 !===============================================================================
@@ -369,38 +368,6 @@ double precision disale(3,nnod), xyzno0(3,nnod)
 
 !===============================================================================
 
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-!===============================================================================
-! 0.  This test allows the user to ensure that the version of this subroutine
-!       used is that from his case definition, and not that from the library.
-!     If a file from the GUI is used, this subroutine may not be mandatory,
-!       thus the default (library reference) version returns immediately.
-!===============================================================================
-if(iihmpr.eq.1) then
-  return
-else
-  write(nfecra,9000)
-  call csexit (1)
-endif
-
- 9000 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : stop in definition of boundary conditions   ',/,&
-'@    =========                                               ',/,&
-'@     ALE Method has been activated                          ',/,&
-'@     User subroutine ''usalcl'' must be completed           ',/,&
-'@                                                            ',/,&
-'@  The calculation will not be run                           ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
-
 !--------
 ! Formats
 !--------
