@@ -10,7 +10,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -42,19 +42,10 @@
 #include <ple_coupling.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "cs_base.h"
-#include "cs_coupling.h"
-#include "cs_sat_coupling.h"
-#include "cs_syr_coupling.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -66,7 +57,7 @@ BEGIN_C_DECLS
  *
  * \brief Code couplings definition with SYRTHES and Code_Saturne.
  *
- * See \subpage user_coupling for examples.
+ * See \ref user_coupling for examples.
  */
 /*----------------------------------------------------------------------------*/
 
@@ -76,15 +67,16 @@ BEGIN_C_DECLS
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Define global options for couplings.
+ * \brief Define couplings with other instances of Code_Saturne.
  *
- * These options allow defining the time step synchronization policy,
- * as well as a time step multiplier.
+ * This is done by calling the \ref cs_sat_coupling_define function for each
+ * coupling to add.
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_saturne_coupling
 void
-cs_user_coupling(void)
+cs_user_saturne_coupling(void)
 {
 
 }
@@ -98,6 +90,7 @@ cs_user_coupling(void)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_syrthes_coupling
 void
 cs_user_syrthes_coupling(void)
 {
@@ -106,17 +99,29 @@ cs_user_syrthes_coupling(void)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Define couplings with other instances of Code_Saturne.
+ * \brief Compute a volume exchange coefficient for SYRTHES couplings.
  *
- * This is done by calling the \ref cs_sat_coupling_define function for each
- * coupling to add.
+ * \param[in]   coupling_id   Syrthes coupling id
+ * \param[in]   syrthes_name  name of associated Syrthes instance
+ * \param[in]   n_elts        number of associated cells
+ * \param[in]   elt_ids       associated cell ids
+ * \param[out]  h_vol         associated exchange coefficient (size: n_elts)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_syrthes_coupling_volume_h
 void
-cs_user_saturne_coupling(void)
+cs_user_syrthes_coupling_volume_h(int               coupling_id,
+                                  const char       *syrthes_name,
+                                  cs_lnum_t         n_elts,
+                                  const cs_lnum_t   elt_ids[],
+                                  cs_real_t         h_vol[])
 {
-
+  CS_UNUSED(coupling_id);
+  CS_UNUSED(syrthes_name);
+  CS_UNUSED(n_elts);
+  CS_UNUSED(elt_ids);
+  CS_UNUSED(h_vol);
 }
 
 /*----------------------------------------------------------------------------*/
