@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -38,20 +38,10 @@
 #include <assert.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include <bft_mem.h>
-#include <bft_printf.h>
-
-#include "cs_domain.h"
-
-/*----------------------------------------------------------------------------
- * Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_gwf_soil.h"
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -70,14 +60,10 @@ BEGIN_C_DECLS
  */
 /*----------------------------------------------------------------------------*/
 
-/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
-
 /*============================================================================
  * Private function prototypes
  *============================================================================*/
 
-/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
-
 /*============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -92,13 +78,13 @@ BEGIN_C_DECLS
 */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_gwf_setup
 void
 cs_user_gwf_setup(cs_domain_t   *domain)
 {
   CS_UNUSED(domain);
 }
 
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Retrieve the bulk density related to a soil structure
@@ -108,6 +94,7 @@ cs_user_gwf_setup(cs_domain_t   *domain)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_gwf_get_soil_density
 void
 cs_user_gwf_get_soil_density(const cs_gwf_soil_t   *soil,
                              cs_real_t             *density)
