@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -30,7 +30,7 @@
 !>
 !> \brief Initialize variables
 !>
-!> See \subpage cs_user_initialization for examples.
+!> See \ref cs_user_initialization for examples.
 !>
 !> This subroutine is called at beginning of the computation
 !> (restart or not) before the loop time step.
@@ -40,7 +40,7 @@
 !>
 !> Modification of the behaviour law of physical quantities (rom, viscl,
 !> viscls, cp) is not done here. It is the purpose of the user subroutine
-!> \ref usphyv (in cs_user_physical_properties.f90)
+!> \ref cs_user_physical_properties
 !>
 !> \c rom and \c viscl values are equal to \c ro0 and \c viscl0 or initialize
 !> by reading the restart file.
@@ -58,7 +58,7 @@
 !> functions as described \ref field "here".
 !>
 !> Example of field ids:
-!> - Density:                        \c irom
+!> - Density:                        \c icrom
 !> - Dynamic molecular viscosity:    \c iviscl
 !> - Turbulent viscosity:            \c ivisct
 !> - Specific heat:                  \c icp
@@ -111,6 +111,9 @@ use mesh
 use field
 use turbomachinery
 
+use cs_c_bindings
+use cs_f_interfaces
+
 !===============================================================================
 
 implicit none
@@ -125,37 +128,13 @@ double precision dt(ncelet)
 
 ! INSERT_VARIABLE_DEFINITIONS_HERE
 
-integer, allocatable, dimension(:) :: lstelt
-
 !===============================================================================
 
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-
-if (1.eq.1) then
-!       Tag to know if a call to this subroutine has already been done
-  iusini = 0
-  return
-endif
-
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
-!===============================================================================
-! Initialization
-!===============================================================================
-
-allocate(lstelt(ncel)) ! temporary array for cells selection
-
 ! INSERT_MAIN_CODE_HERE
 
-!--------
-! Formats
-!--------
-
 !----
 ! End
 !----
 
-deallocate(lstelt) ! temporary array for cells selection
-
 return
 end subroutine cs_user_f_initialization
