@@ -15,7 +15,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -48,37 +48,10 @@
 #include <string.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_error.h"
-#include "bft_mem.h"
-#include "bft_printf.h"
-
-#include "fvm_defs.h"
-#include "fvm_selector.h"
-
-#include "cs_base.h"
-#include "cs_mesh_boundary.h"
-#include "cs_join.h"
-#include "cs_join_perio.h"
-#include "cs_mesh.h"
-#include "cs_mesh_quantities.h"
-#include "cs_mesh_bad_cells.h"
-#include "cs_mesh_boundary_layer.h"
-#include "cs_mesh_extrude.h"
-#include "cs_mesh_smoother.h"
-#include "cs_mesh_warping.h"
-#include "cs_parall.h"
-#include "cs_post.h"
-#include "cs_preprocessor_data.h"
-#include "cs_selector.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -90,7 +63,7 @@ BEGIN_C_DECLS
  *
  * \brief Definition and modification of the calculation mesh.
  *
- * See \subpage cs_user_mesh for examples.
+ * See \ref cs_user_mesh for examples.
  */
 /*----------------------------------------------------------------------------*/
 
@@ -104,6 +77,7 @@ BEGIN_C_DECLS
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_mesh_input
 void
 cs_user_mesh_input(void)
 {
@@ -116,6 +90,7 @@ cs_user_mesh_input(void)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_join
 void
 cs_user_join(void)
 {
@@ -128,6 +103,7 @@ cs_user_join(void)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_periodicity
 void
 cs_user_periodicity(void)
 {
@@ -140,6 +116,7 @@ cs_user_periodicity(void)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_mesh_warping
 void
 cs_user_mesh_warping(void)
 {
@@ -154,10 +131,11 @@ cs_user_mesh_warping(void)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_mesh_boundary
 void
 cs_user_mesh_boundary(cs_mesh_t  *mesh)
 {
-
+  CS_UNUSED(mesh);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -168,10 +146,11 @@ cs_user_mesh_boundary(cs_mesh_t  *mesh)
 */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_mesh_modify
 void
 cs_user_mesh_modify(cs_mesh_t  *mesh)
 {
-
+  CS_UNUSED(mesh);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -182,10 +161,11 @@ cs_user_mesh_modify(cs_mesh_t  *mesh)
 */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_mesh_smoothe
 void
 cs_user_mesh_smoothe(cs_mesh_t  *mesh)
 {
-
+  CS_UNUSED(mesh);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -198,10 +178,11 @@ cs_user_mesh_smoothe(cs_mesh_t  *mesh)
 */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_mesh_save
 void
 cs_user_mesh_save(cs_mesh_t  *mesh)
 {
-
+  CS_UNUSED(mesh);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -213,9 +194,43 @@ cs_user_mesh_save(cs_mesh_t  *mesh)
 */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_mesh_bad_cells_tag
 void
 cs_user_mesh_bad_cells_tag(cs_mesh_t             *mesh,
                            cs_mesh_quantities_t  *mesh_quantities)
+{
+  CS_UNUSED(mesh);
+  CS_UNUSED(mesh_quantities);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Apply partial modifications to the mesh after the preprocessing
+ *        and initial postprocessing mesh building stage.
+ *
+ * \param[in,out] mesh  pointer to a cs_mesh_t structure
+ * \param[in,out] mesh_quantities pointer to a cs_mesh_quantities_t structure
+*/
+/*----------------------------------------------------------------------------*/
+
+#pragma weak cs_user_mesh_modify_partial
+void
+cs_user_mesh_modify_partial(cs_mesh_t             *mesh,
+                            cs_mesh_quantities_t  *mesh_quantities)
+{
+  CS_UNUSED(mesh);
+  CS_UNUSED(mesh_quantities);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define a cartesian mesh.
+*/
+/*----------------------------------------------------------------------------*/
+
+#pragma weak cs_user_mesh_cartesian_define
+void
+cs_user_mesh_cartesian_define(void)
 {
 
 }
