@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -29,7 +29,7 @@
 !> \brief User subroutines for input of calculation parameters (Fortran modules).
 !>        These subroutines are called in all cases.
 !>
-!>  See \subpage f_parameters for examples.
+!>  See \ref f_parameters for examples.
 !>
 !>   If the Code_Saturne GUI is used, this file is not required (but may be
 !>   used to override parameters entered through the GUI, and to set
@@ -71,8 +71,7 @@
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[in]     ixmlpu        indicates if the XML file from the GUI is used
-!>                              (1 : yes, 0 : no)
+!> \param[in]     ixmlpu        indicates if an XML file from the GUI is used  !
 !______________________________________________________________________________!
 
 subroutine usppmo &
@@ -91,6 +90,7 @@ use ppincl
 use ppcpfu
 use coincl
 use radiat
+use ctincl
 use cs_c_bindings
 
 !===============================================================================
@@ -104,7 +104,6 @@ integer ixmlpu
 return
 end subroutine usppmo
 
-
 !===============================================================================
 
 !> \brief User subroutine for input of model selection parameters.
@@ -118,13 +117,11 @@ end subroutine usppmo
 !>                              used (1: yes, 0: no
 !> \param[in, out] iturb        turbulence model
 !> \param[in, out] itherm       thermal model
-!> \param[in, out] iale         ale module
-!> \param[in, out] ivofmt       vof method
-!> \param[in, out] icavit       cavitation model
+!> \param[in, out] iale         ALE module
 !______________________________________________________________________________!
 
 subroutine usipph &
- ( ixmlpu, iturb , itherm, iale , ivofmt, icavit )
+ ( ixmlpu, iturb , itherm, iale )
 
 !===============================================================================
 ! Module files
@@ -140,7 +137,7 @@ implicit none
 ! Arguments
 
 integer ixmlpu
-integer iturb, itherm, iale, ivofmt, icavit
+integer iturb, itherm, iale
 
 ! Local variables
 
@@ -171,17 +168,22 @@ integer iturb, itherm, iale, ivofmt, icavit
 return
 end subroutine usipph
 
-
 !===============================================================================
 
 !> \brief User subroutine for the input of additional user parameters.
 !
+!>  This subroutine allows setting parameters
+!>  which do not already appear in the other subroutines of this file.
+!>
+!>  It is possible to add or remove parameters.
+!>  The number of physical properties and variables is known here.
+!
 !-------------------------------------------------------------------------------
 ! Arguments
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[in]     nmodpp         number of active specific physics models
+!> \param[in]     nmodpp         number of active specific physical models     !
 !______________________________________________________________________________!
 
 subroutine usipsu &
@@ -200,7 +202,6 @@ use cstphy
 use entsor
 use parall
 use period
-use ihmpre
 use albase
 use ppppar
 use ppthch
@@ -211,6 +212,11 @@ use field
 use cavitation
 use post
 use rotation
+use atincl
+use atsoil
+use atchem
+use atimbr
+use sshaerosol
 use cs_c_bindings
 
 !===============================================================================
@@ -223,14 +229,6 @@ integer nmodpp
 
 !===============================================================================
 
-!>  This subroutine allows setting parameters
-!>  which do not already appear in the other subroutines of this file.
-!>
-!>  It is possible to add or remove parameters.
-!>  The number of physical properties and variables is known here.
-
-!===============================================================================
-
 !----
 ! Formats
 !----
@@ -238,7 +236,6 @@ integer nmodpp
 return
 end subroutine usipsu
 
-
 !===============================================================================
 
 !> \brief User subroutine for the input of additional user parameters for
@@ -269,7 +266,6 @@ use entsor
 use field
 use parall
 use period
-use ihmpre
 use post
 use ppppar
 use ppthch
@@ -297,52 +293,6 @@ integer nmodpp
 return
 end subroutine usipes
 
-
-!===============================================================================
-
-
-!> \brief Initialize non-standard calculation options for the atmospheric version.
-
-!-------------------------------------------------------------------------------
-! Arguments
-!______________________________________________________________________________.
-!  mode           name          role                                           !
-!______________________________________________________________________________!
-
-subroutine usati1
-
-!===============================================================================
-! Module files
-!===============================================================================
-
-use paramx
-use dimens
-use numvar
-use optcal
-use cstphy
-use entsor
-use cstnum
-use ppppar
-use atincl
-use atsoil
-use atchem
-use atimbr
-use siream
-
-!===============================================================================
-
-implicit none
-
-!===============================================================================
-
-!----
-! End
-!----
-
-return
-end subroutine usati1
-
-
 !===============================================================================
 ! Purpose:
 ! -------
@@ -382,7 +332,6 @@ use cstphy
 use entsor
 use cstnum
 use parall
-use ihmpre
 use period
 use ppppar
 use ppthch
@@ -431,6 +380,7 @@ use optcal
 use cstphy
 use parall
 use period
+use ppincl
 use ppppar
 use radiat
 
@@ -464,7 +414,6 @@ subroutine uscfx1
 !===============================================================================
 
 use paramx
-use ihmpre
 use dimens
 use numvar
 use optcal
@@ -488,35 +437,6 @@ implicit none
 
 !===============================================================================
 
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-!===============================================================================
-! 0.  This test allows the user to ensure that the version of this subroutine
-!       used is that from his case definition, and not that from the library.
-!     This subroutine is  mandatory for compressible flow,
-!       thus the default (library reference) version stops immediately.
-!===============================================================================
-
-if (iihmpr.eq.0) then
-  write(nfecra,9000)
-  call csexit (1)
-endif
-
- 9000 format(                                                     &
-'@',/,                                                            &
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',/,                                                            &
-'@ @@ WARNING:    stop in compressible flow options definition',/,&
-'@    =======',/,                                                 &
-'@     The user subroutine ''uscfx1'' must be completed.',/,      &
-'@',/,                                                            &
-'@  The calculation will not be run.',/,                          &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',/)
-
-
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
 !----
 ! End
 !----
@@ -554,7 +474,6 @@ subroutine uscfx2
 !===============================================================================
 
 use paramx
-use ihmpre
 use dimens
 use numvar
 use optcal
@@ -573,35 +492,6 @@ implicit none
 
 !===============================================================================
 
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-!===============================================================================
-! 0.  This test allows the user to ensure that the version of this subroutine
-!       used is that from his case definition, and not that from the library.
-!     This subroutine is  mandatory for compressible flow,
-!       thus the default (library reference) version stops immediately.
-!===============================================================================
-
-if (iihmpr.eq.0) then
-  write(nfecra,9000)
-  call csexit (1)
-endif
-
- 9000 format(                                                     &
-'@',/,                                                            &
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',/,                                                            &
-'@ @@ WARNING:    stop in data input for compressible flow',/,    &
-'@    =======',/,                                                 &
-'@     The user subroutine ''uscfx2'' must be completed',/,       &
-'@',/,                                                            &
-'@  The calculation will not be run.',/,                          &
-'@',/,                                                            &
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',/)
-
-
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
 !----
 ! End
 !----
@@ -646,40 +536,6 @@ implicit none
 
 !===============================================================================
 
-!===============================================================================
-
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-!===============================================================================
-! 0. This test allows the user to ensure that the version of this subroutine
-!       used is that from his case definition, and not that from the library.
-!     If a file from the GUI is used, this subroutine may not be mandatory,
-!       thus the default (library reference) version returns immediately.
-!===============================================================================
-
-if (1.eq.1) then
-  write(nfecra,9000)
-  call csexit (1)
-endif
-
- 9000 format(                                                     &
-'@',/,                                                            &
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',/,                                                            &
-'@ @@ WARNING:    stop in data input',/,                          &
-'@    =======',/,                                                 &
-'@     The user subroutine ''cs_user_cooling_towers''',/,         &
-'@     must be completed',/,                                      &
-'@     for the cooling tower module',/,                           &
-'@                                                            ',/,&
-'@',/,                                                            &
-'@  The calculation will not be run.',/,                          &
-'@',/,                                                            &
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',/)
-
-!===============================================================================
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
 !----
 ! End
 !----
@@ -705,7 +561,6 @@ subroutine user_darcy_ini1
 ! Module files
 !===============================================================================
 
-use ihmpre, only: iihmpr
 use entsor
 use darcy_module
 
@@ -715,37 +570,6 @@ implicit none
 
 !===============================================================================
 
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-!===============================================================================
-! 0. This test allows the user to ensure that the version of this subroutine
-!       used is that from his case definition, and not that from the library.
-!     If a file from the GUI is used, this subroutine may not be mandatory,
-!       thus the default (library reference) version returns immediately.
-!===============================================================================
-
-if (iihmpr.eq.0) then
-  write(nfecra,9000)
-  call csexit (1)
-endif
-
- 9000 format(                                                     &
-'@',/,                                                            &
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',/,                                                            &
-'@ @@ WARNING:    stop in data input',/,                          &
-'@    =======',/,                                                 &
-'@     The user subroutine ''user_darcy_ini1'' ',/,               &
-'@     must be completed',/,                                      &
-'@                                                            ',/,&
-'@',/,                                                            &
-'@  The calculation will not be run.',/,                          &
-'@',/,                                                            &
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@',/)
-
-!===============================================================================
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
 !----
 ! End
 !----
