@@ -1,5 +1,5 @@
 /*============================================================================
- * This function is called each time step to define physical properties
+ * User definition of physical properties.
  *============================================================================*/
 
 /* VERS */
@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -34,6 +34,7 @@
 
 #include <assert.h>
 #include <math.h>
+#include <string.h>
 
 #if defined(HAVE_MPI)
 #include <mpi.h>
@@ -46,37 +47,10 @@
 #include <ple_coupling.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-#include "bft_printf.h"
-
-#include "cs_base.h"
-#include "cs_field.h"
-#include "cs_field_pointer.h"
-#include "cs_field_operator.h"
-#include "cs_mesh.h"
-#include "cs_mesh_quantities.h"
-#include "cs_halo.h"
-#include "cs_halo_perio.h"
-#include "cs_log.h"
-#include "cs_parameters.h"
-#include "cs_prototypes.h"
-#include "cs_rotation.h"
-#include "cs_time_moment.h"
-#include "cs_time_step.h"
-#include "cs_turbomachinery.h"
-#include "cs_selector.h"
-
-#include "cs_post.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -98,16 +72,79 @@ BEGIN_C_DECLS
 /*!
  * \brief Function called at each time step to define physical properties.
  *
- * \param[in]  mesh             pointer to a cs_mesh_t structure
- * \param[in]  mesh_quantities  pointer to a cs_mesh_quantities_t structure
+ * \param[in, out]  domain   pointer to a cs_domain_t structure
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_physical_properties
 void
-cs_user_physical_properties(const cs_mesh_t             *mesh,
-                            const cs_mesh_quantities_t  *mesh_quantities)
+cs_user_physical_properties(cs_domain_t   *domain)
 {
+  CS_NO_WARN_IF_UNUSED(domain);
+}
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief User definition of enthalpy to temperature conversion.
+ *
+ * This allows overwriting the solver defaults if necessary.
+ *
+ * This function may be called on a per-zone basis, so as to allow different
+ * conversion relations in zones representing solids or different fluids.
+ *
+ * \param[in, out]  domain   pointer to a cs_domain_t structure
+ * \param[in]       z        zone (volume or boundary) applying to current call
+ * \param[in]       z_local  if true, h and t arrays are defined in a compact
+ *                           (contiguous) manner for this zone only;
+ *                           if false, h and t are defined on the zone's parent
+ *                           location (usually all cells or boundary faces)
+ * \param[in]       h        enthalpy values
+ * \param[in, out]  t        temperature values
+ */
+/*----------------------------------------------------------------------------*/
+
+#pragma weak cs_user_physical_properties_h_to_t
+void
+cs_user_physical_properties_h_to_t(cs_domain_t      *domain,
+                                   const cs_zone_t  *z,
+                                   bool              z_local,
+                                   const cs_real_t   h[],
+                                   cs_real_t         t[])
+{
+  CS_NO_WARN_IF_UNUSED(domain);
+  CS_NO_WARN_IF_UNUSED(z);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief User definition of temperature to enthalpy conversion.
+ *
+ * This allows overwriting the solver defaults if necessary.
+ *
+ * This function may be called on a per-zone basis, so as to allow different
+ * conversion relations in zones representing solids or different fluids.
+ *
+ * \param[in, out]  domain   pointer to a cs_domain_t structure
+ * \param[in]       z        zone (volume or boundary) applying to current call
+ * \param[in]       z_local  if true, h and t arrays are defined in a compact
+ *                           (contiguous) manner for this zone only;
+ *                           if false, h and t are defined on the zone's parent
+ *                           location (usually all cells or boundary faces)
+ * \param[in]       h        temperature values
+ * \param[in, out]  t        enthalpy values
+ */
+/*----------------------------------------------------------------------------*/
+
+#pragma weak cs_user_physical_properties_t_to_h
+void
+cs_user_physical_properties_t_to_h(cs_domain_t      *domain,
+                                   const cs_zone_t  *z,
+                                   bool              z_local,
+                                   const cs_real_t   t[],
+                                   cs_real_t         h[])
+{
+  CS_NO_WARN_IF_UNUSED(domain);
+  CS_NO_WARN_IF_UNUSED(z);
 }
 
 /*----------------------------------------------------------------------------*/
