@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -36,34 +36,10 @@
 #include "string.h"
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-
-#include "cs_base.h"
-#include "cs_field.h"
-#include "cs_geom.h"
-#include "cs_interpolate.h"
-#include "cs_mesh.h"
-#include "cs_selector.h"
-#include "cs_parall.h"
-#include "cs_post.h"
-#include "cs_post_util.h"
-#include "cs_probe.h"
-#include "cs_time_plot.h"
-
-#include "cs_field_pointer.h"
-#include "cs_parameters.h"
-#include "cs_physical_constants.h"
-#include "cs_turbulence_model.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -88,6 +64,7 @@ BEGIN_C_DECLS
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_postprocess_writers
 void
 cs_user_postprocess_writers(void)
 {
@@ -104,6 +81,7 @@ cs_user_postprocess_writers(void)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_postprocess_meshes
 void
 cs_user_postprocess_meshes(void)
 {
@@ -118,6 +96,7 @@ cs_user_postprocess_meshes(void)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_postprocess_probes
 void
 cs_user_postprocess_probes(void)
 {
@@ -150,6 +129,7 @@ cs_user_postprocess_probes(void)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_postprocess_values
 void
 cs_user_postprocess_values(const char            *mesh_name,
                            int                    mesh_id,
@@ -165,7 +145,19 @@ cs_user_postprocess_values(const char            *mesh_name,
                            const cs_lnum_t        vertex_list[],
                            const cs_time_step_t  *ts)
 {
-
+  CS_UNUSED(mesh_name);
+  CS_UNUSED(mesh_id);
+  CS_UNUSED(cat_id);
+  CS_UNUSED(probes);
+  CS_UNUSED(n_cells);
+  CS_UNUSED(n_i_faces);
+  CS_UNUSED(n_b_faces);
+  CS_UNUSED(n_vertices);
+  CS_UNUSED(cell_list);
+  CS_UNUSED(i_face_list);
+  CS_UNUSED(b_face_list);
+  CS_UNUSED(vertex_list);
+  CS_UNUSED(ts);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -180,12 +172,15 @@ cs_user_postprocess_values(const char            *mesh_name,
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_postprocess_activate
 void
 cs_user_postprocess_activate(int     nt_max_abs,
                              int     nt_cur_abs,
                              double  t_cur_abs)
 {
-
+  CS_UNUSED(nt_max_abs);
+  CS_UNUSED(nt_cur_abs);
+  CS_UNUSED(t_cur_abs);
 }
 
 /*----------------------------------------------------------------------------*/
