@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -41,41 +41,10 @@
 #endif
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-#include "bft_printf.h"
-
-#include "cs_base.h"
-#include "cs_field.h"
-#include "cs_field_pointer.h"
-#include "cs_field_operator.h"
-#include "cs_math.h"
-#include "cs_mesh.h"
-#include "cs_mesh_location.h"
-#include "cs_mesh_quantities.h"
-#include "cs_halo.h"
-#include "cs_log.h"
-#include "cs_parall.h"
-#include "cs_parameters.h"
-#include "cs_physical_constants.h"
-#include "cs_physical_model.h"
-#include "cs_prototypes.h"
-#include "cs_restart.h"
-#include "cs_rotation.h"
-#include "cs_time_step.h"
-#include "cs_selector.h"
-#include "cs_rad_transfer.h"
-
-#include "cs_post.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -90,13 +59,9 @@ BEGIN_C_DECLS
  * \brief User function for input of radiative transfer parameters:
  *        absorption coefficient and net radiation flux.
  *
- *  See \subpage cs_user_radiative_transfer for examples.
+ *  See \ref cs_user_radiative_transfer for examples.
  */
 
-/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
-
-/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
-
 /*=============================================================================
  * Public function definitions
  *============================================================================*/
@@ -107,6 +72,7 @@ BEGIN_C_DECLS
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_radiative_transfer_parameters
 void
 cs_user_radiative_transfer_parameters(void)
 {
@@ -128,19 +94,18 @@ cs_user_radiative_transfer_parameters(void)
  * In the case of the P-1 model, we check that the optical length is at
  * least of the order of 1.
  *
- * \param[in]     bc_type       boundary face types
- * \param[in]     dt            time step (per cell)
- * \param[out]    ck            medium's absorption coefficient
- *                              (zero if transparent)
+ * \param[in]   bc_type  boundary face types
+ * \param[out]  ck       medium's absorption coefficient (zero if transparent)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_rad_transfer_absorption
 void
-cs_user_rad_transfer_absorption(const int         bc_type[],
-                                const cs_real_t   dt[],
-                                cs_real_t         ck[])
+cs_user_rad_transfer_absorption(const int  bc_type[],
+                                cs_real_t  ck[])
 {
-
+  CS_UNUSED(bc_type);
+  CS_UNUSED(ck);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -150,11 +115,10 @@ cs_user_rad_transfer_absorption(const int         bc_type[],
  * The density of net radiation flux must be calculated
  * consistently with the boundary conditions of the intensity.
  * The density of net flux is the balance between the radiative
- * emiting part of a boudary face (and not the reflecting one)
+ * emiting part of a boundary face (and not the reflecting one)
  * and the radiative absorbing part.
  *
  * \param[in]   bc_type   boundary face types
- * \param[in]   dt        time step (per cell)
  * \param[in]   coefap    boundary condition work array for the luminance
  *                         (explicit part)
  * \param[in]   coefbp    boundary condition work array for the luminance
@@ -173,9 +137,9 @@ cs_user_rad_transfer_absorption(const int         bc_type[],
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_rad_transfer_net_flux
 void
 cs_user_rad_transfer_net_flux(const int        bc_type[],
-                              const cs_real_t  dt[],
                               const cs_real_t  coefap[],
                               const cs_real_t  coefbp[],
                               const cs_real_t  cofafp[],
@@ -188,7 +152,18 @@ cs_user_rad_transfer_net_flux(const int        bc_type[],
                               const cs_real_t  ck[],
                               cs_real_t        net_flux[])
 {
-
+  CS_UNUSED(bc_type);
+  CS_UNUSED(coefap);
+  CS_UNUSED(coefbp);
+  CS_UNUSED(cofafp);
+  CS_UNUSED(cofbfp);
+  CS_UNUSED(twall);
+  CS_UNUSED(qincid);
+  CS_UNUSED(xlam);
+  CS_UNUSED(epa);
+  CS_UNUSED(eps);
+  CS_UNUSED(ck);
+  CS_UNUSED(net_flux);
 }
 
 /*----------------------------------------------------------------------------*/
