@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -41,50 +41,10 @@
 #endif
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-#include "bft_printf.h"
-
-#include "cs_1d_wall_thermal.h"
-#include "cs_base.h"
-#include "cs_boundary_zone.h"
-#include "cs_fan.h"
-#include "cs_field.h"
-#include "cs_gui_util.h"
-#include "cs_field_pointer.h"
-#include "cs_field_operator.h"
-#include "cs_math.h"
-#include "cs_mesh.h"
-#include "cs_mesh_location.h"
-#include "cs_mesh_quantities.h"
-#include "cs_halo.h"
-#include "cs_halo_perio.h"
-#include "cs_log.h"
-#include "cs_multigrid.h"
-#include "cs_parameters.h"
-#include "cs_physical_constants.h"
-#include "cs_physical_properties.h"
-#include "cs_prototypes.h"
-#include "cs_rotation.h"
-#include "cs_sles.h"
-#include "cs_sles_it.h"
-#include "cs_time_moment.h"
-#include "cs_time_step.h"
-#include "cs_turbomachinery.h"
-#include "cs_selector.h"
-#include "cs_rad_transfer.h"
-#include "cs_thermal_model.h"
-
-#include "cs_post.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -96,10 +56,6 @@ BEGIN_C_DECLS
 
 /*! \file cs_user_radiative_transfer_bcs.c */
 
-/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
-
-/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
-
 /*=============================================================================
  * Public function definitions
  *============================================================================*/
@@ -108,7 +64,7 @@ BEGIN_C_DECLS
 /*!
  * \brief User definition of radiative transfer boundary conditions.
  *
- * See \subpage cs_user_radiative_transfer for examples.
+ * See \ref cs_user_radiative_transfer for examples.
  *
  * \warning the temperature unit here is the Kelvin
  *
@@ -129,15 +85,16 @@ BEGIN_C_DECLS
  *
  * The following face characteristics must be set:
  *  - isothp(face_id) boundary face type
- *               = itpimp -> Gray wall with fixed inside temperature
- *               = ipgrno -> Gray wall with fixed outside temperature
- *               = iprefl -> Reflecting wall with fixed outside temperature
- *               = ifgrno -> Gray wall with fixed conduction flux
- *               = ifrefl -> Reflecting wall with fixed conduction flux
- *  - tintp(face_id) inside wall temperature (Kelvin)
- *               initialize thwall at the first time step.
- *               If isothp = itpimp, the value of thwall is fixed to tintp
- *               In the other case, tintp is only for initialization.
+ *    * CS_BOUNDARY_RAD_WALL_GRAY:
+ *      Gray wall with temperature based on fluid BCs
+ *    * CS_BOUNDARY_RAD_WALL_GRAY_EXTERIOR_T:
+ *      Gray wall with fixed outside temperature
+ *    * CS_BOUNDARY_RAD_WALL_REFL_EXTERIOR_T:
+ *      Reflecting wall with fixed outside temperature
+ *    * CS_BOUNDARY_RAD_WALL_GRAY_COND_FLUX:
+ *      Gray wall with fixed conduction flux
+ *    * CS_BOUNDARY_RAD_WALL_REFL_COND_FLUX:
+ *      Reflecting wall with fixed conduction flux
  *
  * Depending on the value of isothp, other values may also need to be set:
  *  - rcodcl = conduction flux
@@ -149,23 +106,7 @@ BEGIN_C_DECLS
  * \param[in]     nvar          total number of variable BC's
  * \param[in]     bc_type       boundary face types
  * \param[in]     icodcl        boundary face code
- *                                - 1  -> Dirichlet
- *                                - 2  -> convective outlet
- *                                - 3  -> flux density
- *                                - 4  -> sliding wall and u.n=0 (velocity)
- *                                - 5  -> friction and u.n=0 (velocity)
- *                                - 6  -> roughness and u.n=0 (velocity)
- *                                - 9  -> free inlet/outlet (velocity)
- *                                inflowing possibly blocked
  * \param[in]     isothp        boundary face type for radiative transfer
- *                                - itpimp -> Gray wall with fixed inside temp
- *                                - ipgrno -> Gray wall with fixed outside temp
- *                                - iprefl -> Reflecting wall with fixed
- *                                         outside temp
- *                                - ifgrno -> Gray wall with fixed
- *                                      conduction flux
- *                                - ifrefl -> Reflecting wall with fixed
- *                                      conduction flux
  * \param[out]    tmin          min allowed value of the wall temperature
  * \param[out]    tmax          max allowed value of the wall temperature
  * \param[in]     tx            relaxation coefficient (0 < tx < 1)
@@ -181,10 +122,10 @@ BEGIN_C_DECLS
  * \param[out]    epap          thickness (m)
  * \param[out]    epsp          emissivity (>0)
  * \param[out]    textp         outside temperature (K)
- * \param[out]    tintp         initial inside temperature (K)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_radiative_transfer_bcs
 void
 cs_user_radiative_transfer_bcs(int               nvar,
                                const int         bc_type[],
@@ -202,10 +143,25 @@ cs_user_radiative_transfer_bcs(int               nvar,
                                cs_real_t         xlamp[],
                                cs_real_t         epap[],
                                cs_real_t         epsp[],
-                               cs_real_t         textp[],
-                               cs_real_t         tintp[])
+                               cs_real_t         textp[])
 {
-
+  CS_UNUSED(nvar);
+  CS_UNUSED(bc_type);
+  CS_UNUSED(icodcl);
+  CS_UNUSED(isothp);
+  CS_UNUSED(tmin);
+  CS_UNUSED(tmax);
+  CS_UNUSED(tx);
+  CS_UNUSED(dt);
+  CS_UNUSED(rcodcl);
+  CS_UNUSED(thwall);
+  CS_UNUSED(qincid);
+  CS_UNUSED(hfcnvp);
+  CS_UNUSED(flcnvp);
+  CS_UNUSED(xlamp);
+  CS_UNUSED(epap);
+  CS_UNUSED(epsp);
+  CS_UNUSED(textp);
 }
 
 /*----------------------------------------------------------------------------*/
