@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -39,25 +39,10 @@
 #include <string.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_error.h"
-#include "bft_mem.h"
-#include "bft_printf.h"
-
-#include "cs_base.h"
-#include "cs_mesh.h"
-#include "cs_mesh_quantities.h"
-#include "cs_post.h"
-#include "cs_restart.h"
-#include "cs_time_plot.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -69,7 +54,7 @@ BEGIN_C_DECLS
  *
  * \brief User solver setting and implementation
  *
- * See \subpage user_solver for examples.
+ * See \ref user_solver for examples.
  */
 /*----------------------------------------------------------------------------*/
 
@@ -85,6 +70,7 @@ BEGIN_C_DECLS
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_solver_set
 int
 cs_user_solver_set(void)
 {
@@ -101,11 +87,13 @@ cs_user_solver_set(void)
  */
 /*----------------------------------------------------------------------------*/
 
+#pragma weak cs_user_solver
 void
 cs_user_solver(const cs_mesh_t             *mesh,
                const cs_mesh_quantities_t  *mesh_quantities)
 {
-
+  CS_UNUSED(mesh);
+  CS_UNUSED(mesh_quantities);
 }
 
 /*----------------------------------------------------------------------------*/
