@@ -3,7 +3,7 @@
 !
 ! 1) Momentum equation (coupled solver)
 ! 2) Species transport
-! 3) Turbulence (k-epsilon, k-omega, Rij-epsilon, v2-f, Spalart-Allmaras)
+! 3) Turbulence (Rij-epsilon)
 !===============================================================================
 
 !-------------------------------------------------------------------------------
@@ -12,7 +12,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -31,19 +31,19 @@
 !-------------------------------------------------------------------------------
 
 !===============================================================================
-! Purpose:
-! -------
-
 !> \file cs_user_source_terms.f90
 !>
-!> \brief Additional right-hand side source terms
+!> \brief User subroutines for additional right-hand side source terms
 !>
-!> See \subpage cs_user_source_terms and \subpage cs_user_source_terms-scalar_in_a_channel
-!> for examples.
+!> See \ref cs_user_source_terms and
+!> \ref cs_user_source_terms-scalar_in_a_channel for examples.
 !>
+!===============================================================================
 !> \brief Additional right-hand side source terms for velocity components equation
 !> (Navier-Stokes)
 !>
+!> \deprecated Use \ref cs_user_source_terms instead.
+!>
 !> \section ustsnv_use  Usage
 !>
 !> The additional source term is decomposed into an explicit part (\c crvexp) and
@@ -64,7 +64,7 @@
 !> the routine. It is not needed to do it in the routine (waste of CPU time).
 !>
 !> \remark The additional force on \f$ x_i \f$ direction is given by
-!>  \c crvexp(i, iel) + vel(j, iel)* crvimp(j, i).
+!>  \c crvexp(i, iel) + vel(j, iel)* crvimp(j, i, iel).
 !>
 !> For stability reasons, Code_Saturne will not add -crvimp directly to the
 !> diagonal of the matrix, but Max(-crvimp,0). This way, the crvimp term is
@@ -97,12 +97,10 @@
 !> \param[in]     icepdc        index number of cells with head loss terms
 !> \param[in]     icetsm        index number of cells with mass source terms
 !> \param[in]     itypsm        type of mass source term for each variable
-!>                               (see \ref cs_user_mass_source_terms)
 !> \param[in]     dt            time step (per cell)
 !> \param[in]     ckupdc        head loss coefficient
 !> \param[in]     smacel        value associated to each variable in the mass
-!>                               source terms or mass rate (see
-!>                               \ref cs_user_mass_source_terms)
+!>                              source terms or mass rate
 !> \param[out]    crvexp        explicit part of the source term
 !> \param[out]    crvimp        implicit part of the source term
 !_______________________________________________________________________________
@@ -145,7 +143,7 @@ integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 double precision crvexp(3,ncelet), crvimp(3,3,ncelet)
 
 ! Local variables
@@ -173,7 +171,7 @@ allocate(lstelt(ncel))
 ! Formats
 !--------
 
- 1000 format(' User source terms for variable ',A8,/)
+ 1000 format(' User source terms for variable ', a8,/)
 
 !----
 ! End
@@ -189,13 +187,11 @@ end subroutine ustsnv
 !===============================================================================
 
 !===============================================================================
-! Purpose:
-! -------
-
 !>    User subroutine.
 !> \brief    Additional right-hand side source terms for scalar equations (user
 !>     scalars and specific physics scalars).
 !>
+!> \deprecated Use \ref cs_user_source_terms instead.
 !>
 !> Usage
 !> -----
@@ -285,12 +281,10 @@ end subroutine ustsnv
 !> \param[in]     icepdc        index number of cells with head loss terms
 !> \param[in]     icetsm        index number of cells with mass source terms
 !> \param[in]     itypsm        type of mass source term for each variable
-!> \param[in]                    (see \ref cs_user_mass_source_terms)
 !> \param[in]     dt            time step (per cell)
 !> \param[in]     ckupdc        head loss coefficient
 !> \param[in]     smacel        value associated to each variable in the mass
-!> \param[in]                    source terms or mass rate (see \ref
-!>                               cs_user_mass_source_terms)
+!> \param[in]                   source terms or mass rate
 !> \param[out]    crvexp        explicit part of the source term
 !> \param[out]    crvimp        implicit part of the source term
 !______________________________________________________________________________!
@@ -332,7 +326,7 @@ integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 double precision crvexp(ncelet), crvimp(ncelet)
 
 ! Local variables
@@ -376,13 +370,10 @@ end subroutine ustssc
 !===============================================================================
 
 !===============================================================================
-! Purpose:
-! -------
-
-!>    User subroutine.
 !> \brief    Additional right-hand side source terms for vectorial equations
 !>           (user vectors and specific physics vectors).
 !>
+!> \deprecated Use \ref cs_user_source_terms instead.
 !>
 !> Usage
 !> -----
@@ -437,7 +428,7 @@ end subroutine ustssc
 !>
 
 !>
-!> STEP SOURCE TERMS
+!> STEEP SOURCE TERMS
 !>===================
 !> In case of a complex, non-linear source term, say F(f), for scalar f, the
 !> easiest method is to implement the source term explicitely.
@@ -473,12 +464,10 @@ end subroutine ustssc
 !> \param[in]     icepdc        index number of cells with head loss terms
 !> \param[in]     icetsm        index number of cells with mass source terms
 !> \param[in]     itypsm        type of mass source term for each variable
-!> \param[in]                    (see \ref cs_user_mass_source_terms)
 !> \param[in]     dt            time step (per cell)
 !> \param[in]     ckupdc        head loss coefficient
 !> \param[in]     smacel        value associated to each variable in the mass
-!> \param[in]                    source terms or mass rate
-!>                               (see \ref cs_user_mass_source_terms)
+!> \param[in]                   source terms or mass rate
 !> \param[out]    crvexp        explicit part of the source term
 !> \param[out]    crvimp        implicit part of the source term
 !______________________________________________________________________________!
@@ -520,7 +509,7 @@ integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 double precision crvexp(3,ncelet), crvimp(3,3,ncelet)
 
 ! Local variables
@@ -561,164 +550,14 @@ deallocate(lstelt)
 return
 end subroutine ustsvv
 
-
 !===============================================================================
 
 !===============================================================================
-! Purpose:
-! -------
-
-!> \brief Additional right-hand side source terms for turbulence models
-!>
-!> \section cs_user_turbulence_source_terms_use  Usage
-!>
-!> The additional source term is decomposed into an explicit part (crvexp) and
-!> an implicit part (crvimp) that must be provided here.
-!> The resulting equations solved by the code are:
-!> \f[
-!>  \rho \norm{\vol{\celli}} \DP{\varia} + ....
-!>   = \tens{crvimp} \varia + \vect{crvexp}
-!> \f]
-!> where \f$ \varia \f$ is the turbulence field of index \c f_id
-!>
-!> Note that crvexp, crvimp are defined after the Finite Volume
-!> integration over the cells, so they include the "volume" term. More precisely:
-!>   - crvexp is expressed in kg.m2/s2
-!>   - crvimp is expressed in kg/s
-!>
-!> The crvexp, crvimp arrays are already initialized to 0 before
-!> entering the routine. It is not needed to do it in the routine (waste of CPU time).
-!>
-!> For stability reasons, Code_Saturne will not add -crvimp directly to the
-!> diagonal of the matrix, but Max(-crvimp,0). This way, the crvimp term is
-!> treated implicitely only if it strengthens the diagonal of the matrix.
-!> However, when using the second-order in time scheme, this limitation cannot
-!> be done anymore and -crvimp is added directly. The user should therefore test
-!> the negativity of crvimp by himself.
-!>
-!> When using the second-order in time scheme, one should supply:
-!>   - crvexp at time n
-!>   - crvimp at time n+1/2
-!>
-!> The selection of cells where to apply the source terms is based on a getcel
-!> command. For more info on the syntax of the \ref getcel command, refer to the
-!> user manual or to the comments on the similar command \ref getfbr in the routine
-!> \ref cs_user_boundary_conditions.
-!
-!-------------------------------------------------------------------------------
-
-!-------------------------------------------------------------------------------
-! Arguments
-!______________________________________________________________________________.
-!  mode           name          role                                           !
-!______________________________________________________________________________!
-!> \param[in]     nvar          total number of variables
-!> \param[in]     nscal         total number of scalars
-!> \param[in]     ncepdp        number of cells with head loss terms
-!> \param[in]     ncesmp        number of cells with mass source terms
-!> \param[in]     f_id          field index of the current turbulent variable
-!> \param[in]     icepdc        index number of cells with head loss terms
-!> \param[in]     icetsm        index number of cells with mass source terms
-!> \param[in]     itypsm        type of mass source term for each variable
-!>                               (see \ref cs_user_mass_source_terms)
-!> \param[in]     ckupdc        head loss coefficient
-!> \param[in]     smacel        value associated to each variable in the mass
-!>                               source terms or mass rate (see
-!>                               \ref cs_user_mass_source_terms)
-!> \param[out]    crvexp        explicit part of the source term
-!> \param[out]    crvimp        implicit part of the source term
-!_______________________________________________________________________________
-
-subroutine cs_user_turbulence_source_terms &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   f_id   ,                                                       &
-   icepdc , icetsm , itypsm ,                                     &
-   ckupdc , smacel ,                                              &
-   crvexp , crvimp )
-
-!===============================================================================
-
-!===============================================================================
-! Module files
-!===============================================================================
-
-use paramx
-use numvar
-use entsor
-use optcal
-use cstphy
-use parall
-use period
-use mesh
-use field
-use cs_f_interfaces
-use cs_c_bindings
-
-!===============================================================================
-
-implicit none
-
-! Arguments
-
-integer          nvar   , nscal
-integer          ncepdp , ncesmp
-integer          f_id
-
-integer          icepdc(ncepdp)
-integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
-
-double precision dt(ncelet)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
-double precision crvexp(ncelet), crvimp(ncelet)
-
-! Local variables
-
-integer, allocatable, dimension(:) :: lstelt
-
-!===============================================================================
-
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-!===============================================================================
-
-if (.true.) return
-
-!===============================================================================
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
-
-!===============================================================================
-! 1. Initialization
-!===============================================================================
-
-! Allocate a temporary array for cells selection
-allocate(lstelt(ncel))
-
-!--------
-! Formats
-!--------
-
- 1000 format(' User source terms for turbulence model',/)
-
-!----
-! End
-!----
-
-! Deallocate the temporary array
-deallocate(lstelt)
-
-return
-end subroutine cs_user_turbulence_source_terms
-
-!===============================================================================
-
-
-!===============================================================================
-! Purpose:
-! -------
-
 !> \brief Additional right-hand side source terms for turbulence models and
 !>irijco =1
 !>
+!> \deprecated Use \ref cs_user_source_terms instead.
+!>
 !> \section cs_user_rij_source_terms_use  Usage
 !>
 !> The additional source term is decomposed into an explicit part (crvexp) and
@@ -769,11 +608,9 @@ end subroutine cs_user_turbulence_source_terms
 !> \param[in]     icepdc        index number of cells with head loss terms
 !> \param[in]     icetsm        index number of cells with mass source terms
 !> \param[in]     itypsm        type of mass source term for each variable
-!>                               (see \ref cs_user_mass_source_terms)
 !> \param[in]     ckupdc        head loss coefficient
 !> \param[in]     smacel        value associated to each variable in the mass
-!>                               source terms or mass rate (see
-!>                               \ref cs_user_mass_source_terms)
+!>                              source terms or mass rate
 !> \param[out]    crvexp        explicit part of the source term
 !> \param[out]    crvimp        implicit part of the source term
 !_______________________________________________________________________________
@@ -817,7 +654,7 @@ integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 double precision crvexp(6,ncelet), crvimp(6,6,ncelet)
 
 ! Local variables
