@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -51,6 +51,8 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
+!>                               - 13 Dirichlet for the advection operator and
+!>                                    Neumann for the diffusion operator
 !> \param[in]     itrifb        indirection for boundary faces ordering
 !> \param[in,out] itypfb        boundary face types
 !> \param[out]    izfppp        boundary face zone number
@@ -60,8 +62,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughness
-!>                                 in m if icodcl=6
+!>                                 (negative if gain) in w/m2
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
 !>                                 -# for the pressure \f$ \Delta t
@@ -91,18 +92,22 @@ use cstnum
 use entsor
 use parall
 use period
-use ihmpre
 use ppppar
 use ppthch
 use coincl
 use cpincl
 use ppincl
 use ppcpfu
+use atchem
 use atincl
+use atsoil
 use ctincl
 use cs_fuel_incl
 use mesh
 use field
+use turbomachinery
+use iso_c_binding
+use cs_c_bindings
 
 !===============================================================================
 
@@ -125,6 +130,7 @@ double precision rcodcl(nfabor,nvar,3)
 integer          ifac, ii
 integer          izone
 integer          ilelt, nlelt
+integer          f_id_rough, f_id_t_rough
 double precision d2s3
 double precision zref, xuref
 double precision ustar, rugd, rugt
@@ -132,6 +138,8 @@ double precision zent, xuent, xvent
 double precision xkent, xeent
 
 integer, allocatable, dimension(:) :: lstelt
+double precision, dimension(:), pointer :: bpro_roughness
+double precision, dimension(:), pointer :: bpro_roughness_t
 !< [loc_var_dec]
 
 !===============================================================================
@@ -166,9 +174,9 @@ rugt = rugd
 
 !< [example_1]
 call getfbr('11',nlelt,lstelt)
-!==========
-!   - Zone number (from 1 to n)
-izone = 1
+
+! Get a new zone number (1 <= izone <= nozppm)
+izone = maxval(izfppp) + 1
 
 do ilelt = 1, nlelt
 
@@ -180,6 +188,9 @@ do ilelt = 1, nlelt
   ! - Boundary conditions are prescribed from the meteo profile
   iprofm(izone) = 1
 
+  ! - Chemical boundary conditions are prescribed from the chemistry profile
+  iprofc(izone) = 1
+
   ! - boundary condition type can be set to ientre or i_convective_inlet
 
   itypfb(ifac) = ientre
@@ -199,9 +210,9 @@ enddo
 
 !< [example_2]
 call getfbr('21',nlelt,lstelt)
-!==========
-!   -  Zone number (from 1 to n)
-izone = 2
+
+! Get a new zone number (1 <= izone <= nozppm)
+izone = maxval(izfppp) + 1
 
 do ilelt = 1, nlelt
 
@@ -213,6 +224,9 @@ do ilelt = 1, nlelt
   ! - Boundary conditions are prescribed from the meteo profile
   iprofm(izone) = 1
 
+  ! - Chemical boundary conditions are prescribed from the chemistry profile
+  iprofc(izone) = 1
+
   ! - Assign inlet boundary conditions
   itypfb(ifac) = ientre
 
@@ -226,10 +240,9 @@ enddo
 
 !< [example_3]
 call getfbr('31',nlelt,lstelt)
-!==========
 
-!   - Zone number (from 1 to n)
-izone = 3
+! Get a new zone number (1 <= izone <= nozppm)
+izone = maxval(izfppp) + 1
 
 do ilelt = 1, nlelt
 
@@ -241,14 +254,17 @@ do ilelt = 1, nlelt
   ! - Boundary conditions are prescribed from the meteo profile
   iprofm(izone) = 1
 
+  ! - Chemical boundary conditions are prescribed from the chemistry profile
+  iprofc(izone) = 1
+
   ! - Dynamical variables are prescribed with a rough log law
   zent=cdgfbo(3,ifac)
 
-  ustar=xkappa*xuref/log((zref+rugd)/rugd)
-  xuent=ustar/xkappa*log((zent+rugd)/rugd)
+  ustar = xkappa*xuref/log((zref+rugd)/rugd)
+  xuent = ustar/xkappa*log((zent+rugd)/rugd)
   xvent = 0.d0
-  xkent=ustar**2/sqrt(cmu)
-  xeent=ustar**3/xkappa/(zent+rugd)
+  xkent = ustar**2/sqrt(cmu)
+  xeent = ustar**3/xkappa/(zent+rugd)
 
   itypfb(ifac) = ientre
 
@@ -291,14 +307,13 @@ do ilelt = 1, nlelt
   endif
 
 enddo
-!< [example_3]
 
+!< [example_3]
 ! --- Prescribe at boundary faces of color '12' an outlet for all phases
 call getfbr('12', nlelt, lstelt)
-!==========
 
-!   - Zone number (from 1 to n)
-izone = 4
+! Get a new zone number (1 <= izone <= nozppm)
+izone = maxval(izfppp) + 1
 
 do ilelt = 1, nlelt
 
@@ -315,13 +330,17 @@ do ilelt = 1, nlelt
 
 enddo
 
-! --- Prescribe at boundary faces of color 15 a rough wall for all phases
+! --- Prescribe at boundary faces of color 15 a rough wall
 !< [example_4]
-call getfbr('15', nlelt, lstelt)
-!==========
+call field_get_id_try("boundary_roughness", f_id_rough)
+if (f_id_rough.ge.0) call field_get_val_s(f_id_rough, bpro_roughness)
+call field_get_id_try("boundary_thermal_roughness", f_id_t_rough)
+if (f_id_t_rough.ge.0) call field_get_val_s(f_id_t_rough, bpro_roughness_t)
 
-!   - Zone number (from 1 to n)
-izone = 5
+call getfbr('15', nlelt, lstelt)
+
+! Get a new zone number (1 <= izone <= nozppm)
+izone = maxval(izfppp) + 1
 
 do ilelt = 1, nlelt
 
@@ -336,24 +355,23 @@ do ilelt = 1, nlelt
   itypfb(ifac)   = iparug
 
   ! Roughness for velocity: rugd
-  rcodcl(ifac,iu,3) = rugd
+  bpro_roughness(ifac) = rugd
 
-  ! Roughness for scalars:
-  rcodcl(ifac,iv,3) = rugt
+  ! Roughness for scalars (if required)
+  if (f_id_rough.ge.0) &
+    bpro_roughness_t(ifac) = 0.01d0
 
   ! By default zero flux for scalars
 
-
 enddo
 !< [example_4]
 
 ! --- Prescribe at boundary faces of color 4 a symmetry for all phases
 !< [example_5]
 call getfbr('4', nlelt, lstelt)
-!==========
 
-!   - Zone number (from 1 to n)
-izone = 6
+! Get a new zone number (1 <= izone <= nozppm)
+izone = maxval(izfppp) + 1
 
 do ilelt = 1, nlelt
 
