@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -63,8 +63,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughness
-!>                                 in m if icodcl=6
+!>                                 (negative if gain) in w/m2
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
 !>                                 -# for the pressure \f$ \Delta t
@@ -94,7 +93,6 @@ use cstnum
 use entsor
 use parall
 use period
-use ihmpre
 use ppppar
 use ppthch
 use coincl
@@ -338,7 +336,11 @@ else
     call parrsm(2, acc)
   endif
 
-  fmul = fmprsc/(acc(1)/acc(2)) ! 1 / estimate flow multiplier
+  if (acc(1).le.epzero) then
+     fmul = 0.d0 ! zero velocity in bulk domain
+  else
+     fmul = fmprsc/(acc(1)/acc(2)) ! 1 / estimate flow multiplier
+  endif
 
   ! Apply BC
 
