@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -58,8 +58,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughness
-!>                                 in m if icodcl=6
+!>                                 (negative if gain) in w/m2
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
 !>                                 -# for the pressure \f$ \Delta t
@@ -89,7 +88,6 @@ use cstnum
 use entsor
 use parall
 use period
-use ihmpre
 use ppppar
 use ppthch
 use coincl
@@ -217,12 +215,12 @@ do ilelt = 1, nlelt
     ! We will also be careful if necessary to use a more precise
     ! formula for the dynamic viscosity use in the calculation of
     ! the Reynolds number (especially if it is variable, it may be
-    ! useful to take the law from 'usphyv'. Here, we use by default
+    ! useful to take the law from 'cs_user_physical_properties'. Here, we use by default
     ! the 'viscl0" value.
 
     ! Regarding the density, we have acess to its value at boundary
     ! faces (romb) so this value is the one used here (specifically,
-    ! it is consistent with the processing in 'usphyv', in case of
+    ! it is consistent with the processing in 'cs_user_physical_properties', in case of
     ! variable density)
 
     ! Hydraulic diameter
