@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2018 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -102,7 +102,7 @@ double precision tpar
 
 !< [loc_var_dec]
 integer          ieltcd, ii, iz
-integer          ifac, iel, iesp, iscal
+integer          ifac, iel, iscal
 integer          ivarh
 integer          ilelt, nlelt
 integer          izone
