@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -31,29 +31,14 @@
 /*----------------------------------------------------------------------------
  * Standard C library headers
  *----------------------------------------------------------------------------*/
+
 #include <math.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "cs_base.h"
-#include "cs_mesh.h"
-#include "cs_math.h"
-#include "cs_mesh_quantities.h"
-#include "cs_elec_model.h"
-#include "bft_mem.h"
-#include "bft_printf.h"
-#include "cs_field.h"
-#include "cs_field_pointer.h"
-#include "cs_parall.h"
-#include "cs_physical_model.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -102,7 +87,7 @@ cs_user_scaling_elec(const cs_mesh_t             *mesh,
   const cs_real_3_t *cdgfac = (const cs_real_3_t *) mesh_quantities->i_face_cog;
 
   cs_elec_option_t *elec_opt = cs_get_glob_elec_option();
-  const int kivisl = cs_field_key_id("scalar_diffusivity_id");
+  const int kivisl = cs_field_key_id("diffusivity_id");
 
   int ielarc = cs_glob_physical_model_flag[CS_ELECTRIC_ARCS];
 
