@@ -9,7 +9,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -48,38 +48,10 @@
 #include <ple_coupling.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-#include "bft_printf.h"
-
-#include "cs_base.h"
-#include "cs_balance_by_zone.h"
-#include "cs_field.h"
-#include "cs_field_pointer.h"
-#include "cs_field_operator.h"
-#include "cs_mesh.h"
-#include "cs_mesh_quantities.h"
-#include "cs_halo.h"
-#include "cs_halo_perio.h"
-#include "cs_log.h"
-#include "cs_parameters.h"
-#include "cs_prototypes.h"
-#include "cs_rotation.h"
-#include "cs_time_moment.h"
-#include "cs_time_step.h"
-#include "cs_turbomachinery.h"
-#include "cs_selector.h"
-
-#include "cs_post.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -105,36 +77,33 @@ BEGIN_C_DECLS
  *
  * It has a very general purpose, although it is recommended to handle
  * mainly postprocessing or data-extraction type operations.
+ *
+ * \param[in, out]  domain   pointer to a cs_domain_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_user_extra_operations(void)
+cs_user_extra_operations(cs_domain_t     *domain)
 {
-  //!< [example_1]
-
-  cs_balance_by_zone("all[]",
-                     "temperature");
-
-  //!< [example_1]
-
-  //!< [example_2]
+  /*! [example_1] */
+  cs_balance_by_zone("all[]", "temperature");
+  /*! [example_1] */
 
+  /*! [example_2] */
   cs_balance_by_zone("box[-0.5, 1.3, 0.0, 1.0, 1.9, 1.0]",
                      "scalar1");
+  /*! [example_2] */
 
-  //!< [example_2]
-
-  //!< [example_3]
+  /*! [example_3] */
   cs_real_t normal[3] = {0., 0., 1.,};
 
   cs_surface_balance("selection_criterion", "scalar1", normal);
-  //!< [example_3]
+  /*! [example_3] */
 
   /* More advanced usage for pressure drop */
 
   {
-    /*< [example_4] */
+    /*! [example_4] */
     const char criteria[] = "zone_group";
 
     cs_lnum_t   n_selected_cells = 0;
@@ -142,7 +111,7 @@ cs_user_extra_operations(void)
 
     cs_real_t balance[CS_BALANCE_P_N_TERMS];
 
-    BFT_MALLOC(selected_cells, cs_glob_mesh->n_cells, cs_lnum_t);
+    BFT_MALLOC(selected_cells, domain->mesh->n_cells, cs_lnum_t);
 
     cs_selector_get_cell_list(criteria,
                               &n_selected_cells,
@@ -158,22 +127,21 @@ cs_user_extra_operations(void)
     cs_balance_term_t  mass_in_idx = CS_BALANCE_MASS_IN;
     cs_balance_term_t  mass_out_idx = CS_BALANCE_MASS_OUT;
 
-    bft_printf("inlet mass flow  (scalar 1): %g\n",
+    bft_printf("inlet mass flow  (scalar 1): %g\n"
                "outlet mass flow (scalar 1): %g\n",
                balance[mass_in_idx],
                balance[mass_out_idx]);
-    /*< [example_4] */
+    /*! [example_4] */
   }
-  //!< [example_5]
 
+  /*! [example_5] */
   cs_pressure_drop_by_zone("zone_group");
-
-  //!< [example_5]
+  /*! [example_5] */
 
   /* More advanced usage for pressure drop */
 
+  /*! [example_6] */
   {
-    /*< [example_6] */
     const char criteria[] = "zone_group";
 
     cs_lnum_t   n_selected_cells = 0;
@@ -181,7 +149,7 @@ cs_user_extra_operations(void)
 
     cs_real_t balance[CS_BALANCE_P_N_TERMS];
 
-    BFT_MALLOC(selected_cells, cs_glob_mesh->n_cells, cs_lnum_t);
+    BFT_MALLOC(selected_cells, domain->mesh->n_cells, cs_lnum_t);
 
     cs_selector_get_cell_list(criteria,
                               &n_selected_cells,
@@ -196,12 +164,12 @@ cs_user_extra_operations(void)
     cs_balance_p_term_t  rhou_in_idx = CS_BALANCE_P_RHOU_IN;
     cs_balance_p_term_t  rhou_out_idx = CS_BALANCE_P_RHOU_OUT;
 
-    bft_printf("inlet mass flow  (rho.u): %g\n",
+    bft_printf("inlet mass flow  (rho.u): %g\n"
                "outlet mass flow (rho.u): %g\n",
                balance[rhou_in_idx],
                balance[rhou_out_idx]);
-    /*< [example_6] */
   }
+  /*! [example_6] */
 }
 
 /*----------------------------------------------------------------------------*/
