@@ -9,7 +9,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -36,6 +36,7 @@
 
 #include <assert.h>
 #include <math.h>
+#include <stdio.h>
 
 #if defined(HAVE_MPI)
 #include <mpi.h>
@@ -48,36 +49,10 @@
 #include <ple_coupling.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-#include "bft_printf.h"
-
-#include "cs_base.h"
-#include "cs_field.h"
-#include "cs_field_pointer.h"
-#include "cs_field_operator.h"
-#include "cs_mesh.h"
-#include "cs_mesh_quantities.h"
-#include "cs_halo.h"
-#include "cs_halo_perio.h"
-#include "cs_log.h"
-#include "cs_parall.h"
-#include "cs_parameters.h"
-#include "cs_prototypes.h"
-#include "cs_time_step.h"
-#include "cs_turbomachinery.h"
-#include "cs_selector.h"
-
-#include "cs_post.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -90,6 +65,8 @@ BEGIN_C_DECLS
  * \brief This function is called at the end of each time step, and has a very
  * general purpose (i.e. anything that does not have another dedicated
  * user function).
+ *
+ * \param[in, out]  domain   pointer to a cs_domain_t structure
  */
 /*----------------------------------------------------------------------------*/
 
@@ -102,36 +79,45 @@ BEGIN_C_DECLS
  *----------------------------------------------------------------------------*/
 
 void
-cs_user_extra_operations(void)
+cs_user_extra_operations(cs_domain_t     *domain)
 {
 
   /* Local variables */
+
+  /*! [local_variables] */
+
   cs_lnum_t n_faces;
   cs_lnum_t *face_list;
 
-  int cell_id, cell_id1, cell_id2, face_id;
-  int nt_cur = cs_glob_time_step->nt_cur;
+  cs_lnum_t cell_id, cell_id1, cell_id2, face_id;
+  int nt_cur = domain->time_step->nt_cur;
 
-  const cs_mesh_t *m = cs_glob_mesh;
-  const cs_mesh_quantities_t *fvq = cs_glob_mesh_quantities;
+  const cs_mesh_t *m = domain->mesh;
+  const cs_mesh_quantities_t *mq = domain->mesh_quantities;
 
-  const int n_cells = m->n_cells;
-  const int n_cells_ext = m->n_cells_with_ghosts;
-  const int n_i_faces = m->n_i_faces;
-  const int n_b_faces = m->n_b_faces;
+  const cs_lnum_t n_cells = m->n_cells;
+  const cs_lnum_t n_cells_ext = m->n_cells_with_ghosts;
+  const cs_lnum_t n_i_faces = m->n_i_faces;
+  const cs_lnum_t n_b_faces = m->n_b_faces;
 
   const cs_lnum_2_t *i_face_cells = (const cs_lnum_2_t *)m->i_face_cells;
   const cs_lnum_t *b_face_cells = (const cs_lnum_t *)m->b_face_cells;
 
-  const cs_real_t *cell_vol = fvq->cell_vol;
-  const cs_real_3_t *diipb = (const cs_real_3_t *)fvq->diipb;
-  const cs_real_t *b_face_surf = (const cs_real_t *)fvq->b_face_surf;
+  const cs_real_t *cell_vol = mq->cell_vol;
+  const cs_real_3_t *diipb = (const cs_real_3_t *)mq->diipb;
+  const cs_real_t *b_face_surf = (const cs_real_t *)mq->b_face_surf;
+
+  /*! [local_variables] */
+
+  /*! [physical_fields] */
 
   /* Get physical fields */
   const cs_real_t *dt = CS_F_(dt)->val;
   const cs_real_t *rho = CS_F_(rho)->val;
   const cs_field_t *h = cs_field_by_name_try("enthalpy");
 
+  /*! [physical_fields] */
+
   /*-------------------------------------------------------------------------
    * This example computes energy balance relative to enthalpy
    * We assume that we want to compute balances (convective and diffusive)
@@ -167,6 +153,8 @@ cs_user_extra_operations(void)
     mass_o_balance: constribution from mass suctions
     tot_balance   : total balance */
 
+  /*! [init] */
+
   double vol_balance = 0.;
   double div_balance = 0.;
   double a_wall_balance = 0.;
@@ -228,11 +216,18 @@ cs_user_extra_operations(void)
     }
   }
 
+  /*! [init] */
+
   /* 2. Compute the balance at time step n
     ======================================
+  */
 
+  /*! [computation] */
+
+  /*
     --> Balance on interior volumes
-        --------------------------- */
+        ---------------------------
+  */
 
   for (cell_id = 0; cell_id < n_cells; cell_id++) {
     vol_balance += cell_vol[cell_id] * rho[cell_id]
@@ -287,7 +282,7 @@ cs_user_extra_operations(void)
 
   cs_selector_get_b_face_list("2 or 3 or 4 or 7", &n_faces, face_list);
 
-  for (int i = 0; i < n_faces; i++) {
+  for (cs_lnum_t i = 0; i < n_faces; i++) {
 
     face_id = face_list[i];
     cell_id = b_face_cells[face_id]; // associated boundary cell
@@ -308,7 +303,7 @@ cs_user_extra_operations(void)
   */
   cs_selector_get_b_face_list("6", &n_faces, face_list);
 
-  for (int i = 0; i < n_faces; i++) {
+  for (cs_lnum_t i = 0; i < n_faces; i++) {
 
     face_id = face_list[i];
     cell_id = b_face_cells[face_id]; // associated boundary cell
@@ -328,7 +323,7 @@ cs_user_extra_operations(void)
   */
   cs_selector_get_b_face_list("8 or 9", &n_faces, face_list);
 
-  for (int i = 0; i < n_faces; i++) {
+  for (cs_lnum_t i = 0; i < n_faces; i++) {
 
     face_id = face_list[i];
     cell_id = b_face_cells[face_id]; // associated boundary cell
@@ -348,7 +343,7 @@ cs_user_extra_operations(void)
   */
   cs_selector_get_b_face_list("1", &n_faces, face_list);
 
-  for (int i = 0; i < n_faces; i++) {
+  for (cs_lnum_t i = 0; i < n_faces; i++) {
 
     face_id = face_list[i];
     cell_id = b_face_cells[face_id]; // associated boundary cell
@@ -368,7 +363,7 @@ cs_user_extra_operations(void)
   */
   cs_selector_get_b_face_list("5", &n_faces, face_list);
 
-  for (int i = 0; i < n_faces; i++) {
+  for (cs_lnum_t i = 0; i < n_faces; i++) {
 
     face_id = face_list[i];
     cell_id = b_face_cells[face_id]; // associated boundary cell
@@ -408,9 +403,14 @@ cs_user_extra_operations(void)
               + sym_balance + in_balance + out_balance + mass_i_balance
               + mass_o_balance;
 
+  /*! [computation] */
+
+
   /* 3. Write the balance at time step n
     ==================================== */
 
+  /*! [writing] */
+
   bft_printf("\n   ** Enthalpy balance **\n"
              "      ----------------\n"
              "-----------"
@@ -426,6 +426,8 @@ cs_user_extra_operations(void)
     sym_balance, in_balance, out_balance,
     mass_i_balance, mass_o_balance, tot_balance);
 
+  /*! [writing] */
+
 }
 
 END_C_DECLS
