@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -43,45 +43,10 @@
 #include <assert.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_printf.h"
-#include "bft_error.h"
-#include "bft_mem.h"
-
-#include "fvm_periodicity.h"
-
-#include "cs_base.h"
-#include "cs_halo.h"
-#include "cs_interface.h"
-#include "cs_math.h"
-#include "cs_mesh.h"
-#include "cs_mesh_quantities.h"
-#include "cs_order.h"
-#include "cs_parall.h"
-#include "cs_prototypes.h"
-#include "cs_random.h"
-#include "cs_search.h"
-#include "cs_time_step.h"
-#include "cs_timer_stats.h"
-#include "cs_thermal_model.h"
-
-#include "cs_field.h"
-#include "cs_field_pointer.h"
-
-#include "cs_lagr.h"
-#include "cs_lagr_new.h"
-#include "cs_lagr_particle.h"
-#include "cs_lagr_stat.h"
-#include "cs_lagr_sde.h"
-#include "cs_lagr_geom.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_lagr_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -221,6 +186,7 @@ _inlet2(cs_lagr_particle_set_t  *p_set,
  */
 /*----------------------------------------------------------------------------*/
 
+/*! [lagr_ef] */
 void
 cs_user_lagr_ef(cs_real_t            dt_p,
                 const cs_real_t      taup[],
@@ -241,7 +207,9 @@ cs_user_lagr_ef(cs_real_t            dt_p,
     fextla[ip][1] = 0;
     fextla[ip][2] = 0;
   }
+
 }
+/*! [lagr_ef] */
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -258,11 +226,16 @@ cs_user_lagr_ef(cs_real_t            dt_p,
 void
 cs_user_lagr_extra_operations(const cs_real_t  dt[])
 {
+  /*! [lagr_init] */
+
   cs_lagr_particle_set_t  *p_set = cs_lagr_get_particle_set();
   const cs_lagr_attribute_map_t *p_am = p_set->p_am;
 
+  /*! [lagr_init] */
+
   /* Example: computation of the particle mass flow rate on 4 planes
      --------------------------------------------------------------- */
+  /*! [lagr_example] */
 
   {
     cs_real_t zz[4] = {0.1e0, 0.15e0, 0.20e0, 0.25e0};
@@ -284,23 +257,19 @@ cs_user_lagr_extra_operations(const cs_real_t  dt[])
 
         unsigned char *part = p_set->p_buffer + p_am->extents * npt;
 
-        cs_lnum_t iel = cs_lagr_particle_get_cell_id(part, p_am);
+        cs_lnum_t iel = cs_lagr_particle_get_lnum(part, p_am, CS_LAGR_CELL_ID);
 
-        if( iel >= 0 ) {
+        const cs_real_t *part_coords
+          = cs_lagr_particle_attr_const(part, p_am, CS_LAGR_COORDS);
+        const cs_real_t *prev_part_coords
+          = cs_lagr_particle_attr_n_const(part, p_am, 1, CS_LAGR_COORDS);
 
-          const cs_real_t *part_coords
-            = cs_lagr_particle_attr_const(part, p_am, CS_LAGR_COORDS);
-          const cs_real_t *prev_part_coords
-            = cs_lagr_particle_attr_n_const(part, p_am, 1, CS_LAGR_COORDS);
-
-          if (    part_coords[0] > zz[iplan]
-              && prev_part_coords[0] <= zz[iplan])
-            _m_flow[iplan] +=  cs_lagr_particle_get_real(part, p_am,
-                                                        CS_LAGR_STAT_WEIGHT)
-                             * cs_lagr_particle_get_real(part, p_am,
-                                                         CS_LAGR_MASS);
-
-        }
+        if (    part_coords[0] > zz[iplan]
+            && prev_part_coords[0] <= zz[iplan])
+          _m_flow[iplan] +=  cs_lagr_particle_get_real(part, p_am,
+                                                       CS_LAGR_STAT_WEIGHT)
+                           * cs_lagr_particle_get_real(part, p_am,
+                                                       CS_LAGR_MASS);
 
       }
 
@@ -308,11 +277,13 @@ cs_user_lagr_extra_operations(const cs_real_t  dt[])
 
     cs_real_t stat_age = cs_lagr_stat_get_age();
 
-    for (cs_lnum_t iplan = 0; iplan < 4; iplan++)
+    for (int iplan = 0; iplan < 4; iplan++)
       bft_printf(" Particle mass flow at Z(%d): %e14.5)",
                  iplan,
                  _m_flow[iplan]/stat_age);
   }
+
+  /*! [lagr_example] */
 }
 
 /*----------------------------------------------------------------------------*/
@@ -328,11 +299,13 @@ cs_user_lagr_extra_operations(const cs_real_t  dt[])
  */
 /*----------------------------------------------------------------------------*/
 
+/*! [lagr_imposed_motion] */
 void
 cs_user_lagr_imposed_motion(const cs_real_t  coords[3],
                             cs_real_t        dt,
                             cs_real_t        disp[3])
 {
+
   /* Angular velocity */
   cs_real_t omega = 1.0;
 
@@ -345,7 +318,9 @@ cs_user_lagr_imposed_motion(const cs_real_t  coords[3],
   disp[0] = 0.;
   disp[1] = rcost * (cos(omega*dt) - 1.0 ) - rsint * sin(omega*dt);
   disp[2] = rsint * (cos(omega*dt) - 1.0 ) + rcost * sin(omega*dt);
+
 }
+/*! [lagr_imposed_motion] */
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -358,7 +333,7 @@ cs_user_lagr_imposed_motion(const cs_real_t  coords[3],
  *
  * This function is called for each injection zone and set. Particles
  * with ids between \c pset->n_particles and \c n_elts are initialized
- * but may be modidied by this function.
+ * but may be modified by this function.
  *
  * \param[in,out]  particles         particle set
  * \param[in]      zis               zone injection set data
@@ -371,6 +346,7 @@ cs_user_lagr_imposed_motion(const cs_real_t  coords[3],
  */
 /*----------------------------------------------------------------------------*/
 
+/*! [lagr_inj] */
 void
 cs_user_lagr_in(cs_lagr_particle_set_t         *particles,
                 const cs_lagr_injection_set_t  *zis,
@@ -378,6 +354,7 @@ cs_user_lagr_in(cs_lagr_particle_set_t         *particles,
                 const cs_lnum_t                 particle_face_id[],
                 const cs_real_t                 visc_length[])
 {
+
   const int ntcabs = cs_glob_time_step->nt_cur;
 
   cs_lagr_zone_data_t  *lagr_bdy_conditions
@@ -409,6 +386,7 @@ cs_user_lagr_in(cs_lagr_particle_set_t         *particles,
     cs_lagr_particles_set_real(particles, p_id, CS_LAGR_STAT_WEIGHT, 0.01);
 
     /* User variables */
+
     for (int attr_id = CS_LAGR_USER;
          attr_id < CS_LAGR_USER + cs_glob_lagr_model->n_user_variables;
          attr_id++) {
@@ -425,6 +403,7 @@ cs_user_lagr_in(cs_lagr_particle_set_t         *particles,
      the particle injection. */
 
   /* if new particles have entered the domain  */
+
   for (cs_lnum_t ip = particle_range[0]; ip < particle_range[1]; ip++) {
     _inlet2(particles, ip);
   }
@@ -432,12 +411,11 @@ cs_user_lagr_in(cs_lagr_particle_set_t         *particles,
   /*
    * Trick to average the statistics at iteration nstist
    * starting from an unsteady two-coupling calculation
-   *                                                      */
-  if (cs_glob_time_step->nt_cur > cs_glob_lagr_stat_options->nstist) {
+   */
 
+  if (cs_glob_time_step->nt_cur > cs_glob_lagr_stat_options->nstist) {
     cs_glob_lagr_source_terms->nstits = cs_glob_lagr_stat_options->nstist;
     cs_glob_lagr_time_scheme->isttio = 1;
-
   }
 
   /* Simulation of the instantaneous turbulent fluid flow velocities seen
@@ -458,7 +436,7 @@ cs_user_lagr_in(cs_lagr_particle_set_t         *particles,
    *     velocities of the flow seen.
    *   - the reconstruction must be adapted to the case. */
 
-  if (false) {
+  {
     int time_id = 1;
     if (cs_glob_time_step->nt_cur == 1)
       time_id = 0;
@@ -467,6 +445,7 @@ cs_user_lagr_in(cs_lagr_particle_set_t         *particles,
                               visc_length);
   }
 }
+/*! [lagr_inj] */
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -476,42 +455,29 @@ cs_user_lagr_in(cs_lagr_particle_set_t         *particles,
  * This function is called in a loop on the particles, so be careful
  * to avoid too costly operations.
  *
- *                m   Cp
- *                 p    p
- *       Tau = ---------------
- *          c          2
- *                PI d    h
- *                    p    e
  *
- *      Tau  : Thermal relaxation time (value to be computed)
- *         c
+ *      \tau_c = \frac{m_p{C_p}_p}{PId_p^2h_e}
  *
- *      m    : Particle mass
- *       p
+ *      \tau_c  : Thermal relaxation time (value to be computed)
  *
- *      Cp   : Particle specific heat
- *        p
+ *      m_p    : Particle mass
  *
- *      d    : Particle diameter
- *       p
+ *      {C_p}_p   : Particle specific heat
  *
- *      h    : Coefficient of thermal exchange
- *       e
+ *      d_p    : Particle diameter
+ *
+ *      h_e    : Coefficient of thermal exchange
  *
  *  The coefficient of thermal exchange is calculated from a Nusselt number,
  *  itself evaluated by a correlation (Ranz-Marshall by default)
  *
- *             h  d
- *              e  p
- *      Nu = --------  = 2 + 0.55 Re **(0.5) Prt**(0.33)
- *            Lambda                p
+ *      \nu =  \frac{h_ed_p}{\lambda} = 2 + 0.55{\Re_e}_p^{0.5}P_{rt}^{0.33}
  *
- *      Lambda : Thermal conductivity of the carrier field
+ *      \lambda : Thermal conductivity of the carrier field
  *
- *      Re     : Particle Reynolds number
- *        p
+ *      {\Re_e}_p     : Particle Reynolds number
  *
- *      Prt    : Prandtl number
+ *      P_{rt}    : Prandtl number
  *
  * \param[in]   id_p   particle id
  * \param[in]   re_p   particle Reynolds number
@@ -527,6 +493,7 @@ cs_user_lagr_in(cs_lagr_particle_set_t         *particles,
  */
 /*----------------------------------------------------------------------------*/
 
+/*! [lagr_particle_relax_time] */
 void
 cs_user_lagr_rt(cs_lnum_t        id_p,
                 cs_real_t        re_p,
@@ -592,6 +559,7 @@ cs_user_lagr_rt(cs_lnum_t        id_p,
 
   taup[id_p] = rho_p / rho_f / fdr;
 }
+/*! [lagr_particle_relax_time] */
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -616,6 +584,7 @@ cs_user_lagr_rt(cs_lnum_t        id_p,
  */
 /*----------------------------------------------------------------------------*/
 
+/*! [lagr_thermal_relax_time] */
 void
 cs_user_lagr_rt_t(cs_lnum_t        id_p,
                   cs_real_t        re_p,
@@ -648,6 +617,7 @@ cs_user_lagr_rt_t(cs_lnum_t        id_p,
 
   tauc[id_p]= diam * diam * rho_p * cp_p  / ( fnus * 6.0 * rho_f * cp_f * k_f);
 }
+/*! [lagr_thermal_relax_time] */
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -681,6 +651,7 @@ cs_user_lagr_rt_t(cs_lnum_t        id_p,
  */
 /*----------------------------------------------------------------------------*/
 
+/*! [lagr_SDE] */
 void
 cs_user_lagr_sde(const cs_real_t  dt[],
                  cs_real_t        taup[],
@@ -710,30 +681,26 @@ cs_user_lagr_sde(const cs_real_t  dt[],
     for (cs_lnum_t npt = 0; npt < p_set->n_particles; npt++) {
 
       unsigned char *part = p_set->p_buffer + p_am->extents * npt;
-      cs_lnum_t iel = cs_lagr_particle_get_cell_id(part, p_am);
+      cs_lnum_t iel = cs_lagr_particle_get_lnum(part, p_am, CS_LAGR_CELL_ID);
 
       cs_real_t *usr_var
         = cs_lagr_particle_attr_n(part, p_am, 0, CS_LAGR_USER);
       cs_real_t *prev_usr_var
         = cs_lagr_particle_attr_n(part, p_am, 1, CS_LAGR_USER);
 
-      if (iel >= 0) {
+      /* Characteristic time tca of the differential equation,
+         This example must be adapted to the case */
+      tcarac[npt] = 1.0;
 
-        /* Characteristic time tca of the differential equation,
-           This example must be adapted to the case */
-        tcarac[npt] = 1.0;
+      /* Prediction at the first substep;
+         This example must be adapted to the case */
+      if (cs_glob_lagr_time_step->nor == 1)
+        pip[npt] = prev_usr_var[i];
 
-        /* Prediction at the first substep;
-           This example must be adapted to the case */
-        if (cs_glob_lagr_time_step->nor == 1)
-          pip[npt] = prev_usr_var[i];
-
-        /* Correction at the second substep;
-           This example must be adapted to the case */
-        else
-          pip[npt] = usr_var[i];
-
-      }
+      /* Correction at the second substep;
+         This example must be adapted to the case */
+      else
+        pip[npt] = usr_var[i];
 
     }
 
@@ -747,6 +714,7 @@ cs_user_lagr_sde(const cs_real_t  dt[],
   BFT_FREE(tcarac);
   BFT_FREE(pip);
 }
+/*! [lagr_SDE] */
 
 /*----------------------------------------------------------------------------*/
 
