@@ -1,10 +1,10 @@
 !-------------------------------------------------------------------------------
 
-!                      Code_Saturne version 4.0-alpha
-!                      --------------------------
+!VERS
+
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2021 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -130,7 +130,7 @@ enddo
 ! coupled.
 !===============================================================================
 
-if (icond.eq.1) then
+if (icondv.eq.0) then
 
   ! Choice the way to impose the wall temperature (tmet)
   ! at the solid/fluid interface:
@@ -191,7 +191,7 @@ endif
 !===============================================================================
 
 !-- pointer to the specific heat
-if (icp.gt.0) call field_get_val_s(iprpfl(icp), cpro_cp)
+if (icp.ge.0) call field_get_val_s(icp, cpro_cp)
 
 !-- pointer to the enthalpy value
 ivarh = isca(iscalt)
