@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -41,36 +41,10 @@
 #endif
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-#include "bft_printf.h"
-
-#include "cs_advection_field.h"
-#include "cs_base.h"
-#include "cs_domain_setup.h"
-#include "cs_equation.h"
-#include "cs_equation_param.h"
-#include "cs_field.h"
-#include "cs_math.h"
-#include "cs_mesh.h"
-#include "cs_mesh_location.h"
-#include "cs_mesh_quantities.h"
-#include "cs_multigrid.h"
-#include "cs_halo.h"
-#include "cs_param.h"
-#include "cs_property.h"
-#include "cs_sles.h"
-#include "cs_time_step.h"
-#include "cs_walldistance.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -78,11 +52,11 @@ BEGIN_C_DECLS
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \file cs_user_parameters-base.c
+ * \file cs_user_parameters-cdo-condif.c
  *
  * \brief User functions for input of calculation parameters.
  *
- * See \subpage parameters for examples.
+ * See \ref parameters for examples.
  */
 /*----------------------------------------------------------------------------*/
 
@@ -93,18 +67,19 @@ BEGIN_C_DECLS
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Give the explicit definition of the advection field.
- *         pt_ids is optional. If not NULL, it enables to access in coords
- *         at the right location and the same thing to fill retval if compact
- *         is set to false
- *         Rely on a generic function pointer for an analytic function
+ *         Generic function pointer for an evaluation relying on an analytic
+ *         function
+ *         pt_ids is optional. If not NULL, it enables to access to the coords
+ *         array with an indirection. The same indirection can be applied to
+ *         fill retval if dense_output is set to false.
  *
- * \param[in]      time       when ?
- * \param[in]      n_elts     number of elements to consider
- * \param[in]      pt_ids     list of elements ids (to access coords and fill)
- * \param[in]      coords     where ?
- * \param[in]      compact    true:no indirection, false:indirection for filling
- * \param[in]      input      NULL or pointer to a structure cast on-the-fly
- * \param[in, out] retval     result of the function
+ * \param[in]      time          when ?
+ * \param[in]      n_pts         number of elements to consider
+ * \param[in]      pt_ids        list of elements ids (in coords and retval)
+ * \param[in]      xyz           where ? Coordinates array
+ * \param[in]      dense_output  perform an indirection in res or not
+ * \param[in]      input         NULL or pointer to a structure cast on-the-fly
+ * \param[in, out] res           resulting value(s). Must be allocated.
  */
 /*----------------------------------------------------------------------------*/
 
@@ -113,7 +88,7 @@ _define_adv_field(cs_real_t           time,
                   cs_lnum_t           n_pts,
                   const cs_lnum_t    *pt_ids,
                   const cs_real_t    *xyz,
-                  bool                compact,
+                  bool                dense_output,
                   void               *input,
                   cs_real_t          *res)
 {
@@ -123,7 +98,7 @@ _define_adv_field(cs_real_t           time,
   for (cs_lnum_t p = 0; p < n_pts; p++) {
 
     const cs_lnum_t  id = (pt_ids == NULL) ? p : pt_ids[p];
-    const cs_lnum_t  ii = compact ? p : id;
+    const cs_lnum_t  ii = dense_output ? p : id;
     const cs_real_t  *pxyz = xyz + 3*id;
     cs_real_t  *pres = res + 3*ii;
 
@@ -137,18 +112,19 @@ _define_adv_field(cs_real_t           time,
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Give the explicit definition of the dirichlet boundary conditions
- *         pt_ids is optional. If not NULL, it enables to access in coords
- *         at the right location and the same thing to fill retval if compact
- *         is set to false
- *         Rely on a generic function pointer for an analytic function
+ *         Generic function pointer for an evaluation relying on an analytic
+ *         function
+ *         pt_ids is optional. If not NULL, it enables to access to the coords
+ *         array with an indirection. The same indirection can be applied to
+ *         fill retval if dense_output is set to false.
  *
- * \param[in]      time      when ?
- * \param[in]      n_elts    number of elements to consider
- * \param[in]      pt_ids    list of elements ids (to access coords and fill)
- * \param[in]      coords    where ?
- * \param[in]      compact   true:no indirection, false:indirection for filling
- * \param[in]      input     NULL or pointer to a structure cast on-the-fly
- * \param[in, out] res       result of the function
+ * \param[in]      time          when ?
+ * \param[in]      n_pts         number of elements to consider
+ * \param[in]      pt_ids        list of elements ids (in coords and retval)
+ * \param[in]      xyz           where ? Coordinates array
+ * \param[in]      dense_output  perform an indirection in res or not
+ * \param[in]      input         NULL or pointer to a structure cast on-the-fly
+ * \param[in, out] res           resulting value(s). Must be allocated.
  */
 /*----------------------------------------------------------------------------*/
 
@@ -157,25 +133,24 @@ _define_bcs(cs_real_t           time,
             cs_lnum_t           n_pts,
             const cs_lnum_t    *pt_ids,
             const cs_real_t    *xyz,
-            bool                compact,
+            bool                dense_output,
             void               *input,
             cs_real_t          *res)
 {
   CS_UNUSED(time);
   CS_UNUSED(input);
 
-  const double  pi = 4.0*atan(1.0);
+  const double  pi = cs_math_pi;
   for (cs_lnum_t p = 0; p < n_pts; p++) {
 
     const cs_lnum_t  id = (pt_ids == NULL) ? p : pt_ids[p];
-    const cs_lnum_t  ii = compact ? p : id;
+    const cs_lnum_t  ii = dense_output ? p : id;
     const cs_real_t  *_xyz = xyz + 3*id;
     const double  x = _xyz[0], y = _xyz[1], z = _xyz[2];
 
     res[ii] = 1 + sin(pi*x)*sin(pi*(y+0.5))*sin(pi*(z+0.25));
 
   }
-
 }
 
 /*============================================================================
@@ -210,22 +185,25 @@ cs_user_model(void)
 
   /*! [param_cdo_domain_boundary] */
   {
+    cs_domain_t  *domain = cs_glob_domain;
+    cs_boundary_t  *bdy = domain->boundaries;
+
     /* Choose a boundary by default */
-    cs_domain_boundary_set_default(CS_DOMAIN_BOUNDARY_WALL);
+    cs_boundary_set_default(bdy, CS_BOUNDARY_WALL);
 
     /* Add a new boundary
-     *  >> cs_domain_add_boundary(type_of_boundary, zone_name);
+     *  >> cs_domain_add_boundary(boundary, type_of_boundary, zone_name);
      *
      * zone_name is either a predefined one or user-defined one
      * type_of_boundary is one of the following keywords:
-     *   CS_DOMAIN_BOUNDARY_WALL,
-     *   CS_DOMAIN_BOUNDARY_INLET,
-     *   CS_DOMAIN_BOUNDARY_OUTLET,
-     *   CS_DOMAIN_BOUNDARY_SYMMETRY
+     *   CS_BOUNDARY_WALL,
+     *   CS_BOUNDARY_INLET,
+     *   CS_BOUNDARY_OUTLET,
+     *   CS_BOUNDARY_SYMMETRY
      */
 
-    cs_domain_boundary_add(CS_DOMAIN_BOUNDARY_INLET, "in");
-    cs_domain_boundary_add(CS_DOMAIN_BOUNDARY_OUTLET, "out");
+    cs_boundary_add(bdy, CS_BOUNDARY_INLET, "in");
+    cs_boundary_add(bdy, CS_BOUNDARY_OUTLET, "out");
   }
   /*! [param_cdo_domain_boundary] */
 
@@ -277,20 +255,18 @@ cs_user_model(void)
 
   /*! [param_cdo_wall_distance] */
   {
+    /*  Activate predefined module as the computation of the wall distance */
     cs_walldistance_activate();
   }
   /*! [param_cdo_wall_distance] */
 
   /*! [param_cdo_add_user_equation] */
   {
-    /* Add a new user equation:
-       Set the default boundary condition among:
-       CS_PARAM_BC_HMG_DIRICHLET or
-       CS_PARAM_BC_HMG_NEUMANN
-
-       By default, initial values are set to zero (or the value given by the
-       restart file in case of restart).
-    */
+    /* Add a new user equation.
+     *   The default boundary condition has to be chosen among:
+     *    CS_PARAM_BC_HMG_DIRICHLET
+     *    CS_PARAM_BC_HMG_NEUMANN
+     */
 
     cs_equation_add_user("AdvDiff.Upw", // equation name
                          "Pot.Upw",     // associated variable field name
@@ -339,24 +315,29 @@ cs_user_model(void)
   }
   /*! [param_cdo_add_user_adv_field] */
 
-  /*! [param_cdo_add_user_adv_field_opt] */
+  /*! [param_cdo_add_adv_field] */
+  {
+    /* Add a user-defined advection field named "adv_field"  */
+    cs_advection_field_status_t  adv_status =
+      CS_ADVECTION_FIELD_USER                 | /* = user-defined */
+      CS_ADVECTION_FIELD_TYPE_VELOCITY_VECTOR | /* = define by a vector field */
+      CS_ADVECTION_FIELD_DEFINE_AT_VERTICES   | /* = add a field at vertices */
+      CS_ADVECTION_FIELD_DEFINE_AT_BOUNDARY_FACES;  /* = add boundary fluxes */
+
+    cs_adv_field_t  *adv = cs_advection_field_add("adv_field", adv_status);
+  }
+  /*! [param_cdo_add_adv_field] */
+
+  /*! [param_cdo_add_user_adv_field_post] */
   {
     /* Retrieve an advection field named "adv_field"  */
     cs_adv_field_t  *adv = cs_advection_field_by_name("adv_field");
 
     /* Compute the Courant number (if unsteady simulation) */
-    cs_advection_field_set_option(adv, CS_ADVKEY_POST_COURANT);
+    cs_advection_field_set_postprocess(adv, CS_ADVECTION_FIELD_POST_COURANT);
 
-    /* Set other advanced options: for instance, define an interpolation of the
-       advection field at vertices */
-    cs_advection_field_set_option(adv, CS_ADVKEY_DEFINE_AT_VERTICES);
-
-    /* Both options in one call */
-    cs_advection_field_set_option(adv,
-                                  CS_ADVKEY_POST_COURANT |
-                                  CS_ADVKEY_DEFINE_AT_VERTICES);
   }
-  /*! [param_cdo_add_user_adv_field_opt] */
+  /*! [param_cdo_add_user_adv_field_post] */
 
 }
 
@@ -367,12 +348,16 @@ cs_user_model(void)
  * At the calling point of this function, most model-related most variables
  * and other fields have been defined, so specific settings related to those
  * fields may be set here.
+ *
+ * \param[in, out]   domain    pointer to a cs_domain_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_user_parameters(void)
+cs_user_parameters(cs_domain_t   *domain)
 {
+  CS_UNUSED(domain);
+
   /* ─────────────────────────────────────────────────────────────────────────
    *  Modify the setting of an equation using a generic process
    *  cf. the DOXYGEN documentation (website or in your installation path)
@@ -406,7 +391,7 @@ cs_user_parameters(void)
 #endif
     cs_equation_set_param(eqp, CS_EQKEY_ITSOL_MAX_ITER, "2500");
     cs_equation_set_param(eqp, CS_EQKEY_ITSOL_EPS, "1e-12");
-    cs_equation_set_param(eqp, CS_EQKEY_ITSOL_RESNORM, "false");
+    cs_equation_set_param(eqp, CS_EQKEY_ITSOL_RESNORM_TYPE, "false");
 
   }
   /*! [param_cdo_numerics] */
@@ -433,9 +418,9 @@ cs_user_linear_solvers(void)
 
     /* In case of a in-house K-cylcle multigrid as a preconditioner of a
        linear iterative solver */
-    if (eqp->itsol_info.precond == CS_PARAM_PRECOND_AMG) {
+    if (eqp->sles_param->precond == CS_PARAM_PRECOND_AMG) {
       /* If multigrid is the chosen preconditioner */
-      if (eqp->itsol_info.amg_type == CS_PARAM_AMG_HOUSE_K) {
+      if (eqp->sles_param->amg_type == CS_PARAM_AMG_HOUSE_K) {
         /* If this is a K-cycle multigrid */
 
         /* Retrieve the different context structures to apply additional
@@ -472,16 +457,16 @@ cs_user_linear_solvers(void)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  - Specify the elements such as properties, advection fields,
- *           user-defined equations and modules which have been previously
- *           added.
+ * \brief  Specify the elements such as properties, advection fields,
+ *         user-defined equations and modules which have been previously
+ *         added.
  *
  * \param[in, out]   domain    pointer to a cs_domain_t structure
 */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_user_cdo_finalize_setup(cs_domain_t   *domain)
+cs_user_finalize_setup(cs_domain_t   *domain)
 {
   CS_UNUSED(domain);
 
@@ -518,11 +503,8 @@ cs_user_cdo_finalize_setup(cs_domain_t   *domain)
 
     cs_advection_field_def_by_analytic(adv, _define_adv_field, NULL);
 
-    /* Enable also the defintion of the advection field at mesh vertices */
-    cs_advection_field_set_option(adv, CS_ADVKEY_DEFINE_AT_VERTICES);
-
     /* Activate the post-processing of the related Courant number */
-    cs_advection_field_set_option(adv, CS_ADVKEY_POST_COURANT);
+    cs_advection_field_set_postprocess(adv, CS_ADVECTION_FIELD_POST_COURANT);
   }
   /*! [param_cdo_setup_advfield] */
 
@@ -539,7 +521,7 @@ cs_user_cdo_finalize_setup(cs_domain_t   *domain)
      -> eq is the structure related to the equation to set
      -> type of boundary condition:
         CS_PARAM_BC_DIRICHLET, CS_PARAM_BC_HMG_DIRICHLET,
-        CS_PARAM_BC_NEUMANN, CS_PARAM_BC_HMG_NEUMANN, CS_PARAM_BC_ROBIN
+        CS_PARAM_BC_NEUMANN_FULL, CS_PARAM_BC_HMG_NEUMANN, CS_PARAM_BC_ROBIN
 
      >> cs_equation_add_bc_by_value(eqp,
                                     bc_type,
@@ -589,21 +571,27 @@ cs_user_cdo_finalize_setup(cs_domain_t   *domain)
 
   /*! [param_cdo_copy_settings] */
   {
-    /* Copy the settings for AdvDiff.Upw */
+    /* Copy the settings from AdvDiff.Upw to AdvDiff.SG */
     cs_equation_param_t  *eqp_ref = cs_equation_param_by_name("AdvDiff.Upw");
-    cs_equation_t *eq = cs_equation_by_name("AdvDiff.SG");
+
+    /* Another example to retrieve the cs_equation_param structure */
+    cs_equation_t  *eq = cs_equation_by_name("AdvDiff.SG");
     cs_equation_param_t  *eqp = cs_equation_get_param(eq);
 
-    /* Copy the settings */
-    cs_equation_param_update_from(eqp_ref, eqp);
+    /* Copy the settings to start from a common state as in the reference
+     * equation (in this case, do not copy the associated field id since these
+     * are two different equations with their own variable field) */
+    bool  copy_field_id = false;
+    cs_equation_copy_param_from(eqp_ref, eqp, copy_field_id);
 
     /* Keep all the settings from "AdvDiff.Upw and then only change the
        advection scheme for the second equation */
     cs_equation_set_param(eqp, CS_EQKEY_ADV_SCHEME, "sg");
 
     /* Call this function to be sure that the linear solver is set to what
-       one wants */
-    cs_equation_param_set_sles(eqp, cs_equation_get_field_id(eq));
+       one wants (if there is no modification of the SLES parameters, this
+       step can be skipped) */
+    cs_equation_param_set_sles(eqp);
 
   }
   /*! [param_cdo_copy_settings] */
