@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -47,33 +47,10 @@
 #include <ple_coupling.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-#include "bft_printf.h"
-
-#include "cs_advection_field.h"
-#include "cs_base.h"
-#include "cs_domain_setup.h"
-#include "cs_field.h"
-#include "cs_gwf.h"
-#include "cs_math.h"
-#include "cs_mesh.h"
-#include "cs_mesh_location.h"
-#include "cs_mesh_quantities.h"
-#include "cs_param.h"
-#include "cs_physical_model.h"
-#include "cs_property.h"
-#include "cs_prototypes.h"
-#include "cs_time_step.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -86,7 +63,7 @@ BEGIN_C_DECLS
  * \brief User functions for setting a calcultion using the groundwater flow
  *        module with CDO schemes
  *
- * See \subpage parameters for examples.
+ * See \ref parameters for examples.
  */
 /*----------------------------------------------------------------------------*/
 
@@ -100,27 +77,28 @@ static const double k1 = 1e5, k2 = 1;
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Get the explicit definition of the problem for the Richards eq.
- *         pt_ids is optional. If not NULL, it enables to access in coords
- *         at the right location and the same thing to fill retval if compact
- *         is set to false
- *         Rely on a generic function pointer for an analytic function
+ *         Generic function pointer for an evaluation relying on an analytic
+ *         function
+ *         pt_ids is optional. If not NULL, it enables to access to the coords
+ *         array with an indirection. The same indirection can be applied to
+ *         fill retval if dense_output is set to false.
  *
- * \param[in]      time      when ?
- * \param[in]      n_elts    number of elements to consider
- * \param[in]      pt_ids    list of elements ids (to access coords and fill)
- * \param[in]      coords    where ?
- * \param[in]      compact   true:no indirection, false:indirection for filling
- * \param[in]      input     NULL or pointer to a structure cast on-the-fly
- * \param[in, out] retval    result of the function
+ * \param[in]      time          when ?
+ * \param[in]      n_elts        number of elements to consider
+ * \param[in]      pt_ids        list of elements ids (in coords and retval)
+ * \param[in]      xyz           where ? Coordinates array
+ * \param[in]      dense_output  perform an indirection in retval or not
+ * \param[in]      input         NULL or pointer to a structure cast on-the-fly
+ * \param[in, out] retval        resulting value(s). Must be allocated.
  */
 /*----------------------------------------------------------------------------*/
 
 static inline void
 get_tracer_sol(cs_real_t          time,
-               cs_lnum_t          n_points,
+               cs_lnum_t          n_elts,
                const cs_lnum_t   *pt_ids,
                const cs_real_t   *xyz,
-               bool               compact,
+               bool               dense_output,
                void              *input,
                cs_real_t         *retval)
 {
@@ -130,9 +108,9 @@ get_tracer_sol(cs_real_t          time,
   const double  magnitude = 2*k1/(k1 + k2);
   const double  x_front = magnitude * time;
 
-  if (pt_ids != NULL && !compact) {
+  if (pt_ids != NULL && !dense_output) {
 
-    for (cs_lnum_t  i = 0; i < n_points; i++) {
+    for (cs_lnum_t  i = 0; i < n_elts; i++) {
 
       const cs_lnum_t  id = pt_ids[i];
       const double  x = xyz[3*id];
@@ -143,9 +121,9 @@ get_tracer_sol(cs_real_t          time,
     }
 
   }
-  else if (pt_ids != NULL && compact) {
+  else if (pt_ids != NULL && dense_output) {
 
-    for (cs_lnum_t  i = 0; i < n_points; i++) {
+    for (cs_lnum_t  i = 0; i < n_elts; i++) {
       const double  x = xyz[3*pt_ids[i]];
       if (x <= x_front)
         retval[i] = 1;
@@ -156,7 +134,7 @@ get_tracer_sol(cs_real_t          time,
   }
   else {
 
-    for (cs_lnum_t  i = 0; i < n_points; i++) {
+    for (cs_lnum_t  i = 0; i < n_elts; i++) {
       const double  x = xyz[3*i];
       if (x <= x_front)
         retval[i] = 1;
@@ -184,19 +162,22 @@ get_tracer_sol(cs_real_t          time,
 void
 cs_user_model(void)
 {
+  cs_domain_t  *domain = cs_glob_domain;
+
   /* ======================
      Boundary of the domain
      ====================== */
 
+  cs_boundary_t  *bdy = domain->boundaries;
+
   /* Choose a boundary by default */
-  cs_domain_boundary_set_default(CS_DOMAIN_BOUNDARY_SYMMETRY);
+  cs_boundary_set_default(bdy, CS_BOUNDARY_SYMMETRY);
 
   /* Add new boundaries */
-  cs_domain_boundary_add(CS_DOMAIN_BOUNDARY_INLET, "left");
-  cs_domain_boundary_add(CS_DOMAIN_BOUNDARY_OUTLET, "right");
+  cs_boundary_add(bdy, CS_BOUNDARY_INLET, "left");
+  cs_boundary_add(bdy, CS_BOUNDARY_OUTLET, "right");
 
   /* Activate CDO/HHO module so that main additional structure are built */
-  cs_domain_t  *domain = cs_glob_domain;
 
   cs_domain_set_cdo_mode(domain, CS_DOMAIN_CDO_MODE_ONLY);
 
@@ -237,8 +218,7 @@ cs_user_model(void)
      ================================
 
      For the groundwater flow module:
-     >> cs_gwf_activate(permeability_type,
-                        richards_flag);
+     >> cs_gwf_activate(permeability_type, option_flag);
 
      * permeability_type is one of the following keywords:
        CS_PROPERTY_ISO, CS_PROPERTY_ORTHO or CS_PROPERTY_ANISO
@@ -259,12 +239,24 @@ cs_user_model(void)
 
   /*! [param_cdo_activate_gwf] */
   {
+    cs_flag_t  option_flag = 0;
 
-  cs_gwf_activate(CS_PROPERTY_ISO, 0); // no flag to set
-
+    cs_gwf_activate(CS_PROPERTY_ISO, option_flag);
   }
   /*! [param_cdo_activate_gwf] */
 
+  /*! [param_cdo_activate_gwf_b] */
+  {
+    /* Take into account the gravity effect */
+    cs_flag_t  option_flag = CS_GWF_GRAVITATION;
+
+    /* In this case, the gravity vector has to be defined (either using the GUI
+       or in cs_user_parameters() function */
+
+    cs_gwf_activate(CS_PROPERTY_ISO, option_flag);
+  }
+  /*! [param_cdo_activate_gwf_b] */
+
   /* =========
      Add soils (must be done before adding tracers)
      ========= */
@@ -287,12 +279,13 @@ cs_user_model(void)
      For standard tracer:
        cs_gwf_add_tracer(eqname, varname);
      For user-defined tracer
-       cs_gwf_add_tracer_user(eqname, varname, setup_func);
+       cs_gwf_add_user_tracer(eqname, varname, setup_func);
   */
 
   /*! [param_cdo_gwf_add_tracer] */
   {
-    cs_gwf_add_tracer("Tracer_01","C1");
+    cs_gwf_tracer_model_t  model = 0; /* default model */
+    cs_gwf_add_tracer(model, "Tracer_01","C1");
   }
   /*! [param_cdo_gwf_add_tracer] */
 
@@ -300,7 +293,7 @@ cs_user_model(void)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  - Specify the elements such as properties, advection fields,
+ * \brief  Specify the elements such as properties, advection fields,
  *           user-defined equations and modules which have been previously
  *           added.
  *
@@ -309,7 +302,7 @@ cs_user_model(void)
 /*----------------------------------------------------------------------------*/
 
 void
-cs_user_cdo_finalize_setup(cs_domain_t   *domain)
+cs_user_finalize_setup(cs_domain_t   *domain)
 {
   CS_UNUSED(domain);
 
