@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -55,55 +55,15 @@
 #include <ple_coupling.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-#include "bft_printf.h"
-
-#include "cs_base.h"
-#include "cs_ctwr.h"
-#include "cs_equation_param.h"
-#include "cs_fan.h"
-#include "cs_field.h"
-#include "cs_field_pointer.h"
-#include "cs_field_operator.h"
-#include "cs_gui_util.h"
-#include "cs_grid.h"
-#include "cs_internal_coupling.h"
-#include "cs_math.h"
-#include "cs_mesh.h"
-#include "cs_mesh_location.h"
-#include "cs_mesh_quantities.h"
-#include "cs_halo.h"
-#include "cs_halo_perio.h"
-#include "cs_log.h"
-#include "cs_multigrid.h"
-#include "cs_parameters.h"
-#include "cs_physical_constants.h"
-#include "cs_post.h"
-#include "cs_post_util.h"
-#include "cs_prototypes.h"
-#include "cs_rotation.h"
-#include "cs_sles.h"
-#include "cs_sles_it.h"
-#include "cs_time_moment.h"
-#include "cs_time_step.h"
-#include "cs_turbomachinery.h"
-#include "cs_selector.h"
-#include "cs_rad_transfer.h"
+#include "cs_headers.h"
 
 #if defined(HAVE_PETSC)
 #include "cs_sles_petsc.h"
 #endif
 
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
-
 /*----------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
@@ -114,7 +74,7 @@ BEGIN_C_DECLS
  *
  * \brief Linear solvers examples.
  *
- * See \subpage parameters for examples.
+ * See \ref parameters for examples.
  *
  */
 /*----------------------------------------------------------------------------*/
@@ -144,9 +104,10 @@ BEGIN_C_DECLS
 
 /*! [sles_petsc_hook_1] */
 static void
-_petsc_p_setup_hook(const void  *context,
-                    KSP          ksp)
+_petsc_p_setup_hook(void   *context,
+                    KSP     ksp)
 {
+  CS_UNUSED(context);
   PC pc;
 
   KSPSetType(ksp, KSPCG);   /* Preconditioned Conjugate Gradient */
@@ -163,9 +124,10 @@ _petsc_p_setup_hook(const void  *context,
 
 /*! [sles_petsc_hook_gamg] */
 static void
-_petsc_p_setup_hook_gamg(const void  *context,
+_petsc_p_setup_hook_gamg(void        *context,
                          KSP          ksp)
 {
+  CS_UNUSED(context);
   PC pc;
 
   KSPSetType(ksp, KSPCG);   /* Preconditioned Conjugate Gradient */
@@ -181,9 +143,10 @@ _petsc_p_setup_hook_gamg(const void  *context,
 
 /*! [sles_petsc_hook_bamg] */
 static void
-_petsc_p_setup_hook_bamg(const void  *context,
+_petsc_p_setup_hook_bamg(void        *context,
                          KSP          ksp)
 {
+  CS_UNUSED(context);
   PC pc;
 
   KSPSetType(ksp, KSPCG);   /* Preconditioned Conjugate Gradient */
@@ -212,14 +175,15 @@ _petsc_p_setup_hook_bamg(const void  *context,
 
 /*! [sles_petsc_hook_view] */
 static void
-_petsc_p_setup_hook_view(const void  *context,
+_petsc_p_setup_hook_view(void        *context,
                          KSP          ksp)
 {
+  CS_UNUSED(context);
   const char *p = getenv("CS_USER_PETSC_MAT_VIEW");
 
   if (p != NULL) {
 
-    /* Get system and preconditioner matrixes */
+    /* Get system and preconditioner matrices */
 
     Mat a, pa;
     KSPGetOperators(ksp, &a, &pa);
@@ -274,14 +238,21 @@ void
 cs_user_sles_petsc_hook(void               *context,
                         KSP                 ksp)
 {
-  /*! [sles_petsc_cdo_hook] */
-  cs_equation_param_t  *eqp = (cs_equation_param_t *)context;
+  CS_UNUSED(ksp);
 
-  if (cs_equation_param_has_name(eqp, "Name_Of_The_Equation")) {
+  /*! [sles_petsc_cdo_hook] */
+  cs_param_sles_t  *slesp = (cs_param_sles_t *)context;
+
+  if (slesp == NULL)
+    return;
+
+  /* Usually the name of the equation or the field id of the associated
+     variable */
+  if (strcmp(slesp->name, "Name_Of_The_System") == 0) {
 
     /* Assume a PETSc version greater or equal to 3.7.0 */
-    if (eqp->itsol_info.precond == CS_PARAM_PRECOND_AMG) {
-      if (eqp->itsol_info.amg_type == CS_PARAM_AMG_BOOMER) {
+    if (slesp->precond == CS_PARAM_PRECOND_AMG) {
+      if (slesp->amg_type == CS_PARAM_AMG_HYPRE_BOOMER) {
 
         PetscOptionsSetValue(NULL,
                              "-pc_hypre_boomeramg_strong_threshold", "0.7");
@@ -316,7 +287,7 @@ cs_user_linear_solvers(void)
 {
   /* Available native iterative linear solvers are:
    *
-   *  CS_SLES_PCG                 (preconditionned conjugate gradient)
+   *  CS_SLES_PCG                 (preconditioned conjugate gradient)
    *  CS_SLES_JACOBI              (Jacobi)
    *  CS_SLES_BICGSTAB            (Bi-conjugate gradient stabilized)
    *  CS_SLES_BICGSTAB2           (BiCGStab2)
@@ -367,7 +338,7 @@ cs_user_linear_solvers(void)
     cs_sles_t *sles_p = cs_sles_find_or_add(CS_F_(p)->id, NULL);
     cs_sles_set_post_output(sles_p, CS_POST_WRITER_DEFAULT);
 
-    cs_sles_t *sles_u = cs_sles_find_or_add(CS_F_(u)->id, NULL);
+    cs_sles_t *sles_u = cs_sles_find_or_add(CS_F_(vel)->id, NULL);
     cs_sles_set_post_output(sles_u, CS_POST_WRITER_DEFAULT);
   }
   /*! [sles_viz_1] */
@@ -408,15 +379,20 @@ cs_user_linear_solvers(void)
   }
   /*! [sles_mgp_1] */
 
-
   /* Set parallel grid merging options for all multigrid solvers */
   /*-------------------------------------------------------------*/
 
   /*! [sles_mg_parall] */
-  cs_grid_set_merge_options(4,   /* # of ranks merged at a time */
-                            300, /* mean # of cells under which we merge */
-                            500, /* global # of cells under which we merge */
-                            1);  /* # of ranks under which we do not merge */
+  {
+    cs_multigrid_t *mg = cs_multigrid_define(CS_F_(p)->id,
+                                             NULL,
+                                             CS_MULTIGRID_V_CYCLE);
+
+    cs_multigrid_set_merge_options(mg,
+                                   4,    /* # of ranks merged at a time */
+                                   300,  /* mean # of cells under which we merge */
+                                   500); /* global # of cells under which we merge */
+  }
   /*! [sles_mg_parall] */
 
   /* Example: conjugate gradient preconditioned by multigrid for pressure */
@@ -454,7 +430,6 @@ cs_user_linear_solvers(void)
   }
   /*! [sles_mgp_2] */
 
-
   /* Set a non-default linear solver for DOM radiation. */
   /*----------------------------------------------------*/
 
@@ -480,7 +455,6 @@ cs_user_linear_solvers(void)
   /* Example: activate convergence plot for pressure */
   /*-------------------------------------------------*/
 
-
   /*! [sles_plot_1] */
   {
     const cs_field_t *f = CS_F_(p);
@@ -525,7 +499,6 @@ cs_user_linear_solvers(void)
   }
   /*! [sles_petsc_1] */
 
-
   /* Setting pressure solver with PETSc */
   /*------------------------------------*/
 
@@ -593,7 +566,6 @@ cs_user_linear_solvers(void)
   }
   /*! [sles_petsc_gamg_2] */
 
-
   /* Setting global options for PETSc with HYPRE BoomerAMG preconditioner */
   /*----------------------------------------------------------------------*/
 
@@ -649,6 +621,18 @@ cs_user_linear_solvers(void)
 
 #endif /* defined(HAVE_PETSC) */
 
+  /* Setting pressure solver with AMGX */
+  /*-----------------------------------*/
+
+#if defined(HAVE_AMGX)
+  /*! [sles_amgx] */
+  {
+    cs_sles_amgx_t *amgx_p = cs_sles_amgx_define(CS_F_(p)->id, NULL);
+
+    cs_sles_amgx_set_config_file(amgx_p, "PCG_CLASSICAL_V_JACOBI.json");
+  }
+  /*! [sles_amgx] */
+#endif /* defined(HAVE_AMGX) */
 }
 
 /*----------------------------------------------------------------------------*/
