@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -40,27 +40,10 @@
 #include <string.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_error.h"
-#include "bft_mem.h"
-#include "bft_printf.h"
-
-#include "cs_base.h"
-#include "cs_file.h"
-#include "cs_grid.h"
-#include "cs_matrix.h"
-#include "cs_matrix_default.h"
-#include "cs_parall.h"
-#include "cs_partition.h"
-#include "cs_renumber.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -72,7 +55,7 @@ BEGIN_C_DECLS
  *
  * \brief Matrix tuning example.
  *
- * See \subpage cs_user_performance_tuning for examples.
+ * See \ref cs_user_performance_tuning for examples.
  */
 /*----------------------------------------------------------------------------*/
 
@@ -95,29 +78,12 @@ cs_user_matrix_tuning(void)
 
   /* Set tuning runs (defaults) */
 
-  cs_matrix_set_tuning_runs(10,   /* n_min_products */
+  cs_matrix_set_tuning_runs(20,   /* n_min_products */
                             0.5); /* t_measure */
 
-  /* Activate tuning for selected matrix fill types. */
+  /* Force default for selected types */
 
-  cs_matrix_set_tuning(CS_MATRIX_SCALAR, 1);
-
-  cs_matrix_set_tuning(CS_MATRIX_SCALAR_SYM, 1);
-
-  /* Force variant for selected types */
-
-  cs_matrix_variant_t *mv
-    = cs_matrix_variant_create(CS_MATRIX_MSR,
-                               cs_glob_mesh->i_face_numbering);
-  cs_matrix_variant_set_func(mv,
-                             cs_glob_mesh->i_face_numbering,
-                             CS_MATRIX_BLOCK_D,
-                             2,
-                             "default");
-
-  cs_matrix_set_variant(CS_MATRIX_BLOCK_D, mv);
-
-  cs_matrix_variant_destroy(&mv);
+  cs_matrix_default_set_type(CS_MATRIX_BLOCK_D, CS_MATRIX_MSR);
 
   /* Also allow tuning for multigrid for all expected levels
    * (we rarely have more than 10 or 11 levels except for huge meshes). */
