@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -40,27 +40,10 @@
 #include <string.h>
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_error.h"
-#include "bft_mem.h"
-#include "bft_printf.h"
-
-#include "cs_base.h"
-#include "cs_file.h"
-#include "cs_grid.h"
-#include "cs_matrix.h"
-#include "cs_matrix_default.h"
-#include "cs_parall.h"
-#include "cs_partition.h"
-#include "cs_renumber.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -72,7 +55,7 @@ BEGIN_C_DECLS
  *
  * \brief Mesh numbering example.
  *
- * See \subpage cs_user_performance_tuning for examples.
+ * See \ref cs_user_performance_tuning for examples.
  */
 /*----------------------------------------------------------------------------*/
 
@@ -110,7 +93,8 @@ cs_user_numbering(void)
      CS_RENUMBER_CELLS_NONE,          /* cells_pre_numbering */
      CS_RENUMBER_CELLS_NONE,          /* cells_numbering */
      CS_RENUMBER_I_FACES_MULTIPASS,   /* interior faces numbering */
-     CS_RENUMBER_B_FACES_THREAD);     /* boundary faces numbering */
+     CS_RENUMBER_B_FACES_THREAD,      /* boundary faces numbering */
+     CS_RENUMBER_VERTICES_NONE);      /* vertices numbering */
 
   /*! [performance_tuning_numbering] */
 }
