@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -36,34 +36,10 @@
 #include "string.h"
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-
-#include "cs_base.h"
-#include "cs_field.h"
-#include "cs_geom.h"
-#include "cs_interpolate.h"
-#include "cs_mesh.h"
-#include "cs_selector.h"
-#include "cs_parall.h"
-#include "cs_post.h"
-#include "cs_post_util.h"
-#include "cs_probe.h"
-#include "cs_time_plot.h"
-
-#include "cs_field_pointer.h"
-#include "cs_parameters.h"
-#include "cs_physical_constants.h"
-#include "cs_turbulence_model.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -273,13 +249,26 @@ cs_user_postprocess_values(const char            *mesh_name,
 
     const cs_real_3_t *cell_cen = (const cs_real_3_t *)mq->cell_cen;
 
-    const cs_real_3_t *vel = (const cs_real_3_t *)CS_F_(u)->val;
+    const cs_real_3_t *vel = (const cs_real_3_t *)CS_F_(vel)->val;
     const cs_real_t uref = cs_glob_turb_ref_values->uref;
     const cs_real_t uref2 = uref*uref;
 
     const char *name = cs_probe_set_get_name(probes);
 
-    const cs_time_step_t *ts_post = (ts->nt_cur == ts->nt_max) ? NULL : ts;
+    bool is_profile = false;
+    cs_probe_set_get_post_info(probes,
+                               NULL,
+                               NULL,
+                               &is_profile,
+                               NULL,
+                               NULL,
+                               NULL,
+                               NULL,
+                               NULL);
+
+    const cs_time_step_t *ts_post = ts;
+    if (is_profile && ts->nt_cur == ts->nt_max)
+      ts = NULL;
 
     /* Common variables */
 
@@ -311,8 +300,10 @@ cs_user_postprocess_values(const char            *mesh_name,
 
       /* Reynolds stresses */
 
-      const cs_turb_model_t *turb_mdl = cs_glob_turb_model;
+      const cs_turb_model_t  *turb_mdl = cs_get_glob_turb_model();
+      assert(turb_mdl != NULL);
       const cs_turb_rans_model_t *turb_rans_mdl = cs_glob_turb_rans_model;
+      assert(turb_rans_mdl != NULL);
 
       cs_real_6_t *rij = NULL;
       BFT_MALLOC(rij, n_cells, cs_real_6_t);
@@ -321,7 +312,12 @@ cs_user_postprocess_values(const char            *mesh_name,
           || turb_mdl->itytur == 5
           || turb_mdl->itytur == 6) {
 
-        cs_post_evm_reynolds_stresses(n_cells, cell_list, rij);
+        cs_field_interpolate_t interpolation_type = CS_FIELD_INTERPOLATE_MEAN;
+        cs_post_evm_reynolds_stresses(interpolation_type,
+                                      n_cells,
+                                      cell_list,
+                                      NULL, /* coords */
+                                      rij);
 
       } else if (turb_mdl->itytur == 3 && turb_rans_mdl->irijco == 0) {
 
@@ -340,7 +336,7 @@ cs_user_postprocess_values(const char            *mesh_name,
         cs_real_6_t *cvar_rij = (cs_real_6_t *)CS_F_(rij)->val;
         for (cs_lnum_t i = 0; i < n_cells; i++) {
           cs_lnum_t c_id = cell_list[i];
-          for (cs_lnum_t j = 0; i < 6; i++)
+          for (cs_lnum_t j = 0; j < 6; j++)
             rij[i][j] = cvar_rij[c_id][j];
         }
 
