@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -36,34 +36,10 @@
 #include "string.h"
 
 /*----------------------------------------------------------------------------
- *  Local headers
+ * Local headers
  *----------------------------------------------------------------------------*/
 
-#include "bft_mem.h"
-#include "bft_error.h"
-
-#include "cs_base.h"
-#include "cs_field.h"
-#include "cs_geom.h"
-#include "cs_interpolate.h"
-#include "cs_mesh.h"
-#include "cs_selector.h"
-#include "cs_parall.h"
-#include "cs_post.h"
-#include "cs_post_util.h"
-#include "cs_probe.h"
-#include "cs_time_plot.h"
-
-#include "cs_field_pointer.h"
-#include "cs_parameters.h"
-#include "cs_physical_constants.h"
-#include "cs_turbulence_model.h"
-
-/*----------------------------------------------------------------------------
- *  Header for the current file
- *----------------------------------------------------------------------------*/
-
-#include "cs_prototypes.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -94,8 +70,8 @@ _i_faces_select_example(void         *input,
   CS_UNUSED(input);
 
   cs_lnum_t i, face_id;
-  cs_lnum_t n_families = 0;
-  cs_int_t *family_list = NULL;
+  int n_families = 0;
+  int *family_list = NULL;
   int *family_mask = NULL;
 
   cs_lnum_t n_i_faces = 0;
@@ -109,7 +85,7 @@ _i_faces_select_example(void         *input,
 
   /* Build mask on families matching groups "2" (1), "3" (2) */
 
-  BFT_MALLOC(family_list, m->n_families, cs_int_t);
+  BFT_MALLOC(family_list, m->n_families, int);
   BFT_MALLOC(family_mask, m->n_families, int);
 
   for (i = 0; i < m->n_families; i++)
@@ -229,7 +205,7 @@ _he_fraction_05_select(void        *input,
 
   const cs_mesh_t *m = cs_glob_mesh;
 
-  cs_field_t *f = cs_field_by_name("He_fraction"); /* Get access to field */
+  cs_field_t *f = cs_field_by_name_try("He_fraction"); /* Get access to field */
 
   if (f == NULL)
     bft_error(__FILE__, __LINE__, 0,
@@ -501,9 +477,7 @@ cs_user_postprocess_meshes(void)
   /* Example: attach default txt histogram writer on boundary mesh */
 
   /*! [post_attach_mesh_1] */
-  {
-    cs_post_mesh_attach_writer(CS_POST_MESH_BOUNDARY, CS_POST_WRITER_HISTOGRAMS);
-  }
+  cs_post_mesh_attach_writer(CS_POST_MESH_BOUNDARY, CS_POST_WRITER_HISTOGRAMS);
   /*! [post_attach_mesh_1] */
 
   /*--------------------------------------------------------------------------*/
@@ -511,10 +485,30 @@ cs_user_postprocess_meshes(void)
   /* Example: attach user tex histogram writer of id 6 on volume mesh */
 
   /*! [post_attach_mesh_2] */
-  {
-    cs_post_mesh_attach_writer(CS_POST_MESH_VOLUME, 6);
-  }
+  cs_post_mesh_attach_writer(CS_POST_MESH_VOLUME, 6);
   /*! [post_attach_mesh_2] */
+
+  /*--------------------------------------------------------------------------*/
+
+  /* Example: output specific field on mesh with all associated writers */
+
+  /*! [post_attach_field_1] */
+  cs_post_mesh_attach_field(4,
+                            CS_POST_WRITER_ALL_ASSOCIATED,
+                            cs_field_id_by_name("pressure"),
+                            -1);
+  /*! [post_attach_field_1] */
+  /*--------------------------------------------------------------------------*/
+
+  /* Example: output z-component of velocity field on mesh with
+     a given writer */
+
+  /*! [post_attach_field_2] */
+  cs_post_mesh_attach_field(4,
+                            1,
+                            CS_F_(vel)->id,
+                            2);
+  /*! [post_attach_field_2] */
 }
 
 /*----------------------------------------------------------------------------*/
@@ -541,11 +535,34 @@ cs_user_postprocess_probes(void)
     cs_probe_set_add_probe(pset, 0.25, 0.025, 0.025, "M1");
     cs_probe_set_add_probe(pset, 0.50, 0.025, 0.025, "M2");
     cs_probe_set_add_probe(pset, 0.75, 0.025, 0.025, "M3");
-
   }
   /*! [post_define_probes_1] */
 
+  /*! [post_define_probes_2] */
+  {
+    const cs_real_t coords[][3] = {{0.25, 0.025, 0.025},
+                                   {0.50, 0.025, 0.025},
+                                   {0.75, 0.025, 0.025}};
+    const char *labels[] = {"M1", "M2", "M3"};
+
+    cs_probe_set_t  *pset = cs_probe_set_create_from_array("Monitoring",
+                                                           3,
+                                                           coords,
+                                                           labels);
+  }
+  /*! [post_define_probes_2] */
+
+  /*! [post_set_probes_interpolate] */
+  {
+    cs_probe_set_t  *pset = cs_probe_set_get("probes");
+
+    cs_probe_set_option(pset, "interpolation", "1");
+  }
+  /*! [post_set_probes_interpolate] */
+
   /* Add a first profile */
+
+  /*! [post_define_profile_1] */
   {
     cs_coord_3_t  start = {0., 0.025, 0.025};
     cs_coord_3_t  end = {1., 0.025, 0.025};
@@ -559,36 +576,58 @@ cs_user_postprocess_probes(void)
 
     cs_probe_set_associate_writers(pset, 1, writer_ids);
   }
+  /*! [post_define_profile_1] */
 
   /* Add a second profile attached to boundary vertices */
+
+  /*! [post_define_profile_2] */
   {
     cs_coord_3_t  start = {0., 0., 0.};
     cs_coord_3_t  end = {1., 0., 0.};
 
-    cs_probe_set_create_from_segment("P1",    // name
-                                     11,      // n_probes
-                                     start,   // start coordinates
-                                     end);    // end coordinates
-  }
-
-  /* Add a second profile attached to boundary vertices */
-  {
-    cs_coord_3_t  start = {0., 0., 0.};
-    cs_coord_3_t  end = {1., 0., 0.};
-    int  writer_ids[] = {5};
-
-    cs_probe_set_t *pset =
-      cs_probe_set_create_from_segment("P2",     // name
-                                       21,       // n_probes
-                                       start,    // start coordinate
-                                       end);     // end coordinate
+    cs_probe_set_t  *pset =
+      cs_probe_set_create_from_segment("P2",    // name
+                                       11,      // n_probes
+                                       start,   // start coordinates
+                                       end);    // end coordinates
 
+    int  writer_ids[] = {2};
     cs_probe_set_associate_writers(pset, 1, writer_ids);
 
     cs_probe_set_option(pset, "boundary", "true");
     cs_probe_set_snap_mode(pset, CS_PROBE_SNAP_VERTEX);
   }
+  /*! [post_define_profile_2] */
 
+  /* Define output on a profile */
+
+  /*! [post_define_profile_3] */
+  {
+    cs_coord_3_t  start = {0., 0.025, 0.025};
+    cs_coord_3_t  end = {1., 0.025, 0.025};
+    int  writer_ids[] = {2};
+
+    cs_probe_set_t  *pset =
+      cs_probe_set_create_from_segment("Prof4", // name
+                                       11,      // n_probes
+                                       start,   // start coordinates
+                                       end);    // end coordinates
+
+    cs_probe_set_associate_writers(pset, 1, writer_ids);
+
+    cs_probe_set_auto_curvilinear_coords(pset, true);
+    cs_probe_set_auto_var(pset, false);
+
+    cs_probe_set_associate_field(pset,
+                                 CS_POST_WRITER_ALL_ASSOCIATED,
+                                 CS_F_(p)->id,
+                                 -1);
+    cs_probe_set_associate_field(pset,
+                                 CS_POST_WRITER_ALL_ASSOCIATED,
+                                 CS_F_(vel)->id,
+                                 0);
+  }
+  /*! [post_define_profile_3] */
 }
 
 /*----------------------------------------------------------------------------*/
@@ -632,51 +671,62 @@ cs_user_postprocess_values(const char            *mesh_name,
                            const cs_lnum_t        vertex_list[],
                            const cs_time_step_t  *ts)
 {
-  /* Output of k=1/2(R11+R22+R33) for the Rij-epsilon model
+  CS_NO_WARN_IF_UNUSED(probes);
+  CS_NO_WARN_IF_UNUSED(n_vertices);
+  CS_NO_WARN_IF_UNUSED(vertex_list);
+
+  /* Output of k = 1/2 (R11+R22+R33) for the Rij-epsilon model
      ------------------------------------------------------ */
 
   /*< [postprocess_values_ex_1] */
-  if (cat_id == CS_POST_MESH_VOLUME && cs_glob_turb_model->itytur == 3) {
+  if (cat_id == CS_POST_MESH_VOLUME) { /* filter: only for volume
+                                          postprocessing mesh */
 
-    cs_real_t *s_cell;
-    BFT_MALLOC(s_cell, n_cells, cs_real_t);
+    if (cs_glob_turb_model->itytur == 3) {
 
-    if (cs_glob_turb_rans_model->irijco) {
-      const cs_real_6_t *cvar_r = (const cs_real_6_t *)(CS_F_(rij)->val);
-      for (cs_lnum_t i = 0; i < n_cells; i++) {
-        cs_lnum_t cell_id = cell_list[i];
-        s_cell[i] = 0.5* (  cvar_r[cell_id][0]
-                          + cvar_r[cell_id][1]
-                          + cvar_r[cell_id][2]);
+      cs_real_t *s_cell;
+      BFT_MALLOC(s_cell, n_cells, cs_real_t);
+
+      if (cs_glob_turb_rans_model->irijco) {
+        const cs_real_6_t *cvar_r = (const cs_real_6_t *)(CS_F_(rij)->val);
+        for (cs_lnum_t i = 0; i < n_cells; i++) {
+          cs_lnum_t cell_id = cell_list[i];
+          s_cell[i] = 0.5* (  cvar_r[cell_id][0]
+                            + cvar_r[cell_id][1]
+                            + cvar_r[cell_id][2]);
+        }
       }
+
+      else {
+        const cs_real_t *cvar_r11 = CS_F_(r11)->val;
+        const cs_real_t *cvar_r22 = CS_F_(r22)->val;
+        const cs_real_t *cvar_r33 = CS_F_(r33)->val;
+
+        for (cs_lnum_t i = 0; i < n_cells; i++) {
+          cs_lnum_t cell_id = cell_list[i];
+          s_cell[i] = 0.5* (  cvar_r11[cell_id]
+                            + cvar_r22[cell_id]
+                            + cvar_r33[cell_id]);
+        }
+
+      }
+
+      cs_post_write_var(mesh_id,
+                        CS_POST_WRITER_ALL_ASSOCIATED,  /* writer id filter */
+                        "Turb energy",                  /* var_name */
+                        1,                              /* var_dim */
+                        true,                           /* interlace, */
+                        false,                          /* use_parent */
+                        CS_POST_TYPE_cs_real_t,         /* var_type */
+                        s_cell,                         /* cel_vals */
+                        NULL,                           /* i_face_vals */
+                        NULL,                           /* b_face_vals */
+                        ts);
+
+      BFT_FREE(s_cell);
+
     }
 
-    else {
-      const cs_real_t *cvar_r11 = CS_F_(r11)->val;
-      const cs_real_t *cvar_r22 = CS_F_(r22)->val;
-      const cs_real_t *cvar_r33 = CS_F_(r33)->val;
-
-      for (cs_lnum_t i = 0; i < n_cells; i++) {
-        cs_lnum_t cell_id = cell_list[i];
-        s_cell[i] = 0.5* (  cvar_r11[cell_id]
-                          + cvar_r22[cell_id]
-                          + cvar_r33[cell_id]);
-      }
-    }
-
-    cs_post_write_var(mesh_id,
-                      CS_POST_WRITER_ALL_ASSOCIATED,  /* writer id filter */
-                      "Turb energy",                  /* var_name */
-                      1,                              /* var_dim */
-                      true,                           /* interlace, */
-                      false,                          /* use_parent */
-                      CS_POST_TYPE_cs_real_t,         /* var_type */
-                      s_cell,                         /* cel_vals */
-                      NULL,                           /* i_face_vals */
-                      NULL,                           /* b_face_vals */
-                      ts);
-
-    BFT_FREE(s_cell);
   }
   /*< [postprocess_values_ex_1] */
 
@@ -752,7 +802,7 @@ cs_user_postprocess_values(const char            *mesh_name,
     if (f != NULL)
       cs_post_write_var(mesh_id,
                         CS_POST_WRITER_ALL_ASSOCIATED,  /* writer id filter */
-                        f->dim,                         /* var_name */
+                        f->name,                        /* var_name */
                         1,                              /* var_dim */
                         true,                           /* interlace, */
                         true,                           /* use_parent */
@@ -780,7 +830,7 @@ cs_user_postprocess_values(const char            *mesh_name,
 
         cs_post_write_var(mesh_id,
                           CS_POST_WRITER_ALL_ASSOCIATED,  /* writer id filter */
-                          f->dim,                         /* var_name */
+                          f->name,                        /* var_name */
                           1,                              /* var_dim */
                           true,                           /* interlace, */
                           true,                           /* use_parent */
@@ -814,6 +864,9 @@ cs_user_postprocess_activate(int     nt_max_abs,
                              int     nt_cur_abs,
                              double  t_cur_abs)
 {
+  CS_NO_WARN_IF_UNUSED(nt_cur_abs);
+  CS_NO_WARN_IF_UNUSED(t_cur_abs);
+
   /* Use the cs_post_activate_writer() function to force the
    * "active" or "inactive" flag for a specific writer or for all
    * writers for the current time step.
